// 定义相关接口
interface Message {
    FileFlag: boolean;
    FileData: string;
    Fd3: string;
    Fd10: string;
    Fd12: string;
    IntegrateFd(bitMapData: Uint8Array): Uint8Array;
    AnilyzeFd(msgData: Uint8Array): boolean;
}

// 模拟 PlatformSettings 类
class PlatformSettings {
    static Encoding = {
        getBytes: (str: string) => new TextEncoder().encode(str),
        getString: (bytes: Uint8Array) => new TextDecoder().decode(bytes)
    };
}

// 模拟 PlatformLogger 类
class PlatformLogger {
    static SystemErrorInfo(message: string, ex: any) {
        console.error(`${message}`, ex);
    }

    static CommunicationInfo(info: string) {
        console.log(info);
    }
}

// 模拟 LoginUserInfo
const LoginUserInfo = {
    KinbrNo: "",
    TtyName: ""
};

// 模拟 MsgParames
const MsgParames = {
    MsgType: "",
    KeyStr: ""
};

// 模拟 MsgPackage
const MsgPackage = {
    Encrypt: (data: Uint8Array, key: Uint8Array, mode: number) => {
        // 加密逻辑待实现
    }
};

// 模拟 CommonSettings
const CommonSettings = {
    KeyStr: ""
};

export class Msg858304 {
    private static instance: Msg858304;

    private constructor() {}

    static GetInstance(): Msg858304 {
        if (!this.instance) {
            this.instance = new Msg858304();
        }
        return this.instance;
    }

    Integrate(msg: Message): Uint8Array {
        let returnData: Uint8Array | null = null;

        try {
            returnData = this.DoIntegrate(msg);
        } catch (ex) {
            PlatformLogger.SystemErrorInfo("组包发生异常：", ex);
            throw ex;
        }
        return returnData!;
    }

    Analyze(msg: Message, data: Uint8Array): boolean {
        try {
            this.DoAnalyze(msg, data);
        } catch (ex) {
            return false;
        }
        return true;
    }

    private DoIntegrate(msg: Message): Uint8Array {
        let msgData: Uint8Array;
        let totalLen: number;
        let strLen: string;
        let msgDataLen: number;
        let msgTitle: string;
        let msgKindByte: Uint8Array;
        let bitMapByte = new Uint8Array(16);
        let bitDataByte: Uint8Array;
        let totalMsgData: Uint8Array;

        const fileFlag = msg.FileFlag ? 0x7f : 0xff;

        try {
            msgKindByte = PlatformSettings.Encoding.getBytes("00000");
            bitDataByte = msg.IntegrateFd(bitMapByte);

            msgDataLen = msgKindByte.length + bitMapByte.length + bitDataByte.length;
            totalLen = msgDataLen + 60;
            strLen = totalLen.toString().padStart(8, '0');
            msgTitle = msgDataLen.toString().padStart(4, '0').padStart(60, ' ');

            msgData = new Uint8Array(msgDataLen);
            for (let i = 0; i < msgData.length; i++) {
                if (i < msgKindByte.length) {
                    msgData[i] = msgKindByte[i];
                } else if (i < msgKindByte.length + bitMapByte.length) {
                    msgData[i] = bitMapByte[i - msgKindByte.length];
                } else {
                    msgData[i] = bitDataByte[i - msgKindByte.length - bitMapByte.length];
                }
            }

            // 模拟加密
            // MsgPackage.Encrypt(msgData, PlatformSettings.Encoding.getBytes(CommonSettings.KeyStr), 1);

            const len8 = PlatformSettings.Encoding.getBytes(strLen);
            const msgTitleByte = PlatformSettings.Encoding.getBytes(msgTitle);
            totalMsgData = new Uint8Array(len8.length + msgTitleByte.length + msgData.length + 1);
            for (let j = 0; j < totalMsgData.length - 1; j++) {
                if (j < len8.length) {
                    totalMsgData[j] = len8[j];
                } else if (j < len8.length + msgTitleByte.length) {
                    totalMsgData[j] = msgTitleByte[j - len8.length];
                } else {
                    totalMsgData[j] = msgData[j - len8.length - msgTitleByte.length];
                }
            }
            totalMsgData[totalMsgData.length - 1] = fileFlag;

            if (msg.FileFlag) {
                const file = PlatformSettings.Encoding.getBytes(msg.FileData);
                const lenData = PlatformSettings.Encoding.getBytes(file.length.toString().padStart(8, '0'));
                const fileName = PlatformSettings.Encoding.getBytes((LoginUserInfo.KinbrNo + LoginUserInfo.TtyName).padEnd(30));
                const ret = new Uint8Array(totalMsgData.length + fileName.length + lenData.length + file.length + 1);
                ret.set(totalMsgData, 0);
                ret.set(fileName, totalMsgData.length);
                ret.set(lenData, totalMsgData.length + fileName.length);
                ret.set(file, totalMsgData.length + fileName.length + lenData.length);
                ret[ret.length - 1] = 0xff;
                totalMsgData = ret;
            }
        } catch (ex) {
            PlatformLogger.SystemErrorInfo("组包发生异常！", ex);
            throw ex;
        }
        return totalMsgData;
    }

    private GetMsgT(msg: Message, msgDataLen: string): string {
        let clnm = "".padStart(5, ' ');
        let svcnm = "".padStart(15, ' ');
        let reqtype = "4";
        let branchNo = msg.Fd3.padLeft(10, ' ');
        let tty = msg.Fd10.padLeft(12, ' ');
        let titaOr8583 = "8";
        let cbs = "".padStart(10, ' ');
        let macflg = "".padStart(1, ' ');
        let mac = "".padStart(8, ' ');
        let result = "0";
        let len = msgDataLen.padLeft(4, '0');

        return clnm + svcnm + reqtype + branchNo + tty + titaOr8583 + cbs + macflg + mac + result + len;
    }

    private DoAnalyze(msg: Message, returnData: Uint8Array): void {
        try {
            const ret = returnData.slice(8);
            const msg60 = ret.slice(0, 60);
            const tmpLen = msg60.slice(56, 60);
            let msgDataLen = parseInt(PlatformSettings.Encoding.getString(tmpLen), 10);

            if (msgDataLen > 68) {
                msgDataLen -= 68;
            }

            const msgData = ret.slice(60, 60 + msgDataLen);
            // 模拟解密
            // MsgPackage.Encrypt(msgData, PlatformSettings.Encoding.getBytes(CommonSettings.KeyStr), 0);

            if (!msg.AnilyzeFd(msgData)) {
                throw new Error("8583通讯解包操作失败！具体情况请查看日志！");
            }

            const fileLen = returnData.length - 68 - msgDataLen - 38 - 1;
            if (fileLen > 0) {
                const fileData = returnData.slice(68 + msgDataLen + 38 + 1);
                msg.FileData = PlatformSettings.Encoding.getString(fileData);
                PlatformLogger.CommunicationInfo("\n文件数据:\n" + msg.FileData);
            }
        } catch (ex) {
            PlatformLogger.SystemErrorInfo("解析报文失败！", ex);
            throw ex;
        }
    }
}