import Message from "@/views/front/develop/Communication/Message";
import PlatformLogger from "@/views/front/platfrom/common/LogSystem/PlatformLogger";
import IPackage from './IPackage'
import { ServiceSettings } from "@/views/front/develop/ServiceProxy.Ext/ServiceSettings"
import TradeHandle from '../../ServiceProxy.Ext/ServiceHelper/TradeHandle'
import { PlatformSettings } from "@/views/front/platfrom/common/RunningParameters/PlatformSettings";
import { PrintManagerHandle } from "@/views/front/develop/ServiceProxy.Ext/ServiceHelper/PrintManagerHandle";
import SocketManager from "@/views/front/develop/Communication/SocketListener/SocketManager"
import { Msg858301 } from "./Msg858301";
import { Msg858302 } from "./Msg858302";
import { Msg858303 } from "./Msg858303";
import { MsgXml01 } from "./MsgXml01";

// 定义相关类型
enum PackageType {
    Msg858301,
    Msg858302,
    Msg858304,
    MsgXml01
}

export default class MsgPackage {
    // hulei 报文类型集合 根据不同的报文类型获取不同的单例模型 原文件中是Dictionary类型字典 typescript中没有对应的类型 这里可以自定义Dictionary类型
    static PackageList: {} = {//[key in PackageType]: IPackage
        // [PackageType.Msg858301]: {

        // },
        // [PackageType.Msg858302]: {
        //     Msg858302.getInstance(),
        // },
        // [PackageType.Msg858304]: {
        //     Msg858303.getInstance(),
        // },
        // [PackageType.MsgXml01]: {
        //     MsgXml01.getInstance(),
        // }
    };

    static Trade(msg: Message, packageType: PackageType, transit: string): boolean {
        let integrateData: Uint8Array;
        let totalData: Uint8Array;
        let returnData: Uint8Array;
        let fileData: Uint8Array = new Uint8Array(0);
        let retunValue = false;

        if (!this.PackageList[packageType]) {
            PlatformLogger.TradeErrorInfo("MsgPackage.Trade:不支持的报文类型！-->" + PackageType[packageType], null);
            return false;
        }

        try {
            const packageObj = this.PackageList[packageType];
            integrateData = packageObj.integrate(msg);

            const fileName = msg.Fd3.replace(/\s/g, '') + msg.Fd10.replace(/\s/g, '');
            const watch = new Date();
            let list: Uint8Array<ArrayBufferLike>[];

            if (ServiceSettings.IsRecordPrintData) {
                const msgType = msg.IsMainTrade ? PackageType[packageType] : '';
                try {
                    const encoder = new TextEncoder();

                    TradeHandle.DoTrade(transit, integrateData, msg.Fd3, fileName, encoder.encode(msg.fileData), null, msg.SerialNumber, msgType).then(x => {
                        list = x;
                    })
                } catch {
                    list = [];
                }

                if (msg.IsMainTrade && (list.length === 0)) {
                    for (let i = 0; i < 3; i++) {
                        try {
                            const data = PrintManagerHandle.GetPrintExtDataById(msg.SerialNumber);
                            if (data) {
                                PlatformLogger.SystemInfo("MsgPackage:主通讯补偿成功！" + msg.SerialNumber);
                                // list = [data.Message];
                                // if (data.MsgFile && data.MsgFile.length > 0) {
                                //     list.push(data.MsgFile);
                                // }打印的数据暂时不处理hulei
                                break;
                            }
                        } catch (ex) {
                            PlatformLogger.SystemErrorInfo("MsgPackage:主通讯补偿失败！" + msg.SerialNumber, ex);
                        }
                        setTimeout(() => { }, 200);
                    }
                    TradeHandle.WriteImportantLog("主通讯触发补偿", msg.SerialNumber, (list.length > 0 ? "成功" : "失败"), "DoTrade", "2", SocketManager.GetLocalIpAddressList());
                }
            } else {
                const encoder = new TextEncoder();
                // PlatformSettings.Encoding.getBytes(msg.fileData) 用encoder.encode(msg.fileData) 代替hulei
                TradeHandle.DoTrade(transit, integrateData, msg.Fd3, fileName, encoder.encode(msg.fileData), null, msg.SerialNumber).then(
                    X => {
                        list = X

                    }
                )

            }

            const endTime = new Date();
            const elapsedMilliseconds = endTime.getTime() - watch.getTime();
            if (elapsedMilliseconds > 2000) {
                TradeHandle.WriteImportantLog("通讯时间统计", elapsedMilliseconds.toString(), msg.Fd16, "DoTrade", "2", "0.0.0.0");//hulei获取IP地址 暂时注销SocketManager.GetLocalIpAddressList().join(',')
            }

            msg.IsMainTrade = false;
            returnData = list[0];

            if (list.length > 1) {
                fileData = list[1];
            }

            let fileDataFlag = 0;
            if (returnData && returnData.length > 0) {
                if (returnData.length < 20) {
                    PlatformLogger.SystemErrorInfo(`MsgPackage:返回报文长度太短，无法获取附件标志！长度: ${returnData.length}`, new Error());
                }
                fileDataFlag = returnData[20];
            } else {
                msg.Fd12 = "COMM";
                return false;
            }

            msg.fileFlag = false;
            totalData = new Uint8Array(returnData.length + fileData.length);
            totalData.set(returnData, 0);
            totalData.set(fileData, returnData.length);

            retunValue = packageObj.analyze(msg, totalData);
        } catch (ex) {
            PlatformLogger.SystemErrorInfo("Trade: 函数出错. ", ex);
            return false;
        }

        return retunValue;
    }

    static Encrypt(myData: Uint8Array, key: Uint8Array, flg: number): boolean {
        if (true) return true;

        const klen = key.length;
        try {
            for (let i = 0; i < myData.length; i++) {
                if (flg === 1) {
                    if ((myData[i] + key[i % klen]) > 255) {
                        const value = (myData[i] + key[i % klen]).toString(2);
                        myData[i] = parseInt(value.slice(-8), 2);
                    } else {
                        myData[i] = myData[i] + key[i % klen];
                    }
                } else {
                    if (myData[i] < key[i % klen]) {
                        const value = myData[i].toString(2).padStart(8, '0');
                        myData[i] = parseInt('1' + value, 2) - key[i % klen];
                    } else {
                        myData[i] = myData[i] - key[i % klen];
                    }
                }
            }
        } catch (ex) {
            PlatformLogger.SystemErrorInfo("加密解密出错！", ex);
            throw ex;
        }
        return true;
    }

    static Clone(obj: Message, source: Message): void {
        for (const key in source) {
            if (source.hasOwnProperty(key)) {
                (obj as any)[key] = (source as any)[key];
            }
        }
    }

    static CheckByte(data: Uint8Array): void {
        if (!data || data.length === 0) return;

        for (let i = 0; i < data.length; i++) {
            if (data[i] < 128) continue;

            if (i + 1 >= data.length) continue;

            const tmpData = new Uint8Array([data[i], data[i + 1]]);
            const decoder = new TextDecoder('utf-8');
            // const tmpStr = PlatformSettings.Encoding.getString(tmpData);//hulei 原始逻辑需要引用到系统system.IO中的getstring()  将字节转换成string
            const tmpStr = decoder.decode(tmpData);
            if (this.IsSbc(tmpStr) && !this.IsChinese(tmpStr)) {//判断是否全是全角和中文
                i++;
                continue;
            }

            if (data[i] >= 0x81 && data[i] <= 0xa0) {
                if (i + 1 >= data.length) continue;
                if (data[i + 1] < 0x40 || data[i + 1] > 0xfe) {
                    data[i] = 61;
                    continue;
                }
                i++;
            } else if (data[i] >= 0xa1 && data[i] <= 0xa9) {
                if (i + 1 >= data.length) continue;
                if (data[i + 1] < 0xa1 || data[i + 1] > 0xef) {
                    data[i] = 61;
                    continue;
                }
                i++;
            } else if (data[i] >= 0xaa && data[i] <= 0xfe) {
                if (i + 1 >= data.length) continue;
                if (data[i + 1] < 0x40) {
                    data[i] = 61;
                    continue;
                } else if (data[i + 1] > 0xa0) {
                    if (data[i] >= 0xb0 && data[i] <= 0xf7) {
                        if (data[i + 1] < 0xa1 || data[i + 1] > 0xfe) {
                            data[i] = 61;
                            continue;
                        }
                    }
                }
                i++;
            } else {
                data[i] = 61;
            }
        }
    }

    private static IsSbc(str: string): boolean {
        return str.length * 2 === new TextEncoder().encode(str).length;
    }

    private static IsChinese(word: string): boolean {
        if (!word) return false;
        const rx = /^[\u4e00-\u9fa5]$/;
        return rx.test(word);
    }
}
