/*----------------------------------------------------------------------------------------------
 
       Copyright (C) DHCC Ȩ

       ϵͳ:
         :  BufferManager
         Ҫ: 
       :  Hu Ha
       :  01/18/2010 17:13:01
         :    No.                   Ver.No        ޸             																																																																										

------------------------------------------------------------------------------------------------*/

using System;
using System.Collections.Generic;
using System.Text;

namespace TellerSystem.Communication.SocketListener
{
    /// <summary>
    /// ṩպͷͻĹ
    /// </summary>
    public sealed class BufferManager
    {
        private byte[] m_receiveBuffer;
        private byte[] m_sendBuffer;

        private int m_maxRequest;
        private int m_receiveBufferSize;
        private int m_sendBufferSize;

        private int m_bufferBlockIndex;
        private Stack<int> m_bufferBlockIndexStack;

        private static object Locker = new object();

        /// <summary>
        /// Initializes a new instance of the <see cref="BufferManager"/> class.
        /// </summary>
        /// <param name="maxRequest">The max request.</param>
        /// <param name="receivevBufferSize">Size of the receivev buffer.</param>
        /// <param name="sendBufferSize">Size of the send buffer.</param>
        public BufferManager(int maxRequest, int receivevBufferSize, int sendBufferSize)
        {
            m_maxRequest = maxRequest;
            m_receiveBufferSize = receivevBufferSize;
            m_sendBufferSize = sendBufferSize;

            m_bufferBlockIndex = 0;
            m_bufferBlockIndexStack = new Stack<int>(maxRequest);

            m_receiveBuffer = new byte[m_receiveBufferSize * m_maxRequest];
            m_sendBuffer = new byte[m_sendBufferSize * m_maxRequest];
        }

        /// <summary>
        /// ջܳ
        /// </summary>
        public int ReceiveBufferSize
        {
            get { return m_receiveBufferSize; }
        }

        /// <summary>
        /// ͻܳ
        /// </summary>
        public int SendBufferSize
        {
            get { return m_sendBufferSize; }
        }

        /// <summary>
        /// ջ
        /// </summary>
        public byte[] ReceiveBuffer
        {
            get { return m_receiveBuffer; }
        }

        /// <summary>
        /// ͻ
        /// </summary>
        public byte[] SendBuffer
        {
            get { return m_sendBuffer; }
        }

        /// <summary>
        /// ͷŻ
        /// </summary>
        /// <param name="bufferBlockIndex"></param>
        public void FreeBufferBlockIndex(int bufferBlockIndex)
        {
            if (bufferBlockIndex >= this.m_maxRequest || bufferBlockIndex < 0)
            {
                return;
            }

            lock (Locker)
            {
                if (this.m_bufferBlockIndexStack.Contains(bufferBlockIndex))
                    return;

                m_bufferBlockIndexStack.Push(bufferBlockIndex);
            }
        }

        /// <summary>
        /// ȡе
        /// </summary>
        /// <returns></returns>
        public int GetBufferBlockIndex()
        {
            lock (Locker)
            {
                int blockIndex = -1;

                // ùͷŵĻ
                if (m_bufferBlockIndexStack.Count > 0)
                {
                    blockIndex = m_bufferBlockIndexStack.Pop();
                }
                else
                {
                    // δû
                    if (m_bufferBlockIndex < m_maxRequest)
                    {
                        blockIndex = m_bufferBlockIndex++;
                    }
                }

                return blockIndex;
            }
        }

        /// <summary>
        /// ȡջƫ
        /// </summary>
        /// <param name="bufferBlockIndex">˽ջ</param>
        /// <returns>ջƫ</returns>
        public int GetReceiveBufferOffset(int bufferBlockIndex)
        {
            if (bufferBlockIndex >= this.m_maxRequest)
            {
                throw new ArgumentException(string.Format(":{0}", this.m_maxRequest.ToString()));
            }

            if (bufferBlockIndex < 0)
            {
                return 0;
            }

            return bufferBlockIndex * m_receiveBufferSize;
        }

        /// <summary>
        /// ȡͻƫ
        /// </summary>
        /// <param name="bufferBlockIndex">˷ͻ</param>
        /// <returns>ͻƫ</returns>
        public int GetSendBufferOffset(int bufferBlockIndex)
        {
            if (bufferBlockIndex >= this.m_maxRequest)
            {
                throw new ArgumentException(string.Format(":{0}", this.m_maxRequest.ToString()));
            }

            if (bufferBlockIndex < 0)
            {
                return 0;
            }

            return bufferBlockIndex * m_sendBufferSize;
        }

        /// <summary>
        /// 
        /// </summary>
        public void Clear()
        {
            m_bufferBlockIndexStack.Clear();
            m_receiveBuffer = null;
            m_sendBuffer = null;
        }
    }
}