import { ref, reactive, defineComponent } from 'vue'
import { TradeBusHelper } from "@/views/front/develop/Constract/ContractBase/TradeBusHelper"
import { TradeInfoHelper } from "@/views/front/develop/Constract/ContractBase/TradeInfoHelper"
import TradeAPIsHelper from "@/views/front/develop/Constract/ContractBase/TradeAPIsHelper"
import TradeResourcesHelper from "@/views/front/develop/Constract/ContractBase/TradeAPIsHelper"
import { ITradeHelper } from "@/views/front/develop/Constract/ContractBase/ITradeHelper"
//import { PaperlessEntry } from './PaperlessEntry'
//import { CashCodeEntry } from './CashCodeEntry'
import PlatformLogger from "@/views/front/platfrom/common/LogSystem/PlatformLogger";
import { ServiceSettings } from "@/views/front/develop/ServiceProxy.Ext/ServiceSettings"
import TradeHandle from '../../ServiceProxy.Ext/ServiceHelper/TradeHandle'
import { PrintAttributes } from "../../PrintTemplate/Extension/PrintAttributes";
import Message from '../../Communication/Message'
import TradePage from '@/views/front/platfrom/presentation/PageFunctions/TradePage'
import { PageContainer } from '@/views/front/platfrom/presentation/PageFunctions/PageContainer'
import TradeModel from '@/views/front/platfrom/common/BasicFunctions/PageFunctions/TradeModel'
//import TradeModel from "@/views/front/platfrom/common/BasicFunctions/";


/**
 * 页面基类，填充通用的业务接口API
 * 注意:该对象需要支持序列化/反序列化
 */
export abstract class PageBase implements TradePage, ITradeHelper {
    get TradeModel(): TradeModel {
        throw new Error('Method not implemented.')
    }
    set TradeModel(value: TradeModel) {
        throw new Error('Method not implemented.')
    }
    get Ready(): { add: () => void; remove: () => void } {
        throw new Error('Method not implemented.')
    }
    Dispose(): void {
        throw new Error('Method not implemented.')
    }
    get OwnerContainer(): PageContainer {
        throw new Error('Method not implemented.')
    }
    SetOwnerContainer(pageContainer: PageContainer): void {
        throw new Error('Method not implemented.')
    }
    get InstanceId(): string {
        throw new Error('Method not implemented.')
    }
    SetInstanceId(guid: string): void {
        throw new Error('Method not implemented.')
    }
    get ViewStateData(): { [key: string]: any } {
        throw new Error('Method not implemented.')
    }
    SetViewStateData(viewStateData: { [key: string]: any }): void {
        throw new Error('Method not implemented.')
    }
    get PropertyChanged(): { add: () => void; remove: () => void } {
        throw new Error('Method not implemented.')
    }
    protected raisePropertyChanged(propertyName: string): void {
        throw new Error('Method not implemented.')
    }
    // }
    // 冠字号对象
    // private cashCode: CashCodeEntry | null = null
    // get CashCode(): CashCodeEntry {
    //     if (!this.cashCode) {
    //         this.cashCode = new CashCodeEntry()
    //     }
    //     return this.cashCode
    // }
    // 银行标签
    Serialize(msg: Message): any {
        throw new Error('Method not implemented.')
    }
    // 业务总线
    private _tradeBus: TradeBusHelper | null = null
    get TradeBus(): TradeBusHelper {
        if (!this._tradeBus) {
            this._tradeBus = new TradeBusHelper(this)
        }
        return this._tradeBus
    }

    // 交易信息
    private _tradeInfo: TradeInfoHelper | null = null
    get TradeInfo(): TradeInfoHelper {
        if (!this._tradeInfo) {
            this._tradeInfo = new TradeInfoHelper(this)
        }
        return this._tradeInfo
    }

    // 操作接口
    private _tradeAPIs: TradeAPIsHelper | null = null
    get TradeAPIs(): TradeAPIsHelper {
        if (!this._tradeAPIs) {
            this._tradeAPIs = new TradeAPIsHelper(this)
        }
        return this._tradeAPIs
    }

    // 资源集合
    private _tradeResources: TradeResourcesHelper | null = null
    get TradeResources(): TradeResourcesHelper {
        if (!this._tradeResources) {
            this._tradeResources = new TradeResourcesHelper(this)
        }
        return this._tradeResources
    }

    // 无纸化对象 hulei  无纸化和冠字号没用 暂时注销
    // private paperless: PaperlessEntry | null = null
    // get Paperless(): PaperlessEntry {
    //     if (!this.paperless) {
    //         this.paperless = new PaperlessEntry()
    //     }
    //     return this.paperless
    // }

    // 冠字号对象
    // private cashCode: CashCodeEntry | null = null
    // get CashCode(): CashCodeEntry {
    //     if (!this.cashCode) {
    //         this.cashCode = new CashCodeEntry()
    //     }
    //     return this.cashCode
    // }

    // 银行标签
    get BankFlag(): string {
        return ServiceSettings.BankFlag
    }

    // 打印事件
    private pagePrintedHandlers: ((attr: PrintAttributes) => void)[] = []

    /**
     * 初始化页面
     */
    public initPage(): boolean {
        return true
    }

    /**
     * 页面加载完成
     */
    protected onTradePageLoaded(): void {
        let flag = false
        try {
            // 加载页面初始化
            flag = this.initPage()
            if (flag) {
                // 焦点管理逻辑
                // 将交易打开的时间登记到bus
                try {
                    if (!this.TradeBus.get('__StartTime__')) {
                        this.TradeBus.set('__StartTime__', new Date().toISOString())
                    }
                } catch (ex) {
                    TradeHandle.WriteImportantLog('获取交易打开时间异常', ex.message, 'StartTime', "", "", "");
                }
            }
        } catch (e) {
            PlatformLogger.TradeErrorInfo('调用InitPage失败！', e,)// TradeModel.Code hulei trademodel 不明确类
        }
        if (!flag) {
            // 关闭页面逻辑
        }
    }

    /**
     * 记录打印日志
     */
    private log2PagePrinted(log: string): void {
        try {
            PlatformLogger.SystemInfo(`Log2PagePrinted:start 【${log}】=>${this.pagePrintedHandlers.length}`)
            if (this.pagePrintedHandlers.length === 0) return
            PlatformLogger.SystemInfo(`Log2PagePrinted:list length=>${this.pagePrintedHandlers.length}`)
        } catch { }
    }

    /**
     * 触发打印后事件
     */
    public onPagePrinted(attr: PrintAttributes): void {
        PlatformLogger.SystemInfo(`OnPagePrinted:触发事件=>${this.pagePrintedHandlers.length}`)
        if (this.pagePrintedHandlers.length > 0) {
            this.log2PagePrinted('OnPagePrinted.start')
            this.pagePrintedHandlers.forEach(handler => handler(attr))
            this.log2PagePrinted('OnPagePrinted.end')
        }
    }

    /**
     * 添加打印事件监听
     */
    public addPagePrintedListener(handler: (attr: PrintAttributes) => void): void {
        this.pagePrintedHandlers.push(handler)
    }

    /**
     * 移除打印事件监听
     */
    public removePagePrintedListener(handler: (attr: PrintAttributes) => void): void {
        this.pagePrintedHandlers = this.pagePrintedHandlers.filter(h => h !== handler)
    }

    /**
     * 清理缓存数据
     */
    public clear(): void {
        // this.paperless = null
        this._tradeAPIs = null
        this._tradeBus = null
        this._tradeInfo = null
        this._tradeResources = null
    }
}

// // Vue组件封装
// export const PageBaseComponent = defineComponent({
//     extends: PageBase,
//     methods: {
//         initPage(): boolean {
//             return super.initPage()
//         },
//         onPagePrinted(attr: PrintAttributes): void {
//             super.onPagePrinted(attr)
//         }
//     }
// })
