using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 检查柜员凭证最小号
    /// 根据相关要素，检查柜员凭证最小号
    /// </summary>
    public class ChecksTellerSmallestCertificate : TFOutput
    {
        #region Output
        /// <summary>
        /// 标志位 0730
        /// [空格表示没有查询相关数据]第一位[0]介质属性是否存在(0-存在 1-不存在) 第二位[1]是否需要输入卡号(0-否 1是) 第三位[2]卡号是否已经开户(0-否 1-是)第四位[3]是否是次小号码(0-是 1-否)第五位[4]查询最小号是否成功(0-成功 1-失败)第六位[5]卡凭证生成卡号是否成功(0-成功 1-失败)第七位[6]检验位是否正确(0-正确 1-错误)第八位[7]是否是最小号(0-是 1-否)
        /// </summary>
        public string BZW { get; internal set; }

        /// <summary>
        /// 最小卡号 0370
        /// 仅用于开发环境
        /// </summary>
        public string ZXKH { get; internal set; }

        /// <summary>
        /// 最小凭证号 0580
        /// 仅用于开发环境
        /// </summary>
        public string ZXPZH { get; internal set; }


        #endregion

        #region Input
		/// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 凭证类型 0890
            /// 介质代码 、凭证类型 至少输入一个 
            /// </summary>
            public string PZLX { get; set; }

            /// <summary>
            /// 次小号标志 0700
            /// 1-校验次小号  其他-校验最小号 
            /// </summary>
            public string CXHBZ { get; set; }

            /// <summary>
            /// 介质代号 1161
            /// 介质代码 、凭证类型 至少输入一个 
            /// </summary>
            public string JZDH { get; set; }

            /// <summary>
            /// 卡号 1162
            ///  
            /// </summary>
            public string KH { get; set; }

            /// <summary>
            /// 凭证号码 1168
            ///  
            /// </summary>
            public string PZHM { get; set; }


            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new ChecksTellerSmallestCertificate();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
				MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "K053";
                MessageData.Fd89 = PZLX;
                MessageData.Fd70 = CXHBZ;
                MessageData.Fd116_1 = JZDH;
                MessageData.Fd116_2 = KH;
                MessageData.Fd116_8 = PZHM;

                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }                
                output.BZW = (MessageData.Fd73);
                output.ZXKH = (MessageData.Fd37);
                output.ZXPZH = (MessageData.Fd58);

                return output;
            }
        }
        #endregion
    }
}
