using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 指定一级科目下的活期产品
    /// 根据一级科目号、介质代码查询产品列表
    /// </summary>
    public class ProductUnderLevelSubjects : TFOutput
    {
        #region Output
        /// <summary>
        /// 产品列表
        /// 文件附件:产品代码|产品描述|
        /// </summary>
        public string CPLB { get; internal set; }

        /// <summary>
        /// 产品列表
        /// 文件附件:产品代码|产品描述|
        /// </summary>
        public List<E_CPLB> CPLBData { get; internal set; }

        /// <summary>
        /// 产品列表
        /// 文件附件:产品代码|产品描述|
        /// </summary>
        public class E_CPLB
        {
            /// <summary>
            /// 产品代码
            /// </summary>
            public string CPDM { get; set; }

            /// <summary>
            /// 产品描述
            /// </summary>
            public string CPMS { get; set; }

        }

        #endregion

        #region Input
		/// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 币种 0210
            /// 必输 
            /// </summary>
            public string BZ { get; set; }

            /// <summary>
            /// 产品可以使用的介质代码 0310
            /// 可选输入 
            /// </summary>
            public string CPKYSYDJZDM { get; set; }

            /// <summary>
            /// 客户类型 0670
            /// 0：公用；1：个人；2：公司；3：机构（指政府机关等组织）；4：金融同业；5：内部员工；6金融机构 
            /// </summary>
            public string KHLX { get; set; }

            /// <summary>
            /// 业务种类 0300
            /// 一级科目号 
            /// </summary>
            public string YWZL { get; set; }


            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new ProductUnderLevelSubjects();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
				MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "CXCL";
                MessageData.Fd21 = BZ;
                MessageData.Fd31 = CPKYSYDJZDM;
                MessageData.Fd67 = KHLX;
                MessageData.Fd128 = "0013";
                MessageData.Fd30 = YWZL;

                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }                
                output.CPLBData = AnilyzeFileData<E_CPLB>(MessageData.FileData,
                                            "产品代码|产品描述|",
                                            x =>
                                            {
                                                return new E_CPLB
                                                {
                                                    CPDM = x["产品代码"],
                                                    CPMS = x["产品描述"]
                                                };
                                            });
                output.CPLB = MessageData.FileData;

                return output;
            }
        }
        #endregion
    }
}
