using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 黑名单信息信息
    /// 通过证件信息查询黑名单信息
    /// </summary>
    public class QueryBlackList : TFOutput
    {
        #region Output
        /// <summary>
        /// 黑名单列表
        /// 黑名单类型|黑名单状态|客户类型|证件类型|证件号码|客户名称|黑名单状态|信息来源|上单原因|上单日期|上单操作机构|上单操作柜员|上单操作流水|注销日期|注销操作机构|注销操作员|注销操作流水|
        /// </summary>
        public string HMDLB { get; internal set; }

        /// <summary>
        /// 黑名单列表
        /// 黑名单类型|黑名单状态|客户类型|证件类型|证件号码|客户名称|黑名单状态|信息来源|上单原因|上单日期|上单操作机构|上单操作柜员|上单操作流水|注销日期|注销操作机构|注销操作员|注销操作流水|
        /// </summary>
        public List<E_HMDLB> HMDLBData { get; internal set; }

        /// <summary>
        /// 黑名单列表
        /// 黑名单类型|黑名单状态|客户类型|证件类型|证件号码|客户名称|黑名单状态|信息来源|上单原因|上单日期|上单操作机构|上单操作柜员|上单操作流水|注销日期|注销操作机构|注销操作员|注销操作流水|
        /// </summary>
        public class E_HMDLB
        {
            /// <summary>
            /// 黑名单类型
            /// </summary>
            public string HMDLX { get; set; }

            /// <summary>
            /// 黑名单状态
            /// </summary>
            public string HMDZT { get; set; }

            /// <summary>
            /// 客户类型
            /// </summary>
            public string KHLX { get; set; }

            /// <summary>
            /// 证件类型
            /// </summary>
            public string ZJLX { get; set; }

            /// <summary>
            /// 证件号码
            /// </summary>
            public string ZJHM { get; set; }

            /// <summary>
            /// 客户名称
            /// </summary>
            public string KHMC { get; set; }

            /// <summary>
            /// 黑名单状态
            /// </summary>
            public string HMDZT_1 { get; set; }

            /// <summary>
            /// 信息来源
            /// </summary>
            public string XXLY { get; set; }

            /// <summary>
            /// 上单原因
            /// </summary>
            public string SDYY { get; set; }

            /// <summary>
            /// 上单日期
            /// </summary>
            public string SDRQ { get; set; }

            /// <summary>
            /// 上单操作机构
            /// </summary>
            public string SDCZJG { get; set; }

            /// <summary>
            /// 上单操作柜员
            /// </summary>
            public string SDCZGY { get; set; }

            /// <summary>
            /// 上单操作流水
            /// </summary>
            public string SDCZLS { get; set; }

            /// <summary>
            /// 注销日期
            /// </summary>
            public string ZXRQ { get; set; }

            /// <summary>
            /// 注销操作机构
            /// </summary>
            public string ZXCZJG { get; set; }

            /// <summary>
            /// 注销操作员
            /// </summary>
            public string ZXCZY { get; set; }

            /// <summary>
            /// 注销操作流水
            /// </summary>
            public string ZXCZLS { get; set; }

        }

        #endregion

        #region Input
		/// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 户名 0260
            ///  
            /// </summary>
            public string HM { get; set; }

            /// <summary>
            /// 证件号码 0620
            ///  
            /// </summary>
            public string ZJHM { get; set; }

            /// <summary>
            /// 客户类型 0670
            ///  
            /// </summary>
            public string KHLX { get; set; }

            /// <summary>
            /// 证件类型 0680
            ///  
            /// </summary>
            public string ZJLX { get; set; }


            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new QueryBlackList();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
				MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "CXCL";
                MessageData.Fd26 = HM;
                MessageData.Fd62 = ZJHM;
                MessageData.Fd67 = KHLX;
                MessageData.Fd68 = ZJLX;
                MessageData.Fd128 = "0038";

                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }                
                output.HMDLBData = AnilyzeFileData<E_HMDLB>(MessageData.FileData,
                                            "黑名单类型|黑名单状态|客户类型|证件类型|证件号码|客户名称|黑名单状态|信息来源|上单原因|上单日期|上单操作机构|上单操作柜员|上单操作流水|注销日期|注销操作机构|注销操作员|注销操作流水|",
                                            x =>
                                            {
                                                return new E_HMDLB
                                                {
                                                    HMDLX = x["黑名单类型"],
                                                    HMDZT = x["黑名单状态"],
                                                    KHLX = x["客户类型"],
                                                    ZJLX = x["证件类型"],
                                                    ZJHM = x["证件号码"],
                                                    KHMC = x["客户名称"],
                                                    HMDZT_1 = x["黑名单状态"],
                                                    XXLY = x["信息来源"],
                                                    SDYY = x["上单原因"],
                                                    SDRQ = x["上单日期"],
                                                    SDCZJG = x["上单操作机构"],
                                                    SDCZGY = x["上单操作柜员"],
                                                    SDCZLS = x["上单操作流水"],
                                                    ZXRQ = x["注销日期"],
                                                    ZXCZJG = x["注销操作机构"],
                                                    ZXCZY = x["注销操作员"],
                                                    ZXCZLS = x["注销操作流水"]
                                                };
                                            });
                output.HMDLB = MessageData.FileData;

                return output;
            }
        }
        #endregion
    }
}
