using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 现金下发/收缴、领入/上缴申请或审批等记录查询
    /// 根据相关要素，返回相应的该柜员已申请在途的现金信息列表
    /// </summary>
    public class QueryCashIssuedIncomeRecords : TFOutput
    {
        #region Output
        /// <summary>
        /// 已申请在途现金列表
        /// 后台返回文件，内容包括：状态|申请编号|申请柜员|申请机构|申请模式|币种|申请时间|申请金额|申请原因|拒绝原因|一百元|五十元|二十元|十元|五元|二元|一元|五角|二角|一角|
        /// </summary>
        public string YSQZTXJLB { get; internal set; }

        /// <summary>
        /// 已申请在途现金列表
        /// 后台返回文件，内容包括：状态|申请编号|申请柜员|申请机构|申请模式|币种|申请时间|申请金额|申请原因|拒绝原因|一百元|五十元|二十元|十元|五元|二元|一元|五角|二角|一角|
        /// </summary>
        public List<E_YSQZTXJLB> YSQZTXJLBData { get; internal set; }

        /// <summary>
        /// 已申请在途现金列表
        /// 后台返回文件，内容包括：状态|申请编号|申请柜员|申请机构|申请模式|币种|申请时间|申请金额|申请原因|拒绝原因|一百元|五十元|二十元|十元|五元|二元|一元|五角|二角|一角|
        /// </summary>
        public class E_YSQZTXJLB
        {
            /// <summary>
            /// 状态
            /// </summary>
            public string ZT { get; set; }

            /// <summary>
            /// 申请编号
            /// </summary>
            public string SQBH { get; set; }

            /// <summary>
            /// 申请柜员
            /// </summary>
            public string SQGY { get; set; }

            /// <summary>
            /// 申请机构
            /// </summary>
            public string SQJG { get; set; }

            /// <summary>
            /// 申请模式
            /// </summary>
            public string SQMS { get; set; }

            /// <summary>
            /// 币种
            /// </summary>
            public string BZ { get; set; }

            /// <summary>
            /// 申请时间
            /// </summary>
            public string SQSJ { get; set; }

            /// <summary>
            /// 申请金额
            /// </summary>
            public string SQJE { get; set; }

            /// <summary>
            /// 申请原因
            /// </summary>
            public string SQYY { get; set; }

            /// <summary>
            /// 拒绝原因
            /// </summary>
            public string JJYY { get; set; }

            /// <summary>
            /// 一百元
            /// </summary>
            public string YBY { get; set; }

            /// <summary>
            /// 五十元
            /// </summary>
            public string WSY { get; set; }

            /// <summary>
            /// 二十元
            /// </summary>
            public string ESY { get; set; }

            /// <summary>
            /// 十元
            /// </summary>
            public string SY { get; set; }

            /// <summary>
            /// 五元
            /// </summary>
            public string WY { get; set; }

            /// <summary>
            /// 二元
            /// </summary>
            public string EY { get; set; }

            /// <summary>
            /// 一元
            /// </summary>
            public string YY { get; set; }

            /// <summary>
            /// 五角
            /// </summary>
            public string WJ { get; set; }

            /// <summary>
            /// 二角
            /// </summary>
            public string EJ { get; set; }

            /// <summary>
            /// 一角
            /// </summary>
            public string YJ { get; set; }

        }

        #endregion

        #region Input
		/// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 申请日期(起始) 0440
            /// (null) 
            /// </summary>
            public string SQRQQS { get; set; }

            /// <summary>
            /// 申请日期(结束) 0450
            /// (null) 
            /// </summary>
            public string SQRQJS { get; set; }

            /// <summary>
            /// 申请模式 0670
            /// 0申请模式 1预约模式 
            /// </summary>
            public string SQMS { get; set; }

            /// <summary>
            /// 状态 0680
            /// 1提出申请 2收回申请 3批准申请 4拒绝申请 5下发待领入 6正式领入 7上缴待收入 8收缴 
            /// </summary>
            public string ZT { get; set; }

            /// <summary>
            /// 机构号 0910
            /// (null) 
            /// </summary>
            public string JGH { get; set; }

            /// <summary>
            /// 柜员号 0920
            /// (null) 
            /// </summary>
            public string GYH { get; set; }

            /// <summary>
            /// 标志位 0720
            /// 审批机构查询该域不传值 申请机构查询该域传1 
            /// </summary>
            public string BZW { get; set; }

            /// <summary>
            /// 申请类型 0660
            /// 0上缴申请 1下发申请 
            /// </summary>
            public string SQLX { get; set; }
            /// <summary>
            /// 交易类型 0690
            /// 0下发
            /// </summary>
            public string TMPLX { get; set; }

            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new QueryCashIssuedIncomeRecords();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
				MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "CXCL";
                MessageData.Fd44 = SQRQQS;
                MessageData.Fd45 = SQRQJS;
                MessageData.Fd67 = SQMS;
                MessageData.Fd68 = ZT;
                MessageData.Fd91 = JGH;
                MessageData.Fd92 = GYH;
                MessageData.Fd128 = "0001";
                MessageData.Fd72 = BZW;
                MessageData.Fd66 = SQLX;
                MessageData.Fd69 = TMPLX;
                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }                
                output.YSQZTXJLBData = AnilyzeFileData<E_YSQZTXJLB>(MessageData.FileData,
                                            "状态|申请编号|申请柜员|申请机构|申请模式|币种|申请时间|申请金额|申请原因|拒绝原因|一百元|五十元|二十元|十元|五元|二元|一元|五角|二角|一角|",
                                            x =>
                                            {
                                                return new E_YSQZTXJLB
                                                {
                                                    ZT = x["状态"],
                                                    SQBH = x["申请编号"],
                                                    SQGY = x["申请柜员"],
                                                    SQJG = x["申请机构"],
                                                    SQMS = x["申请模式"],
                                                    BZ = x["币种"],
                                                    SQSJ = x["申请时间"],
                                                    SQJE = x["申请金额"],
                                                    SQYY = x["申请原因"],
                                                    JJYY = x["拒绝原因"],
                                                    YBY = x["一百元"],
                                                    WSY = x["五十元"],
                                                    ESY = x["二十元"],
                                                    SY = x["十元"],
                                                    WY = x["五元"],
                                                    EY = x["二元"],
                                                    YY = x["一元"],
                                                    WJ = x["五角"],
                                                    EJ = x["二角"],
                                                    YJ = x["一角"]
                                                };
                                            });
                output.YSQZTXJLB = MessageData.FileData;

                return output;
            }
        }
        #endregion
    }
}
