using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 凭证审批下发/收缴、收入/上缴申请等信息记录查询
    /// 根据相关要素，返回相应的等待该柜员审批的下发申请记录列表
    /// </summary>
    public class QueryCertificateIssuedIncome : TFOutput
    {
        #region Output
        /// <summary>
        /// 待审批下发申请记录
        /// 后台返回文件，内容包括：申请编号|申请序号|凭证类别|凭证类型|申请柜员|申请机构|申请模式|申请数量|审批数量|申请时间|申请状态|制卡批次
        /// </summary>
        public string DSPXFSQJL { get; internal set; }

        /// <summary>
        /// 待审批下发申请记录
        /// 后台返回文件，内容包括：申请编号|申请序号|凭证类别|凭证类型|申请柜员|申请机构|申请模式|申请数量|审批数量|申请时间|申请状态|制卡批次
        /// </summary>
        public List<E_DSPXFSQJL> DSPXFSQJLData { get; internal set; }

        /// <summary>
        /// 待审批下发申请记录
        /// 后台返回文件，内容包括：申请编号|申请序号|凭证类别|凭证类型|申请柜员|申请机构|申请模式|申请数量|审批数量|申请时间|申请状态|制卡批次
        /// </summary>
        public class E_DSPXFSQJL
        {
            /// <summary>
            /// 申请编号
            /// </summary>
            public string SQBH { get; set; }

            /// <summary>
            /// 申请序号
            /// </summary>
            public string SQXH { get; set; }

            /// <summary>
            /// 凭证类别
            /// </summary>
            public string PZLB { get; set; }

            /// <summary>
            /// 凭证类型
            /// </summary>
            public string PZLX { get; set; }
            /// <summary>
            /// 凭证类型代码
            /// </summary>
            public string PZLXDM { get; set; }
            /// <summary>
            /// 凭证状态代码
            /// </summary>
            public string PZZTDM { get; set; }
            /// <summary>
            /// 凭证状态
            /// </summary>
            public string PZZT { get; set; }
            /// <summary>
            /// 申请柜员
            /// </summary>
            public string SQGY { get; set; }

            /// <summary>
            /// 申请机构
            /// </summary>
            public string SQJG { get; set; }

            /// <summary>
            /// 申请机构名称
            /// </summary>
            public string SQJGMC { get; set; }

            /// <summary>
            /// 申请模式
            /// </summary>
            public string SQMS { get; set; }

            /// <summary>
            /// 申请数量
            /// </summary>
            public string SQSL { get; set; }

            /// <summary>
            /// 审批数量
            /// </summary>
            public string SPSL { get; set; }

            /// <summary>
            /// 申请时间
            /// </summary>
            public string SQSJ { get; set; }

            /// <summary>
            /// 申请状态
            /// </summary>
            public string SQZT { get; set; }

            /// <summary>
            /// 制卡批次
            /// </summary>
            public string ZKPC { get; set; }
        }

        #endregion

        #region Input
		/// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 申请日期(起始) 0440
            /// (null) 
            /// </summary>
            public string SQRQQS { get; set; }

            /// <summary>
            /// 申请日期(终止) 0450
            /// (null) 
            /// </summary>
            public string SQRQZZ { get; set; }

            /// <summary>
            /// 申请模式 0670
            /// 0 申请模式 1 预约模式 
            /// </summary>
            public string SQMS { get; set; }

            /// <summary>
            /// 状态 0680
            /// 1 提出申请 2 收回申请 3 批准申请 4 拒绝申请 5 下发待领入 6 正式领入 7 上缴待收入 8 收缴 
            /// </summary>
            public string ZT { get; set; }

            /// <summary>
            /// 机构号 0910
            /// (null) 
            /// </summary>
            public string JGH { get; set; }

            /// <summary>
            /// 柜员号 0920
            /// (null) 
            /// </summary>
            public string GYH { get; set; }

            /// <summary>
            /// 标志位 0720
            /// 申请柜员查询时为1 审批柜员查询时不传值 
            /// </summary>
            public string BZW { get; set; }

            /// <summary>
            /// 申请类型 0660
            /// 0调入 1调出 
            /// </summary>
            public string SQLX { get; set; }

            /// <summary>
            /// 印章类 0710
            /// 传1只保留印章类，传其他只保留其他
            /// </summary>
            public string IsYZL { get; set; }


            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new QueryCertificateIssuedIncome();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
				MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "CXCL";
                MessageData.Fd44 = SQRQQS;
                MessageData.Fd45 = SQRQZZ;
                MessageData.Fd67 = SQMS;
                MessageData.Fd68 = ZT;
                MessageData.Fd91 = JGH;
                MessageData.Fd92 = GYH;
                MessageData.Fd128 = "0006";
                MessageData.Fd72 = BZW;
                MessageData.Fd66 = SQLX;
                MessageData.Fd71 = IsYZL;

                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }
                if (MessageData.FileData.Contains("凭证状态"))
                {
                    #region 齐丰核心文件，增加凭证状态，凭证状态代码，凭证类型代码 三个字段
                    output.DSPXFSQJLData = AnilyzeFileData<E_DSPXFSQJL>(MessageData.FileData,
                                                "申请编号|申请序号|凭证类别|凭证类型|凭证状态|申请柜员|申请机构|申请模式|申请数量|审批数量|申请时间|申请状态|制卡批次|凭证状态代码|凭证类型代码",
                                                x =>
                                                {
                                                    return new E_DSPXFSQJL
                                                    {
                                                        SQBH = x["申请编号"],
                                                        SQXH = x["申请序号"],
                                                        PZLB = x["凭证类别"],
                                                        PZLX = x["凭证类型"],
                                                        PZZT = x["凭证状态"],
                                                        SQGY = x["申请柜员"],
                                                        SQJG = x["申请机构"],
                                                        SQMS = x["申请模式"],
                                                        SQSL = x["申请数量"],
                                                        SPSL = x["审批数量"],
                                                        SQSJ = x["申请时间"],
                                                        SQZT = x["申请状态"],
                                                        ZKPC = x["制卡批次"],
                                                        PZZTDM = x["凭证状态代码"],
                                                        PZLXDM = x["凭证类型代码"],
                                                    };
                                                });
                    #endregion 齐丰核心文件，增加凭证状态，凭证状态代码，凭证类型代码 三个字段
                }
                else if (MessageData.FileData.Contains("申请机构名称"))
                {
                    #region 原核心文件-通用
                    output.DSPXFSQJLData = AnilyzeFileData<E_DSPXFSQJL>(MessageData.FileData,
                                                "申请编号|申请序号|凭证类别|凭证类型|申请柜员|申请机构|申请机构名称|申请模式|申请数量|审批数量|申请时间|申请状态|制卡批次",
                                                x =>
                                                {
                                                    return new E_DSPXFSQJL
                                                    {
                                                        SQBH = x["申请编号"],
                                                        SQXH = x["申请序号"],
                                                        PZLB = x["凭证类别"],
                                                        PZLX = x["凭证类型"],
                                                        SQGY = x["申请柜员"],
                                                        SQJG = x["申请机构"],
                                                        SQJGMC = x["申请机构名称"],
                                                        SQMS = x["申请模式"],
                                                        SQSL = x["申请数量"],
                                                        SPSL = x["审批数量"],
                                                        SQSJ = x["申请时间"],
                                                        SQZT = x["申请状态"],
                                                        ZKPC = x["制卡批次"]
                                                    };
                                                });
                    #endregion 原核心文件-通用
                }
                else
                {
                    #region 原核心文件-通用
                    output.DSPXFSQJLData = AnilyzeFileData<E_DSPXFSQJL>(MessageData.FileData,
                                                "申请编号|申请序号|凭证类别|凭证类型|申请柜员|申请机构|申请模式|申请数量|审批数量|申请时间|申请状态|制卡批次",
                                                x =>
                                                {
                                                    return new E_DSPXFSQJL
                                                    {
                                                        SQBH = x["申请编号"],
                                                        SQXH = x["申请序号"],
                                                        PZLB = x["凭证类别"],
                                                        PZLX = x["凭证类型"],
                                                        SQGY = x["申请柜员"],
                                                        SQJG = x["申请机构"],
                                                        SQMS = x["申请模式"],
                                                        SQSL = x["申请数量"],
                                                        SPSL = x["审批数量"],
                                                        SQSJ = x["申请时间"],
                                                        SQZT = x["申请状态"],
                                                        ZKPC = x["制卡批次"]
                                                    };
                                                });
                    #endregion 原核心文件-通用
                }
                output.DSPXFSQJL = MessageData.FileData;

                return output;
            }
        }
        #endregion
    }
}
