using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 查询柜员待领入/待收缴凭证信息
    /// 通过柜员号查询该柜员待领入/待收缴凭证信息
    /// </summary>
    public class QueryCollarintoCertificate : TFOutput
    {
        #region Output
        /// <summary>
        /// 凭证信息文件
        /// ~申请编号|申请序号|凭证类别|凭证类型|申请柜员|申请机构|申请模式|数量|申请时间|申请状态|起始号码|终止号码|制卡批次
        /// </summary>
        public string PZXXWJ { get; internal set; }

        /// <summary>
        /// 凭证信息文件
        /// ~申请编号|申请序号|凭证类别|凭证类型|申请柜员|申请机构|申请模式|数量|申请时间|申请状态|起始号码|终止号码|制卡批次
        /// </summary>
        public List<E_PZXXWJ> PZXXWJData { get; internal set; }

        /// <summary>
        /// 凭证信息文件
        /// ~申请编号|申请序号|凭证类别|凭证类型|申请柜员|申请机构|申请模式|数量|申请时间|申请状态|起始号码|终止号码|制卡批次
        /// </summary>
        public class E_PZXXWJ
        {
            /// <summary>
            /// 申请编号
            /// </summary>
            public string SQBH { get; set; }

            /// <summary>
            /// 申请序号
            /// </summary>
            public string SQXH { get; set; }

            /// <summary>
            /// 凭证类别
            /// </summary>
            public string PZLB { get; set; }

            /// <summary>
            /// 凭证类型
            /// </summary>
            public string PZLX { get; set; }
            /// <summary>
            /// 凭证状态
            /// </summary>
            public string PZZT { get; set; }

            /// <summary>
            /// 申请柜员
            /// </summary>
            public string SQGY { get; set; }

            /// <summary>
            /// 申请机构
            /// </summary>
            public string SQJG { get; set; }

            /// <summary>
            /// 申请模式
            /// </summary>
            public string SQMS { get; set; }

            /// <summary>
            /// 数量
            /// </summary>
            public string SL { get; set; }

            /// <summary>
            /// 申请时间
            /// </summary>
            public string SQSJ { get; set; }

            /// <summary>
            /// 申请状态
            /// </summary>
            public string SQZT { get; set; }

            /// <summary>
            /// 起始号码
            /// </summary>
            public string QSHM { get; set; }

            /// <summary>
            /// 终止号码
            /// </summary>
            public string ZZHM { get; set; }

            /// <summary>
            /// 制卡批次
            /// </summary>
            public string ZKPC { get; set; }
        }

        #endregion

        #region Input
        /// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 标志位 0680
            /// 收缴时上级机构查询为1 领入时申请机构查询不传值 
            /// </summary>
            public string BZW { get; set; }

            /// <summary>
            /// 申请状态 0720
            /// 1提出申请 2收回申请 3批准申请 4拒绝申请 5下发待领入 6正式领入 7上缴待收入 8收缴 
            /// </summary>
            public string SQZT { get; set; }

            /// <summary>
            /// 柜员号 0920
            ///  
            /// </summary>
            public string GYH { get; set; }

            /// <summary>
            /// 申请类型 0660
            /// 0调入 1调出 
            /// </summary>
            public string SQLX { get; set; }

            /// <summary>
            /// 印章类 0710
            /// 传1只保留印章类，传其他只保留其他
            /// </summary>
            public string IsYZL { get; set; }

            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new QueryCollarintoCertificate();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
                MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "CXCL";
                MessageData.Fd68 = BZW;
                MessageData.Fd72 = SQZT;
                MessageData.Fd92 = GYH;
                MessageData.Fd128 = "0042";
                MessageData.Fd66 = SQLX;
                MessageData.Fd71 = IsYZL;

                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }
                if (MessageData.FileData.Contains("凭证状态"))
                {
                    #region 齐丰核心文件，增加 凭证状态 字段
                    output.PZXXWJData = AnilyzeFileData<E_PZXXWJ>(MessageData.FileData,
                                                "申请编号|申请序号|凭证类别|凭证类型|凭证状态|申请柜员|申请机构|申请模式|数量|申请时间|申请状态|起始号码|终止号码|制卡批次",
                                                x =>
                                                {
                                                    return new E_PZXXWJ
                                                    {
                                                        SQBH = x["申请编号"],
                                                        SQXH = x["申请序号"],
                                                        PZLB = x["凭证类别"],
                                                        PZLX = x["凭证类型"],
                                                        PZZT = x["凭证状态"],
                                                        SQGY = x["申请柜员"],
                                                        SQJG = x["申请机构"],
                                                        SQMS = x["申请模式"],
                                                        SL = x["数量"],
                                                        SQSJ = x["申请时间"],
                                                        SQZT = x["申请状态"],
                                                        QSHM = x["起始号码"],
                                                        ZZHM = x["终止号码"],
                                                        ZKPC = x["制卡批次"]
                                                    };
                                                });
                    #endregion 齐丰核心文件，增加凭证状态，凭证状态代码，凭证类型代码 三个字段
                }
                else
                {
                    #region 原核心文件-通用
                    output.PZXXWJData = AnilyzeFileData<E_PZXXWJ>(MessageData.FileData,
                                            "申请编号|申请序号|凭证类别|凭证类型|申请柜员|申请机构|申请模式|数量|申请时间|申请状态|起始号码|终止号码|制卡批次",
                                            x =>
                                            {
                                                return new E_PZXXWJ
                                                {
                                                    SQBH = x["申请编号"],
                                                    SQXH = x["申请序号"],
                                                    PZLB = x["凭证类别"],
                                                    PZLX = x["凭证类型"],
                                                    SQGY = x["申请柜员"],
                                                    SQJG = x["申请机构"],
                                                    SQMS = x["申请模式"],
                                                    SL = x["数量"],
                                                    SQSJ = x["申请时间"],
                                                    SQZT = x["申请状态"],
                                                    QSHM = x["起始号码"],
                                                    ZZHM = x["终止号码"],
                                                    ZKPC = x["制卡批次"]
                                                };
                                            });
                    #endregion 原核心文件-通用
                }
                output.PZXXWJ = MessageData.FileData;

                return output;
            }
        }
        #endregion
    }
}
