using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 查询抵质押品更改信息(文件)
    /// 通过合同号查询抵质押品更改信息
    /// </summary>
    public class QueryCollateralInformationChangedOfFile : TFOutput
    {
        #region Output
        /// <summary>
        /// 抵质押列表
        /// 担保合同号|抵质押品编号|抵质押类型|原抵质押物名称|抵质押物名称|原抵质押金额|抵质押金额|原抵质押价值|抵质押价值|修改状态|操作员|交易机构号|交易日期|摘要|
        /// </summary>
        public string DZYLB { get; internal set; }

        /// <summary>
        /// 抵质押列表
        /// 担保合同号|抵质押品编号|抵质押类型|原抵质押物名称|抵质押物名称|原抵质押金额|抵质押金额|原抵质押价值|抵质押价值|修改状态|操作员|交易机构号|交易日期|摘要|
        /// </summary>
        public List<E_DZYLB> DZYLBData { get; internal set; }

        /// <summary>
        /// 抵质押列表
        /// 担保合同号|抵质押品编号|抵质押类型|原抵质押物名称|抵质押物名称|原抵质押金额|抵质押金额|原抵质押价值|抵质押价值|修改状态|操作员|交易机构号|交易日期|摘要|
        /// </summary>
        public class E_DZYLB
        {
            /// <summary>
            /// 担保合同号
            /// </summary>
            public string DBHTH { get; set; }

            /// <summary>
            /// 抵质押品编号
            /// </summary>
            public string DZYPBH { get; set; }

            /// <summary>
            /// 抵质押类型
            /// </summary>
            public string DZYLX { get; set; }

            /// <summary>
            /// 原抵质押物名称
            /// </summary>
            public string YDZYWMC { get; set; }

            /// <summary>
            /// 抵质押物名称
            /// </summary>
            public string DZYWMC { get; set; }

            /// <summary>
            /// 原抵质押金额
            /// </summary>
            public string YDZYJE { get; set; }

            /// <summary>
            /// 抵质押金额
            /// </summary>
            public string DZYJE { get; set; }

            /// <summary>
            /// 原抵质押价值
            /// </summary>
            public string YDZYJZ { get; set; }

            /// <summary>
            /// 抵质押价值
            /// </summary>
            public string DZYJZ { get; set; }

            /// <summary>
            /// 修改状态
            /// </summary>
            public string XGZT { get; set; }

            /// <summary>
            /// 操作员
            /// </summary>
            public string CZY { get; set; }

            /// <summary>
            /// 交易机构号
            /// </summary>
            public string JYJGH { get; set; }

            /// <summary>
            /// 交易日期
            /// </summary>
            public string JYRQ { get; set; }

            /// <summary>
            /// 摘要
            /// </summary>
            public string ZY { get; set; }

        }

        #endregion

        #region Input
		/// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 担保合同号 0380
            ///  
            /// </summary>
            public string DBHTH { get; set; }


            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new QueryCollateralInformationChangedOfFile();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
				MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "CXCL";
                MessageData.Fd38 = DBHTH;
                MessageData.Fd128 = "0058";

                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }                
                output.DZYLBData = AnilyzeFileData<E_DZYLB>(MessageData.FileData,
                                            "担保合同号|抵质押品编号|抵质押类型|原抵质押物名称|抵质押物名称|原抵质押金额|抵质押金额|原抵质押价值|抵质押价值|修改状态|操作员|交易机构号|交易日期|摘要|",
                                            x =>
                                            {
                                                return new E_DZYLB
                                                {
                                                    DBHTH = x["担保合同号"],
                                                    DZYPBH = x["抵质押品编号"],
                                                    DZYLX = x["抵质押类型"],
                                                    YDZYWMC = x["原抵质押物名称"],
                                                    DZYWMC = x["抵质押物名称"],
                                                    YDZYJE = x["原抵质押金额"],
                                                    DZYJE = x["抵质押金额"],
                                                    YDZYJZ = x["原抵质押价值"],
                                                    DZYJZ = x["抵质押价值"],
                                                    XGZT = x["修改状态"],
                                                    CZY = x["操作员"],
                                                    JYJGH = x["交易机构号"],
                                                    JYRQ = x["交易日期"],
                                                    ZY = x["摘要"]
                                                };
                                            });
                output.DZYLB = MessageData.FileData;

                return output;
            }
        }
        #endregion
    }
}
