using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 收费黑白名单查询
    /// 收费黑白名单查询
    /// </summary>
    public class QueryFeeBlackList : TFOutput
    {
        #region Output
        /// <summary>
        /// 名单列表
        /// 名单范围|交易机构|登记机构|客户类型|客户号|户名|控制类型|控制账号|名单性质|交易日期|流水号|操作柜员|授权人|状态|备注|
        /// </summary>
        public string MDLB { get; internal set; }

        /// <summary>
        /// 名单列表
        /// 名单范围|交易机构|登记机构|客户类型|客户号|户名|控制类型|控制账号|名单性质|交易日期|流水号|操作柜员|授权人|状态|备注|
        /// </summary>
        public List<E_MDLB> MDLBData { get; internal set; }

        /// <summary>
        /// 名单列表
        /// 名单范围|交易机构|登记机构|客户类型|客户号|户名|控制类型|控制账号|名单性质|交易日期|流水号|操作柜员|授权人|状态|备注|
        /// </summary>
        public class E_MDLB
        {
            /// <summary>
            /// 名单范围
            /// </summary>
            public string MDFW { get; set; }

            /// <summary>
            /// 交易机构
            /// </summary>
            public string JYJG { get; set; }

            /// <summary>
            /// 登记机构
            /// </summary>
            public string DJJG { get; set; }

            /// <summary>
            /// 客户类型
            /// </summary>
            public string KHLX { get; set; }

            /// <summary>
            /// 客户号
            /// </summary>
            public string KHH { get; set; }

            /// <summary>
            /// 户名
            /// </summary>
            public string HM { get; set; }

            /// <summary>
            /// 控制类型
            /// </summary>
            public string KZLX { get; set; }

            /// <summary>
            /// 控制账号
            /// </summary>
            public string KZZH { get; set; }

            /// <summary>
            /// 名单性质
            /// </summary>
            public string MDXZ { get; set; }

            /// <summary>
            /// 交易日期
            /// </summary>
            public string JYRQ { get; set; }

            /// <summary>
            /// 流水号
            /// </summary>
            public string LSH { get; set; }

            /// <summary>
            /// 操作柜员
            /// </summary>
            public string CZGY { get; set; }

            /// <summary>
            /// 授权人
            /// </summary>
            public string SQR { get; set; }

            /// <summary>
            /// 状态
            /// </summary>
            public string ZT { get; set; }

            /// <summary>
            /// 备注
            /// </summary>
            public string BZ { get; set; }

        }

        #endregion

        #region Input
		/// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 客户号 0280
            /// 名单范围为2.指定客户时输入 
            /// </summary>
            public string KHH { get; set; }

            /// <summary>
            /// 账号 0300
            /// 名单范围为1.指定账户时输入 
            /// </summary>
            public string ZH { get; set; }

            /// <summary>
            /// 客户类型 0670
            /// 1.个人2.对公 
            /// </summary>
            public string KHLX { get; set; }

            /// <summary>
            /// 名单范围 0690
            /// 1.指定账户2.指定客户 
            /// </summary>
            public string MDFW { get; set; }


            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new QueryFeeBlackList();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
				MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "CXCL";
                MessageData.Fd28 = KHH;
                MessageData.Fd30 = ZH;
                MessageData.Fd67 = KHLX;
                MessageData.Fd69 = MDFW;
                MessageData.Fd128 = "0049";

                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }                
                output.MDLBData = AnilyzeFileData<E_MDLB>(MessageData.FileData,
                                            "名单范围|交易机构|登记机构|客户类型|客户号|户名|控制类型|控制账号|名单性质|交易日期|流水号|操作柜员|授权人|状态|备注|",
                                            x =>
                                            {
                                                return new E_MDLB
                                                {
                                                    MDFW = x["名单范围"],
                                                    JYJG = x["交易机构"],
                                                    DJJG = x["登记机构"],
                                                    KHLX = x["客户类型"],
                                                    KHH = x["客户号"],
                                                    HM = x["户名"],
                                                    KZLX = x["控制类型"],
                                                    KZZH = x["控制账号"],
                                                    MDXZ = x["名单性质"],
                                                    JYRQ = x["交易日期"],
                                                    LSH = x["流水号"],
                                                    CZGY = x["操作柜员"],
                                                    SQR = x["授权人"],
                                                    ZT = x["状态"],
                                                    BZ = x["备注"]
                                                };
                                            });
                output.MDLB = MessageData.FileData;

                return output;
            }
        }
        #endregion
    }
}
