using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 查询划款协议登记明细表
    /// 根据账号查询划款协议登记明细表
    /// </summary>
    public class QueryPaymentAgreementRecords : TFOutput
    {
        #region Output
        /// <summary>
        /// 签约明细
        /// 账号|序号|协议|登记机构|主账号|主账号序号|留存金额|划转金额|定期3个月百分比|定期6个月百分比|定期1年百分比|定期2年百分比|定期3年百分比|定期5年百分比|通知1天百分比|通知7天百分比|登记日期|到期日期|续约标志|流水号|柜员|状态|
        /// </summary>
        public string QYMX { get; internal set; }

        /// <summary>
        /// 签约明细
        /// 账号|序号|协议|登记机构|主账号|主账号序号|留存金额|划转金额|定期3个月百分比|定期6个月百分比|定期1年百分比|定期2年百分比|定期3年百分比|定期5年百分比|通知1天百分比|通知7天百分比|登记日期|到期日期|续约标志|流水号|柜员|状态|
        /// </summary>
        public List<E_QYMX> QYMXData { get; internal set; }

        /// <summary>
        /// 签约明细
        /// 账号|序号|协议|登记机构|主账号|主账号序号|留存金额|划转金额|定期3个月百分比|定期6个月百分比|定期1年百分比|定期2年百分比|定期3年百分比|定期5年百分比|通知1天百分比|通知7天百分比|登记日期|到期日期|续约标志|流水号|柜员|状态|
        /// </summary>
        public class E_QYMX
        {
            /// <summary>
            /// 账号
            /// </summary>
            public string ZH { get; set; }

            /// <summary>
            /// 序号
            /// </summary>
            public string XH { get; set; }

            /// <summary>
            /// 协议
            /// </summary>
            public string XY { get; set; }

            /// <summary>
            /// 登记机构
            /// </summary>
            public string DJJG { get; set; }

            /// <summary>
            /// 主账号
            /// </summary>
            public string ZZH { get; set; }

            /// <summary>
            /// 主账号序号
            /// </summary>
            public string ZZHXH { get; set; }

            /// <summary>
            /// 留存金额
            /// </summary>
            public string LCJE { get; set; }

            /// <summary>
            /// 划转金额
            /// </summary>
            public string HZJE { get; set; }

            /// <summary>
            /// 定期3个月百分比
            /// </summary>
            public string DQ3GYBFB { get; set; }

            /// <summary>
            /// 定期6个月百分比
            /// </summary>
            public string DQ6GYBFB { get; set; }

            /// <summary>
            /// 定期1年百分比
            /// </summary>
            public string DQ1NBFB { get; set; }

            /// <summary>
            /// 定期2年百分比
            /// </summary>
            public string DQ2NBFB { get; set; }

            /// <summary>
            /// 定期3年百分比
            /// </summary>
            public string DQ3NBFB { get; set; }

            /// <summary>
            /// 定期5年百分比
            /// </summary>
            public string DQ5NBFB { get; set; }

            /// <summary>
            /// 通知1天百分比
            /// </summary>
            public string TZ1TBFB { get; set; }

            /// <summary>
            /// 通知7天百分比
            /// </summary>
            public string TZ7TBFB { get; set; }

            /// <summary>
            /// 登记日期
            /// </summary>
            public string DJRQ { get; set; }

            /// <summary>
            /// 到期日期
            /// </summary>
            public string DQRQ { get; set; }

            /// <summary>
            /// 续约标志
            /// </summary>
            public string XYBZ { get; set; }

            /// <summary>
            /// 流水号
            /// </summary>
            public string LSH { get; set; }

            /// <summary>
            /// 柜员
            /// </summary>
            public string GY { get; set; }

            /// <summary>
            /// 状态
            /// </summary>
            public string ZT { get; set; }

        }

        #endregion

        #region Input
		/// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 帐号 1021
            ///  
            /// </summary>
            public string ZH { get; set; }

            /// <summary>
            /// 协议类型 1034
            ///  
            /// </summary>
            public string XYLX { get; set; }


            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new QueryPaymentAgreementRecords();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
				MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "CXCL";
                MessageData.Fd102_1 = ZH;
                MessageData.Fd103_4 = XYLX;
                MessageData.Fd128 = "2716";

                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }                
                output.QYMXData = AnilyzeFileData<E_QYMX>(MessageData.FileData,
                                            "账号|序号|协议|登记机构|主账号|主账号序号|留存金额|划转金额|定期3个月百分比|定期6个月百分比|定期1年百分比|定期2年百分比|定期3年百分比|定期5年百分比|通知1天百分比|通知7天百分比|登记日期|到期日期|续约标志|流水号|柜员|状态|",
                                            x =>
                                            {
                                                return new E_QYMX
                                                {
                                                    ZH = x["账号"],
                                                    XH = x["序号"],
                                                    XY = x["协议"],
                                                    DJJG = x["登记机构"],
                                                    ZZH = x["主账号"],
                                                    ZZHXH = x["主账号序号"],
                                                    LCJE = x["留存金额"],
                                                    HZJE = x["划转金额"],
                                                    DQ3GYBFB = x["定期3个月百分比"],
                                                    DQ6GYBFB = x["定期6个月百分比"],
                                                    DQ1NBFB = x["定期1年百分比"],
                                                    DQ2NBFB = x["定期2年百分比"],
                                                    DQ3NBFB = x["定期3年百分比"],
                                                    DQ5NBFB = x["定期5年百分比"],
                                                    TZ1TBFB = x["通知1天百分比"],
                                                    TZ7TBFB = x["通知7天百分比"],
                                                    DJRQ = x["登记日期"],
                                                    DQRQ = x["到期日期"],
                                                    XYBZ = x["续约标志"],
                                                    LSH = x["流水号"],
                                                    GY = x["柜员"],
                                                    ZT = x["状态"]
                                                };
                                            });
                output.QYMX = MessageData.FileData;

                return output;
            }
        }
        #endregion
    }
}
