using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 查询产品存期
    /// 通过定期产品类型查询产品存期
    /// </summary>
    public class QueryProductPeriod : TFOutput
    {
        #region Output
        /// <summary>
        /// 产品存期
        /// 文件格式：存期(大写)+存期单位(汉字)|存期(数字)+存期单位(字母)|
        /// </summary>
        public string CPCQ { get; internal set; }

        /// <summary>
        /// 产品存期
        /// 文件格式：存期(大写)+存期单位(汉字)|存期(数字)+存期单位(字母)|
        /// </summary>
        public List<E_CPCQ> CPCQData { get; internal set; }

        /// <summary>
        /// 产品存期
        /// 文件格式：存期(大写)+存期单位(汉字)|存期(数字)+存期单位(字母)|
        /// </summary>
        public class E_CPCQ
        {
            /// <summary>
            /// 存期描述
            /// </summary>
            public string CQMS { get; set; }

            /// <summary>
            /// 存期值
            /// </summary>
            public string CQZ { get; set; }

        }

        #endregion

        #region Input
		/// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 币种 0210
            /// 01人民币 
            /// </summary>
            public string BZ { get; set; }

            /// <summary>
            /// 产品介质代码 0600
            /// 01人民币 
            /// </summary>
            public string JZDM { get; set; }

            /// <summary>
            /// 定期产品类型 0670
            /// 0：整存整取； 1：零存整取； 2：整存零取； 3：存本取息； 4：通知存款  5：协议存款； 6：定活两便；7：教育储蓄 A: 利润最大化通知存款  
            /// </summary>
            public string CPLX { get; set; }

            /// <summary>
            /// 客户类型 0680
            /// 1个人 2对公 
            /// </summary>
            public string KHLX { get; set; }


            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new QueryProductPeriod();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
				MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "K043";
                MessageData.Fd21 = BZ;
                MessageData.Fd60 = JZDM;
                MessageData.Fd67 = CPLX;
                MessageData.Fd68 = KHLX;

                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }                
                output.CPCQData = AnilyzeFileData<E_CPCQ>(MessageData.FileData,
                                            "存期描述|存期值|",
                                            x =>
                                            {
                                                return new E_CPCQ
                                                {
                                                    CQMS = x["存期描述"],
                                                    CQZ = x["存期值"]
                                                };
                                            });
                output.CPCQ = MessageData.FileData;

                return output;
            }
        }
        #endregion
    }
}
