using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 产品与产品关系表查询-产品一
    /// 根据委托存款类型查询并返回存款列表
    /// </summary>
    public class QueryProductRelations_Pro1 : TFOutput
    {
        #region Output
        /// <summary>
        /// 产品列表
        /// 文件格式： 产品号|产品描述|\n
        /// </summary>
        public string CPLB { get; internal set; }

        /// <summary>
        /// 产品列表
        /// 文件格式： 产品号|产品描述|\n
        /// </summary>
        public List<E_CPLB> CPLBData { get; internal set; }

        /// <summary>
        /// 产品列表
        /// 文件格式： 产品号|产品描述|\n
        /// </summary>
        public class E_CPLB
        {
            /// <summary>
            /// 产品号
            /// </summary>
            public string CPH { get; set; }

            /// <summary>
            /// 产品描述
            /// </summary>
            public string CPMS { get; set; }

        }

        #endregion

        #region Input
		/// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 客户类型 0720
            /// 0：公用；1：个人；2：公司；3：机构（指政府机关等组织）；4：金融同业；5：内部员工；6金融机构 
            /// </summary>
            public string KHLX { get; set; }

            /// <summary>
            /// 关系类型 0930
            /// 1-存款与贷款产品关系，2-存款与收息户关系 
            /// </summary>
            public string GXLX { get; set; }


            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new QueryProductRelations_Pro1();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
				MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "CXCL";
                MessageData.Fd72 = KHLX;
                MessageData.Fd93 = GXLX;
                MessageData.Fd128 = "0015";

                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }                
                output.CPLBData = AnilyzeFileData<E_CPLB>(MessageData.FileData,
                                            "产品号|产品描述|",
                                            x =>
                                            {
                                                return new E_CPLB
                                                {
                                                    CPH = x["产品号"],
                                                    CPMS = x["产品描述"]
                                                };
                                            });
                output.CPLB = MessageData.FileData;

                return output;
            }
        }
        #endregion
    }
}
