using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TellerSystem.Contract.Function
{
    /// <summary>
    /// 待入库卡凭证查询
    /// 根据凭证类别-卡和制卡批次回显卡凭证信息列表
    /// </summary>
    public class QueryStoragingCard : TFOutput
    {
        #region Output
        /// <summary>
        /// 卡凭证种类列表
        /// 后台返回文件，内容包括：申请日期|申请柜员|卡种|数量|起始号码|终止号码|
        /// </summary>
        public string KPZZLLB { get; internal set; }

        /// <summary>
        /// 卡凭证种类列表
        /// 后台返回文件，内容包括：申请日期|申请柜员|卡种|数量|起始号码|终止号码|
        /// </summary>
        public List<E_KPZZLLB> KPZZLLBData { get; internal set; }

        /// <summary>
        /// 卡凭证种类列表
        /// 后台返回文件，内容包括：申请日期|申请柜员|卡种|数量|起始号码|终止号码|
        /// </summary>
        public class E_KPZZLLB
        {
            /// <summary>
            /// 申请日期
            /// </summary>
            public string SQRQ { get; set; }

            /// <summary>
            /// 申请柜员
            /// </summary>
            public string SQGY { get; set; }

            /// <summary>
            /// 卡种
            /// </summary>
            public string KZ { get; set; }

            /// <summary>
            /// 数量
            /// </summary>
            public string SL { get; set; }

            /// <summary>
            /// 起始号码
            /// </summary>
            public string QSHM { get; set; }

            /// <summary>
            /// 终止号码
            /// </summary>
            public string ZZHM { get; set; }

        }

        #endregion

        #region Input
		/// <summary>
        /// 参数清单
        /// </summary>
        public class Input : TFInput
        {
            /// <summary>
            /// 制卡批次号 0630
            /// (null) 
            /// </summary>
            public string ZKPCH { get; set; }


            /// <summary>                 
            /// 定义业务逻辑              
            /// </summary>                
            /// <returns></returns>  
            public override TFOutput Raise()
            {
                //拉取数据                                  
                var output = new QueryStoragingCard();
                PageData.TradeAPIs.InitFd(MessageData);
                PageData.TradeAPIs.CommSend(MessageData);
				MessageData.Fd126_5 = MainCode;
                MessageData.Fd16 = "CXCL";
                MessageData.Fd63 = ZKPCH;
                MessageData.Fd128 = "0003";

                PageData.TradeAPIs.CallServer(MessageData);
                if (MessageData.Fd12 != "0000")
                {
                    output.IsFailed = true;
                    output.FailureCause = PageData.TradeAPIs.GetError(MessageData);
                }                
                output.KPZZLLBData = AnilyzeFileData<E_KPZZLLB>(MessageData.FileData,
                                            "申请日期|申请柜员|卡种|数量|起始号码|终止号码|",
                                            x =>
                                            {
                                                return new E_KPZZLLB
                                                {
                                                    SQRQ = x["申请日期"],
                                                    SQGY = x["申请柜员"],
                                                    KZ = x["卡种"],
                                                    SL = x["数量"],
                                                    QSHM = x["起始号码"],
                                                    ZZHM = x["终止号码"]
                                                };
                                            });
                output.KPZZLLB = MessageData.FileData;

                return output;
            }
        }
        #endregion
    }
}
