﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.IO;
using System.Collections.ObjectModel;
using TellerSystem.Library.Ext.Function;
using Platform.Common.RunningParameters;
using AForge.Video.DirectShow;
using System.Drawing;
using AForge.Video;
using System.Drawing.Imaging;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using AForge.Controls;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// CamPlayer.xaml 的交互逻辑
    /// </summary>
    public partial class CamPlayer
    {
        public CamPlayer(string DeviceType)
        {
            InitializeComponent();
            DefaultDevice = DeviceType;
            InitPage();
        }

        /// <summary>
        /// 依次为标识，VID，PID
        /// </summary>
        private Dictionary<string, Tuple<string, string>> DevicesTagDit = new Dictionary<string, Tuple<string, string>>
        {
           {"PZ",new Tuple<string,string>("0ac8","3370")},
           {"KH",new Tuple<string,string>("0c45","6340")},
           {"GY",new Tuple<string,string>("c456","6341")},
        };
        /// <summary>
        /// 版本控制标志
        /// </summary>
        private string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
        /// <summary>
        /// 是否是拍照产生的图片
        /// </summary>
        bool isPhoto = false;

        Window win = null;

        ObservableCollection<MediaParaEntry> collect = new ObservableCollection<MediaParaEntry>();

        /// <summary>
        /// 
        /// </summary>
        public string DefaultDevice = "KH";

        /// <summary>
        /// 
        /// </summary>
        public DHCPhoto Photo = new DHCPhoto();


        VideoCapabilities getVideoCapabilities(VideoCapabilities[] videos)
        {
            for (int i = 0; i < videos.Length; i++)
            {
                var item = videos[i];
                if (flag == "V2")
                {
                    //鄢陵
                    if (DefaultDevice == "KH" || DefaultDevice == "GY")
                    {
                        if (item.FrameSize.Width == 1280 && item.FrameSize.Height == 960)
                        {
                            return item;
                        }
                    }
                    //凭证和补录
                    else
                    {
                        if (item.FrameSize.Width == 1600 && item.FrameSize.Height == 1200)
                        {
                            return item;
                        }
                    }
                }
                //阳光远程授权
                if (flag == "V8")
                {
                    if (item.FrameSize.Width == 1280 && item.FrameSize.Height == 960)
                    {
                        return item; 
                    }
                    if (item.FrameSize.Width == 1280 && item.FrameSize.Height == 1024)
                    {
                        return item;
                    }
                }
                else if (flag == "V21")
                {
                    if (DefaultDevice == "KH" || DefaultDevice == "GY")
                    {
                        if (item.FrameSize.Width == 1280 && item.FrameSize.Height == 960)
                        {
                            return item;
                        }
                    }
                    //凭证和补录
                    else
                    {
                        if (item.FrameSize.Width == 2592 && item.FrameSize.Height == 1944)
                        {
                            return item;
                        }
                        //if (flag == "V21")
                        //{
                            var item1 = 0;
                            for (int j = 0; j < videos.Length; j++)
                            {
                                if (DefaultDevice != "KH" && DefaultDevice != "GY")
                                {
                                    if (item1 < videos[j].FrameSize.Width)
                                    {
                                        item1 = videos[j].FrameSize.Width;
                                    }
                                }
                            }
                            for (int j = 0; j < videos.Length; j++)
                            {
                                if (DefaultDevice != "KH" && DefaultDevice != "GY")
                                {
                                    if (videos[j].FrameSize.Width == item1)
                                    {
                                        return videos[j];
                                    }
                                }
                            }
                        //}
                    }
                }
                else
                {
                    //客户和柜员 
                    if (DefaultDevice == "KH" || DefaultDevice == "GY")
                    {
                        if (item.FrameSize.Width == 1280 && item.FrameSize.Height == 720)
                        {
                            return item; 
                        }
                    }
                    //凭证和补录
                    else
                    {
                        if (item.FrameSize.Width == 1920 && item.FrameSize.Height == 1080)
                        {
                            return item; 
                        }
                    }
                }

            }
            return videos[videos.Length - 1];
        }

        void openCamera(FilterInfo info)
        {
            var videoCapture = new VideoCaptureDevice(info.MonikerString);
            videoCapture.VideoResolution = getVideoCapabilities(videoCapture.VideoCapabilities);
            setHostWH(videoCapture.VideoResolution.FrameSize);
            vsp.VideoSource = videoCapture;
            vsp.NewFrame += new VideoSourcePlayer.NewFrameHandler(this.Vsp_NewFrame);
            vsp.Start();
        }

        //图像缓存
        private Bitmap bmp;
        private void Vsp_NewFrame(object sender, ref Bitmap image)
        {
            bmp = image;
        }

        public BitmapImage BitmapToBitmapImage(Bitmap bitmap)
        {
            using (MemoryStream stream = new MemoryStream())
            {
                Bitmap bm2 = new Bitmap(bitmap.Width, bitmap.Height);
                Graphics g = Graphics.FromImage(bm2);
                //重新复制bitmap--解决GDI+
                g.DrawImageUnscaled(bitmap, 0, 0);
                g.Dispose();
                bm2.Save(stream, ImageFormat.Png); // 坑点：格式选Bmp时，不带透明度

                stream.Position = 0;
                BitmapImage result = new BitmapImage();
                result.BeginInit();
                result.CacheOption = BitmapCacheOption.OnLoad;
                result.StreamSource = stream;
                result.EndInit();
                result.Freeze();
                return result;
            }
        }


        public void closeCamera()
        {
            vsp.NewFrame -= Vsp_NewFrame;
            if (vsp.IsRunning)
            {   // 停止视频
                vsp.SignalToStop();
                vsp.WaitForStop();
            }
        }


        FilterInfo getDefaultFilter(FilterInfoCollection devices)
        {
            string devTag = DefaultDevice;
            if (devTag == "BL")
            {
                devTag = "PZ";
            }
            FilterInfo result = null;
            String vid = "vid_" + DevicesTagDit[devTag].Item1;
            String pid = "pid_" + DevicesTagDit[devTag].Item2;
            for (int i = 0; i < devices.Count; i++)
            {
                var item = devices[i];
                if (item.MonikerString.Contains(vid) && item.MonikerString.Contains(pid))
                {
                    result = item;
                }
            }
            if (result == null)
            {
                result = devices[0];
            }
            return result;
        }


        public bool InitPage()
        {
            if (ConfigManager.GetInstance().GetConfigValue("CameraEnabled", ConfigType.User).ToLower() == "true")
            {
                DevicesTagDit.Clear();
                DevicesTagDit.Add("PZ", new Tuple<string, string>(ConfigManager.GetInstance().GetConfigValue("PZVid", ConfigType.User), ConfigManager.GetInstance().GetConfigValue("PZPid", ConfigType.User)));
                DevicesTagDit.Add("KH", new Tuple<string, string>(ConfigManager.GetInstance().GetConfigValue("KHVid", ConfigType.User), ConfigManager.GetInstance().GetConfigValue("KHPid", ConfigType.User)));
                DevicesTagDit.Add("GY", new Tuple<string, string>(ConfigManager.GetInstance().GetConfigValue("GYVid", ConfigType.User), ConfigManager.GetInstance().GetConfigValue("GYPid", ConfigType.User)));

            }
            var devices = new FilterInfoCollection(FilterCategory.VideoInputDevice);
            if (devices == null || devices.Count < 1)
            {
                MessageBox.Show("请检查摄像头是否连接");
                win.Close();
                return false;
            }
            FilterInfo dev = getDefaultFilter(devices);
            for (int i = 0; i < devices.Count; i++)
            {
                var item = devices[i];
                collect.Add(new MediaParaEntry() { Text = item.Name, Device = item, IsRun = dev == item });
            }
            buttons.ItemsSource = collect;
            openCamera(dev);
            return true;
        }

        //modify by hulei   20190409   新增 window 关闭界面时 关掉摄像头，以防被占用 start
        void CamPlayer_Loaded(object sender, RoutedEventArgs e)
        {
            win = this.FindVisualTreeParent(x => x is Window) as Window;
            var buttonGroup = win.FirstVisualTreeChildren(x => x is StackPanel && (x as StackPanel).Name == "buttonsStackPanel") as StackPanel;
            if (buttonGroup.Name == "buttonsStackPanel")
            {
                buttonGroup.Visibility = Visibility.Collapsed;
            }
            var image = win.FirstVisualTreeChildren(x => x is FrameworkElement && (x as FrameworkElement).Name == "image") as FrameworkElement;
            if (image.Name == "image")
            {
                image.Visibility = Visibility.Collapsed;
            }
            win.Closed += new EventHandler(win_Closed);
        }
        void win_Closed(object sender, EventArgs e)
        {
            closeCamera();
        }
        //modify by hulei   20190409   新增 window 关闭界面时 关掉摄像头，以防被占用 end 

        /// <summary>
        /// 拍照
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void photoBtn_Click(object sender, RoutedEventArgs e)
        {
            isPhoto = true;
            if (bmp == null)
            {
                MessageBox.Show("请重新点击拍照！");
            }
            Photo.Source = BitmapToBitmapImage(bmp);
            closeCamera();
            win.Close();
        }

        private void RadioButton_Checked(object sender, RoutedEventArgs e)
        {
            RadioButton rd = sender as RadioButton;
            MediaParaEntry entry = rd.Tag as MediaParaEntry;
            closeCamera();
            openCamera(entry.Device);
        }

        private void cancelBtn_Click(object sender, RoutedEventArgs e)
        {
            isPhoto = false;
            closeCamera();
            win.Close();
        }


        void setHostWH(System.Drawing.Size frameSize)
        {
            double rWidth;
            double rHeight;
            double pWidth = 780;
            double pHeight = 540;
            double wRate = (double)frameSize.Width / pWidth;
            double hRate = (double)frameSize.Height / pHeight;
            if (wRate > 1 || hRate > 1)
            {
                if (wRate > hRate)
                {
                    //以宽标准缩放
                    rWidth = pWidth;
                    rHeight = pWidth / frameSize.Width * frameSize.Height;
                }
                else
                {
                    rWidth = pHeight / frameSize.Height * frameSize.Width;
                    rHeight = pHeight;
                }
            }
            else
            {
                rWidth = (double)frameSize.Width;
                rHeight = (double)frameSize.Height;
            }
            host.Width = rWidth;
            host.Height = rHeight;
        }

    }

    public class DHCPhoto : INotifyPropertyChanged
    {
        BitmapSource source = null;
        public BitmapSource Source
        {
            get { return source; }
            set
            {
                source = value;
                data = null;
                Notify("Source");
            }
        }
        string devicetype = "KH";
        public string DeviceType
        {

            get { return devicetype; }
            set
            {
                devicetype = value;
                Notify("DeviceType");
            }

        }


        byte[] data = null;
        int qualityLevel = 100;
        public byte[] Data
        {
            get
            {

                if (data == null && Source != null)
                {
                    if (DeviceType == "KH")
                    {
                        qualityLevel = 30;
                    }
                    else if (DeviceType == "GY")
                    {
                        qualityLevel = 30;
                    }
                    else
                    {
                        string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
                        if (FLAG == "V21")
                        {
                            qualityLevel = 30;
                        }
                        else
                        {
                            qualityLevel = 80;
                        }
                    }
                    using (var ms = new MemoryStream())
                    {
                        BitmapEncoder encoder = new JpegBitmapEncoder() { QualityLevel = qualityLevel };
                        encoder.Frames.Add(BitmapFrame.Create(source as BitmapSource));
                        encoder.Save(ms);
                        ms.Flush();
                        data = ms.ToArray();
                        ms.Close();
                    }
                }
                return data;
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        void Notify(string propName)
        {
            if (this.PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propName));
            }
        }
    }

    public class MediaParaEntry : INotifyPropertyChanged
    {
        public string text = string.Empty;
        /// <summary>
        /// 
        /// </summary>
        public string Text
        {
            get { return text; }
            set { text = value; Notify("Text"); }
        }

        private FilterInfo device;

        public FilterInfo Device
        {
            get { return device; }
            set { device = value; Notify("Device"); }
        }

        private bool isRun = false;
        /// <summary>
        /// 
        /// </summary>
        public bool IsRun
        {
            get { return isRun; }
            set { isRun = value; Notify("IsRun"); }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        void Notify(string propName)
        {
            if (this.PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propName));
            }
        }
    }


}
