﻿/*****************************************************************************************
系统名     综合业务系统2.0
程序名     交易2301的源代码
类名       T2301      
概要   	   客户信息维护
作成者     XXX
作成日     
履历       No.        日期          Ver.No       修改人          描述
*****************************************************************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using AxDOCCAMERAOCXLib;
using Platform.Library;
using Platform.Presentation.PageFunctions;
using TellerSystem.Library.Ext.Function;
using Application = System.Windows.Application;
using TellerSystem.Library.Ext.TradeExtension;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// CameraViewer.xaml 的交互逻辑
    /// </summary>
    [DesignTimeVisible(false)]
    public partial class CameraViewer:TradePage
    {
        private bool _cutBorder = false;
        public CameraViewer(bool cutBorderFlag = false)
        {
            InitializeComponent();
            _cutBorder = cutBorderFlag;
        }

        //摄像头
        private AxDoccameraOcx myCamera = new AxDoccameraOcx();
        //图像数据
        private byte[] imageData = null;

        #region  WPF调用OCX

        /// <summary>
        /// OCX初始化
        /// </summary>
        private void InitOcx()
        {
            myCamera.Width = 533;
            myCamera.Height = 400;
            Host.Child = myCamera;
            myCamera.EndInit();
        }

        #endregion

        #region 操作事件


        /// <summary>
        /// 打开摄像头
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOpen_Click(object sender, RoutedEventArgs e)
        {
            //清空上一次采集到的图像
            imageData = null;

            //保存图片
            if (myCamera.bSaveBMP(AppDomain.CurrentDomain.BaseDirectory, "TBMPimage"))
            {
                //this.ShowMessageBox("成功采集图像！");
                FileStream aFileStream = new FileStream(AppDomain.CurrentDomain.BaseDirectory + "TBMPimage.BMP", FileMode.Open);
                imageData = new byte[aFileStream.Length];
                aFileStream.Read(imageData, 0, imageData.Length);
                aFileStream.Close();
            }
            else
            {
                this.ShowMessageBox("采集图像失败！");
            }

            if (_cutBorder)
            {
                imageData = ImageCutHelper.CutIDCARDBorder(imageData);
            }
            if (imageData != null)
            {
                var imageSourceConverter = new ImageSourceConverter();
                smallImage.Source = imageSourceConverter.ConvertFrom(new MemoryStream(imageData)) as ImageSource;
                smallImage.ToolTip = new System.Windows.Controls.ToolTip()
                                         {
                                             MaxWidth = 533,
                                             MinWidth = 400,
                                             Content = new Image { Source = smallImage.Source, Stretch = Stretch.Uniform }
                                         };
            }
        }

        /// <summary>
        /// 保存
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, RoutedEventArgs e)
        {
            //关闭
            if (myCamera != null)
            {
                myCamera.bStopPlay();
            }

            if (imageData == null)
            {
                this.ShowMessageBox("没有采集到数据，请重新拍照！");
                return;
            }
            else
            {
                Dictionary<string, object> dic = new Dictionary<string, object>();
                dic.Add("ImageData", imageData);
                this.SetViewStateData(dic);
                this.OwnerContainer.ClosePage();
            }
        }

        /// <summary>
        /// 取消操作
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            //关闭
            if (myCamera != null)
            {
                myCamera.bStopPlay();
            }

            this.Close();
        }

        /// <summary>
        /// 浏览选择文件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBrowse_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new Microsoft.Win32.OpenFileDialog();
            dialog.ShowDialog(Application.Current.MainWindow);
            if (string.IsNullOrEmpty(dialog.FileName)) return;
            imageData = File.ReadAllBytes(dialog.FileName);
        }

        #endregion

        /// <summary>
        /// 初始并打开摄像头
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TradePage_Loaded(object sender, RoutedEventArgs e)
        {
            InitOcx();

            myCamera.vSetDelHBFlag(true);
            //打开摄像头
            if (!myCamera.bStartPlay())
            {
                //TradeLogger.Error("打开摄像头失败!");
                this.Close();
            }
        }

        /// <summary>
        /// 关闭摄像头
        /// </summary>
        public void CloseCamera()
        {
            //关闭
            if (myCamera != null)
            {
                myCamera.bStopPlay();
            }
        }

    }
}
