﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.IO;
using Platform.Controls.ControlsHelper;
using Platform.Library;
using TellerSystem.Communication;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using System.Collections.Generic;
using Platform.Common.RunningParameters;

/******************************平台Dll*****************************/

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// CheckSealVerification.xaml 的交互逻辑
    /// 鑫万佳的验印对接
    /// </summary>
    public partial class CheckSealVerification
    {
        #region Ctor
        public CheckSealVerification()
        {
            InitializeComponent();
            Record = new YYCheckerResult();
            Record.IsOnline = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("Is_CheckYY") == "true";
        }
        string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
        private void TradePage_Loaded(object sender, RoutedEventArgs e)
        {
            cbb_Result.SetItems(new Dictionary<string, string> { { "验印通过", "000" }, { "验印未过", "001" }, { "尚无结果", "999" }, { "验印系统Down机", "301" } });
            tb_yyls.Text = LoginUserInfo.TradeDate;
            this.SetFocus(tb_yyls);
            tb_yyls.Select(tb_yyls.Text.Length, 0);
        }
        #endregion

        #region Property

        /// <summary>
        /// 验印结果
        /// </summary>
        public YYCheckerResult Record { get; private set; }
        #endregion

       
        /// <summary>
        /// 解析图像数据
        /// </summary>
        /// <param name="imageStream"></param>
        private ImageSource CreateImage(String imageStream)
        {
            ImageSource ret = null;
            try
            {
                if (!string.IsNullOrEmpty(imageStream))
                {
                    //把BASE64转码字符串，转成byte数组
                    byte[] arr = Convert.FromBase64String(imageStream);
                    //定义流
                    MemoryStream ms = new MemoryStream(arr);
                    var bitImg = new BitmapImage();
                    bitImg.BeginInit();
                    //ms.Seek(0, SeekOrigin.Begin);
                    bitImg.StreamSource = ms;
                    bitImg.EndInit();
                    ret = bitImg;
                }
            }
            catch (Exception e)
            {
                this.ShowWarnMessageBox("加载图片数据出错！");
            }
            return ret;
        }

        #region Event

        /// <summary>
        /// 验印核查
        /// </summary>
        private void btnCheck_Click(object sender, RoutedEventArgs e)
        {
            Message message = new Message(Message.BitMap.Agn);
            this.InitFd(message);
            this.CommSend(message);
            message.Fd16 = "2001";
            message.Fd18 = "DZYY";
            message.Fd37 = tb_yyls.Text;

            this.TradeAPIs.CallAgn(message);
            if (message.Fd12 != "0000")
            {
                this.TradeAPIs.ShowInfoMessageBox(message.Fd74);
                this.TradeAPIs.SetFocus();
                return;
            }
            cbb_Result.SetSelectedValue(message.Fd47.Trim());
            Record.ZH = tb_ZH.Text = message.Fd38.Trim();
            Record.PZNO = tb_PZNO.Text = message.Fd44.Trim();
            Record.JE = tb_JE.Text = message.Fd39.Trim().ToDecimal().ToString();
            Record.Type = tb_PZTYPE.Text = message.Fd82.Trim();
            Record.CPRQ = tb_CPRQ.Text = message.Fd46.Trim();
            Record.YYLS = tb_yyls.Text.Trim();
            Record.IsComplated = cbb_Result.GetSelectedValue() == "000" ;
            
            string _result = cbb_Result.GetSelectedValue();
            switch(_result)
            {
                case "000":
                case "001":
                    imgPhoto.Source = CreateImage(message.FileData);
                    this.TradeAPIs.SetFocus(btnView);
                    break;
                case "999":
                    this.TradeAPIs.ShowInfoMessageBox("查询尚无结果！");
                    this.TradeAPIs.SetFocus();
                    break;
                case "301":
                    this.TradeAPIs.ShowInfoMessageBox("验印系统Down机！");
                    this.TradeAPIs.SetFocus();
                    break;
            }
        }


        private void btnBack_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
        #endregion

        private void btnView_Click(object sender, RoutedEventArgs e)
        {
            view detail = new view();
            detail.img.Source = imgPhoto.Source;
            if (detail.img.Source != null)
            {
                this.ShowPage(detail, "详细凭证");
            }
            else 
            {
                this.TradeAPIs.ShowInfoMessageBox("暂无图像，无法查看！");
                this.TradeAPIs.SetFocus();
            }
        }

    }

    /// <summary>
    /// 核查记录
    /// </summary>
    public class YYCheckerResult
    {
        /// <summary>
        /// 是否验印
        /// </summary>
        public bool IsOnline { get; internal set; }

        /// <summary>
        /// 是否完成验印 
        /// </summary>
        public bool IsComplated { get; internal set; }

        /// <summary>
        /// 账号
        /// </summary>
        public string ZH { get; internal set; }

        /// <summary>
        /// 金额
        /// </summary>
        public string JE { get; internal set; }

        /// <summary>
        /// 凭证种类
        /// </summary>
        public string Type { get; internal set; }

        /// <summary>
        /// 凭证号码
        /// </summary>
        public string PZNO { get; internal set; }

        /// <summary>
        /// 出票日期
        /// </summary>
        public string CPRQ { get; internal set; }

        /// <summary>
        /// 验印流水
        /// </summary>
        public string YYLS { get; internal set; }

    }
}