﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Globalization;
using Platform.Presentation.FocusManagers;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// DateTextBox.xaml 的交互逻辑
    /// </summary>
    public partial class DateInputControl : UserControl
    {
        private Platform.Presentation.FocusManagers.FocusManager focus;
        public DateInputControl()
        {
            InitializeComponent();
            this.datePicker.DateValidationError += new EventHandler<DatePickerDateValidationErrorEventArgs>(datePicker_DateValidationError);
            this.Loaded += (sender, e) =>
            {
                if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this)) return;
                focus = Platform.Presentation.FocusManagers.FocusManager.GetManagerInstance((DependencyObject)this) as Platform.Presentation.FocusManagers.FocusManager;
                if (focus == null) throw new ArgumentNullException("无法找到焦点控制模块！");
            };
            Platform.Presentation.FocusManagers.FocusManager.AddPreviewGotFocusHandler(this, PreviewGotFocus);
            Platform.Presentation.FocusManagers.FocusManager.AddPreviewLostFocusHandler(this, PreviewLostFocus);
            this.PreviewKeyDown += new System.Windows.Input.KeyEventHandler(DateInputControl_PreviewKeyDown);
        }


        public DateTime? Date
        {
            get { return this.datePicker.SelectedDate; }
            set { this.datePicker.SelectedDate = value; }
        }

        /// <summary>
        /// 输入日期
        /// </summary>
        public string DateValue
        {

            get { return Date.HasValue ? Date.Value.ToString("yyyyMMdd") : string.Empty; }
            set
            {
                int year, month, day;
                DateTime date;
                if (value.Length == 8
                    && int.TryParse(value.Substring(0, 4), out year)
                    && int.TryParse(value.Substring(4, 2), out month)
                    && int.TryParse(value.Substring(6, 2), out day)
                    && DateTime.TryParse(string.Format("{0}/{1}/{2}", year, month, day), out date))
                    Date = date;
                else
                    Date = null;
            }
        }

        void datePicker_DateValidationError(object sender, DatePickerDateValidationErrorEventArgs e)
        {
            var datePicker = sender as DatePicker;
            if (datePicker != null)
            {
                var text = e.Text;
                DateTime dateTime;
                if (DateTime.TryParseExact(text, "yyyyMMdd", CultureInfo.CurrentUICulture, DateTimeStyles.None, out dateTime))
                {
                    datePicker.SelectedDate = dateTime;
                    Date = dateTime;
                }
                else
                {
                    datePicker.SelectedDate = null;
                    Date = null;
                }
            }
        }

        void DateInputControl_PreviewKeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {

            if (e.Key == Key.Up)
            {
                datePicker.Focus(); 
                focus.MoveFocus(FocusNavigation.Backward);
            }
            else if (e.Key == Key.Down || e.Key == Key.Tab || e.Key == Key.Enter)
            {
                datePicker.Focus(); 
                focus.MoveFocus(FocusNavigation.Forward);
            }
            else if (e.Key == Key.Left)
            {
                datePicker.Focus(); 
                focus.MoveFocus(FocusNavigation.Backward);
            }
            else if (e.Key == Key.Right)
            {
                datePicker.Focus();   
                focus.MoveFocus(FocusNavigation.Forward);
            }
        }

        private void PreviewGotFocus(object sender, RoutedEventArgs e)
        {
            datePicker.Focus();        
        }

        private void PreviewLostFocus(object sender, RoutedEventArgs e)
        {
            datePicker.Focus();
        }


    }
}
