﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Presentation.FocusManagers;
using Platform.Controls.ControlsHelper;
using Platform.Presentation.Interfaces;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// DateSpanControl.xaml 的交互逻辑
    /// </summary>
    public partial class DateSpanControl : UserControl
    {

        #region 事件
        public delegate void DateLostFocusHandle(TextBox beginBox,TextBox endBox);
        public event DateLostFocusHandle BeginDateLostFocus;
        public event DateLostFocusHandle EndDateLostFocus;
        //public static readonly RoutedEvent BeginDateLostFocusEvent = EventManager.RegisterRoutedEvent("BeginDateLostFocus", RoutingStrategy.Direct, typeof(RoutedEventHandler), typeof(DateSpanControl));

        //public event RoutedEventHandler BeginDateLostFocus
        //{
        //    add { this.AddHandler(BeginDateLostFocusEvent,value);}
        //    remove { this.RemoveHandler(BeginDateLostFocusEvent, value); }
        //}

        //public static readonly RoutedEvent EndDateLostFocusEvent = EventManager.RegisterRoutedEvent("EndDateLostFocus", RoutingStrategy.Direct, typeof(RoutedEventHandler), typeof(DateSpanControl));

        //public event RoutedEventHandler EndDateLostFocus
        //{
        //    add { this.AddHandler(EndDateLostFocusEvent, value); }
        //    remove { this.RemoveHandler(EndDateLostFocusEvent, value); }
        //}
        #endregion

        #region 属性
        /// <summary>
        /// 设置或获取时间值的必输性，true为必输
        /// </summary>
        public bool IsNotEmpty
        {
            get { return (bool)GetValue(IsNotEmptyProperty); }
            set { SetValue(IsNotEmptyProperty, value); }
        }

        public static readonly DependencyProperty IsNotEmptyProperty =
            DependencyProperty.Register("IsNotEmpty", typeof(bool), typeof(DateSpanControl), new UIPropertyMetadata(false, new PropertyChangedCallback(IsNotEmptyChangedCallback)));



        public string BeginDate
        {
            get { return (string)GetValue(BeginDateProperty); }
            set { SetValue(BeginDateProperty, value); }
        }


        public static readonly DependencyProperty BeginDateProperty =
            DependencyProperty.Register("BeginDate", typeof(string), typeof(DateSpanControl), new UIPropertyMetadata(""));



        public string EndDate
        {
            get { return (string)GetValue(EndDateProperty); }
            set { SetValue(EndDateProperty, value); }
        }

        public static readonly DependencyProperty EndDateProperty =
            DependencyProperty.Register("EndDate", typeof(string), typeof(DateSpanControl), new UIPropertyMetadata(""));



        public bool IsVerifiy
        {
            get { return (bool)GetValue(IsVerifiyProperty); }
            set { SetValue(IsVerifiyProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsVerifiy.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsVerifiyProperty =
            DependencyProperty.Register("IsVerifiy", typeof(bool), typeof(DateSpanControl), new UIPropertyMetadata(true));


        
        #endregion

        #region 方法
        public DateSpanControl()
        {
            InitializeComponent();
        }

        static void IsNotEmptyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DateSpanControl ds = d as DateSpanControl;
            if (ds != null)
            {
                StackPanel sp_Content = ds.Content as StackPanel;
                if (sp_Content != null)
                {
                    TextBox tb_Begin = sp_Content.Children[0] as TextBox;
                    TextBox tb_End = sp_Content.Children[2] as TextBox;
                    if ((bool)e.NewValue)
                    {
                        FocusManagerExt.SetAssertNotEmpty(tb_Begin, true);
                        FocusManagerExt.SetAssertNotEmpty(tb_End, true);
                    }
                    else
                    {
                        FocusManagerExt.SetAssertNotEmpty(tb_Begin, false);
                        FocusManagerExt.SetAssertNotEmpty(tb_End, false);
                    }

                }
            }

        }
        #endregion

        private void End_date_LostFocus(object sender, RoutedEventArgs e)
        {
            if (this.IsVerifiy )
            {
                if(string.IsNullOrEmpty(this.Beg_date.Text)||string.IsNullOrEmpty(this.End_date.Text))
                    return;

                if (long.Parse(this.End_date.Text) < long.Parse(this.Beg_date.Text))
                {
                    this.End_date.ShowTip("结束日期不能小于开始日期");
                    IPage page = Platform.Common.BasicFunctions.DependencyObjectExtensions.FindVisualTreeAncestor(this, x => { return (x is IPage); }) as IPage;
                    page.SetFocus();
                    return;
                }
            }

            if (this.EndDateLostFocus != null)
                this.EndDateLostFocus(this.Beg_date,this.End_date);
        }

        private void Beg_date_LostFocus(object sender, RoutedEventArgs e)
        {
            if (this.BeginDateLostFocus != null)
                this.BeginDateLostFocus(this.Beg_date, this.End_date);
        }
    }
}
