﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Controls.ControlsHelper;
using Platform.Library;
using TellerSystem.Library.Ext.Function;
using System.Text.RegularExpressions;
using Platform.Controls.ComponentContainer;
using TellerSystem.Library.Ext.TradeExtension;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// CheckMoney.xaml 的交互逻辑
    /// </summary>
    public partial class DetailEdit
    {
        #region 构造函数

        public DetailEdit()
        {
            InitializeComponent();
        }

        #endregion

        #region 变量

        #region 传输数据所用的字典

        /// <summary>
        /// 传输数据所用的字典
        /// </summary>
        private Dictionary<string, string> DetailInfoDic = new Dictionary<string, string>()
        {
            {"ProjectCode", ""},
            {"ProjectName", ""},
            {"StandardCode", ""},
            {"StandardName", ""},
            {"UnitOfMeasurement", ""},
            {"ChargingCount", ""},
            {"Amount", ""},
            {"ParmKPRQ", ""},
            {"ParmDWBM", ""},
            {"ParmSFJM", ""},
            {"ParmJKFS", ""},
            {"ParmIsOperate", ""}
        };

        #endregion

        #region 列名到打控件名的映射字典

        /// <summary>
        /// 列名到打控件名的映射字典
        /// </summary>
        private Dictionary<string, string> ColumnName = new Dictionary<string, string>()
        {
            {"项目编码", "ProjectCode"},
            {"项目名称", "ProjectName"},
            {"标准编码", "StandardCode"},
            {"标准名称", "StandardName"},
            {"计量单位", "UnitOfMeasurement"},
            {"计费数量", "ChargingCount"},
            {"金额", "Amount"}
        };

        #endregion

        /// <summary>
        /// 输入参数类
        /// </summary>
        public DetailEditEntity DetailEditEntity { internal get; set; }

        /// <summary>
        /// 输出参数类
        /// </summary>
        public DetailEditResult DetailEditResult { get; internal set; }

        #endregion

        #region 逻辑代码

        /// <summary>
        /// 界面初始化数据
        /// </summary>
        /// <returns></returns>
        public override bool InitPage()
        {
            /*********************************************
             * 此处需要填充的代码包括：
             * 1.界面初始数据
             * ******************************************/
            //如果数据源不为null用数据源，为null加载默认列名
            //做数据恢复用，方便重新编辑。
            if (DetailEditEntity.DetailEditResult.SourceDT != null)
                this.Detail_DataBox.DataBinding(DetailEditEntity.DetailEditResult.SourceDT);
            else
                this.Detail_DataBox.DataBinding("", new string[] { "项目编码", "项目名称", "标准编码", "标准名称", "计量单位", "计费数量", "金额" });
            //实例化结果类
            DetailEditResult = new DetailEditResult();
            //如果没有数据，焦点停留在新增按钮上
            if (!this.Detail_DataBox.DataBoxHost.Rows().Any())
                this.SetFocus(AddDetail);
            else
                this.SetFocus(Detail_DataBox);
            return true;
        }

        #endregion

        #region 内部逻辑

        /// <summary>
        /// 新增一条明细
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AddDetail_OnClick(object sender, RoutedEventArgs e)
        {
            if (Detail_DataBox.DataBoxHost.Rows().Count() == 5)
            {
                this.ShowWarnMessageBox("只能添加五条明细！");
                return;
            }
            //设置字典的值
            SetDicValue(null);
            //明细详情窗口
            DetailInformation df = new DetailInformation()
            {
                DetailInfoDic = this.DetailInfoDic
            };
            this.ShowPage(df);
            DetailInfoDic = df.DetailInfoDic;
            //没有操作，不添加
            if (String.CompareOrdinal(DetailInfoDic["ParmIsOperate"], "true") != 0) return;
            //查找重复项
            if (CheckDetailExist()) return;
            //添加一项
            this.Detail_DataBox.AddRow(GetDicRowData());
            //清除字典的值
            ClearDicValue();
        }

        /// <summary>
        /// 修改一条明细
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ModifyDetail_OnClick(object sender, RoutedEventArgs e)
        {
            var selectedRow = this.Detail_DataBox.GetSelectSingleRow();
            if(selectedRow == null) return;
            SetDicValue(selectedRow);

            //明细详情窗口
            var df = new DetailInformation()
            {
                DetailInfoDic = this.DetailInfoDic
            };
            this.ShowPage(df);
            DetailInfoDic = df.DetailInfoDic;

            //没有操作，不修改
            if (String.CompareOrdinal(DetailInfoDic["ParmIsOperate"], "true") != 0) return;

            //如果修改了项目编码和标准编码的组合，需要验证是否重复。
            if (!(Equals(selectedRow["项目编码"], DetailInfoDic["ProjectCode"]) ||
                Equals(selectedRow["标准编码"], DetailInfoDic["StandardCode"]))
                && CheckDetailExist())
            {
                return;
            }

            //修改选中列
            Detail_DataBox.UpdateRow(GetDicRowData() );
            //清除字典数据
            ClearDicValue();
        }

        /// <summary>
        /// 删除一条明细
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DeleteDetail_OnClick(object sender, RoutedEventArgs e)
        {
            this.Detail_DataBox.DeleteSelectRows();
        }

        /// <summary>
        /// 退出系统
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Quit_OnClick(object sender, RoutedEventArgs e)
        {
            if (this.ShowMessageBox("确定保存并退出吗？", "系统提示", MessageBoxButton.YesNo) != MessageBoxResult.Yes)
            {
                return;
            }
            //传出数据源
            DetailEditResult.SourceDT = Detail_DataBox.GetDataForDataTable();
            //传出数据
            GetDataForString();
            DetailEditResult.TotalMoney =
                this.Detail_DataBox.DataBoxHost.Items.Cast<DataRowView>()
                    .Sum(view => decimal.Parse(view.Row["金额"].ToString()));
            this.Close();
        }

        /// <summary>
        /// 检查明细是否存在
        /// </summary>
        /// <param name="projectCode">项目编码</param>
        /// <param name="StandardCode">标准编码</param>
        /// <returns>true存在，false不存在</returns>
        private bool CheckDetailExist()
        {
            var correctRow = Detail_DataBox.DataBoxHost.Items.Cast<DataRowView>()
                 .ToList()
                 .Find(x => Equals(x.Row["项目编码"], DetailInfoDic["ProjectCode"]) && Equals(x.Row["标准编码"],  DetailInfoDic["StandardCode"]));
            if (correctRow != null)
            {
                this.ShowErrorMessageBox("该条明细已存在！");
                return true;
            }
            return false;
        }

        /// <summary>
        /// 得到表格数据
        /// 兼容老系统，每行前面加上"||"
        /// </summary>
        private void GetDataForString()
        {
            var dataTable = Detail_DataBox.GetDataForDataTable();
            if (dataTable == null) return;

            StringBuilder fileData = new StringBuilder();

            //构造数据
            for (var i = 0; i < dataTable.Rows.Count; i++)
            {
                for (var j = 0; j < dataTable.Columns.Count; j++)
                {
                    if (j == 0)
                    {
                        fileData.Append("||");
                    }
                    fileData.Append(dataTable.Rows[i][j].ToString() + "|");
                }

                fileData.Append("\n");
            }
            DetailEditResult.DetailData = fileData.ToString();

        }

        /// <summary>
        /// 清除字典的值
        /// </summary>
        private void ClearDicValue()
        {
            var colName = DetailInfoDic.Keys.ToArray().Where(x => !x.StartsWith("Parm"));
            foreach (string val in colName)
            {
                DetailInfoDic[val] = "";
            }
        }

        /// <summary>
        /// 设置字典的值
        /// </summary>
        /// <param name="row"></param>
        private void SetDicValue(DataRow row)
        {
            if (row != null)
            {
                foreach (KeyValuePair<string, string> keyValuePair in ColumnName)
                    this.DetailInfoDic[keyValuePair.Value] = row[keyValuePair.Key].ToString().Trim();
            }
            this.DetailInfoDic["ParmKPRQ"] = this.DetailEditEntity.KPRQParm;
            this.DetailInfoDic["ParmDWBM"] = this.DetailEditEntity.DWBMParm;
            this.DetailInfoDic["ParmSFJM"] = this.DetailEditEntity.SFJMParm;
            this.DetailInfoDic["ParmJKFS"] = this.DetailEditEntity.JKFSParm;
        }

        /// <summary>
        /// 得到字典中列数据的值
        /// </summary>
        /// <returns></returns>
        private string[] GetDicRowData()
        {
            return new string[]
            {
                DetailInfoDic["ProjectCode"],
                DetailInfoDic["ProjectName"],
                DetailInfoDic["StandardCode"],
                DetailInfoDic["StandardName"],
                DetailInfoDic["UnitOfMeasurement"],
                DetailInfoDic["ChargingCount"],
                DetailInfoDic["Amount"]
            };
        }

        #endregion
    }

    #region 参数类

    /// <summary>
    /// 明细详情参数类
    /// </summary>
    public class DetailEditEntity
    {

        /// <summary>
        /// 开票日期，交易中传过来
        /// </summary>
        public string KPRQParm { internal get; set; }

        /// <summary>
        /// 单位编码，交易中传过来
        /// </summary>
        public string DWBMParm { internal get; set; }

        /// <summary>
        /// 是否减免，交易中传过来
        /// </summary>
        public string SFJMParm { internal get; set; }

        /// <summary>
        /// 缴库方式，交易中传过来
        /// </summary>
        public string JKFSParm { internal get; set; }

        public DetailEditResult DetailEditResult { internal get; set; }
    }

    /// <summary>
    /// 明细详情结果类
    /// </summary>
    public class DetailEditResult
    {
        public string DetailData { get; internal set; }

        public DataTable SourceDT { get; internal set; }

        public decimal TotalMoney { get; internal set; }
    }

    #endregion
}
