﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Controls.ControlsHelper;
using Platform.Library;
using TellerSystem.Library.Ext.Function;
using System.Text.RegularExpressions;
using System.Windows.Forms.VisualStyles;
using Platform.Controls.NumericContainer;
using TellerSystem.Communication;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// CheckMoney.xaml 的交互逻辑
    /// </summary>
    public partial class DetailInformation
    {
        #region 构造函数

        public DetailInformation()
        {
            InitializeComponent();
        }

        #endregion

        #region 变量

        public Dictionary<string,string> DetailInfoDic = new Dictionary<string, string>(); 

        #endregion

        #region 逻辑代码

        /// <summary>
        /// 界面初始化数据
        /// </summary>
        /// <returns></returns>
        public override bool InitPage()
        {
            /*********************************************
             * 此处需要填充的代码包括：
             * 1.界面初始数据
             * ******************************************/

            if (DetailInfoDic.ContainsKey("ProjectCode") &&
                !string.IsNullOrWhiteSpace(DetailInfoDic["ProjectCode"]))
            {
                foreach (
                    KeyValuePair<string, string> keyValuePair in DetailInfoDic.Where(x => !x.Value.StartsWith("Parm")))
                {
                    TextBox textbox = this.FindName(keyValuePair.Key) as TextBox;
                    if (textbox != null) textbox.Text = keyValuePair.Value;
                }
            }
            DetailInfoDic["ParmIsOperate"] = "false";
            return true;
        }

        #endregion

        #region 内部逻辑

        /// <summary>
        /// 项目编码焦点事件
        /// 获取项目名称
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ProjectCode_OnLostFocus(object sender, RoutedEventArgs e)
        {
            Message _msg = new Message(Message.BitMap.Agn);
            this.TradeAPIs.InitFd(_msg);
            this.TradeAPIs.CommSend(_msg);
            _msg.Fd16 = "6998";
            _msg.Fd60 = "FSSR";
            _msg.Fd22 = "06";
            _msg.Fd123 = string.Format("select fxmmc from fssr_hbxmxx where fxmbm = '{0}'", this.ProjectCode.Text.Trim());
            this.CallAgnTips(_msg);
            if (_msg.Fd12 != "0000")
            {
                this.TradeAPIs.ShowErrorMessageBox(this.TradeAPIs.GetError(_msg));
                return;
            }
            if (_msg.FileData == "")
            {
                this.TradeAPIs.ShowTip("请输入有效项目编码!");
                this.SetFocus();
                return;
            }
            this.ProjectName.Text = _msg.FileData.Split('|')[0];
        }

        /// <summary>
        /// 标准编码焦点事件
        /// 获取标准名称，计量单位
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StandardCode_OnLostFocus(object sender, RoutedEventArgs e)
        {
            Message _msg = new Message(Message.BitMap.Agn);
            this.TradeAPIs.InitFd(_msg);
            this.TradeAPIs.CommSend(_msg);
            _msg.Fd16 = "6998";
            _msg.Fd60 = "FSSR";
            _msg.Fd22 = "06";
            _msg.Fd123 = string.Format("select fbzmc,fjldw from fssr_xmbzxx where fxmbm = '{0}' and fbzbm = '{1}'",
                this.ProjectCode.Text.Trim(), this.StandardCode.Text.Trim());
            this.CallAgnTips(_msg);
            if (_msg.Fd12 != "0000")
            {
                this.TradeAPIs.ShowErrorMessageBox(this.TradeAPIs.GetError(_msg));
                return;
            }
            if (_msg.FileData == "")
            {
                this.TradeAPIs.ShowTip("请输入有效标准编码!");
                this.SetFocus();
                return;
            }
            this.StandardName.Text = _msg.FileData.Split('|')[0];
            this.UnitOfMeasurement.Text = _msg.FileData.Split('|')[1];
        }

        /// <summary>
        /// 确定点击事件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Message _msg = new Message(Message.BitMap.Agn);
            this.TradeAPIs.InitFd(_msg);
            this.TradeAPIs.CommSend(_msg);
            _msg.Fd16 = "6949";
            _msg.Fd22 = "06";
            _msg.Fd60 = "FSSR";
            _msg.Fd44 = DetailInfoDic["ParmKPRQ"];
            _msg.Fd30 = DetailInfoDic["ParmDWBM"];
            _msg.Fd31 = this.ProjectCode.Text;
            _msg.Fd32 = this.StandardCode.Text;
            _msg.Fd67 = DetailInfoDic["ParmSFJM"];
            _msg.Fd23 = DetailInfoDic["ParmJKFS"];
            _msg.Fd40 = this.TradeAPIs.Itoa(decimal.Parse(this.ChargingCount.Text), "%016.0f");
            _msg.Fd41 = this.TradeAPIs.Itoa(decimal.Parse(this.Amount.Text), "%016.0f");
            this.TradeAPIs.CallAgnTips(_msg);
            if (_msg.Fd12 != "0000")
            {
                if (this.ShowMessageBox("此条明细存在问题：" + this.TradeAPIs.GetError(_msg) + "。点击确定继续添加,取消重新编辑。"
                    , "系统提示", MessageBoxButton.YesNo) != MessageBoxResult.Yes)
                {
                    this.SetFocus(ProjectCode);
                    return;
                }
            }
            DetailInfoDic["ParmIsOperate"] = "true";
            var colName =  DetailInfoDic.Keys.ToArray().Where(x => !x.StartsWith("Parm"));
            foreach (string key in colName)
            {
                var textBox = this.FindName(key) as TextBox;
                if (textBox != null)
                    DetailInfoDic[key] = textBox.Text.Trim();
            }
            var currencyEditor = this.FindName("Amount") as CurrencyEditor;
            DetailInfoDic["Amount"] = currencyEditor.Val;

            this.Close();
        }

        #endregion

    }


}
