﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Printing;
using System.IO;
using System.Diagnostics;
using System.Linq;
using Platform.ServiceProxy.ServiceHelper;
using Platform.Library;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Library.Ext.TradeExtension;
using System.Text.RegularExpressions;
using Platform.Common.LogSystem;
using System.Runtime.InteropServices;
using System.Drawing.Printing;
using Platform.Common.RunningParameters;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// 文本打印交互逻辑
    /// </summary>
    [DesignTimeVisible(false)]
    public partial class FileViewer
    {
        #region 界面初始化

        /// <summary>
        /// 界面初始化
        /// </summary>
        public FileViewer()
        {
            InitializeComponent();
            reportTextBox.IsReadOnly = true;
            pc.EventOnlyPage += new EventHandler(rolesPage_OnlyPage);
            pc.EventFristPage += new EventHandler(rolesPage_FirstPage);
            pc.EventPage += new EventHandler(rolesPage_Page);
            pc.EventLastPage += new EventHandler(rolesPage_LastPage);
            Platform.Controls.TradeExtension.PageDialog.IsNormal = true;
            btnExport.Visibility = FLAG == "V2" ? Visibility.Visible : Visibility.Collapsed;
            //浚县增加导出功能
            if (FLAG == "V23" || FLAG == "V11")
            {
                setSepMargin();
                btnExportlast.Visibility = Visibility.Visible;
            }
        }

        #endregion 界面初始化

        #region 变量

        /// <summary>
        /// 字符编码
        /// </summary>
        //const string CodingStr = "gb2312";
        const string CodingStr = "UTF-8";

        /// <summary>
        /// 处理的报表文件路径
        /// </summary>
        string _filename = "";

        /// <summary>
        /// 报表分页数据集合
        /// </summary>
        List<string> DataPages;

        /// <summary>
        /// UI线程等待
        /// </summary>
        bool wait = true;

        /// <summary>
        /// 页面文本
        /// </summary>
        StringBuilder Page;

        /// <summary>
        /// 记录每次从服务器获取的字节数
        /// </summary>
        int bytecount = 0;
        /// <summary>
        /// 每次取数据的字节长度,默认10K
        /// </summary>
        int defaultByteLength = 10240;

        /// <summary>
        /// 字节的总长度
        /// </summary>
        int len = 0;
        /// <summary>
        /// 分页
        /// </summary>
        PageController pc = new PageController();
        /// <summary>
        /// 选择打印机的名称
        /// </summary>
        string printName = null;
        /// <summary>
        /// 版本控制标志
        /// </summary>
        private string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
        /// <summary>
        /// 核心返回文件
        /// </summary>
        string _textStr = string.Empty;
        #endregion

        #region 报表批量打印
        #region 设置间距
        //设置toolbar的组件间距
        private void setSepMargin()
        {
            Thickness myThickness = new Thickness();
            myThickness.Left = 2;
            myThickness.Top = 2;
            myThickness.Right = 2;
            myThickness.Bottom = 2;
            foreach (var item in this.toolBar.Items)
            {

                if (item is Separator)
                {
                    Separator sep = item as Separator;
                    sep.Margin = myThickness;
                }
            }
        }
        #endregion
        /// <summary>
        ///  报表批量打印时
        /// </summary>
        /// <returns></returns> 
        private bool InitReport()
        {
            byte[] text = FilesHandle.GetFileBlock(_filename, 0, defaultByteLength);
            DataPages = new List<string>();
            StringBuilder Page = new StringBuilder();
            while (text != null)
            {
                bytecount += text.Length;
                List<int> indexs = GetSplitChar(text);
                if (indexs.Count != 0)
                {
                    int position = 0;
                    for (int i = 0; i < indexs.Count; i++)
                    {
                        int index = indexs[i];
                        Page.Append(Encoding.GetEncoding(CodingStr).GetString(text, position, index - position).Replace("\r\n", "\n"));
                        DataPages.Add(Page.ToString());
                        Page = new StringBuilder();
                        position = index + 1;
                    }
                    Page.Append(Encoding.GetEncoding(CodingStr).GetString(text, position, text.Length - position).Replace("\r\n", "\n"));
                }
                else
                {
                    Page.Append(Encoding.GetEncoding(CodingStr).GetString(text, 0, text.Length).Replace("\r\n", "\n"));
                }
                if (len - bytecount > defaultByteLength)
                    text = FilesHandle.GetFileBlock(_filename, bytecount, defaultByteLength + bytecount);
                if (len - bytecount <= defaultByteLength)
                    text = FilesHandle.GetFileBlock(_filename, bytecount, len);  //每次取10k字节的数据
                if (len - bytecount <= 0)
                {
                    //退出循环
                    break;
                }
            }
            DataPages.Add(Page.ToString());
            return true;
        }
        #endregion

        #region  根据文件名，获取服务器上文件字节流
        /// <summary>
        /// 从服务器获取小块数据量
        /// </summary>
        /// <param name="filename"></param>
        /// <returns></returns>
        private bool GetLessFile(string filename)
        {
            DataPages = new List<string>();
            Page = new StringBuilder();
            byte[] text = null;
            text = FilesHandle.GetFileBlock(_filename, 0, len);
            if (text == null || text.Length < 5)
            {
                return false;
            }
            Page.Append(Encoding.GetEncoding(CodingStr).GetString(text, 0, text.Length).Replace("\r\n", "\n"));
            DataPages.Add(Page.ToString());
            reportTextBox.AppendText(DataPages[0]);
            if (DataPages.Count > 2)
            {
                pc.FristPage();
            }
            pc.CountPage(DataPages.Count);
            this.SumPage.Text = "共 " + DataPages.Count + " 页";
            //读取完成，允许打印
            btnPrint.IsEnabled = true;
            btnQuickPrint.IsEnabled = true;
            btnExit.Focus();
            return true;
        }
        /// <summary>
        /// 从服务器获取大量数据
        /// </summary>
        /// <param name="filename"></param>
        /// <returns></returns>
        private void GetMoreFile()
        {
            //开启一个线程去服务器上获取数据
            BackgroundWorker bgw_InitReport = new BackgroundWorker();
            bgw_InitReport.WorkerReportsProgress = true;
            bgw_InitReport.DoWork += new DoWorkEventHandler(bgw_InitReport_DoWork);
            bgw_InitReport.ProgressChanged += new ProgressChangedEventHandler(bgw_InitReport_ProgressChanged);
            bgw_InitReport.RunWorkerCompleted += new RunWorkerCompletedEventHandler(bgw_InitReport_RunWorkerCompleted);
            bgw_InitReport.RunWorkerAsync();
        }

        #endregion

        #region 根据文件名，获取服务器上文件
        /// <summary>
        /// 根据文件名，获取服务器上文件
        /// 初始化报表
        /// </summary>
        /// <param name="filename">文件路径</param>
        /// <param name="Batch">是否批量打印处理</param>
        /// <returns>True | False</returns>
        public bool Init(string filename, bool Batch)
        {
            if (string.IsNullOrEmpty(filename))
            {
                return false;
            }
            this._filename = filename;
            len = FilesHandle.GetFileBlockLength(_filename);

            if (!MoreThanByteLength(len))
            {
                GetLessFile(_filename);
            }
            else
            {
                GetMoreFile();
                while (wait)
                {
                    if (DataPages != null)
                    {
                        if (DataPages.Count == 1)
                        {
                            reportTextBox.AppendText("\n" + DataPages[0] + "\n\n");
                            wait = false;
                        }
                    }
                }
            }
            PrintCommand(null, null);
            return true;
        }
        #endregion

        #region  根据字符串，初始化报表
        /// <summary>
        /// 根据字符串，初始化报表
        /// </summary>
        /// <param name="textStr">文本字符串</param>
        /// <returns></returns>
        public bool TxtInit(string textStr, int pageRowCount = 0)
        {
            //textStr=textStr.Replace("\t", "    ");
            _textStr = textStr;
            if (string.IsNullOrEmpty(textStr))
            {
                return false;
            }
            DataPages = new List<string>();
            if (pageRowCount > 0)
            {
                //按照指定的页行数进行分页判断
                var reader = new StringReader(textStr);
                var line = reader.ReadLine();
                var counter = 1;
                var pageData = new StringBuilder();
                while (line != null)
                {
                    pageData.AppendLine(line);
                    if (counter == pageRowCount)
                    {
                        DataPages.Add(pageData.ToString());
                        pageData.Clear();
                        counter = 0;
                    }
                    line = reader.ReadLine();
                    counter++;
                }
                if (pageData.Length > 0)
                {
                    DataPages.Add(pageData.ToString());
                }
            }
            else
            {
                //采用解析/f分页符的方式进行分页判定
                int position = 0;
                List<int> indexs = GetStrIndex(textStr);
                for (int i = 0; i < indexs.Count; i++)
                {
                    //分隔符索引
                    int index = indexs[i];
                    //页数据存入DataPages中
                    DataPages.Add(textStr.Substring(position, index - position));
                    //新的开始位置
                    position = index + 1;
                }
                string tempstr = textStr.Substring(position, textStr.Length - position);
                Regex r = new Regex("^[0-9]{1,}$"); //表示数字的范围 ^符号是开始，$是关闭
                Regex cn = new Regex("[\u4e00-\u9fa5]+");//汉字范围
                Regex zm = new Regex(@"^[0-9a-zA-Z]{0,12}$");//字母范围
                if (r.IsMatch(tempstr) || cn.IsMatch(tempstr)) //过滤不是汉字、数字、字母的显示
                {
                    DataPages.Add(tempstr);
                }
            }
            if (DataPages.Count < 1)
                return false; //没有加载数据s
            reportTextBox.AppendText(DataPages[0]);
            if (DataPages.Count > 2)
            {
                pc.FristPage();
            }
            pc.CountPage(DataPages.Count);
            this.SumPage.Text = "共 " + DataPages.Count + " 页";
            //读取完成，允许打印
            btnPrint.IsEnabled = true;
            btnQuickPrint.IsEnabled = true;
            btnExit.Focus();
            return true;
        }
        #endregion

        #region 异步从服务器获取数据

        /// <summary>
        /// 异步从服务器取数据
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bgw_InitReport_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker bgw = sender as BackgroundWorker;
            byte[] text = FilesHandle.GetFileBlock(_filename, 0, defaultByteLength);
            DataPages = new List<string>();
            Page = new StringBuilder();
            while (text != null)
            {
                bytecount += text.Length;
                List<int> indexs = GetSplitChar(text);
                if (indexs.Count != 0)
                {
                    int position = 0;
                    for (int i = 0; i < indexs.Count; i++)
                    {
                        int index = indexs[i];
                        Page.Append(Encoding.GetEncoding(CodingStr).GetString(text, position, index - position).Replace("\r\n", "\n"));
                        DataPages.Add(Page.ToString());
                        bgw.ReportProgress(1);
                        Page = new StringBuilder();
                        position = index + 1;
                    }
                    Page.Append(Encoding.GetEncoding(CodingStr).GetString(text, position, text.Length - position).Replace("\r\n", "\n"));
                }
                else
                {
                    Page.Append(Encoding.GetEncoding(CodingStr).GetString(text, 0, text.Length).Replace("\r\n", "\n"));
                }
                if (len - bytecount > defaultByteLength)
                    text = FilesHandle.GetFileBlock(_filename, bytecount, defaultByteLength + bytecount);
                if (len - bytecount <= defaultByteLength)
                    text = FilesHandle.GetFileBlock(_filename, bytecount, len);  //每次取10k字节的数据
                if (len - bytecount <= 0)
                {
                    //退出循环
                    break;
                }
            }
            if (!string.IsNullOrWhiteSpace(Page.ToString().Trim()))
            {
                DataPages.Add(Page.ToString());
                //reportTextBox.Dispatcher.BeginInvoke(() => { reportTextBox.AppendText(Page.ToString()); },DispatcherPriority.Normal);
                //报告进度
                bgw.ReportProgress(1);
            }
        }

        /// <summary>
        /// 异步操作数据更新触发
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bgw_InitReport_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            DownCurrentPage.Text = "已下载 " + DataPages.Count + " 页,请耐心等待！";
            if (DataPages.Count > 1 && DataPages.Count < 3)
            {
                pc.FristPage();
            }
        }

        /// <summary>
        /// 异步操作完成
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bgw_InitReport_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            pc.CountPage(DataPages.Count);
            DownCurrentPage.Text = "下载完成,总共" + DataPages.Count + " 页";
            this.SumPage.Text = "共 " + DataPages.Count + " 页";
            //读取完成，允许打印
            btnPrint.IsEnabled = true;
            btnQuickPrint.IsEnabled = true;
            btnExit.Focus();
            if (e.Error != null)
            {
                this.ShowMessageBox("报表加载错误，请联系科技部!");
            }
        }

        #endregion 异步从服务器获取数据

        #region ToolBar操作

        /// <summary>
        /// 打印
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PrintCommand(object sender, RoutedEventArgs e)
        {
            //创建一个PrintDocument的实例
            PrintDialog printDialog = new PrintDialog();
            printDialog.UserPageRangeEnabled = true;
            if (printDialog.PrintQueue == null)
            {
                this.ShowMessageBox("未找到默认打印机!");
                return;
            }
            printDialog.PrintTicket = printDialog.PrintQueue.UserPrintTicket;
            printDialog.PrintTicket.PageOrientation = PageOrientation.Portrait;
            printDialog.PrintQueue = LocalPrintServer.GetDefaultPrintQueue();
            if (printDialog.ShowDialog() == true)
            {
                printName = printDialog.PrintQueue.FullName; //获取打印机的名称
                this.PerformPrintRun(printDialog);
            }

        }
        //快速打印
        private void PrintQuickCommand(object sender, RoutedEventArgs e)
        {
            //创建一个PrintDocument的实例
            PrintDialog printDialog = new PrintDialog();
            printDialog.UserPageRangeEnabled = true;
            if (printDialog.PrintQueue == null)
            {
                this.ShowMessageBox("未找到默认打印机!");
                return;
            }
            printDialog.PrintTicket = printDialog.PrintQueue.UserPrintTicket;
            printDialog.PrintTicket.PageOrientation = PageOrientation.Portrait;
            printDialog.PrintQueue = LocalPrintServer.GetDefaultPrintQueue();
            printName = printDialog.PrintQueue.FullName; //获取打印机的名称
            if (string.IsNullOrWhiteSpace(printName))
            {
                this.ShowMessageBox("不存在默认的打印机");
                return;
            }
            this.PerformPrintRun(printDialog);
        }
        /// <summary>
        /// 开始执行打印
        /// </summary>
        private void PerformPrintRun(PrintDialog printDialog)
        {
            CloseProcess("cmd.exe");
            int startIndex = 0;
            int endIndex = 0;
            //获取应用程序的当前工作目录
            string filepath = Directory.GetCurrentDirectory() + "\\PrintDir";
            //获取应用程序的当前工作目录的盘符
            //string disk = Directory.GetDirectoryRoot(filepath);
            string txtName = "print" + string.Format("{0:yyyyMMddHHmmss}", DateTime.Now) + ".txt";
            string txtPath = filepath + "\\" + txtName;

            DirectoryInfo info = new DirectoryInfo(@filepath);
            //检查是否存在此目录
            if (info.Exists)
                //info.Delete();
                Directory.Delete(filepath, true);
            //创建目录
            info.Create();
            if (printDialog.PageRangeSelection == PageRangeSelection.AllPages)
            {
                startIndex = 0;
                endIndex = DataPages.Count - 1;
            }
            else
            {
                startIndex = printDialog.PageRange.PageFrom - 1;
                endIndex = printDialog.PageRange.PageTo - 1;
                endIndex = endIndex < DataPages.Count - 1 ? endIndex : DataPages.Count - 1;
            }
            if (!File.Exists(txtPath))
            {
                if (FLAG != "V10" && FLAG != "V13" && FLAG != "V14" && FLAG != "V18" && FLAG != "V17" && FLAG != "V16" && FLAG != "V21" && FLAG != "V22" && FLAG != "V23" && FLAG != "V24")//齐丰项目win7打印导致程序卡死，更换方法后打印出现乱码，so 采用电脑的默认编码格式
                {
                    using (StreamWriter streamWriter = new StreamWriter(txtPath, true, Encoding.GetEncoding(CodingStr)))
                    {
                        for (int i = startIndex; i <= endIndex; i++)
                        {
                            streamWriter.Write(DataPages[i]);

                            if (i != endIndex)
                            {
                                streamWriter.WriteLine('\f');
                            }
                        }
                        streamWriter.Flush();
                        streamWriter.Close();
                    }
                }
                else
                {
                    using (StreamWriter streamWriter = new StreamWriter(txtPath, true, Encoding.Default))
                    {
                        for (int i = startIndex; i <= endIndex; i++)
                        {
                            streamWriter.Write(DataPages[i]);

                            if (i != endIndex)
                            {
                                streamWriter.WriteLine('\f');
                            }
                        }
                        streamWriter.Flush();
                        streamWriter.Close();
                    }



                }
            }
            if (FLAG != "V10" && FLAG != "V13" && FLAG != "V14" && FLAG != "V18" && FLAG != "V17" && FLAG != "V16" && FLAG != "V21" && FLAG != "V22" && FLAG != "V23" && FLAG != "V24")//齐丰WIN7电脑调用该方法异常关闭。故采用cmd命令打印。
            {
                //采用底层打印
                RealPrinter.SendToPrinter(printDialog.PrintQueue.FullName, txtPath);
                return;
            }
            string[] strPrint = printDialog.PrintQueue.FullName.Replace("\\\\", "\\").Split(new char[] { '\\' });
            var result = from s in strPrint where !string.IsNullOrEmpty(s) select s;
            string cmdStr = "";
            txtPath = string.Format("\"{0}\"", txtPath);
            if (strPrint.Length < 2)
                cmdStr = string.Format("Print /d:{0} {1}", printDialog.PrintQueue.QueuePort.Name, txtPath);//本地打印
            else
                cmdStr = "Print " + "/d:\\" + "\\" + result.FirstOrDefault() + "\\" + "\"" + printDialog.PrintQueue.ShareName + "\"" + " " + txtPath;//共享打印机           
            //调用cmd打印命令
            var cmd = Cmd(cmdStr);


        }

        #region 底层打印调用
        /// <summary>
        ///打印机类 windows底层函数的打印机相关操作
        /// </summary>
        public class RealPrinter
        {
            public struct DOCINFO
            {
                [MarshalAs(UnmanagedType.LPWStr)]
                public string pDocName;
                [MarshalAs(UnmanagedType.LPWStr)]
                public string pOutputFile;
                [MarshalAs(UnmanagedType.LPWStr)]
                public string pDataType;
            }

            [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = false,
           CallingConvention = CallingConvention.StdCall)]
            public static extern long OpenPrinter(string pPrinterName, ref IntPtr phPrinter,
            int pDefault);
            [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = false,
           CallingConvention = CallingConvention.StdCall)]
            public static extern long StartDocPrinter(IntPtr hPrinter, int Level,
            ref DOCINFO pDocInfo);

            [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = true,
           CallingConvention = CallingConvention.StdCall)]
            public static extern long StartPagePrinter(IntPtr hPrinter);
            [DllImport("winspool.drv", CharSet = CharSet.Ansi, ExactSpelling = true,
           CallingConvention = CallingConvention.StdCall)]
            public static extern long WritePrinter(IntPtr hPrinter, string data,
            int buf, ref int pcWritten);

            [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = true,
           CallingConvention = CallingConvention.StdCall)]
            public static extern long EndPagePrinter(IntPtr hPrinter);

            [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = true,
           CallingConvention = CallingConvention.StdCall)]
            public static extern long EndDocPrinter(IntPtr hPrinter);

            [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = true,
           CallingConvention = CallingConvention.StdCall)]
            public static extern long ClosePrinter(IntPtr hPrinter);

            /// <summary>
            /// 打印文件
            /// </summary>
            /// <param name="filePath"></param>
            public static void SendToPrinter(string printerName, string filePath)
            {
                //打开打印机
                int pcWritten = 0;
                IntPtr lhPrinter = new System.IntPtr();
                var di = new DOCINFO();
                RealPrinter.OpenPrinter(printerName, ref lhPrinter, 0);
                try
                {
                    RealPrinter.StartDocPrinter(lhPrinter, 1, ref di);
                    RealPrinter.StartPagePrinter(lhPrinter);
                    //执行打印
                    var data = File.ReadAllText(filePath);
                    //Encoding.GetEncoding("gb2312").GetBytes(testStr)
                    RealPrinter.WritePrinter(lhPrinter, data, PlatformSettings.Encoding.GetBytes(data).Length, ref pcWritten);
                    //RealPrinter.WritePrinter(lhPrinter, data, Encoding.GetEncoding("ISO-8859-1").GetBytes(data).Length, ref pcWritten);


                    ////printStr = PRT_OPEN;
                    ////RealPrinter.WritePrinter(lhPrinter, printStr, printStr.Length, ref pcWritten);
                    //data = "\x11\x18";
                    //RealPrinter.WritePrinter(lhPrinter, data, data.Length, ref pcWritten);
                    ////update by lic 20111125 黄行不要初始化打印机 start
                    ////printStr = "\x10\x42";
                    ////RealPrinter.WritePrinter(lhPrinter, printStr, printStr.Length, ref pcWritten);
                    ////update by lic 20111125 黄行不要初始化打印机 end
                    //data = "\x1bR";
                    //RealPrinter.WritePrinter(lhPrinter, data, data.Length, ref pcWritten);
                    //data = "\x1b\x25\x39\x00";
                    //RealPrinter.WritePrinter(lhPrinter, data, data.Length, ref pcWritten);
                    //data = ((char)LineSpace).ToString(); ;
                    //RealPrinter.WritePrinter(lhPrinter, data, data.Length, ref pcWritten);

                }
                catch (Exception ex)
                {
                    //记录日志
                    PlatformLogger.SystemErrorInfo("打印机故障，核心报表打印失败！", ex);
                }
                finally
                {
                    if (lhPrinter != IntPtr.Zero)
                    {
                        //退纸
                        string printStr = "\x0c";
                        RealPrinter.WritePrinter(lhPrinter, printStr, printStr.Length, ref pcWritten);
                        RealPrinter.EndPagePrinter(lhPrinter);
                        RealPrinter.EndDocPrinter(lhPrinter);
                        RealPrinter.ClosePrinter(lhPrinter);
                    }
                }
            }
        }
        #endregion

        #region cmd命令操作
        /// <summary>
        /// 运行Cmd命令
        /// </summary>
        /// <param name="cmd">命令</param>
        /// <returns></returns>
        public static string Cmd(string command)
        {
            string output = "";
            try
            {
                //实例化一个Process类,启动一个独立进程
                Process cmd = new Process();
                //设定程序名
                cmd.StartInfo.FileName = "cmd.exe";
                ////设定程序执行参数
                //cmd.StartInfo.Arguments = "/c "+command;
                //关闭Shell的使用
                cmd.StartInfo.UseShellExecute = false;
                ////重定向标准输入
                cmd.StartInfo.RedirectStandardInput = true;
                cmd.StartInfo.RedirectStandardOutput = true;
                ////重定向错误输出
                cmd.StartInfo.RedirectStandardError = true;
                ////设置不显示窗口
                cmd.StartInfo.CreateNoWindow = true;
                cmd.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
                //启动
                cmd.Start();
                cmd.StandardInput.AutoFlush = true;
                cmd.StandardInput.WriteLine(command.ToString());
                cmd.StandardInput.WriteLine("exit");
                //从输出流取得命令执行结果
                output = cmd.StandardOutput.ReadToEnd();
                PlatformLogger.SystemInfo("报表打印:" + output);
                //MessageBox.Show(output);
                cmd.WaitForExit();
                cmd.Close();
            }
            catch (Exception e)
            {
                throw e;
            }
            return output;
        }
        /// <summary>
        /// 关闭进程
        /// </summary>
        /// <param name="ProcName">进程名称</param>
        /// <returns></returns>
        public static bool CloseProcess(string ProcName)
        {
            bool result = false;
            System.Collections.ArrayList procList = new System.Collections.ArrayList();
            string tempName = "";
            int begpos;
            int endpos;
            foreach (System.Diagnostics.Process thisProc in System.Diagnostics.Process.GetProcesses())
            {
                tempName = thisProc.ToString();
                begpos = tempName.IndexOf("(") + 1;
                endpos = tempName.IndexOf(")");
                tempName = tempName.Substring(begpos, endpos - begpos);
                procList.Add(tempName);
                if (tempName == ProcName)
                {
                    if (!thisProc.CloseMainWindow())
                        thisProc.Kill(); // 当发送关闭窗口命令无效时强行结束进程
                    result = true;
                }
            }
            return result;
        }
        #endregion

        #region cmd命令执行打印
        private void PrintTxt(List<string> DataPage)
        {


            ////删除/拷贝
            //try
            //{
            //    File.Delete(filepath);
            //}
            //catch (Exception e)
            //{
            //    throw e;
            //}

        }
        #endregion



        /// <summary>
        /// 退出
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ExitCommand(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// 首页
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FirstPageCommand(object sender, RoutedEventArgs e)
        {
            pc.FristPage();
            RtbBindingText(pc.CurPage);
        }

        /// <summary>
        /// 上一页
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PreviouePageCommand(object sender, RoutedEventArgs e)
        {
            pc.PreviousPage();
            RtbBindingText(pc.CurPage);
        }

        /// <summary>
        /// 下一页
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NextPageCommand(object sender, RoutedEventArgs e)
        {
            pc.NextPage();
            RtbBindingText(pc.CurPage);

        }

        /// <summary>
        /// 尾页
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LastPageCommand(object sender, RoutedEventArgs e)
        {
            pc.LastPage();
            RtbBindingText(pc.CurPage);
        }
        /// <summary>
        /// 跳转页码
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void GoToPageCommand(object sender, RoutedEventArgs e)
        {
            if (String.IsNullOrWhiteSpace(pageNumberTextBox.Text))
            {
                this.ShowMessageBox("请输入页码!");
                return;
            }
            if (Convert.ToInt32(pageNumberTextBox.Text) > DataPages.Count)
            {
                this.ShowMessageBox("输入的页码超出所有的页!总共" + DataPages.Count + "页");
                pageNumberTextBox.Text = "";
                return;
            }
            if (Convert.ToInt32(pageNumberTextBox.Text) == 0)
            {
                this.ShowMessageBox("输入的页码不正确,不存在0页");
                pageNumberTextBox.Text = "";
                return;
            }
            if (Convert.ToInt32(pageNumberTextBox.Text) == 1)
            {
                pc.FristPage();
                RtbBindingText(1);
            }
            if (Convert.ToInt32(pageNumberTextBox.Text) == DataPages.Count)
            {
                pc.LastPage();
                RtbBindingText(DataPages.Count);
            }
            if (Convert.ToInt32(pageNumberTextBox.Text) < DataPages.Count && Convert.ToInt32(pageNumberTextBox.Text) > 1)
            {
                rolesPage_Page(sender, e);
                RtbBindingText(Convert.ToInt32(pageNumberTextBox.Text));
            }
        }


        #endregion ToolBar操作

        #region 跳转文本验证

        private void pageNumberPasting(object sender, DataObjectPastingEventArgs e)
        {
            if (e.DataObject.GetDataPresent(typeof(String)))
            {
                String text = (String)e.DataObject.GetData(typeof(String));
                if (!isNumberic(text))
                { e.CancelCommand(); }
            }
            else { e.CancelCommand(); }
        }

        private void pageNumberPreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Space)
                e.Handled = true;
        }

        private void pageNumberPreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            if (!isNumberic(e.Text))
            {
                e.Handled = true;
            }
            else
                e.Handled = false;
        }

        /// <summary>
        /// isDigit是否是数字
        /// </summary>
        /// <param name="_string"></param>
        /// <returns></returns>
        public static bool isNumberic(string _string)
        {
            if (string.IsNullOrEmpty(_string))
                return false;
            foreach (char c in _string)
            {
                if (!char.IsDigit(c))
                    return false;
            }
            return true;
        }

        #endregion 跳转文本验证

        #region 快捷键

        /// <summary>
        /// 获取键盘快捷键
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TradePage_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Up:
                    {
                        if (BtnPreviouePage.IsEnabled)
                        {
                            PreviouePageCommand(sender, e);
                        }
                        break;
                    }
                case Key.Down:
                    {
                        if (BtnNextPage.IsEnabled)
                        {
                            NextPageCommand(sender, e);
                        }
                        break;
                    }
                case Key.P:
                    {
                        if (btnPrint.IsEnabled)
                        {
                            PrintCommand(sender, e);
                        }
                        break;
                    }
                case Key.F:
                    {
                        if (btnQuickPrint.IsEnabled)
                        {
                            PrintQuickCommand(sender, e);
                        }
                        break;
                    }
                case Key.Escape:
                    {
                        if (btnExit.IsEnabled)
                        {
                            ExitCommand(sender, e);
                        }
                        break;
                    }
                case Key.Home:
                    {
                        if (BtnFirstPage.IsEnabled)
                        {
                            FirstPageCommand(sender, e);
                        }

                        break;
                    }
                case Key.End:
                    {
                        if (BtnLastPage.IsEnabled)
                        {
                            LastPageCommand(sender, e);
                        }
                        break;
                    }
                case Key.Return:
                    {
                        if (pageNumberTextBox.Focus())
                        {
                            GoToPageCommand(sender, e);
                        }
                        break;
                    }
                default: break;
            }
        }

        #endregion 快捷键

        #region 辅助方法

        /// <summary>
        /// 遍历byte数组,寻找分页符
        /// </summary>
        /// <param name="text"></param>
        /// <returns>返回查找到的索引集合</returns>
        private List<int> GetSplitChar(byte[] text)
        {
            List<int> indexs = new List<int>();
            for (int i = 0; i < text.Length; i++)
            {
                if (text[i] == (byte)'\f')
                {
                    indexs.Add(i);
                }
                //除去内容中的\0字符
                if (text[i] == (byte)'\0')
                {
                    text[i] = (byte)' ';
                }
            }
            return indexs;
        }

        /// <summary>
        ///  当前字节的长度与总长度比较
        /// </summary>
        /// <param name="len">取得数据的字节的总长度</param>
        /// <returns>True| False</returns>
        private bool MoreThanByteLength(int len)
        {
            if (defaultByteLength < len)
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// 解析当前字符出现分页的索引
        /// </summary>
        /// <param name="textStr">传入字符串</param>
        /// <returns>返回出现分页的索引</returns>
        private List<int> GetStrIndex(string textStr)
        {
            List<int> indexs = new List<int>();
            for (int i = 0; i < textStr.Length; i++)
            {
                if (textStr[i] == '\f')
                {
                    indexs.Add(i);
                }
            }
            return indexs;
        }
        #endregion

        #region   设置点击状态
        //仅有一页
        void rolesPage_OnlyPage(object sender, EventArgs e)
        {
            BtnFirstPage.IsEnabled = false;
            BtnPreviouePage.IsEnabled = false;
            BtnNextPage.IsEnabled = false;
            BtnLastPage.IsEnabled = false;
            btnGoToPage.IsEnabled = false;
        }

        //首页
        void rolesPage_FirstPage(object sender, EventArgs e)
        {
            BtnFirstPage.IsEnabled = false;
            BtnPreviouePage.IsEnabled = false;
            BtnNextPage.IsEnabled = true;
            BtnLastPage.IsEnabled = true;
            btnGoToPage.IsEnabled = true;
        }

        //中间页
        void rolesPage_Page(object sender, EventArgs e)
        {
            BtnFirstPage.IsEnabled = true;
            BtnPreviouePage.IsEnabled = true;
            BtnNextPage.IsEnabled = true;
            BtnLastPage.IsEnabled = true;
            btnGoToPage.IsEnabled = true;
        }
        //尾页
        void rolesPage_LastPage(object sender, EventArgs e)
        {
            BtnFirstPage.IsEnabled = true;
            BtnPreviouePage.IsEnabled = true;
            BtnNextPage.IsEnabled = false;
            BtnLastPage.IsEnabled = false;
            btnGoToPage.IsEnabled = true;
        }


        #endregion

        #region   文本操作
        /// <summary>
        /// 绑定文本值
        /// </summary>
        /// <param name="index"></param>
        private void RtbBindingText(int index)
        {
            ClearRichTextBox();
            reportTextBox.AppendText("\n" + DataPages[index - 1].Replace("\n\n\n\n", "\n").Replace("\n\n\n", "\n").Replace("\n\n", "\n") + "\n\n");
            CurrentPage.Text = "第 " + index + " 页";
        }
        /// <summary>
        /// 清空RichtextBox文本
        /// </summary>
        private void ClearRichTextBox()
        {
            var clearRichTextBox = reportTextBox.Document;
            clearRichTextBox.Blocks.Clear();
        }
        #endregion

        #region 导出
        /// <summary>
        /// 导出文本
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ExportCommand(object sender, RoutedEventArgs e)
        {
            string file = "";
            string rootpath = AppDomain.CurrentDomain.BaseDirectory;
            var folderBrowserDialog = new System.Windows.Forms.FolderBrowserDialog();
            System.Windows.Forms.DialogResult result = folderBrowserDialog.ShowDialog();
            if (result == System.Windows.Forms.DialogResult.OK)
            {
                var folderName = folderBrowserDialog.SelectedPath;
                rootpath = folderName;
            }
            else
            {
                this.ShowMessageBox("导出失败!");
                return;

            }
            if (!Directory.Exists(rootpath))//若文件夹不存在则新建文件夹   
            {
                Directory.CreateDirectory(rootpath); //新建文件夹
            }
            try
            {
                file = _textStr;
                string path = rootpath + "//";
                if (!Directory.Exists(path))//若文件夹不存在则新建文件夹   
                {
                    Directory.CreateDirectory(path); //新建文件夹
                }
                string fileName = "print" + string.Format("{0:yyyyMMddHHmmss}", DateTime.Now) + ".txt";
                string filePath = path + fileName;
                StreamWriter sw = new StreamWriter(filePath);
                //向创建的文件中写入内容
                sw.WriteLine(file);
                sw.Close();
                file = string.Empty;
                this.ShowMessageBox("导出成功！导出文件名为：" + fileName);
            }
            catch (Exception)
            {
                this.ShowMessageBox("导出失败！");
            }

        }

        private static string towview(string file)
        {
            file = file.Replace("─", "──");//替换─
            file = file.Replace("┴", "─┴");//替换─
            file = file.Replace("┬", "─┬");//替换─
            file = file.Replace("┼", "─┼");//替换─
            Regex r = new Regex("│");
            Regex rege = new Regex("│", RegexOptions.Compiled);
            StringBuilder sbfile = new StringBuilder();
            int count = 0;
            var tp = file.Split('\n');
            for (int i = 0; i < tp.Length; i++)
            {
                count = rege.Matches(tp[i]).Count;
                if (count > 2)
                {
                    tp[i] = r.Replace(tp[i], " │", count - 2, tp[i].IndexOf('│') + 1);
                }
                sbfile.AppendLine(tp[i]);
            }
            file = sbfile.ToString();
            return file;
        }
        #endregion
        /// <summary>
        /// 导出文本
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ExportpagesCommand(object sender, RoutedEventArgs e)
        {
            string file = "";
            string rootpath = AppDomain.CurrentDomain.BaseDirectory;
            var folderBrowserDialog = new System.Windows.Forms.FolderBrowserDialog();
            System.Windows.Forms.DialogResult result = folderBrowserDialog.ShowDialog();
            if (result == System.Windows.Forms.DialogResult.OK)
            {
                var folderName = folderBrowserDialog.SelectedPath;
                rootpath = folderName;
            }
            else
            {
                this.ShowMessageBox("导出失败!");
                return;

            }
            if (!Directory.Exists(rootpath))//若文件夹不存在则新建文件夹   
            {
                Directory.CreateDirectory(rootpath); //新建文件夹
            }
            try
            {
                file = _textStr;
                StringBuilder sbfile = new StringBuilder();

                //采用解析/f分页符的方式进行分页判定
                int position = 0;
                int pageSize = 64;
                List<string> listPages = new List<string>();
                List<int> indexs = GetStrIndex(file);
                for (int i = 0; i < indexs.Count; i++)
                {
                    //分隔符索引
                    int index = indexs[i];
                    //页数据存入DataPages中
                    listPages.Add(file.Substring(position, index - position));
                    //新的开始位置
                    position = index + 1;
                }
                string tempstr = file.Substring(position, file.Length - position);
                Regex r = new Regex("^[0-9]{1,}$"); //表示数字的范围 ^符号是开始，$是关闭
                Regex cn = new Regex("[\u4e00-\u9fa5]+");//汉字范围
                Regex zm = new Regex(@"^[0-9a-zA-Z]{0,12}$");//字母范围
                if (r.IsMatch(tempstr) || cn.IsMatch(tempstr)) //过滤不是汉字、数字、字母的显示
                {
                    listPages.Add(tempstr);
                }
                StringBuilder files = new StringBuilder();
                foreach (var page in listPages)
                {
                    var tp = page.Split('\n');
                    for (int i = 0; i < pageSize; i++)
                    {
                        if (tp.Length > i)
                        {
                            string line = tp[i];
                            if (line.Contains("剪----------------裁----------------线"))
                            {
                                line = line.Remove(2, 14);
                                line = line.Remove(line.Length - 14, 14);
                            }
                            files.AppendLine(line);
                            if (line.Contains("单位经办人签名"))
                            {
                                files.AppendLine(tp[i + 1]);
                                files.AppendLine(tp[i + 1]);
                            }
                        }
                        else
                        {
                            files.AppendLine("");
                        }
                    }
                }
                string path = rootpath + "//";
                if (!Directory.Exists(path))//若文件夹不存在则新建文件夹   
                {
                    Directory.CreateDirectory(path); //新建文件夹
                }
                string fileName = "print" + string.Format("{0:yyyyMMddHHmmss}", DateTime.Now) + ".txt";
                string filePath = path + fileName;
                StreamWriter sw = new StreamWriter(filePath);
                //向创建的文件中写入内容
                sw.WriteLine(files.ToString().TrimEnd('\n'));
                sw.Close();
                file = string.Empty;
                this.ShowMessageBox("导出成功！导出文件名为：" + fileName);
            }
            catch (Exception)
            {
                this.ShowMessageBox("导出失败！");
            }
        }

    }
}