﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Printing;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using Platform.Common.BasicFunctions;
using Platform.Common.RunningParameters;
using Platform.Controls.ControlsHelper;
using Platform.Controls.TradeExtension;
using Platform.Library;
using Platform.Presentation.Interfaces;
using Platform.Presentation.PageFunctions;
using Platform.Presentation.PagesHelper;
using Platform.Print;
using TellerSystem.Communication;
using TellerSystem.Controls.Ext.PrintsHelper;
using TellerSystem.Devices.Interface;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Library.Ext.TradeExtension;
using TellerSystem.Library.Ext.Variables;
using TellerSystem.PMS.XML;
using TellerSystem.PrintTemplate.Extension;
using TellerSystem.ServiceProxy.Ext.ServiceEntitys;

/******************************平台Dll*****************************/

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// IdCheckViewer.xaml 的交互逻辑
    /// </summary>
    public partial class LeadTradeViewer
    {
        #region Field

        /// <summary>
        ///     打印页面容器
        /// </summary>
        private ContentControl _panle;

        private bool[] _title;

        /// <summary>
        ///     偏移控制
        /// </summary>
        private TranslateTransform _ttf;

        private int _preIndex = 0;

        #endregion Field

        #region Ctor

        public LeadTradeViewer()
        {
            InitializeComponent();
        }

        #endregion Ctor

        #region 公开的属性与方法

        /// <summary>
        /// 公共报文对象
        /// </summary>
        public Message Msg { get; set; }

        /// <summary>
        /// 数据总线，用于交易间的数据传递
        /// </summary>
        public Dictionary<string, object> DataBus = new Dictionary<string, object>();

        /// <summary>
        ///公共的xml报文，用于存放各个交易的xml报文对象
        /// </summary>
        private Dictionary<string, XML_reader_mgr> xml_reade_list = new Dictionary<string, XML_reader_mgr>();

        /// <summary>
        ///取值公共的xml报文，用于存放各个交易的xml报文对象
        /// </summary>
        public Dictionary<string, XML_reader_mgr> Xml_reade_list
        {
            get { return this.xml_reade_list; }
        }

        /// <summary>
        /// 如果新增加key，则保存对象
        /// 如果已经存在key对应的对象，则修改
        /// </summary>
        /// <param name="key"></param>
        /// <param name="xml_read_handle"></param>
        public void SetItemXmlReaderMgr(string key, XML_reader_mgr xml_read_handle)
        {
            if (this.xml_reade_list.ContainsKey(key))
            {
                this.xml_reade_list[key] = xml_read_handle;
            }
            else
            {
                this.xml_reade_list.Add(key, xml_read_handle);
            }
        }

        /// <summary>
        /// 当前页的索引，从0开始
        /// </summary>
        public int CurrentLeadIndex { get; set; }

        /// <summary>
        /// 数据源
        /// </summary>
        public IList<LeadItem> LeadSource { get; set; }

        /// <summary>
        /// 设置或获取下一步按钮的可用性
        /// </summary>
        public bool IsNextBtnEnable
        {
            set { btnNext.IsEnabled = value; }
        }

        private int _stepIndex;

        /// <summary>
        /// 跳转到指定步骤，从0开始
        /// </summary>
        public int StepIndex
        {
            get { return _stepIndex; }
            set
            {
                _stepIndex = value;
                btnStep.Visibility = Visibility.Visible;
            }
        }

        #endregion 公开的属性与方法

        #region 路由 下一步事件

        //创建和注册ReFlesh事件，该事件路由策略为Direct
        public static readonly RoutedEvent LeadNextEvent = EventManager.RegisterRoutedEvent("LeadNext",
            RoutingStrategy.Direct, typeof(RoutedEventHandler), typeof(LeadTradeViewer));

        //CLR事件的包装器
        public event RoutedEventHandler LeadNext
        {
            add { AddHandler(LeadNextEvent, value); }
            remove { RemoveHandler(LeadNextEvent, value); }
        }

        //触发ReFleshEvent
        private void RaiseLeadNextEvent()
        {
            var newEventArgs = new RoutedEventArgs(LeadTradeViewer.LeadNextEvent);
            RaiseEvent(newEventArgs);
        }

        #endregion 路由 下一步事件

        #region 路由 完成事件

        //创建和注册ReFlesh事件，该事件路由策略为Direct
        public static readonly RoutedEvent LeadCompletedEvent = EventManager.RegisterRoutedEvent("LeadCompleted",
            RoutingStrategy.Direct, typeof(RoutedEventHandler), typeof(LeadTradeViewer));

        public event RoutedEventHandler LeadCompleted
        {
            add { AddHandler(LeadCompletedEvent, value); }
            remove { RemoveHandler(LeadCompletedEvent, value); }
        }

        private void RaiseLeadCompletedEvent()
        {
            var newEventArgs = new RoutedEventArgs(LeadTradeViewer.LeadCompletedEvent);
            RaiseEvent(newEventArgs);
        }

        #endregion 路由 完成事件

        #region 路由 该步完成事件

        //创建和注册ReFlesh事件，该事件路由策略为Direct
        public static readonly RoutedEvent TradeCompletedEvent = EventManager.RegisterRoutedEvent("TradeCompleted",
            RoutingStrategy.Direct, typeof(RoutedEventHandler), typeof(LeadTradeViewer));

        public event RoutedEventHandler TradeCompleted
        {
            add { AddHandler(TradeCompletedEvent, value); }
            remove { RemoveHandler(TradeCompletedEvent, value); }
        }

        private void RaiseTradeCompletedEvent()
        {
            var newEventArgs = new RoutedEventArgs(TradeCompletedEvent);
            RaiseEvent(newEventArgs);
        }

        #endregion 路由 该步完成事件

        #region 初始化

        /// <summary>
        ///     存储初始信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StackPanel_Loaded(object sender, RoutedEventArgs e)
        {
            if (LeadSource == null || LeadHeader.Children.Count >= LeadSource.Count) return;

            _panle = sender as ContentControl;
            _ttf = new TranslateTransform();
            if (_panle != null)
            {
                _panle.RenderTransform = _ttf;
            }
            _title = new bool[LeadSource.Count];
            InitTitle(Direction.None);
            DisplayBtnContent();
        }

        #endregion 初始化

        #region 事件

        /// <summary>
        /// 上一步
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPrevious_Click(object sender, RoutedEventArgs e)
        {
            if (CurrentLeadIndex <= 0) return;
            if (CurrentLeadIndex > LeadSource.Count - 1) CurrentLeadIndex = LeadSource.Count - 1;
            CurrentLeadIndex -= 1;
            InitTitle(Direction.Up);
            DisplayBtnContent();
            //滑动效果
            AnimateControl(FlowDirection.RightToLeft);
        }

        /// <summary>
        /// 下一步
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNext_Click(object sender, RoutedEventArgs e)
        {
            if (CurrentLeadIndex > LeadSource.Count - 1)
            {
                return;
            }

            if (CurrentLeadIndex < LeadSource.Count - 1)  // "下一步"
            {
                CurrentLeadIndex += 1;
                InitTitle(Direction.Next);
                DisplayBtnContent();
                //滑动效果
                AnimateControl(FlowDirection.LeftToRight);

                //公开路由方法
                RaiseLeadNextEvent();
            }
            else   // "完成"
            {
                CurrentLeadIndex += 1;
                //公开路由方法
                RaiseLeadCompletedEvent();
            }
        }

        /// <summary>
        /// 跳过
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStep_Click(object sender, RoutedEventArgs e)
        {
            if (StepIndex < 0 || StepIndex > LeadSource.Count - 1) return;
            CurrentLeadIndex = StepIndex;
            InitTitle(Direction.Jump);
            DisplayBtnContent();
            //滑动效果
            AnimateControl(FlowDirection.LeftToRight);

            btnStep.Visibility = Visibility.Hidden;
        }

        /// <summary>
        /// 动画
        /// </summary>
        /// <param name="dir"></param>
        private void AnimateControl(FlowDirection dir)
        {
            //var sb = new Storyboard();

            //var ks = new DoubleAnimationUsingKeyFrames();
            ////偏移量为实际宽度
            //var width = 710;
            //width *= Math.Abs(CurrentLeadIndex - _preIndex);
            //var k1 = new EasingDoubleKeyFrame
            //{
            //    KeyTime = KeyTime.FromTimeSpan(TimeSpan.FromSeconds(0.5)),
            //    Value = dir == FlowDirection.LeftToRight ? _ttf.X - width : _ttf.X + width,
            //    EasingFunction = new SineEase { EasingMode = EasingMode.EaseIn }
            //};

            //ks.KeyFrames.Add(k1);
            //sb.Children.Add(ks);
            //Storyboard.SetTarget(ks, _panle);
            //Storyboard.SetTargetProperty(ks, new PropertyPath("(UIElement.RenderTransform).(TranslateTransform.X)"));

            //sb.Begin();

            //记录本次索引
            _preIndex = CurrentLeadIndex;
        }

        /// <summary>
        /// 页面显示要素的处理
        /// </summary>
        private void DisplayBtnContent()
        {
            FleshThemes();

            LeadSource[CurrentLeadIndex].PageContent.SetValue(Platform.Presentation.FocusManagers.FocusManager.EnableFocusControlProperty, false);
            //去掉提交模版的提交
            var submit = LeadSource[CurrentLeadIndex].PageContent as TellerSystem.Controls.Ext.TradePageBase.SubmitPage;
            if (submit != null)
            {
                submit.Loaded -= (sender, e) => { };
                submit.Loaded += (sender, e) =>
                {
                    if (submit.SubmitButton != null)
                    {
                        submit.SubmitButton.Visibility = Visibility.Hidden;
                    }
                };
                lv_Leads.Content = submit;
            }
            else
            {
                lv_Leads.Content = LeadSource[CurrentLeadIndex].PageContent;
            }
            //解决databox弹出窗时，trademodel为空的问题
            var model = new TradeModel()
            {
                Type = "trade",
            };
            PageHelper.SetTradeModel(lv_Leads.Content as DependencyObject, model);
            FleshPage();
        }

        /// <summary>
        /// 刷新流程显示
        /// </summary>
        private void FleshThemes()
        {
            var parent = GetParentObject<ContentPresenter>(lv_Leads, "tradepageviewer");
            if (parent != null) parent.Width = lv_Leads.Width;

            //按钮
            btnPrevious.IsEnabled = true;
            if (CurrentLeadIndex == 0) btnPrevious.IsEnabled = false;
            if (CurrentLeadIndex == LeadSource.Count - 1) btnNext.Content = "完成";
            if (CurrentLeadIndex > 0)
            {
                btnPrevious.IsEnabled = true;
            }
            if (CurrentLeadIndex < LeadSource.Count - 1)
            {
                btnNext.Content = "下一步";
            }

            #region 处理标题滚动

            var i = 0;
            //标题
            LeadHeader.Children.Clear();
            foreach (var item in LeadSource)
            {
                var tb = new TextBlock
                 {
                     Text = string.IsNullOrEmpty(item.Description) ? " " : item.Description,
                     Margin = new Thickness(8, 0, 0, 0),
                     FontSize = 14,
                     //Height = 30,
                     Padding = new Thickness(2, 5, 2, 5),
                     //Background = Brushes.Blue,
                     Foreground = Brushes.Blue,
                 };
                TextBlock txt = new TextBlock();
                txt.FontSize = 20;
                txt.Text = " >";
                //i++;
                if (CurrentLeadIndex == i++)
                {
                    tb.Background = Brushes.Blue;
                    tb.Foreground = Brushes.White;
                }

                if (!_title[i - 1])
                {
                    //前面需要隐藏时，只处理一次。并继续下一步操作
                    if (i - 1 < CurrentLeadIndex)
                    {
                        if (_title[i])
                        {
                            tb.Text = "...";
                            LeadHeader.Children.Add(tb);
                        }
                        continue;
                    }
                    //后面需要隐藏时，只处理一次。并中断
                    else
                    {
                        tb.Text = "...";
                        LeadHeader.Children.Add(tb);
                        break;
                    }
                }

                LeadHeader.Children.Add(tb);
                if (i != _title.Length)
                {
                    LeadHeader.Children.Add(txt);
                }
            }

            #endregion 处理标题滚动
        }

        /// <summary>
        /// 记录动态变化标题
        /// </summary>
        private void InitTitle(Direction direction)
        {
            var txtLen = LeadHeader.ActualWidth - 16;

            if (CurrentLeadIndex == 0)
            {
                for (var i = 0; i < LeadSource.Count; i++)
                {
                    _title[i] = true;
                    txtLen -= SystemFunction.Len(string.IsNullOrEmpty(LeadSource[i].Description) ? " " : LeadSource[i].Description) * 7 + 12;
                    if (txtLen < 0)
                    {
                        _title[i] = false;
                    }
                }
            }
            else
            {
                var num = 0;
                if (direction == Direction.Next)
                {
                    for (var i = 0; i < _title.Length; i++)
                    {
                        if (_title[i]) num++;
                        if (!_title[i] && CurrentLeadIndex == i - 2)
                        {
                            _title[i - num] = false;
                            _title[i] = true;
                            break;
                        }
                    }
                }
                else if (direction == Direction.Up)
                {
                    for (var j = _title.Length - 1; j >= 0; j--)
                    {
                        if (_title[j]) num++;
                        if (!_title[j] && CurrentLeadIndex == j + 2)
                        {
                            _title[j + num] = false;
                            _title[j] = true;
                            break;
                        }
                    }
                }
            }
        }

        /// <summary>
        ///  刷新焦点列表
        /// </summary>
        private void FleshPage()
        {
            var page = this.FindVisualTreeAncestor(x => x is IPage);
            if (page != null)
            {
                this.Dispatcher.BeginInvoke(new Action(() =>
                {
                    var fm = Platform.Presentation.FocusManagers.FocusManager.GetManagerInstance(page);
                    if (fm != null)
                    {
                        fm.Refresh();
                    }
                }), System.Windows.Threading.DispatcherPriority.Render, null);
            }
        }

        /// <summary>
        /// 查找父节点
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="obj"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        public T GetParentObject<T>(DependencyObject obj, string name = "") where T : FrameworkElement
        {
            DependencyObject parent = VisualTreeHelper.GetParent(obj);
            while (parent != null)
            {
                if (parent is T && ((T)parent).Name.ToLower() == name)
                {
                    return (T)parent;
                }
                parent = VisualTreeHelper.GetParent(parent);
            }
            return null;
        }

        #endregion 事件
    }

    public class LeadTradeViewerCtl
    {
        /// <summary>
        /// 向导容器
        /// </summary>
        private LeadTradeViewer _lt;

        public LeadTradeViewer lt
        {
            get { return _lt; }
            set { _lt = value; }
        }

        /// <summary>
        /// 设置或获取下一步按钮的可用性
        /// </summary>
        public bool IsNextBtnEnable
        {
            set
            {
                if (_lt != null)
                {
                    _lt.btnNext.IsEnabled = value;
                }
            }
        }

        /// <summary>
        /// 如果新增加key，则保存对象
        /// 如果已经存在key对应的对象，则修改
        /// </summary>
        /// <param name="key"></param>
        /// <param name="xml_read_handle"></param>
        public void SetItemXmlReaderMgr(string key, XML_reader_mgr xml_read_handle)
        {
            if (_lt != null)
            {
                _lt.SetItemXmlReaderMgr(key, xml_read_handle);
            }
        }

        /// <summary>
        /// 放置数据
        /// </summary>
        /// <param name="key"></param>
        /// <param name="obj"></param>
        public void SetDataBus(string key, object obj)
        {
            //只在流程控制中使用
            if (_lt != null)
            {
                _lt.DataBus.SetDictionaryItem(key, obj);
            }
        }

        public object GetDataBus(string key)
        {
            if (_lt.DataBus.ContainsKey(key))
            {
                return lt.DataBus[key];
            }
            else
            {
                return "";
            }
        }
    }

    public class LeadItem
    {
        /// <summary>
        /// 页面路径
        /// </summary>
        public ContentControl PageContent { get; set; }

        /// <summary>
        /// 描述
        /// </summary>
        public string Description { get; set; }
    }

    public enum Direction
    {
        Next,
        Up,
        Jump,
        None,
    }
}