// 为了在Vue中模拟WPF的功能，这里需要一些浏览器的API来替代
// 注意：在Vue中，没有直接对应的Visual和DependencyObject概念，这里使用Element和HTMLElement来模拟

// 获取DPI因子
export const getDpiFactor = (target: HTMLElement): number => {
    const source = window.document.defaultView;
    if (!source) {
        return 1.0;
    }
    const devicePixelRatio = window.devicePixelRatio || 1;
    return 1 / devicePixelRatio;
};

// 获取指定类型的祖先元素
export const getAncestor = <T extends HTMLElement>(target: HTMLElement): T | null => {
    let parent = target.parentElement;
    if (parent instanceof HTMLElement) {
        if (parent instanceof HTMLElement as unknown as T) {
            return parent as T;
        }
        return getAncestor<T>(parent);
    }
    return null;
};

// 获取指定类型的后代元素
export const getDesendentChild = <T extends HTMLElement>(target: HTMLElement): T | null => {
    const childCount = target.children.length;
    if (childCount === 0) {
        return null;
    }
    for (let i = 0; i < childCount; i++) {
        const current = target.children[i] as HTMLElement;
        if (current instanceof HTMLElement as unknown as T) {
            return current as T;
        }
        const desendent = getDesendentChild<T>(current);
        if (desendent) {
            return desendent;
        }
    }
    return null;
};