﻿/*
 * 消息弹出
 * 2014、12、25
 * 郝成阳
 * 
 * 
 * */
using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media.Animation;
using System.Windows.Media;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// 消息弹出
    /// </summary>
    public partial class TellerMessageBox : INotifyPropertyChanged
    {
        
        private bool _animationRan;

        public TellerMessageBox(string message, MessageBoxButton button, MessageBoxImage icon, string[] buttonConent)
        {
            _animationRan = false;

            InitializeComponent();

            Owner = Application.Current.MainWindow;

            CreateButtons(button, buttonConent);

            CreateImage(icon);

            MessageText.Text = message;
        }
        /// <summary>
        /// 提示消息返回值
        /// </summary>
        public MessageBoxResult MessageBoxResult { get; set; }

        #region INotifyPropertyChanged 成员

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region 创建按钮

        /// <summary>
        /// 根据需求创建消息框的按钮
        /// </summary>
        /// <param name="button">按钮选择</param>
        /// <param name="defaultResult"></param>
        private void CreateButtons(MessageBoxButton button, string[] buttonConent)
        {
            switch (button)
            {
                case MessageBoxButton.OK:
                    ButtonsPanel.Children.Add(CreateButton(buttonConent[0], MessageBoxResult.OK));
                    break;
                case MessageBoxButton.OKCancel:
                    ButtonsPanel.Children.Add(CreateButton(buttonConent[0], MessageBoxResult.OK));
                    ButtonsPanel.Children.Add(CreateButton(buttonConent[1], MessageBoxResult.Cancel));
                    break;
                case MessageBoxButton.YesNoCancel:
                    ButtonsPanel.Children.Add(CreateButton(buttonConent[0], MessageBoxResult.Yes));
                    ButtonsPanel.Children.Add(CreateButton(buttonConent[1], MessageBoxResult.No));
                    ButtonsPanel.Children.Add(CreateButton(buttonConent[2], MessageBoxResult.Cancel));
                    break;
                case MessageBoxButton.YesNo:
                    ButtonsPanel.Children.Add(CreateButton(buttonConent[0], MessageBoxResult.Yes));
                    ButtonsPanel.Children.Add(CreateButton(buttonConent[1], MessageBoxResult.No));
                    break;
                default:
                    throw new ArgumentOutOfRangeException("button");
            }
        }

        /// <summary>
        /// 创建按钮
        /// </summary>
        /// <param name="defaultResult"></param>
        /// <param name="content"></param>
        /// <returns></returns>
        private Button CreateButton(string content, MessageBoxResult result)
        {
            var okButton = new Button
                             {
                                 Name = "okButton",
                                 Content = content,
                                 IsDefault = true,
                                 Tag = result,
                             };

            okButton.Click += ButtonClick;

            return okButton;
        }
        /// <summary>
        /// 按钮触发事件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonClick(object sender, RoutedEventArgs e)
        {
            MessageBoxResult = (MessageBoxResult)(sender as Button).Tag;

            Close();
        }

        #endregion



        /// <summary>
        /// 创建图标图像
        /// </summary>
        /// <param name="icon"></param>
        private void CreateImage(MessageBoxImage icon)
        {
            switch (icon)
            {
                case MessageBoxImage.None:
                    ImagePlaceholder.Visibility = Visibility.Collapsed;
                    break;
                case MessageBoxImage.Information:
                    ImagePlaceholder.Source = Application.Current.Resources["MessageBoxMetroInfo"] as ImageSource;
                    break;
                case MessageBoxImage.Question:
                    ImagePlaceholder.Source = Application.Current.Resources["MessageBoxMetroInfo"] as ImageSource;
                    break;
                case MessageBoxImage.Warning:
                    ImagePlaceholder.Source = Application.Current.Resources["MessageBoxMetroWarn"] as ImageSource;
                    break;
                case MessageBoxImage.Error:
                    ImagePlaceholder.Source = Application.Current.Resources["MessageBoxMetroError"] as ImageSource;
                    break;
                default:
                    throw new ArgumentOutOfRangeException("icon");
            }
        }

        public void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler temp = PropertyChanged;
            if (temp != null)
            {
                temp(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        /// <summary>
        ///拖动
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_MouseDown(object sender, MouseButtonEventArgs e)
        {
            DragMove();
        }

        /// <summary>
        /// 加载显示动画
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // 这是在设定高度后的宽度
            SizeToContent = SizeToContent.Height;

            var animation = TryFindResource("LoadAnimation") as Storyboard;

            animation.Begin(this);
        }

        /// <summary>
        /// 关闭动画显示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MessageBoxWindow_Closing(object sender, CancelEventArgs e)
        {
            if (!_animationRan)
            {
                e.Cancel = true;

                var animation = TryFindResource("UnloadAnimation") as Storyboard;

                animation.Completed += AnimationCompleted;

                animation.Begin(this);
            }
        }

        /// <summary>
        /// 信号关闭动画并关闭窗口
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AnimationCompleted(object sender, EventArgs e)
        {
            _animationRan = true;

            Close();
        }

        #region 消息提醒
        /// <summary>
        /// 消息提醒
        /// </summary>
        /// <param name="message"></param>
        /// <returns></returns>
        public static MessageBoxResult ShowInfoMessageBox(string message)
        {
            return Show(message, new []{"确定"},MessageBoxButton.OK,
                       MessageBoxImage.Information, MessageBoxResult.OK);
        }
        /// <summary>
        /// 自定义消息提醒
        /// </summary>
        /// <param name="message"></param>
        /// <param name="details"></param>
        /// <param name="showCancel"></param>
        /// <param name="options"></param>
        /// <returns></returns>
        public static MessageBoxResult ShowInfoMessageBox(string message, string[] buttonConent, MessageBoxButton button)
        {
            return Show(message, buttonConent, button,
                       MessageBoxImage.Information, MessageBoxResult.OK);
        }
        #endregion

        #region 询问消息提醒
        /// <summary>
        /// 询问消息
        /// </summary>
        /// <param name="message"></param>
        /// <param name="details"></param>
        /// <param name="showCancel"></param>
        /// <param name="options"></param>
        /// <returns></returns>
        public static MessageBoxResult ShowQuestion(string message, string[] buttonConent, MessageBoxButton button)
        {
            return Show(message, buttonConent, button,
                     MessageBoxImage.Question, MessageBoxResult.Yes);
        }
        #endregion

        #region 警告消息

        /// <summary>
        /// 提示警告消息
        /// </summary>
        /// <param name="message"></param>
        /// <param name="details"></param>
        /// <param name="showCancel"></param>
        /// <param name="options"></param>
        /// <returns></returns>
        public static MessageBoxResult ShowWarning(string message, string[] buttonConent, MessageBoxButton button)
        {
            return Show(message, buttonConent, button,
                        MessageBoxImage.Warning, MessageBoxResult.OK);
        }
        #endregion

        #region 错误信息
        /// <summary>
        /// 错误消息提示
        /// </summary>
        /// <param name="message"></param>
        /// <param name="details"></param>
        /// <param name="showCancel"></param>
        /// <param name="options"></param>
        /// <returns></returns>
        public static MessageBoxResult ShowError(string message, MessageBoxButton button = MessageBoxButton.OK)
        {
            return Show(message, new string[] { "确定" }, button,
                       MessageBoxImage.Error, MessageBoxResult.OK);
        }
        #endregion

        #region 弹出
        /// <summary>
        /// 显示消息对话框
        /// </summary>
        /// <param name="owner">该消息框的父窗口</param>
        /// <param name="message">消息文本</param>
        /// <param name="button">要显示的按钮</param>
        /// <param name="icon">消息图标</param>
        /// <param name="defaultResult">默认返回值</param>
        /// <param name="options">消息类型</param>
        /// <returns></returns>
        public static MessageBoxResult Show(string message, string[] buttonConent,
                                            MessageBoxButton button,
                                            MessageBoxImage icon = MessageBoxImage.None,
                                            MessageBoxResult defaultResult = MessageBoxResult.None)
        {
            return ShowDialog(message, buttonConent, button, icon);
        }

        /// <summary>
        /// 显示指定的参数消息框
        /// </summary>
        /// <param name="owner">该消息框的父窗口</param>
        /// <param name="message">消息文本</param>
        /// <param name="details">其他文本</param>
        /// <param name="button">要显示的按钮</param>
        /// <param name="icon">消息图标</param>
        /// <param name="defaultResult">默认返回值</param>
        /// <param name="options">消息类型</param>
        /// <returns>选择的按钮</returns>
        public static MessageBoxResult ShowDialog(string message, string[] buttonConent,
                                            MessageBoxButton button, MessageBoxImage icon = MessageBoxImage.None)
        {
            var result = Application.Current.Dispatcher.Invoke(new Func<MessageBoxResult>(() =>
            {
                var messageBox = new TellerMessageBox(message, button, icon, buttonConent);

                messageBox.ShowDialog();

                return messageBox.MessageBoxResult;
            }));

            return result is MessageBoxResult ? (MessageBoxResult)result : MessageBoxResult.None;
        }

        #endregion
    }
}