﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using TellerSystem.Library.Ext.TradeExtension;
using TellerSystem.Library.Ext.Function;
namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// 省市区控件
    /// </summary>
    public partial class ProvinceCityCounty : UserControl
    {
        public ProvinceCityCounty()
        {
            InitializeComponent();
            this.Content.DataContext = this;
        }
        /// <summary>
        /// 省市区控件数据源
        /// </summary>
        private IList<Province> _provinces = new List<Province>();

        public static readonly DependencyProperty ProvinceProperty = DependencyProperty.Register("Province", typeof(string), typeof(ProvinceCityCounty), new FrameworkPropertyMetadata(string.Empty));
        public static readonly DependencyProperty CityProperty = DependencyProperty.Register("City", typeof(string), typeof(ProvinceCityCounty), new FrameworkPropertyMetadata(string.Empty));
        public static readonly DependencyProperty CountyProperty = DependencyProperty.Register("County", typeof(string), typeof(ProvinceCityCounty), new FrameworkPropertyMetadata(string.Empty));

        /// <summary>
        /// 省市区控件数据源
        /// </summary>
        public IList<Province> Provinces
        {
            get
            {
                return _provinces;
            }
            set
            {
                _provinces.Clear();
                if (value != null)
                {
                    foreach (Province province in value)
                    {
                        _provinces.Add(province);
                    }
                    if(_provinces.FirstOrDefault() != null)
                    {
                        Province = _provinces.FirstOrDefault().Value;
                    }
                }
            }
        }

        /// <summary>
        /// 省份
        /// </summary>
        public string Province
        {
            get
            {
                return (string)GetValue(ProvinceProperty);
            }
            set
            {
                SetValue(ProvinceProperty, value);
            }
        }
        /// <summary>
        /// 市
        /// </summary>
        public string City
        {
            get
            {
                return (string)GetValue(CityProperty);
            }
            set
            {
                SetValue(CityProperty, value);
            }
        }
        /// <summary>
        /// 县区
        /// </summary>
        public string County
        {
            get
            {
                return (string)GetValue(CountyProperty);
            }
            set
            {
                SetValue(CountyProperty, value);
            }
        }

        private void ComboBoxProvince_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (string.IsNullOrEmpty(City) && ComboBoxProvince.SelectedItem != null && (ComboBoxProvince.SelectedItem as Province).Child.Count > 0)
            {
                City = (ComboBoxProvince.SelectedItem as Province).Child[0].Value;
            }
        }

        private void ComboBoxCity_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (string.IsNullOrEmpty(County) && ComboBoxCity.SelectedItem != null && (ComboBoxCity.SelectedItem as City).Child.Count > 0)
            {
                County = (ComboBoxCity.SelectedItem as City).Child[0].Value;
            }
        }
    }
}
