﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Linq;
using System.Reflection;
using System.ComponentModel;
using System.Windows.Markup;
using System.Collections.ObjectModel;

namespace TellerSystem.Controls.Ext.Controls
{
	/// <summary>
	/// StateProgress.xaml 的交互逻辑
	/// </summary>
    [DefaultProperty("Operates")]
    [ContentProperty("Operates")]
	public partial class StateProgress : UserControl
	{
       	public StateProgress()
		{
			this.InitializeComponent();
            this.Loaded += (s, e) => Update();
        }

        ObservableCollection<TextBlock> _operates;

        /// <summary>
        /// 获取操作区域
        /// </summary>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public ObservableCollection<TextBlock> Operates
        {
            get
            {
                if (this._operates == null)
                {
                    this._operates = new ObservableCollection<TextBlock>();
                }
                return this._operates;
            }
        }

        /// <summary>
        /// 获取或设置当前信息
        /// </summary>
        public string CurrentMessage
        {
            get { return tbMsg.Text; }
            set 
            {
                tbMsg.Text = value;
            }
        }


        private int _distance = 90;

        /// <summary>
        /// 获取或设置距离
        /// </summary>
        public int Distance
        {
            get { return _distance; }
            set 
            {
                if (_distance != value)
                {
                    _distance = value;
                    Update();
                }
            }
        }

        /// <summary>
        /// 获取或设置使用横向还是纵向
        /// </summary>
        public Orientation Orientation
        {
            get { return stackPanel.Orientation; }
            set 
            {
                if (stackPanel.Orientation != value)
                {
                    stackPanel.Orientation = value;
                    Update();
                }
            }
        }

        private int _currentProgress;

        /// <summary>
        /// 获取或设置当前进度
        /// </summary>
        public int CurrentProgress
        {
            get { return _currentProgress; }
            set
            {
                if (value < 0 || value > _stateCount)
                {
                    throw new ArgumentException("当前进度必须大于0,小于状态数", "CurrentProgress");
                }

                if (_currentProgress != value)
                {
                    _currentProgress = value;
                    Update();
                }
            }
        }
        

        private int _stateCount = 4;

        /// <summary>
        /// 获取或设置状态数
        /// </summary>
        public int StateCount
        {
            get { return _stateCount; }
            set 
            {
                if (value < 2 || value > 9)
                {
                    throw new ArgumentException("状态数量只能在2-9之间", "StateCount");
                }

                if (_stateCount != value)
                {
                    _stateCount = value;
                    Update();
                }
            }
        }

        void Update()
        {
            if (Operates.Count != 0 && _stateCount != Operates.Count)
            {
                throw new InvalidOperationException("操作区数量必须与状态数量一致");
            }
            int margin = _distance / 2;
            Thickness thick = new Thickness(margin, 0, margin, 0);

            if (Orientation == System.Windows.Controls.Orientation.Horizontal)
            {
                thick = new Thickness(margin, 0, margin, 0);

                backPath.HorizontalAlignment = System.Windows.HorizontalAlignment.Left;
                backPath.VerticalAlignment = System.Windows.VerticalAlignment.Center;

                backPath.Margin = new Thickness(margin, 0, 0, 0);

                backPath.Width = (28 + _distance) * (_stateCount - 1) + 4;
                backPath.Height = 6;

                forePath.HorizontalAlignment = System.Windows.HorizontalAlignment.Left;
                forePath.VerticalAlignment = System.Windows.VerticalAlignment.Center;

                forePath.Margin = new Thickness(margin, 0, 0, 0);

                if (_currentProgress == _stateCount)
                {
                    forePath.Width = backPath.Width;
                }
                else
                {
                    var foreWidth = (28 + _distance) * (_currentProgress - 1) + margin + 28;
                    forePath.Width = foreWidth < 0 ? 0 : foreWidth;
                }
                
                forePath.Height = 2;

                if (_currentProgress == 0)
                {
                    tbMsg.Visibility = System.Windows.Visibility.Collapsed;
                }
                else
                {
                    tbMsg.Visibility = System.Windows.Visibility.Visible;
                    tbMsg.Margin = new Thickness((28 + _distance) * (_currentProgress - 1), 0, 0, 0);
                }
            }
            else
            {
                rootGrid.RowDefinitions.Clear();

                thick = new Thickness(0, margin, 0, margin);

                backPath.HorizontalAlignment = System.Windows.HorizontalAlignment.Center;
                backPath.VerticalAlignment = System.Windows.VerticalAlignment.Top;

                backPath.Margin = new Thickness(0, margin, 0, 0);

                backPath.Height = (28 + _distance) * (_stateCount - 1) + 4;
                backPath.Width = 6;

                forePath.HorizontalAlignment = System.Windows.HorizontalAlignment.Center;
                forePath.VerticalAlignment = System.Windows.VerticalAlignment.Top;

                forePath.Margin = new Thickness(0, margin, 0, 0);

                if (_currentProgress == _stateCount)
                {
                    forePath.Height = backPath.Height;
                }
                else
                {
                    var foreHeight = (28 + _distance) * (_currentProgress - 1) + margin + 28;
                    forePath.Height = foreHeight < 0 ? 0 : foreHeight;
                }
                
                forePath.Width = 2;

                if (_currentProgress == 0)
                {
                    tbMsg.Visibility = System.Windows.Visibility.Collapsed;
                }
                else
                {
                    tbMsg.Visibility = System.Windows.Visibility.Visible;
                    tbMsg.Margin = new Thickness(0, (28 + _distance) * (_currentProgress - 1), 0, 0);
                }
            }

            stackPanel.Children.Clear();

            for (int i = 0; i < _stateCount; i++)
            {
                PairStateSign sign = new PairStateSign();
                sign.CurrentState = i + 1;
                sign.Margin = thick;

                sign.CurrentState = i < _currentProgress ? -1 : i + 1;

                stackPanel.Children.Add(sign);

                if (i < Operates.Count)
                {
                    var tb = Operates[i] as TextBlock;
                    tb.IsEnabled = i == _currentProgress;
                }
            }
        }

        public override void OnApplyTemplate()
        {
            stackPanelText.Children.Clear();

            foreach (var tb in Operates)
            {
                tb.TextAlignment = TextAlignment.Center;
                tb.Width = 28 + _distance;
                tb.TextWrapping = TextWrapping.Wrap;

                stackPanelText.Children.Add(tb);
            }

            base.OnApplyTemplate();
        }
	}
}