﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TellerSystem.Controls.Ext.ControlsHelper;
using TellerSystem.Contract.ContractBase;
using System.ComponentModel;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// TBC_Curreny.xaml 的交互逻辑
    /// </summary>
    public partial class TBC_Curreny
    {
        public TBC_Curreny()
        {
            InitializeComponent();
            selectName = string.Empty;
            selectValue = string.Empty;
        }
        /// <summary>
        /// 父页面对象
        /// </summary>
        public PageBase ParentPage { get; private set; }
        public static readonly RoutedEvent TBCLostFocus = EventManager.RegisterRoutedEvent("TBCLostFocus", RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(TBC_Curreny));
        public event RoutedEventHandler TBCLostFocusEvent
        {
            add { base.AddHandler(TBCLostFocus, value); }

            remove { base.RemoveHandler(TBCLostFocus, value); }

        }

        #region 属性
        /// <summary>
        /// 设置或获取值的必输性，true为必输
        /// </summary>
        public bool IsNotEmpty
        {
            get { return (bool)GetValue(IsNotEmptyProperty); }
            set { SetValue(IsNotEmptyProperty, value); }
        }

        public static readonly DependencyProperty IsNotEmptyProperty =
            DependencyProperty.Register("IsNotEmpty", typeof(bool), typeof(DateSpanControl), new UIPropertyMetadata(false));
        /// <summary>
        /// 弹出窗口的标题
        /// </summary>
        public string WinTitle
        {
            get { return (string)GetValue(WinTitleProperty); }
            set { SetValue(WinTitleProperty, value); }
        }

        public static readonly DependencyProperty WinTitleProperty =
            DependencyProperty.Register("WinTitle", typeof(string), typeof(DateSpanControl), new UIPropertyMetadata(""));
    
        /// <summary>
        /// 设置要选中的值
        /// </summary>
        public int SetValue
        {
            get { return (int)GetValue(SetValueProperty); }
            set { SetValue(SetValueProperty, value); }
        }

        public static readonly DependencyProperty SetValueProperty =
            DependencyProperty.Register("SetValue", typeof(int), typeof(DateSpanControl), new UIPropertyMetadata(-1));
        /// <summary>
        /// 设置选中的名称
        /// </summary>
        public int SetName
        {
            get { return (int)GetValue(SetNameProperty); }
            set { SetValue(SetNameProperty, value); }
        }

        // Using a DependencyProperty as the backing store for MyProperty.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SetNameProperty =
            DependencyProperty.RegisterAttached("SetName", typeof(int), typeof(DateSpanControl), new UIPropertyMetadata(2));


        /// <summary>
        /// 附件
        /// </summary>
        public string FileData;
        /// <summary>
        /// 列集合
        /// </summary>
        public string[] Columns;
        private string selectValue;
        /// <summary>
        /// 选中的值
        /// </summary>
        public string SelectValue
        {
            get { return selectValue; }
            set
            {
                selectValue = value;
                Tx_Value.Text = selectValue;
            }
        }

        private string selectName;
        /// <summary>
        /// 选中的名称
        /// </summary>
        public string SelectName
        {
            get { return selectName; }
            set
            {
                selectName = value;
                Tx_Name.Text = selectName;
                Tx_Name.Text = (SelectValue == null||selectValue==string.Empty) ? SelectName : "[" + SelectValue + "]" + SelectName;
            }
        }

        #endregion
        /// <summary>
        /// 查询
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Button_cur_no_Search_Click(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(FileData))
            {
                this.TradeAPIs.ShowInfoMessageBox("选择窗口源为空");
                this.TradeAPIs.SetFocus(Tx_Name);
                return; 
            }
            var selectedData = ParentPage.ShowDataBoxViewer(FileData, "Result", Columns, 550, 300, WinTitle);
            this.Tx_Name.Text = string.Empty;
            if (selectedData == null && IsNotEmpty)
            { 
                this.TradeAPIs.ShowInfoMessageBox("不能为空");
                this.TradeAPIs.SetFocus(Tx_Name);
                return; 
            }
            var d = selectedData as System.Data.DataRow;
            if (d != null && d.ItemArray.Length > SetName && d.ItemArray.Length > SetValue)
            {
                SelectValue = SetValue >= 0 ? selectedData[SetValue].ToString() : null;
                SelectName = selectedData[SetName].ToString();
                Tx_Name.Text = SelectValue == null ? SelectName : "[" + SelectValue + "]" + SelectName;
            }
        }
        /// <summary>
        /// 焦点离开
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Button_cur_no_Search_LostFocus(object sender, RoutedEventArgs e)
        {
            if (IsNotEmpty && this.Tx_Name.Text.Trim() == string.Empty)
            {
                this.TradeAPIs.ShowInfoMessageBox("不能为空");
                this.TradeAPIs.SetFocus(Button_cur_no_Search);
                return;
            }
            RaiseEvent(new RoutedEventArgs(TBC_Curreny.TBCLostFocus, this));
            
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            if (DesignerProperties.GetIsInDesignMode(this)) return;//设计时不加载Load
            ParentPage = (Platform.Common.BasicFunctions.DependencyObjectExtensions.FindVisualTreeAncestor
                (this, (x) => { return x is PageBase; })) as PageBase;
        }
    }
}
