﻿using System;
using System.Data;
using System.Linq;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media;
using System.Text;
using System.ComponentModel;
namespace TellerSystem.Controls.Ext.Controls
{
    public class DataGridTreeViewItem : TreeViewItem, INotifyPropertyChanged
   {
       
       private bool _ischecked=false;

       public bool IsChecked
       {
           get { return _ischecked; }
           set
           {
               _ischecked = value;
               RaisePropertyChanged("IsChecked");
               
               if (this.HasItems)
               {
                   if (value)
                   {
                       this.ExpandSubtree();
                   }
                   else
                   {
                       this.IsExpanded = false;
                   }

                   this.Items.Cast<object>().ToList().ForEach(c =>
                       {
                           if ((this.ItemContainerGenerator.ContainerFromItem(c) as DataGridTreeViewItem).IsHold)
                           {
                               (this.ItemContainerGenerator.ContainerFromItem(c) as DataGridTreeViewItem).IsChecked = value;
                           }
                       });
               }
           }
       }



       public bool IsHold
       {
           get { return (bool)GetValue(IsHoldProperty); }
           set { SetValue(IsHoldProperty, value); }
       }

       // Using a DependencyProperty as the backing store for IsHold.  This enables animation, styling, binding, etc...
       public static readonly DependencyProperty IsHoldProperty =
           DependencyProperty.Register("IsHold", typeof(bool), typeof(DataGridTreeViewItem), new UIPropertyMetadata(true));

       
       
      private int _level = -1;

      /// <summary>
      /// 获取层次等级
      /// </summary>
      public int Level
      {
         get
         {
            if (_level == -1)
            {
                DataGridTreeViewItem parent = ItemsControl.ItemsControlFromItemContainer(this) as DataGridTreeViewItem;
               _level = (parent != null) ? parent.Level + 1 : 0;
            }
            return _level;
         }
      }

      public DataGridTreeViewItem ParentItem 
      {
          get
          {
              DependencyObject dpobj = null;
              for (dpobj = VisualTreeHelper.GetParent(this); dpobj != null && !(dpobj is DHCCTreeViewList)&&!(dpobj is DataGridTreeViewItem); dpobj = VisualTreeHelper.GetParent(dpobj));

              if (dpobj is DHCCTreeViewList)
              {
                  return null;
              }

              return dpobj as DataGridTreeViewItem;
          }
      }

      private void SetIsChecked(DataGridTreeViewItem item, bool value)
      {
          item._ischecked = value;
          item.RaisePropertyChanged("IsChecked");
      }

      /// <summary>
      /// 设置状态
      /// </summary>
      /// <param name="value"></param>
      /// <param name="updateChildren"></param>
      /// <param name="updateParent"></param>
      private void SetCheckState(bool value, bool updateChildren, bool updateParent)
      {
          if (value != IsChecked)
          {
              _ischecked = value;
          }

          if (updateChildren)
          {
              if (HasItems)
              {
                  this.ExpandSubtree();
                  this.Items.Cast<object>().ToList().ForEach(c => (this.ItemContainerGenerator.ContainerFromItem(c) as DataGridTreeViewItem).SetCheckState(value, true, false));
              }
          }

          RaisePropertyChanged("IsChecked");

          if (updateParent)
          {
              VerifyTreeState(ParentItem);
          }
      }

      private void VerifyTreeState(DataGridTreeViewItem item)
      {
          if (item==null)
          {
              return;
          }

          if (item.Items.Cast<object>().ToList().All(c=>(item.ItemContainerGenerator.ContainerFromItem(c) as DataGridTreeViewItem).IsChecked==true))
          {
              SetIsChecked(item, true);
              item.IsExpanded = true;
          }
          else if  (item.Items.Cast<object>().ToList().All(c=>(item.ItemContainerGenerator.ContainerFromItem(c) as DataGridTreeViewItem).IsChecked==false))
          {
              SetIsChecked(item, false);
              item.IsExpanded = false;
          }
          //else if (Node.IsChecked != null)
          //{
          //    SetIsChecked(Node, null);
          //    Node.IsExpanded = true;
          //}

         VerifyTreeState(item.ParentItem);

      }

      protected override DependencyObject GetContainerForItemOverride()
      {
          var li = new DataGridTreeViewItem {OverridesDefaultStyle = true};
          return li;
      }

      protected override bool IsItemItsOwnContainerOverride(object item)
      {
          return item is DataGridTreeViewItem;
      }

      public event PropertyChangedEventHandler PropertyChanged;

      public void RaisePropertyChanged(string propertyName)
      {
          if (this.PropertyChanged != null)
          {
              this.PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyName));
          }
      }
   }
}
