﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection.Emit;
using System.Reflection;
using System.Data;
using System.Collections;

namespace TellerSystem.Controls.Ext.Controls
{
   internal static class DynamicClassHelp
   {
      /// <summary>
      /// 动态创建类型
      /// 给定DataTable所有列为string类型属性，添加Children属性
      /// 例如：给定DataTable有三列col1,col2,col3，那么得到的类型有四个属性，
      /// 分别为col1(string),col2(string),col3(string),Children(object[])
      /// 有四个字段，分别为m_strcol1(string),m_strcol2(string),m_strcol3(string),m_Children(object[])
      /// </summary>
      /// <param name="dt">数据源</param>
      /// <returns></returns>
      internal static Type CreateDynamicPropertyClass(DataTable dt)
      {
         string strDynamicAssemblyName = "RunTimeClassAssembly";
         string strDynamicModuleName = "RunTimeModule";
         string strDynamicClassName = "RunTimeClass" + DateTime.Now.Ticks.ToString();//类名不重复
         string strPropertyName = string.Empty;

         Type typeDynamic = null;
         AppDomain currentDomain = null;
         TypeBuilder typeBuilder = null;
         ILGenerator ilGenerator = null;
         AssemblyName assemblyName = null;
         FieldBuilder fieldBuilder = null;
         ModuleBuilder moduleBuilder = null;
         MethodBuilder methodBuilder = null;
         PropertyBuilder propertyBuilder = null;
         AssemblyBuilder assemblyBuilder = null;
         MethodAttributes getSetAttr;
         Type[] methodArgs = { typeof(string) };
         IEnumerator objAttributes = null;

         currentDomain = AppDomain.CurrentDomain;
         assemblyName = new AssemblyName();
         assemblyName.Name = strDynamicAssemblyName;

         assemblyBuilder = currentDomain.DefineDynamicAssembly(assemblyName, AssemblyBuilderAccess.Run);
         moduleBuilder = assemblyBuilder.DefineDynamicModule(strDynamicModuleName);
         typeBuilder = moduleBuilder.DefineType(strDynamicClassName, TypeAttributes.Public);

         if (dt != null)
         {
            objAttributes = dt.Columns.GetEnumerator();
            while (objAttributes.MoveNext())
            {
               DataColumn col = (DataColumn)objAttributes.Current;

               strPropertyName = col.ColumnName;
               //添加字段，全部使用String类型
               fieldBuilder = typeBuilder.DefineField("m_str_" + strPropertyName, typeof(string), FieldAttributes.Public);
               propertyBuilder = typeBuilder.DefineProperty(strPropertyName, System.Reflection.PropertyAttributes.HasDefault, typeof(string), null);
               getSetAttr = MethodAttributes.Public | MethodAttributes.SpecialName | MethodAttributes.HideBySig;
               //属性get方法
               methodBuilder = typeBuilder.DefineMethod("get_" + strPropertyName, getSetAttr, typeof(string), Type.EmptyTypes);
               ilGenerator = methodBuilder.GetILGenerator();
               ilGenerator.Emit(OpCodes.Ldarg_0);
               ilGenerator.Emit(OpCodes.Ldfld, fieldBuilder);
               ilGenerator.Emit(OpCodes.Ret);
               propertyBuilder.SetGetMethod(methodBuilder);

               //属性set方法
               methodBuilder = typeBuilder.DefineMethod("set_" + strPropertyName,
                  MethodAttributes.Public | MethodAttributes.HideBySig | MethodAttributes.SpecialName,
                  typeof(void), methodArgs);
               ilGenerator = methodBuilder.GetILGenerator();
               ilGenerator.Emit(OpCodes.Ldarg_0);
               ilGenerator.Emit(OpCodes.Ldarg_1);
               ilGenerator.Emit(OpCodes.Stfld, fieldBuilder);
               ilGenerator.Emit(OpCodes.Ret);
               propertyBuilder.SetSetMethod(methodBuilder);
            }
         }

         //添加Children属性，类型为object[]，
         //可以直接指定当前类数组，使用typeBuilder.UnderlyingSystemType.MakeArrayType()
         //但是后续无法进行强类型转换，故直接使用object[]
         strPropertyName = "Children";
         fieldBuilder = typeBuilder.DefineField("m_" + strPropertyName, typeof(object[]), FieldAttributes.Public);
         propertyBuilder = typeBuilder.DefineProperty(strPropertyName, System.Reflection.PropertyAttributes.HasDefault, typeof(object[]), null);
         getSetAttr = MethodAttributes.Public | MethodAttributes.SpecialName | MethodAttributes.HideBySig;
         //属性get方法
         methodBuilder = typeBuilder.DefineMethod("get_" + strPropertyName, getSetAttr, typeof(object[]), Type.EmptyTypes);
         ilGenerator = methodBuilder.GetILGenerator();
         ilGenerator.Emit(OpCodes.Ldarg_0);
         ilGenerator.Emit(OpCodes.Ldfld, fieldBuilder);
         ilGenerator.Emit(OpCodes.Ret);
         propertyBuilder.SetGetMethod(methodBuilder);

         //属性set方法
         methodBuilder = typeBuilder.DefineMethod("set_" + strPropertyName,
            MethodAttributes.Public | MethodAttributes.HideBySig | MethodAttributes.SpecialName,
            typeof(void), methodArgs);
         ilGenerator = methodBuilder.GetILGenerator();
         ilGenerator.Emit(OpCodes.Ldarg_0);
         ilGenerator.Emit(OpCodes.Ldarg_1);
         ilGenerator.Emit(OpCodes.Stfld, fieldBuilder);
         ilGenerator.Emit(OpCodes.Ret);
         propertyBuilder.SetSetMethod(methodBuilder);
         
         //创建类型
         typeBuilder.CreateType();
         typeDynamic = moduleBuilder.GetType(strDynamicClassName);

         return typeDynamic;
      }

      /// <summary>
      /// 给DataRow类型加扩展方法，用以获取指定类型的实例
      /// 请配合动态创建类型一起使用
      /// </summary>
      /// <param name="type">动态创建的类型</param>
      /// <param name="dr">数据源</param>
      /// <param name="children">子项</param>
      /// <returns></returns>
      internal static object GetEntity(this DataRow dr, Type type, object[] children = null)
      {
         //创建指定类型实例
         object objReturn = Activator.CreateInstance(type, null, null);

         //一行数据是一个实体，一列是实体的一个属性，这里给字段赋值
         for (int i = 0; i < dr.Table.Columns.Count; i++)
         {
            string strPropertyName = dr.Table.Columns[i].ColumnName;
            FieldInfo fieldInfo = type.GetField("m_str_" + strPropertyName);
            fieldInfo.SetValue(objReturn, dr[i].ToString());
         }

         //给子项字段赋值
         {
            string strPropertyName = "Children";

            FieldInfo fieldInfo = type.GetField("m_" + strPropertyName);
            fieldInfo.SetValue(objReturn, children);
         }

         return objReturn;
      }
   }
}
