﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Timers;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;
using System.ComponentModel;
using Platform.Controls.ComponentContainer;
using Platform.Presentation.FocusManagers;
using TellerSystem.Library.Ext;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// TreeView_List.xaml 的交互逻辑
    /// </summary>
    public partial class TreeView_List : UserControl, INotifyPropertyChanged
    {
        public TreeView_List()
        {

            InitializeComponent();
            _interval.AutoReset = true;
            _interval.Elapsed += interval_Elapsed;
            TxtFilter.PreviewKeyDown += txtFilter_PreviewKeyDown;
        }
        bool _isShowSelectBar = true;
        bool _isShowPageBar = true;


        private int _total;

        /// <summary>
        /// 获取总数据数
        /// </summary>
        public int Total
        {
            get { return _total; }
            private set
            {
                _total = value;
                RaisePropertyChanged("Total");
            }
        }


        private ObservableCollection<RuleInfoPSet> _sourceshow;

        private ObservableCollection<RuleInfoPSet> _source;
        /// <summary>
        /// 获取或设置数据源
        /// </summary>
        public ObservableCollection<RuleInfoPSet> Source
        {
            get
            {
                return _source;
            }
            set
            {
                _source = value;
            }
        }

        /// <summary>
        /// 获取控件中的TreeViewList控件
        /// </summary>
        public DataGridTreeView TreeViewList
        {
            get { return this.treeViewList; }
        }


        public bool IsShowCheckBox
        {
            get { return (bool)GetValue(IsShowCheckBoxProperty); }
            set { SetValue(IsShowCheckBoxProperty, value); }
        }
        public static readonly DependencyProperty IsShowCheckBoxProperty =
            DependencyProperty.Register("IsShowCheckBox", typeof(bool), typeof(TreeView_List), new UIPropertyMetadata(false));


        /// <summary>
        /// 获得或设置是否显示分页栏
        /// </summary>
        public bool IsShowPageBar
        {
            get
            {
                return _isShowPageBar;
            }
            set
            {
                if (_isShowPageBar != value)
                {
                    _isShowPageBar = value;
                    RaisePropertyChanged("IsShowPageBar");

                    if (_source == null) return;
                    _sourceshow = _source;

                    if (_isShowPageBar)
                    {
                        Total = _sourceshow.Count;
                        dataPager.PageIndex = 1;
                        dataPager_PageChanged(null, new PageChangedEventArgs(null, dataPager.PageSize, dataPager.PageIndex));
                        dataPager.RaisePageChanged();
                    }
                    else
                    {
                        BindData(_sourceshow);
                    }
                }
            }
        }

        /// <summary>
        /// 获得或设置是否显示搜索栏
        /// </summary>
        public bool IsShowSelectBar
        {
            get
            {
                return _isShowSelectBar;
            }
            set
            {
                _isShowSelectBar = value;
                GridSelectBar.Visibility = value ? Visibility.Visible : Visibility.Collapsed;
            }
        }


        /*
         *  对于筛选进行缓冲处理
         */
        readonly Timer _interval = new Timer(400);
        bool _isKeyEvent;

        void interval_Elapsed(object sender, ElapsedEventArgs e)
        {
            _interval.Stop();
            _isKeyEvent = false;
            this.Dispatcher.BeginInvoke(new Action(() =>
            {
                _sourceshow = new ObservableCollection<RuleInfoPSet>();
                var text = TxtFilter.Text;
                Action<ObservableCollection<RuleInfoPSet>> action = null;
                action = item =>
                    {
                        if (item == null) return;
                        item.ToList().ForEach(x =>
                            {
                                if (x.RuleNo.Contains(text) || x.RuleName.Contains(text) || x.RuleRemark.Contains(text) || x.RuleType.Contains(text))
                                {
                                    _sourceshow.Add(x);
                                }
                                else
                                {
                                    action(x.Children);
                                }
                            });
                    };
                if (string.IsNullOrEmpty(TxtFilter.Text.Trim()))
                {
                    _sourceshow = _source;
                }
                else
                {
                    action(_source);
                }

                if (IsShowPageBar)
                {
                    Total = _sourceshow.Count;
                    dataPager.PageIndex = 1;
                    dataPager_PageChanged(null, new PageChangedEventArgs(null, dataPager.PageSize, dataPager.PageIndex));
                    dataPager.RaisePageChanged();
                }
                else
                    TxtFilter.Clear();
            }));
        }

        private void txt_TextChanged(object sender, TextChangedEventArgs e)
        {
            BtnClear.Visibility = TxtFilter.Text.Length > 0 ? System.Windows.Visibility.Visible : System.Windows.Visibility.Collapsed;

            if (_sourceshow == null || _sourceshow.Count == 0) return;

            if (!_isKeyEvent) return;
            _interval.Stop();
            _interval.Start();
        }

        private void btnClear_Click(object sender, RoutedEventArgs e)
        {
            TxtFilter.Clear();
            _interval.Stop();
            _interval.Start();
            _sourceshow = _source;
        }

        void txtFilter_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            _isKeyEvent = true;

            //如果在筛选框上点击ESC或者Enter,则设置焦点到DataGrid上
            if (e.Key == Key.Escape || e.Key == Key.Enter)
            {
                treeViewList.Focus();

                e.Handled = true;//切换焦点的时候阻止路由,这里可能会再修改
            }
            //这里用来过滤一些可能出错的符号'\之类
            else if (e.Key == Key.Oem7 || e.Key == Key.Oem4 || e.Key == Key.Oem6)
            {
                e.Handled = true;
            }
        }

        /// <summary>
        /// 绑定数据
        /// </summary>
        public void Binding()
        {
            if (Source == null) return;
            TxtFilter.Clear();//外部调用时,先清空筛选框
            Binding(Source);
            Action<DataGridTreeViewItem> action = null;
            action = item =>
                {
                    if (item == null || !item.HasItems) return;
                    item.ExpandSubtree();
                    item.Items.Cast<object>()
                        .ToList()
                        .ForEach(c => action(item.ItemContainerGenerator.ContainerFromItem(c) as DataGridTreeViewItem));
                    item.IsExpanded = false;
                };

            treeViewList.Items.Cast<object>().ToList().ForEach(c => action(treeViewList.ItemContainerGenerator.ContainerFromItem(c) as DataGridTreeViewItem));
            SetChecked();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="dtSource">数据源</param>
        /// <exception cref="ArgumentNullException">dtSource 或 columns 参数为 null</exception>
        public void Binding(ObservableCollection<RuleInfoPSet> dtSource)
        {
            if (dtSource == null)
            {
                throw new ArgumentNullException("dtSource");
            }

            if (_sourceshow == null)
            {
                _sourceshow = dtSource;
            }

            if (IsShowPageBar)
            {
                Total = _sourceshow.Count;

                dataPager.RaisePageChanged();
            }
            else
            {
                BindData(_sourceshow);
            }
        }

        void BindData(IEnumerable<RuleInfoPSet> source)
        {
            this.treeViewList.SetItemsSource(source);
        }

        private void dataPager_PageChanged(object sender, PageChangedEventArgs args)
        {
            if (_sourceshow == null) return;

            var source = new ObservableCollection<RuleInfoPSet>();

            foreach (var a in _sourceshow.Skip((args.PageIndex - 1) * args.PageSize).Take(args.PageSize))
            {
                source.Add(a);
            }
            BindData(source);
        }

        public event PropertyChangedEventHandler PropertyChanged;
        public void RaisePropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler tmp = System.Threading.Interlocked.CompareExchange(
                ref PropertyChanged, null, null);
            if (tmp != null)
            {
                tmp(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public List<RuleInfoPSet> GetRows(Predicate<RuleInfoPSet> aFunc)
        {
            return treeViewList.ItemsSource.Cast<RuleInfoPSet>().ToList().FindAll(aFunc);
        }

        public void CheckedChanged(string ruleId, bool ifchecked)
        {
            Action<ObservableCollection<RuleInfoPSet>> action = null;
            action = (item) =>
            {
                if (item == null) return;
                foreach (var ruleInfopset in item)
                {
                    if (ruleInfopset.RuleId == ruleId)
                    {
                        #region 子项勾选
                        Action<ObservableCollection<RuleInfoPSet>> action2 = null;
                        action2 = (item2) =>
                        {
                            if (item2 == null) return;
                            foreach (var ruleInfoPSet2 in item2)
                            {
                                ruleInfoPSet2.IsChecked = ifchecked;
                                //#region 根据菜单勾选设置功能可编辑状态
                                //if (ruleInfoPSet2.RuleTypeCode == "1")
                                //{
                                //    if (ruleInfoPSet2.IsChecked == true)
                                //    {
                                //        foreach (var ruleInfoPSet in ruleInfoPSet2.Children)
                                //        {
                                //            ruleInfoPSet.IsEnabled = true;
                                //        }
                                //    }
                                //    else
                                //    {
                                //        foreach (var ruleInfoPSet in ruleInfoPSet2.Children)
                                //        {
                                //            ruleInfoPSet.IsEnabled = false;
                                //        }
                                //    }

                                //} 
                                //#endregion
                                action2(ruleInfoPSet2.Children);
                            }
                        };
                        action2(ruleInfopset.Children);
                        #endregion

                        #region 父项勾选

                        Action<RuleInfoPSet> action3 = null;
                        action3 = (item3) =>
                        {
                            if (item3 == null) return;
                            if (item3.Children.All(x => x.IsChecked == true))
                            {
                                item3.IsChecked = true;
                            }
                            else if (item3.Children.All(x => x.IsChecked == false))
                            {
                                item3.IsChecked = false;
                            }
                            else
                            {
                                item3.IsChecked = null;
                            }
                            action3(item3.Parent);
                        };

                        //分类才设置
                        if (ruleInfopset.RuleTypeCode != "2")
                        {
                            action3(ruleInfopset.Parent);
                        }
                        else
                        {
                            if (ruleInfopset.IsChecked == true)
                            {
                                var a = ruleInfopset.Parent.Children.Where(ruleInfoPSet => ruleInfoPSet.IsChecked != true).ToList();
                                ruleInfopset.Parent.IsChecked = true;
                                foreach (var ruleInfoPSet in from ruleInfoPSet in ruleInfopset.Parent.Children from infoPSet in a where ruleInfoPSet.RuleId==infoPSet.RuleId select ruleInfoPSet)
                                {
                                    ruleInfoPSet.IsChecked = false;
                                }
                            }
                        }
                        #endregion
                        break;
                    }
                    else
                    {
                        action(ruleInfopset.Children);
                    }
                }
            };

            action(_source);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ruleId"></param>
        /// <param name="ifchecked"></param>
        /// <param name="type">0全部，1 父级， 2 子集</param>
        public void CheckedChanged(string ruleId, bool ifchecked, int type = 0)
        {
            Action<ObservableCollection<RuleInfoPSet>> action = null;
            action = (item) =>
            {
                if (item == null) return;
                foreach (var ruleInfopset in item)
                {
                    if (ruleInfopset.RuleId == ruleId)
                    {
                        if (type == 0 || type == 2)
                        {
                            #region 子项勾选
                            Action<ObservableCollection<RuleInfoPSet>> action2 = null;
                            action2 = (item2) =>
                            {
                                if (item2 == null) return;
                                foreach (var ruleInfoPSet2 in item2)
                                {
                                    ruleInfoPSet2.IsChecked = ifchecked;
                                    action2(ruleInfoPSet2.Children);
                                }
                            };

                            action2(ruleInfopset.Children);
                            #endregion
                        }

                        if (type == 0 || type == 1)
                        {
                            #region 父项勾选

                            Action<RuleInfoPSet> action3 = null;
                            action3 = (item3) =>
                            {
                                if (item3 == null) return;
                                if (item3.Children.All(x => x.IsChecked == true))
                                {
                                    item3.IsChecked = true;
                                }
                                else if (item3.Children.All(x => x.IsChecked == false))
                                {
                                    item3.IsChecked = false;
                                }
                                else
                                {
                                    item3.IsChecked = null;
                                }
                                action3(item3.Parent);
                            };

                            action3(ruleInfopset.Parent);
                            #endregion
                        }
                        break;
                    }
                    else
                    {
                        action(ruleInfopset.Children);
                    }
                }
            };

            action(_source);
        }
        public void SetChecked()
        {
            Action<ObservableCollection<RuleInfoPSet>> action = null;
            action = (item) =>
            {
                if (item == null) return;
                foreach (var ruleInfopset in item)
                {
                    if (!ruleInfopset.Children.Any())
                    {
                        Action<RuleInfoPSet> action3 = null;
                        action3 = (item3) =>
                        {
                            if (item3 == null) return;
                            //是交易的话不做处理
                            if (item3.RuleTypeCode == "1") return;
                            if (item3.Children.All(x => x.IsChecked == true))
                            {
                                item3.IsChecked = true;
                            }
                            else if (item3.Children.All(x => x.IsChecked == false))
                            {
                                item3.IsChecked = false;
                            }
                            else
                            {
                                item3.IsChecked = null;
                            }
                            action3(item3.Parent);
                        };

                        action3(ruleInfopset.Parent);
                    }
                    else
                    {
                        action(ruleInfopset.Children);
                    }
                }
            };

            action(_source);
        }
    }
}
