﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// WorkFlowNodeView.xaml 的交互逻辑
    /// </summary>
    public partial class WorkFlowNodeView : UserControl
    {
        public WorkFlowNodeView()
        {
            InitializeComponent();
        }

        #region 依赖属性
        /// <summary>
        /// 节点描述
        /// </summary>
        public string Description
        {
            get { return (string)GetValue(DescriptionProperty); }
            set { SetValue(DescriptionProperty, value); }
        }

        public static readonly DependencyProperty DescriptionProperty =
            DependencyProperty.Register("Description", typeof(string), typeof(WorkFlowNodeView), new UIPropertyMetadata(string.Empty));


        /// <summary>
        /// 节点当前状态
        /// </summary>
        public WorkFlowNodeState State
        {
            get { return (WorkFlowNodeState)GetValue(StateProperty); }
            set { SetValue(StateProperty, value); }
        }

        public static readonly DependencyProperty StateProperty =
            DependencyProperty.Register("State", typeof(WorkFlowNodeState), typeof(WorkFlowNodeView), new UIPropertyMetadata(WorkFlowNodeState.NextCannotDo));


        /// <summary>
        /// 当前节点的父节点
        /// </summary>
        public WorkFlowNodeView ParentNode
        {
            get { return (WorkFlowNodeView)GetValue(ParentNodeProperty); }
            set { SetValue(ParentNodeProperty, value); }
        }

        public static readonly DependencyProperty ParentNodeProperty =
            DependencyProperty.Register("ParentNode", typeof(WorkFlowNodeView), typeof(WorkFlowNodeView), new UIPropertyMetadata(null));



        /// <summary>
        /// 当前节点的子节点集合
        /// </summary>
        public List<WorkFlowNodeView> Children
        {
            get { return (List<WorkFlowNodeView>)GetValue(ChildrenProperty); }
            set { SetValue(ChildrenProperty, value); }
        }

        public static readonly DependencyProperty ChildrenProperty =
            DependencyProperty.Register("Children", typeof(List<WorkFlowNodeView>), typeof(WorkFlowNodeView), new UIPropertyMetadata(null));


        /// <summary>
        /// 节点序号
        /// </summary>
        public string Sequence
        {
            get { return (string)GetValue(SequenceProperty); }
            set { SetValue(SequenceProperty, value); }
        }

        public static readonly DependencyProperty SequenceProperty =
            DependencyProperty.Register("Sequence", typeof(string), typeof(WorkFlowNodeView), new UIPropertyMetadata("1"));
        #endregion

        #region 事件
        public event EventHandler<WorkFlowArgs> DoAction;
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            if (DoAction != null)
            {
                DoAction(sender, new WorkFlowArgs() { NodeInfo = this });
            }
        }
        #endregion

        #region
        #endregion
    }

    public enum WorkFlowNodeState
    {
        /// <summary>
        /// 做完还能做
        /// </summary>
        ProcessedCanBeDone,

        /// <summary>
        /// 做完不能做
        /// </summary>
        ProcessedCanNotDo,

        /// <summary>
        /// 正在做且不能做
        /// </summary>
        ProcessingCanBeDone,

        /// <summary>
        /// 正在做且还能做
        /// </summary>
        ProcessingCanNotDo,

        /// <summary>
        /// 还没做且能做
        /// </summary>
        NextCanBeDone,

        /// <summary>
        /// 还没做且不能做
        /// </summary>
        NextCannotDo,

    }

    public class StateToBoolConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            WorkFlowNodeState state = (WorkFlowNodeState)value;
            switch (state)
            {
                case WorkFlowNodeState.ProcessedCanBeDone:
                    return true;
                case WorkFlowNodeState.ProcessedCanNotDo:
                    return false;
                case WorkFlowNodeState.ProcessingCanBeDone:
                    return true;
                case WorkFlowNodeState.ProcessingCanNotDo:
                    return false;
                case WorkFlowNodeState.NextCanBeDone:
                    return true;
                case WorkFlowNodeState.NextCannotDo:
                    return false;
                default:
                    return false;
            }
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class PathConvert: IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            var btn = value as WorkFlowNodeView;
            if (btn == null)
                return null;
            double height = btn.Height;
            double width = btn.Width;
            btn.Margin = new Thickness(-1 * width * 0.06, 0, 1, 0);


            PathFigure figure = new PathFigure();


            figure.StartPoint = new Point(0, 0);

            figure.Segments.Add(new LineSegment() { Point = new Point(0.06 * width, height / 2), IsStroked = true });
            figure.Segments.Add(new LineSegment() { Point = new Point(0, height), IsStroked = true });
            figure.Segments.Add(new LineSegment() { Point = new Point(0.94 * width,height)});
            figure.Segments.Add(new LineSegment() { Point = new Point(width, height / 2), IsStroked = true });
            figure.Segments.Add(new LineSegment() { Point = new Point(0.94 * width, 0), IsStroked = true });
            figure.Segments.Add(new LineSegment() { Point = new Point(0, 0) });

            PathGeometry geometry = new PathGeometry();
            geometry.Figures.Add(figure);
            return geometry;
        }


        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class WorkFlowArgs : EventArgs
    {
        public WorkFlowNodeView NodeInfo { get; set; }
    }
}
