﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using TellerSystem.ServiceProxy.Ext;
using Platform.Controls.ControlsHelper;
using Platform.Presentation.FocusManagers;

namespace TellerSystem.Controls.Ext.ControlsAttributes
{
    /// <summary>
    /// 申明控件附加属性
    /// </summary>
    public class ControlsAttributes
    {
        #region 文本框水印文本
        public static string GetWaterMarkText(DependencyObject obj)
        {
            return (string)obj.GetValue(WaterMarkTextProperty);
        }

        public static void SetWaterMarkText(DependencyObject obj, string value)
        {
            obj.SetValue(WaterMarkTextProperty, value);
        }

        public static readonly DependencyProperty WaterMarkTextProperty =
             DependencyProperty.RegisterAttached("WaterMarkText", typeof(string), typeof(ControlsAttributes), new UIPropertyMetadata(string.Empty));
        #endregion

        #region 姓名标准点转换
        public static bool GetIsFmtCEName(DependencyObject obj)
        {
            return (bool)obj.GetValue(IsFmtCENameProperty);
        }

        public static void SetIsFmtCEName(DependencyObject obj, bool value)
        {
            obj.SetValue(IsFmtCENameProperty, value);
        }

        // Using a DependencyProperty as the backing store for IsFmtCEName.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsFmtCENameProperty =
            DependencyProperty.RegisterAttached("IsFmtCEName", typeof(bool), typeof(ControlsAttributes), new UIPropertyMetadata(false
                , new PropertyChangedCallback((d, e) =>
                {
                    var textBox = d as System.Windows.Controls.TextBox;
                    if (textBox != null)
                    {
                        if ((bool)e.NewValue)
                            Platform.Presentation.FocusManagers.FocusManager.AddPreviewLostFocusHandler(textBox, TextChangedToFmtCEName);
                        else
                            Platform.Presentation.FocusManagers.FocusManager.RemovePreviewLostFocusHandler(textBox, TextChangedToFmtCEName);
                    }
                })));
        static void TextChangedToFmtCEName(object sender, RoutedEventArgs e)
        {
            var textBox = sender as System.Windows.Controls.TextBox;
            if (textBox != null)
            {
                textBox.Text = TellerSystem.Communication.Message.FmtCEName(textBox.Text);
            }
        }
        #endregion

        #region 按钮点击间隔控制
        public static int GetButtonClickDelay(DependencyObject obj)
        {
            return (int)obj.GetValue(ButtonClickDelayProperty);
        }

        public static void SetButtonClickDelay(DependencyObject obj, int value)
        {
            obj.SetValue(ButtonClickDelayProperty, value);
        }

        // Using a DependencyProperty as the backing store for ButtonClickDelay.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ButtonClickDelayProperty =
            DependencyProperty.RegisterAttached("ButtonClickDelay", typeof(int), typeof(ControlsAttributes), new UIPropertyMetadata(0, new PropertyChangedCallback((d, e) =>
            {
                var btn = d as Button;
                if (btn == null) return;
                btn.PreviewMouseLeftButtonUp -= btn_PreviewMouseLeftButtonUp;
                btn.PreviewKeyDown -= btn_PreviewKeyDown;
                //非0配置时，启用
                if ((int)e.NewValue != 0)
                {
                    btn.PreviewMouseLeftButtonUp += btn_PreviewMouseLeftButtonUp;
                    btn.PreviewKeyDown += btn_PreviewKeyDown;
                }
            })));

        static void btn_PreviewMouseLeftButtonUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            CheckButtonClick(sender as Button, e);
        }

        static void btn_PreviewKeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {
            if (e.Key == System.Windows.Input.Key.Enter)
                CheckButtonClick(sender as Button, e);
        }
        // Using a DependencyProperty as the backing store for ClickTimestamp.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ClickTimestampProperty =
            DependencyProperty.RegisterAttached("ClickTimestamp", typeof(DateTime?), typeof(ControlsAttributes), new UIPropertyMetadata(null));

        private static void CheckButtonClick(Button btn, RoutedEventArgs e)
        {
            if (btn == null) return;
            var fm = FocusManager.GetManagerInstance(btn);
            //只处理按钮在焦点控制列表中的情况
            if (fm == null || fm._focusList == null || !fm._focusList.Contains(btn)) return;
            //控制处理
            var delay = GetButtonClickDelay(btn);
            //若指定延迟小于0，则采用参数统一配置，默认3s
            if (delay < 0) delay = ServiceSettings.GetConfig("ButtonClickDelay", 3000);
            //执行按钮点击延迟处理
            var timestamp = (DateTime?)btn.GetValue(ClickTimestampProperty);
            var timeSpan = DateTime.Now - timestamp;
            if (timeSpan == null || timeSpan.Value.TotalMilliseconds > delay)
            {
                btn.RemoveTip();
                btn.SetValue(ClickTimestampProperty, DateTime.Now);
                return;
            }
            btn.ShowTip("请勿频繁点击，稍后再试...");
            e.Handled = true;
            return;
        }
        #endregion
    }
}
