﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Collections;
using Platform.Presentation.PageFunctions;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    /// 
    /// </summary>
    public class ExpanderGrid : Control
    {
        public delegate void ExpanderGridButtonClickHandle(ExpanderGridItem row, ExpanderGridItemButton button);
        public delegate void ExpanderGridItemAreaIsChangedHandle(ExpanderGridItem row);
        public event ExpanderGridButtonClickHandle ItemButtonClick;
        public event ExpanderGridItemAreaIsChangedHandle ItemAreaIsVisible;
        public event ExpanderGridItemAreaIsChangedHandle ItemAreaIsHidden;

        static ExpanderGrid()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(ExpanderGrid), new FrameworkPropertyMetadata(typeof(ExpanderGrid)));
        }

        /// <summary>
        /// 构造此控件实例
        /// </summary>
        public ExpanderGrid()
        {
            this.ExpanderGridItems = new ObservableCollection<ExpanderGridItem>();
        }

        /// <summary>
        /// 项集合
        /// </summary>
        public ObservableCollection<ExpanderGridItem> ExpanderGridItems
        {
            get { return (ObservableCollection<ExpanderGridItem>)GetValue(ExpanderGridItemsProperty); }
            set { SetValue(ExpanderGridItemsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Items.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ExpanderGridItemsProperty =
            DependencyProperty.Register("ExpanderGridItems", typeof(ObservableCollection<ExpanderGridItem>), typeof(ExpanderGrid), new UIPropertyMetadata(null));

        /// <summary>
        /// 
        /// </summary>
        public IEnumerable ItemsSouce
        {
            set
            {
                foreach (var n in value)
                {
                    ExpanderGridItem item = (ExpanderGridItem)n;

                    item.Parent = this;

                    for (int i = 0; i < item.Buttons.Count; i++)
                    {
                        item.Buttons[i].Parent = item;
                    }

                    for (int i = 0; i < item.Texts.Count; i++)
                    {
                        item.Texts[i].Parent = item;
                    }

                    this.ExpanderGridItems.Add(item);

                    //
                    //if (item.ExpandArea != null)
                    //    ExpanderGrid.SetExpanderGridItem(item.ExpandArea, item);
                }
            }
        }

        internal void OnButtonClick(ExpanderGridItem row, ExpanderGridItemButton button)
        {
            if (this.ItemButtonClick != null)
                this.ItemButtonClick(row, button);
        }

        internal void OnItemAreaIsVisible(ExpanderGridItem row)
        {
            if (this.ItemAreaIsVisible != null)
                this.ItemAreaIsVisible(row);
        }

        internal void OnItemAreaIsHidden(ExpanderGridItem row)
        {
            if (this.ItemAreaIsHidden != null)
                this.ItemAreaIsHidden(row);
        }

        /// <summary>
        /// 收缩起所有的已展开的项
        /// </summary>
        public bool IsCollapse
        {
            set
            {
                foreach (var item in this.ExpanderGridItems)
                {
                    if (item.ExpandAreaIsVisibility == System.Windows.Visibility.Visible)
                        item.ExpandAreaIsVisibility = System.Windows.Visibility.Collapsed;
                }
            }
        }

        /// <summary>
        /// 给每一个usercontrol附加一个usercontrol所在的ExpanderGridItem供在usercontrol里面也能操作ExpanderGridItem
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public static ExpanderGridItem GetExpanderGridItem(DependencyObject obj)
        {
            return (ExpanderGridItem)obj.GetValue(ExpanderGridItemProperty);
        }

        public static void SetExpanderGridItem(DependencyObject obj, ExpanderGridItem value)
        {
            obj.SetValue(ExpanderGridItemProperty, value);
        }

        // Using a DependencyProperty as the backing store for ExpanderGridItem.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ExpanderGridItemProperty =
            DependencyProperty.RegisterAttached("ExpanderGridItem", typeof(ExpanderGridItem), typeof(ExpanderGrid), new UIPropertyMetadata(null));
    }

    /// <summary>
    /// ExpanderGrid的行对象
    /// </summary>
    public class ExpanderGridItem : DependencyObject
    {
        //private Grid GD = null;
        public ExpanderGrid Parent
        {
            get;
            internal set;
        }

        /// <summary>
        /// 构造ExpanderGrid的行对象
        /// </summary>
        public ExpanderGridItem()
        {
            // this.Parent = parentControl;
            this.Buttons = new ObservableCollection<ExpanderGridItemButton>();
            this.Texts = new ObservableCollection<ExpanderGridItemText>();
            this.ExpandArea = new PageContainer();
        }

        /// <summary>
        /// 
        /// </summary>
        public object CommandTag
        {
            get { return (object)GetValue(CommandTagProperty); }
            set { SetValue(CommandTagProperty, value); }
        }

        // Using a DependencyProperty as the backing store for CommandTag.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty CommandTagProperty =
            DependencyProperty.Register("CommandTag", typeof(object), typeof(ExpanderGridItem), new UIPropertyMetadata(null));

        /// <summary>
        /// 
        /// </summary>
        public ObservableCollection<ExpanderGridItemText> Texts
        {
            get { return (ObservableCollection<ExpanderGridItemText>)GetValue(TextsProperty); }
            set { SetValue(TextsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Texts.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TextsProperty =
            DependencyProperty.Register("Texts", typeof(ObservableCollection<ExpanderGridItemText>), typeof(ExpanderGridItem), new UIPropertyMetadata(null));

        /// <summary>
        /// 按钮集合
        /// </summary>
        public ObservableCollection<ExpanderGridItemButton> Buttons
        {
            get { return (ObservableCollection<ExpanderGridItemButton>)GetValue(ButtonsProperty); }
            set { SetValue(ButtonsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Buttons.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ButtonsProperty =
            DependencyProperty.Register("Buttons", typeof(ObservableCollection<ExpanderGridItemButton>), typeof(ExpanderGridItem), new UIPropertyMetadata(null));

        /// <summary>
        /// 自定义编辑区域
        /// </summary>
        public PageContainer ExpandArea
        {
            get { return (PageContainer)GetValue(ExpandAreaProperty); }
            set { SetValue(ExpandAreaProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ExpandArea.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ExpandAreaProperty =
            DependencyProperty.Register("ExpandArea", typeof(PageContainer), typeof(ExpanderGridItem), new UIPropertyMetadata(null, new PropertyChangedCallback(PropertyChangedCallback)));

        /// <summary>
        /// 
        /// </summary>
        public Visibility ExpandAreaIsVisibility
        {
            get { return (Visibility)GetValue(ExpandAreaIsVisibilityProperty); }
            set { SetValue(ExpandAreaIsVisibilityProperty, value); }
        }

        private static void PropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            if (e.NewValue != null)
                ExpanderGrid.SetExpanderGridItem((DependencyObject)e.NewValue, (ExpanderGridItem)d);
        }

        // Using a DependencyProperty as the backing store for ExpandAreaIsVisibility.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ExpandAreaIsVisibilityProperty =
            DependencyProperty.Register("ExpandAreaIsVisibility", typeof(Visibility), typeof(ExpanderGridItem), new UIPropertyMetadata(Visibility.Collapsed, new PropertyChangedCallback(ExpandAreaIsVisibilityIsChanged)));

        private static void ExpandAreaIsVisibilityIsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ExpanderGridItem item = (ExpanderGridItem)d;

            if ((Visibility)e.NewValue == Visibility.Visible)
            {
                foreach (var n in item.Parent.ExpanderGridItems)
                    if (n != item)
                        n.ExpandAreaIsVisibility = Visibility.Collapsed;

                item.ExpandAreaIsVisibility = Visibility.Visible;

                item.Parent.OnItemAreaIsVisible(item);
            }
            else
            {
                item.ExpandAreaIsVisibility = Visibility.Collapsed;

                item.Parent.OnItemAreaIsHidden(item);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        internal double ClickButtonPoint
        {
            get { return (double)GetValue(ClickButtonPointProperty); }
            set { SetValue(ClickButtonPointProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ClickButtonPoint.  This enables animation, styling, binding, etc...
        internal static readonly DependencyProperty ClickButtonPointProperty =
            DependencyProperty.Register("ClickButtonPoint", typeof(double), typeof(ExpanderGridItem), new UIPropertyMetadata(0.00));
    }

    public class ExpanderGridItemButton : DependencyObject
    {
        public ExpanderGridItemButton()
        {
            //this.Parent = parentItem;
        }

        /// <summary>
        /// 
        /// </summary>
        public ImageSource ImageSource
        {
            get { return (ImageSource)GetValue(ImageSourceProperty); }
            set { SetValue(ImageSourceProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ImageSource.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ImageSourceProperty =
            DependencyProperty.Register("ImageSource", typeof(ImageSource), typeof(ExpanderGridItemButton), new UIPropertyMetadata(null));

        /// <summary>
        /// 
        /// </summary>
        public string Text
        {
            get { return (string)GetValue(TextProperty); }
            set { SetValue(TextProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Text.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TextProperty =
            DependencyProperty.Register("Text", typeof(string), typeof(ExpanderGridItemButton), new UIPropertyMetadata("未命名"));

        /// <summary>
        /// 
        /// </summary>
        public object CommandTag
        {
            get { return (object)GetValue(CommandTagProperty); }
            set { SetValue(CommandTagProperty, value); }
        }

        // Using a DependencyProperty as the backing store for CommandTag.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty CommandTagProperty =
            DependencyProperty.Register("CommandTag", typeof(object), typeof(ExpanderGridItemButton), new UIPropertyMetadata(null));

        /// <summary>
        /// 是否启用按钮
        /// </summary>
        public bool IsEnabled
        {
            get { return (bool)GetValue(IsEnabledProperty); }
            set { SetValue(IsEnabledProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsEnabled.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsEnabledProperty =
            DependencyProperty.Register("IsEnabled", typeof(bool), typeof(ExpanderGridItemButton), new UIPropertyMetadata(true));

        public ExpanderGridItem Parent
        {
            get;
            internal set;
        }
        /// <summary>
        /// 必须为public否则无法触发此命令
        /// </summary>
        public MyCommand CommandInstance
        {
            get
            {
                return new MyCommand(this);
            }
        }
    }

    /// <summary>
    /// 
    /// </summary>
    public class ExpanderGridItemText : DependencyObject
    {
        /// <summary>
        /// 
        /// </summary>
        public ExpanderGridItemText()
        {
            //this.Parent = parentItem;
        }

        /// <summary>
        /// 
        /// </summary>
        public ImageSource ImageSource
        {
            get { return (ImageSource)GetValue(ImageSourceProperty); }
            set { SetValue(ImageSourceProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ImageSource.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ImageSourceProperty =
            DependencyProperty.Register("ImageSource", typeof(ImageSource), typeof(ExpanderGridItemText), new UIPropertyMetadata(null));

        /// <summary>
        /// 
        /// </summary>
        public string Text
        {
            get { return (string)GetValue(TextProperty); }
            set { SetValue(TextProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Text.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TextProperty =
            DependencyProperty.Register("Text", typeof(string), typeof(ExpanderGridItemText), new UIPropertyMetadata("未命名"));

        /// <summary>
        /// 
        /// </summary>
        public ExpanderGridItem Parent
        {
            get;
            internal set;
        }
    }

    /// <summary>
    /// 
    /// </summary>
    public class MyCommand : ICommand
    {
        private ExpanderGridItemButton ParentButton = null;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="parentButton"></param>
        public MyCommand(ExpanderGridItemButton parentButton)
        {
            this.ParentButton = parentButton;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns></returns>
        public bool CanExecute(object parameter)
        {
            return true;
        }

        /// <summary>
        /// 
        /// </summary>
        public event EventHandler CanExecuteChanged;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="parameter"></param>
        public void Execute(object parameter)
        {
            Button bt = (Button)parameter;
            Point point = bt.TranslatePoint(new Point(0, 0), (UIElement)this.ParentButton.Parent.Parent);

            this.ParentButton.Parent.ClickButtonPoint = point.X;

            this.ParentButton.Parent.Parent.OnButtonClick(this.ParentButton.Parent, this.ParentButton);
        }
    }

    /// <summary>
    /// 
    /// </summary>
    public class DenoteConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            Thickness tk = new Thickness();
            //tk.Left = 15;
            tk.Left = (double)value + 10;
            tk.Top = 0;
            tk.Right = 0;
            tk.Bottom = 0;
            return tk;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
