﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Globalization;

namespace TellerSystem.Controls.Ext
{

    /// <summary>
    /// 处理类型
    /// </summary>
    public enum HandleType
    {
        /// <summary>
        /// 已处理
        /// </summary>
        Handled,
        /// <summary>
        /// //未处理
        /// </summary>
        UnHandled,
        /// <summary>
        /// 正在处理
        /// </summary>
        Handleding//
    }

    /// <summary>
    /// 按照步骤 1a 或 1b 操作，然后执行步骤 2 以在 XAML 文件中使用此自定义控件。
    ///
    /// 步骤 1a) 在当前项目中存在的 XAML 文件中使用该自定义控件。
    /// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
    /// 元素中:
    ///
    ///     xmlns:MyNamespace="clr-namespace:TellerSystem.Controls.Ext"
    ///
    ///
    /// 步骤 1b) 在其他项目中存在的 XAML 文件中使用该自定义控件。
    /// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
    /// 元素中:
    ///
    ///     xmlns:MyNamespace="clr-namespace:TellerSystem.Controls.Ext;assembly=TellerSystem.Controls.Ext"
    ///
    /// 您还需要添加一个从 XAML 文件所在的项目到此项目的项目引用，
    /// 并重新生成以避免编译错误:
    ///
    ///     在解决方案资源管理器中右击目标项目，然后依次单击
    ///     “添加引用”->“项目”->[选择此项目]
    ///
    ///
    /// 步骤 2)
    /// 继续操作并在 XAML 文件中使用控件。
    ///
    ///     <MyNamespace:CustomControl1/>
    ///
    /// </summary>
    [TemplatePart(Name = "PART_IMAGE", Type = typeof(Image))]
    public class FlowControl : Control
    {
        static FlowControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(FlowControl), new FrameworkPropertyMetadata(typeof(FlowControl)));
        }

        #region 基本属性
        public int CurrentStep { get; set; }
        #endregion

        #region 依赖属性
        private static readonly DependencyProperty FlowNameProperty =
            DependencyProperty.Register("FlowName", typeof(string), typeof(FlowControl), new PropertyMetadata(""));

        /// <summary>
        /// 流程节点名称
        /// </summary>
        public string FlowName 
        {
            get { return (string)GetValue(FlowNameProperty); }
            set { SetValue(FlowNameProperty, value); }
        }


        private static readonly DependencyProperty CurrentStateProperty =
            DependencyProperty.Register("CurrentState", typeof(HandleType), typeof(FlowControl), new PropertyMetadata(HandleType.UnHandled,
            new PropertyChangedCallback(OnStateChanged)));

        /// <summary>
        /// 流程节点状态
        /// </summary>
        public HandleType CurrentState
        {
            get { return (HandleType)GetValue(CurrentStateProperty); }
            set { SetValue(CurrentStateProperty, value); }
        }


        /// <summary>
        /// 是否还有后续步骤
        /// </summary>
        public bool ArrowIsVisible
        {
            get { return (bool)GetValue(ArrowIsVisibleProperty); }
            set { SetValue(ArrowIsVisibleProperty, value); }
        }

        public static readonly DependencyProperty ArrowIsVisibleProperty =
            DependencyProperty.Register("ArrowIsVisible", typeof(bool), typeof(FlowControl), new UIPropertyMetadata(true));


        /// <summary>
        /// 流程步骤集合
        /// </summary>
        public ObservableCollection<FlowControl> FlowLists
        {
            get
            {
                return (ObservableCollection<FlowControl>)GetValue(FlowListsProperty);
            }
            set
            {
                SetValue(FlowListsProperty, value);
            }
        }

        public static readonly DependencyProperty FlowListsProperty =
            DependencyProperty.Register("FlowLists", typeof(ObservableCollection<FlowControl>), typeof(FlowControl), new UIPropertyMetadata(null));



        /// <summary>
        /// 附加流程属性
        /// </summary>
        public static FlowControl GetFlow(DependencyObject obj)
        {
            return (FlowControl)obj.GetValue(FlowProperty);
        }

        /// <summary>
        /// 获取流程属性
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="value"></param>
        public static void SetFlow(DependencyObject obj, FlowControl value)
        {
            obj.SetValue(FlowProperty, value);
        }
        public static readonly DependencyProperty FlowProperty =
            DependencyProperty.Register("Flow", typeof(FlowControl), typeof(FlowControl), new UIPropertyMetadata(null));
        #endregion

        #region 依赖事件
        public static readonly RoutedEvent StateChangedEvent = EventManager.RegisterRoutedEvent("StateChanged",
            RoutingStrategy.Bubble, typeof(RoutedPropertyChangedEventHandler<HandleType>), typeof(FlowControl));

        public event RoutedPropertyChangedEventHandler<string> StateChanged
        {
            add { AddHandler(StateChangedEvent, value); }
            remove { RemoveHandler(StateChangedEvent, value); }
        }
        #endregion


        #region 状态改变回调方法
        private static void OnStateChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            FlowControl target = (FlowControl)d;
            HandleType oldState = (HandleType)e.OldValue;
            HandleType newState = target.CurrentState;
            target.OnStateChanged(oldState, newState);
        }

        protected virtual void OnStateChanged(HandleType oldState, HandleType newState)
        {
            RaiseEvent(new RoutedPropertyChangedEventArgs<HandleType>(oldState, newState, StateChangedEvent));
            if (imgState != null)
            {
                imgState.Source = new BitmapImage(new Uri(GetState(newState),UriKind.Relative));
            }
        }
        #endregion

        #region 控件模板
        Image imgState;
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            imgState = Template.FindName("PART_IMAGE", this) as Image;
        }
        #endregion

        #region 状态图标
        private static string GetState(HandleType type) 
        {
            string bgSource;
              if (type == HandleType.Handled) //已处理
                {
                    bgSource = "pack://application:,,,/TellerSystem.Controls.Ext;component/ControlsExt/Resources/led_red.png";
                }
                else if (type == HandleType.UnHandled)//未处理
                {
                    bgSource = "pack://application:,,,/TellerSystem.Controls.Ext;component/ControlsExt/Resources/led_green.png";
                }
                else
                {
                    bgSource = "pack://application:,,,/TellerSystem.Controls.Ext;component/ControlsExt/Resources/led_yellow.png";
                }
            return bgSource;
        }
        #endregion

        #region 公开操作
        /// <summary>
        /// 下一步流程显示
        /// </summary>
        public void SetNextStepFlow() 
        {
            if (CurrentStep < FlowLists.Count -1)
            {
                 FlowLists[CurrentStep].CurrentState = HandleType.Handled;
                 FlowLists[CurrentStep + 1].CurrentState = HandleType.Handleding;
                 CurrentStep++;
            }
            else
            {
                FlowLists[CurrentStep].CurrentState = HandleType.Handled;
            }
        }

        /// <summary>
        /// 初始化流程显示
        /// </summary>
        /// <param name="count">流程步骤总数</param>
        public void InitFlow()
        {
            FlowLists[FlowLists.Count - 1].ArrowIsVisible = false;
        }

        /// <summary>
        /// 设置当前流程
        /// </summary>
        /// <param name="index">当前流程索引</param>
        /// <param name="index">当前交易码</param>
        public void SetCurrentFlowStep(int index)
        {
            FlowLists[index - 1].CurrentState = HandleType.Handleding;
            CurrentStep = index - 1;
        }
        #endregion 
    }

    /// <summary>
    /// Enum与Source转换
    /// </summary>
    public sealed class EnumToSourceConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            string bgSource;

            if ((HandleType)value == HandleType.Handled) //已处理
            {
                bgSource = "pack://application:,,,/TellerSystem.Controls.Ext;component/ControlsExt/Resources/led_red.png";
            }
            else if ((HandleType)value == HandleType.UnHandled)//未处理
            {
                bgSource = "pack://application:,,,/TellerSystem.Controls.Ext;component/ControlsExt/Resources/led_green.png";
            }
            else
            {
                bgSource = "pack://application:,,,/TellerSystem.Controls.Ext;component/ControlsExt/Resources/led_yellow.png";
            }
            return bgSource;
        }
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            string bgSource = "";
            return bgSource = "pack://application:,,,/TellerSystem.Controls.Ext;component/ControlsExt/Resources/led_green.png";
        }
    }

    /// <summary>
    /// Enum与Visibility转换
    /// </summary>
    public sealed class EnumToVisibilityConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if ((HandleType)value == HandleType.Handled) //已处理
            {
                return Visibility.Collapsed;
            }
            else if ((HandleType)value == HandleType.UnHandled)//未处理
            {
                return Visibility.Collapsed;
            }
            else
            {
                return Visibility.Visible;
            }
        }
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return value is Visibility && (Visibility)value == Visibility.Visible;
        }
    }

    /// <summary>
    /// Bool与Visibility转换
    /// </summary>
    public sealed class BooleanToVisibilityConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return (value is bool && (bool)value) ? Visibility.Visible : Visibility.Collapsed;
        }
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return value is Visibility && (Visibility)value == Visibility.Visible;
        }
    }

}
