﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Presentation.FocusManagers;
using System.Globalization;
namespace TellerSystem.Controls.Ext.ControlsExt
{
    public class HourMinute:TextBox
    {
        static HourMinute()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(HourMinute),new FrameworkPropertyMetadata(typeof(HourMinute)));
        }
        protected int _currentIndex;
        private Platform.Presentation.FocusManagers.FocusManager _focus;
        public HourMinute()
        {
            //禁用输入法
            InputMethod.SetIsInputMethodEnabled(this,false);
            this.Loaded += new RoutedEventHandler(HourMinute_Loaded);
            this.Text = "00:00";
            this.HorizontalAlignment = HorizontalAlignment.Left;
        }

        void HourMinute_Loaded(object sender,RoutedEventArgs e)
        {
            if(System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
                return;
            _focus = Platform.Presentation.FocusManagers.FocusManager.GetManagerInstance((DependencyObject)this) as Platform.Presentation.FocusManagers.FocusManager;
            if(_focus == null)
                throw new ArgumentNullException("无法找到焦点控制模块！");
            this.PreviewKeyDown += new System.Windows.Input.KeyEventHandler(OnPreviewKeyDown);
            this.TextChanged += new TextChangedEventHandler(HourMinute_TextChanged);
        }

        void HourMinute_TextChanged(object sender,TextChangedEventArgs e)
        {
            if(this.CaretIndex < this.Text.IndexOf(':'))  //如果光标在冒号之前，即停留在小时上
            {
                if(this.Text.IndexOf(':')> 2)  //小时数输入的大于2
                {
                    this.Text = this.Text.Substring(0,2) + this.Text.Substring(3);  //取小时的前两位加冒号以后的数据
                }
            }
            else if(this.CaretIndex == this.Text.IndexOf(':') && this.Text.Length>5 )
            {
                this.Text =this.Text.Substring(0,2) + ":" + this.Text.Replace(":","").Substring(2,2);
                _currentIndex = _currentIndex + 1;
            }
            else if(this.CaretIndex > this.Text.IndexOf(':'))  //如果光标在冒号之后 ，即停留在分钟上
            {
                if(this.Text.Length - this.Text.IndexOf(':') - 1 > 2)  //分钟数输入的大于2
                {
                    this.Text = this.Text.Substring(0,this.Text.Length - 1);
                }
                else
                {
                    if(this.Text.Substring(this.Text.IndexOf(':') + 1).Length == 1)
                    {
                        this.Text = this.Text + "0";
                    }
                }
            }
            if(this.Text.IndexOf(':')>1)  //防止出现:00的现象
            {
                if(Int32.Parse(this.Text.Substring(0,this.Text.IndexOf(':'))) > 23)
                {
                    this.Text = this.Text.Substring(0,1) + ":" + this.Text.Substring(this.Text.IndexOf(':') + 1);
                }
            }
            if(Int32.Parse(this.Text.Substring(this.Text.IndexOf(':') + 1)) > 59)
            {
                this.Text = this.Text.Substring(0,this.Text.Length - 1);
            }
            DateTime value;
            DateTime.TryParse((this.Text),CultureInfo.GetCultureInfo("zh-CN"),DateTimeStyles.None,out value);
            this.Text = string.Format("{0:t}",value);
            this.CaretIndex = _currentIndex;
        }

        protected void OnPreviewKeyDown(object sender,System.Windows.Input.KeyEventArgs e)
        {               
            if(e.Key == Key.Up)
            {
                _focus.MoveFocus(FocusNavigation.Backward);
                e.Handled = true;
            }
            else if(e.Key == Key.Down || e.Key == Key.Tab || e.Key==Key.Return)
            {
                _focus.MoveFocus(FocusNavigation.Forward);
                e.Handled = true;
            }
            else if(e.Key == Key.Left)//左移
            {

            }
            else if(e.Key == Key.Right)//右移
            {

            }
            else if(e.Key == Key.Back)//退格键
            {
                if(!string.IsNullOrEmpty(this.Text) && this.CaretIndex != 0)
                {
                    if(this.Text[this.CaretIndex - 1] == ':')
                    {
                        CaretIndex--;
                        _currentIndex = CaretIndex - 1;
                    }
                    else
                    {
                        _currentIndex = CaretIndex - 1;
                    }

                }
            }
            else if(e.Key == Key.Delete)//按Delete键
            {
                if(!string.IsNullOrEmpty(this.Text) && CaretIndex != this.Text.Length)
                {
                    if(this.Text[this.CaretIndex] == ':')
                    {
                        CaretIndex++;
                        _currentIndex = CaretIndex;
                    }
                    else
                    {
                        _currentIndex = CaretIndex;
                    }
                }
            }
            else if(!IsDigitalKey(e.Key))
            {
                _currentIndex = CaretIndex;
                e.Handled = true;
            }
            else
            {
                _currentIndex = CaretIndex + 1;

            }
            //base.OnPreviewKeyDown(e);
        }

        protected bool IsDigitalKey(Key key)
        {
            var value = (int)key;
            return (value >= 34 && value <= 43) || (value >= 74 && value <= 83);
        }
    }
}
