﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;
using Platform.Controls.ComponentContainer;
using TellerSystem.Library.Ext.TradeExtension;
using TellerSystem.Contract.ContractBase;
using Platform.Common.BasicFunctions;
using TellerSystem.Library.Ext.Function;
using System.ComponentModel;

namespace TellerSystem.Controls.Ext.ControlsExt
{
    /// <summary>
    /// 按照步骤 1a 或 1b 操作，然后执行步骤 2 以在 XAML 文件中使用此自定义控件。
    ///
    /// 步骤 1a) 在当前项目中存在的 XAML 文件中使用该自定义控件。
    /// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
    /// 元素中:
    ///
    ///     xmlns:MyNamespace="clr-namespace:TellerSystem.Controls.Ext.ControlsExt"
    ///
    ///
    /// 步骤 1b) 在其他项目中存在的 XAML 文件中使用该自定义控件。
    /// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
    /// 元素中:
    ///
    ///     xmlns:MyNamespace="clr-namespace:TellerSystem.Controls.Ext.ControlsExt;assembly=TellerSystem.Controls.Ext.ControlsExt"
    ///
    /// 您还需要添加一个从 XAML 文件所在的项目到此项目的项目引用，
    /// 并重新生成以避免编译错误:
    ///
    ///     在解决方案资源管理器中右击目标项目，然后依次单击
    ///     “添加引用”->“项目”->[浏览查找并选择此项目]
    ///
    ///
    /// 步骤 2)
    /// 继续操作并在 XAML 文件中使用控件。
    ///
    ///     <MyNamespace:QueryControl/>
    ///
    /// </summary>
    [TemplatePart(Name = "PART_DataHost", Type = typeof(DataBox))]
    public class QueryControl : ContentControl
    {
        #region 主框架
        static QueryControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(QueryControl), new FrameworkPropertyMetadata(typeof(QueryControl)));
        }

        /// <summary>
        /// 查询维护控件
        /// </summary>
        public QueryControl()
        {
            //命令绑定
            CommandBindings.Add(new CommandBinding(QueryCommand, (sender, e) =>
            {
                IsBusy = true;
                //执行查询动作
                RaiseQuery();
                IsBusy = false;
            }, (sender, e) =>
            {
                e.CanExecute = !IsBusy;
            }));
            //CommandBindings.Add(new CommandBinding(InsertCommand, (sender, e) =>
            //{
            //    //执行增加动作
            //    RaiseQueryItem(QueryItemAction.Insert);
            //}, (sender, e) =>
            //{
            //    e.CanExecute = true;
            //}));
        }

        /// <summary>
        /// 查询中
        /// </summary>
        public bool IsBusy { get; private set; }

        #region 是否启用添加功能
        private bool _CanInsertItem = false;

        private CommandBinding _insertCB;
        /// <summary>
        /// 是否启用添加
        /// </summary>
        public bool CanInsertItem
        {
            get { return _CanInsertItem; }
            set
            {
                _CanInsertItem = value;
                if (_insertCB == null)
                {
                    _insertCB = new CommandBinding(InsertCommand, (sender, e) =>
                    {
                        //执行增加动作
                        RaiseQueryItem(QueryItemAction.Insert);
                    }, (sender, e) =>
                    {
                        e.CanExecute = true;
                    });
                    //InputBindings.Add(new KeyBinding(InsertCommand, Key.A, ModifierKeys.Control));
                }
                if (_CanInsertItem)
                    CommandBindings.Add(_insertCB);
                else
                    CommandBindings.Remove(_insertCB);
            }
        }
        #endregion

        public PageBase ParentPage { get; private set; }

        /// <summary>
        /// 加载模板
        /// </summary>
        public override void OnApplyTemplate()
        {
            _dataHost = this.GetTemplateChild("PART_DataHost") as DataBox;
            _dataHost.Height = double.NaN;
            _dataHost.Width = double.NaN;

            ParentPage = this.FindVisualTreeAncestor(x => x is PageBase) as PageBase;
            base.OnApplyTemplate();
        }

        #region Commands
        /// <summary>
        /// 查询命令
        /// </summary>
        public static readonly RoutedUICommand QueryCommand = new RoutedUICommand("查询", "Query", typeof(QueryControl));
        /// <summary>
        /// 增加命令
        /// </summary>
        public static readonly RoutedUICommand InsertCommand = new RoutedUICommand("增加", "Insert", typeof(QueryControl));
        #endregion
        #endregion

        #region 查询条件
        /******************************************************
         * 1、查询条件界面由Content属性决定，该部分为外部定义
         * 2、定义查询按钮动作的执行主体事件，由外部定义，仅限定数据接口
         * ****************************************************/
        /// <summary>
        /// 触发查询
        /// </summary>
        public void RaiseQuery()
        {
            //TODO:清空数据，防止历史数据干扰
            _dataHost.InitDataBox();
            if (Query == null) return;
            var args = new QueryEventArgs(_dataHost);
            Query(this, args);
            RaiseShowLink();
        }
        /// <summary>
        /// 触发查询的事件，需要填充数据源
        /// </summary>
        public event EventHandler<QueryEventArgs> Query;
        /// <summary>
        /// 查询事件参数
        /// </summary>
        public class QueryEventArgs : EventArgs
        {
            public QueryEventArgs(DataBox dataBox)
            {
                DataHost = dataBox;
            }
            public DataBox DataHost { get; private set; }
        }
        #endregion

        #region 查询结果
        /******************************************************
         * 1、查询结果界面默认采用DataBox控件为主体
         * 2、允许使用自定义模板进行处理，此时DataBox相关的处理全部失效
         * ****************************************************/
        #region 默认
        /// <summary>
        /// PART_DataHost
        /// </summary>
        private DataBox _dataHost;

        /// <summary>
        /// 获取当前的集合数据-string
        /// </summary>
        /// <returns></returns>
        public string GetDataForString()
        {
            return _dataHost.GetDataForString();
        }

        /// <summary>
        /// 获取当前的集合数据-DataTable
        /// </summary>
        /// <returns></returns>
        public DataTable GetDataForDataTable()
        {
            return _dataHost.GetDataForDataTable();
        }

        /// <summary>
        /// 是否显示分页
        /// </summary>
        public bool IsShowPageBar
        {
            get { return _dataHost.IsShowPageBar; }
            set { _dataHost.IsShowPageBar = value; }
        }
        #endregion

        #region 页眉页脚


        public object ResultHeader
        {
            get { return (object)GetValue(ResultHeaderProperty); }
            set { SetValue(ResultHeaderProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ResultHeader.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ResultHeaderProperty =
            DependencyProperty.Register("ResultHeader", typeof(object), typeof(QueryControl), new UIPropertyMetadata(null));



        public object ResultFooter
        {
            get { return (object)GetValue(ResultFooterProperty); }
            set { SetValue(ResultFooterProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ResultFooter.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ResultFooterProperty =
            DependencyProperty.Register("ResultFooter", typeof(object), typeof(QueryControl), new UIPropertyMetadata(null));


        #endregion

        #region 自定义查询列表
        /// <summary>
        /// 是否自定义结果界面
        /// </summary>
        public bool IsCustomizeResult { get; set; }
        //TODO:如何限定自定义控件的呈现
        #endregion
        #endregion

        #region 信息维护
        /// <summary>
        /// 查询单一对象的承载控件
        /// </summary>
        public IQueryItem QueryItem { get; set; }

        /// <summary>
        /// 是否采用延迟触发QueryItem.RaiseAction，即IQueryItem元素Loaded之后执行
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public static bool GetIsDelayRaise(DependencyObject obj)
        {
            return (bool)obj.GetValue(IsDelayRaiseProperty);
        }

        /// <summary>
        /// 是否采用延迟触发QueryItem.RaiseAction，即IQueryItem元素Loaded之后执行
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="value"></param>
        public static void SetIsDelayRaise(DependencyObject obj, bool value)
        {
            obj.SetValue(IsDelayRaiseProperty, value);
        }

        // Using a DependencyProperty as the backing store for IsDelayRaise.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsDelayRaiseProperty =
            DependencyProperty.RegisterAttached("IsDelayRaise", typeof(bool), typeof(QueryControl), new UIPropertyMetadata(false));

        /// <summary>
        /// 触发维护操作
        /// </summary>
        /// <param name="action"></param>
        void RaiseQueryItem(QueryItemAction action)
        {
            if (QueryItem == null) return;
            //非增加，必须选中一条
            if (action != QueryItemAction.Insert && _dataHost.GetSelectSingleRow() == null) return;
            var args = new RaiseItemActionArgs(_dataHost, action) { IsCompleted = false };
            //执行前检查，若自行处理完毕，则退出
            if (RaiseItemAction != null) RaiseItemAction(this, args);
            if (args.Cancel) return;
            //初始化
            QueryItem = Activator.CreateInstance(QueryItem.GetType()) as IQueryItem;
            if (GetIsDelayRaise(QueryItem as DependencyObject))
            {
                (QueryItem as FrameworkElement).ToolTip = args;//Tag会被覆盖，所以换了ToolTip
                (QueryItem as FrameworkElement).Loaded += (sender, e) =>
                {
                    var item = sender as IQueryItem;
                    if (item == null) return;
                    var arg = (item as FrameworkElement).ToolTip as RaiseItemActionArgs;
                    (item as FrameworkElement).ToolTip = null;
                    item.RaiseAction(arg);
                };
            }
            else
            {
                QueryItem.RaiseAction(args);
            }
            Platform.Presentation.Interfaces.IPage content;
            //若绑定的对象就是IPage，直接打开
            if (QueryItem is Platform.Presentation.Interfaces.IPage)
                content = QueryItem as Platform.Presentation.Interfaces.IPage;
            else
                content = new Platform.Presentation.PageFunctions.TradePage { Content = QueryItem };
            ParentPage.ShowPage(content, "提示窗口");
            //执行完毕，重新触发一次后续交易
            args.IsCompleted = true;
            //结束
            QueryItem.RaiseAction(args);
            if (RaiseItemAction != null) RaiseItemAction(this, args);
        }

        void QueryControl_Loaded(object sender, RoutedEventArgs e)
        {
            throw new NotImplementedException();
        }
        #region 事件
        /// <summary>
        /// 触发Item操作事件
        /// </summary>
        public event EventHandler<RaiseItemActionArgs> RaiseItemAction;
        /// <summary>
        /// RaiseItemAction事件参数
        /// </summary>
        public class RaiseItemActionArgs : CancelEventArgs
        {
            public RaiseItemActionArgs(DataBox dataBox, QueryItemAction action)
            {
                DataHost = dataBox;
                ItemAction = action;
                Cancel = false;
                AttachData = new Dictionary<string, object>();
            }

            public DataBox DataHost { get; private set; }
            /// <summary>
            /// 操作方式
            /// </summary>
            public QueryItemAction ItemAction { get; private set; }
            /// <summary>
            /// 是否完成
            /// </summary>
            public bool IsCompleted { get; internal set; }
            /// <summary>
            /// 
            /// </summary>
            public Dictionary<string, object> AttachData { get; set; }
        }

        #endregion
        /// <summary>
        /// 是否显示编辑链接
        /// </summary>
        public bool LinkEdit { get; set; }
        /// <summary>
        /// 是否显示查看链接
        /// </summary>
        public bool LinkView { get; set; }
        /// <summary>
        /// 是否显示删除链接
        /// </summary>
        public bool LinkDelete { get; set; }

        void RaiseShowLink()
        {
            if (LinkView)
                _dataHost.AddHyperlinkColumn("查看", (sender, e) =>
                {
                    RaiseQueryItem(QueryItemAction.View);
                }, Key.V);
            if (LinkEdit)
                _dataHost.AddHyperlinkColumn("编辑", (sender, e) =>
                {
                    RaiseQueryItem(QueryItemAction.Update);
                }, Key.E);
            if (LinkDelete)
                _dataHost.AddHyperlinkColumn("删除", (sender, e) =>
                {
                    RaiseQueryItem(QueryItemAction.Delete);
                }, Key.D);
        }
        #endregion
    }
    /// <summary>
    /// 查询单单一对象
    /// </summary>
    public interface IQueryItem
    {
        void RaiseAction(QueryControl.RaiseItemActionArgs args);
    }

    public enum QueryItemAction
    {
        /// <summary>
        /// 添加对象
        /// </summary>
        Insert,
        /// <summary>
        /// 更新对象
        /// </summary>
        Update,
        /// <summary>
        /// 查询对象
        /// </summary>
        View,
        /// <summary>
        /// 删除对象
        /// </summary>
        Delete
    }
}
