﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Controls.ControlsHelper;

namespace TellerSystem.Controls.Ext.ControlsExt
{
    /// <summary>
    /// 按照步骤 1a 或 1b 操作，然后执行步骤 2 以在 XAML 文件中使用此自定义控件。
    ///
    /// 步骤 1a) 在当前项目中存在的 XAML 文件中使用该自定义控件。
    /// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
    /// 元素中:
    ///
    ///     xmlns:MyNamespace="clr-namespace:TellerSystem.Controls.Ext.ControlsExt"
    ///
    ///
    /// 步骤 1b) 在其他项目中存在的 XAML 文件中使用该自定义控件。
    /// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
    /// 元素中:
    ///
    ///     xmlns:MyNamespace="clr-namespace:TellerSystem.Controls.Ext.ControlsExt;assembly=TellerSystem.Controls.Ext.ControlsExt"
    ///
    /// 您还需要添加一个从 XAML 文件所在的项目到此项目的项目引用，
    /// 并重新生成以避免编译错误:
    ///
    ///     在解决方案资源管理器中右击目标项目，然后依次单击
    ///     “添加引用”->“项目”->[浏览查找并选择此项目]
    ///
    ///
    /// 步骤 2)
    /// 继续操作并在 XAML 文件中使用控件。
    ///
    ///     <MyNamespace:RangeBox/>
    ///
    /// </summary>
    public class RangeBox : Control
    {
        static RangeBox()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(RangeBox), new FrameworkPropertyMetadata(typeof(RangeBox)));
        }
        public RangeBox()
        {
            PART_ComboBox = new ComboBox { Width = 55, Visibility = Visibility.Collapsed };
            PART_ComboBox.Items.Add(new ComboBoxItem { Content = "录入" });
            PART_ComboBox.Items.Add(new ComboBoxItem { Content = "长期" });
            PART_ComboBox.SelectedIndex = 0;
            PART_ComboBox.SelectionChanged += PART_ComboBox_SelectionChanged;
        }

        #region ComboBox

        private ComboBox PART_ComboBox;

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            var bd = Template.FindName("PART_ComboBox_Border", this) as Border;
            bd.Child = PART_ComboBox;
        }

        void End_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (!AllowLongTerm) return;
            //若赋值为长期，则设置效果
            if (End.Text.Equals("99999999") || End.Text.Equals("99991231"))
            {
                PART_ComboBox.SelectedIndex = 1;
            }
            else {
                PART_ComboBox.SelectedIndex = 0;
            }
        }

        void PART_ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (!AllowLongTerm) return;
            if (LongTerm)
            {
                End.Visibility = Visibility.Hidden;
                End.Text = "99991231";
            }
            else
            {
                End.Visibility = Visibility.Visible;
            }
        }

        public bool AllowLongTerm
        {
            get
            {
                return PART_ComboBox.Visibility == Visibility.Visible;
            }
            set
            {
                PART_ComboBox.Visibility = value ? Visibility.Visible : Visibility.Collapsed;
            }
        }

        /// <summary>
        /// 是否长期有效
        /// </summary>
        public bool LongTerm
        {
            get
            {
                return PART_ComboBox.SelectedIndex == 1;
            }
        }
        #endregion

        #region propdp
        public TextBox Begin
        {
            get { return (TextBox)GetValue(BeginProperty); }
            set { SetValue(BeginProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Begin.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty BeginProperty =
            DependencyProperty.Register("Begin", typeof(TextBox), typeof(RangeBox), new UIPropertyMetadata(null));

        public TextBox End
        {
            get { return (TextBox)GetValue(EndProperty); }
            set { SetValue(EndProperty, value); }
        }

        // Using a DependencyProperty as the backing store for End.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty EndProperty =
            DependencyProperty.Register("End", typeof(TextBox), typeof(RangeBox),
                new UIPropertyMetadata(null, new PropertyChangedCallback((d, e) =>
                {
                    if (e.NewValue == null) return;
                    var element = d as RangeBox;
                    if (element == null) return;
                    element.ProcessEnd();
                })));

        private void ProcessEnd()
        {
            End.TextChanged += End_TextChanged;
            PART_ComboBox.SetBinding(ComboBox.IsEnabledProperty, new Binding { Source = End, Path = new PropertyPath(TextBox.IsEnabledProperty) });
            PART_ComboBox.SetBinding(ComboBox.IsReadOnlyProperty, new Binding { Source = End, Path = new PropertyPath(TextBox.IsReadOnlyProperty) });
        }
        #endregion





    }
}
