﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Effects;
using System.Windows.Media.Media3D;

namespace TellerSystem.Controls.Ext.Controls
{
    /// <summary>
    ///     Rotate3DContainer.xaml 的交互逻辑
    /// </summary>
    public class Rotate3DContainer : Grid
    {
        #region Field
        private Storyboard _back2FrontStory;
        private Border _backWarpper;
        private Storyboard _front2BackStory;
        private Border _frontWarpper;
        #endregion

        #region Property

        /// <summary>
        /// 当前显示项
        /// </summary>
        public int CurrentIndex { get; set; }
        #endregion

        #region Ctor
        /// <summary>
        ///  构造函数
        /// </summary>
        public Rotate3DContainer()
        {
            Loaded += Container_Loaded;
        }
        #endregion

        #region Event
        private void Container_Loaded(object sender, RoutedEventArgs e)
        {
            Init();
        }
        #endregion

        #region Method
        private void Init()
        {
            if (Children.Count == 2)
            {
                var array = new UIElement[2];
                Children.CopyTo(array, 0);
                Children.Clear();

                _frontWarpper = new Border
                {
                    Child = array[0],
                    HorizontalAlignment = HorizontalAlignment.Stretch,
                    VerticalAlignment = VerticalAlignment.Stretch,
                    Background = Brushes.Transparent
                };
                _backWarpper = new Border
                {
                    Child = array[1],
                    Opacity = 0,
                    Visibility = Visibility.Hidden,
                    HorizontalAlignment = HorizontalAlignment.Stretch,
                    VerticalAlignment = VerticalAlignment.Stretch,
                    Background = Brushes.Transparent
                };

                Viewport3D viewport = Get3DView();
                viewport.HorizontalAlignment = HorizontalAlignment.Stretch;
                viewport.VerticalAlignment = VerticalAlignment.Stretch;
                //viewport.Height = _frontWarpper.Child.DesiredSize.Height;
                //viewport.Width = _frontWarpper.Child.DesiredSize.Width;

                _front2BackStory = new Storyboard
                {
                    Children = new TimelineCollection
                    {
                        GetShowHideAnimation(viewport, 0, 800),
                        GetShowHideAnimation(_backWarpper, 800, -1),
                        GetShowHideAnimation(_frontWarpper, -1, 50),
                        GetFadeAnimation(_frontWarpper, 0, -1, 50),
                        GetFadeAnimation(_backWarpper, 1, 750, 50),
                        GetCameraMoveAnimation(0, 0, 1, 0, 0, 1.4, 0, 400, viewport),
                        GetRotateAnimation(0, -180, 0.3, 0.3, 50, 750)
                    }
                };

                _back2FrontStory = new Storyboard
                {
                    Children = new TimelineCollection
                    {
                        GetShowHideAnimation(viewport, 0, 800),
                        GetShowHideAnimation(_frontWarpper, 800, -1),
                        GetShowHideAnimation(_backWarpper, -1, 50),
                        GetFadeAnimation(_backWarpper, 0, -1, 50),
                        GetFadeAnimation(_frontWarpper, 1, 750, 50),
                        GetCameraMoveAnimation(0, 0, 1, 0, 0, 1.4, 0, 400, viewport),
                        GetRotateAnimation(180, 360, 0.3, 0.3, 50, 750)
                    }
                };

                Children.Add(_frontWarpper);
                Children.Add(_backWarpper);
                Children.Add(viewport);
            }
        }

        private DoubleAnimation GetFadeAnimation(UIElement target, int toOpacity, int beginTime, int duration)
        {
            var result = new DoubleAnimation(toOpacity, new Duration(TimeSpan.FromMilliseconds(duration)));
            if (beginTime >= 0)
            {
                result.BeginTime = TimeSpan.FromMilliseconds(beginTime);
            }
            Storyboard.SetTarget(result, target);
            Storyboard.SetTargetProperty(result, new PropertyPath("Opacity"));
            return result;
        }

        private Point3DAnimation GetCameraMoveAnimation(double x1, double y1, double z1, double x2, double y2, double z2,
            int beginTime, int duration, Viewport3D view)
        {
            var result = new Point3DAnimation(new Point3D(x1, y1, z1), new Point3D(x2, y2, z2),
                new Duration(TimeSpan.FromMilliseconds(duration)))
            {
                AutoReverse = true,
                BeginTime = TimeSpan.FromMilliseconds(beginTime),
                DecelerationRatio = 0.3
            };
            Storyboard.SetTarget(result, view);
            Storyboard.SetTargetProperty(result, new PropertyPath("Camera.Position"));
            return result;
        }

        private DoubleAnimation GetRotateAnimation(double fromDegree, double toDegree, double accelerationRatio,
            double decelerationRatio, int beginTime, int duration)
        {
            var result = new DoubleAnimation(fromDegree, toDegree, new Duration(TimeSpan.FromMilliseconds(duration)),
                FillBehavior.HoldEnd)
            {
                AccelerationRatio = accelerationRatio,
                DecelerationRatio = decelerationRatio,
                BeginTime = TimeSpan.FromMilliseconds(beginTime)
            };
            Storyboard.SetTargetName(result, "AxisAngleRotation3D");
            Storyboard.SetTargetProperty(result, new PropertyPath("Angle"));
            return result;
        }

        private Viewport3D Get3DView()
        {
            var viewport = new Viewport3D
            {
                Camera =
                    new PerspectiveCamera(new Point3D(0, 0, 1), new Vector3D(0, 0, -1), new Vector3D(0, 1, 0), 52),
                Visibility = Visibility.Hidden
            };
            viewport.Children.Add(GetLightVisual3D());
            viewport.Children.Add(GetSceneVisual3D(_frontWarpper, _backWarpper));
            return viewport;
        }

        private ModelVisual3D GetLightVisual3D()
        {
            var group = new Model3DGroup();
            group.Children.Add(new DirectionalLight(Color.FromRgb(0x44, 0x44, 0x44), new Vector3D(0, 0, -1)));
            group.Children.Add(new AmbientLight(Color.FromRgb(0xBB, 0xBB, 0xBB)));
            return new ModelVisual3D {Content = group};
        }

        private ModelVisual3D GetSceneVisual3D(Border frontElement, Border backElement)
        {
            double w = 1.0;
            double h = default(double);
            var element = frontElement.Child as FrameworkElement;
            if (element != null)
            {
                h = element.ActualHeight/element.ActualWidth;
            }

            var meshgmod = new MeshGeometry3D
            {
                TriangleIndices = new Int32Collection(new[] {0, 1, 2, 2, 3, 0}),
                TextureCoordinates =
                    new PointCollection(new[] {new Point(0, h), new Point(w, h), new Point(w, 0), new Point(0, 0)}),
                Positions =
                    new Point3DCollection(new[]
                    {
                        new Point3D(-w/2, -h/2, 0), new Point3D(w/2, -h/2, 0), new Point3D(w/2, h/2, 0),
                        new Point3D(-w/2, h/2, 0)
                    })
            };

            var frontBrush = new VisualBrush(frontElement.Child)
            {
                Stretch = Stretch.Uniform
            };

            var backBrush = new VisualBrush(backElement.Child)
            {
                Stretch = Stretch.Uniform,
                RelativeTransform = new ScaleTransform(-1, 1, 0.5, 0)
            };

            var rotate = new AxisAngleRotation3D(new Vector3D(0, 1, 0), 0);
            RegisterName("AxisAngleRotation3D", rotate);

            var gmode3D = new GeometryModel3D
            {
                Geometry = meshgmod,
                Material = new DiffuseMaterial(frontBrush),
                BackMaterial = new DiffuseMaterial(backBrush),
                Transform = new RotateTransform3D(rotate)
            };
            return new ModelVisual3D {Content = gmode3D};
        }

        private ObjectAnimationUsingKeyFrames GetShowHideAnimation(UIElement element, int showTime, int hideTime)
        {
            var frame = new ObjectAnimationUsingKeyFrames();
            Storyboard.SetTarget(frame, element);
            Storyboard.SetTargetProperty(frame, new PropertyPath("Visibility"));
            if (showTime >= 0)
            {
                frame.KeyFrames.Add(new DiscreteObjectKeyFrame(Visibility.Visible, TimeSpan.FromMilliseconds(showTime)));
            }
            if (hideTime >= 0)
            {
                frame.KeyFrames.Add(new DiscreteObjectKeyFrame(Visibility.Hidden, TimeSpan.FromMilliseconds(hideTime)));
            }
            return frame;
        }

        /// <summary>
        ///     翻转
        /// </summary>
        /// <param name="isReverse">旋转方向</param>
        public void Turn(bool isReverse = true)
        {
            Storyboard target;

            double fromAngle = 0;
            double step = 180;

            if (_frontWarpper.Opacity > double.Epsilon)
            {
                target = _front2BackStory;
                CurrentIndex = 1;
            }
            else
            {
                fromAngle = 180;
                target = _back2FrontStory;
                CurrentIndex = 0;
            }

            var direction = (DoubleAnimation) target.Children[6];

            if (!isReverse)
            {
                step = -180;
            }
            direction.From = fromAngle;
            direction.To = fromAngle + step;



            target.Begin(this);
        }
        #endregion
    }
}