﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Shapes;
using System.Windows.Media;
using System.Globalization;

namespace TellerSystem.Controls.Ext.ControlsExt
{

    public class TradePanel : Panel
    {
        public TradePanel()
        {
            this.Margin = new Thickness(0, 5, 0, 0);
        }

        #region 字段
        //奇数列与偶数列的默认右边距
        private double _marginKV = 10;

        //偶数列占容器总宽度的比例
        private double evenColumn = 0.35;

        //奇数列占容器总宽度的比例
        private double oddColumn = 0.65;

        //元素上下之间的间隔值
        private double _marginTop = 8;

        //各列宽度
        private List<double> _columnSize = new List<double>();

        //用于画线
        List<Line> lineList = new List<Line>();

        //用于存储每行的行高信息
        private Dictionary<int, double> rowDetail = new Dictionary<int, double>();

        private int _columns = 4;

        FormattedText ft;

        #endregion

        #region 属性
        public bool IsShowLine
        {
            get { return (bool)GetValue(IsShowLineProperty); }
            set { SetValue(IsShowLineProperty, value); }
        }

        public static readonly DependencyProperty IsShowLineProperty =
            DependencyProperty.Register("IsShowLine", typeof(bool), typeof(TradePanel), new FrameworkPropertyMetadata(false, FrameworkPropertyMetadataOptions.AffectsRender));

        public string Header
        {
            get { return (string)GetValue(HeaderProperty); }
            set { SetValue(HeaderProperty, value); }
        }

        public static readonly DependencyProperty HeaderProperty =
            DependencyProperty.Register("Header", typeof(string), typeof(TradePanel), new FrameworkPropertyMetadata(string.Empty, FrameworkPropertyMetadataOptions.AffectsRender));

        public int Columns
        {
            get { return _columns; }
            set { _columns = value; }
        }
        #endregion

        #region  跨列属性
        public static int GetColumnSpan(DependencyObject obj)
        {
            return (int)obj.GetValue(ColumnSpanProperty);
        }
        public static void SetColumnSpan(DependencyObject obj, int value)
        {
            obj.SetValue(ColumnSpanProperty, value);
        }
        public static readonly DependencyProperty ColumnSpanProperty =
            DependencyProperty.RegisterAttached("ColumnSpan", typeof(int), typeof(TradePanel), new FrameworkPropertyMetadata(1, FrameworkPropertyMetadataOptions.AffectsParentArrange));
        #endregion

        #region  单独居中属性
        public static bool GetIsCentralAlone(DependencyObject obj)
        {
            return (bool)obj.GetValue(IsCentralAloneProperty);
        }
        public static void SetIsCentralAlone(DependencyObject obj, bool value)
        {
            obj.SetValue(IsCentralAloneProperty, value);
        }
        public static readonly DependencyProperty IsCentralAloneProperty =
            DependencyProperty.RegisterAttached("IsCentralAlone", typeof(bool), typeof(TradePanel), new FrameworkPropertyMetadata(false, FrameworkPropertyMetadataOptions.AffectsParentArrange));
        #endregion

        #region 子控件分组

        public static string GetGroupName(DependencyObject obj)
        {
            return (string)obj.GetValue(GroupNameProperty);
        }

        public static void SetGroupName(DependencyObject obj, string value)
        {
            obj.SetValue(GroupNameProperty, value);
        }

        // Using a DependencyProperty as the backing store for GroupName.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty GroupNameProperty =
            DependencyProperty.RegisterAttached("GroupName", typeof(string), typeof(TradePanel), new UIPropertyMetadata(string.Empty));

        #region 相关的处理方法
        /// <summary>
        /// 针对组元素进行集中处理
        /// </summary>
        /// <param name="groupName"></param>
        /// <param name="action"></param>
        public void RaiseGroupAction(string groupName, Action<UIElement> action)
        {
            foreach (var item in this.Children)
            {
                var element = item as UIElement;
                if (element == null) continue;
                if (GetGroupName(element) == groupName)
                    action(element);
            }
        }
        #endregion
        #endregion

        #region 方法
        /// <summary>
        /// 根据指定元素所在的列数，获取其横坐标
        /// </summary>
        /// <param name="col">元素所在的列数</param>
        /// <param name="ele">指定元素</param>
        /// <returns>该列元素的横坐标</returns>
        private double GetX(int col, FrameworkElement ele)
        {
            double re = 0;
            if (GetIsCentralAlone(ele))//添加居中分支
            {
                double sum = 0;
                for (int i = 0; i < _columnSize.Count; i++)
                {
                    sum += _columnSize[i];
                }
                if (ele.Width.Equals(double.NaN))
                {
                    re = _marginKV;
                }
                else
                {
                    re = (sum - ele.DesiredSize.Width) / 2;
                }

                return re;
            }
            else
            {
                int span = GetColumnSpan(ele);
                if (span + col > _columnSize.Count)
                {
                    span = _columnSize.Count - col;
                }
                double totalWidth = 0;
                if (col == 0)
                {
                    for (int i = 0; i < span; i++)
                    {
                        totalWidth += _columnSize[i];
                    }
                    re = totalWidth - ele.DesiredSize.Width - _marginKV;
                    return re;
                }
                else
                {
                    for (int i = 0; i < col; i++)
                    {
                        re += _columnSize[i];
                    }
                    if (col % 2 == 0)
                    {
                        for (int i = 0; i < span; i++)
                        {
                            totalWidth += _columnSize[col + i];
                        }
                        re += (totalWidth - ele.DesiredSize.Width - _marginKV);
                    }
                    return re;
                }
            }
        }

        protected override Size ArrangeOverride(Size finalSize)
        {
            double fontHeight = 0;
            if (!string.IsNullOrEmpty(Header))
            {
                fontHeight = ft.Height;
            }
            int currentColumn = 0;
            double lineHight = 0;
            double currentY = _marginTop + fontHeight;
            int totalColumn = _columnSize.Count;

            for (int i = 0; i < Children.Count; i++)
            {
                var ele = Children[i] as FrameworkElement;
                if (ele != null && ele.Visibility != Visibility.Collapsed)//保证后面的控件往前串
                {
                    if (GetIsCentralAlone(ele)) //设置居中分支
                    {

                        currentColumn = 0;
                        if (lineHight == 0)   //整行隐藏
                        {
                            currentY += lineHight;
                        }
                        else
                        {
                            currentY += _marginTop + lineHight;
                        }
                        if (ele.Width.Equals(double.NaN))
                        {
                            ele.Arrange(new Rect(GetX(currentColumn, ele), currentY + GetY(i, ele, rowDetail), finalSize.Width - _marginKV * 2, ele.DesiredSize.Height));
                        }
                        else
                        {
                            ele.Arrange(new Rect(GetX(currentColumn, ele), currentY + GetY(i, ele, rowDetail), ele.DesiredSize.Width, ele.DesiredSize.Height));
                        }

                        lineHight = ele.DesiredSize.Height;
                        if (lineHight == 0)   //整行隐藏
                        {
                            currentY += lineHight;
                        }
                        else
                        {
                            currentY += _marginTop + lineHight;
                        }
                        lineHight = 0;
                    }
                    else
                    {
                        int cs = GetColumnSpan(ele);
                        if (cs == 0)
                        {
                            cs = 1;
                            SetColumnSpan(ele, 1);
                        }
                        if (cs + currentColumn > totalColumn) //准备换行
                        {
                            currentColumn = 0;
                            if (lineHight == 0)   //整行隐藏
                            {
                                currentY += lineHight;
                            }
                            else
                            {
                                currentY += _marginTop + lineHight;
                            }
                            lineHight = 0;
                        }
                        if (ele.DesiredSize.Height > lineHight)
                        {
                            lineHight = ele.DesiredSize.Height;
                        }
                        ele.Arrange(new Rect(GetX(currentColumn, ele), currentY + GetY(i, ele, rowDetail), ele.DesiredSize.Width, ele.DesiredSize.Height));
                        currentColumn += cs;
                    }
                }
            }
            return finalSize;
        }

        protected override Size MeasureOverride(Size availableSize)
        {
            double fontHeight = 0;
            if (!string.IsNullOrEmpty(Header))
            {
                FontFamily family = new FontFamily("宋体");
                Typeface headerFace = new Typeface(family, FontStyles.Normal, FontWeights.Bold, FontStretches.Normal);
                //#FF0071C0
                ft = new FormattedText(Header, CultureInfo.CurrentCulture, FlowDirection.LeftToRight, headerFace, 14, new SolidColorBrush(Color.FromArgb(0xFF, 0x00, 0x71, 0xC0)));// Brushes.Blue);
                fontHeight = ft.Height;
            }
            for (int i = 0; i < Children.Count; i++)
            {
                var ele = Children[i] as FrameworkElement;
                if (ele != null && ele.Visibility != Visibility.Collapsed)//保证后面的控件往前串
                {
                    ele.Measure(availableSize);
                }
            }
            //宽度不要求自适应，如果没有给定宽度值，需要设置一个默认宽度
            if (double.IsInfinity(availableSize.Width))
            {
                availableSize.Width = 780;
            }
            _columnSize.Clear();
            double total = 0;
            if (Columns % 2 != 0)
            {
                Columns += 1;
                for (int i = 0; i < Columns - 2; i++)
                {
                    double columnWidth = 0;
                    if (i % 2 == 0)
                    {
                        columnWidth = availableSize.Width * (1.0 * 2 / Columns) * evenColumn;

                    }
                    else
                    {
                        columnWidth = availableSize.Width * (1.0 * 2 / Columns) * oddColumn;

                    }
                    total += columnWidth;
                    _columnSize.Add(columnWidth);
                }
                _columnSize.Add(availableSize.Width - total);
                Columns -= 1;
            }
            else
            {
                for (int i = 0; i < Columns; i++)
                {
                    double columnWidth = 0;
                    if (i % 2 == 0)
                    {
                        columnWidth = availableSize.Width * (1.0 * 2 / Columns) * evenColumn;

                    }
                    else
                    {
                        columnWidth = availableSize.Width * (1.0 * 2 / Columns) * oddColumn;
                    }
                    _columnSize.Add(columnWidth);
                }
            }
            SetRowDetail(availableSize);
            //没设高度采用自适应，设置高度采用设置好的高度
            if (double.IsInfinity(availableSize.Height))
            {
                double totalActualHeight = GetTotalHeight(rowDetail);
                if (totalActualHeight == 0)
                {
                    if (string.IsNullOrEmpty(Header))
                    {
                        availableSize.Height = 0;
                    }
                    else
                    {
                        availableSize.Height = fontHeight;
                    }
                }
                else
                {
                    if (string.IsNullOrEmpty(Header))
                    {
                        //一个_marginTop代表第一行与顶边的距离，另一个margin代表最后一行与底边的距离
                        availableSize.Height = totalActualHeight + _marginTop * 2;
                    }
                    else
                    {
                        availableSize.Height = totalActualHeight + _marginTop * 2 + fontHeight;
                    }

                }
            }
            double sum = 0;
            lineList.Clear();
            for (int j = 0; j < Columns - 1; j++)
            {
                sum += _columnSize[j];
                Line line = new Line();
                line.X1 = sum;
                line.X2 = sum;
                line.Y1 = 0;
                line.Y2 = availableSize.Height;
                lineList.Add(line);
            }
            return availableSize;
        }

        protected override void OnRender(DrawingContext dc)
        {
            base.OnRender(dc);
            if (IsShowLine)
            {
                foreach (var line in lineList)
                {
                    dc.DrawLine(new Pen(Brushes.RoyalBlue, 1), new Point(line.X1, line.Y1), new Point(line.X2, line.Y2));
                }
            }
            if (!string.IsNullOrEmpty(Header))
            {
                FontFamily family = new FontFamily("宋体");
                Typeface headerFace = new Typeface(family, FontStyles.Normal, FontWeights.Bold, FontStretches.Normal);
                //#FF0071C0
                ft = new FormattedText(Header, CultureInfo.CurrentCulture, FlowDirection.LeftToRight, headerFace, 14, new SolidColorBrush(Color.FromArgb(0xFF, 0x00, 0x71, 0xC0)));// Brushes.Blue);
                Pen p = new Pen();
                p.Thickness = 0.4;
                p.Brush = Brushes.RoyalBlue;
                Pen p2 = new Pen();
                Rect rect = new Rect(0, 0, this.ActualWidth, this.ActualHeight);
                RectangleGeometry rg = new RectangleGeometry(rect, 0, 0);
                Rect rect2 = new Rect(1, 1, this.ActualWidth - 2, ft.Height);
                RectangleGeometry rg2 = new RectangleGeometry(rect2, 0, 0);
                dc.DrawGeometry(null, p, rg);
                //C0EBEBEB
                dc.DrawGeometry(new SolidColorBrush(Color.FromArgb(0xcA, 0xeb, 0xeb, 0xeb)), null, rg2);
                dc.DrawText(ft, new Point(9, 1));
            }
        }

        private void SetRowDetail(Size size)
        {
            rowDetail.Clear();
            double lineHight = 0;
            //每行最后一个元素的下标
            int lineeleNumber = 0;
            int totalColumn = _columnSize.Count;
            int currentColumn = 0;
            for (int i = 0; i < Children.Count; i++)
            {
                var ele = Children[i] as FrameworkElement;
                if (ele != null && ele.Visibility != Visibility.Collapsed)
                {
                    if (GetIsCentralAlone(ele))  //添加设置居中的分支
                    {
                        if (currentColumn == 0)
                        {
                            lineeleNumber = i;
                            lineHight = ele.DesiredSize.Height;
                            rowDetail.Add(lineeleNumber, lineHight);
                            lineHight = 0;
                        }
                        else
                        {
                            currentColumn = 0;
                            lineeleNumber = i - 1;
                            rowDetail.Add(lineeleNumber, lineHight);
                            lineHight = ele.DesiredSize.Height;
                            lineeleNumber = i;
                            rowDetail.Add(lineeleNumber, lineHight);
                            lineHight = 0;
                        }

                    }
                    else
                    {
                        int cs = GetColumnSpan(ele);
                        if (cs == 0)
                        {
                            cs = 1;
                            SetColumnSpan(ele, 1);
                        }
                        if (cs + currentColumn > totalColumn) //准备换行
                        {
                            currentColumn = 0;
                            lineeleNumber = i - 1;
                            rowDetail.Add(lineeleNumber, lineHight);
                            lineHight = 0;
                        }
                        if (ele.DesiredSize.Height > lineHight)
                        {
                            lineHight = ele.DesiredSize.Height;
                        }
                        currentColumn += cs;
                    }
                }
            }
            int lastIndex = Children.Count - 1;
            if (!rowDetail.ContainsKey(lastIndex))
            {
                rowDetail.Add(lastIndex, lineHight);
            }
        }

        private double GetY(int index, FrameworkElement ele, Dictionary<int, double> dic)
        {
            double y = 0;
            double height = ele.DesiredSize.Height;

            foreach (KeyValuePair<int, double> kvp in dic)
            {
                if (index <= kvp.Key)
                {
                    y = (kvp.Value - height) / 2.0;
                    break;
                }

            }
            return y; ;
        }

        /// <summary>
        /// 获取子元素的实际高度总和(包括间隙)
        /// </summary>
        private double GetTotalHeight(Dictionary<int, double> dic)
        {
            //计算行数
            int lines = 0;
            double height = 0;
            foreach (var item in dic)
            {
                if (item.Value > 0)
                {
                    height += item.Value;
                    lines++;
                }
            }
            //加上行与行之间的高度
            if (lines > 1)
            {
                height += (lines - 1) * _marginTop;
            }
            return height;
        }
        #endregion

    }
}
