﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TellerSystem.Contract.ContractBase;
using TellerSystem.Contract.Function;
using Platform.Common.RunningParameters;
using System.Windows;
using TellerSystem.Communication;
using TellerSystem.Library.Ext.TradeExtension;

namespace TellerSystem.Controls.Ext.ControlsHelper
{
    /// <summary>
    /// 业务API集合
    /// </summary>
    public static class BusinessFunction
    {
        #region 公共
        #endregion

        #region 客户管理
        #endregion

        #region 存款业务
        /// <summary>
        /// 通过账号，决定一条账户记录
        /// </summary>
        /// <param name="target"></param>
        /// <param name="accountNo"></param>
        /// <returns></returns>
        public static QueryDepositAccount BF_Deposit_GetAccountInfo(this TradeAPIsHelper target, string accountNo)
        {
            var output = new QueryDepositAccount();
            var page = target.Page;
            var subacinfo = target.TF_Deposit_QueryChildAccountList(new QueryChildAccountList.Input
            {
                ZH = accountNo
            });
            if (subacinfo.IsFailed)
            {
                target.ShowErrorMessageBox(subacinfo.FailureCause);
                target.SetFocus();
                return null;
            }
            var acSeqn = string.Empty;
            if (!string.IsNullOrEmpty(subacinfo.ZZHXX))
            {
                var selectedData = ShowPages.ShowDataBoxViewer(target.Page, subacinfo.ZZHXX, new string[0]);
                if (selectedData == null)
                {
                    output.SetFailure("没有可选账号信息！");
                    return output;
                }
                acSeqn = selectedData["账户序号"].ToString();
            }
            else
            {
                output.SetFailure("没有可选账号信息！");
                return output;
            }
            return output = target.TF_Deposit_QueryDepositAccount(new QueryDepositAccount.Input { ZH = accountNo, XH = acSeqn });
        }
        /// 根据凭证类别获取凭证类型码
        /// </summary>
        /// <param name="target">ssss</param>
        /// <param name="PZLB">凭证类别</param>
        /// <param name="PZLX">凭证类型</param>
        /// <returns></returns>
        public static string GetVoucherTypeCode(this TradeAPIsHelper target, string PZLB, string PZLX)
        {

            var tf = target.TF_Public_QueryComboBoxData(new QueryComboBoxData.Input()
            {
                LBBM = "0011",
                FJTJ = "{NOTE_KIND}|" + GetCategoryCode(target, PZLB) + "|",
            });
            if (tf.IsFailed)
            {
                target.ShowErrorMessageBox(tf.FailureCause);
            }
            var ret = tf.CDXXData.FirstOrDefault(P => P.MS == PZLX);
            return ret == null ? "" : ret.LXZ;
        }
        /// <summary>
        /// 获取凭证类别码
        /// </summary>
        /// <param name="?"></param>
        /// <returns></returns>
        public static string GetCategoryCode(this TradeAPIsHelper target, string PZLB)
        {
            var tf = target.TF_Public_QueryComboBoxData(new QueryComboBoxData.Input()
            {
                LBBM = "0003"
            });
            if (tf.IsFailed)
            {
                target.ShowErrorMessageBox(tf.FailureCause);
                return string.Empty;
            }
            var ret = tf.CDXXData.FirstOrDefault(P => P.MS == PZLB);
            return ret == null ? "" : ret.LXZ;
        }
        /// 根据凭证类别获取凭证类型码_印章
        /// </summary>
        /// <param name="target">ssss</param>
        /// <param name="PZLB">凭证类别</param>
        /// <param name="PZLX">凭证类型</param>
        /// <returns></returns>
        public static string GetVoucherTypeCode_YZ(this TradeAPIsHelper target, string PZLB, string PZLX)
        {

            var tf = target.TF_Public_QueryComboBoxData(new QueryComboBoxData.Input()
            {
                LBBM = "0011",
                FJTJ = "{NOTE_KIND}|" + GetCategoryCode_YZ(target, PZLB) + "|",
            });
            if (tf.IsFailed)
            {
                target.ShowErrorMessageBox(tf.FailureCause);
            }
            var ret = tf.CDXXData.FirstOrDefault(P => P.MS == PZLX);
            return ret == null ? "" : ret.LXZ;
        }
        /// <summary>
        /// 获取凭证类别码
        /// </summary>
        /// <param name="?"></param>
        /// <returns></returns>
        public static string GetCategoryCode_YZ(this TradeAPIsHelper target, string PZLB)
        {
            var tf = target.TF_Public_QueryComboBoxData(new QueryComboBoxData.Input()
            {
                LBBM = "000Z"
            });
            if (tf.IsFailed)
            {
                target.ShowErrorMessageBox(tf.FailureCause);
                return string.Empty;
            }
            var ret = tf.CDXXData.FirstOrDefault(P => P.MS == PZLB);
            return ret == null ? "" : ret.LXZ;
        }
        /// <summary>
        /// 获取模式号码
        /// </summary>
        /// <param name="target"></param>
        /// <param name="str"></param>
        /// <returns></returns>
        public static string GetNumber(this TradeAPIsHelper target, string name)
        {
            var tf = target.TF_Public_QueryComboBoxData(new QueryComboBoxData.Input()
            {
                LBBM = "0015"
            });
            if (tf.IsFailed)
            {
                target.ShowErrorMessageBox(tf.FailureCause);
                return string.Empty;
            }
            var ret = tf.CDXXData.FirstOrDefault(P => P.MS == name);
            return ret == null ? "" : ret.LXZ;
        }
        /// <summary>
        /// 检查支付密码器
        /// </summary>
        /// <param name="target"></param>
        /// <param name="zh">账号</param>
        /// <param name="cprq">出票日期</param>
        /// <param name="jyje">交易金额</param>
        /// <param name="pzlx">凭证种类</param>
        /// <param name="pzhm">凭证号码</param>
        /// <param name="mm">密码</param>
        public static bool CheckPayPassword(this TradeAPIsHelper target, string zh, string cprq,
                                      string jyje, string pzlx, string pzhm, string mm)
        {
            //检测开关:IsSkipCheckPayPassword
            if (string.Compare(ConfigManager.GetInstance().GetConfigValue("IsSkipCheckPayPassword", ConfigType.System), "true", true) == 0)
            {
                if (target.ShowQuestionMessageBox("支付密码验证:是否进行支付密码验证？") != MessageBoxResult.Yes)
                    return true;
            }
            //获取验证方式
            //调用支付密码方式接口
            var pwdtype = target.TF_Deposit_PasswordAuthentication(new PasswordAuthentication.Input()
            {
                ZH = zh,
                CPRQ = cprq,
                YZBZ = "4"
            });
            if (pwdtype.IsFailed)
            {
                //this.ShowMessageBox(pwdtype.FailureCause);
                target.ShowErrorMessageBox(pwdtype.FailureCause);
                return false;
            }
            //给全局变量赋值
            var type = pwdtype.PZMMYZFS;

            var pwd = new PasswordAuthentication.Input();
            //密码单支付
            if (type == "0")
            {
                pwd.YZBZ = "2";
            }
            //支付密码器支付
            if (type == "1")
            {
                pwd.YZBZ = "3";//验证标识
                //pwd.GYH = LoginUserInfo.TellerNo;//柜员号
                pwd.PMJE = jyje;//交易金额
            }
            pwd.ZH = zh;
            pwd.MM = mm;//密码
            pwd.CPRQ = cprq;//出票日期
            pwd.PZLX = pzlx;//凭证类型
            pwd.PZHM = pzhm;//凭证号码
            var passWord = target.TF_Deposit_PasswordAuthentication(pwd);
            if (passWord.IsFailed)
            {
                target.ShowErrorMessageBox(passWord.FailureCause);
                return false;
            }
            return true;
        }

        /// <summary>
        /// 检查验印流水
        /// </summary>
        /// <param name="target"></param>
        /// <param name="ZH">账号</param>
        /// <param name="PZHM">凭证号码</param>
        /// <param name="JYJE">交易金额</param>
        /// <param name="CPRQ">出票日期</param>
        /// <param name="PZZL">凭证种类</param>
        /// <param name="YYLSH">验印流水号</param>
        /// <returns></returns>
        public static bool CheckYanYinWater(this TradeAPIsHelper target, string ZH, string PZHM, string JYJE,
            string CPRQ, string PZZL, string YYLSH)
        {
            //检测开关:IsSkipCheckYanYinWater
            if (string.Compare(ConfigManager.GetInstance().GetConfigValue("IsSkipCheckYanYinWater", ConfigType.System), "true", true) == 0)
            {
                if (target.ShowQuestionMessageBox("验印流水验证:是否进行验印流水验证？") != MessageBoxResult.Yes)
                    return true;
            }
            //拉取数据        
            var message = new Message(Message.BitMap.Agn);
            //target.InitFd(message);
            target.CommSend(message);
            message.Fd61 = "8200";
            message.Fd60 = "CIS";//业务标识
            message.Fd30 = ZH;//付款人账号
            message.Fd31 = PZHM;//凭证号码
            message.Fd39 = JYJE;//交易金额
            message.Fd45 = CPRQ;//出票日期
            message.Fd52 = PZZL;//凭证种类
            message.Fd62 = YYLSH.PadLeft(16, '0');//验印流水号
            //对方已经支持新老数据转换
            ////使用旧柜员号、旧机构号
            ////旧开户行机构号
            //message.Fd2 = LoginAuthentication.Instance.KinbrComment;
            ////旧机构号
            //message.Fd3 = LoginAuthentication.Instance.KinbrComment;
            ////旧柜员号
            //message.Fd7 = LoginAuthentication.Instance.UserComment;
            target.CallAgnCIS(message);//callagn_cis();
            if (message.Fd12 != "0000")
            {
                target.ShowErrorMessageBox(target.GetError(message));
                return false;
            }
            //恢复新柜员号、新机构号
            message.Fd2 = LoginUserInfo.KinbrNo;
            message.Fd3 = LoginUserInfo.KinbrNo;
            message.Fd7 = LoginUserInfo.TellerNo;
            var tmpstat1 = message.Fd34;
            var tmpstat2 = message.Fd35;
            if (tmpstat1 == "0")
            {
                target.ShowErrorMessageBox("该账户未验印!");
                return false;
            }
            else if (tmpstat1 == "2")
            {
                target.ShowErrorMessageBox("自动验印不通过!");
                return false;
            }
            else if (tmpstat1 == "4")
            {
                target.ShowErrorMessageBox("人工验印不通过!");
                return false;
            }
            return true;
        }
        #endregion

        /// <summary>
        /// 检查磁道信息
        /// </summary>
        /// <param name="target"></param>
        /// <param name="textBox">待检查的控件Text内容是否来源于外设</param>
        /// <param name="queryDepositAccount">账户信息</param>
        /// <param name="Msr2">二磁信息</param>
        /// <param name="Msr3">三磁信息</param>
        /// <param name="checkType">校验方式</param>
        /// <returns>是否越过磁道信息检查</returns>
        public static bool CheckCardMsrInfo(this TradeAPIsHelper target, System.Windows.Controls.TextBox textBox, QueryDepositAccount queryDepositAccount, string Msr2 = null, string Msr3 = null, string checkType = "1")
        {
            //检测开关:InDeviceDebugMode
            if (string.Compare(ConfigManager.GetInstance().GetConfigValue("InDeviceDebugMode", ConfigType.System), "true", true) == 0)
            {
                if (target.ShowQuestionMessageBox("磁道信息校验:是否进行卡、折、一本通磁道信息校验？") != MessageBoxResult.Yes)
                    return true;
            }
            //卡 折 一本通 必须刷磁
            if (target.CheckCardBin(queryDepositAccount.CLJZH.Trim()) ||
                queryDepositAccount.JZZL.Trim().Equals(TellerSystem.Library.Ext.Variables.UserVariable.JZZL_DQYBT) ||
                queryDepositAccount.JZZL.Trim().Equals(TellerSystem.Library.Ext.Variables.UserVariable.JZZL_CZ))
            {
                if (!target.CheckMsrDataByDevice(textBox))
                {
                    target.ShowInfoMessageBox("卡、折、一本通必须刷磁");
                    return false;
                }
                //校验二磁三磁信息
                var shortTrading = target.TF_Public_QueryCardMsrInfo(
                    new QueryCardMsrInfo.Input()
                    {
                        ZH = textBox.Text.Trim(),
                        JYFS = checkType,
                        MSR2 = Msr2,
                        MSR3 = Msr3,
                    });
                if (shortTrading.IsFailed)
                {
                    target.ShowErrorMessageBox(shortTrading.FailureCause);
                    return false;
                }
            }
            return true;
        }

        #region 代理人标识
        public static bool GetProxyStatus(DependencyObject obj)
        {
            return (bool)obj.GetValue(ProxyStatusProperty);
        }

        public static void SetProxyStatus(DependencyObject obj, bool value)
        {
            obj.SetValue(ProxyStatusProperty, value);
        }

        // Using a DependencyProperty as the backing store for ProxyStatus.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ProxyStatusProperty =
            DependencyProperty.RegisterAttached("ProxyStatus", typeof(bool), typeof(BusinessFunction), new UIPropertyMetadata(false,
                (d, e) =>
                {
                    //当代理状态发生变化，同步处理SkipFaceComp标识
                    var page = d as PageBase;
                    page.TradeAPIs.IsSkipFaceComp((bool)e.NewValue);
                }));

        /// <summary>
        /// 获取或设置当前交易是否为代理状态
        /// </summary>
        /// <param name="target"></param>edssss
        /// <param name="proxy"></param>
        /// <returns></returns>
        public static bool IsProxyStatus(this TradeAPIsHelper target, bool? proxy = null)
        {
            if (proxy == null)
                return GetProxyStatus(target.Page);
            SetProxyStatus(target.Page, proxy.Value);
            return proxy.Value;
        }
        #endregion

        #region 跳过人脸识别
        public static bool GetSkipFaceComp(DependencyObject obj)
        {
            return (bool)obj.GetValue(SkipFaceCompProperty);
        }

        public static void SetSkipFaceComp(DependencyObject obj, bool value)
        {
            obj.SetValue(SkipFaceCompProperty, value);
        }

        // Using a DependencyProperty as the backing store for SkipFaceComp.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SkipFaceCompProperty =
            DependencyProperty.RegisterAttached("SkipFaceComp", typeof(bool), typeof(BusinessFunction), new UIPropertyMetadata(false));

        /// <summary>
        /// 是否跳过人脸对比标识
        /// </summary>
        /// <param name="target"></param>
        /// <param name="flag"></param>
        /// <returns></returns>
        public static bool IsSkipFaceComp(this TradeAPIsHelper target, bool? flag = null)
        {
            if (flag == null)
                return GetSkipFaceComp(target.Page);
            SetSkipFaceComp(target.Page, flag.Value);
            return flag.Value;
        }

        /// <summary>
        /// 获取或设置当前交易是否为存单开户
        /// </summary>
        /// <param name="target"></param>edssss
        /// <param name="proxy"></param>
        /// <returns></returns>
        [Obsolete("过期的调用，兼容保留。请使用IsSkipFaceComp")]
        public static bool IsJZStatus(this TradeAPIsHelper target, bool? proxy = null)
        {
            return IsSkipFaceComp(target, proxy);
        }
        #endregion
    }
}
