import { ref } from 'vue';

// 假设这些类型在其他文件中定义，这里简单声明
interface TradeAPIsHelper {
    Page: any;
    TF_Deposit_QueryChildAccountList: (input: any) => any;
    ShowErrorMessageBox: (message: string) => void;
    SetFocus: () => void;
    TF_Deposit_QueryDepositAccount: (input: any) => any;
    TF_Public_QueryComboBoxData: (input: any) => any;
    TF_Deposit_PasswordAuthentication: (input: any) => any;
    ShowQuestionMessageBox: (message: string) => any;
    CommSend: (message: any) => void;
    CallAgnCIS: (message: any) => void;
    GetError: (message: any) => string;
    ShowInfoMessageBox: (message: string) => void;
    CheckCardBin: (cljzh: string) => boolean;
    CheckMsrDataByDevice: (textBox: any) => boolean;
    IsSkipFaceComp: (flag?: boolean | null) => boolean;
}

interface QueryDepositAccount {
    SetFailure: (message: string) => void;
}

interface ConfigManager {
    static GetInstance: () => {
        GetConfigValue: (key: string, type: any) => string;
    };
}

interface Message {
    static BitMap: {
        Agn: any;
    };
    Fd61: string;
    Fd60: string;
    Fd30: string;
    Fd31: string;
    Fd39: string;
    Fd45: string;
    Fd52: string;
    Fd62: string;
    Fd12: string;
    Fd2: string;
    Fd3: string;
    Fd7: string;
    Fd34: string;
    Fd35: string;
}

interface DependencyObject {
    GetValue: (property: any) => any;
    SetValue: (property: any, value: any) => void;
}

interface PageBase {
    TradeAPIs: TradeAPIsHelper;
}

// 业务 API 集合
export class BusinessFunction {
    // 公共部分暂时为空

    // 客户管理部分暂时为空

    // 存款业务
    /**
     * 通过账号，决定一条账户记录
     * @param target 
     * @param accountNo 
     * @returns 
     */
    static BF_Deposit_GetAccountInfo(target: TradeAPIsHelper, accountNo: string): QueryDepositAccount | null {
        const output = {} as QueryDepositAccount;
        const page = target.Page;
        const subacinfo = target.TF_Deposit_QueryChildAccountList({
            ZH: accountNo
        });
        if (subacinfo.IsFailed) {
            target.ShowErrorMessageBox(subacinfo.FailureCause);
            target.SetFocus();
            return null;
        }
        let acSeqn = '';
        if (subacinfo.ZZHXX) {
            const selectedData = ShowPages.ShowDataBoxViewer(page, subacinfo.ZZHXX, []);
            if (!selectedData) {
                output.SetFailure('没有可选账号信息！');
                return output;
            }
            acSeqn = selectedData["账户序号"].toString();
        } else {
            output.SetFailure('没有可选账号信息！');
            return output;
        }
        return target.TF_Deposit_QueryDepositAccount({ ZH: accountNo, XH: acSeqn });
    }

    /**
     * 根据凭证类别获取凭证类型码
     * @param target 
     * @param PZLB 凭证类别
     * @param PZLX 凭证类型
     * @returns 
     */
    static GetVoucherTypeCode(target: TradeAPIsHelper, PZLB: string, PZLX: string): string {
        const tf = target.TF_Public_QueryComboBoxData({
            LBBM: "0011",
            FJTJ: "{NOTE_KIND}|" + this.GetCategoryCode(target, PZLB) + "|",
        });
        if (tf.IsFailed) {
            target.ShowErrorMessageBox(tf.FailureCause);
        }
        const ret = tf.CDXXData.find((P: any) => P.MS === PZLX);
        return ret? ret.LXZ : '';
    }

    /**
     * 获取凭证类别码
     * @param target 
     * @param PZLB 
     * @returns 
     */
    static GetCategoryCode(target: TradeAPIsHelper, PZLB: string): string {
        const tf = target.TF_Public_QueryComboBoxData({
            LBBM: "0003"
        });
        if (tf.IsFailed) {
            target.ShowErrorMessageBox(tf.FailureCause);
            return '';
        }
        const ret = tf.CDXXData.find((P: any) => P.MS === PZLB);
        return ret? ret.LXZ : '';
    }

    /**
     * 根据凭证类别获取凭证类型码_印章
     * @param target 
     * @param PZLB 凭证类别
     * @param PZLX 凭证类型
     * @returns 
     */
    static GetVoucherTypeCode_YZ(target: TradeAPIsHelper, PZLB: string, PZLX: string): string {
        const tf = target.TF_Public_QueryComboBoxData({
            LBBM: "0011",
            FJTJ: "{NOTE_KIND}|" + this.GetCategoryCode_YZ(target, PZLB) + "|",
        });
        if (tf.IsFailed) {
            target.ShowErrorMessageBox(tf.FailureCause);
        }
        const ret = tf.CDXXData.find((P: any) => P.MS === PZLX);
        return ret? ret.LXZ : '';
    }

    /**
     * 获取凭证类别码
     * @param target 
     * @param PZLB 
     * @returns 
     */
    static GetCategoryCode_YZ(target: TradeAPIsHelper, PZLB: string): string {
        const tf = target.TF_Public_QueryComboBoxData({
            LBBM: "000Z"
        });
        if (tf.IsFailed) {
            target.ShowErrorMessageBox(tf.FailureCause);
            return '';
        }
        const ret = tf.CDXXData.find((P: any) => P.MS === PZLB);
        return ret? ret.LXZ : '';
    }

    /**
     * 获取模式号码
     * @param target 
     * @param name 
     * @returns 
     */
    static GetNumber(target: TradeAPIsHelper, name: string): string {
        const tf = target.TF_Public_QueryComboBoxData({
            LBBM: "0015"
        });
        if (tf.IsFailed) {
            target.ShowErrorMessageBox(tf.FailureCause);
            return '';
        }
        const ret = tf.CDXXData.find((P: any) => P.MS === name);
        return ret? ret.LXZ : '';
    }

    /**
     * 检查支付密码器
     * @param target 
     * @param zh 账号
     * @param cprq 出票日期
     * @param jyje 交易金额
     * @param pzlx 凭证种类
     * @param pzhm 凭证号码
     * @param mm 密码
     * @returns 
     */
    static CheckPayPassword(target: TradeAPIsHelper, zh: string, cprq: string,
        jyje: string, pzlx: string, pzhm: string, mm: string): boolean {
        // 检测开关:IsSkipCheckPayPassword
        if (ConfigManager.GetInstance().GetConfigValue("IsSkipCheckPayPassword", "System").toLowerCase() === "true") {
            if (target.ShowQuestionMessageBox("支付密码验证:是否进行支付密码验证？")!== 'Yes')
                return true;
        }
        // 获取验证方式
        // 调用支付密码方式接口
        const pwdtype = target.TF_Deposit_PasswordAuthentication({
            ZH: zh,
            CPRQ: cprq,
            YZBZ: "4"
        });
        if (pwdtype.IsFailed) {
            target.ShowErrorMessageBox(pwdtype.FailureCause);
            return false;
        }
        // 给全局变量赋值
        const type = pwdtype.PZMMYZFS;

        const pwd = {} as any;
        // 密码单支付
        if (type === "0") {
            pwd.YZBZ = "2";
        }
        // 支付密码器支付
        if (type === "1") {
            pwd.YZBZ = "3";// 验证标识
            // pwd.GYH = LoginUserInfo.TellerNo;// 柜员号
            pwd.PMJE = jyje;// 交易金额
        }
        pwd.ZH = zh;
        pwd.MM = mm;// 密码
        pwd.CPRQ = cprq;// 出票日期
        pwd.PZLX = pzlx;// 凭证类型
        pwd.PZHM = pzhm;// 凭证号码
        const passWord = target.TF_Deposit_PasswordAuthentication(pwd);
        if (passWord.IsFailed) {
            target.ShowErrorMessageBox(passWord.FailureCause);
            return false;
        }
        return true;
    }

    /**
     * 检查验印流水
     * @param target 
     * @param ZH 账号
     * @param PZHM 凭证号码
     * @param JYJE 交易金额
     * @param CPRQ 出票日期
     * @param PZZL 凭证种类
     * @param YYLSH 验印流水号
     * @returns 
     */
    static CheckYanYinWater(target: TradeAPIsHelper, ZH: string, PZHM: string, JYJE: string,
        CPRQ: string, PZZL: string, YYLSH: string): boolean {
        // 检测开关:IsSkipCheckYanYinWater
        if (ConfigManager.GetInstance().GetConfigValue("IsSkipCheckYanYinWater", "System").toLowerCase() === "true") {
            if (target.ShowQuestionMessageBox("验印流水验证:是否进行验印流水验证？")!== 'Yes')
                return true;
        }
        // 拉取数据        
        const message = new Message(Message.BitMap.Agn);
        // target.InitFd(message);
        target.CommSend(message);
        message.Fd61 = "8200";
        message.Fd60 = "CIS";// 业务标识
        message.Fd30 = ZH;// 付款人账号
        message.Fd31 = PZHM;// 凭证号码
        message.Fd39 = JYJE;// 交易金额
        message.Fd45 = CPRQ;// 出票日期
        message.Fd52 = PZZL;// 凭证种类
        message.Fd62 = YYLSH.padStart(16, '0');// 验印流水号
        // 对方已经支持新老数据转换
        // // 使用旧柜员号、旧机构号
        // // 旧开户行机构号
        // message.Fd2 = LoginAuthentication.Instance.KinbrComment;
        // // 旧机构号
        // message.Fd3 = LoginAuthentication.Instance.KinbrComment;
        // // 旧柜员号
        // message.Fd7 = LoginAuthentication.Instance.UserComment;
        target.CallAgnCIS(message);// callagn_cis();
        if (message.Fd12!== "0000") {
            target.ShowErrorMessageBox(target.GetError(message));
            return false;
        }
        // 恢复新柜员号、新机构号
        message.Fd2 = "LoginUserInfo.KinbrNo";
        message.Fd3 = "LoginUserInfo.KinbrNo";
        message.Fd7 = "LoginUserInfo.TellerNo";
        const tmpstat1 = message.Fd34;
        const tmpstat2 = message.Fd35;
        if (tmpstat1 === "0") {
            target.ShowErrorMessageBox("该账户未验印!");
            return false;
        } else if (tmpstat1 === "2") {
            target.ShowErrorMessageBox("自动验印不通过!");
            return false;
        } else if (tmpstat1 === "4") {
            target.ShowErrorMessageBox("人工验印不通过!");
            return false;
        }
        return true;
    }

    /**
     * 检查磁道信息
     * @param target 
     * @param textBox 待检查的控件Text内容是否来源于外设
     * @param queryDepositAccount 账户信息
     * @param Msr2 二磁信息
     * @param Msr3 三磁信息
     * @param checkType 校验方式
     * @returns 是否越过磁道信息检查
     */
    static CheckCardMsrInfo(target: TradeAPIsHelper, textBox: any, queryDepositAccount: QueryDepositAccount, Msr2: string | null = null, Msr3: string | null = null, checkType: string = "1"): boolean {
        // 检测开关:InDeviceDebugMode
        if (ConfigManager.GetInstance().GetConfigValue("InDeviceDebugMode", "System").toLowerCase() === "true") {
            if (target.ShowQuestionMessageBox("磁道信息校验:是否进行卡、折、一本通磁道信息校验？")!== 'Yes')
                return true;
        }
        // 卡 折 一本通 必须刷磁
        if (target.CheckCardBin(queryDepositAccount.CLJZH.trim()) ||
            queryDepositAccount.JZZL.trim() === "TellerSystem.Library.Ext.Variables.UserVariable.JZZL_DQYBT" ||
            queryDepositAccount.JZZL.trim() === "TellerSystem.Library.Ext.Variables.UserVariable.JZZL_CZ") {
            if (!target.CheckMsrDataByDevice(textBox)) {
                target.ShowInfoMessageBox("卡、折、一本通必须刷磁");
                return false;
            }
            // 校验二磁三磁信息
            const shortTrading = target.TF_Public_QueryCardMsrInfo({
                ZH: textBox.Text.trim(),
                JYFS: checkType,
                MSR2: Msr2,
                MSR3: Msr3,
            });
            if (shortTrading.IsFailed) {
                target.ShowErrorMessageBox(shortTrading.FailureCause);
                return false;
            }
        }
        return true;
    }

    // 代理人标识
    static GetProxyStatus(obj: DependencyObject): boolean {
        return obj.GetValue(ProxyStatusProperty) as boolean;
    }

    static SetProxyStatus(obj: DependencyObject, value: boolean): void {
        obj.SetValue(ProxyStatusProperty, value);
    }

    // Using a DependencyProperty as the backing store for ProxyStatus.  This enables animation, styling, binding, etc...
    static readonly ProxyStatusProperty = {
        // 这里简单模拟依赖属性，实际 Vue 中需要不同实现
        get: (obj: DependencyObject) => obj.GetValue(ProxyStatusProperty),
        set: (obj: DependencyObject, value: boolean) => obj.SetValue(ProxyStatusProperty, value)
    };

    /**
     * 获取或设置当前交易是否为代理状态
     * @param target 
     * @param proxy 
     * @returns 
     */
    static IsProxyStatus(target: TradeAPIsHelper, proxy: boolean | null = null): boolean {
        if (proxy === null)
            return this.GetProxyStatus(target.Page);
        this.SetProxyStatus(target.Page, proxy);
        return proxy;
    }

    // 跳过人脸识别
    static GetSkipFaceComp(obj: DependencyObject): boolean {
        return obj.GetValue(SkipFaceCompProperty) as boolean;
    }

    static SetSkipFaceComp(obj: DependencyObject, value: boolean): void {
        obj.SetValue(SkipFaceCompProperty, value);
    }

    // Using a DependencyProperty as the backing store for SkipFaceComp.  This enables animation, styling, binding, etc...
    static readonly SkipFaceCompProperty = {
        // 这里简单模拟依赖属性，实际 Vue 中需要不同实现
        get: (obj: DependencyObject) => obj.GetValue(SkipFaceCompProperty),
        set: (obj: DependencyObject, value: boolean) => obj.SetValue(SkipFaceCompProperty, value)
    };

    /**
     * 是否跳过人脸对比标识
     * @param target 
     * @param flag 
     * @returns 
     */
    static IsSkipFaceComp(target: TradeAPIsHelper, flag: boolean | null = null): boolean {
        if (flag === null)
            return this.GetSkipFaceComp(target.Page);
        this.SetSkipFaceComp(target.Page, flag);
        return flag;
    }

    /**
     * 获取或设置当前交易是否为存单开户
     * @param target 
     * @param proxy 
     * @returns 
     */
    static IsJZStatus(target: TradeAPIsHelper, proxy: boolean | null = null): boolean {
        return this.IsSkipFaceComp(target, proxy);
    }
}