﻿using System.Collections.Generic;
using System.Data;
using System.IO;
using Platform.Controls.ControlsHelper;
using Platform.Library;
using Platform.Presentation.Interfaces;
using TellerSystem.Communication;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Library.Ext.TradeExtension;
using TellerSystem.Controls.Ext.Controls;
using TellerSystem.Devices.Interface;
using TellerSystem.ServiceProxy.Ext.ServiceEntitys;
using TellerSystem.Contract.ContractBase;
using System.Windows;
using TellerSystem.Contract.Contracts;
using System.Windows.Controls;
using System;
using Platform.Common.RunningParameters;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.Contract.Function;
using TellerSystem.Controls.Ext.ControlEntity;

namespace TellerSystem.Controls.Ext.ControlsHelper
{
    public static class ShowPages
    {
        private static string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
        /// <summary>
        /// 文件查看——fview函数
        /// </summary>
        /// <param name="target"></param>
        /// <param name="fileInfo"></param>
        /// <param name="isFileData"></param>
        /// <returns></returns>
        public static bool ShowFileViewer(this IPage target, string fileInfo, bool isFileData = true)
        {
            var fileViewer = new FileViewer();
            if (isFileData)
            {
                fileViewer.TxtInit(fileInfo);
            }
            else
            {
                fileViewer.Init(fileInfo, false);
            }
            target.ShowPage(fileViewer, "查看文件");
            return true;
        }
        /// <summary>
        /// 文件查看--fview 函数
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="fileInfo"></param>
        /// <param name="isFileData"></param>
        /// <returns></returns>
        public static bool ShowFileViewer(this TradeAPIsHelper helper, string fileInfo, bool isFileData = true)
        {
            return helper.Page.ShowFileViewer(fileInfo, isFileData);
        }
        /// <summary>
        /// 文件查看——fview函数
        /// </summary>
        /// <param name="target"></param>
        /// <param name="fileInfo"></param>
        /// <param name="isFileData"></param>
        /// <returns></returns>
        public static bool ShowFileViewer(this IPage target, string fileData, int pageRowCount)
        {

            if (pageRowCount < 1)
            {
                target.ShowWarnMessageBox("ShowFileViewer：参数pageRowCount必须大于0！");
                return false;
            }
            var fileViewer = new FileViewer();
            fileViewer.TxtInit(fileData, pageRowCount);

            target.ShowPage(fileViewer, "查看文件");
            return true;
        }

        /// <summary>
        /// 文件查看--fview 函数
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="fileInfo"></param>
        /// <param name="isFileData"></param>
        /// <returns></returns>
        public static bool ShowFileViewer(this TradeAPIsHelper helper, string fileInfo, int pageRowCount)
        {
            return helper.Page.ShowFileViewer(fileInfo, pageRowCount);
        }
        /// <summary>
        /// 身份核查
        /// </summary>
        /// <param name="target"></param>
        /// <param name="idNo"></param>
        /// <param name="idName"></param>
        /// <returns></returns>
        public static IdCheckerResult IdCheck(this TradeAPIsHelper helper, TextBox textBox, string name = null)
        {
            return IdCheck(helper, textBox, name, false);
        }

        private static IdCheckerResult cache_IdCheck_LastPerson;

        private static IdCheckerResult IdCheck(this TradeAPIsHelper helper, TextBox textBox, string name, bool force = false)
        {
            string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            var data = DeviceManager.GetIdentityCardData(textBox);
            //GetIdentityCardData方法内部已经处理了非设备录入情况，故无需判断
            if (data.HasError)
            {
                //throw new ArgumentException("IdCheck:无法获取到对象的身份证对象！");
                data = new DeviceEntity<IdentityCardEntity>
                {
                    HasError = false,
                    Data = new IdentityCardEntity { IdNumber = textBox.Text, Name = name }
                };
            }
            if (!string.IsNullOrEmpty(name) && string.IsNullOrEmpty(data.Data.Name))
                data.Data.Name = name;
            //同一客户连续身份核查，去重
            if (cache_IdCheck_LastPerson != null)
            {
                if (cache_IdCheck_LastPerson.DeviceInfo.IdNumber.Trim() == data.Data.IdNumber.Trim())
                    return cache_IdCheck_LastPerson;
                cache_IdCheck_LastPerson = null;//未匹配，则取消缓存
            }
            var page = new IdCheckViewer(data.Data, !helper.IsSkipFaceComp());
            //强制开启标志
            if (force) page.Record.IsOnline = true;
            ////检查状态，非身份证情况下
            if (DeviceManager.GetIdentityCardType(textBox) != IdentityCardType.IdentityCard)
                page.Record.IsOnline = false;
            //郑州没有联网核查 所以不让弹窗核查 直接返回成功 edit by jun  20150522
            if (page.Record.IsOnline && !string.IsNullOrWhiteSpace(textBox.Text))
            {
                if (helper.Page is PageBase && (helper.Page as PageBase).TradeModel != null)
                {
                    page.tb_reason.Text = (helper.Page as PageBase).TradeModel.Name;
                    string codes = flag != "V5" ? "1000|1002|2003|2000|2351|2004|2014|2020|2200|2202|2203|2205|2216|2217|2300|5801|5834|5030|2998" : "1000|1002|2000|2003|2004|2020|2998|5801|5834|3422|1524|2002|2006|2022|2201|2029|6006|4421|5724|5725|5728|4150|5729|8124";
                    if (flag == "V8")
                    {
                        codes = "1000|1002|2003|2000|2350|2351|2004|2014|2020|2200|2202|2203|2205|2216|2217|2300|5801|5834|5030|2998|2543|3998|3401";
                    }
                    //若菜单已配置，则无需再检查交易号
                    if (helper.GetTradeMetadata("isIdCheckCombo") == "true" || codes.Contains((helper.Page as PageBase).TradeModel.Code))
                    {
                        helper.Page.ShowPage(page);
                        //核查成功，进行缓存登记
                        if (page.Record.IsOnline && page.Record.IsComplated && !page.Record.Reject2RLSB)
                            cache_IdCheck_LastPerson = page.Record;
                    }
                    else
                        page.Record.IsComplated = true;
                }
                else
                {
                    page.Record.IsComplated = true;
                }
                //helper.Page.ShowPage(page);
            }
            else
            {
                page.Record.IsComplated = true;
            }
            return page.Record;
        }

        #region IdCheckCombo
        /// <summary>
        /// 设置输入控件失去焦点时触发弹窗身份核查。name=null时关闭弹窗
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="textBox"></param>
        /// <param name="name"></param>
        public static void SetIdCheckCombo(this TradeAPIsHelper helper, TextBox textBox, string name = null)
        {
            Platform.Presentation.FocusManagers.FocusManager.RemovePreviewLostFocusHandler(textBox, tb_IdCheckCombo);
            if (name != null)
            {
                helper.Page.TradeBus["IdCheckCombo_Name"] = name;
                Platform.Presentation.FocusManagers.FocusManager.AddPreviewLostFocusHandler(textBox, tb_IdCheckCombo);
            }
        }

        private static void tb_IdCheckCombo(object sender, RoutedEventArgs e)
        {
            var tb = sender as TextBox;
            if (tb == null) return;
            var page = tb.FindVisualTreeParent(x => x is PageBase) as PageBase;
            if (page == null) return;
            if (page.ViewStateData == null)//补充对象,防止异常
                page.SetViewStateData(new Dictionary<string, object>());
            ////检查状态，若使用读卡器但非身份证情况下
            //if (DeviceManager.GetIdentityCardReader(tb) != IdentityCardReaderType.Input && DeviceManager.GetIdentityCardType(tb) != IdentityCardType.IdentityCard) return;
            //检查交易配置；增加全局开关，默认开启
            if (ServiceProxy.Ext.ServiceSettings.GetConfig("isIdCheckCombo", true) && page.TradeAPIs.GetTradeMetadata("isIdCheckCombo") == "true")
            {
                var record = page.TradeAPIs.IdCheck(tb, page.TradeBus["IdCheckCombo_Name"], true);
                //未开启联网核查，直接通过
                if (!record.IsOnline) return;
                var fm = Platform.Presentation.FocusManagers.FocusManager.GetManagerInstance(tb);
                if (!record.IsComplated)
                    fm.IsFocusContinue = page.TradeAPIs.ShowQuestionMessageBox("联网身份核查未通过，是否继续？") == MessageBoxResult.Yes;
                else if (record.Reject2RLSB)
                    fm.IsFocusContinue = false;
                return;
            }
        }
        #endregion
        #region DZYY 电子验印
        /// <summary>
        /// 发起验印查询--仅适用于鑫万佳
        /// </summary>
        /// <param name="target"></param>
        /// <param name="YYNo"></param>
        /// <returns></returns>
        [Obsolete("请使用新的验印处理API")]
        public static YYCheckerResult YYCheck(this TradeAPIsHelper helper)
        {
            var page = new CheckSealVerification();

            if (page.Record.IsOnline)
            {
                helper.Page.ShowPage(page, "验印页面");
            }
            else
            {
                page.Record.IsComplated = true;
            }
            return page.Record;
        }

        /// <summary>
        /// 发起验印查询，返回验印结果
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="dzyy"></param>
        /// <returns>true:验印成功；false:验印失败；null:未开启验印检查</returns>
        public static bool? YYCheck(this TradeAPIsHelper helper, DzyyEntity dzyy)
        {
            //目前支持2种验印模式，鑫万佳和中科金财
            var mode = ServiceProxy.Ext.ServiceSettings.GetConfig("DZYY.Mode", "");
            var KinbrNo = ServiceProxy.Ext.ServiceSettings.GetConfig("DZYY.KinbrNo", "");
            dzyy.Message = "检查配置";
            switch (mode)
            {
                case "XWJ"://鑫万佳
                    var page = new CheckSealVerification();
                    if (page.Record.IsOnline)
                    {
                        helper.Page.ShowPage(page, "验印页面");
                        if (!page.Record.IsComplated)
                        {
                            dzyy.Message = "验印未完成！";
                            return false;
                        }
                        //检查验印数据
                        if (!string.IsNullOrEmpty(dzyy.AccNo) && dzyy.AccNo != page.Record.ZH)
                        {
                            dzyy.Message = "当前账号与验印结果：" + page.Record.ZH + "不一致，请确认！";
                            return false;
                        }
                        if (!string.IsNullOrEmpty(dzyy.Amt) && dzyy.Amt != page.Record.JE)
                        {
                            dzyy.Message = "当前金额与验印结果：" + page.Record.JE + "不一致，请确认！";
                            return false;
                        }
                        if (!string.IsNullOrEmpty(dzyy.Voucher) && dzyy.Voucher != page.Record.Type)
                        {
                            dzyy.Message = "当前凭证类型与验印结果：" + page.Record.Type + "不一致，请确认！";
                            return false;
                        }
                        if (!string.IsNullOrEmpty(dzyy.VoucherNo) && dzyy.VoucherNo != page.Record.PZNO)
                        {
                            dzyy.Message = "当前凭证号码与验印结果：" + page.Record.PZNO + "不一致，请确认！";
                            return false;
                        }
                        if (!string.IsNullOrEmpty(dzyy.VoucherDate) && dzyy.VoucherDate != page.Record.CPRQ)
                        {
                            dzyy.Message = "当前出票日期与验印结果：" + page.Record.CPRQ + "不一致，请确认！";
                            return false;
                        }
                        dzyy.SerialNo = page.Record.YYLS;
                        dzyy.Message = "验印完成";
                        //登记验印流水到流水附加表
                        helper.TraderecordsExp("SealNo", dzyy.SerialNo);
                        return true;
                    }
                    else
                        dzyy.Message = "验印检查未开启";
                    break;
                case "ZK"://中科金财
                    if (helper.ShowQuestionMessageBox("该交易是否需要电子验印？") == MessageBoxResult.Yes)
                    {
                        var page_zk = new CheckSealVerification_ZK();
                        if (page_zk.Record.IsOnline && KinbrNo.Contains(LoginUserInfo.KinbrNo.Substring(0, 3)))
                        {
                            page_zk._zh = dzyy.AccNo;
                            page_zk.tb_JE.Text = dzyy.Amt;
                            page_zk.tb_PZTYPE.Text = dzyy.Voucher;
                            page_zk.tb_PZNO.Text = dzyy.VoucherNo;
                            page_zk.tb_CPRQ.Text = dzyy.VoucherDate;
                            helper.Page.ShowPage(page_zk, "验印页面");
                            dzyy.Message = "验证完成：" + page_zk.Record.IsComplated;
                            dzyy.YYFlag = page_zk.Record.YYFlag;
                            if (!page_zk.Record.IsComplated)
                            {
                                dzyy.Message = "验印未通过，请继续进行验印!";
                            }
                            dzyy.SerialNo = page_zk.Record.YYLS;
                            return page_zk.Record.IsComplated;
                        }
                        else
                            dzyy.Message = "验印检查未开启";
                    }
                    else
                    {

                        if (!AuthAction(new Message() { Fd96_W = "fd96w" }, helper))
                        {
                            dzyy.Message = "验印选手工验印，但授权未通过，请重新验印!";
                            return false;
                        }
                        dzyy.YYFlag = "手工验印，授权通过";
                    }
                    break;
                case ""://未配置
                    break;
                default:
                    throw new NotSupportedException("不支持的验印模型：" + mode);
            }
            return null;
        }
        #endregion

        /// <summary>
        /// 身份核查 - 该方法已经否决，请使用TradeAPIs.IdCheck(...)
        /// </summary>
        /// <param name="target"></param>
        /// <param name="idNo"></param>
        /// <param name="idName"></param>
        /// <returns></returns>       
        public static IdCheckRecords IdCheck(this IPage target, IdentityCardEntity info)
        {
            var page = new IdCheckViewer(info);
            if (target is PageBase && (target as PageBase).TradeModel != null)
            {
                page.tb_reason.Text = (target as PageBase).TradeModel.Name;
            }
            target.ShowPage(page);
            return new IdCheckRecords
            {
                Branch = LoginUserInfo.KinbrNo,
                Businessdate = LoginUserInfo.TradeDate,
                Checkcasuse = page.Record.Reason,
                Checkresult = "0" + page.Record.Result,
                Idno = page.Record.IdNumber,
                Marking = page.Record.IsComplated ? "1" : "0",
                Proof = string.Empty,
                Name = page.Record.Name,
                Teller = LoginUserInfo.TellerNo,
                Type = page.Record.Type
            };
        }

        /// <summary>
        /// 调用9986，查询收费
        /// </summary>
        /// <param name="target"></param>
        /// <param name="_msg"></param>
        /// <returns></returns>
        public static bool Call_9986(this IPage target, Message _msg)
        {
            string fd16 = _msg.Fd16;
            string fd9_3 = _msg.Fd9_3; //调用者，设置的默认的收费方式
            //国密改造 新增变量
            string fd122_I = string.Empty;
            target.CommSend(_msg);
            _msg.Fd16 = "9986";
            _msg.Fd12 = fd16;
            target.CallServer(_msg);
            _msg.Fd16 = fd16;
            //国密改造 保存加密密码,防止清空
            if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") == "true")
            {
                fd122_I = _msg.Fd122_I;
            }

            //放到下面来
            if (_msg.Fd122_F == "1")
            {
                _msg.ClearFd(122);
                _msg.Fd122_F = "1";
            }
            else
            {
                _msg.ClearFd(122);
            }
            //国密改造 重新给Fd122_I赋值
            if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") == "true")
            {
                _msg.Fd122_I = fd122_I;
            }
            //决定是否执行收费
            if (_msg.Fd12 == "P047")
            {
                if (!string.IsNullOrWhiteSpace(fd9_3) && string.IsNullOrWhiteSpace(_msg.Fd9_3))
                {
                    _msg.Fd9_3 = fd9_3;
                }
                var fee = new FeeViewer(_msg);
                fee.Is_Wy = false;
                target.ShowPage(fee, "收费界面");
                if (!fee.IsFinishFee)
                {
                    target.ShowMessageBox("收费失败！");
                    return false;
                }
                //若收费成功判断是否存在打印凭条
                if (fee.PrintFee != null && target is ISubmitContract)
                {
                    (target as ISubmitContract).SubmitEntry.MorePrintList.RemoveAll(x => x is PrintsHelper.Print_Fee);
                    (target as ISubmitContract).SubmitEntry.MorePrintList.Add(fee.PrintFee);
                }
                return true;
            }
            else if (_msg.Fd12 != "0000" && _msg.Fd12 != "P047")
            {
                target.ShowMessageBox(target.GetError(_msg));
                return false;
            }
            else if (_msg.Fd12 == "0000")
            {
                if (!string.IsNullOrEmpty(_msg.Fd9_1) && !string.IsNullOrEmpty(_msg.Fd9_2) && !string.IsNullOrEmpty(_msg.Fd9_3))
                {
                    _msg.Fd9_3 = " ";
                }
            }
            return true;
        }
        /// <summary>
        /// 调用9986，查询收费
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="_msg"></param>
        /// <returns></returns>
        public static bool Call_9986(this TradeAPIsHelper helper, Message _msg)
        {
            return helper.Page.Call_9986(_msg);
        }

        /// <summary>
        /// 调用9986_wy，查询收费
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="_msg"></param>
        /// <returns></returns>
        public static bool Call_9986_wy(this IPage target, Message _msg)
        {
            string fd16 = _msg.Fd16;
            string fd9_3 = _msg.Fd9_3; //调用者，设置的默认的收费方式

            //国密改造 新增变量
            string fd122_I = string.Empty;
            target.CommSend(_msg);
            _msg.Fd16 = "9986";
            _msg.Fd12 = fd16;
            target.CallServer(_msg);
            _msg.Fd16 = fd16;
            //国密改造 保存加密密码,防止清空
            if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") == "true")
            {
                fd122_I = _msg.Fd122_I;
            }
            //放到下面来
            if (_msg.Fd122_F == "1")
            {
                _msg.ClearFd(122);
                _msg.Fd122_F = "1";
            }
            else
            {
                _msg.ClearFd(122);
            }
            //国密改造 重新给Fd122_I赋值
            if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") == "true")
            {
                _msg.Fd122_I = fd122_I;
            }
            //决定是否执行收费
            if (_msg.Fd12 == "P047")
            {
                if (!string.IsNullOrWhiteSpace(fd9_3) && string.IsNullOrWhiteSpace(_msg.Fd9_3))
                {
                    _msg.Fd9_3 = fd9_3;
                }
                var fee = new FeeViewer(_msg);
                fee.Is_Wy = true;
                target.ShowPage(fee, "收费界面");
                if (!fee.IsFinishFee)
                {
                    target.ShowMessageBox("收费失败！");
                    return false;
                }
                return true;
            }
            else if (_msg.Fd12 != "0000" && _msg.Fd12 != "P047")
            {
                target.ShowMessageBox(target.GetError(_msg));
                return false;
            }
            else if (_msg.Fd12 == "0000")
            {
                if (!string.IsNullOrEmpty(_msg.Fd9_1) && !string.IsNullOrEmpty(_msg.Fd9_2) && !string.IsNullOrEmpty(_msg.Fd9_3))
                {
                    _msg.Fd9_3 = " ";
                }
            }
            return true;// helper.Page.Call_9986(_msg);
        }

        /// <summary>
        /// 调用9986_wy，查询收费
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="_msg"></param>
        /// <returns></returns>
        public static bool Call_9986_wy(this TradeAPIsHelper helper, Message _msg)
        {
            return helper.Page.Call_9986_wy(_msg);
        }

        ///// <summary>
        ///// 查看图片
        ///// </summary>
        ///// <returns></returns>
        //public static bool ScanImage(this IPage target)
        //{
        //    var reader = new StringReader(target.GetTradeDataItem<string>("CaptureImageList"));
        //    var list = new List<string>();
        //    string line = reader.ReadLine();
        //    while (!string.IsNullOrEmpty(line))
        //    {
        //        list.Add(line);
        //        line = reader.ReadLine();
        //    }
        //    var imgid = new string[list.Count];
        //    for (int i = 0; i < list.Count; i++)
        //    {
        //        imgid[i] = list[i];
        //    }
        //    var showImg = new Imageshow(imgid);
        //    target.ShowPage(showImg, "附件录入");
        //    return true;
        //}


        /// <summary>
        /// 查看XML格式的表格
        /// </summary>
        /// <param name="target"></param>
        /// <param name="sequence">XML数据源</param>
        /// <param name="nodeName">xml目录名</param>
        /// <param name="columnHeaders">列名</param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="header"></param>
        /// <returns></returns>
        public static DataRow ShowDataBoxViewer(this IPage target, string sequence, string nodeName, string[] columnHeaders, double width = 0, double height = 0, string header = "")
        {
            if (string.IsNullOrWhiteSpace(sequence) || string.IsNullOrWhiteSpace(nodeName))
                return null;
            sequence = DataBoxExtension.AnalyzeXMLtoStr(sequence, nodeName);
            var dataBoxeViewer = new DataBoxViewer();
            dataBoxeViewer.LoadData(sequence, columnHeaders);
            dataBoxeViewer.LoadPageSize(width, height);
            target.ShowPage(dataBoxeViewer, header);
            return dataBoxeViewer.Tag as DataRow;
        }
        /// <summary>
        /// 查看XML格式的表格
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="sequence">XML数据源</param>
        /// <param name="nodeName">xml目录名</param>
        /// <param name="columnHeaders">列名</param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="header"></param>
        /// <returns></returns>
        public static DataRow ShowDataBoxViewer(this TradeAPIsHelper helper, string sequence, string nodeName, string[] columnHeaders, double width = 0, double height = 0, string header = "")
        {
            return helper.Page.ShowDataBoxViewer(sequence, nodeName, columnHeaders, width, height, header);
        }
        /// <summary>
        /// 查看表格-数据源是DataTable类型的
        /// </summary>
        /// <param name="target">IPage对象</param>
        /// <param name="dt">DataTable对象</param>
        /// <param name="isShowPageBar">是否显示PageBar</param>
        /// <param name="width">宽（单位：像素）</param>
        /// <param name="height">高（单位：像素）</param>
        /// <param name="header">Header</param>
        /// <returns></returns>
        public static DataRow[] ShowDataBoxViewerReturnMultiRows(this IPage target, DataTable dt, bool isShowPageBar = true, double width = 0, double height = 0, string header = "")
        {
            var dataBoxeViewer = new DataBoxViewer { MultiRows = true };
            dataBoxeViewer.DataBox1.IsShowPageBar = isShowPageBar;
            dataBoxeViewer.LoadData(dt);
            dataBoxeViewer.LoadPageSize(width, height);
            target.ShowPage(dataBoxeViewer, header);
            return dataBoxeViewer.Tag as DataRow[];
        }
        /// <summary>
        /// 弹窗拍照
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="sequence">XML数据源</param>
        /// <param name="nodeName">xml目录名</param>
        /// <param name="columnHeaders">列名</param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="header"></param>
        /// <returns></returns>
        public static DHCPhoto ShowCamPlayer(this TradeAPIsHelper helper, string defaultDevice, bool isAlloChangedDevice = true)
        {
            //var player = new CamPlayer();
            //player.IsAlloChangedDevice = isAlloChangedDevice;
            //player.DefaultDevice = defaultDevice;
            //tar.ShowPage(player, "图片采集弹窗");
            return helper.Page.ShowCamPlayer(defaultDevice, isAlloChangedDevice);
        }
        //public static DHCPhoto ShowCamPlayer(this IPage target, string defaultDevice, bool isAlloChangedDevice = true)
        //{
        //    var player = new CamPlayer(defaultDevice);
        //    //player.IsAlloChangedDevice = isAlloChangedDevice;
        //    player.DefaultDevice = defaultDevice;
        //    target.ShowPage(player, "图片采集弹窗");
        //    player.closeCamera();
        //    return player.Photo;
        //}
        public static DHCPhoto ShowCamPlayer(this IPage target, string defaultDevice, bool isAlloChangedDevice = true)
        {
            var player = new CamPlayer(defaultDevice);
            player.DefaultDevice = defaultDevice;
            target.ShowMessageBox(player, "图片采集弹窗");
            player.closeCamera();
            return player.Photo;
        }

        /// <summary>
        /// 查看表格
        /// </summary>
        /// <param name="target"></param>
        /// <param name="sequence"></param>
        /// <param name="columnHeaders"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="header"></param>
        /// <returns></returns>
        public static DataRow ShowDataBoxViewer(this IPage target, string sequence, string[] columnHeaders, double width = 0, double height = 0, string header = "")
        {
            if (string.IsNullOrWhiteSpace(sequence))
                return null;
            var dataBoxeViewer = new DataBoxViewer();
            dataBoxeViewer.LoadData(sequence, columnHeaders);
            dataBoxeViewer.LoadPageSize(width, height);
            target.ShowPage(dataBoxeViewer, header);
            return dataBoxeViewer.Tag as DataRow;
        }
        /// <summary>
        /// 查看表格
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="sequence"></param>
        /// <param name="columnHeaders"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="header"></param>
        /// <returns></returns>
        public static DataRow ShowDataBoxViewer(this TradeAPIsHelper helper, string sequence, string[] columnHeaders, double width = 0, double height = 0, string header = "")
        {
            return helper.Page.ShowDataBoxViewer(sequence, columnHeaders, width, height, header);
        }
        /// <summary>
        /// 查看表格
        /// </summary>
        /// <param name="target"></param>
        /// <param name="sequence"></param>
        /// <param name="columnHeaders"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="header"></param>
        /// <returns></returns>
        public static DataRow[] ShowDataBoxViewerReturnMultiRows(this IPage target, string sequence, string[] columnHeaders, double width = 0, double height = 0, string header = "")
        {
            if (string.IsNullOrWhiteSpace(sequence))
                return new DataRow[0];
            var dataBoxeViewer = new DataBoxViewer { MultiRows = true };
            dataBoxeViewer.LoadData(sequence, columnHeaders);
            dataBoxeViewer.LoadPageSize(width, height);
            target.ShowPage(dataBoxeViewer, header);
            return dataBoxeViewer.Tag as DataRow[];
        }
        /// <summary>
        /// 查看表格
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="sequence"></param>
        /// <param name="columnHeaders"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="header"></param>
        /// <returns></returns>
        public static DataRow[] ShowDataBoxViewerReturnMultiRows(this TradeAPIsHelper helper, string sequence, string[] columnHeaders, double width = 0, double height = 0, string header = "")
        {
            return helper.Page.ShowDataBoxViewerReturnMultiRows(sequence, columnHeaders, width, height, header);
        }
        /// <summary>
        /// 根据DataTable初始化DataBox弹出框
        /// </summary>
        /// <param name="target"></param>
        /// <param name="tablename"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="header"></param>
        /// <returns></returns>
        public static DataRow ShowDataBoxViewer(this IPage target, DataTable tablename, double width = 0, double height = 0, string header = "")
        {
            var dataBoxeViewer = new DataBoxViewer();
            dataBoxeViewer.LoadData(tablename);
            dataBoxeViewer.LoadPageSize(width, height);
            target.ShowPage(dataBoxeViewer, header);
            return dataBoxeViewer.Tag as DataRow;
        }
        /// <summary>
        /// 根据DataTable初始化DataBox弹出框。注意：该API在升腾终端机上无法正常显示，请弃用
        /// </summary>
        /// <param name="helper"></param>
        /// <param name="tablename"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="header"></param>
        /// <returns></returns>
        public static DataRow ShowDataBoxViewer(this TradeAPIsHelper helper, DataTable tablename, double width = 0, double height = 0, string header = "")
        {
            return ShowDataBoxViewer(helper.Page, tablename, width, height, header);
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="target"></param>
        /// <returns></returns>
        public static List<PictureEntity> ShowPictureCaptureViewer(this IPage target)
        {
            var viewer = new PictureCaptureViewer();
            target.ShowPage(viewer, "影像采集");

            return viewer.Pictures;
        }

        /// <summary>
        /// 授权方法
        /// </summary>
        /// <param name="msg"></param>
        /// <returns></returns>
        private static bool AuthAction(Message msg, TradeAPIsHelper helper)
        {
            // 组织授权检查要素
            TellerSystem.ServiceProxy.Ext.ServiceEntitys.PageData[] page = new TellerSystem.ServiceProxy.Ext.ServiceEntitys.PageData[1];
            page[0] = new TellerSystem.ServiceProxy.Ext.ServiceEntitys.PageData() { index = 1, datas = helper.Page.SaveTradePage((FrameworkElement)helper.Page.Content) };
            AuthConditionElement element = new AuthConditionElement();

            AuthManager c = new AuthManager();
            c.AsksuperResultMessage += new AuthManager.AsksuperResultHandle(AsksuperResultMessage);
            return c.DoAuth(helper.Page, element, msg, page, "0", "", "", "", "电子验印授权", "", "");
        }

        private static void AsksuperResultMessage(AsksuperResultType result, TradeBusData busData)
        {
            switch (result)
            {
                //case AsksuperResultType.Success:
                //    this.ShowMessageBox("授权已经通过，将提交交易。");
                //    break;
                case AsksuperResultType.Failed:
                    MessageBox.Show("授权没有通过。");
                    break;
                //case AsksuperResultType.Undo:
                //    this.ShowMessageBox("授权已撤销。");
                //    break;
                default:
                    break;
            }
        }
        /// <summary>
        /// 二代收费
        /// </summary>
        /// <param name="target"></param>
        /// <param name="_msg"></param>
        /// <returns></returns>
        public static bool Call_9986_CNAPS2(this IPage target, Message _msg)
        {            //国密改造 新增变量           
            string fd122_I = string.Empty;
            string fd16 = _msg.Fd16;
            string fd9_3 = _msg.Fd9_3; //调用者，设置的默认的收费方式
            string fd96_c = _msg.Fd96_C;
            string fd96_7 = _msg.Fd96_7;
            target.CommSend(_msg);
            _msg.Fd16 = "9986";
            _msg.Fd12 = fd16;
            _msg.Fd96_C = fd96_c;
            _msg.Fd96_7 = fd96_7;
            if (_msg.Fd96_C == "LVPS")
            {
                _msg.Fd96_9 = "1400";
            }
            else if (_msg.Fd96_C == "HVPS")
            {
                _msg.Fd96_9 = "1300";
            }
            else if (_msg.Fd96_C == "IBPS" && FLAG == "V0")//中牟超网收费
            {
                _msg.Fd96_9 = "0921";
            }
            target.CallServer(_msg);
            _msg.Fd16 = fd16;
            //国密改造 保存加密密码,防止清空
            if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") == "true")
            {
                fd122_I = _msg.Fd122_I;
            }
            //放到下面来
            if (_msg.Fd122_F == "1")
            {
                _msg.ClearFd(122);
                _msg.Fd122_F = "1";
            }
            else
            {
                _msg.ClearFd(122);
            }
            //国密改造 重新给Fd122_I赋值
            if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") == "true")
            {
                _msg.Fd122_I = fd122_I;
            }
            //决定是否执行收费
            if (_msg.Fd12 == "P047")
            {
                if (!string.IsNullOrWhiteSpace(fd9_3) && string.IsNullOrWhiteSpace(_msg.Fd9_3))
                {
                    _msg.Fd9_3 = fd9_3;
                }
                var fee = new FeeViewer(_msg);
                fee.Is_Wy = false;
                fee.Is_Canps2 = true;
                target.ShowPage(fee, "收费界面");
                if (!fee.IsFinishFee)
                {
                    target.ShowMessageBox("收费失败！");
                    return false;
                }
                //大石桥远程授权交易截图加收费截图：5801、5834
                if ((FLAG == "V11" || FLAG == "V21") && (_msg.Fd16 == "6233" || _msg.Fd16 == "6234"))
                {
                    //target.GetTradeInfoData().TradeScreens.Clear();
                    foreach (var item in target.GetTradeInfoData().TradeScreens)
                    {
                        if (item.Key == "fee")
                        {
                            target.GetTradeInfoData().TradeScreens.Remove("fee");
                        }
                    }
                    target.GetTradeInfoData().TradeScreens.Add("fee", AuthManager.GetPageImages(fee));
                }
                //若收费成功判断是否存在打印凭条
                if (fee.PrintFee != null && fee.PrintFeeGuest != null && target is ISubmitContract)
                {
                    //按照需求删掉柜员凭条 2015年5月22日 
                    //(target as ISubmitContract).SubmitEntry.MorePrintList.RemoveAll(x => x is PrintsHelper.Print_Fee);
                    //(target as ISubmitContract).SubmitEntry.MorePrintList.Add(fee.PrintFee);
                    //按照需求新增客户凭条 2015年5月22日 
                    (target as ISubmitContract).SubmitEntry.MorePrintList.RemoveAll(x => x is PrintsHelper.Print_FeeGuest);
                    (target as ISubmitContract).SubmitEntry.MorePrintList.Add(fee.PrintFeeGuest);
                }

                return true;
            }
            else if (_msg.Fd12 != "0000" && _msg.Fd12 != "P047")
            {
                target.ShowMessageBox(target.GetError(_msg));
                return false;
            }
            else if (_msg.Fd12 == "0000")
            {
                if (!string.IsNullOrEmpty(_msg.Fd9_1) && !string.IsNullOrEmpty(_msg.Fd9_2) && !string.IsNullOrEmpty(_msg.Fd9_3))
                {
                    _msg.Fd9_3 = " ";
                }
            }
            return true;
        }

        #region 手机三要素校验
        /// <summary>
        /// 手机三要素校验
        /// </summary>
        /// <param name="target"></param>
        /// <param name="idNo"></param>
        /// <param name="idName"></param>
        /// <returns></returns>
        public static bool? MobilePhoneCheck(this TradeAPIsHelper helper, string idno, string name, string phone)
        {
            if (string.IsNullOrEmpty(idno) || string.IsNullOrEmpty(name) || string.IsNullOrEmpty(phone)) return null;
            var page = new MobilePhoneCheckViewer(idno, name, phone);
            helper.Page.ShowPage(page, "手机号三要素查验");
            return page.Result;
        }
        #endregion
    }
}