import { IdCheckerResult, TradeAPIsHelper, IPage, IdentityCardEntity, IdCheckRecords, Message, DzyyEntity, DataRow, DataTable, PictureEntity, AuthConditionElement, PageData, AsksuperResultType, TradeBusData, DHCPhoto } from './types'; // 假设这些类型定义在 types.ts 文件中
import { DeviceManager, TradeManagerHandle, LoginUserInfo, ServiceProxy, AuthManager, DataBoxExtension } from './services'; // 假设这些服务在 services.ts 文件中
import { FileViewer, IdCheckViewer, CheckSealVerification, CheckSealVerification_ZK, FeeViewer, DataBoxViewer, CamPlayer, PictureCaptureViewer, MobilePhoneCheckViewer } from './components'; // 假设这些组件在 components.ts 文件中

// 假设这是从配置中获取的常量
let FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");

// 文件查看——fview函数
export function ShowFileViewer(target: IPage, fileInfo: string, isFileData: boolean = true): boolean {
    const fileViewer = new FileViewer();
    if (isFileData) {
        fileViewer.TxtInit(fileInfo);
    } else {
        fileViewer.Init(fileInfo, false);
    }
    target.ShowPage(fileViewer, "查看文件");
    return true;
}

// 文件查看--fview 函数
export function ShowFileViewerHelper(helper: TradeAPIsHelper, fileInfo: string, isFileData: boolean = true): boolean {
    return helper.Page.ShowFileViewer(fileInfo, isFileData);
}

// 文件查看——fview函数
export function ShowFileViewerWithPageRowCount(target: IPage, fileData: string, pageRowCount: number): boolean {
    if (pageRowCount < 1) {
        target.ShowWarnMessageBox("ShowFileViewer：参数pageRowCount必须大于0！");
        return false;
    }
    const fileViewer = new FileViewer();
    fileViewer.TxtInit(fileData, pageRowCount);
    target.ShowPage(fileViewer, "查看文件");
    return true;
}

// 文件查看--fview 函数
export function ShowFileViewerHelperWithPageRowCount(helper: TradeAPIsHelper, fileInfo: string, pageRowCount: number): boolean {
    return helper.Page.ShowFileViewer(fileInfo, pageRowCount);
}

// 身份核查
export function IdCheck(helper: TradeAPIsHelper, textBox: HTMLInputElement, name: string | null = null): IdCheckerResult {
    return IdCheckInternal(helper, textBox, name, false);
}

let cache_IdCheck_LastPerson: IdCheckerResult | null = null;

function IdCheckInternal(helper: TradeAPIsHelper, textBox: HTMLInputElement, name: string | null, force: boolean = false): IdCheckerResult {
    const flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
    const data = DeviceManager.GetIdentityCardData(textBox);
    if (data.HasError) {
        data = {
            HasError: false,
            Data: { IdNumber: textBox.value, Name: name }
        };
    }
    if (name &&!data.Data.Name) {
        data.Data.Name = name;
    }
    if (cache_IdCheck_LastPerson) {
        if (cache_IdCheck_LastPerson.DeviceInfo.IdNumber.trim() === data.Data.IdNumber.trim()) {
            return cache_IdCheck_LastPerson;
        }
        cache_IdCheck_LastPerson = null;
    }
    const page = new IdCheckViewer(data.Data,!helper.IsSkipFaceComp());
    if (force) {
        page.Record.IsOnline = true;
    }
    if (DeviceManager.GetIdentityCardType(textBox)!== 'IdentityCard') {
        page.Record.IsOnline = false;
    }
    if (page.Record.IsOnline && textBox.value) {
        if (helper.Page instanceof PageBase && helper.Page.TradeModel) {
            page.tb_reason.textContent = helper.Page.TradeModel.Name;
            let codes = flag!== "V5"? "1000|1002|2003|2000|2351|2004|2014|2020|2200|2202|2203|2205|2216|2217|2300|5801|5834|5030|2998" : "1000|1002|2000|2003|2004|2020|2998|5801|5834|3422|1524|2002|2006|2022|2201|2029|6006|4421|5724|5725|5728|4150|5729|8124";
            if (flag === "V8") {
                codes = "1000|1002|2003|2000|2350|2351|2004|2014|2020|2200|2202|2203|2205|2216|2217|2300|5801|5834|5030|2998|2543|3998|3401";
            }
            if (helper.GetTradeMetadata("isIdCheckCombo") === "true" || codes.includes(helper.Page.TradeModel.Code)) {
                helper.Page.ShowPage(page);
                if (page.Record.IsOnline && page.Record.IsComplated &&!page.Record.Reject2RLSB) {
                    cache_IdCheck_LastPerson = page.Record;
                }
            } else {
                page.Record.IsComplated = true;
            }
        } else {
            page.Record.IsComplated = true;
        }
    } else {
        page.Record.IsComplated = true;
    }
    return page.Record;
}

// 设置输入控件失去焦点时触发弹窗身份核查。name=null时关闭弹窗
export function SetIdCheckCombo(helper: TradeAPIsHelper, textBox: HTMLInputElement, name: string | null = null): void {
    const focusManager = Platform.Presentation.FocusManagers.FocusManager;
    focusManager.RemovePreviewLostFocusHandler(textBox, tb_IdCheckCombo);
    if (name) {
        helper.Page.TradeBus["IdCheckCombo_Name"] = name;
        focusManager.AddPreviewLostFocusHandler(textBox, tb_IdCheckCombo);
    }
}

function tb_IdCheckCombo(sender: EventTarget | null, e: Event): void {
    const tb = sender as HTMLInputElement;
    if (!tb) return;
    const page = tb.closest('.page-base') as PageBase;
    if (!page) return;
    if (!page.ViewStateData) {
        page.SetViewStateData(new Map<string, any>());
    }
    if (ServiceProxy.Ext.ServiceSettings.GetConfig("isIdCheckCombo", true) && page.TradeAPIs.GetTradeMetadata("isIdCheckCombo") === "true") {
        const record = page.TradeAPIs.IdCheck(tb, page.TradeBus["IdCheckCombo_Name"], true);
        if (!record.IsOnline) return;
        const fm = Platform.Presentation.FocusManagers.FocusManager.GetManagerInstance(tb);
        if (!record.IsComplated) {
            fm.IsFocusContinue = page.TradeAPIs.ShowQuestionMessageBox("联网身份核查未通过，是否继续？") === "Yes";
        } else if (record.Reject2RLSB) {
            fm.IsFocusContinue = false;
        }
        return;
    }
}

// 发起验印查询--仅适用于鑫万佳
export function YYCheckOld(helper: TradeAPIsHelper): any {
    const page = new CheckSealVerification();
    if (page.Record.IsOnline) {
        helper.Page.ShowPage(page, "验印页面");
    } else {
        page.Record.IsComplated = true;
    }
    return page.Record;
}

// 发起验印查询，返回验印结果
export function YYCheck(helper: TradeAPIsHelper, dzyy: DzyyEntity): boolean | null {
    const mode = ServiceProxy.Ext.ServiceSettings.GetConfig("DZYY.Mode", "");
    const KinbrNo = ServiceProxy.Ext.ServiceSettings.GetConfig("DZYY.KinbrNo", "");
    dzyy.Message = "检查配置";
    switch (mode) {
        case "XWJ":
            const page = new CheckSealVerification();
            if (page.Record.IsOnline) {
                helper.Page.ShowPage(page, "验印页面");
                if (!page.Record.IsComplated) {
                    dzyy.Message = "验印未完成！";
                    return false;
                }
                if (dzyy.AccNo && dzyy.AccNo!== page.Record.ZH) {
                    dzyy.Message = `当前账号与验印结果：${page.Record.ZH}不一致，请确认！`;
                    return false;
                }
                if (dzyy.Amt && dzyy.Amt!== page.Record.JE) {
                    dzyy.Message = `当前金额与验印结果：${page.Record.JE}不一致，请确认！`;
                    return false;
                }
                if (dzyy.Voucher && dzyy.Voucher!== page.Record.Type) {
                    dzyy.Message = `当前凭证类型与验印结果：${page.Record.Type}不一致，请确认！`;
                    return false;
                }
                if (dzyy.VoucherNo && dzyy.VoucherNo!== page.Record.PZNO) {
                    dzyy.Message = `当前凭证号码与验印结果：${page.Record.PZNO}不一致，请确认！`;
                    return false;
                }
                if (dzyy.VoucherDate && dzyy.VoucherDate!== page.Record.CPRQ) {
                    dzyy.Message = `当前出票日期与验印结果：${page.Record.CPRQ}不一致，请确认！`;
                    return false;
                }
                dzyy.SerialNo = page.Record.YYLS;
                dzyy.Message = "验印完成";
                helper.TraderecordsExp("SealNo", dzyy.SerialNo);
                return true;
            } else {
                dzyy.Message = "验印检查未开启";
            }
            break;
        case "ZK":
            if (helper.ShowQuestionMessageBox("该交易是否需要电子验印？") === "Yes") {
                const page_zk = new CheckSealVerification_ZK();
                if (page_zk.Record.IsOnline && KinbrNo.includes(LoginUserInfo.KinbrNo.substring(0, 3))) {
                    page_zk._zh = dzyy.AccNo;
                    page_zk.tb_JE.value = dzyy.Amt;
                    page_zk.tb_PZTYPE.value = dzyy.Voucher;
                    page_zk.tb_PZNO.value = dzyy.VoucherNo;
                    page_zk.tb_CPRQ.value = dzyy.VoucherDate;
                    helper.Page.ShowPage(page_zk, "验印页面");
                    dzyy.Message = `验证完成：${page_zk.Record.IsComplated}`;
                    dzyy.YYFlag = page_zk.Record.YYFlag;
                    if (!page_zk.Record.IsComplated) {
                        dzyy.Message = "验印未通过，请继续进行验印!";
                    }
                    dzyy.SerialNo = page_zk.Record.YYLS;
                    return page_zk.Record.IsComplated;
                } else {
                    dzyy.Message = "验印检查未开启";
                }
            } else {
                if (!AuthAction(new Message({ Fd96_W: "fd96w" }), helper)) {
                    dzyy.Message = "验印选手工验印，但授权未通过，请重新验印!";
                    return false;
                }
                dzyy.YYFlag = "手工验印，授权通过";
            }
            break;
        case "":
            break;
        default:
            throw new Error(`不支持的验印模型：${mode}`);
    }
    return null;
}

// 身份核查 - 该方法已经否决，请使用TradeAPIs.IdCheck(...)
export function IdCheckOld(target: IPage, info: IdentityCardEntity): IdCheckRecords {
    const page = new IdCheckViewer(info);
    if (target instanceof PageBase && target.TradeModel) {
        page.tb_reason.textContent = target.TradeModel.Name;
    }
    target.ShowPage(page);
    return {
        Branch: LoginUserInfo.KinbrNo,
        Businessdate: LoginUserInfo.TradeDate,
        Checkcasuse: page.Record.Reason,
        Checkresult: `0${page.Record.Result}`,
        Idno: page.Record.IdNumber,
        Marking: page.Record.IsComplated? "1" : "0",
        Proof: "",
        Name: page.Record.Name,
        Teller: LoginUserInfo.TellerNo,
        Type: page.Record.Type
    };
}

// 调用9986，查询收费
export function Call_9986(target: IPage, _msg: Message): boolean {
    const fd16 = _msg.Fd16;
    const fd9_3 = _msg.Fd9_3;
    let fd122_I = "";
    target.CommSend(_msg);
    _msg.Fd16 = "9986";
    _msg.Fd12 = fd16;
    target.CallServer(_msg);
    _msg.Fd16 = fd16;
    if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") === "true") {
        fd122_I = _msg.Fd122_I;
    }
    if (_msg.Fd122_F === "1") {
        _msg.ClearFd(122);
        _msg.Fd122_F = "1";
    } else {
        _msg.ClearFd(122);
    }
    if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") === "true") {
        _msg.Fd122_I = fd122_I;
    }
    if (_msg.Fd12 === "P047") {
        if (fd9_3 &&!_msg.Fd9_3) {
            _msg.Fd9_3 = fd9_3;
        }
        const fee = new FeeViewer(_msg);
        fee.Is_Wy = false;
        target.ShowPage(fee, "收费界面");
        if (!fee.IsFinishFee) {
            target.ShowMessageBox("收费失败！");
            return false;
        }
        if (fee.PrintFee && target instanceof ISubmitContract) {
            const submitEntry = target.SubmitEntry;
            submitEntry.MorePrintList = submitEntry.MorePrintList.filter(x =>!(x instanceof PrintsHelper.Print_Fee));
            submitEntry.MorePrintList.push(fee.PrintFee);
        }
        return true;
    } else if (_msg.Fd12!== "0000" && _msg.Fd12!== "P047") {
        target.ShowMessageBox(target.GetError(_msg));
        return false;
    } else if (_msg.Fd12 === "0000") {
        if (_msg.Fd9_1 && _msg.Fd9_2 && _msg.Fd9_3) {
            _msg.Fd9_3 = " ";
        }
    }
    return true;
}

// 调用9986，查询收费
export function Call_9986Helper(helper: TradeAPIsHelper, _msg: Message): boolean {
    return helper.Page.Call_9986(_msg);
}

// 调用9986_wy，查询收费
export function Call_9986_wy(target: IPage, _msg: Message): boolean {
    const fd16 = _msg.Fd16;
    const fd9_3 = _msg.Fd9_3;
    let fd122_I = "";
    target.CommSend(_msg);
    _msg.Fd16 = "9986";
    _msg.Fd12 = fd16;
    target.CallServer(_msg);
    _msg.Fd16 = fd16;
    if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") === "true") {
        fd122_I = _msg.Fd122_I;
    }
    if (_msg.Fd122_F === "1") {
        _msg.ClearFd(122);
        _msg.Fd122_F = "1";
    } else {
        _msg.ClearFd(122);
    }
    if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") === "true") {
        _msg.Fd122_I = fd122_I;
    }
    if (_msg.Fd12 === "P047") {
        if (fd9_3 &&!_msg.Fd9_3) {
            _msg.Fd9_3 = fd9_3;
        }
        const fee = new FeeViewer(_msg);
        fee.Is_Wy = true;
        target.ShowPage(fee, "收费界面");
        if (!fee.IsFinishFee) {
            target.ShowMessageBox("收费失败！");
            return false;
        }
        return true;
    } else if (_msg.Fd12!== "0000" && _msg.Fd12!== "P047") {
        target.ShowMessageBox(target.GetError(_msg));
        return false;
    } else if (_msg.Fd12 === "0000") {
        if (_msg.Fd9_1 && _msg.Fd9_2 && _msg.Fd9_3) {
            _msg.Fd9_3 = " ";
        }
    }
    return true;
}

// 调用9986_wy，查询收费
export function Call_9986_wyHelper(helper: TradeAPIsHelper, _msg: Message): boolean {
    return helper.Page.Call_9986_wy(_msg);
}

// 查看XML格式的表格
export function ShowDataBoxViewer(target: IPage, sequence: string, nodeName: string, columnHeaders: string[], width: number = 0, height: number = 0, header: string = ""): DataRow | null {
    if (!sequence ||!nodeName) return null;
    sequence = DataBoxExtension.AnalyzeXMLtoStr(sequence, nodeName);
    const dataBoxeViewer = new DataBoxViewer();
    dataBoxeViewer.LoadData(sequence, columnHeaders);
    dataBoxeViewer.LoadPageSize(width, height);
    target.ShowPage(dataBoxeViewer, header);
    return dataBoxeViewer.Tag as DataRow;
}

// 查看XML格式的表格
export function ShowDataBoxViewerHelper(helper: TradeAPIsHelper, sequence: string, nodeName: string, columnHeaders: string[], width: number = 0, height: number = 0, header: string = ""): DataRow | null {
    return helper.Page.ShowDataBoxViewer(sequence, nodeName, columnHeaders, width, height, header);
}

// 查看表格-数据源是DataTable类型的
export function ShowDataBoxViewerReturnMultiRows(target: IPage, dt: DataTable, isShowPageBar: boolean = true, width: number = 0, height: number = 0, header: string = ""): DataRow[] | null {
    const dataBoxeViewer = new DataBoxViewer();
    dataBoxeViewer.MultiRows = true;
    dataBoxeViewer.DataBox1.IsShowPageBar = isShowPageBar;
    dataBoxeViewer.LoadData(dt);
    dataBoxeViewer.LoadPageSize(width, height);
    target.ShowPage(dataBoxeViewer, header);
    return dataBoxeViewer.Tag as DataRow[];
}

// 弹窗拍照
export function ShowCamPlayer(helper: TradeAPIsHelper, defaultDevice: string, isAlloChangedDevice: boolean = true): DHCPhoto {
    return helper.Page.ShowCamPlayer(defaultDevice, isAlloChangedDevice);
}

// 查看表格
export function ShowDataBoxViewerSimple(target: IPage, sequence: string, columnHeaders: string[], width: number = 0, height: number = 0, header: string = ""): DataRow | null {
    if (!sequence) return null;
    const dataBoxeViewer = new DataBoxViewer();
    dataBoxeViewer.LoadData(sequence, columnHeaders);
    dataBoxeViewer.LoadPageSize(width, height);
    target.ShowPage(dataBoxeViewer, header);
    return dataBoxeViewer.Tag as DataRow;
}

// 查看表格
export function ShowDataBoxViewerSimpleHelper(helper: TradeAPIsHelper, sequence: string, columnHeaders: string[], width: number = 0, height: number = 0, header: string = ""): DataRow | null {
    return helper.Page.ShowDataBoxViewerSimple(sequence, columnHeaders, width, height, header);
}

// 查看表格
export function ShowDataBoxViewerReturnMultiRowsSimple(target: IPage, sequence: string, columnHeaders: string[], width: number = 0, height: number = 0, header: string = ""): DataRow[] | null {
    if (!sequence) return [];
    const dataBoxeViewer = new DataBoxViewer();
    dataBoxeViewer.MultiRows = true;
    dataBoxeViewer.LoadData(sequence, columnHeaders);
    dataBoxeViewer.LoadPageSize(width, height);
    target.ShowPage(dataBoxeViewer, header);
    return dataBoxeViewer.Tag as DataRow[];
}

// 查看表格
export function ShowDataBoxViewerReturnMultiRowsSimpleHelper(helper: TradeAPIsHelper, sequence: string, columnHeaders: string[], width: number = 0, height: number = 0, header: string = ""): DataRow[] | null {
    return helper.Page.ShowDataBoxViewerReturnMultiRowsSimple(sequence, columnHeaders, width, height, header);
}

// 根据DataTable初始化DataBox弹出框
export function ShowDataBoxViewerFromDataTable(target: IPage, tablename: DataTable, width: number = 0, height: number = 0, header: string = ""): DataRow | null {
    const dataBoxeViewer = new DataBoxViewer();
    dataBoxeViewer.LoadData(tablename);
    dataBoxeViewer.LoadPageSize(width, height);
    target.ShowPage(dataBoxeViewer, header);
    return dataBoxeViewer.Tag as DataRow;
}

// 根据DataTable初始化DataBox弹出框。注意：该API在升腾终端机上无法正常显示，请弃用
export function ShowDataBoxViewerFromDataTableHelper(helper: TradeAPIsHelper, tablename: DataTable, width: number = 0, height: number = 0, header: string = ""): DataRow | null {
    return ShowDataBoxViewerFromDataTable(helper.Page, tablename, width, height, header);
}

// 查看影像采集
export function ShowPictureCaptureViewer(target: IPage): PictureEntity[] {
    const viewer = new PictureCaptureViewer();
    target.ShowPage(viewer, "影像采集");
    return viewer.Pictures;
}

// 授权方法
function AuthAction(msg: Message, helper: TradeAPIsHelper): boolean {
    const page: PageData[] = [
        { index: 1, datas: helper.Page.SaveTradePage(helper.Page.Content as HTMLElement) }
    ];
    const element = new AuthConditionElement();
    const c = new AuthManager();
    c.AsksuperResultMessage = (result: AsksuperResultType, busData: TradeBusData) => {
        AsksuperResultMessage(result, busData);
    };
    return c.DoAuth(helper.Page, element, msg, page, "0", "", "", "", "电子验印授权", "", "");
}

function AsksuperResultMessage(result: AsksuperResultType, busData: TradeBusData): void {
    switch (result) {
        case "Failed":
            alert("授权没有通过。");
            break;
        default:
            break;
    }
}

// 二代收费
export function Call_9986_CNAPS2(target: IPage, _msg: Message): boolean {
    let fd122_I = "";
    const fd16 = _msg.Fd16;
    const fd9_3 = _msg.Fd9_3;
    const fd96_c = _msg.Fd96_C;
    const fd96_7 = _msg.Fd96_7;
    target.CommSend(_msg);
    _msg.Fd16 = "9986";
    _msg.Fd12 = fd16;
    _msg.Fd96_C = fd96_c;
    _msg.Fd96_7 = fd96_7;
    if (_msg.Fd96_C === "LVPS") {
        _msg.Fd96_9 = "1400";
    } else if (_msg.Fd96_C === "HVPS") {
        _msg.Fd96_9 = "1300";
    } else if (_msg.Fd96_C === "IBPS" && FLAG === "V0") {
        _msg.Fd96_9 = "0921";
    }
    target.CallServer(_msg);
    _msg.Fd16 = fd16;
    if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") === "true") {
        fd122_I = _msg.Fd122_I;
    }
    if (_msg.Fd122_F === "1") {
        _msg.ClearFd(122);
        _msg.Fd122_F = "1";
    } else {
        _msg.ClearFd(122);
    }
    if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") === "true") {
        _msg.Fd122_I = fd122_I;
    }
    if (_msg.Fd12 === "P047") {
        if (fd9_3 &&!_msg.Fd9_3) {
            _msg.Fd9_3 = fd9_3;
        }
        const fee = new FeeViewer(_msg);
        fee.Is_Wy = false;
        fee.Is_Canps2 = true;
        target.ShowPage(fee, "收费界面");
        if (!fee.IsFinishFee) {
            target.ShowMessageBox("收费失败！");
            return false;
        }
        if ((FLAG === "V11" || FLAG === "V21") && (_msg.Fd16 === "6233" || _msg.Fd16 === "6234")) {
            const tradeInfoData = target.GetTradeInfoData();
            tradeInfoData.TradeScreens = tradeInfoData.TradeScreens.filter(item => item.Key!== "fee");
            tradeInfoData.TradeScreens.push({ Key: "fee", Value: AuthManager.GetPageImages(fee) });
        }
        if (fee.PrintFee && fee.PrintFeeGuest && target instanceof ISubmitContract) {
            const submitEntry = target.SubmitEntry;
            submitEntry.MorePrintList = submitEntry.MorePrintList.filter(x =>!(x instanceof PrintsHelper.Print_FeeGuest));
            submitEntry.MorePrintList.push(fee.PrintFeeGuest);
        }
        return true;
    } else if (_msg.Fd12!== "0000" && _msg.Fd12!== "P047") {
        target.ShowMessageBox(target.GetError(_msg));
        return false;
    } else if (_msg.Fd12 === "0000") {
        if (_msg.Fd9_1 && _msg.Fd9_2 && _msg.Fd9_3) {
            _msg.Fd9_3 = " ";
        }
    }
    return true;
}

// 手机三要素校验
export function MobilePhoneCheck(helper: TradeAPIsHelper, idno: string, name: string, phone: string): boolean | null {
    if (!idno ||!name ||!phone) return null;
    const page = new MobilePhoneCheckViewer(idno, name, phone);
    helper.Page.ShowPage(page, "手机号三要素查验");
    return page.Result;
}