﻿using System.Collections.Generic;
using Platform.Print;
using TellerSystem.PrintTemplate.Bankbook;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    public class CXCZBankbook
    {
        private static CXCZBankbook _instance;

        public static CXCZBankbook GetInstance()
        {
            if (_instance == null)
            {
                _instance = new CXCZBankbook();
            }
            return _instance;
        }

        /// <summary>
        /// 单页行数
        /// </summary>
        private const int PrintPageNum = 12;

        private int _sumNum = 96;
        /// <summary>
        /// 折最大行
        /// </summary>
        public int SumNum
        {
            get { return _sumNum; }
            set { _sumNum = value; }
        }

        /// <summary>
        /// 储蓄存折首页数据
        /// </summary>
        public CXCZFirstPageEntity FirstPageData { get; set; }

        /// <summary>
        /// 储蓄存折数据页数据
        /// </summary>
        public List<CXCZPageEntity> PageData { get; set; }

        /// <summary>
        /// 储蓄存折尾页数据
        /// </summary>
        public CXCZEndPageEntity EndPageData { get; set; }

        /// <summary>
        /// 打印存折页，替代PrintPage方法
        /// </summary>
        public List<PrintContainer> PrintBankBookPage()
        {
            var retData = new List<PrintContainer>();
            List<CXCZPageEntity> tempDatas = new List<CXCZPageEntity>();
            int pageTatolLine = PrintPageNum * 2;
            //本次要打印的总行数
            int account = 0;
            //当前页要打印的行数，从1开始
            int currentPrintLine = 0;
            if (PageData.Count == 0)
            {
                return retData;
            }
            if (PageData != null)
            {
                account = PageData.Count;
                //开户
                if (account == 1 && PageData[0].Line == 1)
                {
                    var entry = new PrintBankBook_活期首页.PrintBankBook_活期首页Entry
                    {
                        AccountNo = FirstPageData.AcNo,
                        AccountName = FirstPageData.AcName,
                        Currency = FirstPageData.CurNo,
                        //AccountBank = FirstPageData.OpnBrname,
                        //AccountDate = FirstPageData.OpnDate,
                        Type = FirstPageData.Type,
                        DepositType = FirstPageData.DepositType,
                        PasswordOrSeal = FirstPageData.DrawMode,
                        PaySysCode = FirstPageData.PaySysCode,
                        Species = FirstPageData.Species,
                        SignDate = FirstPageData.OpnDate,
                        Interestrate = FirstPageData.Interestrate,
                        OpenBrno = FirstPageData.OpnBrname,
                        NoteNo = FirstPageData.CZNum
                    };
                    PrintBankBook_活期首页 first = new PrintBankBook_活期首页(entry);
                    first.Description = "请打印第1页";
                    first.PageData = PageData;
                    first.Setvalue();
                    retData.Clear();
                    retData.Add(first);
                }

                //用于更换介质打扉页并打出多条记录
                else if (FirstPageData != null && account > 0)
                {
                    var entry = new PrintBankBook_活期首页.PrintBankBook_活期首页Entry
                    {
                        AccountNo = FirstPageData.AcNo,
                        AccountName = FirstPageData.AcName,
                        Currency = FirstPageData.CurNo,
                        //AccountBank = FirstPageData.OpnBrname,
                        //AccountDate = FirstPageData.OpnDate,
                        Type = FirstPageData.Type,
                        DepositType = FirstPageData.DepositType,
                        PasswordOrSeal = FirstPageData.DrawMode,
                        PaySysCode = FirstPageData.PaySysCode,
                        Species = FirstPageData.Species,
                        SignDate = FirstPageData.OpnDate,
                        Interestrate = FirstPageData.Interestrate,
                        OpenBrno = FirstPageData.OpnBrname,
                        NoteNo = FirstPageData.CZNum
                    };
                    PrintBankBook_活期首页 first = new PrintBankBook_活期首页(entry);
                    first.Description = "请打印第1页";
                    int tmp = 0;
                    int extra = 0;
                    tmp = PageData[0].Line % pageTatolLine;
                    currentPrintLine = tmp == 0 ? 1 : tmp;
                    //首页换页
                    if (PrintPageNum - currentPrintLine + 1 < account)
                    {
                        tempDatas.Clear();
                        for (int i = 1; i < currentPrintLine; i++)
                        {
                            tempDatas.Add(new CXCZPageEntity());
                        }
                        tempDatas.AddRange(PageData.GetRange(0, PrintPageNum - currentPrintLine + 1));
                        first.PageData = tempDatas;
                        first.Setvalue();
                        retData.Clear();
                        retData.Add(first);
                        extra = account - (PrintPageNum - currentPrintLine + 1);
                        tmp = (PageData[account - extra].Line - PrintPageNum) % pageTatolLine;
                        currentPrintLine = tmp == 0 ? 1 : tmp;
                        while (extra > 0)
                        {
                            //当前已经打印到尾页
                            if (PageData[account - extra].Line > (SumNum - PrintPageNum))
                            {
                                PrintBankBook_活期尾页 last = new PrintBankBook_活期尾页(new PrintBankBook_活期尾页.PrintBankBook_活期尾页Entry());
                                tempDatas.Clear();
                                if (extra > PrintPageNum)
                                {
                                    for (int i = 1; i < currentPrintLine; i++)
                                    {
                                        tempDatas.Add(new CXCZPageEntity());
                                    }
                                    tempDatas.AddRange(PageData.GetRange(account - extra, PrintPageNum));
                                    extra = 0;
                                    //Todo提示换证  
                                    last.Description = "请换折";
                                }
                                else
                                {
                                    for (int i = 1; i < currentPrintLine; i++)
                                    {
                                        tempDatas.Add(new CXCZPageEntity());
                                    }
                                    tempDatas.AddRange(PageData.GetRange(account - extra, extra));
                                    extra = 0;
                                }
                                last.PageData3 = tempDatas;
                                last.SetValue();
                                retData.Add(last);
                            }
                            //打印正常页面
                            else
                            {
                                PrintBankBook_活期页面 page = new PrintBankBook_活期页面(new PrintBankBook_活期页面.PrintBankBook_活期页面Entry());
                                tempDatas.Clear();
                                if (extra >= pageTatolLine)
                                {
                                    for (int i = 1; i < currentPrintLine; i++)
                                    {
                                        tempDatas.Add(new CXCZPageEntity());
                                    }
                                    tempDatas.AddRange(PageData.GetRange(account - extra, pageTatolLine));
                                    extra = extra - (pageTatolLine - currentPrintLine + 1);
                                    if (extra > 0)
                                    {
                                        tmp = PageData[account - extra].Line % PrintPageNum;
                                        currentPrintLine = tmp == 0 ? 1 : tmp;
                                    }
                                }
                                else
                                {
                                    for (int i = 1; i < currentPrintLine; i++)
                                    {
                                        tempDatas.Add(new CXCZPageEntity());
                                    }
                                    tempDatas.AddRange(PageData.GetRange(account - extra, extra));
                                    extra = 0;
                                }
                                page.PageData3 = tempDatas;
                                page.SetValue();
                                retData.Add(page);
                            }
                        }
                    }
                    //首页不换页
                    else if (PrintPageNum - currentPrintLine + 1 >= account)
                    {
                        tempDatas.Clear();
                        for (int i = 1; i < currentPrintLine; i++)
                        {
                            tempDatas.Add(new CXCZPageEntity());
                        }
                        tempDatas.AddRange(PageData.GetRange(0, account));
                        first.PageData = tempDatas;
                        first.Setvalue();
                        retData.Clear();
                        retData.Add(first);
                    }

                }

                //正常打印数据页
                else
                {
                    int tmp = 0;
                    int extra = 0; 
                    //带首页
                    if (PageData[0].Line <= PrintPageNum)
                    {
                        tmp = PageData[0].Line % pageTatolLine;
                        currentPrintLine = tmp == 0 ? 1 : tmp;
                        //首页换页
                        if (PrintPageNum - currentPrintLine + 1 < account)
                        {
                            PrintBankBook_活期首页 first = new PrintBankBook_活期首页(new PrintBankBook_活期首页.PrintBankBook_活期首页Entry());
                            first.Description = "请打印第1页";
                            tempDatas.Clear();
                            for (int i = 1; i < currentPrintLine; i++)
                            {
                                tempDatas.Add(new CXCZPageEntity());
                            }
                            tempDatas.AddRange(PageData.GetRange(0, PrintPageNum - currentPrintLine + 1));
                            first.PageData = tempDatas;
                            first.Setvalue();
                            retData.Clear();
                            retData.Add(first);
                            extra = account - (PrintPageNum - currentPrintLine + 1);
                            tmp = (PageData[account - extra].Line - PrintPageNum) % pageTatolLine;
                            currentPrintLine = tmp == 0 ? 1 : tmp;
                            while (extra > 0)
                            {
                                //当前已经打印到尾页
                                if (PageData[account - extra].Line > (SumNum - PrintPageNum))
                                {
                                    PrintBankBook_活期尾页 last = new PrintBankBook_活期尾页(new PrintBankBook_活期尾页.PrintBankBook_活期尾页Entry());
                                    tempDatas.Clear();
                                    if (extra > PrintPageNum)
                                    {
                                        for (int i = 1; i < currentPrintLine; i++)
                                        {
                                            tempDatas.Add(new CXCZPageEntity());
                                        }
                                        tempDatas.AddRange(PageData.GetRange(account - extra, PrintPageNum));
                                        extra = 0;
                                        //Todo提示换证  
                                        last.Description = "请换折";
                                    }
                                    else
                                    {
                                        for (int i = 1; i < currentPrintLine; i++)
                                        {
                                            tempDatas.Add(new CXCZPageEntity());
                                        }
                                        tempDatas.AddRange(PageData.GetRange(account - extra, extra));
                                        extra = 0;
                                    }
                                    last.PageData3 = tempDatas;
                                    last.SetValue();
                                    retData.Add(last);
                                }
                                //打印正常页面
                                else
                                {
                                    PrintBankBook_活期页面 page = new PrintBankBook_活期页面(new PrintBankBook_活期页面.PrintBankBook_活期页面Entry());
                                    tempDatas.Clear();
                                    if (extra >= pageTatolLine)
                                    {
                                        for (int i = 1; i < currentPrintLine; i++)
                                        {
                                            tempDatas.Add(new CXCZPageEntity());
                                        }
                                        tempDatas.AddRange(PageData.GetRange(account - extra, pageTatolLine));
                                        extra = extra - (pageTatolLine - currentPrintLine + 1);
                                        if (extra > 0)
                                        {
                                            tmp = PageData[account - extra].Line % PrintPageNum;
                                            currentPrintLine = tmp == 0 ? 1 : tmp;
                                        }
                                        //tmp = PageData[account - extra].Line % PrintPageNum;
                                        //currentPrintLine = tmp == 0 ? 1 : tmp;
                                    }
                                    else
                                    {
                                        for (int i = 1; i < currentPrintLine; i++)
                                        {
                                            tempDatas.Add(new CXCZPageEntity());
                                        }
                                        tempDatas.AddRange(PageData.GetRange(account - extra, extra));
                                        extra = 0;
                                    }
                                    page.PageData3 = tempDatas;
                                    page.SetValue();
                                    retData.Add(page);
                                }
                            }
                        }

                        //首页不换页
                        else if (PrintPageNum - currentPrintLine + 1 >= account)
                        {
                            PrintBankBook_活期首页 first = new PrintBankBook_活期首页(new PrintBankBook_活期首页.PrintBankBook_活期首页Entry());
                            first.Description = "请打印第1页";
                            tempDatas.Clear();
                            for (int i = 1; i < currentPrintLine; i++)
                            {
                                tempDatas.Add(new CXCZPageEntity());
                            }
                            tempDatas.AddRange(PageData.GetRange(0, account));
                            first.PageData = tempDatas;
                            first.Setvalue();
                            retData.Clear();
                            retData.Add(first);
                        }

                    }

                    //不带首页
                    else
                    {
                        extra = 0;
                        tmp = (PageData[0].Line - PrintPageNum) % pageTatolLine;
                        currentPrintLine = tmp == 0 ? pageTatolLine : tmp;
                        //数据页换页
                        if (pageTatolLine - currentPrintLine + 1 < account)
                        {

                            PrintBankBook_活期页面 page = new PrintBankBook_活期页面(new PrintBankBook_活期页面.PrintBankBook_活期页面Entry());
                            tempDatas.Clear();
                            for (int i = 1; i < currentPrintLine; i++)
                            {
                                tempDatas.Add(new CXCZPageEntity());
                            }
                            tempDatas.AddRange(PageData.GetRange(0, pageTatolLine - currentPrintLine + 1));
                            page.PageData3 = tempDatas;
                            page.SetValue();
                            retData.Clear();
                            retData.Add(page);
                            extra = account - (pageTatolLine - currentPrintLine + 1);
                            tmp = (PageData[account - extra].Line - PrintPageNum) % pageTatolLine;
                            currentPrintLine = tmp == 0 ? pageTatolLine : tmp;
                            while (extra > 0)
                            {
                                //当前已经打印到尾页
                                if (PageData[account - extra].Line > (SumNum - PrintPageNum))
                                {
                                    PrintBankBook_活期尾页 last = new PrintBankBook_活期尾页(new PrintBankBook_活期尾页.PrintBankBook_活期尾页Entry());
                                    tempDatas.Clear();
                                    if (extra > PrintPageNum)
                                    {
                                        for (int i = 1; i < currentPrintLine; i++)
                                        {
                                            tempDatas.Add(new CXCZPageEntity());
                                        }
                                        tempDatas.AddRange(PageData.GetRange(account - extra, PrintPageNum));
                                        extra = 0;
                                        //Todo提示换证  
                                        last.Description = "请换折";
                                    }
                                    else
                                    {
                                        for (int i = 1; i < currentPrintLine; i++)
                                        {
                                            tempDatas.Add(new CXCZPageEntity());
                                        }
                                        tempDatas.AddRange(PageData.GetRange(account - extra, extra));
                                        extra = 0;
                                    }
                                    last.PageData3 = tempDatas;
                                    last.SetValue();
                                    retData.Add(last);
                                }
                                //打印正常页面
                                else
                                {
                                    page = new PrintBankBook_活期页面(new PrintBankBook_活期页面.PrintBankBook_活期页面Entry());
                                    tempDatas.Clear();
                                    if (extra >= pageTatolLine)
                                    {
                                        for (int i = 1; i < currentPrintLine; i++)
                                        {
                                            tempDatas.Add(new CXCZPageEntity());
                                        }
                                        tempDatas.AddRange(PageData.GetRange(account - extra, pageTatolLine));
                                        extra = extra - (pageTatolLine - currentPrintLine + 1);
                                        if (extra > 0 )
                                        {
                                            tmp = (PageData[account - extra].Line - PrintPageNum) % pageTatolLine;
                                            currentPrintLine = tmp == 0 ? 1 : tmp;
                                        }
                                        //tmp = (PageData[account - extra].Line - PrintPageNum) % pageTatolLine;
                                        //currentPrintLine = tmp == 0 ? 1 : tmp;
                                    }
                                    else
                                    {
                                        for (int i = 1; i < currentPrintLine; i++)
                                        {
                                            tempDatas.Add(new CXCZPageEntity());
                                        }
                                        tempDatas.AddRange(PageData.GetRange(account - extra, extra));
                                        extra = 0;
                                    }
                                    page.PageData3 = tempDatas;
                                    page.SetValue();
                                    retData.Add(page);
                                }
                            }
                        }
                        //数据页不换页
                        else if (pageTatolLine - currentPrintLine + 1 >= account)
                        {
                            //当前已经打印到尾页
                            if (PageData[0].Line > (SumNum - PrintPageNum))
                            {
                                PrintBankBook_活期尾页 last = new PrintBankBook_活期尾页(new PrintBankBook_活期尾页.PrintBankBook_活期尾页Entry());
                                tempDatas.Clear();
                                if (account > SumNum - PageData[0].Line + 1)
                                {
                                    for (int i = 1; i < currentPrintLine; i++)
                                    {
                                        tempDatas.Add(new CXCZPageEntity());
                                    }
                                    tempDatas.AddRange(PageData.GetRange(0, SumNum - PageData[0].Line + 1));
                                    //Todo提示换证  
                                    last.Description = "请换折";
                                }
                                else
                                {
                                    for (int i = 1; i < currentPrintLine; i++)
                                    {
                                        tempDatas.Add(new CXCZPageEntity());
                                    }
                                    tempDatas.AddRange(PageData.GetRange(0, account));
                                }
                                last.PageData3 = tempDatas;
                                last.SetValue();
                                retData.Add(last);
                            }
                            else
                            {
                                PrintBankBook_活期页面 page = new PrintBankBook_活期页面(new PrintBankBook_活期页面.PrintBankBook_活期页面Entry());
                                tempDatas.Clear();
                                for (int i = 1; i < currentPrintLine; i++)
                                {
                                    tempDatas.Add(new CXCZPageEntity());
                                }
                                tempDatas.AddRange(PageData.GetRange(0, PageData.Count));
                                page.PageData3 = tempDatas;
                                page.SetValue();
                                retData.Clear();
                                retData.Add(page);
                            }
                        }
                    }
                }
            }

            return retData;
        }
    }
}
