﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml;
using Platform.Common.LogSystem;
using Platform.Common.RunningParameters;
using TellerSystem.PrintTemplate.PrintTemplateData;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.Communication;
using TellerSystem.Communication.MessageHelper;
using TellerSystem.PrintTemplate.ElecSeal;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 活期存折首页
    /// </summary>
    public partial class PrintBankBook_活期首页 : IPrintContract
    {
        public PrintBankBook_活期首页(PrintBankBook_活期首页Entry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }
        public Contract.Entry.PrintEntry PrintData
        {
            get;
            set;
        }

        public void LoadPrintData()
        {
            var entry = PrintData as PrintBankBook_活期首页Entry;
            if (entry == null || string.IsNullOrEmpty(entry.AccountName)) return;
            TB_AccountName.Text = entry.AccountName;
            TB_Currency.Text = entry.Currency;
            TB_AccountNo.Text = entry.AccountNo;
            TB_CZ.Text = entry.Type;
            //var date = DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd", null);
            var date = entry.SignDate.ToDateTime();
            TB_Date_Year.Text = date.Year.ToString();
            TB_Date_Month.Text = date.Month.ToString();
            TB_Date_Day.Text = date.Day.ToString();

            TB_PasswordOrSeal.Text = entry.PasswordOrSeal;
            //TB_Species.Text = "活期存折";
            TB_Species.Text = entry.Species;
            TB_Interestrate.Text = decimal.Parse(entry.Interestrate).ToString() + "%";
            TB_OpenBrno.Text = entry.OpenBrno;
            TB_BankNo.Text = "支付行号：" + entry.PaySysCode;
            TB_BankNo.Text = entry.PaySysCode;
            TB_NoteNO.Text = entry.NoteNo;
            //电子印章
            string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            if (!string.IsNullOrEmpty(LoginUserInfo.KinbrNo))
            {
                if (LoginUserInfo.KinbrNo.Substring(0, 3) == "601")//融丰
                {
                    this.E_Seal_Location = new Point(280, 90);
                }
                else if (LoginUserInfo.KinbrNo.Substring(0, 3) != "820" && FLAG != "V5")//阳光（大石桥不打印20200925）
                {
                    this.E_Seal_Location = new Point(319, 100);
                }
                //青隆电子签章
                else if (FLAG == "V5")
                {
                    this.ElecSealHelper.Sign2Point(new Point(280, 90), SealKind.Biz);
                }
            }
        }

        public class PrintBankBook_活期首页Entry : PrintEntry
        {
            /**********************************
             * 定义实体属性
             * 1.打印逻辑用到的要素
             * *******************************/
            #region 打印属性
            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }

            /// <summary>
            /// 币种
            /// </summary>
            public string Currency { get; set; }

            /// <summary>
            /// 账号
            /// </summary>
            public string AccountNo { get; set; }

            /// <summary>
            /// 类型
            /// </summary>
            public string Type { get; set; }

            /// <summary>
            /// 存款类别
            /// </summary>
            public string DepositType { get; set; }

            /// <summary>
            /// 印密
            /// </summary>
            public string PasswordOrSeal { get; set; }

            /// <summary>
            /// 种类
            /// </summary>
            public string Species { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public string Interestrate { get; set; }
            /// <summary>
            /// 支付行号
            /// </summary>
            public string PaySysCode { get; set; }

            /// <summary>
            /// 签发日期
            /// </summary>
            public string SignDate { get; set; }
            /// <summary>
            /// 开户行
            /// </summary>
            public string OpenBrno { get; set; }
            /// <summary>
            /// 凭证号
            /// </summary>
            public string NoteNo { get; set; }


            #endregion
        }


        public List<CXCZPageEntity> PageData { get; set; }

        public void Setvalue()
        {
            if (PageData == null) return;
            if (PageData.Count == 0) return;
            //郏县、新密需求 存折打印偏上，需调整
            var FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            switch (FLAG)
            {
                case "V1"://郏县
                    BBP_Bottom.SetValue(Canvas.TopProperty, 312.0);
                    break;
                case "V3"://新密
                    BBP_Bottom.SetValue(Canvas.TopProperty, 312.0);
                    break;
                case "V5"://山东
                    BBP_Bottom.SetValue(Canvas.TopProperty, 322.0);
                    CvsCanvas.SetValue(Canvas.TopProperty, -10.0);
                    break;
                case "V6"://淇县、濮阳
                    BBP_Bottom.SetValue(Canvas.TopProperty, 297.0);
                    CvsCanvas.SetValue(Canvas.TopProperty, -10.0);
                    break;
                case "V8"://阳光
                    BBP_Bottom.SetValue(Canvas.TopProperty, 297.0);
                    CvsCanvas.SetValue(Canvas.TopProperty, -10.0);
                    break;
                case "V12"://长春融丰
                case "V11"://大石桥隆丰
                    BBP_Bottom.SetValue(Canvas.TopProperty, 312.0);
                    CvsCanvas.SetValue(Canvas.TopProperty, -10.0);
                    break;
                case "V9"://汇通
                    TB_OpenBrno.SetValue(Canvas.TopProperty, 193.0);
                    TB_OpenBrno.SetValue(Canvas.LeftProperty, 80.0);
                    TB_OpenBrno.SetValue(Canvas.HeightProperty, 40.0);
                    TB_OpenBrno.SetValue(Canvas.WidthProperty, 150.0);
                    break;
                case "V21":
                case "V22":
                case "V13":
                case "V10":
                    BBP_Bottom.SetValue(Canvas.TopProperty, 312.0);
                    TB_Interestrate.Visibility = Visibility.Collapsed;
                    TB_NoteNO.Visibility = Visibility.Collapsed;
                    break;
                default:
                    break;
            }
            BBP_Bottom.LoadOldData(PageData);
        }
    }
}