﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// BankBookPage.xaml 的交互逻辑
    /// </summary>
    public partial class BankBookPageQF : UserControl
    {
        public BankBookPageQF()
        {
            InitializeComponent();
            MaxLineCount = 12;

            Data = new DataTable();
            Data.Columns.Add("Date");
            Data.Columns.Add("Summary");
            Data.Columns.Add("XH");//增加笔序号
            Data.Columns.Add("CQ");//存期
            Data.Columns.Add("LV");//利率
            Data.Columns.Add("DepWinhType");
            Data.Columns.Add("Balance");
            Data.Columns.Add("Remark");
            if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
            {
                for (int i = 0; i < MaxLineCount; i++)
                {
                    Data.Rows.Add("Date", "Summary", "XH", "CQ", "LV", "DepWinhType", "Balance", "Remark");
                }
            }
            IC_Page.ItemsSource = Data.DefaultView;
        }

        public DataTable Data { get; private set; }

        /// <summary>
        /// 最大的打印行数
        /// </summary>
        public int MaxLineCount { get; set; }

        /// <summary>
        /// 加载数据，返回值为实际处理的数据条目
        /// </summary>
        /// <param name="data"></param>
        /// <param name="startLineNo"></param>
        /// <returns></returns>
        public int LoadData(IEnumerable<BankBookPageQFLine> data, int startLineNo)
        {
            if (startLineNo < 0 || startLineNo >= MaxLineCount)
                throw new ArgumentOutOfRangeException("startLineNo不能小于0");
            Data.Rows.Clear();

            for (int i = 0; i < startLineNo; i++)
            {
                Data.Rows.Add("", "", "", "", "", "", "", "");
            }
            var count = 0;
            foreach (var item in data)
            {
                count++;
                Data.Rows.Add(item.Date == null ? string.Empty : item.Date.Value.ToString("yyyyMMdd"),
                    item.Summary,
                    item.XH,
                    item.CQ,
                    item.LV,
                    item.DepWinhType,
                    //item.Withdrawals == null ? string.Empty : item.Withdrawals.Value.ToString("N2"),
                    //item.Deposit == null ? string.Empty : item.Deposit.Value.ToString("N2"),
                    item.Balance == null ? string.Empty : item.Balance.Value.ToString("N2"),
                    item.Remark);
                if (startLineNo + count >= MaxLineCount) break;
            }
            return count;
        }

        public void LoadOldData(IEnumerable<CXCZPageEntity> data)
        {
            //string s = string.Empty;
            var ss = new BankBookPageQFLine();
            var newData = data.Select(x =>
            {
                var ret = new BankBookPageQFLine();
                if (!string.IsNullOrEmpty(x.DateInfo))
                {
                    ret.Date = DateTime.ParseExact(x.DateInfo, "yyyyMMdd", null);
                    ret.Summary = x.Description;
                    ret.XH = x.Number;
                    ret.CQ = x.Deadline;
                    //老数据 活期存折更换介质未返回利率
                    if (!string.IsNullOrEmpty(x.Interestrate.Trim()))
                    {
                        try
                        {
                            if (double.Parse(x.Interestrate) != 0)
                            {
                                ret.LV = double.Parse(x.Interestrate).ToString();

                            }
                            else
                            {
                                ret.LV = string.Empty;
                            }
                        }
                        catch (Exception ex)
                        {
                            ret.LV = string.Empty;
                        }
                    }
                    else
                    {
                        ret.LV = string.Empty;
                    }
                    decimal withdrawals, deposit, balance;
                    ret.DepWinhType = x.DepWinhType;
                    //if (decimal.TryParse(x.Withdrawal, out withdrawals))
                    //    ret.Withdrawals = withdrawals;
                    //if (decimal.TryParse(x.Deposit, out deposit))
                    //    ret.Deposit = deposit;
                    if (decimal.TryParse(x.Balance, out balance))
                        ret.Balance = balance;
                    ret.Remark = x.BankOperate;
                }
                ss = ret;
                return ret;
            });
            var tt = ss;
            LoadData(newData, 0);
        }

        public void LoadOldData(IEnumerable<CXCZPageQFEntity> data)
        {
            //string s = string.Empty;
            var ss = new BankBookPageQFLine();
            var newData = data.Select(x =>
            {
                var ret = new BankBookPageQFLine();
                if (!string.IsNullOrEmpty(x.DateInfo))
                {
                    ret.Date = DateTime.ParseExact(x.DateInfo, "yyyyMMdd", null);
                    ret.Summary = x.Description;
                    ret.XH = x.Number;
                    ret.CQ = x.Deadline;
                    //老数据 活期存折更换介质未返回利率
                    if (!string.IsNullOrEmpty(x.Interestrate.Trim()))
                    {
                        try
                        {
                            if (double.Parse(x.Interestrate) != 0)
                            {
                                ret.LV = double.Parse(x.Interestrate).ToString();

                            }
                            else
                            {
                                ret.LV = string.Empty;
                            }
                        }
                        catch (Exception ex)
                        {
                            ret.LV = string.Empty;
                        }
                    }
                    else
                    {
                        ret.LV = string.Empty;
                    }
                    decimal withdrawals, deposit, balance;
                    ret.DepWinhType = x.DepWinhType;
                    //if (decimal.TryParse(x.Withdrawal, out withdrawals))
                    //    ret.Withdrawals = withdrawals;
                    //if (decimal.TryParse(x.Deposit, out deposit))
                    //    ret.Deposit = deposit;
                    if (decimal.TryParse(x.Balance, out balance))
                        ret.Balance = balance;
                    ret.Remark = x.BankOperate;
                }
                ss = ret;
                return ret;
            });
            var tt = ss;
            LoadData(newData, 0);
        }

        /// <summary>
        /// 存折行数据
        /// </summary>
        public class BankBookPageQFLine
        {
            /// <summary>
            /// 发生日期
            /// </summary>
            public DateTime? Date { get; set; }
            /// <summary>
            /// 摘要
            /// </summary>
            public string Summary { get; set; }
            /// <summary>
            /// 笔序号
            /// </summary>
            public string XH { get; set; }
            /// <summary>
            /// 存期
            /// </summary>
            public string CQ { get; set; }
            /// <summary>
            /// 利率
            /// </summary>
            public string LV { get; set; }
            ///// <summary>
            ///// 支出
            ///// </summary>
            //public decimal? Withdrawals { get; set; }
            ///// <summary>
            ///// 存入
            ///// </summary>
            //public decimal? Deposit { get; set; }
            /// <summary>
            ///  贷方（存入用"+")
            ///  借方（支出用"-")
            /// </summary>
            public string DepWinhType { get; set; }
            /// <summary>
            /// 余额
            /// </summary>
            public decimal? Balance { get; set; }
            /// <summary>
            /// 银行备考
            /// </summary>
            public string Remark { get; set; }
        }
    }
}
