﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// BankBookPage.xaml 的交互逻辑
    /// </summary>
    public partial class BankBookPageXX : UserControl
    {
        public BankBookPageXX()
        {
            InitializeComponent();
            MaxLineCount = 12;

            Data = new DataTable();
            Data.Columns.Add("ZZH");
            Data.Columns.Add("Date");
            Data.Columns.Add("Summary");
            Data.Columns.Add("CR");
            Data.Columns.Add("ZC");
            Data.Columns.Add("YE");
            Data.Columns.Add("Kinbr");
            Data.Columns.Add("Teller");
            if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
            {
                for (int i = 0; i < MaxLineCount; i++)
                {
                    Data.Rows.Add("ZZH", "Date", "Summary", "CR", "ZC", "YE", "Kinbr", "Teller");
                }
            }
            IC_Page.ItemsSource = Data.DefaultView;
        }

        public DataTable Data { get; private set; }

        /// <summary>
        /// 最大的打印行数
        /// </summary>
        public int MaxLineCount { get; set; }

        /// <summary>
        /// 加载数据，返回值为实际处理的数据条目
        /// </summary>
        /// <param name="data"></param>
        /// <param name="startLineNo"></param>
        /// <returns></returns>
        public int LoadData(IEnumerable<BankBookPageXXLine> data, int startLineNo)
        {
            if (startLineNo < 0 || startLineNo >= MaxLineCount)
                throw new ArgumentOutOfRangeException("startLineNo不能小于0");
            Data.Rows.Clear();

            for (int i = 0; i < startLineNo; i++)
            {
                Data.Rows.Add("", "", "", "", "", "", "", "");
            }
            var count = 0;
            foreach (var item in data)
            {
                count++;
                Data.Rows.Add(
                    item.ZZH,
                    item.Date == null ? string.Empty : item.Date.Value.ToString("yyyyMMdd"),
                    item.Summary,
                    item.Deposit == null ? string.Empty : item.Deposit.Value.ToString("N2"),
                    item.Withdrawals == null ? string.Empty : item.Withdrawals.Value.ToString("N2"),
                    item.YE,
                    item.Kinbr,
                    item.Teller);
                if (startLineNo + count >= MaxLineCount) break;
            }
            return count;
        }

        public void LoadOldData(IEnumerable<CXCZPageXXEntity> data)
        {
            var ss = new BankBookPageXXLine();
            var newData = data.Select(x =>
            {
                var ret = new BankBookPageXXLine();
                if (!string.IsNullOrEmpty(x.DateInfo))
                {
                    ret.ZZH = x.Number;
                    ret.Date = DateTime.ParseExact(x.DateInfo, "yyyyMMdd", null);
                    ret.Summary = x.Description;
                    decimal withdrawals, deposit, balance;
                    if (decimal.TryParse(x.Deposit, out deposit))
                        ret.Deposit = deposit;
                    if (decimal.TryParse(x.Withdrawal, out withdrawals))
                        ret.Withdrawals = withdrawals;
                    if (decimal.TryParse(x.Balance, out balance))
                        ret.YE = balance;
                    ret.Kinbr = x.Kinbr;
                    ret.Teller = x.TellerNo;
                }
                ss = ret;
                return ret;
            });
            var tt = ss;
            LoadData(newData, 0);
        }

        /// <summary>
        /// 存折行数据
        /// </summary>
        public class BankBookPageXXLine
        {
            /// <summary>
            /// 子账号
            /// </summary>
            public string ZZH { get; set; }
            /// <summary>
            /// 发生日期
            /// </summary>
            public DateTime? Date { get; set; }
            /// <summary>
            /// 摘要
            /// </summary>
            public string Summary { get; set; }
            /// <summary>
            /// 支出
            /// </summary>
            public decimal? Withdrawals { get; set; }
            /// <summary>
            /// 存入
            /// </summary>
            public decimal? Deposit { get; set; }
            /// <summary>
            /// 余额
            /// </summary>
            public decimal? YE { get; set; }
            /// <summary>
            /// 网点号
            /// </summary>
            public string Kinbr { get; set; }
            /// <summary>
            /// 柜员号
            /// </summary>
            public string Teller { get; set; }
        }
    }
}
