﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Platform.Common.RunningParameters;
using Platform.Print;
using TellerSystem.Communication;
using TellerSystem.Library.Ext.Function;
using TellerSystem.PrintTemplate.Bankbook;
using Platform.Common.LogSystem;

namespace TellerSystem.Controls.Ext.PrintsHelper.DQCZ
{
    /// <summary>
    /// 存折打印辅助类
    /// </summary>
    public class BankBookHelper
    {
        const string GANG = "-------";

        #region 活期存折

        /// <summary>
        /// 打印储蓄存折首页
        /// </summary>
        /// <param name="msg">报文</param>
        /// <returns></returns>
        public static List<PrintContainer> PrintCXCZFirstPage(Message msg)
        {
            var aCxczBankbook = CXCZBankbook.GetInstance();

            try
            {
                AnalyzeCXCZData(aCxczBankbook, msg, "FirstPage");
            }
            catch (Exception e)
            {
                PlatformLogger.SystemErrorInfo("打印解析出错:", e);
                MessageBox.Show("打印解析出错");
            }

            return aCxczBankbook.PrintFirstPage();
        }

        public static List<PrintContainer> PrintCXCZFirstPage_CZFY(Message msg)
        {
            var aCxczBankbook = CXCZBankbook.GetInstance();
            try
            {
                AnalyzeCXCZData(aCxczBankbook, msg, "Page");
            }
            catch (Exception e)
            {
                PlatformLogger.SystemErrorInfo("打印解析出错:", e);
                MessageBox.Show("打印解析出错");
            }
            return aCxczBankbook.PrintFirstPage();
        }

        /// <summary>
        /// 打印更换介质首页
        /// </summary>
        /// <param name="msg">报文</param>
        /// <returns></returns>
        public static List<PrintContainer> PrintGHJZFirstPage(Message msg)
        {
            var aCxczBankbook = CXCZBankbook.GetInstance();
            try
            {
                AnalyzeCXCZData(aCxczBankbook, msg, "GHJZ");
            }
            catch (Exception e)
            {
                PlatformLogger.SystemErrorInfo("打印解析出错:", e);
                MessageBox.Show("打印解析出错");
            }
            return aCxczBankbook.PrintFirstPage();
        }

        /// <summary>
        /// 打印储蓄存折数据页
        /// </summary>
        /// <param name="msg">报文</param>
        /// <returns></returns>
        public static List<PrintContainer> PrintCXCZPage(Message msg)
        {
            var aCxczBankbook = CXCZBankbook.GetInstance();
            try
            {
                AnalyzeCXCZData(aCxczBankbook, msg, "Page");
            }
            catch (Exception e)
            {
                PlatformLogger.SystemErrorInfo("打印解析出错:", e);
                MessageBox.Show("打印解析出错");
            }
            return aCxczBankbook.PrintPage();
        }

        /// <summary>
        /// 打印储蓄存折数据页
        /// </summary>
        /// <param name="msg">报文</param>
        /// <returns></returns>
        public static List<PrintContainer> PrintCXCZPage(Message msg, string flg)
        {
            var aCxczBankbook = CXCZBankbook.GetInstance();
            try
            {
                AnalyzeCXCZData(aCxczBankbook, msg, flg);
            }
            catch (Exception e)
            {
                PlatformLogger.SystemErrorInfo("打印解析出错:", e);
                MessageBox.Show("打印解析出错");
            }
            return aCxczBankbook.PrintPage();
        }

        /// <summary>
        /// 不打印转账储蓄存折数据页
        /// </summary>
        /// <param name="msg">报文</param>
        /// <returns></returns>
        public static List<PrintContainer> UnPrintCXZZPage(Message msg, string acno)
        {
            var cxcz = CXCZBankbook.GetInstance();

            try
            {
                cxcz.FirstPageData = null;
                cxcz.EndPageData = null;
                cxcz.PageData = new List<CXCZPageEntity>();

                var czfy = msg.AnilyzePrintFile("CZFY|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|||@rate|@tx_amt||@zy|@deac_type|@cur_name|@tx_code||@ac_no1|||@ac_seqn|@ic_date|@prt_line|@prdt_no|@page_no|@brf|@tmp_ac_no|@tmp_tx_name||||@br_name|@br_tele|@note_no||@accno|@zy|@mbfebkno||@shebaono|@tctd_flag|@txbrname||@tctd|");
                if (czfy.Count != 0)
                {
                    var draw = string.Empty;
                    var depositType = string.Empty;
                    var firstPageData = new CXCZFirstPageEntity();
                    //if (opna[0]["@zy"].Equals(" 【 凭密支取 】"))
                    //{
                    //    draw = " 凭密支取 ";
                    //}
                    //else if (opna[0]["@zy"].Equals(" 【 密码兼证件支取 】"))
                    //{
                    //    draw = " 密码兼证件支取 ";
                    //}

                    if (czfy[0]["@prdt_no"].Equals("101"))
                    {
                        depositType = "活期储蓄";
                    }

                    firstPageData.AcNo = czfy[0]["@ac_no"];
                    firstPageData.AcName = czfy[0]["@name"];
                    firstPageData.CZNum = czfy[0]["@note_no"];
                    firstPageData.CurNo = czfy[0]["@cur_name"];
                    firstPageData.DepositType = depositType;
                    firstPageData.DrawMode = czfy[0]["@zy"];
                    firstPageData.OpnBrname = czfy[0]["@br_name"];
                    firstPageData.OpnDate = czfy[0]["@opn_date"];
                    firstPageData.PaySysCode = czfy[0]["@mbfebkno"];
                    firstPageData.DepositType = czfy[0]["@title"];
                    cxcz.FirstPageData = firstPageData;

                    //TODO:数据组装
                    var pageData = new CXCZPageEntity();
                    pageData.Balance = czfy[0]["@tx_amt1"].ToDecimal().ToString("N2").PadLeft(12);
                    pageData.DateInfo = czfy[0]["@opn_date"];
                    pageData.Deposit = pageData.Balance;
                    pageData.Description = czfy[0]["@brf"];
                    pageData.IsVisibility = Visibility.Visible;
                    pageData.Line = 0;
                    pageData.TellerNo = czfy[0]["@br_tele"];
                    pageData.Withdrawal = "";
                    cxcz.PageData.Add(pageData);
                    return cxcz.PrintPage();
                }

                //普通页数据
                var comPage = msg.AnilyzePrintFile("CXCZ|@txday|@dscpt|@txamt|@dbflag|@avbal|@tlrno||@brno|@tty|@lineno||@txcode|@term|@rate|@ac_seqn|@srate|@sterm|@zc_flag|@ac_no");
                if (comPage.Count != 0)
                {
                    CXCZPageEntity tmpPageData = null;
                    string ytxamt = string.Empty;
                    string avbal = string.Empty;
                    int curLine = 0;
                    foreach (var alineData in comPage.Where(dic => dic["@ac_no"] == acno))
                    {
                        tmpPageData = new CXCZPageEntity();
                        tmpPageData.DateInfo = alineData["@txday"];
                        tmpPageData.Description = alineData["@dscpt"];
                        tmpPageData.AcNo = alineData["@ac_no"];
                        if (alineData["@dbflag"].Equals("0"))
                        {
                            ytxamt = alineData["@txamt"].ToDecimal().ToString("N2").PadLeft(12);
                            tmpPageData.Withdrawal = ytxamt;
                        }
                        else
                        {
                            ytxamt = alineData["@txamt"].ToDecimal().ToString("N2").PadLeft(12);
                            tmpPageData.Deposit = ytxamt;
                        }
                        avbal = alineData["@avbal"].ToDecimal().ToString("N2").PadLeft(12);
                        tmpPageData.Balance = avbal;
                        tmpPageData.TellerNo = alineData["@tlrno"];
                        if (!int.TryParse(alineData["@lineno"], out curLine))
                        {
                            throw new Exception("储蓄存折打印数据有问题！");
                        }
                        tmpPageData.Line = curLine;
                        tmpPageData.IsVisibility = Visibility.Visible;
                        cxcz.PageData.Add(tmpPageData);
                    }
                    return cxcz.PrintPage();
                }
            }
            catch (Exception e)
            {
                PlatformLogger.SystemErrorInfo("打印解析出错:", e);
                MessageBox.Show("打印解析出错");
            }


            return cxcz.PrintPage();
        }

        /// <summary>
        /// 打印储蓄存折尾页
        /// </summary>
        /// <param name="msg"></param>
        /// <returns></returns>
        public static List<PrintContainer> PrintEndPage(Message msg)
        {
            var aCxczBankbook = CXCZBankbook.GetInstance();
            try
            {
                AnalyzeCXCZData(aCxczBankbook, msg, "EndPage");
            }
            catch (Exception e)
            {
                PlatformLogger.SystemErrorInfo("打印解析出错:", e);
                MessageBox.Show("打印解析出错");
            }
            return aCxczBankbook.PrintEndPage();
        }

        /// <summary>
        /// 储蓄存折数据解析
        /// </summary>
        /// <param name="cxcz">储蓄存折</param>
        /// <param name="msg">报文</param>
        /// <returns></returns>
        private static bool AnalyzeCXCZData(CXCZBankbook cxcz, Message msg, string flag)
        {
            //开户首页数据OPNA|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|@term|@term_type|@rate|@tx_amt1|@email|@zy|@deac_type|@cur_name|@tx_code|@memo|@ac_no1||@ct_name|@ac_seqn|@ic_date|@prt_line|@prdt_no||@brf|@tmp_ac_no|@tmp_tx_name|@tel|@id_name|@id_no|@br_name|@br_tele|@note_no||||@mbfebkno|@card_flag|@tx_br_name|@nxybrno
            //储蓄存折数据CXCZ|@txday|@dscpt|@txamt|@dbflag|@avbal|@tlrno||@brno|@tty|@lineno|||@term|@rate|@ac_seqn|@srate|@sterm|@zc_flag
            //存折数据初始化
            cxcz.FirstPageData = null;
            cxcz.EndPageData = null;
            cxcz.PageData = new List<CXCZPageEntity>();

            if (flag == "FirstPage")
            {
                //首页数据
                var opna = msg.AnilyzePrintFile("OPNA|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|@term|@term_type|@rate|@tx_amt|@email|@zy|@deac_type|@cur_name|@tx_code|@memo|@ac_no1||@ct_name|@ac_seqn|@ic_date|@prt_line|@prdt_no||@brf|@tmp_ac_no|@tmp_tx_name|@tel|@id_name|@id_no|@br_name|@br_tele|@note_no||||@mbfebkno|@card_flag|@tx_br_name|@nxybrno");
                if (opna.Count != 0)
                {
                    var draw = string.Empty;
                    var depositType = string.Empty;
                    var firstPageData = new CXCZFirstPageEntity();
                    //if (opna[0]["@zy"].Equals(" 【 凭密支取 】"))
                    //{
                    //    draw = " 凭密支取 ";
                    //}
                    //else if (opna[0]["@zy"].Equals(" 【 密码兼证件支取 】"))
                    //{
                    //    draw = " 密码兼证件支取 ";
                    //}

                    if (opna[0]["@prdt_no"].Equals("101"))
                    {
                        depositType = "活期储蓄";
                    }

                    firstPageData.AcNo = opna[0]["@ac_no"];
                    firstPageData.AcName = opna[0]["@name"];
                    firstPageData.CZNum = opna[0]["@note_no"];
                    firstPageData.CurNo = opna[0]["@cur_name"];
                    firstPageData.DepositType = depositType;
                    firstPageData.DrawMode = opna[0]["@zy"];
                    firstPageData.OpnBrname = opna[0]["@br_name"];
                    firstPageData.OpnDate = opna[0]["@opn_date"];
                    firstPageData.PaySysCode = opna[0]["@mbfebkno"];
                    firstPageData.DepositType = opna[0]["@title"];
                    if (opna[0]["@tmp_tx_name"] == "定期开卡")
                    {
                        firstPageData.InterestRate = opna[0]["@rate"] + "%";
                    }
                    cxcz.FirstPageData = firstPageData;

                    //TODO:数据组装
                    var pageData = new CXCZPageEntity();
                    pageData.Balance = opna[0]["@tx_amt1"].ToDecimal().ToString("N2").PadLeft(12);
                    pageData.DateInfo = opna[0]["@opn_date"];
                    pageData.Deposit = pageData.Balance;
                    pageData.Description = opna[0]["@memo"];
                    pageData.IsVisibility = Visibility.Visible;
                    pageData.Line = 0;
                    pageData.TellerNo = opna[0]["@tel"];
                    pageData.Withdrawal = "";
                    cxcz.PageData.Add(pageData);
                    return true;
                }
            }
            else if (flag == "Page")
            {
                var czfy = msg.AnilyzePrintFile("CZFY|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|||@rate|@tx_amt||@zy|@deac_type|@cur_name|@tx_code||@ac_no1|||@ac_seqn|@ic_date|@prt_line|@prdt_no|@page_no|@brf|@tmp_ac_no|@tmp_tx_name||||@br_name|@br_tele|@note_no||@accno|@zy|@mbfebkno||@shebaono|@tctd_flag|@txbrname||@tctd|");
                if (czfy.Count != 0)
                {
                    var draw = string.Empty;
                    var depositType = string.Empty;
                    var firstPageData = new CXCZFirstPageEntity();
                    //if (opna[0]["@zy"].Equals(" 【 凭密支取 】"))
                    //{
                    //    draw = " 凭密支取 ";
                    //}
                    //else if (opna[0]["@zy"].Equals(" 【 密码兼证件支取 】"))
                    //{
                    //    draw = " 密码兼证件支取 ";
                    //}

                    if (czfy[0]["@prdt_no"].Equals("101"))
                    {
                        depositType = "活期储蓄";
                    }

                    firstPageData.AcNo = czfy[0]["@ac_no"];
                    firstPageData.AcName = czfy[0]["@name"];
                    firstPageData.CZNum = czfy[0]["@note_no"];
                    firstPageData.CurNo = czfy[0]["@cur_name"];
                    firstPageData.DepositType = depositType;
                    firstPageData.DrawMode = czfy[0]["@zy"];
                    firstPageData.OpnBrname = czfy[0]["@br_name"];
                    firstPageData.OpnDate = czfy[0]["@opn_date"];
                    firstPageData.PaySysCode = czfy[0]["@mbfebkno"];
                    firstPageData.DepositType = czfy[0]["@title"];
                    cxcz.FirstPageData = firstPageData;

                    //TODO:数据组装
                    var pageData = new CXCZPageEntity();
                    pageData.Balance = czfy[0]["@tx_amt1"].ToDecimal().ToString("N2").PadLeft(12);
                    pageData.DateInfo = czfy[0]["@opn_date"];
                    pageData.Deposit = pageData.Balance;
                    pageData.Description = czfy[0]["@brf"];
                    pageData.IsVisibility = Visibility.Visible;
                    pageData.Line = 0;
                    pageData.TellerNo = czfy[0]["@br_tele"];
                    pageData.Withdrawal = "";
                    cxcz.PageData.Add(pageData);
                    return true;
                }

                //普通页数据
                var comPage = msg.AnilyzePrintFile("CXCZ|@txday|@dscpt|@txamt|@dbflag|@avbal|@tlrno||@brno|@tty|@lineno||@txcode|@term|@rate|@ac_seqn|@srate|@sterm|@zc_flag|@ac_no");
                if (comPage.Count != 0)
                {
                    CXCZPageEntity tmpPageData = null;
                    string ytxamt = string.Empty;
                    string avbal = string.Empty;
                    int curLine = 0;
                    foreach (var alineData in comPage)
                    {
                        tmpPageData = new CXCZPageEntity();
                        tmpPageData.DateInfo = alineData["@txday"];
                        tmpPageData.Description = alineData["@dscpt"];
                        tmpPageData.AcNo = alineData["@ac_no"];
                        if (alineData["@dbflag"].Equals("0"))
                        {
                            ytxamt = alineData["@txamt"].ToDecimal().ToString("N2").PadLeft(12);
                            tmpPageData.Withdrawal = ytxamt;
                        }
                        else
                        {
                            ytxamt = alineData["@txamt"].ToDecimal().ToString("N2").PadLeft(12);
                            tmpPageData.Deposit = ytxamt;
                        }
                        avbal = alineData["@avbal"].ToDecimal().ToString("N2").PadLeft(12);
                        tmpPageData.Balance = avbal;
                        tmpPageData.TellerNo = alineData["@tlrno"];
                        if (!int.TryParse(alineData["@lineno"], out curLine))
                        {
                            throw new Exception("储蓄存折打印数据有问题！");
                        }
                        tmpPageData.Line = curLine;
                        tmpPageData.IsVisibility = Visibility.Visible;
                        cxcz.PageData.Add(tmpPageData);
                    }
                    return true;
                }
            }
            //存折转账
            else if (flag == "CZZZ")
            {
                //普通页数据
                var comPage = msg.AnilyzePrintFile("CXCZ|@txday|@dscpt|@txamt|@dbflag|@avbal|@tlrno||@brno|@tty|@lineno||@txcode|@term|@rate|@ac_seqn|@srate|@sterm|@zc_flag|@ac_no");
                if (comPage.Count != 0)
                {
                    CXCZPageEntity tmpPageData = null;
                    string ytxamt = string.Empty;
                    string avbal = string.Empty;
                    int curLine = 0;
                    foreach (var alineData in comPage)
                    {
                        tmpPageData = new CXCZPageEntity();
                        tmpPageData.DateInfo = alineData["@txday"];
                        tmpPageData.Description = alineData["@dscpt"];
                        tmpPageData.AcNo = alineData["@ac_no"];
                        if (alineData["@dbflag"].Equals("0"))
                        {
                            ytxamt = alineData["@txamt"].ToDecimal().ToString("N2").PadLeft(12);
                            tmpPageData.Withdrawal = ytxamt;
                        }
                        else
                        {
                            ytxamt = alineData["@txamt"].ToDecimal().ToString("N2").PadLeft(12);
                            tmpPageData.Deposit = ytxamt;
                        }
                        avbal = alineData["@avbal"].ToDecimal().ToString("N2").PadLeft(12);
                        tmpPageData.Balance = avbal;
                        tmpPageData.TellerNo = alineData["@tlrno"];
                        if (!int.TryParse(alineData["@lineno"], out curLine))
                        {
                            throw new Exception("储蓄存折打印数据有问题！");
                        }
                        tmpPageData.Line = curLine;
                        tmpPageData.IsVisibility = Visibility.Visible;
                        cxcz.PageData.Add(tmpPageData);
                    }
                    return true;
                }
            }
            //更换介质
            else if (flag == "GHJZ")
            {
                var czfy = msg.AnilyzePrintFile("CZFY|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|||@rate|@tx_amt1||@zy|@deac_type|@cur_name|@tx_code||@ac_no1|||@ac_seqn|@ic_date|@prt_line|@prdt_no|@page_no|@brf|@tmp_ac_no|@tmp_tx_name||||@br_name|@br_tele|@note_no||@accno|@zy|@mbfebkno||@shebaono|@tctd_flag|@txbrname||@tctd|");
                if (czfy.Count != 0)
                {
                    var draw = string.Empty;
                    var depositType = string.Empty;
                    var firstPageData = new CXCZFirstPageEntity();

                    if (czfy[0]["@prdt_no"].Equals("101"))
                    {
                        depositType = "活期储蓄";
                    }

                    firstPageData.AcNo = czfy[0]["@ac_no"];
                    firstPageData.AcName = czfy[0]["@name"];
                    firstPageData.CZNum = czfy[0]["@note_no"];
                    firstPageData.CurNo = czfy[0]["@cur_name"];
                    firstPageData.DepositType = depositType;
                    firstPageData.DrawMode = czfy[0]["@zy"];
                    firstPageData.OpnBrname = czfy[0]["@br_name"];
                    firstPageData.OpnDate = czfy[0]["@opn_date"];
                    firstPageData.PaySysCode = czfy[0]["@mbfebkno"];
                    firstPageData.DepositType = czfy[0]["@title"];

                    firstPageData.InterestRate = czfy[0]["@rate"] + "%";

                    cxcz.FirstPageData = firstPageData;
                }

                //普通页数据
                var comPage = msg.AnilyzePrintFile("CXCZ|@txday|@dscpt|@txamt|@dbflag|@avbal|@tlrno||@brno|@tty|@lineno||@txcode|@term|@rate|@ac_seqn|@srate|@sterm|@zc_flag|@ac_no");
                if (comPage.Count != 0)
                {
                    CXCZPageEntity tmpPageData = null;
                    string ytxamt = string.Empty;
                    string avbal = string.Empty;
                    int curLine = 0;
                    foreach (var alineData in comPage)
                    {
                        tmpPageData = new CXCZPageEntity();
                        tmpPageData.DateInfo = alineData["@txday"];
                        tmpPageData.Description = alineData["@dscpt"];
                        tmpPageData.AcNo = alineData["@ac_no"];
                        if (alineData["@dbflag"].Equals("0"))
                        {
                            ytxamt = alineData["@txamt"].ToDecimal().ToString("N2").PadLeft(12);
                            tmpPageData.Withdrawal = ytxamt;
                        }
                        else
                        {
                            ytxamt = alineData["@txamt"].ToDecimal().ToString("N2").PadLeft(12);
                            tmpPageData.Deposit = ytxamt;
                        }
                        avbal = string.Format("{0}", SystemFunction.Comma(alineData["@avbal"], 12));
                        tmpPageData.Balance = avbal;
                        tmpPageData.TellerNo = alineData["@tlrno"];
                        if (!int.TryParse(alineData["@lineno"], out curLine))
                        {
                            throw new Exception("储蓄存折打印数据有问题！");
                        }
                        tmpPageData.Line = curLine;
                        tmpPageData.IsVisibility = Visibility.Visible;
                        cxcz.PageData.Add(tmpPageData);
                    }
                    return true;
                }
            }
            else if (flag == "EndPage")
            {
                //尾页数据
                var LXQD = msg.AnilyzePrintFile("LXQD|@ac_no|@name|@tx_amt|@title|@opn_date|@opn_br_no|@av_bal|@all_inter|@in_tax|@tax_rate|@tax_inter|@after_inter|@c_txamt|@zero_amt|@inst_type|@memo|@ac_seqn|@term||@brf|@ac_no1||@mtr_date||@rate|@prdt_no||||||");
                if (LXQD.Count != 0)
                {
                    var endPage = new CXCZEndPageEntity();

                    endPage.AcNo = LXQD[0]["@ac_no"];
                    endPage.AcName = LXQD[0]["@name"];
                    endPage.XHDate = LoginUserInfo.TradeDate;
                    endPage.BenJin = LXQD[0]["@tx_amt"];
                    endPage.LiXi = LXQD[0]["@all_inter"];
                    endPage.ShuiJin = LXQD[0]["@in_tax"];
                    endPage.BenXiHeJi = LXQD[0]["@c_txamt"];
                    endPage.Teller = LoginUserInfo.TellerNo;

                    cxcz.EndPageData = endPage;
                }
            }

            //解析数据错误
            if (cxcz.FirstPageData == null
                && cxcz.EndPageData == null
                && cxcz.PageData.Count == 0)
            {
                return false;
            }

            return true;
        }

        #endregion 活期存折

        #region 定期存折

        /// <summary>
        /// 定期存折数据解析
        /// </summary>
        /// <param name="dqcz">定期存折</param>
        /// <param name="msg">报文</param>
        /// <returns></returns>
        private static bool AnalyzeDQCZData(DQCZBankbook dqcz, Message msg)
        {
            dqcz.FirstPageData = null;
            dqcz.SubAcTitleList = new List<DQCZSubAcTitleEntity>();
            //首页数据
            var opna = msg.AnilyzePrintFile("OPNA|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|@term|@term_type|@rate|@tx_amt1|@email|@zy|@deac_type|@cur_name|@tx_code|@memo|@ac_no1||@ct_name|@ac_seqn|@ic_date|@prt_line|@prdt_no|@page_no|@brf|@tmp_ac_no|@tmp_tx_name|@tel|@id_name|@id_no|@br_name|@br_tele|@note_no|@zdzc|||@mbfebkno|@card_flag|@tx_br_name|@nxybrno|@tctd_flag");
            var YBTZ = msg.AnilyzePrintFile("YBTZ|@ac_seqn|@tx_date|@brf|@tmp_term|@term_type|@mtr_date|@tx_amt|@bal|@rate|@tel|@prtl|@opn_br_no|@tty|@hst_cnt|@tx_code|@cz|@zdzc|@cur_no|@ic_date");
            var YBHY = msg.AnilyzePrintFile("YBHY|@ac_seqn|@tx_date|@brf|@tmp_term|@term_type|@mtr_date|@tx_amt|@bal|@rate|@tel|@zdzc|@opn_br_no|@tty|@hst_cnt|@prt_page|@ac_name|@prdt_name|@prtl");
            var CZFY = msg.AnilyzePrintFile("CZFY|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|||@rate|@tx_amt1||@zy|@deac_type|@cur_name|@tx_code||@ac_no1|||@ac_seqn|@ic_date|@prt_line|@prdt_no|@page_no|@brf|@tmp_ac_no|@tmp_tx_name||||@br_name|@br_tele|@note_no||@accno|@zy1|@mbfebkno||@shebaono|@tctd_flag|@txbrname||@tctd|");
            //换折明细子账户
            var YBHZ = msg.AnilyzePrintFile("YBHZ|@ac_seqn|@tx_date|@brf|@tmp_term|@term_type|@mtr_date|@tx_amt|@bal|@rate|@tel|@prtl|@opn_br_no|@tty|@hst_cnt|@tx_code|@cz|@zdzc|@cur_no|@ic_date|");
            //子账户没传户名需要从首页取
            var czfYname = "";

            #region OPNA

            foreach (var item in opna)
            {

                if (Int32.Parse(item["@ac_seqn"]) >= 200)
                {
                //第一个子账户,存折头信息   判断是不是第一个子账户 第一个子账户是值为“1”，打印第一页数据  
                if (item["@ac_seqn"] == "200")
                {
                    var firstPageData = new DQCZFirstPageEntity();
                    firstPageData.AcNo = item["@ac_no"];
                    firstPageData.AcName = item["@name"];
                    firstPageData.DrawMode = item["@zy"];
                    firstPageData.OpnBr = item["@br_name"];
                    firstPageData.OpnDate = item["@opn_date"];
                    firstPageData.Teller = item["@tel"];
                    firstPageData.CzNum = item["@note_no"];
                    dqcz.FirstPageData = firstPageData;

                }
                //存单头信息
                    var SubAcTitle = new DQCZSubAcTitleEntity();
                    SubAcTitle.AcAmt = item["@tx_amt1"];
                    SubAcTitle.AcName = item["@name"];
                    SubAcTitle.AcNoSeqn = Convert.ToInt32(item["@ac_seqn"]);
                    SubAcTitle.CurNo = item["@cur_name"];
                    SubAcTitle.PrintAcSeqn = Convert.ToInt32(item["@page_no"]);
                    SubAcTitle.DualDate = item["@end_date"];
                    SubAcTitle.TxDate = item["@opn_date"];
                    SubAcTitle.PrintLine = Convert.ToInt32(item["@prt_line"]);
                    SubAcTitle.Rate = item["@rate"] + "%";
                    SubAcTitle.AutoDeposit = item["@zdzc"];
                    SubAcTitle.Term = string.IsNullOrWhiteSpace(item["@term"]) ? string.Empty : item["@term"] + item["@term_type"];
                    SubAcTitle.DualDate = item["@end_date"];
                    if (item["@prdt_no"] == "241")//定活两便
                    {
                        SubAcTitle.AutoDeposit = GANG;
                        SubAcTitle.DualDate = GANG;
                        SubAcTitle.Rate = "六折";
                        SubAcTitle.Term = GANG;
                    }
                    if (item["@title"].Contains("通知"))//通知存款不打转存标志
                    {
                        SubAcTitle.AutoDeposit = GANG;
                    }
                    SubAcTitle.SubAcData = SubAcTitle.GetNullData();

                    if (YBTZ.Count == 0 || YBTZ[0]["@tx_code"] != "2323")//如果为存单提入，不打印第一条开户记录
                    {
                        //存单数据信息
                        var dqczData = new DQCZSubAcEntity();
                        dqczData.Amt = item["@tx_amt1"];
                        dqczData.Anotherbr = LoginUserInfo.KinbrNo; //item["@opn_br_no"];//修改经办行,改为登录行号
                        dqczData.BegDate = item["@ic_date"];
                        dqczData.CurNo = item["@cur_name"];
                        dqczData.DepoistType = GetTitle(item["@title"]);
                        // dqczData.DepoistType = item["@title"];
                        dqczData.DespInfo = item["@brf"];
                        dqczData.Seqn = item["@ac_seqn"];
                        dqczData.Teller = item["@tel"];
                        dqczData.Term = string.IsNullOrWhiteSpace(item["@term"]) ? string.Empty : item["@term"] + item["@term_type"];
                        dqczData.TxDate = item["@opn_date"];
                        //定活两便
                        if (item["@prdt_no"] == "241")
                        {
                            dqczData.RateAmt = "六折";
                            dqczData.Term = "";
                        }
                        else
                        {
                            dqczData.RateAmt = item["@rate"] + "%";
                        }
                        dqczData.IsVisibility = Visibility.Visible;

                        SubAcTitle.SubAcData[0] = dqczData;
                    }

                    dqcz.SubAcTitleList.Add(SubAcTitle);
                }
            }

            #endregion OPNA

            #region CZFY

            foreach (var item in CZFY)
            {
                //第一个子账户,存折头信息
                var firstPageData = new DQCZFirstPageEntity();
                firstPageData.AcNo = item["@ac_no"];
                firstPageData.AcName = item["@name"];
                firstPageData.CzNum = item["@note_no"];
                firstPageData.DrawMode = item["@zy"] == "密" ? "密码支取" : item["@zy"];
                firstPageData.OpnBr = item["@br_name"];
                firstPageData.OpnDate = item["@opn_date"];
                firstPageData.Teller = item["@br_tele"];
                dqcz.FirstPageData = firstPageData;
                czfYname = item["@name"];
            }

            #endregion CZFY

            #region YBHZ

            //YBTZ|@ac_seqn|@tx_date|@brf|@tmp_term|@term_type|@mtr_date|@tx_amt|@bal|@rate|@tel|@prtl|@opn_br_no|@tty|@hst_cnt|@tx_code|@cz|@zdzc|@cur_no|
            foreach (var item in YBHZ)
            {
                var dqczData = new DQCZSubAcEntity
                {
                    Amt = item["@tx_amt"],
                    Anotherbr = LoginUserInfo.KinbrNo, //item["@opn_br_no"];//修改经办行,改为登录行号
                    //BegDate = item["@tx_date"],
                    CurNo = item["@cur_no"],
                    DepoistType = item["@cz"].Contains("通知存款") ? "通知" : item["@cz"].Substring(0, 2),
                    DespInfo = item["@brf"],
                    RateAmt = item["@rate"] + "%",
                    Seqn = item["@ac_seqn"],
                    Teller = item["@tel"],
                    Term = string.IsNullOrWhiteSpace(item["@tmp_term"]) ? string.Empty : item["@tmp_term"] + item["@term_type"],
                    TxDate = item["@tx_date"]
                };

                if (!string.IsNullOrWhiteSpace(item["@ic_date"]))
                {
                    dqczData.BegDate = item["@ic_date"];
                }

                if (item["@cz"] == "定活两便存款" || item["@cz"] == "定活")
                {
                    dqczData.RateAmt = "六折";
                    dqczData.Term = "";
                }
                int prtl = Convert.ToInt32(item["@prtl"]);
                dqczData.PrintNo = prtl % 7 == 0 ? 7 : prtl % 7;
                dqczData.IsVisibility = Visibility.Visible;
                var subAcTitle = dqcz.SubAcTitleList.Where(p => p.AcNoSeqn == Convert.ToInt32(item["@ac_seqn"])).ToList();
                if (subAcTitle.Count == 0)
                {
                    var subAcTitleEntity = new DQCZSubAcTitleEntity();
                    subAcTitleEntity.PrintAcSeqn = Convert.ToInt32(item["@ac_seqn"]);
                    subAcTitleEntity.SubAcData = subAcTitleEntity.GetNullData();
                    subAcTitleEntity.SubAcData[dqczData.PrintNo - 1] = dqczData;
                    subAcTitleEntity.AcName = czfYname;
                    subAcTitleEntity.AcNoSeqn = Convert.ToInt32(item["@ac_seqn"]);
                    subAcTitleEntity.CurNo = item["@cur_no"];
                    decimal dec;
                    decimal.TryParse(item["@bal"], out dec);
                    subAcTitleEntity.AcAmt = dec.ToString("N2");
                    subAcTitleEntity.AutoDeposit = item["@zdzc"];
                    subAcTitleEntity.TxDate = item["@tx_date"];

                    if (!string.IsNullOrWhiteSpace(item["@ic_date"]))
                    {
                        subAcTitleEntity.TxDate = item["@ic_date"];
                    }

                    subAcTitleEntity.DualDate = item["@mtr_date"];
                    subAcTitleEntity.Term = item["@tmp_term"] + item["@term_type"];
                    subAcTitleEntity.Rate = item["@rate"] + "%";
                    dqcz.SubAcTitleList.Add(subAcTitleEntity);
                    subAcTitleEntity.PrintLine = Convert.ToInt32(item["@prtl"]);
                    if (item["@cz"] == "定活两便存款" || item["@cz"] == "定活")
                    {
                        subAcTitleEntity.Rate = "六折";
                        subAcTitleEntity.Term = GANG;
                        subAcTitleEntity.AutoDeposit = GANG;
                        subAcTitleEntity.DualDate = GANG;
                    }
                    if (item["@cz"].Contains("通知"))//通知存款不打转存标志
                    {
                        subAcTitleEntity.AutoDeposit = GANG;
                    }
                }
                else
                {
                    if (subAcTitle.Count == 1)
                    {
                        subAcTitle[0].SubAcData[dqczData.PrintNo - 1] = dqczData;
                        subAcTitle[0].PrintLine = Convert.ToInt32(item["@prtl"]);
                    }
                    else
                    {
                        foreach (var dqczSubAcTitleEntity in subAcTitle.Where(dqczSubAcTitleEntity => dqczSubAcTitleEntity.PrintPageNo == ((Convert.ToInt32(item["@prtl"]) - 1) / 7 + 1)))
                        {
                            dqczSubAcTitleEntity.SubAcData[dqczData.PrintNo - 1] = dqczData;
                            dqczSubAcTitleEntity.PrintLine = Convert.ToInt32(item["@prtl"]);
                        }
                    }
                }
            }

            #endregion YBHZ

            #region YBHY

            //YBHY|@ac_seqn|@tx_date|@brf|@tmp_term|@term_type|@mtr_date|@tx_amt      |@bal         |@rate  |@tel     |@zdzc |@opn_br_no|@tty       |@hst_cnt      |@prt_page|@ac_name|@prdt_name
            //YBHY账户序号 |开户日期|摘要|存期     |存期单位  |到期日   |开户金额      |余额         |利率   |开户柜员|0     |开户机构  |终端号     |此账户打印行数|打印页 |户名  |
            foreach (var item in YBHY)
            {
                var subAcTitle = dqcz.SubAcTitleList.Where(p => p.AcNoSeqn == Convert.ToInt32(item["@ac_seqn"])).ToList();
                if (subAcTitle.Count != 0)
                {
                    if (subAcTitle.Count == 0)
                    {
                        subAcTitle[0].PrintLine = Convert.ToInt32(item["@prtl"]);
                        subAcTitle[0].ChangePageStr = " 转至第[" + item["@prt_page"] + "]页";
                    }
                    else
                    {
                        foreach (var dqczSubAcTitleEntity in subAcTitle.Where(dqczSubAcTitleEntity => dqczSubAcTitleEntity.PrintPageNo == ((Convert.ToInt32(item["@prtl"]) - 1) / 7 + 1)))
                        {
                            dqczSubAcTitleEntity.ChangePageStr = " 转至第[" + item["@prt_page"] + "]页";
                            dqczSubAcTitleEntity.PrintLine = Convert.ToInt32(item["@prtl"]);
                        }
                    }
                }
                else
                {
                    var subAcTitleEntity = new DQCZSubAcTitleEntity();
                    subAcTitleEntity.PrintLine = Convert.ToInt32(item["@prtl"]) - 1;//Modify By NanQi 不修改大逻辑的情况下做此修改
                    subAcTitleEntity.AcNoSeqn = Convert.ToInt32(item["@ac_seqn"]);
                    subAcTitleEntity.PrintAcSeqn = Convert.ToInt32(item["@ac_seqn"]);
                    subAcTitleEntity.SubAcData = subAcTitleEntity.GetNullData();
                    subAcTitleEntity.ChangePageStr = " 转至第[" + item["@prt_page"] + "]页";
                    dqcz.SubAcTitleList.Add(subAcTitleEntity);
                }
                var subAcTitleEntity2 = new DQCZSubAcTitleEntity();
                subAcTitleEntity2.AcName = item["@ac_name"];
                subAcTitleEntity2.AcNoSeqn = Convert.ToInt32(item["@ac_seqn"]);
                subAcTitleEntity2.CurNo = "人民币";
                subAcTitleEntity2.AcAmt = item["@tx_amt"];
                subAcTitleEntity2.AutoDeposit = item["@zdzc"];
                subAcTitleEntity2.TxDate = item["@tx_date"];
                subAcTitleEntity2.DualDate = item["@mtr_date"];
                subAcTitleEntity2.Term = item["@tmp_term"] + item["@term_type"];
                subAcTitleEntity2.Rate = item["@rate"] + "%";
                subAcTitleEntity2.PrintLine = (Convert.ToInt32(item["@prt_page"]) - 1) * 7 + 1;
                subAcTitleEntity2.SubAcData = subAcTitleEntity2.GetNullData();
                dqcz.SubAcTitleList.Add(subAcTitleEntity2);
            }

            #endregion YBHY

            #region YBTZ

            //YBTZ|@ac_seqn|@tx_date|@brf|@tmp_term|@term_type|@mtr_date|@tx_amt |@bal|@rate|@tel    |@prtl   |@opn_br_no|@tty  |@hst_cnt        |@tx_code|@cz|@zdzc|@cur_no|
            //YBTZ账户序号 |开户日期|摘要|存期     |存期单位  |到期日   |交易金额|余额|利率 |开户柜员|打印行数|开户机构  |终端号|定期主表打印行数|2104|

            int lxqdIndex = 0;

            foreach (var item in YBTZ)
            {
                var dqczData = new DQCZSubAcEntity();
                dqczData.Amt = item["@tx_amt"];
                dqczData.Anotherbr = LoginUserInfo.KinbrNo; //item["@opn_br_no"];//修改经办行,改为登录行号
                dqczData.BegDate = item["@tx_date"];
                dqczData.CurNo = item["@cur_no"];
                dqczData.DepoistType = item["@cz"].Contains("通知存款") ? "通知" : item["@cz"].Substring(0, 2);
                dqczData.DespInfo = item["@brf"];

                if (string.IsNullOrWhiteSpace(item["@ic_date"]))
                {
                    //从QKPT中取起息日
                    var QKPT = msg.AnilyzePrintFile("QKPT|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@mtr_date|@term|@term_type|@rate|@av_bal|@email|@zy|@deac_type|@cur_name|@tx_code|@memo|@ac_no1||@ct_name|@ac_seqn|@ic_date|@prt_line|@prdt_no||@brf|@tmp_ac_no|@tmp_tx_name|@tel|@id_name|@id_no|@br_name|@br_tele|@note_no||@acc_hrt");
                    if (QKPT.Count > 0)
                    {
                        if (lxqdIndex / 2 < QKPT.Count)
                        {
                            dqczData.BegDate = QKPT[lxqdIndex / 2]["@ic_date"];
                        }
                    }
                    else if (opna.Count > 0)
                    {
                        dqczData.BegDate = opna[0]["@ic_date"];
                    }
                }
                else
                {
                    dqczData.BegDate = item["@ic_date"];
                }


                //部提需要打印利息而不是利率
                var LXQD = msg.AnilyzePrintFile("LXQD|@ac_no|@name|@tx_amt|@title|@opn_date|@opn_br_no|@av_bal|@all_inter|@in_tax|@tax_rate|@tax_inter|@after_inter|@c_txamt|@zero_amt|@inst_type|@memo|@ac_seqn|@term||@brf|@ac_no1||@mtr_date||@rate|@prdt_no||||||");
                if (item["@brf"].Contains("余额"))
                {
                    if (item["@cz"] == "定活两便存款" || item["@cz"] == "定活")
                    {
                        dqczData.RateAmt = "六折";
                    }
                    else
                    {
                        dqczData.RateAmt = SystemFunction.Comma2(item["@rate"], "N6"); //利率
                        dqczData.RateAmt = string.IsNullOrWhiteSpace(dqczData.RateAmt) ? string.Empty : dqczData.RateAmt + "%";
                    }
                }
                else if (lxqdIndex < LXQD.Count)
                {
                    dqczData.RateAmt = SystemFunction.Comma2(LXQD[lxqdIndex]["@all_inter"]); //应付利息 
                    lxqdIndex++;
                }

                dqczData.Seqn = item["@ac_seqn"];
                dqczData.Teller = item["@tel"];
                dqczData.Term = string.IsNullOrWhiteSpace(item["@tmp_term"]) ? string.Empty : item["@tmp_term"] + item["@term_type"];
                dqczData.TxDate = item["@tx_date"];
                int prtl = Convert.ToInt32(item["@prtl"]);
                prtl = prtl < 1 ? 1 : prtl;//增加容错机制
                int pageNo = (prtl - 1) / 7 + 1;
                dqczData.PrintNo = prtl % 7 == 0 ? 7 : prtl % 7;
                dqczData.IsVisibility = Visibility.Visible;

                if (item["@cz"] == "定活两便存款" || item["@cz"] == "定活")
                {
                    dqczData.Term = "";
                }

                var subAcTitle2 = dqcz.SubAcTitleList.Where(p => p.PrintPageNo == pageNo).ToList();
                if (subAcTitle2.Count == 0)
                {
                    var subAcTitleEntity = new DQCZSubAcTitleEntity();
                    subAcTitleEntity.PrintAcSeqn = Convert.ToInt32(item["@ac_seqn"]);
                    subAcTitleEntity.AcNoSeqn = Convert.ToInt32(item["@ac_seqn"]);
                    subAcTitleEntity.SubAcData = subAcTitleEntity.GetNullData();
                    subAcTitleEntity.SubAcData[dqczData.PrintNo - 1] = dqczData;
                    subAcTitleEntity.PrintLine = prtl;
                    dqcz.SubAcTitleList.Add(subAcTitleEntity);
                }

                var subAcTitle = dqcz.SubAcTitleList.Where(p => p.AcNoSeqn == Convert.ToInt32(item["@ac_seqn"])).ToList();
                if (subAcTitle.Count == 0)
                {
                    var subAcTitleEntity = new DQCZSubAcTitleEntity();
                    subAcTitleEntity.PrintAcSeqn = Convert.ToInt32(item["@ac_seqn"]);
                    subAcTitleEntity.AcNoSeqn = Convert.ToInt32(item["@ac_seqn"]);
                    subAcTitleEntity.SubAcData = subAcTitleEntity.GetNullData();
                    subAcTitleEntity.SubAcData[dqczData.PrintNo - 1] = dqczData;
                    subAcTitleEntity.PrintLine = prtl;
                    dqcz.SubAcTitleList.Add(subAcTitleEntity);
                }
                else if (subAcTitle.Count == 1 && subAcTitle[0].PrintPageNo == pageNo)
                {
                    subAcTitle[0].SubAcData[dqczData.PrintNo - 1] = dqczData;
                    subAcTitle[0].PrintLine = prtl;
                }
                else
                {
                    foreach (var dqczSubAcTitleEntity in subAcTitle.Where(dqczSubAcTitleEntity => dqczSubAcTitleEntity.PrintPageNo == pageNo))
                    {
                        dqczSubAcTitleEntity.SubAcData[dqczData.PrintNo - 1] = dqczData;
                        dqczSubAcTitleEntity.PrintLine = prtl;
                    }
                }



            }

            #endregion YBTZ

            //排序子账户打印顺序
            dqcz.SubAcTitleList = dqcz.SubAcTitleList.OrderBy(x => x.PrintPageNo).ToList();
            return true;
        }

        /// <summary>
        ///
        /// </summary>
        /// <param name="msg"></param>
        /// <returns></returns>
        public static List<PrintContainer> PrintDQCZ(Message msg)
        {
            var aDQCZBankbook = DQCZBankbook.GetInstance();

            try
            {
                AnalyzeDQCZData(aDQCZBankbook, msg);
            }
            catch (Exception e)
            {
                PlatformLogger.SystemErrorInfo("打印解析出错:", e);
                MessageBox.Show("打印解析出错");
            }

            return aDQCZBankbook.PrintDQCZ();
        }

        static string GetTitle(string title)
        {
            var ret = title.Length > 4 ? title.Substring(0, 4) : title;

            switch (ret)
            {
                case "整存整取":
                    ret = "整整";
                    break;
                case "定活两便":
                    ret = "定活";
                    break;
                case "通知存款":
                    ret = "通知";
                    break;
                default:
                    break;
            }

            return ret;
        }

        #endregion 定期存折
    }
}