﻿using System.Collections.Generic;
using System.Linq;
using Platform.Print;

namespace TellerSystem.Controls.Ext.PrintsHelper.DQCZ
{
    public class CXCZBankbook
    {
        private static CXCZBankbook _instance;

        public static CXCZBankbook GetInstance()
        {
            if(_instance == null)
            {
                _instance = new CXCZBankbook();
            }
            return _instance;
        }

        /// <summary>
        /// 单页行数
        /// </summary>
        private const int PrintPageNum = 10;

        /// <summary>
        /// 折最大行
        /// </summary>
        private const int SumNum = 100;

        /// <summary>
        /// 储蓄存折首页数据
        /// </summary>
        public CXCZFirstPageEntity FirstPageData { get; set; }

        /// <summary>
        /// 储蓄存折数据页数据
        /// </summary>
        public List<CXCZPageEntity> PageData { get; set; }

        /// <summary>
        /// 储蓄存折尾页数据
        /// </summary>
        public CXCZEndPageEntity EndPageData { get; set; }

        /// <summary>
        /// 打印首页
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public List<PrintContainer> PrintFirstPage()
        {
            List<PrintContainer> retData = new List<PrintContainer>();
            if (FirstPageData == null)
                return retData;

            CXCZFirstPage cxczFirstPage = new CXCZFirstPage();
            cxczFirstPage.Description = "请打印第1页";
            cxczFirstPage.Number = FirstPageData.CZNum;
            cxczFirstPage.AccountNo = FirstPageData.AcNo;
            cxczFirstPage.AccountName = FirstPageData.AcName;
            cxczFirstPage.Currency = FirstPageData.CurNo;
            cxczFirstPage.AccountBank = FirstPageData.OpnBrname;
            cxczFirstPage.AccountDate = FirstPageData.OpnDate;
            cxczFirstPage.DepositType = FirstPageData.DepositType;
            cxczFirstPage.PasswordOrSeal = FirstPageData.DrawMode;
            cxczFirstPage.PaySysCode = FirstPageData.PaySysCode;
            cxczFirstPage.InterestRate = FirstPageData.InterestRate;


           // var tmpFpPage = new List<CXCZPageEntity>();
           // tmpFpPage.AddRange(PageData);
           // cxczFirstPage.PageData = tmpFpPage;
           // cxczFirstPage.Setvalue();

           // retData.Add(cxczFirstPage);
            retData.AddRange(PrintPage(cxczFirstPage));
            return retData;
        }

        /// <summary>
        /// 打印数据页
        /// </summary>
        /// <returns></returns>
        public List<PrintContainer> PrintPage(CXCZFirstPage firstPage = null)
        {
            var retData = new List<PrintContainer>();
            var aDemoDatas = new List<CXCZPage>();

            //是否转账有介质
            var isZZYJZ = PageData.GroupBy(page => page.AcNo).Count() > 1;

            foreach (var acList in PageData.GroupBy(page => page.AcNo))
            {
                var isInit = true;
                CXCZFirstPage aCxczFirstPage = null;
                var tmpFpPage = new List<CXCZPageEntity>();

                int subPage = 0;
                int curLine = 0;
                CXCZPage tmpSubPage = null;
                //初始化的数据
                List<CXCZPageEntity> initRows = null;

                //开户情况
                if (firstPage == null && FirstPageData != null)
                {
                    aCxczFirstPage = new CXCZFirstPage();
                    aCxczFirstPage.Description = "请打印第1页！";
                    tmpFpPage = new List<CXCZPageEntity>();
                    tmpFpPage.AddRange(acList);
                    aCxczFirstPage.PageData = tmpFpPage;
                    aCxczFirstPage.Setvalue();
                    //aCxczFirstPage.ListView7.ItemsSource = tmpFpPage;
                    retData.Add(aCxczFirstPage);
                    return retData;
                }

                //计算当前页及行数
                curLine = acList.First().Line;
                if (curLine <= PrintPageNum)
                {
                    if (firstPage == null)
                    {
                        aCxczFirstPage = new CXCZFirstPage();
                    }
                    else
                    {
                        aCxczFirstPage = firstPage;
                    }

                    string desc = string.Format("请打印第{0}页！", (subPage + 1).ToString());
                    if (isZZYJZ)
                    {
                        desc = string.Format("请打印在帐号为{0}的第1页！", acList.First().AcNo);
                    }
                    aCxczFirstPage.Description = desc;
                    subPage = 0;
                }
                else
                {
                    subPage = (curLine - PrintPageNum - 1) / (PrintPageNum * 2) + 1;
                }

                CXCZPageEntity tmpEntity = null;
                //存折上半部分数据
                var tmpPgPart1 = new List<CXCZPageEntity>();
                //存折下半部分数据
                var tmpPgPart2 = new List<CXCZPageEntity>();

                //当前页初始化部分
                initRows = new List<CXCZPageEntity>();
                if ((curLine - 1) % PrintPageNum != 0)
                {
                    if (subPage == 0)
                    {
                        for (int j = 0; j < curLine - 1; j++)
                        {
                            initRows.Add(new CXCZPageEntity());
                        }
                    }
                    else
                    {
                        for (int j = PrintPageNum + PrintPageNum * 2 * (subPage - 1); j < curLine - 1; j++)
                        {
                            initRows.Add(new CXCZPageEntity());
                        }
                    }
                }

                //不是第0页(页码为0)时,把subpage更新为前一页码
                if (subPage > 0)
                {
                    subPage--;
                }

                foreach (var alinedata in acList)
                {
                    tmpEntity = alinedata;
                    curLine = alinedata.Line;
                    //首页下半部分数据
                    if (curLine <= PrintPageNum)
                    {
                        if (isInit)
                        {
                            tmpFpPage.AddRange(initRows);
                            isInit = false;
                        }
                        tmpFpPage.Add(tmpEntity);
                    }
                    else
                    {
                        //数据页数据
                        if (subPage == 0)
                        {
                            //切第1页时
                            if (curLine / PrintPageNum > subPage)
                            {
                                tmpPgPart1 = new List<CXCZPageEntity>();
                                tmpPgPart2 = new List<CXCZPageEntity>();
                                if (isInit)
                                {
                                    int tmpTick = 0;
                                    foreach (var cxczEntity in initRows)
                                    {
                                        //奇数--对应数据页上半部分
                                        if (tmpTick < PrintPageNum)
                                        {
                                            tmpPgPart1.Add(cxczEntity);
                                        }
                                        else
                                        {
                                            //偶数--对应数据页下半部分
                                            tmpPgPart2.Add(cxczEntity);
                                        }
                                        tmpTick++;
                                    }
                                    isInit = false;
                                }
                                subPage++;
                                tmpSubPage = new CXCZPage();
                                string desc = string.Format("请打印第{0}页！", (subPage + 1).ToString());
                                if (isZZYJZ)
                                {
                                    desc = string.Format("请打印在帐号为{0}的第{1}页！", alinedata.AcNo, (subPage + 1).ToString());
                                }
                                tmpSubPage.Description = desc;
                                aDemoDatas.Add(tmpSubPage);
                            }
                        }
                        else
                        {
                            //切第2、3...页时
                            if (curLine > PrintPageNum + PrintPageNum * 2 * subPage)
                            {
                                tmpPgPart1 = new List<CXCZPageEntity>();
                                tmpPgPart2 = new List<CXCZPageEntity>();
                                if (isInit)
                                {
                                    int tmpTick = 0;
                                    foreach (var cxczEntity in initRows)
                                    {
                                        //奇数--对应数据页上半部分
                                        if (tmpTick < PrintPageNum)
                                        {
                                            tmpPgPart1.Add(cxczEntity);
                                        }
                                        else
                                        {
                                            //偶数--对应数据页下半部分
                                            tmpPgPart2.Add(cxczEntity);
                                        }
                                        tmpTick++;
                                    }
                                    isInit = false;
                                }
                                subPage++;
                                tmpSubPage = new CXCZPage();
                                string desc = string.Format("请打印第{0}页！", (subPage + 1).ToString());
                                if (isZZYJZ)
                                {
                                    desc = string.Format("请打印在帐号为{0}的第{1}页！", alinedata.AcNo, (subPage + 1).ToString());
                                }
                                tmpSubPage.Description = desc;
                                aDemoDatas.Add(tmpSubPage);
                            }
                        }

                        //奇数--对应数据页上半部分
                        if (((curLine - 1) / PrintPageNum) % 2 != 0)
                        {
                            tmpPgPart1.Add(tmpEntity);
                        }
                        else
                        {
                            //偶数--对应数据页下半部分
                            tmpPgPart2.Add(tmpEntity);
                        }

                        if (tmpSubPage != null)
                        {
                            //绑定数据页的上下部分
                            if (tmpPgPart1.Count == PrintPageNum)
                            {
                                tmpSubPage.PageData = tmpPgPart1;
                                tmpSubPage.Setvalueup();
                            }
                            if (tmpPgPart2.Count == PrintPageNum)
                            {
                                tmpSubPage.PageData2 = tmpPgPart2;
                                tmpSubPage.Setvaluedown();
                            }

                            //if (tmpSubPage.ListView7.ItemsSource == null && tmpPgPart1.Count == PrintPageNum)
                            //{
                            //    tmpSubPage.ListView7.ItemsSource = tmpPgPart1;
                            //}
                            //if (tmpSubPage.listView1.ItemsSource == null && tmpPgPart2.Count == PrintPageNum)
                            //{
                            //    tmpSubPage.listView1.ItemsSource = tmpPgPart2;
                            //}
                        }
                    }
                }

                //绑定数据页的上下部分
                if (tmpSubPage != null)
                {
                    if (tmpPgPart1.Count < PrintPageNum)
                    {
                        tmpSubPage.PageData = tmpPgPart1;
                        tmpSubPage.Setvalueup();
                        // tmpSubPage.ListView7.ItemsSource = tmpPgPart1;
                    }
                    if (tmpPgPart2.Count < PrintPageNum)
                    {
                        tmpSubPage.PageData2 = tmpPgPart2;
                        tmpSubPage.Setvaluedown();
                        // tmpSubPage.listView1.ItemsSource = tmpPgPart2;
                    }
                }

                //绑定首页数据
                if (tmpFpPage.Count != 0)
                {
                    if (aCxczFirstPage != null)
                    {
                        aCxczFirstPage.PageData = tmpFpPage;
                        aCxczFirstPage.Setvalue();
                        //aCxczFirstPage.ListView7.ItemsSource = tmpFpPage;
                        retData.Add(aCxczFirstPage);
                    }
                }
            }
            
            retData.AddRange(aDemoDatas);
            return retData;
        }

        /// <summary>
        /// 打印储蓄存折尾页
        /// </summary>
        /// <returns></returns>
        public List<PrintContainer> PrintEndPage()
        {
            List<PrintContainer> retData = new List<PrintContainer>();

            if (EndPageData == null)
                return retData;
            CXCZEndPage aCXCZEndPage = new CXCZEndPage();
            aCXCZEndPage.Description = "请打印最后一页！";
            aCXCZEndPage.AcName = EndPageData.AcName;
            aCXCZEndPage.AcNo = EndPageData.AcNo;
            aCXCZEndPage.BenJin = EndPageData.BenJin;
            aCXCZEndPage.BenXiHeJi = EndPageData.BenXiHeJi;
            aCXCZEndPage.LiXi = EndPageData.LiXi;
            aCXCZEndPage.ShuiJin = EndPageData.ShuiJin;
            aCXCZEndPage.Teller = EndPageData.Teller;
            aCXCZEndPage.XHDate = EndPageData.XHDate;
            retData.Add(aCXCZEndPage);
            return retData;
        }

    }
}
