﻿using System.Windows;
using System.Collections.Generic;

namespace TellerSystem.Controls.Ext.PrintsHelper.DQCZ
{
    /// <summary>
    /// 定期存折单行数据实体
    /// </summary>
    public class DQCZSubAcEntity
    {
        /// <summary>
        /// 打印序号
        /// </summary>
        public int PrintNo { get; set; }

        /// <summary>
        /// 序号
        /// </summary>
        public string Seqn { get; set; }

        /// <summary>
        /// 交易日期
        /// </summary>
        public string TxDate { get; set; }

        /// <summary>
        /// 币种
        /// </summary>
        public string CurNo { get; set; }

        /// <summary>
        /// 摘要
        /// </summary>
        public string DespInfo { get; set; }

        /// <summary>
        /// 起息日
        /// </summary>
        public string BegDate { get; set; }

        /// <summary>
        /// 储种
        /// </summary>
        public string DepoistType { get; set; }

        /// <summary>
        /// 存期
        /// </summary>
        public string Term { get; set; }

        private string _Amt = string.Empty;
        /// <summary>
        /// 金额
        /// </summary>
        public string Amt
        {
            get
            {
                double amt = 0;
                if (double.TryParse(_Amt, out amt))
                {
                    return amt.ToString("N2");
                }
                else
                {
                    return _Amt;
                }
            }
            set
            {
                _Amt = value;
            }
        }

        /// <summary>
        /// 利息
        /// </summary>
        public string RateAmt { get; set; }

        /// <summary>
        /// 操作员
        /// </summary>
        public string Teller { get; set; }

        /// <summary>
        /// 经办机构
        /// </summary>
        public string Anotherbr { get; set; }

        /// <summary>
        /// 是否显示
        /// </summary>
        public Visibility IsVisibility { get; set; }

    }

    /// <summary>
    /// 子账户实体
    /// </summary>
    public class DQCZSubAcTitleEntity
    {
        /// <summary>
        /// 账号序号
        /// </summary>
        public int AcNoSeqn { get; set; }

        public string CurNo { get; set; }

        /// <summary>
        /// 第几个子账户
        /// </summary>
        public int PrintAcSeqn
        {
            get;
            set;
        }

        ///// <summary>
        ///// 第几页
        ///// </summary>
        //public int PrintPageNo
        //{
        //    get { return PrintAcSeqn / 2 + 1; }
        //}

        /// <summary>
        /// 打印行数
        /// </summary>
        public int PrintLine
        {
            get;
            set;
        }

        /// <summary>
        /// 第几页
        /// </summary>
        public int PrintPageNo
        {
            get { return (PrintLine - 1) / 7 + 1; }
        }

        ///// <summary>
        ///// 0表示上半页，1表示下半页
        ///// </summary>
        //public int UpDownPage
        //{
        //    get { return PrintAcSeqn % 2; }
        //}

        /// <summary>
        /// 户名
        /// </summary>
        public string AcName { get; set; }

        private string _AcAmt = string.Empty;
        /// <summary>
        /// 金额
        /// </summary>
        public string AcAmt
        {
            get
            {
                double amt = 0;
                if (double.TryParse(_AcAmt, out amt))
                {
                    return amt.ToString("N2");
                }
                else
                {
                    return _AcAmt;
                }
            }
            set
            {
                _AcAmt = value;
            }
        }

        /// <summary>
        /// 交易日期
        /// </summary>
        public string TxDate { get; set; }

        /// <summary>
        /// 到期日期
        /// </summary>
        public string DualDate { get; set; }

        /// <summary>
        /// 存期
        /// </summary>
        public string Term { get; set; }

        /// <summary>
        /// 利率
        /// </summary>
        public string Rate { get; set; }

        /// <summary>
        /// 自动转存
        /// </summary>
        public string AutoDeposit { get; set; }

        /// <summary>
        /// 自动转存
        /// </summary>
        public string ChangePageStr { get; set; }

        /// <summary>
        /// 子账户数据
        /// </summary>
        public List<DQCZSubAcEntity> SubAcData { get; set; }

        public List<DQCZSubAcEntity> GetNullData()
        {
            List<DQCZSubAcEntity> list = new List<DQCZSubAcEntity>();
            for (int i = 0; i < 7; i++)
            {
                list.Add(new DQCZSubAcEntity());
            }
            return list;
        }
    }

    /// <summary>
    /// 定期存折首页
    /// </summary>
    public class DQCZFirstPageEntity
    {
        /// <summary>
        /// 账号
        /// </summary>
        public string AcNo { get; set; }

        /// <summary>
        /// 存折号码
        /// </summary>
        public string CzNum { get; set; }

        /// <summary>
        /// 户名
        /// </summary>
        public string AcName { get; set; }

        /// <summary>
        ///  支取方式
        /// </summary>
        public string DrawMode { get; set; }

        /// <summary>
        /// 经办人员
        /// </summary>
        public string Teller { get; set; }

        /// <summary>
        /// 签发日期
        /// </summary>
        public string OpnDate { get; set; }

        /// <summary>
        /// 开户网点
        /// </summary>
        public string OpnBr { get; set; }
    }
}
