﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// BankBookPage.xaml 的交互逻辑
    /// </summary>
    public partial class LoanCardBook : UserControl
    {
        public LoanCardBook()
        {
            InitializeComponent();
            MaxLineCount = 12;

            Data = new DataTable();
            Data.Columns.Add("Date"); //日期
            Data.Columns.Add("AccountLoanNo");//贷款账号
            Data.Columns.Add("XH");//方式
            Data.Columns.Add("EndDate");//到期日
            Data.Columns.Add("Summary");//交易金额
            Data.Columns.Add("LV");//利率
            Data.Columns.Add("Balance");//剩余额度
            if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
            {
                for (int i = 0; i < MaxLineCount; i++)
                {
                    Data.Rows.Add("Date", "AccountLoanNo", "XH", "EndDate", "Summary", "LV","Balance");
                }
            }
            IC_Page.ItemsSource = Data.DefaultView;
        }

        public DataTable Data { get; private set; }

        /// <summary>
        /// 最大的打印行数
        /// </summary>
        public int MaxLineCount { get; set; }

        /// <summary>
        /// 加载数据，返回值为实际处理的数据条目
        /// </summary>
        /// <param name="data"></param>
        /// <param name="startLineNo"></param>
        /// <returns></returns>
        public int LoadData(IEnumerable<BankBookPageLine> data, int startLineNo)
        {
            if (startLineNo < 0 || startLineNo >= MaxLineCount)
                throw new ArgumentOutOfRangeException("startLineNo不能小于0");
            Data.Rows.Clear();

            for (int i = 0; i < startLineNo; i++)
            {
                Data.Rows.Add("", "", "", "", "", "", "", "");
            }
            var count = 0;
            foreach (var item in data)
            {
                count++;
                Data.Rows.Add(item.Date == null ? string.Empty : item.Date.Value.ToString("yyyyMMdd"),
                    item.AccountLoanNo,
                    item.XH,
                    item.EndDate,
                    item.Summary,
                    item.LV,
                    item.Balance
                    );
                if (startLineNo + count >= MaxLineCount) break;
            }
            return count;
        }

        public void LoadOldData(IEnumerable<SDDKEndPageEntity> data)
        {
            //string s = string.Empty;
            var ss = new BankBookPageLine();
            var newData = data.Select(x =>
            {
                var ret = new BankBookPageLine();
                if (!string.IsNullOrEmpty(x.AccountLoanNo))
                {
                    ret.Date = DateTime.ParseExact(x.Date, "yyyyMMdd", null);
                    ret.AccountLoanNo = x.AccountLoanNo;
                    ret.XH = x.XH;
                    ret.EndDate = x.EndDate;
                    ret.LV = x.LV;
                    ret.Summary = x.Summary;
                    ret.Balance = x.Balance;
                }
                ss = ret;
                return ret;
            });
            var tt = ss;
            LoadData(newData, 0);
        }

        /// <summary>
        /// 存折行数据
        /// </summary>
        public class BankBookPageLine
        {
            /// <summary>
            /// 发生日期
            /// </summary>
            public DateTime? Date { get; set; }
            /// <summary>
            /// 贷款账号
            /// </summary>
            public string AccountLoanNo { get; set; }
            /// <summary>   
            /// 笔序号
            /// </summary>
            public string XH { get; set; }
            /// <summary>
            /// 到期日
            /// </summary>
            public string EndDate { get; set; }
            /// <summary>
            /// 交易金额
            /// </summary>
            public string Summary { get; set; }
            /// <summary>
            /// 利率
            /// </summary>
            public string LV { get; set; }
           
            /// <summary>
            /// 剩余余额
            /// </summary>
            public string Balance { get; set; }
        }
    }
}
