﻿using System.Collections.Generic;
using Platform.Print;
using TellerSystem.PrintTemplate.Bankbook;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    public class SDDKZBankbook
    {
        private static SDDKZBankbook _instance;

        public static SDDKZBankbook GetInstance()
        {
            if (_instance == null)
            {
                _instance = new SDDKZBankbook();
            }
            return _instance;
        }

        /// <summary>
        /// 单页行数
        /// </summary>
        private const int PrintPageNum = 12;

        private int _sumNum = 24;
        /// <summary>
        /// 折最大行
        /// </summary>
        public int SumNum
        {
            get { return _sumNum; }
            set { _sumNum = value; }
        }

        /// <summary>
        /// 贷款证首页数据
        /// </summary>
        public SDDKFirstPageEntity FirstPageData { get; set; }

        /// <summary>
        /// 贷款证中间页 
        /// </summary>
        public SDDKPageEntity PageData { get; set; }

        /// <summary>
        /// 贷款证尾页
        /// </summary>
        public List<SDDKEndPageEntity> EndPageData { get; set; }

        /// <summary>
        /// 打印数据页，替代PrintPage方法
        /// </summary>
        public List<PrintContainer> PrintBankBookPage()
        {
            var retData = new List<PrintContainer>();
            List<SDDKEndPageEntity> tempDatas = new List<SDDKEndPageEntity>();
                //总行数
            int pageTatolLine = PrintPageNum * 2;
            //本次要打印的总行数
            int account = 0;
            //当前页要打印的行数，从1开始
            int currentPrintLine = 0;
            if (EndPageData.Count == 0)
            {
                return retData;
            }
            if (EndPageData != null)
            {
                account = EndPageData.Count;
                //山东贷款证无需换页,只一页，直接换证
                var tmp = EndPageData[0].Line % pageTatolLine;
                currentPrintLine = tmp == 0 ? EndPageData[0].Line : tmp;
               
                    LoanCardBook_贷款尾页 last = new LoanCardBook_贷款尾页(new LoanCardBook_贷款尾页.LoanCardBook_贷款尾页Entry());
                    tempDatas.Clear();
                    if (account >= SumNum - EndPageData[0].Line + 1)
                    {
                        //贷款证存在自动还本且不打印的情况，但核心记录行数依次加1。
                        //解决核心记录行数大于24，而当前打印行数小于24的情况。
                        if (EndPageData[0].Line > 24)
                        {
                            for (int i = 1; i < 24; i++)
                            {
                                tempDatas.Add(new SDDKEndPageEntity());
                            }
                            tempDatas.AddRange(EndPageData.GetRange(0, 1));
                        }
                        else
                        {
                            for (int i = 1; i < currentPrintLine; i++)
                            {
                                tempDatas.Add(new SDDKEndPageEntity());
                            }
                            tempDatas.AddRange(EndPageData.GetRange(0, SumNum - EndPageData[0].Line + 1));
                        }
                        //Todo提示换证  
                        last.Description = "请换折";
                    }
                    else
                    {
                        for (int i = 1; i < currentPrintLine; i++)
                        {
                            tempDatas.Add(new SDDKEndPageEntity());
                        }
                        tempDatas.AddRange(EndPageData.GetRange(0, account));
                    }              
                    last.PageData3 = tempDatas;
                    last.SetValue();
                    retData.Add(last);
               
            }

            return retData;
        }
    }
}
