﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// BankBookPage.xaml 的交互逻辑
    /// </summary>
    public partial class BBP_YBZCZ : UserControl
    {
        public BBP_YBZCZ()
        {
            InitializeComponent();

            Data = new DataTable();
            Data.Columns.Add("Number");
            Data.Columns.Add("Date");
            Data.Columns.Add("Seqn");
            Data.Columns.Add("Summary");
            Data.Columns.Add("DepositTerm");
            Data.Columns.Add("Amount");
            Data.Columns.Add("Balance");
            Data.Columns.Add("InterestRate");
            Data.Columns.Add("DateEnd");
            Data.Columns.Add("Remark");
            
            if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
            {
                for (int i = 0; i < MaxLineCount; i++)
                {
                    Data.Rows.Add("Number", "Date", "Seqn", "Summary", "DepositTerm", "Amount", "Balance", "InterestRate", "DateEnd", "Remark");
                }
            }
            IC_Page.ItemsSource = Data.DefaultView;
        }

        public DataTable Data { get; private set; }

        /// <summary>
        /// 最大的打印行数
        /// </summary>
        public const int MaxLineCount = 12;

        /// <summary>
        /// 加载数据，返回值为实际处理的数据条目
        /// </summary>
        /// <param name="data"></param>
        /// <param name="startLineNo"></param>
        /// <returns></returns>
        public void LoadData(IEnumerable<BankBookPageLine> data)
        {
            Data.Rows.Clear();
            for (int i = 1; i < MaxLineCount; i++)
            {
                var line = data.FirstOrDefault(x => (x.Number) % MaxLineCount == i );
                if (line == null)
                    Data.Rows.Add("", "", "", "", "", "", "", "", "", "");
                else
                    Data.Rows.Add((line.Number).ToString(),
                        line.Date == null ? string.Empty : line.Date.Value.ToString("yyyyMMdd"),
                        line.Seqn,
                        line.Summary,
                        line.DepositTerm,
                        line.Amount == null ? string.Empty :GetPrefixBySummary(line.Summary.Trim()) +string.Format("{0:N2}", line.Amount),
                        line.Balance == null ? string.Empty :string.Format("{0:N2}", line.Balance),
                        line.InterestRate == null ? string.Empty : line.InterestRate.ToString(),
                        line.DateEnd == null ? string.Empty : line.DateEnd.Value.ToString("yyyyMMdd"),
                        line.Remark);
            }
            var EndLine = data.FirstOrDefault(x => (x.Number) % MaxLineCount == 0);
            if (EndLine != null)
            {
                Data.Rows.Add((EndLine.Number).ToString(),
                        EndLine.Date == null ? string.Empty : EndLine.Date.Value.ToString("yyyyMMdd"),
                        EndLine.Seqn,
                        EndLine.Summary,
                        EndLine.DepositTerm,
                        EndLine.Amount == null ? string.Empty : GetPrefixBySummary(EndLine.Summary.Trim()) + string.Format("{0:N2}", EndLine.Amount),
                        EndLine.Balance == null ? string.Empty : string.Format("{0:N2}", EndLine.Balance),
                        EndLine.InterestRate == null ? string.Empty : EndLine.InterestRate.ToString(),
                        EndLine.DateEnd == null ? string.Empty : EndLine.DateEnd.Value.ToString("yyyyMMdd"),
                        EndLine.Remark);
            }
        }

        //private string FormatType(string type)
        //{
        //    if (type == "整存整取")
        //        return "整整";
        //    if (type == "定活两便")
        //        return "定活";
        //    if (type == "通知存款")
        //        return "通知";
        //    if (type.Length > 2)
        //        return type.Substring(0, 2);
        //    return type;
        //}

        private string GetPrefixBySummary(string summary)
        {
            switch (summary)
            {
                case "开户":
                case "提入":
                case "换证":
                    return "+";
                case "部提":
                case "提出":
                case "销户":
                    return "-";
                default:
                    return string.Empty;
            }
        }

        /// <summary>
        /// 存折行数据
        /// </summary>
        public class BankBookPageLine
        {
            /// <summary>
            /// 打印行号，从1开始
            /// </summary>
            public int Number { get; set; }
            /// <summary>
            /// 序号
            /// </summary>
            public string Seqn { get; set; }
            /// <summary>
            /// 发生日期
            /// </summary>
            public DateTime? Date { get; set; }
            /// <summary>
            /// 摘要
            /// </summary>
            public string Summary { get; set; }
            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }
            /// <summary>
            /// 到期日
            /// </summary>
            public DateTime? DateEnd { get; set; }
            /// <summary>
            /// 利率
            /// </summary>
            public decimal? InterestRate { get; set; }
            /// <summary>
            /// 本金
            /// </summary>
            public decimal? Amount { get; set; }
            /// <summary>
            /// 余额
            /// </summary>
            public decimal? Balance { get; set; }
            /// <summary>
            /// 操作员
            /// </summary>
            public string Remark { get; set; }
        }
    }
}
