﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// BankBookPage.xaml 的交互逻辑
    /// </summary>
    public partial class QF_YBTCZ : UserControl
    {
        public QF_YBTCZ()
        {
            InitializeComponent();
            Data = new DataTable();
            Data.Columns.Add("Date");
            Data.Columns.Add("Summary");
            Data.Columns.Add("XH");//序号
            Data.Columns.Add("CQ");//存期
            Data.Columns.Add("LV");//利率
            Data.Columns.Add("DepWinhType");
            Data.Columns.Add("Balance");
            Data.Columns.Add("Remark");
            if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
            {
                for (int i = 0; i < MaxLineCount; i++)
                {
                    Data.Rows.Add("Date", "Summary", "XH", "CQ", "LV", "DepWinhType", "Balance", "Remark");
                }
            }
            IC_Page.ItemsSource = Data.DefaultView;
        }

        public DataTable Data { get; private set; }

        /// <summary>
        /// 最大的打印行数
        /// </summary>
        public const int MaxLineCount = 12;

        /// <summary>
        /// 加载数据，返回值为实际处理的数据条目
        /// </summary>
        /// <param name="data"></param>
        /// <param name="startLineNo"></param>
        /// <returns></returns>
        public void LoadData(IEnumerable<BankBookPageLine> data)
        {
            Data.Rows.Clear();
            for (int i = 1; i < MaxLineCount; i++)
            {
                var line = data.FirstOrDefault(x => (x.Number) % MaxLineCount == i );
                if (line == null)
                    Data.Rows.Add("", "", "", "", "", "", "", "");
                else
                    Data.Rows.Add(
                        line.Date == null ? string.Empty : line.Date.Value.ToString("yyyyMMdd"),
                        line.Summary,
                        line.Seqn,
                        line.DepositTerm,
                        line.InterestRate == null ? string.Empty : string.Format("{0:N3}", line.InterestRate),
                        line.Amount == null ? string.Empty :GetPrefixBySummary(line.Summary.Trim()) +string.Format("{0:N2}", line.Amount),
                        line.Balance == null ? string.Empty :string.Format("{0:N2}", line.Balance),
                        line.Remark);
            }
            var EndLine = data.FirstOrDefault(x => (x.Number) % MaxLineCount == 0);
            if (EndLine != null)
            {
                Data.Rows.Add(
                        EndLine.Date == null ? string.Empty : EndLine.Date.Value.ToString("yyyyMMdd"),
                        EndLine.Summary,
                        EndLine.Seqn,
                        EndLine.DepositTerm,
                        EndLine.InterestRate == null ? string.Empty : string.Format("{0:N3}", EndLine.InterestRate),
                        EndLine.Amount == null ? string.Empty : GetPrefixBySummary(EndLine.Summary.Trim()) + string.Format("{0:N2}", EndLine.Amount),
                        EndLine.Balance == null ? string.Empty : string.Format("{0:N2}", EndLine.Balance),
                        EndLine.Remark);
            }
        }

        //private string FormatType(string type)
        //{
        //    if (type == "整存整取")
        //        return "整整";
        //    if (type == "定活两便")
        //        return "定活";
        //    if (type == "通知存款")
        //        return "通知";
        //    if (type.Length > 2)
        //        return type.Substring(0, 2);
        //    return type;
        //}

        private string GetPrefixBySummary(string summary)
        {
            string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            if (FLAG == "V19")
            {
                switch (summary)
                {
                    case "开户":
                    case "提入":
                    case "换证":
                    case "换折":
                        return "+";
                    case "部提":
                    case "提出":
                    case "销户":
                        return "-";
                    default:
                        return string.Empty;
                }
            }
            else
            {
                switch (summary)
                {
                    case "开户":
                    case "提入":
                    case "换证":
                        return "+";
                    case "部提":
                    case "提出":
                    case "销户":
                        return "-";
                    default:
                        return string.Empty;
                }
            }           
        }

        /// <summary>
        /// 存折行数据
        /// </summary>
        public class BankBookPageLine
        {
            /// <summary>
            /// 打印行号，从1开始
            /// </summary>
            public int Number { get; set; }
            /// <summary>
            /// 序号
            /// </summary>
            public string Seqn { get; set; }
            /// <summary>
            /// 发生日期
            /// </summary>
            public DateTime? Date { get; set; }
            /// <summary>
            /// 摘要
            /// </summary>
            public string Summary { get; set; }
            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }
            /// <summary>
            /// 利率
            /// </summary>
            public decimal? InterestRate { get; set; }
            /// <summary>
            /// 本金
            /// </summary>
            public decimal? Amount { get; set; }
            /// <summary>
            /// 余额
            /// </summary>
            public decimal? Balance { get; set; }
            /// <summary>
            /// 操作员
            /// </summary>
            public string Remark { get; set; }
        }
    }
}
