﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Platform.Common.RunningParameters;
using Platform.Print;
using TellerSystem.Communication;
using TellerSystem.Library.Ext.Function;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    public class SDXX_YBTCZ
    {
        #region Ctor

        public SDXX_YBTCZ(Message data)
        {
            PrintList = new List<PrintContainer>();
            Data = data;
            InitData();
        }

        /// <summary>
        /// 数据
        /// </summary>
        public Message Data { get; private set; }

        #endregion Ctor

        #region Method

        #endregion Method

        #region Internal

        public List<PrintContainer> PrintList { get; private set; }

        private void InitData()
        {
            /* 规则
             * OPNA：独立出现，内部包含一条明细。当账户序号为1时，为首次开存折，需要打印首页
             * YBTZ：独立出现，当部提或销户时，出现1条
             * CZFY：换折数据，仅包含首页信息。与YBHZ、YBTZ联合使用
             * YBHZ：换折时需要打印的数据
             * 注意---OPNA、CZFY解析串一致，YBTZ、YBHZ解析串一致
             * */
            PrintList.Clear();
            //序号：ac_seqn 日期：ic_date 摘要：memo 种类：cpfl 存期：term+term_type 到期日：end_date 利率：rate 本金：tx_amt1 利息：  操作员：当前操作员
            var opnaList = Data.AnilyzePrintFile("OPNA|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|@term|@term_type|@rate|@tx_amt1|@email|@zy|@deac_type|@cur_name|@tx_code|@memo|@ac_no1|@lx|@ct_name|@ac_seqn|@ic_date|@prt_line|@prdt_no|@page_no|@brf|@tmp_ac_no|@tmp_tx_name|@tel|@id_name|@id_no|@br_name|@br_tele|@note_no|@zdzc|||@mbfebkno|@card_flag|@tx_br_name|@nxybrno|@tctd_flag|@cpfl|");
            //YBTZ账户序号 |开户日期|摘要|存期     |存期单位  |到期日   |交易金额|余额|利率 |开户柜员|打印行数|开户机构  |终端号|定期主表打印行数|2104|
            var ybtzList = Data.AnilyzePrintFile("YBTZ|@ac_seqn|@tx_date|@brf|@tmp_term|@term_type|@mtr_date|@tx_amt|@bal|@rate|@tel|@prtl|@opn_br_no|@tty|@hst_cnt|@tx_code|@cz|@zdzc|@cur_no|@ic_date|@cpfl|@lixi|");
            var czfyList = Data.AnilyzePrintFile("CZFY|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|@term|@term_type|@rate|@tx_amt1|@email|@zy|@deac_type|@cur_name|@tx_code|@memo|@ac_no1|@lx|@ct_name|@ac_seqn|@ic_date|@prt_line|@prdt_no|@page_no|@brf|@tmp_ac_no|@tmp_tx_name|@tel|@id_name|@id_no|@br_name|@br_tele|@note_no||@accno|@my|@mbfebkno||@shebaono|@tctd_flag|@txbrname||@tctd_flag|@cpfl");
            var ybhzList = Data.AnilyzePrintFile("YBHZ|@ac_seqn|@tx_date|@brf|@tmp_term|@term_type|@mtr_date|@tx_amt|@bal|@rate|@tel|@prtl|@opn_br_no|@tty|@hst_cnt|@tx_code|@cz|@zdzc|@cur_no|@ic_date|@cpfl|@lixi|");
            //明细条目
            var lines = new List<XX_YBTCZ.BankBookPageLine>();
            //检查OPNA标签
            if (opnaList.Count > 0 && ybtzList.Count == 0)
            {
                //若账户序号为200，则代表新开存折，需打印首页
                var line = GetPageLine(opnaList[0], "OPNA");
                if (opnaList[0]["@ac_seqn"] == "200")
                {
                    //创建存折首页
                    var entry = new XX_YBZCZ_FirstPage.XX_YBZCZ_FirstPageEntry();
                    entry.HM = opnaList[0]["@name"];
                    entry.ZH = opnaList[0]["@ac_no"];
                    entry.Kinbro = opnaList[0]["@br_name"];
                    entry.QFRQ = opnaList[0]["@opn_date"];
                    entry.TCTD = opnaList[0]["@tctd_flag"];
                    entry.ZFTJ = opnaList[0]["@zy"];
                    var p = new XX_YBZCZ_FirstPage(entry);
                    //首条信息必定打印在首页
                    p.SetLines(new List<XX_YBTCZ.BankBookPageLine> { line });
                    PrintList.Add(p);
                    return;
                }
                //非开立折，纳入明细中
                lines.Add(line);
            }
            //解析YBTZ、TBHZ明细
            lines.AddRange(ybtzList.Select(x => GetPageLine(x, "YBTZ")));
            lines.AddRange(ybhzList.Select(x => GetPageLine(x, "YBHZ")));
            if (czfyList.Count > 0)
            {
                //创建存折首页
                var entry = new XX_YBZCZ_FirstPage.XX_YBZCZ_FirstPageEntry();
                entry.HM = czfyList[0]["@name"];
                entry.ZH = czfyList[0]["@ac_no"];
                entry.Kinbro = czfyList[0]["@opn_br_no"];
                entry.QFRQ = czfyList[0]["@opn_date"];
                entry.TCTD = czfyList[0]["@tctd_flag"];
                var p = new XX_YBZCZ_FirstPage(entry);
                //筛选打印在首页的数据，与首页一起参与打印
                var mx = lines.Where(x => { return x.Number < XX_YBTCZ.MaxLineCount; }).ToList();
                p.SetLines(mx);
                PrintList.Add(p);
                mx.ForEach(x => lines.Remove(x));
            }
            //打印其他页的数据
            PrintList.AddRange(GetPrints(lines));
        }

        /// <summary>
        /// 创建打印行对象
        /// </summary>
        /// <param name="dict"></param>
        /// <param name="type"></param>
        /// <returns></returns>
        private XX_YBTCZ.BankBookPageLine GetPageLine(Dictionary<string, string> dict, string type)
        {
            switch (type)
            {
                case "OPNA":
                    return new XX_YBTCZ.BankBookPageLine
                    {
                        Number = (int)(dict["@page_no"].ToDecimal() - 1) * 3 + 1,
                        ZZH = dict["@ac_seqn"],
                        Date = dict["@ic_date"].ToDateTime(),
                        Summary = dict["@memo"],
                        DepositTerm = dict["@term"] + dict["@term_type"],
                        QXRQ = dict["@opn_date"],
                        DQRQ = dict["@end_date"],
                        BZ = "人民币",
                        Amt = dict["@tx_amt1"].ToDecimal(),
                        InterestRate = dict["@rate"].ToDecimal(),
                        YQ = "",
                        Kinbr = LoginUserInfo.KinbrNo,
                        Teller = LoginUserInfo.TellerNo
                    };
                case "YBTZ":
                    return new XX_YBTCZ.BankBookPageLine
                    {
                        Number=(int)(dict["@prtl"].ToDecimal()/10*3+(dict["@prtl"].ToDecimal()%10)),
                        ZZH = dict["@ac_seqn"],
                        Date = dict["@tx_date"].ToDateTime(),
                        Summary = dict["@brf"],
                        DepositTerm = dict["@tmp_term"] + dict["@term_type"],
                        QXRQ = dict["@mtr_date"],
                        DQRQ = "",
                        BZ = "人民币",
                        Amt = dict["@tx_amt"].ToDecimal(),
                        InterestRate = dict["@rate"].ToDecimal(),
                        YQ = "",
                        Kinbr = LoginUserInfo.KinbrNo,
                        Teller = dict["@tel"]
                    };
                case "YBHZ":
                    return new XX_YBTCZ.BankBookPageLine
                    {
                        Number = (int)(dict["@prtl"].ToDecimal() / 10 * 3 + (dict["@prtl"].ToDecimal() % 10)),
                        ZZH = dict["@ac_seqn"],
                        Date = dict["@tx_date"].ToDateTime(),
                        Summary = dict["@brf"],
                        DepositTerm = dict["@tmp_term"] + dict["@term_type"],
                        QXRQ = dict["@mtr_date"],
                        DQRQ = "",
                        BZ = "人民币",
                        Amt = dict["@tx_amt"].ToDecimal(),
                        InterestRate = dict["@rate"].ToDecimal(),
                        YQ = "",
                        Kinbr = LoginUserInfo.KinbrNo,
                        Teller = dict["@tel"]
                    };
                default:
                    break;
            }

            return null;
        }

        /// <summary>
        /// 创建打印对象
        /// </summary>
        /// <param name="lines"></param>
        /// <returns></returns>
        private List<PrintContainer> GetPrints(List<XX_YBTCZ.BankBookPageLine> lines)
        {
            var ret = new List<PrintContainer>();
            //计算
            var first = lines.Where(x => { return x.Number <= XX_YBTCZ.MaxLineCount; }).ToList();
            if (first.Count > 0)
            {
                var p = new XX_YBZCZ_FirstPage(new XX_YBZCZ_FirstPage.XX_YBZCZ_FirstPageEntry());
                p.SetLines(first);
                ret.Add(p);
            }
            var page1 = lines.Where(x => { return x.Number > XX_YBTCZ.MaxLineCount && x.Number <= XX_YBTCZ.MaxLineCount * 3; }).ToList();
            if (page1.Count > 0)
            {
                var p = new XX_YBZCZ_Page(new XX_YBZCZ_Page.XX_YBZCZ_PageEntry());
                p.SetLines(page1);
                p.Description = "请翻到存折第2页进行打印";
                ret.Add(p);
            }
            var page2 = lines.Where(x => { return x.Number > XX_YBTCZ.MaxLineCount * 3 && x.Number <= XX_YBTCZ.MaxLineCount * 5; }).ToList();
            if (page2.Count > 0)
            {
                var p = new XX_YBZCZ_Page(new XX_YBZCZ_Page.XX_YBZCZ_PageEntry());
                p.SetLines(page2);
                p.Description = "请翻到存折第3页进行打印";
                ret.Add(p);
            }
            var page3 = lines.Where(x => { return x.Number > XX_YBTCZ.MaxLineCount * 5 && x.Number <= XX_YBTCZ.MaxLineCount * 7; }).ToList();
            if (page3.Count > 0)
            {
                var p = new XX_YBZCZ_Page(new XX_YBZCZ_Page.XX_YBZCZ_PageEntry());
                p.SetLines(page3);
                p.Description = "请翻到存折第4页进行打印";
                ret.Add(p);
            }
            var last = lines.Where(x => { return x.Number > XX_YBTCZ.MaxLineCount * 7; }).ToList();
            if (last.Count > 0)
            {
                var p = new XX_YBZCZ_LastPage(new XX_YBZCZ_LastPage.XX_YBZCZ_LastPageEntry());
                p.SetLines(last);
                ret.Add(p);
            }
            return ret;
        }

        #endregion Internal
    }
}