﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// BankBookPage.xaml 的交互逻辑
    /// </summary>
    public partial class XX_YBTCZ : UserControl
    {
        public XX_YBTCZ()
        {
            InitializeComponent();
            Data = new DataTable();
            Data.Columns.Add("ZZH");
            Data.Columns.Add("Date");
            Data.Columns.Add("Summary");
            Data.Columns.Add("CQ");//存期
            Data.Columns.Add("QXRQ");//利率
            Data.Columns.Add("DQRQ");
            Data.Columns.Add("BZ");
            Data.Columns.Add("Amt");
            Data.Columns.Add("LV");
            Data.Columns.Add("YQ");
            Data.Columns.Add("Kinbr");
            Data.Columns.Add("Teller");
            if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
            {
                for (int i = 0; i < MaxLineCount; i++)
                {
                    Data.Rows.Add("ZZH", "Date", "Summary", "CQ", "QXRQ", "DQRQ", "BZ", "Amt", "LV", "YQ", "Kinbr", "Teller");
                }
            }
            IC_Page.ItemsSource = Data.DefaultView;
        }

        public DataTable Data { get; private set; }

        /// <summary>
        /// 最大的打印行数
        /// </summary>
        public const int MaxLineCount = 12;

        /// <summary>
        /// 加载数据，返回值为实际处理的数据条目
        /// </summary>
        /// <param name="data"></param>
        /// <param name="startLineNo"></param>
        /// <returns></returns>
        public void LoadData(IEnumerable<BankBookPageLine> data)
        {
            Data.Rows.Clear();
            for (int i = 1; i < MaxLineCount; i++)
            {
                var line = data.FirstOrDefault(x => (x.Number) % MaxLineCount == i );
                if (line == null)
                    Data.Rows.Add("", "", "", "", "", "", "", "", "", "", "", "");
                else
                    Data.Rows.Add(
                        line.ZZH,
                        line.Date == null ? string.Empty : line.Date.Value.ToString("yyyyMMdd"),
                        line.Summary,
                        line.DepositTerm,
                        line.QXRQ,
                        line.DQRQ,
                        line.BZ,
                        line.Amt,
                        line.InterestRate == null ? string.Empty : line.InterestRate.ToString().Trim('0'),
                        line.YQ,
                        line.Kinbr,
                        line.Teller);
            }
            var EndLine = data.FirstOrDefault(x => (x.Number) % MaxLineCount == 0);
            if (EndLine != null)
            {
                Data.Rows.Add(
                        EndLine.ZZH,
                        EndLine.Date == null ? string.Empty : EndLine.Date.Value.ToString("yyyyMMdd"),
                        EndLine.Summary,
                        EndLine.DepositTerm,
                        EndLine.QXRQ,
                        EndLine.DQRQ,
                        EndLine.BZ,
                        EndLine.Amt,
                        EndLine.InterestRate == null ? string.Empty : EndLine.InterestRate.ToString().Trim('0'),
                        EndLine.YQ,
                        EndLine.Kinbr,
                        EndLine.Teller);
            }
        }

        private string GetPrefixBySummary(string summary)
        {
            switch (summary)
            {
                case "开户":
                case "提入":
                case "换证":
                    return "+";
                case "部提":
                case "提出":
                case "销户":
                    return "-";
                default:
                    return string.Empty;
            }
        }

        /// <summary>
        /// 存折行数据
        /// </summary>
        public class BankBookPageLine
        {
            /// <summary>
            /// 打印行号，从1开始
            /// </summary>
            public int Number { get; set; }
            /// <summary>
            /// 子账号
            /// </summary>
            public string ZZH { get; set; }
            /// <summary>
            /// 发生日期
            /// </summary>
            public DateTime? Date { get; set; }
            /// <summary>
            /// 摘要
            /// </summary>
            public string Summary { get; set; }
            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }
            /// <summary>
            /// 起息日期
            /// </summary>
            public string QXRQ { get; set; }
            /// <summary>
            /// 到期日期
            /// </summary>
            public string DQRQ { get; set; }
            /// <summary>
            /// 币种
            /// </summary>
            public string BZ { get; set; }
            /// <summary>
            /// 金额
            /// </summary>
            public decimal? Amt { get; set; }
            /// <summary>
            /// 利率
            /// </summary>
            public decimal? InterestRate { get; set; }
            /// <summary>
            /// 约期
            /// </summary>
            public string YQ { get; set; }
            /// <summary>
            /// 机构
            /// </summary>
            public string Kinbr { get; set; }
            /// <summary>
            /// 柜员号
            /// </summary>
            public string Teller { get; set; }
        }
    }
}
