﻿using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using System;
using System.Windows;
using System.Windows.Controls;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    ///  承兑汇票
    /// </summary>
    public partial class Print_AcceptanceDraft : IPrintContract
    {
        public Print_AcceptanceDraft(Print_AcceptanceDraftEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }
        public Contract.Entry.PrintEntry PrintData { get; private set; }

        /// <summary>
        /// 凭证的打印数据实体定义
        /// </summary>
        public class Print_AcceptanceDraftEntry : PrintEntry
        {
            /**********************************
             * 定义实体属性
             * 1.打印逻辑用到的要素
             * *******************************/
            /// <summary>
            /// 出票人全称
            /// </summary>
            public string CPRQC { get; set; }

            /// <summary>
            /// 收款人全称
            /// </summary>
            public string SKRQC { get; set; }

            /// <summary>
            /// 出票人账号
            /// </summary>
            public string CPRZH { get; set; }

            /// <summary>
            /// 收款人账号
            /// </summary>
            public string SKRZH { get; set; }

            /// <summary>
            /// 付款行全称
            /// </summary>
            public string FKHQC { get; set; }

            /// <summary>
            /// 收款人开户银行
            /// </summary>
            public string SKRKHYH { get; set; }

            /// <summary>
            /// 出票金额
            /// </summary>
            public decimal CPJE { get; set; }

            /// <summary>
            /// 汇票到期日
            /// </summary>
            public string HPDQR { get; set; }

            /// <summary>
            /// 承兑协议编号
            /// </summary>
            public string CDXYBH { get; set; }

            /// <summary>
            /// 付款行行号
            /// </summary>
            public string FKHHH { get; set; }

            /// <summary>
            /// 付款行地址
            /// </summary>
            public string FKHDZ { get; set; }

            /// <summary>
            /// 金额小写字符间加空格
            /// </summary>
            public string JEXEZF { get; set; }
        }

        public void LoadPrintData()
        {
            /// <summary>
            /// 版本控制标志
            /// </summary>
            string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            var entry = PrintData as Print_AcceptanceDraftEntry;
            //传入要素
            CPRQC.Text = entry.CPRQC;
            SKRQC.Text = entry.SKRQC;
            CPRZH.Text = entry.CPRZH;
            SKRZH.Text = entry.SKRZH;
            FKHQC.Text = entry.FKHQC;
            SKRKHYH.Text = entry.SKRKHYH;

            if ( !string.IsNullOrEmpty(entry.JEXEZF))
            {
                JEXX.Visibility = Visibility.Visible;
                CPJE.FontSize = 20;
                if (FLAG == "V19")
                {
                    SdCanvas.SetValue(Canvas.TopProperty, -10.00);
                    SdCanvas.SetValue(Canvas.LeftProperty, -9.00);
                    CPJE.SetValue(Canvas.TopProperty, 160.00);
                }
                else
                {
                    SdCanvas.SetValue(Canvas.TopProperty, -11.00);
                    SdCanvas.SetValue(Canvas.LeftProperty, -16.00);
                }
            }
            JEXX.Text = "¥" + entry.JEXEZF;
            CPJE.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.CPJE);
            //汇票到期日 大写
            var DATA = entry.HPDQR.ToDateTime();
            var  DQMonth = entry.PageData.TradeAPIs.ChangeToCI(DATA.Month, false, true);
            var DQDay = entry.PageData.TradeAPIs.ChangeToCI(DATA.Day, false, true);

            if ((DQMonth=="贰") ||(DQMonth=="壹"))
            {
                DQMonth = "零" + DQMonth;
            }
            if (DQMonth.Contains("壹拾零"))
            {
                DQMonth = "零壹拾";
            }
            //TB_Date_Month.Text = DQMonth;
            switch (DQDay)
            {
                case "贰拾零":
                    DQDay = "零贰拾";
                    break;
                case "壹拾零":
                    DQDay = "零壹拾";
                    break;
                case "叁拾零":
                    DQDay = "零叁拾";
                    break;
                case "壹":
                    DQDay = "零壹";
                    break;
                case "贰":
                    DQDay = "零贰";
                    break;
                case "叁":
                    DQDay = "零叁";
                    break;
                case "肆":
                    DQDay = "零肆";
                    break;
                case "伍":
                    DQDay = "零伍";
                    break;
                case "陆":
                    DQDay = "零陆";
                    break;
                case "柒":
                    DQDay = "零柒";
                    break;
                case "捌":
                    DQDay = "零捌";
                    break;
                case "玖":
                    DQDay = "零玖";
                    break;
                default:
                    DQDay = DQDay;
                    break;
            }

            HPDQR.Text = entry.PageData.TradeAPIs.ChangeToCI(DATA.Year, false) + "年"
                + DQMonth + "月"
                + DQDay + "日";
            CDXYBH.Text = entry.CDXYBH;
            FKHHH.Text = entry.FKHHH.ToString();
            FKHDZ.Text = entry.FKHDZ;

            //公共数据
            //日期为营业日期
            var date = LoginUserInfo.TradeDate.ToDateTime();

            TB_Date_Year.Text = entry.PageData.TradeAPIs.ChangeToCI(date.Year, false);
            string Month = entry.PageData.TradeAPIs.ChangeToCI(date.Month, false, true);
            if ((Month=="贰") ||( Month=="壹"))
            {
                Month = "零" + Month;
            }
            if (Month.Contains("壹拾零"))
            {
                Month = "零壹拾";
            }
            TB_Date_Month.Text = Month;
             var myday =  entry.PageData.TradeAPIs.ChangeToCI(date.Day, false, true);
             switch (myday)
             {
                 case "贰拾零":
                     myday = "零贰拾";
                     break;
                 case "壹拾零":
                     myday = "零壹拾";
                     break;
                 case "叁拾零":
                     myday = "零叁拾";
                     break;
                 case "壹":
                     myday = "零壹";
                     break;
                 case "贰":
                     myday = "零贰";
                     break;
                 case "叁":
                     myday = "零叁";
                     break;
                 case "肆":
                     myday = "零肆";
                     break;
                 case "伍":
                     myday = "零伍";
                     break;
                 case "陆":
                     myday = "零陆";
                     break;
                 case "柒":
                     myday = "零柒";
                     break;
                 case "捌":
                     myday = "零捌";
                     break;
                 case "玖":
                     myday = "零玖";
                     break;
                 default:
                     myday = myday;
                     break;
             }
            string Myday = myday;
            TB_Date_Day.Text = Myday;
            ////复核等公共要素
            ////由于区域不足，仅打印姓名即可
            //var review = PrintAttribute.Review;
            //var index = PrintAttribute.Review.IndexOf('(');
            //if (index > 0)
            //    review = review.Substring(0, index);
            //TB_Recheck.Text = TB_Recheck1.Text = review;
            //TB_Handling.Text = LoginUserInfo.TellerName;
            //TB_Tally.Text = LoginUserInfo.TellerName;
            //TB_TestKey.Text = entry.TestKey;           
            if (FLAG == "V11")
            {
                SdCanvas.Margin = new Thickness(0,-5,0,0);
                //SKRKHYH.SetValue(Canvas.LeftProperty, 376.00);//大石桥 开户银行 左移 

                SKRKHYH.FontSize = CPRQC.FontSize = 10;//出票人全称、收款人账户 缩小字体确保打印完整
            }

        }
    }
}
