﻿using System;
using System.Windows;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using System.ComponentModel;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// Print_AccountApplication.xaml 的交互逻辑
    /// </summary>
    public partial class Print_AccountApplication : IPrintContract
    {
        public Print_AccountApplication(Print_AccountApplicationEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        /// <summary>
        /// 打印实体
        /// </summary>
        public PrintEntry PrintData { get; set; }

        /// <summary>
        /// 凭证的打印数据实体定义
        /// </summary>
        public class Print_AccountApplicationEntry : PrintEntry
        {
            /**********************************
             * 定义实体属性
             * 1.打印逻辑用到的要素
             * *******************************/
            /// <summary>
            /// 账号
            /// </summary>
            public string tb_Ac_no { get; set; }
            /// <summary>
            /// 户名
            /// </summary>
            public string tb_Acname { get; set; }
            /// <summary>
            /// 交易类型
            /// </summary>
            public string tb_tradeType { get; set; }
            /// <summary>
            /// 开户金额
            /// </summary>
            public string tb_AcNum { get; set; }

            /// <summary>
            /// 证件类型
            /// </summary>
            public string tb_IDtype { get; set; }
            /// <summary>
            /// 证件号码
            /// </summary>
            public string tb_IDNum { get; set; }
            /// <summary>
            /// 凭证号码
            /// </summary>
            public string tb_PZNum { get; set; }
            /// <summary>
            /// 开户日期
            /// </summary>
            public string tb_OpenDate { get; set; }
            /// <summary>
            /// 代理人名称
            /// </summary>
            public string tb_ProxyName { get; set; }
            /// <summary>
            /// 代理人证件类型
            /// </summary>
            public string tb_ProxyIDType { get; set; }
            /// <summary>
            /// 代理人证件号码
            /// </summary>
            public string tb_ProxyIDNum { get; set; }
            /// <summary>
            /// 授权柜员
            /// </summary>
            public string SQGY { get; set; }
            /// <summary>
            /// 经办人
            /// </summary>
            public string JBR { get; set; }
            /// <summary>
            /// 会计分录
            /// </summary>
            public string KJFL { get; set; }
            /// <summary>
            /// 机构名称
            /// </summary>
            public string tb_jgmc { get; set; }
            /// <summary>
            /// 代码
            /// </summary>
            public string tb_dm { get; set; }
            /// <summary>
            /// 交易
            /// </summary>
            public string tb_jy { get; set; }
            /// <summary>
            /// 日期
            /// </summary>
            public string tb_rq { get; set; }
            /// <summary>
            /// 柜员
            /// </summary>
            public string tb_gy { get; set; }
            /// <summary>
            /// 流水号
            /// </summary>
            public string tb_ywls { get; set; }
            /// <summary>
            /// 柜员流水
            /// </summary>
            public string tb_gyls { get; set; }
            /// <summary>
            /// 备注
            /// </summary>
            public string tb_bz { get; set; }

            public string isHasProxy { get; set; }
        }
        

         //<summary>
         //加载打印数据
         //</summary>
        public void LoadPrintData()
        {
            /**********************************
            * 加载数据到打印界面
            * 1.数据来源，PrintData.MessageData、PrintData.PageBase
            * *******************************/

            var entry = PrintData as Print_AccountApplicationEntry;
            if (entry.isHasProxy.Equals("Y"))
            {
                tb_ProxyName.Visibility = Visibility.Visible;
                tb_ProxyIDType.Visibility = Visibility.Visible;
                tb_ProxyIDNum.Visibility = Visibility.Visible;
            }
            tb_ProxyName.SetPrintTextBlock(entry.tb_ProxyName);
            tb_ProxyIDType.SetPrintTextBlock(entry.tb_ProxyIDType);
            tb_ProxyIDNum.SetPrintTextBlock(entry.tb_ProxyIDNum);
            tb_Ac_no.SetPrintTextBlock(entry.tb_Ac_no);
            tb_Acname.SetPrintTextBlock(entry.tb_Acname);
            tb_tradeType.SetPrintTextBlock(entry.tb_tradeType);
            //tb_Amt.SetPrintTextBlock(SystemFunction.Delspace(SystemFunction.Comma(entry.OpnAmt, 16)));
            tb_AcNum.SetPrintTextBlock(SystemFunction.Delspace(SystemFunction.Comma(entry.tb_AcNum, 16)));
            tb_IDtype.SetPrintTextBlock(entry.tb_IDtype);
            tb_IDNum.SetPrintTextBlock(entry.tb_IDNum);
            tb_PZNum.SetPrintTextBlock(entry.tb_PZNum);
            tb_OpenDate.SetPrintTextBlock(entry.tb_OpenDate);

            tb_jy.SetPrintTextBlock(this.PrintAttribute.TradeModel.Name + "(" + this.PrintAttribute.TradeModel.Code + ")");//交易名称
            tb_ywls.SetPrintTextBlock(this.PrintAttribute.BusinessTransaction);//业务流水
            SQGY.Text=this.PrintAttribute.Authentic;//授权柜员
            tb_gyls.SetPrintTextBlock(this.PrintAttribute.SerialNumber);//柜员流水号 
            tb_rq.SetPrintTextBlock(LoginUserInfo.TradeDate);//日期
            tb_gy.SetPrintTextBlock(LoginUserInfo.TellerName);//柜员
            tb_jgmc.SetPrintTextBlock(LoginUserInfo.KinbrName);//机构名称
            tb_dm.SetPrintTextBlock(entry.tb_dm);//代码
            JBR.Text=LoginUserInfo.TellerName;//经办人
            
            KJFL.LoadData(entry.KJFL);
        }
    }
}
