﻿using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using System;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 银行汇票
    /// </summary>
    public partial class Print_BillOfExchange : IPrintContract
    {
        public Print_BillOfExchange(Print_BillOfExchangeEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }
        public Contract.Entry.PrintEntry PrintData { get; private set; }

        /// <summary>
        /// 凭证的打印数据实体定义
        /// </summary>
        public class Print_BillOfExchangeEntry : PrintEntry
        {
            /**********************************
             * 定义实体属性
             * 1.打印逻辑用到的要素
             * *******************************/
            /// <summary>
            /// 付款行名称
            /// </summary>
            public string PaymentBankName { get; set; }

            /// <summary>
            /// 付款行号
            /// </summary>
            public string PaymentBankNo { get; set; }

            /// <summary>
            /// 收款人
            /// </summary>
            public string PayeeName { get; set; }

            /// <summary>
            /// 出票金额
            /// </summary>
            public decimal TicketAmount { get; set; }

            /// <summary>
            /// 结算金额
            /// </summary>
            public decimal SettlementAmount { get; set; }

            /// <summary>
            /// 申请人
            /// </summary>
            public string Apply { get; set; }

            /// <summary>
            /// 帐号
            /// </summary>
            public string Account { get; set; }

            /// <summary>
            /// 出票行
            /// </summary>
            public string TicketBankName { get; set; }

            /// <summary>
            /// 出票行号
            /// </summary>
            public string TicketBankNo { get; set; }

            /// <summary>
            /// 备注
            /// </summary>
            public string Remark { get; set; }

            /// <summary>
            /// 密押
            /// </summary>
            public string TestKey { get; set; }
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_BillOfExchangeEntry;
            //传入要素
            TB_PaymentBankName.Text = entry.PaymentBankName;
            TB_PaymentBankNo.Text = entry.PaymentBankNo;
            TB_PayeeName.Text = entry.PayeeName;
            TB_TicketAmount.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.TicketAmount);
            TB_SettlementAmountMax.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.SettlementAmount);
            TB_SettlementAmountMin.Text = entry.SettlementAmount.ToString();
            TB_Apply.Text = entry.Apply;
            TB_Account.Text = entry.Account;
            TB_TicketBankName.Text = entry.TicketBankName.Replace("股份有限公司", "");
            TB_TicketBankNo.Text = entry.TicketBankNo;
            TB_Remark.Text = entry.Remark;

            //公共数据
            //日期为营业日期
            var date = LoginUserInfo.TradeDate.ToDateTime();

            TB_Date_Year.Text = entry.PageData.TradeAPIs.ChangeToCI(date.Year, false);
            TB_Date_Month.Text = entry.PageData.TradeAPIs.ChangeToCI(date.Month, false, true);
            TB_Date_Day.Text = entry.PageData.TradeAPIs.ChangeToCI(date.Day, false, true);
            //复核等公共要素
            //由于区域不足，仅打印姓名即可
            var review = PrintAttribute.Review;
            var index = PrintAttribute.Review.IndexOf('(');
            if (index > 0)
                review = review.Substring(0, index);
            TB_Recheck.Text = TB_Recheck1.Text = review;
            TB_Handling.Text = LoginUserInfo.TellerName;
            TB_Tally.Text = LoginUserInfo.TellerName;
            TB_TestKey.Text = entry.TestKey;
        }
    }
}