﻿using System;
using System.Windows;
using System.Windows.Controls;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.PrintTemplate.Base;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// Page1.xaml 的交互逻辑
    /// </summary>
    public partial class Print_BusinessVoucher_Deposit : IPrintContract
    {
        public Print_BusinessVoucher_Deposit(Print_BusinessVoucher_Deposit_Entry entry)
        {
            InitializeComponent();
            PrintData = entry;
            LoadPrintData();
        }

        public Contract.Entry.PrintEntry PrintData { get; private set; }
        public class Print_BusinessVoucher_Deposit_Entry : PrintEntry
        {
            /// <summary>
            /// 银行留存联账号
            /// </summary>
            public string Acno { get; set; }
            /// <summary>
            /// 客户回单账户
            /// </summary>
            public string Acno1 { get; set; }

            /// <summary>
            /// 账户序号
            /// </summary>
            public string ZHXH { get; set; }

            /// <summary>
            /// 开户行名
            /// </summary>
            public string OpnName { get; set; }

            /// <summary>
            /// 账户户名
            /// </summary>
            public string ZHHM { get; set; }

            /// <summary>
            /// 交易金额
            /// </summary>
            public decimal TxAmt { get; set; }

            /// <summary>
            /// 账户余额
            /// </summary>
            public string tbAccount { get; set; }

            /// <summary>
            /// 余额
            /// </summary>
            public decimal Bal { get; set; }
            /// <summary>
            /// 利率
            /// </summary>
            public string Rate { get; set; }

            /// <summary>
            /// 储种
            /// </summary>
            public string PrdtName { get; set; }

            /// <summary>
            /// 存期
            /// </summary>
            // public string Term { get; set; }

            /// <summary>
            /// 凭证号码
            /// </summary>
            public string NoteNo { get; set; }

            /// <summary>
            /// 科目
            /// </summary>
            public string Subject { get; set; }

            /// <summary>
            /// 是否带介质
            /// </summary>
            public bool HasJZ { get; set; }

            /// <summary>
            /// 是否显示"是否带介质"
            /// </summary>
            public bool DisplayHasJZ { get; set; }

            /// <summary>
            /// 现转标志
            /// </summary>
            public string CtInd { get; set; }

            /// <summary>
            /// 交易类型
            /// </summary>
            public string TradeType { get; set; }

            /// <summary>
            /// 对方账号
            /// </summary>
            public string DfAcno { get; set; }

            /// <summary>
            /// 对方户名
            /// </summary>
            public string DfAcname { get; set; }

            /// <summary>
            /// 币种
            /// </summary>
            public string BZ { get; set; }
            /// <summary>
            /// 流水号
            /// </summary>
            public string LSH { get; set; }
            /// <summary>
            /// 表头
            /// </summary>
            public string Title { get; set; }
            /// <summary>
            /// 留密户
            /// </summary>
            public string Lmh { get; set; }
            /// <summary>
            ///自动转存
            /// </summary>
            public string ZDZC { get; set; }
            /// <summary>
            /// 通存标志
            /// </summary>
            public string TCBZ { get; set; }
            /// <summary>
            /// 账户类型
            /// </summary>
            public string PZHLX { get; set; }
            /// <summary>
            /// 证件类型
            /// </summary>
            public string zjlx { get; set; }
            /// <summary>
            /// 证件号码
            /// </summary>
            public string zjhm { get; set; }

            public string DLRBZ { get; set; }
            public string DLRMC { get; set; }
            public string DLRHM { get; set; }
            public string DLRSJH { get; set; }
            public string DLRZJLX { get; set; }
            public string FLBS { get; set; }
            /// <summary>
            /// 存款保险条例
            /// </summary>
            public string BZ1 { get; set; }
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_BusinessVoucher_Deposit_Entry;
            if (entry == null)
                return;
            TB_title.Text = entry.Title;


            TB_Prdt1.Text = "储种:{0}";


            switch (entry.TCBZ[4].ToString().Trim())
            {
                case "0":
                    TB_TCBZ.SetPrintTextBlock("非通存通兑");
                    break;
                case "1":
                    TB_TCBZ.SetPrintTextBlock("通存通兑");
                    break;
                case "2":
                    TB_TCBZ.SetPrintTextBlock("通存非通兑");
                    break;
                default:
                    TB_TCBZ.SetPrintTextBlock("通存通兑");
                    break;
            }

            TB_OPNNAME.SetPrintTextBlock(entry.OpnName);
            TB_Acno1.SetPrintTextBlock(entry.Acno);


            TB_ZHXH.SetPrintTextBlock(entry.ZHXH);
            TB_ZHXH2.SetPrintTextBlock(entry.Acno.Substring(0, 6) + "****" + entry.Acno.Substring(entry.Acno.Length - 4, 4) + "-" + entry.ZHXH);
            TB_HM1.SetPrintTextBlock(entry.ZHHM);

            TB_Note1.SetPrintTextBlock(entry.NoteNo.PadLeft(TellerSystem.Library.Ext.Variables.UserVariable.Len_VoucherNo, '0'));
            var type = UserFunction.ChangeMoneySymbol(entry.BZ);
            TB_TxAmt1.SetPrintTextBlock(type + entry.TxAmt.ToString("N2"));

            //天山新增证件类型,证件号码
            ZJLX.SetPrintTextBlock(entry.zjlx);
            ZJHM.SetPrintTextBlock(entry.zjhm);
            if (entry.tbAccount != null)
            {
                tb_Account.Visibility = Visibility.Visible;
                tb_Account.SetPrintTextBlock(type + entry.tbAccount.ToDecimal().ToString("N2"));
            }
            if (entry.PZHLX != null)
            {
                ZHLX.Visibility = Visibility.Visible;
                ZHLX.SetPrintTextBlock(entry.PZHLX);
            }
            if (entry.FLBS == "true")
            {
                FLBS.Visibility = Visibility.Visible;
            }
            if (entry.HasJZ)
            {
                TB_HasMdm1.SetPrintTextBlock("是");
            }
            else
            {
                TB_HasMdm1.SetPrintTextBlock("否");
            }
            TB_HasMdm1.Visibility = entry.DisplayHasJZ ? Visibility.Visible : Visibility.Hidden;
            TB_Prdt1.SetPrintTextBlock(entry.PrdtName);
            TB_rq.SetPrintTextBlock(LoginUserInfo.TradeDate);
            TB_jyh.SetPrintTextBlock(LoginUserInfo.KinbrNo);
            TB_TradeType1.SetPrintTextBlock(entry.TradeType);
            TB_CtInd1.SetPrintTextBlock(entry.CtInd);
            // TB_CtInd2.SetPrintTextBlock(entry.CtInd);
            if (string.IsNullOrEmpty(entry.DfAcno))
            {
                TB_DfAcno1.Visibility = Visibility.Hidden;
            }
            else
            {
                TB_DfAcno1.SetPrintTextBlock(entry.DfAcno);
            }
            if (string.IsNullOrEmpty(entry.DfAcname))
            {
                TB_DfAcname1.Visibility = Visibility.Hidden;
            }
            else
            {
                TB_DfAcname1.SetPrintTextBlock(entry.DfAcname);
            }
            TB_SFLM.SetPrintTextBlock(entry.Lmh);
            if (!string.IsNullOrEmpty(entry.ZDZC))
            {
                TB_ZDZC.Visibility = Visibility.Visible;
                TB_ZDZC.SetPrintTextBlock(entry.ZDZC);

            }
            //公共要素
            var tradedate = LoginUserInfo.TradeDate.ToDateTime();
            if (!string.IsNullOrEmpty(entry.Rate))
            {
                TB_Rate1.SetPrintTextBlock(string.Format("{0:N6}%", entry.Rate));
            }
            else
            {
                TB_Rate1.SetPrintTextBlock(string.Empty);

            }
            //代理人信息
            if (entry.DLRBZ == "Y")
            {
                if (!string.IsNullOrWhiteSpace(entry.DLRZJLX))
                {
                    TB_DLRZJLX.Visibility = Visibility.Visible;
                }
                if (!string.IsNullOrWhiteSpace(entry.DLRHM))
                {
                    TB_DLRHM.Visibility = Visibility.Visible;
                }
                if (!string.IsNullOrWhiteSpace(entry.DLRMC))
                {
                    TB_DLRMC.Visibility = Visibility.Visible;
                }
                if (!string.IsNullOrWhiteSpace(entry.DLRSJH))
                {
                    TB_DLRSJH.Visibility = Visibility.Visible;
                }
                TB_DLRZJLX.SetPrintTextBlock(entry.DLRZJLX);
                TB_DLRHM.SetPrintTextBlock(entry.DLRHM);
                TB_DLRMC.SetPrintTextBlock(entry.DLRMC);
                TB_DLRSJH.SetPrintTextBlock(entry.DLRSJH);
            }
            //中牟需求 新储种过长 右侧整体向右偏移两个字节
            TB_HM1.SetValue(Canvas.LeftProperty, 298.00);
            TB_Rate1.SetValue(Canvas.LeftProperty, 298.00);
            TB_rq.SetValue(Canvas.LeftProperty, 298.00);
            TB_DfAcno1.SetValue(Canvas.LeftProperty, 298.00);
            TB_SFLM.SetValue(Canvas.LeftProperty, 298.00);
            TB_DfAcname1.SetValue(Canvas.LeftProperty, 298.00);
            TB_DLRHM.SetValue(Canvas.LeftProperty, 298.00);
            TB_DLRSJH.SetValue(Canvas.LeftProperty, 298.00);

            TB_OPNNAME.SetValue(Canvas.LeftProperty, 500.00);
            TB_jyh.SetValue(Canvas.LeftProperty, 500.00);
            TB_TradeType1.SetValue(Canvas.LeftProperty, 500.00);
            TB_HasMdm1.SetValue(Canvas.LeftProperty, 500.00);
            TB_ZDZC.SetValue(Canvas.LeftProperty, 500.00);
            tb_Account.SetValue(Canvas.LeftProperty, 500.00);
            TB_HasMdm1.SetValue(Canvas.TopProperty, 192.00);
            tb_Account.SetValue(Canvas.TopProperty, 248.00);
            //柜员流水、业务流水、TB_title一行
            TB_title.SetValue(Canvas.LeftProperty, 394.00);
            //账户余额不显示
            tb_Account.Visibility = Visibility.Collapsed;
        }

    }
}