﻿using System;
using System.Windows;
using System.Windows.Controls;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.PrintTemplate.Base;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// Print_PersonalAccountingVoucher.xaml 的交互逻辑
    /// </summary>
    public partial class Print_BusinessVoucher_Drawxh : IPrintContract
    {
        public Print_BusinessVoucher_Drawxh(Print_BusinessVoucher_Drawxh_Entry entry)
        {
            InitializeComponent();
            PrintData = entry;
            LoadPrintData();
        }
        public class Print_BusinessVoucher_Drawxh_Entry : PrintEntry
        {
            /// <summary>
            /// 标题
            /// </summary>
            public string Title { get; set; }

            /// <summary>
            /// 结算类型
            /// </summary>
            public string JSLX { get; set; }

            /// <summary>
            /// 账户类型
            /// </summary>
            public string ZHLX { get; set; }

            /// <summary>
            /// 账号
            /// </summary>
            public string Acno { get; set; }

            /// <summary>
            /// 账户序号
            /// </summary>
            public string ZHXH { get; set; }

            /// <summary>
            /// 账户户名
            /// </summary>
            public string ZHHM { get; set; }
            /// <summary>
            /// 开户日
            /// </summary>
            public string Opndata { get; set; }
            /// <summary>
            /// 到期日
            /// </summary>
            public string DataEnd { get; set; }

            /// <summary>
            /// 开户行名
            /// </summary>
            // public string OpnName { get; set; }

            /// <summary>
            /// 储种
            /// </summary>
            public string PrdtName { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public string Rate { get; set; }

            /// <summary>
            /// 交易类型
            /// </summary>
            public string TradeType { get; set; }

            /// <summary>
            /// 交易金额
            /// </summary>
            public decimal TxAmt { get; set; }



            /// <summary>
            /// 流水号
            /// </summary>
            // public string Lsh { get; set; }

            /// <summary>
            /// 利息
            /// </summary>
            public decimal Lx { get; set; }

            /// <summary>
            /// 利息税
            /// </summary>
            public decimal Lxs { get; set; }

            /// <summary>
            /// 自动转存
            /// </summary>
            public string Zdzc { get; set; }

            /// <summary>
            /// 实付利息
            /// </summary>
            public decimal Sflx { get; set; }

            /// <summary>
            /// 备注信息1
            /// </summary>
            public string Bzinfo1 { get; set; }
            /// <summary>
            /// 开户机构10
            /// </summary>
            public string Khjg { get; set; }
            /// <summary>
            /// 实付本息税
            /// </summary>
            public decimal Sfbxs { get; set; }

            /// <summary>
            /// 备注信息2
            /// </summary>
            public string Bzinfo2 { get; set; }

            /// <summary>
            /// 对方账号
            /// </summary>
            public string DfAcno { get; set; }

            /// <summary>
            /// 对方户名
            /// </summary>
            public string DfAcname { get; set; }
            /// <summary>
            /// 通存标志
            /// </summary>
            public string TCBZ { get; set; }
            /// <summary>
            /// 前置利息
            /// </summary>
            public string QZLX { get; set; }

            /// <summary>
            /// 是否代理
            /// </summary>
            public string SFDL { get; set; }
            /// <summary>
            /// 代理人名称
            /// </summary>
            public string DLRMC { get; set; }
            /// <summary>
            /// 代理人号码
            /// </summary>
            public string DLRHM { get; set; }
            /// <summary>
            /// 代理人手机号
            /// </summary>
            public string DLRSJH { get; set; }
            /// <summary>
            /// 代理人证件类型
            /// </summary>
            public string DLRZJLX { get; set; }
            /// <summary>
            /// 备注信息3
            /// </summary>
            public string Bzinfo3 { get; set; }

            /// <summary>
            /// 利息转入方式
            /// </summary>
            public string LXZRFS { get; set; }

            /// <summary>
            /// 利息转入账号
            /// </summary>
            public string LXZRZH { get; set; }
            /// <summary>
            /// 凭证号
            /// </summary>
            public string PZH { get; set; }
        }
        public Contract.Entry.PrintEntry PrintData { get; set; }

        public void LoadPrintData()
        {
            var flag = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            var entry = PrintData as Print_BusinessVoucher_Drawxh_Entry;
            if (entry == null)
                return;
            TB_title.Text = entry.Title;
            TB_Prdt1.Text = "储种:{0}";
            switch (entry.TCBZ[4].ToString().Trim())
            {
                case "0":
                    TB_TCBZ.SetPrintTextBlock("非通存通兑");
                    break;
                case "1":
                    TB_TCBZ.SetPrintTextBlock("通存通兑");
                    break;
                case "2":
                    TB_TCBZ.SetPrintTextBlock("通存非通兑");
                    break;
                default:
                    TB_TCBZ.SetPrintTextBlock("通存通兑");
                    break;
            }
            if (flag == "V0")
            {
                if (TB_ZHXH.Visibility == Visibility.Collapsed)
                {
                    TB_PZH.SetValue(Canvas.TopProperty, 134.0);
                }
                TB_PZH.Visibility = Visibility.Visible;
                TB_PZH.SetPrintTextBlock(entry.PZH);
            }

            TB_ZHXH.SetPrintTextBlock(entry.ZHXH);
            TB_HM1.SetPrintTextBlock(entry.ZHHM);
            TB_Acno1.SetPrintTextBlock(entry.Acno);
            TB_OPNData.SetPrintTextBlock(entry.Opndata);
            TB_Prdt1.SetPrintTextBlock(entry.PrdtName);

            if (entry.ZHLX.Equals("2"))
            {
                TB_JSLX.Visibility = Visibility.Hidden;
            }
            else
            {
                TB_JSLX.SetPrintTextBlock(entry.JSLX);
            }
            if (!string.IsNullOrEmpty(entry.Rate))
            {
                TB_Rate1.SetPrintTextBlock(string.Format("{0:N6}%", entry.Rate));

            }
            else
            {
                TB_Rate1.SetPrintTextBlock(string.Empty);


            }
            TB_Data.SetPrintTextBlock(LoginUserInfo.TradeDate);
            TB_TradeType.SetPrintTextBlock(entry.TradeType);
            //var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            var type = UserFunction.ChangeMoneySymbol("01");
            TB_TxAmt1.SetPrintTextBlock(type + entry.TxAmt.ToString("N2"));
            // TB_lsh.SetPrintTextBlock(this.PrintAttribute.SerialNumber);
            TB_lx.SetPrintTextBlock(type + entry.Lx.ToString("N2"));
            TB_lxs.SetPrintTextBlock(type + entry.Lxs.ToString("N2"));
            //TB_czy.SetPrintTextBlock(LoginUserInfo.TellerNo); 
            //TB_czy.SetPrintTextBlock("(" + LoginUserInfo.TellerName + ")" + LoginUserInfo.TellerNo);
            //TB_czy.Text = LoginUserInfo.TellerNo;

            if (!string.IsNullOrEmpty(entry.Zdzc))
            {
                TB_zdzc.Visibility = Visibility.Visible;
                TB_zdzc.SetPrintTextBlock(entry.Zdzc);
            }
            else
            {
                TB_zdzc.Visibility = Visibility.Hidden;
            }
            if (!string.IsNullOrEmpty(entry.QZLX))
            {
                if (LoginUserInfo.TradeDate.ToDecimal() < entry.DataEnd.ToDecimal())
                {
                    tbkQZLX3.SetPrintTextBlock("¥" + entry.QZLX);
                    tbkQZLX3.Visibility = Visibility.Visible;
                    tbkQZLX.Visibility = Visibility.Collapsed;
                    tbkQZLX2.Visibility = Visibility.Collapsed;
                }
                else if (LoginUserInfo.TradeDate.ToDecimal() == entry.DataEnd.ToDecimal())
                {
                    tbkQZLX.SetPrintTextBlock("¥" + entry.QZLX);

                    tbkQZLX.Visibility = Visibility.Visible;
                    tbkQZLX3.Visibility = Visibility.Collapsed;
                    tbkQZLX2.Visibility = Visibility.Collapsed;
                }
                else
                {
                    tbkQZLX2.SetPrintTextBlock("¥" + entry.QZLX);

                    tbkQZLX2.Visibility = Visibility.Visible;
                    tbkQZLX.Visibility = Visibility.Collapsed;
                    tbkQZLX3.Visibility = Visibility.Collapsed;
                }
            }
            else
            {
                tbkQZLX.Visibility = Visibility.Collapsed;
            }
            TB_sflx.SetPrintTextBlock(type + entry.Sflx.ToString("N2"));

            TB_info1.Text = entry.Bzinfo1;
            // TB_dqlxs.SetPrintTextBlock(type + entry.Dqlxs.ToString("N2"));
            TB_khjg.SetPrintTextBlock(entry.Khjg);
            TB_jyjg.SetPrintTextBlock(LoginUserInfo.KinbrNo);
            TB_sfbx.SetPrintTextBlock(type + entry.Sfbxs.ToString("N2"));

            TB_info2.Text = entry.Bzinfo2;
            TB_info3.Text = entry.Bzinfo3;
            TB_LXZRFS.SetPrintTextBlock(entry.LXZRFS);
            TB_LXZRZH.SetPrintTextBlock(entry.LXZRZH);
            //业务流水
            // TB_lsh.SetPrintTextBlock(this.PrintAttribute.SerialNumber);
            //核心流水
            // tb_ls.SetPrintTextBlock(this.PrintAttribute.BusinessTransaction);
            // TB_sqy.SetPrintTextBlock(this.PrintAttribute.Authentic);//授权柜员号
            // TB_sqy.Text = this.PrintAttribute.Authno;
            if (string.IsNullOrEmpty(entry.DfAcno))
            {
                TB_DfAcno1.Visibility = Visibility.Hidden;

            }
            else
            {
                TB_DfAcno1.SetPrintTextBlock(entry.DfAcno);

            }
            if (string.IsNullOrEmpty(entry.DfAcname))
            {
                TB_DfAcname1.Visibility = Visibility.Hidden;

            }
            else
            {
                TB_DfAcname1.SetPrintTextBlock(entry.DfAcname);

            }

            //代理人信息
            if (entry.SFDL == "Y")
            {
                if (!string.IsNullOrWhiteSpace(entry.DLRZJLX))
                {
                    TB_DLRZJLX.Visibility = Visibility.Visible;
                }
                if (!string.IsNullOrWhiteSpace(entry.DLRHM))
                {
                    TB_DLRHM.Visibility = Visibility.Visible;
                }
                if (!string.IsNullOrWhiteSpace(entry.DLRMC))
                {
                    TB_DLRMC.Visibility = Visibility.Visible;
                }
                if (!string.IsNullOrWhiteSpace(entry.DLRSJH))
                {
                    TB_DLRSJH.Visibility = Visibility.Visible;
                }
                TB_DLRZJLX.SetPrintTextBlock(entry.DLRZJLX);
                TB_DLRHM.SetPrintTextBlock(entry.DLRHM);
                TB_DLRMC.SetPrintTextBlock(entry.DLRMC);
                TB_DLRSJH.SetPrintTextBlock(entry.DLRSJH);
            }



            TB_HM1.SetValue(Canvas.LeftProperty, 295.00);
            TB_Rate1.SetValue(Canvas.LeftProperty, 295.00);
            TB_TxAmt1.SetValue(Canvas.LeftProperty, 295.00);
            TB_lxs.SetValue(Canvas.LeftProperty, 295.00);
            TB_jyjg.SetValue(Canvas.LeftProperty, 295.00);
            TB_DfAcname1.SetValue(Canvas.LeftProperty, 295.00);
            TB_JSLX.SetValue(Canvas.LeftProperty, 295.00);
            TB_DLRSJH.SetValue(Canvas.LeftProperty, 295.00);
            TB_DLRHM.SetValue(Canvas.LeftProperty, 295.00);
            //交易复核人 
            //if (!string.IsNullOrWhiteSpace(PrintAttribute.Review))
            //{
            //    var review = PrintAttribute.Review;
            //    TB_sqy.Text = review.Substring(review.IndexOf("(") + 1, 6);
            //}
            //else
            //{
            //    TB_sqy.Visibility = Visibility.Collapsed;
            //}
            //授权柜员号
            //TB_Super.SetPrintTextBlock(PrintAttribute.Authno);
            // TB_Super.Visibility = !string.IsNullOrWhiteSpace(PrintAttribute.Authno) ? Visibility.Visible : Visibility.Collapsed;
            //柜员流水、业务流水、TB_title一行
            TB_title.SetValue(Canvas.LeftProperty, 364.00);
            //  tb_ls.SetValue(Canvas.TopProperty, 72.00);
            //  TB_lsh.SetValue(Canvas.TopProperty, 72.00);
        }
    }



}
