﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.PrintTemplate;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 储蓄存单
    /// </summary>
    public partial class Print_CashCheck : IPrintContract
    {
        public Print_CashCheck(Print_CashCheckentry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData
        {
            get;
            private set;
        }

        /// <summary>
        /// 
        /// </summary>
        public class Print_CashCheckentry : PrintEntry
        {
            ///// <summary>
            ///// 营业日期
            ///// </summary>
            //public string date { get; set; }

            /// <summary>
            /// 付款行名称
            /// </summary>
            public string payBankName { get; set; }

            /// <summary>
            /// 收款人
            /// </summary>
            public string payee { get; set; }

            /// <summary>
            /// 出票人账号
            /// </summary>
            public string CPRNo { get; set; }

            /// <summary>
            /// 金额大写
            /// </summary>
            public string Money { get; set; }

            /// <summary>
            /// 用途
            /// </summary>
            public string use { get; set; }

            /// <summary>
            /// 密码
            /// </summary>
            public string psw { get; set; }

            /// <summary>
            /// 上列款项请存
            /// </summary>
            public string SLKXQC { get; set; }

            /// <summary>
            /// 我账户内支付
            /// </summary>
            public string WZHNZF { get; set; }

            ///// <summary>
            ///// 复核
            ///// </summary>
            //public string check { get; set; }

            ///// <summary>
            ///// 记账
            ///// </summary>
            //public string JZ { get; set; }
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_CashCheckentry;
            if (entry == null)
                return;
                        
                var date = LoginUserInfo.TradeDate;
                dateYear.Text = date.Trim().Substring(0, 3);
                dateMonth.Text = date.Trim().Substring(4, 5);
                dateDay.Text = date.Trim().Substring(6, 7);
            
            if (entry.payee != null)
                SKR.Text = entry.payee;
            if (entry.CPRNo != null)
                CPRZH.Text = entry.CPRNo;
            if (entry.Money != null)
                JE.Text = GetMoneyByNum(entry.Money);
            //主要用来截取存期的描述部分
            if (!string.IsNullOrEmpty(entry.use))
            {
                  Use.Text = entry.use;
            }
    
            if(string.IsNullOrEmpty(entry.psw))
            {
                Password.Text = entry.psw; 
            }


            if (string.IsNullOrEmpty(entry.WZHNZF))
            {
                W_ZHNZF.Text = entry.WZHNZF;
            }
            if (string.IsNullOrEmpty(entry.SLKXQC))
            {
                S_LKXQC.Text = entry.SLKXQC;
            }
            var review = PrintAttribute.Review;
            var index = PrintAttribute.Review.IndexOf('(');
            if (index > 0)
                review = review.Substring(0, index);
            FH.Text = review;
            JZTeller.Text = LoginUserInfo.TellerName; 
      
        }

        /// <summary>
        /// 将数字金额转化为大写金额
        /// </summary>
        private string GetMoneyByNum(string source)
        {
            string str = "";
            string[] temp = { "零","壹", "贰", "叁", "肆", "伍", "陆", "柒", "捌", "玖"};
            if (!string.IsNullOrEmpty(source))
            {
                for (int i = 0; i < source.Length; i++)
                {
                    if (source[i] > 47 && source[i] < 58)
                    {
                        str += temp[source[i] - 48];
                        break;
                    }
                    else 
                    {
                        MessageBox.Show("转化字符串中含有非数字字符，请仔细检查！");
                        return "error";
                    }
                }
            }
            return str;
        }
    }
}
