﻿using System;
using System.Windows;
using System.Windows.Controls;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using System.ComponentModel;
using System.Data.SqlTypes;
using TellerSystem.Library.Ext.Variables;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// Print_AccountApplication.xaml 的交互逻辑
    /// </summary>
    public partial class Print_ClearingwithdrawalSD : IPrintContract
    {
        public Print_ClearingwithdrawalSD(Print_ClearingwithdrawalEntrySD entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        /// <summary>
        /// 打印实体
        /// </summary>
        public PrintEntry PrintData { get; set; }

        /// <summary>
        /// 凭证的打印数据实体定义
        /// </summary>
        public class Print_ClearingwithdrawalEntrySD : PrintEntry
        {
            /**********************************
             * 定义实体属性
             * 1.打印逻辑用到的要素
             * *******************************/
            /// <summary>
            /// 取款账号
            /// </summary>
            public string tb_Ac_no { get; set; }
            /// <summary>
            /// 取款户名
            /// </summary>
            public string tb_Acname { get; set; }
            /// <summary>
            /// 交易摘要
            /// </summary>
            public string tb_JYZY { get; set; }

            /// <summary>
            /// 交易类型
            /// </summary>
            public string tb_tradeType { get; set; }
            /// <summary>
            /// 开户金额
            /// </summary>
            public string tb_AcNum { get; set; }

            /// <summary>
            /// 证件类型
            /// </summary>
            public string tb_IDtype { get; set; }
            /// <summary>
            /// 证件号码
            /// </summary>
            public string tb_IDNum { get; set; }

            /// <summary>
            /// 凭证类型
            /// </summary>
            public string tb_PZLX { get; set; }

            /// <summary>
            /// 凭证类型判断
            /// </summary>
            public string tb_PZLXPD { get; set; }
            /// <summary>
            /// 凭证号码
            /// </summary>
            public string tb_PZNum { get; set; }
         
            /// <summary>
            /// 会计分录
            /// </summary>
            public string KJFL { get; set; }
            /// <summary>
            /// 机构名称
            /// </summary>
            public string tb_jgmc { get; set; }
            /// <summary>
            /// 代码
            /// </summary>
            public string tb_dm { get; set; }
            /// <summary>
            /// 交易
            /// </summary>
            public string tb_jy { get; set; }
            /// <summary>
            /// 日期
            /// </summary>
            public string tb_rq { get; set; }
          
            /// <summary>
            /// 流水号
            /// </summary>
            public string tb_ywls { get; set; }
            /// <summary>
            /// 柜员流水
            /// </summary>
            public string tb_gyls { get; set; }

            /// <summary>
            /// 户名
            /// </summary>
            public string tb_HM { get; set; }

            /// <summary>
            /// 交易金额
            /// </summary>
            public string tb_JYJE{ get; set; }
            /// <summary>
            /// 账户余额
            /// </summary>
            public string tb_ZHYE { get; set; }
            /// <summary>
            /// 账户类型
            /// </summary>
            public string tb_ZHLX { get; set; }
            /// <summary>
            /// 购买日期
            /// </summary>
            public string tb_GMRQ { get; set; }
            /// <summary>
            /// 验印通过标志
            /// </summary>
            public string yyFlag { get; set; }
       
           
        }


        //<summary>
        //加载打印数据
        //</summary>
        public void LoadPrintData()
        {
            /**********************************
            * 加载数据到打印界面
            * 1.数据来源，PrintData.MessageData、PrintData.PageBase
            * *******************************/

            var entry = PrintData as Print_ClearingwithdrawalEntrySD;

          
            tb_Ac_no.SetPrintTextBlock(entry.tb_Ac_no);//取款账号
            tb_Acname.SetPrintTextBlock(entry.tb_Acname);//取款户名
            tb_tradeType.SetPrintTextBlock(entry.tb_PZLX); //凭证类型
            tb_AcNum.SetPrintTextBlock(SystemFunction.Delspace(SystemFunction.Comma(entry.tb_AcNum, 16)));//交易金额

            tb_PZNum.SetPrintTextBlock(entry.tb_PZNum.PadLeft(TellerSystem.Library.Ext.Variables.UserVariable.Len_VoucherNo, '0'));//凭证号码
            tb_jy.SetPrintTextBlock(this.PrintAttribute.TradeModel.Name + "(" + this.PrintAttribute.TradeModel.Code + ")");//交易名称
            tb_ywls.SetPrintTextBlock(this.PrintAttribute.BusinessTransaction);//业务流水
            FHGY.Text = this.PrintAttribute.Review;//复核柜员
            tb_gyls.SetPrintTextBlock(this.PrintAttribute.SerialNumber);//柜员流水号 
            tb_JYZY.SetPrintTextBlock(entry.tb_JYZY);//交易摘要
            tb_ZHYE.SetPrintTextBlock(entry.tb_ZHYE);//账户余额
            tb_ZHLX.SetPrintTextBlock(entry.tb_ZHLX);//账户类型
            YearBlock.Text = LoginUserInfo.TradeDate.ToString().Substring(0, 4);
            MothBlock.Text = LoginUserInfo.TradeDate.ToString().Substring(4, 2);
            DayBlock.Text = LoginUserInfo.TradeDate.ToString().Substring(6, 2);

            //购买日期的显示
            if (entry.tb_PZLXPD != "299" && entry.tb_PZLXPD !=UserVariable.PZZL_SDDGZ)
            {
                GMRQ.Visibility = Visibility.Visible;
                GMRQ.SetPrintTextBlock(entry.tb_GMRQ);
            }

            //户名的显示
            if (!string.IsNullOrEmpty(entry.tb_HM))
            {
                HmBlock.Visibility = Visibility.Visible;
                this.HmBlock.SetPrintTextBlock(entry.tb_HM);
            }
            //账户证件类型 显示赋值
            if (!string.IsNullOrEmpty(entry.tb_IDNum))
            {
                tb_IDNum.Visibility = tb_IDtype.Visibility = Visibility.Visible;
                this.tb_IDtype.SetPrintTextBlock(entry.tb_IDtype);
                this.tb_IDNum.SetPrintTextBlock(entry.tb_IDNum);
            }

            tb_jgmc.SetPrintTextBlock(LoginUserInfo.KinbrName);//机构名称
            tb_dm.SetPrintTextBlock(entry.tb_dm);//代码
            JBR.Text = LoginUserInfo.TellerName;//经办人
            if (!string.IsNullOrEmpty(entry.yyFlag))
            {
                YYflag.Text = entry.yyFlag;
            }
            //黄骅青隆增加印章
            string FLAG = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            if (FLAG == "V5" && LoginUserInfo.KinbrNo.Substring(0, 3) == "302")
            {
                this.E_Seal_Location = new Point(554, 200);
                tb_ZHLX.SetValue(Canvas.LeftProperty, 393.00);
                tb_gyls.SetValue(Canvas.LeftProperty, 393.00);
            }
            if (!string.IsNullOrEmpty(this.PrintAttribute.Authno))
            {
                SQGY.Visibility=Visibility.Visible;
                SQGY.SetPrintTextBlock(this.PrintAttribute.Authno);
            }

        }
    }
}
