﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 单位单位定期存单
    /// </summary>
    public partial class Print_EnterpriseRegularlyCDs : IPrintContract
    {
        public Print_EnterpriseRegularlyCDs(Print_EnterpriseRegularlyCDsEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }


        public Contract.Entry.PrintEntry PrintData
        {
            get;
            private set;
        }

        public class Print_EnterpriseRegularlyCDsEntry : PrintEntry
        {
            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }
            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }
            /// <summary>
            /// 证实书号
            /// </summary>
            public string ConfirmationNo { get; set; }
            /// <summary>
            /// 金额
            /// </summary>
            public decimal Amount { get; set; }
            /// <summary>
            /// 期限
            /// </summary>
            public int Months { get; set; }
            /// <summary>
            /// 利率
            /// </summary>
            public decimal InterestRate { get; set; }
            public DateTime? DateEnd { get; set; }

            /// <summary>
            /// 自动转存
            /// </summary>
            public string Autozc { get; set; }
            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_EnterpriseRegularlyCDsEntry;
            TB_AccountName.Text = entry.AccountName;
            TB_Account.Text = entry.Account;
            TB_ConfirmationNo.Text = entry.ConfirmationNo;
            TB_AmountMax.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
            var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            TB_AmountMin.Text = type + entry.Amount.ToString("N2");
            TB_Months.Text = entry.Months.ToString();
            TB_InterestRate.Text = string.Format("{0:N6}%", entry.InterestRate);
            if (entry.DateEnd != null)
            {
                TB_DateEnd.Text = entry.DateEnd.Value.ToString("yyyy/MM/dd");
            }
            else
            {
                TB_DateEnd.Text = "---";
            }
            TB_AutoZC.Text = entry.Autozc;
            //公共要素
            var date = entry.PageData.TradeAPIs.ToDateTime(LoginUserInfo.TradeDate);
            TB_TicketDate.Text = date.ToString("yyyy/MM/dd");
            TB_Recheck.Text = PrintAttribute.Review;
            TB_Handling.Text = LoginUserInfo.TellerNo + "(" + LoginUserInfo.TellerName + ")";
            TB_Operation.Text = LoginUserInfo.TellerNo;
        }
    }
}
