﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 单位单位定期存单
    /// </summary>
    public partial class Print_EnterpriseRegularlyCDs_HT : IPrintContract
    {
        string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");

        public Print_EnterpriseRegularlyCDs_HT(Print_EnterpriseRegularlyCDs_HTEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }


        public Contract.Entry.PrintEntry PrintData
        {
            get;
            private set;
        }

        public class Print_EnterpriseRegularlyCDs_HTEntry : PrintEntry
        {
            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }
            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }
            /// <summary>
            /// 证实书号
            /// </summary>
            public string ConfirmationNo { get; set; }
            /// <summary>
            /// 金额
            /// </summary>
            public decimal Amount { get; set; }
            /// <summary>
            /// 期限
            /// </summary>
            public string Months { get; set; }
            /// <summary>
            /// 利率
            /// </summary>
            public decimal InterestRate { get; set; }

            /// <summary>
            /// 到期日期
            /// </summary>
            public DateTime? DateEnd { get; set; }

            /// <summary>
            /// 存入日期
            /// </summary>
            public DateTime? DateStart { get; set; }
            /// <summary>
            /// 自动转存
            /// </summary>
            public string Autozc { get; set; }
            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }
            /// <summary>
            /// 客户经理号
            /// </summary>
            public string CustomerManager { get; set; }
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_EnterpriseRegularlyCDs_HTEntry;
            TB_AccountName.Text = entry.AccountName;
            TB_Account.Text = entry.Account;
            TB_ConfirmationNo.Text = entry.ConfirmationNo;
            TB_AmountMax.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
            var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            TB_AmountMin.Text = type + entry.Amount.ToString("N2");
            TB_Months.Text = entry.Months.ToString();
            TB_InterestRate.Text = string.Format("{0:N6}%", entry.InterestRate);
            if (entry.DateEnd != null)
            {
                TB_DateEndYear.Text = entry.DateEnd.Value.ToString("yyyyMMdd").Substring(0,4);
                TB_DateEndMonths.Text = entry.DateEnd.Value.ToString("yyyyMMdd").Substring(4, 2);
                TB_DateEndDays.Text = entry.DateEnd.Value.ToString("yyyyMMdd").Substring(6, 2);
            }
            else
            {
                TB_DateEndYear.Text = TB_DateEndMonths.Text = TB_DateEndDays.Text = "---";
            }
            //TB_AutoZC.Text = entry.Autozc;
            //公共要素
            var date = entry.PageData.TradeAPIs.ToDateTime(LoginUserInfo.TradeDate);
            if (entry.DateStart != null)
            {
                TB_TicketDateYear.Text = entry.DateStart.Value.ToString("yyyyMMdd").Substring(0, 4);
                TB_Months1.Text = entry.DateStart.Value.ToString("yyyyMMdd").Substring(4, 2);
                TB_Days1.Text = entry.DateStart.Value.ToString("yyyyMMdd").Substring(6, 2);
                if (FLAG == "V11")
                {
                    TB_TicketDateYear1.Text = entry.DateStart.Value.ToString("yyyyMMdd").Substring(0, 4);
                    TB_Months2.Text = entry.DateStart.Value.ToString("yyyyMMdd").Substring(4, 2);
                    TB_Days2.Text = entry.DateStart.Value.ToString("yyyyMMdd").Substring(6, 2);
                }
                if (FLAG == "V9")
                {
                    TB_TicketDateYear1.Text = date.ToString("yyyyMMdd").Substring(0, 4);
                    TB_Months2.Text = date.ToString("yyyyMMdd").Substring(4, 2);
                    TB_Days2.Text = date.ToString("yyyyMMdd").Substring(6, 2);
                }
            }
            else
            {
                TB_TicketDateYear.Text = TB_Months1.Text = TB_Days1.Text = "---";
                if (FLAG == "V11")
                {
                    TB_TicketDateYear1.Text = TB_Months2.Text = TB_Days2.Text = "---";
                }
            }
            TB_Recheck.Text = PrintAttribute.Authentic;
            TB_Handling.Text = LoginUserInfo.TellerNo + "(" + LoginUserInfo.TellerName + ")";
            if (!string.IsNullOrWhiteSpace(entry.CustomerManager))
            {
                TB_CustomerManager.Text = "客户经理号: " + entry.CustomerManager;
            }
            if (FLAG != "V11")
            {
                this.E_Seal_Location = new Point(62, 207);
            }

            //20190611 修改冠县、新泰、蒙阴的单位定期存单的凭证打印  修改人：张若禺
            if (FLAG == "V16" || FLAG == "V17" || FLAG == "V18" || FLAG == "V19")
            {
                TB_AccountName.SetValue(Canvas.TopProperty, 63.0);
                TB_AccountName.SetValue(Canvas.LeftProperty, 139.0);

                TB_Account.SetValue(Canvas.TopProperty, 93.0);
                TB_Account.SetValue(Canvas.LeftProperty, 139.0);

                TB_InterestRate.SetValue(Canvas.TopProperty, 120.0);
                TB_InterestRate.SetValue(Canvas.LeftProperty, 139.0);

                TB_TicketDateYear.SetValue(Canvas.TopProperty, 63.0);
                TB_TicketDateYear.SetValue(Canvas.LeftProperty, 471.0);

                TB_Months1.SetValue(Canvas.TopProperty, 63.0);
                TB_Months1.SetValue(Canvas.LeftProperty, 522.0);

                TB_Days1.SetValue(Canvas.TopProperty, 63.0);
                TB_Days1.SetValue(Canvas.LeftProperty, 563.0);

                TB_Months.SetValue(Canvas.TopProperty, 93.0);

                TB_DateEndYear.SetValue(Canvas.TopProperty, 120.0);
                TB_DateEndYear.SetValue(Canvas.LeftProperty, 471.0);

                TB_DateEndMonths.SetValue(Canvas.TopProperty, 120.0);
                TB_DateEndMonths.SetValue(Canvas.LeftProperty, 522.0);

                TB_DateEndDays.SetValue(Canvas.TopProperty, 120.0);
                TB_DateEndDays.SetValue(Canvas.LeftProperty, 563.0);

                TB_AmountMin.SetValue(Canvas.TopProperty, 163.0);

                TB_Recheck.SetValue(Canvas.TopProperty, 290.0);
                TB_Recheck.SetValue(Canvas.LeftProperty, 381.0);
                TB_Recheck.SetValue(Canvas.WidthProperty, 121.0);

                TB_Handling.SetValue(Canvas.TopProperty, 291.0);
                TB_Handling.SetValue(Canvas.LeftProperty, 577.0);

            }
            if (FLAG == "V11")
            {
                TB_AmountMax.SetValue(Canvas.TopProperty, 185.0);
                TB_AmountMax.SetValue(Canvas.LeftProperty, 180.0);
                TB_AmountMin.SetValue(Canvas.TopProperty, 175.0);
                TB_AmountMin.SetValue(Canvas.LeftProperty, 515.0);
                TB_AmountMin.FontSize = 20;
                TB_TicketDateYear1.Visibility = TB_Months2.Visibility = TB_Days2.Visibility = Visibility.Visible;
                
            }
            if (FLAG == "V9")
            {
                TB_TicketDateYear1.Visibility = TB_Months2.Visibility = TB_Days2.Visibility = Visibility.Visible;
            }
        }


       
    }
}
