﻿using System;
using System.Globalization;
using System.Linq;
using System.Windows;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using System.Windows.Controls;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    ///单位定期存款证实书
    /// </summary>
    public partial class Print_EnterpriseRegularlyConfirmation : IPrintContract
    {
        public Print_EnterpriseRegularlyConfirmation(Print_EnterpriseRegularlyConfirmationEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public PrintEntry PrintData
        {
            get;
            set;
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_EnterpriseRegularlyConfirmationEntry;
           // TB_Subject.Text = entry.Subject;
            if (entry.DateStart != null)
                TB_DateStart.Text = entry.DateStart.Value.ToString("yyyy/MM/dd");
            if (entry.DateInterest != null)
                TB_DateInterest.Text = entry.DateInterest.Value.ToString("yyyy/MM/dd");
            TB_AccessType.Text = entry.AccessType;
            TB_zqfs.Text = entry.zqfs;
            TB_DepositTerm.Text = entry.DepositTerm;
            if (entry.DateEnd != null)
                TB_DateEnd.Text = entry.DateEnd.Value.ToString("yyyy/MM/dd");
            TB_Operation.Text = entry.Operation;
            TB_Account.Text = entry.Account;
            TB_AccountName.Text = entry.AccountName;
            TB_AmountMax.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
            var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            TB_AmountMin.Text = type + entry.Amount.ToString("N2");
            var FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            if (!entry.AccessType.Contains("通知存款"))
            {
                //TB_InterestRate.Text = string.Format("{0:N6}%", entry.InterestRate);
                TB_zdzc.Text = entry.zdzc == "到期不转存" ? "否" : "是";
                if (FLAG == "V2" && LoginUserInfo.KinbrNo.Substring(0, 3) == "152")
                {
                    //整体字体缩小，每一项都不要出格
                    TB_DateStart.FontSize = TB_AmountMin.FontSize = TB_DepositTerm.FontSize = TB_zqfs.FontSize = TB_InterestRate.FontSize = TB_DateInterest.FontSize = TB_DateEnd.FontSize = TB_dqx.FontSize = TB_zdzc.FontSize = TB_Operation.FontSize = 10;
                    //存款利息精确到小数点后3位，例如3.500%
                    TB_InterestRate.Text = string.Format("{0:N3}%", entry.InterestRate);
                    TB_InterestRate.SetValue(Canvas.LeftProperty, 292.00);
                    //到期利息，每三位加逗号
                    TB_dqx.Text = double.Parse(entry.dqx).ToString("N2");
                    //期限，支取方式 右移
                    TB_DepositTerm.SetValue(Canvas.LeftProperty, 186.00);
                    TB_zqfs.SetValue(Canvas.LeftProperty, 227.00);
                }
                else
                {
					TB_InterestRate.Text = string.Format("{0:N6}%", entry.InterestRate);
                    TB_dqx.Text = entry.dqx;
                }
            }
            else
            {
                TB_InterestRate.Text = string.Empty;
                TB_zdzc.Text = string.Empty;
                TB_dqx.Text = string.Empty;
                if (FLAG == "V0")//中牟通知存款到期日
                {
                    TB_DateEnd.Text = "   ---";
                }
            }
            //20150527z证实书不打复核人 魏文哲提出
            //公共要素
           // TB_Recheck.Text = PrintAttribute.Review;
            TB_Recheck.Text = "";
            TB_Info.Text = entry.Info;
            //TB_Handling.Text = LoginUserInfo.TellerNo + "(" + LoginUserInfo.TellerName + ")";
            switch (FLAG)
            {
                case"V8":
                case "V12"://长春融丰
                    TB_AccountName.SetValue(Canvas.TopProperty, 98.00);
                    TB_Account.SetValue(Canvas.TopProperty, 98.00);
                    TB_AccessType.SetValue(Canvas.TopProperty, 98.00);
                    TB_AmountMax.SetValue(Canvas.TopProperty, 175.00);
                    break;
                case "V5":
                    this.ElecSealHelper.Sign2Point(new Point(490, 150), TellerSystem.PrintTemplate.ElecSeal.SealKind.Biz);
                    BZ.Visibility = Visibility.Visible;
                    if (LoginUserInfo.KinbrNo.Substring(0, 3) == "311") //周村 需要调整最后一行往上5个单位
                    {
                        TB_DateStart.SetValue(Canvas.TopProperty, 255.00);
                        TB_AmountMin.SetValue(Canvas.TopProperty, 255.00);
                        TB_DepositTerm.SetValue(Canvas.TopProperty, 255.00);
                        TB_zqfs.SetValue(Canvas.TopProperty, 255.00);
                        TB_InterestRate.SetValue(Canvas.TopProperty, 255.00);
                        TB_DateInterest.SetValue(Canvas.TopProperty, 255.00);
                        TB_DateEnd.SetValue(Canvas.TopProperty, 255.00);
                        TB_dqx.SetValue(Canvas.TopProperty, 255.00);
                        TB_zdzc.SetValue(Canvas.TopProperty, 255.00);
                        TB_Operation.SetValue(Canvas.TopProperty, 255.00);
                    }
                    break;
                default:
                    break;
            }
        }

        public class Print_EnterpriseRegularlyConfirmationEntry : PrintEntry
        {
            /// <summary>
            /// 科目
            /// </summary>
          //  public string Subject { get; set; }

            /// <summary>
            /// 存入日
            /// </summary>
            public DateTime? DateStart { get; set; }

            /// <summary>
            /// 起息日
            /// </summary>
            public DateTime? DateInterest { get; set; }

            /// <summary>
            /// 密/印标志
            /// </summary>
            public string AccessType { get; set; }

            /// <summary>
            /// 信息
            /// </summary>
            public string Info { get; set; }

            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public decimal? InterestRate { get; set; }

            /// <summary>
            /// 到期日
            /// </summary>
            public DateTime? DateEnd { get; set; }

            /// <summary>
            /// 操作
            /// </summary>
            public string Operation { get; set; }

            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }

            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }

            /// <summary>
            /// 存入金额
            /// </summary>
            public decimal Amount { get; set; }

            /// <summary>
            /// 自动转存
            /// </summary>
            public string zdzc { get; set; } 

            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }
            /// <summary>
            /// 到期息
            /// </summary>
            public string dqx { get; set; }
             /// <summary>
            /// 支取方式
            /// </summary>
            public string zqfs { get; set; }
        }
    }
}