﻿using System;
using System.Globalization;
using System.Linq;
using System.Windows;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using System.Windows.Controls;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    ///单位定期存款证实书
    /// </summary>
    public partial class Print_EnterpriseRegularlyConfirmationTJ : IPrintContract
    {
        public Print_EnterpriseRegularlyConfirmationTJ(Print_EnterpriseRegularlyConfirmationTJEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public PrintEntry PrintData
        {
            get;
            set;
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_EnterpriseRegularlyConfirmationTJEntry;

            string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");

            this.Year.Text = LoginUserInfo.TradeDate.Substring(0, 4).Trim();
            this.Month.Text = LoginUserInfo.TradeDate.Substring(4, 2).Trim();
            this.Day.Text = LoginUserInfo.TradeDate.Substring(6, 2).Trim();

            TB_Account.Text = entry.Account;
            TB_AccountName.Text = entry.AccountName;
            if (entry.AccessType.Contains("单位定期") && flag == "V21")
            {
                TB_AccessType.Text = "单位定期存款";
            }
            else
            {
                TB_AccessType.Text = entry.AccessType;
            }
            TB_AmountMax.Text = "人民币" + entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);

            if (entry.DateInterest != null)
                TB_DateInterest.Text = entry.DateInterest.Value.ToString("yyyy/MM/dd");
            TB_zqfs.Text = entry.zqfs;
            TB_DepositTerm.Text = entry.DepositTerm;
            if (entry.DateEnd != null)
                TB_DateEnd.Text = entry.DateEnd.Value.ToString("yyyy/MM/dd");
            if (!entry.AccessType.Contains("通知存款"))
            {
                TB_InterestRate.Text = entry.InterestRate.ToString().Trim('0')+"%";
                TB_zdzc.Text = entry.zdzc == "到期不转存" ? "否" : "是";
            }
            else
            {
                if (flag == "V21")
                {
                    TB_InterestRate.Text = entry.InterestRate.ToString().Trim('0') + "%"; 
                }
                else
                {
                    TB_InterestRate.Text = string.Empty;
                }
                TB_zdzc.Text = string.Empty;
            }
            TB_Recheck.Text = "";
            TB_sq.Text = this.PrintAttribute.Authentic;
            TB_Handling.Text = LoginUserInfo.TellerName + "(" + LoginUserInfo.TellerNo + ")";
            
            if (flag == "V21")
            {
                canvas_main.Margin = new Thickness(0, -20, 0, 0);
            }
            if (flag == "V22")
            {
                BZ.Visibility = Visibility.Visible;
            }
        }

        public class Print_EnterpriseRegularlyConfirmationTJEntry : PrintEntry
        {
            /// <summary>
            /// 起息日
            /// </summary>
            public DateTime? DateInterest { get; set; }

            /// <summary>
            /// 密/印标志
            /// </summary>
            public string AccessType { get; set; }

            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public decimal? InterestRate { get; set; }

            /// <summary>
            /// 到期日
            /// </summary>
            public DateTime? DateEnd { get; set; }

            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }

            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }

            /// <summary>
            /// 存入金额
            /// </summary>
            public decimal Amount { get; set; }

            /// <summary>
            /// 自动转存
            /// </summary>
            public string zdzc { get; set; }

            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }

            /// <summary>
            /// 支取方式
            /// </summary>
            public string zqfs { get; set; }
        }
    }
}