﻿using System;
using System.Globalization;
using System.Linq;
using System.Windows;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using System.Windows.Controls;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    ///单位定期存款证实书
    /// </summary>
    public partial class Print_EnterpriseRegularlyConfirmationXX : IPrintContract
    {
        public Print_EnterpriseRegularlyConfirmationXX(Print_EnterpriseRegularlyConfirmationXXEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public PrintEntry PrintData
        {
            get;
            set;
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_EnterpriseRegularlyConfirmationXXEntry;
            TB_KinbrNo.Text = LoginUserInfo.KinbrName;
            TB_AccessType.Text = entry.AccessType;
            TB_Account.Text = entry.Account;
            TB_AccountName.Text = entry.AccountName;
            TB_AmountMax.Text = "人民币" + entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
           
            if (entry.DateStart != null)
                TB_DateStart.Text = entry.DateStart.Value.ToString("yyyy/MM/dd");
            if (entry.DateInterest != null)
                TB_DateInterest.Text = entry.DateInterest.Value.ToString("yyyy/MM/dd");           
                                                        
            var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            TB_AmountMin.Text = type + entry.Amount.ToString("N2");
            if (!entry.AccessType.Contains("通知存款"))
            {
                TB_DepositTerm.Text = entry.DepositTerm;
                TB_InterestRate.Text = entry.InterestRate.ToString().Trim('0')+"%";
                if (entry.DateEnd != null)
                    TB_DateEnd.Text = entry.DateEnd.Value.ToString("yyyy/MM/dd");
            }
            else
            {
                TB_DepositTerm.Text = string.Empty;
                TB_InterestRate.Text = string.Empty;
                TB_DateEnd.Text = string.Empty;
            }
        
            TB_Info.Text = entry.Info;                       
        }

        public class Print_EnterpriseRegularlyConfirmationXXEntry : PrintEntry
        {
            
            /// <summary>
            /// 存入日
            /// </summary>
            public DateTime? DateStart { get; set; }

            /// <summary>
            /// 起息日
            /// </summary>
            public DateTime? DateInterest { get; set; }
            
            /// <summary>
            /// 信息
            /// </summary>
            public string Info { get; set; }

            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public decimal? InterestRate { get; set; }

            /// <summary>
            /// 到期日
            /// </summary>
            public DateTime? DateEnd { get; set; }
          
            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }

            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }

            /// <summary>
            /// 存入金额
            /// </summary>
            public decimal Amount { get; set; }

            /// <summary>
            /// 储种
            /// </summary>
            public string AccessType { get; set; }

            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }
                        
        }
    }
}