﻿using System;
using System.Globalization;
using System.Linq;
using System.Windows;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 担保保证金
    /// </summary>
    public partial class Print_GuaranteeSpecialCDs : IPrintContract
    {
        /// <summary>
        /// 
        /// </summary>
        public Print_GuaranteeSpecialCDs(Print_GuaranteeSpecialCDsEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }


        /// <summary>
        /// 
        /// </summary>
        public Contract.Entry.PrintEntry PrintData
        {
            get;
            set;
        }
        /// <summary>
        /// 
        /// </summary>
        public void LoadPrintData()
        {
            var entry = PrintData as Print_GuaranteeSpecialCDsEntry;
            TB_Subject.Text = entry.Subject;
            if (entry.DateStart != null)
                TB_DateStart.Text = entry.DateStart.Value.ToString("yyyy/MM/dd");
            if (entry.DateInterest != null)
                TB_DateInterest.Text = entry.DateInterest.Value.ToString("yyyy/MM/dd");
            TB_AccessType.Text = entry.AccessType;
            TB_Type.Text = entry.Type;
            TB_DepositTerm.Text = entry.DepositTerm;
            TB_InterestRate.Text = string.Format("{0:N6}%",entry.InterestRate);
            if (entry.DateEnd != null)
                TB_DateEnd.Text = entry.DateEnd.Value.ToString("yyyy/MM/dd");
            TB_Operation.Text = entry.Operation;
            TB_Account.Text = entry.Account;
            TB_AccountName.Text = entry.AccountName;
            TB_AmountMax.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
            var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            TB_AmountMin.Text = type + entry.Amount.ToString("N2");
            //公共要素
            TB_Recheck.Text = PrintAttribute.Review;
            TB_Handling.Text = LoginUserInfo.TellerNo + "(" + LoginUserInfo.TellerName + ")";
        }

        /// <summary>
        /// 
        /// </summary>
        public class Print_GuaranteeSpecialCDsEntry : PrintEntry
        {
            /// <summary>
            /// 科目
            /// </summary>
            public string Subject { get; set; }

            /// <summary>
            /// 存入日
            /// </summary>
            public DateTime? DateStart { get; set; }

            /// <summary>
            /// 起息日
            /// </summary>
            public DateTime? DateInterest { get; set; }

            /// <summary>
            /// 密/印标志
            /// </summary>
            public string AccessType { get; set; }

            /// <summary>
            /// 储种
            /// </summary>
            public string Type { get; set; }

            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public decimal InterestRate { get; set; }

            /// <summary>
            /// 到期日
            /// </summary>
            public DateTime? DateEnd { get; set; }

            /// <summary>
            /// 操作
            /// </summary>
            public string Operation { get; set; }

            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }

            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }

            /// <summary>
            /// 存入金额
            /// </summary>
            public decimal Amount { get; set; }

            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }
        }
    }
}