﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// Print_LostPage_CCRF.xaml 的交互逻辑
    /// </summary>
    public partial class Print_LostPage_CCRF : IPrintContract
    {
        public Print_LostPage_CCRF(Print_LostPage_CCRF_Entry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }
        public Contract.Entry.PrintEntry PrintData { get; private set; }
        public void LoadPrintData()
        {
            var entry = PrintData as Print_LostPage_CCRF_Entry;
            if (entry == null) return;
            Year.Text = entry.Year;
            Month.Text = entry.Month;
            Day.Text = entry.Day;
            KHMC.Text = entry.KHMC;
            KHDZ.Text = entry.KHDZ;
            KHDH.Text = entry.KHDH;
            SFSFZ.Visibility = entry.SFSFZ?Visibility.Visible:Visibility.Hidden;
            KHZJHM.Text = string.Join(" ", entry.KHZJHM.ToArray<char>());
            SLRQ.SetPrintTextBlock(entry.SLRQ);
            GSSQSBH.SetPrintTextBlock(entry.GSSQSBH);
            ZH.SetPrintTextBlock(entry.ZH);
            HM.SetPrintTextBlock(entry.HM);
            GSZL.SetPrintTextBlock(entry.GSZL);
            GSPZHM.SetPrintTextBlock(entry.GSPZHM);
            SQ.SetPrintTextBlock(entry.SQ);
            JB.SetPrintTextBlock(entry.JB);
            LSH.SetPrintTextBlock(entry.LSH);
            HX_CLSJ.SetPrintTextBlock(entry.HX_CLSJ);
            HX_CLJG.SetPrintTextBlock(entry.HX_CLJG);
            HX_ZH.SetPrintTextBlock(entry.HX_ZH);
            HX_SQ.SetPrintTextBlock(entry.HX_SQ);
            HX_JB.SetPrintTextBlock(entry.HX_JB);
            HX_LSH.SetPrintTextBlock(entry.HX_LSH);
            if (string.IsNullOrEmpty(entry.HX_CLSJ))
            {  //电子印章
                this.E_Seal_Location = new Point(505, 675);
            }
            else
            {
                //电子印章
                this.E_Seal_Location = new Point(505, 925);
            }
            if (entry.SFDYCDY == "Y")
            {
                //若第一次打印,挂失处理显示，后续处理隐藏
                canvas1.Visibility = Visibility.Visible;
                canvas2.Visibility = Visibility.Collapsed;
               
            }
            else {
                //第二次打印，挂失处理隐藏，后续处理显示
                canvas1.Visibility = Visibility.Collapsed;
                canvas2.Visibility = Visibility.Visible;
            }
        }
        public class Print_LostPage_CCRF_Entry : PrintEntry {
            /// <summary>
            /// 年
            /// </summary>
            public string Year { get; set; }
            /// <summary>
            /// 月
            /// </summary>
            public string Month { get; set; }
            /// <summary>
            /// 日
            /// </summary>
            public string Day { get; set; }
            /// <summary>
            /// 客户名称
            /// </summary>
            public string KHMC { get; set; }
            /// <summary>
            /// 客户地址
            /// </summary>
            public string KHDZ { get; set; }
            /// <summary>
            /// 客户电话
            /// </summary>
            public string KHDH { get; set; }
            /// <summary>
            /// 是否身份证
            /// </summary>
            public bool SFSFZ { get; set; }
            /// <summary>
            /// 客户证件号码
            /// </summary>
            public string KHZJHM { get; set; }
            /// <summary>
            /// 受理日期
            /// </summary>
            public string SLRQ { get; set; }
            /// <summary>
            /// 挂失申请书编号
            /// </summary>
            public string GSSQSBH { get; set; }
            /// <summary>
            /// 账号
            /// </summary>
            public string ZH { get; set; }
            /// <summary>
            /// 户名
            /// </summary>
            public string HM { get; set; }
            /// <summary>
            /// 挂失种类
            /// </summary>
            public string GSZL { get; set; }
            /// <summary>
            /// 挂失凭证号码
            /// </summary>
            public string GSPZHM { get; set; }
            /// <summary>
            /// 授权
            /// </summary>
            public string SQ { get; set; }
            /// <summary>
            /// 经办
            /// </summary>
            public string JB { get; set; }
            /// <summary>
            /// 流水号
            /// </summary>
            public string LSH { get; set; }
            /// <summary>
            /// 受理日期(后续处理)
            /// </summary>
            public string HX_CLSJ { get; set; }
            /// <summary>
            /// 处理结果(后续处理)
            /// </summary>
            public string HX_CLJG { get; set; }
            /// <summary>
            /// 账号(后续处理)
            /// </summary>
            public string HX_ZH { get; set; }
            /// <summary>
            /// 授权(后续处理)
            /// </summary>
            public string HX_SQ { get; set; }
            /// <summary>
            /// 经办(后续处理)
            /// </summary>
            public string HX_JB { get; set; }
            /// <summary>
            /// 流水号(后续处理)
            /// </summary>
            public string HX_LSH { get; set; }
            /// <summary>
            /// 是否第一次打印
            /// </summary>
            public string SFDYCDY { get; set; }
        }

    }
}
