﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// Interaction logic for Print_PersonalAccountingVoucher_Close.xaml
    /// </summary>
    public partial class Print_PersonalAccountingVoucher_Close : IPrintContract
    {
        public Print_PersonalAccountingVoucher_Close(Print_PersonalAccountingVoucher_CloseEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
            this.BusinessType = "销户";
        }

        public class Print_PersonalAccountingVoucher_CloseEntry : PrintEntry
        {
            /// <summary>
            /// 账号
            /// </summary>
            public string Acno { get; set; }

            /// <summary>
            /// 账户序号
            /// </summary>
            public string ZHXH { get; set; }

            /// <summary>
            /// 账户户名
            /// </summary>
            public string ZHHM { get; set; }

            /// <summary>
            /// 开户行名称
            /// </summary>
            public string OpnBrName { get; set; }

            /// <summary>
            /// 储种
            /// </summary>
            public string PrdtName { get; set; }

            /// <summary>
            /// 存期
            /// </summary>
            public string Term { get; set; }

            /// <summary>
            /// 凭证号码
            /// </summary>
            public string NoteNo { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public decimal Rate { get; set; }

            /// <summary>
            /// 凭证号码
            /// </summary>
            public string PZHM { get; set; }

            /// <summary>
            /// 交易金额
            /// </summary>
            public decimal PayAmt { get; set; }

            /// <summary>
            /// 余额
            /// </summary>
            public decimal Bal { get; set; }

            /// <summary>
            /// 现转标志
            /// </summary>
            public string DtInd { get; set; }

            /// <summary>
            /// 交易类型--销户方式
            /// </summary>
            public string TradeType { get; set; }

            /// <summary>
            /// 对方账号
            /// </summary>
            public string DfAcno { get; set; }

            /// <summary>
            /// 对方户名
            /// </summary>
            public string DfAcname { get; set; }
           

            /// <summary>
            /// 科目
            /// </summary>
            public string Subject { get; set; }
            
        }

        #region IPrintContract Members

        public Contract.Entry.PrintEntry PrintData { get; set; }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_PersonalAccountingVoucher_CloseEntry;
            if (entry == null)
                return;
            TB_Acno1.SetPrintTextBlock(entry.Acno);
            TB_Acno2.SetPrintTextBlock(entry.Acno);
            TB_ZHXH.SetPrintTextBlock(entry.ZHXH);
            TB_HM1.SetPrintTextBlock(entry.ZHHM);
            TB_HM2.SetPrintTextBlock(entry.ZHHM);
            TB_OPNNAME.SetPrintTextBlock(entry.OpnBrName);
            TB_Rate1.SetPrintTextBlock(string.Format("{0:N6}%", entry.Rate));
            TB_Rate2.SetPrintTextBlock(string.Format("{0:N6}%", entry.Rate));
            TB_Prdt1.SetPrintTextBlock(entry.PrdtName);
            TB_Prdt2.SetPrintTextBlock(entry.PrdtName);
            if(string.IsNullOrEmpty(entry.Term))
            {
                TB_Term1.Visibility = Visibility.Hidden;
                TB_Term2.Visibility = Visibility.Hidden;
            }
            else
            {
                TB_Term1.SetPrintTextBlock(entry.Term);
                TB_Term2.SetPrintTextBlock(entry.Term);
            }
            TB_Note1.SetPrintTextBlock(entry.NoteNo.PadLeft(TellerSystem.Library.Ext.Variables.UserVariable.Len_VoucherNo, '0'));
            TB_TxAmt1.SetPrintTextBlock("¥" + entry.PayAmt.ToString("N2"));
            TB_TxAmt2.SetPrintTextBlock("¥" + entry.PayAmt.ToString("N2"));
            TB_Bal1.SetPrintTextBlock("¥" + entry.Bal.ToString("N2"));
            TB_Bal2.SetPrintTextBlock("¥" + entry.Bal.ToString("N2"));
            TB_CtInd1.SetPrintTextBlock(entry.DtInd);
            TB_CtInd2.SetPrintTextBlock(entry.DtInd);
            TB_TradeType1.SetPrintTextBlock(entry.TradeType);
            if(string.IsNullOrEmpty(entry.DfAcno))
            {
                TB_DfAcno1.Visibility = Visibility.Hidden;
                TB_DfAcno2.Visibility = Visibility.Hidden;
            }
            else
            {
                TB_DfAcno1.SetPrintTextBlock(entry.DfAcno);
                TB_DfAcno2.SetPrintTextBlock(entry.DfAcno);
            }
            if (string.IsNullOrEmpty(entry.DfAcname))
            {
                TB_DfAcname1.Visibility = Visibility.Hidden;
                TB_DfAcname2.Visibility = Visibility.Hidden;
            }
            else
            {
                TB_DfAcname1.SetPrintTextBlock(entry.DfAcname);
                TB_DfAcname2.SetPrintTextBlock(entry.DfAcname);
            }
            //TB_Subject.Text = entry.Subject;

            //公共要素
            //交易日期
            TB_Teller2.SetPrintTextBlock(LoginUserInfo.TellerNo);
        }

        #endregion
    }
}
