﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using Platform.Common.RunningParameters;
using System.Data;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 存款证明
    /// </summary>
    public partial class Print_PersonalCDsRecord : IPrintContract
    {
        public Print_PersonalCDsRecord(Print_PersonalCDsRecordEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData { get; private set; }

        /// <summary>
        /// 凭证的打印数据实体定义
        /// </summary>
        public class Print_PersonalCDsRecordEntry : PrintEntry
        {
            /**********************************
             * 定义实体属性
             * 1.打印逻辑用到的要素
             * *******************************/
            /// <summary>
            /// 客户名称
            /// </summary>
            public string CifName { get; set; }

            /// <summary>
            /// 客户英文名称
            /// </summary>
            public string CifNameEn { get; set; }

            /// <summary>
            /// 截止日期
            /// </summary>
            public DateTime DateEnd { get; set; }

            /// <summary>
            /// 存款记录集合
            /// </summary>
            public List<CDsRecord> Records { get; set; }

            public class CDsRecord
            {
                /// <summary>
                /// 账号
                /// </summary>
                public string Account { get; set; }
                /// <summary>
                /// 种类
                /// </summary>
                public string Type { get; set; }
                /// <summary>
                /// 金额
                /// </summary>
                public decimal Amount { get; set; }
                /// <summary>
                /// 存入日
                /// </summary>
                public DateTime? DateStart { get; set; }
            }
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_PersonalCDsRecordEntry;
            //传入要素 
            TB_CifName.Text = entry.CifName;
            TB_CifNameEn.Text = entry.CifNameEn;
            TB_DateEnd_Year.Text = entry.DateEnd.Year.ToString();
            TB_DateEnd_Month.Text = entry.DateEnd.Month.ToString();
            TB_DateEnd_Day.Text = entry.DateEnd.Day.ToString();
            TB_DateEnd.Text = entry.DateEnd.ToString("dd/MM/yyyy");
            //集合数据
            var table = new DataTable();
            table.Columns.Add("Account");
            table.Columns.Add("Type");
            table.Columns.Add("Amount");
            table.Columns.Add("DateStart");
            entry.Records.ForEach(x =>
            {
                if (x.DateStart.HasValue)
                {
                    table.Rows.Add(x.Account,
                                   x.Type,
                                   entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                                   x.DateStart.Value.ToString("yyyy/MM/dd")
                        );
                }
                else
                {
                    table.Rows.Add(x.Account,
                                  x.Type,
                                  entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                                  "****"
                       );
                }
            });
            IC_RecordList.ItemsSource = table.DefaultView;
            //公共要素
            var date = DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd", null);
            TB_Date.Text = date.ToString("yyyy/MM/dd");
        }
    }
}
