﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using Platform.Common.RunningParameters;
using System.Data;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 存款证明
    /// </summary>
    public partial class Print_PersonalCDsRecordLF : IPrintContract
    {
        public Print_PersonalCDsRecordLF(Print_PersonalCDsRecordLFEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData { get; private set; }

        /// <summary>
        /// 凭证的打印数据实体定义
        /// </summary>
        public class Print_PersonalCDsRecordLFEntry : PrintEntry
        {
            /**********************************
             * 定义实体属性
             * 1.打印逻辑用到的要素
             * *******************************/
            /// <summary>
            /// 客户名称
            /// </summary>
            public string CifName { get; set; }

            /// <summary>
            /// 客户英文名称
            /// </summary>
            public string CifNameEn { get; set; }

            /// <summary>
            /// 截止日期
            /// </summary>
            public DateTime DateEnd { get; set; }

            /// <summary>
            /// 身份证
            /// </summary>
            public string IDNO { get; set; }
            /// <summary>
            /// 身份证英文
            /// </summary>
            public string IDNOENG { get; set; }

            /// <summary>
            /// 存款金额
            /// </summary>
            public string AccountAll { get; set; }

            /// <summary>
            /// 存款记录集合
            /// </summary>
            public List<CDsRecord> Records { get; set; }

            public class CDsRecord
            {
                /// <summary>
                /// 账号
                /// </summary>
                public string Account { get; set; }
                /// <summary>
                /// 种类
                /// </summary>
                public string Type { get; set; }
                /// <summary>
                /// 金额
                /// </summary>
                public decimal Amount { get; set; }
                /// <summary>
                /// 到期日
                /// </summary>
                public DateTime? DateEnd { get; set; }
            }
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_PersonalCDsRecordLFEntry;
            //传入要素 
            TB_CifName.Text = entry.CifName;
            TB_CifNameEn.Text = entry.CifNameEn;
            TB_IDNO.Text = entry.IDNO;
            TB_IDNOEeglish.Text = entry.IDNOENG;
            //CKJE.Text = entry.AccountAll;
            //TB_DateEnd_hour.Text = entry.DateEnd.Hour.ToString();
            //TB_DateEnd_min.Text = entry.DateEnd.mi.ToString();
            TB_DateEnd.Text = entry.DateEnd.ToString("yyyy      MM      dd");
            TB_DateEnd_EN.Text = entry.DateEnd.ToString("yyyy/MM/dd");
            //集合数据
            var table = new DataTable();
            table.Columns.Add("Account");
            table.Columns.Add("Amount");
            table.Columns.Add("Type");
            table.Columns.Add("DateStart");
            entry.Records.ForEach(x =>
            {
                if (x.DateEnd.HasValue)
                {
                    table.Rows.Add(x.Account,
                                   x.Type,
                                   entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                                   x.DateEnd.Value.ToString("yyyy/MM/dd")
                        );
                }
                else
                {
                    table.Rows.Add(x.Account,
                                  x.Type,
                                  entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                                  "****"
                       );
                }
            });
            IC_RecordList.ItemsSource = table.DefaultView;

            //公共要素
            var date = DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd", null);
            TB_Date.Text = date.ToString("yyyy     MM     dd");
            TB_Date_EN.Text = date.ToString("yyyy/MM/dd");

            //版本控制标志
            string FLAG = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            if (FLAG == "V11") //大石桥打印元素整体偏右下，调整
            {
                TB_IDNO.SetValue(Canvas.LeftProperty, 532.00);
                TB_IDNO.SetValue(Canvas.TopProperty, 202.00);
                TB_IDNOEeglish.SetValue(Canvas.LeftProperty, 232.00);
                TB_IDNOEeglish.SetValue(Canvas.TopProperty, 223.00);
                TB_CifName.SetValue(Canvas.LeftProperty, 153.00);
                TB_CifName.SetValue(Canvas.TopProperty, 270.00);
                TB_Date.SetValue(Canvas.LeftProperty, 377.00);
                TB_Date.SetValue(Canvas.TopProperty, 270.00);
                TB_Date_EN.SetValue(Canvas.LeftProperty, 274.00);
                TB_Date_EN.SetValue(Canvas.TopProperty, 294.00);
                TB_CifNameEn.SetValue(Canvas.LeftProperty, 463.00);
                TB_CifNameEn.SetValue(Canvas.TopProperty, 294.00);
                CKJEENG.SetValue(Canvas.LeftProperty, 335.00);
                CKJEENG.SetValue(Canvas.TopProperty, 315.00);
                IC_RecordList.SetValue(Canvas.LeftProperty, 92.00);
                IC_RecordList.SetValue(Canvas.TopProperty, 389.00);
                TB_DateEnd.SetValue(Canvas.LeftProperty, 346.00);
                TB_DateEnd.SetValue(Canvas.TopProperty, 528.00);
                TB_DateEnd_EN.SetValue(Canvas.LeftProperty, 444.00);
                TB_DateEnd_EN.SetValue(Canvas.TopProperty, 554.00);
            }
        }
    }
}
