﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using Platform.Common.RunningParameters;
using System.Data;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 存款证明
    /// </summary>
    public partial class Print_PersonalCDsRecordSD : IPrintContract
    {
        public Print_PersonalCDsRecordSD(Print_PersonalCDsRecordSDEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData { get; private set; }
        string FLAG = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
        /// <summary>
        /// 到期日期
        /// </summary>
        public string DQRQ;
        /// <summary>
        /// 凭证的打印数据实体定义
        /// </summary>
        public class Print_PersonalCDsRecordSDEntry : PrintEntry
        {
            /**********************************
             * 定义实体属性
             * 1.打印逻辑用到的要素
             * *******************************/
            /// <summary>
            /// 客户名称
            /// </summary>
            public string CifName { get; set; }

            /// <summary>
            /// 客户英文名称
            /// </summary>
            public string CifNameEn { get; set; }

            /// <summary>
            /// 凭证号
            /// </summary>
            public string PZHM { get; set; }

            /// <summary>
            /// 证件号码
            /// </summary>
            public string ZJHM { get; set; }

            /// <summary>
            /// 截止日期
            /// </summary>
            public DateTime DateEnd { get; set; }


            /// <summary>
            /// 存款记录集合
            /// </summary>
            public List<CDsRecord> Records { get; set; }

            public class CDsRecord
            {
                /// <summary>
                /// 户名
                /// </summary>
                public string HM { get; set; }
             
                /// <summary>
                /// 账号
                /// </summary>
                public string Account { get; set; }
                /// <summary>
                /// 币种
                /// </summary>
                public string Currency { get; set; }
                /// <summary>
                /// 金额
                /// </summary>
                public decimal Amount { get; set; }
                /// <summary>
                /// 存入日
                /// </summary>
                public DateTime? DateStart { get; set; }
                /// <summary>
                /// 到期日
                /// </summary>
                public DateTime? EndDate { get; set; }
            }
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_PersonalCDsRecordSDEntry;
            //传入要素 
            TB_CifName.Text = entry.CifName;
            TB_CifNameEn.Text = entry.CifNameEn;
            TB_DateEnd_Year.Text = entry.DateEnd.Year.ToString();
            TB_DateEnd_Month.Text = entry.DateEnd.Month.ToString();
            TB_DateEnd_Day.Text = entry.DateEnd.Day.ToString();
            TB_DateEnd.Text = entry.DateEnd.ToString("dd/MM/yyyy");

            EndyearBlock.Text = entry.DateEnd.Year.ToString();
            EndmothBlock.Text = entry.DateEnd.Month.ToString();
            EnddayBlock.Text = entry.DateEnd.Day.ToString();
            PzhmBlock.Text = entry.PZHM;
            IdnoBlock.Text = entry.ZJHM;

            //集合数据
            var table = new DataTable();
            table.Columns.Add("HM");
            table.Columns.Add("Account");
            table.Columns.Add("Currency");
            table.Columns.Add("Amount");
            table.Columns.Add("DateStart");
            table.Columns.Add("EndDate");
            int CKXH = 0;
            if (FLAG == "V2" && LoginUserInfo.KinbrNo.Substring(0, 3) == "152")
            {
                entry.Records.ForEach(x =>
                {
                    if (x.DateStart.HasValue)
                    {
                        if (x.EndDate.Value.ToString("yyyy/MM/dd") == "9999/12/31")
                        {
                            DQRQ = "/";
                        }
                        else
                        {
                            DQRQ = x.EndDate.Value.ToString("yyyy/MM/dd");
                        }
                        table.Rows.Add(x.HM,
                                       x.Account+"    ",
                                       x.Currency,
                                       entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                                       x.DateStart.Value.ToString("yyyy/MM/dd"),
                                       DQRQ
                            );
                        CKXH++;
                    }
                    else
                    {
                        table.Rows.Add(x.HM,
                                       x.Account,
                                       x.Currency,
                                      entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                                      "****",
                                      "****"
                           );
                    }
                });
            }
            else{
            entry.Records.ForEach(x =>
            {
                if (x.DateStart.HasValue)
                {
                    table.Rows.Add(x.HM,
                                   x.Account,
                                   x.Currency,
                                   entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                                   x.DateStart.Value.ToString("yyyy/MM/dd"),
                                   x.EndDate.Value.ToString("yyyy/MM/dd")
                        );
                    CKXH++;
                }
                else
                {
                    table.Rows.Add(x.HM,
                                   x.Account,
                                   x.Currency,
                                  entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                                  "****",
                                  "****"
                       );
                }
            });
            }
            BsxxBlock.Text = CKXH.ToString();
            BsdxBlock.Text = entry.PageData.TradeAPIs.ChangeToCI(CKXH, false, true);
            IC_RecordList.ItemsSource = table.DefaultView;
            //公共要素
            var date = DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd", null);
            TB_Date.Text = date.ToString("yyyy/MM/dd");
            if (FLAG == "V5") {
                this.ElecSealHelper.Sign2Point(new Point(330, 870), TellerSystem.PrintTemplate.ElecSeal.SealKind.Biz);
            }
            if (FLAG == "V2" && LoginUserInfo.KinbrNo.Substring(0, 3) == "152")
            {
                TB_CifNameEn.SetValue(Canvas.TopProperty, 211.00);//230.00
                TB_CifName.SetValue(Canvas.TopProperty, 235.00);//254.00
                IdnoBlock.SetValue(Canvas.TopProperty, 226.00);//245.00
                EndyearBlock.SetValue(Canvas.TopProperty, 272.00);//291.00
                EndmothBlock.SetValue(Canvas.TopProperty, 272.00);//291.00
                EnddayBlock.SetValue(Canvas.TopProperty, 272.00);//291.00
                BsxxBlock.SetValue(Canvas.TopProperty, 272.00);//291.00
                BsdxBlock.SetValue(Canvas.TopProperty, 272.00);//291.00
                IC_RecordList.SetValue(Canvas.TopProperty, 378.00);//397.00
                TB_DateEnd_Year.SetValue(Canvas.TopProperty, 601.00);//620.00
                TB_DateEnd_Month.SetValue(Canvas.TopProperty, 601.00);//620.00
                TB_DateEnd_Day.SetValue(Canvas.TopProperty, 601.00);//620
                TB_DateEnd.SetValue(Canvas.TopProperty, 614.00);//633.00
                TB_CifName.SetValue(Canvas.LeftProperty, 120);//
                IdnoBlock.SetValue(Canvas.LeftProperty, 405);//
                EndyearBlock.SetValue(Canvas.LeftProperty, 104);//
                EndmothBlock.SetValue(Canvas.LeftProperty, 164);//
                EndmothBlock.SetValue(Canvas.LeftProperty, 207.00);
                BsxxBlock.SetValue(Canvas.LeftProperty, 311.00);//
                BsdxBlock.SetValue(Canvas.LeftProperty, 403.00);//
                TB_DateEnd_Year.SetValue(Canvas.LeftProperty, 156.00);//
                TB_DateEnd_Month.SetValue(Canvas.LeftProperty, 223.00);//
                TB_DateEnd_Day.SetValue(Canvas.LeftProperty, 269.00);//
                IC_RecordList.SetValue(Canvas.LeftProperty, 50.00);//
            }
        }
    }
}
