﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using Platform.Common.RunningParameters;
using System.Data;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.Library.Ext.Function;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 存款证明
    /// </summary>
    public partial class Print_PersonalCDsRecordTJ : IPrintContract
    {
        public Print_PersonalCDsRecordTJ(Print_PersonalCDsRecordEntryTJ entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData { get; private set; }

        /// <summary>
        /// 凭证的打印数据实体定义
        /// </summary>
        public class Print_PersonalCDsRecordEntryTJ : PrintEntry
        {
            /**********************************
             * 定义实体属性
             * 1.打印逻辑用到的要素
             * *******************************/

            /// <summary>
            /// 截止日期
            /// </summary>
            public DateTime DateEnd { get; set; }

            /// <summary>
            /// 交易日期
            /// </summary>
            public DateTime JYRQ { get; set; }

            /// <summary>
            /// 申请用途
            /// </summary>
            public string SQYT { get; set; }

            /// <summary>
            /// 客户名称
            /// </summary>
            public string CifName { get; set; }

            /// <summary>
            /// 客户名称英文
            /// </summary>
            public string CifNameEN { get; set; }

            /// <summary>
            /// 证件类型
            /// </summary>
            public string IDTYPE { get; set; }

            /// <summary>
            /// 证件号码
            /// </summary>
            public string IDNO { get; set; }

            /// <summary>
            /// 地址
            /// </summary>
            //public string ADDR { get; set; }

            /// <summary>
            /// 存款金额
            /// </summary>
            public string AccountAll { get; set; }

            /// <summary>
            /// 存款记录集合
            /// </summary>
            public List<CDsRecord> Records { get; set; }

            public class CDsRecord
            {
                /// <summary>
                /// 账号序号
                /// </summary>
                public string AccountXH { get; set; }
                /// <summary>
                /// 账号
                /// </summary>
                public string Account { get; set; }
                /// <summary>
                /// 币种
                /// </summary>
                public string Currency { get; set; }
                /// <summary>
                /// 金额
                /// </summary>
                public decimal Amount { get; set; }
                /// <summary>
                /// 种类
                /// </summary>
                public string Type { get; set; }
                /// <summary>
                /// 存入日
                /// </summary>
                public DateTime? DateStart { get; set; }
                /// <summary>
                /// 到期日
                /// </summary>
                public DateTime? DateEnd { get; set; }
            }
        }

        public void LoadPrintData()
        {
            string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            var entry = PrintData as Print_PersonalCDsRecordEntryTJ;

            TB_DateEnd_Year.Text = entry.DateEnd.Year.ToString();
            TB_DateEnd_Month.Text = entry.DateEnd.Month.ToString();
            TB_DateEnd_Day.Text = entry.DateEnd.Day.ToString();
            TB_DateEnd_Year1.Text = entry.DateEnd.Year.ToString();
            TB_DateEnd_Month1.Text = entry.DateEnd.Month.ToString();
            TB_DateEnd_Day1.Text = entry.DateEnd.Day.ToString();
            TB_DateEnd_Year2.Text = entry.DateEnd.Year.ToString();
            TB_DateEnd_Month2.Text = entry.DateEnd.Month.ToString();
            TB_DateEnd_Day2.Text = entry.DateEnd.Day.ToString();
            TB_DateEnd_Year3.Text = entry.DateEnd.Year.ToString();
            TB_DateEnd_Month3.Text = entry.DateEnd.Month.ToString();
            TB_DateEnd_Day3.Text = entry.DateEnd.Day.ToString();
            if (FLAG == "V22")
            {
                DateEnd_Year.Text = entry.JYRQ.Year.ToString();
                DateEnd_Month.Text = entry.JYRQ.Month.ToString();
                DateEnd_Day.Text = entry.JYRQ.Day.ToString();
                DateEnd_Year.Visibility = DateEnd_Month.Visibility = DateEnd_Day.Visibility = Visibility.Visible;
            }
            
            //传入要素 
            TB_CifName.Text = entry.CifName;
            if (FLAG == "V22")
            {
                TB_CifName1.Text = entry.SQYT;
            }
            else
            {
                TB_CifName1.Text = entry.CifName;
                TB_CifNameEN.Text = entry.CifNameEN;
            }
            TB_IDTYPE.Text = entry.IDTYPE;
            TB_IDNO.Text = entry.IDNO;
            TB_ADDR.Text = "";
            CKJE.Text = "人民币" + SystemFunction.ChangeToCNY(entry.AccountAll.ToDecimal());
            CKJEMin.Text = entry.AccountAll;

            //集合数据
            var table = new DataTable();
            table.Columns.Add("AccountXH");
            table.Columns.Add("Account");
            table.Columns.Add("Variety");
            table.Columns.Add("Currency");
            table.Columns.Add("Amount");
            table.Columns.Add("Type");
            table.Columns.Add("DateStart");
            table.Columns.Add("DateEnd");
            entry.Records.ForEach(x =>
            {
                if (x.DateStart.HasValue)
                {
                    table.Rows.Add(x.AccountXH,
                                   x.Account,
                                   "",
                                   x.Currency,
                                   entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                                   x.Type,
                                   x.DateStart.Value.ToString("yyyy/MM/dd"),
                                   x.DateEnd.Value.ToString("yyyy/MM/dd")
                        );
                }
                else
                {
                    table.Rows.Add(x.AccountXH,
                                   x.Account,
                                   "",
                                   x.Currency,
                                  entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                                  x.Type,
                                  "****",
                                  "****"
                       );
                }
            });
            IC_RecordList.ItemsSource = table.DefaultView;
            
            if (FLAG == "V22")
            {
                PYL.Margin = new Thickness(0,-25,0,0);
            }
        }
    }
}
