﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using Platform.Common.RunningParameters;
using System.Data;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 存款证明
    /// </summary>
    public partial class Print_PersonalCDsRecordYG : IPrintContract
    {
        public Print_PersonalCDsRecordYG(Print_PersonalCDsRecordEntryYG entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData { get; private set; }

        /// <summary>
        /// 凭证的打印数据实体定义
        /// </summary>
        public class Print_PersonalCDsRecordEntryYG : PrintEntry
        {
            /**********************************
             * 定义实体属性
             * 1.打印逻辑用到的要素
             * *******************************/
            /// <summary>
            /// 客户名称
            /// </summary>
            public string CifName { get; set; }

            /// <summary>
            /// 客户英文名称
            /// </summary>
            public string CifNameEn { get; set; }

            /// <summary>
            /// 截止日期
            /// </summary>
            public DateTime DateEnd { get; set; }

            /// <summary>
            /// 身份证
            /// </summary>
            public string IDNO { get; set; }
            /// <summary>
            /// 身份证英文
            /// </summary>
            public string IDNOENG { get; set; }

            /// <summary>
            /// 存款金额
            /// </summary>
            public string AccountAll { get; set; }

            /// <summary>
            /// 存款记录集合
            /// </summary>
            public List<CDsRecord> Records { get; set; }

            public class CDsRecord
            {
                /// <summary>
                /// 账号
                /// </summary>
                public string Account { get; set; }
                /// <summary>
                /// 种类
                /// </summary>
                public string Type { get; set; }
                /// <summary>
                /// 金额
                /// </summary>
                public decimal Amount { get; set; }
                /// <summary>
                /// 存入日
                /// </summary>
                public DateTime? DateStart { get; set; }
            }
        }

        public void LoadPrintData()
        {
            string FLAG = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            var entry = PrintData as Print_PersonalCDsRecordEntryYG;
            //传入要素 
            TB_CifName.Text = entry.CifName;
            TB_CifNameEn.Text = entry.CifNameEn;
            TB_IDNO.Text = entry.IDNO;
            TB_IDNOEeglish.Text = entry.IDNOENG;
            CKJE.Text = entry.AccountAll;
            TB_DateEnd_Year.Text = entry.DateEnd.Year.ToString();
            TB_DateEnd_Month.Text = entry.DateEnd.Month.ToString();
            TB_DateEnd_Day.Text = entry.DateEnd.Day.ToString();
            //新密版本要求证书打印有小时分钟   20190830  融丰增加时分显示 20221024
            if (FLAG == "V3" || FLAG=="V12")
            {
                TB_DateEnd_hour.Text = entry.DateEnd.Hour.ToString();
                TB_DateEnd_min.Text = entry.DateEnd.Minute.ToString();
            }
            TB_DateEnd.Text = entry.DateEnd.ToString("dd/MM/yyyy");
            //集合数据
            var table = new DataTable();
            table.Columns.Add("Account");
            table.Columns.Add("Type");
            table.Columns.Add("Amount");
            table.Columns.Add("DateStart");
            entry.Records.ForEach(x =>
            {
                if (x.DateStart.HasValue)
                {
                    table.Rows.Add(x.Account,
                                   x.Type,
                                   entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                                   x.DateStart.Value.ToString("yyyy/MM/dd")
                        );
                }
                //else
                //{
                //    table.Rows.Add(x.Account,
                //                  x.Type,
                //                  entry.PageData.TradeAPIs.Comma(x.Amount.ToString("N2"), 16),
                //                  "****"
                //       );
                //}
            });
            IC_RecordList.ItemsSource = table.DefaultView;

            //公共要素
            var date = DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd", null);
            TB_Date.Text = date.ToString("yyyy    MM    dd");
            //加载电子印章，是否显示的控制机制在内部处理
            this.E_Seal_Location = new Point(534, 892);
        }
    }
}
