﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.Communication;
using TellerSystem.Communication.MessageHelper;
using TellerSystem.PrintTemplate.ElecSeal;
using TellerSystem.PrintTemplate.Base;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 储蓄存单
    /// </summary>
    public partial class Print_PersonalRegularlyCDs : IPrintContract
    {
        public Print_PersonalRegularlyCDs(Print_PersonalRegularlyCDsEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData
        {
            get;
            private set;
        }

        /// <summary>
        /// 版本控制标志
        /// </summary>
        private string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");

        /// <summary>
        /// 
        /// </summary>
        public class Print_PersonalRegularlyCDsEntry : PrintEntry
        {
            /// <summary>
            /// 存入日
            /// </summary>
            public DateTime? DateStart { get; set; }

            /// <summary>
            /// 起息日
            /// </summary>
            public DateTime? DateInterest { get; set; }

            /// <summary>
            /// 密/印标志
            /// </summary>
            public string AccessType { get; set; }

            /// <summary>
            /// 储种
            /// </summary>
            public string Type { get; set; }

            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public string InterestRate { get; set; }

            /// <summary>
            /// 到期日
            /// </summary>
            public DateTime? DateEnd { get; set; }

            /// <summary>
            /// 操作
            /// </summary>
            public string Operation { get; set; }

            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }

            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }

            /// <summary>
            /// 存入金额
            /// </summary>
            public decimal Amount { get; set; }

            /// <summary>
            /// 自动转存标志
            /// </summary>
            public string AutoZC { get; set; }
            /// <summary>
            /// 开户行
            /// </summary>
            public string OpnBrname { get; set; }

            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }

            /// <summary>
            /// 到期息
            /// </summary>
            public string dqx { get; set; }
            /// <summary>
            /// 标题
            /// </summary>
            public string Title { get; set; }
            /// <summary>
            /// 柜员机构名称
            /// </summary>
            public string KinbrName { get; set; }
            /// <summary>
            /// 存息账号
            /// </summary>
            public string CXZH { get; set; }
            /// <summary>
            /// 凭证号
            /// </summary>
            public string PZH = string.Empty;
        }

        public void LoadPrintData()
        {
            if (flag == "V0" || flag == "V8" || flag == "V12" || flag == "V1")
            {
                TB_Type.FontSize = 12;
            }
            var entry = PrintData as Print_PersonalRegularlyCDsEntry;
            if (entry == null)
                return;
            if (entry.DateStart != null)
                TB_DateStart.Text = entry.DateStart.Value.ToString("yyyy/MM/dd");
            if (entry.DateInterest != null)
                TB_DateInterest.Text = entry.DateInterest.Value.ToString("yyyy/MM/dd");
            TB_AccessType.Text = entry.AccessType;
            //齐丰需求：储种为98天定期存款，198天定期存款，328天定期存款.....998天定期存款时，储种打印为整存整取。
            //修改很无奈,杀个程序员祭天吧！
            if ((flag == "V10" || flag == "V13" || flag == "V14" || flag == "V16" || flag == "V17" || flag == "V18" || flag == "V21" || flag == "V22" || flag == "V23" || flag == "V24") && entry.Type.Contains("8天"))
            {
                entry.Type = "整存整取";
            }
            TB_Type.Text = entry.Type;
            if (string.IsNullOrEmpty(entry.DepositTerm))
            {
                TB_DepositTerm.Text = "---";
            }
            else
            {
                TB_DepositTerm.Text = entry.DepositTerm;
            }
            if (!string.IsNullOrEmpty(entry.InterestRate))
            {
                if (LoginUserInfo.KinbrNo.Substring(0, 3) == "302")//黄骅
                {
                    if (TB_DepositTerm.Text == "三年")
                    {
                        TB_InterestRate.Text = string.Format("{0:N4}%", Convert.ToDouble(entry.InterestRate).ToString("0.0000")); 
                    }
                    else
                    {
                        TB_InterestRate.Text = string.Format("{0:N3}%", Convert.ToDouble(entry.InterestRate).ToString("0.000"));
                    }
                }
                else
                {
                    TB_InterestRate.Text = string.Format("{0:N6}%", entry.InterestRate);
                }
            }
            if (entry.DateEnd != null)
            {
                TB_DateEnd.Text = entry.DateEnd.Value.ToString("yyyy/MM/dd");
            }
            else
            {
                TB_DateEnd.Text = "---";
            }
            TB_Account.Text = entry.Account;
            TB_AccountName.Text = entry.AccountName;
            TB_AmountMax.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
            var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            TB_AmountMin.Text = type + entry.Amount.ToString("N2");
            if (!string.IsNullOrEmpty(entry.AutoZC))
            {
                TB_AutoZC.Text = entry.AutoZC == "到期自动转存" ? "是" : "否";
            }

            TB_dqx.Text = entry.dqx;
            TB_TITLE.Text = entry.Title;
            if (flag == "V5")
            {
                if (entry.Title == "通存不通兑" || entry.Title == "通存通兑")
                {
                    TB_TITLE.Visibility = Visibility.Collapsed;
                }
                TB_TITLE.SetValue(Canvas.TopProperty, 15.00);
                this.ElecSealHelper.Sign2Point(new Point(480, 139), SealKind.Biz);
            }
            else {
                //加载电子印章，是否显示的控制机制在内部处理
                this.E_Seal_Location = new Point(491, 124);
            }
            //公共要素
            TB_Recheck.Text = PrintAttribute.Review;
            //业务要求操作传柜员号
            TB_Operation.Text = LoginUserInfo.TellerNo;
            //郏县、新密需求 存折打印偏上，需调整
            var FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            switch (FLAG)
            {
                case "V0"://中牟
                    break;
                case "V5":
                    if (LoginUserInfo.KinbrNo.Substring(0, 3) != "310")//东阿沿用旧存单
                    {
                        this.PrintType = PrintTypes.PersonalRegularlyCDs_QL;
                    }
                    TB_dqx.Visibility = Visibility.Collapsed;
                    TB_AutoZC.SetValue(Canvas.LeftProperty, 537.0);
                    TB_Operation.SetValue(Canvas.LeftProperty, 587.0);

                    //TB_dqx.SetValue(Canvas.LeftProperty,472.00);
                    TB_DateEnd.SetValue(Canvas.LeftProperty, 421.00);
                    TB_DateInterest.SetValue(Canvas.LeftProperty, 335.00);
                    //TB_InterestRate.SetValue(Canvas.LeftProperty, 285.00);
                    TB_AccessType.SetValue(Canvas.LeftProperty, 222.00);
                    TB_InterestRate.SetValue(Canvas.LeftProperty, 285.00);
                    //TB_DepositTerm.SetValue(Canvas.LeftProperty, 182.00);
                    //TB_AmountMin.SetValue(Canvas.LeftProperty, 80.00);
                    //TB_DateStart.SetValue(Canvas.LeftProperty, 18.00);
                    this.TB_CXZH.SetPrintTextBlock(entry.CXZH);//山东 按期付息产品开户 需要增加 收息账户
                    if (!string.IsNullOrWhiteSpace(entry.CXZH))
                    {
                        TB_CXZH.Visibility = Visibility.Visible;
                    }
                    if (LoginUserInfo.KinbrNo.Substring(0, 3) == "308" && !string.IsNullOrEmpty(entry.dqx))//莘县 整存整取需要到期息
                    {
                        //TB_dqx.Visibility = Visibility.Visible;
                        //TB_dqx.SetValue(Canvas.LeftProperty, 465.00);
                        //TB_DateEnd.SetValue(Canvas.LeftProperty, 400.00);
                        //TB_DateInterest.SetValue(Canvas.LeftProperty, 330.00);
                    }
                    if (LoginUserInfo.KinbrNo.Substring(0, 3) == "302")//黄骅位置调整
                    {
                        TB_TITLE.SetValue(Canvas.TopProperty, 15.00);
                        TB_Account.SetValue(Canvas.TopProperty, 87.00);//账号
                        TB_Type.SetValue(Canvas.TopProperty, 87.00);//储种
                        TB_AmountMax.SetValue(Canvas.TopProperty, 149.00);//大写金额

                        TB_DateStart.SetValue(Canvas.LeftProperty, 20.00);//存入日
                        TB_AmountMin.SetValue(Canvas.LeftProperty, 77.00);//金额
                        TB_DepositTerm.SetValue(Canvas.LeftProperty, 202.00);//期限
                        TB_AccessType.SetValue(Canvas.LeftProperty, 251.00);//支取方式
                        TB_InterestRate.SetValue(Canvas.LeftProperty, 313.00);//利率
                        TB_InterestRate.Width = 80;
                        TB_DateInterest.SetValue(Canvas.LeftProperty, 380.00);//起息日
                        TB_DateEnd.SetValue(Canvas.LeftProperty, 456.00);//到期日
                        TB_AutoZC.SetValue(Canvas.LeftProperty, 568.00);//自动转存

                        TB_Operation.SetValue(Canvas.LeftProperty, 608.00);
                        TB_Handling.SetValue(Canvas.TopProperty, 290.00);
                        TB_Operation.Text = LoginUserInfo.TellerName;//操作员
                        TB_Handling.Text = LoginUserInfo.TellerName;//经办
                        if (!string.IsNullOrEmpty(entry.PZH))
                        {
                            TB_PZH.SetPrintTextBlock(entry.PZH);//凭证号
                            TB_PZH.Visibility = Visibility.Visible;
                        }
                    }
                    if (LoginUserInfo.KinbrNo.Substring(0, 3) == "309" || LoginUserInfo.KinbrNo.Substring(0, 3) == "308")//高唐、莘县 整存整取需要到期息
                    {
                        //TB_dqx.Visibility = Visibility.Visible;
                        //TB_dqx.SetValue(Canvas.LeftProperty, 485.00);
                        //TB_dqx.SetValue(Canvas.TopProperty, 248.00);
                        //TB_AutoZC.SetValue(Canvas.LeftProperty, 560.00);//自动转存
                        TB_DepositTerm.SetValue(Canvas.LeftProperty, 182.00);//期限
                        TB_AccessType.SetValue(Canvas.LeftProperty, 231.00);//支取方式
                        TB_InterestRate.SetValue(Canvas.LeftProperty, 303.00);//利率
                        //TB_InterestRate.Width = 80;
                        TB_DateInterest.SetValue(Canvas.LeftProperty, 380.00);//起息日
                        TB_DateEnd.SetValue(Canvas.LeftProperty, 456.00);//到期日
                    }
                    if (LoginUserInfo.KinbrNo.Substring(0, 3) == "310" && !string.IsNullOrEmpty(entry.dqx))//东阿 整存整取需要到期息
                    {
                        TB_dqx.Visibility = Visibility.Visible;
                        TB_dqx.SetValue(Canvas.LeftProperty, 485.00);
                    }
                    BZ.Visibility = Visibility.Visible;
                    break;
                case "V8"://阳光
                case "V12"://长春融丰
                    //向右偏移
                    TB_dqx.SetValue(Canvas.LeftProperty, 472.00);
                    TB_DateEnd.SetValue(Canvas.LeftProperty, 399.00);
                    TB_DateInterest.SetValue(Canvas.LeftProperty, 328.00);
                    TB_InterestRate.SetValue(Canvas.LeftProperty, 285.00);
                    TB_AccessType.SetValue(Canvas.LeftProperty, 219.00);
                    TB_DepositTerm.SetValue(Canvas.LeftProperty, 182.00);
                    TB_AmountMin.SetValue(Canvas.LeftProperty, 80.00);
                    TB_DateStart.SetValue(Canvas.LeftProperty, 18.00);
                    //向上偏移
                    TB_dqx.SetValue(Canvas.TopProperty, 237.00);
                    TB_DateEnd.SetValue(Canvas.TopProperty, 237.00);
                    TB_DateInterest.SetValue(Canvas.TopProperty, 237.00);
                    TB_InterestRate.SetValue(Canvas.TopProperty, 237.00);
                    TB_AccessType.SetValue(Canvas.TopProperty, 237.00);
                    TB_DepositTerm.SetValue(Canvas.TopProperty, 237.00);
                    TB_AmountMin.SetValue(Canvas.TopProperty, 237.00);
                    TB_DateStart.SetValue(Canvas.TopProperty, 237.00);
                    TB_AutoZC.SetValue(Canvas.TopProperty, 237.00);
                    TB_Operation.SetValue(Canvas.TopProperty, 237.00);
                    TB_AccountName.SetValue(Canvas.TopProperty, 88.00);
                    TB_TITLE.SetValue(Canvas.TopProperty, 17.00);
                    TB_Account.SetValue(Canvas.TopProperty, 88.00);
                    TB_Type.SetValue(Canvas.TopProperty, 88.00);
                    TB_AmountMax.SetValue(Canvas.TopProperty, 150.00);
                    TB_Super.SetValue(Canvas.TopProperty, 283.00);
                    TB_Recheck.SetValue(Canvas.TopProperty, 283.00);
                    TB_Handling.SetValue(Canvas.TopProperty, 283.00);
                    if (flag == "V8")
                    {
                        BZ.Visibility = Visibility.Visible;
                    }
                    break;
                case "V14":
                case "V21":
                case "V22":
                case "V13":
                case "V10"://齐丰
                case "V16"://冠县齐丰
                case "V17"://蒙阴齐丰
                case "V18"://新泰齐丰
                case "V19"://哈密天山
                case "V23"://浚县
                case "V24"://确山
                    TB_DateStart.SetValue(Canvas.LeftProperty, 18.00);
                    TB_DateStart.SetValue(Canvas.TopProperty, 235.00);
                    TB_AmountMin.SetValue(Canvas.LeftProperty, 78.00);
                    TB_AmountMin.SetValue(Canvas.TopProperty, 235.00);
                    TB_DepositTerm.SetValue(Canvas.LeftProperty, 180.00);
                    TB_DepositTerm.SetValue(Canvas.TopProperty, 235.00);
                    TB_AccessType.SetValue(Canvas.LeftProperty, 216.00);
                    TB_AccessType.SetValue(Canvas.TopProperty, 235.00);
                    TB_InterestRate.SetValue(Canvas.LeftProperty, 280.00);
                    TB_InterestRate.SetValue(Canvas.TopProperty, 235.00);
                    TB_DateInterest.SetValue(Canvas.LeftProperty, 320.00);
                    TB_DateInterest.SetValue(Canvas.TopProperty, 235.00);
                    TB_DateEnd.SetValue(Canvas.LeftProperty, 391.00);
                    TB_DateEnd.SetValue(Canvas.TopProperty, 235.00);
                    TB_dqx.SetValue(Canvas.LeftProperty, 452.00);
                    TB_dqx.SetValue(Canvas.TopProperty, 235.00);
                    TB_AutoZC.SetValue(Canvas.LeftProperty, 542.00);
                    TB_AutoZC.SetValue(Canvas.TopProperty, 235.00);
                    TB_Operation.SetValue(Canvas.LeftProperty, 595.00);
                    TB_Operation.SetValue(Canvas.TopProperty, 235.00);
                    break;
                default:
                    TB_dqx.SetValue(Canvas.LeftProperty, 472.00);
                    TB_DateEnd.SetValue(Canvas.LeftProperty, 399.00);
                    TB_DateInterest.SetValue(Canvas.LeftProperty, 328.00);
                    TB_InterestRate.SetValue(Canvas.LeftProperty, 285.00);
                    TB_AccessType.SetValue(Canvas.LeftProperty, 219.00);
                    TB_DepositTerm.SetValue(Canvas.LeftProperty, 182.00);
                    TB_AmountMin.SetValue(Canvas.LeftProperty, 80.00);
                    TB_DateStart.SetValue(Canvas.LeftProperty, 18.00);
                    break;
            }
        }

        /// <summary>
        /// 获取数字字符所在的索引
        /// </summary>
        private int GetNumberLetterIndex(string source)
        {
            int index = -1;
            string[] temp = { "一", "二", "三", "四", "五", "六", "七", "八", "九", "十" };
            if (!string.IsNullOrEmpty(source))
            {
                for (int i = 0; i < temp.Length; i++)
                {
                    if (source.Contains(temp[i]))
                    {
                        index = source.IndexOf(temp[i]);
                        break;
                    }
                }
            }
            return index;
        }
    }
}
