﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 储蓄存单
    /// </summary>
    public partial class Print_PersonalRegularlyCDsLF : IPrintContract
    {
        public Print_PersonalRegularlyCDsLF(Print_PersonalRegularlyCDsLFEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData
        {
            get;
            private set;
        }



        /// <summary>
        /// 
        /// </summary>
        public class Print_PersonalRegularlyCDsLFEntry : PrintEntry
        {
            /// <summary>
            /// 存入日
            /// </summary>
            public DateTime? DateStart { get; set; }

            /// <summary>
            /// 起息日
            /// </summary>
            public DateTime? DateInterest { get; set; }

            /// <summary>
            /// 密/印标志
            /// </summary>
            public string AccessType { get; set; }

            /// <summary>
            /// 储种
            /// </summary>
            public string Type { get; set; }

            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public string InterestRate { get; set; }

            /// <summary>
            /// 到期日
            /// </summary>
            public DateTime? DateEnd { get; set; }

            /// <summary>
            /// 操作
            /// </summary>
            public string Operation { get; set; }

            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }

            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }

            /// <summary>
            /// 存入金额
            /// </summary>
            public decimal Amount { get; set; }

            /// <summary>
            /// 自动转存标志
            /// </summary>
            public string AutoZC { get; set; }
            /// <summary>
            /// 开户行
            /// </summary>
            public string OpnBrname { get; set; }

            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }

            /// <summary>
            /// 到期息
            /// </summary>
            public string dqx { get; set; }
            /// <summary>
            /// 标题
            /// </summary>
            public string Title { get; set; }
            /// <summary>
            /// 柜员机构名称
            /// </summary>
            public string KinbrName { get; set; }
            /// <summary>
            /// 通存通兑    
            /// </summary>
            public string TCTD { get; set; }
            /// <summary>
            /// 存息账号
            /// </summary>
            public string CXZH { get; set; }
            /// <summary>
            /// 挂失新换字样
            /// </summary>
            public string GSXH { get; set; }
        }


        public void LoadPrintData()
        {
            var entry = PrintData as Print_PersonalRegularlyCDsLFEntry;
            if (entry == null)
                return;
            if (entry.DateStart != null)
                TB_DateStart.Text = entry.DateStart.Value.ToString("yyyyMMdd");
            if (entry.DateInterest != null)
                TB_DateInterest.Text = entry.DateInterest.Value.ToString("yyyyMMdd");
            TB_AccessType.Text = entry.AccessType;
            TB_Type.Text = entry.Type;
            this.TB_CXZH.SetPrintTextBlock(entry.CXZH);
            if (entry.Type.Contains("鸿运来") || entry.Type.Contains("月月盈") || entry.Type.Contains("季季盈")) 
            {
                TB_txt.Visibility = entry.Type.Contains("鸿运来") ? Visibility.Visible : Visibility.Collapsed;
                if (!string.IsNullOrWhiteSpace(entry.CXZH)) { TB_CXZH.Visibility = Visibility.Visible; }
            }
            if (string.IsNullOrEmpty(entry.DepositTerm))
            {
                TB_DepositTerm.Text = "---";
            }
            else
            {
                TB_DepositTerm.Text = entry.DepositTerm;
            }
            if (!string.IsNullOrEmpty(entry.InterestRate))
            {
                TB_InterestRate.Text = Convert.ToDouble(entry.InterestRate).ToString() + "%";
            }
            if (entry.DateEnd != null)
            {
                TB_DateEnd.Text = entry.DateEnd.Value.ToString("yyyyMMdd");
            }
            else
            {
                TB_DateEnd.Text = "---";
            }
            TB_Account.Text = entry.Account;
            TB_AccountName.Text = entry.AccountName;
            TB_AmountMax.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
            var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            TB_AmountMin.Text = type + entry.Amount.ToString("N2");
            if (!string.IsNullOrEmpty(entry.AutoZC))
            {
                TB_AutoZC.Text = entry.AutoZC == "到期自动转存" ? "是" : "否";
            }
            TB_Tctd.Text = entry.TCTD;
            TB_dqx.Text = entry.dqx;
            TB_TITLE.Text = entry.Title;
            //公共要素
            TB_Recheck.Text = PrintAttribute.Authno;
            //业务要求操作传柜员号
            TB_Operation.Text = LoginUserInfo.TellerNo;
            TB_Handling.Text = LoginUserInfo.TellerNo;
            TB_GSXH.Text = entry.GSXH;
            #region 打印印章


            //var textBolck = new Canvas() { Height = 200, Width = 200 };
            //var TestFlag = new Image() { Height = 200, Width = 200 };
            //tb_qz.Source =
            //    new BitmapImage(new Uri("/TellerSystem.PrintTemplate;component/Base/Images/"+entry.KinbrName+".jpg",
            //        UriKind.Relative));
            //Canvas.SetLeft(TestFlag, 529.00);
            //Canvas.SetTop(TestFlag, 159.00);
            //textBolck.Children.Add(TestFlag);
            //var testBH = new TextBlock()
            //{
            //    Height = 16,
            //    Width = 92,
            //    Text = "0000000000"
            //};
            //Canvas.SetLeft(testBH, 61.00);
            //Canvas.SetTop(testBH, 53.00);
            //testBH.FontWeight = FontWeights.UltraBlack;
            //testBH.Foreground = new SolidColorBrush(Colors.Red);
            //textBolck.Children.Add(testBH);
            //this.DrawingUIElement(574, 418, textBolck);
            #endregion

        }

        /// <summary>
        /// 获取数字字符所在的索引
        /// </summary>
        private int GetNumberLetterIndex(string source)
        {
            int index = -1;
            string[] temp = { "一", "二", "三", "四", "五", "六", "七", "八", "九", "十" };
            if (!string.IsNullOrEmpty(source))
            {
                for (int i = 0; i < temp.Length; i++)
                {
                    if (source.Contains(temp[i]))
                    {
                        index = source.IndexOf(temp[i]);
                        break;
                    }
                }
            }
            return index;
        }
    }
}
