﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 储蓄存单
    /// </summary>
    public partial class Print_PersonalRegularlyCDsQF : IPrintContract
    {
        public Print_PersonalRegularlyCDsQF(Print_PersonalRegularlyCDsQFEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData
        {
            get;
            private set;
        }

        

        /// <summary>
        /// 
        /// </summary>
        public class Print_PersonalRegularlyCDsQFEntry : PrintEntry
        {
            /// <summary>
            /// 存入日
            /// </summary>
            public DateTime? DateStart { get; set; }

            /// <summary>
            /// 起息日
            /// </summary>
            public DateTime? DateInterest { get; set; }

            /// <summary>
            /// 密/印标志
            /// </summary>
            public string AccessType { get; set; }

            /// <summary>
            /// 储种
            /// </summary>
            public string Type { get; set; }

            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public string InterestRate { get; set; }

            /// <summary>
            /// 到期日
            /// </summary>
            public DateTime? DateEnd { get; set; }

            /// <summary>
            /// 操作
            /// </summary>
            public string Operation { get; set; }

            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }

            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }

            /// <summary>
            /// 存入金额
            /// </summary>
            public decimal Amount { get; set; }

            /// <summary>
            /// 自动转存标志
            /// </summary>
            public string AutoZC { get; set; }
            /// <summary>
            /// 开户行
            /// </summary>
            public string OpnBrname { get; set; }

            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }

            /// <summary>
            /// 到期息
            /// </summary>
            public string dqx { get; set; }
            /// <summary>
            /// 标题
            /// </summary>
            public string Title { get; set; }
            /// <summary>
            /// 柜员机构名称
            /// </summary>
            public string KinbrName { get; set; }

            public string GH { get; set; }
            /// <summary>
            /// 产品类型 
            /// </summary>
            public string cp_type { get; set; }
        }


        public void LoadPrintData()
        {
            string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            var entry = PrintData as Print_PersonalRegularlyCDsQFEntry;
            if (entry == null)
                return;
            if (entry.DateStart != null)
                TB_DateStart.Text = entry.DateStart.Value.ToString("yyyy/MM/dd");
            if (entry.DateInterest != null)
                TB_DateInterest.Text = entry.DateInterest.Value.ToString("yyyy/MM/dd");
            TB_AccessType.Text = entry.AccessType;
            //齐丰需求：储种为98天定期存款，198天定期存款，328天定期存款.....998天定期存款时，储种打印为整存整取。
            //修改很无奈,杀个程序员祭天吧！
            if (entry.Type.Contains("8天"))
            {
                entry.Type = "整存整取";
            }
            if (flag == "V10" && entry.cp_type == "整存整取") //莱山 产品类型是整存整取的存单打印储种全部打印“整存整取”
            {
                entry.Type = "整存整取";
            }
            TB_Type.Text = entry.Type;
            if (string.IsNullOrEmpty(entry.DepositTerm))
            {
                TB_DepositTerm.Text = "---";
            }
            else
            {
                TB_DepositTerm.Text = entry.DepositTerm;
            }
            if (!string.IsNullOrEmpty(entry.InterestRate))
            {
                TB_InterestRate.Text = Convert.ToDouble(entry.InterestRate).ToString() + "%";
            }
            if (entry.DateEnd != null)
            {
                TB_DateEnd.Text = entry.DateEnd.Value.ToString("yyyy/MM/dd");
            }
            else
            {
                TB_DateEnd.Text = "---";
            }
            TB_Account.Text = entry.Account;
            TB_AccountName.Text = entry.AccountName;
            TB_AmountMax.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
            var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            TB_AmountMin.Text = type + entry.Amount.ToString("N2");
            if (!string.IsNullOrEmpty(entry.AutoZC))
            {
                TB_AutoZC.Text = entry.AutoZC == "到期自动转存" ? "是" : "否";
            }

            TB_dqx.Text = entry.dqx;
            TB_TITLE.Text = entry.Title;
            //公共要素
            if (flag != "V19")
            {
                TB_Recheck.Text = PrintAttribute.Review;
            }
            else
            {
                TB_Recheck.Text = PrintAttribute.Authentic;
            }
            //业务要求操作传柜员号
            TB_Operation.Text = LoginUserInfo.TellerNo;
            if (flag == "V19")
            {
                TB_Handling.Text = LoginUserInfo.TellerNo;
                //整体调整位移
                PYL.Margin = new Thickness(0, 10, 0, 0);
            }
            
            if (flag == "V10") //莱山-个人定期存单，往上调整“通存通兑”位置
            {
                TB_TITLE.SetValue(Canvas.TopProperty, 18.00);
                this.E_Seal_Location = new Point(531, 121);

                BZ.Visibility = Visibility.Visible;
            }
            else
            {
                this.E_Seal_Location = new Point(489, 134);
            }
            if (flag == "V14")
            {
                BY.Visibility = Visibility.Visible;
            }
            if (flag == "V17")
            {
                BZ.Visibility = Visibility.Visible; 
            }
        }

        /// <summary>
        /// 获取数字字符所在的索引
        /// </summary>
        private int GetNumberLetterIndex(string source)
        {
            int index = -1;
            string[] temp = { "一", "二", "三", "四", "五", "六", "七", "八", "九", "十" };
            if (!string.IsNullOrEmpty(source))
            {
                for (int i = 0; i < temp.Length; i++)
                {
                    if (source.Contains(temp[i]))
                    {
                        index = source.IndexOf(temp[i]);
                        break;
                    }
                }
            }
            return index;
        }
    }
}
