﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.PrintTemplate.Base;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 储蓄存单
    /// </summary>
    public partial class Print_PersonalRegularlyCDsTJ : IPrintContract
    {
        public Print_PersonalRegularlyCDsTJ(Print_PersonalRegularlyCDsTJEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
          
        }
        string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
        public Contract.Entry.PrintEntry PrintData
        {
            get;
            private set;
        }

        

        /// <summary>
        /// 
        /// </summary>
        public class Print_PersonalRegularlyCDsTJEntry : PrintEntry
        {
            /// <summary>
            /// 起息日
            /// </summary>
            public DateTime? DateInterest { get; set; }
            /// <summary>
            /// 储种
            /// </summary>
            public string Type { get; set; }
            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }
            /// <summary>
            /// 利率
            /// </summary>
            public string InterestRate { get; set; }
            /// <summary>
            /// 到期日
            /// </summary>
            public DateTime? DateEnd { get; set; }
            /// <summary>
            /// 操作
            /// </summary>
            public string Operation { get; set; }
            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }
            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }
            /// <summary>
            /// 存入金额
            /// </summary>
            public decimal Amount { get; set; }
            /// <summary>
            /// 自动转存标志
            /// </summary>
            public string AutoZC { get; set; }
            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }
            /// <summary>
            /// 到期息
            /// </summary>
            public string dqx { get; set; }
            /// <summary>
            /// 标题
            /// </summary>
            public string Title { get; set; }
            /// <summary>
            /// 支取方式
            /// </summary>
            public string ZQFS { get; set; }
            /// <summary>
            /// 挂失更换打印标志
            /// </summary>
            public string GSGH { get; set; }
        }


        public void LoadPrintData()
        {
            //固始开启防伪字体
            if (flag == "V21")
            {
                var uri = RES.GetUri("Font/iconfont.ttf");
                if (uri != null)
                    //p.FontFamily = new System.Windows.Media.FontFamily(uri.LocalPath + "#number,Yu Gothic");
                    this.FontFamily = new System.Windows.Media.FontFamily(uri.LocalPath + "#number,Microsoft YaHei");
            }
            var entry = PrintData as Print_PersonalRegularlyCDsTJEntry;
            if (entry == null)
                return;
            if (entry.DateInterest != null)
                TB_DateInterest.Text = entry.DateInterest.Value.ToString("yyyy/MM/dd");
            TB_Type.Text = entry.Type;
            if (string.IsNullOrEmpty(entry.DepositTerm))
            {
                TB_DepositTerm.Text = "---";
            }
            else
            {
                TB_DepositTerm.Text = entry.DepositTerm;
            }
            if (!string.IsNullOrEmpty(entry.InterestRate))
            {
                TB_InterestRate.Text = Convert.ToDouble(entry.InterestRate).ToString() + "%";
            }
            if (entry.DateEnd != null)
            {
                TB_DateEnd.Text = entry.DateEnd.Value.ToString("yyyy/MM/dd");
            }
            else
            {
                TB_DateEnd.Text = "---";
            }
            TB_Account.Text = entry.Account;
            TB_AccountName.Text = entry.AccountName;
            if (flag == "V22" || flag == "V21")
            {
                TB_AmountMin.Text = UserFunction.ChangeMoneySymbol("01") + entry.Amount;
                TB_AmountMin.Visibility = Visibility.Visible;
                if (flag == "V21")
                {
                    TB_AmountMin.FontSize = 16;
                }
            }
            TB_AmountMax.Text = "人民币" + entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
            var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            if (!string.IsNullOrEmpty(entry.AutoZC))
            {
                TB_AutoZC.Text = entry.AutoZC == "到期自动转存" ? "自动转存" : "非自动转存";
            }
            TB_dqx.Text = UserFunction.ChangeMoneySymbol("01") + entry.dqx;
            TB_TITLE.Text = entry.Title;
            TB_Recheck.Text = PrintAttribute.Review;
            if (flag == "V21")// && entry.Type.Contains("开门红"))
            {
                TB_Operation.Text = LoginUserInfo.TellerName + "(" + LoginUserInfo.TellerNo + ")";
                TB_sq.Text = PrintAttribute.Authentic;
            }
            else
            {
                //业务要求操作传柜员号
                TB_Operation.Text = LoginUserInfo.TellerNo;
                TB_sq.Text = PrintAttribute.Authno;
            }
            TB_Year.Text = LoginUserInfo.TradeDate.Substring(0, 4).Trim();
            TB_Month.Text = LoginUserInfo.TradeDate.Substring(4, 2).Trim();
            TB_Day.Text = LoginUserInfo.TradeDate.Substring(6, 2).Trim();
            if (flag == "V21")
            {
                PYL.Margin = new Thickness(-25, -15, 0, 0);
                TB_ZQFS.Visibility = Visibility.Visible;
                TB_ZQFS.Text = "凭" + entry.ZQFS;//支取方式
                if (entry.GSGH == "挂失更换")//挂失更换 打印“凭证补发”字样
                {
                    TB_GSGH.Visibility = Visibility.Visible;
                }
                TB_AmountMax.FontSize = 16;
                TB_AmountMax.Height = 38;
            }
        }

        /// <summary>
        /// 获取数字字符所在的索引
        /// </summary>
        private int GetNumberLetterIndex(string source)
        {
            int index = -1;
            string[] temp = { "一", "二", "三", "四", "五", "六", "七", "八", "九", "十" };
            if (!string.IsNullOrEmpty(source))
            {
                for (int i = 0; i < temp.Length; i++)
                {
                    if (source.Contains(temp[i]))
                    {
                        index = source.IndexOf(temp[i]);
                        break;
                    }
                }
            }
            return index;
        }
    }
}
