﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 储蓄存单
    /// </summary>
    public partial class Print_PersonalRegularlyCDs_DY : IPrintContract
    {
        public Print_PersonalRegularlyCDs_DY(Print_PersonalRegularlyCDs_DYEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData
        {
            get;
            private set;
        }

        /// <summary>
        /// 
        /// </summary>
        public class Print_PersonalRegularlyCDs_DYEntry : PrintEntry
        {
            /// <summary>
            /// 存入日
            /// </summary>
            public DateTime? DateStart { get; set; }

            /// <summary>
            /// 起息日
            /// </summary>
            public DateTime? DateInterest { get; set; }

            /// <summary>
            /// 密/印标志
            /// </summary>
            public string AccessType { get; set; }

            /// <summary>
            /// 储种
            /// </summary>
            public string Type { get; set; }

            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public string InterestRate { get; set; }

            /// <summary>
            /// 到期日
            /// </summary>
            public DateTime? DateEnd { get; set; }

            /// <summary>
            /// 操作
            /// </summary>
            public string Operation { get; set; }

            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }

            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }

            /// <summary>
            /// 存入金额
            /// </summary>
            public decimal Amount { get; set; }

            /// <summary>
            /// 自动转存标志
            /// </summary>
            public string AutoZC { get; set; }
            /// <summary>
            /// 开户行
            /// </summary>
            public string OpnBrname { get; set; }

            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }
            
            /// <summary>
            /// 到期息
            /// </summary>
            public string dqx { get; set; }
            /// <summary>
            /// 标题
            /// </summary>
            public string Title { get; set; }

            /// <summary>
            /// 利息前置产品标志
            /// </summary>
            public string QZBZ { get; set; }

            /// <summary>
            /// 通兑标志
            /// </summary>
            public string TDBZ { get; set; }
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_PersonalRegularlyCDs_DYEntry;
            if (entry == null)
                return;
            //if (entry.DateStart != null)
            //    TB_DateStart.Text = entry.DateStart.Value.ToString("yyyy/MM/dd");
            if (entry.DateInterest != null)
                TB_DateInterest.Text = entry.DateInterest.Value.ToString("yyyy/MM/dd");
            TB_AccessType.Text = entry.AccessType;
            
            TB_Type.Text = entry.Type;
            if(string.IsNullOrEmpty(entry.DepositTerm))
            {
                TB_DepositTerm.Text = "---"; 
            }
            else
            {
                TB_DepositTerm.Text = entry.DepositTerm; 
            }
            if (!string.IsNullOrEmpty(entry.InterestRate))
            {
                TB_InterestRate.Text = string.Format("{0:N6}%", entry.InterestRate);
            }
            if (entry.DateEnd != null)
            {
                TB_DateEnd.Text = entry.DateEnd.Value.ToString("yyyy/MM/dd");
            }
            else
            {
                TB_DateEnd.Text = "---";
            }
            //TB_Bz.Text = entry.Bz;
            Year.Text = LoginUserInfo.TradeDate.Substring(0, 4);
            Month.Text = LoginUserInfo.TradeDate.Substring(4, 2);
            Day.Text = LoginUserInfo.TradeDate.Substring(6, 2);
            TB_Bz.Text = "人民币";
            TB_Account.Text = entry.Account;
            TB_AccountName.Text = entry.AccountName;
            TB_AmountMax.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
            var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            TB_AmountMin.Text = type + entry.Amount.ToString("N2");
            //if (!string.IsNullOrEmpty(entry.AutoZC))
            //{
            //    TB_AutoZC.Text = entry.AutoZC == "到期自动转存" ? "是" : "否";
            //}
            //TB_dqx.Text = entry.dqx;
            TB_TITLE.Text = entry.Title;
            TB_TD.Text = entry.TDBZ;
            //公共要素
            TB_Recheck.Text = PrintAttribute.Review;
            //业务要求操作传柜员号
            TB_Operation.Text = LoginUserInfo.TellerNo;
            TB_ZDZC.SetPrintTextBlock(entry.AutoZC);
        }

        /// <summary>
        /// 获取数字字符所在的索引
        /// </summary>
        private int GetNumberLetterIndex(string source)
        {
            int index=-1;
            string[] temp = { "一", "二", "三", "四", "五", "六", "七", "八", "九", "十" };
            if (!string.IsNullOrEmpty(source))
            {
                for (int i = 0; i < temp.Length; i++)
                {
                    if (source.Contains(temp[i]))
                    {
                        index = source.IndexOf(temp[i]);
                        break;
                    }
                }
            }
            return index;
        }
    }
}
