﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.Communication;
using TellerSystem.Communication.MessageHelper;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// 浚县储蓄存单
    /// </summary>
    public partial class Print_PersonalRegularlyCDs_QS : IPrintContract
    {
        public Print_PersonalRegularlyCDs_QS(Print_PersonalRegularlyCDs_QSEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData
        {
            get;
            private set;
        }

        /// <summary>
        /// 版本控制标志
        /// </summary>
        private string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");

        /// <summary>
        /// 
        /// </summary>
        public class Print_PersonalRegularlyCDs_QSEntry : PrintEntry
        {
            /// <summary>
            /// 起息日
            /// </summary>
            public DateTime? DateInterest { get; set; }

            /// <summary>
            /// 密/印标志
            /// </summary>
            public string AccessType { get; set; }

            /// <summary>
            /// 储种
            /// </summary>
            public string Type { get; set; }

            /// <summary>
            /// 存期
            /// </summary>
            public string DepositTerm { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public string InterestRate { get; set; }

            /// <summary>
            /// 到期日
            /// </summary>
            public DateTime? DateEnd { get; set; }
            /// <summary>
            /// 操作
            /// </summary>
            public string Operation { get; set; }

            /// <summary>
            /// 账号
            /// </summary>
            public string Account { get; set; }

            /// <summary>
            /// 户名
            /// </summary>
            public string AccountName { get; set; }

            /// <summary>
            /// 存入金额
            /// </summary>
            public decimal Amount { get; set; }
            /// <summary>
            /// 币种
            /// </summary>
            public string Bz { get; set; }
            /// <summary>
            /// 标题
            /// </summary>
            public string Title { get; set; }

        }

        public void LoadPrintData()
        {           
            var entry = PrintData as Print_PersonalRegularlyCDs_QSEntry;
            if (entry == null)
                return;
            if (entry.DateInterest != null)
                TB_DateInterest.Text = entry.DateInterest.Value.ToString("yyyyMMdd");
            TB_AccessType.Text = entry.AccessType;           
            if (string.IsNullOrEmpty(entry.DepositTerm))
            {
                TB_DepositTerm.Text = "---";
            }
            else
            {
                TB_DepositTerm.Text = entry.DepositTerm;
            }
            if (!string.IsNullOrEmpty(entry.InterestRate))
            {
                TB_InterestRate.Text = string.Format("{0:N6}%", entry.InterestRate);
            }
            if (entry.DateEnd != null)
            {
                TB_DateEnd.Text = entry.DateEnd.Value.ToString("yyyyMMdd");
            }
            else
            {
                TB_DateEnd.Text = "---";
            }            
            Year.Text = LoginUserInfo.TradeDate.Substring(0, 4);
            Month.Text = LoginUserInfo.TradeDate.Substring(4, 2);
            Day.Text = LoginUserInfo.TradeDate.Substring(6, 2);
            TB_Account.Text = entry.Account;
            TB_AccountName.Text = entry.AccountName;
            TB_AmountMax.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
            TB_Bz.Text = "人民币";
            var type = UserFunction.ChangeMoneySymbol(entry.Bz);
            TB_AmountMin.Text = type + entry.Amount.ToString("N2");
            TB_TITLE.Text = entry.Title;
            TB_Type.Text = entry.Type;
            //公共要素
            //业务要求操作传柜员号
            TB_Operation.Text = LoginUserInfo.TellerNo;
            TB_Recheck.Text = PrintAttribute.Authno;
            TB_Handling.Text = LoginUserInfo.TellerNo;
            //加载电子印章，是否显示的控制机制在内部处理
            this.E_Seal_Location = new Point(491, 124);
        }

        /// <summary>
        /// 获取数字字符所在的索引
        /// </summary>
        private int GetNumberLetterIndex(string source)
        {
            int index = -1;
            string[] temp = { "一", "二", "三", "四", "五", "六", "七", "八", "九", "十" };
            if (!string.IsNullOrEmpty(source))
            {
                for (int i = 0; i < temp.Length; i++)
                {
                    if (source.Contains(temp[i]))
                    {
                        index = source.IndexOf(temp[i]);
                        break;
                    }
                }
            }
            return index;
        }
    }
}
