﻿using System;
using System.Windows;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    /// Interaction logic for Print_PersonalTrnsferDraft.xaml
    /// </summary>
    public partial class Print_PersonalTrnsferDraft : IPrintContract
    {
        public Print_PersonalTrnsferDraft(Print_PersonalTrnsferDraftEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public class Print_PersonalTrnsferDraftEntry : PrintEntry
        {
            /// <summary>
            /// 银行留存联账号
            /// </summary>
            public string Acno { get; set; }
            /// <summary>
            /// 客户回单账号
            /// </summary>
            public string Acno1 { get; set; }
            /// <summary>
            /// 账户户名
            /// </summary>
            public string ZHHM { get; set; }

            /// <summary>
            /// 开户行名
            /// </summary>
            public string OpnBrno { get; set; }

            /// <summary>
            /// 储种
            /// </summary>
            public string PrdtName { get; set; }

            /// <summary>
            /// 利率
            /// </summary>
            public string Rate { get; set; }

            /// <summary>
            /// 交易金额
            /// </summary>
            public decimal TxAmt { get; set; }
            /// <summary>
            /// 账户余额
            /// </summary>
            public string tbAccount { get; set; }

            /// <summary>
            /// 账户类型
            /// </summary>
            public string Zhlx { get; set; }

            /// <summary>
            /// 交易类型
            /// </summary>
            public string TradeType { get; set; }

            /// <summary>
            /// 现转标志
            /// </summary>
            public string CtInd { get; set; }

            /// <summary>
            /// 对方户名
            /// </summary>
            public string DfAcname { get; set; }

            /// <summary>
            /// 银行留存联对方账号
            /// </summary>
            public string DfAcno { get; set; }

            /// <summary>
            /// 客户回单对方账号
            /// </summary>
            public string DfAcno1 { get; set; }

            /// <summary>
            /// 通存通兑标志
            /// </summary>
            public string TCBZ { get; set; }

            /// <summary>
            /// 汇款用途
            /// </summary>
            public string YT { get; set; }

        }

        #region IPrintContract Members

        public Contract.Entry.PrintEntry PrintData { get; private set; }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_PersonalTrnsferDraftEntry;
            if (entry == null)
                return;
            switch (entry.TCBZ[4].ToString().Trim())
            {
                case "0":
                    TB_TCBZ.SetPrintTextBlock("非通存通兑");
                    break;
                case "1":
                    TB_TCBZ.SetPrintTextBlock("通存通兑");
                    break;
                case "2":
                    TB_TCBZ.SetPrintTextBlock("通存非通兑");
                    break;
                default:
                    TB_TCBZ.SetPrintTextBlock("通存通兑");
                    break;
            }
            TB_Acno1.SetPrintTextBlock(entry.Acno);
            //TB_Acno2.SetPrintTextBlock(entry.Acno);
            if (entry.Acno1 != null)
            {
                TB_Acno2.Text=entry.Acno1;
            }
            else
            {
                TB_Acno2.Text=entry.Acno;
            }
            TB_HM1.SetPrintTextBlock(entry.ZHHM);
            TB_HM2.Text=entry.ZHHM;
            TB_OPNbrno.SetPrintTextBlock(entry.OpnBrno);
            TB_Prdt1.SetPrintTextBlock(entry.PrdtName);
            TB_CtInd1.SetPrintTextBlock(entry.CtInd);
            YT.SetPrintTextBlock(entry.YT);
            if (!string.IsNullOrEmpty(entry.Rate))
            {
                TB_Rate1.SetPrintTextBlock(string.Format("{0:N6}%", entry.Rate));
            }
            else
            {
                TB_Rate1.SetPrintTextBlock(string.Empty);
            }

            TB_lsh.SetPrintTextBlock(this.PrintAttribute.SerialNumber);
            //TB_Term1.SetPrintTextBlock(entry.Term);
            var type = UserFunction.ChangeMoneySymbol("01");
            TB_TxAmt1.SetPrintTextBlock(type + entry.TxAmt.ToString("N2"));
            TB_TxAmt2.Text=type + entry.TxAmt.ToString("N2");
            if (entry.tbAccount != null)
            {
                tb_Account.Visibility = Visibility.Visible;
                tb_Account.SetPrintTextBlock(type + entry.tbAccount);
            }
            TB_data.Text=LoginUserInfo.TradeDate;
            TB_Czy.Text = LoginUserInfo.TellerNo;
            if (string.IsNullOrEmpty(entry.DfAcno))
            {
                TB_DfAcno1.Visibility = Visibility.Hidden;
                TB_DfAcno2.Visibility = Visibility.Hidden;
            }
            else
            {
                TB_DfAcno1.SetPrintTextBlock(entry.DfAcno);
                if (entry.DfAcno1 != null)
                {
                    TB_DfAcno2.Text=entry.DfAcno1;
                }
                else
                {
                    TB_DfAcno2.Text=entry.DfAcno;
                }

            }
            if (string.IsNullOrEmpty(entry.DfAcname))
            {
                TB_DfAcname1.Visibility = Visibility.Hidden;
                TB_DfAcname2.Visibility = Visibility.Hidden;
            }
            else
            {
                TB_DfAcname1.SetPrintTextBlock(entry.DfAcname);
                TB_DfAcname2.Text=entry.DfAcname;
            }


            //业务流水
            TB_lsh.SetPrintTextBlock(this.PrintAttribute.SerialNumber);
            //核心流水
            tb_ls.SetPrintTextBlock(this.PrintAttribute.BusinessTransaction);
            TB_sqy.Text = this.PrintAttribute.Authno;
        }

        #endregion
    }
}
