﻿using System;
using System.Globalization;
using System.Windows;
using Platform.Common.RunningParameters;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;

namespace TellerSystem.Controls.Ext.PrintsHelper
{
    /// <summary>
    ///本票
    /// </summary>
    public partial class Print_PromissoryNote : IPrintContract
    {
        public Print_PromissoryNote(Print_PromissoryNoteEntry entry)
        {
            InitializeComponent();
            PrintData = entry;
        }

        public Contract.Entry.PrintEntry PrintData
        {
            get;
            private set;
        }

        public class Print_PromissoryNoteEntry : PrintEntry
        {
            /// <summary>
            /// 收款人
            /// </summary>
            public string Payee { get; set; }
            /// <summary>
            /// 申请人
            /// </summary>
            public string Apply { get; set; }
            /// <summary>
            /// 金额
            /// </summary>
            public decimal Amount { get; set; }
            /// <summary>
            /// 密押
            /// </summary>
            public string TestKey { get; set; }
            /// <summary>
            /// 行号
            /// </summary>
            public string BankNo { get; set; }
            /// <summary>
            /// 备注
            /// </summary>
            public string Remark { get; set; }
            /// <summary>
            /// 转账true 现金false
            /// </summary>
            public bool IsTransfer { get; set; }
        }

        public void LoadPrintData()
        {
            var entry = PrintData as Print_PromissoryNoteEntry;
            TB_Payee.Text = entry.Payee;
            TB_Apply.Text = entry.Apply;
            TB_AmountMax.Text = entry.PageData.TradeAPIs.ChangeToCNY(entry.Amount);
            TB_AmountMin.Text = entry.Amount.ToString();
            TB_TestKey.Text = entry.TestKey;
            TB_BankNo.Text = entry.BankNo;
            TB_Remark.Text = entry.Remark;
            CB_Transfer.IsChecked = entry.IsTransfer;
            CB_Cash.IsChecked = !entry.IsTransfer;
            //公共要素
            var date = entry.PageData.TradeAPIs.ToDateTime(LoginUserInfo.TradeDate);
            TB_Date_Year.Text = entry.PageData.TradeAPIs.ChangeToCI(date.Year, false);
            TB_Date_Month.Text = entry.PageData.TradeAPIs.ChangeToCI(date.Month, false);
            TB_Date_Day.Text = entry.PageData.TradeAPIs.ChangeToCI(date.Day, false);

            TB_Cashier.Text = LoginUserInfo.TellerName + "(" + LoginUserInfo.TellerNo + ")";
            TB_Recheck.Text = PrintAttribute.Review;
            TB_Handling.Text = LoginUserInfo.TellerName + "(" + LoginUserInfo.TellerNo + ")";
        }
    }
}