import { onMounted, ref } from 'vue';
import { WorkflowPage } from './WorkflowPage'; // 假设WorkflowPage已定义
import { TradeInfoData } from 'TellerSystem.Library.Ext.TradeExtension'; // 假设该模块已存在

// 定义BlankTradePage类
export class BlankTradePage extends WorkflowPage {
    constructor() {
        super();
        // 设置交易信息数据
        this.setTradeInfo(new TradeInfoData());
    }

    // 静态方法，重写默认样式键
    static overrideDefaultStyleKey() {
        // 在Vue中没有直接对应重写元数据的方式，这里简单注释说明
        // 可在组件样式中设置默认样式
        console.log('Override default style key for BlankTradePage');
    }

    // 应用模板时的逻辑
    onApplyTemplate() {
        onMounted(() => {
            const scrollViewer = ref<HTMLElement | null>(null);
            // 模拟获取模板子元素
            scrollViewer.value = document.getElementById('TradeScrollViewer');

            if (scrollViewer.value && !this.isInDesignMode()) {
                const content = scrollViewer.value.firstElementChild as HTMLElement;
                content.style.textAlign = 'left';

                scrollViewer.value.addEventListener('requestBringIntoView', (e) => {
                    const targetRect = e.target.getBoundingClientRect();
                    const scroll = scrollViewer.value as HTMLElement;
                    if (!isNaN(targetRect.y) && targetRect.y > scroll.offsetHeight * 2 / 3) {
                        scroll.scrollTop = scroll.scrollTop + targetRect.y - scroll.offsetHeight * 2 / 3;
                    }
                });
            }
        });
    }

    // 模拟DesignerProperties.GetIsInDesignMode
    private isInDesignMode() {
        // 这里简单返回false，实际可根据环境变量判断
        return false;
    }

    // 模拟设置交易信息的方法
    private setTradeInfo(info: TradeInfoData) {
        // 这里可添加具体的设置逻辑
        console.log('Set trade info:', info);
    }
}

// 调用静态方法重写默认样式键
BlankTradePage.overrideDefaultStyleKey();