﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TellerSystem.Contract.ContractBase;
using System.Windows.Controls;
using System.Windows;
using TellerSystem.Library.Ext.Function;
using Platform.Common.BasicFunctions;
using TellerSystem.Library.Ext.TradeExtension;
using System.Reflection;
using System.Windows.Input;

namespace TellerSystem.Controls.Ext.TradePageBase
{
    public class HomePageBase : PageBase
    {
        public void LoadTradeHost(UIElement ui)
        {
            Processor = new TradeProcessor(ui);
        }

        protected TradeProcessor Processor
        {
            get;
            private set;
        }

        #region 定义一个代理类，用来包装_tradeHost
        /// <summary>
        /// 提供交易加载处理机制的代理类，跟ui的沟通务必通过此类包装，以方便后续替换ui控件
        /// </summary>
        protected class TradeProcessor : ContentControl
        {
            public TradeProcessor(UIElement ui)
            {
                element = ui;
                this.Content = new Border { Child = ui, Visibility = Visibility.Hidden };
                this.Loaded += (sender, e) =>
                {
                    this.Dispatcher.BeginInvoke(new Action(() =>
                    {
                        (this.Content as Border).Visibility = Visibility.Visible;
                    }), System.Windows.Threading.DispatcherPriority.SystemIdle, null);

                };
            }
            private UIElement element;

            /// <summary>
            /// 打开参数指定的交易
            /// </summary>
            /// <param name="code"></param>
            public void OpenPage(string code)
            {
                var list = TradeMenuManager.Instance.GetTradeModel(x => x.Code == code);
                if (list.Count() == 0) return;
                var model = list.First();
                OpenPage(model);
            }

            /// <summary>
            /// 打开参数指定的交易
            /// </summary>
            /// <param name="code"></param>
            public void OpenPage(TradeModel model)
            {
                //TradePageExtension.OpenTradePage(element as Platform.Presentation.Interfaces.IPage, model.Action, TradePageExtension.OpenOptions.Tab);
                dynamic d = element;
                // public void menutree_LClick(object sender, RoutedEventArgs e)
                d.menutree_LClick(element, new RoutedEventArgs(Button.ClickEvent, model));
            }

            /// <summary>
            /// 退出系统,mode:0-大退，1-重新登陆
            /// </summary>
            /// <param name="mode"></param>
            public void Exit(int mode = 0)
            {
                var toolBar = element.FirstVisualTreeChild(x => (x as FrameworkElement).Name == "ToolBar");
                var t = toolBar.GetType();
                var m = t.GetMethod(mode == 0 ? "OnLoginOut" : "OnLoginAgain", BindingFlags.NonPublic | BindingFlags.Instance);
                m.Invoke(toolBar, new object[] { toolBar, null });
            }

            /// <summary>
            /// 锁屏
            /// </summary>
            public void Lock()
            {
                dynamic toolBar = element.FirstVisualTreeChild(x => (x as FrameworkElement).Name == "ToolBar");
                toolBar.OnLoginLock(toolBar, null);
            }

            /// <summary>
            /// 签到签退工作流
            /// </summary>
            public void WorkNavigation()
            {
                var ctrl = element.FirstVisualTreeChild(x => x.GetType().Name == "QuickToolBar");
                var t = ctrl.GetType();
                var m = t.GetMethod("WorkNavigation_Click", BindingFlags.NonPublic | BindingFlags.Instance);
                m.Invoke(ctrl, new object[] { ctrl, null });
            }

            /// <summary>
            /// 触发工具栏功能
            /// </summary>
            /// <param name="text"></param>
            public void QuickTool(string text)
            {
                dynamic ctrl = element.FirstVisualTreeChild(x => x.GetType().Name == "QuickToolBar");
                List<object> list = ctrl.ItemsSource;
                foreach (var item in list)
                {
                    var t = item.GetType();
                    if (t.GetProperty("Text").GetValue(item, null).Equals(text))
                    {
                        dynamic click = t.GetProperty("Click").GetValue(item, null);
                        click(null, null);
                        break;
                    }
                }
            }

            /// <summary>
            /// 发送按键信息
            /// </summary>
            /// <param name="key"></param>
            public void SendKey(Key key)
            {
                //private void GridPreviewKeyDown(object sender, KeyEventArgs e)
                var t = element.GetType();
                var m = t.GetMethod("GridPreviewKeyDown", BindingFlags.NonPublic | BindingFlags.Instance);
                m.Invoke(element, new object[] { element, new KeyEventArgs(Keyboard.PrimaryDevice, PresentationSource.FromVisual(element), 1, key) { RoutedEvent = UIElement.PreviewKeyUpEvent } });
            }
        }
        #endregion
    }
}
