import { ref, onMounted } from 'vue';

// 假设这些类型已经在其他文件中定义，这里先做类型声明
interface UIElement {
    FirstVisualTreeChild: (filter: (x: any) => boolean) => any;
    // 其他可能的属性和方法
}

interface TradeModel {
    Code: string;
    Action: any;
    // 其他可能的属性和方法
}

interface TradeMenuManager {
    Instance: {
        GetTradeModel: (filter: (x: TradeModel) => boolean) => TradeModel[];
    };
}

interface PageBase {
    // 可能的属性和方法
}

interface RoutedEventArgs {
    RoutedEvent: any;
    Source: any;
}

// 模拟 Button 类
class Button {
    static ClickEvent = 'click';
}

// 模拟 PresentationSource 类
class PresentationSource {
    static FromVisual = (visual: any) => null;
}

// 模拟 Keyboard 类
class Keyboard {
    static PrimaryDevice = null;
}

// 模拟 Key 类型
type Key = string;

// 模拟 TradePageExtension 类
class TradePageExtension {
    static OpenTradePage = (page: any, action: any, options: any) => { };
    static OpenOptions = {
        Tab: 'tab'
    };
}

// 模拟 Border 类
class Border {
    Child: UIElement;
    Visibility: string;
    constructor() {
        this.Visibility = 'hidden';
    }
}

// 模拟 ContentControl 类
class ContentControl {
    Content: any;
    Loaded: (callback: (sender: any, e: any) => void) => void;
    Dispatcher: {
        BeginInvoke: (action: () => void, priority: any, args: any[]) => void;
    };
    constructor() {
        this.Loaded = (callback) => {
            // 模拟加载完成回调
            setTimeout(() => {
                callback(this, null);
            }, 0);
        };
        this.Dispatcher = {
            BeginInvoke: (action, priority, args) => {
                setTimeout(action, 0);
            }
        };
    }
}

// TradeProcessor 类
class TradeProcessor extends ContentControl {
    private element: UIElement;

    constructor(ui: UIElement) {
        super();
        this.element = ui;
        this.Content = new Border();
        this.Content.Child = ui;
        this.Loaded((sender, e) => {
            this.Dispatcher.BeginInvoke(() => {
                (this.Content as Border).Visibility = 'visible';
            }, null, []);
        });
    }

    // 打开参数指定的交易
    OpenPage(code: string) {
        const list = TradeMenuManager.Instance.GetTradeModel(x => x.Code === code);
        if (list.length === 0) return;
        const model = list[0];
        this.OpenPage(model);
    }

    // 打开参数指定的交易
    OpenPage(model: TradeModel) {
        // TradePageExtension.OpenTradePage(this.element as any, model.Action, TradePageExtension.OpenOptions.Tab);
        const d = this.element as any;
        d.menutree_LClick(this.element, new RoutedEventArgs({ RoutedEvent: Button.ClickEvent, Source: model }));
    }

    // 退出系统,mode:0-大退，1-重新登陆
    Exit(mode = 0) {
        const toolBar = this.element.FirstVisualTreeChild(x => (x as any).Name === 'ToolBar');
        const t = toolBar.constructor;
        const methodName = mode === 0 ? 'OnLoginOut' : 'OnLoginAgain';
        const m = t.prototype[methodName];
        if (m) {
            m.call(toolBar, toolBar, null);
        }
    }

    // 锁屏
    Lock() {
        const toolBar = this.element.FirstVisualTreeChild(x => (x as any).Name === 'ToolBar') as any;
        toolBar.OnLoginLock(toolBar, null);
    }

    // 签到签退工作流
    WorkNavigation() {
        const ctrl = this.element.FirstVisualTreeChild(x => x.constructor.name === 'QuickToolBar');
        const t = ctrl.constructor;
        const m = t.prototype['WorkNavigation_Click'];
        if (m) {
            m.call(ctrl, ctrl, null);
        }
    }

    // 触发工具栏功能
    QuickTool(text: string) {
        const ctrl = this.element.FirstVisualTreeChild(x => x.constructor.name === 'QuickToolBar') as any;
        const list = ctrl.ItemsSource as any[];
        for (const item of list) {
            const t = item.constructor;
            if (t.prototype['Text'] && t.prototype['Text'].call(item) === text) {
                const click = t.prototype['Click'].call(item);
                if (click) {
                    click(null, null);
                }
                break;
            }
        }
    }

    // 发送按键信息
    SendKey(key: Key) {
        const t = this.element.constructor;
        const m = t.prototype['GridPreviewKeyDown'];
        if (m) {
            m.call(this.element, this.element, new RoutedEventArgs({
                RoutedEvent: UIElement.prototype.PreviewKeyUpEvent,
                Source: new KeyboardEvent('keyup', { key })
            }));
        }
    }
}

// HomePageBase 类
class HomePageBase extends PageBase {
    private _processor = ref<TradeProcessor | null>(null);

    get Processor() {
        return this._processor.value;
    }

    LoadTradeHost(ui: UIElement) {
        this._processor.value = new TradeProcessor(ui);
    }
}

export { HomePageBase };