﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.BasicFunctions;
using TellerSystem.Communication;
using TellerSystem.Contract.ContractBase;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Contract.Function;
using TellerSystem.Controls.Ext.Controls;
using TellerSystem.Controls.Ext.PrintsHelper;
using TellerSystem.Library.Ext.Function;

namespace TellerSystem.Controls.Ext.TradePageBase
{
    public class LeadSubmitPage : PageBase, ISubmitContract, IAuthorizeContract, IRecheckContract
    {
        #region 构造

        static LeadSubmitPage()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(LeadSubmitPage), new FrameworkPropertyMetadata(typeof(LeadSubmitPage)));
        }

        #region 命令

        /// <summary>
        /// 提交命令
        /// </summary>
        public static readonly RoutedUICommand SubmitCommand = new RoutedUICommand("提交", "SubmitCommand", typeof(LeadSubmitPage));

        /// <summary>
        /// 下一步命令
        /// </summary>
        public static readonly RoutedUICommand NextCommand = new RoutedUICommand("下一步", "NextCommand", typeof(LeadSubmitPage));

        /// <summary>
        /// 跳过命令
        /// </summary>
        public static readonly RoutedUICommand SkipCommand = new RoutedUICommand("跳过", "SkipButton", typeof(LeadSubmitPage));

        /// <summary>
        /// 上一步
        /// </summary>
        public static readonly RoutedUICommand PreCommand = new RoutedUICommand("上一步", "PreButton", typeof(LeadSubmitPage));

        #endregion 命令

        #region 操作按钮控制

        private FrameworkElement _buttonViewer, _submitButton, _skipButton, _preButton, _nextButton;

        private FrameworkElement _lv_Leads, _leadHeader;

        /// <summary>
        /// 启用模板时
        /// </summary>
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            _buttonViewer = GetTemplateChild("ButtonViewerPanel") as FrameworkElement;
            _submitButton = GetTemplateChild("SubmitButton") as FrameworkElement;
            _skipButton = GetTemplateChild("SkipButton") as FrameworkElement;
            _preButton = GetTemplateChild("PreButton") as FrameworkElement;
            _nextButton = GetTemplateChild("NextButton") as FrameworkElement;

            _lv_Leads = GetTemplateChild("TradePageViewer") as FrameworkElement;
            _leadHeader = GetTemplateChild("LeadHeader") as FrameworkElement;

            //控件滚动条定位
            var scrollViewer = GetTemplateChild("TradeScrollViewer") as ScrollViewer;
            if (scrollViewer != null && !DesignerProperties.GetIsInDesignMode(this))
            {
                ((FrameworkElement)scrollViewer.Content).HorizontalAlignment = System.Windows.HorizontalAlignment.Left;
                scrollViewer.RequestBringIntoView += (sender, e) =>
                {
                    var scroll = sender as ScrollViewer;
                    if (scroll != null)
                    {
                        if (!double.IsInfinity(e.TargetRect.Y) && e.TargetRect.Y > scroll.ActualHeight * 2 / 3)
                        {
                            scroll.ScrollToVerticalOffset(scroll.VerticalOffset + e.TargetRect.Y - scroll.ActualHeight * 2 / 3);
                        }
                    }
                };
            }
        }

        /// <summary>
        /// 设置操作区域的可见性
        /// </summary>
        /// <param name="visibility"></param>
        protected void ShowBottonViewer(Visibility visibility)
        {
            if (_buttonViewer != null)
            {
                _buttonViewer.Visibility = visibility;
            }
        }

        /// <summary>
        /// 在操作区域添加按钮
        /// </summary>
        /// <param name="buttonMsg"></param>
        /// <param name="clickEventHandler"></param>
        /// <param name="index"></param>
        protected Button AddOperateButton(string buttonMsg, RoutedEventHandler clickEventHandler, int index = 0)
        {
            Button button = null;
            var panel = _buttonViewer as Panel;
            if (panel != null)
            {
                button = new Button { Content = buttonMsg, Margin = new Thickness(0, 0, 10, 0) };
                button.Click += clickEventHandler;
                if (index > panel.Children.Count)
                    index = panel.Children.Count;
                panel.Children.Insert(index, button);
            }
            return button;
        }

        /// <summary>
        /// 提交按钮
        /// </summary>
        public Button SubmitButton
        {
            get { return _submitButton as Button; }
        }

        /// <summary>
        /// 跳过按钮
        /// </summary>
        public Button SkipButton
        {
            get { return _skipButton as Button; }
        }

        /// <summary>
        /// 上一步按钮
        /// </summary>
        public Button PreButton
        {
            get { return _preButton as Button; }
        }

        /// <summary>
        /// 下一步按钮
        /// </summary>
        public Button NextButton
        {
            get { return _nextButton as Button; }
        }

        /// <summary>
        ///
        /// </summary>
        public ContentPresenter TradePageViewer
        {
            get { return _lv_Leads as ContentPresenter; }
        }

        public WrapPanel LeadHeader
        {
            get { return _leadHeader as WrapPanel; }
        }

        #endregion 操作按钮控制

        /// <summary>
        /// 构造函数
        /// </summary>
        public LeadSubmitPage()
        {
            #region 绑定命令

            //提交按钮
            CommandBindings.Add(new CommandBinding(SubmitCommand,
                 (sender, e) =>
                 {
                     //RaiseSubmit();
                     ShowBottonViewer(Visibility.Collapsed);
                     ContractManager.RaiseSubmit(this);
                     ShowBottonViewer(Visibility.Visible);
                 }, (sender, e) =>
                 {
                     e.CanExecute = true;
                 }));

            //下一步按钮
            CommandBindings.Add(new CommandBinding(NextCommand,
                 (sender, e) =>
                 {
                     btnNext_Click();
                 }, (sender, e) =>
                 {
                     e.CanExecute = true;
                 }));

            //跳过按钮
            CommandBindings.Add(new CommandBinding(SkipCommand,
                (sender, e) =>
                {
                    btnStep_Click();
                }, (sender, e) =>
                {
                    e.CanExecute = true;
                }));

            //上一步按钮
            CommandBindings.Add(new CommandBinding(PreCommand,
                (sender, e) =>
                {
                    btnPrevious_Click();
                }, (sender, e) =>
                {
                    e.CanExecute = true;
                }));

            #endregion 绑定命令

            #region 构建授权实例

            AuthorizeEntries = new List<AuthorizeEntry>();
            var authorizeEntry = new AuthorizeEntry("0");
            authorizeEntry.PreviewAuthorize += (sender, e) =>
            {
                PreviewAuthorize(e);
            };
            authorizeEntry.RemoteAuthorized += (sender, e) =>
            {
                SubmitEntry.LoadData(authorizeEntry.PageData, authorizeEntry.MessageData);
                SubmitEntry.RaiseSubmit();
            };
            AuthorizeEntries.Add(authorizeEntry);

            #endregion 构建授权实例

            #region 构建复核实例

            RecheckEntries = new List<RecheckEntry>();
            var recheckEntry = new RecheckEntry("0");
            recheckEntry.PreviewRecheck += (sender, e) =>
            {
                PreviewRecheck(e);
            };
            RecheckEntries.Add(recheckEntry);

            #endregion 构建复核实例

            this.Loaded += (s1, e1) =>
            {
                var flag = Platform.Common.RunningParameters.ConfigManager.GetInstance().GetConfigValue("QBDJ", Platform.Common.RunningParameters.ConfigType.Client);
                if (flag != "true") return;
                var list = Platform.Common.BasicFunctions.DependencyObjectExtensions.FindVisualTreeChildren(s1 as DependencyObject, x => x is Platform.Controls.NumericContainer.CurrencyEditor);
                foreach (var item in list)
                {
                    Platform.Presentation.FocusManagers.FocusManager.AddPreviewLostFocusHandler(item, (sender, e) =>
                    {
                        var element = sender as Platform.Controls.NumericContainer.CurrencyEditor;
                        if (element == null) return;
                        var money = new TellerSystem.Controls.Ext.Controls.TBC_MoneyInfo();
                        money.Init(element.Text);
                        var d = new Platform.Controls.TradeExtension.PageDialog();
                        d.Open(this, money, "券别登记");
                        if (!money.IsOk)
                            Platform.Controls.ControlsHelper.ControlHelper.SetFocus(this, element);
                    });
                }

                #region leadsource

                if (LeadSource == null || LeadHeader.Children.Count >= LeadSource.Count) return;
                InitBtnContent();
                //_panle = sender as ContentControl;
                //_ttf = new TranslateTransform();
                //if (_panle != null)
                //{
                //    _panle.RenderTransform = _ttf;
                //}

                // var source = new List<ContentControl>();
                foreach (var item in LeadSource)
                {
                    //  source.Add(item.PageContent);
                    LeadHeader.Children.Add(new TextBlock
                    {
                        Text = item.Description,
                        Margin = new Thickness(0, 0, 8, 0),
                        FontSize = 14,
                        Height = 30,
                        Padding = new Thickness(2, 5, 2, 5),
                    });
                }
                //lv_Leads.ItemsSource = source;

                DisplayBtnContent();

                #endregion leadsource
            };
        }

        #endregion 构造

        #region ISubmitContract

        public virtual bool PreviewSubmitting(out Message message)
        {
            message = new Message();
            return false;
        }

        public SubmitEntry SubmitEntry
        {
            get;
            protected set;
        }

        #endregion ISubmitContract

        #region IAuthorizeContract

        /// <summary>
        /// 授权逻辑入口集合
        /// </summary>
        public List<AuthorizeEntry> AuthorizeEntries
        {
            get;
            private set;
        }

        /// <summary>
        /// 提交部分授权数据采集入口
        /// </summary>
        /// <param name="args"></param>
        public virtual void PreviewAuthorize(AuthorizeEntry.PreviewAuthorizeEventArgs args)
        {
            return;
        }

        #endregion IAuthorizeContract

        #region IRecheckContract

        /// <summary>
        /// 复核逻辑入口集合
        /// </summary>
        public List<RecheckEntry> RecheckEntries
        {
            get;
            private set;
        }

        /// <summary>
        /// 提交部分复核数据采集入口
        /// </summary>
        /// <param name="args"></param>
        public virtual void PreviewRecheck(RecheckEntry.PreviewRecheckEventArgs args)
        {
            return;
        }

        #endregion IRecheckContract

        //////////////////////////////////////////////////////////////////////////////////////

        #region Field

        /// <summary>
        ///     打印页面容器
        /// </summary>
        private ContentControl _panle;

        /// <summary>
        ///     偏移控制
        /// </summary>
        private TranslateTransform _ttf;

        private int _preIndex = 0;

        #endregion Field

        #region 公开的属性与方法

        /// <summary>
        /// 当前页的索引，从0开始
        /// </summary>
        public int CurrentLeadIndex { get; set; }

        /// <summary>
        /// 数据源
        /// </summary>
        public IList<LeadItem> LeadSource { get; set; }

        /// <summary>
        /// 设置或获取下一步按钮的可用性
        /// </summary>
        public bool IsNextBtnEnable
        {
            set { NextButton.IsEnabled = value; }
        }

        private int _stepIndex;

        /// <summary>
        /// 跳转到指定步骤，从0开始
        /// </summary>
        public int StepIndex
        {
            get { return _stepIndex; }
            set
            {
                _stepIndex = value;
                SkipButton.Visibility = Visibility.Visible;
            }
        }

        #endregion 公开的属性与方法

        //#region 路由 下一步事件

        ////创建和注册ReFlesh事件，该事件路由策略为Direct
        //public static readonly RoutedEvent LeadNextEvent = EventManager.RegisterRoutedEvent("LeadNext",
        //    RoutingStrategy.Direct, typeof(RoutedEventHandler), typeof(LeadTradeViewer));

        ////CLR事件的包装器
        //public event RoutedEventHandler LeadNext
        //{
        //    add { AddHandler(LeadNextEvent, value); }
        //    remove { RemoveHandler(LeadNextEvent, value); }
        //}

        ////触发ReFleshEvent
        //private void RaiseLeadNextEvent()
        //{
        //    var newEventArgs = new RoutedEventArgs(LeadTradeViewer.LeadNextEvent);
        //    RaiseEvent(newEventArgs);
        //}

        //#endregion 路由 下一步事件

        //#region 路由 完成事件

        ////创建和注册ReFlesh事件，该事件路由策略为Direct
        //public static readonly RoutedEvent LeadCompletedEvent = EventManager.RegisterRoutedEvent("LeadCompleted",
        //    RoutingStrategy.Direct, typeof(RoutedEventHandler), typeof(LeadTradeViewer));

        //public event RoutedEventHandler LeadCompleted
        //{
        //    add { AddHandler(LeadCompletedEvent, value); }
        //    remove { RemoveHandler(LeadCompletedEvent, value); }
        //}

        //private void RaiseLeadCompletedEvent()
        //{
        //    var newEventArgs = new RoutedEventArgs(LeadTradeViewer.LeadCompletedEvent);
        //    RaiseEvent(newEventArgs);
        //}

        //#endregion 路由 完成事件

        //#region 路由 该步完成事件

        ////创建和注册ReFlesh事件，该事件路由策略为Direct
        //public static readonly RoutedEvent TradeCompletedEvent = EventManager.RegisterRoutedEvent("TradeCompleted",
        //    RoutingStrategy.Direct, typeof(RoutedEventHandler), typeof(LeadTradeViewer));

        //public event RoutedEventHandler TradeCompleted
        //{
        //    add { AddHandler(TradeCompletedEvent, value); }
        //    remove { RemoveHandler(TradeCompletedEvent, value); }
        //}

        //private void RaiseTradeCompletedEvent()
        //{
        //    var newEventArgs = new RoutedEventArgs(TradeCompletedEvent);
        //    RaiseEvent(newEventArgs);
        //}

        //#endregion 路由 该步完成事件

        #region 初始化

        /// <summary>
        ///     存储初始信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StackPanel_Loaded(object sender, RoutedEventArgs e)
        {
            if (LeadSource == null || LeadHeader.Children.Count >= LeadSource.Count) return;

            _panle = sender as ContentControl;
            _ttf = new TranslateTransform();
            if (_panle != null)
            {
                _panle.RenderTransform = _ttf;
            }

            // var source = new List<ContentControl>();
            foreach (var item in LeadSource)
            {
                //  source.Add(item.PageContent);
                LeadHeader.Children.Add(new TextBlock
                {
                    Text = item.Description,
                    Margin = new Thickness(0, 0, 8, 0),
                    FontSize = 14,
                    Height = 30,
                    Padding = new Thickness(2, 5, 2, 5),
                });
            }
            //lv_Leads.ItemsSource = source;

            DisplayBtnContent();
        }

        #endregion 初始化

        #region 事件

        /// <summary>
        /// 上一步
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPrevious_Click()
        {
            if (CurrentLeadIndex <= 0) return;
            if (CurrentLeadIndex > LeadSource.Count - 1) CurrentLeadIndex = LeadSource.Count - 1;
            CurrentLeadIndex -= 1;
            DisplayBtnContent();
            //滑动效果
            AnimateControl(FlowDirection.RightToLeft);
        }

        /// <summary>
        /// 下一步
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNext_Click()
        {
            if (CurrentLeadIndex > LeadSource.Count - 1)
            {
                return;
            }

            if (CurrentLeadIndex < LeadSource.Count - 1)
            {
                CurrentLeadIndex += 1;
                DisplayBtnContent();
                //滑动效果
                AnimateControl(FlowDirection.LeftToRight);

                //公开路由方法
                //RaiseLeadNextEvent();
            }
            else
            {
                CurrentLeadIndex += 1;
                //公开路由方法
                //RaiseLeadCompletedEvent();
            }
        }

        /// <summary>
        /// 跳过
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStep_Click()
        {
            if (StepIndex < 0 || StepIndex > LeadSource.Count - 1) return;
            CurrentLeadIndex = StepIndex;
            DisplayBtnContent();
            //滑动效果
            AnimateControl(FlowDirection.LeftToRight);

            SkipButton.Visibility = Visibility.Hidden;
        }

        /// <summary>
        /// 动画
        /// </summary>
        /// <param name="dir"></param>
        private void AnimateControl(FlowDirection dir)
        {
            //var sb = new Storyboard();

            //var ks = new DoubleAnimationUsingKeyFrames();
            ////偏移量为实际宽度
            //var width = 710;
            //width *= Math.Abs(CurrentLeadIndex - _preIndex);
            //var k1 = new EasingDoubleKeyFrame
            //{
            //    KeyTime = KeyTime.FromTimeSpan(TimeSpan.FromSeconds(0.5)),
            //    Value = dir == FlowDirection.LeftToRight ? _ttf.X - width : _ttf.X + width,
            //    EasingFunction = new SineEase { EasingMode = EasingMode.EaseIn }
            //};

            //ks.KeyFrames.Add(k1);
            //sb.Children.Add(ks);
            //Storyboard.SetTarget(ks, _panle);
            //Storyboard.SetTargetProperty(ks, new PropertyPath("(UIElement.RenderTransform).(TranslateTransform.X)"));

            //sb.Begin();

            //记录本次索引
            _preIndex = CurrentLeadIndex;
        }

        /// <summary>
        /// 页面显示要素的处理
        /// </summary>
        private void InitBtnContent()
        {
            if (LeadSource == null) return;
            if (LeadSource.Count == 1)
            {
                PreButton.Visibility = Visibility.Collapsed;
                NextButton.Visibility = Visibility.Collapsed;
                SkipButton.Visibility = Visibility.Collapsed;
                SubmitButton.Visibility = Visibility.Visible;
            }
            else if (LeadSource.Count > 1)
            {
                PreButton.Visibility = Visibility.Collapsed;
                NextButton.Visibility = Visibility.Collapsed;
                SkipButton.Visibility = Visibility.Visible;
                SubmitButton.Visibility = Visibility.Collapsed;
            }
        }

        /// <summary>
        /// 页面显示要素的处理
        /// </summary>
        private void DisplayBtnContent()
        {
            PreButton.IsEnabled = true;
            if (CurrentLeadIndex == 0)
            {
                PreButton.IsEnabled = false;
            }

            if (CurrentLeadIndex == LeadSource.Count - 1)
            {
                SubmitButton.Visibility = Visibility.Visible;
            }
            else
            {
                SubmitButton.Visibility = Visibility.Collapsed;
            }

            if (CurrentLeadIndex > 0)
            {
                PreButton.IsEnabled = true;
            }

            if (CurrentLeadIndex < LeadSource.Count - 1)
            {
                NextButton.Visibility = Visibility.Visible;
            }
            else
            {
                NextButton.Visibility = Visibility.Collapsed;
            }

            var i = 0;
            foreach (TextBlock item in LeadHeader.Children)
            {
                if (item == null) continue;
                if (CurrentLeadIndex == i++)
                {
                    item.Background = Brushes.Blue;
                    item.Foreground = Brushes.White;
                    continue;
                }
                item.Background = Brushes.Transparent;
                item.Foreground = Brushes.Black;
            }
            TradePageViewer.Content = LeadSource[CurrentLeadIndex].PageContent;
        }

        #endregion 事件

        public class LeadItem
        {
            /// <summary>
            /// 页面路径
            /// </summary>
            public ContentControl PageContent { get; set; }

            /// <summary>
            /// 描述
            /// </summary>
            public string Description { get; set; }
        }
    }
}