import { ref, onMounted, watch, type Ref } from 'vue';

// 模拟部分类型定义，实际使用时需要根据项目情况调整
interface Message {
    // 这里可以根据实际的 Message 类补充属性
}

interface AuthorizeEntry {
    powerCode: string;
    PreviewAuthorize: (sender: any, e: AuthorizeEntry.PreviewAuthorizeEventArgs) => void;
    RemoteAuthorized: (sender: any, e: any) => void;
    PageData: any;
    MessageData: any;
    // 这里可以根据实际的 AuthorizeEntry 类补充属性和方法
}

namespace AuthorizeEntry {
    export class PreviewAuthorizeEventArgs {
        // 这里可以根据实际的 PreviewAuthorizeEventArgs 类补充属性
    }
}

interface RecheckEntry {
    powerCode: string;
    PreviewRecheck: (sender: any, e: RecheckEntry.PreviewRecheckEventArgs) => void;
    // 这里可以根据实际的 RecheckEntry 类补充属性和方法
}

namespace RecheckEntry {
    export class PreviewRecheckEventArgs {
        // 这里可以根据实际的 PreviewRecheckEventArgs 类补充属性
    }
}

interface LeadItem {
    PageContent: any; // 实际使用时可根据 PageContent 的类型精确指定
    Description: string;
}

interface FrameworkElement {
    Visibility: string;
    // 这里可以根据实际的 FrameworkElement 类补充属性和方法
}

interface ScrollViewer {
    Content: any;
    RequestBringIntoView: (sender: any, e: any) => void;
    ScrollToVerticalOffset: (offset: number) => void;
    VerticalOffset: number;
    ActualHeight: number;
    // 这里可以根据实际的 ScrollViewer 类补充属性和方法
}

interface Button {
    Content: string;
    Margin: any;
    Click: (sender: any, e: any) => void;
    IsEnabled: boolean;
    Visibility: string;
    // 这里可以根据实际的 Button 类补充属性和方法
}

interface ContentPresenter {
    Content: any;
    // 这里可以根据实际的 ContentPresenter 类补充属性和方法
}

interface WrapPanel {
    Children: any[];
    // 这里可以根据实际的 WrapPanel 类补充属性和方法
}

interface RoutedUICommand {
    // 这里可以根据实际的 RoutedUICommand 类补充属性和方法
}

// 模拟 ContractManager 类
class ContractManager {
    static RaiseSubmit(page: any) {
        // 这里可以实现实际的提交逻辑
    }
}

// 模拟 Platform.Common.RunningParameters.ConfigManager 类
class ConfigManager {
    static GetInstance() {
        return {
            GetConfigValue(key: string, type: string) {
                // 这里可以实现实际的获取配置值逻辑
                return '';
            }
        };
    }
}

// 模拟 Platform.Common.BasicFunctions.DependencyObjectExtensions 类
class DependencyObjectExtensions {
    static FindVisualTreeChildren(obj: any, predicate: (x: any) => boolean) {
        // 这里可以实现实际的查找子元素逻辑
        return [];
    }
}

// 模拟 Platform.Presentation.FocusManagers.FocusManager 类
class FocusManager {
    static AddPreviewLostFocusHandler(item: any, handler: (sender: any, e: any) => void) {
        // 这里可以实现实际的添加焦点丢失处理逻辑
    }
}

// 模拟 TellerSystem.Controls.Ext.Controls.TBC_MoneyInfo 类
class TBC_MoneyInfo {
    IsOk: boolean = false;
    Init(text: string) {
        // 这里可以实现实际的初始化逻辑
    }
}

// 模拟 Platform.Controls.TradeExtension.PageDialog 类
class PageDialog {
    Open(page: any, content: any, title: string) {
        // 这里可以实现实际的打开对话框逻辑
    }
}

// 模拟 Platform.Controls.ControlsHelper.ControlHelper 类
class ControlHelper {
    static SetFocus(page: any, element: any) {
        // 这里可以实现实际的设置焦点逻辑
    }
}

export default {
    setup() {
        // 命令
        const SubmitCommand: RoutedUICommand = new (class implements RoutedUICommand {
            // 这里可以根据实际的 RoutedUICommand 类补充属性和方法
        })();
        const NextCommand: RoutedUICommand = new (class implements RoutedUICommand {
            // 这里可以根据实际的 RoutedUICommand 类补充属性和方法
        })();
        const SkipCommand: RoutedUICommand = new (class implements RoutedUICommand {
            // 这里可以根据实际的 RoutedUICommand 类补充属性和方法
        })();
        const PreCommand: RoutedUICommand = new (class implements RoutedUICommand {
            // 这里可以根据实际的 RoutedUICommand 类补充属性和方法
        })();

        // 操作按钮控制
        const _buttonViewer: Ref<FrameworkElement | null> = ref(null);
        const _submitButton: Ref<FrameworkElement | null> = ref(null);
        const _skipButton: Ref<FrameworkElement | null> = ref(null);
        const _preButton: Ref<FrameworkElement | null> = ref(null);
        const _nextButton: Ref<FrameworkElement | null> = ref(null);

        const _lv_Leads: Ref<FrameworkElement | null> = ref(null);
        const _leadHeader: Ref<FrameworkElement | null> = ref(null);

        const OnApplyTemplate = () => {
            // 这里需要根据 Vue 的方式来获取模板子元素，以下是模拟实现
            _buttonViewer.value = document.getElementById('ButtonViewerPanel') as FrameworkElement;
            _submitButton.value = document.getElementById('SubmitButton') as FrameworkElement;
            _skipButton.value = document.getElementById('SkipButton') as FrameworkElement;
            _preButton.value = document.getElementById('PreButton') as FrameworkElement;
            _nextButton.value = document.getElementById('NextButton') as FrameworkElement;

            _lv_Leads.value = document.getElementById('TradePageViewer') as FrameworkElement;
            _leadHeader.value = document.getElementById('LeadHeader') as FrameworkElement;

            const scrollViewer = document.getElementById('TradeScrollViewer') as ScrollViewer;
            if (scrollViewer) {
                (scrollViewer.Content as FrameworkElement).style.horizontalAlignment = 'left';
                scrollViewer.RequestBringIntoView = (sender, e) => {
                    const scroll = sender as ScrollViewer;
                    if (scroll) {
                        if (!isNaN(e.TargetRect.Y) && e.TargetRect.Y > scroll.ActualHeight * 2 / 3) {
                            scroll.ScrollToVerticalOffset(scroll.VerticalOffset + e.TargetRect.Y - scroll.ActualHeight * 2 / 3);
                        }
                    }
                };
            }
        };

        const ShowBottonViewer = (visibility: string) => {
            if (_buttonViewer.value) {
                _buttonViewer.value.Visibility = visibility;
            }
        };

        const AddOperateButton = (buttonMsg: string, clickEventHandler: (sender: any, e: any) => void, index: number = 0): Button | null => {
            let button: Button | null = null;
            const panel = _buttonViewer.value as unknown as { Children: any[] };
            if (panel) {
                button = {
                    Content: buttonMsg,
                    Margin: { left: 0, top: 0, right: 10, bottom: 0 },
                    Click: clickEventHandler,
                    IsEnabled: true,
                    Visibility: 'visible'
                };
                if (index > panel.Children.length) {
                    index = panel.Children.length;
                }
                panel.Children.splice(index, 0, button);
            }
            return button;
        };

        const getSubmitButton = (): Button | null => {
            return _submitButton.value as Button;
        };

        const getSkipButton = (): Button | null => {
            return _skipButton.value as Button;
        };

        const getPreButton = (): Button | null => {
            return _preButton.value as Button;
        };

        const getNextButton = (): Button | null => {
            return _nextButton.value as Button;
        };

        const getTradePageViewer = (): ContentPresenter | null => {
            return _lv_Leads.value as ContentPresenter;
        };

        const getLeadHeader = (): WrapPanel | null => {
            return _leadHeader.value as WrapPanel;
        };

        // 绑定命令
        const bindCommands = () => {
            // 提交按钮
            // 这里需要根据 Vue 的方式来绑定命令，以下是模拟实现
            const submitCommandBinding = {
                execute: (sender: any, e: any) => {
                    ShowBottonViewer('collapsed');
                    ContractManager.RaiseSubmit(this);
                    ShowBottonViewer('visible');
                },
                canExecute: (sender: any, e: any) => {
                    e.CanExecute = true;
                }
            };

            // 下一步按钮
            const nextCommandBinding = {
                execute: (sender: any, e: any) => {
                    btnNext_Click();
                },
                canExecute: (sender: any, e: any) => {
                    e.CanExecute = true;
                }
            };

            // 跳过按钮
            const skipCommandBinding = {
                execute: (sender: any, e: any) => {
                    btnStep_Click();
                },
                canExecute: (sender: any, e: any) => {
                    e.CanExecute = true;
                }
            };

            // 上一步按钮
            const preCommandBinding = {
                execute: (sender: any, e: any) => {
                    btnPrevious_Click();
                },
                canExecute: (sender: any, e: any) => {
                    e.CanExecute = true;
                }
            };
        };

        // 构建授权实例
        const AuthorizeEntries: AuthorizeEntry[] = [];
        const authorizeEntry = {
            powerCode: '0',
            PreviewAuthorize: (sender: any, e: AuthorizeEntry.PreviewAuthorizeEventArgs) => {
                PreviewAuthorize(e);
            },
            RemoteAuthorized: (sender: any, e: any) => {
                // 这里需要根据实际的 SubmitEntry 类来实现逻辑
                // SubmitEntry.LoadData(authorizeEntry.PageData, authorizeEntry.MessageData);
                // SubmitEntry.RaiseSubmit();
            },
            PageData: null,
            MessageData: null
        };
        AuthorizeEntries.push(authorizeEntry);

        // 构建复核实例
        const RecheckEntries: RecheckEntry[] = [];
        const recheckEntry = {
            powerCode: '0',
            PreviewRecheck: (sender: any, e: RecheckEntry.PreviewRecheckEventArgs) => {
                PreviewRecheck(e);
            }
        };
        RecheckEntries.push(recheckEntry);

        const CurrentLeadIndex: Ref<number> = ref(0);
        const LeadSource: Ref<LeadItem[] | null> = ref(null);
        const IsNextBtnEnable: Ref<boolean> = ref(true);
        const _stepIndex: Ref<number> = ref(0);

        const StepIndex = {
            get: () => _stepIndex.value,
            set: (value: number) => {
                _stepIndex.value = value;
                if (getSkipButton()) {
                    getSkipButton()!.Visibility = 'visible';
                }
            }
        };

        onMounted(() => {
            const flag = ConfigManager.GetInstance().GetConfigValue('QBDJ', 'Client');
            if (flag !== 'true') return;
            const list = DependencyObjectExtensions.FindVisualTreeChildren(document.body, (x) => x instanceof HTMLElement);
            list.forEach((item) => {
                FocusManager.AddPreviewLostFocusHandler(item, (sender, e) => {
                    const element = sender as HTMLElement;
                    if (element) {
                        const money = new TBC_MoneyInfo();
                        money.Init(element.textContent || '');
                        const d = new PageDialog();
                        d.Open(this, money, '券别登记');
                        if (!money.IsOk) {
                            ControlHelper.SetFocus(this, element);
                        }
                    }
                });
            });

            if (LeadSource.value && getLeadHeader() && getLeadHeader()!.Children.length < LeadSource.value.length) {
                InitBtnContent();
                LeadSource.value.forEach((item) => {
                    const textBlock = document.createElement('div');
                    textBlock.textContent = item.Description;
                    textBlock.style.margin = '0 0 8px 0';
                    textBlock.style.fontSize = '14px';
                    textBlock.style.height = '30px';
                    textBlock.style.padding = '2px 5px';
                    getLeadHeader()!.Children.push(textBlock);
                });
                DisplayBtnContent();
            }
        });

        const PreviewSubmitting = (): [boolean, Message] => {
            const message: Message = {} as Message;
            return [false, message];
        };

        let SubmitEntry: any = null;

        const PreviewAuthorize = (args: AuthorizeEntry.PreviewAuthorizeEventArgs) => {
            return;
        };

        const PreviewRecheck = (args: RecheckEntry.PreviewRecheckEventArgs) => {
            return;
        };

        const StackPanel_Loaded = (sender: any, e: any) => {
            if (LeadSource.value && getLeadHeader() && getLeadHeader()!.Children.length < LeadSource.value.length) {
                // 这里需要根据 Vue 的方式来处理逻辑，以下是模拟实现
                LeadSource.value.forEach((item) => {
                    const textBlock = document.createElement('div');
                    textBlock.textContent = item.Description;
                    textBlock.style.margin = '0 0 8px 0';
                    textBlock.style.fontSize = '14px';
                    textBlock.style.height = '30px';
                    textBlock.style.padding = '2px 5px';
                    getLeadHeader()!.Children.push(textBlock);
                });
                DisplayBtnContent();
            }
        };

        const btnPrevious_Click = () => {
            if (CurrentLeadIndex.value <= 0) return;
            if (CurrentLeadIndex.value > (LeadSource.value?.length || 0) - 1) {
                CurrentLeadIndex.value = (LeadSource.value?.length || 0) - 1;
            }
            CurrentLeadIndex.value -= 1;
            DisplayBtnContent();
            AnimateControl('right-to-left');
        };

        const btnNext_Click = () => {
            if (CurrentLeadIndex.value > (LeadSource.value?.length || 0) - 1) {
                return;
            }

            if (CurrentLeadIndex.value < (LeadSource.value?.length || 0) - 1) {
                CurrentLeadIndex.value += 1;
                DisplayBtnContent();
                AnimateControl('left-to-right');
            } else {
                CurrentLeadIndex.value += 1;
                // 公开路由方法
                // RaiseLeadCompletedEvent();
            }
        };

        const btnStep_Click = () => {
            if (StepIndex.get() < 0 || StepIndex.get() > (LeadSource.value?.length || 0) - 1) return;
            CurrentLeadIndex.value = StepIndex.get();
            DisplayBtnContent();
            AnimateControl('left-to-right');
            if (getSkipButton()) {
                getSkipButton()!.Visibility = 'hidden';
            }
        };

        const AnimateControl = (dir: string) => {
            // 这里需要根据 Vue 的方式来实现动画逻辑，以下是模拟实现
            // const width = 710;
            // width *= Math.abs(CurrentLeadIndex.value - _preIndex);
            // 这里可以使用 Vue 的动画库来实现动画效果
            // 记录本次索引
            // _preIndex = CurrentLeadIndex.value;
        };

        const InitBtnContent = () => {
            if (!LeadSource.value) return;
            if (LeadSource.value.length === 1) {
                if (getPreButton()) {
                    getPreButton()!.Visibility = 'collapsed';
                }
                if (getNextButton()) {
                    getNextButton()!.Visibility = 'collapsed';
                }
                if (getSkipButton()) {
                    getSkipButton()!.Visibility = 'collapsed';
                }
                if (getSubmitButton()) {
                    getSubmitButton()!.Visibility = 'visible';
                }
            } else if (LeadSource.value.length > 1) {
                if (getPreButton()) {
                    getPreButton()!.Visibility = 'collapsed';
                }
                if (getNextButton()) {
                    getNextButton()!.Visibility = 'collapsed';
                }
                if (getSkipButton()) {
                    getSkipButton()!.Visibility = 'visible';
                }
                if (getSubmitButton()) {
                    getSubmitButton()!.Visibility = 'collapsed';
                }
            }
        };

        const DisplayBtnContent = () => {
            if (getPreButton()) {
                getPreButton()!.IsEnabled = true;
            }
            if (CurrentLeadIndex.value === 0) {
                if (getPreButton()) {
                    getPreButton()!.IsEnabled = false;
                }
            }

            if (CurrentLeadIndex.value === (LeadSource.value?.length || 0) - 1) {
                if (getSubmitButton()) {
                    getSubmitButton()!.Visibility = 'visible';
                }
            } else {
                if (getSubmitButton()) {
                    getSubmitButton()!.Visibility = 'collapsed';
                }
            }

            if (CurrentLeadIndex.value > 0) {
                if (getPreButton()) {
                    getPreButton()!.IsEnabled = true;
                }
            }

            if (CurrentLeadIndex.value < (LeadSource.value?.length || 0) - 1) {
                if (getNextButton()) {
                    getNextButton()!.Visibility = 'visible';
                }
            } else {
                if (getNextButton()) {
                    getNextButton()!.Visibility = 'collapsed';
                }
            }

            if (getLeadHeader()) {
                getLeadHeader()!.Children.forEach((item: HTMLElement, i: number) => {
                    if (item) {
                        if (CurrentLeadIndex.value === i) {
                            item.style.backgroundColor = 'blue';
                            item.style.color = 'white';
                        } else {
                            item.style.backgroundColor = 'transparent';
                            item.style.color = 'black';
                        }
                    }
                });
            }

            if (getTradePageViewer() && LeadSource.value) {
                getTradePageViewer()!.Content = LeadSource.value[CurrentLeadIndex.value].PageContent;
            }
        };

        return {
            // 暴露需要在模板中使用的方法和属性
            CurrentLeadIndex,
            LeadSource,
            IsNextBtnEnable,
            StepIndex,
            OnApplyTemplate,
            ShowBottonViewer,
            AddOperateButton,
            getSubmitButton,
            getSkipButton,
            getPreButton,
            getNextButton,
            getTradePageViewer,
            getLeadHeader,
            bindCommands,
            btnPrevious_Click,
            btnNext_Click,
            btnStep_Click,
            InitBtnContent,
            DisplayBtnContent
        };
    }
};