﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using Platform.Presentation.FocusManagers;
using Platform.Print;
using TellerSystem.Communication;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Library.Ext.TradeInterfaces;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using FocusInput = System.Windows.Input;
using System;
using Platform.Common.RunningParameters;
using Platform.Library;
using Platform.Presentation.PageFunctions;
using Platform.Presentation.Interfaces;
using TellerSystem.Library.Ext.TradeExtension;
using System.Xml;
using Platform.ServiceProxy.ServiceFunction.Entitys;
using TellerSystem.ServiceProxy.Ext.ServiceEntitys;
using System.Collections;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Data;
using System.IO;
using System.Xml.Linq;
using TellerSystem.Library.Ext.Variables;
using TellerSystem.Controls.Ext.Controls;
using Platform.Common.BasicFunctions;
using TellerSystem.PrintTemplate.Bankbook;
using Platform.Common.LogSystem;
using TellerSystem.Controls.Ext.PrintsHelper;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;

namespace TellerSystem.Controls.Ext.TradePageBase
{
    /// <summary>
    /// 标准的TradePage结构
    /// </summary>
    public class NormalTradePage : SubmitPage
    {
        #region 初始化
        public NormalTradePage()
        {
            #region 默认值
            TransitMessage = _msg;
            TradeInfoData.SetTradeInfo(this, new Library.Ext.TradeExtension.TradeInfoData());
            #endregion 默认值
            //关联提交逻辑入口
            SubmitEntry = new STNormalTradePage();
            //加载授权逻辑入口
            /*********************************************
             * 默认加载PowerCode为空的授权逻辑入口
             * 1、若页面存在中途授权情况，请添加权限编号和相应的授权逻辑入口
             * 2、复杂授权流程，通过交易个性化去实现
             * ******************************************/

        }
        #endregion

        #region 逻辑处理
        /// <summary>
        /// 提交前数据组装
        /// </summary>
        /// <param name="message">提交的报文对象</param>
        /// <returns></returns>
        public override bool PreviewSubmitting(out Message message)
        {
            /*********************************************
             * 此处需要填充的代码包括：
             * 1.业务处理的Message对象
             * 2.总线数据，主要提供打印逻辑使用
             * ******************************************/
            message = null;
            if (!PreviewSubmitting()) return false;
            message = TransitMessage;
            return true;
        }

        class STNormalTradePage : SubmitEntry
        {
            public STNormalTradePage()
            {
                this.IsCustomizeVerify = true;
            }

            public NormalTradePage NTP
            {
                get
                {
                    return this.PageData as NormalTradePage;
                }
            }

            public override bool SubmitComplete()
            {
                if (!NTP.Submitted())
                    return false;
                NTP.ShowCallbackPage();
                return true;
                //throw new NotImplementedException();
            }

            public override bool Finally()
            {
                return NTP.FinallyWork();
                //throw new NotImplementedException();
            }

            public override bool CreatePrintList(out List<PrintContainer> printList)
            {
                printList = NTP.Print().Cast<PrintContainer>().ToList();
                return true;
            }
        }
        #endregion


        #region 属性
        /// <summary>
        /// 8583报文通信对象
        /// </summary>
        protected Message _msg = new Message();

        /// <summary>
        /// 提交时，使用用的通讯对象Message实例
        /// </summary>
        public Message TransitMessage
        {
            get;
            set;
        }

        /// <summary>
        /// 交易视图信息集合
        /// </summary>
        public TradeInfoData TradeInfo
        {
            get
            {
                return this.GetTradeInfoData();
            }
        }

        /// <summary>
        /// 前台流水号
        /// </summary>
        public string SerialNo
        {
            get { return TradeInfo.SerialNumber; }
            set
            {
                //set无效
            }
        }

        /// <summary>
        /// 前台交易号
        /// </summary>
        public string TradeNo
        {
            get { return this.TradeModel.Code; }
            set
            {
                //set无效
            }
        }

        ///// <summary>
        ///// 交易版本
        ///// </summary>
        //public string Version
        //{
        //    get;
        //    private set;
        //}

        /// <summary>
        /// 弹出框是否按ESC关闭窗体2013-8-10 翟国栋（提供交易处理交易中途退出交易问题）
        /// True:ESC退出对话框，交易中断,return 
        /// False:正常退出，交易继续
        /// </summary>
        public bool IsDialogEscKeyClose
        {
            get { return Platform.Controls.TradeExtension.PageDialog.flagEsc; }
        }

        #endregion 属性

        #region 框架控制


        #region 流程控制方法
        /// <summary>
        /// 触发提交操作
        /// </summary>
        protected void RaiseSubmit()
        {
            SubmitCommand.Execute(null, SubmitButton);
        }

        /// <summary>
        /// 设置回显页面的呈现状态
        /// </summary>
        protected void ShowCallbackPage()
        {
            this.IsEnabled = false;
            ShowBottonViewer(System.Windows.Visibility.Collapsed);
            //呈现回显页面
            var panel = this.Content as Panel;
            if (panel == null) return;
            Panel callbackPanel = null;
            foreach (UIElement item in panel.Children)
            {
                if (item is Panel && (item as Panel).Name == "Canvas_CallbackDisplay")
                {
                    callbackPanel = item as Panel;
                }
            }
            if (callbackPanel != null && callbackPanel.Children.Count > 0)
            {
                foreach (UIElement item in panel.Children)
                {
                    item.Visibility = Visibility.Collapsed;
                }
                callbackPanel.Visibility = Visibility.Visible;
                //this.ShowPageMessageBoxExt("交易完成！",
                //    new Dictionary<System.Windows.Input.Key, string> { { System.Windows.Input.Key.Enter, "确定" } });
            }
        }

        /// <summary>
        /// 还原界面操作
        /// </summary>
        /// <returns></returns>
        protected virtual bool RestorePage()
        {
            return true;
        }

        /// <summary>
        /// 为提交工作准备数据
        /// </summary>
        /// <returns></returns>
        protected virtual bool PreviewSubmitting()
        {
            return false;
        }

        /// <summary>
        /// 提交完毕，执行结果判定
        /// </summary>
        /// <returns></returns>
        protected virtual bool Submitted()
        {
            return false;
        }

        ///// <summary>
        ///// 执行打印操作
        ///// </summary>
        ///// <returns></returns>
        protected virtual ICollection Print()
        {
            return new List<PrintContainer>();
        }

        /// <summary>
        /// 交易完成，执行后续操作
        /// </summary>
        /// <returns></returns>
        protected virtual bool FinallyWork()
        {
            return true;
        }

        /// <summary>
        /// 组织待监督数据
        /// </summary>
        /// <returns></returns>
        protected virtual void Super()
        {
        }

        #endregion 流程控制方法

        /// <summary>
        /// 执行打印操作
        /// </summary>
        /// <param name="printList"></param>
        protected void DoPrintWork(ICollection printList)
        {
            this.PrintCertificate(printList.OfType<PrintContainer>().Select(item => item as PrintContainer).ToArray());
        }
        #endregion 框架控制
    }

}
