import { ref } from 'vue';

// 假设的类型定义，需要根据实际情况完善
interface Message {
  // 这里可根据实际 Message 类的属性进行补充
}

interface TradeInfoData {
  SerialNumber: string;
  // 其他可能的属性
}

interface PrintContainer {
  // 这里可根据实际 PrintContainer 类的属性进行补充
}

// 假设的 SubmitPage 基类
class SubmitPage {
  // 可根据实际基类的属性和方法进行补充
}

// 假设的 STNormalTradePage 继承的 SubmitEntry 类
class SubmitEntry {
  IsCustomizeVerify: boolean;
  PageData: any;
  constructor() {
    this.IsCustomizeVerify = false;
    this.PageData = null;
  }
  SubmitComplete(): boolean {
    throw new Error('Method not implemented.');
  }
  Finally(): boolean {
    throw new Error('Method not implemented.');
  }
  CreatePrintList(printList: PrintContainer[]): boolean {
    throw new Error('Method not implemented.');
  }
}

class NormalTradePage extends SubmitPage {
  // 属性部分
  private _msg: Message = {} as Message;
  public TransitMessage: Message;
  public TradeInfo: TradeInfoData;
  public SerialNo: string;
  public TradeNo: string;
  public IsDialogEscKeyClose: boolean;

  // 提交逻辑入口
  public SubmitEntry: STNormalTradePage;

  constructor() {
    super();
    // 初始化默认值
    this.TransitMessage = this._msg;
    const tradeInfo = new (class implements TradeInfoData {
      SerialNumber = '';
    })();
    // 模拟 TradeInfoData.SetTradeInfo 方法
    this.TradeInfo = tradeInfo;

    // 关联提交逻辑入口
    this.SubmitEntry = new STNormalTradePage();
    this.SubmitEntry.PageData = this;
  }

  // 提交前数据组装
  public PreviewSubmitting(): [boolean, Message | null] {
    let message: Message | null = null;
    if (!this.PreviewSubmitting()) return [false, null];
    message = this.TransitMessage;
    return [true, message];
  }

  // 框架控制部分方法

  // 触发提交操作
  protected RaiseSubmit(): void {
    // 模拟 SubmitCommand.Execute 方法
    console.log('SubmitCommand.Execute');
  }

  // 设置回显页面的呈现状态
  protected ShowCallbackPage(): void {
    this.IsEnabled = false;
    this.ShowBottonViewer('Collapsed');
    // 呈现回显页面
    const panel = this.Content as HTMLElement;
    if (!panel) return;
    let callbackPanel: HTMLElement | null = null;
    for (let i = 0; i < panel.children.length; i++) {
      const item = panel.children[i] as HTMLElement;
      if (item.tagName === 'DIV' && item.id === 'Canvas_CallbackDisplay') {
        callbackPanel = item;
      }
    }
    if (callbackPanel && callbackPanel.children.length > 0) {
      for (let i = 0; i < panel.children.length; i++) {
        const item = panel.children[i] as HTMLElement;
        item.style.display = 'none';
      }
      callbackPanel.style.display = 'block';
    }
  }

  // 还原界面操作
  protected RestorePage(): boolean {
    return true;
  }

  // 为提交工作准备数据
  protected PreviewSubmitting(): boolean {
    return false;
  }

  // 提交完毕，执行结果判定
  protected Submitted(): boolean {
    return false;
  }

  // 执行打印操作
  protected Print(): PrintContainer[] {
    return [];
  }

  // 交易完成，执行后续操作
  protected FinallyWork(): boolean {
    return true;
  }

  // 组织待监督数据
  protected Super(): void {
    // 实现逻辑
  }

  // 执行打印操作
  protected DoPrintWork(printList: PrintContainer[]): void {
    const printArray = printList.filter(item => item) as PrintContainer[];
    this.PrintCertificate(printArray);
  }

  // 模拟的辅助方法
  private IsEnabled: boolean = true;
  private Content: HTMLElement | null = null;
  private ShowBottonViewer(state: string): void {
    console.log(`ShowBottonViewer: ${state}`);
  }
  private PrintCertificate(printArray: PrintContainer[]): void {
    console.log('PrintCertificate', printArray);
  }
}

class STNormalTradePage extends SubmitEntry {
  constructor() {
    super();
    this.IsCustomizeVerify = true;
  }

  get NTP(): NormalTradePage {
    return this.PageData as NormalTradePage;
  }

  public SubmitComplete(): boolean {
    if (!this.NTP.Submitted())
      return false;
    this.NTP.ShowCallbackPage();
    return true;
  }

  public Finally(): boolean {
    return this.NTP.FinallyWork();
  }

  public CreatePrintList(): [boolean, PrintContainer[]] {
    const printList = this.NTP.Print();
    return [true, printList];
  }
}

export { NormalTradePage };