﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Presentation.PageFunctions;
using TellerSystem.Contract.ContractBase;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using System.ComponentModel;
using Platform.Controls.ComponentContainer;
using TellerSystem.Controls.Ext.ControlsExt;
using TellerSystem.Communication;
using Platform.Common.BasicFunctions;
using Platform.Library;
using TellerSystem.Library.Ext.TradeExtension;
using TellerSystem.Library.Ext.Function;
using System.Data;
using TellerSystem.Communication.MessageHelper;

namespace TellerSystem.Controls.Ext.TradePageBase
{
    /// <summary>
    /// 按照步骤 1a 或 1b 操作，然后执行步骤 2 以在 XAML 文件中使用此自定义控件。
    ///
    /// 步骤 1a) 在当前项目中存在的 XAML 文件中使用该自定义控件。
    /// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
    /// 元素中:
    ///
    ///     xmlns:MyNamespace="clr-namespace:TellerSystem.Controls.Ext.TradePageBase"
    ///
    ///
    /// 步骤 1b) 在其他项目中存在的 XAML 文件中使用该自定义控件。
    /// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
    /// 元素中:
    ///
    ///     xmlns:MyNamespace="clr-namespace:TellerSystem.Controls.Ext.TradePageBase;assembly=TellerSystem.Controls.Ext.TradePageBase"
    ///
    /// 您还需要添加一个从 XAML 文件所在的项目到此项目的项目引用，
    /// 并重新生成以避免编译错误:
    ///
    ///     在解决方案资源管理器中右击目标项目，然后依次单击
    ///     “添加引用”->“项目”->[浏览查找并选择此项目]
    ///
    ///
    /// 步骤 2)
    /// 继续操作并在 XAML 文件中使用控件。
    ///
    ///     <MyNamespace:SearchPage/>
    ///
    /// </summary>
    public class SearchPage : WorkflowPage
    {
        #region 构造
        static SearchPage()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(SearchPage), new FrameworkPropertyMetadata(typeof(SearchPage)));
        }

        /// <summary>
        /// 查询控件
        /// </summary>
        public QueryControl QueryControl { get; private set; }

        /// <summary>
        /// 是否自定义验证
        /// </summary>
        public bool IsCustomizeVerify { get; set; }

        /// <summary>
        /// 是否启用查询按钮授权检查
        /// </summary>
        public bool IsAuthorizeOfSearch { get; set; }

        /// <summary>
        /// 初始化
        /// </summary>
        public override void OnApplyTemplate()
        {
            //处理某些查询交易设置了高度，导致呈现效果出错的问题
            if (this.Height != double.NaN)
                this.Height = double.NaN;

            base.OnApplyTemplate();
            QueryControl = this.GetTemplateChild("TradePageViewer") as QueryControl;
            //查询处理逻辑
            QueryControl.Query += (sender, e) =>
            {
                Message message = null;
                if (!PreviewQuerying(out message)) return;
                //指定主交易代码
                if (message == null) return;
                if (message.TransitNode == TransitType.CallServer)
                {
                    message.Fd126_5 = this.TradeInfo.MainCode;
                }
                RoutedEventHandler handler = (senderObj, eObj) =>
                {
                    var dataBox = senderObj as DataBox;
                    if (dataBox == null) return;
                    ChooseRows(dataBox);
                };

                var action = new Action<Message, DataBox>((msg, dbox) =>
                {
                    //执行通讯操作
                    // if (!this.CallServer(message)) return;
                    if (!this.DoTransit(msg)) return;

                    if (!IsCustomizeVerify)
                    {
                        if (msg.Fd12 != "0000")
                        {
                            this.TradeAPIs.ShowInfoMessageBox(this.GetError(msg));
                            return;
                        }
                        if (string.IsNullOrWhiteSpace(msg.FileData))
                        {
                            this.ShowMessageBox("没有找到符合条件的数据！");
                            return;
                        }
                    }
                    if (!QueryComplete(msg, dbox)) return;
                    Platform.Presentation.FocusManagers.FocusManager.RemoveLostFocusHandler(dbox, handler);
                    Platform.Presentation.FocusManagers.FocusManager.AddLostFocusHandler(dbox, handler);
                });
                //授权判定
                if (IsAuthorizeOfSearch)
                    this.RaiseDefaultAuthorize(message, () => action(message, e.DataHost));
                else
                    action(message, e.DataHost);
            };

            //控件滚动条定位
            var scrollViewer = GetTemplateChild("TradeScrollViewer") as ScrollViewer;
            if (scrollViewer != null && !DesignerProperties.GetIsInDesignMode(this))
            {
                ((FrameworkElement)scrollViewer.Content).HorizontalAlignment = System.Windows.HorizontalAlignment.Left;
                scrollViewer.RequestBringIntoView += (sender, e) =>
                {
                    var scroll = sender as ScrollViewer;
                    if (scroll != null)
                    {
                        if (!double.IsInfinity(e.TargetRect.Y) && e.TargetRect.Y > scroll.ActualHeight * 2 / 3)
                        {
                            scroll.ScrollToVerticalOffset(scroll.VerticalOffset + e.TargetRect.Y - scroll.ActualHeight * 2 / 3);
                        }
                    }
                };
            }
        }
        #endregion

        #region Query
        /// <summary>
        /// 提交前
        /// </summary>
        /// <param name="message"></param>
        /// <returns></returns>
        public virtual bool PreviewQuerying(out Message message)
        {
            message = null;
            return false;
        }

        /// <summary>
        /// 提交后
        /// </summary>
        /// <param name="message"></param>
        /// <returns></returns>
        public virtual bool QueryComplete(Message message, DataBox dataHost)
        {

            return false;
        }

        /// <summary>
        /// 选择数据行
        /// </summary>
        /// <param name="dataHost"></param>
        /// <returns></returns>
        public virtual void ChooseRows(DataBox dataHost)
        {

        }
        #endregion
    }
}
