﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.BasicFunctions;
using TellerSystem.Contract.ContractBase;
using System.ComponentModel;
using TellerSystem.Contract.Contracts;
using TellerSystem.Communication;
using TellerSystem.Contract.Entry;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Controls.Ext.Controls;
using TellerSystem.Controls.Ext.PrintsHelper;
using TellerSystem.Contract.Function;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using Platform.Print;
using TellerSystem.Communication.SocketListener;
using Platform.Common.LogSystem;

namespace TellerSystem.Controls.Ext.TradePageBase
{
    public class SubmitPage : PageBase, ISubmitContract, IAuthorizeContract, IRecheckContract, IPaymentFeeContract, IEvaluationContract
    {
        #region 构造
        static SubmitPage()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(SubmitPage), new FrameworkPropertyMetadata(typeof(SubmitPage)));
        }

        #region 命令

        /// <summary>
        /// 提交命令
        /// </summary>
        public static readonly RoutedUICommand SubmitCommand = new RoutedUICommand("提交", "SubmitCommand", typeof(SubmitPage));
        /// <summary>
        /// 取消命令
        /// </summary>
        public static readonly RoutedUICommand CancelCommand = new RoutedUICommand("取消", "CancelButton", typeof(SubmitPage));

        #endregion 命令

        #region 操作按钮控制

        private FrameworkElement _buttonViewer, _submitButton, _cancelButton;

        /// <summary>
        ///
        /// </summary>
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            _buttonViewer = GetTemplateChild("ButtonViewerPanel") as FrameworkElement;
            _submitButton = GetTemplateChild("SubmitButton") as FrameworkElement;
            _cancelButton = GetTemplateChild("CancelButton") as FrameworkElement;

            //控件滚动条定位
            var scrollViewer = GetTemplateChild("TradeScrollViewer") as ScrollViewer;
            if (scrollViewer != null && !DesignerProperties.GetIsInDesignMode(this))
            {
                ((FrameworkElement)scrollViewer.Content).HorizontalAlignment = System.Windows.HorizontalAlignment.Left;
                scrollViewer.RequestBringIntoView += (sender, e) =>
                {
                    var scroll = sender as ScrollViewer;
                    if (scroll != null)
                    {
                        if (!double.IsInfinity(e.TargetRect.Y) && e.TargetRect.Y > scroll.ActualHeight * 2 / 3)
                        {
                            scroll.ScrollToVerticalOffset(scroll.VerticalOffset + e.TargetRect.Y - scroll.ActualHeight * 2 / 3);
                        }
                    }
                };
            }
        }

        /// <summary>
        /// 设置操作区域的可见性
        /// </summary>
        /// <param name="visibility"></param>
        protected void ShowBottonViewer(Visibility visibility)
        {
            if (_buttonViewer != null)
            {
                _buttonViewer.Visibility = visibility;
            }
        }

        /// <summary>
        /// 在操作区域添加按钮
        /// </summary>
        /// <param name="buttonMsg"></param>
        /// <param name="clickEventHandler"></param>
        /// <param name="index"></param>
        protected Button AddOperateButton(string buttonMsg, RoutedEventHandler clickEventHandler, int index = 0)
        {
            Button button = null;
            var panel = _buttonViewer as Panel;
            if (panel != null)
            {
                button = new Button { Content = buttonMsg, Margin = new Thickness(0, 0, 10, 0) };
                button.Click += clickEventHandler;
                if (index > panel.Children.Count)
                    index = panel.Children.Count;
                panel.Children.Insert(index, button);
            }
            return button;
        }

        /// <summary>
        /// 提交按钮
        /// </summary>
        public Button SubmitButton
        {
            get { return _submitButton as Button; }
        }

        /// <summary>
        /// 取消按钮
        /// </summary>
        public Button CancelButton
        {
            get { return _cancelButton as Button; }
        }

        #endregion 操作按钮控制

        /// <summary>
        /// 构造函数
        /// </summary>
        public SubmitPage()
        {
            #region 绑定命令

            //提交按钮
            CommandBindings.Add(new CommandBinding(SubmitCommand,
                 (sender, e) =>
                 {
                     //RaiseSubmit();
                     ShowBottonViewer(Visibility.Collapsed);
                     ContractManager.RaiseSubmit(this);
                     ShowBottonViewer(Visibility.Visible);
                 }, (sender, e) =>
                 {
                     e.CanExecute = true;
                 }));
            //取消按钮
            CommandBindings.Add(new CommandBinding(CancelCommand,
                (sender, e) =>
                {
                    this.Close();
                }, (sender, e) =>
                {
                    e.CanExecute = true;
                }));

            #endregion 绑定命令

            #region 构建授权实例
            AuthorizeEntries = new List<AuthorizeEntry>();
            var authorizeEntry = new AuthorizeEntry("0");
            authorizeEntry.PreviewAuthorize += (sender, e) =>
            {
                PreviewAuthorize(e);
            };
            authorizeEntry.RemoteAuthorized += (sender, e) =>
            {
                var entry = sender as AuthorizeEntry;
                SubmitEntry.LoadData(entry.PageData, entry.MessageData);
                SubmitEntry.RaiseSubmit();
            };
            AuthorizeEntries.Add(authorizeEntry);
            #endregion

            #region 构建复核实例
            RecheckEntries = new List<RecheckEntry>();
            var recheckEntry = new RecheckEntry("0");
            recheckEntry.PreviewRecheck += (sender, e) =>
            {
                PreviewRecheck(e);
            };
            RecheckEntries.Add(recheckEntry);
            #endregion

            this.PagePrinted += (s1, e1) => ContractManager.RaiseEvaluation(this);

            this.Dispatcher.BeginInvoke(new Action<PageBase>(s1 =>
            {
                if (!TradeAPIs.CheckWorkState(true))
                {
                    this.Close();
                    return;
                }
                if (!TradeAPIs.CheckDFState(true))
                {
                    this.Close();
                    return;
                }
                var flag = Platform.Common.RunningParameters.ConfigManager.GetInstance().GetConfigValue("QBDJ", Platform.Common.RunningParameters.ConfigType.Client);
                if (flag != "true") return;
                var list = Platform.Common.BasicFunctions.DependencyObjectExtensions.FindVisualTreeChildren(s1 as DependencyObject, x => x is Platform.Controls.NumericContainer.CurrencyEditor);
                foreach (var item in list)
                {
                    Platform.Presentation.FocusManagers.FocusManager.AddPreviewLostFocusHandler(item, (sender, e) =>
                    {
                        var element = sender as Platform.Controls.NumericContainer.CurrencyEditor;
                        if (element == null) return;
                        var money = new TellerSystem.Controls.Ext.Controls.TBC_MoneyInfo();
                        money.Init(element.Text);
                        var d = new Platform.Controls.TradeExtension.PageDialog();
                        d.Open(this, money, "券别登记");
                        if (!money.IsOk)
                            Platform.Controls.ControlsHelper.ControlHelper.SetFocus(this, element);
                    });
                }
            }), System.Windows.Threading.DispatcherPriority.Background, this);
        }
        #endregion

        #region ISubmitContract
        public virtual bool PreviewSubmitting(out Message message)
        {
            message = new Message();
            return false;
        }

        public SubmitEntry SubmitEntry
        {
            get;
            protected set;
        }
        #endregion

        #region IAuthorizeContract
        /// <summary>
        /// 授权逻辑入口集合
        /// </summary>
        public List<AuthorizeEntry> AuthorizeEntries
        {
            get;
            private set;
        }
        /// <summary>
        /// 提交部分授权数据采集入口
        /// </summary>
        /// <param name="args"></param>
        public virtual void PreviewAuthorize(AuthorizeEntry.PreviewAuthorizeEventArgs args)
        {

            return;
        }
        #endregion

        #region IRecheckContract
        /// <summary>
        /// 复核逻辑入口集合
        /// </summary>
        public List<RecheckEntry> RecheckEntries
        {
            get;
            private set;
        }
        /// <summary>
        /// 提交部分复核数据采集入口
        /// </summary>
        /// <param name="args"></param>
        public virtual void PreviewRecheck(RecheckEntry.PreviewRecheckEventArgs args)
        {

            return;
        }
        #endregion

        #region IPaymentFeeContract
        /// <summary>
        /// 检查收费情况
        /// </summary>
        /// <param name="message"></param>
        /// <param name="isPaymentFee"></param>
        /// <returns></returns>
        public bool CheckFee(PaymentFeeArguments args, out bool isPaymentFee)
        {
            isPaymentFee = false;
            var _msg = args.Message;
            //throw new NotImplementedException();
            string fd16 = _msg.Fd16;
            //国密改造 新增变量
            string fd122_I = string.Empty;
            //调用者，设置的默认的收费方式
            args.Metadata.SetDictionaryItem("DefaultPaymentFeeType", _msg.Fd9_3);

            this.TradeAPIs.CommSend(_msg);
            _msg.Fd16 = "9986";
            _msg.Fd12 = fd16;
            this.TradeAPIs.CallServer(_msg);
            _msg.Fd16 = fd16;
            //国密改造 保存加密密码,防止清空
            if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") == "true")
            {
                fd122_I = _msg.Fd122_I;
            }
            //放到下面来
            if (_msg.Fd122_F == "1")
            {
                _msg.ClearFd(122);
                _msg.Fd122_F = "1";
            }
            else
            {
                _msg.ClearFd(122);
            }
            //国密改造 重新给Fd122_I赋值
            if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") == "true")
            {
                _msg.Fd122_I = fd122_I;
            }
            //决定是否执行收费
            if (_msg.Fd12 == "0000")
            {
                if (!string.IsNullOrEmpty(_msg.Fd9_1) && !string.IsNullOrEmpty(_msg.Fd9_2) && !string.IsNullOrEmpty(_msg.Fd9_3))
                {
                    _msg.Fd9_3 = " ";
                }
                isPaymentFee = false;
            }
            else if (_msg.Fd12 == "P047")
            {
                isPaymentFee = true;
                return true;
            }
            else
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// 执行收费决策
        /// </summary>
        /// <param name="message"></param>
        /// <returns></returns>
        public bool RaisePaymentFee(PaymentFeeArguments args)
        {
            var fd9_3 = args.Metadata.GetDictionaryItem<string>("DefaultPaymentFeeType");
            var _msg = args.Message;

            if (!string.IsNullOrWhiteSpace(fd9_3) && string.IsNullOrWhiteSpace(_msg.Fd9_3))
            {
                _msg.Fd9_3 = fd9_3;
            }
            var fee = new FeeViewer(_msg);
            fee.Is_Wy = false;
            if (_msg.FileData.IndexOf("存款证明手续费") != -1)
            {
                fee._is_Cunkuan = true;
            }
            this.ShowPage(fee, "收费界面");
            if (!fee.IsFinishFee)
            {
                this.TradeAPIs.ShowErrorMessageBox("收费失败！");
                return false;
            }
            //大石桥远程授权交易截图加收费截图：2998挂失、2300、2205、7997
            string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            if (FLAG == "V11" && (_msg.Fd16 == "2405" || _msg.Fd16 == "2105" || _msg.Fd16 == "3522" || _msg.Fd16 == "5105"))
            {
                if (TradeInfo.TradeScreens.Keys.Contains("fee"))
                {
                    TradeInfo.TradeScreens.Remove("fee");
                    TradeInfo.TradeScreens.Add("fee", AuthManager.GetPageImages(fee));
                }
                else
                {
                    TradeInfo.TradeScreens.Add("fee", AuthManager.GetPageImages(fee));
                }
            }
            //远程授权交易截图加收费截图：2998挂失、7997挂失、2205、7997出售凭证、2998更换介质
            if (FLAG == "V21" && (_msg.Fd16 == "2405" || _msg.Fd16 == "5112" || _msg.Fd16 == "3522" || _msg.Fd16 == "5105" || _msg.Fd16 == "2410"))
            {
                if (TradeInfo.TradeScreens.Keys.Contains("fee"))
                {
                    TradeInfo.TradeScreens.Remove("fee");
                    TradeInfo.TradeScreens.Add("fee", AuthManager.GetPageImages(fee));
                }
                else
                {
                    TradeInfo.TradeScreens.Add("fee", AuthManager.GetPageImages(fee));
                }
            }
            //添加收费凭证
            if (SubmitEntry != null && fee.PrintFeeGuest != null)
            {
                //每个交易仅需要添置一个收费凭证
                //去掉银行留存单 直接打客户回单
                if (FLAG == "V11" && _msg.Fd16 == "5105")
                {
                }
                else if ((FLAG == "V11" && _msg.Fd16 != "5105") || FLAG == "V21" || FLAG == "V22")//大石桥 非7997凭证出售 打印两张，天骄也打印两张
                {
                    SubmitEntry.MorePrintList.RemoveAll(x => x is Print_Fee);
                    SubmitEntry.MorePrintList.Add(fee.PrintFee);
                    SubmitEntry.MorePrintList.RemoveAll(x => x is Print_FeeGuest);
                    SubmitEntry.MorePrintList.Add(fee.PrintFeeGuest);
                }
                else
                {
                    SubmitEntry.MorePrintList.RemoveAll(x => x is Print_FeeGuest);
                    SubmitEntry.MorePrintList.Add(fee.PrintFeeGuest);
                }
            }
            else
            {
                SubmitEntry.MorePrintList.RemoveAll(x => x is Print_Fee);
                SubmitEntry.MorePrintList.RemoveAll(x => x is Print_FeeGuest);
            }
            return true;
        }
        #endregion

        #region IEvaluationContract
        public virtual bool PreviewEvaluation(EvaluationRecord record)
        {
            return false;
        }
        #endregion
    }
}
