﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using Platform.Presentation.FocusManagers;
using Platform.Print;
using TellerSystem.Communication;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Library.Ext.TradeInterfaces;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using FocusInput = System.Windows.Input;
using System;
using Platform.Common.RunningParameters;
using Platform.Library;
using Platform.Presentation.PageFunctions;
using Platform.Presentation.Interfaces;
using TellerSystem.Library.Ext.TradeExtension;
using System.Xml;
using TellerSystem.Library.Ext.TradeExtension;
using Platform.ServiceProxy.ServiceFunction.Entitys;
using TellerSystem.ServiceProxy.Ext.ServiceEntitys;
using System.Collections;
using System.Windows.Input;
using System.Windows.Media;
using System.IO;
using System.Reflection;
using TellerSystem.Contract.ContractBase;
using TellerSystem.Contract.Contracts;
using TellerSystem.Contract.Entry;
using TellerSystem.Contract.Function;

namespace TellerSystem.Controls.Ext.TradePageBase
{
    /// <summary>
    /// 支持流程的页面类
    /// </summary>
    public class WorkflowPage : PageBase, IAuthorizeContract, IRecheckContract, IEvaluationContract
    {
        #region Ctor
        public WorkflowPage()
        {
            AuthorizeEntries = new List<AuthorizeEntry>();
            //授权
            var entry = new AuthorizeEntry();
            entry.PreviewAuthorize += (sender, e) => { PreviewAuthorize(e); };
            AuthorizeEntries.Add(entry);
            RecheckEntries = new List<RecheckEntry>();
            this.PagePrinted += (s1, e1) => ContractManager.RaiseEvaluation(this);
            this.Dispatcher.BeginInvoke(new Action<PageBase>(s1 =>
            {
                //查询模板，默认不检查柜员签到
                bool query = false;
                if (this.TradeModel == null)
                {
                    query = true;
                }
                else
                {
                    query = this is SearchPage || this.TradeModel.Name.Contains("查询");
                }
                if (!TradeAPIs.CheckWorkState(!query))
                {
                    this.Close();
                    return;
                }
                if (!TradeAPIs.CheckDFState(!query))
                {
                    this.Close();
                    return;
                }
            }), System.Windows.Threading.DispatcherPriority.Background, this);
        }
        #endregion

        #region IAuthorizeContract
        /// <summary>
        /// 授权逻辑入口集合
        /// </summary>
        public List<AuthorizeEntry> AuthorizeEntries
        {
            get;
            private set;
        }

        /// <summary>
        /// 提交部分授权数据采集入口
        /// </summary>
        /// <param name="args"></param>
        public virtual void PreviewAuthorize(AuthorizeEntry.PreviewAuthorizeEventArgs args)
        {
            return;
        }

        private EventHandler _authorized;

        /// <summary>
        /// 触发默认的授权流程
        /// </summary>
        /// <param name="message"></param>
        public void RaiseDefaultAuthorize(Message message, Action authorized)
        {
            this.AuthorizeEntries[0].RemoteAuthorized -= _authorized;
            _authorized = (sender, e) => { authorized(); };
            this.AuthorizeEntries[0].RemoteAuthorized += _authorized;
            if (this.RaiseAuthorize(message))
            {
                //无需授权或本地时，自动调用授权通过事件
                //TODO:此次主要是为了照顾Submit模板的调用流程，可以抽空重新定义授权模块
                AuthDataHelper authdata = new AuthDataHelper(this);
                if (authdata.AuthWay != "remote")
                    _authorized(this, new EventArgs());
            }
        }
        #endregion

        #region IRecheckContract
        /// <summary>
        /// 复核逻辑入口集合
        /// </summary>
        public List<RecheckEntry> RecheckEntries
        {
            get;
            private set;
        }
        /// <summary>
        /// 提交部分复核数据采集入口
        /// </summary>
        /// <param name="args"></param>
        public virtual void PreviewRecheck(RecheckEntry.PreviewRecheckEventArgs args)
        {

            return;
        }
        #endregion

        #region IEvaluationContract
        public virtual bool PreviewEvaluation(EvaluationRecord record)
        {
            return false;
        }
        #endregion
    }
}
