﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Controls.ControlsHelper;
using Platform.Library;
using TellerSystem.Contract.Function;
using TellerSystem.Controls.Ext.ControlsHelper;
using TellerSystem.Devices.Config;
using TellerSystem.Devices.Interface;
using TellerSystem.Library.Ext.TradeExtension;
using TellerSystem.Library.Ext.Variables;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.Contract.ContractBase;
using TellerSystem.Library.Ext.Function;

namespace TellerSystem.Controls.Ext.TradePageComponent
{
    /// <summary>
    /// 输入账户组件
    /// </summary>
    public partial class TBC_AcInput : TBC_ComponentBase<TBC_AcInputInput, TBC_AcInputOutput>
    {
        #region 构造
        /// <summary>
        /// 构造函数
        /// </summary>
        public TBC_AcInput()
        {
            InitializeComponent();
        }
        #endregion
        
        #region 内部逻辑
        /// <summary>
        /// 获取账户信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tb_Acno_LostFocus(object sender, RoutedEventArgs e)
        {
            //输入为读卡方式
            if (DeviceManager.GetSwipeCardData(tb_Acno) != null)
            {
                ComponentOutput.AccountMsrInfo = DeviceManager.GetSwipeCardData(tb_Acno).Data;

            }
            else
            {
                ComponentOutput.AccountMsrInfo = new SwipeCardReaderEntity();
            }
            //输入为读IC芯片方式
            if (DeviceManager.GetICCardData(tb_Acno) != null)
            {
                ComponentOutput.AccountICInfo = DeviceManager.GetICCardData(tb_Acno).Data;
            }
            else
            {
                ComponentOutput.AccountICInfo = new ICCardEntity();
            }
            //查询子账户信息
            var acinfo = this.Page.TradeAPIs.TF_Deposit_QueryChildAccountList(new QueryChildAccountList.Input()
                {
                    ZH = tb_Acno.Text.Trim(),
                    ZHZTBZ = ComponentHelper.GLXHBZ ? "1" : "",
                    ZHLX = ComponentHelper.CXZHLX,
                    XCBZ = string.Empty
                }
            );
            if (acinfo.IsFailed)
            {
                this.Page.TradeAPIs.ShowTip(acinfo.FailureCause);
                this.Page.SetFocus();
                return;
            }

            if (acinfo.ZZHXXData.Count > 0)
            {
                //账户序号|开户日期|产品名称|账户余额|账户状态|冻结状态|止付状态|
                var selectedData = this.Page.ShowDataBoxViewer(acinfo.ZZHXX, new string[] { "账户序号", "", "开户日期", "产品名称", "账户余额", "账户状态", "冻结状态", "止付状态" });
                tb_Acseqn.Text = string.Empty;
                if (selectedData == null)
                {
                    this.Page.TradeAPIs.ShowTip(this.Page.TradeAPIs.GetNotifyMessage(NotifyMessage.N039));
                    this.Page.SetFocus();
                    return;
                }
                tb_Acseqn.Text = selectedData["账户序号"].ToString();
            }
            else
            {
                this.Page.TradeAPIs.ShowTip("没有可选的未销户的子账号信息！");
                this.Page.SetFocus();
                return;
            }

            //版本控制定义
            string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            //查询账号信息
            var acInfo = this.Page.TradeAPIs.TF_Deposit_QueryDepositAccount(new QueryDepositAccount.Input()
            {
                ZH = tb_Acno.Text.Trim(),
                XH = tb_Acseqn.Text.Trim(),
                SXBZ = (FLAG == "V21" || FLAG == "V22" || FLAG == "V19") ? "1" : "",
                SFTS = (FLAG == "V0") ? "1" : ""
            });
            if (acInfo.IsFailed)
            {

                this.Page.TradeAPIs.ShowTip(acInfo.FailureCause);
                this.Page.SetFocus();
                return;
            }
            if (acInfo.BZW.Substring(59, 1).Equals("2"))
            {
                this.Page.TradeAPIs.ShowInfoMessageBox("联网核查!此用户是灰名单用户!请注意!");
            }
            else if (acInfo.BZW.Substring(59, 1).Equals("1"))
            {
                this.Page.TradeAPIs.ShowTip("联网核查!此用户是黑名单用户!!!");
                this.Page.SetFocus();
                return;
            }
            //版本控制定义
            //string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            TradeAPIsHelper tardeApis = new TradeAPIsHelper(this.Page);
            if (FLAG == "V9" && tardeApis.Page is PageBase && (tardeApis.Page as PageBase).TradeModel != null && (tardeApis.Page as PageBase).TradeModel.Code == "2201")//辰州，2201 账户状态非正常 不允许维护
            {
                if (acInfo.ZHZT =="3")
                {
                    this.Page.TradeAPIs.ShowTip("该账户已挂失");
                    this.Page.SetFocus();
                    return;
                }
                if (!string.IsNullOrEmpty(acInfo.ESLBZ) && acInfo.ESLBZ.Substring(0, 1) == "1")
                {
                    if (acInfo.ZHDJZT != "0")
                    {
                        String message = acInfo.ZHDJZT == "1" ? "该账户完全冻结" : (acInfo.ZHDJZT == "2" ? "该账户只进不出" : "该账户部分冻结");
                        this.Page.TradeAPIs.ShowTip(message);
                        this.Page.SetFocus();
                        return;
                    }
                    if (acInfo.ZHZFZT != "0")
                    {
                        String message = acInfo.ZHZFZT == "1" ? "该账户全部止付" : (acInfo.ZHZFZT == "2" ? "该账户只进不出" : "该账户部分止付");
                        this.Page.TradeAPIs.ShowTip(message);
                        this.Page.SetFocus();
                        return;
                    }
                }
            }
            if (ComponentHelper.AccountFilter != null && !ComponentHelper.AccountFilter(acInfo))
            {
                //验证失败
                this.Page.SetFocus();
                return;
            }

            //更新账号信息
            ComponentOutput.AccountInfo = acInfo;
        }
        #endregion


        #region  属性

        /// <summary>
        /// 标题信息
        /// </summary>
        public string Header
        {
            get { return (string)GetValue(HeaderProperty); }
            set { SetValue(HeaderProperty, value); }
        }

        public static readonly DependencyProperty HeaderProperty =
            DependencyProperty.Register("Header", typeof(string), typeof(TBC_AcInput), new FrameworkPropertyMetadata("账户信息", new PropertyChangedCallback((sender, e) =>
                                                                                                                                                                   {
                                                                                                                                                                       var tmpcontrol = sender as TBC_AcInput;
                                                                                                                                                                       if (tmpcontrol == null)
                                                                                                                                                                           return;
                                                                                                                                                                       tmpcontrol.tradepanel.Header = e.NewValue.ToString();
                                                                                                                                                                   })));
        #endregion

        #region  对外公开方法

        /// <summary>
        /// 设置账户信息
        /// </summary>
        /// <param name="acno">账号</param>
        /// <param name="isReadonly">是否可读</param>
        public void SetAcno(string acno, bool isReadonly)
        {
            tb_Acno.Text = acno;
            tb_Acno.IsReadOnly = isReadonly;
            tb_Acseqn.Text = string.Empty;
        }

        /// <summary>
        /// 设置账号输入方式
        /// </summary>
        /// <param name="inputtype"></param>
        public void SetAcnoInputType(SwipeCardReaderType inputtype)
        {
            DeviceManager.SetSwipeCardReaderTracksReader(tb_Acno, inputtype);
        }

        #endregion
    }

    #region 数据限定
    /// <summary>
    /// 组件数据交互类
    /// </summary>
    public class TBC_AcInputInput : TBC_ComponentInput
    {
        #region 构造
        public TBC_AcInputInput()
        {
            //初始化数据
            GLXHBZ = true;
            CXZHLX = string.Empty;
        }
        #endregion

        #region 控制属性
        /// <summary>
        /// 是否过滤销户标志 true:过滤 false:不过滤
        /// </summary>
        public bool GLXHBZ { get; set; }

        /// <summary>
        /// 1:只查询活期 2:只查询定期 空值:返回所有活期、定期子账户 
        /// </summary>
        public string CXZHLX { get; set; }

        /// <summary>
        /// 账户查询结果过滤
        /// </summary>
        public Predicate<QueryDepositAccount> AccountFilter { get; set; }

        #endregion

        #region 交互方法

        #endregion
    }

    /// <summary>
    /// 组件数据输出类
    /// </summary>
    public class TBC_AcInputOutput : TBC_ComponentOutput
    {
        /// <summary>
        /// 输出账户信息
        /// </summary>
        public QueryDepositAccount AccountInfo { get; internal set; }

        /// <summary>
        /// 输出账户磁道信息
        /// </summary>
        public SwipeCardReaderEntity AccountMsrInfo { get; internal set; }

        /// <summary>
        /// 输出账户IC芯片信息
        /// </summary>
        public ICCardEntity AccountICInfo { get; internal set; }
    }
    #endregion
}
