﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TellerSystem.Contract.Function;
using Platform.Controls.ControlsHelper;
using TellerSystem.Library.Ext.Variables;
using TellerSystem.Controls.Ext.TradePageBase;
using TellerSystem.Controls.Ext.ControlsHelper;
using TellerSystem.Library.Ext.Function;
using Platform.Library;
using Platform.Controls.NumericContainer;

namespace TellerSystem.Controls.Ext.TradePageComponent
{
    /// <summary>
    /// 账号信息组件
    /// </summary>
    public partial class TBC_AccountInfo : TBC_ComponentBase
    {
        private AccountInfoArgs accountInfoArgs = null;

        /// <summary>
        /// 构造函数
        /// </summary>
        public TBC_AccountInfo()
        {
            InitializeComponent();

            // 介质类型
            this.CblCashFlag.SetItems(UserVariable.GetMdmType());

            // 加载币种
            this.CblCurNo.SetItems(UserVariable.Getbz());

            //支取标志
            CblPasswordFlag.SetItems(new Dictionary<string, string> { { "密码支取", "Y" }, { "无密支取", "N" } });
            CblPasswordFlag.SelectedIndex = 0;

            // 证件类型
            this.CblCertType.SetItems(UserVariable.GetPerIdType());

            this.accountInfoArgs = new AccountInfoArgs();
            this.accountInfoArgs.GetAccountName = this.TxtAcName;
            this.accountInfoArgs.GetAccountNo=this.AccountNo;
            this.accountInfoArgs.GetAccountSequence = this.AccountSequence; ;
            this.accountInfoArgs.GetCblCashFlag=this.CblCashFlag;
            this.accountInfoArgs.GetCblCertType=this.CblCertType;
            this.accountInfoArgs.GetCblCurNo=this.CblCurNo;
            this.accountInfoArgs.GetCblPasswordFlag=this.CblPasswordFlag;
            this.accountInfoArgs.GetCurBal=this.CurBal;
            this.accountInfoArgs.GetTxtCertNo=this.TxtCertNo;
            this.accountInfoArgs.GetPassword = this.TxtPassword;
        }

        /// <summary>
        /// 账号失去焦点
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AccountNo_LostFocus(object sender, RoutedEventArgs e)
        {
            QueryChildAccountList.Input input = new QueryChildAccountList.Input();
            input.ZH = this.AccountNo.Text;

            var list = this.Page.TradeAPIs.TF_Deposit_QueryChildAccountList(input);
            if (list.IsFailed)
            {
                this.Page.ShowErrorMessageBox(list.FailureCause);
                return ;
            }

            if (list.ZZHXXData.Count > 0)
            {
                //账户序号|开户日期|产品名称|账户余额|账户状态|冻结状态|止付状态|
                var selectedData = this.Page.ShowDataBoxViewer(list.ZZHXX, new string[] { "账户序号", "开户日期", "产品名称", "账户余额", "账户状态" });
                AccountSequence.Text = "";
                if (selectedData == null)
                {
                    this.Page.ShowMessageBox("没有选择账号信息!", "警告");
                    this.Page.SetFocus(AccountNo);
                    return;
                }
                AccountSequence.Text = selectedData["账户序号"].ToString();
                this.Page.SetFocus(AccountSequence);
            }
            else
            {
                this.Page.ShowMessageBox("没有可选账号信息!", "警告");
                this.Page.SetFocus(AccountNo);
                return;
            }
        }

        /// <summary>
        /// 序号失去焦点
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AccountSequence_LostFocus(object sender, RoutedEventArgs e)
        {
            QueryDepositAccount.Input inputZh = new QueryDepositAccount.Input();
            inputZh.XH = this.AccountSequence.Text;
            inputZh.ZH = this.AccountNo.Text;
            this.accountInfoArgs.GuestInfo = this.Page.TradeAPIs.TF_Deposit_QueryDepositAccount(inputZh);

            if (this.accountInfoArgs.GuestInfo.IsFailed)
            {
                this.Page.ShowErrorMessageBox(this.accountInfoArgs.GuestInfo.FailureCause);
                return;
            }

            //
            this.TxtAcName.Text = this.accountInfoArgs.GuestInfo.HM;
            this.CurBal.Text = this.accountInfoArgs.GuestInfo.YE;
            this.CblCashFlag.SetSelectedValue(this.accountInfoArgs.GuestInfo.JZZL);
            this.CblCurNo.SetSelectedValue(this.accountInfoArgs.GuestInfo.BZ);
            this.CblPasswordFlag.SetSelectedValue(this.accountInfoArgs.GuestInfo.SFPM);
            this.CblCertType.SetSelectedValue(this.accountInfoArgs.GuestInfo.ZJLX);

            if (this.accountInfoArgs.GuestInfo.SFPM == "Y")
                this.TxtPassword.IsEnabled = true;
            else
                this.TxtPassword.IsEnabled = false;
        }

        public class AccountInfoArgs
        {
            /// <summary>
            /// 此组件的客户信息
            /// </summary>
            public QueryDepositAccount GuestInfo
            {
                get;
                internal set;
            }

            /// <summary>
            /// 账号
            /// </summary>
            public TextBox GetAccountNo
            {
                get;
                internal set;
            }

            /// <summary>
            /// 获取账号序号
            /// </summary>
            public TextBox GetAccountSequence
            {
                get;
                internal set;
            }

            /// <summary>
            /// 获取账号户名
            /// </summary>
            public TextBox GetAccountName
            {
                get;
                internal set;
            }

            /// <summary>
            /// 获取账号余额
            /// </summary>
            public CurrencyEditor GetCurBal
            {
                get;
                internal set;
            }

            /// <summary>
            /// 介质种类
            /// </summary>
            public ComboBox GetCblCashFlag
            {
                get;
                internal set;
            }

            /// <summary>
            /// 币种
            /// </summary>
            public ComboBox GetCblCurNo
            {
                get;
                internal set;
            }

            /// <summary>
            /// 支取标志
            /// </summary>
            public ComboBox GetCblPasswordFlag
            {
                get;
                internal set;
            }

            /// <summary>
            /// 证件类型
            /// </summary>
            public ComboBox GetCblCertType
            {
                get;
                internal set;
            }

            /// <summary>
            /// 证件号码
            /// </summary>
            public TextBox GetTxtCertNo
            {
                get;
                internal set;
            }

            /// <summary>
            /// 密码框
            /// </summary>
            public PasswordBox GetPassword
            {
                get; 
                internal set;
            }
        }

        private void TxtCertNo_LostFocus(object sender, RoutedEventArgs e)
        {
            if (this.TxtCertNo.Text != this.Controls.GuestInfo.ZJH)
            {
                this.Page.ShowMessageBox("证件号不正确。");
                this.TxtCertNo.Text = "";
                this.Page.SetFocus();
                return;
            }

            this.OnComponentIsLostFocused();
        }

        public AccountInfoArgs Controls
        {
            get
            {
                return this.accountInfoArgs;
            }
        }

        private void TxtPassword_LostFocus(object sender, RoutedEventArgs e)
        {
            //密码验证
            var mmcheck = this.Page.TradeAPIs.TF_Deposit_PasswordAuthentication(new PasswordAuthentication.Input()
            {
                MM = TxtPassword.Password,
                ZH = AccountNo.Text.Trim()
            });
            if (mmcheck.IsFailed)
            {
                this.Page.ShowMessageBox(mmcheck.FailureCause);
                this.Page.SetFocus(TxtPassword);
                TxtPassword.Password = "";
                return;
            }
        }

        private void TxtPassword_GotFocus(object sender, RoutedEventArgs e)
        {
            this.TxtPassword.Password = "";
        }
    }
}
