﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TellerSystem.Controls.Ext.TradePageBase;
using System.Windows;
using System.Windows.Controls;
using Platform.Presentation.Interfaces;
using TellerSystem.Contract.ContractBase;
using Platform.Common.BasicFunctions;
using Platform.Presentation.FocusManagers;

namespace TellerSystem.Controls.Ext.TradePageComponent
{
    /// <summary>
    /// 组件的基类
    /// </summary>
    public class TBC_ComponentBase<T1, T2> : UserControl
        where T1 : TBC_ComponentInput, new()
        where T2 : TBC_ComponentOutput, new()
    {
        #region 构造
        public TBC_ComponentBase()
        {
            ComponentHelper = new T1();
            ComponentOutput = new T2();

            this.Loaded += TBC_ComponentBase_Loaded;
        }

        void TBC_ComponentBase_Loaded(object sender, RoutedEventArgs e)
        {
            this.Loaded -= TBC_ComponentBase_Loaded;
            InitComponent();
        }
        #endregion

        #region 属性
        private PageBase _page;
        /// <summary>
        /// 页面元素
        /// </summary>
        public PageBase Page
        {
            get
            {
                if (_page == null)
                {
                    _page = this.FindVisualTreeAncestor(x => x is PageBase) as PageBase;
                }
                return _page;
            }
        }

        /// <summary>
        /// 组件数据交互
        /// </summary>
        public T1 ComponentHelper { get; private set; }
        /// <summary>
        /// 组件数据输出
        /// </summary>
        public T2 ComponentOutput { get; private set; }
        #endregion

        #region 方法
        #region 重载
        /// <summary>
        /// 组件初始化
        /// </summary>
        protected virtual void InitComponent()
        {
        }
        #endregion

        #region 入口\出口
        /// <summary>
        /// 触发组件订阅的dhcc:FocusManager.GotFocus事件
        /// </summary>
        protected void RaiseGotFocus()
        {
            RaiseEvent(new RoutedEventArgs(FocusManager.GotFocusEvent, this));
        }

        /// <summary>
        /// 触发组件订阅的dhcc:FocusManager.LostFocus事件
        /// </summary>
        protected void RaiseLostFocus()
        {
            RaiseEvent(new RoutedEventArgs(FocusManager.LostFocusEvent, this));
        }
        #endregion
        #endregion
    }

    #region 抽象类限定
    /// <summary>
    /// 组件数据交互限定
    /// </summary>
    public abstract class TBC_ComponentInput
    {
    }

    /// <summary>
    /// 组件数据输出限定
    /// </summary>
    public abstract class TBC_ComponentOutput
    {
    }
    #endregion
}
