﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using Platform.Controls.ControlsHelper;
using TellerSystem.Contract.ContractBase;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Contract.Function;
using TellerSystem.Library.Ext.Variables;
using Platform.Common.BasicFunctions;
using Platform.Presentation.FocusManagers;
using TellerSystem.Library.Ext.Function;

namespace TellerSystem.Controls.Ext.TradePageComponent
{
    /// <summary>
    /// 转入账户信息
    /// </summary>
    public partial class TBC_InAccountInfo : UserControl
    {
        #region 属性
        /// <summary>
        /// 用于记录是否允许转入主账号
        /// </summary>
        private bool _zzh_enbale = false;
        /// <summary>
        /// 父页面对象
        /// </summary>
        public PageBase ParentPage { get; private set; }
        public InAccountInfo accountInfo = null;
        #endregion

        #region 路由事件
        #region GotFocus-路由事件
        /// <summary>
        /// 路由事件，得到焦点后发生
        /// </summary>
        public static readonly RoutedEvent GotFocusEvent = EventManager.RegisterRoutedEvent("GotFocus",
            RoutingStrategy.Direct, typeof(RoutedEventHandler), typeof(TBC_InAccountInfo));

        public static void AddGotFocusHandler(DependencyObject element, RoutedEventHandler handler)
        {
            element.AddHandler(GotFocusEvent, handler);
        }

        public static void RemoveGotFocusHandler(DependencyObject element, RoutedEventHandler handler)
        {
            element.RemoveHandler(GotFocusEvent, handler);
        }

        #endregion

        #region LostFocus-路由事件
        /// <summary>
        /// LostFocus，失去焦点后发生
        /// </summary>
        public static readonly RoutedEvent LostFocusEvent = EventManager.RegisterRoutedEvent("LostFocus",
            RoutingStrategy.Direct, typeof(RoutedEventHandler), typeof(TBC_InAccountInfo));

        public static void AddLostFocusHandler(DependencyObject element, RoutedEventHandler handler)
        {
            element.AddHandler(LostFocusEvent, handler);
        }

        public static void RemoveLostFocusHandler(DependencyObject element, RoutedEventHandler handler)
        {
            element.RemoveHandler(LostFocusEvent, handler);
        }
        #endregion

        bool RaiseFocusManagerEvent(DependencyObject target, RoutedEvent routedEvent)
        {
            var IsFocusContinue = true;

            if (target == null) return false;
            var args = new RoutedEventArgs { RoutedEvent = routedEvent };
            target.RaiseEvent(args);

            return IsFocusContinue;
        }
        #endregion

        public TBC_InAccountInfo()
        {
            InitializeComponent();
            TranInPrdtName.SetItems(UserVariable.GetNoteType());
            TranInPrdtName.SelectedIndex = 0;
            TranInCurNo.SetItems(UserVariable.GetCurNo());
            TranInCurNo.SelectedIndex = 0;
            this.Loaded += (TBC_InAccountInfo_Loaded);
            this.GotFocus += new RoutedEventHandler(TBC_InAccountInfo_GotFocus);
            this.LostFocus += new RoutedEventHandler(TBC_InAccountInfo_LostFocus);
            if (accountInfo == null)
                accountInfo = new InAccountInfo();

        }
        void TBC_InAccountInfo_GotFocus(object sender, RoutedEventArgs args)
        {
            RaiseFocusManagerEvent(sender as FrameworkElement, GotFocusEvent);
        }
        void TBC_InAccountInfo_LostFocus(object sender, RoutedEventArgs args)
        {
            RaiseFocusManagerEvent(sender as FrameworkElement, LostFocusEvent);
        }
        //组件Load事件
        void TBC_InAccountInfo_Loaded(object sender, RoutedEventArgs args)
        {
            this.Loaded -= (TBC_InAccountInfo_Loaded);
            if (DesignerProperties.GetIsInDesignMode(this)) return;//设计时不加载Load
            ParentPage = (Platform.Common.BasicFunctions.DependencyObjectExtensions.FindVisualTreeAncestor
                (this, (x) => { return x is PageBase; })) as PageBase;

        }
        //转入账号
        private void TranInActNo_LostFocus(object sender, RoutedEventArgs e)
        {
            if (ParentPage.TradeAPIs.SubStr(TranInActNo.Text, 0, 3) == "407"
                || ParentPage.TradeAPIs.SubStr(TranInActNo.Text, 0, 3) == "416"
                || ParentPage.TradeAPIs.SubStr(TranInActNo.Text, 0, 3) == "408")
            {
                TranInActNo.ShowTip("不能记此科目的帐务!");
                ParentPage.SetFocus(TranInActNo);
                return;
            }
            if (_zzh_enbale == false && TranInActNo.Text == accountInfo.ActNo)
            {
                ParentPage.TradeAPIs.ShowInfoMessageBox("转入账户和销户账户不能相同");
                ParentPage.TradeAPIs.SetFocus(TranInActNo);
                return;
            }
            //限制对公账户不能在此销户
            if (!(ParentPage.TradeAPIs.CheckCardBin(TranInActNo.Text))
                && ParentPage.TradeAPIs.SubStr(TranInActNo.Text, 0, 1).ToDecimal() > 4)
            {
                ParentPage.TradeAPIs.ShowInfoMessageBox("非个人账户不允许在此交易转账,请检查!");
                ParentPage.SetFocus(TranInActNo);
                return;
            }
            //通过转入账号和现转标志获取户名和账户序号
            var InActInFo = ParentPage.TradeAPIs.TF_Deposit_QueryDepositAccount(new QueryDepositAccount.Input
                {
                    ZH = TranInActNo.Text,
                    XH = TranInActSeqn.Text
                });
            if (!InActInFo.IsFailed)
            {
                ParentPage.TradeAPIs.ShowInfoMessageBox(InActInFo.FailureCause);
                ParentPage.SetFocus(TranInActNo);
                return;
            }
            //账户类型
            if (InActInFo.ZHLX != "1")
            {
                TranInActNo.ShowTip("转出账户为非活期账户,请检查!");
                ParentPage.SetFocus(TranInActNo);
                return;
            }
            TranInActSeqn.Text = InActInFo.XH;//序号
            TranInActName.Text = InActInFo.HM;//户名            
        }
        //账户序号
        private void TranInActSeqn_LostFocus(object sender, RoutedEventArgs e)
        {
            //根据账号和账户序号查询账户信息
            var accountInfo = ParentPage.TradeAPIs.TF_Deposit_QueryDepositAccount(new QueryDepositAccount.Input { ZH = TranInActNo.Text, XH = TranInActSeqn.Text });
            //冻结支付状态校验
            if (accountInfo.ZHZT == "1")
            {
                ParentPage.TradeAPIs.ShowInfoMessageBox("账户" + "[" + TranInActNo.Text.Trim() + "]" + "已被全部冻结或止付，不能存款!");
                ParentPage.TradeAPIs.SetFocus(TranInActSeqn);
                return;
            }
            else if (accountInfo.ZHZT == "2")
            {
                ParentPage.TradeAPIs.ShowInfoMessageBox("账户" + "[" + TranInActSeqn.Text.Trim() + "]" + "处于只进不出状态!");
            }
            if (accountInfo.ZFJE.ToDecimal() > 0 || accountInfo.DJJE.ToDecimal() > 0)
            {
                ParentPage.TradeAPIs.ShowInfoMessageBox("此户冻结" + accountInfo.DJJE.ToDecimal() + ",止付" + accountInfo.ZFJE.ToDecimal());
            }
            TranInPrdtName.SetSelectedValue(accountInfo.JZZL);
            TranInCurNo.SetSelectedValue(accountInfo.BZ);
        }

        /// <summary>
        /// 设置传入账号数据
        /// </summary>
        /// <param name="accountInfo"></param>
        public void SetInAccountInfo(InAccountInfo accountInfo)
        {
            this.accountInfo = accountInfo;
        }
        /// <summary>
        /// 获取转入账号信息
        /// </summary>
        /// <returns></returns>
        public InAccountInfo GetInAccountInfo()
        {
            InAccountInfo info = new InAccountInfo();
            info.InActNo = TranInActNo.Text;
            info.InActSeqn = TranInActSeqn.Text;
            info.InActName = TranInActName.Text;
            info.InPrdtName = TranInPrdtName.GetSelectedValue();
            info.InCurNo = TranInCurNo.GetSelectedValue();
            info.InAmt = TranInAmt.Text;
            return info;
        }
        /// <summary>
        ///  转入账户信息实体
        /// </summary>
        public class InAccountInfo
        {
            /// <summary>
            /// 卡号(传入参数) 校验转入账号和销户账号不能相同
            /// </summary>
            public string ActNo { get; set; }
            /// <summary>
            /// 现转标志 现金、转账
            /// </summary>
            public string CashFlag { get; set; }
            /// <summary>
            /// 转入账号
            /// </summary>
            public string InActNo { get; set; }
            /// <summary>
            /// 账户序号
            /// </summary>
            public string InActSeqn { get; set; }
            /// <summary>
            /// 户名
            /// </summary>
            public string InActName { get; set; }
            /// <summary>
            /// 介质总类
            /// </summary>
            public string InPrdtName { get; set; }
            /// <summary>
            /// 币种
            /// </summary>
            public string InCurNo { get; set; }
            /// <summary>
            /// 输入金额
            /// </summary>
            public string InAmt { get; set; }
        }
    }
}
