﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Platform.Common.RunningParameters;
using Platform.Controls.ControlsHelper;
using Platform.Controls.NumericContainer;
using Platform.Library;
using TellerSystem.Contract.Function;
using TellerSystem.Controls.Ext.ControlsExt;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Library.Ext.Variables;
using TellerSystem.Controls.Ext.ControlsHelper;
using TellerSystem.Library.Ext.TradeExtension;
using TellerSystem.Devices.Interface;
using TellerSystem.Devices.Config;

namespace TellerSystem.Controls.Ext.TradePageComponent
{
    /// <summary>
    /// 输入账户组件
    /// </summary>
    public partial class TBC_TransOutAccountInfo : TBC_ComponentBase<TBC_TransOutAccountInfoInput, TBC_TransOutAccountInfoOutput>
    {
        #region 构造
        /// <summary>
        /// 构造函数
        /// </summary>
        public TBC_TransOutAccountInfo()
        {
            InitializeComponent();
        }
        #endregion

        #region 内部逻辑

        /// <summary>
        /// 账号
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TRAN_AC_NO_LostFocus(object sender, RoutedEventArgs e)
        {
            ComponentOutput.AccMsrInfo = new SwipeCardReaderEntity();

            if (this.Page.TradeAPIs.Len(TRAN_AC_NO.Text) <= 7 && this.Page.TradeAPIs.SubStr(this.TRAN_AC_NO.Text, 0, 1) == "5")
            {
                this.Page.TradeAPIs.ShowErrorMessageBox("不可记此科目！");
                this.Page.TradeAPIs.SetFocus();
                return;
            }

            //9731
            var result = this.Page.TradeAPIs.TF_Deposit_QueryDepositAccount(new QueryDepositAccount.Input
            {
                ZH = TRAN_AC_NO.Text
            });
            if (result.IsFailed)
            {
                this.Page.TradeAPIs.ShowTip(result.FailureCause);
                this.Page.TradeAPIs.SetFocus();
                return;
            }

            OPN_BR_NO = result.KHXH;
            CardData = TellerSystem.Library.Ext.Function.SystemFunction.GetCardTracks(TRAN_AC_NO);

            var substr = this.Page.TradeAPIs.SubStr((TRAN_AC_NO.Text), 0, 6);    //承德卡
            if (substr == "622936" || substr == "623060")
            {
                if (this.Page.TradeAPIs.Len(CardData.Data.TracksTwo) < 36)
                {
                    this.Page.TradeAPIs.ShowTip("承德卡帐号，请刷卡！");
                    this.Page.TradeAPIs.SetFocus();
                    return;
                }
                this.SEQNO_102.Text = result.XH;
                SEQNO_102.Visibility = Visibility.Visible;
                NSEQNO_102.Visibility = Visibility.Visible;
            }
            else
            {
                SEQNO_102.Visibility = Visibility.Hidden;
                NSEQNO_102.Visibility = Visibility.Hidden;
                if (result.KHLX == "1" && result.JSLX != "D")
                {
                    this.Page.TradeAPIs.ShowErrorMessageBox("非结算户不允许转帐！");
                    this.Page.TradeAPIs.SetFocus();
                    return;
                }
                if (result.KHLX == "2" && result.KHXH != LoginUserInfo.KinbrNo)
                {
                    this.Page.TradeAPIs.ShowErrorMessageBox("对公户请到开户行结算！");
                    this.Page.TradeAPIs.SetFocus();
                    return;
                }
                if (result.KHLX == "2")
                {
                    SEQNO_102.Visibility = Visibility.Visible;
                    NSEQNO_102.Visibility = Visibility.Visible;
                    var res = this.Page.TradeAPIs.TF_Deposit_QueryChildAccountList(new QueryChildAccountList.Input()
                    {
                        ZH = this.TRAN_AC_NO.Text.Trim(),
                        ZHZTBZ = "1",
                        ZHLX = "1",
                        XCBZ = string.Empty
                    });
                    if (res.IsFailed)
                    {
                        this.Page.TradeAPIs.ShowTip(res.FailureCause);
                        this.Page.SetFocus();
                        return;
                    }
                    if (res.ZZHXXData.Count > 0)
                    {
                        var selectedData = this.Page.ShowDataBoxViewer(res.ZZHXX, new string[] { "账户序号", "开户日期", "产品名称", "账户余额", "账户状态", " 冻结状态", "止付状态" });
                        if (selectedData == null)
                        {
                            this.Page.TradeAPIs.ShowTip(this.Page.TradeAPIs.GetNotifyMessage(NotifyMessage.N039));
                            this.Page.SetFocus();
                            return;
                        }
                        SEQNO_102.Text = selectedData["账户序号"].ToString();
                    }
                }
                else
                    DoAccuntInfo(result);
            }
        }
        /// <summary>
        /// 账号序号
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SEQNO_102_LostFocus(object sender, RoutedEventArgs e)
        {
            var substr = this.Page.TradeAPIs.SubStr((TRAN_AC_NO.Text), 0, 6);

            var result = this.Page.TradeAPIs.TF_Deposit_QueryDepositAccount(new QueryDepositAccount.Input
            {
                ZH = TRAN_AC_NO.Text,
                XH = SEQNO_102.Text
            });
            if (result.IsFailed)
            {
                this.Page.TradeAPIs.ShowTip(result.FailureCause);
                this.Page.TradeAPIs.SetFocus();
                return;
            }
            if (this.Page.TradeAPIs.SubStr(result.CPH, 0, 1) == "2")
            {
                this.Page.TradeAPIs.ShowErrorMessageBox("此交易不支持卡内活期户！");
                this.Page.TradeAPIs.SetFocus();
                return;
            }
            DoAccuntInfo(result);

            //this.Page.TradeAPIs.SetItems(TRAN_CTY, new Dictionary<string, string> { { "储蓄卡", "020" }, { "借记磁条卡折", "024" } });

        }

        private void DoAccuntInfo(QueryDepositAccount obj)
        {
            #region init data
            //摘要选项
            var zyList = this.Page.TradeAPIs.TF_Public_QueryComboBoxData(new QueryComboBoxData.Input { LBBM = "0002" });
            ZY_TP2.SetItems(zyList.CDXXData.Select(x => new KeyValuePair<string, string>(x.MS, x.LXZ)));
            //凭证种类 
            var ctyList = this.Page.TradeAPIs.TF_Public_QueryComboBoxData(new QueryComboBoxData.Input { LBBM = obj.KHLX == "2" ? "0016" : "N001" });
            TRAN_CTY.SetItems(ctyList.CDXXData.Select(x => new KeyValuePair<string, string>(x.MS, x.LXZ)));
            //证件类型
            var list1 = this.Page.TradeAPIs.TF_Public_QueryComboBoxData(new QueryComboBoxData.Input { LBBM = "C002", });
            TRAN_CET.SetItems(list1.CDXXData.Select(x => new KeyValuePair<string, string>(x.MS, x.LXZ)));
            this.Page.TradeAPIs.SetItems(TRAN_PDOT, new Dictionary<string, string> { { "是", "Y" }, { "否", "N" } });
            this.Page.TradeAPIs.SetItems(TRAN_CDOT, new Dictionary<string, string> { { "是", "Y" }, { "否", "N" } });
            this.Page.TradeAPIs.SetItems(TRAN_CUR, new Dictionary<string, string> { { "人民币", "01" } });

            #endregion

            TRAN_BAL.Text = obj.YE;
            //_tran_sdot = result.SFPYJZQ; 

            if (TRAN_BAL.Val.ToDecimal() < TRAN_AMT.Text.ToDecimal())
            {
                this.Page.TradeAPIs.ShowInfoMessageBox("账号余额【" + TRAN_BAL.Val + "】不足！");
                this.Page.TradeAPIs.SetFocus();
                return;
            }
            KHLX = obj.KHLX;
            bk.Visibility = Visibility.Visible;

            if (KHLX == "2")
            {
                //出票日期
                NCPRQ.Visibility = Visibility.Visible;
                CPRQ.Visibility = Visibility.Visible;
                //摘要
                NZY_TP2.Visibility = Visibility.Visible;
                ZY_TP2.Visibility = Visibility.Visible;
                NZY_TP.Visibility = Visibility.Visible;
                ZY_TP.Visibility = Visibility.Visible;
                this.Page.TradeAPIs.SetSelectedValue(ZY_TP2, "01");
                this.Page.TradeAPIs.SetSelectedValue(TRAN_CTY, "299");
                TRAN_CTY.IsEnabled = true;
                TRAN_CNM.IsEnabled = true;
                TRAN_CNM.Text = "";

                //凭证、证件和密码
                NTRAN_CDOT.Visibility = Visibility.Collapsed;
                NTRAN_PDOT.Visibility = Visibility.Collapsed;
                TRAN_CDOT.Visibility = Visibility.Collapsed;
                TRAN_PDOT.Visibility = Visibility.Collapsed;
                NTRAN_PWD.Visibility = Visibility.Collapsed;
                TRAN_PWD.Visibility = Visibility.Collapsed;
                NTRAN_CET.Visibility = Visibility.Collapsed;
                NTRAN_CNUM.Visibility = Visibility.Collapsed;
                TRAN_CET.Visibility = Visibility.Collapsed;
                TRAN_CNUM.Visibility = Visibility.Collapsed;
            }
            else
            {
                //摘要
                NZY_TP2.Visibility = Visibility.Collapsed;
                ZY_TP2.Visibility = Visibility.Collapsed;
                NZY_TP.Visibility = Visibility.Collapsed;
                ZY_TP.Visibility = Visibility.Collapsed;
                //出票日期
                NCPRQ.Visibility = Visibility.Collapsed;
                CPRQ.Visibility = Visibility.Collapsed;
                NZP_PWD.Visibility = Visibility.Collapsed;
                ZP_PWD.Visibility = Visibility.Collapsed;
                NYYTRACENO.Visibility = Visibility.Collapsed;
                YYTRACENO.Visibility = Visibility.Collapsed;

                this.Page.TradeAPIs.SetSelectedValue(TRAN_CTY, obj.PZZL);
                TRAN_CTY.IsEnabled = false;
                TRAN_CNM.IsEnabled = false;

                //凭证、证件和密码
                NTRAN_CDOT.Visibility = Visibility.Visible;
                NTRAN_PDOT.Visibility = Visibility.Visible;
                TRAN_CDOT.Visibility = Visibility.Visible;
                TRAN_PDOT.Visibility = Visibility.Visible;
                NTRAN_PWD.Visibility = Visibility.Visible;
                TRAN_PWD.Visibility = Visibility.Visible;
                NTRAN_CET.Visibility = Visibility.Visible;
                NTRAN_CNUM.Visibility = Visibility.Visible;
                TRAN_CET.Visibility = Visibility.Visible;
                TRAN_CNUM.Visibility = Visibility.Visible;

                if (obj.SFPM == "Y")
                {
                    this.TRAN_PWD.IsEnabled = true;
                }
                else
                {
                    this.NTRAN_PWD.Visibility = Visibility.Collapsed;
                    this.TRAN_PWD.Visibility = Visibility.Collapsed;
                }

                if (obj.SFPZJZQ == "Y")
                {
                    TRAN_CET.IsEnabled = true;
                    TRAN_CNUM.IsEnabled = true;
                }
                else
                {
                    NTRAN_CET.Visibility = Visibility.Collapsed;
                    NTRAN_CNUM.Visibility = Visibility.Collapsed;
                    TRAN_CET.Visibility = Visibility.Collapsed;
                    TRAN_CNUM.Visibility = Visibility.Collapsed;
                }
            }
            if (TRAN_NAME.Visibility == Visibility.Visible) TRAN_NAME.Text = obj.HM;
            if (TRAN_PDN.Visibility == Visibility.Visible) TRAN_PDN.Text = obj.CPMC;        //产品名称
            if (TRAN_CUR.Visibility == Visibility.Visible) this.Page.TradeAPIs.SetSelectedValue(TRAN_CUR, obj.BZ);
            if (TRAN_CTY.Visibility == Visibility.Visible) this.Page.TradeAPIs.SetSelectedValue(TRAN_CTY, obj.PZZL);
            if (TRAN_CNM.Visibility == Visibility.Visible) TRAN_CNM.Text = obj.PZH;
            if (TRAN_PDOT.Visibility == Visibility.Visible) this.Page.TradeAPIs.SetSelectedValue(TRAN_PDOT, obj.SFPM);    //密码标志
            if (TRAN_CDOT.Visibility == Visibility.Visible) this.Page.TradeAPIs.SetSelectedValue(TRAN_CDOT, obj.SFPZJZQ);
            if (TRAN_CET.Visibility == Visibility.Visible) this.Page.TradeAPIs.SetSelectedValue(TRAN_CET, obj.ZJLX);        //证件类型
            if (TRAN_CNUM.Visibility == Visibility.Visible) TRAN_CNUM.Text = obj.ZJHM;       //证件号码
            if (CPRQ.Visibility == Visibility.Visible) CPRQ.Text = LoginUserInfo.TradeDate;

        }
        /// <summary>
        /// 凭证号码
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TRAN_CNM_LostFocus(object sender, RoutedEventArgs e)
        {
            if (this.Page.TradeAPIs.GetSelectedValue(TRAN_CTY) != "299")
            {
                NYYTRACENO.Visibility = Visibility.Visible;
                YYTRACENO.Visibility = Visibility.Visible;

                NCPRQ.Visibility = Visibility.Visible;
                CPRQ.Visibility = Visibility.Visible;

                //msg.Fd16 = "0900";
                var result = this.Page.TradeAPIs.TF_Deposit_CheckCustomerCertificate(new CheckCustomerCertificate.Input()
                {
                    ZH = TRAN_AC_NO.Text,
                    PZLX = this.Page.TradeAPIs.GetSelectedValue(TRAN_CTY),
                    PZHM = TRAN_CNM.Text,
                });
                if (result.IsFailed)
                {
                    this.Page.TradeAPIs.ShowTip(result.FailureCause);
                    this.Page.TradeAPIs.SetFocus();
                    return;
                }
                if (result.PZZT != "1")
                {
                    var pz = string.Empty;
                    switch (result.PZZT)
                    {
                        case "0": pz = "无效"; break;
                        case "2": pz = "挂失"; break;
                        case "3": pz = "回收"; break;
                        case "4": pz = "作废"; break;
                        case "5": pz = "使用"; break;
                    }

                    this.Page.TradeAPIs.ShowInfoMessageBox("当前凭证状态【" + pz + "】");
                    this.Page.TradeAPIs.SetFocus();
                    return;
                }
            }
            else
            {
                NYYTRACENO.Visibility = Visibility.Collapsed;
                YYTRACENO.Visibility = Visibility.Collapsed;
                NCPRQ.Visibility = Visibility.Collapsed;
                CPRQ.Visibility = Visibility.Collapsed;
                NZP_PWD.Visibility = Visibility.Collapsed;
                ZP_PWD.Visibility = Visibility.Collapsed;
            }
        }
        /// <summary>
        /// 出票日期
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CPRQ_LostFocus(object sender, RoutedEventArgs e)
        {
            var pzzl = this.Page.TradeAPIs.GetSelectedValue(TRAN_CTY);
            if (pzzl == UserVariable.PZZL_XJZP || pzzl == UserVariable.PZZL_ZZZP || pzzl == "167" || pzzl == "002" || pzzl == "168" || pzzl == "189")
            {
                if (this.Page.TradeAPIs.Func(CPRQ.Text, LoginUserInfo.TradeDate) > 10)
                {
                    this.Page.TradeAPIs.ShowTip("支票有效期限已过10天！");
                    this.Page.TradeAPIs.SetFocus();
                    return;
                }
                var tf = this.Page.TradeAPIs.TF_Deposit_PasswordAuthentication(new PasswordAuthentication.Input()
                {
                    GYH = LoginUserInfo.TellerNo,
                    ZH = TRAN_AC_NO.Text,
                    CPRQ = CPRQ.Text,
                    PZHM = TRAN_CNM.Text,
                    PZLX = this.Page.TradeAPIs.GetSelectedValue(TRAN_CTY),
                    YZBZ = "4",
                });
                if (tf.IsFailed)
                {
                    this.Page.TradeAPIs.ShowTip(tf.FailureCause);
                    this.Page.TradeAPIs.SetFocus();
                    return;
                }
                NZP_PWD.Visibility = Visibility.Visible;
                ZP_PWD.Visibility = Visibility.Visible;
                if (tf.PZMMYZFS == "0")
                {
                    this.Page.TradeAPIs.ShowInfoMessageBox("校验密码单!!");
                }
                else if (tf.PZMMYZFS == "1")
                {
                    this.Page.TradeAPIs.ShowInfoMessageBox("校验支付密码器!!!");
                }
                else
                {
                    NZP_PWD.Visibility = Visibility.Collapsed;
                    ZP_PWD.Visibility = Visibility.Collapsed;
                }
                //给全局变量赋值
                TansPassWordType = tf.PZMMYZFS;
            }
            else
            {
                NZP_PWD.Visibility = Visibility.Collapsed;
                ZP_PWD.Visibility = Visibility.Collapsed;
            }
        }
        /// <summary>
        /// 凭证密码
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ZP_PWD_LostFocus(object sender, RoutedEventArgs e)
        {
            var pwdlen = this.Page.TradeAPIs.Len(ZP_PWD.Text);
            if (pwdlen != 16 && pwdlen != 10)
            {
                this.Page.TradeAPIs.ShowErrorMessageBox("密码长度必须为10位或16位！");
                this.Page.TradeAPIs.SetFocus();
                return;
            }
            var tf = this.Page.TradeAPIs.TF_Deposit_PasswordAuthentication(new PasswordAuthentication.Input()
            {
                GYH = LoginUserInfo.TellerNo,
                YZBZ = TansPassWordType == "0" ? "2" : "3",
                ZH = TRAN_AC_NO.Text,
                CPRQ = CPRQ.Text,
                PZHM = TRAN_CNM.Text,
                PZLX = this.Page.TradeAPIs.GetSelectedValue(TRAN_CTY),
                MM = ZP_PWD.Text,
                PMJE = TRAN_AMT.Val,
            });
            if (tf.IsFailed)
            {
                this.Page.TradeAPIs.ShowTip(tf.FailureCause);
                this.Page.TradeAPIs.SetFocus();
                return;
            }

        }
        /// <summary>
        /// 密码
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TRAN_PWD_LostFocus(object sender, RoutedEventArgs e)
        {
            //msg.Fd16 = "9317";
            var zhmmyz = this.Page.TradeAPIs.TF_Deposit_PasswordAuthentication(new PasswordAuthentication.Input()
            {
                ZH = TRAN_AC_NO.Text,
                MM = TRAN_PWD.Password,
                YZBZ = "0",
            });
            if (zhmmyz.IsFailed)
            {
                this.Page.TradeAPIs.ShowTip(zhmmyz.FailureCause);
                this.Page.TradeAPIs.SetFocus();
                return;
            }
        }
        /// <summary>
        /// 摘要
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ZY_TP2_LostFocus(object sender, RoutedEventArgs e)
        {
            if (this.Page.TradeAPIs.GetSelectedValue(ZY_TP2) != "31")
            {
                ZY_TP.Text = this.Page.TradeAPIs.GetSelectedKey(ZY_TP2);
                ZY_TP.IsEnabled = false;
            }
            else
            {
                ZY_TP.Text = "";
                ZY_TP.IsEnabled = true;
            }
        }
        /// <summary>
        /// 验印流水
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void YYTRACENO_LostFocus(object sender, RoutedEventArgs e)
        {
            if (!this.Page.TradeAPIs.CheckYanYinWater(TRAN_AC_NO.Text, TRAN_CNM.Text, TRAN_AMT.Val, CPRQ.Text, this.Page.TradeAPIs.GetSelectedValue(TRAN_CTY), YYTRACENO.Text))
            {
                this.Page.TradeAPIs.ShowTip("验印流水失败");
                this.Page.TradeAPIs.SetFocus();
                return;
            }
        }
        #endregion

        #region 属性
        public DeviceEntity<SwipeCardReaderEntity> CardData { get; private set; }
        /// <summary>
        /// 开户类型
        /// </summary>
        public string KHLX { get; private set; }
        /// <summary>
        /// 开户行号
        /// </summary> 
        public string OPN_BR_NO { get; private set; }
        /// <summary>
        /// 转出金额
        /// </summary> 
        public string TANSAMT { set { TRAN_AMT.Text = value; } }
        /// <summary>
        /// 账号
        /// </summary>
        public TextBox TansAcno { get { return TRAN_AC_NO; } }
        /// <summary>
        /// 账号序号
        /// </summary>
        public string TansAcseq { get { return this.Page.TradeAPIs.Delspace(SEQNO_102.Text); } }
        /// <summary>
        /// 产品名称
        /// </summary>
        public string TansPdname { get { return this.Page.TradeAPIs.Delspace(TRAN_PDN.Text); } }
        /// <summary>
        /// 账号户名
        /// </summary>
        public string TansName { get { return this.Page.TradeAPIs.Delspace(TRAN_NAME.Text); } }
        /// <summary>
        /// 币种
        /// </summary>
        public ComboBox TansCurType { get { return TRAN_CUR; } }
        /// <summary>
        /// 余额
        /// </summary>
        public string TansBal { get { return this.Page.TradeAPIs.Delspace(TRAN_BAL.Val); } }
        /// <summary>
        /// 凭证号码
        /// </summary>
        public string TansPnum { get { return this.Page.TradeAPIs.Delspace(TRAN_CNM.Text); } }
        /// <summary>
        /// 出票日期
        /// </summary>
        public string TansCprq { get { return this.Page.TradeAPIs.Delspace(CPRQ.Text); } }
        /// <summary>
        /// 凭证类型
        /// </summary>
        public ComboBox TansPtype { get { return TRAN_CTY; } }
        /// <summary>
        /// 密码支取
        /// </summary>
        public ComboBox TansPdot { get { return TRAN_PDOT; } }
        /// <summary>
        /// 证件支取
        /// </summary>
        public ComboBox TansCdot { get { return TRAN_CDOT; } }
        /// <summary>
        /// 密码
        /// </summary>
        public string TansPwd { get { return this.Page.TradeAPIs.Delspace(TRAN_PWD.Password); } }
        /// <summary>
        /// 证件类型
        /// </summary>
        public ComboBox TansCType { get { return TRAN_CET; } }
        /// <summary>
        /// 证件号码
        /// </summary>
        public string TansCnum { get { return this.Page.TradeAPIs.Delspace(TRAN_CNUM.Text); } }
        /// <summary>
        /// 摘要选项
        /// </summary>
        public ComboBox TansZYtype { get { return ZY_TP2; } }
        /// <summary>
        /// 摘要
        /// </summary>
        public string TansZY { get { return this.Page.TradeAPIs.Delspace(ZY_TP.Text); } }
        /// <summary>
        /// 验印流水
        /// </summary>
        public string TansYY { get { return this.Page.TradeAPIs.Delspace(YYTRACENO.Text); } }
        /// <summary>
        /// 支付密码类型
        /// </summary>
        public string TansPassWordType { get; private set; }

        #endregion
    }

    #region 数据限定
    /// <summary>
    /// 组件数据交互类
    /// </summary>
    public class TBC_TransOutAccountInfoInput : TBC_ComponentInput
    {

    }

    /// <summary>
    /// 组件数据输出类
    /// </summary>
    public class TBC_TransOutAccountInfoOutput : TBC_ComponentOutput
    {

        /// <summary>
        /// 输出账户磁道信息
        /// </summary>
        public SwipeCardReaderEntity AccMsrInfo { get; internal set; }
    }
    #endregion
}
