﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace TellerSystem.Library.Ext.ExternalSystem
{
    /// <summary>
    /// 调用windows相关api方法与常量
    /// </summary>
    public class Win32API
    {
        #region 相关常量
        public const int WS_VSCROLL = 0x00200000;
        public const int SWP_NOOWNERZORDER = 0x200;
        public const int SWP_NOREDRAW = 0x8;
        public const int SWP_NOZORDER = 0x4;
        public const int SWP_SHOWWINDOW = 0x0040;
        public const int WS_EX_MDICHILD = 0x40;
        public const int SWP_FRAMECHANGED = 0x20;
        public const int SWP_NOACTIVATE = 0x10;
        public const int SWP_ASYNCWINDOWPOS = 0x4000;
        public const int SWP_NOMOVE = 0x2;
        public const int SWP_NOSIZE = 0x1;
        public const int GWL_STYLE = (-16);
        public const int WS_VISIBLE = 0x10000000;
        public const int WM_CLOSE = 0x10;
        public const int WS_CHILD = 0x40000000;
        public const int WM_CHAR = 0X102;
        public const int WM_KEYDOWN = 0X100;
        public const int WM_KEYUP = 0X101;
        public const int WM_SYSKEYUP = 0X105;
        public const int WM_SYSKEYDOWN = 0X104;
        /// <summary>
        /// //显示最小化
        /// </summary>
        public const int SW_SHOWMINIMIZED = 2;
        /// <summary>
        /// 窗口命令
        /// </summary>
        public const int WM_SYSCOMMAND = 0x0112;
        /// <summary>
        /// 最大化窗口
        /// </summary>
        public const int SC_MAXIMIZE = 0xF030;
        /// <summary>
        /// 键盘插入键
        /// </summary>
        public const int INSERT = 0x2D;
        public const int WM_SETFOCUS = 0x0007;
        public const int WM_ACTIVATE = 0x0006;
        #endregion

        #region 外部方法
        /// <summary>
        /// 给窗口发消息
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="Msg"></param>
        /// <param name="wParam"></param>
        /// <param name="lParam"></param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern int SendMessage(IntPtr hWnd, int Msg, int wParam, int lParam);

        /// <summary>
        /// 给窗口发送
        /// </summary>
        /// <param name="hWnd">接收窗口句柄</param>
        /// <param name="Msg">发送的消息</param>
        /// <param name="wParam"></param>
        /// <param name="lParam"></param>
        /// <returns></returns>
        [DllImport("User32.dll", EntryPoint = "SendMessage")]
        public static extern int SendMessage(IntPtr hWnd, int Msg, IntPtr wParam, string lParam);

        /// <summary>
        /// 设置窗口样式
        /// </summary>
        /// <param name="hwnd">窗口句柄</param>
        /// <param name="nIndex">属性索引</param>
        /// <param name="dwNewLong">属性值</param>
        /// <returns></returns>
        [DllImport("user32.dll", SetLastError = true)]
        public static extern long SetWindowLong(IntPtr hwnd, int nIndex, int dwNewLong);

        /// <summary>
        /// 设置窗口的父窗口
        /// </summary>
        /// <param name="hWndChild">预设置窗口句柄</param>
        /// <param name="hWndNewParent">父窗口句柄</param>
        /// <returns></returns>
        [DllImport("user32.dll", SetLastError = true)]
        public static extern long SetParent(IntPtr hWndChild, IntPtr hWndNewParent);

        /// <summary>
        /// 设置输入焦点
        /// </summary>
        /// <param name="winHandl">获取焦点的窗口或者控件句柄</param>
        /// <returns></returns>
        [DllImport("user32.dll", SetLastError = true)]
        public static extern IntPtr SetFocus(IntPtr winHandl);

        /// <summary>
        /// 激活窗口
        /// </summary>
        /// <param name="hWnd">需要激活的窗口句柄</param>
        /// <returns></returns>
        [DllImport("user32.dll", SetLastError = true)]
        public static extern IntPtr SetActiveWindow(IntPtr hWnd);

        /// <summary>
        /// 获取hwnd代表的窗口的屏幕显示区域
        /// </summary>
        /// <param name="hwnd">窗口句柄引用</param>
        /// <param name="rect">屏幕显示的窗口矩形区域</param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern bool GetWindowRect(HandleRef hwnd, out NativeRECT rect);

        /// <summary>
        /// 设置鼠标在屏幕上的位置
        /// </summary>
        /// <param name="X">以左上角为0点的横向坐标</param>
        /// <param name="Y">以左上角为0点的纵向坐标</param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern bool SetCursorPos(int X, int Y);

        /// <summary>
        /// 执行鼠标事件
        /// </summary>
        /// <param name="flags">鼠标的按键标识</param>
        /// <param name="dx">触发事件时鼠标的横向偏移量</param>
        /// <param name="dy">>触发事件时鼠标的纵向偏移量</param>
        /// <param name="data"></param>
        /// <param name="extraInfo"></param>
        [DllImport("user32.dll")]
        public static extern void mouse_event(MouseEventFlag flags, int dx, int dy, uint data, UIntPtr extraInfo);
        #endregion
    }
}
