﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using TellerSystem.ServiceProxy.Ext.TradeBusinessEntitys;

namespace TellerSystem.Library.Ext.ExternalSystem
{
    /// <summary>
    /// 外接系统应用程序管理类
    /// 目前该类只针对netterm的打开
    /// 如有其它系统，需要根据系统特点做相应调整
    /// </summary>
    public class Win32App
    {
        private Process process;
        private IntPtr appMainFormHandle;

        /// <summary>
        /// 外接应用程序主窗口句柄
        /// </summary>
        public IntPtr AppMainFormHandle
        {
            get { return appMainFormHandle; }
            private set { appMainFormHandle = value; }
        }

        /// <summary>
        /// 外接系统基本信息对象
        /// </summary>
        public TT_Extsys_ExtuserFromByGroup ExtSystemInfo
        {
            get;
            set;
        }

        public Win32App(TT_Extsys_ExtuserFromByGroup extSysInfo)
        {
            this.ExtSystemInfo = extSysInfo;
        }

        /// <summary>
        /// 打开外接应用程序
        /// </summary>
        /// <returns>外接应用程序主窗口</returns>
        public IntPtr StartApp()
        {
            ProcessStartInfo psi = new ProcessStartInfo(this.ExtSystemInfo.ExtAddress);
            psi.RedirectStandardOutput = true;
            psi.WindowStyle = ProcessWindowStyle.Maximized;
            psi.UseShellExecute = false;
            process = System.Diagnostics.Process.Start(psi);
            // Wait for process to be created and enter idle condition
            process.WaitForInputIdle();
            // 最小化
            Win32API.SendMessage(process.MainWindowHandle,Win32API.WM_SYSCOMMAND, (IntPtr)Win32API.SC_MAXIMIZE, "");
           // this.process.Add(this.entity, process);

            // 获取主窗口句柄
            return this.appMainFormHandle = process.MainWindowHandle;
        }

        /// <summary>
        /// 将外接应用程序宿主到指定的窗口中
        /// </summary>
        /// <param name="parentHandle">父窗口句柄</param>
        public void HostInTo(IntPtr parentHandle)
        {
            // 去掉窗口边框
            Win32API.SetWindowLong(this.appMainFormHandle, Win32API.GWL_STYLE, Win32API.WS_CHILD | Win32API.WS_VISIBLE | Win32API.WS_VSCROLL);
            Win32API.SetParent(this.appMainFormHandle, parentHandle);
        }

        public void DestroyWindow()
        {
            process.Kill();
            //Win32API.DestroyWindow(this.appMainFormHandle);
        }

        /// <summary>
        /// 设置焦点
        /// </summary>
        public void SetFocus()
        {
            //Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_ACTIVATE, 0, 0);
            //Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_SETFOCUS, 0, 0);

            Win32API.SetActiveWindow(this.appMainFormHandle);
            Win32API.SetFocus(this.appMainFormHandle);
            Win32API.SetCursorPos(300, 300);
        }

        /// <summary>
        /// 发送一条消息
        /// </summary>
        /// <param name="msg">消息内容</param>
        public void SendLine(string msg)
        {
            if (this.appMainFormHandle.ToInt32() != 0)
            {
                this.SendLine(msg,this.appMainFormHandle);
            }
        }

        /// <summary>
        /// 往指定窗口发送一条消息
        /// </summary>
        /// <param name="msg">消息内容</param>
        /// <param name="handle">接收消息窗口</param>
        private void SendLine(string msg,IntPtr handle)
        {
            byte[] ch = (ASCIIEncoding.ASCII.GetBytes(msg));
            for (int i = 0; i < ch.Length; i++)
            {
                Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_CHAR, ch[i], 0);
            }
        }

        public void LoadOPSystem()
        {
            this.SendLine("winqt");
            this.SendEnter();
            this.SendLine("winqt");
        }

        public void OpenSoftWare()
        {
            this.SendLine("cibas");
            this.SendEnter();
        }

        public void LoginSoftWare()
        {
            this.SendLine(this.ExtSystemInfo.ExtUserId);
            this.SendEnter();
            this.SendLine(this.ExtSystemInfo.ExtPsw);
        }

        public void OpenTrade()
        {
            Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_KEYDOWN, Win32API.INSERT, 0); //输入ENTER（0x0d）
            Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_KEYUP, Win32API.INSERT, 0);
            this.SendLine("3000");
        }

        /// <summary>
        /// 给当前窗口发送回车键
        /// </summary>
        public void SendEnter()
        {
            Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_KEYDOWN, 0X0D, 0); //输入ENTER（0x0d）
            Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_KEYUP, 0X0D, 0);
        }
    }
}
