import { spawn } from 'child_process';
import Win32API from '@/views/front/develop/Library.Ext/ExternalSystem/CS/Win32API';
import { TT_Extsys_ExtuserFromByGroup } from "@/views/front/develop/ServiceProxy.Ext/TradeBusinessEntitys/TT_Extsys_ExtuserFromByGroup"



// // 假设TT_Extsys_ExtuserFromByGroup接口定义
// interface TT_Extsys_ExtuserFromByGroup {
//     ExtAddress: string;
//     ExtUserId: string;
//     ExtPsw: string;
// }

export class Win32App {
    private process: any;
    private appMainFormHandle: number = 0;
    private extSystemInfo: TT_Extsys_ExtuserFromByGroup;

    constructor(extSysInfo: TT_Extsys_ExtuserFromByGroup) {
        this.extSystemInfo = extSysInfo;
    }

    get AppMainFormHandle(): number {
        return this.appMainFormHandle;
    }

    get ExtSystemInfo(): TT_Extsys_ExtuserFromByGroup {
        return this.extSystemInfo;
    }

    set ExtSystemInfo(value: TT_Extsys_ExtuserFromByGroup) {
        this.extSystemInfo = value;
    }

    // 启动外部应用程序
    public StartApp(): number {
        const psi = {
            command: this.extSystemInfo.ExtAddress,
            windowsVerbatimArguments: true,
            stdio: 'pipe',
            windowsHide: false
        };

        this.process = spawn(psi.command, [], {
            shell: true,
            windowsVerbatimArguments: psi.windowsVerbatimArguments,
            // stdio: psi.stdio, hulei-/ 不太理解意思。 暂时注掉
            windowsHide: psi.windowsHide
        });

        // 在Node.js中无法直接获取窗口句柄，这里需要额外处理
        // 假设通过某种方式获取到了窗口句柄
        this.appMainFormHandle = 12345; // 示例值，实际需要获取真实句柄

        // 最大化窗口
        Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_SYSCOMMAND, Win32API.SC_MAXIMIZE, 100);

        return this.appMainFormHandle;
    }

    // 将外部应用程序窗口嵌入到父窗口中
    public HostInTo(parentHandle: number): void {
        Win32API.SetWindowLong(this.appMainFormHandle, Win32API.GWL_STYLE,
            Win32API.WS_CHILD | Win32API.WS_VISIBLE | Win32API.WS_VSCROLL);
        Win32API.SetParent(this.appMainFormHandle, parentHandle);
    }

    // 销毁窗口
    public DestroyWindow(): void {
        this.process.kill();
    }

    // 设置焦点
    public SetFocus(): void {
        Win32API.SetActiveWindow(this.appMainFormHandle);
        Win32API.SetFocus(this.appMainFormHandle);
        Win32API.SetCursorPos(300, 300);
    }

    // 发送消息
    public SendLine(msg: string): void {
        if (this.appMainFormHandle !== 0) {
            this.SendLineToHandle(msg, this.appMainFormHandle);
        }
    }

    private SendLineToHandle(msg: string, handle: number): void {
        const ch = Buffer.from(msg, 'ascii');
        for (let i = 0; i < ch.length; i++) {
            Win32API.SendMessage(handle, Win32API.WM_CHAR, ch[i], 0);
        }
    }

    // 系统操作流程
    public LoadOPSystem(): void {
        this.SendLine("winqt");
        this.SendEnter();
        this.SendLine("winqt");
    }

    public OpenSoftWare(): void {
        this.SendLine("cibas");
        this.SendEnter();
    }

    public LoginSoftWare(): void {
        this.SendLine(this.extSystemInfo.ExtUserId);
        this.SendEnter();
        this.SendLine(this.extSystemInfo.ExtPsw);
    }

    public OpenTrade(): void {
        Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_KEYDOWN, Win32API.INSERT, 0);
        Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_KEYUP, Win32API.INSERT, 0);
        this.SendLine("3000");
    }

    // 发送回车键
    public SendEnter(): void {
        Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_KEYDOWN, 0x0D, 0);
        Win32API.SendMessage(this.appMainFormHandle, Win32API.WM_KEYUP, 0x0D, 0);
    }
}
