// 假设这些类型在其他文件中定义，这里先简单声明

import { Win32App } from "@/views/front/develop/Library.Ext/ExternalSystem/CS/Win32App";

class ScriptHelper {
    /**
     * 分析脚本，取出外接系统相关参数
     * @param script 脚本内容
     * @param app Win32应用对象
     */
    public static Analysis(script: string, app: Win32App): void {
        const xDoc = new DOMParser().parseFromString(script, 'text/xml');
        // to do 取出外接系统相关参数
        // 比如机器名称，密码，服务器地址等相关信息
    }

    /**
     * 执行脚本
     * @param htmlDoc HTML文档对象
     * @param script 脚本内容
     * @param app Win32应用对象 huleiSYS  System.Windows.Forms  这里暂时使用弃用的类型
     */
    public static ExcutScript(htmlDoc: HTMLDocument, script: string, app: Win32App): void {
        const xDoc = XDocument.Parse(script.toLowerCase());
        const root = xDoc.Element('externalsystemautologin');

        // 赋值
        if (root) {
            const loginParameter = root.Element('loginparameter');
            if (loginParameter) {
                for (const item of loginParameter.Elements()) {
                    const element = this.GetElement(item, htmlDoc);
                    if (element === null) {
                        // 没有取到HtmlElement
                        continue;
                    }

                    const property = item.Element('property');
                    if (property) {
                        for (const prop of property.Attributes()) {
                            element.setAttribute(prop.Name.LocalName, prop.Value === '@name' ? app.ExtSystemInfo.ExtName : app.ExtSystemInfo.ExtPsw);
                        }
                    }
                }
            }
        }

        // 登录
        if (root) {
            const loginMethod = root.Element('loginmethod');
            if (loginMethod) {
                for (const item of loginMethod.Elements()) {
                    const method = item.Element('method');
                    if (!method) continue;

                    // 如果是Item
                    if (item.Name.LocalName.toLowerCase() === 'item') {
                        const element = this.GetElement(item, htmlDoc);
                        if (element === null) {
                            // 没有取到HtmlElement
                            continue;
                        }

                        const attName = method.Attribute('name');
                        const attParameter = method.Attribute('parameter');
                        if (!attName) return;

                        if (!attParameter) {
                            // element.InvokeMember(attName.Value); huleiSYS 这里element没有InvokeMember需要具体实现
                        } else {
                            // element.InvokeMember(attName.Value, attParameter.Value);//huleiSYS 这里element没有InvokeMember需要具体实现
                        }
                    }
                    // 如果是脚本
                    else if (item.Name.LocalName.toLowerCase() === 'script') {
                        const attName = method.Attribute('name');
                        const attParameter = method.Attribute('parameter');
                        if (!attName) return;

                        if (!attParameter) {
                            // htmlDoc.InvokeScript(attName.Value);//hulei-s 这里element没有InvokeScript需要具体实现
                        } else {
                            // htmlDoc.InvokeScript(attName.Value, [attParameter.Value]); 同上
                        }
                    }
                }
            }
        }
    }

    /**
     * 根据XElement获取HtmlElement
     * @param item XElement对象
     * @param htmlDoc HTML文档对象
     * @returns HtmlElement对象或null
     */
    private static GetElement(item: Element, htmlDoc: HTMLDocument): HTMLElement | null {
        // const attID = item.attributes('id');

        // // 如果att中有ID，使用ID选择器选择
        // if (attID) {
        //     return htmlDoc.GetElementById(attID.Value);
        // }

        // const attName = item.Attribute('name');

        // // 如果有Name
        // if (attName) {
        //     const items = htmlDoc.All[attName.Value];

        //     // 如果唯一
        //     if (items && items.Children.Count === 1) {
        //         return items;
        //     }
        // }

        // let flg = true;
        // const attTag = item.Attribute('tagname');

        // // 其他
        // for (let i = 0; i < htmlDoc.Body.All.length; i++) {
        //     const element = htmlDoc.Body.All[i];

        //     // 如果有TagName
        //     if (attTag) {
        //         if (element.TagName.toLowerCase() !== attTag.Value.toLowerCase()) {
        //             continue;
        //         }
        //     }

        //     // 如果有Name
        //     if (attName) {
        //         if (element.Name.toLowerCase() !== attName.Value.toLowerCase()) {
        //             continue;
        //         }
        //     }

        //     flg = true;

        //     // 其他选择器取并集
        //     for (const att of item.Attributes().filter(att => att.Value && att.Name.LocalName.toLowerCase() !== 'tagname' && att.Name.LocalName.toLowerCase() !== 'name')) {
        //         if (element.GetAttribute(att.Name.LocalName).toLowerCase() !== att.Value.toLowerCase()) {
        //             flg = false;
        //             break;
        //         }
        //     }

        //     if (flg) {
        //         return element;
        //     }
        // }

        return null;
    }
}

// 由于没有实际的XDocument.Parse方法，这里简单模拟
namespace XDocument {
    export function Parse(script: string) {
        ///这里可以使用xmldom库 但是目前安装不上库: XDocument
        // 这里只是简单返回一个模拟对象，实际需要根据XML解析逻辑实现
        return {
            Element: (p0?: string) => null
        };
    }
}

export { ScriptHelper };
