// 模拟部分依赖

import Message from "../../Communication/Message";
import { PrintAttributes } from "../../PrintTemplate/Extension/PrintAttributes";

// 这里只是简单模拟，实际需要根据具体实现来补充
interface IPage {
    OpenTradePage(arg0: string, Dialog: OpenOptions, arg2: null, bus: any, arg4: string): unknown;
    CallServer(msg: Message): unknown;
    OwnerContainer: any;
    ShowWarnMessageBox: (msg: string) => void;
    GetTradeInfoData: () => any;
    CreateSerialNumber: () => string;
    CreateTellerSerialNumber: (serialid: string) => string;
    CreatePrintNum: (serialId: string) => string;
    CheckLoginStatus: () => boolean;
    CheckCashQuota: () => void;
    SetViewStateData: (data: any) => void;
    ViewStateData: {
        SetDictionaryItem: (key: string, value: any) => void;
    };
    InstanceId: string;
    Close: () => any;
    ShowErrorMessageBox: (msg: string) => void;
    ShowQuestionMessageBox: (msg: string) => any;
    ShowInfoMessageBox: (msg: string) => void;
    InitFd: (msg: any) => void;
    CommSend: (msg: any) => void;
    DoTransit: (msg: any) => void;
    callAgn: (msg: any) => void;
    getError: (msg: any) => string;
    previewSubmitting: () => Promise<any>;
    submitEntry: {
        loadData: (page: IPage, message: any) => boolean;
        raiseSubmit: () => boolean;
    };
    authorizeEntries: {
        find: (callback: (entry: any) => boolean) => any;
    };
    recheckEntries: {
        find: (callback: (entry: any) => boolean) => any;
    };
    checkFee: (args: any, isPaymentFee: boolean) => boolean;
    raisePaymentFee: (args: any) => boolean;
    previewEvaluation: (record: any) => boolean;
    getPrintAttributes: () => any;
    printCertificate: (pl: any[]) => void;
}

interface FrameworkElement {
    ActualWidth: number;
    ActualHeight: number;
}

interface XmlDocument {
    InnerXml: string;
    DocumentElement: any;
    SelectSingleNode: (node: string) => any;
    CreateElement: (name: string) => any;
}

interface MemoryStream {
    Position: number;
    Length: number;
    Read: (buffer: any[], offset: number, length: number) => number;
    Flush: () => void;
    Close: () => void;
}
//hulei C:\Program Files (x86)\Reference Assemblies\Microsoft\Framework\.NETFramework\v4.0\PresentationCore.dll 微软引用库中的方法
interface RenderTargetBitmap {
    Render: (element: FrameworkElement) => void;
}

interface BitmapSource {
    // 简单模拟
}

interface Image {
    Source: BitmapSource;
}

interface ComboBox {
    Items: any[];
    SelectedIndex: number;
    Name: string;
}

interface DataGrid {
    ItemsSource: any;
    Columns: any[];
    Name: string;
}

interface DataBox {
    Parent: any;
}

interface PrintContainer {
    Device: any;
    DocumentName: string;
    DocumentId: string;
    PrintToXps: () => any;
    BackgroundImage: any;
    Width: number;
    Height: number;
    Description: string;
}

interface TT_PrintData {
    BusinessId: string;
    Description: string;
    FormId: string;
    Id: string;
    PrintData: any[];
    ByteLength: string;
    TradeNo: string;
}

interface PrintViewer {
    PrintSource: any[];
    Printing: (s: any, e: any) => void;
    OpenPrintOnWindow: (target: IPage, flag: boolean) => void;
}

interface Process {
    StartInfo: {
        FileName: string;
    };
    Start: () => boolean;
    HasExited: boolean;
    Kill: () => void;
    Refresh: () => void;
    WorkingSet64: number;
    StartTime: Date;
    StandardOutput: {
        ReadLine: () => string | null;
    };
}
const enum OpenOptions {
    Tab = "Tab",
    Dialog = "Dialog",
    NextPage = "NextPage",
    NewPage = "NewPage",
    CloseAndNewPage = "CloseAndNewPage",
}
// 定义命名空间
namespace TellerSystem.Library.Ext.Function {
    export class TradePageExtension {
        // 交易界面还原相关
        public static SaveTradePage(target: IPage, rootElement: FrameworkElement): number[] {
            //hulei 交易还原逻辑使用的是微软提供的引用包中的逻辑，在新的系统中需要重新实现
            const rtb = "";// new RenderTargetBitmap((rootElement.ActualWidth), (rootElement.ActualHeight), 96, 96, null);
            // rtb.Render(rootElement);
            return this.BitmapSourceToByte(rtb);
        }
        //hulei 交易还原逻辑使用的是微软提供的引用包中的逻辑，在新的系统中需要重新实现
        // public static RestoreTradePage(target: IPage, tradepagedata: number[]): FrameworkElement {
        //     // const image = new Image();
        //     // image.Source = this.ByteToBitmapSource(tradepagedata);
        //     // return image as FrameworkElement;
        // }

        private static DealSpecialControl(rootElement: FrameworkElement, pagedata: XmlDocument, specialitems: { [key: string]: any }, flag: number) {
            const elements = []; // 这里需要实现 LogicalTreeHelper.GetChildren 的逻辑
            //hulei此处需要处理按节点处理。
            // for (const element of elements) {
            //     const child = element as FrameworkElement;
            //     if (!child) continue;

            //     if (child instanceof ComboBox) {
            //         if (flag === 0) {
            //             if (!specialitems[child.Name]) {
            //                 const items: { [key: string]: any } = {};
            //                 let j = 0;
            //                 if (child.Items.length > 0) {
            //                     for (const item of child.Items) {
            //                         j++;
            //                         if (item)
            //                             items[j.toString()] = item;
            //                     }
            //                 }
            //                 this.InsertInfoToXml(pagedata, "ComBoxInfo", child.Name, this.Serialize(items), child.SelectedIndex.toString());
            //                 items["selectedindex"] = child.SelectedIndex;
            //                 specialitems[child.Name] = items;
            //                 child.Items = [];
            //             }
            //         } else if (flag === 1) {
            //             if (Object.keys(specialitems).length !== 0) {
            //                 for (const tmp of Object.entries(specialitems)) {
            //                     if (tmp[0] === child.Name) {
            //                         for (const tmp1 of Object.entries(tmp[1])) {
            //                             if (tmp1[0] === "selectedindex")
            //                                 child.SelectedIndex = tmp1[1];
            //                             else {
            //                                 child.Items.push(tmp1[1]);
            //                             }
            //                         }
            //                         break;
            //                     }
            //                 }
            //             } else {
            //                 const items = this.Deserialize<{ [key: string]: any }>(this.GetDataFromXml(pagedata, "ComBoxInfo", child.Name, "value"));
            //                 for (const tmp of Object.entries(items)) {
            //                     child.Items.push(tmp[1]);
            //                 }
            //                 let index = 0;
            //                 const indexStr = this.GetDataFromXml(pagedata, "ComBoxInfo", child.Name, "selectedindex");
            //                 if (indexStr)
            //                     index = parseInt(indexStr);
            //                 child.SelectedIndex = index;
            //             }
            //         }
            //     } else if (child instanceof DataGrid) {
            //         if (flag === 0) {
            //             if (!specialitems[child.Name]) {
            //                 specialitems[child.Name] = child.ItemsSource;
            //                 const items: { [key: string]: any } = {};
            //                 let j = 0;
            //                 const colname: any[] = new Array(child.Columns.length);
            //                 for (const column of child.Columns) {
            //                     colname[j] = column.Header;
            //                     j++;
            //                 }
            //                 items["colname"] = colname;
            //                 j = 0;
            //                 for (const item of child.Items) {
            //                     j++;
            //                     const row = item as any;
            //                     if (row)
            //                         items[j.toString()] = row.Row.ItemArray;
            //                 }
            //                 this.InsertInfoToXml(pagedata, "DataGrid", child.Name, this.Serialize(items), "");
            //                 child.ItemsSource = null;
            //             }
            //         } else if (flag === 1) {
            //             if (Object.keys(specialitems).length !== 0) {
            //                 for (const tmp of Object.entries(specialitems)) {
            //                     if (tmp[0] === child.Name) {
            //                         child.ItemsSource = tmp[1];
            //                         break;
            //                     }
            //                 }
            //             } else {
            //                 const items = this.Deserialize<{ [key: string]: any }>(this.GetDataFromXml(pagedata, "DataGrid", child.Name, "value"));
            //                 const table = []; // 这里需要实现 DataTable 的逻辑
            //                 for (const tmp of Object.entries(items)) {
            //                     if (tmp[0] === "colname") {
            //                         for (const colname of tmp[1]) {
            //                             // table.Columns.Add(colname.toString());
            //                         }
            //                     } else {
            //                         // table.Rows.Add(tmp[1]);
            //                     }
            //                 }
            //                 child.ItemsSource = table;
            //             }
            //         }
            //     } else if (child instanceof DataBox) {
            //         const parent = child.Parent as FrameworkElement;
            //         if (parent) {
            //             // 这里需要实现移除子元素的逻辑
            //             this.DealSpecialControl(parent, pagedata, specialitems, flag);
            //             break;
            //         }
            //     }

            //     if (child && !false) // 这里需要实现 FocusManager.GetIsInternalKeyboardControlling 的逻辑
            //         this.DealSpecialControl(child, pagedata, specialitems, flag);
            // }
        }

        private static InsertInfoToXml(pagedataxml: XmlDocument, node: string, name: string, value: string, selectedindex: string) {
            try {
                if (pagedataxml) {
                    const root = pagedataxml.DocumentElement;
                    const nodetmp = root.SelectSingleNode(node);
                    const xe = pagedataxml.CreateElement("info");
                    xe.setAttribute("name", name);
                    xe.setAttribute("selectedindex", selectedindex);
                    xe.setAttribute("value", value);
                    nodetmp.appendChild(xe);
                }
            } catch (e) {
                throw e;
            }
        }

        private static GetDataFromXml(pagedataxml: XmlDocument, node: string, name: string, property: string): string {
            let result = null;
            try {
                const root = pagedataxml.DocumentElement;
                const nodetmp = root.SelectSingleNode(node);
                for (const tmp of nodetmp.childNodes) {
                    if (tmp.attributes["name"].value === name) {
                        result = tmp.attributes[property].value;
                        break;
                    }
                }
            } catch (e) {
                throw e;
            }
            return result;
        }

        // 序列化obj和反序列化string
        private static Serialize<T>(obj: T): string {
            if (!obj)
                return null;
            const formatter = null; // 这里需要实现 BinaryFormatter 的逻辑
            //hulei C:\Program Files (x86)\Reference Assemblies\Microsoft\Framework\.NETFramework\v4.0\mscorlib.dll
            const stream = "";//new MemoryStream();
            // formatter.Serialize(stream, obj);
            // stream.Position = 0;
            const buffer = null;// new Array(stream.Length);
            // stream.Read(buffer, 0, buffer.Length);
            // stream.Flush();
            // stream.Close();
            return btoa(String.fromCharCode.apply(null, buffer));
        }

        public static SerializePage<T>(target: IPage, obj: T): string {
            return this.Serialize<T>(obj);
        }

        public static DeserializePage<T>(target: IPage, str: string): T {
            return this.Deserialize<T>(str);
        }

        private static Deserialize<T>(str: string): T {
            const obj = null; // 这里需要实现默认值逻辑
            const formatter = null; // 这里需要实现 BinaryFormatter 的逻辑
            const buffer = atob(str).split('').map((char) => char.charCodeAt(0));
            //hulei C:\Program Files (x86)\Reference Assemblies\Microsoft\Framework\.NETFramework\v4.0\mscorlib.dll const stream = new MemoryStream(buffer);
            // obj = formatter.Deserialize(stream);
            // stream.Flush();
            // stream.Close();
            return obj;
        }

        // 打印API
        public static SetProcessWorkingSetSize(proc: any, min: number, max: number): boolean {
            // 这里需要实现具体逻辑
            return false;
        }

        public static ClearSystemMemory(maxSize: number) {
            try {
                //hulei // C:\Program Files (x86)\Reference Assemblies\Microsoft\Framework\.NETFramework\v4.0\System.dll
                // const p = new Process();
                // const oldSize = p.WorkingSet64 / 1024 / 1024;
                // if (oldSize > maxSize) {
                //     this.SetProcessWorkingSetSize(p, -1, -1);
                //     p.Refresh();
                //     const newSize = p.WorkingSet64 / 1024 / 1024;
                //     // TradeHandle.WriteImportantLog("定时回收内存！", string.Format("内存情况[{0}M->{1}M][{2}]", oldSize, newSize, maxSize), "", "", "0", string.Join(",", SocketManager.GetLocalIpAddressList()));
                // }
            } catch (ex) {
                // TradeHandle.WriteImportantLog("定时回收内存！发生异常...", ex.Message + "\n" + ex.StackTrace, "", "", "0", string.Join(",", SocketManager.GetLocalIpAddressList()));
            }
        }

        private static FLAG = null; // 这里需要实现 TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID 的逻辑

        public static PrintCertificate(target: IPage, authForms: PrintContainer[]) {
            // try {
            //     const processe = []; // 这里需要实现 Process.GetProcessesByName 的逻辑
            //     if (processe.length <= 0) {
            //         // C:\Program Files (x86)\Reference Assemblies\Microsoft\Framework\.NETFramework\v4.0\System.dll
            //         // const printProcess = new Process();
            //         // printProcess.StartInfo.FileName = null; // 这里需要实现 Path.Combine 的逻辑
            //         // printProcess.Start();
            //     }
            // } catch (ex) {
            //     // PlatformLogger.SystemErrorInfo("打印前检查进程发生异常！", ex);
            //     // TradeHandle.WriteImportantLog("打印前检查进程发生异常！", ex.Message + "\n" + ex.StackTrace, "", "", "0", string.Join(",", SocketManager.GetLocalIpAddressList()));
            // }
            // if (!authForms || authForms.length === 0)
            //     return;
            // target.CheckCashQuota();
            // const pList: { [key: PrintContainer]: [any, TT_PrintData] } = {};
            // const printAttr = target.GetPrintAttributes();
            // try {
            //     authForms = authForms.filter((x) => x && x.Device === null).map((item) => {
            //         if (!this.GetPrintElement(item))
            //             this.SetPrintElement(item, printAttr);
            //         return item;
            //     });
            //     // 这里需要实现 dynamic 的逻辑
            //     // const paperless = target.Paperless;
            //     // paperless.Check(authForms, target);
            // } catch (e) { }

            // const action = (item: PrintContainer) => {
            //     try {
            //         if (!item || item.Device !== null) return;
            //         if (!this.GetPrintElement(item))
            //             this.SetPrintElement(item, printAttr);

            //         const xpsFile = item.PrintToXps();
            //         if (item instanceof any) // 这里需要实现 PrintTemplate.Base.TemplateBase 的逻辑
            //             xpsFile = (item as any).ProcessXps(xpsFile);
            //         // PlatformLogger.TradeInfo("文件名:" + xpsFile.Name + "文件路径" + xpsFile.Directory);
            //         // PlatformLogger.TradeInfo("文件路径:" + System.IO.Path.Combine("print", xpsFile.Name));
            //         const tt = new TT_PrintData();
            //         tt.BusinessId = target.GetTradeInfoData().SystemSerialNumber;
            //         tt.Description = item.DocumentName;
            //         tt.FormId = item.DocumentId;
            //         tt.Id = null; // 这里需要实现 System.Guid.NewGuid().ToString("n") 的逻辑
            //         tt.PrintData = null; // 这里需要实现 File.ReadAllBytes 的逻辑
            //         tt.ByteLength = tt.PrintData.length.toString();
            //         tt.TradeNo = target.GetTradeInfoData().TradeCode;

            //         pList[item] = [xpsFile, tt];
            //     } catch (ex) {
            //         // TradeHandle.WriteImportantLog("产生XPS时发生异常...", ex.Message + "\n" + ex.StackTrace, "", "", "0", string.Join(",", SocketManager.GetLocalIpAddressList()));
            //     }
            // };

            // const uploadTask = new Promise((resolve, reject) => {
            //     try {
            //         // 这里需要实现补偿逻辑
            //         // 全部完成，执行上传
            //         // PrintManagerHandle.SavePrintData(Object.values(pList).map((x) => x[1]));
            //         resolve();
            //     } catch (ex) {
            //         // TradeHandle.WriteImportantLog("上传打印信息时发生异常...", ex.Message + "\n" + ex.StackTrace, "", "", "0", string.Join(",", SocketManager.GetLocalIpAddressList()));
            //         reject(ex);
            //     }
            // });

            // for (const item of authForms) {
            //     for (let i = 0; i < 5; i++) {
            //         action(item);
            //         if (!item || pList[item])
            //             break;
            //         // 这里需要实现 Thread.Sleep 的逻辑
            //     }
            // }
            // uploadTask.then(() => {
            //     // 提取有效的PrintAttributes
            //     printAttr = this.GetPrintElement(authForms[0]);
            //     try {
            //         // 这里需要实现 dynamic 的逻辑
            //         // target.Paperless.Proc(pList);
            //     } catch (e) { }
            //     // 尝试采用新的方案执行打印过程
            //     let flag = false;
            //     target.OwnerContainer.ModalInvoke(() => {
            //         flag = this.ProcessPrint(pList, (id) => {
            //             uploadTask.then(() => {
            //                 try {
            //                     const detail = new TT_PrintDataDetail();
            //                     detail.AuthTellerNo = "";
            //                     detail.Description = "";
            //                     detail.FormId = id;
            //                     detail.Id = null; // 这里需要实现 System.Guid.NewGuid().ToString("n") 的逻辑
            //                     detail.TellerNo = null; // 这里需要实现 LoginUserInfo.TellerNo 的逻辑
            //                     // PrintManagerHandle.AddPrintDataHistory(detail);
            //                 } catch (ex) {
            //                     // TradeHandle.WriteImportantLog("记录打印数据时发生异常...", ex.Message + "\n" + ex.StackTrace, "", "", "0", string.Join(",", SocketManager.GetLocalIpAddressList()));
            //                 }
            //             });
            //         });
            //     }, "打印机处理中，请稍候...");
            //     if (flag) {
            //         if (Object.keys(pList).length > 0)
            //             target.ShowWarnMessageBox("打印清单未全部完成！如非主动取消打印，请检查业务流程！");
            //         try {
            //             // 这里需要实现 dynamic 的逻辑
            //             // target.OnPagePrinted(printAttr);
            //         } catch (ex) {
            //             // PlatformLogger.SystemErrorInfo("打印完成后，调用打印完成事件发生异常！", ex);
            //         }
            //         return;
            //     }
            //     for (const item of Object.entries(pList)) {
            //         const isPrinted = false;
            //         const viewer = new PrintViewer();
            //         viewer.PrintSource = [
            //             {
            //                 XpsFile: item[1][0],
            //                 PrintObject: item[0],
            //             },
            //         ];
            //         viewer.Printing = (s, e) => {
            //             isPrinted = true;
            //         };
            //         viewer.OpenPrintOnWindow(target, true);
            //         if (isPrinted) {
            //             Promise.resolve().then(() => {
            //                 try {
            //                     uploadTask.then(() => {
            //                         const data = item[1];
            //                         const detail = new TT_PrintDataDetail();
            //                         detail.AuthTellerNo = "";
            //                         detail.Description = "";
            //                         detail.FormId = data[1].Id;
            //                         detail.Id = null; // 这里需要实现 System.Guid.NewGuid().ToString("n") 的逻辑
            //                         detail.TellerNo = null; // 这里需要实现 LoginUserInfo.TellerNo 的逻辑
            //                         // PrintManagerHandle.AddPrintDataHistory(detail);
            //                     });
            //                 } catch (ex) {
            //                     // TradeHandle.WriteImportantLog("记录打印数据时发生异常...", ex.Message + "\n" + ex.StackTrace, "", "", "0", string.Join(",", SocketManager.GetLocalIpAddressList()));
            //                 }
            //             });
            //         }
            //     }
            //     try {
            //         // PlatformLogger.SystemInfo("打印完毕，准备触发事件OnPagePrinted！");
            //         // 这里需要实现 dynamic 的逻辑
            //         // target.OnPagePrinted(printAttr);
            //     } catch (ex) {
            //         // PlatformLogger.SystemErrorInfo("打印完成后，调用打印完成事件发生异常！", ex);
            //     }
            //     // 打印完毕，进行内存清理
            //     this.ClearSystemMemory(500);
            // });
            //打印票据逻辑报错需要重新重构处理 hulei
        }

        private static ProcessPrint(pList: { [key: string]: [any, TT_PrintData] }, printed?: (id: string) => void): boolean {
            // 检查是否采用此方案
            if (!false) return false; // 这里需要实现 ServiceSettings.GetConfig 的逻辑
            const path = "TellerSystem.Previewer.exe";//hulei 调用exe方法需要重写  打印的逻辑可以忽略
            const arg0 = "";
            Object.entries(pList).forEach((p) => {
                // 这里需要实现 ui 线程访问控件数据的逻辑
                const background = "";
                //报错了hulei arg0 += `@${p[1][1].Id}|${p[0].DocumentName}|${p[0].Width}|${p[0].Height}|${p[1][0].FullName}|${background}|${false}|${p[0].IsNotingPreview}|${p[0].Description}|\n`;
            });
            const msg = "";
            // PlatformLogger.SystemInfo("准备开始打印：" + Object.keys(pList).length);
            const cnt = Object.keys(pList).length;
            // 这里需要实现 ProcessExe 的逻辑
            return false;
        }

        public static GetPrintAttributes(target: IPage): PrintAttributes {
            const tradeInfo = target.GetTradeInfoData();
            const attr = new PrintAttributes();
            attr.BusinessTransaction = "";
            attr.Review = null; // 这里需要实现 AfterSuperviseHandle.GetUserNameByCode 的逻辑
            attr.SystemSerialNumber = target.CreateSerialNumber();
            attr.PrintNumber = tradeInfo.PrintNum;
            attr.Authentic = null; // 这里需要实现 AfterSuperviseHandle.GetUserNameByCode 的逻辑
            attr.TradeModel = null; // 这里需要实现 PageHelper.GetTradeModel 的逻辑
            attr.Authno = tradeInfo.TradeBus["AuthTeller"];
            return attr;
        }

        public static Sync(PrintUIData: { [key: string]: string }, msg: string, fromEntry: boolean): boolean {
            if (!PrintUIData) return false;
            if (fromEntry) {
                let sw = "";
                for (const item of Object.entries(PrintUIData)) {
                    sw += `${item[0]}:${item[1]}\n`;
                }
                msg = sw;
            } else {
                Object.keys(PrintUIData).forEach((key) => delete PrintUIData[key]);
                const sr = msg.split('\n');
                for (const line of sr) {
                    if (line) {
                        const index = line.indexOf(':');
                        if (index > 0) {
                            PrintUIData[line.substring(0, index)] = line.substring(index + 1);
                        }
                    }
                }
            }
            return true;
        }

        // BitmapSource与Byte[]互转
        public static BitmapSourceToByte(bitmapSource: BitmapSource): number[] {
            const encoder1 = null; // 这里需要实现 PngBitmapEncoder 的逻辑
            encoder1.Frames.push(null); // 这里需要实现 BitmapFrame.Create 的逻辑
            const data = null;
            // const ms = new MemoryStream();//hulei VUE中图片与字节互转。
            // encoder1.Save(ms);
            // const data = new Array(ms.Length);
            // ms.Read(data, 0, data.Length);
            // ms.Flush();
            // ms.Close();
            return data;
        }

        public static ByteToBitmapSource(byteData: number[]): BitmapSource {
            const bi = null; // 这里需要实现 BitmapImage 的逻辑 //hulei VUE ByteToBitmapSource中图片与字节互转。
            // bi.BeginInit();
            // bi.StreamSource = new MemoryStream(byteData);
            // bi.EndInit();
            return bi;
        }

        // 工作流
        // public static GetWorkFlow(page: IPage): WorkFlow {
        //     const wf = new WorkFlow(page);
        //     return wf;
        // }

        // 打开交易页面相关


        public static PreOpenTradeCode = "";

        public static OpenTradePage(target: IPage, action: string, option: OpenOptions = OpenOptions.NewPage, inputData: { [key: string]: any } = null, tradeBus: any = null, titleName: string = ""): string {
            if (!target || !action) return null;
            const page = null; // 这里需要实现 PageFactory.CreatePage 的逻辑
            if (!page) {
                target.ShowWarnMessageBox("交易打开失败：未找到相关交易文件！");
                // PlatformLogger.TradeErrorInfo(string.Format("OpenTradePage：PageFactory.CreatePage失败，IPage=[{0}]", action), null);
                return null;
            }
            page.SetViewStateData({});
            if (titleName) {
                (page as any).Tag = titleName;
            }
            if (inputData) {
                for (const x of Object.entries(inputData)) {
                    page.ViewStateData.SetDictionaryItem(x[0], x[1]);
                }
            }
            return this.OpenTradePageWithPage(target, page, option, tradeBus);
        }

        public static OpenTradePageWithReload(target: IPage, action: string, reLoaded: () => void, option: OpenOptions = OpenOptions.NewPage, inputData: { [key: string]: any } = null, tradeBus: any = null): string {
            if (!target || !action) return null;
            const page = null; // 这里需要实现 PageFactory.CreatePage 的逻辑
            if (!page) {
                target.ShowWarnMessageBox("交易打开失败：未找到相关交易文件！");
                // PlatformLogger.TradeErrorInfo(string.Format("OpenTradePage：PageFactory.CreatePage失败，IPage=[{0}]", action), null);
                return null;
            }
            page.SetViewStateData({});
            if (inputData) {
                for (const x of Object.entries(inputData)) {
                    page.ViewStateData.SetDictionaryItem(x[0], x[1]);
                }
            }
            return this.OpenTradePageWithPage(target, page, option, tradeBus, reLoaded);
        }

        public static OpenTradePageWithPage(target: IPage, content: IPage, option: OpenOptions = OpenOptions.NewPage, tradeBus: any = null, reLoaded?: () => void): string {
            if (!target || !content) return null;
            let model = null; // 这里需要实现 PageHelper.GetTradeModel 的逻辑
            const action = content.constructor.name;
            switch (option) {
                case OpenOptions.Tab:
                    if (!model)
                        model = this.GetTradeModelByAction(target, action);
                    if (!model) {
                        target.ShowWarnMessageBox("交易打开失败：未配置权限！");
                        // PlatformLogger.TradeErrorInfo(string.Format("OpenTradePage：Tab方式打开失败，未配置权限，IPage=[{0}]", action), null);
                        return null;
                    }
                    if (!target.CheckLoginStatus())
                        return null;
                    // PageHelper.SetTradeModel(content, model);
                    const tradeInfo = {
                        TradeCode: model.Code,
                        TradeName: model.Name,
                        TradeBus: tradeBus || {},
                    };
                    // TradeInfoData.SetTradeInfo(content, tradeInfo);
                    const mainPage = null; // 这里需要实现 MainPageManager.GetMainPage 的逻辑
                    if (!mainPage || !mainPage.OpenTabPage(content)) {
                        target.ShowWarnMessageBox("交易打开失败：主窗体创建交易容器失败！");
                        // PlatformLogger.TradeErrorInfo(string.Format("OpenTradePage：Tab方式打开失败，主窗体创建交易容器失败，IPage=[{0}]", action), null);
                        return null;
                    }
                    this.PreOpenTradeCode = model.Code;
                    break;
                case OpenOptions.Dialog:
                    // 这里需要实现获取 TradeModel 和 TradeInfo 的逻辑
                    if (!model || !tradeInfo) {
                        target.ShowWarnMessageBox("交易打开失败：非法的主交易页面！");
                        // PlatformLogger.TradeErrorInfo(string.Format("OpenTradePage：Dialog方式打开失败，无法获取主界面的TradeInfo，IPage=[{0}]", action), null);
                        return null;
                    }
                    // PageHelper.SetTradeModel(content, model);
                    // TradeInfoData.SetTradeInfo(content, tradeInfo);
                    const dialog = null; // 这里需要实现 PageDialog 的逻辑
                    let header = tradeInfo.TradeName;
                    if ((content as any).Tag)
                        header = (content as any).Tag;
                    // 这里需要实现 UI 缩放逻辑
                    dialog.Open(target, content, header);
                    if (content.ViewStateData) {
                        for (const x of Object.entries(content.ViewStateData)) {
                            target.ViewStateData.SetDictionaryItem(x[0], x[1]);
                        }
                    }
                    break;
                case OpenOptions.NextPage:
                    // 这里需要实现获取 TradeModel 和 TradeInfo 的逻辑
                    if (!model || !tradeInfo) {
                        target.ShowWarnMessageBox("交易打开失败：非法的主交易页面！");
                        // PlatformLogger.TradeErrorInfo(string.Format("OpenTradePage：NextPage方式打开失败，无法获取主界面的TradeInfo，IPage=[{0}]", action), null);
                        return null;
                    }
                    // PageHelper.SetTradeModel(content, model);
                    // TradeInfoData.SetTradeInfo(content, tradeInfo);
                    target.OwnerContainer.PushPage(content, (x) => {
                        if (x.ViewStateData) {
                            for (const y of Object.entries(x.ViewStateData)) {
                                target.ViewStateData.SetDictionaryItem(y[0], y[1]);
                            }
                        }
                        if (reLoaded)
                            reLoaded();
                    });
                    break;
                case OpenOptions.NewPage:
                    if (!model)
                        model = this.GetTradeModelByAction(target, action);
                    if (!model) {
                        target.ShowWarnMessageBox("交易打开失败：未配置权限！");
                        // PlatformLogger.TradeErrorInfo(string.Format("OpenTradePage：NewPage方式打开失败，未配置权限，IPage=[{0}]", action), null);
                        return null;
                    }
                    // PageHelper.SetTradeModel(content, model);
                    const newTradeInfo = {
                        TradeCode: model.Code,
                        TradeName: model.Name,
                        TradeBus: tradeBus,
                    };
                    // TradeInfoData.SetTradeInfo(content, newTradeInfo);
                    target.OwnerContainer.PushPage(content, (x) => {
                        if (x.ViewStateData) {
                            for (const y of Object.entries(x.ViewStateData)) {
                                target.ViewStateData.SetDictionaryItem(y[0], y[1]);
                            }
                        }
                        if (reLoaded)
                            reLoaded();
                    });
                    break;
                case OpenOptions.CloseAndNewPage:
                    if (!model)
                        model = this.GetTradeModelByAction(target, action);
                    if (!model) {
                        target.ShowWarnMessageBox("交易打开失败：未配置权限！");
                        // PlatformLogger.TradeErrorInfo(string.Format("OpenTradePage：NewPage方式打开失败，未配置权限，IPage=[{0}]", action), null);
                        return null;
                    }
                    // PageHelper.SetTradeModel(content, model);
                    const closeAndNewTradeInfo = {
                        TradeCode: model.Code,
                        TradeName: model.Name,
                        TradeBus: tradeBus,
                    };
                    // TradeInfoData.SetTradeInfo(content, closeAndNewTradeInfo);
                    target.OwnerContainer.PushPage(content, (x) => {
                        if (x.ViewStateData) {
                            for (const y of Object.entries(x.ViewStateData)) {
                                target.ViewStateData.SetDictionaryItem(y[0], y[1]);
                            }
                        }
                        x.Close();
                        if (reLoaded)
                            reLoaded();
                    });
                    break;
                default:
                    return null;
            }
            return content.InstanceId;
        }

        public static CheckLoginStatus(target: IPage): boolean {
            let error = "检查登录状态初始化";
            const config = null; // 这里需要实现 TradeManagerHandle.GetSysconfigList 的逻辑
            if (config.length > 0) {
                const dtime = null; // 这里需要实现 config[0].Configvalue.ToDateTime 的逻辑
                error = "参数【登录时间限制】设置格式异常，请联系科技部！";
                let flag = false;
                if (dtime) {
                    const sTime = null; // 这里需要实现 TradeHandle.GetServerTime().ToDateTime 的逻辑
                    error = "获取服务端时间失败，请重试！";
                    if (sTime) {
                        const deviation = new Date().getTime() - sTime.getTime();
                        const start = "";//hulei 重新处理定时器 new Date(Process.GetCurrentProcess().StartTime.getTime() - deviation);
                        error = `柜员登录状态已过期，请重新登录！\n登录时间：${start}\n限制时间：${dtime}`;
                        flag = start > dtime;
                    }
                }
                if (!flag) {
                    error = `柜员状态检查未通过！原因：\n${error}`;
                    // PlatformLogger.TradeErrorInfo(error, null);
                    target.ShowErrorMessageBox(error);
                }
                return flag;
            }
            return true;
        }

        private static GetTradeModelByAction(target: IPage, action: string, isCheckAll: boolean = false): any {
            const data = null; // 这里需要实现 TradeMenuManager.Instance.GetTradeModel 的逻辑
            if (data && data.length > 0)
                return data[0];
            return null;
        }

        // 兼容方法
        public static CloseAndOpenTradePage(target: IPage, actionName: string, importArgs: { [key: string]: any } = null): string {
            const tradePageContainer = target.OwnerContainer;
            if (!tradePageContainer) {
                return null;
            }
            return ""; //target.OpenTradePage(actionName, OpenOptions.CloseAndNewPage, importArgs);//hulei 中间应该用VUE中的路由跳转打开新的页面
        }

        public static OpenPage(target: IPage, actionName: string, importArgs: { [key: string]: any } = null): string {
            const tradePageContainer = target.OwnerContainer;
            if (!tradePageContainer) {
                return null;
            }
            return "";// target.OpenTradePage(actionName, OpenOptions.Tab, importArgs);//hulei 中间应该用VUE中的路由跳转打开新的页面
        }

        public static Close(target: IPage): { [key: string]: any } {
            const tradePageContainer = target.OwnerContainer;
            if (!tradePageContainer)
                return null;
            if (target.InstanceId === null)
                return {};
            const result = tradePageContainer.ClosePage();
            // 关闭可能造成整个系统焦点丢失，故重新置换焦点
            // Application.Current.MainWindow.Activate();
            return result;
        }

        public static ShowPage(target: IPage, content: IPage, titleName: string = ""): { [key: string]: any } {
            let result = null;
            if (content) {
                (content as any).Tag = titleName;
                // if (target.OpenTradePage(content, OpenOptions.Dialog) !== null) {
                //     result = content.ViewStateData;
                // }
            }
            return result;
        }

        public static ShowPageWithName(target: IPage, pageFullName: string, titleName: string, initArgs: { [key: string]: any } = null): { [key: string]: any } {
            //const guid = target.OpenTradePage(pageFullName, OpenOptions.Dialog, initArgs, null, titleName);//hulei 中间应该用VUE中的路由跳转打开新的页面
            return target.ViewStateData;
        }

        public static ShowPageWithAction(target: IPage, action: string): { [key: string]: any } {
            return this.ShowPageWithName(target, action, null, null);
        }

        public static OpenPageWithContainer(target: IPage, action: string, container: any, importArgs: { [key: string]: any } = null): string {
            if (!container) {
                return null;
            }
            const flag = false; // 这里需要实现 ConfigManager.GetInstance().GetConfigValue 的逻辑
            if (!false && flag) { // 这里需要实现 PlatformSettings.IsDebugMode 的逻辑
                try {
                    const uc = null; // 这里需要实现 UpgradeClass 的逻辑
                    uc.UpdateTradeDll(action);
                } catch (ex) {
                    // PlatformLogger.SystemErrorInfo("更新服务器连接失败", ex);
                }
            }
            const page = this.CreatePage(target, action, importArgs);
            if (!page) return null;
            return container.PushPage(page, (x) => {
                if (x.ViewStateData) {
                    for (const y of Object.entries(x.ViewStateData)) {
                        target.ViewStateData.SetDictionaryItem(y[0], y[1]);
                    }
                }
            });
        }

        public static CreatePage(target: IPage, action: string, importArgs: { [key: string]: any } = null): IPage {
            if (!target || !action) return null;
            const page = null; // 这里需要实现 PageFactory.CreatePage 的逻辑
            if (!page) {
                target.ShowWarnMessageBox("交易打开失败：未找到相关交易文件！");
                // PlatformLogger.TradeErrorInfo(string.Format("OpenTradePage：PageFactory.CreatePage失败，IPage=[{0}]", action), null);
                return null;
            }
            const model = null; // 这里需要实现 PageHelper.GetTradeModel 的逻辑
            const tradeInfo = null; // 这里需要实现 TradeInfoData.GetTradeInfo 的逻辑
            if (!model || !tradeInfo) {
                target.ShowWarnMessageBox("交易打开失败：非法的主交易页面！");
                // PlatformLogger.TradeErrorInfo(string.Format("OpenPage：target无法获取主界面的TradeInfo，IPage=[{0}]", action), null);
                return null;
            }
            tradeInfo.SystemSerialNumber = "";
            // PageHelper.SetTradeModel(page, model);
            // TradeInfoData.SetTradeInfo(page, tradeInfo);
            page.SetViewStateData({});
            if (importArgs) {
                for (const x of Object.entries(importArgs)) {
                    page.ViewStateData.SetDictionaryItem(x[0], x[1]);
                }
            }
            return page;
        }

        // 流水号相关
        public static CreateSerialNumber(target: IPage): string {
            const data = null; // 这里需要实现 TradeInfoData.GetTradeInfo 的逻辑
            if (!data) return "";
            if (!data.SystemSerialNumber) {
                const localip = null; // 这里需要实现 SystemFunction.GetIPv4Addresses 的逻辑
                data.SystemSerialNumber = null; // 这里需要实现 WorkFlowHandle.GenNextSerialNo 的逻辑
            }
            return data.SystemSerialNumber;
        }

        public static ClearSerialNumber(target: IPage) {
            const data = null; // 这里需要实现 TradeInfoData.GetTradeInfo 的逻辑
            if (data && data.SystemSerialNumber) {
                data.SerialNumber = "";
                data.SystemSerialNumber = "";
            }
        }

        public static CreateTellerSerialNumber(target: IPage, serialid: string): string {
            const data = null; // 这里需要实现 TradeInfoData.GetTradeInfo 的逻辑
            if (!data) return "";
            if (!data.SerialNumber) {
                data.SerialNumber = null; // 这里需要实现 WorkFlowHandle.GenTellerSerialNo 的逻辑
            }
            return data.SerialNumber;
        }

        public static CreatePrintNum(target: IPage, serialId: string): string {
            const data = null; // 这里需要实现 TradeInfoData.GetTradeInfo 的逻辑
            if (!data) return "";
            data.PrintNum = null; // 这里需要实现 PrintManagerHandle.GetPrintNum 的逻辑
            return data.PrintNum;
        }

        public static GetHideTradeTabCount(): number {
            return Object.keys(this._dict).length;
        }

        public static HideTradeTab(target: IPage, page: IPage): boolean {
            let tabPage = page;
            const number = null; // 这里需要实现 TradeInfoData.GetTradeInfo 的逻辑
            const dialog = null; // 这里需要实现 DependencyObjectExtensions.FindVisualTreeAncestor 的逻辑
            if (dialog) {
                if (!dialog.OwnerPage) {
                    throw new Error("HideTradeTab:无法处理OwnerPage为null的PageDialogBase！");
                }
                tabPage = dialog.OwnerPage;//hulei 中间应该用VUE中的路由跳转打开新的页面
            }
            const guid = null; // 这里需要实现 MainPageManager.GetMainPage().HideTab 的逻辑
            if (guid === null)
                return false;
            if (dialog)
                dialog.Hide();
            if (this._dict[guid])
                delete this._dict[guid];
            this._dict[guid] = [number, dialog, page];
            return true;
        }

        public static ShowTradeTab(target: IPage, serinalNumber: string): IPage {
            const data = Object.entries(this._dict).find((x) => x[1][0] === serinalNumber);
            if (!data) return null;
            const page = null; //hulei 这里需要实现 MainPageManager.GetMainPage().ShowTab 的逻辑
            if (data[1][1]) {
                data[1][1].ShowDialog();
            }
            return data[1][2];
        }

        public static ShowTradeTabWithAction(target: IPage, serinalNumber: string, aciton: (page: IPage) => void) {
            const data = Object.entries(this._dict).find((x) => x[1][0] === serinalNumber);
            if (!data) {
                aciton(null);
                return;
            }
            const page = null; // hulei 这里需要实现 MainPageManager.GetMainPage().ShowTab 的逻辑
            aciton(data[1][2]);
            if (data[1][1]) {
                data[1][1].ShowDialog();
            }
        }

        private static _dict: { [key: string]: [string, any, IPage] } = {};

        // 现金配额相关
        private static _lastBalanceMoney: { [key: string]: number };

        public static GetTellerBalanceMoney(target: IPage): { [key: string]: number } {
            const _msg = null; // 这里需要实现 Message 的逻辑
            target.InitFd(_msg);
            target.CommSend(_msg);
            _msg.Fd16 = "0011";
            _msg.Fd92 = null; // 这里需要实现 LoginUserInfo.TellerNo 的逻辑
            target.DoTransit(_msg);
            if (_msg.Fd12 !== "0000" || !_msg.FileData)
                return null;
            return _msg.FileData.split('\n').map((x) => x.split('|')).filter((x) => x.length > 1).reduce((acc, x) => {
                acc[x[0]] = parseFloat(x[1]);
                return acc;
            }, {});
        }

        public static CheckCashQuota(target: IPage) {
            //检查开关hulei 原逻辑 开关 可以暂时注销掉
            //  if (Platform.Common.RunningParameters.ConfigManager.GetInstance().GetConfigValue("CashQuota", ConfigType.System).ToLower() != "true")
            // return;
            // if (null.toLowerCase() !== "true") // 这里需要实现 Platform.Common.RunningParameters.ConfigManager.GetInstance().GetConfigValue 的逻辑
            //     return;
            const current = this.GetTellerBalanceMoney(target);
            if (!this._lastBalanceMoney) {
                this._lastBalanceMoney = current;
                return;
            }
            const cash = current["01"] - this._lastBalanceMoney["01"];
            if (cash !== 0) {
                const bus = target.GetTradeInfoData().TradeBus;
                let flag = false;
                do {
                    bus["__CashQuota__"] = cash.toString();
                    target.OpenTradePage("Trade.OperationManage.T9901", OpenOptions.Dialog, null, bus, "现金配款");
                    flag = parseFloat(bus["__CashQuota__"]) === 0 || target.ShowQuestionMessageBox("配款未完成，是否重新配款？\n如果不做配款，后续需要做配款调整！") !== "Yes";
                } while (!flag);
                this._lastBalanceMoney = current;
            }
        }

        // UI要素提取
        public static PositioningUI(target: IPage, keycode: string[]): [string, FrameworkElement][] {
            const list: [string, FrameworkElement][] = [];
            // const root = target as FrameworkElement; //hulei 这里报错
            keycode = keycode.filter((x) => x && x.trim());
            // 这里需要实现寻找符合条件的控件的逻辑
            return list;
        }

        public static PositioningUIText(target: IPage, keycode: string[], useValue: boolean = false): { [key: string]: { [key: string]: string } } {//{ [key: string]: { [key: FrameworkElement]: string } } FrameworkElement 改为string
            const dict: { [key: string]: { [key: string]: string } } = {};
            // this.PositioningUI(target, keycode).forEach((x) => {
            //     let val = "";
            //     const ui = x[1];
            //     if (ui instanceof TextBox) {
            //         val = (ui as TextBox).Text;
            //         if (!useValue) {
            //             const symbol = ui.getAttribute("Platform.Controls.NumericContainer.CurrencyBase.CurrencySymbolProperty");
            //             val = (ui instanceof Platform.Controls.NumericContainer.CurrencyEditor ? symbol : "") + val
            //                 + (ui instanceof Platform.Controls.NumericContainer.CurrencyRateEditor ? symbol : "");
            //         }
            //     } else if (ui instanceof ComboBox) {
            //         if ((ui as ComboBox).SelectedItem && typeof (ui as ComboBox).SelectedItem === "object") {
            //             const item = (ui as ComboBox).SelectedItem as { [key: string]: string };
            //             val = useValue ? item.value : item.key;
            //         }
            //     }
            //     if (!dict[x[0]])
            //         dict[x[0]] = {};
            //     dict[x[0]][ui] = val;
            // });
            return dict;
        }
    }
}
