﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TellerSystem.Library.Ext.NPOI;
using System.Data;
using System.IO;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;

namespace TellerSystem.Library.Ext.NPOI.Product
{
    /// <summary>
    /// 
    /// </summary>
    public class excel_03 : NPOIExcel
    {
        /// <summary>
        /// 将excel中的指定sheet页转换为datatable
        /// </summary>
        /// <param name="filePath"></param>
        /// <param name="sheetIndex"></param>
        /// <returns></returns>
        public override DataTable ExcelToTable(string filePath, int sheetIndex = 0)
        {
            DataTable dt = new DataTable();
            using (FileStream fs = new FileStream(filePath, FileMode.Open, FileAccess.Read))
            {
                HSSFWorkbook hssfworkbook = new HSSFWorkbook(fs);
                ISheet sheet = hssfworkbook.GetSheetAt(sheetIndex);
                //表头  
                IRow header = sheet.GetRow(sheet.FirstRowNum);
                foreach (var cell in header.Cells)
                {
                    if (string.IsNullOrWhiteSpace(cell.StringCellValue))
                        break;
                    dt.Columns.Add(new DataColumn(cell.StringCellValue));
                }
                //数据
                for (int rowIndex = sheet.FirstRowNum + 1; rowIndex <= sheet.LastRowNum; rowIndex++)
                {
                    DataRow dr = dt.NewRow();
                    IRow row = sheet.GetRow(rowIndex);
                    if (row != null)
                    {
                        bool flag = false;
                        for (int cellIndex = 0; cellIndex < dt.Columns.Count; cellIndex++)
                        {
                            if (row.GetCell(cellIndex) == null)
                                continue;
                            if (!string.IsNullOrEmpty(row.GetCell(cellIndex).ToString()))
                                flag = true;
                            dr[cellIndex] = getCellStringValueAllCase(row.GetCell(cellIndex));
                        }
                        if (flag)
                            dt.Rows.Add(dr);
                    }
                }
            }
            return dt;
        }
        /// <summary>
        /// 将DataTable转换成Excel中的sheet页，其中sheet页的名称为DataTable的名称
        /// </summary>
        /// <param name="dt"></param>
        /// <returns></returns>
        public override byte[] DataTableToExcel(DataTable dt)
        {
            HSSFWorkbook book = new HSSFWorkbook();
            ISheet sheet = book.CreateSheet(string.IsNullOrEmpty(dt.TableName) ? "DataTable" : dt.TableName);
            //第一行为列头
            IRow row = sheet.CreateRow(0);
            for (int i = 0; i < dt.Columns.Count; i++)
            {
                string caption = dt.Columns[i].Caption;
                string colName = dt.Columns[i].ColumnName;
                row.CreateCell(i).SetCellValue(string.IsNullOrWhiteSpace(caption) ? colName : caption);
            }
            for (int i = 0; i < dt.Rows.Count; i++)
            {
                IRow row2 = sheet.CreateRow(i + 1);
                for (int j = 0; j < dt.Columns.Count; j++)
                {
                    row2.CreateCell(j).SetCellValue(Convert.ToString(dt.Rows[i][j]));
                }
            }
            using (MemoryStream ms = new MemoryStream())
            {
                book.Write(ms);
                return ms.ToArray();
            }
        }
        /// <summary>
        /// 将DataSet转换成Excel中的sheet页，其中sheet页的名称为对应的DataTable的名称
        /// </summary>
        /// <param name="ds"></param>
        /// <returns></returns>
        public override byte[] DataSetToExcel(DataSet ds)
        {
            HSSFWorkbook book = new HSSFWorkbook();
            if (ds == null || ds.Tables.Count <= 0)
            {
                return null;
            }
            foreach (DataTable dt in ds.Tables)
            {
                ISheet sheet = book.CreateSheet(string.IsNullOrEmpty(dt.TableName) ? "DataTable" : dt.TableName);
                //第一行为列头
                IRow row = sheet.CreateRow(0);
                for (int i = 0; i < dt.Columns.Count; i++)
                {
                    row.CreateCell(i).SetCellValue(dt.Columns[i].ColumnName);
                }
                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    IRow row2 = sheet.CreateRow(i + 1);
                    for (int j = 0; j < dt.Columns.Count; j++)
                    {
                        row2.CreateCell(j).SetCellValue(Convert.ToString(dt.Rows[i][j]));
                    }
                }
            }
            using (MemoryStream ms = new MemoryStream())
            {
                book.Write(ms);
                return ms.ToArray();
            }
        }
        /// <summary>
        /// 将单元格的值转化为字符串
        /// </summary>
        /// <param name="tCell"></param>
        /// <returns></returns>
        private string getCellStringValueAllCase(ICell tCell)
        {
            string tempValue = "";
            switch (tCell.CellType)
            {
                case CellType.Blank:
                    break;
                case CellType.Boolean:
                    tempValue = tCell.BooleanCellValue.ToString();
                    break;
                case CellType.Error:
                    break;
                case CellType.Formula:
                    IFormulaEvaluator fe = WorkbookFactory.CreateFormulaEvaluator(tCell.Sheet.Workbook);
                    var cellValue = fe.Evaluate(tCell);
                    switch (cellValue.CellType)
                    {
                        case CellType.Blank:
                            break;
                        case CellType.Boolean:
                            tempValue = cellValue.BooleanValue.ToString();
                            break;
                        case CellType.Error:
                            break;
                        case CellType.Formula:
                            break;
                        case CellType.Numeric:
                            tempValue = cellValue.NumberValue.ToString();
                            break;
                        case CellType.String:
                            tempValue = cellValue.StringValue.ToString();
                            break;
                        case CellType.Unknown:
                            break;
                        default:
                            break;
                    }
                    break;
                case CellType.Numeric:

                    if (DateUtil.IsCellDateFormatted(tCell))
                    {
                        tempValue = tCell.DateCellValue.ToString("yyyy-MM-dd");
                    }
                    else
                    {
                        tempValue = tCell.NumericCellValue.ToString();
                    }
                    break;
                case CellType.String:
                    tempValue = tCell.StringCellValue.Trim();
                    break;
                case CellType.Unknown:
                    break;
                default:
                    break;
            }
            return tempValue;
        }
    }
}
