// 引入必要的类型
import { ObservableCollection } from 'some-observable-collection-library'; // 假设存在这个库用于模拟ObservableCollection

// 假设TT_RuleInfo的定义
interface TT_RuleInfo {
    RuleId: string;
    RulePId: string;
    RuleNo: string;
    RuleName: string;
    RuleRemark: string;
    RuleTypeCode: string;
    RuleType: string;
    ExtId: string;
    RuleComment: string;
}

// 定义RuleInfoPSet类
class RuleInfoPSet {
    private _ruleId: string;
    private _rulePId: string;
    private _ruleNo: string;
    private _ruleName: string;
    private _ruleRemark: string;
    private _ruleTypeCode: string;
    private _ruleType: string;
    private _extId: string;
    private _ruleComment: string;
    private _ischecked: boolean | null = false;
    private _isenable: boolean = false;
    private _parent: RuleInfoPSet | null = null;
    private readonly _children: ObservableCollection<RuleInfoPSet> = new ObservableCollection<RuleInfoPSet>();

    // 事件处理相关
    private propertyChangedHandlers: ((propertyName: string) => void)[] = [];

    constructor(
        ruleId: string,
        rulePId: string,
        ruleNo: string,
        ruleName: string,
        ruleRemark: string,
        ruleTypeCode: string,
        ruleType: string,
        extId: string,
        ruleComment: string,
        list?: TT_RuleInfo[],
        powerlist?: TT_RuleInfo[],
        grouppowerdeleteentitys?: TT_RuleInfo[],
        grouppowerentitys?: TT_RuleInfo[]
    ) {
        this._ruleId = ruleId;
        this._rulePId = rulePId;
        this._ruleNo = ruleNo;
        this._ruleName = ruleName;
        this._ruleRemark = ruleRemark;
        this._ruleTypeCode = ruleTypeCode;
        this._ruleType = ruleType;
        this._extId = extId;
        this._ruleComment = ruleComment;

        if (list) {
            list.forEach((drElement) => {
                if (drElement.RulePId === ruleId && drElement.RuleId!== ruleId) {
                    let newRuleInfo: RuleInfoPSet;
                    if (powerlist && grouppowerdeleteentitys && grouppowerentitys) {
                        newRuleInfo = new RuleInfoPSet(
                            drElement.RuleId,
                            drElement.RulePId,
                            drElement.RuleNo,
                            drElement.RuleName,
                            drElement.RuleRemark,
                            drElement.RuleTypeCode,
                            drElement.RuleType,
                            drElement.ExtId,
                            drElement.RuleComment,
                            list,
                            powerlist,
                            grouppowerdeleteentitys,
                            grouppowerentitys
                        );
                        newRuleInfo.IsEnabled = false;
                        newRuleInfo.Parent = this;
                        if (powerlist.some((x) => x.RuleId === drElement.RuleId) ||
                            (!grouppowerdeleteentitys.some((x) => x.RuleId === drElement.RuleId) &&
                                grouppowerentitys.some((x) => x.RuleId === drElement.RuleId))) {
                            newRuleInfo.IsChecked = true;
                        }
                    } else if (powerlist) {
                        newRuleInfo = new RuleInfoPSet(
                            drElement.RuleId,
                            drElement.RulePId,
                            drElement.RuleNo,
                            drElement.RuleName,
                            drElement.RuleRemark,
                            drElement.RuleTypeCode,
                            drElement.RuleType,
                            drElement.ExtId,
                            drElement.RuleComment,
                            list,
                            powerlist
                        );
                        newRuleInfo.IsEnabled = false;
                        newRuleInfo.IsChecked = powerlist.some((x) => x.RuleId === drElement.RuleId);
                        newRuleInfo.Parent = this;
                    } else {
                        newRuleInfo = new RuleInfoPSet(
                            drElement.RuleId,
                            drElement.RulePId,
                            drElement.RuleNo,
                            drElement.RuleName,
                            drElement.RuleRemark,
                            drElement.RuleTypeCode,
                            drElement.RuleType,
                            drElement.ExtId,
                            drElement.RuleComment,
                            list
                        );
                    }
                    this._children.add(newRuleInfo);
                }
            });
        }
    }

    // 以下是属性的getter和setter
    get RuleId(): string {
        return this._ruleId;
    }

    set RuleId(value: string) {
        this._ruleId = value;
        this.onPropertyChanged('RuleId');
    }

    get RulePId(): string {
        return this._rulePId;
    }

    set RulePId(value: string) {
        this._rulePId = value;
        this.onPropertyChanged('RulePId');
    }

    get RuleNo(): string {
        return this._ruleNo;
    }

    set RuleNo(value: string) {
        this._ruleNo = value;
        this.onPropertyChanged('RuleNo');
    }

    get RuleName(): string {
        return this._ruleName;
    }

    set RuleName(value: string) {
        this._ruleName = value;
        this.onPropertyChanged('RuleName');
    }

    get RuleRemark(): string {
        return this._ruleRemark;
    }

    set RuleRemark(value: string) {
        this._ruleRemark = value;
        this.onPropertyChanged('RuleRemark');
    }

    get RuleTypeCode(): string {
        return this._ruleTypeCode;
    }

    set RuleTypeCode(value: string) {
        this._ruleTypeCode = value;
        this.onPropertyChanged('RuleTypeCode');
    }

    get RuleType(): string {
        return this._ruleType;
    }

    set RuleType(value: string) {
        this._ruleType = value;
        this.onPropertyChanged('RuleType');
    }

    get ExtId(): string {
        return this._extId;
    }

    set ExtId(value: string) {
        this._extId = value;
        this.onPropertyChanged('ExtId');
    }

    get RuleComment(): string {
        return this._ruleComment;
    }

    set RuleComment(value: string) {
        this._ruleComment = value;
        this.onPropertyChanged('RuleComment');
    }

    get IsChecked(): boolean | null {
        return this._ischecked;
    }

    set IsChecked(value: boolean | null) {
        this._ischecked = value;
        this.onPropertyChanged('IsChecked');
    }

    get IsEnabled(): boolean {
        return this._isenable;
    }

    set IsEnabled(value: boolean) {
        this._isenable = value;
        this.onPropertyChanged('IsEnabled');
    }

    get Parent(): RuleInfoPSet | null {
        return this._parent;
    }

    set Parent(value: RuleInfoPSet | null) {
        this._parent = value;
        this.onPropertyChanged('Parent');
    }

    get Children(): ObservableCollection<RuleInfoPSet> {
        return this._children;
    }

    // 事件触发方法
    onPropertyChanged(propertyName: string): void {
        this.propertyChangedHandlers.forEach((handler) => handler(propertyName));
    }

    // 模拟事件订阅方法
    addPropertyChangedHandler(handler: (propertyName: string) => void): void {
        this.propertyChangedHandlers.push(handler);
    }
}

export { RuleInfoPSet };