﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Diagnostics;
using Platform.Common.RunningParameters;
using System.Windows.Media.Imaging;
using System.Net;
using Newtonsoft.Json;
using System.Net.Cache;
using Newtonsoft.Json.Linq;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.Library.Ext.Function;
using Platform.Common.LogSystem;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class AICapacity
    {
        private static AICapacity instence = null;
        /// <summary>
        /// token信息
        /// </summary>
        private string token;
        /// <summary>
        /// ip端口
        /// </summary>
        private string ipCom;
        /// <summary>
        /// 机构号前3位
        /// </summary>
        private string jgh;
        /// <summary>
        /// 生态平台用户名
        /// </summary>
        private string userCode;
        /// <summary>
        /// 生态平台密钥
        /// </summary>
        private string password;
        /// <summary>
        /// 生态平台对接柜面的系统名
        /// </summary>
        private string sysName;
        /// <summary>
        /// 人脸比对阈值
        /// </summary>
        private string rlbd_yz;
        /// <summary>
        /// 人脸比对开关
        /// </summary>
        private bool rlbz_isOpen;
        /// <summary>
        /// 上次获取时间
        /// </summary>
        private DateTime lastTime = DateTime.Now.AddMinutes(-10);
        /// <summary>
        /// 版本标志
        /// </summary>
        string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
        /// <summary>
        /// 柜面参数配置表
        /// </summary>
        private List<ServiceProxy.Ext.TradeBusinessEntitys.TT_SysConfig> sysConfigList;
        /// <summary>
        /// 柜面参数是否可用
        /// </summary>
        private bool isEnble;
        /// <summary>
        /// 错误信息
        /// </summary>
        private string ErrMes;
        private AICapacity()
        {
            sysConfigList = TradeManagerHandle.GetSysconfigList(null);
            jgh = LoginUserInfo.KinbrNo.Substring(0, 3);
            userCode = GetSysConfig("AI_用户名");
            password = GetSysConfig("AI_密钥");
            sysName = GetSysConfig("AI_系统名");
            rlbz_isOpen = GetSysConfig("AI_人脸比对_开关").ToLower() == "true";
            rlbd_yz = GetSysConfig("AI_人脸比对_阈值");
            var list = GetSysConfig("AI_地址").Split('|');
            ipCom = list.Length == 1 ? list[0] : list.FirstOrDefault(x =>
            {
                //检测网络连通性,使用获取token接口来测试
                var request = System.Net.WebRequest.Create("http://" + x + "/core/auth/getVirtualToken");
                try
                {
                    request.Timeout = 10000;//10s内无反应，视为无效
                    request.GetResponse();
                    return true;
                }
                catch (Exception ex)
                {
                    //返回非200的正常响应，也认为配置可用
                    return (ex is WebException && (ex as WebException).Status == WebExceptionStatus.ProtocolError);
                }
                finally
                {
                    request.Abort();
                }
            });
            isEnble = RLSB_isEnble();
        }
        /// <summary>
        /// 人脸比对开关
        /// </summary>
        public bool IsOpen
        {
            get { return rlbz_isOpen; }
        }

        /// <summary>
        /// 柜面人脸比对参数是否可用
        /// </summary>
        public bool IsEnble
        {
            get { return isEnble; }
        }

        /// <summary>
        /// 获取柜面配置的参数值，优先取带机构号前三位的参数值
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        private string GetSysConfig(string key)
        {
            string value = "";
            //青隆做升级处理，加上机构号前三位
            string key_ql = key + "_" + jgh;
            if (sysConfigList.Exists(x => x.Configname == key_ql))
            {
                value = sysConfigList.Find(x => x.Configname == key_ql).Configvalue;
            }
            else if (sysConfigList.Exists(x => x.Configname == key))
            {
                value = sysConfigList.Find(x => x.Configname == key).Configvalue;
            }
            return value;
        }
        /// <summary>
        /// 获取人脸比对结果
        /// </summary>
        /// <param name="img">身份核查照片信息</param>
        /// <returns></returns>
        public bool? GetResultFromRLBD(string img, out string result)
        {
            return GetResultFromRLBD(img, out result, null);
        }

        public bool? GetResultFromRLBD(string img, out string result, ExtData ext)
        {
            result = "人脸比对失败（未开启服务）！";
            if (!rlbz_isOpen)
                return null;
            result = "柜面人脸比对参数配置错误！";
            if (isEnble)
            {
                result = "人脸比对失败（联网核查的照片信息为空）！";
                if (!string.IsNullOrEmpty(img))
                {
                    string img_GPY = AICapacity.GetInstence().getCusPhoto();
                    result = "人脸比对失败（拍摄的照片信息为空）！";
                    if (!string.IsNullOrEmpty(img_GPY))
                    {
                        var score = faceMach(img, img_GPY, ext);
                        var f = score.ToDecimal() >= rlbd_yz.ToDecimal();
                        if (string.IsNullOrEmpty(ErrMes))
                            result = (f ? "通过" : "未通过") + "(" + score + ")！";
                        else
                            result = (f ? "通过" : "未通过") + "(" + score + ErrMes + ")！";
                        //result = "人脸比对结果:" + (f ? "通过" : "未通过") + "(" + score + ")！";
                        return f;
                    }
                }
            }
            return false;
        }
        /// <summary>
        /// 判断人脸识别参数是否可用
        /// </summary>
        /// <returns></returns>
        private bool RLSB_isEnble()
        {
            if (string.IsNullOrEmpty(userCode) || string.IsNullOrEmpty(password) || string.IsNullOrEmpty(sysName) || string.IsNullOrEmpty(rlbd_yz) || string.IsNullOrEmpty(ipCom))
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// 获取客户拍照信息
        /// </summary>
        /// <returns></returns>
        private string getCusPhoto()
        {
            var entity = Devices.DeviceHelper.Capture(Devices.Interface.CameraKind.Minor2Outside);
            if (entity.HasError) return string.Empty;
            byte[] ystp = null;
            using (var ms = new MemoryStream(entity.Data))
            {
                BitmapImage bitmapImage = new BitmapImage();
                bitmapImage.BeginInit();
                bitmapImage.CacheOption = BitmapCacheOption.OnLoad;
                bitmapImage.StreamSource = ms;
                bitmapImage.EndInit();
                bitmapImage.Freeze();
                //压缩图片
                ystp = SystemFunction.GetPhotoBytesForSize_QL(bitmapImage, 100, 150);
            }
            return ystp == null ? string.Empty : Convert.ToBase64String(ystp);
        }

        /// <summary>
        /// AI调用通讯
        /// </summary>
        /// <param name="path">路径</param>
        /// <param name="req">请求参数</param>
        /// <param name="accessToken">token</param>
        /// <returns></returns>
        private string AIRequest(string path, string req, string accessToken, string method = "POST")
        {
            StringBuilder loginfo = new StringBuilder();
            loginfo.AppendLine("通讯接口:" + path);
            loginfo.AppendLine("操作柜员:" + LoginUserInfo.TellerNo);
            loginfo.AppendLine("发生时间:" + DateTime.Now.ToString("HHmmss"));
            loginfo.AppendLine("日志信息:通讯报文:" + req);
            PlatformLogger.CommunicationInfo(loginfo.ToString());
            //先根据用户请求的uri构造请求地址
            string serviceUrl = "http://" + ipCom + path;
            //创建Web访问对象
            HttpWebRequest webRequest = (HttpWebRequest)WebRequest.Create(serviceUrl);
            webRequest.ServicePoint.Expect100Continue = false;
            //把用户传过来的数据转成“UTF-8”的字节流
            byte[] buf = System.Text.Encoding.GetEncoding("UTF-8").GetBytes(req);
            webRequest.Method = method;
            webRequest.ContentLength = buf.Length;
            webRequest.ContentType = "application/json";
            webRequest.CachePolicy = new RequestCachePolicy(RequestCacheLevel.NoCacheNoStore);
            webRequest.Accept = "*/*";
            if (!string.IsNullOrEmpty(accessToken))
                webRequest.Headers.Add("Authorization", accessToken);
            if (method == "POST")
            {
                //发送请求
                Stream stream = webRequest.GetRequestStream();
                stream.Write(buf, 0, buf.Length);
                stream.Close();
            }
            //获取接口返回值
            //通过Web访问对象获取响应内容
            HttpWebResponse webResponse = (HttpWebResponse)webRequest.GetResponse();
            //通过响应内容流创建StreamReader对象，因为StreamReader更高级更快
            StreamReader reader = new StreamReader(webResponse.GetResponseStream(), Encoding.UTF8);
            //string returnXml = HttpUtility.UrlDecode(reader.ReadToEnd());//如果有编码问题就用这个方法
            string returnXml = reader.ReadToEnd();//利用StreamReader就可以从响应内容从头读到尾
            reader.Close();
            webResponse.Close();
            loginfo = new StringBuilder();
            loginfo.AppendLine("通讯接口:" + path);
            loginfo.AppendLine("操作柜员:" + LoginUserInfo.TellerNo);
            loginfo.AppendLine("发生时间:" + DateTime.Now.ToString("HHmmss"));
            loginfo.AppendLine("日志信息:通讯响应:" + returnXml);
            PlatformLogger.CommunicationInfo(loginfo.ToString());
            return returnXml;
        }

        #region Simple Call
        public abstract class Req
        {
            [JsonProperty("extData")]
            public ExtData ExtData { get; set; }
        }

        public class ExtData
        {
            public ExtData(string serialNo, Dictionary<string, string> extInfo = null)
            {
                Branch = LoginUserInfo.KinbrNo;
                Operator = LoginUserInfo.TellerNo;
                SerialNo = serialNo;
                ExtInfo = extInfo ?? new Dictionary<string, string> { };
            }

            [JsonProperty("branch")]
            public string Branch { get; private set; }

            [JsonProperty("operator")]
            public string Operator { get; private set; }

            [JsonProperty("serialNo")]
            public string SerialNo { get; private set; }

            [JsonProperty("extInfo")]
            public Dictionary<string, string> ExtInfo { get; private set; }
        }

        public class Resp
        {
            private JObject root;

            public Resp(string data)
            {
                HasError = true;
                ErrorMsg = data;
                try
                {
                    root = JsonConvert.DeserializeObject(data) as JObject;
                    HasError = Value("errcode", -1) != 0;
                    ErrorMsg = Value("errmsg", "未提取到有效信息");
                }
                catch { }
            }

            public bool HasError { get; internal set; }

            public string ErrorMsg { get; internal set; }

            /// <summary>
            /// 提取要素
            /// </summary>
            /// <typeparam name="T"></typeparam>
            /// <param name="path"></param>
            /// <param name="def"></param>
            /// <returns></returns>
            public T Value<T>(string path, T def)
            {
                if (root == null) return def;
                var ele = root.SelectToken(path);
                if (ele == null) return def;
                return ele.ToObject<T>();
            }
        }

        /// <summary>
        /// post请求
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="path"></param>
        /// <param name="req"></param>
        /// <returns></returns>
        public Resp AIRequest<T>(string path, T req)
            where T : Req, new()
        {
            return AIRequest2Resp(path, JsonConvert.SerializeObject(req));
        }

        /// <summary>
        /// get请求
        /// </summary>
        /// <param name="path"></param>
        /// <param name="parms"></param>
        /// <returns></returns>
        public Resp AIRequest(string path, Dictionary<string, string> parms)
        {
            var p = "?" + string.Join("&", parms.Select(x => string.Format("{0}={1}", x.Key, x.Value)));
            return AIRequest2Resp(path + p, string.Empty, "GET");
        }

        public Resp AIRequest2Resp(string path, string req, string method = "POST")
        {
            var accessToken = getTokenInfo();
            if (string.IsNullOrEmpty(accessToken))
                return new Resp("token获取失败！");
            try
            {
                var data = AIRequest(path, req, accessToken, method);
                return new Resp(data);
            }
            catch (Exception ex)
            {
                return new Resp("AI平台调用异常！" + ex.Message);
            }
        }

        public class UniversalReq<T> : Req
        {
            [JsonProperty("buscode")]
            public string BizCode { get; set; }

            [JsonProperty("group")]
            public string Group { get; set; }

            [JsonProperty("data")]
            public T Data { get; set; }
        }

        public Resp AIRequest2Universal<T>(string bizCode, T data, ExtData ext = null)
        {
            var req = new UniversalReq<T>
            {
                BizCode = bizCode,
                Group = "real",
                Data = data,
                ExtData = ext,
            };
            return AIRequest("/tech/ai/celerity/universal", req);
        }
        #endregion

        #region 文件服务
        /// <summary>
        /// 上传文件
        /// </summary>
        /// <param name="path">上传服务器地址</param>
        /// <param name="filePath">文件路径</param>
        /// <param name="fileName">文件名称</param>
        /// <param name="accessToken"></param>
        /// <returns></returns>
        public Resp UploadFile(byte[] fileByte, string fileName, int days = 1)
        {
            var resp = new Resp("");
            try
            {
                //先根据用户请求的uri构造请求地址
                string serviceUrl = "http://" + ipCom + "/upload/";

                StringBuilder loginfo = new StringBuilder();
                loginfo.AppendLine("通讯接口:" + serviceUrl);
                loginfo.AppendLine("操作柜员:" + LoginUserInfo.TellerNo);
                loginfo.AppendLine("发生时间:" + DateTime.Now.ToString("HHmmss"));
                loginfo.AppendLine("日志信息:通讯报文:上传文件名:" + fileName);
                PlatformLogger.CommunicationInfo(loginfo.ToString());

                //创建Web访问对象
                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(serviceUrl);
                CookieContainer cookieContainer = new CookieContainer();
                request.CookieContainer = cookieContainer;
                request.AllowAutoRedirect = true;
                request.MaximumResponseHeadersLength = 1024;
                request.Method = "POST";
                string boundary = DateTime.Now.Ticks.ToString("X"); // 随机分隔线
                request.ContentType = "multipart/form-data;charset=utf-8;boundary=----WebKitFormBoundary" + boundary;
                byte[] itemBoundaryBytes = Encoding.UTF8.GetBytes("\r\n------WebKitFormBoundary" + boundary + "\r\n");
                byte[] endBoundaryBytes = Encoding.UTF8.GetBytes("\r\n------WebKitFormBoundary" + boundary + "--\r\n");
                //请求头部信息 
                StringBuilder sbHeader = new StringBuilder(string.Format("Content-Disposition:form-data;name=\"file\";filename=\"{0}\"\r\nContent-Type:application/octet-stream\r\n\r\n", fileName, 2));
                //请求头部信息 
                StringBuilder sbbody = new StringBuilder(string.Format("Content-Disposition:form-data;name=\"saveDay\";\r\n\r\n"));
                byte[] postHeaderBytes = Encoding.UTF8.GetBytes(sbHeader.ToString());
                byte[] postbodyBytes = Encoding.UTF8.GetBytes(sbbody.ToString());
                byte[] bodyBytes = Encoding.UTF8.GetBytes(days.ToString());
                Stream postStream = request.GetRequestStream();
                postStream.Write(itemBoundaryBytes, 0, itemBoundaryBytes.Length);
                postStream.Write(postHeaderBytes, 0, postHeaderBytes.Length);
                postStream.Write(fileByte, 0, fileByte.Length);
                postStream.Write(itemBoundaryBytes, 0, itemBoundaryBytes.Length);
                postStream.Write(postbodyBytes, 0, postbodyBytes.Length);
                postStream.Write(bodyBytes, 0, bodyBytes.Length);
                postStream.Write(endBoundaryBytes, 0, endBoundaryBytes.Length);
                postStream.Close();
                HttpWebResponse res;

                res = (HttpWebResponse)request.GetResponse();
                StreamReader sr = new StreamReader(res.GetResponseStream(), Encoding.UTF8);
                var content = sr.ReadToEnd();

                loginfo = new StringBuilder();
                loginfo.AppendLine("通讯接口:" + serviceUrl);
                loginfo.AppendLine("操作柜员:" + LoginUserInfo.TellerNo);
                loginfo.AppendLine("发生时间:" + DateTime.Now.ToString("HHmmss"));
                loginfo.AppendLine("上传文件名:" + fileName);
                loginfo.AppendLine("日志信息:通讯响应:" + content);
                PlatformLogger.CommunicationInfo(loginfo.ToString());
                return new Resp(content);
            }
            catch (Exception ex)
            {
                return new Resp(ex.ToString());
            }
        }
        #endregion

        /// <summary>
        /// 获取人脸比对token信息
        /// </summary>
        /// <returns></returns>
        private string getTokenInfo()
        {
            //token信息15分钟失效，暂控制在10分钟之内。
            if (lastTime.AddMinutes(10) < DateTime.Now)
            {
                var path = "/core/auth/getVirtualToken";
                TokenInfoReq data = new TokenInfoReq();
                if (!RLSB_isEnble())
                {
                    return "";
                }

                //比较参数配置表里面的机构号
                data.userCode = userCode;
                data.password = password;
                data.sysName = sysName;
                string req = JsonConvert.SerializeObject(data);
                var res = AIRequest(path, req, "");
                JObject jo = (JObject)JsonConvert.DeserializeObject(res);
                if (jo.GetValue("errcode").ToString() == "0")
                {
                    lastTime = DateTime.Now;
                    token = jo.GetValue("data")["accessToken"].ToString();
                }
                else { return ""; }
            }
            return token;
        }

        /// <summary>
        /// 人脸比对
        /// </summary>
        /// <param name="img_SFHC">身份核查照片</param>
        /// <param name="img_GPY">高拍仪照片</param>
        /// <returns></returns>
        private string faceMach(string img_SFHC, string img_GPY, ExtData ext = null)
        {
            //传入数据
            var data = new FaceMachReq()
            {
                Async = false,
                LiveCode = "",
                ExtData = ext ?? new ExtData(string.Empty),
            };
            var image = new TuPian()
            {
                Origin = "self",
                Data = img_SFHC,
            };
            var faceData = new FaceData()
            {
                Type = "image",
                VideoType = "SILENT",
                Origin = "self",
                Data = img_GPY,
            };
            data.image = new TuPian();
            data.faceData = new FaceData();
            data.image = image;
            data.faceData = faceData;
            var path = "/tech/ai/face/match";
            string grade = "";
            var resp = AIRequest(path, data);
            ErrMes = resp.ErrorMsg;//默认
            if (!resp.HasError)
            {
                grade = resp.Value("data.result.score", string.Empty);
                ErrMes = "";
            }
            else if (resp.Value("errcode", -1) == 1715)
            {
                ErrMes = "人脸识别剩余次数不足，请充值";
            }
            return grade;
        }

        public static AICapacity GetInstence()
        {
            if (instence == null)
            {
                instence = new AICapacity();
            }
            return instence;
        }
    }
    public class FaceMachReq : AICapacity.Req
    {
        [JsonProperty("async")]
        public Boolean Async { get; set; }
        [JsonProperty("liveCode")]
        public string LiveCode { get; set; }
        [JsonProperty("image")]
        public TuPian image { get; set; }
        [JsonProperty("faceData")]
        public FaceData faceData { get; set; }
    }
    public class TuPian
    {
        [JsonProperty("origin")]
        public string Origin { get; set; }
        [JsonProperty("data")]
        public string Data { get; set; }

    }
    public class FaceData
    {
        [JsonProperty("type")]
        public string Type { get; set; }
        [JsonProperty("videoType")]
        public string VideoType { get; set; }
        [JsonProperty("origin")]
        public string Origin { get; set; }
        [JsonProperty("data")]
        public string Data { get; set; }
    }
    public class TokenInfoReq
    {
        public string sysName { get; set; }
        public string userCode { get; set; }
        public string password { get; set; }
    }
}
