// 模拟依赖的类和命名空间
class TradeManagerHandle {
    static GetSysconfigList(_: any) {
        return [];
    }
    static TT_SYSCONFIG_GetConfigValueByConfigID(id: string) {
        return "";
    }
}

class LoginUserInfo {
    static get KinbrNo() {
        return "";
    }
    static get TellerNo() {
        return "";
    }
    static get TradeDate() {
        return "";
    }
}

class PlatformLogger {
    static CommunicationInfo(info: string) {
        console.log(info);
    }
}

class Devices {
    static DeviceHelper = {
        Capture(cameraKind: any) {
            return {
                HasError: false,
                Data: new Uint8Array()
            };
        }
    };
    static Interface = {
        CameraKind: {
            Minor2Outside: "Minor2Outside"
        }
    };
}

class SystemFunction {
    static GetPhotoBytesForSize_QL(bitmapImage: any, width: number, height: number) {
        return new Uint8Array();
    }
}

// 定义 ExtData 类
class ExtData {
    Branch: string;
    Operator: string;
    SerialNo: string;
    ExtInfo: { [key: string]: string };

    constructor(serialNo: string, extInfo?: { [key: string]: string }) {
        this.Branch = LoginUserInfo.KinbrNo;
        this.Operator = LoginUserInfo.TellerNo;
        this.SerialNo = serialNo;
        this.ExtInfo = extInfo || {};
    }
}

// 定义 Resp 类
class Resp {
    HasError: boolean;
    ErrorMsg: string;
    private root: any;

    constructor(data: string) {
        this.HasError = true;
        this.ErrorMsg = data;
        try {
            this.root = JSON.parse(data);
            this.HasError = this.Value("errcode", -1) !== 0;
            this.ErrorMsg = this.Value("errmsg", "未提取到有效信息");
        } catch (error) {
            // 忽略解析错误
        }
    }

    Value<T>(path: string, def: T): T {
        if (!this.root) return def;
        const parts = path.split('.');
        let current = this.root;
        for (const part of parts) {
            if (!current[part]) return def;
            current = current[part];
        }
        return current as T;
    }
}

// 定义 FaceMachReq 类
class FaceMachReq {
    @JsonProperty("async")
    Async: boolean;
    @JsonProperty("liveCode")
    LiveCode: string;
    @JsonProperty("image")
    image: TuPian;
    @JsonProperty("faceData")
    faceData: FaceData;
    @JsonProperty("extData")
    ExtData: ExtData;

    constructor() {
        this.Async = false;
        this.LiveCode = "";
        this.image = new TuPian();
        this.faceData = new FaceData();
        this.ExtData = new ExtData("");
    }
}

// 定义 TuPian 类
class TuPian {
    @JsonProperty("origin")
    Origin: string;
    @JsonProperty("data")
    Data: string;

    constructor() {
        this.Origin = "self";
        this.Data = "";
    }
}

// 定义 FaceData 类
class FaceData {
    @JsonProperty("type")
    Type: string;
    @JsonProperty("videoType")
    VideoType: string;
    @JsonProperty("origin")
    Origin: string;
    @JsonProperty("data")
    Data: string;

    constructor() {
        this.Type = "image";
        this.VideoType = "SILENT";
        this.Origin = "self";
        this.Data = "";
    }
}

// 定义 TokenInfoReq 类
class TokenInfoReq {
    sysName: string;
    userCode: string;
    password: string;
}

// 定义 AICapacity 类
class AICapacity {
    private static instence: AICapacity | null = null;
    private token: string = "";
    private ipCom: string = "";
    private jgh: string = "";
    private userCode: string = "";
    private password: string = "";
    private sysName: string = "";
    private rlbd_yz: string = "";
    private rlbz_isOpen: boolean = false;
    private lastTime: Date = new Date(Date.now() - 10 * 60 * 1000);
    private flag: string = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
    private sysConfigList: any[] = [];
    private isEnble: boolean = false;
    private ErrMes: string = "";

    private constructor() {
        this.sysConfigList = TradeManagerHandle.GetSysconfigList(null);
        this.jgh = LoginUserInfo.KinbrNo.substring(0, 3);
        this.userCode = this.GetSysConfig("AI_用户名");
        this.password = this.GetSysConfig("AI_密钥");
        this.sysName = this.GetSysConfig("AI_系统名");
        this.rlbz_isOpen = this.GetSysConfig("AI_人脸比对_开关").toLowerCase() === "true";
        this.rlbd_yz = this.GetSysConfig("AI_人脸比对_阈值");
        const list = this.GetSysConfig("AI_地址").split('|');
        this.ipCom = list.length === 1 ? list[0] : list.find(async (x) => {
            try {
                const response = await fetch(`http://${x}/core/auth/getVirtualToken`, {
                    method: 'GET',
                    timeout: 10000
                });
                return true;
            } catch (error) {
                return error instanceof TypeError && error.message.includes('NetworkError');
            }
        }) || "";
        this.isEnble = this.RLSB_isEnble();
    }

    static GetInstence(): AICapacity {
        if (!this.instence) {
            this.instence = new AICapacity();
        }
        return this.instence;
    }

    get IsOpen(): boolean {
        return this.rlbz_isOpen;
    }

    get IsEnble(): boolean {
        return this.isEnble;
    }

    private GetSysConfig(key: string): string {
        let value = "";
        const key_ql = key + "_" + this.jgh;
        const itemWithQl = this.sysConfigList.find(x => x.Configname === key_ql);
        if (itemWithQl) {
            value = itemWithQl.Configvalue;
        } else {
            const item = this.sysConfigList.find(x => x.Configname === key);
            if (item) {
                value = item.Configvalue;
            }
        }
        return value;
    }

    GetResultFromRLBD(img: string, ext?: ExtData): { result: string; success?: boolean } {
        let result = "人脸比对失败（未开启服务）！";
        if (!this.rlbz_isOpen) {
            return { result };
        }
        result = "柜面人脸比对参数配置错误！";
        if (this.isEnble) {
            result = "人脸比对失败（联网核查的照片信息为空）！";
            if (img) {
                const img_GPY = this.getCusPhoto();
                result = "人脸比对失败（拍摄的照片信息为空）！";
                if (img_GPY) {
                    const score = this.faceMach(img, img_GPY, ext);
                    const f = parseFloat(score) >= parseFloat(this.rlbd_yz);
                    if (!this.ErrMes) {
                        result = (f ? "通过" : "未通过") + `(${score})！`;
                    } else {
                        result = (f ? "通过" : "未通过") + `(${score}${this.ErrMes})！`;
                    }
                    return { result, success: f };
                }
            }
        }
        return { result, success: false };
    }

    private RLSB_isEnble(): boolean {
        return !!this.userCode && !!this.password && !!this.sysName && !!this.rlbd_yz && !!this.ipCom;
    }

    private getCusPhoto(): string {
        const entity = Devices.DeviceHelper.Capture(Devices.Interface.CameraKind.Minor2Outside);
        if (entity.HasError) return "";
        let ystp: Uint8Array | null = null;
        const blob = new Blob([entity.Data], { type: 'image/jpeg' });
        const bitmapImage = URL.createObjectURL(blob);
        ystp = SystemFunction.GetPhotoBytesForSize_QL(bitmapImage, 100, 150);
        return ystp ? btoa(String.fromCharCode.apply(null, Array.from(ystp))) : "";
    }

    private async AIRequest(path: string, req: string, accessToken: string, method: string = "POST"): Promise<string> {
        const loginfo = [];
        loginfo.push(`通讯接口:${path}`);
        loginfo.push(`操作柜员:${LoginUserInfo.TellerNo}`);
        loginfo.push(`发生时间:${new Date().toLocaleTimeString().replace(/:/g, '')}`);
        loginfo.push(`日志信息:通讯报文:${req}`);
        PlatformLogger.CommunicationInfo(loginfo.join('\n'));

        const serviceUrl = `http://${this.ipCom}${path}`;
        const headers: { [key: string]: string } = {
            'Content-Type': 'application/json',
            'Accept': '*/*'
        };
        if (accessToken) {
            headers['Authorization'] = accessToken;
        }

        const response = await fetch(serviceUrl, {
            method,
            headers,
            body: method === "POST" ? req : undefined
        });

        const returnXml = await response.text();

        const loginfoAfter = [];
        loginfoAfter.push(`通讯接口:${path}`);
        loginfoAfter.push(`操作柜员:${LoginUserInfo.TellerNo}`);
        loginfoAfter.push(`发生时间:${new Date().toLocaleTimeString().replace(/:/g, '')}`);
        loginfoAfter.push(`日志信息:通讯响应:${returnXml}`);
        PlatformLogger.CommunicationInfo(loginfoAfter.join('\n'));

        return returnXml;
    }

    private async getTokenInfo(): Promise<string> {
        if (new Date(this.lastTime.getTime() + 10 * 60 * 1000) < new Date()) {
            const path = "/core/auth/getVirtualToken";
            const data = new TokenInfoReq();
            if (!this.RLSB_isEnble()) {
                return "";
            }
            data.userCode = this.userCode;
            data.password = this.password;
            data.sysName = this.sysName;
            const req = JSON.stringify(data);
            const res = await this.AIRequest(path, req, "");
            try {
                const jo = JSON.parse(res);
                if (jo.errcode.toString() === "0") {
                    this.lastTime = new Date();
                    this.token = jo.data.accessToken;
                } else {
                    return "";
                }
            } catch (error) {
                return "";
            }
        }
        return this.token;
    }

    private async faceMach(img_SFHC: string, img_GPY: string, ext?: ExtData): Promise<string> {
        const data = new FaceMachReq();
        data.Async = false;
        data.LiveCode = "";
        data.ExtData = ext || new ExtData("");
        const image = new TuPian();
        image.Origin = "self";
        image.Data = img_SFHC;
        const faceData = new FaceData();
        faceData.Type = "image";
        faceData.VideoType = "SILENT";
        faceData.Origin = "self";
        faceData.Data = img_GPY;
        data.image = image;
        data.faceData = faceData;
        const path = "/tech/ai/face/match";
        let grade = "";
        const resp = await this.AIRequest2Resp(path, JSON.stringify(data));
        this.ErrMes = resp.ErrorMsg;
        if (!resp.HasError) {
            grade = resp.Value("data.result.score", "");
            this.ErrMes = "";
        } else if (resp.Value("errcode", -1) === 1715) {
            this.ErrMes = "人脸识别剩余次数不足，请充值";
        }
        return grade;
    }

    async AIRequest2Resp(path: string, req: string, method: string = "POST"): Promise<Resp> {
        const accessToken = await this.getTokenInfo();
        if (!accessToken) {
            return new Resp("token获取失败！");
        }
        try {
            const data = await this.AIRequest(path, req, accessToken, method);
            return new Resp(data);
        } catch (error) {
            return new Resp(`AI平台调用异常！${error.message}`);
        }
    }

    async UploadFile(fileByte: Uint8Array, fileName: string, days: number = 1): Promise<Resp> {
        try {
            const serviceUrl = `http://${this.ipCom}/upload/`;
            const loginfo = [];
            loginfo.push(`通讯接口:${serviceUrl}`);
            loginfo.push(`操作柜员:${LoginUserInfo.TellerNo}`);
            loginfo.push(`发生时间:${new Date().toLocaleTimeString().replace(/:/g, '')}`);
            loginfo.push(`日志信息:通讯报文:上传文件名:${fileName}`);
            PlatformLogger.CommunicationInfo(loginfo.join('\n'));

            const formData = new FormData();
            const blob = new Blob([fileByte], { type: 'application/octet-stream' });
            formData.append('file', blob, fileName);
            formData.append('saveDay', days.toString());

            const response = await fetch(serviceUrl, {
                method: 'POST',
                body: formData
            });

            const content = await response.text();

            const loginfoAfter = [];
            loginfoAfter.push(`通讯接口:${serviceUrl}`);
            loginfoAfter.push(`操作柜员:${LoginUserInfo.TellerNo}`);
            loginfoAfter.push(`发生时间:${new Date().toLocaleTimeString().replace(/:/g, '')}`);
            loginfoAfter.push(`上传文件名:${fileName}`);
            loginfoAfter.push(`日志信息:通讯响应:${content}`);
            PlatformLogger.CommunicationInfo(loginfoAfter.join('\n'));

            return new Resp(content);
        } catch (error) {
            return new Resp(error.toString());
        }
    }
}

// 模拟 JsonProperty 装饰器
function JsonProperty(name: string) {
    return function (target: any, propertyKey: string) {
        // 这里可以实现更多逻辑，当前仅作标记
    };
}

export { AICapacity };