// 模拟 ComboBoxDataGrid 类型
class ComboBoxDataGrid {
    selectedItem: DataRowView | null;

    constructor() {
        this.selectedItem = null;
    }
}

// 模拟 DataRowView 类型
class DataRowView {
    row: DataRow;

    constructor(row: DataRow) {
        this.row = row;
    }
}

// 模拟 DataRow 类型
class DataRow {
    itemArray: any[];

    constructor(itemArray: any[]) {
        this.itemArray = itemArray;
    }
}

// 定义扩展方法所在的命名空间
namespace TellerSystem {
    export namespace Library {
        export namespace Ext {
            export namespace TradeExtension {
                export class ComboBoxDataGridExtension {
                    /**
                     * 获取当前选择行
                     * @param cbxDataGrid ComboBoxDataGrid
                     * @returns 当前行(CurrentRow)
                     */
                    static GetSelectRow(cbxDataGrid: ComboBoxDataGrid): DataRow | null {
                        const drv = cbxDataGrid.selectedItem;
                        if (drv === null) return null;

                        return drv.row;
                    }

                    /**
                     * 获取当前选择行的值
                     * @param cbxDataGrid ComboBoxDataGrid
                     * @returns DataRow.ItemArray
                     */
                    static GetSelectRowValue(cbxDataGrid: ComboBoxDataGrid): string[] | null {
                        const row = this.GetSelectRow(cbxDataGrid);
                        if (row === null) return null;

                        return row.itemArray.map(obj => obj.toString());
                    }
                }
            }
        }
    }
}

export default TellerSystem.Library.Ext.TradeExtension.ComboBoxDataGridExtension;