﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TellerSystem.Communication;
using Platform.Presentation.Interfaces;
using TellerSystem.Library.Ext.Function;
using System.Data;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using Platform.Common.RunningParameters;

namespace TellerSystem.Library.Ext.TradeExtension
{
    class Currency2DicManager
    {
        public Currency2DicManager()
        {
            _dicCurrency = new Dictionary<string, string>();
            _dicTicketDistinct = new Dictionary<string, string>();
            _dicKinbr = new Dictionary<string, string>();
        }
        /// <summary>
        /// 单例
        /// </summary>
        private static Currency2DicManager _instance;
        /// <summary>
        /// 单例
        /// </summary>
        public static Currency2DicManager Instance
        {
            get 
            {
                if (_instance == null)
                    _instance = new Currency2DicManager();
                return _instance; 
            }
        }
        /// <summary>
        /// 币种
        /// </summary>
        private static Dictionary<string, string> _dicCurrency;
        /// <summary>
        /// 券别
        /// </summary>
        private static Dictionary<string, string> _dicTicketDistinct;
        /// <summary>
        /// 机构信息
        /// </summary>
        private static Dictionary<string, string> _dicKinbr;

        /// <summary>
        /// 获取币种字典
        /// </summary>
        /// <returns></returns>
        public Dictionary<string, string> GetDicCurrency()
        {
            if (_dicCurrency.Count==0)
            {
                Message message = new Message();
                SystemFunction.InitFd(null, message);
                SystemFunction.CommSend(null, message);
                message.Fd16 = "2016";
                var dic = new Dictionary<string, string> ();
                message.FileData = UserFunction.ConvertDictionaryToXML(dic);
                message.TransitNode = TellerSystem.Communication.MessageHelper.TransitType.CallNoFileSys;
                if (message.DoTransit() && message.Fd12 == "0000")
                {
                    DataTable dt = UserFunction.ConvertXMLToDataTable(message.FileData);
                    foreach (DataRow row in dt.Rows)
                    {
                        _dicCurrency.Add(row["CurrencyNo"].ToString(), row["CurrencyName"].ToString());
                    }
                }
            }
            return _dicCurrency;
 
        }
        /// <summary>
        /// 获取券别字典
        /// </summary>
        /// <returns></returns>
        public Dictionary<string, string> GetDicTicketDistinct()
        {
            if (_dicTicketDistinct.Count == 0)
            {
                Message message = new Message();
                SystemFunction.InitFd(null, message);
                SystemFunction.CommSend(null, message);
                message.Fd16 = "2015";
                var dic = new Dictionary<string, string>();
                message.FileData = UserFunction.ConvertDictionaryToXML(dic);
                message.TransitNode = TellerSystem.Communication.MessageHelper.TransitType.CallNoFileSys;
                if (message.DoTransit() && message.Fd12 == "0000")
                {
                    DataTable dt = UserFunction.ConvertXMLToDataTable(message.FileData);
                    foreach (DataRow row in dt.Rows)
                    {
                        _dicTicketDistinct.Add(row["CouponsNo"].ToString(), row["CouponsName"].ToString());
                    }
                }
            }
            return _dicTicketDistinct;
        }

        /// <summary>
        /// 获取机构信息的字典
        /// </summary>
        /// <returns></returns>
        public Dictionary<string, string> GetDicKinbr()
        {
            if (_dicKinbr.Count == 0)
            {
                var entitys = TradeManagerHandle.GetAllKinbrInfo(LoginUserInfo.KinbrNo);
                foreach (var entity in entitys)
                {
                    _dicKinbr.Add(entity.KinbrName, entity.KinbrNo);
                }
            }
            return _dicKinbr;
        }
    }
}
