﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Linq;
using Platform.Common.RunningParameters;
using Platform.Controls.ComponentContainer;
using Platform.Controls.ControlsHelper;
using Platform.Presentation.FocusManagers;
using System.Text.RegularExpressions;
using Platform.Common.BasicFunctions;
using Forms = System.Windows.Forms;
using Platform.Presentation.Interfaces;
using Platform.Presentation.PageFunctions;
using TellerSystem.Library.Ext.Function;
using Platform.Library;
using Microsoft.Reporting.WinForms;
using System.Threading.Tasks;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using Platform.Common.LogSystem;
using System.Diagnostics;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.PrintTemplate.Report;
using TellerSystem.PrintTemplate.Extension;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public static class DataBoxExtension
    {
        #region CONST

        //TODO:行列分隔符，暂时写死
        const char COL_SPLITTER = '|';
        const char ROW_SPLITTER = '\n';
        const char MONEY_SYMBOL = '$';
        const char DATE_SYMBOL = '#';
        const char HEAD_SYMBOL = '~';
        const char LEFT_SYMBOL = '@';

        #endregion CONST

        #region Extension

        public static void InitDataBox(this DataBox dataBox)
        {
            dataBox.DataBoxHost.ItemsSource = null;
            dataBox.DataBoxHost.Columns.Clear();

            RemoveAllEventHandler(dataBox.DataBoxHost, "_dataBindBeginEvent", "DataBindBeginEvent");
            RemoveAllEventHandler(dataBox.DataBoxHost, "_dataBindEndEvent", "DataBindEndEvent");
            RemoveAllEventHandler(dataBox, "_hotkeyControlEvent", "HotkeyControlEvent");
        }

        public static DataTable GetDataForDataTable(this DataBox dataBox)
        {
            return dataBox.DataSource;

        }

        public static string GetDataForString(this DataBox dataBox)
        {
            return GetDataForString(dataBox, null);
        }

        public static string GetDataForString(this DataBox dataBox, string col_splitter)
        {
            if (string.IsNullOrEmpty(col_splitter)) col_splitter = COL_SPLITTER.ToString();
            var dataTable = GetDataForDataTable(dataBox);
            if (dataTable == null) return null;

            StringBuilder fileData = new StringBuilder(
                dataTable.Rows.Count * dataTable.Columns.Count * 10);

            //TODO 暂时不保存FileData

            //如果源数据有列标题，则还以源数据的标题为返回数据的标题
            //if (!string.IsNullOrEmpty(FileData) && FileData.Split(rowSplitter)[0].StartsWith("~"))
            //{
            //    fileData += FileData.Split(rowSplitter)[0] + rowSplitter;
            //}

            //构造数据
            for (var i = 0; i < dataTable.Rows.Count; i++)
            {
                for (var j = 0; j < dataTable.Columns.Count; j++)
                {
                    fileData.Append(dataTable.Rows[i][j].ToString() + col_splitter);
                }

                fileData.Append(ROW_SPLITTER.ToString());
            }
            return fileData.ToString();
        }
        /// <summary>
        /// 解析XML格式的字符串
        /// </summary>
        /// <param name="xmlSource">XML数据源</param>
        /// <param name="nodeName">Node目录名</param>
        /// <returns>返回用|分隔节点，\n分隔行的字符串</returns>
        public static string AnalyzeXMLtoStr(string xmlSource, string nodeName)
        {
            var result = string.Empty;
            if (string.IsNullOrWhiteSpace(xmlSource)) return result;

            System.Xml.XmlDocument objXmlDoc = new System.Xml.XmlDocument();
            try
            {
                objXmlDoc.LoadXml(xmlSource);

                System.Xml.XmlNodeList nodes = objXmlDoc.GetElementsByTagName(nodeName);

                foreach (System.Xml.XmlNode xmlNode in nodes)
                {
                    for (int j = 0; j < xmlNode.ChildNodes.Count; j++)
                    {
                        result += xmlNode.ChildNodes[j].InnerText + "|";
                    }
                    result += "\n";
                }
            }
            catch
            {
                result = string.Empty;
            }
            return result;
        }
        /// <summary>
        /// 绑定DataTable数据源并自动设置列宽
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="dtSource"></param>
        public static void BindDataTable(this DataBox databox, DataTable dtSource)
        {
            //databox.DataBoxHost.DataBindEndEvent -= databox.DataBoxHost.AutoColumnWidth;
            //databox.DataBoxHost.DataBindEndEvent += databox.DataBoxHost.AutoColumnWidth;
            if (databox.IsShowPrintButton)
            {
                //if (!databox.IsCustomPrintOperate)
                //{
                databox.PrintButtonClick -= new RoutedEventHandler(databox_PrintButtonClick);
                databox.PrintButtonClick += new RoutedEventHandler(databox_PrintButtonClick);
                //}
            }
            if (databox.IsShowExportButton)
            {
                databox.ExportButtonClick -= new RoutedEventHandler(databox_ExportButtonClick);
                databox.ExportButtonClick += new RoutedEventHandler(databox_ExportButtonClick);
            }
            databox.DataBoxHost.AutoGenerateColumns = false;
            BuildDataGridColumn(databox, dtSource);
            databox.DataBinding(dtSource);
        }

        static void databox_PrintButtonClick(object sender, RoutedEventArgs e)
        {
            DataBox db = (sender as DependencyObject).FindVisualTreeAncestor(d => d is DataBox) as DataBox;

            if (db != null)
            {
                db.CustomerPrintPreview();
            }
            else
            {
                MessageBox.Show("未找到DataBox对象，无法产生打印对象！");
            }
            //{
            //    //if (db.PrintModel == PrintModel.DirectPrint)
            //    //{

            //    //}
            //    if (true)
            //    {
            //        db.CustomerPrintPreview("测试");
            //        //db.PrintPreview("测试");
            //    }
            //    else
            //    {
            //        db.PrintPreview("");
            //    }
            //}
        }
        static void databox_ExportButtonClick(object sender, RoutedEventArgs e)
        {
            DataBox db = (sender as DependencyObject).FindVisualTreeAncestor(d => d is DataBox) as DataBox;
            if (db != null)
            {
                DataTable table = db.GetDataTableForPrint();
                if (table != null && table.Rows.Count > 0)
                {
                    System.Windows.Forms.SaveFileDialog saveDialog = new System.Windows.Forms.SaveFileDialog();
                    saveDialog.Filter = "Excel工作簿|*.xls";
                    saveDialog.ShowDialog();
                    var outputFile = saveDialog.FileName;
                    if (outputFile.IndexOf(":", System.StringComparison.Ordinal) < 0)
                        return; //被点了取消 
                    var isByNPOI = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("IsByNPOI");
                    if (isByNPOI != null && isByNPOI.ToUpper() == "TRUE")
                    {
                        ExcelFuntion.ExportToExcel(table, outputFile);
                    }
                    else
                    {
                        ExcelFuntion.ExportExcelByDataTable(table, outputFile);
                    }
                }
            }
        }

        /// <summary>
        /// 列宽自动设置 如果总列宽小于控件宽度 则最后一列拉伸
        /// </summary>
        /// <param name="databox"></param>
        public static void AutoColumnWidth(this DataGrid databox)
        {
            double DataActualWidth = databox.RowHeaderActualWidth;

            int columnCount = databox.Columns.Count;
            if (columnCount == 0) return;

            for (int i = 0; i < columnCount; i++)
            {
                DataActualWidth += databox.Columns[i].ActualWidth;
            }

            if (databox.ActualWidth > DataActualWidth)
            {
                databox.Columns[columnCount - 1].Width =
                    //new DataGridLength(1, DataGridLengthUnitType.Star);
                    new DataGridLength(databox.Columns[columnCount - 1].Width.Value + databox.ActualWidth - DataActualWidth);
            }
        }

        /// <summary>
        /// 数据绑定,显示所有数据
        /// </summary>
        /// <param name="databox">DataBox</param>
        /// <param name="fileData">数据源</param>
        /// <param name="columnName">显示指定列名,如果数据源中没有列名,必须显示指定</param>
        public static void DataBinding(this DataBox databox, string fileData, params string[] columnName)
        {
            DataBinding(databox, null, fileData, columnName);
        }

        /// <summary>
        /// 数据绑定,显示所有数据。支持自定义列分隔符
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="separator"></param>
        /// <param name="fileData"></param>
        /// <param name="columnName"></param>
        public static void DataBinding(this DataBox databox, string[] separator, string fileData, params string[] columnName)
        {
            if (columnName.All(str => string.IsNullOrWhiteSpace(str)))
            {
                columnName = null;
            }

            DataBinding(databox, separator, fileData, DisplayLevelEnum.All, columnName);
        }

        /// <summary>
        /// 数据绑定,根据数据显示等级对于数据进行相应处理
        /// </summary>
        /// <param name="databox">DataBox</param>
        /// <param name="fileData">数据源</param>
        /// <param name="level">数据显示等级</param>
        /// <param name="columnNames">显示指定列名,如果数据源中没有列名,必须显示指定</param>
        public static void DataBinding(this DataBox databox, string fileData, DisplayLevelEnum level, params string[] columnNames)
        {
            DataBinding(databox, null, fileData, level, columnNames);
        }

        /// <summary>
        /// 数据绑定,根据数据显示等级对于数据进行相应处理。提供自定义的列分隔符
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="fileData"></param>
        /// <param name="separator"></param>
        /// <param name="level"></param>
        /// <param name="columnNames"></param>
        public static void DataBinding(this DataBox databox, string[] separator, string fileData, DisplayLevelEnum level, params string[] columnNames)
        {
            if (separator == null || separator.Length == 0) separator = new string[] { COL_SPLITTER.ToString() };
            string[] columns = null;
            string[] rows = null;
            DataTable dtResult = null;
            List<int> lstCorrectionIndex = null;

            if (!string.IsNullOrWhiteSpace(fileData))
            {
                //依据原有规则，对每一行的最后分隔符进行过滤
                rows = fileData.Split(new string[] { ROW_SPLITTER.ToString(), "\r\n" }, StringSplitOptions.None).Select(line =>
                {
                    var sp = separator.FirstOrDefault(x => line.EndsWith(x));
                    return (string.IsNullOrEmpty(sp)) ? line : line.Substring(0, line.Length - sp.Length);
                }).ToArray();
                if (rows == null || rows.Length == 0) return;

                //如果第一行为~开头 把第一行的列标题解析出来做列名
                if (rows.First().StartsWith(HEAD_SYMBOL.ToString()))
                {
                    columns = rows.First().TrimStart(HEAD_SYMBOL).Split(separator, StringSplitOptions.None);
                    rows = rows.Skip(1).ToArray();
                }

                if (rows.Length == 0)
                {
                    //没有数据
                }
                else
                {
                    //无法明确其意图，分析为处理最后一行仅为分隔符数据的行置空；改为rows设置时统一处理
                    //if (rows[rows.Length - 1].EndsWith(COL_SPLITTER.ToString()))
                    //{
                    //    rows[rows.Length - 1] = rows[rows.Length - 1].TrimEnd(COL_SPLITTER);
                    //}
                }
            }

            //如果传入columnNames则使用columnNames做列名
            if (columnNames != null && columnNames.Length > 0)
            {
                columns = columnNames;
            }
            else if (columns != null)
            {
                //使用数据中解析的列名
            }
            else
            {
                throw new ArgumentNullException("数据源中没有指定列名时,必须显示指定列名.");
            }

            switch (level)
            {
                case DisplayLevelEnum.All:
                    dtResult = GetAll(columns, rows, separator);
                    break;
                case DisplayLevelEnum.Right:
                    dtResult = GetRight(columns, rows, separator);
                    break;
                case DisplayLevelEnum.Intelligence:
                    var result = GetIntelligence(columns, rows, separator);
                    dtResult = result.Item1;
                    lstCorrectionIndex = result.Item2;
                    break;
                default:
                    throw new Exception("switch (level) default");
            }

            if (dtResult == null) return;

            databox.BindDataTable(dtResult);

            bool flgColumn = columnNames != null && columnNames.Length > 0;

            for (int i = 0; i < dtResult.Columns.Count; i++)
            {
                if ((flgColumn && i >= columns.Length) || columns.Length <= i || string.IsNullOrWhiteSpace(columns[i]))
                {
                    SetColumnsVisible(databox, i, false);
                }
                //更换对齐策略 wangquan 20140621
                //else if (!flgColumn && !columns[i].StartsWith(LEFT_SYMBOL.ToString()))
                //{
                //    SetColumnsAlign(databox, i, TextAlignment.Right);
                //}
                ///金额列也左对齐 edit by jun 20150509
                //else if (dtResult.Rows.Count > 0 && Regex.IsMatch(dtResult.Rows[0][i].ToString(), @"^[0-9,]*\.[0-9]*$"))
                //{
                //    //检索该列数据，是否符合金额的条件
                //    SetColumnsAlign(databox, i, TextAlignment.Right);
                //}
            }
        }

        /// <summary>
        /// 设置列名
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="columnIndex"></param>
        /// <param name="header"></param>
        public static void SetColumnsHeader(this DataBox databox, int columnIndex, string header)
        {
            Action DataBindEndHandler = null;
            DataBindEndHandler = () =>
            {
                int index = GetColumnIndex(databox, columnIndex);
                if (databox.DataBoxHost.Columns.Count <= index) return;

                databox.DataBoxHost.Columns[index].Header = header;
            };

            if (databox.GetIsBinding()) DataBindEndHandler();
            databox.DataBoxHost.DataBindEndEvent += DataBindEndHandler;
        }

        /// <summary>
        /// 设置列宽
        /// 对打印报表列宽度有效，可实现换行
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="columnIndex"></param>
        /// <param name="width"></param>
        public static void SetColumnsWidth(this DataBox databox, int columnIndex, double width)
        {
            Action DataBindEndHandler = null;
            DataBindEndHandler = () =>
            {
                int index = GetColumnIndex(databox, columnIndex);
                if (databox.DataBoxHost.Columns.Count <= index) return;

                databox.DataBoxHost.Columns[index].Width = width;
            };

            if (databox.GetIsBinding()) DataBindEndHandler();
            databox.DataBoxHost.DataBindEndEvent += DataBindEndHandler;
        }

        /// <summary>
        /// 设置列的显示状态
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="columnIndex"></param>
        /// <param name="visible"></param>
        public static void SetColumnsVisible(this DataBox databox, int columnIndex, bool visible)
        {
            Action DataBindEndHandler = null;
            DataBindEndHandler = () =>
            {
                int index = GetColumnIndex(databox, columnIndex);
                if (databox.DataBoxHost.Columns.Count <= index) return;

                databox.DataBoxHost.Columns[index].Visibility = visible ? Visibility.Visible : Visibility.Collapsed;
            };

            if (databox.GetIsBinding()) DataBindEndHandler();
            databox.DataBoxHost.DataBindEndEvent += DataBindEndHandler;
        }

        /// <summary>
        /// 设置列的格式化
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="columnIndex"></param>
        /// <param name="format"></param>
        public static void SetColumnsFormat(this DataBox databox, int columnIndex, string format)
        {
            Action DataBindEndHandler = null;
            DataBindEndHandler = () =>
            {
                int index = GetColumnIndex(databox, columnIndex);
                if (databox.DataBoxHost.Columns.Count <= index) return;

                var column = databox.DataBoxHost.Columns[index] as DataGridTextColumn;
                if (column == null) throw new Exception("只能对文本列进行格式化");

                var tmpColumn = new DataGridTextColumn();

                Binding bind = new Binding();
                bind.Path = ((Binding)column.Binding).Path;
                bind.StringFormat = format;
                tmpColumn.Binding = bind;
                tmpColumn.Header = column.Header;

                databox.DataBoxHost.Columns[index] = tmpColumn;
            };

            if (databox.GetIsBinding()) DataBindEndHandler();
            databox.DataBoxHost.DataBindEndEvent += DataBindEndHandler;
        }

        /// <summary>
        /// 设置列的文本对齐方式
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="columnIndex"></param>
        /// <param name="align"></param>
        public static void SetColumnsAlign(this DataBox databox, int columnIndex, TextAlignment align)
        {
            Action DataBindEndHandler = null;

            DataBindEndHandler = () =>
            {
                int index = GetColumnIndex(databox, columnIndex);
                if (databox.DataBoxHost.Columns.Count <= index) return;

                var column = databox.DataBoxHost.Columns[index] as DataGridBoundColumn;
                if (column == null) throw new Exception("只能对DataGridBoundColumn类型的列进行格式化");

                //if (column.ElementStyle != null)
                //{
                //    column.ElementStyle.Setters.Add(new Setter(FrameworkElement.HorizontalAlignmentProperty, align));
                //}
                Style style = new Style();
                Setter setter = new Setter(FrameworkElement.HorizontalAlignmentProperty, (HorizontalAlignment)
                    (align == TextAlignment.Right ? TextAlignment.Center :
                     align == TextAlignment.Center ? TextAlignment.Right : align));
                style.Setters.Add(setter);

                column.ElementStyle = style;
            };

            if (databox.GetIsBinding()) DataBindEndHandler();
            databox.DataBoxHost.DataBindEndEvent += DataBindEndHandler;
        }

        /// <summary>
        /// 获取当前选择的所有行
        /// </summary>
        /// <param name="databox">DataBox</param>
        /// <returns>当前选择的所有行DataRow集合</returns>
        public static DataRow[] GetSelectMultiRows(this DataBox databox)
        {
            List<DataRow> lst = new List<DataRow>();

            if (!databox.IsMultiSelected)
            {
                lst.Add(databox.GetSelectSingleRow());
                return lst.ToArray();
            }

            DataRowView drv;

            foreach (var index in databox.DataBoxHost.Rows()
                .Where(dgr => DataGridHelper.GetIsChecked(dgr) ?? false)
                .Select(dgr => dgr.GetIndex()))
            {
                drv = databox.DataBoxHost.Items[index] as DataRowView;
                if (drv == null) return null;

                lst.Add(drv.Row);
            }

            return ConvertDataRow(lst.ToArray());

            // return lst.ToArray();
        }

        /// <summary>
        /// 获取选中的行
        /// </summary>
        /// <param name="databox">DataBox</param>
        /// <returns>当前行(CurrentRow)</returns>
        public static DataRow GetSelectSingleRow(this DataBox databox)
        {
            DataRowView drv = databox.DataBoxHost.SelectedItem as DataRowView;
            if (drv == null) return null;

            var row = ConvertDataRow(drv.Row).Single();

            return row;
        }

        /// <summary>
        /// 获取当前行的值
        /// </summary>
        /// <param name="databox">DataBox</param>
        /// <returns>DataRow.ItemArray</returns>
        public static string[] GetSelectRowValue(this DataBox databox)
        {
            DataRow row = GetSelectSingleRow(databox);
            if (row == null) return null;

            return row.ItemArray.Select(obj => obj.ToString()).ToArray();
        }

        /// <summary>
        /// 将指定列名的列转换为ComboBoxColumn
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="colName"></param>
        /// <param name="dic"></param>
        public static void SetSel(this DataBox databox, string colName, Dictionary<string, string> dic)
        {
            if (string.IsNullOrEmpty(colName))
            {
                throw new ArgumentNullException("colName");
            }

            Action DataBindEndHandler = null;

            DataBindEndHandler = () =>
            {
                DataTable dt = databox.GetDataForDataTable();

                if (dt == null || dt.Columns.Count == 0)
                {
                    throw new Exception("DataBox Columns is not exist, 请在绑定后进行SetSel");
                }

                int colIndex = -1;

                for (int i = 0; i < dt.Columns.Count; i++)
                {
                    if (dt.Columns[i].ColumnName == colName)
                    {
                        colIndex = i;
                        break;
                    }
                }

                if (colIndex == -1)
                {
                    throw new ArgumentException("colName is not exist");
                }
                if (databox.IsMultiSelected || databox.IsShowRadioColumn) colIndex++;
                DataGridComboBoxColumn col = new DataGridComboBoxColumn();
                col.ItemsSource = SystemFunction.Observable2Dictionary(dic);
                col.DisplayMemberPath = "Key";
                col.SelectedValuePath = "Value";
                col.SelectedValueBinding = new Binding(colName);
                col.Header = colName;

                if (databox.DataBoxHost.Columns.Count > colIndex)
                    databox.DataBoxHost.Columns[colIndex] = col;
            };

            if (databox.GetIsBinding()) DataBindEndHandler();
            databox.DataBoxHost.DataBindEndEvent += DataBindEndHandler;
        }

        /// <summary>
        /// 将指定列名的列转换为ComboBoxColumn,显示字典的Value
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="colName"></param>
        /// <param name="dic"></param>
        public static void TranslateColumn(this DataBox databox, string colName, Dictionary<string, string> dic)
        {
            if (string.IsNullOrEmpty(colName))
            {
                throw new ArgumentNullException("colName");
            }

            Action DataBindEndHandler = null;

            DataBindEndHandler = () =>
            {
                DataTable dt = databox.GetDataForDataTable();

                if (dt == null || dt.Columns.Count == 0)
                {
                    throw new Exception("DataBox Columns is not exist, 请在绑定后进行SetSel");
                }

                int colIndex = -1;

                for (int i = 0; i < dt.Columns.Count; i++)
                {
                    if (dt.Columns[i].ColumnName == colName)
                    {
                        colIndex = i;
                        break;
                    }
                }

                if (colIndex == -1)
                {
                    throw new ArgumentException("colName is not exist");
                }
                if (databox.IsMultiSelected || databox.IsShowRadioColumn) colIndex++;
                DataGridComboBoxColumn col = new DataGridComboBoxColumn();
                col.ItemsSource = SystemFunction.Observable2Dictionary(dic);
                col.DisplayMemberPath = "Value";
                col.SelectedValuePath = "Key";
                col.SelectedValueBinding = new Binding(colName);
                col.Header = colName;

                if (databox.DataBoxHost.Columns.Count > colIndex)
                    databox.DataBoxHost.Columns[colIndex] = col;
            };

            if (databox.GetIsBinding()) DataBindEndHandler();
            databox.DataBoxHost.DataBindEndEvent += DataBindEndHandler;
        }

        /// <summary>
        /// 添加列
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="column"></param>
        public static void AddColumn(this DataBox databox, DataGridColumn column)
        {
            Action DataBindEndHandler = null;

            DataBindEndHandler = () =>
            {
                databox.DataBoxHost.Columns.Add(column);
            };

            if (databox.GetIsBinding()) DataBindEndHandler();
            databox.DataBoxHost.DataBindEndEvent += DataBindEndHandler;
        }

        /// <summary>
        /// 添加带下划线的列的内容
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="colName"></param>
        /// <param name="click"></param>
        /// <param name="hotkey"></param>
        public static void AddHyperlinkColumn(this DataBox databox, string colName, RoutedEventHandler click, Key hotkey = Key.None)
        {
            DataGridTemplateColumn dgc = new DataGridTemplateColumn();
            dgc.Header = colName;

            DataTemplate dt = new DataTemplate();
            FrameworkElementFactory fefTextBlock = new FrameworkElementFactory(typeof(TextBlock));
            fefTextBlock.SetValue(TextBlock.MarginProperty, new Thickness(5, 0, 5, 0));
            fefTextBlock.SetValue(TextBlock.TextWrappingProperty, TextWrapping.Wrap);
            fefTextBlock.SetValue(TextBlock.TextAlignmentProperty, TextAlignment.Center);
            FrameworkElementFactory fefHyperlink = new FrameworkElementFactory(typeof(Hyperlink));
            fefHyperlink.AddHandler(Hyperlink.ClickEvent, click);

            FrameworkElementFactory fefRun = new FrameworkElementFactory(typeof(Run));
            fefRun.SetValue(Run.TextProperty, hotkey == Key.None ? colName :
                string.Format(colName + "({0})", hotkey.ToString()));

            if (hotkey != Key.None)
            {
                databox.HotkeyControlEvent += e =>
                {
                    if (e.Key == hotkey && databox.DataBoxHost.IsKeyboardFocusWithin)
                    {
                        click(databox.DataBoxHost, e);
                        return false;
                    }

                    return true;
                };
            }

            fefTextBlock.AppendChild(fefHyperlink);
            fefHyperlink.AppendChild(fefRun);

            dt.VisualTree = fefTextBlock;

            dgc.CellTemplate = dt;
            databox.AddColumn(dgc);
        }

        /// <summary>
        /// 添加行
        /// </summary>
        /// <param name="newRow"></param>
        /// <returns></returns>
        public static bool AddRow(this DataGrid databox, DataRow newRow)
        {
            try
            {
                databox.GetTable().Rows.Add(newRow);
            }
            catch (Exception)
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// 获取数据源
        /// </summary>
        /// <param name="databox"></param>
        /// <returns></returns>
        public static DataTable GetTable(this DataGrid databox)
        {
            var dv = databox.ItemsSource as DataView;
            if (dv == null) return null;

            return dv.Table;
        }

        /// <summary>
        /// 添加行
        /// </summary>
        /// <param name="values"></param>
        /// <returns></returns>
        public static bool AddRow(this DataGrid databox, params object[] values)
        {
            try
            {
                databox.GetTable().Rows.Add(values);
            }
            catch (Exception)
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// 修改当前选中行
        /// </summary>
        /// <param name="values"></param>
        /// <returns></returns>
        public static bool UpdateRow(this DataGrid databox, params object[] values)
        {
            DataRowView drv = databox.SelectedItem as DataRowView;
            if (drv == null) return false;

            DataRow row = drv.Row;

            try
            {
                row.BeginEdit();

                for (int i = 0; i < row.ItemArray.Length; i++)
                {
                    row[i] = values[i];
                }

                row.EndEdit();
            }
            catch (Exception)
            {
                row.CancelEdit();
                return false;
            }

            return true;
        }

        #region DataBox操作方法

        /// <summary>
        /// 添加行
        /// </summary>
        /// <param name="newRow"></param>
        /// <returns></returns>
        public static bool AddRow(this DataBox target, DataRow newRow)
        {
            //更新数据源 2013-8-28            
            return target.DataBoxHost.AddRow(newRow);
        }

        /// <summary>
        /// 添加行
        /// </summary>
        /// <param name="values"></param>
        /// <returns></returns>
        public static bool AddRow(this DataBox target, params object[] values)
        {
            //更新数据源 2013-8-28            
            return target.DataBoxHost.AddRow(values);
        }

        /// <summary>
        /// 修改当前选中行
        /// </summary>
        /// <param name="values"></param>
        /// <returns></returns>
        public static bool UpdateRow(this DataBox target, params object[] values)
        {
            return target.DataBoxHost.UpdateRow(values);
        }

        // 不能直接使用 i作为 行index     by slzh
        // /// <summary>
        // /// 删除选中的所有行(多选和单选)
        // /// </summary>
        // /// <returns>布尔型True|False</returns>
        // public static bool DeleteSelectRows(this DataBox target)
        // {
        //     var rows = target.GetSelectMultiRows();
        //     if (rows == null || rows.Length == 0) return false;
        //     DataRow[] newRows = new DataRow[rows.Count()];
        //     DataRowView drv;
        //
        //
        //     for (int i = 0; i < newRows.Count(); i++)
        //     {
        //         drv = target.DataBoxHost.Items[i] as DataRowView;
        //         if (drv == null) return false;
        //         newRows[i] = drv.Row;
        //     }
        //     foreach (var row in newRows)
        //     {
        //         row.Delete();
        //     }
        //     return true;
        // }

        /// <summary>
        /// 删除选中的所有行(多选和单选)
        /// </summary>
        /// <returns>布尔型True|False</returns>
        public static bool DeleteSelectRows(this DataBox databox)
        {
            DataRowView drv;
            if (!databox.IsMultiSelected)
            {
                //databox.GetSelectSingleRow().Delete();
                drv = databox.DataBoxHost.SelectedItem as DataRowView;
                if (drv == null) return true;
                drv.Row.Delete();
                databox.DataSource.AcceptChanges();
                ///临时解决存在分页条时,删除databox记录能够及时刷新的问题
                if (databox.IsShowPageBar)
                {
                    databox.BindDataTable(databox.DataSource);
                }
                return true;
            }

            List<DataRow> lst = new List<DataRow>();

            foreach (var index in databox.DataBoxHost.Rows().Where(dgr => DataGridHelper.GetIsChecked(dgr) ?? false).Select(dgr => dgr.GetIndex()))
            {
                drv = databox.DataBoxHost.Items[index] as DataRowView;
                if (drv == null) return true;
                lst.Add(drv.Row);
            }

            foreach (DataRow r in lst)
            {
                r.Delete();
            }

            databox.DataSource.AcceptChanges();
            ///临时解决存在分页条时,删除databox记录能够及时刷新的问题
            if (databox.IsShowPageBar)
            {
                databox.BindDataTable(databox.DataSource);
            }
            return true;
        }

        #endregion DataBox操作方法

        #region Print

        /// <summary>
        /// 打印列
        /// </summary>
        public static readonly DependencyProperty PrintColumnsProperty = DependencyProperty.RegisterAttached("PrintColumns", typeof(string[]), typeof(DataBoxExtension), new UIPropertyMetadata(null));
        /// <summary>
        /// 页边距，以CM为单位
        /// </summary>
        public static readonly DependencyProperty PageMarginProperty = DependencyProperty.RegisterAttached("PageMargin", typeof(Thickness), typeof(DataBoxExtension), new PropertyMetadata(new Thickness(0, 0, 0, 0.5)));
        /// <summary>
        /// 是否纵向打印
        /// </summary>
        public static readonly DependencyProperty IsVerticalProperty = DependencyProperty.RegisterAttached("IsVertical", typeof(bool), typeof(DataBoxExtension), new PropertyMetadata(false));
        /// <summary>
        /// 设置databox是否纵向打印,true值就竖着打印
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="value"></param>
        public static void SetIsVertical(this DataBox databox, bool value)
        {
            databox.DataBoxHost.SetValue(IsVerticalProperty, value);
        }

        public static void SetPageMargin(this DataBox databox, Thickness value)
        {
            databox.DataBoxHost.SetValue(PageMarginProperty, value);
        }

        public static void SetPrintColumns(this DataBox databox, params string[] columns)
        {
            databox.DataBoxHost.SetValue(PrintColumnsProperty, columns);
        }

        /// <summary>
        /// 是否采用自定义打印
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public static bool GetIsCustomizePrint(DataBox obj)
        {
            return (bool)obj.GetValue(IsCustomizePrintProperty);
        }

        public static void SetIsCustomizePrint(DataBox obj, bool value)
        {
            obj.SetValue(IsCustomizePrintProperty, value);
        }

        // Using a DependencyProperty as the backing store for IsCustomizePrint.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsCustomizePrintProperty =
            DependencyProperty.RegisterAttached("IsCustomizePrint", typeof(bool), typeof(DataBoxExtension), new UIPropertyMetadata(false));

        public static void PrintPreview(this DataBox databox, string title, string topMsg = null, string bottomMsg = null)
        {
            //弹窗之前杀进程
            Process[] processes = Process.GetProcessesByName("Platform.PrintService");
            if (processes.Length > 0)
            {
                foreach (Process process in processes)
                {
                    if (!process.HasExited)
                        process.Kill();
                }
            }
            //是否采用自定义打印
            if (!GetIsCustomizePrint(databox))
                return;
            try
            {
                DataTable dt = GetDataForDataTable(databox);

                if (databox.IsMultiSelected)
                {
                    var rows = databox.GetSelectMultiRows();
                    if (rows != null && rows.Length > 0)
                    {
                        var page = databox.FindVisualTreeAncestor(d => d is IPage) as IPage ?? LogicalTreeHelper.GetChildren(Application.Current.MainWindow).OfType<PageContainer>().FirstOrDefault().CurrentPage;
                        if (page != null)
                        {
                            var result = page.ShowMessageBox("是否只打印选择项？", "打印询问", MessageBoxButton.YesNo);
                            if (result == MessageBoxResult.Yes)
                            {
                                dt = dt.Clone();
                                foreach (var row in rows)
                                {
                                    dt.ImportRow(row);
                                }
                            }
                        }
                    }
                }

                if (dt == null || dt.Rows.Count == 0 || string.IsNullOrEmpty(title))
                {
                    //
                    return;
                }

                foreach (DataColumn col in dt.Columns)
                {
                    if (IsContainsNumberString(col.ColumnName))
                    {
                        string str = RemoveNumber(col.ColumnName);
                        col.ColumnName = RemoveNumber(col.ColumnName);
                    }
                }

                #region plan1
                //Window win = new Window();
                //win.Icon = Application.Current.MainWindow.Icon;
                //win.Title = title;
                //win.WindowState = WindowState.Maximized;
                //win.ShowInTaskbar = false;
                //Microsoft.Reporting.WinForms.ReportViewer rv = new Microsoft.Reporting.WinForms.ReportViewer();
                ////rv.ShowToolBar = false;
                //rv.Dock = Forms.DockStyle.Fill;
                //rv.SetDisplayMode(Microsoft.Reporting.WinForms.DisplayMode.PrintLayout);

                //win.Content = new System.Windows.Forms.Integration.WindowsFormsHost() { Child = rv };

                //using (Stream stream = ReportBuild(databox.DataBoxHost))
                //{
                //    rv.LocalReport.LoadReportDefinition(stream);
                //    rv.LocalReport.DataSources.Add(new Microsoft.Reporting.WinForms.ReportDataSource(
                //        "DataGridReportDataSet", dt));
                //    rv.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                //        "TitleReportParameter", title));

                //    StringBuilder strDate = new StringBuilder(LoginUserInfo.TradeDate);
                //    strDate.Insert(6, "月");
                //    strDate.Insert(4, "年");
                //    strDate.Append("日");
                //    strDate.Insert(0, "打印日期:");
                //    rv.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                //        "TradeDate", strDate.ToString()));
                //    rv.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                //        "TellerName", LoginUserInfo.TellerName));
                //    rv.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                //        "CustomParameterTop", topMsg));
                //    rv.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                //       "CustomParameterBottom", bottomMsg));
                //    rv.RefreshReport();

                //    win.ShowDialog();

                //    string tempPath = Environment.GetEnvironmentVariable("TEMP");
                //    string tmp = Path.Combine(tempPath, "tmp.rdlc");

                //    File.Delete(tmp);
                //}
                #endregion


                #region plan2
                ReportWindow rpt = new ReportWindow();
                List<ReportParameter> list;
                using (Stream stream = ReportBuild(databox.DataBoxHost, out list))
                {
                    rpt.Report.SetDisplayMode(Microsoft.Reporting.WinForms.DisplayMode.PrintLayout);
                    rpt.Report.ZoomMode = Microsoft.Reporting.WinForms.ZoomMode.FullPage;

                    rpt.Report.LocalReport.LoadReportDefinition(stream);
                    //加载后才能设定参数
                    if (list != null && list.Count > 0)
                    {
                        rpt.Report.LocalReport.EnableExternalImages = true;//开启外部图片支持
                        rpt.Report.LocalReport.SetParameters(list);
                    }
                    rpt.Report.LocalReport.DataSources.Add(new Microsoft.Reporting.WinForms.ReportDataSource(
                        "DataGridReportDataSet", dt));
                    rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                        "TitleReportParameter", title));

                    StringBuilder strDate = new StringBuilder(LoginUserInfo.TradeDate);
                    strDate.Insert(6, "月");
                    strDate.Insert(4, "年");
                    strDate.Append("日");
                    strDate.Insert(0, "打印日期:");
                    /// <summary>
                    /// 版本控制标志
                    /// </summary>
                    string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
                    if ((title == "柜员凭证表" || title == "现金管理") && flag == "V9")
                    {
                        StringBuilder strTime = new StringBuilder(TradeHandle.GetServerTime());// 获取服务端的时间  "2020-10-10 14:40:00"
                        strTime.Remove(0, 11);//获取时分秒
                        strTime.Remove(2, 1);
                        strTime.Remove(4, 3);
                        strTime.Insert(2, "时");
                        strTime.Insert(5, "分");
                        strDate.Append(strTime);
                    }
                    if (!((title == "客户账户综合查询" || title == "柜员凭证表" || title == "现金管理") && flag == "V11"))
                    {
                        rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter("Background_Seal", string.Empty));
                    }
                    rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                        "TradeDate", strDate.ToString()));
                    rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                        "TellerName", LoginUserInfo.TellerName));
                    rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                        "CustomParameterTop", topMsg));
                    rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                       "CustomParameterBottom", bottomMsg));
                    rpt.Report.RefreshReport();

                    //var page = databox.FindVisualTreeAncestor(d => d is IPage) as IPage;
                    //page.ShowPage(rpt, title);
                    rpt.ShowDialog();


                    string tempPath = Environment.GetEnvironmentVariable("TEMP");
                    string tmp = Path.Combine(tempPath, "tmp.rdlc");

                    File.Delete(tmp);
                }
                #endregion
            }
            catch
            {
            }
        }

        /// <summary>
        /// 该方法待修改，暂不使用
        /// </summary>
        private static void CustomerPrintPreview(this DataBox databox, string topMsg = null, string bottomMsg = null)
        {
            //弹窗之前杀进程
            Process[] processes = Process.GetProcessesByName("Platform.PrintService");
            if (processes.Length > 0)
            {
                foreach (Process process in processes)
                {
                    if (!process.HasExited)
                        process.Kill();
                }
            }
            //是否采用自定义打印
            if (GetIsCustomizePrint(databox))
                return;
            var page = databox.FindVisualTreeAncestor(d => d is IPage) as IPage ?? LogicalTreeHelper.GetChildren(Application.Current.MainWindow).OfType<PageContainer>().FirstOrDefault().CurrentPage;
            string Titel = string.Empty;
            if (page != null)
            {
                var tradeModel = Platform.Presentation.PagesHelper.PageHelper.GetTradeModel(page as DependencyObject);
                Titel = tradeModel.Name;
            }
            DataTable dt = GetDataTableForPrint(databox);
            // DataTable dt = GetDataForDataTable(databox);
            //string s = GetDataForString(databox);
            //for (int i = 0; i < databox.DataBoxHost.Columns.Count; i++)
            //{
            //    if (databox.DataBoxHost.Columns[i] is DataGridComboBoxColumn)
            //    {
            //        DataGridComboBoxColumn b = (databox.DataBoxHost.Columns[i] as DataGridComboBoxColumn);
            //        IEnumerable<KeyValuePair<string, string>> kvp = b.ItemsSource as IEnumerable<KeyValuePair<string, string>>;
            //        if (dt.Columns.Contains(b.Header.ToString()))
            //        {

            //        }
            //    }
            //}
            if (databox.IsMultiSelected)
            {
                var rows = databox.GetSelectMultiRows();
                if (rows != null && rows.Length > 0)
                {
                    if (page != null)
                    {
                        var result = page.ShowMessageBox("是否只打印选择项？", "打印询问", MessageBoxButton.YesNo);
                        if (result == MessageBoxResult.Yes)
                        {
                            dt = dt.Clone();
                            foreach (var row in rows)
                            {
                                dt.ImportRow(row);
                            }
                        }
                    }
                }
            }

            if (dt == null || dt.Rows.Count == 0)
            {
                //
                MessageBox.Show("数据为空！无法产生打印报表...");
                return;
            }

            foreach (DataColumn col in dt.Columns)
            {
                //@hcy去掉表头的特殊字符
                col.ColumnName = col.ColumnName.Replace("\\", "");
                if (IsContainsNumberString(col.ColumnName))
                {
                    string str = RemoveNumber(col.ColumnName);
                    col.ColumnName = RemoveNumber(col.ColumnName);
                }
            }


            #region plan2
            ReportWindow rpt = new ReportWindow();
            rpt.Report.RenderingComplete += new RenderingCompleteEventHandler(Report_RenderingComplete);
            rpt.Source = databox;
            List<ReportParameter> list;
            using (Stream stream = ReportBuild(databox.DataBoxHost, out list))
            {
                rpt.Report.SetDisplayMode(Microsoft.Reporting.WinForms.DisplayMode.PrintLayout);
                //rpt.Report.ZoomMode = Microsoft.Reporting.WinForms.ZoomMode.FullPage;
                rpt.Report.LocalReport.LoadReportDefinition(stream);
                //加载后才能设定参数
                if (list != null && list.Count > 0)
                {
                    rpt.Report.LocalReport.EnableExternalImages = true;//开启外部图片支持
                    rpt.Report.LocalReport.SetParameters(list);
                }
                rpt.Report.LocalReport.DataSources.Add(new Microsoft.Reporting.WinForms.ReportDataSource(
                    "DataGridReportDataSet", dt));
                rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                    "TitleReportParameter", Titel));
                StringBuilder strDate = new StringBuilder(LoginUserInfo.TradeDate);
                // 齐丰需要增加打印时间  获取服务端的时间  modify by  hulei 20181121 start
                StringBuilder strTime = new StringBuilder(TradeHandle.GetServerTime());// 获取服务端的时间  "2014-07-12 15:07:25"
                strDate.Insert(6, "月");
                strDate.Insert(4, "年");
                strDate.Append("日");
                strDate.Insert(0, "打印日期:");
                rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                    "TradeDate", strDate.ToString()));
                strTime.Remove(0, 11);//获取时分秒
                strTime.Insert(0, "打印时间:");
                rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                 "TradeTime", strTime.ToString()));
                /// <summary>
                /// 版本控制标志
                /// </summary>
                string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
                if (!((Titel == "客户账户综合查询" || Titel == "凭证查询" || Titel == "现金管理") && flag == "V11"))
                {
                    rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter("Background_Seal", string.Empty));
                }
                // 齐丰需要增加打印时间  获取服务端的时间  modify  by  hulei 20181121 end 
                //rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                //    "TellerName", LoginUserInfo.TellerName));
                //rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                //    "CustomParameterTop", topMsg));
                //rpt.Report.LocalReport.SetParameters(new Microsoft.Reporting.WinForms.ReportParameter(
                //   "CustomParameterBottom", bottomMsg));
                rpt.Report.RefreshReport();
                rpt.Report.Tag = rpt;
                _IPage = page;
                //if (false)
                //{
                //    CreateExcel(rpt.Report);
                //}
                //else
                //{
            }
            #endregion
        }
        static IPage _IPage;
        //static ReportWindow rpt = new ReportWindow();
        /// <summary>
        /// 执行打印
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        static void Report_RenderingComplete(object sender, RenderingCompleteEventArgs e)
        {
            ReportViewer rep = sender as ReportViewer;
            var pageSetting = rep.GetPageSettings().Landscape = true;
            //var rpt = rep.Tag as ReportWindow;
            //var deviceQuestion = new Platform.Controls.TradeExtension.PageMessageBox();
            //var result = deviceQuestion.Show(_IPage, "是否打印预览？是：预览 否：直接打印",
            //                      "打印提示", MessageBoxButton.YesNo, MessageBoxImage.Question, MessageBoxResult.Yes);
            //if (result == MessageBoxResult.Yes)
            //{
            //    rpt.ShowDialog();
            //}
            //else
            //{
            rep.PrintDialog();

            //}
            //rpt.Report.RenderingComplete -= new RenderingCompleteEventHandler(Report_RenderingComplete);
            //rpt.Close();
        }

        /// <summary>
        ///创建用于导出或打印的数据源
        /// </summary>
        public static DataTable GetDataTableForPrint(this DataBox databox)
        {
            DataTable data = GetDataForDataTable(databox).Copy();
            Dictionary<string, string> dic = new Dictionary<string, string>();
            var datagrid = databox.DataBoxHost;
            var printcols = datagrid.GetValue(PrintColumnsProperty) as string[];
            List<DataGridColumn> printColumns = datagrid.Columns.Where(
            col => (col is DataGridTextColumn || col is DataGridComboBoxColumn) &&
             col.Visibility == System.Windows.Visibility.Visible && (printcols == null ||
              printcols.Contains(col.Header.ToString()))).ToList<DataGridColumn>();
            foreach (var item in printColumns)
            {
                dic.Add(item.Header.ToString(), item.SortMemberPath.ToString());
            }
            for (int i = 0; i < data.Columns.Count; )
            {
                if (dic.ContainsKey(data.Columns[i].ColumnName) || dic.ContainsValue(data.Columns[i].ColumnName))
                {
                    i++;
                    continue;
                }
                else
                {
                    data.Columns.Remove(data.Columns[i]);
                }
            }
            return data;
        }

        /// <summary>
        /// 将报表导成excel
        /// </summary>
        private static void CreateExcel(ReportViewer report)
        {
            Warning[] warnings;
            string[] streamids;
            string mimeType;
            string encoding;
            string extension;

            byte[] bytes = report.LocalReport.Render(
                "Excel", null, out mimeType, out encoding, out extension,
               out streamids, out warnings);
            var saveDialog = new System.Windows.Forms.SaveFileDialog { FileName = "DataBoxToExcel" };
            System.Windows.Forms.DialogResult result = saveDialog.ShowDialog();
            if (result == System.Windows.Forms.DialogResult.OK)
            {
                FileStream fs = new FileStream(saveDialog.FileName + "." + extension, FileMode.Create);
                fs.Write(bytes, 0, bytes.Length);
                fs.Close();
            }

        }

        private static Stream ReportBuild(System.Windows.Controls.DataGrid dataGrid, out List<ReportParameter> reportParameters)
        {
            var PrintColumns = dataGrid.GetValue(PrintColumnsProperty) as string[];
            var totalWidth = dataGrid.Columns.Where(
                col => (col is DataGridTextColumn || col is DataGridComboBoxColumn) &&
                    col.Visibility == System.Windows.Visibility.Visible &&
                    col.ActualWidth > 0 && (PrintColumns == null ||
                    PrintColumns.Contains(col.Header.ToString()))).Sum(col => GetInByDIU(col.ActualWidth) * 2.54);
            /*
             * 2013-10-16
             * DataBox报表打印增加自动判断横向纵向功能
             * 这里使用厘米为单位，A4纸为21CM * 29.7CM 减去4.49的边距得到横竖最大宽度
             * DIU转IN有误差，可能与字体有关，故增加20%的修正（* 1.2）
             * CM与IN转换为2.54
             * 对于横向依然过宽，给予选择隐藏某些列的方案，暂时未完成
             * 
             * By NanQi
             */
            // string path = @"Report\DataBoxReport\DataGridReportTemplate_Horizontal.rdlc";
            //string path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Report", "DataBoxReport", "DataGridReportTemplate_Vertical.rdlc");
            string path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "RES", "Report", "DataBoxReport", "DataGridReportTemplate_Vertical.rdlc");
            if (!File.Exists(path))
                path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Report", "DataBoxReport", "DataGridReportTemplate_Vertical.rdlc");
            // if ((bool)dataGrid.GetValue(IsVerticalProperty))//手动传参
            //{
            //   path = @"Report\DataBoxReport\DataGridReportTemplate_Vertical.rdlc";
            //}
            //构建Background_Seal参数,若存在则加入返回列表
            reportParameters = new List<ReportParameter>();
            var page = dataGrid.FindVisualTreeAncestor(x => x is IPage) as IPage;
            string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            var codes = "4112";
            if (flag == "V16" && codes.Contains(page.GetPrintAttributes().TradeModel.Code))
            {
                path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "RES", "V16", "Report", "DataBoxReport", "DataGridReportTemplate_Vertical.rdlc");
            }
            ReportParameter reportParameter = ReportEntity.Rdlc2Background_Seal(path
                , PrintContainerExt.GetPrintElement(page as DependencyObject) ?? page.GetPrintAttributes());
            if (reportParameter != null) reportParameters.Add(reportParameter);

            //const double VERTICAL_MAX_WIDTH = 16.51d;//21 - 4.49
            //const double HORIZONTAL_MAX_WIDTH = 25.21d;//29.7 - 4.49

            //if (totalWidth > VERTICAL_MAX_WIDTH && totalWidth <= HORIZONTAL_MAX_WIDTH)
            //{
            //    path = @"Report\DataBoxReport\DataGridReportTemplate_Horizontal.rdlc";
            //}
            //else if (totalWidth > HORIZONTAL_MAX_WIDTH)
            //{
            //    path = @"Report\DataBoxReport\DataGridReportTemplate_Horizontal.rdlc";
            //    //UNDONE 横向仍然过宽，这种情况需要弹出对话框选择隐藏某些列 By NanQi
            //}

            XDocument doc = XDocument.Load(path);

            XNamespace xName = doc.Root.GetDefaultNamespace();
            XNamespace rdName = doc.Root.GetNamespaceOfPrefix("rd");

            #region 设置页边距
            var PageMargin = (Thickness)dataGrid.GetValue(PageMarginProperty);

            var LeftMargin = doc.Descendants(xName + "LeftMargin").Single();
            var RightMargin = doc.Descendants(xName + "RightMargin").Single();
            var TopMargin = doc.Descendants(xName + "TopMargin").Single();
            var BottomMargin = doc.Descendants(xName + "BottomMargin").Single();

            //LeftMargin.Value = PageMargin.Left + "cm";
            //RightMargin.Value = PageMargin.Right + "cm";
            //TopMargin.Value = PageMargin.Top + "cm";
            //BottomMargin.Value = PageMargin.Bottom + "cm";

            #endregion


            var Fields = doc.Descendants(xName + "Fields").Single();
            var Field = Fields.Elements().First();

            var TablixColumns = doc.Descendants(xName + "TablixColumns").Single();
            var TablixCells = doc.Descendants(xName + "TablixCells");

            var TablixColumn = TablixColumns.Descendants(xName + "TablixColumn").Single();
            var cellsColumn = TablixCells.First();
            var cellsData = TablixCells.Skip(1).Single();

            var cellColumn = cellsColumn.Descendants(xName + "TablixCell").Single();
            var cellData = cellsData.Descendants(xName + "TablixCell").Single();

            var TablixColumnHierarchy = doc.Descendants(xName + "TablixColumnHierarchy").Single();
            var TablixMembers = TablixColumnHierarchy.Element(xName + "TablixMembers");
            var TablixMember = TablixMembers.Element(xName + "TablixMember");

            Fields.RemoveAll();
            TablixColumns.RemoveAll();
            cellsColumn.RemoveAll();
            cellsData.RemoveAll();
            TablixMembers.RemoveAll();

            int count = 0;
            foreach (var column in dataGrid.Columns.Where(
                col => (col is DataGridTextColumn || col is DataGridComboBoxColumn) &&
                    col.Visibility == System.Windows.Visibility.Visible &&
                    col.ActualWidth > 0 && (PrintColumns == null ||
                    PrintColumns.Contains(col.Header.ToString()))))
            {
                column.Header = column.Header.ToString().Replace("\\", "");
                //增加TablixColumn,Width
                TablixColumn.Element(xName + "Width").Value = GetInByDIU(column.ActualWidth).ToString("F5") + "in"; ;
                TablixColumns.Add(new XElement(TablixColumn));

                //增加表格的标题,Name,Value
                count++;
                var TextboxColumn = cellColumn.Descendants(xName + "Textbox").Single();
                TextboxColumn.Attribute("Name").Value = "Textbox" + count.ToString();

                var ValueColumn = cellColumn.Descendants(xName + "Value").Single();
                ValueColumn.Value = column.Header.ToString();

                var DefaultNameColumn = cellColumn.Descendants(rdName + "DefaultName").Single();
                DefaultNameColumn.Value = "Textbox" + count.ToString();

                cellsColumn.Add(new XElement(cellColumn));

                string colName = column.SortMemberPath.Replace("\\", "");

                //增加表格的内容
                var TextboxData = cellData.Descendants(xName + "Textbox").Single();


                var ValueData = cellData.Descendants(xName + "Value").Single();
                if (IsContainsNumberString(colName))
                {
                    TextboxData.Attribute("Name").Value = RemoveNumber(colName);
                    ValueData.Value = "=Fields!" + RemoveNumber(colName) + ".Value";
                }
                else
                {
                    TextboxData.Attribute("Name").Value = colName;
                    ValueData.Value = "=Fields!" + colName + ".Value";
                }



                var DefaultNameData = cellData.Descendants(rdName + "DefaultName").Single();
                DefaultNameData.Value = colName;

                cellsData.Add(new XElement(cellData));

                //TablixColumnHierarchy
                TablixMembers.Add(new XElement(TablixMember));

                //Fields
                if (IsContainsNumberString(colName))
                {
                    Field.Attribute("Name").Value = RemoveNumber(colName);
                    Field.Element(xName + "DataField").Value = RemoveNumber(colName);
                }
                else
                {
                    Field.Attribute("Name").Value = colName;
                    Field.Element(xName + "DataField").Value = colName;
                }


                Fields.Add(new XElement(Field));
            }

            var ReportID = doc.Descendants(rdName + "ReportID").Single();
            ReportID.Value = Guid.NewGuid().ToString();

            string tempPath = Environment.GetEnvironmentVariable("TEMP");
            string tmp = Path.Combine(tempPath, "tmp.rdlc");
            //doc.Save(ms);
            doc.Save(tmp);
            FileStream fs = new FileStream(tmp, FileMode.Open);
            return fs;
            //byte[] buffer = Encoding.UTF8.GetBytes(doc.Declaration +
            //    Environment.NewLine + doc.ToString());
            //MemoryStream stream = new MemoryStream(buffer);
            //return stream;
        }

        #endregion Print

        #region Select

        /// <summary>
        /// 设置全选/全不选
        /// </summary>
        /// <param name="databox"></param>
        /// <param name="isCheckedAll"></param>
        public static void SetCheckedAll(this DataBox databox, bool isCheckedAll)
        {
            DataGridHelper.SetIsChecked(databox.DataBoxHost, isCheckedAll);
        }

        #endregion Select

        #endregion Extension

        #region private

        private static void RemoveAllEventHandler(object obj, string eventName, string propEvent)
        {
            FieldInfo field = obj.GetType().GetField(eventName, BindingFlags.NonPublic | BindingFlags.Instance);
            if (field == null) return;

            EventInfo eve = obj.GetType().GetEvent(propEvent);
            if (eve == null) return;

            var fieldValue = field.GetValue(obj);
            if (fieldValue == null) return;

            Delegate dele = fieldValue as Delegate;
            if (dele == null) return;

            dele.GetInvocationList().ToList().ForEach(d =>
            {
                eve.RemoveEventHandler(obj, d);
            });
        }

        private static bool GetIsBinding(this DataBox databox)
        {
            return databox.DataSource != null;
        }

        private static void BuildDataGridColumn(DataBox databox, DataTable dt)
        {
            Action DataBindBeginHandler = GetBuildDataGridColumn(databox);
            databox.DataBoxHost.DataBindBeginEvent -= DataBindBeginHandler;

            DataBindBeginHandler = () =>
            {
                DataGridTextColumn dgc = null;

                foreach (DataColumn column in dt.Columns)
                {
                    dgc = new DataGridTextColumn();
                    dgc.Binding = new Binding(column.ColumnName);
                    dgc.Header = column.ColumnName;

                    databox.DataBoxHost.Columns.Add(dgc);
                }
            };

            SetBuildDataGridColumn(databox, DataBindBeginHandler);
            databox.DataBoxHost.DataBindBeginEvent += DataBindBeginHandler;
        }

        private static int GetColumnIndex(DataBox databox, int columnIndex)
        {
            int index = columnIndex;

            if (databox.DataBoxHost.Columns.Count > 0 &&
                databox.DataBoxHost.Columns[0] is DataGridTemplateColumn &&
                (databox.IsShowRadioColumn || databox.IsMultiSelected))
            {
                index++;
            }

            return index;
        }

        private static double GetInByDIU(double diu)
        {
            return diu / 96 * 1.1;// 1.2;
            //return din.ToString("F5") + "in";
        }

        private static DataTable GetAll(string[] columns, string[] rows, string[] separator)
        {
            if (separator == null || separator.Length == 0) separator = new string[] { COL_SPLITTER.ToString() };
            DataTable dtResult = new DataTable("AllTable");

            #region 添加列

            for (int i = 0; i < columns.Length; i++)
            {
                string col = RedressColumnName(columns[i]);
                DataColumn dataColumn = new DataColumn(col);

                if (col.StartsWith(LEFT_SYMBOL.ToString()))
                {
                    dataColumn.ColumnName = col.TrimStart(LEFT_SYMBOL);
                }
                else if (col.StartsWith(MONEY_SYMBOL.ToString()))
                {
                    dataColumn.ColumnName = col.TrimStart(MONEY_SYMBOL);
                }
                //else if (col.StartsWith(DATE_SYMBOL.ToString()))
                //{
                //    dataColumn.ColumnName = col.TrimStart(DATE_SYMBOL);
                //    dataColumn.DataType = typeof(DateTime);
                //}

                AddColumn(dtResult, dataColumn);
            }

            #endregion 添加列

            if (rows == null) return dtResult;

            #region 添加数据

            string[] strCell;
            for (int i = 0; i < rows.Length; i++)
            {
                var readLine = rows[i];
                if (string.IsNullOrWhiteSpace(readLine)) continue;

                strCell = readLine.Split(separator, StringSplitOptions.None);
                //如果数据的列数不等于DataTable的列数
                if (strCell.Length > dtResult.Columns.Count)
                {
                    int tmpDataColumnCount = dtResult.Columns.Count;
                    for (var j = 0; j < strCell.Length - tmpDataColumnCount; j++)
                    {
                        dtResult.Columns.Add();
                    }
                }

                try
                {
                    dtResult.Rows.Add(strCell);
                }
                catch (Exception e)
                {
                    //MessageBox.Show(e.Message + Environment.NewLine + readLine);
                }
            }

            #endregion 添加数据

            return dtResult;
        }

        private static DataTable GetRight(string[] columns, string[] rows, string[] separator)
        {
            if (separator == null || separator.Length == 0) separator = new string[] { COL_SPLITTER.ToString() };
            DataTable dtResult = new DataTable("RightTable");

            var tmpRows = rows.Select(str => str.Split(separator, StringSplitOptions.None));

            var group = tmpRows.GroupBy(row => row.Length)
                .Select(g => new { g.Key, Count = g.Count() })
                .OrderByDescending(a => a.Count);

            //获取最多的相同的列数,确保能正确显示的数据最多
            int columnCount = group.First().Key;

            //表示有参差不齐的情况
            if (group.Count() > 1)
            {
                //获取最多的相同的列数,确保能正确显示的数据最多
                //筛选掉参差不齐数据
                tmpRows = tmpRows.Where(row => row.Length == columnCount);
            }

            #region 添加列

            if (columns.Length > columnCount)
            {
                columns = columns.Take(columnCount).ToArray();
            }
            else if (columns.Length < columnCount)
            {
                string[] tmpColumns = new string[columnCount].Select(str => string.Empty).ToArray();
                Array.Copy(columns, tmpColumns, columns.Length);

                columns = tmpColumns;
            }
            //保存不能转换为金额的列的索引
            List<int> lstNotMoneyIndex = new List<int>();

            //表示列中有金额
            if (columns.Any(col => col.StartsWith(MONEY_SYMBOL.ToString())))
            {
                //判断数据中对应的列是否都能转换为金额,否则将列置为String
                decimal tmpDec = 0;

                int[] moneyIndex = columns.Select((col, i) =>
                {
                    if (col.StartsWith(MONEY_SYMBOL.ToString())) return (int?)i;
                    else return null;
                }).Where(i => i.HasValue).Select(i => i ?? 0).ToArray();

                foreach (var index in moneyIndex)
                {
                    if (tmpRows.Any(row => index < row.Length && !decimal.TryParse(row[index], out tmpDec))) lstNotMoneyIndex.Add(index);
                }
            }

            for (int i = 0; i < columns.Length; i++)
            {
                string col = RedressColumnName(columns[i]);
                DataColumn dataColumn = new DataColumn(col, typeof(string));

                if (col.StartsWith(LEFT_SYMBOL.ToString()))
                {
                    dataColumn.ColumnName = col.TrimStart(LEFT_SYMBOL);
                }
                else if (col.StartsWith(MONEY_SYMBOL.ToString()))
                {
                    dataColumn.ColumnName = col.TrimStart(MONEY_SYMBOL);
                    if (!lstNotMoneyIndex.Contains(i)) dataColumn.DataType = typeof(decimal);
                }
                //else if (col.StartsWith(DATE_SYMBOL.ToString()))
                //{
                //    dataColumn.ColumnName = col.TrimStart(DATE_SYMBOL);
                //    dataColumn.DataType = typeof(DateTime);
                //}

                AddColumn(dtResult, dataColumn);
            }

            #endregion 添加列

            #region 添加数据

            foreach (var row in tmpRows)
            {
                try
                {
                    dtResult.Rows.Add(row);
                }
                catch (Exception e)
                {
                    //MessageBox.Show(e.Message + Environment.NewLine + string.Join("|", row));
                }
            }

            #endregion 添加数据

            return dtResult;
        }

        private static Tuple<DataTable, List<int>> GetIntelligence(string[] columns, string[] rows, string[] separator)
        {
            if (separator == null || separator.Length == 0) separator = new string[] { COL_SPLITTER.ToString() };
            DataTable dtResult = new DataTable("IntelligenceTable");
            List<int> lstCorrectionIndex = new List<int>();

            var tmpRows = rows.Select(str => str.Split(separator, StringSplitOptions.None));

            var group = tmpRows.GroupBy(row => row.Length)
                .Select(g => new { g.Key, Count = g.Count() })
                .OrderByDescending(a => a.Count);

            //获取最多的相同的列数,确保能正确显示的数据最多
            int columnCount = group.First().Key;

            //获得认定正确的数据 是一个二维数组
            List<string[]> finalRows = tmpRows.Where(row => row.Length == columnCount).ToList();//表示最终认定的数据

            //表示有参差不齐的情况 进行数据矫正
            if (group.Count() > 1)
            {
                int[][] arrCellLength = new int[columnCount][];

                var tmpArr1 = finalRows.Select(row => row.Select(str => str.Length).ToArray()).ToArray();
                int[][] tmpArr2 = new int[columnCount][];

                for (int i = 0; i < columnCount; i++)
                {
                    int[] tmpValue = new int[tmpArr1.Length];

                    for (int j = 0; j < tmpArr1.Length; j++)
                    {
                        tmpValue[j] = tmpArr1[j][i];
                    }

                    tmpArr2[i] = tmpValue;
                }

                for (int i = 0; i < columnCount; i++)
                {
                    var tmpGroup = tmpArr2[i].GroupBy(length => length).Select(g => new { g.Key, Count = g.Count() })
                        .OrderByDescending(a => a.Count);
                    arrCellLength[i] = tmpGroup.Where(a => a.Count > tmpArr1.Length / tmpGroup.Count() / 2).Select(a => a.Key).ToArray();
                }

                var lstRows = tmpRows.ToList();
                for (int i = 0; i < lstRows.Count; i++)
                {
                    string[] tmpRow = new string[columnCount];//临时行
                    int offset = 0;
                    string[] row = lstRows[i];
                    //某行的列数大于认定的列数,则认为其数据中有|
                    if (row.Length > columnCount)
                    {
                        //标示是否长度已经相等
                        bool flg = false;

                        for (int j = 0; j < columnCount; j++)
                        {
                            //没有内容或长度相等或大于
                            if (flg || string.IsNullOrWhiteSpace(row[j]) ||
                                arrCellLength[j].Contains(row[j].Length) ||
                                arrCellLength[j].Any(cell => row[j].Length > cell))
                            {
                                tmpRow[j] = row[j + offset];
                            }
                            else
                            {
                                //TODO 在此对单元格进行特殊标注,或者在下面对一行数据进行特殊标注
                                tmpRow[j] = row[j + offset] + row[j + offset + 1];
                                offset++;

                                if (row.Length - columnCount == offset)
                                {
                                    flg = true;
                                }
                            }
                        }

                        lstCorrectionIndex.Add(finalRows.Count);
                        finalRows.Add(tmpRow);
                    }
                    //某行的列数小于认定的列数,则认为其数据中有\n
                    else if (row.Length < columnCount)
                    {
                        //有下一行并且下一行列数不正确
                        if (i + 1 < lstRows.Count && lstRows[i + 1].Length != columnCount)
                        {
                            lstRows[i + 1] = lstRows[i].Concat(lstRows[i + 1]).ToArray();
                            continue;
                        }
                    }
                }

                //列数过多的数据已经矫正并且标注
            }

            #region 添加列

            if (columns.Length > columnCount)
            {
                columns = columns.Take(columnCount).ToArray();
            }
            else if (columns.Length < columnCount)
            {
                string[] tmpColumns = new string[columnCount].Select(str => string.Empty).ToArray();
                Array.Copy(columns, tmpColumns, columns.Length);

                columns = tmpColumns;
            }
            //保存不能转换为金额的列的索引
            List<int> lstNotMoneyIndex = new List<int>();

            //表示列中有金额
            if (columns.Any(col => col.StartsWith(MONEY_SYMBOL.ToString())))
            {
                //判断数据中对应的列是否都能转换为金额,否则将列置为String
                decimal tmpDec = 0;

                int[] moneyIndex = columns.Select((col, i) =>
                {
                    if (col.StartsWith(MONEY_SYMBOL.ToString())) return (int?)i;
                    else return null;
                }).Where(i => i.HasValue).Select(i => i ?? 0).ToArray();

                foreach (var index in moneyIndex)
                {
                    if (finalRows.Any(row => index < row.Length && !decimal.TryParse(row[index], out tmpDec))) lstNotMoneyIndex.Add(index);
                }
            }

            for (int i = 0; i < columns.Length; i++)
            {
                string col = RedressColumnName(columns[i]);
                DataColumn dataColumn = new DataColumn(col, typeof(string));

                if (col.StartsWith(LEFT_SYMBOL.ToString()))
                {
                    dataColumn.ColumnName = col.TrimStart(LEFT_SYMBOL);
                }
                else if (col.StartsWith(MONEY_SYMBOL.ToString()))
                {
                    dataColumn.ColumnName = col.TrimStart(MONEY_SYMBOL);
                    if (!lstNotMoneyIndex.Contains(i)) dataColumn.DataType = typeof(decimal);
                }
                //else if (col.StartsWith(DATE_SYMBOL.ToString()))
                //{
                //    dataColumn.ColumnName = col.TrimStart(DATE_SYMBOL);
                //    dataColumn.DataType = typeof(DateTime);
                //}

                AddColumn(dtResult, dataColumn);
            }

            #endregion 添加列

            #region 添加数据

            foreach (var row in finalRows)
            {
                try
                {
                    dtResult.Rows.Add(row);
                }
                catch (Exception e)
                {
                    //MessageBox.Show(e.Message + Environment.NewLine + string.Join("|", row));
                }
            }

            #endregion 添加数据

            return new Tuple<DataTable, List<int>>(dtResult, lstCorrectionIndex);
        }

        private static string RedressColumnName(string colName)
        {
            if (colName.StartsWith(LEFT_SYMBOL.ToString()))
            {
                return LEFT_SYMBOL.ToString() + colName.TrimStart(LEFT_SYMBOL).Replace('/', '\\').Trim();
            }
            else if (colName.StartsWith(MONEY_SYMBOL.ToString()))
            {
                return MONEY_SYMBOL.ToString() + colName.TrimStart(MONEY_SYMBOL).Replace('/', '\\').Trim();
            }
            else
            {
                return colName.Replace('/', '\\').Trim();
            }
        }

        private static void AddColumn(DataTable dt, DataColumn dataColumn)
        {
            int errorRename = 0;

            string tmpColumnName = dataColumn.ColumnName;

            //如果有重名,循环重命名
            while (true)
            {
                try
                {
                    dt.Columns.Add(dataColumn);
                    break;
                }
                catch (DuplicateNameException)
                {
                    errorRename++;
                    dataColumn.ColumnName = tmpColumnName + "_" + errorRename.ToString();
                }
                catch (Exception)
                {
                }
            }
        }

        private static DataRow[] ConvertDataRow(params DataRow[] dataRow)
        {
            if (dataRow == null || dataRow.Length == 0) return null;

            var table = dataRow[0].Table.Clone();

            var table_new = new DataTable("new table");

            table_new.Columns.Add("__No__", typeof(int));

            int columnCount = table.Columns.Count;

            for (int i = 0; i < columnCount; i++)
            {
                table_new.Columns.Add
                (
                    table.Columns[i].ColumnName,
                    table.Columns[i].DataType
                );
            }

            for (int i = 0; i < dataRow.Length; i++)
            {
                var row = table_new.NewRow();
                row[0] = i + 1;

                for (int j = 0; j < columnCount; j++)
                {
                    row[j + 1] = dataRow[i][j];
                }

                table_new.Rows.Add(row);
            }

            return table_new.Rows.Cast<DataRow>().ToArray();
        }

        #endregion private

        #region Attached

        private static Action GetBuildDataGridColumn(DependencyObject obj)
        {
            return (Action)obj.GetValue(BuildDataGridColumnProperty);
        }

        private static void SetBuildDataGridColumn(DependencyObject obj, Action value)
        {
            obj.SetValue(BuildDataGridColumnProperty, value);
        }

        private static readonly DependencyProperty BuildDataGridColumnProperty =
            DependencyProperty.RegisterAttached("BuildDataGridColumn", typeof(Action), typeof(DataBoxExtension), new UIPropertyMetadata(null));

        #endregion Attached

        #region 判断是否包含数字的正则表达式
        public static bool IsContainsNumberString(string str)
        {
            if (str == null || str == "") return false;
            Regex regNum = new Regex("^[0-9]");

            if (regNum.IsMatch(str))
            {
                return true;
            }
            else
            {
                return false;
            }
        }



        /// <summary>  
        /// 去掉字符串中的数字  
        /// </summary>  
        /// <param name="key"></param>  
        /// <returns></returns>  
        public static string RemoveNumber(string key)
        {
            return Regex.Replace(key, @"\d", "");
        }

        #endregion
    }

    /// <summary>
    /// 数据显示级别 请根据实际情况进行选择
    /// </summary>
    public enum DisplayLevelEnum
    {
        /// <summary>
        /// 所有全部都显示
        /// </summary>
        All,
        /// <summary>
        /// 只显示正确的,对于列数不符合的数据不予显示
        /// </summary>
        Right,
        /// <summary>
        /// 智能矫正显示数据,能显示所有数据,对于矫正的数据会进行标注(暂时出现在最下面)
        /// </summary>
        Intelligence
    }
}