﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Office.Interop.Excel;
using Microsoft.Office.Interop;
using System.Diagnostics;
using System.Data;
using System.IO;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class ExcelToTxt
    {
        #region Feilds
        private Stopwatch wath = new Stopwatch();

        /// <summary>
        /// 转换后的文件文本内容
        /// </summary>
        private string changedFileContant = string.Empty;

        private bool isHasTotalMoneyAndTotalCout = false;

        private List<string> tableColumNames = new List<string>();

        private string sourceFileFullName = string.Empty;

        private System.Data.DataTable tableTypeData;
        #endregion

        #region Properties

        /// <summary>
        /// 总金额
        /// </summary>
        public double TotalMoney
        {
            get;
            set;
        }

        /// <summary>
        /// 总笔数
        /// </summary>
        public int TotalCount
        {
            get;
            set;
        }

        /// <summary>
        /// 文件转换后表格的所有列名称
        /// </summary>
        public List<string> TableColumNames
        {
            get
            {
                if (this.tableColumNames.Count == 0)
                {
                    foreach (DataColumn dc in this.TableTypeData.Columns)
                    {
                        this.tableColumNames.Add(dc.ColumnName);
                    }
                }
                return this.tableColumNames;
            }
            set
            {
                if (value.Count > 0)
                {
                    foreach (string colName in value)
                    {
                        if (!this.TableTypeData.Columns.Contains(colName))
                        {
                            this.TableTypeData.Columns.Add(new DataColumn(colName));
                        }
                    }
                }
            }
        }

        /// <summary>
        /// 目标文件头
        /// </summary>
        public string TargetFileHead { get; set; }

        /// <summary>
        /// 目标文件尾
        /// </summary>
        public string TargetFileEnd { get; set; }

        /// <summary>
        /// 数据的表格表现形式
        /// </summary>
        public System.Data.DataTable TableTypeData
        {
            get
            {
                return this.tableTypeData;
            }
            private set
            {
                this.tableTypeData = value;
                if (value.Columns.Count == 0 && this.TableColumNames.Count > 0)
                {
                    foreach (string colName in this.TableColumNames)
                    {
                        this.TableTypeData.Columns.Add(new DataColumn(colName));
                    }
                }
            }
        }

        /// <summary>
        /// 源文件类型
        /// </summary>
        public FileType SourceFileType
        {
            get;
            private set;
        }

        /// <summary>
        /// 源文件名称
        /// </summary>
        public string SourceFileName
        {
            get { return this.SourceFileFullName.Split("\\".ToCharArray()).Last(); }
        }

        /// <summary>
        /// 转换后的文件名称
        /// </summary>
        public string TargetFileName
        {
            get { return this.TargetFileFullName.Split("\\".ToCharArray()).Last(); }
        }

        /// <summary>
        /// 源文件（转换前的excel或者txt的完整名称）
        /// </summary>
        public string SourceFileFullName
        {
            get { return this.sourceFileFullName; }
            set
            {
                this.sourceFileFullName = value;
                if (value.ToLower().EndsWith(".txt"))
                {
                    this.SourceFileType = FileType.TXT;
                }
                else
                {
                    this.SourceFileType = FileType.EXCEL;
                }
            }
        }

        /// <summary>
        /// 目标文件（转换后的txt的完整名称）
        /// </summary>
        public string TargetFileFullName
        {
            get;
            set;
        }

        /// <summary>
        /// 转换后的文件文本内容
        /// </summary>
        public string ChangedFileContant
        {
            get
            {
                if (this.isHasTotalMoneyAndTotalCout)
                {
                    return this.TargetFileHead + this.changedFileContant + this.TargetFileEnd;
                }
                else
                {
                    return string.Format(this.TargetFileHead + "\n" + this.changedFileContant + this.TargetFileEnd, this.TotalCount.ToString(), this.TotalMoney.ToString());
                }
            }
        }
        #endregion

        public ExcelToTxt(string sourceFile, string targetFile) : this()
        {
            this.SourceFileFullName = sourceFile;
            this.TargetFileFullName = targetFile;
        }

        public ExcelToTxt()
        {
            this.TableTypeData = new System.Data.DataTable();
            this.TargetFileHead = "TOTAL NO ";
            this.TargetFileEnd = "\nEND";
        }

        public bool ChangeExcelToTxt()
        {
            if (string.IsNullOrEmpty(this.SourceFileFullName))
            {
                throw new ArgumentException("找不到指定路径:" + this.SourceFileFullName + "的一部分。", "SourceFile");
            }
            return this.ChangeExcelToTxt(this.SourceFileFullName);
        }

        /// <summary>
        /// 将txt转换成指定的格式
        /// </summary>
        /// <param name="fileFullName">需转换的文件路径</param>
        /// <param name="changedFileContant">转换后的文本内容</param>
        /// <returns>转换是否成功</returns>
        public bool ChangeExcelToTxt(string fileFullName)
        {
            return ChangeExcelToTxt(fileFullName, 0, 0, false, 1);
        }

        /// <summary>
        /// 将excel文件转换成指定的txt格式
        /// </summary>
        /// <param name="filePath">需转换的文件路径</param>
        /// <param name="changedFileContant">转换后的文本内容</param>
        /// <param name="headLineCount">sheet中表头的行数</param>
        /// <param name="needName">是否需要表中的名称</param>
        /// <param name="beginLine">表开始的行数，即就是表中名称的行标(从1开始)</param>
        /// <returns>转换是否成功</returns>
        public bool ChangeExcelToTxt(string filePath, int headLineCount, int endLineCount, bool hasColumnTitle, int beginLine)
        {
            bool result = false;
            this.SourceFileFullName = filePath;
            if (string.IsNullOrEmpty(this.SourceFileFullName))
            {
                throw new ArgumentException("找不到指定路径:" + this.SourceFileFullName + "的一部分。", "SourceFile");
            }
            try
            {
                if (this.SourceFileFullName.EndsWith(".xls") || this.SourceFileFullName.EndsWith(".xlsx"))
                {
                    this.changedFileContant = this.ExcelToText(filePath, headLineCount, endLineCount, hasColumnTitle, beginLine);
                }
            }
            catch (Exception)
            {
                changedFileContant = string.Empty;
            }
            return result;
        }

        /// <summary>
        /// 只处理了第一个sheet页，如果有更多sheet页需要做调整  
        /// </summary>
        /// <param name="filePath">文件路径</param>
        /// <param name="headLineCount">sheet中表头的行数</param> 
        /// <param name="needName">是否需要表中的名称</param>
        /// <param name="beginLine">表开始的行数，即就是表中名称的行标</param>
        private string ExcelToText(string filePath, int headLineCount, int endLineCount, bool hasColumnTitle, int beginLine)
        {
            isHasTotalMoneyAndTotalCout = true;
            string head = string.Empty;
            string data = string.Empty;
            this.TableTypeData.Clear();
            Microsoft.Office.Interop.Excel.Application app = new Microsoft.Office.Interop.Excel.Application();
            Sheets sheets;
            Workbook workbook = null;
            object oMissiong = System.Reflection.Missing.Value;
            System.Data.DataTable dt = new System.Data.DataTable();

            //当前行索引
            int currentRowIndex = 0;
            //当前列索引
            int currentColIndex = 0;
            wath.Start();

            try
            {
                if (app == null)
                {
                    return head + data;
                }
                workbook = app.Workbooks.Open(filePath, oMissiong, oMissiong, oMissiong, oMissiong, oMissiong, oMissiong, oMissiong, oMissiong, oMissiong, oMissiong, oMissiong, oMissiong, oMissiong, oMissiong);
                sheets = workbook.Worksheets;
                Worksheet worksheet = (Worksheet)sheets.get_Item(1);//读取第一张表   
                //总行数
                int rowCount = worksheet.UsedRange.Rows.Count;
                if (worksheet == null)
                    return head + data;

                #region 处理表头
                for (int i = beginLine; i < beginLine + headLineCount; i++)
                {
                    currentRowIndex = i;
                    foreach (Range rc in worksheet.UsedRange.Rows[i].Columns)
                    {
                        currentColIndex++;
                        var cells = ((dynamic)rc).Cells;
                        if (cells != null && !string.IsNullOrEmpty(cells.Text.Trim()))
                        {
                            if (cells.Text.Contains("："))
                            {
                                continue;
                            }
                            head += cells.Text.Trim() + " ";
                        }
                    }
                }
                #endregion

                #region 处理表中数据内容
                for (int i = beginLine + headLineCount; i <= rowCount - endLineCount; i++)
                {
                    string tempData = string.Empty;
                    currentColIndex = 0;
                    currentRowIndex = i;
                    if (hasColumnTitle)
                    {
                        foreach (Range rc in worksheet.UsedRange.Rows[i].Columns)
                        {
                            currentColIndex++;
                            var cells = ((dynamic)rc).Cells;
                            if (cells != null && !string.IsNullOrEmpty(cells.Text.Trim()))
                            {
                                this.TableTypeData.Columns.Add(cells.Text.Trim());
                            }
                        }
                        hasColumnTitle = false;
                        continue;
                    }
                    //处理表里面的数据内容
                    #region Data
                    System.Data.DataRow dr = this.TableTypeData.NewRow();
                    bool isAddRow = false;
                    #region
                    foreach (Range rc in worksheet.UsedRange.Rows[i].Columns)
                    {
                        currentColIndex++;
                        string temp = string.Empty;
                        var cells = ((dynamic)rc).Cells;
                        if (cells != null && !string.IsNullOrEmpty(cells.Text.Trim()))
                        {
                            string t = temp = cells.Text;
                            DateTime dateTime;
                            if ((t.Contains(":") || t.Contains("-") || t.Contains("/")) && DateTime.TryParse(t, out dateTime))
                            {
                                if (dateTime.Hour != 0)
                                {
                                    t = dateTime.ToString("hhmmss");
                                    temp = dateTime.ToString("hh:mm:ss");
                                }
                                else
                                {
                                    t = dateTime.ToString("yyyyMMdd");
                                    temp = dateTime.ToString("yyyy-MM-dd");
                                }
                            }
                            tempData += t.Trim() + " ";
                            dr[currentColIndex - 1] = temp;
                            isAddRow = true;
                        }
                    }
                    if (isAddRow)
                    {
                        this.TableTypeData.Rows.Add(dr);
                    }
                    #endregion
                    if (!string.IsNullOrEmpty(tempData.Trim()))
                    {
                        data += "\n"+tempData.TrimEnd(' ');
                    }
                    #endregion
                }
                #endregion

                #region 处理表尾
                for (int i = rowCount - endLineCount + 1; i <= rowCount; i++)
                {
                    currentColIndex = 0;
                    currentRowIndex = i;
                    string temp = string.Empty;
                    foreach (Range rc in worksheet.UsedRange.Rows[i].Columns)
                    {
                        currentColIndex++;
                        var cells = ((dynamic)rc).Cells;
                        if (cells != null && !string.IsNullOrEmpty(cells.Text.Trim()))
                        {
                            if (cells.Text.Contains("："))
                            {
                                continue;
                            }
                            temp += cells.Text.Trim() + " ";
                        }
                    }
                    head = temp + head;
                }
                #endregion

                wath.Stop();
                TimeSpan ts = wath.Elapsed;
                return head + data;
            }
            catch (Exception e)
            {
                throw new Exception(string.Format("‘{0}’行，‘{1}’列初数据转换错误。", currentRowIndex,currentColIndex));
            }
            finally
            {
                workbook.Close(false, oMissiong, oMissiong);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(workbook);
                workbook = null;
                app.Workbooks.Close();
                app.Quit();
                System.Runtime.InteropServices.Marshal.ReleaseComObject(app);
                app = null;
                GC.Collect();
                GC.WaitForPendingFinalizers();
            }
        }

        public bool Save()
        {
            if (string.IsNullOrEmpty(this.TargetFileFullName))
            {
                throw new ArgumentException("找不到指定路径:" + this.TargetFileFullName + "的一部分。", "TargetFile");
            }
            return this.Save(this.TargetFileFullName, this.ChangedFileContant);
        }

        public bool Save(string targetFile)
        {
            if (string.IsNullOrEmpty(targetFile))
            {
                throw new ArgumentException("找不到指定路径:" + targetFile + "的一部分。", "TargetFile");
            }
            return this.Save(targetFile, this.ChangedFileContant);
        }

        /// <summary>
        /// 保存成*.txt文件件
        /// </summary>
        /// <param name="targetFile">保存文件名称</param>
        public bool Save(string targetFile, string fileContant)
        {
            this.TargetFileFullName = targetFile;
            using (FileStream fs = File.OpenWrite(targetFile))
            {
                byte[] bt = Encoding.Default.GetBytes(fileContant);
                fs.Write(bt, 0, bt.Count());
                fs.Flush();
                fs.Close();
            }
            return true;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sourceFile">原文件名</param>
        /// <param name="countColumnName">需要计算的列名（如果没有需要计算的列，传“”或者String.Empty）</param>
        public void TxtToDataTableColumnName(string sourceFile, string countColumnName)
        {
            string temp = string.Empty;
            temp = File.ReadAllText(sourceFile, Encoding.Default);
            temp = temp.Replace("\r\n", "\n");
            this.TargetFileHead = (this.isHasTotalMoneyAndTotalCout = temp.StartsWith("TOTAL")) ? "" : this.TargetFileHead + "{0} {1}";
            this.changedFileContant = temp.Replace('^', ' ');
            this.ToDataTable(" ", countColumnName);
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sourceFile">原文件名</param>
        /// <param name="countColumnName">需要计算的列名（如果没有需要计算的列，传“”或者String.Empty）</param>
        /// <param name="splitChar">txt文件的分隔符</param>
        public void TxtToDataTableColumnName(string sourceFile, string countColumnName,string splitChar)
        {
            string temp = string.Empty;
            temp = File.ReadAllText(sourceFile, Encoding.Default);
            temp = temp.Replace("\r\n", "\n");
            this.TargetFileHead = (this.isHasTotalMoneyAndTotalCout = temp.StartsWith("TOTAL")) ? "" : this.TargetFileHead + "{0} {1}";
            this.changedFileContant = temp.Replace('^', ' ');
            this.ToDataTable(splitChar, countColumnName);
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sourceFile">原文件名</param>
        /// <param name="countColumnName">需要计算的列名（如果没有需要计算的列，传“”或者String.Empty）</param>
        /// /// <param name="t">是否忽略文件第一行</param>
        public void TxtToDataTableColumnName(string sourceFile, string countColumnName,bool t)
        {
            string temp = string.Empty;
            temp = File.ReadAllText(sourceFile, Encoding.Default);
            temp = temp.Replace("\r\n", "\n");
            this.TargetFileHead = (this.isHasTotalMoneyAndTotalCout = temp.StartsWith("TOTAL")) ? "" : this.TargetFileHead + "{0} {1}";
            this.changedFileContant = temp.Replace('^', ' ');
            this.ToDataTable(" ", countColumnName,t);
        }

        /// <summary>
        /// 专门对应txt格式s文件
        /// </summary>
        /// <param name="sourceFile">源文件名称</param>
        /// <param name="count">总笔数</param>
        /// <param name="totalMoney">总金额</param>
        /// <param name="beginLine">开始行</param>
        /// <param name="splitChar">分割字符</param>
        public void TxtToDataTable(string sourceFile)
        {
            this.TotalCount = 0;
            List<string> needLines = new List<string>();
            string[] lines = File.ReadAllLines(sourceFile, Encoding.Default);
            for (int i = 0; i < lines.Length; i++)
            {
                if (lines[i].StartsWith("U"))
                {
                    this.TotalCount++;
                    needLines.Add(lines[i]);
                    this.changedFileContant += lines[i] + "\n";
                }
            }
            this.TargetFileHead += "{0} {1} ";
            this.changedFileContant = this.changedFileContant.Trim("\n".ToCharArray()).Replace('^', ' ');
            this.ToDataTable(" ", string.Empty);
        }

        /// <summary>
        /// 对于批量返回的txt文件
        /// </summary>
        /// <param name="sourceFile">源文件名称</param>
        /// <param name="count">总笔数</param>
        /// <param name="totalMoney">总金额</param>
        /// <param name="splitChar">分割字符</param>
        public void TxtToDataTable(string sourceFile, string splitChar)
        {
            this.changedFileContant = File.ReadAllText(sourceFile, Encoding.Default);
            this.ToDataTable(splitChar, string.Empty);
        }

        private void ToDataTable(string splitChar, string countColumn)
        {
            string[] strs = this.ChangedFileContant.Split('\n');
            int i = 0;
            //是否计算总笔数
            bool needAddTotalCount = false;
            //是否计算总金额
            bool needCountTotalMoney = false;
            if (strs != null)
            {
                this.TableTypeData.Clear();
                foreach (string item in strs)
                {   
                    if (item.StartsWith("END"))
                    {
                        break;
                    }
                    if (string.IsNullOrEmpty(item.Trim()))
                    {
                        continue;  
                    }
                    if (item=='\n'.ToString())
                    {
                        continue;
                    }
                    string[] strT = item.Split(splitChar.ToCharArray(),StringSplitOptions.None);
                    if (strT == null || strT.Length < 1)
                    {
                        throw new FileFormatException("不可处理的文件格式。");
                    }
                    if (i == 0)
                    {
                        this.TargetFileHead = strT[0] + " " + strT[1] + " {0} {1} ";
                        this.TotalCount = Convert.ToInt32(strT[2]);
                        needAddTotalCount = this.TotalCount > 0 ? false : true;
                        this.TotalMoney = Convert.ToDouble(strT[3]);
                        needCountTotalMoney = this.TotalMoney > 0 ? false : true;
                        if (strT.Length > 4)
                        {
                            for (int j = 4; j < strT.Length; j++)
                            {
                                this.TargetFileHead += strT[j];
                            }
                        }
                    }
                    else
                    {
                        DataRow dr = this.TableTypeData.NewRow();
                        for (int j = 0; j < strT.Length; j++)
                        {
                            dr[j] = strT[j];
                        }
                        if (!string.IsNullOrEmpty(countColumn) && needCountTotalMoney)
                        {
                            if (!Convert.IsDBNull(dr[countColumn])&&dr[countColumn]!=null)
                            {
                                this.TotalMoney += Convert.ToDouble(dr[countColumn]);
                            }
                            
                        }   

                        this.TotalCount = needAddTotalCount? this.TotalCount+1:this.TotalCount;
                        this.TableTypeData.Rows.Add(dr);
                    }
                    i++;
                }
            }
            else
            {
                throw new FileFormatException("不可处理的文件格式。");
            }

        }

        private void ToDataTable(string splitChar, string countColumn,bool t)
        {
            string[] strs = this.ChangedFileContant.Split('\n');
            int i = 0;
            //是否计算总笔数
            bool needAddTotalCount = false;
            //是否计算总金额
            bool needCountTotalMoney = false;
            if (strs != null)
            {
                this.TableTypeData.Clear();
                foreach (string item in strs)
                {
                    if (item.StartsWith("END"))
                    {
                        break;
                    }
                    if (string.IsNullOrEmpty(item.Trim()))
                    {
                        continue;
                    }
                    if (!t&&item.StartsWith("TOTAL"))
                    {
                        continue;
                    }
                    string[] strT = item.Split(splitChar.ToCharArray(), StringSplitOptions.None);
                    if (strT == null || strT.Length < 1)
                    {
                        throw new FileFormatException("不可处理的文件格式。");
                    }
                    if (t)
                    {
                        if (t)
                        {
                            this.TargetFileHead = strT[0] + " " + strT[1] + " {0} {1} ";
                            this.TotalCount = Convert.ToInt32(strT[2]);
                            needAddTotalCount = this.TotalCount > 0 ? false : true;
                            this.TotalMoney = Convert.ToDouble(strT[3]);
                            needCountTotalMoney = this.TotalMoney > 0 ? false : true;
                        }
                        if (strT.Length > 4)
                        {
                            for (int j = 4; j < strT.Length; j++)
                            {
                                this.TargetFileHead += strT[j];
                            }
                        }
                    }
                    else
                    {
                        DataRow dr = this.TableTypeData.NewRow();
                        for (int j = 0; j < strT.Length; j++)
                        {
                            dr[j] = strT[j];
                        }
                        if (!string.IsNullOrEmpty(countColumn) && needCountTotalMoney)
                        {
                            this.TotalMoney += Convert.ToDouble(dr[countColumn]);
                        }

                        this.TotalCount = needAddTotalCount ? this.TotalCount + 1 : this.TotalCount;
                        this.TableTypeData.Rows.Add(dr);
                    }
                    i++;
                }
            }
            else
            {
                throw new FileFormatException("不可处理的文件格式。");
            }

        }
        /// <summary>
        /// DataTable转换成txt
        /// </summary>
        /// <param name="dt">带转换的数据表</param>
        /// <param name="countColumnName">表中要计算的数据列名</param>
        /// <param name="state">是否计算总金额</param>
        public void DataTableToTxt(System.Data.DataTable dt, string countColumnName, bool state)
        {

            this.TotalCount = 0;
            this.TotalMoney = 0;
            this.tableTypeData = dt;
            string data = string.Empty;
            foreach (DataRow dr in dt.Rows)
            {
                this.TotalCount++;
                string drdata = string.Empty;
                foreach (var item in dr.ItemArray)
                {
                    drdata += item.ToString() + " ";
                }
                data += drdata + "\n";
                if (state)
                {
                    if (!string.IsNullOrEmpty(countColumnName))
                    {
                        this.TotalMoney += Convert.ToDouble(dr[countColumnName]);
                    }
                }

            }
            this.TargetFileHead += "TOTAL NO " + TotalCount + " " + TotalMoney + "\n";
            this.changedFileContant = data.Trim("\n".ToCharArray());

        }
    

        /// <summary>
        /// DataTable转换成txt
        /// </summary>
        /// <param name="dt">带转换的数据表</param>
        /// <param name="countColumnName">表中要计算的数据列名</param>
        public void DataTableToTxt(System.Data.DataTable dt, string countColumnName)
        {
            this.TargetFileHead += "{0} {1} ";
            this.TotalCount = 0;
            this.TotalMoney = 0;
            this.tableTypeData = dt;
            string data = string.Empty;
            foreach (DataRow dr in dt.Rows)
            {
                this.TotalCount++;
                string drdata = string.Empty;
                foreach (var item in dr.ItemArray)
                {
                    drdata += item.ToString() + " ";
                }
                data += drdata + "\n";
                if (!string.IsNullOrEmpty(countColumnName))
                {
                    this.TotalMoney += Convert.ToDouble(dr[countColumnName]);
                }
            }
            this.changedFileContant = data.Trim("\n".ToCharArray());
        }

        /// <summary>
        /// DataTable转换成txt(不计算总金额）
        /// </summary>
        /// <param name="dt">带转换的数据表</param>
       
        public void DataTableToTxt(System.Data.DataTable dt)
        {
            this.TargetFileHead += "{0} {1} ";
            this.TotalCount = 0;
            this.TotalMoney = 0;
            this.tableTypeData = dt;
            string data = string.Empty;
            foreach (DataRow dr in dt.Rows)
            {
                this.TotalCount++;
                string drdata = string.Empty;
                foreach (var item in dr.ItemArray)
                {
                    drdata += item.ToString() + " ";
                }
                data += drdata + "\n";
     
            }
            this.changedFileContant = data.Trim("\n".ToCharArray());
        }

        public enum FileType
        {
            /// <summary>
            /// 文本文件
            /// </summary>
            TXT,
            /// <summary>
            /// excel表格文件
            /// </summary>
            EXCEL
        }
        /// <summary>
        /// 统计文件中的总金额，总笔数
        /// </summary>
        /// <param name="s">要统计的字符串</param>
        /// <param name="totalmoney">金额所在列号</param>
        public void TotalCountAndTotalMoney(string s, int totalmoney)
        {

            int j = 0;
            string[] str1 = s.Split('\n');
            string s2 = string.Empty;
            foreach (string item in str1)
            {
                string[] str2 = item.Split(' ');
                if (item == "" || item == "END")
                {
                    continue;
                }
                if (str2[0] == "TOTAL")
                {
                    continue;
                }
                this.TotalMoney = this.TotalMoney + Convert.ToDouble(str2[totalmoney]);
                j++;
            }
            this.TotalCount = j;
            str1[0] = " "+this.TotalCount.ToString() + " " + this.TotalMoney.ToString();
          //  this.TargetFileHead = str1[0] + '\n';
            for (int n = 0; n < str1.Length; n++)
            {

                if (str1[n] == "")
                {
                    continue;
                }

                if (str1[n + 1] != "END")
                    s2 = s2 + str1[n] + '\n';
                else
                {
                    s2 = s2 + str1[n];
                    break;
                }
            }
            this.changedFileContant = s2;
        }


    }
}
