// 模拟依赖的类型和模块
type DataTable = {
    columns: string[];
    rows: any[][];
    clear(): void;
    newRow(): any;
    addRow(row: any): void;
};

type Worksheet = {
    UsedRange: {
        Rows: {
            item(index: number): any;
        };
        Columns: any;
    };
};

type Workbook = {
    Close(saveChanges?: boolean, fileName?: any, routeWorkbook?: any): void;
};

type ExcelApplication = {
    Workbooks: {
        Open(filePath: string, ...args: any[]): Workbook;
        Close(): void;
    };
    Quit(): void;
};

// 定义 FileType 枚举
enum FileType {
    TXT,
    EXCEL
}

class ExcelToTxt {
    private wath: { start(): void; stop(): void; elapsed: { totalMilliseconds: number } } = {
        start() {},
        stop() {},
        elapsed: { totalMilliseconds: 0 }
    };
    private changedFileContant: string = "";
    private isHasTotalMoneyAndTotalCout: boolean = false;
    private tableColumNames: string[] = [];
    private sourceFileFullName: string = "";
    private tableTypeData: DataTable = {
        columns: [],
        rows: [],
        clear() {
            this.columns = [];
            this.rows = [];
        },
        newRow() {
            return {};
        },
        addRow(row: any) {
            this.rows.push(row);
        }
    };

    public TotalMoney: number = 0;
    public TotalCount: number = 0;
    public TargetFileHead: string = "TOTAL NO ";
    public TargetFileEnd: string = "\nEND";
    public SourceFileType: FileType = FileType.EXCEL;
    public TargetFileFullName: string = "";

    constructor(sourceFile?: string, targetFile?: string) {
        if (sourceFile && targetFile) {
            this.SourceFileFullName = sourceFile;
            this.TargetFileFullName = targetFile;
        }
        this.tableTypeData.clear();
    }

    get TableColumNames(): string[] {
        if (this.tableColumNames.length === 0) {
            this.tableColumNames = this.tableTypeData.columns;
        }
        return this.tableColumNames;
    }

    set TableColumNames(value: string[]) {
        if (value.length > 0) {
            value.forEach((colName) => {
                if (!this.tableTypeData.columns.includes(colName)) {
                    this.tableTypeData.columns.push(colName);
                }
            });
        }
    }

    get SourceFileName(): string {
        return this.sourceFileFullName.split("\\").pop() || "";
    }

    get TargetFileName(): string {
        return this.TargetFileFullName.split("\\").pop() || "";
    }

    set SourceFileFullName(value: string) {
        this.sourceFileFullName = value;
        if (value.toLowerCase().endsWith(".txt")) {
            this.SourceFileType = FileType.TXT;
        } else {
            this.SourceFileType = FileType.EXCEL;
        }
    }

    get ChangedFileContant(): string {
        if (this.isHasTotalMoneyAndTotalCout) {
            return this.TargetFileHead + this.changedFileContant + this.TargetFileEnd;
        } else {
            return `${this.TargetFileHead}\n${this.changedFileContant}${this.TargetFileEnd}`.replace(
                "{0}",
                this.TotalCount.toString()
            ).replace("{1}", this.TotalMoney.toString());
        }
    }

    public ChangeExcelToTxt(): boolean {
        if (!this.sourceFileFullName) {
            throw new Error(`找不到指定路径:${this.sourceFileFullName}的一部分。`);
        }
        return this.ChangeExcelToTxt(this.sourceFileFullName);
    }

    public ChangeExcelToTxt(fileFullName: string): boolean {
        return this.ChangeExcelToTxt(fileFullName, 0, 0, false, 1);
    }

    public ChangeExcelToTxt(
        filePath: string,
        headLineCount: number,
        endLineCount: number,
        hasColumnTitle: boolean,
        beginLine: number
    ): boolean {
        let result = false;
        this.SourceFileFullName = filePath;
        if (!this.sourceFileFullName) {
            throw new Error(`找不到指定路径:${this.sourceFileFullName}的一部分。`);
        }
        try {
            if (filePath.endsWith(".xls") || filePath.endsWith(".xlsx")) {
                this.changedFileContant = this.ExcelToText(filePath, headLineCount, endLineCount, hasColumnTitle, beginLine);
            }
        } catch (error) {
            this.changedFileContant = "";
        }
        return result;
    }

    private ExcelToText(
        filePath: string,
        headLineCount: number,
        endLineCount: number,
        hasColumnTitle: boolean,
        beginLine: number
    ): string {
        this.isHasTotalMoneyAndTotalCout = true;
        let head = "";
        let data = "";
        this.tableTypeData.clear();
        const app: ExcelApplication = {
            Workbooks: {
                Open() {
                    return {
                        Close() {}
                    };
                },
                Close() {}
            },
            Quit() {}
        };
        let workbook: Workbook | null = null;
        try {
            this.wath.start();
            workbook = app.Workbooks.Open(filePath);
            const sheets = workbook;
            const worksheet: Worksheet = {
                UsedRange: {
                    Rows: {
                        item(index: number) {
                            return {
                                Columns: {
                                    forEach(callback: (rc: any) => void) {
                                        // 模拟列遍历
                                    }
                                }
                            };
                        }
                    },
                    Columns: {}
                }
            };
            const rowCount = 100; // 模拟总行数

            // 处理表头
            for (let i = beginLine; i < beginLine + headLineCount; i++) {
                let currentColIndex = 0;
                worksheet.UsedRange.Rows.item(i).Columns.forEach((rc: any) => {
                    currentColIndex++;
                    const cells = rc.Cells;
                    if (cells && cells.Text.trim()) {
                        if (!cells.Text.includes("：")) {
                            head += cells.Text.trim() + " ";
                        }
                    }
                });
            }

            // 处理表中数据内容
            for (let i = beginLine + headLineCount; i <= rowCount - endLineCount; i++) {
                let tempData = "";
                let currentColIndex = 0;
                if (hasColumnTitle) {
                    worksheet.UsedRange.Rows.item(i).Columns.forEach((rc: any) => {
                        currentColIndex++;
                        const cells = rc.Cells;
                        if (cells && cells.Text.trim()) {
                            this.tableTypeData.columns.push(cells.Text.trim());
                        }
                    });
                    hasColumnTitle = false;
                    continue;
                }
                const dr = this.tableTypeData.newRow();
                let isAddRow = false;
                worksheet.UsedRange.Rows.item(i).Columns.forEach((rc: any) => {
                    currentColIndex++;
                    let temp = "";
                    const cells = rc.Cells;
                    if (cells && cells.Text.trim()) {
                        let t = cells.Text;
                        let dateTime = new Date(t);
                        if (t.includes(":") || t.includes("-") || t.includes("/")) {
                            if (!isNaN(dateTime.getTime())) {
                                if (dateTime.getHours() !== 0) {
                                    t = dateTime.toLocaleTimeString("en-US", { hour12: false, hour: "2-digit", minute: "2-digit", second: "2-digit" }).replace(/:/g, "");
                                    temp = dateTime.toLocaleTimeString("en-US", { hour12: false, hour: "2-digit", minute: "2-digit", second: "2-digit" });
                                } else {
                                    t = dateTime.toISOString().split("T")[0].replace(/-/g, "");
                                    temp = dateTime.toISOString().split("T")[0];
                                }
                            }
                        }
                        tempData += t.trim() + " ";
                        dr[currentColIndex - 1] = temp;
                        isAddRow = true;
                    }
                });
                if (isAddRow) {
                    this.tableTypeData.addRow(dr);
                }
                if (tempData.trim()) {
                    data += `\n${tempData.trimEnd()}`;
                }
            }

            // 处理表尾
            for (let i = rowCount - endLineCount + 1; i <= rowCount; i++) {
                let currentColIndex = 0;
                let temp = "";
                worksheet.UsedRange.Rows.item(i).Columns.forEach((rc: any) => {
                    currentColIndex++;
                    const cells = rc.Cells;
                    if (cells && cells.Text.trim()) {
                        if (!cells.Text.includes("：")) {
                            temp += cells.Text.trim() + " ";
                        }
                    }
                });
                head = temp + head;
            }

            this.wath.stop();
            return head + data;
        } catch (error) {
            throw new Error(`数据转换错误。`);
        } finally {
            if (workbook) {
                workbook.Close(false);
            }
            app.Workbooks.Close();
            app.Quit();
        }
    }

    public Save(): boolean {
        if (!this.TargetFileFullName) {
            throw new Error(`找不到指定路径:${this.TargetFileFullName}的一部分。`);
        }
        return this.Save(this.TargetFileFullName, this.ChangedFileContant);
    }

    public Save(targetFile: string): boolean {
        if (!targetFile) {
            throw new Error(`找不到指定路径:${targetFile}的一部分。`);
        }
        return this.Save(targetFile, this.ChangedFileContant);
    }

    public Save(targetFile: string, fileContant: string): boolean {
        this.TargetFileFullName = targetFile;
        // 模拟文件写入
        console.log(`将内容写入文件: ${targetFile}`);
        return true;
    }

    public TxtToDataTableColumnName(sourceFile: string, countColumnName: string, splitChar?: string, t?: boolean): void {
        let temp = "";
        // 模拟文件读取
        temp = temp.replace("\r\n", "\n");
        this.TargetFileHead = temp.startsWith("TOTAL") ? "" : this.TargetFileHead + "{0} {1}";
        this.changedFileContant = temp.replace("^", " ");
        if (splitChar && t !== undefined) {
            this.ToDataTable(splitChar, countColumnName, t);
        } else if (splitChar) {
            this.ToDataTable(splitChar, countColumnName);
        } else {
            this.ToDataTable(" ", countColumnName);
        }
    }

    public TxtToDataTable(sourceFile: string, splitChar?: string): void {
        this.TotalCount = 0;
        let needLines: string[] = [];
        // 模拟文件读取
        const lines: string[] = [];
        lines.forEach((line) => {
            if (line.startsWith("U")) {
                this.TotalCount++;
                needLines.push(line);
                this.changedFileContant += line + "\n";
            }
        });
        this.TargetFileHead += "{0} {1} ";
        this.changedFileContant = this.changedFileContant.trim().replace("^", " ");
        if (splitChar) {
            this.ToDataTable(splitChar, "");
        } else {
            this.ToDataTable(" ", "");
        }
    }

    private ToDataTable(splitChar: string, countColumn: string, t?: boolean): void {
        const strs = this.ChangedFileContant.split("\n");
        let i = 0;
        let needAddTotalCount = false;
        let needCountTotalMoney = false;
        if (strs) {
            this.tableTypeData.clear();
            strs.forEach((item) => {
                if (item.startsWith("END")) {
                    return;
                }
                if (!item.trim()) {
                    return;
                }
                const strT = item.split(splitChar);
                if (!strT || strT.length < 1) {
                    throw new Error("不可处理的文件格式。");
                }
                if (i === 0) {
                    if (t) {
                        this.TargetFileHead = `${strT[0]} ${strT[1]} {0} {1} `;
                        this.TotalCount = parseInt(strT[2], 10);
                        needAddTotalCount = this.TotalCount > 0 ? false : true;
                        this.TotalMoney = parseFloat(strT[3]);
                        needCountTotalMoney = this.TotalMoney > 0 ? false : true;
                    }
                    if (strT.length > 4) {
                        for (let j = 4; j < strT.length; j++) {
                            this.TargetFileHead += strT[j];
                        }
                    }
                } else {
                    const dr = this.tableTypeData.newRow();
                    strT.forEach((value, index) => {
                        dr[index] = value;
                    });
                    if (countColumn && needCountTotalMoney) {
                        const colIndex = this.tableTypeData.columns.indexOf(countColumn);
                        if (colIndex !== -1 && dr[colIndex] !== undefined) {
                            this.TotalMoney += parseFloat(dr[colIndex]);
                        }
                    }
                    this.TotalCount = needAddTotalCount ? this.TotalCount + 1 : this.TotalCount;
                    this.tableTypeData.addRow(dr);
                }
                i++;
            });
        } else {
            throw new Error("不可处理的文件格式。");
        }
    }

    public DataTableToTxt(dt: DataTable, countColumnName?: string, state?: boolean): void {
        this.TotalCount = 0;
        this.TotalMoney = 0;
        this.tableTypeData = dt;
        let data = "";
        dt.rows.forEach((dr) => {
            this.TotalCount++;
            let drdata = "";
            dr.forEach((item) => {
                drdata += item.toString() + " ";
            });
            data += drdata + "\n";
            if (state && countColumnName) {
                const colIndex = dt.columns.indexOf(countColumnName);
                if (colIndex !== -1 && dr[colIndex] !== undefined) {
                    this.TotalMoney += parseFloat(dr[colIndex]);
                }
            }
        });
        if (state) {
            this.TargetFileHead += `TOTAL NO ${this.TotalCount} ${this.TotalMoney}\n`;
        } else {
            this.TargetFileHead += "{0} {1} ";
        }
        this.changedFileContant = data.trim();
    }

    public TotalCountAndTotalMoney(s: string, totalmoney: number): void {
        let j = 0;
        const str1 = s.split("\n");
        let s2 = "";
        str1.forEach((item) => {
            const str2 = item.split(" ");
            if (!item || item === "END") {
                return;
            }
            if (str2[0] === "TOTAL") {
                return;
            }
            if (str2[totalmoney] !== undefined) {
                this.TotalMoney += parseFloat(str2[totalmoney]);
            }
            j++;
        });
        this.TotalCount = j;
        str1[0] = ` ${this.TotalCount} ${this.TotalMoney}`;
        str1.forEach((item, index) => {
            if (!item) {
                return;
            }
            if (str1[index + 1] !== "END") {
                s2 += item + "\n";
            } else {
                s2 += item;
            }
        });
        this.changedFileContant = s2;
    }
}

export { ExcelToTxt, FileType };