﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TellerSystem.ServiceProxy.Ext.WebAPI.Auth.Entity;
using System.Threading;
using System.IO;
using Platform.Common.LogSystem;
using Platform.Common.RunningParameters;
using System.Net;
using System.Windows;
using System.Text.RegularExpressions;
using System.Diagnostics;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using System.Windows.Media;
using TellerSystem;
using System.Windows.Media.Imaging;
using TellerSystem.Library.Ext.Function;
using System.Drawing.Imaging;
using System.Drawing;
using System.Drawing.Drawing2D;


namespace TellerSystem.Library.Ext.TradeExtension
{
    /// <summary>
    /// FTP接口类
    /// </summary>
    public class FtpHandle_TS
    {
        public static string YXLC = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("YXLC");
        /// <summary>
        /// 压缩图片
        /// </summary>
        public static byte[] ystp;
        /// <summary>
        /// 上传文件
        /// </summary>
        /// <param name="fileinfo">需要上传的文件</param>
        /// <param name="targetDir">目标路径</param>
        /// <param name="hostname">ftp地址</param>
        /// <param name="username">ftp用户名</param>
        /// <param name="password">ftp密码</param>
        public static void UploadFile(FileInfo fileinfo, string targetDir, string hostname, string username, string password)
        {
            //1. check target
            string target;
            if (targetDir.Trim() == "")
            {
                return;
            }
            //target = Guid.NewGuid().ToString();  //使用临时文件名
            target = fileinfo.Name;//文件名

            //string URI = "FTP://" + hostname + "/image/image/" + targetDir + "/" + target;
            string URI = "FTP://" + hostname + "/" + targetDir + "/" + target;
            System.Net.FtpWebRequest ftp = GetRequest(URI, username, password);

            ftp.Method = WebRequestMethods.Ftp.GetFileSize;
            ftp.ContentLength = fileinfo.Length;
            //设置FTP命令 设置所要执行的FTP命令，
            //ftp.Method = System.Net.WebRequestMethods.Ftp.ListDirectoryDetails;//假设此处为显示指定路径下的文件列表
            ftp.Method = System.Net.WebRequestMethods.Ftp.UploadFile;
            //指定文件传输的数据类型
            ftp.UseBinary = true;
            ftp.UsePassive = true;
            
            //告诉ftp文件大小
            ftp.ContentLength = fileinfo.Length;
            //缓冲大小设置为2KB
            const int BufferSize = 2048;
            byte[] content = new byte[BufferSize - 1 + 1];
            int dataRead;

            //打开一个文件流 (System.IO.FileStream) 去读上传的文件
            using (FileStream fs = fileinfo.OpenRead())
            {
                try
                {
                    //把上传的文件写入流
                    using (Stream rs = ftp.GetRequestStream())
                    {
                        do
                        {
                            //每次读文件流的2KB
                            dataRead = fs.Read(content, 0, BufferSize);
                            rs.Write(content, 0, dataRead);
                        } while (!(dataRead < BufferSize));
                        rs.Close();
                    }

                }
                catch (Exception ex) { TradeHandle.WriteImportantLog("写文件异常", "写文件异常", "pic", "pp"); }
                finally
                {
                    fs.Close();
                }

            }

            //ftp = null;
            ////设置FTP命令
            //ftp = GetRequest(URI, username, password);
            //ftp.Method = System.Net.WebRequestMethods.Ftp.Rename; //改名
            //ftp.RenameTo = fileinfo.Name;
            //try
            //{
            //    ftp.GetResponse();
            //}
            //catch (Exception ex)
            //{
            //    TradeHandle.WriteImportantLog("文件重命名异常", "文件重命名异常", "pic", "pp");
            //    ftp = GetRequest(URI, username, password);
            //    ftp.Method = System.Net.WebRequestMethods.Ftp.DeleteFile; //删除
            //    try
            //    {
            //        ftp.GetResponse();
            //    }
            //    catch (Exception ex1)
            //    {
            //        TradeHandle.WriteImportantLog("留存服务器删除文件异常", "留存服务器删除文件异常", "pic", "pp");
            //        throw ex1;
            //    }
            //    throw ex;
            //}
            //finally
            //{
            //    //fileinfo.Delete();
            //}

            // 可以记录一个日志  "上传" + fileinfo.FullName + "上传到" + "FTP://" + hostname + "/" + targetDir + "/" + fileinfo.Name + "成功." );
            ftp = null;

            #region
            /*****
             *FtpWebResponse
             * ****/
            //FtpWebResponse ftpWebResponse = (FtpWebResponse)ftp.GetResponse();
            #endregion
        }

        private static FtpWebRequest GetRequest(string URI, string username, string password)
        {
            //根据服务器信息FtpWebRequest创建类的对象
            FtpWebRequest result = (FtpWebRequest)FtpWebRequest.Create(URI);
            //提供身份验证信息
            result.Credentials = new System.Net.NetworkCredential(username, password);
            //设置请求完成之后是否保持到FTP服务器的控制连接，默认值为true
            result.KeepAlive = false;
            return result;
        }

        /// 在ftp服务器上创建目录
        /// </summary>
        /// <param name="dirName">创建的目录名称</param>
        /// <param name="ftpHostIP">ftp地址</param>
        /// <param name="username">用户名</param>
        /// <param name="password">密码</param>
        public static void MakeDir(string dirName, string ftpHostIP, string username, string password)
        {
            if (!ftpIsExistsFile(LoginUserInfo.TradeDate, YXLC.Split('|')[0].Trim(), YXLC.Split('|')[1].Trim(), YXLC.Split('|')[2].Trim()))
            {
                string uri = null;
                try
                {
                    //uri = "ftp://" + ftpHostIP + "/image/image/" + dirName;
                    uri = "ftp://" + ftpHostIP + "/" + dirName;
                    System.Net.FtpWebRequest ftp = GetRequest(uri, username, password);
                    ftp.Method = WebRequestMethods.Ftp.MakeDirectory;

                    FtpWebResponse response = (FtpWebResponse)ftp.GetResponse();
                    response.Close();
                }
                catch (Exception ex)
                {
                    TradeHandle.WriteImportantLog("创建目录失败", uri, "pic", "pp");
                    //MessageBox.Show(ex.Message);
                }
            }
            
        }

        /// <summary>
        /// 判断ftp服务器上该目录是否存在
        /// </summary>
        /// <param name="dirName"></param>
        /// <param name="ftpHostIP"></param>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        public static bool ftpIsExistsFile(string dirName, string ftpHostIP, string username, string password)
        {
            bool flag = false;
            try
            {
                //string uri = "ftp://" + ftpHostIP + "/image/image/";// +dirName;
                string uri = "ftp://" + ftpHostIP;// +dirName;
                System.Net.FtpWebRequest ftp = GetRequest(uri, username, password);
                ftp.Method = WebRequestMethods.Ftp.ListDirectory;

                FtpWebResponse response = (FtpWebResponse)ftp.GetResponse();
                StreamReader reader = new StreamReader(response.GetResponseStream());
                string line = null;
                while((line = reader.ReadLine()) != null)
                {
                    if (line != null && line.Contains(LoginUserInfo.TradeDate))
                    {
                        flag = true;
                    }
                }
                //string line = reader.ReadLine();
                //if (line != null && line.Contains(LoginUserInfo.TradeDate))
                //{
                //    flag = true;
                //}
                //else
                //{
                //    flag = false;
                //}
                response.Close();
            }
            catch (Exception)
            {
                TradeHandle.WriteImportantLog("判断ftp服务器上该目录是否存在方法异常", "判断ftp服务器上该目录是否存在方法异常", "pic", "pp");
                flag = false;
            }
            return flag;
        }

        /// <summary>
        /// 执行拍照动作并保存图片至指定路径下
        /// </summary>
        /// <param name="IdNo"></param>
        /// <returns></returns>
        public static void photograph(string IdNo)
        {
            string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            string devicesTagStr = null;
            //执行拍摄动作
            Process[] processes = Process.GetProcessesByName("CameraPlayer");
            if (processes.Length > 0)
            {
                foreach (Process process in processes)
                {
                    if (!process.HasExited)
                        process.Kill();
                }
            }
            if (devicesTagStr == null)
            {
                string kh = String.Format("KH|{0}|{1};", ConfigManager.GetInstance().GetConfigValue("KHVid", ConfigType.User), ConfigManager.GetInstance().GetConfigValue("KHPid", ConfigType.User));
                devicesTagStr = kh;
            }
            string saveImgDir = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, @"picture", LoginUserInfo.TradeDate);
            //创建头像存储路径
            DirectoryInfo dir = null;
            if (!File.Exists(saveImgDir))
            {
                dir = Directory.CreateDirectory(saveImgDir);
            }
            //DateTime.Now.ToString("yyyyMMddHHmmss")
            string fileName = LoginUserInfo.TradeDate + "_" + LoginUserInfo.TellerNo + "_" + IdNo + ".png";
            string fileName1 = DateTime.Now.ToString("yyyyMMddHHmmss") + "_" + LoginUserInfo.TellerNo + "_" + IdNo + ".png";
            string outImgPath = System.IO.Path.Combine(@"picture", LoginUserInfo.TradeDate, fileName);
            string outImgPath1 = System.IO.Path.Combine(@"picture", LoginUserInfo.TradeDate, fileName1);
            Process printProcess = new Process();
            printProcess.StartInfo.FileName = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, @"Devices\CameraPlayer\CameraPlayer.exe");
            printProcess.StartInfo.Arguments = FLAG + " " + "KH" + " " + devicesTagStr + " " + outImgPath;
            printProcess.Start();
            printProcess.WaitForExit();
            string saveImgPath = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, outImgPath);
            string saveImgPath1 = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, outImgPath1);
            Bitmap ystph;
            if (File.Exists(saveImgPath))
            {
                BitmapImage bitmapImage = new BitmapImage();
                bitmapImage.BeginInit();
                bitmapImage.CacheOption = BitmapCacheOption.OnLoad;
                bitmapImage.UriSource = new Uri(saveImgPath);//szPath为图片的全路径
                bitmapImage.EndInit();
                bitmapImage.Freeze();
                CompressImage(saveImgPath, saveImgPath1,720,1280,90);
            }


            string getDirecName = LoginUserInfo.TradeDate;
            FtpHandle_TS.MakeDir(getDirecName, YXLC.Split('|')[0].Trim(), YXLC.Split('|')[1].Trim(), YXLC.Split('|')[2].Trim());
            //FtpHandle_TS.MakeDir(getDirecName, "16.5.21.61", "c6front", "c6front321");

            DirectoryInfo dire = dir as DirectoryInfo;
            if (dire == null) return;
            FileSystemInfo[] files = dire.GetFileSystemInfos();
            for (int i = 0; i < files.Length; i++)
            {
                FileInfo fi = files[i] as FileInfo;
                if (fi != null && fi.Name == fileName1)
                {
                    DirectoryInfo DirecObj = fi.Directory;
                    string DireObjName = DirecObj.Name;
                    FtpHandle_TS.UploadFile(fi, DireObjName, YXLC.Split('|')[0].Trim(), YXLC.Split('|')[1].Trim(), YXLC.Split('|')[2].Trim());
                    //FtpHandle_TS.UploadFile(fi, DireObjName, "16.5.21.61", "c6front", "c6front321");
                }
            }
            //判断文件夹是否还存在--清理当前拍摄照片的原文件
            if (Directory.Exists(saveImgDir))
            {
                //去除文件夹和子文件的只读属性
                System.IO.DirectoryInfo fileInfo = new DirectoryInfo(saveImgDir);
                fileInfo.Attributes = FileAttributes.Normal & FileAttributes.Directory;
                System.IO.File.SetAttributes(saveImgDir, System.IO.FileAttributes.Normal);
                foreach (string f in Directory.GetFileSystemEntries(saveImgDir))
                {
                    if (File.Exists(f) && f == saveImgPath)
                    {
                        //如果有子文件删除文件
                        File.Delete(f);
                    }
                }
                ////删除空文件夹
                //Directory.Delete(saveImgDir);
            }

            string filePath = "picture";
            FileSystemInfo filesInfo = new DirectoryInfo(filePath);
            ClearFiles(filesInfo);
        }

        /// <summary>
        /// 清除本地图片文件
        /// </summary>
        public static void ClearFiles(FileSystemInfo file)
        {
            DirectoryInfo dire = file as DirectoryInfo;
            if (dire == null) return;
            FileSystemInfo[] files = dire.GetFileSystemInfos();
            string date = string.Empty;
            if (LoginUserInfo.TradeDate.Length == 8)
                date = LoginUserInfo.TradeDate.Substring(0, 4) + "-" + LoginUserInfo.TradeDate.Substring(4, 2) + "-" + LoginUserInfo.TradeDate.Substring(6, 2);
            DateTime dt = DateTime.Now;
            if (!DateTime.TryParse(date, out dt))
                return;
            string[] dateList = new string[3];
            dateList[0] = LoginUserInfo.TradeDate;
            dateList[1] = string.Format("{0:yyyyMMdd}", dt.AddDays(-1));
            dateList[2] = string.Format("{0:yyyyMMdd}", dt.AddDays(-2));
            for (int i = 0; i < files.Length; i++)
            {
                bool flag = true;
                string finame = files[i].Name;
                string fipath = files[i].FullName;
                for (int y = 0; y < dateList.Length; y++)
                {
                    if (files[i].Name == dateList[y])
                    {
                        flag = false;
                        continue;
                    }

                }
                if (flag)
                {
                    DirectoryInfo dirInfo = new DirectoryInfo(fipath);
                    //删除目录及其子目录
                    dirInfo.Delete(true);
                }


            }
        }
        public static Bitmap BytesToBitmap(byte[] Bytes)
        {
            MemoryStream stream = null;
            try
            {
                stream = new MemoryStream(Bytes);
                return new Bitmap((Image)new Bitmap(stream));
            }
            catch (ArgumentNullException ex)
            {
                throw ex;
            }
            catch (ArgumentException ex)
            {
                throw ex;
            }
            finally
            {
                stream.Close();
            }
        }

        public static bool CompressImage(string sFile, string dFile, int dHeight, int dWidth, int flag)
        {


            System.Drawing.Image iSource = System.Drawing.Image.FromFile(sFile);
            ImageFormat tFormat = iSource.RawFormat;
            int sW = 0, sH = 0;
            //按比例缩放  
            System.Windows.Size tem_size = new System.Windows.Size(iSource.Width, iSource.Height);


            if (tem_size.Width > dHeight || tem_size.Width > dWidth)
            {
                if ((tem_size.Width * dHeight) > (tem_size.Height * dWidth))
                {
                    sW = dWidth;
                    sH = (dWidth * Convert.ToInt32(tem_size.Height)) / Convert.ToInt32(tem_size.Width);
                }
                else
                {
                    sH = dHeight;
                    sW = (Convert.ToInt32(tem_size.Width) * dHeight) / Convert.ToInt32(tem_size.Height);
                }
            }
            else
            {
                sW = Convert.ToInt32(tem_size.Width);
                sH = Convert.ToInt32(tem_size.Height);
            }
            Bitmap ob = new Bitmap(dWidth, dHeight);
            Graphics g = Graphics.FromImage(ob);
            g.CompositingQuality = CompositingQuality.HighQuality;
            g.SmoothingMode = SmoothingMode.HighQuality;
            g.InterpolationMode = InterpolationMode.HighQualityBicubic;
            g.DrawImage(iSource, new Rectangle((dWidth - sW) / 2, (dHeight - sH) / 2, sW, sH), 0, 0, iSource.Width, iSource.Height, GraphicsUnit.Pixel);
            g.Dispose();
            //以下代码为保存图片时，设置压缩质量  
            EncoderParameters ep = new EncoderParameters();
            long[] qy = new long[1];
            qy[0] = flag;//设置压缩的比例1-100  
            EncoderParameter eParam = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, qy);
            ep.Param[0] = eParam;
            try
            {
                ImageCodecInfo[] arrayICI = ImageCodecInfo.GetImageEncoders();
                ImageCodecInfo jpegICIinfo = null;
                for (int x = 0; x < arrayICI.Length; x++)
                {
                    if (arrayICI[x].FormatDescription.Equals("JPEG"))
                    {
                        jpegICIinfo = arrayICI[x];
                        break;
                    }
                }
                if (jpegICIinfo != null)
                {
                    ob.Save(dFile, jpegICIinfo, ep);//dFile是压缩后的新路径  
                }
                else
                {
                    ob.Save(dFile, tFormat);
                }
                return true;
            }
            catch
            {
                return false;
            }
            finally
            {
                iSource.Dispose();
                ob.Dispose();
            }


        }
    }
}
