// 模拟依赖的类和方法
class TradeManagerHandle {
    static TT_SYSCONFIG_GetConfigValueByConfigID(id: string): string {
        return "";
    }
}

class LoginUserInfo {
    static get TradeDate(): string {
        return "";
    }
    static get TellerNo(): string {
        return "";
    }
}

class ConfigManager {
    static GetInstance() {
        return {
            GetConfigValue: (key: string, type: any) => ""
        };
    }
}

class TradeHandle {
    static WriteImportantLog(title: string, content: string, type: string, subType: string) {
        console.log(`[${title}] ${content}`);
    }
}

// 模拟文件系统操作
const fs = require('fs');
const path = require('path');
const child_process = require('child_process');

// 模拟 FTP 操作
const ftp = require('basic-ftp');

class FtpHandle_TS {
    static YXLC = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("YXLC");
    static ystp: Buffer;

    static async UploadFile(filePath: string, targetDir: string, hostname: string, username: string, password: string) {
        if (!targetDir.trim()) {
            return;
        }

        const client = new ftp.Client();
        try {
            await client.access({
                host: hostname,
                user: username,
                password: password,
                secure: false
            });

            const fileName = path.basename(filePath);
            await client.ensureDir(targetDir);
            await client.uploadFrom(filePath, `${targetDir}/${fileName}`);
        } catch (error) {
            TradeHandle.WriteImportantLog("写文件异常", "写文件异常", "pic", "pp");
        } finally {
            client.close();
        }
    }

    static async MakeDir(dirName: string, ftpHostIP: string, username: string, password: string) {
        if (!await this.ftpIsExistsFile(dirName, ftpHostIP, username, password)) {
            const client = new ftp.Client();
            try {
                await client.access({
                    host: ftpHostIP,
                    user: username,
                    password: password,
                    secure: false
                });
                await client.ensureDir(dirName);
            } catch (error) {
                TradeHandle.WriteImportantLog("创建目录失败", `ftp://${ftpHostIP}/${dirName}`, "pic", "pp");
            } finally {
                client.close();
            }
        }
    }

    static async ftpIsExistsFile(dirName: string, ftpHostIP: string, username: string, password: string) {
        const client = new ftp.Client();
        try {
            await client.access({
                host: ftpHostIP,
                user: username,
                password: password,
                secure: false
            });
            const list = await client.list();
            return list.some(item => item.name === dirName);
        } catch (error) {
            TradeHandle.WriteImportantLog("判断ftp服务器上该目录是否存在方法异常", "判断ftp服务器上该目录是否存在方法异常", "pic", "pp");
            return false;
        } finally {
            client.close();
        }
    }

    static async photograph(IdNo: string) {
        const FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
        let devicesTagStr = null;

        const processes = child_process.execSync('tasklist').toString().split('\n').filter(line => line.includes('CameraPlayer'));
        processes.forEach(() => {
            child_process.execSync('taskkill /F /IM CameraPlayer.exe');
        });

        if (!devicesTagStr) {
            const kh = `KH|${ConfigManager.GetInstance().GetConfigValue("KHVid", "User")}|${ConfigManager.GetInstance().GetConfigValue("KHPid", "User")};`;
            devicesTagStr = kh;
        }

        const saveImgDir = path.join(process.cwd(), 'picture', LoginUserInfo.TradeDate);
        if (!fs.existsSync(saveImgDir)) {
            fs.mkdirSync(saveImgDir, { recursive: true });
        }

        const fileName = `${LoginUserInfo.TradeDate}_${LoginUserInfo.TellerNo}_${IdNo}.png`;
        const fileName1 = `${new Date().toISOString().replace(/[:.-]/g, '')}_${LoginUserInfo.TellerNo}_${IdNo}.png`;
        const outImgPath = path.join('picture', LoginUserInfo.TradeDate, fileName);
        const outImgPath1 = path.join('picture', LoginUserInfo.TradeDate, fileName1);

        const printProcess = child_process.spawn(path.join(process.cwd(), 'Devices', 'CameraPlayer', 'CameraPlayer.exe'), [FLAG, "KH", devicesTagStr, outImgPath]);
        await new Promise((resolve) => printProcess.on('close', resolve));

        if (fs.existsSync(outImgPath)) {
            // 模拟图片压缩
            fs.copyFileSync(outImgPath, outImgPath1);
        }

        await this.MakeDir(LoginUserInfo.TradeDate, this.YXLC.split('|')[0].trim(), this.YXLC.split('|')[1].trim(), this.YXLC.split('|')[2].trim());

        const files = fs.readdirSync(saveImgDir);
        files.forEach(file => {
            if (file === fileName1) {
                const filePath = path.join(saveImgDir, file);
                this.UploadFile(filePath, LoginUserInfo.TradeDate, this.YXLC.split('|')[0].trim(), this.YXLC.split('|')[1].trim(), this.YXLC.split('|')[2].trim());
            }
        });

        // 清理原文件
        if (fs.existsSync(outImgPath)) {
            fs.unlinkSync(outImgPath);
        }

        this.ClearFiles(new fs.Stats());
    }

    static ClearFiles(file: fs.Stats) {
        const filePath = 'picture';
        if (!fs.lstatSync(filePath).isDirectory()) {
            return;
        }

        const date = LoginUserInfo.TradeDate;
        let dt: Date;
        if (date.length === 8) {
            const year = date.substring(0, 4);
            const month = date.substring(4, 6);
            const day = date.substring(6, 8);
            dt = new Date(`${year}-${month}-${day}`);
        } else {
            return;
        }

        const dateList = [
            LoginUserInfo.TradeDate,
            new Date(dt.getTime() - 24 * 60 * 60 * 1000).toISOString().split('T')[0].replace(/-/g, ''),
            new Date(dt.getTime() - 2 * 24 * 60 * 60 * 1000).toISOString().split('T')[0].replace(/-/g, '')
        ];

        const dirs = fs.readdirSync(filePath);
        dirs.forEach(dir => {
            if (!dateList.includes(dir)) {
                fs.rmSync(path.join(filePath, dir), { recursive: true, force: true });
            }
        });
    }

    static BytesToBitmap(bytes: Buffer) {
        // 由于 TypeScript 没有 Bitmap 类型，这里简单返回字节数组
        return bytes;
    }

    static CompressImage(sFile: string, dFile: string, dHeight: number, dWidth: number, flag: number) {
        // 模拟图片压缩
        fs.copyFileSync(sFile, dFile);
        return true;
    }
}

export { FtpHandle_TS };