﻿using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.IO;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class ImageCutHelper
    {
        /*
            输入：
            char *filename 含有绝对路径的文件名
            int iCutMode 裁剪方式
			            1 整幅图像整体裁剪
			            2 先分为左右两部分，然后再次裁剪
                        BOOL bDegbugFlag 是否调试 TRUE 调试输出 FALSE 不调试
                        char *SavePath 裁剪后的保存路径
            输出：
                0 切割成功
	            1 裁剪方式错误(没有找到对应文件)
	            9 图像质量太差，无法裁剪
        */
        [DllImport("FLowBankImageMfcDll.dll")]
        private static extern int HKPD_CUT(string filename, int iMode, bool bDebugFlag, string SavePath);

        /// <summary>
        /// 切割汇款凭证，图形集合依次为 -0.大写金额- 1.小写金额- 2.业务类型时间-   3.转出方-  4.转入方-
        /// </summary>
        /// <param name="data"></param>
        /// <param name="cutData"></param>
        /// <returns></returns>
        public static bool CutHKPD(byte[] data, out List<byte[]> cutData)
        {
            cutData = new List<byte[]>();

            //将源数据保存为图片
            var file = new FileInfo(Path.Combine("temp", "HKPD.bmp"));
            if (!file.Directory.Exists) file.Directory.Create();
            var dir = new DirectoryInfo(Path.Combine("temp", "HKPD"));
            try
            {
                File.WriteAllBytes(file.FullName, data);
                if (!file.Exists) return false;
                if (!dir.Exists) dir.Create();

                var ret = HKPD_CUT(file.FullName, 1, false, dir.FullName);
                if (ret != 0) return false;

                //检查裁剪数据
                var f = new FileInfo(Path.Combine(dir.FullName, "大写金额.bmp"));
                if (!f.Exists) return false;
                cutData.Add(File.ReadAllBytes(f.FullName));

                f = new FileInfo(Path.Combine(dir.FullName, "小写金额.bmp"));
                if (!f.Exists) return false;
                cutData.Add(File.ReadAllBytes(f.FullName));

                f = new FileInfo(Path.Combine(dir.FullName, "业务类型时间.bmp"));
                if (!f.Exists) return false;
                cutData.Add(File.ReadAllBytes(f.FullName));

                f = new FileInfo(Path.Combine(dir.FullName, "转出方.bmp"));
                if (!f.Exists) return false;
                cutData.Add(File.ReadAllBytes(f.FullName));

                f = new FileInfo(Path.Combine(dir.FullName, "转入方.bmp"));
                if (!f.Exists) return false;
                cutData.Add(File.ReadAllBytes(f.FullName));
            }
            catch
            {
                return false;
            }
            finally
            {
                File.Delete(file.FullName);
                Directory.Delete(dir.FullName, true);
            }
            return true;
        }


        /*
            功能：组织机构代码的裁剪
            输入：
               char *filename 含有绝对路径的文件名
               BOOL bDegbugFlag 是否调试 TRUE 调试输出 FALSE 不调试
               char *SavePath 裁剪后的保存路径
               char *ORGNO 组织机构代码
               char *ORGNAME 组织名称
           输出：
               0 成功
               1 裁剪方式错误
               7 目录不存在或者格式错误
               8 不存在图像文件或图像格式错误 必须为bmp格式
               9 图像质量太差，无法裁剪
        */
        [DllImport("FLowBankImageMfcDll.dll")]
        private static extern int ORGNOTE_CUT(string filename, bool bDebugFlag, string SavePath, StringBuilder ORGNO, StringBuilder ORGNAME);

        /// <summary>
        /// 切割组织机构代码，图形集合依次为 -0.{[识别的组织机构代码证号],[组织机构代码证号图像数据]}- 1.{[识别的组织机构代码证名],[组织机构代码证名图像数据]} 
        /// </summary>
        /// <param name="data"></param>
        /// <param name="cutData"></param>
        /// <returns></returns>
        public static bool CutORGNOTE(byte[] data, out Dictionary<string, byte[]> cutData)
        {
            cutData = new Dictionary<string, byte[]>();

            //将源数据保存为图片
            var file = new FileInfo(Path.Combine("temp", "ORGNOTE.bmp"));
            if (!file.Directory.Exists) file.Directory.Create();
            var dir = new DirectoryInfo(Path.Combine("temp", "ORGNOTE"));
            try
            {
                File.WriteAllBytes(file.FullName, data);
                if (!file.Exists) return false;
                if (!dir.Exists) dir.Create();

                var ORGNO = new StringBuilder();
                var ORGNAME = new StringBuilder();
                var ret = ORGNOTE_CUT(file.FullName, false, dir.FullName, ORGNO, ORGNAME);
                if (ret != 0) return false;

                //检查裁剪数据
                var f = new FileInfo(Path.Combine(dir.FullName, "机构代码.bmp"));
                if (!f.Exists) return false;
                var ORGNO_Value = ORGNO.ToString();
                var index = ORGNO_Value.IndexOf(':');
                if (index >= 0)
                {
                    ORGNO_Value = ORGNO_Value.Substring(index + 1);
                }
                cutData.Add(ORGNO_Value.Replace(" ", ""), File.ReadAllBytes(f.FullName));

                f = new FileInfo(Path.Combine(dir.FullName, "机构名称.bmp"));
                if (!f.Exists) return false;
                var ORGNAME_Value = ORGNAME.ToString();
                index = ORGNAME_Value.IndexOf(':');
                if (index >= 0)
                {
                    ORGNAME_Value = ORGNAME_Value.Substring(index + 1);
                }
                cutData.Add(ORGNAME_Value.Replace(" ", ""), File.ReadAllBytes(f.FullName));
            }
            catch
            {
                return false;
            }
            finally
            {
                File.Delete(file.FullName);
                Directory.Delete(dir.FullName, true);
            }
            return true;
        }

        /*
           功能：身份证的裁剪
           输入：
           char *filename 含有绝对路径的文件名
           BOOL bDegbugFlag 是否调试 TRUE 调试输出 FALSE 不调试
           char *SavePath 裁剪后的保存路径
           char *IDNAME 身份证名称
           char *IDNO 身份证号码
           输出：
               0		成功
               1		//initialization failed
               2		//unsupported image format
               3		// file operating error ( reading or writing)
               4		// writing file error
               5		// no enough memory
               6		//no recognized result
               7		//软件加密锁未连接，或加密锁驱动未安装
               8		//Getcharinfo()该字符没有字符信息，比如：空格，制表符，回车换行符
               9		//Getcharinfo()传入索引超过总字符数
               10		//Getcharinfo(),本页没有文字识别结果
               11		// errors not listed above
               12		// dll load error
               13		// 参数错误
               14		// 版面分析无结果
               15		裁剪失败，没有定位到具体位置
               16		打开图像文件错误
               17		保存路径为空
           */
        //int  IDCARD_CUT(char *filename,BOOL bDebugFlag,char *SavePath,char *IdName,char *IdNo)
        [DllImport("FLowBankImageMfcDll.dll")]
        private static extern int IDCARD_CUT(string filename, bool bDebugFlag, string SavePath, StringBuilder IDNAME, StringBuilder IDNO, bool bCutEdge);

        /// <summary>
        /// 切割组织机构代码，图形集合依次为 -0.{[识别的组织机构代码证号],[组织机构代码证号图像数据]}- 1.{[识别的组织机构代码证名],[组织机构代码证名图像数据]} 
        /// </summary>
        /// <param name="data"></param>
        /// <param name="cutData"></param>
        /// <returns></returns>
        public static bool CutIDCARD(byte[] data, out Dictionary<string, byte[]> cutData)
        {
            cutData = new Dictionary<string, byte[]>();

            //将源数据保存为图片
            var file = new FileInfo(Path.Combine("temp", "IDCARD.bmp"));
            if (!file.Directory.Exists) file.Directory.Create();
            var dir = new DirectoryInfo(Path.Combine("temp", "IDCARD"));
            try
            {
                File.WriteAllBytes(file.FullName, data);
                if (!file.Exists) return false;
                if (!dir.Exists) dir.Create();

                var IDNAME = new StringBuilder();
                var IDNO = new StringBuilder();
                var ret = IDCARD_CUT(file.FullName, false, dir.FullName, IDNAME, IDNO, false);
                if (ret != 0) return false;

                //检查裁剪数据
                var f = new FileInfo(Path.Combine(dir.FullName, "身份证名称.bmp"));
                if (!f.Exists) return false;
                //var IDNAME_Value = IDNAME.ToString();
                //if (IDNAME_Value.Length > 4)
                //    IDNAME_Value = IDNAME_Value.Substring(4);
                cutData.Add(IDNAME.ToString().Replace(" ", ""), File.ReadAllBytes(f.FullName));

                f = new FileInfo(Path.Combine(dir.FullName, "身份证号码.bmp"));
                if (!f.Exists) return false;
                //var IDNO_Value = IDNO.ToString();
                //for (int i = 0; i < IDNO_Value.Length; i++)
                //{
                //    var c = IDNO_Value[i];
                //    if (c >= '0' && c <= '9')
                //    {
                //        IDNO_Value = IDNO_Value.Substring(i);
                //        break;
                //    }
                //}
                cutData.Add(IDNO.ToString().Replace(" ", ""), File.ReadAllBytes(f.FullName));
            }
            catch
            {
                return false;
            }
            finally
            {
                File.Delete(file.FullName);
                Directory.Delete(dir.FullName, true);
            }
            return true;
        }

        /*
            功能：身份证的裁剪-删掉图像的边缘
            输入：
            char *filename 含有绝对路径的文件名
            BOOL bDegbugFlag 是否调试 TRUE 调试输出 FALSE 不调试
            char *SavePath 裁剪后的保存路径
            char *SaveName 保存裁剪后的名称
            BOOL bCutEdge 是否删除原图像 TRUE 删除 FALSE 不删除
            int  iSaveMode 保存模式
            			0	与原图相同
            			1	灰度图像
            			2	二值图像
            输出：
            	0		成功
            
        **/
        [DllImport("FLowBankImageMfcDll.dll")]
        private static extern int IDCARD_CUT_EDGE(string filename, bool bDegbugFlag, string SavePath, string SaveName, bool bDelete, int iSaveMode);

        /// <summary>
        /// 处理身份证图片，返回去黑边之后的数据
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public static byte[] CutIDCARDBorder(byte[] data)
        {
            byte[] retData = null;
            //将源数据保存为图片
            var file = new FileInfo(Path.Combine("temp", "IDCARD.bmp"));
            if (!file.Directory.Exists) file.Directory.Create();
            var dir = new DirectoryInfo(Path.Combine("temp", "IDCARD"));
            try
            {
                File.WriteAllBytes(file.FullName, data);
                if (file.Exists)
                {
                    if (!dir.Exists) dir.Create();
                    var f = new FileInfo(Path.Combine(dir.FullName, "去边身份证.bmp"));
                    var ret = IDCARD_CUT_EDGE(file.FullName, false, dir.FullName, f.Name, false, 1);
                    if (ret == 0 && f.Exists)
                    {
                        retData = File.ReadAllBytes(f.FullName);
                    }
                }
            }
            catch
            {
            }
            finally
            {
                File.Delete(file.FullName);
                Directory.Delete(dir.FullName, true);
            }
            return retData;
        }
    }
}
