// 模拟依赖的类和方法
class FileSystemInfo {
    exists: boolean;
    fullName: string;

    constructor(exists: boolean, fullName: string) {
        this.exists = exists;
        this.fullName = fullName;
    }
}

class DirectoryInfo extends FileSystemInfo {
    constructor(exists: boolean, fullName: string) {
        super(exists, fullName);
    }

    getFileSystemInfos(): FileSystemInfo[] {
        // 模拟实现，实际需要根据文件系统操作实现
        return [];
    }

    delete(recursive: boolean): void {
        // 模拟删除操作
    }
}

class TradeHandle {
    static FtpUpLoad(fileName: string, path: string, fileData: number[]): number {
        // 模拟 FTP 上传操作
        return 0;
    }
}

class LoginUserInfo {
    static get TradeDate(): string {
        return "";
    }
}

class LoggerFTP {
    static FileName: string = "Logs";
    private static fileData: number[] | null = null;
    private static filePath: string = "";
    private static localLogPath: string[] = [];

    static IsExistLogFile(file: FileSystemInfo, currentDate: string): boolean {
        if (!file.exists) {
            return false;
        }
        const dire = file as DirectoryInfo;
        if (!dire) {
            return false;
        }
        const files = dire.getFileSystemInfos();
        const fileCount = files.filter(x => parseFloat(x.fullName.split('/').pop() || '') < parseFloat(currentDate));
        return fileCount.length > 0;
    }

    private static getFiles(path: string): string[] {
        const strFileNames: string[] = []; // 模拟获取文件列表
        const strDirectories: string[] = []; // 模拟获取子目录列表

        strFileNames.forEach(filename => {
            this.localLogPath.push(filename);
        });

        strDirectories.forEach(dir => {
            this.getFiles(dir);
        });

        return this.localLogPath;
    }

    static GetFileSystemInfos(file: FileSystemInfo): void {
        this.localLogPath = [];
        if (!file.exists) {
            return;
        }
        const list = this.getFiles(file.fullName);

        list.forEach(item => {
            if (!item.includes(LoginUserInfo.TradeDate)) {
                const upLastName = item.substring(item.indexOf(this.FileName));
                let upPath = "";
                const upPathArray = upLastName.split('\\');
                for (let i = 0; i < upPathArray.length; i++) {
                    if (i < upPathArray.length - 1) {
                        upPath += upPathArray[i] + "/";
                    }
                }
                // 模拟读取文件字节数据
                this.fileData = []; 
                let result = 0;
                try {
                    result = TradeHandle.FtpUpLoad(upPathArray[upPathArray.length - 1], upPath, this.fileData);
                } catch (e) {
                    // 忽略异常
                }
            }
        });
    }

    static ClearFiles(file: FileSystemInfo): void {
        const dire = file as DirectoryInfo;
        if (!dire) {
            return;
        }
        const files = dire.getFileSystemInfos();
        let date = "";
        if (LoginUserInfo.TradeDate.length === 8) {
            date = `${LoginUserInfo.TradeDate.substring(0, 4)}-${LoginUserInfo.TradeDate.substring(4, 2)}-${LoginUserInfo.TradeDate.substring(6, 2)}`;
        }
        let dt = new Date();
        if (isNaN(Date.parse(date))) {
            return;
        }
        dt = new Date(date);
        const dateList: string[] = [
            LoginUserInfo.TradeDate,
            new Date(dt.getTime() - 24 * 60 * 60 * 1000).toISOString().slice(0, 10).replace(/-/g, ''),
            new Date(dt.getTime() - 2 * 24 * 60 * 60 * 1000).toISOString().slice(0, 10).replace(/-/g, '')
        ];

        files.forEach(fileInfo => {
            let flag = true;
            const finame = fileInfo.fullName.split('/').pop() || '';
            dateList.forEach(dateItem => {
                if (finame === dateItem) {
                    flag = false;
                }
            });
            if (flag) {
                const dirInfo = new DirectoryInfo(true, fileInfo.fullName);
                dirInfo.delete(true);
            }
        });
    }
}

export { LoggerFTP };