// 引入必要的模块
import { CultureInfo, DateTime, DateTimeStyles } from 'typescript-dotnet-commonjs';

// 定义 MessageBase 类
export class MessageBase {
    // 消息发送者
    public sender: any;
    // 消息预期接收者
    public target: any;

    // 无参构造函数
    constructor();
    // 带 sender 参数的构造函数
    constructor(sender: any);
    // 带 sender 和 target 参数的构造函数
    constructor(sender?: any, target?: any) {
        if (sender) {
            this.sender = sender;
        }
        if (target) {
            this.target = target;
        }
    }

    /**
     * 格式化日期时间字符串
     * @param dateTime 待格式化的日期时间字符串，格式为 'yyyyMMddHHmmss'
     * @returns 格式化后的日期时间字符串，格式为 'yyyy-MM-dd HH:mm:ss'
     */
    protected formatDateTime(dateTime: string): string {
        let ret = '';
        try {
            const parsedDateTime = DateTime.parseExact(
                dateTime,
                'yyyyMMddHHmmss',
                new CultureInfo('zh-CN', true),
                DateTimeStyles.AllowInnerWhite
            );
            ret = parsedDateTime.toString('yyyy-MM-dd HH:mm:ss');
        } catch (error) {
            console.error('日期时间格式化失败:', error);
        }
        return ret;
    }
}