﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Windows;
using System.Windows.Documents;
using Platform.Common.RunningParameters;
using Platform.Library;
using Platform.Presentation.Interfaces;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Library.Ext.TradeInterfaces;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Library.Ext.TradeExtension
{
    /// <summary>
    ///     NotificationMessageBox.xaml 的交互逻辑
    /// </summary>
    public partial class NotificationMessageBox : INotification
    {
        #region Field
        private static NotificationMessageBox _instance;

        private static dynamic _mainPage;
        #endregion

        #region Property
        /// <summary>
        /// 单例实体
        /// </summary>
        public static NotificationMessageBox Instance
        {
            get
            {
                if (_instance == null)
                {
                    _instance = new NotificationMessageBox();
                }
                return _instance;
            }
        }

        #endregion
        
        #region DependencyProperty
        /// <summary>
        /// 数据源
        /// </summary>
        NotificationMessage Source
        {
            get { return (NotificationMessage)GetValue(SourceProperty); }
            set { SetValue(SourceProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Source.  This enables animation, styling, binding, etc...
        /// <summary>
        /// 消息数据源
        /// </summary>
        public static readonly DependencyProperty SourceProperty =
            DependencyProperty.Register("Source", typeof(NotificationMessage), typeof(NotificationMessageBox));
        #endregion

        #region Ctor
        /// <summary>
        ///     构造通知窗体对象
        /// </summary>
        private NotificationMessageBox()
        {
            InitializeComponent();
        }
        #endregion

        #region Method
        /// <summary>
        /// 注册监听
        /// </summary>
        public void Register()
        {
            Messenger.Default.Register(this, new Action<NotificationMessage>(NewMessage));
        }

        /// <summary>
        /// 注销监听
        /// </summary>
        public void Unregister()
        {
            Messenger.Default.Unregister<NotificationMessage>(this);
        }

        /// <summary>
        /// 初始化监听
        /// </summary>
        public static void Init()
        {
            Instance.Register();
        }

        /// <summary>
        /// 新消息提示
        /// </summary>
        /// <param name="message"></param>
        private void NewMessage(NotificationMessage message)
        {
            if (message == null)
                return;

            if (_mainPage == null)
            {
                _mainPage = MainPageManager.GetMainPage();

                if (_mainPage == null)
                    throw new Exception("无法找到主界面！");
            }

            var messageBox = new NotificationMessageBox { Source = message };

            _mainPage.ShowBottomDialog("消息提示", messageBox);
        }
        #endregion

        #region Event
        /// <summary>
        /// 打开消息盒子
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Hlink_OpenMessageBox_Click(object sender, RoutedEventArgs e)
        {
            var hyperlink = sender as Hyperlink;

            if (hyperlink == null || string.IsNullOrWhiteSpace(hyperlink.Tag.ToString()))
                return;

            //显示公告消息盒子
            string userid = PlatformManagerHandle.Get_TT_LOGINUSER_UserID(LoginUserInfo.TellerNo.Trim());
            dynamic mainPage = MainPageManager.GetMainPage();
            if (mainPage != null)
            {
                try
                {
                    mainPage.ShowNotifyBox(userid, hyperlink.Tag.ToString());
                }
                catch (Exception)
                {
                }
            }
        }
        #endregion
    }
}