﻿using System;
using System.Collections.Generic;
using System.Windows;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Library.Ext.TradeInterfaces;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Library.Ext.TradeExtension
{
    /// <summary>
    ///     NotificationMessageBox.xaml 的交互逻辑
    /// </summary>
    public partial class TaskMessageBox : INotification
    {
        #region Field

        private static TaskMessageBox _instance;

        private static dynamic _mainPage;

        #endregion

        #region Property

        /// <summary>
        ///     单例实体
        /// </summary>
        public static TaskMessageBox Instance
        {
            get
            {
                if (_instance == null)
                {
                    _instance = new TaskMessageBox();
                }
                return _instance;
            }
        }

        #endregion

        #region DependencyProperty

        /// <summary>
        /// 消息数据源
        /// </summary>
        public static readonly DependencyProperty SourceProperty =
            DependencyProperty.Register("Source", typeof (TaskMessage), typeof (TaskMessageBox));

        /// <summary>
        ///     数据源
        /// </summary>
        private TaskMessage Source
        {
            get { return (TaskMessage) GetValue(SourceProperty); }
            set { SetValue(SourceProperty, value); }
        }

        #endregion

        #region Ctor

        /// <summary>
        ///     构造通知窗体对象
        /// </summary>
        public TaskMessageBox()
        {
            InitializeComponent();
        }

        #endregion

        #region Event

        /// <summary>
        ///     执行按钮响应事件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_Perform_PreviewKeyDown(object sender, RoutedEventArgs e)
        {
            Dictionary<string, string> tradedata = WorkFlowHandle.GetTradedataBySerialNo(Source.Key);

            if (tradedata == null)
            {
                btn_Perform.IsEnabled = false;
                return;
            }

            if (tradedata.ContainsKey("WorkItemId"))
            {
                tradedata["WorkItemId"] = Source.Other;
            }
            else
            {
                tradedata.Add("WorkItemId", Source.Other);
            }
            if ("Restore".Equals(Source.Trade)) //交易还原
            {
                // 这里授权结果使用的的是AuthFlag，复核使用的是SuperFlag
                var bus = new TradeBusData();
                foreach (var item in tradedata)
                    bus[item.Key] = item.Value;
                //TODO:20140422
                //this.RestoreTradePage(tradedata["SerialId"], bus);
                btn_Perform.IsEnabled = false;
            }
            else //打开新交易
            {
                var d = new TradeBusData();
                foreach (var item in tradedata)
                {
                    d[item.Key] = item.Value;
                }
                btn_Perform.IsEnabled = false;
                this.OpenTradePage(Source.Trade, TradePageExtension.OpenOptions.NewPage, null, d);
            }
        }

        #endregion

        #region Method

        /// <summary>
        ///     注册监听
        /// </summary>
        public void Register()
        {
            Messenger.Default.Register(this, new Action<TaskMessage>(NewMessage));
        }

        /// <summary>
        ///     注销监听
        /// </summary>
        public void Unregister()
        {
            Messenger.Default.Unregister<TaskMessage>(this);
        }


        /// <summary>
        ///     初始化监听
        /// </summary>
        public static void Init()
        {
            Instance.Register();
        }

        /// <summary>
        ///     新消息提示
        /// </summary>
        /// <param name="message"></param>
        private void NewMessage(TaskMessage message)
        {
            if (message == null)
                return;

            if (_mainPage == null)
            {
                _mainPage = MainPageManager.GetMainPage();

                if (_mainPage == null)
                    throw new Exception("无法找到主界面！");
            }

            var messageBox = new TaskMessageBox {Source = message};

            _mainPage.ShowBottomDialog("消息提示", messageBox);
        }

        #endregion
    }
}