﻿using System;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class PageController
    {
        /// <summary>
        /// 当前页数
        /// </summary> 
        public int CurPage { get; set; }
        /// <summary>
        /// 总页数
        /// </summary> 
        public int TotalPage { get; set; }
        /// <summary>
        /// 首页
        /// </summary> 
        public event EventHandler EventFristPage;
        /// <summary>
        /// 末页
        /// </summary>
        public event EventHandler EventLastPage;
        /// <summary>
        /// 中间页
        /// </summary>
        public event EventHandler EventPage;
        /// <summary>
        /// 只有一页
        /// </summary>
        public event EventHandler EventOnlyPage;


        /// <summary>
        /// 初始化构造方法
        /// </summary>
        public PageController()
        {
            this.CurPage = 1;
            this.TotalPage = 1;
        }

        /// <summary>
        /// 计算页数
        /// </summary>
        public void CountPage(int totalPage)
        {
            this.TotalPage = totalPage; 
            FireEvent();
        }


        /// <summary>
        /// 首页
        /// </summary>
        public void FristPage()
        {
            this.CurPage = 1;
            FireEvent();
        }

        /// <summary>
        /// 上一页
        /// </summary>
        public void PreviousPage()
        {
            this.CurPage--;
            if (this.CurPage < 1)
            {
                this.CurPage = 1;
            }
            FireEvent();
        }
        /// <summary>
        /// 下一页
        /// </summary>
        public void NextPage()
        {
            this.CurPage++;
            if (this.CurPage > this.TotalPage)
            {
                this.CurPage = this.TotalPage;
            }
            FireEvent();
        }

        /// <summary>
        /// 末页
        /// </summary>
        public void LastPage()
        {
            this.CurPage = this.TotalPage;
            FireEvent();
        }

        /// <summary>
        /// 调用事件
        /// </summary>
        public void FireEvent()
        {
            if (this.TotalPage < 2)
            {
                if (EventOnlyPage != null)
                {
                    EventOnlyPage(this, EventArgs.Empty);
                    return;
                }
            }
            if (CurPage == 1)
            {
                if (EventFristPage != null)
                {
                    EventFristPage(this, EventArgs.Empty);
                    return;
                }
            }
            if (CurPage == this.TotalPage)
            {
                if (EventLastPage != null)
                {
                    EventLastPage(this, EventArgs.Empty);
                    return;
                }
            }
            if (CurPage > 1 && CurPage < this.TotalPage)
            {
                if (EventPage != null)
                {
                    EventPage(this, EventArgs.Empty);
                    return;
                }
            }
        }
    }
}
