// 定义事件处理函数类型
type EventHandler = (sender: PageController, args: EventArgs) => void;

// 定义 EventArgs 类
class EventArgs {
    static Empty = new EventArgs();
}

class PageController {
    // 当前页数
    public CurPage: number;
    // 总页数
    public TotalPage: number;
    // 首页事件
    public EventFristPage?: EventHandler;
    // 末页事件
    public EventLastPage?: EventHandler;
    // 中间页事件
    public EventPage?: EventHandler;
    // 只有一页事件
    public EventOnlyPage?: EventHandler;

    constructor() {
        this.CurPage = 1;
        this.TotalPage = 1;
    }

    // 计算页数
    public CountPage(totalPage: number): void {
        this.TotalPage = totalPage;
        this.FireEvent();
    }

    // 首页
    public FristPage(): void {
        this.CurPage = 1;
        this.FireEvent();
    }

    // 上一页
    public PreviousPage(): void {
        this.CurPage--;
        if (this.CurPage < 1) {
            this.CurPage = 1;
        }
        this.FireEvent();
    }

    // 下一页
    public NextPage(): void {
        this.CurPage++;
        if (this.CurPage > this.TotalPage) {
            this.CurPage = this.TotalPage;
        }
        this.FireEvent();
    }

    // 末页
    public LastPage(): void {
        this.CurPage = this.TotalPage;
        this.FireEvent();
    }

    // 调用事件
    public FireEvent(): void {
        if (this.TotalPage < 2) {
            if (this.EventOnlyPage) {
                this.EventOnlyPage(this, EventArgs.Empty);
                return;
            }
        }
        if (this.CurPage === 1) {
            if (this.EventFristPage) {
                this.EventFristPage(this, EventArgs.Empty);
                return;
            }
        }
        if (this.CurPage === this.TotalPage) {
            if (this.EventLastPage) {
                this.EventLastPage(this, EventArgs.Empty);
                return;
            }
        }
        if (this.CurPage > 1 && this.CurPage < this.TotalPage) {
            if (this.EventPage) {
                this.EventPage(this, EventArgs.Empty);
                return;
            }
        }
    }
}

export { PageController };