﻿using System.Collections.Generic;
using System.Windows.Controls;
using Platform.Controls.ControlsHelper;
using Platform.Library;
using Platform.Presentation.Interfaces;
using TellerSystem.Communication;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Library.Ext.Variables;

namespace TellerSystem.Library.Ext.TradeExtension
{
    /// <summary>
    /// ComboBox实体数据层
    /// </summary>
    public class PerCombBoxData
    {
        private static PerCombBoxData instance;
        private static object _lock = new object();
        private List<PerCombBoxData> comboboxList = new List<PerCombBoxData>();

        private PerCombBoxData()
        {
            _combBoxKeyValue = new Dictionary<string, string>();
        }

        /// <summary>
        /// 初始化一个PerCombBoxData,(单例模式)
        /// </summary>
        /// <returns></returns>
        public static PerCombBoxData GetInstance()
        {
            if (instance == null)
            {
                lock (_lock)
                {
                    if (instance == null)
                    {
                        instance = new PerCombBoxData();
                    }
                }
            }
            return instance;
        }

        private IPage _tradeName;
        private string _combBoxName;
        private string _combBoxMark;
        private Dictionary<string, string> _combBoxKeyValue;

        /// <summary>
        /// 交易名称
        /// </summary>
        public IPage TradeName
        {
            get { return _tradeName; }
            set { _tradeName = value; }
        }

        /// <summary>
        /// ComboBox名称
        /// </summary>
        public string CombBoxName
        {
            get { return _combBoxName; }
            set { _combBoxName = value; }
        }

        /// <summary>
        /// 标识符,同一个交易，同一个Combobox，不同的数据
        /// </summary>
        public string CombBoxNumber
        {
            get { return _combBoxMark; }
            set { _combBoxMark = value; }
        }

        /// <summary>
        /// ComboBox key and Value
        /// </summary>
        public Dictionary<string, string> CombBoxKeyValue
        {
            get { return _combBoxKeyValue; }
            set { _combBoxKeyValue = value; }
        }

        /// <summary>
        /// 把Combobox的值添加列表中
        /// </summary>
        /// <param name="target">Ipage对象</param>
        /// <param name="comboBox">ComboBox对象</param>
        /// <param name="msg">Message通讯对象</param>
        /// <param name="formatString">Combobox展现格式</param>
        /// <param name="defalutCombox">默认Combobox标识符</param>
        /// <returns>modify slzh 因涉及业务 返回错误描述，方便界面提示</returns>
        public Result SetComboBoxList(IPage target, ComboBox comboBox, Message msg, string formatString, string defalutCombox = "Null")
        {
            var result = new Result();
            if (comboBox == null || msg == null || string.IsNullOrEmpty(defalutCombox))
            {
                result.Code = "Erro";
                result.Desc = "系统异常：未设置ComboBox或Message对象";
                return result;
            }
            foreach (var comboboxData in comboboxList)
            {
                if (comboboxData.TradeName.GetType().Name == target.GetType().Name && comboboxData.CombBoxName == comboBox.Name && comboboxData.CombBoxNumber == defalutCombox)
                {
                    comboBox.SetItems(comboboxData.CombBoxKeyValue);
                    return result;
                }
            }
            PerCombBoxData _perCombBoxDate = new PerCombBoxData();
            var boolcallServer = SystemFunction.CallServer(target, msg);
            if (!boolcallServer)
            {
                result.Code = "Erro";
                result.Desc = "CallServer与核心通信失败";
                return result;
            }
            if (msg.Fd12 != "0000")
            {
                result.Code = msg.Fd12;
                result.Desc = target.GetError(msg);
                return result;
            }
            string fileData = msg.FileData;
            var boolSetSel = SystemFunction.SetSel(comboBox, fileData, formatString);
            if (!boolSetSel)
            {
                result.Code = "Erro";
                result.Desc = "系统异常：SetSel设置失败";
                return result;
            }
            _perCombBoxDate.TradeName = target;
            _perCombBoxDate.CombBoxName = comboBox.Name;
            _perCombBoxDate.CombBoxNumber = defalutCombox;
            foreach (KeyValuePair<string, string> item in comboBox.Items)
            {
                _perCombBoxDate.CombBoxKeyValue.Add(item.Key, item.Value);
            }
            comboboxList.Add(_perCombBoxDate);
            return result;
        }
    }
}