// 模拟依赖的类型和函数
interface IPage {}
class ComboBox {
    SetItems(keyValue: { [key: string]: string }) {}
    Items: { [key: string]: string } = {};
    name: string = "";
}
class Message {
    Fd12: string = "";
    FileData: string = "";
}
class Result {
    Code: string = "";
    Desc: string = "";
}
class SystemFunction {
    static CallServer(target: IPage, msg: Message): boolean {
        return true;
    }
    static SetSel(comboBox: ComboBox, fileData: string, formatString: string): boolean {
        return true;
    }
}

class PerCombBoxData {
    private static instance: PerCombBoxData | null = null;
    private static _lock: object = {};
    private comboboxList: PerCombBoxData[] = [];

    private _tradeName: IPage | null = null;
    private _combBoxName: string = "";
    private _combBoxMark: string = "";
    private _combBoxKeyValue: { [key: string]: string } = {};

    private constructor() {
        this._combBoxKeyValue = {};
    }

    static GetInstance(): PerCombBoxData {
        if (!this.instance) {
            // 模拟锁机制
            if (!this.instance) {
                this.instance = new PerCombBoxData();
            }
        }
        return this.instance;
    }

    get TradeName(): IPage | null {
        return this._tradeName;
    }
    set TradeName(value: IPage | null) {
        this._tradeName = value;
    }

    get CombBoxName(): string {
        return this._combBoxName;
    }
    set CombBoxName(value: string) {
        this._combBoxName = value;
    }

    get CombBoxNumber(): string {
        return this._combBoxMark;
    }
    set CombBoxNumber(value: string) {
        this._combBoxMark = value;
    }

    get CombBoxKeyValue(): { [key: string]: string } {
        return this._combBoxKeyValue;
    }
    set CombBoxKeyValue(value: { [key: string]: string }) {
        this._combBoxKeyValue = value;
    }

    SetComboBoxList(target: IPage, comboBox: ComboBox, msg: Message, formatString: string, defalutCombox: string = "Null"): Result {
        const result = new Result();
        if (!comboBox || !msg || defalutCombox === "") {
            result.Code = "Erro";
            result.Desc = "系统异常：未设置ComboBox或Message对象";
            return result;
        }

        for (const comboboxData of this.comboboxList) {
            if (
                comboboxData.TradeName?.constructor.name === target.constructor.name &&
                comboboxData.CombBoxName === comboBox.name &&
                comboboxData.CombBoxNumber === defalutCombox
            ) {
                comboBox.SetItems(comboboxData.CombBoxKeyValue);
                return result;
            }
        }

        const perCombBoxDate = new PerCombBoxData();
        const boolcallServer = SystemFunction.CallServer(target, msg);
        if (!boolcallServer) {
            result.Code = "Erro";
            result.Desc = "CallServer与核心通信失败";
            return result;
        }

        if (msg.Fd12 !== "0000") {
            result.Code = msg.Fd12;
            // 模拟获取错误信息
            result.Desc = "获取错误信息"; 
            return result;
        }

        const fileData = msg.FileData;
        const boolSetSel = SystemFunction.SetSel(comboBox, fileData, formatString);
        if (!boolSetSel) {
            result.Code = "Erro";
            result.Desc = "系统异常：SetSel设置失败";
            return result;
        }

        perCombBoxDate.TradeName = target;
        perCombBoxDate.CombBoxName = comboBox.name;
        perCombBoxDate.CombBoxNumber = defalutCombox;
        for (const key in comboBox.Items) {
            perCombBoxDate.CombBoxKeyValue[key] = comboBox.Items[key];
        }
        this.comboboxList.push(perCombBoxDate);

        return result;
    }
}

export { PerCombBoxData };