// 定义 Province、City 和 County 类
class County {
    constructor(public Value: string, public Key: string) {}
}

class City {
    Child: County[];
    constructor(public Value: string, public Key: string) {
        this.Child = [];
    }
}

class Province {
    Child: City[];
    constructor(public Value: string, public Key: string) {
        this.Child = [];
    }
}

// 模拟 DesignerProperties 类
class DesignerProperties {
    static GetIsInDesignMode(_: any): boolean {
        return false;
    }
}

// 定义 ProvinceCityCountyManager 类
class ProvinceCityCountyManager {
    private static _instance: ProvinceCityCountyManager;
    private _provinces: Province[];

    private constructor() {
        this._provinces = [];
    }

    static get Instance(): ProvinceCityCountyManager {
        if (!this._instance) {
            this._instance = new ProvinceCityCountyManager();
        }
        return this._instance;
    }

    async GetProvinces(): Promise<Province[]> {
        if (this._provinces.length > 0) {
            return this._provinces;
        }

        if (DesignerProperties.GetIsInDesignMode({})) {
            return [];
        }

        try {
            const response = await fetch('Config/PCC.xml');
            const xmlText = await response.text();
            const parser = new DOMParser();
            const doc = parser.parseFromString(xmlText, 'text/xml');

            const provinceNodes = doc.querySelectorAll('/Root/Province');
            const provinces: Province[] = [];

            provinceNodes.forEach((provinceNode) => {
                const provinceCode = provinceNode.getAttribute('Code') || '';
                const provinceText = provinceNode.getAttribute('Text') || '';
                const province = new Province(provinceCode, provinceText);

                const cityNodes = provinceNode.querySelectorAll('City');
                cityNodes.forEach((cityNode) => {
                    const cityCode = cityNode.getAttribute('Code') || '';
                    const cityText = cityNode.getAttribute('Text') || '';
                    const city = new City(cityCode, cityText);

                    const countyNodes = cityNode.querySelectorAll('County');
                    countyNodes.forEach((countyNode) => {
                        const countyCode = countyNode.getAttribute('Code') || '';
                        const countyText = countyNode.getAttribute('Text') || '';
                        const county = new County(countyCode, countyText);
                        city.Child.push(county);
                    });

                    province.Child.push(city);
                });

                provinces.push(province);
            });

            this._provinces = provinces;
            return provinces;
        } catch (error) {
            console.error('加载省市区数据时出错:', error);
            return [];
        }
    }
}

export default ProvinceCityCountyManager;