﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Platform.Controls.ControlsHelper;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class RegexRuleExtension
    {
        #region ctor
        static RegexRuleExtension()
        {
            LoadData();
        }

        /// <summary>
        /// 标志-防止重复加载
        /// </summary>
        static bool flag = false;

        /// <summary>
        /// 加载数据
        /// </summary>
        public static void LoadData()
        {
            if (flag) return;
            flag = true;
            //静态构造方法，加载输入规则
            RegexValidationRules.Instance.AddRule(InputRules.None.ToString(), "", "");
           // RegexValidationRules.Instance.AddRule(InputRules.Telephone.ToString(), @"^\s*$|(\(\d{3,4}\)|\d{3,4}-|\s)?\d{7}", "请输入正确的固定电话号码！");
            RegexValidationRules.Instance.AddRule(InputRules.Telephone.ToString(), @"^([0-9]{3,4}\-[0-9]{6,8}$)|(^\([0-9]{3,4}\)[0-9]{6,8}$)", "请输入正确的固定电话号码,电话加区号！");
            RegexValidationRules.Instance.AddRule(InputRules.Fax.ToString(), @"^\s*$|(\(\d{3,4}\)|\d{3,4}-|\s)?\d{7}", "请输入正确的传真号码！");
            //RegexValidationRules.Instance.AddRule(InputRules.MobilePhone.ToString(), @"^\s*$|1[0-9]{10}", "请输入正确的手机号码！");
            RegexValidationRules.Instance.AddRule(InputRules.MobilePhone.ToString(), @"^1[0-9]{10}$", "请输入正确的手机号码，11位！");
            RegexValidationRules.Instance.AddRule(InputRules.NumberStr.ToString(), @"^\s*$|^[0-9]*$", "请输入数字串！");
            RegexValidationRules.Instance.AddRule(InputRules.PaymentSystem.ToString(), "^[^:%'\"]*$", "不允许使用支付系统规定的保留字！");
            RegexValidationRules.Instance.AddRule(InputRules.NumLetter.ToString(), @"^\s*$|^[A-Za-z0-9]*$", "只允许输入数字和字母！");
            RegexValidationRules.Instance.AddRule(InputRules.NameLetter.ToString(), @"^\s*$|^[a-zA-Z0-9\u4e00-\u9fa5]*$", "只允许输入数字、字母、中文！");
            RegexValidationRules.Instance.AddRule(InputType.Date.ToString(), @"(^99999999$)|(^(?:(?:(?:(?:(?:1[6-9]|[2-9]\d)(?:0[48]|[2468][048]|[13579][26])|(?:(?:16|[2468][048]|[3579][26])00)))(?:0229))|(?:(?:(?:1[6-9]|[2-9]\d)\d{2})(?:(?:(?:0[13578]|1[02])31)|(?:(?:0[13-9]|1[0-2])(?:29|30))|(?:(?:0[1-9])|(?:1[0-2]))(?:0[1-9]|1\d|2[0-8]))))$)", "日期格式20120101，请重新输入。");
            RegexValidationRules.Instance.AddRule(InputRules.RateLetter.ToString(), @"^(-?\d+)(\.\d+)?{0,6}$", "只允许输入浮点数,且小数点后6位");
            RegexValidationRules.Instance.AddRule(InputRules.NoChinese.ToString(), @"^\s*$|[^\u4e00-\u9fa5]", "不允许输入中文！");
            RegexValidationRules.Instance.AddRule(InputRules.TelephoneOrMobilePhone.ToString(), @"^([0-9]{3,4}\-[0-9]{6,8}$)|(^1[0-9]{10}$)|(^\([0-9]{3,4}\)[0-9]{6,8}$)", "手机号11位，电话需要增加区号！");
            RegexValidationRules.Instance.AddRule(InputRules.DoubleNumBer.ToString(), @"^\d+(\.\d+)?$", "只允许输入整数或小数");
            RegexValidationRules.Instance.AddRule(InputRules.DateZM.ToString(), @"(^\d{4}(0[1-9]|1[0-2])$)|(^99999999$)|(^(?:(?:(?:(?:(?:1[6-9]|[2-9]\d)(?:0[48]|[2468][048]|[13579][26])|(?:(?:16|[2468][048]|[3579][26])00)))(?:0229))|(?:(?:(?:1[6-9]|[2-9]\d)\d{2})(?:(?:(?:0[13578]|1[02])31)|(?:(?:0[13-9]|1[0-2])(?:29|30))|(?:(?:0[1-9])|(?:1[0-2]))(?:0[1-9]|1\d|2[0-8]))))$)", "日期格式20120101或者201201，请重新输入。");
            
      }
        #endregion

        #region InputRule - 输入规则
        /// <summary>
        /// 指定用户可输入的数据类型
        /// </summary>
        public static readonly DependencyProperty InputRuleProperty =
            DependencyProperty.RegisterAttached("InputRule", typeof(InputRules), typeof(RegexRuleExtension),
                new UIPropertyMetadata((InputRules.None),
                    new PropertyChangedCallback(OnInputRuleChanged)));
        public static InputRules GetInputType(DependencyObject d)
        {
            return (InputRules)d.GetValue(InputRuleProperty);
        }
        public static void SetInputType(DependencyObject d, InputRules value)
        {
            d.SetValue(InputRuleProperty, value);
        }
        public static void OnInputRuleChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            //直接引导到底层输入控制，具体规则由本类的静态构造方法产生
            TextBoxHelper.SetInputRule(d, e.NewValue.ToString());
        }
        #endregion
    }

    /// <summary>
    /// 输入规则
    /// </summary>
    public enum InputRules
    {
        /// <summary>
        /// 无限制
        /// </summary>
        None,
        /// <summary>
        /// 固定电话
        /// </summary>
        Telephone,
        /// <summary>
        /// 移动电话
        /// </summary>
        MobilePhone,
        /// <summary>
        /// 传真
        /// </summary>
        Fax,
        /// <summary>
        /// 数字串
        /// </summary>
        NumberStr,
        /// <summary>
        /// 支付系统保留字控制
        /// </summary>
        PaymentSystem,
        /// <summary>
        /// 数字字母
        /// </summary>
        NumLetter,
        /// <summary>
        /// 名称串
        /// </summary>
        NameLetter,
        /// <summary>
        /// 正Double数字
        /// </summary>
        RateLetter,
        /// <summary>
        /// 非中文
        /// </summary>
        NoChinese,
        /// <summary>
        /// 电话或手机
        /// </summary>
        TelephoneOrMobilePhone,
        /// <summary>
        /// 正Double数字
        /// </summary>
        DoubleNumBer,
        /// <summary>
        /// 年月日或者年月
        /// </summary>
        DateZM
    }
}