﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.Communication;
using System.Management;
using System.Security.Cryptography;
using Platform.Common.RunningParameters;
using System.Windows;
using TellerSystem.Communication.SocketListener;
using TellerSystem.ServiceProxy.Ext.TradeBusinessEntitys;
using System.Runtime.Remoting.Contexts;
using System.Runtime.CompilerServices;
using System.Windows.Controls;
using Platform.Presentation.Interfaces;
using Microsoft.Win32;
using System.Text.RegularExpressions;
using Platform.Controls.ControlsHelper;

namespace TellerSystem.Library.Ext.TradeExtension
{
    /// <summary>
    /// 单点登录控制
    /// </summary>
    public class LoginAuthentication
    {
        #region 单例模式
        private static LoginAuthentication _instance;
        /// <summary>
        /// 获取单列
        /// </summary>
        /// <returns></returns>
        public static LoginAuthentication Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new LoginAuthentication();
                return _instance;
            }
        }
        #endregion

        #region 私有变量
        private string _tellerNo = string.Empty;
        #endregion

        #region 属性
        /// <summary>
        /// 开户行
        /// </summary>
        public string PaymentNo { get; set; }
        /// <summary>
        /// 用户ID
        /// </summary>
        public string USERID { get; private set; }
        /// <summary>
        /// 用户登录账号
        /// </summary>
        public string USERCODE { get; private set; }
        /// <summary>
        /// 姓名
        /// </summary>
        public string USERNAME { get; private set; }
        /// <summary>
        /// 是否单客户端
        /// </summary>
        public bool ISPIONT { get; private set; }
        /// <summary>
        /// 锁定标志
        /// </summary>
        public bool ISLOGIN { get; private set; }
        /// <summary>
        /// 所属机构
        /// </summary>
        public string KINBRID { get; private set; }
        /// <summary>
        /// ATM管理柜员
        /// </summary>
        public string MNGATMUSER { get; private set; }
        /// <summary>
        /// 用户密码
        /// </summary>
        public string USERPSW { get; private set; }
        /// <summary>
        /// IC卡ID
        /// </summary>
        public string CARDID { get; private set; }
        /// <summary>
        /// 上次修改密码日期
        /// </summary>
        public string PWDCHGDATE { get; private set; }
        /// <summary>
        /// 上次登录IP
        /// </summary>
        public string LOGINIP { get; private set; }
        /// <summary>
        /// 上次登录端口号
        /// </summary>
        public string PORTNO { get; private set; }
        /// <summary>
        /// 登录验证方式
        /// </summary>
        public string PSWLEVEL { get; private set; }
        /// <summary>
        /// 上次修改验证方式日期
        /// </summary>
        public string LEVELCHGDATE { get; private set; }
        /// <summary>
        /// 登录验证方式有效期
        /// </summary>
        public int LEVELDATE { get; private set; }
        /// <summary>
        /// IC卡管理级别
        /// </summary>
        public string CARDLEVEL { get; private set; }
        /// <summary>
        /// 登录修改密码
        /// </summary>
        public string ISCHANGEPSW { get; private set; }
        /// <summary>
        /// 上次登录时间
        /// </summary>
        public string BEFOREDATE { get; private set; }
        /// <summary>
        /// 连续密码错误次数
        /// </summary>
        public int PSWERRCOUNT { get; private set; }
        /// <summary>
        /// 上次密码错误时间
        /// </summary>
        public string PSWTIME { get; private set; }
        /// <summary>
        /// 当前系统日期
        /// </summary>
        public string DATE { get; private set; }
        /// <summary>
        /// 当前柜员状态
        /// </summary>
        public string CHENKSIGN { get; private set; }
        /// <summary>
        /// 最大密码修改周期
        /// </summary>
        public string MaxPswCycle { get; private set; }
        /// <summary>
        /// 是否允许强制登陆
        /// </summary>
        public bool CanForceLogin { get; private set; }
        /// <summary>
        /// 获取或设置机构号
        /// </summary>
        public string KinbrNo { get; private set; }
        /// <summary>
        /// 获取或设置是否跨终端登录
        /// </summary>
        public string IsAcross { get; private set; }

        /// <summary>
        /// 是否允许多角色登录
        /// </summary>
        public bool CanGroupsLogin { get; private set; }

        /// <summary>
        /// Gets IP addresss
        /// </summary>
        public string IPAddress { get; private set; }

        /// <summary>
        /// 用户角色
        /// </summary>
        public List<TT_RoleGroup> Usergroup { get; private set; }
        /// <summary>
        /// 执行强制下线操作
        /// </summary>
        public bool SetUserDown { get; private set; }
        /// <summary>
        /// 是否移动端
        /// </summary>
        public bool Ismobile { get; private set; }

        /// <summary>
        /// 柜员选择角色
        /// </summary>
        public string TellerGroupName
        {
            get
            {
                var ret = string.Empty;
                if (Usergroup != null && Usergroup.Count > 0)
                {
                    var names = new StringBuilder();
                    foreach (var item in Usergroup)
                    {
                        names.Append("," + item.GroupName);
                    }
                    ret = names.ToString().Substring(1);
                }
                return ret;
            }
        }

        /// <summary>
        /// 柜员选择角色值
        /// </summary>
        public string TellerGroupValue
        {
            get
            {
                var ret = string.Empty;
                if (Usergroup != null && Usergroup.Count > 0)
                {
                    var values = new StringBuilder();
                    foreach (var item in Usergroup)
                    {
                        values.Append("," + item.GroupId);
                    }
                    ret = values.ToString().Substring(1);
                }
                return ret;
            }
        }

        /// <summary>
        /// 凭证管理上级机构
        /// </summary>
        public string UpNoteKinbrno { get; private set; }
        /// <summary>
        /// 现金管理上级机构
        /// </summary>
        public string UpCashKinbrno { get; private set; }

        /// <summary>
        /// 上级机构凭证管理员
        /// </summary>
        public string UpNoteteller { get; private set; }
        /// <summary>
        /// 柜员选择角色值
        /// </summary>
        public string UpCashteller { get; private set; }
        /// <summary>
        /// 机构类型
        /// </summary>
        public string kinbrtype { get; private set; }

        /// <summary>
        /// 柜员表备用字段--旧柜员号
        /// </summary>
        public string UserComment { get; private set; }

        /// <summary>
        /// 机构表备用字段--旧机构号
        /// </summary>
        public string KinbrComment { get; private set; }
        /// <summary>
        /// 身份证号
        /// </summary>
        public string IdCarNO { get; private set; }

        /// <summary>
        /// 汇总机构
        /// </summary>
        public string GroupKinbrNo { get; private set; }
        #endregion

        #region 私有方法


        ///<summary>
        /// 获得MAC地址
        ///</summary>
        ///<returns></returns>
        [MethodImplAttribute(MethodImplOptions.Synchronized)]
        public string GetMacAddress()
        {
            //务必设置为同步方法，否则多线程调用会造成Com异常
            string strMAC = "";
            ManagementClass MC = new ManagementClass("Win32_NetworkAdapterConfiguration");
            ManagementObjectCollection MOC = MC.GetInstances();
            foreach (ManagementObject moc in MOC)
            {
                if (moc["IPEnabled"].ToString() == "True")
                {
                    strMAC = moc["MacAddress"].ToString();
                }
            }
            return strMAC;
        }

        #endregion

        /// <summary>
        /// 读取注册表信息
        /// </summary>
        /// <returns>激活码</returns>
        public string ReadZhuce()
        {
            string info = "";
            RegistryKey Key;
            Key = Registry.LocalMachine;
            RegistryKey myreg = Key.OpenSubKey("SAM\\TellerSys");
            if (myreg == null)
            {
                return "";
            }
            // myreg = Key.OpenSubKey("software\\test",true);
            info = myreg.GetValue("TellerSys").ToString();
            myreg.Close();
            return info;
        }
        #region 验证方案
        /// <summary>
        /// 初始化数据
        /// </summary>
        /// <returns></returns>
        public bool InitData(string tellerNo)
        {
            //string[] strData = null;
            string zcm = "";
            if (string.Compare(ConfigManager.GetInstance().GetConfigValue("IsStartHardDiskNumber", ConfigType.System), "true", true) == 0)
            {
                zcm = ReadZhuce();
            }

            if (string.IsNullOrEmpty(tellerNo)) return false;
            var data = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.GetLoginDataByUserCode(tellerNo, zcm);
            if (data == null || data.Count == 0)
                return false;
            var loginData = data[0];
            //解析数据
            this.USERID = loginData.UserId;
            this.USERCODE = loginData.UserCode;
            this.USERNAME = loginData.UserName;
            this.ISPIONT = loginData.IsPiont == "1";
            this.ISLOGIN = loginData.IsLogin == "1";
            this.KINBRID = loginData.KinbrId;
            this.MNGATMUSER = loginData.MngatmUser;
            this.USERPSW = loginData.UserPsw;
            this.CARDID = loginData.CardId;
            this.PWDCHGDATE = loginData.PwdChgDate;
            this.LOGINIP = loginData.LoginIp;
            this.PORTNO = string.IsNullOrEmpty(loginData.PortNo) ? "0" : loginData.PortNo;
            this.PSWLEVEL = loginData.PswLevel;
            this.LEVELCHGDATE = loginData.LevelChgDate;
            this.LEVELDATE = string.IsNullOrEmpty(loginData.LevelDate) ? 0 : Convert.ToInt32(loginData.LevelDate);
            this.CARDLEVEL = loginData.CardLevel;
            this.ISCHANGEPSW = loginData.IsChangePsw;
            this.BEFOREDATE = loginData.BeforeDate;
            this.PSWERRCOUNT = string.IsNullOrEmpty(loginData.PswErrCount) ? 0 : Convert.ToInt32(loginData.PswErrCount);
            this.PSWTIME = loginData.PswTime;
            this.DATE = loginData.Date;
            this.CanForceLogin = loginData.CanForceLogin;
            this.MaxPswCycle = loginData.MaxPswCycle;
            this.CHENKSIGN = loginData.ChenkSign;
            this.KinbrNo = loginData.KinbrNo;
            this.IsAcross = loginData.IsAcross;
            this.CanGroupsLogin = loginData.CanGroupsLogin;
            this.UpNoteKinbrno = loginData.UpNoteKinbrno;
            this.UpCashKinbrno = loginData.UpCashKinbrno;
            this.UpNoteteller = loginData.upnoteteller;
            this.UpCashteller = loginData.upcashteller;
            this.kinbrtype = loginData.kinbrtype;
            this.KinbrComment = loginData.kinbrcomment;
            this.UserComment = loginData.usercomment;
            this.IdCarNO = loginData.idCarNo;
            this.GroupKinbrNo = loginData.GroupKinbrNo;
            this.Ismobile = loginData.IsMobile;
            _tellerNo = tellerNo;
            return true;
        }

        #endregion

        #region 公共方法
        /// <summary>
        /// 登陆操作
        /// </summary>
        /// <param name="tellerNo"></param>
        /// <param name="errorMsg"></param>
        /// <returns></returns>
        public bool LoginCheck(out string errorMsg)
        {

            SetUserDown = false;
            errorMsg = string.Empty;
            if (this.PSWLEVEL == "1")//柜员如果启用指纹校验,就没有必要进行密码验证
            {
                return true;
            }
            //是否允许登陆
            if (!ISLOGIN)
            {
                errorMsg = "该柜员已被停用！";
                return false;
            }
            //登陆方式是否过期
            DateTime dateEnd;
            var dateNow = DateTime.Parse(DATE);
            if (DateTime.TryParse(LEVELCHGDATE, out dateEnd))
            {
                if (dateEnd.AddDays(LEVELDATE) < dateNow)
                {
                    errorMsg = "登录验证方式已过期，禁止登录！";
                    return false;
                }
            }
            //是否进行密码更新，决定于密码修改周期
            DateTime lastPwdChangeData;
            double maxPswCycle;

            if (!DateTime.TryParse(PWDCHGDATE, out lastPwdChangeData)
            || (double.TryParse(MaxPswCycle, out maxPswCycle) && lastPwdChangeData.AddDays(maxPswCycle) < dateNow))
            {
                var dialog = new UpdatePassword(this);
                dialog.Owner = Application.Current.MainWindow;
                dialog.ShowDialog();
                if (!dialog.PageResult)
                {
                    errorMsg = "登陆密码过期，需要进行修改！";
                    return false;
                }
                //密码修改完，要求重新加载
                errorMsg = "密码修改成功！";
                return false;
            }
            return true;
        }

        /// <summary>
        /// 获取用户角色列表
        /// </summary>
        /// <param name="tellerNo"></param>
        /// <param name="errorMsg"></param>
        /// <returns></returns>
        public bool GetTellerGroup(out string errorMsg)
        {
            errorMsg = string.Empty;
            //获取角色列表
            Usergroup = TradeManagerHandle.GetRolegroupsbyUsercode(_tellerNo)
                .Where(ttRoleGroup => !string.IsNullOrEmpty(ttRoleGroup.GroupName))
                 .Where(ttRoleGroup => !string.IsNullOrEmpty(ttRoleGroup.GroupId))
                .ToList();
            if (Usergroup == null || Usergroup.Count == 0)
            {
                errorMsg = "获取柜员角色列表出错，柜员角色列表为空！";
                return false;
            }
            ////角色检查，当同时存在现金管理员和凭证管理员且仍存在其他角色时，做区分处理
            //if (IsCashRole && IsVoucherRole && Usergroup.Count > 2)
            //{
            //    var list = Platform.Common.BasicFunctions.DependencyObjectExtensions.FindVisualTreeChildren(Application.Current.MainWindow, x => x is IPage);
            //    if (list.Count == 0)
            //    {
            //        errorMsg = "未找到主界面，请联系科技部！";
            //        return false;
            //    }
            //    //选择98角色
            //    var ret = Platform.Library.MsgHelper.ShowPageMessageBoxMetro(list.First() as IPage, new TextBlock { FontWeight = FontWeights.Bold, FontSize = 20, Text = string.Format("登陆机构：{0}\n登陆柜员：{1}\n您拥有98柜员角色，请选择想要登陆的角色！", LoginUserInfo.KinbrName, LoginAuthentication.Instance.USERNAME) }, "柜员信息", null, "取消", 3, true, true, true, new string[] { "98柜员", "综合柜员", "取消" });
            //    switch (ret)
            //    {
            //        case "98柜员": Usergroup.RemoveAll(x => x.GroupCode != "000001" && x.GroupCode != "000002"); break;
            //        case "综合柜员": Usergroup.RemoveAll(x => x.GroupCode == "000001" || x.GroupCode == "000002"); break;
            //        default:
            //            return false;
            //    }
            //}
            //TellerGroupName = Usergroup[0].GroupName;
            //TellerGroupValue = Usergroup[0].GroupId;
            return true;
        }

        /// <summary>
        /// 现金管理员
        /// </summary>
        public bool IsCashRole
        {
            get
            {
                return Usergroup.Exists(x => x.GroupCode == "000005"); //TellerGroupCode.Contains("000001");
            }
        }

        /// <summary>
        /// 凭证管理员
        /// </summary>
        public bool IsVoucherRole
        {
            get
            {
                return Usergroup.Exists(x => x.GroupCode == "000005");
            }
        }

        /// <summary>
        /// 验证是否允许单柜员多角色多客户端登录
        /// </summary>
        /// <param name="errorMsg"></param>
        /// <param name="tellerNo"></param>
        /// <param name="group"></param>
        /// <returns></returns>
        public bool GroupCheck(string group, out string errorMsg)
        {
            errorMsg = string.Empty;
            if (CHENKSIGN == "1")
            {
                //终端号判定，若终端号相等，则验证通过
                if (String.CompareOrdinal(PORTNO.ToString(), LoginUserInfo.TtyName) == 0)
                {
                    //查找系统配置---是否允许同一柜员使用不同角色同时登录
                    if (TradeManagerHandle.GetMultiRole() != "1")
                    {
                        //查找当前柜员已经登录的角色
                        var groupstr = TradeManagerHandle.GetLogingroupsByUsercode(_tellerNo);
                        //判断是否选择的角色已经登录
                        if (!groupstr.Contains(group))
                        {
                            errorMsg = "用户已选择其他角色登录！";
                            return false;
                        }
                    }
                }
            }
            return true;
        }

        /// <summary>
        /// 当前柜员状态已登录时的登录验证
        /// </summary>
        /// <param name="tellerNo"></param>
        /// <param name="group"></param>
        /// <param name="errorMsg"></param>
        /// <returns></returns>
        public bool SignCheck(string group, out string errorMsg)
        {
            errorMsg = string.Empty;
            //是否已经登陆，若第一次登陆则验证通过
            if (CHENKSIGN == "1")
            {
                //是否单客户端登陆 
                if (ISPIONT)
                {
                    if (String.CompareOrdinal(PORTNO.ToString(), LoginUserInfo.TtyName) != 0)
                    {
                        //是否允许同一柜员使用不同角色同时登录
                        if (CanGroupsLogin)
                        {
                            //查找当前柜员已经登录的角色
                            var groupstr = TradeManagerHandle.GetLogingroupsByUsercode(_tellerNo);
                            //判断是否选择的角色已经登录
                            if (!groupstr.Contains(group))
                            {
                                return true;
                            }
                        }

                        //是否强制登陆
                        if (!CanForceLogin)
                        {
                            errorMsg = "该柜员已经在其他机器上登陆系统，请核查！";
                            return false;
                        }
                        else
                        {
                            SetUserDown = true;
                        }
                    }
                    //同一客户端可以直接登录
                    //不考虑角色
                }

            }
            return true;
        }

        /// <summary>
        /// 验证密码
        /// </summary>
        /// <param name="password"></param>
        /// <returns></returns>
        public bool CheckPwd(string password, out string errMsg)
        {
            return CheckPwd(password, out errMsg, true);
        }

        public bool CheckPwd(string password, out string errMsg, bool check2Update)
        {
            errMsg = string.Empty;
            if (PSWERRCOUNT >= 3)
            {
                errMsg = "柜员号或密码登陆错误超过3次，当天不允许继续登陆！请联系科技人员！";
                return false;
            }
            string strPwd = _tellerNo + password;
            byte[] b = Encoding.UTF8.GetBytes(strPwd);
            b = new MD5CryptoServiceProvider().ComputeHash(b);
            var ret = new StringBuilder();
            for (int i = 0; i < b.Length; i++)
                ret.Append(b[i].ToString("x").PadLeft(2, '0'));
            if (string.Compare(ret.ToString(), USERPSW) != 0)
            {
                PSWERRCOUNT++;
                TradeManagerHandle.RecordPSWError(USERID);
                errMsg = string.Format("柜员号或密码错误！当日错误次数为{0}！", PSWERRCOUNT);
                return false;
            }
            if (!check2Update) return true;
            var firstLogin = TellerSystem.ServiceProxy.Ext.ServiceSettings.GetConfig("IsFirstLoginModify", "");
            if (!string.IsNullOrEmpty(firstLogin) && firstLogin == "1")
            {
                //检查是否为原始密码
                var systemPwd = TellerSystem.ServiceProxy.Ext.ServiceSettings.GetConfig("Sys003", "");
                if (!string.IsNullOrEmpty(systemPwd) && password == systemPwd)
                {
                    var dialog = new UpdatePassword(this);
                    dialog.Owner = Application.Current.MainWindow;
                    dialog.Title = "首次登录，请修改密码";
                    dialog.ShowDialog();
                    if (!dialog.PageResult)
                    {
                        errMsg = "当前为重置后的原始密码，需要进行修改！";
                        return false;
                    }
                }
            }
            return true;
        }

        static LoginAuthentication()
        {
            var tuple = ServiceProxy.Ext.ServiceSettings.GetConfig<Tuple<int, int>>("UserPwdLength", x =>
            {
                var min = 6;
                var max = 6;
                if (!string.IsNullOrEmpty(x))
                {
                    var tmp = x.Split(",，".ToArray());
                    int.TryParse(tmp[0], out min);
                    if (tmp.Length > 1)
                        int.TryParse(tmp[1], out max);
                    if (min < 6) min = 6;
                    if (max < min) max = min;
                }
                return new Tuple<int, int>(min, max);
            });
            PwdMinLength = tuple.Item1;
            PwdMaxLength = tuple.Item2;
            //添加资源
            var userPwdStyle = new Style(typeof(PasswordBox), Application.Current.TryFindResource(typeof(PasswordBox)) as Style);
            userPwdStyle.Setters.Add(new Setter(Platform.Controls.ControlsHelper.PasswordBoxHelper.MinLengthProperty, PwdMinLength));
            userPwdStyle.Setters.Add(new Setter(PasswordBox.MaxLengthProperty, PwdMaxLength));
            userPwdStyle.Setters.Add(new Setter(Platform.Presentation.FocusManagers.FocusManagerExt.AssertNotEmptyProperty, true));
            Application.Current.Resources.Add("UserPwdStyle", userPwdStyle);
        }

        public static int PwdMinLength { get; private set; }

        public static int PwdMaxLength { get; private set; }

        /// <summary>
        /// 更新密码
        /// </summary>
        /// <param name="password"></param>
        /// <param name="errMsg"></param>
        /// <returns></returns>
        public bool UpdatePwd(string password, out string errMsg)
        {
            errMsg = "密码不允许为空";
            //密码复杂度检查
            if (string.IsNullOrEmpty(password)) return false;
            //1.去前后空格
            password = password.Trim();
            //2.检查密码长度
            errMsg = "密码长度不符";
            if (password.Length < PwdMinLength)
            {
                errMsg += "，不能小于" + PwdMinLength;
                return false;
            }
            if (password.Length > PwdMaxLength)
            {
                errMsg += "，不能大于" + PwdMaxLength;
                return false;
            }
            //3..密码应使用同时包含大小写字母、数字、符号中的三种类型的字符组合
            errMsg = "密码必须同时包含大小写字母、数字、符号";
            var regex = ServiceProxy.Ext.ServiceSettings.GetConfig("UserPwdRegex", "");//@"^(?=.*[A-Z])(?=.*[a-z])(?=.*\d)(?=.*\W)[A-Za-z\d\W]+$");
            if (!string.IsNullOrEmpty(regex) && !Regex.IsMatch(password, regex)) return false;
            //4.简单密码验证
            errMsg = "密码认定简单规则，包含3位及以上连续数字、重复字符或其他情况";
            regex = ServiceProxy.Ext.ServiceSettings.GetConfig("UserPwdRegex2Simple", "");//@"([\w])\1{2,}|012|123|234|345|456|567|678|789|987|876|765|654|543|432|321|210");
            if (!string.IsNullOrEmpty(regex) && Regex.IsMatch(password, regex)) return false;
            //更新操作
            var b = Encoding.UTF8.GetBytes(this.USERCODE + password);
            b = new MD5CryptoServiceProvider().ComputeHash(b);
            var ret = new StringBuilder();
            for (int i = 0; i < b.Length; i++)
                ret.Append(b[i].ToString("x").PadLeft(2, '0'));
            if (TradeManagerHandle.UpdatePSW(this.USERID, ret.ToString()))
            {
                this.USERPSW = ret.ToString();
                ///这里，当修改密码成功后，给几个全局属性赋值，
                if (this.USERCODE == Instance.USERCODE)
                    LoginUserInfo.SetUserInfo("Passwd", password.Trim());
                return true;
            }
            return false;
        }

        /// <summary>
        /// 登记记录状态
        /// </summary>
        /// <param name="sign"></param>
        /// <returns></returns>
        public bool LoginSign(bool sign)
        {
            //if (!TradePowerManager.Instance.IsPowerCheckByUser) return true;
            if (string.IsNullOrEmpty(_tellerNo)) return false;

            if (sign) IPAddress = UserManagerHandle.SortSockAddress(SocketManager.GetLocalIpAddressList());

            var parm = new TT_LoginSign
            {
                ChenkSign = sign ? "1" : "0",
                LoginIp = IPAddress,//获取IP地址 ,
                LoginMarc = GetMacAddress(),
                LoginTime = DateTime.Now.ToString(),
                PortNo = LoginUserInfo.TtyName,//说明:PortNo记录了终端号，用于柜员单客户端登陆。如果终端号相同，登陆不上去
                UserCode = USERCODE,
                UserID = USERID,
                LoginGroups = this.TellerGroupValue
            };
            TradeManagerHandle.RecordLoginSign(
                new List<TT_LoginSign>
            {
                parm
            });
            return true;
        }

        #endregion
    }
}
