// 定义 TT_RoleGroup 类型，假设它在其他地方有完整定义，这里先给出基本结构
interface TT_RoleGroup {
    GroupName: string;
    GroupId: string;
    GroupCode?: string;
}

// 定义 LoginUserInfo 类型，假设它在其他地方有完整定义，这里先给出基本结构
interface LoginUserInfo {
    TtyName: string;
    KinbrName?: string;
}

// 定义 TradeManagerHandle 类型，假设它在其他地方有完整定义，这里先给出基本方法
interface TradeManagerHandle {
    GetLoginDataByUserCode(tellerNo: string, zcm: string): Array<{
        UserId: string;
        UserCode: string;
        UserName: string;
        IsPiont: string;
        IsLogin: string;
        KinbrId: string;
        MngatmUser: string;
        UserPsw: string;
        CardId: string;
        PwdChgDate: string;
        LoginIp: string;
        PortNo: string;
        PswLevel: string;
        LevelChgDate: string;
        LevelDate: string;
        CardLevel: string;
        IsChangePsw: string;
        BeforeDate: string;
        PswErrCount: string;
        PswTime: string;
        Date: string;
        CanForceLogin: boolean;
        MaxPswCycle: string;
        ChenkSign: string;
        KinbrNo: string;
        IsAcross: string;
        CanGroupsLogin: boolean;
        UpNoteKinbrno: string;
        UpCashKinbrno: string;
        upnoteteller: string;
        upcashteller: string;
        kinbrtype: string;
        kinbrcomment: string;
        usercomment: string;
        idCarNo: string;
        GroupKinbrNo: string;
        IsMobile: boolean;
    }>;
    GetRolegroupsbyUsercode(tellerNo: string): TT_RoleGroup[];
    GetMultiRole(): string;
    GetLogingroupsByUsercode(tellerNo: string): string;
    RecordPSWError(userId: string): void;
    UpdatePSW(userId: string, psw: string): boolean;
    RecordLoginSign(loginSigns: Array<{
        ChenkSign: string;
        LoginIp: string;
        LoginMarc: string;
        LoginTime: string;
        PortNo: string;
        UserCode: string;
        UserID: string;
        LoginGroups: string;
    }>): void;
}

// 定义 ServiceProxy 类型，假设它在其他地方有完整定义，这里先给出基本方法
interface ServiceProxy {
    Ext: {
        ServiceSettings: {
            GetConfig<T>(key: string, defaultValue: T): T;
            GetConfig<T>(key: string, parser: (value: string) => T): T;
        };
    };
}

// 定义 UserManagerHandle 类型，假设它在其他地方有完整定义，这里先给出基本方法
interface UserManagerHandle {
    SortSockAddress(ipAddresses: string[]): string;
}

// 定义 SocketManager 类型，假设它在其他地方有完整定义，这里先给出基本方法
interface SocketManager {
    GetLocalIpAddressList(): string[];
}

// 定义 ConfigManager 类型，假设它在其他地方有完整定义，这里先给出基本方法
interface ConfigManager {
    static GetInstance(): {
        GetConfigValue(key: string, configType: string): string;
    };
}

// 定义 UpdatePassword 类型，假设它在其他地方有完整定义，这里先给出基本结构
class UpdatePassword {
    PageResult: boolean;
    constructor(auth: LoginAuthentication) {
        this.PageResult = false;
    }
    ShowDialog(): void {}
}

// 定义 LoginAuthentication 类
class LoginAuthentication {
    private static _instance: LoginAuthentication;
    private _tellerNo: string = '';

    // 属性
    PaymentNo: string = '';
    USERID: string = '';
    USERCODE: string = '';
    USERNAME: string = '';
    ISPIONT: boolean = false;
    ISLOGIN: boolean = false;
    KINBRID: string = '';
    MNGATMUSER: string = '';
    USERPSW: string = '';
    CARDID: string = '';
    PWDCHGDATE: string = '';
    LOGINIP: string = '';
    PORTNO: string = '0';
    PSWLEVEL: string = '';
    LEVELCHGDATE: string = '';
    LEVELDATE: number = 0;
    CARDLEVEL: string = '';
    ISCHANGEPSW: string = '';
    BEFOREDATE: string = '';
    PSWERRCOUNT: number = 0;
    PSWTIME: string = '';
    DATE: string = '';
    CHENKSIGN: string = '';
    MaxPswCycle: string = '';
    CanForceLogin: boolean = false;
    KinbrNo: string = '';
    IsAcross: string = '';
    CanGroupsLogin: boolean = false;
    IPAddress: string = '';
    Usergroup: TT_RoleGroup[] | null = null;
    SetUserDown: boolean = false;
    Ismobile: boolean = false;
    UpNoteKinbrno: string = '';
    UpCashKinbrno: string = '';
    UpNoteteller: string = '';
    UpCashteller: string = '';
    kinbrtype: string = '';
    UserComment: string = '';
    KinbrComment: string = '';
    IdCarNO: string = '';
    GroupKinbrNo: string = '';

    static get Instance(): LoginAuthentication {
        if (!this._instance) {
            this._instance = new LoginAuthentication();
        }
        return this._instance;
    }

    get TellerGroupName(): string {
        let ret = '';
        if (this.Usergroup && this.Usergroup.length > 0) {
            let names = '';
            for (const item of this.Usergroup) {
                names += `,${item.GroupName}`;
            }
            ret = names.slice(1);
        }
        return ret;
    }

    get TellerGroupValue(): string {
        let ret = '';
        if (this.Usergroup && this.Usergroup.length > 0) {
            let values = '';
            for (const item of this.Usergroup) {
                values += `,${item.GroupId}`;
            }
            ret = values.slice(1);
        }
        return ret;
    }

    // 获得MAC地址
    GetMacAddress(): string {
        // 由于浏览器环境限制，无法直接获取MAC地址，这里只是占位实现
        return '';
    }

    // 读取注册表信息
    ReadZhuce(): string {
        // 由于浏览器环境限制，无法直接访问注册表，这里只是占位实现
        return '';
    }

    // 初始化数据
    InitData(tellerNo: string): boolean {
        let zcm = '';
        if (ConfigManager.GetInstance().GetConfigValue('IsStartHardDiskNumber', 'System').toLowerCase() === 'true') {
            zcm = this.ReadZhuce();
        }

        if (!tellerNo) return false;
        const data = TradeManagerHandle.GetLoginDataByUserCode(tellerNo, zcm);
        if (!data || data.length === 0) return false;
        const loginData = data[0];

        this.USERID = loginData.UserId;
        this.USERCODE = loginData.UserCode;
        this.USERNAME = loginData.UserName;
        this.ISPIONT = loginData.IsPiont === '1';
        this.ISLOGIN = loginData.IsLogin === '1';
        this.KINBRID = loginData.KinbrId;
        this.MNGATMUSER = loginData.MngatmUser;
        this.USERPSW = loginData.UserPsw;
        this.CARDID = loginData.CardId;
        this.PWDCHGDATE = loginData.PwdChgDate;
        this.LOGINIP = loginData.LoginIp;
        this.PORTNO = loginData.PortNo || '0';
        this.PSWLEVEL = loginData.PswLevel;
        this.LEVELCHGDATE = loginData.LevelChgDate;
        this.LEVELDATE = loginData.LevelDate ? parseInt(loginData.LevelDate, 10) : 0;
        this.CARDLEVEL = loginData.CardLevel;
        this.ISCHANGEPSW = loginData.IsChangePsw;
        this.BEFOREDATE = loginData.BeforeDate;
        this.PSWERRCOUNT = loginData.PswErrCount ? parseInt(loginData.PswErrCount, 10) : 0;
        this.PSWTIME = loginData.PswTime;
        this.DATE = loginData.Date;
        this.CanForceLogin = loginData.CanForceLogin;
        this.MaxPswCycle = loginData.MaxPswCycle;
        this.CHENKSIGN = loginData.ChenkSign;
        this.KinbrNo = loginData.KinbrNo;
        this.IsAcross = loginData.IsAcross;
        this.CanGroupsLogin = loginData.CanGroupsLogin;
        this.UpNoteKinbrno = loginData.UpNoteKinbrno;
        this.UpCashKinbrno = loginData.UpCashKinbrno;
        this.UpNoteteller = loginData.upnoteteller;
        this.UpCashteller = loginData.upcashteller;
        this.kinbrtype = loginData.kinbrtype;
        this.KinbrComment = loginData.kinbrcomment;
        this.UserComment = loginData.usercomment;
        this.IdCarNO = loginData.idCarNo;
        this.GroupKinbrNo = loginData.GroupKinbrNo;
        this.Ismobile = loginData.IsMobile;
        this._tellerNo = tellerNo;
        return true;
    }

    // 登陆操作
    LoginCheck(): { success: boolean; errorMsg: string } {
        this.SetUserDown = false;
        let errorMsg = '';
        if (this.PSWLEVEL === '1') {
            return { success: true, errorMsg };
        }

        if (!this.ISLOGIN) {
            errorMsg = '该柜员已被停用！';
            return { success: false, errorMsg };
        }

        let dateEnd: Date | null = null;
        const dateNow = new Date(this.DATE);
        if (this.LEVELCHGDATE) {
            dateEnd = new Date(this.LEVELCHGDATE);
            const endDate = new Date(dateEnd.getTime() + this.LEVELDATE * 24 * 60 * 60 * 1000);
            if (endDate < dateNow) {
                errorMsg = '登录验证方式已过期，禁止登录！';
                return { success: false, errorMsg };
            }
        }

        let lastPwdChangeData: Date | null = null;
        let maxPswCycle: number | null = null;

        if (this.PWDCHGDATE) {
            lastPwdChangeData = new Date(this.PWDCHGDATE);
        }
        if (this.MaxPswCycle) {
            maxPswCycle = parseFloat(this.MaxPswCycle);
        }

        if (!lastPwdChangeData || (maxPswCycle && lastPwdChangeData.getTime() + maxPswCycle * 24 * 60 * 60 * 1000 < dateNow.getTime())) {
            const dialog = new UpdatePassword(this);
            dialog.ShowDialog();
            if (!dialog.PageResult) {
                errorMsg = '登陆密码过期，需要进行修改！';
                return { success: false, errorMsg };
            }
            errorMsg = '密码修改成功！';
            return { success: false, errorMsg };
        }

        return { success: true, errorMsg };
    }

    // 获取用户角色列表
    GetTellerGroup(): { success: boolean; errorMsg: string } {
        let errorMsg = '';
        this.Usergroup = TradeManagerHandle.GetRolegroupsbyUsercode(this._tellerNo)
           .filter(ttRoleGroup => ttRoleGroup.GroupName && ttRoleGroup.GroupId);

        if (!this.Usergroup || this.Usergroup.length === 0) {
            errorMsg = '获取柜员角色列表出错，柜员角色列表为空！';
            return { success: false, errorMsg };
        }

        return { success: true, errorMsg };
    }

    // 现金管理员
    get IsCashRole(): boolean {
        return this.Usergroup ? this.Usergroup.some(x => x.GroupCode === '000005') : false;
    }

    // 凭证管理员
    get IsVoucherRole(): boolean {
        return this.Usergroup ? this.Usergroup.some(x => x.GroupCode === '000005') : false;
    }

    // 验证是否允许单柜员多角色多客户端登录
    GroupCheck(group: string): { success: boolean; errorMsg: string } {
        let errorMsg = '';
        if (this.CHENKSIGN === '1') {
            if (this.PORTNO === LoginUserInfo.TtyName) {
                if (TradeManagerHandle.GetMultiRole() !== '1') {
                    const groupstr = TradeManagerHandle.GetLogingroupsByUsercode(this._tellerNo);
                    if (!groupstr.includes(group)) {
                        errorMsg = '用户已选择其他角色登录！';
                        return { success: false, errorMsg };
                    }
                }
            }
        }
        return { success: true, errorMsg };
    }

    // 当前柜员状态已登录时的登录验证
    SignCheck(group: string): { success: boolean; errorMsg: string } {
        let errorMsg = '';
        if (this.CHENKSIGN === '1') {
            if (this.ISPIONT) {
                if (this.PORTNO !== LoginUserInfo.TtyName) {
                    if (this.CanGroupsLogin) {
                        const groupstr = TradeManagerHandle.GetLogingroupsByUsercode(this._tellerNo);
                        if (!groupstr.includes(group)) {
                            return { success: true, errorMsg };
                        }
                    }

                    if (!this.CanForceLogin) {
                        errorMsg = '该柜员已经在其他机器上登陆系统，请核查！';
                        return { success: false, errorMsg };
                    } else {
                        this.SetUserDown = true;
                    }
                }
            }
        }
        return { success: true, errorMsg };
    }

    // 验证密码
    CheckPwd(password: string): { success: boolean; errMsg: string } {
        return this.CheckPwdInner(password, true);
    }

    private CheckPwdInner(password: string, check2Update: boolean): { success: boolean; errMsg: string } {
        let errMsg = '';
        if (this.PSWERRCOUNT >= 3) {
            errMsg = '柜员号或密码登陆错误超过3次，当天不允许继续登陆！请联系科技人员！';
            return { success: false, errMsg };
        }

        const strPwd = this._tellerNo + password;
        const encoder = new TextEncoder();
        const data = encoder.encode(strPwd);
        // 由于浏览器环境中 MD5 算法需要使用第三方库，这里只是占位实现
        const hash = ''; 

        if (hash !== this.USERPSW) {
            this.PSWERRCOUNT++;
            TradeManagerHandle.RecordPSWError(this.USERID);
            errMsg = `柜员号或密码错误！当日错误次数为${this.PSWERRCOUNT}！`;
            return { success: false, errMsg };
        }

        if (!check2Update) return { success: true, errMsg };

        const firstLogin = ServiceProxy.Ext.ServiceSettings.GetConfig('IsFirstLoginModify', '');
        if (firstLogin === '1') {
            const systemPwd = ServiceProxy.Ext.ServiceSettings.GetConfig('Sys003', '');
            if (systemPwd && password === systemPwd) {
                const dialog = new UpdatePassword(this);
                dialog.ShowDialog();
                if (!dialog.PageResult) {
                    errMsg = '当前为重置后的原始密码，需要进行修改！';
                    return { success: false, errMsg };
                }
            }
        }

        return { success: true, errMsg };
    }

    static PwdMinLength: number;
    static PwdMaxLength: number;

    static {
        const tuple = ServiceProxy.Ext.ServiceSettings.GetConfig('UserPwdLength', (x: string) => {
            let min = 6;
            let max = 6;
            if (x) {
                const tmp = x.split(/[,\uff0c]/);
                if (tmp.length > 0) {
                    min = parseInt(tmp[0], 10) || 6;
                }
                if (tmp.length > 1) {
                    max = parseInt(tmp[1], 10) || min;
                }
                if (min < 6) min = 6;
                if (max < min) max = min;
            }
            return { min, max };
        });
        this.PwdMinLength = tuple.min;
        this.PwdMaxLength = tuple.max;
    }

    // 更新密码
    UpdatePwd(password: string): { success: boolean; errMsg: string } {
        let errMsg = '密码不允许为空';
        if (!password) return { success: false, errMsg };

        password = password.trim();
        errMsg = '密码长度不符';
        if (password.length < LoginAuthentication.PwdMinLength) {
            errMsg += `，不能小于${LoginAuthentication.PwdMinLength}`;
            return { success: false, errMsg };
        }
        if (password.length > LoginAuthentication.PwdMaxLength) {
            errMsg += `，不能大于${LoginAuthentication.PwdMaxLength}`;
            return { success: false, errMsg };
        }

        errMsg = '密码必须同时包含大小写字母、数字、符号';
        const regex = ServiceProxy.Ext.ServiceSettings.GetConfig('UserPwdRegex', '');
        if (regex && !new RegExp(regex).test(password)) return { success: false, errMsg };

        errMsg = '密码认定简单规则，包含3位及以上连续数字、重复字符或其他情况';
        const regex2 = ServiceProxy.Ext.ServiceSettings.GetConfig('UserPwdRegex2Simple', '');
        if (regex2 && new RegExp(regex2).test(password)) return { success: false, errMsg };

        const encoder = new TextEncoder();
        const data = encoder.encode(this.USERCODE + password);
        // 由于浏览器环境中 MD5 算法需要使用第三方库，这里只是占位实现
        const hash = ''; 

        if (TradeManagerHandle.UpdatePSW(this.USERID, hash)) {
            this.USERPSW = hash;
            // 假设 LoginUserInfo 存在 SetUserInfo 方法
            if (this.USERCODE === LoginAuthentication.Instance.USERCODE) {
                // LoginUserInfo.SetUserInfo('Passwd', password);
            }
            return { success: true, errMsg: '' };
        }

        return { success: false, errMsg: '密码更新失败' };
    }

    // 登记记录状态
    LoginSign(sign: boolean): boolean {
        if (!this._tellerNo) return false;

        if (sign) {
            this.IPAddress = UserManagerHandle.SortSockAddress(SocketManager.GetLocalIpAddressList());
        }

        const parm = {
            ChenkSign: sign ? '1' : '0',
            LoginIp: this.IPAddress,
            LoginMarc: this.GetMacAddress(),
            LoginTime: new Date().toISOString(),
            PortNo: LoginUserInfo.TtyName,
            UserCode: this.USERCODE,
            UserID: this.USERID,
            LoginGroups: this.TellerGroupValue
        };

        TradeManagerHandle.RecordLoginSign([parm]);
        return true;
    }
}

export { LoginAuthentication };