﻿using System.Windows;
using System.Windows.Input;
using System.Text;
using System.Security.Cryptography;
using TellerSystem.Library.Ext.Function;
using Platform.Common.RunningParameters;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Library.Ext.TradeExtension
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class UpdatePassword : Window
    {
        public UpdatePassword()
        {
            InitializeComponent();
        }

        public UpdatePassword(LoginAuthentication login)
        {
            InitializeComponent();
            this.LoginUser = login;
        }

        /// <summary>
        /// 登录用户
        /// </summary>
        public LoginAuthentication LoginUser { get; private set; }

        /// <summary>
        /// 页面结果 是否成功
        /// </summary>
        public bool PageResult { get; set; }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            UserCood.Text = this.LoginUser.USERCODE;
            UserName.Text = this.LoginUser.USERNAME;

            OldPSW.Focus();
        }
        private void SubmitButton_Click(object sender, RoutedEventArgs e)
        {
            var errMsg = string.Empty;
            if (!LoginUser.CheckPwd(OldPSW.Password, out errMsg, false))
            {
                MessageBox.Show("当前密码与旧密码不符！");
                PSW.Focus();
                return;
            }
            if (OldPSW.Password.Trim() == PSWs.Password.Trim())
            {
                MessageBox.Show("新密码不能与旧密码相同！");
                PSW.Password = PSWs.Password = "";
                PSW.Focus();
                return;
            }
            bool flg = UserFunction.IsSamplePassword(null, PSW.Password.Trim());

            if (flg)
            {
                MessageBox.Show("新密码过于简单！");
                PSW.Password = PSWs.Password = "";
                PSW.Focus();
                return;
            }

            if (PSW.Password.Trim() != PSWs.Password.Trim())
            {
                MessageBox.Show("两次输入的密码不一致！");
                PSW.Password = PSWs.Password = "";
                PSW.Focus();
                return;
            }
            if (LoginUser.UpdatePwd(PSW.Password, out errMsg))
            {
                MessageBox.Show("密码修改成功！");
                PageResult = true;
                this.Close();
            }
            else
            {
                MessageBox.Show("修改失败！" + errMsg);
                PageResult = false;
            }
        }

        private void CancelButton_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        #region UI
        private void OldPSW_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter || e.Key == Key.Down)
            {
                PSW.Focus();
            }
        }

        private void PSW_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter || e.Key == Key.Down)
            {
                PSWs.Focus();
            }
            else if (e.Key == Key.Up)
            {
                OldPSW.Focus();
            }
        }

        private void PSWs_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter || e.Key == Key.Down)
            {
                SubmitButton.Focus();
            }
            else if (e.Key == Key.Up)
            {
                PSW.Focus();
            }
        }

        private void SubmitButton_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Up)
            {
                PSWs.Focus();
            }
        }
        #endregion

        private void OldPSW_LostFocus(object sender, RoutedEventArgs e)
        {
            //if (OldPSW.Password.Length > 0 && OldPSW.Password.Length < 6)
            //{
            //    MessageBox.Show("密码需输入6位！");
            //    OldPSW.Password = "";
            //    var focus = Platform.Presentation.FocusManagers.FocusManager.GetManagerInstance(OldPSW);
            //    focus.SetFocus(OldPSW);
            //    focus.IsFocusContinue = false;
            //    return;
            //}
        }

        private void PSW_LostFocus(object sender, RoutedEventArgs e)
        {
            //if (PSW.Password.Length > 0 && PSW.Password.Length < 6)
            //{
            //    MessageBox.Show("密码需输入6位！");
            //    PSW.Password = "";
            //    var focus = Platform.Presentation.FocusManagers.FocusManager.GetManagerInstance(PSW);
            //    focus.SetFocus(PSW);
            //    focus.IsFocusContinue = false;
            //    return;
            //}
        }

        private void PSWs_LostFocus(object sender, RoutedEventArgs e)
        {
            //if (PSWs.Password.Length > 0 && PSWs.Password.Length < 6)
            //{
            //    MessageBox.Show("密码需输入6位！");
            //    PSWs.Password = "";
            //    var focus = Platform.Presentation.FocusManagers.FocusManager.GetManagerInstance(PSWs);
            //    focus.SetFocus(PSWs);
            //    focus.IsFocusContinue = false;
            //    return;
            //}
        }
    }
}
