﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.Concurrent;
using System.Runtime.CompilerServices;
using TellerSystem.Communication;
using TellerSystem.Library.Ext.Function;
using System.IO;

namespace TellerSystem.Library.Ext.TradeExtension
{
    /// <summary>
    /// 选择项数据管理机
    /// </summary>
    public class SelectionManager
    {
        #region 单例
        private SelectionManager()
        {
            _selectionBuffer = new Dictionary<Tuple<string, Dictionary<string, string>>, Dictionary<string, string>>();
        }
        private static SelectionManager _instance;
        /// <summary>
        /// 数据项管理机单例对象
        /// </summary>
        public static SelectionManager Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new SelectionManager();
                return _instance;
            }
        }
        #endregion

        #region 数据缓存
        /// <summary>
        /// 缓存字典，Key-code,arguments Value-dictionary
        /// </summary>
        private Dictionary<Tuple<string, Dictionary<string, string>>, Dictionary<string, string>> _selectionBuffer;
        #endregion

        #region 接口
        /// <summary>
        /// 拉取列表数据项
        /// </summary>
        /// <param name="target"></param>
        /// <param name="code"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        [MethodImpl(MethodImplOptions.Synchronized)]
        public Dictionary<string, string> PullSelection(string code, Dictionary<string, string> args = null, string mainCode = "def")
        {
            var ret = new Dictionary<string, string>();
            if (!string.IsNullOrEmpty(code))
            {
                //检查缓存，是否存在
                var temp = _selectionBuffer.FirstOrDefault(x =>
                  {
                      if (x.Key.Item1 == code)
                      {
                          //若无参数，则判断缓存是否存在参数
                          if (x.Key.Item2 == null || x.Key.Item2.Count == 0)
                          {
                              return (args == null || args.Count == 0);
                          }
                          //若有参数，先比较个数
                          if (args == null || args.Count != x.Key.Item2.Count)
                              return false;
                          //需要比较参数对象
                          return x.Key.Item2.All(y => args.ContainsKey(y.Key) && args[y.Key] == y.Value);
                      }
                      return false;
                  });
                if (temp.Key != null)
                {
                    ret = temp.Value;
                }
                else
                {
                    //参数，填充123域
                    var arguments = new StringBuilder();
                    if (args != null && args.Count > 0)
                    {
                        foreach (var item in args)
                        {
                            //{NOTE_KIND}|凭证类别值|
                            arguments.Append("{" + item.Key + "}|" + item.Value + "|");
                        }
                    }
                    var MessageData = new Message();
                    SystemFunction.CommSend(null, MessageData);
                    MessageData.Fd126_5 = mainCode;
                    MessageData.Fd16 = "K042";
                    MessageData.Fd64 = code;
                    MessageData.Fd123 = arguments.ToString();

                    if (MessageData.DoTransit() && MessageData.Fd12 == "0000")
                    {
                        //通讯成功,解析数据  格式：类型值|描述|
                        var reader = new StringReader(MessageData.FileData);
                        var line = reader.ReadLine();
                        while (!string.IsNullOrEmpty(line))
                        {
                            if (!line.StartsWith("~"))
                            {
                                var rowData = line.Split('|');
                                if (rowData.Length > 1)
                                    ret.Add(rowData[1], rowData[0]);
                            }
                            line = reader.ReadLine();
                        }
                        //若列表未空，无需加入到缓存
                        if (ret.Count > 0)
                            //加入缓存
                            _selectionBuffer.Add(new Tuple<string, Dictionary<string, string>>(code.Trim(), args), ret);
                    }
                }
            }
            return ret;
        }
        #endregion
    }
}
