// 模拟 Tuple 类型
class Tuple<T1, T2> {
    constructor(public item1: T1, public item2: T2) {}
}

// 模拟 Message 类
class Message {
    Fd126_5: string = '';
    Fd16: string = '';
    Fd64: string = '';
    Fd123: string = '';
    Fd12: string = '';
    FileData: string = '';

    DoTransit(): boolean {
        // 这里需要根据实际情况实现通讯逻辑
        return true;
    }
}

// 模拟 SystemFunction 类
class SystemFunction {
    static CommSend(_: any, __: Message) {
        // 这里需要根据实际情况实现通讯发送逻辑
    }
}

class SelectionManager {
    private static _instance: SelectionManager;
    private _selectionBuffer: Map<Tuple<string, { [key: string]: string } | null>, { [key: string]: string }> = new Map();

    private constructor() {
        this._selectionBuffer = new Map();
    }

    static get Instance(): SelectionManager {
        if (!this._instance) {
            this._instance = new SelectionManager();
        }
        return this._instance;
    }

    PullSelection(code: string, args: { [key: string]: string } | null = null, mainCode: string = 'def'): { [key: string]: string } {
        let ret: { [key: string]: string } = {};
        if (!code) {
            return ret;
        }

        // 检查缓存
        let found = false;
        this._selectionBuffer.forEach((value, key) => {
            if (key.item1 === code) {
                if ((!key.item2 || Object.keys(key.item2).length === 0) && (!args || Object.keys(args).length === 0)) {
                    ret = value;
                    found = true;
                } else if (args && key.item2) {
                    const argKeys = Object.keys(args);
                    const bufferKeys = Object.keys(key.item2);
                    if (argKeys.length === bufferKeys.length) {
                        let allMatch = true;
                        for (const key of argKeys) {
                            if (args[key] !== key.item2[key]) {
                                allMatch = false;
                                break;
                            }
                        }
                        if (allMatch) {
                            ret = value;
                            found = true;
                        }
                    }
                }
            }
        });

        if (found) {
            return ret;
        }

        // 构造参数
        let argumentsStr = '';
        if (args) {
            for (const [key, value] of Object.entries(args)) {
                argumentsStr += `{${key}}|${value}|`;
            }
        }

        const MessageData = new Message();
        SystemFunction.CommSend(null, MessageData);
        MessageData.Fd126_5 = mainCode;
        MessageData.Fd16 = 'K042';
        MessageData.Fd64 = code;
        MessageData.Fd123 = argumentsStr;

        if (MessageData.DoTransit() && MessageData.Fd12 === '0000') {
            const lines = MessageData.FileData.split('\n');
            for (const line of lines) {
                if (line && !line.startsWith('~')) {
                    const rowData = line.split('|');
                    if (rowData.length > 1) {
                        ret[rowData[1]] = rowData[0];
                    }
                }
            }

            if (Object.keys(ret).length > 0) {
                this._selectionBuffer.set(new Tuple(code.trim(), args), ret);
            }
        }

        return ret;
    }
}

export { SelectionManager };