// 定义属性变更事件的回调类型
type PropertyChangedEventHandler = (sender: BatSysInfoUI, eventArgs: PropertyChangedEventArgs) => void;

// 属性变更事件参数类
class PropertyChangedEventArgs {
    constructor(public propertyName: string) {}
}

// 界面双向绑定定义实体
class BatSysInfoUI {
    // 变量
    sysname: string = '';
    batinfoname: string = '';
    batcode: string = '';
    batname: string = '';
    batdesp: string = '';
    syscode: string = '';
    mainseqn: string = '';
    stat: string = '';
    bat_ind: string = '';
    content: string = '';
    txdate: string = '';
    startEnabled: boolean = false;
    lookEnabled: boolean = false;
    timespan: string = '';
    filler1: string = '';
    autoRefreshTime: string = '';
    batcheck: string = '';

    // 属性变更事件
    PropertyChanged?: PropertyChangedEventHandler;

    // 系统名称属性
    get SysName(): string {
        return this.sysname;
    }
    set SysName(value: string) {
        this.sysname = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('SysName'));
        }
    }

    // 批量名称属性
    get BatInfoName(): string {
        return this.batinfoname;
    }
    set BatInfoName(value: string) {
        this.batinfoname = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('BatInfoName'));
        }
    }

    // 批量编码属性
    get BatCode(): string {
        return this.batcode;
    }
    set BatCode(value: string) {
        this.batcode = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('BatCode'));
        }
    }

    // 批量名称属性
    get BatName(): string {
        return this.batname;
    }
    set BatName(value: string) {
        this.batname = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('BatName'));
        }
    }

    // 批量说明属性
    get BatDesp(): string {
        return this.batdesp;
    }
    set BatDesp(value: string) {
        this.batdesp = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('BatDesp'));
        }
    }

    // 启动日期属性
    get TxDate(): string {
        return this.txdate;
    }
    set TxDate(value: string) {
        this.txdate = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('TxDate'));
        }
    }

    // 系统编号属性
    get SysCode(): string {
        return this.syscode;
    }
    set SysCode(value: string) {
        this.syscode = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('SysCode'));
        }
    }

    // 主顺序号属性
    get MainSeqn(): string {
        return this.mainseqn;
    }
    set MainSeqn(value: string) {
        this.mainseqn = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('MainSeqn'));
        }
    }

    // 批量处理状态属性
    get Stat(): string {
        return this.stat;
    }
    set Stat(value: string) {
        this.stat = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('Stat'));
        }
    }

    // 是否自动批量属性
    get Bat_Ind(): string {
        return this.bat_ind;
    }
    set Bat_Ind(value: string) {
        this.bat_ind = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('Bat_Ind'));
        }
    }

    // 批量启动间隔属性
    get Time_Span(): string {
        return this.timespan;
    }
    set Time_Span(value: string) {
        this.timespan = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('Time_Span'));
        }
    }

    // 按钮显示内容属性
    get BatContent(): string {
        return this.content;
    }
    set BatContent(value: string) {
        this.content = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('BatContent'));
        }
    }

    // 系统名称属性
    get Filler1(): string {
        return this.filler1;
    }
    set Filler1(value: string) {
        this.filler1 = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('Filler1'));
        }
    }

    // 当前实体属性
    BatSysInfo: BatSysInfoUI | null = null;

    // 启动按钮属性
    get StartEnabled(): boolean {
        return this.startEnabled;
    }
    set StartEnabled(value: boolean) {
        this.startEnabled = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('StartEnabled'));
        }
    }

    // 启动前是否校验属性
    get BatCheck(): string {
        return this.batcheck;
    }
    set BatCheck(value: string) {
        this.batcheck = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('BatCheck'));
        }
    }

    // 查看详情属性
    get LookEnabled(): boolean {
        return this.lookEnabled;
    }
    set LookEnabled(value: boolean) {
        this.lookEnabled = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('LookEnabled'));
        }
    }

    // 自动刷新的时间间隔(按秒计)属性
    get AutoRefreshTime(): string {
        return this.autoRefreshTime;
    }
    set AutoRefreshTime(value: string) {
        this.autoRefreshTime = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, new PropertyChangedEventArgs('AutoRefreshTime'));
        }
    }
}

// 值转换器接口
interface IValueConverter {
    Convert(value: any, targetType: any, parameter: any, culture: any): any;
    ConvertBack(value: any, targetType: any, parameter: any, culture: any): any;
}

// 批量校验转换器类
class BatCheckConverter implements IValueConverter {
    Convert(value: any, targetType: any, parameter: any, culture: any): any {
        const strValue = value as string;
        if (strValue === '1') {
            return true;
        }
        if (strValue === '0') {
            return false;
        }
        return undefined;
    }

    ConvertBack(value: any, targetType: any, parameter: any, culture: any): any {
        const blcheck = value as boolean;
        if (blcheck) {
            return '1';
        } else {
            return '0';
        }
    }
}

export { BatSysInfoUI, BatCheckConverter };