﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using TellerSystem.ServiceProxy.Ext.ServiceEntitys;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.Library.Ext.Function;
using TellerSystem.Communication;
using TellerSystem.Library.Ext.Function;
using Platform.Common.RunningParameters;
using TellerSystem.Communication.MessageHelper;
using Platform.Presentation.FocusManagers;
using Platform.Presentation.Interfaces;
using Platform.Presentation.PageFunctions;
namespace TellerSystem.Library.Ext.TradeExtension
{
    /// <summary>
    /// 各系统批量操作抽象类
    /// </summary>
    public class BatchAction
    {
        #region 字段集合
        /// <summary>
        /// 报文通讯对象
        /// </summary>
        protected Message _msg = new Message();
        /// <summary>
        /// 批量控制实体
        /// </summary>
        protected BatSysInfo _batSysInfo;
        /// <summary>
        /// 批量信息
        /// </summary>
        public BatSysInfo BatInfo
        {
            get { return _batSysInfo; }
        }

        protected string _message;
        /// <summary>
        /// 当前状态提示信息
        /// </summary>
        public string Message
        {
            get { return _message; }
        }

        #endregion

        #region 方法集合
        /// <summary>
        /// 启动前，校验批量
        /// 是否具备拉起的条件
        /// </summary>
        /// <returns></returns>
        public virtual bool PreviewDoBatRz()
        {
            return true;
        }
        /// <summary>
        /// 执行批量操作
        /// </summary>
        /// <returns></returns>
        public virtual bool DoBatRz()
        {
            return true;
        }
        /// <summary>
        /// 获取系统批次状态
        /// </summary>
        /// <returns></returns>
        public virtual string GetBatStat()
        {
            return "";
        }
        /// <summary>
        /// 更新批量系统的子批量信息
        /// </summary>
        /// <returns></returns>
        public virtual T GetBatRzInfo<T>() where T : class
        {
            T temp = default(T);
            return temp;
        }
        /// <summary>
        /// 获取错误信息
        /// </summary>
        /// <param name="msg"></param>
        /// <returns></returns>
        protected string GetError(Message msg)
        {
            var result = "[COMM]通讯失败，请检查服务是否正常！";
            if (msg != null)
            {
                if (!string.IsNullOrWhiteSpace(msg.Fd12))
                {
                    if (!string.IsNullOrWhiteSpace(msg.Fd13))
                        result = string.Format("[{0}],{1}", msg.Fd12, msg.Fd13.Replace(" ", ""));
                    else
                    {
                        result = SystemFunction.GetError(msg.Fd12);
                    }
                }

            }
            return result;
        }
        /// <summary>
        /// 初始化域
        /// </summary>
        /// <param name="message"></param>
        protected void InitFd(Message message)
        {
            message.Init();
        }
        /// <summary>
        /// 初始化公共域
        /// </summary>
        /// <param name="message"></param>
        protected void CommSend(Message message)
        {
            //开户行机构号
            message.Fd2 = LoginUserInfo.KinbrNo;// LoginUserInfo.Opnbr;
            //机构号
            message.Fd3 = LoginUserInfo.KinbrNo;
            //交易日期
            message.Fd5 = LoginUserInfo.TradeDate;
            //发生时间
            message.Fd6 = DateTime.Now.ToString("HHmmss");
            //柜员号
            message.Fd7 = LoginUserInfo.TellerNo;
            //终端号
            message.Fd10 = LoginUserInfo.TtyName;
            message.Fd96_9 = "0100"; //本行柜面
            message.Fd96_7 = "0";
            message.Fd96_8 = "0";
            message.Fd96_C = "0100"; //本行柜面
            message.Fd96_D = LoginUserInfo.TradeDate;
            message.Fd96_E = message.Fd6;

        }

        /// <summary>
        /// 和平台通讯API
        /// </summary>
        /// <param name="message"></param>
        /// <param name="transitType">TransitType枚举值</param>
        /// <returns></returns>
        protected bool CallAgn(Message message, string transitType)
        {
            var returnValue = false;
            string preTradeNo = message.Fd16;
            message.TransitNode = TransitType.CallAgnRun;
            message.BitMapType = TellerSystem.Communication.Message.BitMap.Agn;
            returnValue = message.DoTransit();
            if (message.Fd16 != preTradeNo && message.Fd12 == "0000")
            {
                message.Fd12 = "COMN";
            }
            return returnValue;
        }
        #endregion
    }
    /// <summary>
    /// 批量执行外围系统实体
    /// </summary>
    public class BatActionKernel : BatchAction
    {
        public BatActionKernel()
        { }

        public BatActionKernel(BatSysInfo batSysInfo)
        {
            //外围系统编号及子批次编号
            if (batSysInfo.syscode.Equals("") && batSysInfo.batcode.Equals(""))
            {
                throw new ArgumentException("参数指定的结帐系统不是结账系统!");
            }
            _batSysInfo = batSysInfo;
        }
        /// <summary>
        /// 校验批次是否可以进行批量结账
        /// </summary>
        /// <returns></returns>
        public override bool PreviewDoBatRz()
        {
            if (_batSysInfo.batcheck == "1")
            {
                InitFd(_msg);
                this.CommSend(_msg);
                _msg.Fd16 = "9006";
                this._msg.Fd21 = _batSysInfo.syscode;
                _msg.Fd47 = _batSysInfo.tx_date;
                _msg.Fd86 = _batSysInfo.batcode;//拉起各系统的批次号
                _msg.Fd18 = "RUNB";
                CallAgn(_msg, "CallAgn");
                if (_msg.Fd12 != "0000")
                {
                    _message = GetError(_msg);
                    return false;
                }
                if (_msg.Fd67 != "Y")
                {
                    _message = string.Format("{0}当前不允许进行结账操作！", _batSysInfo.batname);
                    return false;
                }
            }
            return true;
        }
        /// <summary>
        /// 启动结帐
        /// </summary>
        /// <returns></returns>
        public override bool DoBatRz()
        {
            if (!_batSysInfo.stat.Equals("0") && !_batSysInfo.stat.Equals("3") && !_batSysInfo.stat.Equals("4"))
            {
                _message = string.Format("{0}当前状态并非未结账！不允许进行结账操作！", _batSysInfo.batname);
                return false;
            }

            InitFd(_msg);
            this.CommSend(_msg);
            _msg.Fd16 = "9006";
            this._msg.Fd21 = _batSysInfo.syscode;
            _msg.Fd47 = _batSysInfo.tx_date;
            _msg.Fd86 = _batSysInfo.batcode;//拉起各系统的批次号
            _msg.Fd18 = "RUNB";
            CallAgn(_msg, "CallAgn");
            if (_msg.Fd12 != "0000")
            {
                _message = GetError(_msg);
                return false;
            }


            if (_msg.Fd67 == "Y" || _msg.Fd67 == "Z")
            {
                _message = "拉起跑批成功";
                #region 更改状态、记录开始跑批的时间
                _batSysInfo.stat = "1";
                _batSysInfo.beg_date = DateTime.Now.ToLocalTime().ToString("yyyyMMddHHmmss");
                if (!SysBatchHandle.UpProcessBatSysInfo(_batSysInfo))
                {
                    _message = "批量启动成功，但更新状态失败！请检查！";
                }
                #endregion
                return true;
            }
            else
            {
                _message = "拉起跑批失败";
                return false;
            }
        }

        /// <summary>
        /// 获取系统批量执行的状态
        /// </summary>
        /// <returns></returns>
        public override string GetBatStat()
        {
            /**
             * 获取跑批的状态
             * 0、未处理 
             * 1、正在处理
             * 2、已成功 
             * 3、结账失败
             **/
            string result = "";


            #region 查询系统批次状态
            InitFd(_msg);
            this.CommSend(_msg);
            _msg.Fd16 = "9780";
            _msg.Fd65 = "0001";
            _msg.Fd47 = _batSysInfo.tx_date;
            _msg.Fd21 = _batSysInfo.syscode;
            _msg.Fd86 = _batSysInfo.batcode;//拉起各系统的批次号
            _msg.Fd18 = "RUNB";
            CallAgn(_msg, "CallAgn");
            if (_msg.Fd12 != "0000")
            {
                _message = GetError(_msg);
                result = "3";
                _batSysInfo.stat = result;
            }
            else if (string.IsNullOrEmpty(_msg.Fd12))
            {
                result = "3";
                _message = "未查询到批次状态";
                _batSysInfo.stat = result;
            }
            else if (_msg.Fd67 == "Y")
            {
                result = "2";
                _message = "结账成功";
                _batSysInfo.stat = result;
            }
            else if (_msg.Fd67 == "Z")
            {
                _message = "批量正在执行";
                result = "1";
                _batSysInfo.stat = result;
            }
            else if (_msg.Fd67 == "E")
            {
                _message = "结账失败";
                result = "3";
                _batSysInfo.stat = result;
            }
            else
            {
                _message = "状态异常";
                result = "3";
                _batSysInfo.stat = result;
            }
            #endregion

            return result;
        }

        /// <summary>
        /// 获取跑批系统信息
        /// 查询当天子批量详情 0002
        /// </summary>
        /// <returns></returns>
        public override T GetBatRzInfo<T>()
        {
            List<BatInfo> batlist = new List<BatInfo>();
            InitFd(_msg);
            this.CommSend(_msg);
            this._msg.Fd47 = _batSysInfo.tx_date;
            _msg.Fd16 = "9780";
            _msg.Fd18 = "RUNB";
            if (_batSysInfo.syscode == "06"
                || _batSysInfo.syscode == "08")
            {
                this._msg.Fd65 = "0006";
            }
            else
            {
                this._msg.Fd65 = "0002";
            }
            this._msg.Fd21 = _batSysInfo.syscode;
            _msg.Fd86 = _batSysInfo.batcode;//拉起各系统的批次号
            CallAgn(_msg, "CallAgn");
            if (typeof(T) == batlist.GetType())
            {
                if (_msg.Fd12 != "0000")
                {
                    _message = GetError(_msg);
                    return batlist as T;
                }
                var _fileData = _msg.FileData;
                #region 绑定数据
                BatInfo info_all = new BatInfo
                {
                    batnum = this._msg.Fd109_6,
                    batovernum = this._msg.Fd109_9,
                    batovertime = this._msg.Fd45,
                    remainingnum = this._msg.Fd116_1,
                    expecttime = this._msg.Fd46
                };
                batlist.Add(info_all);
                if (string.IsNullOrWhiteSpace(_fileData))
                    return batlist as T;
                var len = _fileData.Split('\n');//多少行数据        
                foreach (var item in len)
                {
                    if (!string.IsNullOrWhiteSpace(item))
                    {
                        var temp = item.Split(new string[] { "!~!" }, StringSplitOptions.None);
                        if (temp.Length > 5)
                        {
                            BatInfo info = new BatInfo
                            {
                                syscode = _batSysInfo.syscode,
                                batcode = _batSysInfo.batcode,
                                opteller = LoginUserInfo.TellerName,
                                batsubcode = temp[0],
                                batname = temp[1],
                                isredo = temp[2],
                                stat = temp[3],
                                //end_date = temp[4],
                                batnum = temp[4],
                                batovernum = temp[5],
                                batovertime = temp[6],
                                remainingnum = temp[7],
                                expecttime = temp[8],
                                chkteller = temp[10],
                                filler1 = temp[11]

                            };
                            batlist.Add(info);
                        }
                    }
                }
                #endregion
                return batlist as T;
            }
            else if (typeof(T) == _msg.GetType())
            {
                if (_msg.Fd12 != "0000")
                {
                    _message = GetError(_msg);
                    return _msg as T;
                }
                return _msg as T;
            }

            return default(T);
        }
        /// <summary>
        /// 查询跑批系统信息
        /// 查询历史子批量详细0003
        /// </summary>
        /// <returns></returns>
        public List<BatInfo> GetBatRzInfoHistory()
        {
            List<BatInfo> batlist = new List<BatInfo>();
            string _fileData = "";


            #region 查询历史子批量详细信息，传入不通参数平台内部来调度其他系统
            InitFd(_msg);
            this.CommSend(_msg);
            _msg.Fd47 = _batSysInfo.tx_date;
            _msg.Fd16 = "9780";
            _msg.Fd65 = "0003";
            _msg.Fd18 = "RUNB";
            this._msg.Fd21 = _batSysInfo.syscode;
            _msg.Fd86 = _batSysInfo.batcode;//拉起各系统的批次号
            CallAgn(_msg, "CallAgn");
            if (_msg.Fd12 != "0000")
            {
                _message = GetError(_msg);
                return batlist;
            }
            //  string FileName = this._msg.Fd81.ToString().Trim();
            //  _fileData = this.DonwLoadFromOther(FileName);
            _fileData = _msg.FileData;
            #endregion

            #region 绑定数据
            if (string.IsNullOrWhiteSpace(_fileData))
                return batlist;
            var len = _fileData.Split('\n');//多少行数据        
            foreach (var item in len)
            {
                if (!string.IsNullOrWhiteSpace(item))
                {
                    var temp = item.Split(new string[] { "!~!" }, StringSplitOptions.None);
                    if (temp.Length > 5)
                    {
                        BatInfo info = new BatInfo
                        {
                            syscode = _batSysInfo.syscode,
                            batcode = _batSysInfo.batcode,
                            batsubcode = temp[0],
                            batname = temp[1],
                            batdesp = temp[2],
                            stat = temp[3],
                            //end_date = temp[4],
                            batovertime = temp[4],
                            isredo = temp[5],
                            remainingnum = temp[6],
                            opteller = temp[7],
                            chkteller = temp[8],
                            filler1 = temp[9]
                        };
                        batlist.Add(info);
                    }
                }
            }
            #endregion
            return batlist;
        }

        /// <summary>
        /// 查询历史总批量详细0004
        /// </summary>
        /// <returns></returns>
        public List<BatSysInfo> GetBatSysHistory(string bat_date, string sys_code, string bat_code)
        {
            List<BatSysInfo> batlist = new List<BatSysInfo>();
            string _fileData = "";

            #region 其他系统通过联营通转，提供一个短交易，传入不通参数内部来调度其他系统

            InitFd(_msg);
            this.CommSend(_msg);
            _msg.Fd16 = "9780";
            _msg.Fd65 = "0004";
            _msg.Fd21 = sys_code;
            _msg.Fd47 = bat_date;
            _msg.Fd86 = bat_code;
            _msg.Fd18 = "RUNB";
            CallAgn(_msg, "CallAgn");
            if (_msg.Fd12 != "0000")
            {
                _message = GetError(_msg);
                return batlist;
            }
            //string FileName = this._msg.Fd81.ToString().Trim();
            //_fileData = this.DonwLoadFromOther(FileName);
            _fileData = _msg.FileData;
            #endregion

            #region 绑定数据
            if (string.IsNullOrWhiteSpace(_fileData))
                return batlist;
            var len = _fileData.Split('\n');//多少行数据        
            foreach (var item in len)
            {
                if (!string.IsNullOrWhiteSpace(item))
                {
                    var temp = item.Split(new string[] { "!~!" }, StringSplitOptions.None);
                    if (temp.Length > 12)
                    {
                        BatSysInfo info = new BatSysInfo
                        {
                            syscode = sys_code,
                            batcode = bat_code,
                            //batsubcode = temp[0],
                            //batname = temp[1],
                            ///系统名称、总批次名称、总批量说明	、总批量处理状态、开始时间、结束时间、
                            /// 耗时、子批次数量 、执行次数、最后更新时间、操作员、复核员、备注（错误的原因）
                            //总批量说明
                            batdesp = temp[2],
                            //总批量处理状态
                            stat = temp[3],
                            //开始时间
                            beg_date = temp[4],
                            //结束时间
                            end_date = temp[5],
                            //耗时
                            filler2 = temp[6],
                            //子批次数量
                            subcnt = temp[7],
                            //执行次数
                            bat_ind = temp[8],
                            //最后更新时间
                            tx_date = temp[9],
                            //操作员
                            opteller = temp[10],
                            //复核员
                            chkteller = temp[11],
                            //备注
                            filler1 = temp[12]
                        };
                        batlist.Add(info);
                    }
                }
            }
            #endregion

            return batlist;
        }
        /// <summary>
        /// 下载文件内容
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        private string DonwLoadFromKernel(string fileName)
        {
            string fileContent = string.Empty;

            //this.OwnerContainer.ModalInvoke(new Action(() =>
            //{
            //    Thread.Sleep(2000);
            byte[] byteDatas = TradeHandle.DownLoadStreamFromGtp(fileName, "BATCHSERVER");
            fileContent = Encoding.Default.GetString(byteDatas);

            //}), "正在下载文件...");
            return fileContent;
        }

        /// <summary>
        /// 联银通转来的文件内容
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        //private string DonwLoadFromOther(string fileName)
        //{
        //    string fileContent = string.Empty;
        //    byte[] byteDatas = TradeHandle.DownLoadStreamFromGtp(fileName,  "BATCHAGN");
        //    fileContent = Encoding.Default.GetString(byteDatas);
        //    return fileContent;
        //}

    }

}
