// 定义 Message 类，由于不清楚具体实现，这里简单模拟结构
import Message from "@/views/front/develop/Communication/Message";
import { TransitType } from "@/views/front/develop/Communication/MessageHelper/TransitType";
import { BatSysInfo } from "@/views/front/develop/ServiceProxy.Ext/ServiceEntitys/BatSysInfo";
import LoginUserInfo from "@/views/front/platfrom/common/RunningParameters/LoginUserInfo";
import SystemFunction from "@/views/front/develop/Library.Ext/Function/SystemFunction";
import SysBatchHandle from "../../../ServiceProxy.Ext/ServiceHelper/SysBatchHandle";
import { BatInfo } from "../../../ServiceProxy.Ext/ServiceEntitys/BatInfo";
import TradeHandle from "@/views/front/platfrom/serviceproxy/ServiceHelper/TradeHandle";

// 定义 BatSysInfo 类，由于不清楚具体实现，这里简单模拟结构
// 定义 BatInfo 类，由于不清楚具体实现，这里简单模拟结构


// 模拟 LoginUserInfo 对象

// 模拟 SystemFunction 对象


// 模拟 SysBatchHandle 对象

// 模拟 TradeHandle 对象


/**
 * 各系统批量操作抽象类
 */
abstract class BatchAction {
    // 字段集合
    protected _msg: Message = new Message();
    protected _batSysInfo: BatSysInfo;
    public get BatInfo(): BatSysInfo {
        return this._batSysInfo;
    }

    protected _message: string = "";
    public get Message(): string {
        return this._message;
    }

    /**
     * 启动前，校验批量是否具备拉起的条件
     */
    public PreviewDoBatRz(): boolean {
        return true;
    }

    /**
     * 执行批量操作
     */
    public DoBatRz(): boolean {
        return true;
    }

    /**
     * 获取系统批次状态
     */
    public GetBatStat(): string {
        return "";
    }

    /**
     * 更新批量系统的子批量信息
     */
    public GetBatRzInfo<T extends {}>(): T {
        let temp: T | null = null;
        return temp as T;
    }

    /**
     * 获取错误信息
     * @param msg 报文通讯对象
     */
    protected GetError(msg: Message): string {
        let result = "[COMM]通讯失败，请检查服务是否正常！";
        if (msg) {
            if (msg.Fd12 && msg.Fd12.trim() !== "") {
                if (msg.Fd13 && msg.Fd13.trim() !== "") {
                    result = `[${msg.Fd12}],${msg.Fd13.replace(/\s/g, "")}`;
                } else {
                    result = SystemFunction.GetError(msg.Fd12);
                }
            }
        }
        return result;
    }

    /**
     * 初始化域
     * @param message 报文通讯对象
     */
    protected InitFd(message: Message): void {
        message.Init();
    }

    /**
     * 初始化公共域
     * @param message 报文通讯对象
     */
    protected CommSend(message: Message): void {
        // 开户行机构号
        message.Fd2 = LoginUserInfo.KinbrNo;
        // 机构号
        message.Fd3 = LoginUserInfo.KinbrNo;
        // 交易日期
        message.Fd5 = LoginUserInfo.TradeDate;
        // 发生时间
        message.Fd6 = new Date().toLocaleTimeString("zh-CN", { hour12: false, hour: "2-digit", minute: "2-digit", second: "2-digit" }).replace(/:/g, "");
        // 柜员号
        message.Fd7 = LoginUserInfo.TellerNo;
        // 终端号
        message.Fd10 = LoginUserInfo.TtyName;
        message.Fd96_9 = "0100"; // 本行柜面
        message.Fd96_7 = "0";
        message.Fd96_8 = "0";
        message.Fd96_C = "0100"; // 本行柜面
        message.Fd96_D = LoginUserInfo.TradeDate;
        message.Fd96_E = message.Fd6;
    }

    /**
     * 和平台通讯API
     * @param message 报文通讯对象
     * @param transitType TransitType枚举值
     */
    protected CallAgn(message: Message, transitType: string): boolean {
        let returnValue = false;
        let preTradeNo = message.Fd16;
        message.TransitNode = TransitType.CallAgnRun;
        message.BitMapType = Message.BitMap.Agn;
        returnValue = message.DoTransit();
        if (message.Fd16 !== preTradeNo && message.Fd12 === "0000") {
            message.Fd12 = "COMN";
        }
        return returnValue;
    }
}

/**
 * 批量执行外围系统实体
 */
class BatActionKernel extends BatchAction {
    constructor(batSysInfo?: BatSysInfo) {
        super();
        if (batSysInfo) {
            // 外围系统编号及子批次编号
            if (batSysInfo.syscode === "" && batSysInfo.batcode === "") {
                throw new Error("参数指定的结帐系统不是结账系统!");
            }
            this._batSysInfo = batSysInfo;
        }
    }

    /**
     * 校验批次是否可以进行批量结账
     */
    public override PreviewDoBatRz(): boolean {
        if (this._batSysInfo.batcheck === "1") {
            this.InitFd(this._msg);
            this.CommSend(this._msg);
            this._msg.Fd16 = "9006";
            this._msg.Fd21 = this._batSysInfo.syscode;
            this._msg.Fd47 = this._batSysInfo.tx_date;
            this._msg.Fd86 = this._batSysInfo.batcode; // 拉起各系统的批次号
            this._msg.Fd18 = "RUNB";
            this.CallAgn(this._msg, "CallAgn");
            if (this._msg.Fd12 !== "0000") {
                this._message = this.GetError(this._msg);
                return false;
            }
            if (this._msg.Fd67 !== "Y") {
                this._message = `${this._batSysInfo.batname}当前不允许进行结账操作！`;
                return false;
            }
        }
        return true;
    }

    /**
     * 启动结帐
     */
    public override DoBatRz(): boolean {
        if (this._batSysInfo.stat !== "0" && this._batSysInfo.stat !== "3" && this._batSysInfo.stat !== "4") {
            this._message = `${this._batSysInfo.batname}当前状态并非未结账！不允许进行结账操作！`;
            return false;
        }

        this.InitFd(this._msg);
        this.CommSend(this._msg);
        this._msg.Fd16 = "9006";
        this._msg.Fd21 = this._batSysInfo.syscode;
        this._msg.Fd47 = this._batSysInfo.tx_date;
        this._msg.Fd86 = this._batSysInfo.batcode; // 拉起各系统的批次号
        this._msg.Fd18 = "RUNB";
        this.CallAgn(this._msg, "CallAgn");
        if (this._msg.Fd12 !== "0000") {
            this._message = this.GetError(this._msg);
            return false;
        }

        if (this._msg.Fd67 === "Y" || this._msg.Fd67 === "Z") {
            this._message = "拉起跑批成功";
            // 更改状态、记录开始跑批的时间
            this._batSysInfo.stat = "1";
            this._batSysInfo.beg_date = new Date().toISOString().replace(/[-T:.Z]/g, "").slice(0, 14);
            if (!SysBatchHandle.UpProcessBatSysInfo(this._batSysInfo)) {
                this._message = "批量启动成功，但更新状态失败！请检查！";
            }
            return true;
        } else {
            this._message = "拉起跑批失败";
            return false;
        }
    }

    /**
     * 获取系统批量执行的状态
     */
    public override GetBatStat(): string {
        // 获取跑批的状态
        // 0、未处理 
        // 1、正在处理
        // 2、已成功 
        // 3、结账失败
        let result = "";

        // 查询系统批次状态
        this.InitFd(this._msg);
        this.CommSend(this._msg);
        this._msg.Fd16 = "9780";
        this._msg.Fd65 = "0001";
        this._msg.Fd47 = this._batSysInfo.tx_date;
        this._msg.Fd21 = this._batSysInfo.syscode;
        this._msg.Fd86 = this._batSysInfo.batcode; // 拉起各系统的批次号
        this._msg.Fd18 = "RUNB";
        this.CallAgn(this._msg, "CallAgn");
        if (this._msg.Fd12 !== "0000") {
            this._message = this.GetError(this._msg);
            result = "3";
            //     this._batSysInfo.stat = result; hulei 代码逻辑错误
            // } else if (this._msg.Fd12 === "") {
            //     result = "3";
            //     this._message = "未查询到批次状态";
            //     this._batSysInfo.stat = result;
        } else if (this._msg.Fd67 === "Y") {
            result = "2";
            this._message = "结账成功";
            this._batSysInfo.stat = result;
        } else if (this._msg.Fd67 === "Z") {
            this._message = "批量正在执行";
            result = "1";
            this._batSysInfo.stat = result;
        } else if (this._msg.Fd67 === "E") {
            this._message = "结账失败";
            result = "3";
            this._batSysInfo.stat = result;
        } else {
            this._message = "状态异常";
            result = "3";
            this._batSysInfo.stat = result;
        }

        return result;
    }

    /**
     * 获取跑批系统信息
     * 查询当天子批量详情 0002
     */
    public override GetBatRzInfo<T extends {}>(): T {
        let batlist: BatInfo[] = [];
        this.InitFd(this._msg);
        this.CommSend(this._msg);
        this._msg.Fd47 = this._batSysInfo.tx_date;
        this._msg.Fd16 = "9780";
        this._msg.Fd18 = "RUNB";
        if (this._batSysInfo.syscode === "06" || this._batSysInfo.syscode === "08") {
            this._msg.Fd65 = "0006";
        } else {
            this._msg.Fd65 = "0002";
        }
        this._msg.Fd21 = this._batSysInfo.syscode;
        this._msg.Fd86 = this._batSysInfo.batcode; // 拉起各系统的批次号
        this.CallAgn(this._msg, "CallAgn");//批量处理 展示不做hulei-/
        // if (typeof (T) === batlist) {//hulei-r系统类型
        //     if (this._msg.Fd12 !== "0000") {
        //         this._message = this.GetError(this._msg);
        //         return batlist as unknown as T;
        //     }
        //     let _fileData = this._msg.fileData;
        //     // 绑定数据
        //     let info_all: BatInfo = {
        //         batnum: this._msg.Fd109_6,
        //         batovernum: this._msg.Fd109_9,
        //         batovertime: this._msg.Fd45,
        //         remainingnum: this._msg.Fd116_1,
        //         expecttime: this._msg.Fd46,
        //         syscode: "",
        //         batcode: "",
        //         batsubcode: "",
        //         batname: "",
        //         isredo: "",
        //         stat: "",
        //         end_date: "",
        //         opteller: "",
        //         chkteller: "",
        //         filler1: ""
        //     };
        //     batlist.push(info_all);
        //     if (_fileData.trim() === "") {
        //         return batlist as unknown as T;
        //     }
        //     let len = _fileData.split('\n'); // 多少行数据        
        //     for (let item of len) {
        //         if (item.trim() !== "") {
        //             let temp = item.split("!~!");
        //             if (temp.length > 5) {
        //                 let info: BatInfo = {
        //                     syscode: this._batSysInfo.syscode,
        //                     batcode: this._batSysInfo.batcode,
        //                     opteller: LoginUserInfo.TellerName,
        //                     batsubcode: temp[0],
        //                     batname: temp[1],
        //                     isredo: temp[2],
        //                     stat: temp[3],
        //                     //end_date = temp[4],
        //                     batnum: temp[4],
        //                     batovernum: temp[5],
        //                     batovertime: temp[6],
        //                     remainingnum: temp[7],
        //                     expecttime: temp[8],
        //                     chkteller: temp[10],
        //                     filler1: temp[11],
        //                     end_date: ""
        //                 };
        //                 batlist.push(info);
        //             }
        //         }
        //     }
        //     return batlist as unknown as T;
        // } else if (T === Message) {
        //     if (this._msg.Fd12 !== "0000") {
        //         this._message = this.GetError(this._msg);
        //         return this._msg as unknown as T;
        //     }
        //     return this._msg as unknown as T;
        // }

        return null as T;
    }

    /**
     * 查询跑批系统信息
     * 查询历史子批量详细0003
     */
    public GetBatRzInfoHistory(): BatInfo[] {
        let batlist: BatInfo[] = [];
        let _fileData = "";

        // 查询历史子批量详细信息，传入不通参数平台内部来调度其他系统
        this.InitFd(this._msg);
        this.CommSend(this._msg);
        this._msg.Fd47 = this._batSysInfo.tx_date;
        this._msg.Fd16 = "9780";
        this._msg.Fd65 = "0003";
        this._msg.Fd18 = "RUNB";
        this._msg.Fd21 = this._batSysInfo.syscode;
        this._msg.Fd86 = this._batSysInfo.batcode; // 拉起各系统的批次号
        this.CallAgn(this._msg, "CallAgn");
        if (this._msg.Fd12 !== "0000") {
            this._message = this.GetError(this._msg);
            return batlist;
        }
        _fileData = this._msg.fileData;

        // 绑定数据
        if (_fileData.trim() === "") {
            return batlist;
        }
        let len = _fileData.split('\n'); // 多少行数据        
        for (let item of len) {
            if (item.trim() !== "") {
                let temp = item.split("!~!");
                if (temp.length > 5) {
                    let info: BatInfo = {
                        syscode: this._batSysInfo.syscode,
                        batcode: this._batSysInfo.batcode,
                        batsubcode: temp[0],
                        batname: temp[1],
                        batdesp: temp[2],
                        stat: temp[3],
                        //end_date = temp[4],
                        batovertime: temp[4],
                        isredo: temp[5],
                        remainingnum: temp[6],
                        opteller: temp[7],
                        chkteller: temp[8],
                        filler1: temp[9],
                        batnum: "",
                        batovernum: "",
                        expecttime: "",
                        tx_date: "",
                        end_date: "",
                        ChangeFromTable: function (parms: any[]): void {
                            throw new Error("Function not implemented.");
                        },
                        ChangeToTable: function (): string[] {
                            throw new Error("Function not implemented.");
                        }
                    };
                    batlist.push(info);
                }
            }
        }
        return batlist;
    }

    /**
     * 查询历史总批量详细0004
     * @param bat_date 批次日期
     * @param sys_code 系统编码
     * @param bat_code 批次编码
     */
    public GetBatSysHistory(bat_date: string, sys_code: string, bat_code: string): BatSysInfo[] {
        let batlist: BatSysInfo[] = [];
        let _fileData = "";

        // 其他系统通过联营通转，提供一个短交易，传入不通参数内部来调度其他系统
        this.InitFd(this._msg);
        this.CommSend(this._msg);
        this._msg.Fd16 = "9780";
        this._msg.Fd65 = "0004";
        this._msg.Fd21 = sys_code;
        this._msg.Fd47 = bat_date;
        this._msg.Fd86 = bat_code;
        this._msg.Fd18 = "RUNB";
        this.CallAgn(this._msg, "CallAgn");
        if (this._msg.Fd12 !== "0000") {
            this._message = this.GetError(this._msg);
            return batlist;
        }
        _fileData = this._msg.fileData;

        // 绑定数据
        if (_fileData.trim() === "") {
            return batlist;
        }
        let len = _fileData.split('\n'); // 多少行数据        
        for (let item of len) {
            if (item.trim() !== "") {
                let temp = item.split("!~!");
                if (temp.length > 12) {
                    let info: BatSysInfo = {
                        syscode: sys_code,
                        batcode: bat_code,
                        //batsubcode = temp[0],
                        //batname = temp[1],
                        // 系统名称、总批次名称、总批量说明	、总批量处理状态、开始时间、结束时间、
                        // 耗时、子批次数量 、执行次数、最后更新时间、操作员、复核员、备注（错误的原因）
                        // 总批量说明
                        batdesp: temp[2],
                        // 总批量处理状态
                        stat: temp[3],
                        // 开始时间
                        beg_date: temp[4],
                        // 结束时间
                        end_date: temp[5],
                        // 耗时
                        filler2: temp[6],
                        // 子批次数量
                        subcnt: temp[7],
                        // 执行次数
                        bat_ind: temp[8],
                        // 最后更新时间
                        tx_date: temp[9],
                        // 操作员
                        opteller: temp[10],
                        // 复核员
                        chkteller: temp[11],
                        // 备注
                        filler1: temp[12],
                        batname: "",
                        mainseqn: "",
                        elapsetime: undefined,
                        refreshtime: "",
                        batcheck: "",
                        ChangeFromTable: function (parms: any[]): void {
                            throw new Error("Function not implemented.");
                        },
                        ChangeToTable: function (): string[] {
                            throw new Error("Function not implemented.");
                        }
                    };
                    batlist.push(info);
                }
            }
        }
        return batlist;
    }

    /**
     * 下载文件内容
     * @param fileName 文件名
     */
    private DonwLoadFromKernel(fileName: string): string {
        let fileContent = "";
        let byteDatas = TradeHandle.DownLoadStreamFromGtp(fileName, "BATCHSERVER");//hulei-r 缺少方法DownLoadStreamFromGtp
        let decoder = new TextDecoder();
        // fileContent = decoder.decode(byteDatas);
        return fileContent;
    }

    // /**
    //  * 联银通转来的文件内容
    //  * @param fileName 文件名
    //  */
    // private DonwLoadFromOther(fileName: string): string {
    //     let fileContent = "";
    //     let byteDatas = TradeHandle.DownLoadStreamFromGtp(fileName, "BATCHAGN");
    //     let decoder = new TextDecoder();
    //     fileContent = decoder.decode(byteDatas);
    //     return fileContent;
    // }
}
