﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TellerSystem.ServiceProxy.Ext.ServiceEntitys;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class BatchHelper
    {
        /// <summary>
        /// 根据系统编号、批量编号判断该编号批量现在能否执行
        /// </summary>
        /// <param name="syscode">系统编号</param>
        /// <param name="batcode">批量编号</param>
        /// <returns></returns>
        public static bool IsAllowRun(string syscode,string batcode)
        {
            //查询批量顺序执行表数据
            List<BatSysInfo> data = SysBatchHandle.GetBatSysInfos("1=1");
            if(data == null || data.Count == 0)
            {
                return false;
            }

            //查询是否存在该调记录
            var result = data.FindAll(delegate(BatSysInfo batSysInfo)
                             {
                                 if (batSysInfo.syscode.Equals(syscode) && batSysInfo.batcode.Equals(batcode))
                                     return true;
                                 return false;
                             });
            if(result.Count == 0)
            {
                return false;
            }

            //查到的目标批量实例
            BatSysInfo currentItem = result[0];
            //批量状态：0:未处理  1:正在处理  2:已成功  3:结账失败
            foreach (BatSysInfo batSysInfo in data)
            {
                //查询到目标批量实例(未处理状态)
                if (batSysInfo.syscode.Equals(syscode) && batSysInfo.batcode.Equals(batcode))
                {
                   var temp = data.Where(x => Convert.ToInt32(currentItem.mainseqn) > Convert.ToInt32(x.mainseqn) && Convert.ToInt32(currentItem.mainseqn) - Convert.ToInt32(x.mainseqn) < 2).FirstOrDefault();
                   if (temp == null)
                       return true; //说明是第一个元素
                   if (temp.stat != "2")
                   {
                       return false;
                   }
                   else
                   {
                       if (batSysInfo.stat.Equals("0")||batSysInfo.stat.Equals("3"))
                       {
                           return true;
                       }
                   }
                }
                   

                if (batSysInfo.stat.Equals("2"))
                {
                    //已成功执行
                    continue;
                }               
                if (batSysInfo.stat.Equals("1"))
                {
                    //正在处理  同级顺序
                    if (batSysInfo.mainseqn.Equals(currentItem.mainseqn))
                    {
                        continue;
                    }
                    //return false;
                }               
            }
            return false;
        }
       
        /// <summary>
        /// 更新指定批量状态
        /// </summary>
        /// <param name="syscode">系统编号</param>
        /// <param name="batcode">批量编号</param>
        /// <param name="stat">状态 0:未处理  1:正在处理  2:已成功  3:批量失败</param>
        /// <returns></returns>
        public static bool UpdateBatchStat(string syscode,string batcode,string stat)
        {
            List<BatSysInfo> data = SysBatchHandle.GetBatSysInfos("syscode='" + syscode + "' and batcode='" + batcode + "'");
            if(data == null || data.Count == 0)
            {
                return false;
            }
            data[0].stat = stat;
            return SysBatchHandle.UpDateBatSysInfo(data[0]);
        }
    }

}
