﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Platform.Controls.ControlsHelper;
using Platform.Controls.TradeExtension;
using Platform.Presentation.FocusManagers;
using Platform.Presentation.Interfaces;
using TellerSystem.Devices;
using TellerSystem.Devices.Config;
using TellerSystem.Devices.Interface;
using System.Windows.Threading;
using TellerSystem.Library.Ext.Function;
using Platform.Library;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class DeviceManager : DependencyObject
    {
        #region 磁条读卡器相关

        public static DeviceEntity<SwipeCardReaderEntity> GetSwipeCardData(DependencyObject obj)
        {
            return (DeviceEntity<SwipeCardReaderEntity>)obj.GetValue(SwipeCardDataProperty);
        }

        public static void SetSwipeCardData(DependencyObject obj, DeviceEntity<SwipeCardReaderEntity> value)
        {
            obj.SetValue(SwipeCardDataProperty, value);
        }

        /// <summary>
        ///磁条数据附加属性
        /// </summary>
        public static readonly DependencyProperty SwipeCardDataProperty =
            DependencyProperty.RegisterAttached("SwipeCardData", typeof(DeviceEntity<SwipeCardReaderEntity>), typeof(DeviceManager), new UIPropertyMetadata(null));

        public static SwipeCardReaderType? GetSwipeCardReaderTracksReader(DependencyObject obj)
        {
            return (SwipeCardReaderType?)obj.GetValue(SwipeCardReaderTracksReaderProperty);
        }

        public static void SetSwipeCardReaderTracksReader(DependencyObject obj, SwipeCardReaderType? value)
        {
            obj.SetValue(SwipeCardReaderTracksReaderProperty, value);
        }

        /// <summary>
        /// 磁条数据获取方式
        /// </summary>
        public static readonly DependencyProperty SwipeCardReaderTracksReaderProperty =
            DependencyProperty.RegisterAttached("SwipeCardReaderTracksReader", typeof(SwipeCardReaderType?), typeof(DeviceManager),
            new PropertyMetadata(null, new PropertyChangedCallback(OnIsEabledCardReaderChanged)));

        /// <summary>
        /// 磁条读命令模式
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public static int GetSwipeCardCommand(DependencyObject obj)
        {
            return (int)obj.GetValue(SwipeCardCommandProperty);
        }

        public static void SetSwipeCardCommand(DependencyObject obj, int value)
        {
            obj.SetValue(SwipeCardCommandProperty, value);
        }

        // Using a DependencyProperty as the backing store for SwipeCardCommand.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SwipeCardCommandProperty =
            DependencyProperty.RegisterAttached("SwipeCardCommand", typeof(int), typeof(DeviceManager), new UIPropertyMetadata(2));
        #endregion 磁条读卡器相关

        #region 密码键盘相关

        public static DeviceEntity<PassWordEntity> GetPwdData(DependencyObject obj)
        {
            return (DeviceEntity<PassWordEntity>)obj.GetValue(PwdDataProperty);
        }

        public static void SetPwdData(DependencyObject obj, DeviceEntity<PassWordEntity> value)
        {
            obj.SetValue(PwdDataProperty, value);
        }

        /// <summary>
        ///密码数据附加属性
        /// </summary>
        public static readonly DependencyProperty PwdDataProperty =
            DependencyProperty.RegisterAttached("PwdData", typeof(DeviceEntity<PassWordEntity>), typeof(DeviceManager), new UIPropertyMetadata(null));

        public static PassWordKeyBoardReaderType? GetPassWordKeyBoardReader(DependencyObject obj)
        {
            return (PassWordKeyBoardReaderType?)obj.GetValue(PassWordKeyBoardReaderProperty);
        }

        public static void SetPassWordKeyBoardReader(DependencyObject obj, PassWordKeyBoardReaderType? value)
        {
            obj.SetValue(PassWordKeyBoardReaderProperty, value);
        }

        /// <summary>
        /// 密码数据获取方式
        /// </summary>
        public static readonly DependencyProperty PassWordKeyBoardReaderProperty =
            DependencyProperty.RegisterAttached("PassWordKeyBoardReader", typeof(PassWordKeyBoardReaderType?), typeof(DeviceManager),
            new PropertyMetadata(null, new PropertyChangedCallback(OnIsEabledCardReaderChanged)));

        #endregion 密码键盘相关

        #region 二代身份证相关

        public static DeviceEntity<IdentityCardEntity> GetIdentityCardData(DependencyObject obj)
        {
            var data = obj.GetValue(IdentityCardDataProperty) as DeviceEntity<IdentityCardEntity>;
            if (data == null)
            {
                var aTextBox = obj as TextBox;
                if (aTextBox == null)
                    return new DeviceEntity<IdentityCardEntity>();
                return new DeviceEntity<IdentityCardEntity>()
                           {
                               HasError = false,
                               Data = new IdentityCardEntity()
                                          {
                                              IdNumber = aTextBox.Text.Trim()
                                          }
                           };
            }
            return data;
            //return (DeviceEntity<IdentityCardEntity>)obj.GetValue(IdentityCardDataProperty);
        }

        public static void SetIdentityCardData(DependencyObject obj, DeviceEntity<IdentityCardEntity> value)
        {
            obj.SetValue(IdentityCardDataProperty, value);
        }

        /// <summary>
        /// 身份证证数据附加属性
        /// </summary>
        public static readonly DependencyProperty IdentityCardDataProperty =
            DependencyProperty.RegisterAttached("IdentityCardData", typeof(DeviceEntity<IdentityCardEntity>), typeof(DeviceManager), new UIPropertyMetadata(null));

        public static IdentityCardReaderType? GetIdentityCardReader(DependencyObject obj)
        {
            return (IdentityCardReaderType?)obj.GetValue(IdentityCardReaderProperty);
        }

        public static void SetIdentityCardReader(DependencyObject obj, IdentityCardReaderType? value)
        {
            obj.SetValue(IdentityCardReaderProperty, value);
        }

        /// <summary>
        /// 获取身份证信息方式
        /// </summary>
        public static readonly DependencyProperty IdentityCardReaderProperty =
            DependencyProperty.RegisterAttached("IdentityCardReader", typeof(IdentityCardReaderType?), typeof(DeviceManager),
            new PropertyMetadata(null, new PropertyChangedCallback((d, e) =>
            {
                OnIsEabledCardReaderChanged(d, e);
                //调用读卡器情况，为控件配置弹窗身份核查触发开关
                if (d is TextBox)
                {
                    var tb = d as TextBox;
                    tb.Loaded -= tb_Loaded;
                    //延迟触发，否则xaml中设置应发的操作会产生异常
                    if (tb.IsLoaded)
                        tb_Loaded(tb, new RoutedEventArgs(FrameworkElement.LoadedEvent));
                    else
                        tb.Loaded += tb_Loaded;
                }
            })));

        static void tb_Loaded(object sender, RoutedEventArgs e)
        {
            var tb = sender as TextBox;
            tb.Loaded -= tb_Loaded;
            try
            {
                if (DesignerProperties.GetIsInDesignMode(tb)) return;
                var page = (dynamic)(tb.FindVisualTreeParent(x => x is IPage) as IPage);
                string name = null;//默认关闭
                var identityCardReader = GetIdentityCardReader(tb);
                if (identityCardReader != null && identityCardReader != IdentityCardReaderType.Input)
                {
                    if (page.TradeAPIs.GetTradeMetadata("isIdCheckCombo") == "true")
                    {
                        var filter = page.TradeAPIs.GetTradeMetadata("IdCheckComboFilter");
                        if (System.Text.RegularExpressions.Regex.IsMatch(tb.Name, filter))
                            name = string.Empty;//命中时开启
                    }
                }
                var type = AppDomain.CurrentDomain.GetAssemblies().FirstOrDefault(x => x.FullName.StartsWith("TellerSystem.Controls.Ext"))
                    .GetType("TellerSystem.Controls.Ext.ControlsHelper.ShowPages");
                type.InvokeMember("SetIdCheckCombo", System.Reflection.BindingFlags.InvokeMethod | System.Reflection.BindingFlags.Public | System.Reflection.BindingFlags.Static
                   , null, null, new object[] { page.TradeAPIs, tb, name });
            }
            catch (Exception ex)
            {
                Platform.Common.LogSystem.PlatformLogger.SystemErrorInfo("IdentityCardReader:执行SetIdCheckCombo发生异常！", ex);
            }
        }

        public static IdentityCardType GetIdentityCardType(DependencyObject obj)
        {
            return (IdentityCardType)obj.GetValue(IdentityCardTypeProperty);
        }

        public static void SetIdentityCardType(DependencyObject obj, IdentityCardType value)
        {
            obj.SetValue(IdentityCardTypeProperty, value);
        }

        // Using a DependencyProperty as the backing store for IdentityCardType.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IdentityCardTypeProperty =
            DependencyProperty.RegisterAttached("IdentityCardType", typeof(IdentityCardType), typeof(DeviceManager), new UIPropertyMetadata(IdentityCardType.IdentityCard));

        #endregion 二代身份证相关

        #region IC卡相关

        public static readonly DependencyProperty ICCardDataProperty =
            DependencyProperty.RegisterAttached("ICCardData", typeof(DeviceEntity<ICCardEntity>),
                                                typeof(DeviceManager), new UIPropertyMetadata(null));

        public static readonly DependencyProperty ICCardReaderProperty =
            DependencyProperty.RegisterAttached("ICCardReader", typeof(ICCardReaderType?), typeof(DeviceManager),
                                                new UIPropertyMetadata(null, OnIsEabledCardReaderChanged));

        public static DeviceEntity<ICCardEntity> GetICCardData(DependencyObject obj)
        {
            return (DeviceEntity<ICCardEntity>)obj.GetValue(ICCardDataProperty);
        }

        public static void SetICCardData(DependencyObject obj, DeviceEntity<ICCardEntity> value)
        {
            obj.SetValue(ICCardDataProperty, value);
        }

        // IC卡信息


        public static ICCardReaderType? GetICCardReader(DependencyObject obj)
        {
            return (ICCardReaderType?)obj.GetValue(ICCardReaderProperty);
        }

        public static void SetICCardReader(DependencyObject obj, ICCardReaderType? value)
        {
            obj.SetValue(ICCardReaderProperty, value);
        }

        // 获取IC卡信息方式

        #endregion

        #region 业务流程

        /// <summary>
        /// 设置磁卡属性改变
        /// </summary>
        /// <param name="d"></param>
        /// <param name="e"></param>
        private static void OnIsEabledCardReaderChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            // 是否设计时
            if (!(bool)DesignerProperties.IsInDesignModeProperty.GetMetadata(typeof(DependencyObject)).DefaultValue)
            {
                if (d is TextBox || d is PasswordBox)
                {
                    FocusManager.RemoveGotFocusHandler(d, tb_GotFocus);
                    FocusManager.AddGotFocusHandler(d, tb_GotFocus);
                }
            }
        }

        /// <summary>
        /// 获得焦点设备
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private static void tb_GotFocus(object sender, RoutedEventArgs e)
        {
            #region 查找控件所属IPage对象

            IPage parentPage = null;
            var parent = sender as DependencyObject;
            while (parent != null)
            {
                if (parent is IPage)
                {
                    break;
                }
                parent = VisualTreeHelper.GetParent(parent);
            }
            if (parent != null)
            {
                parentPage = parent as IPage;
            }

            #endregion 查找控件所属IPage对象

            var element = sender as FrameworkElement;
            if (element != null)
            {
                element.ToolTip = "按F10可重新调用设备！";
                element.PreviewKeyUp -= ActiveDevice;
                element.PreviewKeyUp += ActiveDevice;
            }

            //磁条读方式业务
            SwipeCardControl(sender, parentPage);

            //密码输入方式业务
            PassWordControl(sender, parentPage);

            //身份证输入方式业务
            IdentityCardControl(sender, parentPage);

            //IC卡输入方式业务
            ICCardControl(sender, parentPage);
        }

        private static void ActiveDevice(object sender, System.Windows.Input.KeyEventArgs e)
        {
            //var pressKey = HotKeyHelper.GetKeyName(e.Key);

            if (e.Key == System.Windows.Input.Key.F10 || e.Key == System.Windows.Input.Key.System)
            {
                tb_GotFocus(sender, null);
                e.Handled = true;
            }
        }

        /// <summary>
        /// 磁条读写器控制流程
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="ownerPage"></param>
        //private static void SwipeCardControl(object sender, IPage ownerPage)
        //{
        //    UserFunction.HasJZ = false;
        //    var tb = sender as TextBox;
        //    if (tb == null)
        //        return;
        //    int command = GetSwipeCardCommand(tb);
        //    tb.ClearValue(SwipeCardDataProperty);
        //    tb.ClearValue(ICCardDataProperty);

        //    //判断父交易容器
        //    if (ownerPage == null) return;

        //    bool isRepeatInput = TextBoxHelper.GetIsRepeatInput(tb);
        //    if (GetSwipeCardReaderTracksReader(tb) == SwipeCardReaderType.Input)
        //    {
        //        //方式一：手工输入
        //        //可输入
        //        SetReadOnly(tb, false);
        //        //重复输入
        //        TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
        //        UserFunction.HasJZ = false;
        //    }
        //    if (GetSwipeCardReaderTracksReader(tb) == SwipeCardReaderType.ReadCard)
        //    {
        //        //方式二：插卡方式
        //        SetReadOnly(tb, true);
        //        TextBoxHelper.SetIsRepeatInput(tb, false);
        //        tb.Dispatcher.BeginInvoke(new Action<TextBox>(x =>
        //        {
        //            while (true)
        //            {
        //                if (DeviceConfig.ICCardEnabled)
        //                {
        //                    var result = QuestionMessage(ownerPage, "确认采取设备读卡号？", MessageBoxButton.OKCancel, MessageBoxResult.OK);
        //                    if (result == MessageBoxResult.OK)
        //                    {
        //                        DeviceEntity<ICCardEntity> data = DeviceHelper.GetICCardNo();
        //                        if (data.HasError)
        //                        {
        //                            string strenable = DeviceConfig.SwipeCardEnabled == true ? "调用IC卡设备失败，准备调用磁条刷卡器！\n" : "调用IC卡设备失败！\n";
        //                            QuestionMessage(ownerPage, strenable, MessageBoxButton.OK, MessageBoxResult.OK);
        //                            if (DeviceConfig.SwipeCardEnabled)
        //                            {
        //                                //修改弹出窗口方式：先读取，失败后再询问是否继续
        //                                var CardTracks = DeviceHelper.ReadCardTracks(command);
        //                                if (CardTracks.HasError)
        //                                {
        //                                    result = QuestionMessage(ownerPage, "磁条刷卡器调用失败，是否重新调用？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes);
        //                                    if (result != MessageBoxResult.Yes)
        //                                    {
        //                                        //用户取消操作，跳出循环
        //                                        break;
        //                                    }
        //                                }
        //                                else
        //                                {
        //                                    //成功读取账号，跳出循环
        //                                    SetSwipeCardData(x, CardTracks);
        //                                    x.Text = CardTracks.Data.CardNo;
        //                                    UserFunction.HasJZ = true;
        //                                    if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
        //                                    {
        //                                        //业务逻辑执行失败，需要提示，否则造成死循环
        //                                        result = QuestionMessage(ownerPage, "是否重新调用设备？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes);
        //                                        if (result != MessageBoxResult.Yes)
        //                                        {
        //                                            //用户取消操作，跳出循环
        //                                            break;
        //                                        }
        //                                        SwipeCardControl(sender, ownerPage);
        //                                    }
        //                                    break;
        //                                }
        //                            }
        //                        }
        //                        else
        //                        {
        //                            //成功读取账号，跳出循环
        //                            SetICCardData(x, data);
        //                            x.Text = data.Data.CardNo;
        //                            UserFunction.HasJZ = true;
        //                            if (!Script(ownerPage, tb.Text))
        //                            {
        //                                tb.Text = null;
        //                                return;
        //                            }
        //                            FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward);
        //                            break;
        //                        }
        //                    }
        //                    else
        //                    {
        //                        //用户取消操作，跳出循环
        //                        break;
        //                    }
        //                }
        //                else if (DeviceConfig.SwipeCardEnabled)
        //                {
        //                    var result = QuestionMessage(ownerPage, "确认采取设备读卡号？", MessageBoxButton.OKCancel, MessageBoxResult.OK);
        //                    if (result == MessageBoxResult.OK)
        //                    {
        //                        var data = DeviceHelper.ReadCardTracks(command);
        //                        if (data.HasError)
        //                        {
        //                            result = QuestionMessage(ownerPage, "磁条刷卡器调用失败，是否重新调用？\n" + data.ErrorInfo, MessageBoxButton.YesNo, MessageBoxResult.Yes);
        //                            if (result != MessageBoxResult.Yes)
        //                            {
        //                                //用户取消操作，跳出循环
        //                                break;
        //                            }
        //                        }
        //                        else
        //                        {
        //                            //成功读取账号，跳出循环
        //                            SetSwipeCardData(x, data);
        //                            x.Text = data.Data.CardNo;
        //                            UserFunction.HasJZ = true;
        //                            if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
        //                            {
        //                                //业务逻辑执行失败，需要提示，否则造成死循环
        //                                result = QuestionMessage(ownerPage, "是否重新调用设备？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes);
        //                                if (result != MessageBoxResult.Yes)
        //                                {
        //                                    //用户取消操作，跳出循环
        //                                    break;
        //                                }
        //                                SwipeCardControl(sender, ownerPage);
        //                            }
        //                            break;
        //                        }
        //                    }
        //                    else
        //                    {
        //                        //用户取消操作，跳出循环
        //                        break;
        //                    }
        //                }
        //                else
        //                {
        //                    //经改造完的手动输入流程（若手动输入，输入项为空时，回车调起外设）
        //                    SetReadOnly(tb, false);
        //                    var oldEmpty = FocusManagerExt.GetAssertNotEmpty(tb);
        //                    FocusManagerExt.SetAssertNotEmpty(tb, oldEmpty);
        //                    TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
        //                    UserFunction.HasJZ = false;
        //                    return;
        //                }
        //            }
        //        }
        //        ), System.Windows.Threading.DispatcherPriority.Input, tb);
        //    }
        //    else if (GetSwipeCardReaderTracksReader(tb) == SwipeCardReaderType.ReadCardOrInput)
        //    {
        //        //第三种方式：可输入插卡方式
        //        //正常流程
        //        tb.Dispatcher.BeginInvoke(new Action<TextBox>(x =>
        //        {
        //            while (true)
        //            {
        //                if (DeviceConfig.ICCardEnabled)
        //                {
        //                    var result = QuestionMessage(ownerPage, "是否调用设备读卡号？\n是：IC读卡器录入  否：手工录入", MessageBoxButton.YesNo, MessageBoxResult.Yes);
        //                    if (result == MessageBoxResult.Yes)
        //                    {
        //                        TextBoxHelper.SetIsRepeatInput(tb, false);
        //                        //磁条读写器操作失败情况
        //                        DeviceEntity<ICCardEntity> data = DeviceHelper.GetICCardNo();
        //                        if (data.HasError)
        //                        {
        //                            string strenable = DeviceConfig.SwipeCardEnabled == true ? "调用IC卡设备失败，准备调用磁条刷卡器！\n是：磁条刷卡器录入  否：手工录入" : "调用IC卡设备失败！\n是：IC读卡器录入  否：手工录入";
        //                            result = QuestionMessage(ownerPage, strenable, MessageBoxButton.YesNo, MessageBoxResult.Yes);
        //                            if (result == MessageBoxResult.Yes)
        //                            {
        //                                if (DeviceConfig.SwipeCardEnabled)
        //                                {
        //                                    //修改弹出窗口方式：先读取，失败后再询问是否继续
        //                                    var CardTracks = DeviceHelper.ReadCardTracks(command);
        //                                    if (CardTracks.HasError)
        //                                    {
        //                                        result = QuestionMessage(ownerPage, "磁条刷卡器调用失败，是否重新调用？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes);
        //                                        if (result != MessageBoxResult.Yes)
        //                                        {
        //                                            //用户取消操作，跳出循环
        //                                            break;
        //                                        }
        //                                    }
        //                                    else
        //                                    {
        //                                        //成功读取账号，跳出循环
        //                                        SetSwipeCardData(x, CardTracks);
        //                                        x.Text = CardTracks.Data.CardNo;
        //                                        UserFunction.HasJZ = true;
        //                                        if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
        //                                        {
        //                                            //业务逻辑执行失败，需要提示，否则造成死循环
        //                                            result = QuestionMessage(ownerPage, "是否重新调用设备？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes);
        //                                            if (result != MessageBoxResult.Yes)
        //                                            {
        //                                                //用户取消操作，跳出循环
        //                                                break;
        //                                            }
        //                                            SwipeCardControl(sender, ownerPage);
        //                                        }
        //                                        break;
        //                                    }
        //                                }
        //                            }
        //                            else
        //                            {
        //                                //经改造完的手动输入流程（若手动输入，输入项为空时，回车调起外设）
        //                                SetReadOnly(tb, false);
        //                                var oldEmpty = FocusManagerExt.GetAssertNotEmpty(tb);
        //                                FocusManagerExt.SetAssertNotEmpty(tb, oldEmpty);
        //                                TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
        //                                UserFunction.HasJZ = false;
        //                                return;
        //                            }
        //                        }
        //                        else
        //                        {
        //                            //成功读取账号，跳出循环
        //                            SetICCardData(tb, data);
        //                            tb.Text = data.Data.CardNo;
        //                            UserFunction.HasJZ = true;
        //                            if (!Script(ownerPage, tb.Text))
        //                            {
        //                                tb.Text = null;
        //                                return;
        //                            }
        //                            FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward);
        //                            return;
        //                        }
        //                    }
        //                    else
        //                    {
        //                        //经改造完的手动输入流程（若手动输入，输入项为空时，回车调起外设）
        //                        SetReadOnly(tb, false);
        //                        var oldEmpty = FocusManagerExt.GetAssertNotEmpty(tb);
        //                        FocusManagerExt.SetAssertNotEmpty(tb, oldEmpty);
        //                        TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
        //                        UserFunction.HasJZ = false;
        //                        return;
        //                    }
        //                }
        //                else if (DeviceConfig.SwipeCardEnabled)
        //                {
        //                    var result = QuestionMessage(ownerPage, "是否调用磁条刷卡器？\n是：设备读卡号  否：手动输入", MessageBoxButton.YesNo, MessageBoxResult.Yes);
        //                    if (result == MessageBoxResult.Yes)
        //                    {
        //                        TextBoxHelper.SetIsRepeatInput(tb, false);
        //                        SetReadOnly(tb, true);
        //                        //磁条读写器操作失败情况
        //                        var data = DeviceHelper.ReadCardTracks(command);
        //                        if (data.HasError)
        //                        {
        //                            result = QuestionMessage(ownerPage, "磁条刷卡器调用失败，是否重新调用？\n" + data.ErrorInfo, MessageBoxButton.YesNo, MessageBoxResult.Yes);
        //                            if (result == MessageBoxResult.No)
        //                            {
        //                                //用户取消操作，跳出循环
        //                                break;
        //                            }
        //                        }
        //                        else
        //                        {
        //                            //成功读取账号，跳出循环
        //                            SetSwipeCardData(tb, data);
        //                            tb.Text = data.Data.CardNo;
        //                            UserFunction.HasJZ = true;
        //                            if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
        //                            {
        //                                SwipeCardControl(sender, ownerPage);
        //                            }
        //                            return;
        //                        }
        //                    }
        //                    else
        //                    {
        //                        //经改造完的手动输入流程（若手动输入，输入项为空时，回车调起外设）
        //                        SetReadOnly(tb, false);
        //                        var oldEmpty = FocusManagerExt.GetAssertNotEmpty(tb);
        //                        FocusManagerExt.SetAssertNotEmpty(tb, oldEmpty);
        //                        TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
        //                        UserFunction.HasJZ = false;
        //                        return;
        //                    }
        //                }
        //                else
        //                {
        //                    //经改造完的手动输入流程（若手动输入，输入项为空时，回车调起外设）
        //                    SetReadOnly(tb, false);
        //                    var oldEmpty = FocusManagerExt.GetAssertNotEmpty(tb);
        //                    FocusManagerExt.SetAssertNotEmpty(tb, oldEmpty);
        //                    TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
        //                    UserFunction.HasJZ = false;
        //                    return;
        //                }
        //            }
        //        }), System.Windows.Threading.DispatcherPriority.Input, tb);
        //    }
        //}

        /// <summary>
        /// 新 磁条读写器控制流程 （主要修改IC卡与磁条卡同时启用的逻辑）
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="ownerPage"></param>
        private static void SwipeCardControl(object sender, IPage ownerPage)
        {
            UserFunction.HasJZ = false;
            var tb = sender as TextBox;
            if (tb == null)
                return;
            int command = GetSwipeCardCommand(tb);
            tb.ClearValue(SwipeCardDataProperty);
            tb.ClearValue(ICCardDataProperty);

            //判断父交易容器
            if (ownerPage == null) return;

            bool isRepeatInput = TextBoxHelper.GetIsRepeatInput(tb);
            if (GetSwipeCardReaderTracksReader(tb) == SwipeCardReaderType.Input)
            {
                //方式一：手工输入
                //可输入
                SetReadOnly(tb, false);
                //重复输入
                TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
                UserFunction.HasJZ = false;
            }
            if (GetSwipeCardReaderTracksReader(tb) == SwipeCardReaderType.ReadCard)
            {
                //方式二：插卡方式
                SetReadOnly(tb, true);
                TextBoxHelper.SetIsRepeatInput(tb, false);
                tb.Dispatcher.BeginInvoke(new Action<TextBox>(x =>
                {
                    while (true)
                    {
                        if (DeviceConfig.ICCardEnabled)
                        {
                            if (DeviceConfig.SwipeCardEnabled)//IC读卡器与磁条读卡器同时启用
                            {
                                int index;
                                string flag = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
                                var tradeModel = Platform.Presentation.PagesHelper.PageHelper.GetTradeModel(ownerPage as DependencyObject);
                                if (tradeModel.Code == "2000" && flag == "V0")
                                    index = 0;
                                else
                                    index = 1;
                                var result = QuestionMessageBox(ownerPage, "请选择录入方式!", new string[] { "IC读卡器", "磁条读写器" }, index);
                                if (result == "IC读卡器")
                                {
                                    DeviceEntity<ICCardEntity> data = DeviceHelper.GetICCardNo();
                                    if (data.HasError)
                                    {
                                        if (QuestionMessage(ownerPage, data.ErrorInfo + "是否重新调用？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes) != MessageBoxResult.Yes)
                                        {
                                            //用户取消操作，跳出循环
                                            break;
                                        }
                                    }
                                    else
                                    {
                                        //成功读取账号，跳出循环
                                        SetICCardData(x, data);
                                        x.Text = data.Data.CardNo;
                                        UserFunction.HasJZ = true;
                                        if (!Script(ownerPage, tb.Text))
                                        {
                                            tb.Text = null;
                                            return;
                                        }
                                        FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward);
                                        break;
                                    }
                                }
                                else if (result == "磁条读写器")
                                {
                                    var CardTracks = DeviceHelper.ReadCardTracks(command);
                                    if (CardTracks.HasError)
                                    {
                                        if (QuestionMessage(ownerPage, "磁条读写器调用失败，是否重新调用？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes) != MessageBoxResult.Yes)
                                        {
                                            //用户取消操作，跳出循环
                                            break;
                                        }
                                    }
                                    else
                                    {
                                        //成功读取账号，跳出循环
                                        SetSwipeCardData(x, CardTracks);
                                        x.Text = CardTracks.Data.CardNo;
                                        UserFunction.HasJZ = true;
                                        if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
                                        {
                                            //业务逻辑执行失败，需要提示，否则造成死循环
                                            var result1 = QuestionMessage(ownerPage, "是否重新调用设备？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes);
                                            if (result1 != MessageBoxResult.Yes)
                                            {
                                                //用户取消操作，跳出循环
                                                break;
                                            }
                                            SwipeCardControl(sender, ownerPage);
                                        }
                                        break;
                                    }
                                }
                            }
                            else  //只启用IC读卡器
                            {
                                if (QuestionMessage(ownerPage, "是否调用设备读卡号？", MessageBoxButton.YesNo, MessageBoxResult.Yes) == MessageBoxResult.Yes)
                                {
                                    DeviceEntity<ICCardEntity> data = DeviceHelper.GetICCardNo();
                                    if (data.HasError)
                                    {
                                        if (QuestionMessage(ownerPage, data.ErrorInfo + "是否重新调用？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes) != MessageBoxResult.Yes)
                                        {
                                            //用户取消操作，跳出循环
                                            break;
                                        }
                                    }
                                    else
                                    {
                                        //成功读取账号，跳出循环
                                        SetICCardData(x, data);
                                        x.Text = data.Data.CardNo;
                                        UserFunction.HasJZ = true;
                                        if (!Script(ownerPage, tb.Text))
                                        {
                                            tb.Text = null;
                                            return;
                                        }
                                        FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward);
                                        break;
                                    }
                                }
                                else
                                {
                                    QuestionMessage(ownerPage, "必须调用设备，请按F10重新调用！\n", MessageBoxButton.OK, MessageBoxResult.None);
                                    break;
                                }
                            }
                        }
                        else if (DeviceConfig.SwipeCardEnabled)//只启用磁条读卡器
                        {
                            if (QuestionMessage(ownerPage, "是否调用磁条刷卡器？", MessageBoxButton.YesNo, MessageBoxResult.Yes) == MessageBoxResult.Yes)
                            {
                                var CardTracks = DeviceHelper.ReadCardTracks(command);
                                if (CardTracks.HasError)
                                {
                                    if (QuestionMessage(ownerPage, "磁条读写器调用失败，是否重新调用？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes) != MessageBoxResult.Yes)
                                    {
                                        //用户取消操作，跳出循环
                                        break;
                                    }
                                }
                                else
                                {
                                    //成功读取账号，跳出循环
                                    SetSwipeCardData(x, CardTracks);
                                    x.Text = CardTracks.Data.CardNo;
                                    UserFunction.HasJZ = true;
                                    if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
                                    {
                                        //业务逻辑执行失败，需要提示，否则造成死循环
                                        var result1 = QuestionMessage(ownerPage, "是否重新调用设备？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes);
                                        if (result1 != MessageBoxResult.Yes)
                                        {
                                            //用户取消操作，跳出循环
                                            break;
                                        }
                                        SwipeCardControl(sender, ownerPage);
                                    }
                                    break;
                                }
                            }
                            else
                            {
                                QuestionMessage(ownerPage, "必须调用设备，请按F10重新调用！\n", MessageBoxButton.OK, MessageBoxResult.None);
                                break;
                            }
                        }
                        else
                        {
                            //经改造完的手动输入流程（若手动输入，输入项为空时，回车调起外设）
                            SetReadOnly(tb, false);
                            var oldEmpty = FocusManagerExt.GetAssertNotEmpty(tb);
                            FocusManagerExt.SetAssertNotEmpty(tb, oldEmpty);
                            TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
                            UserFunction.HasJZ = false;
                            return;
                        }
                    }
                }), System.Windows.Threading.DispatcherPriority.Input, tb);
            }
            else if (GetSwipeCardReaderTracksReader(tb) == SwipeCardReaderType.ReadCardOrInput)
            {
                //方式三：可输入插卡方式
                //正常流程
                tb.Dispatcher.BeginInvoke(new Action<TextBox>(x =>
                {
                    while (true)
                    {
                        if (DeviceConfig.ICCardEnabled)
                        {
                            if (DeviceConfig.SwipeCardEnabled)//IC读卡器与磁条读卡器同时启用
                            {
                                int index;
                                string flag = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
                                var tradeModel = Platform.Presentation.PagesHelper.PageHelper.GetTradeModel(ownerPage as DependencyObject);
                                if (tradeModel.Code == "2000" && flag == "V0")
                                    index = 0;
                                else if (flag == "V21" && (tradeModel.Code == "2350" || tradeModel.Code == "2351"))
                                    index = 2;
                                else if (flag == "V21" && (tradeModel.Code != "2350" && tradeModel.Code != "2351"))
                                    index = 0;
                                else
                                    index = 1;
                                var result = QuestionMessageBox(ownerPage, "请选择录入方式!", new string[] { "IC读卡器", "磁条读写器", "手工录入" }, index);
                                if (result == "IC读卡器")
                                {

                                    DeviceEntity<ICCardEntity> data = DeviceHelper.GetICCardNo();
                                    if (data.HasError)
                                    {
                                        if (QuestionMessage(ownerPage, data.ErrorInfo + "是否重新调用？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes) != MessageBoxResult.Yes)
                                        {
                                            //用户取消操作，跳出循环
                                            break;
                                        }
                                    }
                                    else
                                    {
                                        //成功读取账号，跳出循环
                                        SetICCardData(x, data);
                                        x.Text = data.Data.CardNo;
                                        UserFunction.HasJZ = true;
                                        if (!Script(ownerPage, tb.Text))
                                        {
                                            tb.Text = null;
                                            return;
                                        }
                                        FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward);
                                        break;
                                    }
                                }
                                else if (result == "磁条读写器")
                                {
                                    var CardTracks = DeviceHelper.ReadCardTracks(command);
                                    if (CardTracks.HasError)
                                    {
                                        if (QuestionMessage(ownerPage, "磁条读写器调用失败，是否重新调用？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes) != MessageBoxResult.Yes)
                                        {
                                            //用户取消操作，跳出循环
                                            break;
                                        }
                                    }
                                    else
                                    {
                                        //成功读取账号，跳出循环
                                        SetSwipeCardData(x, CardTracks);
                                        x.Text = CardTracks.Data.CardNo;
                                        UserFunction.HasJZ = true;
                                        if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
                                        {
                                            //业务逻辑执行失败，需要提示，否则造成死循环
                                            var result1 = QuestionMessage(ownerPage, "是否重新调用设备？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes);
                                            if (result1 != MessageBoxResult.Yes)
                                            {
                                                //用户取消操作，跳出循环
                                                break;
                                            }
                                            SwipeCardControl(sender, ownerPage);
                                        }
                                        break;
                                    }
                                }
                                else if (result == "手工录入")
                                {
                                    //经改造完的手动输入流程（若手动输入，输入项为空时，回车调起外设）
                                    SetReadOnly(tb, false);
                                    var oldEmpty = FocusManagerExt.GetAssertNotEmpty(tb);
                                    FocusManagerExt.SetAssertNotEmpty(tb, oldEmpty);
                                    TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
                                    UserFunction.HasJZ = false;
                                    return;
                                }
                            }
                            else  //只启用IC读卡器
                            {
                                if (QuestionMessage(ownerPage, "是否调用设备读卡号？\n是：IC读卡器录入  否：手工录入", MessageBoxButton.YesNo, MessageBoxResult.Yes) == MessageBoxResult.Yes)
                                {
                                    DeviceEntity<ICCardEntity> data = DeviceHelper.GetICCardNo();
                                    if (data.HasError)
                                    {
                                        if (QuestionMessage(ownerPage, data.ErrorInfo + "是否重新调用？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes) != MessageBoxResult.Yes)
                                        {
                                            //用户取消操作，跳出循环
                                            break;
                                        }
                                    }
                                    else
                                    {
                                        //成功读取账号，跳出循环
                                        SetICCardData(x, data);
                                        x.Text = data.Data.CardNo;
                                        UserFunction.HasJZ = true;
                                        if (!Script(ownerPage, tb.Text))
                                        {
                                            tb.Text = null;
                                            return;
                                        }
                                        FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward);
                                        break;
                                    }
                                }
                                else
                                {
                                    //经改造完的手动输入流程（若手动输入，输入项为空时，回车调起外设）
                                    SetReadOnly(tb, false);
                                    var oldEmpty = FocusManagerExt.GetAssertNotEmpty(tb);
                                    FocusManagerExt.SetAssertNotEmpty(tb, oldEmpty);
                                    TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
                                    UserFunction.HasJZ = false;
                                    return;
                                }
                            }
                        }
                        else if (DeviceConfig.SwipeCardEnabled)//只启用磁条读卡器
                        {
                            if (QuestionMessage(ownerPage, "是否调用磁条刷卡器？\n是：磁条刷卡器录入  否：手工录入", MessageBoxButton.YesNo, MessageBoxResult.Yes) == MessageBoxResult.Yes)
                            {
                                var CardTracks = DeviceHelper.ReadCardTracks(command);
                                if (CardTracks.HasError)
                                {
                                    if (QuestionMessage(ownerPage, "磁条读写器调用失败，是否重新调用？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes) != MessageBoxResult.Yes)
                                    {
                                        //用户取消操作，跳出循环
                                        break;
                                    }
                                }
                                else
                                {
                                    //成功读取账号，跳出循环
                                    SetSwipeCardData(x, CardTracks);
                                    x.Text = CardTracks.Data.CardNo;
                                    UserFunction.HasJZ = true;
                                    if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
                                    {
                                        //业务逻辑执行失败，需要提示，否则造成死循环
                                        var result1 = QuestionMessage(ownerPage, "是否重新调用设备？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes);
                                        if (result1 != MessageBoxResult.Yes)
                                        {
                                            //用户取消操作，跳出循环
                                            break;
                                        }
                                        SwipeCardControl(sender, ownerPage);
                                    }
                                    break;
                                }
                            }
                            else
                            {
                                //经改造完的手动输入流程（若手动输入，输入项为空时，回车调起外设）
                                SetReadOnly(tb, false);
                                var oldEmpty = FocusManagerExt.GetAssertNotEmpty(tb);
                                FocusManagerExt.SetAssertNotEmpty(tb, oldEmpty);
                                TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
                                UserFunction.HasJZ = false;
                                return;
                            }
                        }
                        else
                        {
                            //经改造完的手动输入流程（若手动输入，输入项为空时，回车调起外设）
                            SetReadOnly(tb, false);
                            var oldEmpty = FocusManagerExt.GetAssertNotEmpty(tb);
                            FocusManagerExt.SetAssertNotEmpty(tb, oldEmpty);
                            TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
                            UserFunction.HasJZ = false;
                            return;
                        }
                    }
                }), System.Windows.Threading.DispatcherPriority.Input, tb);
            }
        }

        private static bool Script(IPage page, string cardno)
        {
            #region 原代码注掉 没什么用，还影响读IC效率
            //try
            //{
            //    var Dialog = new PageMessageBox();
            //    Communication.Message message = new Communication.Message();
            //    SystemFunction.InitFd(page, message);
            //    SystemFunction.CommSend(page, message);
            //    DeviceHelper.InitICCard("1", message);
            //    //DeviceHelper.IcCardPowerDown();
            //    //if (message.Fd104_1.Length == 0)
            //    //{
            //    //    QuestionMessage(page, "读卡错误，请重新操作", MessageBoxButton.OK, MessageBoxResult.OK);
            //    //    UserFunction.HasJZ = false;
            //    //    return false;
            //    //}
            //    //message.Fd16 = "K100";
            //    message.Fd30 = cardno;
            //    message.Fd104_2 = message.Fd23.PadLeft(3, '0');
            //    //UserFunction.msg104_1 = message.Fd104_1;
            //    //UserFunction.msg104_2 = message.Fd104_2;
            //    //银联柜面通他行不校验账号
            //    //注释掉核心验证20150303 lizheng 平台要求
            //    //if (UserFunction.CheckCardBin(page, cardno))
            //    //{
            //    //    SystemFunction.CallServer(page, message);
            //    //    if (message.Fd12 != "0000")
            //    //    {
            //    //        QuestionMessage(page, "核心K100交易处理失败!", MessageBoxButton.OK, MessageBoxResult.OK);
            //    //        UserFunction.HasJZ = false;
            //    //        return false;
            //    //    }
            //    //}
            //    return true;

            //}
            //catch (Exception ex)
            //{
            //    //DeviceHelper.IcCardPowerDown();
            //    return false;
            //}
            #endregion
            return true;
        }

        /// <summary>
        /// 应焦作客户增加的特殊处理，在可刷可输情况下文本框为空按回车再调用刷卡
        /// </summary>
        /// <param name="asender"></param>
        /// <param name="routedEventArgs"></param>
        private static void SwipeCardPreLostFocus(object asender, RoutedEventArgs routedEventArgs)
        {
            var aTextBox = asender as TextBox;
            if (aTextBox == null)
                return;

            #region 查找控件所属IPage对象

            IPage ownerPage = null;
            var parent = asender as DependencyObject;
            while (parent != null)
            {
                if (parent is IPage)
                {
                    break;
                }
                parent = VisualTreeHelper.GetParent(parent);
            }
            if (parent != null)
            {
                ownerPage = parent as IPage;
            }

            if (ownerPage == null)
                return;

            #endregion 查找控件所属IPage对象

            FocusManager.RemovePreviewLostFocusHandler(aTextBox, SwipeCardPreLostFocus);

            int command = GetSwipeCardCommand(aTextBox);
            if (string.IsNullOrEmpty(aTextBox.Text))
            {
                aTextBox.Dispatcher.BeginInvoke(new Action<TextBox>(x =>
                {
                    while (true)
                    {
                        var deviceQuestion = new PageMessageBox();
                        var result = deviceQuestion.Show(ownerPage, "是：设备读卡号 否：手动输入？",
                                          "磁条读写器操作提示", MessageBoxButton.YesNo, MessageBoxImage.Question, MessageBoxResult.Yes);
                        if (result == MessageBoxResult.Yes)
                        {
                            TextBoxHelper.SetIsRepeatInput(aTextBox, false);
                            //磁条读写器操作失败情况
                            var data = DeviceHelper.ReadCardTracks(command);
                            if (data.HasError)
                            {
                                var deviceDialog = new PageMessageBox();
                                deviceDialog.Show(ownerPage, data.ErrorInfo,
                                                  "磁条读写器操作提示", MessageBoxButton.OK, MessageBoxImage.Information, MessageBoxResult.OK);
                            }
                            else
                            {
                                //成功读取账号，跳出循环
                                aTextBox.Text = data.Data.CardNo;
                                SetSwipeCardData(aTextBox, data);
                                break;
                            }
                        }
                        else
                        {
                            //手工输入，跳出循环
                            TextBoxHelper.SetIsRepeatInput(aTextBox, true);
                            SetReadOnly(aTextBox, false);
                            break;
                        }
                    }
                }), System.Windows.Threading.DispatcherPriority.Input, aTextBox);
            }
        }

        /// <summary>
        /// 密码输入方式控制流程
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="ownerPage"></param>
        private static void PassWordControl(object sender, IPage ownerPage)
        {
            var tb = sender as PasswordBox;
            if (tb == null)
                return;
            tb.ClearValue(PwdDataProperty);

            //没启用设备
            if (DeviceConfig.InDeviceDebugMode && !DeviceConfig.PassWordKBEnabled)
                return;

            if (ownerPage == null)
                return;
            //修正：使用原“IsRepeatInput”属性，不要在这里修改
            bool isRepeatInput = PasswordBoxHelper.GetIsRepeatInput(tb);
            if (GetPassWordKeyBoardReader(tb) == PassWordKeyBoardReaderType.Input)
            {
                //方式一：手工输入
                TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
                SetReadOnly(tb, false);
            }
            else if (GetPassWordKeyBoardReader(tb) == PassWordKeyBoardReaderType.ReadPWBoardOne
                   || GetPassWordKeyBoardReader(tb) == PassWordKeyBoardReaderType.ReadPWBoardOneOrInput)
            {
                //方式二：单次读密码器方式
                //TextBoxHelper.SetIsRepeatInput(tb, false);
                PasswordBoxHelper.SetIsRepeatInput(tb, false);
                SetReadOnly(tb, true);

                tb.Dispatcher.BeginInvoke(new Action<PasswordBox>(x =>
                {
                    //先调用一次设备，失败后才提示
                    while (true)
                    {
                        var pwdData = DeviceHelper.ReadPassWord();
                        if (pwdData.HasError)
                        {
                            var result = QuestionMessage(ownerPage, "密码键盘调用失败，是否重新调用？\n" + pwdData.ErrorInfo, MessageBoxButton.YesNo, MessageBoxResult.Yes);
                            if (result == MessageBoxResult.No)
                            {
                                //用户取消操作，跳出循环
                                break;
                            }
                        }
                        else
                        {
                            //成功读取密码，退出循环
                            PasswordBoxHelper.SetIsUpdating(x, true);
                            x.Password = pwdData.Data.PwdData;
                            PasswordBoxHelper.SetIsUpdating(x, false);
                            SetPwdData(tb, pwdData);
                            if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
                            {
                                //业务逻辑执行失败，需要提升一次
                                var result = QuestionMessage(ownerPage, "是否重新调用设备？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes);
                                if (result == MessageBoxResult.No)
                                {
                                    //用户取消操作，跳出循环
                                    break;
                                }
                                PassWordControl(sender, ownerPage);
                            }
                            break;
                        }
                    }
                }), System.Windows.Threading.DispatcherPriority.Input, tb);
            }
            else if (GetPassWordKeyBoardReader(tb) == PassWordKeyBoardReaderType.ReadPWBoardTwo
                || GetPassWordKeyBoardReader(tb) == PassWordKeyBoardReaderType.ReadPWBoardTwoOrInput)
            {
                //方式三：两次读密码器方式
                //TextBoxHelper.SetIsRepeatInput(tb, false);
                PasswordBoxHelper.SetIsRepeatInput(tb, false);
                SetReadOnly(tb, true);
                tb.Dispatcher.BeginInvoke(new Action<PasswordBox>(x =>
                {
                    while (true)
                    {
                        var pwdData = DeviceHelper.ReadPassWord2();
                        if (pwdData.HasError)
                        {
                            var result = QuestionMessage(ownerPage, "密码键盘调用失败，是否重新调用？\n" + pwdData.ErrorInfo, MessageBoxButton.YesNo, MessageBoxResult.Yes);
                            if (result == MessageBoxResult.No)
                            {
                                //用户取消操作，跳出循环
                                break;
                            }
                        }
                        else
                        {
                            //成功读取密码，退出循环
                            PasswordBoxHelper.SetIsUpdating(x, true);
                            x.Password = pwdData.Data.PwdData;
                            PasswordBoxHelper.SetIsUpdating(x, false);
                            SetPwdData(tb, pwdData);
                            if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
                            {
                                //业务逻辑失败，需要重新提示一次
                                var result = QuestionMessage(ownerPage, "是否重新调用设备？\n", MessageBoxButton.YesNo, MessageBoxResult.Yes);
                                if (result == MessageBoxResult.No)
                                {
                                    //用户取消操作，跳出循环
                                    break;
                                }
                                PassWordControl(sender, ownerPage);
                            }
                            break;
                        }
                    }
                }), System.Windows.Threading.DispatcherPriority.Input, tb);
            }
            //else if (GetPassWordKeyBoardReader(tb) == PassWordKeyBoardReaderType.ReadPWBoardOneOrInput)
            //{
            //    //方式四：单次可输入读密码器方式
            //    tb.Dispatcher.BeginInvoke(new Action<PasswordBox>(x =>
            //    {
            //        while (true)
            //        {
            //            var deviceQuestion = new PageMessageBox();
            //            var result = deviceQuestion.Show(ownerPage, "是：单次读密码 否：手动输入密码？",
            //                              "密码键盘操作提示", MessageBoxButton.YesNo, MessageBoxImage.Question, MessageBoxResult.No);
            //            if (result == MessageBoxResult.Yes)
            //            {
            //                TextBoxHelper.SetIsRepeatInput(tb, false);
            //                SetReadOnly(tb, true);
            //                var pwdData = DeviceHelper.ReadPassWord();
            //                SetPwdData(tb, pwdData);
            //                if (pwdData.HasError)
            //                {
            //                    var deviceDialog = new PageMessageBox();
            //                    deviceDialog.Show(ownerPage, pwdData.ErrorInfo,
            //                                      "密码键盘操作提示", MessageBoxButton.OK, MessageBoxImage.Information, MessageBoxResult.OK);
            //                }
            //                else
            //                {
            //                    //成功读取密码，跳出循环
            //                    x.Password = pwdData.Data.PwdData;
            //                    if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
            //                    {
            //                        PassWordControl(sender, ownerPage);
            //                    }
            //                    break;
            //                }
            //            }
            //            else
            //            {
            //                //手工输入，退出循环
            //                SetReadOnly(tb, false);
            //                TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
            //                break;
            //            }
            //        }
            //    }), System.Windows.Threading.DispatcherPriority.Input, tb);
            //}
            //else if (GetPassWordKeyBoardReader(tb) == PassWordKeyBoardReaderType.ReadPWBoardTwoOrInput)
            //{
            //    tb.Dispatcher.BeginInvoke(new Action<PasswordBox>(x =>
            //    {
            //        while (true)
            //        {
            //            var deviceQuestion = new PageMessageBox();
            //            var result = deviceQuestion.Show(ownerPage, "是：两次读密码 否：手动输入密码？",
            //                              "密码键盘操作提示", MessageBoxButton.YesNo, MessageBoxImage.Question, MessageBoxResult.No);
            //            if (result == MessageBoxResult.Yes)
            //            {
            //                TextBoxHelper.SetIsRepeatInput(tb, false);
            //                SetReadOnly(tb, true);
            //                var pwdData = DeviceHelper.ReadPassWord2();
            //                SetPwdData(tb, pwdData);
            //                if (pwdData.HasError)
            //                {
            //                    var deviceDialog = new PageMessageBox();
            //                    deviceDialog.Show(ownerPage, pwdData.ErrorInfo,
            //                                      "密码键盘操作提示", MessageBoxButton.OK, MessageBoxImage.Information, MessageBoxResult.OK);
            //                }
            //                else
            //                {
            //                    //成功读取密码，跳出循环
            //                    x.Password = pwdData.Data.PwdData;
            //                    if (!FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward))
            //                    {
            //                        PassWordControl(sender, ownerPage);
            //                    }
            //                    break;
            //                }
            //            }
            //            else
            //            {
            //                //手工输入退出循环
            //                SetReadOnly(tb, false);
            //                TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
            //                break;
            //            }
            //        }
            //    }), System.Windows.Threading.DispatcherPriority.Input, tb);
            //}
        }

        /// <summary>
        /// 二代身份证输入方式控制流程
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="ownerPage"></param>
        private static void IdentityCardControl(object sender, IPage ownerPage)
        {
            //string flag = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            var tb = sender as TextBox;
            if (tb == null)
                return;
            tb.ClearValue(IdentityCardDataProperty);

            //没启用设备
            if (DeviceConfig.InDeviceDebugMode && !DeviceConfig.IdentityCardEnabled)
                return;

            if (ownerPage == null)
                return;
            //智能感知其他证件类型
            Intellisense(tb);
            //获取身份证证件类型
            var cardType = GetIdentityCardType(tb);
            //修正：使用原“IsRepeatInput”属性，不要在这里修改
            bool isRepeatInput = TextBoxHelper.GetIsRepeatInput(tb);
            if (Platform.Presentation.FocusManagers.FocusManagerExt.GetAssertNotEmpty(tb))
            {
                if (GetIdentityCardReader(tb) == IdentityCardReaderType.Input)
                {
                    //方式一：手工输入
                    TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
                    SetReadOnly(tb, false);
                }
                else if (GetIdentityCardReader(tb) == IdentityCardReaderType.ReadCard)
                {
                    //方式二： 读二代身份证
                    TextBoxHelper.SetIsRepeatInput(tb, false);
                    SetReadOnly(tb, true);
                    tb.Dispatcher.BeginInvoke(new Action<TextBox>(x =>
                    {
                        while (true)
                        {
                            var result = QuestionMessage(ownerPage, "采用证件读取设备采集数据，请放置好证件！", MessageBoxButton.OK, MessageBoxResult.OK);
                            if (result == MessageBoxResult.OK)
                            {
                                var data = DeviceHelper.ReadIdentityCard(cardType);
                                SetIdentityCardData(tb, data);
                                if (data.HasError)
                                {
                                    result = QuestionMessage(ownerPage, "证件读取设备调用失败，是否重新调用？\n" + data.ErrorInfo, MessageBoxButton.YesNo, MessageBoxResult.Yes);
                                    if (result == MessageBoxResult.No)
                                    {
                                        //用户取消操作，跳出循环
                                        break;
                                    }
                                }
                                else
                                {
                                    //成功读取后跳出循环
                                    tb.Text = data.Data.IdNumber;
                                    FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward);
                                    break;
                                }
                            }
                        }
                    }), System.Windows.Threading.DispatcherPriority.Input, tb);
                }
                else if (GetIdentityCardReader(tb) == IdentityCardReaderType.ReadCardOrInput)
                {
                    //方式三：可输入读二代身份证
                    tb.Dispatcher.BeginInvoke(new Action<TextBox>(x =>
                    {
                        while (true)
                        {
                            var result = QuestionMessage(ownerPage, "是否采用证件读取设备采集数据？\n是：使用设备   否：手输", MessageBoxButton.YesNo, MessageBoxResult.Yes);
                            if (result == MessageBoxResult.Yes)
                            {
                                TextBoxHelper.SetIsRepeatInput(tb, false);
                                SetReadOnly(tb, true);
                                var data = DeviceHelper.ReadIdentityCard(cardType);
                                SetIdentityCardData(tb, data);
                                if (data.HasError)
                                {
                                    result = QuestionMessage(ownerPage, "证件读取设备调用失败，是否重新调用？\n" + data.ErrorInfo, MessageBoxButton.YesNo, MessageBoxResult.Yes);
                                    if (result == MessageBoxResult.No)
                                    {
                                        //用户取消操作，跳出循环
                                        break;
                                    }
                                }
                                else
                                {
                                    //成功读取后跳出循环
                                    tb.Text = data.Data.IdNumber;
                                    FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward);
                                    break;
                                }
                            }
                            else
                            {
                                //手工输入退出循环
                                SetReadOnly(tb, false);
                                TextBoxHelper.SetIsRepeatInput(tb, isRepeatInput);
                                break;
                            }
                        }
                    }), System.Windows.Threading.DispatcherPriority.Input, tb);
                }
            }
        }

        /// <summary>
        /// 分析外国人永久居留身份证及港澳台通行证的情况
        /// </summary>
        /// <param name="tb"></param>
        private static void Intellisense(TextBox tb)
        {
            try
            {
                //未设置的情况不处理
                if (GetIdentityCardReader(tb) == null) return;
                //默认重置成身份证
                SetIdentityCardType(tb, IdentityCardType.IdentityCard);
                var panel = tb.Parent as Panel;
                if (panel == null) return;
                var index = panel.Children.IndexOf(tb);
                if (index - 2 > 0)
                {
                    var cbb = panel.Children[index - 2] as ComboBox;
                    if (cbb == null) return;
                    if (cbb.Text.Contains("外国人居留证") || cbb.Text.Contains("外国人永久居留身份证"))
                    {
                        TextBoxHelper.SetInputType(tb, InputType.String);
                        SetIdentityCardType(tb, IdentityCardType.Foreigner);
                        SetIdentityCardReader(tb, IdentityCardReaderType.ReadCardOrInput);
                    }
                    else if (cbb.Text.Contains("港澳居民来往内地通行证")
                        || cbb.Text.Contains("港澳台居民居住证")
                        || cbb.Text.Contains("台湾同胞来往居民通行证")
                        || cbb.Text.Contains("台湾居民来往大陆通行证"))
                    {
                        TextBoxHelper.SetInputType(tb, InputType.String);
                        SetIdentityCardType(tb, IdentityCardType.HMT);
                        SetIdentityCardReader(tb, IdentityCardReaderType.ReadCardOrInput);
                    }
                }
            }
            catch
            { }
        }

        /// <summary>
        /// 设置某个textbox或者passwordbox为只读
        /// </summary>
        /// <param name="ui"></param>
        /// <param name="isReadonly"></param>
        private static void SetReadOnly(DependencyObject ui, bool isReadonly)
        {
            if (ui is TextBox)
                ((TextBox)ui).IsReadOnly = isReadonly;
            else if (ui is PasswordBox)
                PasswordBoxHelper.SetIsReadOnly(ui, isReadonly);
        }

        /// <summary>
        ///   IC卡输入方式控制流程
        /// </summary>
        /// <param name="sender"></param>
        private static void ICCardControl(object sender, IPage ownerPage)
        {
            UserFunction.HasJZ = false;
            var tb = sender as TextBox;
            if (tb == null)
                return;
            tb.ClearValue(ICCardDataProperty);

            //没启用设备
            if (DeviceConfig.InDeviceDebugMode && !DeviceConfig.ICCardEnabled)
                return;

            //判断父交易容器
            if (ownerPage == null)
                return;

            if (GetICCardReader(tb) == ICCardReaderType.Input)
            {
                //方式一：手工输入
                //可输入
                SetReadOnly(tb, false);
                //重复输入
                TextBoxHelper.SetIsRepeatInput(tb, false);
            }
            else if (GetICCardReader(tb) == ICCardReaderType.ReadCard)
            {
                //方式二：插卡方式
                SetReadOnly(tb, true);
                TextBoxHelper.SetIsRepeatInput(tb, false);

                tb.Dispatcher.BeginInvoke(new Action<TextBox>(x =>
                {
                    while (true)
                    {
                        var result = QuestionMessage(ownerPage, "确认采取设备读卡号？", MessageBoxButton.OKCancel, MessageBoxResult.OK);
                        if (result == MessageBoxResult.OK)
                        {
                            DeviceEntity<ICCardEntity> data = DeviceHelper.GetICCardNo();
                            if (data.HasError)
                            {
                                QuestionMessage(ownerPage, data.ErrorInfo + "是否重新调用？\n", MessageBoxButton.OK, MessageBoxResult.OK);
                            }
                            else
                            {
                                //成功读取账号，跳出循环
                                SetICCardData(x, data);
                                x.Text = data.Data.CardNo;
                                FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward);
                                break;
                            }
                        }
                        else
                        {
                            //用户取消操作，跳出循环
                            break;
                        }
                    }
                }
                                              ), DispatcherPriority.Input, tb);
            }
            else if (GetICCardReader(tb) == ICCardReaderType.ReadCardOrInput)
            {
                //第三种方式：可输入插卡方式
                //正常流程
                tb.Dispatcher.BeginInvoke(new Action<TextBox>(x =>
                {
                    while (true)
                    {
                        var result = QuestionMessage(ownerPage, "是否调用设备读卡号？\n是：IC读卡器录入  否：手工录入", MessageBoxButton.YesNo, MessageBoxResult.Yes);
                        if (result == MessageBoxResult.Yes)
                        {
                            TextBoxHelper.SetIsRepeatInput(tb, false);
                            //磁条读写器操作失败情况
                            DeviceEntity<ICCardEntity> data = DeviceHelper.GetICCardNo();
                            SetICCardData(tb, data);
                            if (data.HasError)
                            {
                                QuestionMessage(ownerPage, data.ErrorInfo + "是否重新调用？\n", MessageBoxButton.OK, MessageBoxResult.OK);
                            }
                            else
                            {
                                //成功读取账号，跳出循环
                                tb.Text = data.Data.CardNo;
                                FocusManager.GetManagerInstance(tb).MoveFocus(FocusNavigation.Forward);
                                return;
                            }
                        }
                        else
                        {
                            //经改造完的手动输入流程（若手动输入，输入项为空时，回车调起外设）
                            SetReadOnly(tb, false);
                            var oldEmpty = FocusManagerExt.GetAssertNotEmpty(tb);
                            FocusManagerExt.SetAssertNotEmpty(tb, oldEmpty);
                            TextBoxHelper.SetIsRepeatInput(tb, false);
                            return;
                        }
                    }
                }), DispatcherPriority.Input, tb);
            }
        }
        #endregion 业务流程

        #region Method
        static MessageBoxResult QuestionMessage(IPage page, string message, MessageBoxButton buttons, MessageBoxResult result)
        {
            string[] btns = null;
            switch (buttons)
            {
                case MessageBoxButton.OK:
                    btns = new string[] { "确认" };
                    break;
                case MessageBoxButton.OKCancel:
                    btns = new string[] { "确认", "取消" };
                    break;
                case MessageBoxButton.YesNo:
                    btns = new string[] { "是", "否" };
                    break;
                case MessageBoxButton.YesNoCancel:
                    btns = new string[] { "是", "否", "取消" };
                    break;
                default:
                    break;
            }
            var ret = page.ShowPageMessageBoxMetro(message, "询问信息", null, dict_MessageBoxResult[result], 3, true, true, false, btns);
            return dict_MessageBoxResult.FirstOrDefault(x => x.Value == ret).Key;

        }
        static Dictionary<MessageBoxResult, string> dict_MessageBoxResult = new Dictionary<MessageBoxResult, string>
        {
            {MessageBoxResult.Cancel,"取消"},
            {MessageBoxResult.No,"否"},
            {MessageBoxResult.None,"无"},
            {MessageBoxResult.OK,"确认"},
            {MessageBoxResult.Yes,"是"} 
        };

        static string QuestionMessageBox(IPage page, string message, string[] results, int defaultIndex, string caption = "询问信息")
        {
            var defaultValue = results.ToList()[results.Length > defaultIndex ? defaultIndex : 0];
            var content = new ContentPresenter { Content = message };
            string result = null;
            //content.Loaded += (sender, e) =>
            //{
            //    //捕获最外层的按键事件
            //    var win = Platform.Common.BasicFunctions.DependencyObjectExtensions.FindVisualTreeAncestor(content, x => x is Window) as Window;
            //    if (win == null) return;
            //    win.PreviewKeyDown += (ss, ee) =>
            //    {
            //        if (results.ContainsKey(ee.Key))
            //        {
            //            result = results[ee.Key];
            //            win.Close();
            //        }
            //    };
            //};
            var ret = page.ShowPageMessageBoxMetro(content, caption, null, defaultValue, 3, true, true, false, results);
            return result ?? ret;
        }
        #endregion
    }

    #region 外设调用方式枚举

    /// <summary>
    /// 磁条数据获取方式
    /// </summary>
    public enum SwipeCardReaderType
    {
        /// <summary>
        /// 手工输入
        /// </summary>
        Input = 0,
        /// <summary>
        /// 刷卡方式
        /// </summary>
        ReadCard = 1,
        /// <summary>
        /// 可输入刷卡方式
        /// </summary>
        ReadCardOrInput = 2,
    }

    /// <summary>
    /// 密码数据获取方式
    /// </summary>
    public enum PassWordKeyBoardReaderType
    {
        /// <summary>
        /// 手工输入
        /// </summary>
        Input = 0,
        /// <summary>
        /// 单次读密码器方式
        /// </summary>
        ReadPWBoardOne = 1,
        /// <summary>
        /// 两次读密码器方式
        /// </summary>
        ReadPWBoardTwo = 2,
        /// <summary>
        /// 单次可输入读密码器方式
        /// </summary>
        ReadPWBoardOneOrInput = 3,
        /// <summary>
        /// 两次可输入读密码器方式
        /// </summary>
        ReadPWBoardTwoOrInput = 4,
    }

    /// <summary>
    /// 身份证号获取方式
    /// </summary>
    public enum IdentityCardReaderType
    {
        /// <summary>
        /// 手工输入
        /// </summary>
        Input = 0,
        /// <summary>
        /// 读二代身份证
        /// </summary>
        ReadCard = 1,
        /// <summary>
        /// 可输入读二代身份证
        /// </summary>
        ReadCardOrInput = 2,
    }

    /// <summary>
    ///  IC卡数据获取方式
    /// </summary>
    public enum ICCardReaderType
    {
        /// <summary>
        ///   手工输入
        /// </summary>
        Input = 0,

        /// <summary>
        ///   插卡方式
        /// </summary>
        ReadCard = 1,

        /// <summary>
        ///   可输入插卡方式
        /// </summary>
        ReadCardOrInput = 2,
    }

    #endregion 外设调用方式枚举
}