﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ProcessCommunication;
using TellerSystem.Communication;
using System.Windows;
using Platform.Common.RunningParameters;
using System.Windows.Input;
using Platform.Common.BasicFunctions;
using System.Diagnostics;
using System.IO;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class ExamSystem
    {
        #region Ctor
        private ExamSystem()
        {
            InExamMode = string.Compare(ConfigManager.GetInstance().GetConfigValue("InExamMode", ConfigType.System), "true", true) == 0;
            ExamEntity = new ExamObject() { IsExamModel = InExamMode };
        }

        private static ExamSystem _instance;
        /// <summary>
        /// 单例模式
        /// </summary>
        public static ExamSystem Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new ExamSystem();
                return _instance;
            }
        }
        #endregion

        #region Property
        /// <summary>
        /// 是否正处理考试模式
        /// </summary>
        public bool InExamMode { get; private set; }

        /// <summary>
        /// 考试模式对象
        /// </summary>
        public ExamObject ExamEntity { get; internal set; }

        /// <summary>
        /// 进程通讯对象
        /// </summary>
        public ProcessCommunication.MessageObject CommunicationObject { get; internal set; }
        #endregion

        #region Command
        private BaseCommand _submitExamCommand;

        /// <summary>
        /// 提交考试系统命令
        /// </summary>
        public BaseCommand SubmitExamCommand
        {
            get
            {
                return _submitExamCommand ?? (_submitExamCommand = new BaseCommand(obj =>
                {
                    if (!InExamMode) return;
                    //若考试系统未启动，则先启动它
                    if (Process.GetProcessesByName("ExamSys.Client").Length == 0)
                    {
                        try
                        {
                            var dir = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "exam");
                            //检查更新脚本
                            FileInfo batFile = new FileInfo(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "exam", "exam.bat"));
                            //写脚本文件
                            StringBuilder sb = new StringBuilder();
                            sb.AppendLine("cd exam");
                            sb.AppendLine("start ExamSys.Client.exe");
                            File.WriteAllText(batFile.FullName, sb.ToString(), Encoding.GetEncoding("gb2312"));

                            System.Diagnostics.ProcessStartInfo process = new System.Diagnostics.ProcessStartInfo(batFile.FullName);
                            process.WindowStyle = System.Diagnostics.ProcessWindowStyle.Hidden;
                            System.Diagnostics.Process.Start(process);
                            Application.Current.Exit += (sender, e) =>
                            {
                                var pl = Process.GetProcessesByName("ExamSys.Client");
                                foreach (var item in pl)
                                {
                                    item.Kill();
                                }
                            };
                        }
                        catch
                        { }
                        return;
                    }
                    if (!string.IsNullOrEmpty(ExamEntity.QuestionKey))
                    {
                        ProcessCommunication.MessageObject message = CommunicationObject;

                        string[] s = new string[] { ExamEntity.QuestionKey };
                        bool istrue = message.SendMessage(s, CommniucationFlag.B);

                        if (!istrue)
                        {
                            MessageBox.Show("向考试系统发送信息失败!", "错误", MessageBoxButton.OK, MessageBoxImage.Error);
                            return;
                        }

                        Application.Current.MainWindow.Visibility = System.Windows.Visibility.Collapsed;
                    }
                }));
            }
        }
        #endregion

        #region Method
        /// <summary>
        /// 初始化考试系统
        /// </summary>
        /// <returns></returns>
        public bool Init()
        {
            if (InExamMode)
            {
                CommunicationObject = new MessageObject("BtoA", "AtoB");
                CommunicationObject.ReceiveMessage += (flag, receiveString) =>
                {
                    #region 与tellersystem的约定A是启动考试模式
                    if (flag == CommniucationFlag.A)
                    {
                        try
                        {
                            if (receiveString.Length != 6)
                            {
                                MessageBox.Show("获取参数错误。", "错误", MessageBoxButton.OK, MessageBoxImage.Error);
                                return;
                            }
                            Application.Current.Dispatcher.Invoke(new Action(() =>
                            {
                                ExamEntity.QuestionKey = receiveString[0];
                                ExamEntity.QuestionName = receiveString[1];
                                ExamEntity.TimeNow = receiveString[2];
                                ExamEntity.StartTime = receiveString[3];
                                ExamEntity.ExamLast = receiveString[4];
                                ExamEntity.LeftTotalSeconds = receiveString[5];
                                Application.Current.MainWindow.Visibility = Visibility.Visible;
                                Application.Current.MainWindow.WindowState = WindowState.Normal;
                            }));
                        }
                        catch (Exception)
                        {
                            //throw;
                        }
                    }
                    #endregion
                };
            }
            return true;
        }

        public void SendExamData(string SendMsg, string RecevieMsg, string tradeCode, string serialNumber)
        {
            if (!InExamMode) return;

            MessageObject msgObject = CommunicationObject;

            if (msgObject != null && ExamEntity.IsExamModel == true && !string.IsNullOrEmpty(ExamEntity.QuestionKey))
            {
                // 返回值定义（题号;交易号;前端流水号;报文文本）

                string[] dl = new string[]
                           {
                             ExamEntity.QuestionKey, 
                               tradeCode==""?"0000":tradeCode,
                              serialNumber,
                               SendMsg, 
                               RecevieMsg
                           };

                bool istrue = msgObject.SendMessage(dl, CommniucationFlag.A);// 与考试系统预定A是传递答案过去

                if (!istrue)
                {
                    MessageBox.Show("向考试系统发送信息失败!", "错误", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
            }
            return;
        }

        public string KGetMessageString(Message msg, string flag)
        {
            if (!InExamMode) return string.Empty;

            // System.Reflection.Assembly a = System.Reflection.Assembly.GetAssembly();
            System.Reflection.PropertyInfo[] info = msg.GetType().GetProperties();

            string s = "";
            string msginfo = "\r" + "\n" + flag + "_域值\r" + "\n";
            for (int i = 0; i < info.Length; i++)
            {
                if (info[i].Name.StartsWith("Fd"))
                {

                    var tempstr = info[i].GetValue(msg, null).ToString().Trim().Replace("\r", "").Replace("\n", "");
                    if (!string.IsNullOrEmpty(tempstr))
                    {
                        msginfo += info[i].Name + "=" + info[i].GetValue(msg, null).ToString().Trim() + "\r" + "\n";
                        s += info[i].Name + "=" + info[i].GetValue(msg, null).ToString().Trim().Replace("\r", "").Replace("\n", "");
                        s += i == info.Length - 1 ? "" : "]";
                    }
                }
            }
            s += "@KSXT#" + msg.FileData;

            msginfo += "\r" + "\n" + flag + "_附件\r" + "\n";
            var file = msg.FileData.Split('|');
            for (int i = 0; i < file.Length; i++)
            {
                msginfo += i.ToString() + "#" + file[i] + "\r" + "\n";
            }
            Platform.Common.LogSystem.PlatformLogger.SystemInfo(msginfo);
            return s;
        }
        #endregion
    }
}
