﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using Platform.Presentation.FocusManagers;
using TellerSystem.Devices;
using TellerSystem.Devices.Interface;
using Platform.Common.RunningParameters;
using Platform.Presentation.Interfaces;
using Platform.Common.BasicFunctions;
using TellerSystem.Library.Ext.Function;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class InputDevice
    {
        #region 附加属性
        #region Input
        public static InputMode GetInput(TextBox obj)
        {
            return (InputMode)obj.GetValue(InputProperty);
        }

        public static void SetInput(TextBox obj, InputMode value)
        {
            obj.SetValue(InputProperty, value);
        }

        // Using a DependencyProperty as the backing store for Input.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty InputProperty =
            DependencyProperty.RegisterAttached("Input", typeof(InputMode), typeof(InputDevice), new UIPropertyMetadata(InputMode.KeyboardOrDevice));
        #endregion

        #region Device
        public static DeviceType GetDevice(TextBox obj)
        {
            return (DeviceType)obj.GetValue(DeviceProperty);
        }

        public static void SetDevice(TextBox obj, DeviceType value)
        {
            obj.SetValue(DeviceProperty, value);
        }

        // Using a DependencyProperty as the backing store for Device.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty DeviceProperty =
            DependencyProperty.RegisterAttached("Device", typeof(DeviceType), typeof(InputDevice), new UIPropertyMetadata(DeviceType.None, new PropertyChangedCallback((sender, e) =>
            {
                var device = (DeviceType)e.NewValue;
                if (device == DeviceType.None)
                {
                    FocusManager.RemovePreviewGotFocusHandler(sender, tb_PreviewGotFocus);
                }
                else
                {
                    FocusManager.RemovePreviewGotFocusHandler(sender, tb_PreviewGotFocus);
                    FocusManager.AddPreviewGotFocusHandler(sender, tb_PreviewGotFocus);
                }
            })));
        #endregion

        #region IsRepeatByPasswordKeyboard
        public static bool GetIsRepeatByPasswordKeyboard(TextBox obj)
        {
            return (bool)obj.GetValue(IsRepeatByPasswordKeyboardProperty);
        }

        public static void SetIsRepeatByPasswordKeyboard(TextBox obj, bool value)
        {
            obj.SetValue(IsRepeatByPasswordKeyboardProperty, value);
        }

        // Using a DependencyProperty as the backing store for IsRepeatByPasswordKeyboard.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsRepeatByPasswordKeyboardProperty =
            DependencyProperty.RegisterAttached("IsRepeatByPasswordKeyboard", typeof(bool), typeof(InputDevice), new UIPropertyMetadata(false));
        #endregion

        #region FingerUser
        public static string GetFingerUser(TextBox obj)
        {
            return (string)obj.GetValue(FingerUserProperty);
        }

        public static void SetFingerUser(TextBox obj, string value)
        {
            obj.SetValue(FingerUserProperty, value);
        }

        // Using a DependencyProperty as the backing store for FingerUser.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty FingerUserProperty =
            DependencyProperty.RegisterAttached("FingerUser", typeof(string), typeof(InputDevice), new UIPropertyMetadata(string.Empty));
        #endregion
        #endregion

        #region Ctor
        public InputDevice(TextBox tb)
        {
            Target = tb;
            ParentPage = tb.FindVisualTreeAncestor(x => x is IPage) as IPage;
        }
        /// <summary>
        /// 目标对象
        /// </summary>
        public TextBox Target { get; private set; }

        public IPage ParentPage { get; private set; }
        #endregion

        #region Method
        static void tb_PreviewGotFocus(object sender, RoutedEventArgs e)
        {
            var tb = sender as TextBox;
            if (tb == null) return;
            var inputDevice = new InputDevice(tb);
            inputDevice.RaiseDevice();
        }

        /// <summary>
        /// 触发设备调用
        /// </summary>
        public void RaiseDevice()
        {
            var device = GetDevice(Target);
            //是否抉择
            var input = GetInput(Target);
            switch (input)
            {
                case InputMode.KeyboardOrDevice:
                    //提示采用输入方式，允许跳过设备
                    var ret = MessageBox.Show("是否使用设备？\nYes:调用设备\nNo:键盘输入\nCancel:关闭交易", "", MessageBoxButton.YesNoCancel);
                    switch (ret)
                    {
                        case MessageBoxResult.Cancel:
                            ParentPage.Close();
                            return;
                        case MessageBoxResult.No:
                            return;
                        default:
                            break;
                    }
                    break;
                case InputMode.FixDevice:
                    //仅允许使用设备输入
                    var ret1 = MessageBox.Show("是否使用设备？\nOk:调用设备\nCancel:关闭交易", "", MessageBoxButton.OKCancel);
                    switch (ret1)
                    {
                        case MessageBoxResult.Cancel:
                            ParentPage.Close();
                            return;
                        default:
                            break;
                    }
                    break;
                default:
                    break;
            }
            switch (device)
            {
                case DeviceType.None:
                    break;
                case DeviceType.SwipeCard:
                    RaiseSwipeCard();
                    break;
                case DeviceType.PasswordKeyboard:
                    RaisePasswordKeyboard();
                    break;
                case DeviceType.IdentityCard:
                    RaiseIdentityCard();
                    break;
                case DeviceType.ICCard:
                    RaiseICCard();
                    break;
                case DeviceType.Finger:
                    RaiseFinger();
                    break;
                default:
                    break;
            }
        }

        private void RaiseFinger()
        {
            //throw new NotImplementedException();
            var fingerUser = GetFingerUser(Target);
            if (string.IsNullOrEmpty(fingerUser))
                fingerUser = LoginUserInfo.TellerNo;
            var data = DeviceHelper.FingerIsMatch(fingerUser);
            if (data.HasError)
                RaiseDevice();
        }

        private void RaiseICCard()
        {
            //throw new NotImplementedException();
        }

        private void RaiseIdentityCard()
        {
            //throw new NotImplementedException();
            var data = DeviceHelper.ReadIdentityCard();
            if (data.HasError)
                RaiseDevice();
        }

        private void RaisePasswordKeyboard()
        {
            //throw new NotImplementedException();
            var mode = GetIsRepeatByPasswordKeyboard(Target);
            DeviceEntity<PassWordEntity> data = null;
            if (mode)
                data = DeviceHelper.ReadPassWord2();
            else
                data = DeviceHelper.ReadPassWord();
            if (data.HasError)
                RaiseDevice();
        }

        private void RaiseSwipeCard()
        {
            //throw new NotImplementedException();
            var data = DeviceHelper.ReadCardTracks();
            if (data.HasError)
                RaiseDevice();
        }
        #endregion
    }

    #region 枚举
    /// <summary>
    /// 输入方式
    /// </summary>
    public enum InputMode
    {
        /// <summary>
        /// 键盘或设备
        /// </summary>
        KeyboardOrDevice,
        /// <summary>
        /// 必须设备输入
        /// </summary>
        FixDevice
    }

    /// <summary>
    /// 设备类型
    /// </summary>
    public enum DeviceType
    {
        /// <summary>
        /// 无
        /// </summary>
        None,
        /// <summary>
        /// 磁条卡
        /// </summary>
        SwipeCard,
        /// <summary>
        /// 密码键盘
        /// </summary>
        PasswordKeyboard,
        /// <summary>
        /// 二代身份证
        /// </summary>
        IdentityCard,
        /// <summary>
        /// IC卡
        /// </summary>
        ICCard,
        /// <summary>
        /// 指纹仪
        /// </summary>
        Finger
    }
    #endregion
}
