// 假设这些类型和类在其他地方定义
enum InputMode {
    KeyboardOrDevice,
    FixDevice
}

enum DeviceType {
    None,
    SwipeCard,
    PasswordKeyboard,
    IdentityCard,
    ICCard,
    Finger
}

interface TextBox {
    GetValue(property: any): any;
    SetValue(property: any, value: any): void;
    FindVisualTreeAncestor(predicate: (x: any) => boolean): any;
}

interface IPage {
    Close(): void;
}

interface DeviceEntity<T> {
    HasError: boolean;
    Data?: T;
}

interface PassWordEntity { }

class DeviceHelper {
    static FingerIsMatch(fingerUser: string): DeviceEntity<any> {
        // 实现逻辑
        return { HasError: false };
    }
    static ReadIdentityCard(): DeviceEntity<any> {
        // 实现逻辑
        return { HasError: false };
    }
    static ReadPassWord(): DeviceEntity<PassWordEntity> {
        // 实现逻辑
        return { HasError: false };
    }
    static ReadPassWord2(): DeviceEntity<PassWordEntity> {
        // 实现逻辑
        return { HasError: false };
    }
    static ReadCardTracks(): DeviceEntity<any> {
        // 实现逻辑
        return { HasError: false };
    }
}

class LoginUserInfo {
    static TellerNo: string = '';
}

class InputDevice {
    static InputProperty = { name: 'Input' };
    static DeviceProperty = { name: 'Device' };
    static IsRepeatByPasswordKeyboardProperty = { name: 'IsRepeatByPasswordKeyboard' };
    static FingerUserProperty = { name: 'FingerUser' };

    Target: TextBox;
    ParentPage: IPage;

    constructor(tb: TextBox) {
        this.Target = tb;
        // this.ParentPage = tb.FindVisualTreeAncestor(x => x instanceof IPage) as IPage;
    }

    static GetInput(obj: TextBox): InputMode {
        return obj.GetValue(this.InputProperty) as InputMode;
    }

    static SetInput(obj: TextBox, value: InputMode): void {
        obj.SetValue(this.InputProperty, value);
    }

    static GetDevice(obj: TextBox): DeviceType {
        return obj.GetValue(this.DeviceProperty) as DeviceType;
    }

    static SetDevice(obj: TextBox, value: DeviceType): void {
        obj.SetValue(this.DeviceProperty, value);
    }

    static GetIsRepeatByPasswordKeyboard(obj: TextBox): boolean {
        return obj.GetValue(this.IsRepeatByPasswordKeyboardProperty) as boolean;
    }

    static SetIsRepeatByPasswordKeyboard(obj: TextBox, value: boolean): void {
        obj.SetValue(this.IsRepeatByPasswordKeyboardProperty, value);
    }

    static GetFingerUser(obj: TextBox): string {
        return obj.GetValue(this.FingerUserProperty) as string;
    }

    static SetFingerUser(obj: TextBox, value: string): void {
        obj.SetValue(this.FingerUserProperty, value);
    }

    static tb_PreviewGotFocus(sender: any, e: any): void {
        const tb = sender as TextBox;
        if (!tb) return;
        const inputDevice = new InputDevice(tb);
        inputDevice.RaiseDevice();
    }

    RaiseDevice(): void {
        const device = InputDevice.GetDevice(this.Target);
        const input = InputDevice.GetInput(this.Target);

        switch (input) {
            case InputMode.KeyboardOrDevice:
                // 模拟 MessageBox.Show
                const ret = prompt('是否使用设备？\nYes:调用设备\nNo:键盘输入\nCancel:关闭交易');
                if (ret === 'Cancel') {
                    this.ParentPage.Close();
                    return;
                } else if (ret === 'No') {
                    return;
                }
                break;
            case InputMode.FixDevice:
                const ret1 = prompt('是否使用设备？\nOk:调用设备\nCancel:关闭交易');
                if (ret1 === 'Cancel') {
                    this.ParentPage.Close();
                    return;
                }
                break;
            default:
                break;
        }

        switch (device) {
            case DeviceType.None:
                break;
            case DeviceType.SwipeCard:
                this.RaiseSwipeCard();
                break;
            case DeviceType.PasswordKeyboard:
                this.RaisePasswordKeyboard();
                break;
            case DeviceType.IdentityCard:
                this.RaiseIdentityCard();
                break;
            case DeviceType.ICCard:
                this.RaiseICCard();
                break;
            case DeviceType.Finger:
                this.RaiseFinger();
                break;
            default:
                break;
        }
    }

    private RaiseFinger(): void {
        let fingerUser = InputDevice.GetFingerUser(this.Target);
        if (!fingerUser) {
            fingerUser = LoginUserInfo.TellerNo;
        }
        const data = DeviceHelper.FingerIsMatch(fingerUser);
        if (data.HasError) {
            this.RaiseDevice();
        }
    }

    private RaiseICCard(): void {
        // 未实现
    }

    private RaiseIdentityCard(): void {
        const data = DeviceHelper.ReadIdentityCard();
        if (data.HasError) {
            this.RaiseDevice();
        }
    }

    private RaisePasswordKeyboard(): void {
        const mode = InputDevice.GetIsRepeatByPasswordKeyboard(this.Target);
        let data: DeviceEntity<PassWordEntity> | null = null;
        if (mode) {
            data = DeviceHelper.ReadPassWord2();
        } else {
            data = DeviceHelper.ReadPassWord();
        }
        if (data && data.HasError) {
            this.RaiseDevice();
        }
    }

    private RaiseSwipeCard(): void {
        const data = DeviceHelper.ReadCardTracks();
        if (data.HasError) {
            this.RaiseDevice();
        }
    }
}

export { InputDevice, InputMode, DeviceType };
