// 模拟 DependencyObject 类
class DependencyObject {
    private properties: { [key: string]: any } = {};

    getValue(property: string): any {
        return this.properties[property];
    }

    setValue(property: string, value: any): void {
        this.properties[property] = value;
    }
}

// 模拟 UIPropertyMetadata 类
class UIPropertyMetadata {
    constructor(public defaultValue: any) {}
}

// 模拟 DependencyProperty 类
class DependencyProperty {
    constructor(
        public name: string,
        public type: any,
        public ownerType: any,
        public metadata: UIPropertyMetadata
    ) {}

    static registerAttached(name: string, type: any, ownerType: any, metadata: UIPropertyMetadata): DependencyProperty {
        return new DependencyProperty(name, type, ownerType, metadata);
    }
}

// 转换 IsChecked 类
export class IsChecked {
    static readonly IsCheckedProperty: DependencyProperty =
        DependencyProperty.registerAttached("IsChecked", Boolean, IsChecked, new UIPropertyMetadata(null));

    static getIsChecked(obj: DependencyObject): boolean {
        return obj.getValue(IsChecked.IsCheckedProperty.name) as boolean;
    }

    static setIsChecked(obj: DependencyObject, value: boolean): void {
        obj.setValue(IsChecked.IsCheckedProperty.name, value);
    }
}