﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class SystemSettingsHelper
    {
        #region Ctor
        public SystemSettingsHelper()
        {
            var list = TradeManagerHandle.GetSysconfigList(null);
            _dict = list.ToDictionary(x => x.Configname, x => x.Configvalue);
        }

        private static SystemSettingsHelper _instance;

        public static SystemSettingsHelper Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new SystemSettingsHelper();
                return _instance;
            }
        }
        #endregion

        #region Data
        private Dictionary<string, string> _dict;
        #endregion

        #region Prop
        /// <summary>
        /// 强制检查更新
        /// </summary>
        public bool IsCheckUpdate
        {
            get
            {
                var flag = true;
                //是否强制自动更新
                if (_dict.ContainsKey("自动更新"))
                {
                    bool.TryParse(_dict["自动更新"], out flag);
                }
                return flag;
            }
        }

        /// <summary>
        /// 是否执行身份联网核查
        /// </summary>
        public bool IsIdCheck
        {
            get
            {
                var flag = true;
                //是否执行身份联网核查
                if (_dict.ContainsKey("联网核查"))
                {
                    bool.TryParse(_dict["联网核查"], out flag);
                }
                return flag;
            }
        }

        /// <summary>
        /// 登陆背景
        /// </summary>
        public string LoginBackgroundFile
        {
            get
            {
                var ret = string.Empty;
                //采用的背景图片
                if (_dict.ContainsKey("登陆背景"))
                {
                    ret = _dict["登陆背景"];
                }
                return ret;
            }
        }
        /// <summary>
        /// 是否检查当前柜员是否是客户
        /// </summary>
        public bool IsTellerCustomer
        {
            get
            {
                var flag = true;
                //是否强制自动更新
                if (_dict.ContainsKey("客户是否是柜员"))
                {
                    bool.TryParse(_dict["客户是否是柜员"], out flag);
                }
                return flag;
            }
        }
        #endregion
    }
}
