// 模拟 ISerializable 接口
interface ISerializable {
    serialize(): string;
    static deserialize(data: string): any;
}

class TradeBusData {
    private tradeBusData: { [key: string]: any } = {};
    private formatter = {
        serialize: (value: any) => JSON.stringify(value),
        deserialize: (data: string) => JSON.parse(data)
    };

    constructor() {
        this.tradeBusData = {};
    }

    public Clear(): void {
        this.tradeBusData = {};
    }

    public get(key: string): string {
        if (this.tradeBusData.hasOwnProperty(key)) {
            return String(this.tradeBusData[key]);
        }
        return '';
    }

    public set(key: string, value: string): void {
        if (this.tradeBusData.hasOwnProperty(key)) {
            this.tradeBusData[key] = value;
        } else {
            this.tradeBusData[key] = value;
        }
    }

    public GetDataItem<T extends ISerializable>(key: string): T | null {
        const newKey = 'Serialize_' + key;
        if (this.tradeBusData.hasOwnProperty(newKey)) {
            return T.deserialize(this.tradeBusData[newKey]) as T;
        }
        return null;
    }

    public SetDataItem<T extends ISerializable>(key: string, value: T): void {
        const newKey = 'Serialize_' + key;
        this.tradeBusData[newKey] = value.serialize();
    }

    public Serialize(): { [key: string]: string } {
        const result: { [key: string]: string } = {};
        for (const [key, value] of Object.entries(this.tradeBusData)) {
            if (key.startsWith('Serialize_')) {
                result[key] = this.formatter.serialize(value);
            } else {
                result[key] = String(value);
            }
        }
        return result;
    }

    public Deserialize(data: { [key: string]: string } | null): boolean {
        if (data === null) {
            return false;
        }
        this.tradeBusData = {};
        for (const [key, value] of Object.entries(data)) {
            if (key.startsWith('Serialize_')) {
                this.tradeBusData[key] = this.formatter.deserialize(value);
            } else {
                this.tradeBusData[key] = value;
            }
        }
        return true;
    }

    public ContainsKey(key: string): boolean {
        return this.tradeBusData.hasOwnProperty(key);
    }
}

class DemoItem implements ISerializable {
    public A: string = '';
    public B: string = '';

    constructor() {}

    public serialize(): string {
        return JSON.stringify({ A: this.A, B: this.B });
    }

    public static deserialize(data: string): DemoItem {
        const obj = JSON.parse(data);
        const demoItem = new DemoItem();
        demoItem.A = obj.A;
        demoItem.B = obj.B;
        return demoItem;
    }
}

export { TradeBusData, DemoItem };