﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Platform.Presentation.Interfaces;
using Platform.Common.BasicFunctions;
using System.Windows;
using Platform.Presentation.PagesHelper;
using Platform.Presentation.PageFunctions;
using System.Windows.Media;
using TellerSystem.Communication;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class TradeInfoData
    {
        #region 基础
        #region 附加到IPage对象
        public static TradeInfoData GetTradeInfo(DependencyObject obj)
        {
            return (TradeInfoData)obj.GetValue(TradeInfoProperty);
        }

        public static void SetTradeInfo(DependencyObject obj, TradeInfoData value)
        {
            obj.SetValue(TradeInfoProperty, value);
        }

        // Using a DependencyProperty as the backing store for TradeInfo.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TradeInfoProperty =
            DependencyProperty.RegisterAttached("TradeInfo", typeof(TradeInfoData), typeof(TradeInfoData), new UIPropertyMetadata(null));
        #endregion
        #endregion

        #region ctor
        public TradeInfoData()
        {
            TradeBus = new TradeBusData();
            SerialNumber = string.Empty;
            SystemSerialNumber = string.Empty;
            TradeCode = string.Empty;
            TradeName = string.Empty;
            TradeScreens = new Dictionary<string, List<ImageSource>>();
        }
        #endregion

        #region 交易信息属性
        /// <summary>
        /// 柜员前端流水号
        /// </summary>
        public string SerialNumber
        {
            get { return TradeBus["SerialNumber"]; }
            set { TradeBus["SerialNumber"] = value; }
        }
        /// <summary>
        /// 系统前端流水号
        /// </summary>
        public string SystemSerialNumber
        {
            get { return TradeBus["SystemSerialNumber"]; }
            internal set { TradeBus["SystemSerialNumber"] = value; }
        }

        /// <summary>
        /// 打印票号
        /// </summary>
        public string PrintNum
        {
            get { return TradeBus["PrintNum"]; }
            internal set { TradeBus["PrintNum"] = value; }
        }
        /// <summary>
        /// 交易号，指代一个完整的业务流程
        /// </summary>
        public string TradeCode
        {
            get;
            internal set;
        }
        /// <summary>
        /// 交易名称，指代一个完整的业务流程
        /// </summary>
        public string TradeName
        {
            get { return TradeBus["TradeName"]; }
            internal set { TradeBus["TradeName"] = value; }
        }
        /// <summary>
        /// 交易截图
        /// </summary>
        public Dictionary<string, List<ImageSource>> TradeScreens
        {
            get;
            set;
        }

        /// <summary>
        /// 登记最近一次的通讯对象
        /// </summary>
        public Message LatestMessage
        {
            get;
            internal set;
        }

        /// <summary>
        /// 总线数据集合
        /// </summary>
        public TradeBusData TradeBus
        {
            get;
            internal set;
        }
        #endregion

        #region 公共方法
        /// <summary>
        /// 设置数据总线
        /// </summary>
        /// <param name="bus"></param>
        public void SetTradeBus(TradeBusData bus)
        {
            TradeBus = bus;
        }

        /// <summary>
        /// 设置交易信息
        /// </summary>
        /// <param name="code"></param>
        /// <param name="name"></param>
        public void SetTradeInfo(string code, string name)
        {
            TradeCode = code;
            TradeName = name;
        }
        #endregion
    }
}
