// 模拟 DependencyObject 类
class DependencyObject {
    private properties: { [key: string]: any } = {};

    getValue(key: string): any {
        return this.properties[key];
    }

    setValue(key: string, value: any): void {
        this.properties[key] = value;
    }
}

// 模拟 UIPropertyMetadata 类
class UIPropertyMetadata {
    constructor(public defaultValue: any) {}
}

// 模拟 TradePowerManager 类
class TradePowerManager {
    static Instance = new TradePowerManager();

    GetTradePowerByCode(tradeCode: string): { RuleNo: string }[] {
        // 这里可以添加实际的逻辑，当前返回空数组
        return [];
    }
}

// 模拟 TradeBusData 类
class TradeBusData {}

// 定义 TradeInfoData 类
class TradeInfoData {
    static readonly TradeInfoProperty = {
        name: 'TradeInfo',
        type: TradeInfoData,
        ownerType: TradeInfoData,
        metadata: new UIPropertyMetadata(null)
    };

    static GetTradeInfo(obj: DependencyObject): TradeInfoData {
        return obj.getValue(this.TradeInfoProperty.name) as TradeInfoData;
    }

    static SetTradeInfo(obj: DependencyObject, value: TradeInfoData): void {
        obj.setValue(this.TradeInfoProperty.name, value);
    }

    SerialNumber: string;
    SystemSerialNumber: string;
    PrintNum: string;
    TradeCode: string;
    TradeName: string;
    TradeBus: TradeBusData;

    constructor() {
        this.SerialNumber = '';
        this.SystemSerialNumber = '';
        this.PrintNum = '';
        this.TradeCode = '';
        this.TradeName = '';
        this.TradeBus = new TradeBusData();
    }

    CheckTradePower(powerCode: string): boolean {
        const list = TradePowerManager.Instance.GetTradePowerByCode(this.TradeCode);
        if (list.length === 0) {
            return false;
        }
        return list.some((item) => item.RuleNo === powerCode);
    }
}

export { TradeInfoData };