﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Platform.Common.BasicFunctions;
using Platform.Common.RunningParameters;
using System.Xml.Linq;
using System.IO;
using TellerSystem.Library.Ext.Variables;
using Platform.ServiceProxy.ServiceHelper;
using Platform.Common.LogSystem;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.Library.Ext.TradeExtension
{
    /// <summary>
    /// 交易菜单管理类
    /// </summary>
    public class TradeMenuManager
    {
        #region 单例模式
        private static TradeMenuManager _instance;
        /// <summary>
        /// 获取单例对象
        /// </summary>
        /// <returns></returns>
        public static TradeMenuManager Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new TradeMenuManager();
                return _instance;
            }
        }
        private TradeMenuManager()
        {
            //_tradeMenuList = new Dictionary<TradeModel, bool?>();
            //加载数据源
            //if (PlatformSettings.IsDebugMode)
            //{
            //    LoadDataByXml();
            //}
            //else
            //{
            //    LoadData();
            //}

            if (!CheakUpdate())
            {
                throw new FileNotFoundException("更新菜单文件出现错误！");
            }

            // LoadDataByXml();
            // LoginUserInfo.SetUserInfo("TradeRight", GetDispalyMenuData().ToArray());
        }
        #endregion

        #region 存储方案
        /// <summary>
        /// 加载的菜单列表，Key=TradeModel对象，Value=菜单项状态【true:显示    false:隐藏    null:无权限】
        /// </summary>
        private Dictionary<TradeModel, bool?> _tradeMenuList;
        private FileInfo _menuFile = new FileInfo(Path.Combine(ClientSettings.ConfigPath
               , ClientSettings.MenuFileName));
        private string _serverMenuFile = ConfigManager.GetInstance().GetConfigValue("ConfigFilePath", ConfigType.System) + "Menu.xml";
        #endregion

        #region 私有方法
        /// <summary>
        /// 加载数据源
        /// </summary>
        private void LoadData()
        {
            throw new InvalidOperationException("未实现从服务端加载菜单数据源的方法！");
        }
        /// <summary>
        /// 加载数据源(来自于xml文件)
        /// </summary>
        private void LoadDataByXml()
        {
            ////更新菜单项
            //if (!CheakUpdate())
            //{
            //    throw new FileNotFoundException("更新菜单文件出现错误！");
            //}
            _tradeMenuList = new Dictionary<TradeModel, bool?>();
            XDocument doc = XDocument.Load(_menuFile.FullName);
            GetModel(doc.Root, null);
            //var data = GetModel(doc.Root, null);
            //foreach (var item in data)
            //{
            //    _tradeMenuList.Add(item, true);
            //}
        }

        private bool CheakUpdate()
        {
            if (Platform.Common.RunningParameters.PlatformSettings.IsDebugMode)
                return true;
            //throw new NotImplementedException();
            if (_menuFile.Exists)
            {
                //检查更新
                if (FilesHandle.CheckMD5(_menuFile.FullName, _serverMenuFile))
                    return true;
            }
            //执行更新
            return FilesHandle.DownloadFile(_menuFile.FullName, _serverMenuFile);
        }

        private void GetModel(XElement parentElement, string parentID)
        {
            foreach (var ele in parentElement.Elements())
            {
                var guid = Guid.NewGuid().ToString();
                var id = ele.Attribute("MenuID");//Guid.NewGuid().ToString();
                if (id != null) guid = id.Value.Trim();
                TradeModel model = null;
                switch (ele.Name.LocalName)
                {
                    case "category":
                        model = new TradeModel()
                                          {
                                              GUID = guid,
                                              Name = ele.Attribute("name").Value.Trim(),
                                              Type = "category",
                                              ParentID = parentID,
                                          };
                        _tradeMenuList.Add(model, true);
                        GetModel(ele, guid);
                        //foreach (var item in GetModel(ele, guid))
                        //    yield return item;

                        break;

                    case "trade":
                        model = new TradeModel()
                                          {
                                              GUID = guid,
                                              Name = ele.Attribute("name").Value.Trim(),
                                              Code = ele.Attribute("menucode").Value,
                                              Action = ele.Attribute("action").Value,
                                              Type = "trade",
                                              ParentID = parentID,
                                          };

                        model.SetMetadata(new Dictionary<string, string>());

                        foreach (var meta in ele.Elements())
                            model.Metadata.Add
                            (
                                meta.Attribute("key").Value,
                                meta.Attribute("value").Value
                            );
                        //检测是否存在重复Code节点
                        var tmp = _tradeMenuList.Keys.FirstOrDefault(x => x.Code == model.Code);
                        if (tmp == null)
                            _tradeMenuList.Add(model, ele.Attribute("State") != null ? ele.Attribute("State").Value != "0" : true);
                        else
                            System.Windows.MessageBox.Show("菜单解析：存在重复Code的交易节点=" + tmp.Code);
                        //yield return model;
                        break;

                    default:
                        break;
                }
            }
        }
        #endregion

        #region 公开方法
        /// <summary>
        /// 权限筛选
        /// </summary>
        /// <param name="predicate"></param>
        public void DoAuthFilter(Predicate<TradeModel> predicate)
        {
            LoadDataByXml();
            LoginUserInfo.SetUserInfo("TradeRight", GetDispalyMenuData().ToArray());
            if (predicate == null) return;
            for (int i = 0; i < _tradeMenuList.Count; i++)
            {
                var item = _tradeMenuList.ElementAt(i);
                //权限筛选
                if (!predicate(item.Key))
                    _tradeMenuList[item.Key] = null;
            }
            LoginUserInfo.SetUserInfo("TradeRight", GetDispalyMenuData().ToArray());
        }




        public void FilterMenus()
        {
            var entitys = TradeManagerHandle.getVacatDatInfo();
            if (entitys == null) return;
            if (entitys.Count == 0) return;
            var entitys1 = entitys.Where(c => c.vacatDay == LoginUserInfo.TradeDate);
            if (entitys1 != null && entitys1.Count() > 0)
            {
                for (int i = 0; i < _tradeMenuList.Count; i++)
                {
                    var item = _tradeMenuList.ElementAt(i);
                    if (item.Key.Type != "category")
                    {
                        var model = item.Key;
                        if (model != null && model.Metadata.ContainsKey("isUseVacaControl"))
                        {
                            if (string.Compare(model.Metadata["isUseVacaControl"], "true", true) == 0)
                            {
                                _tradeMenuList[item.Key] = null;
                            }
                        }
                    }
                }
            }
            LoginUserInfo.SetUserInfo("TradeRight", GetDispalyMenuData().ToArray());
        }
        /// <summary>
        /// 获取显示的菜单数据项集合
        /// </summary>
        /// <returns></returns>
        public IEnumerable<TradeModel> GetDispalyMenuData()
        {
            return _tradeMenuList.Where(x =>
            {
                return x.Value == true;
            }).Select(x => { return x.Key; });
        }

        /// <summary>
        /// 获取符合predicate条件的TradeModel对象，可以选择检索范围
        /// </summary>
        /// <param name="action"></param>
        /// <returns></returns>
        public IEnumerable<TradeModel> GetTradeModel(Predicate<TradeModel> predicate, bool isCheckAll = false)
        {
            var result = _tradeMenuList.Where(x =>
            {
                if (!isCheckAll && x.Value == null)
                    return false;
                if (predicate == null) return true;
                return predicate(x.Key);
            }).Select(x => { return x.Key; });
            return result;
        }

        /// <summary>
        /// 是否菜单中的交易码
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public bool IsCodeOfMenu(string code)
        {
            var result = GetTradeModel(x => x.Code == code, true);
            if (result == null || result.Count() == 0) return false;
            return _tradeMenuList[result.First()] == true;
        }
        #endregion
    }
}
