// 模拟依赖的类和命名空间
class FileInfo {
    constructor(public fullName: string) {}
    get exists(): boolean {
        // 模拟文件存在检查
        return true;
    }
}

class PlatformSettings {
    static get IsDebugMode(): boolean {
        return false;
    }
}

class ClientSettings {
    static get ConfigPath(): string {
        return "";
    }
    static get MenuFileName(): string {
        return "";
    }
}

class ConfigManager {
    static GetInstance() {
        return {
            GetConfigValue: (key: string, type: any) => ""
        };
    }
}

class FilesHandle {
    static CheckMD5(localPath: string, serverPath: string): boolean {
        return true;
    }
    static DownloadFile(localPath: string, serverPath: string): boolean {
        return true;
    }
}

class LoginUserInfo {
    static SetUserInfo(key: string, value: any) {}
}

class TradeManagerHandle {
    static getVacatDatInfo() {
        return [];
    }
}

// 定义 TradeModel 类
class TradeModel {
    GUID: string;
    Name: string;
    Code?: string;
    Action?: string;
    Type: string;
    ParentID?: string;
    Metadata: { [key: string]: string } = {};

    constructor() {
        this.GUID = "";
        this.Name = "";
        this.Type = "";
    }

    SetMetadata(metadata: { [key: string]: string }) {
        this.Metadata = metadata;
    }
}

// 定义 TradeMenuManager 类
class TradeMenuManager {
    private static _instance: TradeMenuManager;
    private _tradeMenuList: Map<TradeModel, boolean | null> = new Map();
    private _menuFile: FileInfo;
    private _serverMenuFile: string;

    private constructor() {
        this._menuFile = new FileInfo(`${ClientSettings.ConfigPath}/${ClientSettings.MenuFileName}`);
        this._serverMenuFile = ConfigManager.GetInstance().GetConfigValue("ConfigFilePath", "System") + "Menu.xml";

        if (!this.CheakUpdate()) {
            throw new Error("更新菜单文件出现错误！");
        }

        // this.LoadDataByXml();
        // LoginUserInfo.SetUserInfo("TradeRight", Array.from(this.GetDispalyMenuData()));
    }

    static get Instance(): TradeMenuManager {
        if (!this._instance) {
            this._instance = new TradeMenuManager();
        }
        return this._instance;
    }

    private LoadData() {
        throw new Error("未实现从服务端加载菜单数据源的方法！");
    }

    private LoadDataByXml() {
        this._tradeMenuList = new Map();
        // 模拟 XDocument.Load
        // const doc = XDocument.Load(this._menuFile.FullName);
        // 由于无法直接解析 XML，这里暂时不实现具体解析逻辑
        // this.GetModel(doc.Root, null);
    }

    private CheakUpdate(): boolean {
        if (PlatformSettings.IsDebugMode) {
            return true;
        }
        if (this._menuFile.exists) {
            if (FilesHandle.CheckMD5(this._menuFile.fullName, this._serverMenuFile)) {
                return true;
            }
        }
        return FilesHandle.DownloadFile(this._menuFile.fullName, this._serverMenuFile);
    }

    private GetModel(parentElement: any, parentID?: string) {
        // 模拟 XML 元素遍历
        // for (const ele of parentElement.Elements()) {
        //     let guid = Math.random().toString(36).substr(2);
        //     const id = ele.getAttribute("MenuID");
        //     if (id) {
        //         guid = id.trim();
        //     }
        //     let model: TradeModel | null = null;
        //     switch (ele.localName) {
        //         case "category":
        //             model = new TradeModel();
        //             model.GUID = guid;
        //             model.Name = ele.getAttribute("name").trim();
        //             model.Type = "category";
        //             model.ParentID = parentID;
        //             this._tradeMenuList.set(model, true);
        //             this.GetModel(ele, guid);
        //             break;
        //         case "trade":
        //             model = new TradeModel();
        //             model.GUID = guid;
        //             model.Name = ele.getAttribute("name").trim();
        //             model.Code = ele.getAttribute("menucode");
        //             model.Action = ele.getAttribute("action");
        //             model.Type = "trade";
        //             model.ParentID = parentID;
        //             model.SetMetadata({});
        //             for (const meta of ele.children) {
        //                 model.Metadata[meta.getAttribute("key")] = meta.getAttribute("value");
        //             }
        //             const tmp = Array.from(this._tradeMenuList.keys()).find(x => x.Code === model.Code);
        //             if (!tmp) {
        //                 const state = ele.getAttribute("State");
        //                 this._tradeMenuList.set(model, state ? state !== "0" : true);
        //             } else {
        //                 console.log(`菜单解析：存在重复 Code 的交易节点=${tmp.Code}`);
        //             }
        //             break;
        //         default:
        //             break;
        //     }
        // }
    }

    public DoAuthFilter(predicate: (model: TradeModel) => boolean) {
        this.LoadDataByXml();
        LoginUserInfo.SetUserInfo("TradeRight", Array.from(this.GetDispalyMenuData()));
        if (!predicate) {
            return;
        }
        for (const [key, _] of this._tradeMenuList) {
            if (!predicate(key)) {
                this._tradeMenuList.set(key, null);
            }
        }
        LoginUserInfo.SetUserInfo("TradeRight", Array.from(this.GetDispalyMenuData()));
    }

    public FilterMenus() {
        const entitys = TradeManagerHandle.getVacatDatInfo();
        if (!entitys || entitys.length === 0) {
            return;
        }
        const entitys1 = entitys.filter(c => c.vacatDay === LoginUserInfo.TradeDate);
        if (entitys1 && entitys1.length > 0) {
            for (const [key, _] of this._tradeMenuList) {
                if (key.Type !== "category") {
                    if (key.Metadata.hasOwnProperty("isUseVacaControl") && key.Metadata["isUseVacaControl"].toLowerCase() === "true") {
                        this._tradeMenuList.set(key, null);
                    }
                }
            }
        }
        LoginUserInfo.SetUserInfo("TradeRight", Array.from(this.GetDispalyMenuData()));
    }

    public GetDispalyMenuData(): TradeModel[] {
        return Array.from(this._tradeMenuList.entries())
            .filter(([_, value]) => value === true)
            .map(([key, _]) => key);
    }

    public GetTradeModel(predicate: (model: TradeModel) => boolean, isCheckAll = false): TradeModel[] {
        return Array.from(this._tradeMenuList.entries())
            .filter(([_, value]) => {
                if (!isCheckAll && value === null) {
                    return false;
                }
                if (!predicate) {
                    return true;
                }
                return predicate(_);
            })
            .map(([key, _]) => key);
    }

    public IsCodeOfMenu(code: string): boolean {
        const result = this.GetTradeModel(x => x.Code === code, true);
        if (!result || result.length === 0) {
            return false;
        }
        return this._tradeMenuList.get(result[0]) === true;
    }
}

export { TradeMenuManager, TradeModel };