﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.ServiceProxy.Ext.TradeBusinessEntitys;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class TradePowerManager
    {
        #region 单列模式
        private static TradePowerManager _instance;
        public static TradePowerManager Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new TradePowerManager();
                return _instance;
            }
        }
        /// <summary>
        /// 是否启用柜员权限控制
        /// </summary>
        public bool IsPowerCheckByUser { get; private set; }

        private TradePowerManager()
        {
            var value = Platform.Common.RunningParameters.ConfigManager.GetInstance().GetConfigValue("IsPowerCheckByUser", Platform.Common.RunningParameters.ConfigType.System);
            var flag = true;
            bool.TryParse(value, out flag);
            IsPowerCheckByUser = flag;
        }
        #endregion

        #region 公开方法
        ///// <summary>
        ///// 初始化数据
        ///// </summary>
        ///// <param name="tellerNo"></param>
        ///// <returns></returns>
        //public bool InitData(string tellerNo)
        //{
        //    if (!IsPowerCheckByUser) return true;
        //    //使用服务端接口查询数据
        //    _ruleInfoList = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.GetPowerByUserCode(tellerNo);
        //    return true;
        //    //使用sql查询数据
        //    var data = GetRuleFromUser(tellerNo, string.Empty);
        //    if (data == null) return false;
        //    //解析数据
        //    _ruleInfoList.Clear();
        //    foreach (var item in data)
        //    {
        //        if (string.IsNullOrEmpty(item)) continue;
        //        var attr = item.Split('|');
        //        if (attr.Length == 8)
        //            _ruleInfoList.Add(new TT_RuleInfo
        //            {
        //                RuleId = attr[0],
        //                RulePId = attr[1],
        //                RuleNo = attr[2],
        //                RuleName = attr[3],
        //                RuleRemark = attr[4],
        //                RuleType = attr[5],
        //                ExtId = attr[6],
        //                RuleComment = attr[7]
        //            });
        //    }
        //    return true;
        //}

        /// <summary>
        /// 初始化数据
        /// </summary>
        /// <param name="tellerNo"></param>
        /// <param name="grouplist"></param>
        /// <returns></returns>
        public bool InitData(string tellerNo,string grouplist)
        {
            if (!IsPowerCheckByUser) return true;
            //使用服务端接口查询数据
            _ruleInfoList = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.GetPowerByUserCode(tellerNo, grouplist);

            #region 此方法废弃
            ////使用sql查询数据
            //var data = GetRuleFromUser(tellerNo, string.Empty);
            //if (data == null) return false;
            ////解析数据
            //_ruleInfoList.Clear();
            //foreach (var item in data)
            //{
            //    if (string.IsNullOrEmpty(item)) continue;
            //    var attr = item.Split('|');
            //    if (attr.Length == 8)
            //        _ruleInfoList.Add(new TT_RuleInfo
            //        {
            //            RuleId = attr[0],
            //            RulePId = attr[1],
            //            RuleNo = attr[2],
            //            RuleName = attr[3],
            //            RuleRemark = attr[4],
            //            RuleType = attr[5],
            //            ExtId = attr[6],
            //            RuleComment = attr[7]
            //        });
            //}
            #endregion
           
            return true;
        }
        /// <summary>
        /// 获取菜单权限列表
        /// </summary>
        /// <returns></returns>
        public List<TT_RuleInfo> GetTradeMenuList()
        {
            return _ruleInfoList.Where(x => { return x.RuleType == "1" || x.RuleType == "0"; }).ToList();
        }
        /// <summary>
        /// 获取菜单项的子功能列表
        /// </summary>
        /// <param name="tradeCode"></param>
        /// <returns></returns>
        public List<TT_RuleInfo> GetTradePowerByCode(string tradeCode)
        {
            var trade = _ruleInfoList.Find(x => x.RuleNo == tradeCode && x.RuleType == "1");
            if (trade == null)
            {
                return new List<TT_RuleInfo>();
            }
            else
            {
                return _ruleInfoList.Where(x => x.RulePId == trade.RuleId && x.RuleType == "2").ToList();
            }
        }
        #endregion

        #region 私有方法
        /// <summary>
        /// 权限列表
        /// </summary>
        private List<TT_RuleInfo> _ruleInfoList = new List<TT_RuleInfo>();

        /// <summary>
        /// 获取用户的权限 根据用户登录账号和密码（密码暂不用传入）
        /// </summary>
        /// <param name="userCode">用户登录账号</param>
        /// <param name="userPsw">用户密码</param>
        /// <returns>权限ID列表【交易ID】（字符串数组）</returns>
        string[] GetRuleFromUser(string userCode, string userPsw)
        {
            //如果用户ID为空则直接返回，本函数不支持查询全部权限
            if (string.IsNullOrEmpty(userCode)) return null;

            //todo:此处需要增加用户账号和密码的验证

            string[] strRult = null;
            try
            {
                StringBuilder strSql = new StringBuilder();
                strSql.Append(" SELECT RULEID,RULEPID,RULENO,RULENAME,RULEREMARK,RULETYPE,EXTID,RULECOMMENT FROM TT_RULEINFO WHERE RULEID IN ( ");
                strSql.Append(" SELECT GROUPRULE.RULEID ");
                strSql.Append("   FROM (SELECT DISTINCT TT_KINBRRULE.RULEID ");
                strSql.Append("           FROM TT_LOGINUSER, TT_KINBRRULE ");
                strSql.Append("          WHERE TT_LOGINUSER.KINBRID = TT_KINBRRULE.KINBRID ");
                strSql.Append("            AND TT_LOGINUSER.USERCODE = '" + userCode.ToString() + "') KINBRRULE,        ");
                strSql.Append("        (SELECT DISTINCT RULEGET.RULEID ");
                strSql.Append("           FROM (SELECT TT_GROUPRULE.RULEID ");
                strSql.Append("                   FROM TT_LOGINUSER, TT_USERROLE, TT_ROLEGROUP, TT_GROUPRULE ");
                strSql.Append("                  WHERE TT_LOGINUSER.USERID = TT_USERROLE.USERID ");
                strSql.Append("                    AND TT_USERROLE.GROUPID = TT_ROLEGROUP.GROUPID ");
                strSql.Append("                    AND TT_ROLEGROUP.GROUPID = TT_GROUPRULE.GROUPID ");
                strSql.Append("                    AND TT_GROUPRULE.DELSIGN = '0' ");
                strSql.Append("                    AND TT_LOGINUSER.USERCODE = '" + userCode.ToString() + "') RULEGET ");
                strSql.Append("          WHERE NOT EXISTS ");
                strSql.Append("          (SELECT RULEID ");
                strSql.Append("                   FROM (SELECT TT_GROUPRULE.RULEID ");
                strSql.Append("                           FROM TT_LOGINUSER, ");
                strSql.Append("                                TT_USERROLE, ");
                strSql.Append("                                TT_ROLEGROUP, ");
                strSql.Append("                                TT_GROUPRULE ");
                strSql.Append("                          WHERE TT_LOGINUSER.USERID = TT_USERROLE.USERID ");
                strSql.Append("                            AND TT_USERROLE.GROUPID = TT_ROLEGROUP.GROUPID ");
                strSql.Append("                            AND TT_ROLEGROUP.GROUPID = TT_GROUPRULE.GROUPID ");
                strSql.Append("                            AND TT_GROUPRULE.DELSIGN = '1' ");
                strSql.Append("                            AND TT_LOGINUSER.USERCODE = '" + userCode.ToString() + "') RULEOUT ");
                strSql.Append("                  WHERE RULEGET.RULEID = RULEOUT.RULEID)) GROUPRULE ");
                strSql.Append("  WHERE KINBRRULE.RULEID = GROUPRULE.RULEID ");
                strSql.Append(")");
                strRult = (string[])UserManagerHandle.ExecuteQuery(strSql.ToString());
            }
            catch (Exception)
            {
                //todo:此处加入异常处理
            }
            return strRult;
        }
        #endregion
    }
}
