// 模拟依赖的类和接口
interface TT_RuleInfo {
    RuleId: string;
    RulePId: string;
    RuleNo: string;
    RuleName: string;
    RuleRemark: string;
    RuleType: string;
    ExtId: string;
    RuleComment: string;
}

class Platform {
    static Common = {
        RunningParameters: {
            ConfigManager: {
                GetInstance: () => ({
                    GetConfigValue: (key: string, type: string) => "",
                }),
            },
        },
    };
}

class TellerSystem {
    static ServiceProxy = {
        Ext: {
            ServiceHelper: {
                TradeManagerHandle: {
                    GetPowerByUserCode: (tellerNo: string, grouplist?: string) => [] as TT_RuleInfo[],
                },
            },
        },
    };
}

class UserManagerHandle {
    static ExecuteQuery = (sql: string) => [] as string[];
}

class TradePowerManager {
    private static _instance: TradePowerManager;
    public IsPowerCheckByUser: boolean;
    private _ruleInfoList: TT_RuleInfo[] = [];

    private constructor() {
        const value = Platform.Common.RunningParameters.ConfigManager.GetInstance().GetConfigValue(
            "IsPowerCheckByUser",
            "System"
        );
        this.IsPowerCheckByUser = value.toLowerCase() === "true";
    }

    public static get Instance(): TradePowerManager {
        if (!this._instance) {
            this._instance = new TradePowerManager();
        }
        return this._instance;
    }

    public InitData(tellerNo: string, grouplist?: string): boolean {
        if (!this.IsPowerCheckByUser) {
            return true;
        }
        this._ruleInfoList = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.GetPowerByUserCode(
            tellerNo,
            grouplist
        );
        return true;
    }

    public GetTradeMenuList(): TT_RuleInfo[] {
        return this._ruleInfoList.filter((x) => x.RuleType === "1" || x.RuleType === "0");
    }

    public GetTradePowerByCode(tradeCode: string): TT_RuleInfo[] {
        const trade = this._ruleInfoList.find((x) => x.RuleNo === tradeCode && x.RuleType === "1");
        if (!trade) {
            return [];
        }
        return this._ruleInfoList.filter((x) => x.RulePId === trade.RuleId && x.RuleType === "2");
    }

    private GetRuleFromUser(userCode: string, userPsw: string): string[] | null {
        if (!userCode) {
            return null;
        }
        // 构造 SQL 语句
        let strSql = `
            SELECT RULEID,RULEPID,RULENO,RULENAME,RULEREMARK,RULETYPE,EXTID,RULECOMMENT FROM TT_RULEINFO WHERE RULEID IN ( 
            SELECT GROUPRULE.RULEID 
              FROM (SELECT DISTINCT TT_KINBRRULE.RULEID 
                      FROM TT_LOGINUSER, TT_KINBRRULE 
                     WHERE TT_LOGINUSER.KINBRID = TT_KINBRRULE.KINBRID 
                       AND TT_LOGINUSER.USERCODE = '${userCode}' ) KINBRRULE,        
                   (SELECT DISTINCT RULEGET.RULEID 
                      FROM (SELECT TT_GROUPRULE.RULEID 
                              FROM TT_LOGINUSER, TT_USERROLE, TT_ROLEGROUP, TT_GROUPRULE 
                             WHERE TT_LOGINUSER.USERID = TT_USERROLE.USERID 
                               AND TT_USERROLE.GROUPID = TT_ROLEGROUP.GROUPID 
                               AND TT_ROLEGROUP.GROUPID = TT_GROUPRULE.GROUPID 
                               AND TT_GROUPRULE.DELSIGN = '0' 
                               AND TT_LOGINUSER.USERCODE = '${userCode}' ) RULEGET 
                     WHERE NOT EXISTS 
                     (SELECT RULEID 
                              FROM (SELECT TT_GROUPRULE.RULEID 
                                      FROM TT_LOGINUSER, 
                                           TT_USERROLE, 
                                           TT_ROLEGROUP, 
                                           TT_GROUPRULE 
                                     WHERE TT_LOGINUSER.USERID = TT_USERROLE.USERID 
                                       AND TT_USERROLE.GROUPID = TT_ROLEGROUP.GROUPID 
                                       AND TT_ROLEGROUP.GROUPID = TT_GROUPRULE.GROUPID 
                                       AND TT_GROUPRULE.DELSIGN = '1' 
                                       AND TT_LOGINUSER.USERCODE = '${userCode}' ) RULEOUT 
                     WHERE RULEGET.RULEID = RULEOUT.RULEID)) GROUPRULE 
              WHERE KINBRRULE.RULEID = GROUPRULE.RULEID 
            )
        `;
        try {
            return UserManagerHandle.ExecuteQuery(strSql);
        } catch (error) {
            // 异常处理逻辑可根据实际情况补充
            return null;
        }
    }
}

export { TradePowerManager, TT_RuleInfo };