// 模拟依赖的类和类型
class Message {
    AnilyzePrintFile(format: string): { [key: string]: string }[] {
        // 这里需要实现具体的解析逻辑
        return [];
    }
}

class CXCZBankbook {
    private static instance: CXCZBankbook;
    FirstPageData: CXCZFirstPageEntity | null = null;
    EndPageData: CXCZEndPageEntity | null = null;
    PageData: CXCZPageEntity[] = [];

    static GetInstance(): CXCZBankbook {
        if (!this.instance) {
            this.instance = new CXCZBankbook();
        }
        return this.instance;
    }

    PrintFirstPage(): PrintContainer[] {
        // 这里需要实现具体的打印逻辑
        return [];
    }

    PrintPage(): PrintContainer[] {
        // 这里需要实现具体的打印逻辑
        return [];
    }

    PrintEndPage(): PrintContainer[] {
        // 这里需要实现具体的打印逻辑
        return [];
    }
}

class DQCZBankbook {
    private static instance: DQCZBankbook;
    FirstPageData: DQCZFirstPageEntity | null = null;
    SubAcTitleList: DQCZSubAcTitleEntity[] = [];

    static GetInstance(): DQCZBankbook {
        if (!this.instance) {
            this.instance = new DQCZBankbook();
        }
        return this.instance;
    }

    PrintDQCZ(): PrintContainer[] {
        // 这里需要实现具体的打印逻辑
        return [];
    }
}

class CXCZFirstPageEntity {
    AcNo: string = "";
    AcName: string = "";
    CZNum: string = "";
    CurNo: string = "";
    DepositType: string = "";
    DrawMode: string = "";
    OpnBrname: string = "";
    OpnDate: string = "";
    PaySysCode: string = "";
}

class CXCZPageEntity {
    DateInfo: string = "";
    Description: string = "";
    Withdrawal: string = "";
    Deposit: string = "";
    Balance: string = "";
    TellerNo: string = "";
    Line: number = 0;
    IsVisibility: any; // 这里需要根据实际情况定义类型
}

class CXCZEndPageEntity {
    AcNo: string = "";
    AcName: string = "";
    XHDate: string = "";
    BenJin: string = "";
    LiXi: string = "";
    ShuiJin: string = "";
    BenXiHeJi: string = "";
    Teller: string = "";
}

class DQCZFirstPageEntity {
    AcNo: string = "";
    AcName: string = "";
    DrawMode: string = "";
    OpnBr: string = "";
    OpnDate: string = "";
    Teller: string = "";
    CzNum: string = "";
}

class DQCZSubAcTitleEntity {
    AcAmt: string = "";
    AcName: string = "";
    AcNoSeqn: number = 0;
    CurNo: string = "";
    PrintAcSeqn: number = 0;
    DualDate: string = "";
    TxDate: string = "";
    PrintLine: number = 0;
    AutoDeposit: string = "";
    Rate: string = "";
    Term: string = "";
    SubAcData: DQCZSubAcEntity[] = [];
    ChangePageStr: string = "";
    PrintPageNo: number = 0;

    GetNullData(): DQCZSubAcEntity[] {
        // 这里需要实现具体的初始化逻辑
        return [];
    }
}

class DQCZSubAcEntity {
    Amt: string = "";
    Anotherbr: string = "";
    BegDate: string = "";
    CurNo: string = "";
    DepoistType: string = "";
    DespInfo: string = "";
    RateAmt: string = "";
    Seqn: string = "";
    Teller: string = "";
    Term: string = "";
    TxDate: string = "";
    PrintNo: number = 0;
    IsVisibility: any; // 这里需要根据实际情况定义类型
}

class PrintContainer {}

class LoginUserInfo {
    static TradeDate: string = "";
    static TellerNo: string = "";
}

class SystemFunction {
    static Comma(value: string, length: number): string {
        // 这里需要实现具体的逻辑
        return value;
    }
}

// 扩展字符串类型以模拟 ToDecimal 方法
declare global {
    interface String {
        ToDecimal(): number;
    }
}

String.prototype.ToDecimal = function () {
    return parseFloat(this.toString());
};

class BankBookHelper {
    static PrintCXCZFirstPage(msg: Message): PrintContainer[] {
        const aCxczBankbook = CXCZBankbook.GetInstance();
        this.AnalyzeCXCZData(aCxczBankbook, msg, "FirstPage");
        return aCxczBankbook.PrintFirstPage();
    }

    static PrintCXCZFirstPage_CZFY(msg: Message): PrintContainer[] {
        const aCxczBankbook = CXCZBankbook.GetInstance();
        this.AnalyzeCXCZData(aCxczBankbook, msg, "Page");
        return aCxczBankbook.PrintFirstPage();
    }

    static PrintGHJZFirstPage(msg: Message): PrintContainer[] {
        const aCxczBankbook = CXCZBankbook.GetInstance();
        this.AnalyzeCXCZData(aCxczBankbook, msg, "GHJZ");
        return aCxczBankbook.PrintFirstPage();
    }

    static PrintCXCZPage(msg: Message): PrintContainer[] {
        const aCxczBankbook = CXCZBankbook.GetInstance();
        this.AnalyzeCXCZData(aCxczBankbook, msg, "Page");
        return aCxczBankbook.PrintPage();
    }

    static PrintEndPage(msg: Message): PrintContainer[] {
        const aCxczBankbook = CXCZBankbook.GetInstance();
        this.AnalyzeCXCZData(aCxczBankbook, msg, "EndPage");
        return aCxczBankbook.PrintEndPage();
    }

    static AnalyzeCXCZData(cxcz: CXCZBankbook, msg: Message, flag: string): boolean {
        cxcz.FirstPageData = null;
        cxcz.EndPageData = null;
        cxcz.PageData = [];

        if (flag === "FirstPage") {
            const opna = msg.AnilyzePrintFile(
                "OPNA|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|@term|@term_type|@rate|@tx_amt|@email|@zy|@deac_type|@cur_name|@tx_code|@memo|@ac_no1||@ct_name|@ac_seqn|@ic_date|@prt_line|@prdt_no||@brf|@tmp_ac_no|@tmp_tx_name|@tel|@id_name|@id_no|@br_name|@br_tele|@note_no||||@mbfebkno|@card_flag|@tx_br_name|@nxybrno"
            );
            if (opna.length !== 0) {
                let depositType = "";
                const firstPageData = new CXCZFirstPageEntity();

                if (opna[0]["@prdt_no"] === "101") {
                    depositType = "活期储蓄";
                }

                firstPageData.AcNo = opna[0]["@ac_no"];
                firstPageData.AcName = opna[0]["@name"];
                firstPageData.CZNum = opna[0]["@note_no"];
                firstPageData.CurNo = opna[0]["@cur_name"];
                firstPageData.DepositType = depositType;
                firstPageData.DrawMode = opna[0]["@zy"];
                firstPageData.OpnBrname = opna[0]["@br_name"];
                firstPageData.OpnDate = opna[0]["@opn_date"];
                firstPageData.PaySysCode = opna[0]["@mbfebkno"];
                firstPageData.DepositType = opna[0]["@title"];
                cxcz.FirstPageData = firstPageData;

                const pageData = new CXCZPageEntity();
                pageData.Balance = opna[0]["@tx_amt1"].ToDecimal().toFixed(2).padStart(12, " ");
                pageData.DateInfo = opna[0]["@opn_date"];
                pageData.Deposit = pageData.Balance;
                pageData.Description = opna[0]["@memo"];
                pageData.IsVisibility = true; // 这里需要根据实际情况设置
                pageData.Line = 0;
                pageData.TellerNo = opna[0]["@tel"];
                pageData.Withdrawal = "";
                cxcz.PageData.push(pageData);
                return true;
            }
        } else if (flag === "Page") {
            const czfy = msg.AnilyzePrintFile(
                "CZFY|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|||@rate|@tx_amt||@zy|@deac_type|@cur_name|@tx_code||@ac_no1|||@ac_seqn|@ic_date|@prt_line|@prdt_no|@page_no|@brf|@tmp_ac_no|@tmp_tx_name||||@br_name|@br_tele|@note_no||@accno|@zy|@mbfebkno||@shebaono|@tctd_flag|@txbrname||@tctd|"
            );
            if (czfy.length !== 0) {
                let depositType = "";
                const firstPageData = new CXCZFirstPageEntity();

                if (czfy[0]["@prdt_no"] === "101") {
                    depositType = "活期储蓄";
                }

                firstPageData.AcNo = czfy[0]["@ac_no"];
                firstPageData.AcName = czfy[0]["@name"];
                firstPageData.CZNum = czfy[0]["@note_no"];
                firstPageData.CurNo = czfy[0]["@cur_name"];
                firstPageData.DepositType = depositType;
                firstPageData.DrawMode = czfy[0]["@zy"];
                firstPageData.OpnBrname = czfy[0]["@br_name"];
                firstPageData.OpnDate = czfy[0]["@opn_date"];
                firstPageData.PaySysCode = czfy[0]["@mbfebkno"];
                firstPageData.DepositType = czfy[0]["@title"];
                cxcz.FirstPageData = firstPageData;

                const pageData = new CXCZPageEntity();
                pageData.Balance = czfy[0]["@tx_amt1"].ToDecimal().toFixed(2).padStart(12, " ");
                pageData.DateInfo = czfy[0]["@opn_date"];
                pageData.Deposit = pageData.Balance;
                pageData.Description = czfy[0]["@brf"];
                pageData.IsVisibility = true; // 这里需要根据实际情况设置
                pageData.Line = 0;
                pageData.TellerNo = czfy[0]["@br_tele"];
                pageData.Withdrawal = "";
                cxcz.PageData.push(pageData);
                return true;
            }

            const comPage = msg.AnilyzePrintFile(
                "CXCZ|@txday|@dscpt|@txamt|@dbflag|@avbal|@tlrno||@brno|@tty|@lineno||@txcode|@term|@rate|@ac_seqn|@srate|@sterm|@zc_flag"
            );
            if (comPage.length !== 0) {
                let tmpPageData: CXCZPageEntity;
                let ytxamt = "";
                let avbal = "";
                let curLine = 0;
                for (const alineData of comPage) {
                    tmpPageData = new CXCZPageEntity();
                    tmpPageData.DateInfo = alineData["@txday"];
                    tmpPageData.Description = alineData["@dscpt"];
                    if (alineData["@dbflag"] === "0") {
                        ytxamt = alineData["@txamt"].ToDecimal().toFixed(2).padStart(12, " ");
                        tmpPageData.Withdrawal = ytxamt;
                    } else {
                        ytxamt = alineData["@txamt"].ToDecimal().toFixed(2).padStart(12, " ");
                        tmpPageData.Deposit = ytxamt;
                    }
                    avbal = alineData["@avbal"].ToDecimal().toFixed(2).padStart(12, " ");
                    tmpPageData.Balance = avbal;
                    tmpPageData.TellerNo = alineData["@tlrno"];
                    curLine = parseInt(alineData["@lineno"], 10);
                    if (isNaN(curLine)) {
                        throw new Error("储蓄存折打印数据有问题！");
                    }
                    tmpPageData.Line = curLine;
                    tmpPageData.IsVisibility = true; // 这里需要根据实际情况设置
                    cxcz.PageData.push(tmpPageData);
                }
                return true;
            }
        } else if (flag === "GHJZ") {
            const czfy = msg.AnilyzePrintFile(
                "CZFY|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|||@rate|@tx_amt1||@zy|@deac_type|@cur_name|@tx_code||@ac_no1|||@ac_seqn|@ic_date|@prt_line|@prdt_no|@page_no|@brf|@tmp_ac_no|@tmp_tx_name||||@br_name|@br_tele|@note_no||@accno|@zy|@mbfebkno||@shebaono|@tctd_flag|@txbrname||@tctd|"
            );
            if (czfy.length !== 0) {
                let depositType = "";
                const firstPageData = new CXCZFirstPageEntity();

                if (czfy[0]["@prdt_no"] === "101") {
                    depositType = "活期储蓄";
                }

                firstPageData.AcNo = czfy[0]["@ac_no"];
                firstPageData.AcName = czfy[0]["@name"];
                firstPageData.CZNum = czfy[0]["@note_no"];
                firstPageData.CurNo = czfy[0]["@cur_name"];
                firstPageData.DepositType = depositType;
                firstPageData.DrawMode = czfy[0]["@zy"];
                firstPageData.OpnBrname = czfy[0]["@br_name"];
                firstPageData.OpnDate = czfy[0]["@opn_date"];
                firstPageData.PaySysCode = czfy[0]["@mbfebkno"];
                firstPageData.DepositType = czfy[0]["@title"];
                cxcz.FirstPageData = firstPageData;
            }

            const comPage = msg.AnilyzePrintFile(
                "CXCZ|@txday|@dscpt|@txamt|@dbflag|@avbal|@tlrno||@brno|@tty|@lineno||@txcode|@term|@rate|@ac_seqn|@srate|@sterm|@zc_flag"
            );
            if (comPage.length !== 0) {
                let tmpPageData: CXCZPageEntity;
                let avbal = "";
                let curLine = 0;
                for (const alineData of comPage) {
                    tmpPageData = new CXCZPageEntity();
                    tmpPageData.DateInfo = alineData["@txday"];
                    tmpPageData.Description = alineData["@dscpt"];
                    avbal = SystemFunction.Comma(alineData["@avbal"], 12);
                    tmpPageData.Balance = avbal;
                    tmpPageData.TellerNo = alineData["@tlrno"];
                    curLine = parseInt(alineData["@lineno"], 10);
                    if (isNaN(curLine)) {
                        throw new Error("储蓄存折打印数据有问题！");
                    }
                    tmpPageData.Line = curLine;
                    tmpPageData.IsVisibility = true; // 这里需要根据实际情况设置
                    cxcz.PageData.push(tmpPageData);
                }
                return true;
            }
        } else if (flag === "EndPage") {
            const LXQD = msg.AnilyzePrintFile(
                "LXQD|@ac_no|@name|@tx_amt|@title|@opn_date|@opn_br_no|@av_bal|@all_inter|@in_tax|@tax_rate|@tax_inter|@after_inter|@c_txamt|@zero_amt|@inst_type|@memo|@ac_seqn|@term||@brf|@ac_no1||@mtr_date||@rate|@prdt_no||||||"
            );
            if (LXQD.length !== 0) {
                const endPage = new CXCZEndPageEntity();

                endPage.AcNo = LXQD[0]["@ac_no"];
                endPage.AcName = LXQD[0]["@name"];
                endPage.XHDate = LoginUserInfo.TradeDate;
                endPage.BenJin = LXQD[0]["@tx_amt"];
                endPage.LiXi = LXQD[0]["@all_inter"];
                endPage.ShuiJin = LXQD[0]["@in_tax"];
                endPage.BenXiHeJi = LXQD[0]["@c_txamt"];
                endPage.Teller = LoginUserInfo.TellerNo;

                cxcz.EndPageData = endPage;
            }
        }

        if (cxcz.FirstPageData === null && cxcz.EndPageData === null && cxcz.PageData.length === 0) {
            return false;
        }

        return true;
    }

    static AnalyzeDQCZData(dqcz: DQCZBankbook, msg: Message): boolean {
        dqcz.FirstPageData = null;
        dqcz.SubAcTitleList = [];

        const opna = msg.AnilyzePrintFile(
            "OPNA|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|@term|@term_type|@rate|@tx_amt1|@email|@zy|@deac_type|@cur_name|@tx_code|@memo|@ac_no1||@ct_name|@ac_seqn|@ic_date|@prt_line|@prdt_no|@page_no|@brf|@tmp_ac_no|@tmp_tx_name|@tel|@id_name|@id_no|@br_name|@br_tele|@note_no|@zdzc|||@mbfebkno|@card_flag|@tx_br_name|@nxybrno|@tctd_flag"
        );
        const YBTZ = msg.AnilyzePrintFile(
            "YBTZ|@ac_seqn|@tx_date|@brf|@tmp_term|@term_type|@mtr_date|@tx_amt|@bal|@rate|@tel|@prtl|@opn_br_no|@tty|@hst_cnt|@tx_code|@cz|@zdzc|@cur_no|"
        );
        const YBHY = msg.AnilyzePrintFile(
            "YBHY|@ac_seqn|@tx_date|@brf|@tmp_term|@term_type|@mtr_date|@tx_amt|@bal|@rate|@tel|@zdzc|@opn_br_no|@tty|@hst_cnt|@prt_page|@ac_name|@prdt_name|@prtl"
        );
        const CZFY = msg.AnilyzePrintFile(
            "CZFY|@ac_no|@name|@tx_amt1|@title|@opn_date|@opn_br_no|@cif_no|@end_date|||@rate|@tx_amt1||@zy|@deac_type|@cur_name|@tx_code||@ac_no1|||@ac_seqn|@ic_date|@prt_line|@prdt_no|@page_no|@brf|@tmp_ac_no|@tmp_tx_name||||@br_name|@br_tele|@note_no||@accno|@zy1|@mbfebkno||@shebaono|@tctd_flag|@txbrname||@tctd|"
        );
        const YBHZ = msg.AnilyzePrintFile(
            "YBHZ|@ac_seqn|@tx_date|@brf|@tmp_term|@term_type|@mtr_date|@tx_amt|@bal|@rate|@tel|@prtl|@opn_br_no|@tty|@hst_cnt|@tx_code|@cz|@zdzc|@cur_no|"
        );

        let czfYname = "";

        for (const item of opna) {
            if (item["@ac_seqn"] === "1") {
                const firstPageData = new DQCZFirstPageEntity();
                firstPageData.AcNo = item["@ac_no"];
                firstPageData.AcName = item["@name"];
                firstPageData.DrawMode = item["@zy"];
                firstPageData.OpnBr = item["@br_name"];
                firstPageData.OpnDate = item["@opn_date"];
                firstPageData.Teller = item["@tel"];
                firstPageData.CzNum = item["@note_no"];
                dqcz.FirstPageData = firstPageData;
            }

            const SubAcTitle = new DQCZSubAcTitleEntity();
            SubAcTitle.AcAmt = item["@tx_amt1"];
            SubAcTitle.AcName = item["@name"];
            SubAcTitle.AcNoSeqn = parseInt(item["@ac_seqn"], 10);
            SubAcTitle.CurNo = item["@cur_name"];
            SubAcTitle.PrintAcSeqn = parseInt(item["@page_no"], 10);
            SubAcTitle.DualDate = item["@end_date"];
            SubAcTitle.TxDate = item["@opn_date"];
            SubAcTitle.PrintLine = parseInt(item["@prt_line"], 10);

            if (item["@prdt_no"] !== "241") {
                SubAcTitle.AutoDeposit = "";
                SubAcTitle.Rate = "六折";
                SubAcTitle.Term = "";
            }

            SubAcTitle.SubAcData = SubAcTitle.GetNullData();

            if (YBTZ.length === 0 || YBTZ[0]["@tx_code"] !== "2323") {
                const dqczData = new DQCZSubAcEntity();
                dqczData.Amt = item["@tx_amt1"];
                dqczData.Anotherbr = item["@opn_br_no"];
                dqczData.BegDate = item["@ic_date"];
                dqczData.CurNo = item["@cur_name"];
                dqczData.DepoistType = item["@title"].includes("通知存款") ? "通知" : item["@title"].substring(0, 2);
                dqczData.DespInfo = item["@brf"];

                if (item["@prdt_no"] === "241") {
                    dqczData.RateAmt = "六折";
                } else {
                    dqczData.RateAmt = item["@rate"] + "%";
                }

                dqczData.Seqn = item["@ac_seqn"];
                dqczData.Teller = item["@tel"];
                dqczData.Term = item["@term"] + item["@term_type"];
                dqczData.TxDate = item["@opn_date"];
                dqczData.IsVisibility = true; // 这里需要根据实际情况设置

                SubAcTitle.SubAcData[0] = dqczData;
            }

            dqcz.SubAcTitleList.push(SubAcTitle);
        }

        for (const item of CZFY) {
            const firstPageData = new DQCZFirstPageEntity();
            firstPageData.AcNo = item["@ac_no"];
            firstPageData.AcName = item["@name"];
            firstPageData.CzNum = item["@note_no"];
            firstPageData.DrawMode = item["@zy"];
            firstPageData.OpnBr = item["@br_name"];
            firstPageData.OpnDate = item["@opn_date"];
            firstPageData.Teller = item["@br_tele"];
            dqcz.FirstPageData = firstPageData;
            czfYname = item["@name"];
        }

        for (const item of YBHZ) {
            const dqczData = new DQCZSubAcEntity();
            dqczData.Amt = item["@tx_amt"];
            dqczData.Anotherbr = item["@opn_br_no"];
            dqczData.BegDate = item["@tx_date"];
            dqczData.CurNo = item["@cur_no"];
            dqczData.DepoistType = item["@cz"].includes("通知存款") ? "通知" : item["@cz"].substring(0, 2);
            dqczData.DespInfo = item["@brf"];
            dqczData.RateAmt = item["@rate"] + "%";
            dqczData.Seqn = item["@ac_seqn"];
            dqczData.Teller = item["@tel"];
            dqczData.Term = item["@tmp_term"] + item["@term_type"];
            dqczData.TxDate = item["@tx_date"];

            if (item["@cz"] === "定活两便存款") {
                dqczData.RateAmt = "六折";
                dqczData.Term = "";
            }

            const prtl = parseInt(item["@prtl"], 10);
            dqczData.PrintNo = prtl % 7 === 0 ? 7 : prtl % 7;
            dqczData.IsVisibility = true; // 这里需要根据实际情况设置

            const subAcTitle = dqcz.SubAcTitleList.filter(
                (p) => p.AcNoSeqn === parseInt(item["@ac_seqn"], 10)
            );
            if (subAcTitle.length === 0) {
                const subAcTitleEntity = new DQCZSubAcTitleEntity();
                subAcTitleEntity.PrintAcSeqn = parseInt(item["@ac_seqn"], 10);
                subAcTitleEntity.SubAcData = subAcTitleEntity.GetNullData();
                subAcTitleEntity.SubAcData[dqczData.PrintNo - 1] = dqczData;
                subAcTitleEntity.AcName = czfYname;
                subAcTitleEntity.AcNoSeqn = parseInt(item["@ac_seqn"], 10);
                subAcTitleEntity.CurNo = item["@cur_no"];
                subAcTitleEntity.AcAmt = item["@bal"];
                subAcTitleEntity.AutoDeposit = item["@zdzc"];
                subAcTitleEntity.TxDate = item["@tx_date"];
                subAcTitleEntity.DualDate = item["@mtr_date"];
                subAcTitleEntity.Term = item["@tmp_term"] + item["@term_type"];
                subAcTitleEntity.Rate = item["@rate"] + "%";
                subAcTitleEntity.PrintLine = parseInt(item["@prtl"], 10);

                if (item["@cz"] === "定活两便存款") {
                    subAcTitleEntity.Rate = "六折";
                    subAcTitleEntity.AutoDeposit = "";
                    subAcTitleEntity.DualDate = "";
                }

                dqcz.SubAcTitleList.push(subAcTitleEntity);
            } else if (subAcTitle.length === 1) {
                subAcTitle[0].SubAcData[dqczData.PrintNo - 1] = dqczData;
                subAcTitle[0].PrintLine = parseInt(item["@prtl"], 10);
            } else {
                for (const dqczSubAcTitleEntity of subAcTitle.filter(
                    (dqczSubAcTitleEntity) =>
                        dqczSubAcTitleEntity.PrintPageNo === Math.floor((parseInt(item["@prtl"], 10) - 1) / 7) + 1
                )) {
                    dqczSubAcTitleEntity.SubAcData[dqczData.PrintNo - 1] = dqczData;
                    dqczSubAcTitleEntity.PrintLine = parseInt(item["@prtl"], 10);
                }
            }
        }

        for (const item of YBHY) {
            const subAcTitle = dqcz.SubAcTitleList.filter(
                (p) => p.AcNoSeqn === parseInt(item["@ac_seqn"], 10)
            );
            if (subAcTitle.length !== 0) {
                if (subAcTitle.length === 1) {
                    subAcTitle[0].PrintLine = parseInt(item["@prtl"], 10);
                    subAcTitle[0].ChangePageStr = ` 转至第[${item["@prt_page"]}]页`;
                } else {
                    for (const dqczSubAcTitleEntity of subAcTitle.filter(
                        (dqczSubAcTitleEntity) =>
                            dqczSubAcTitleEntity.PrintPageNo === Math.floor((parseInt(item["@prtl"], 10) - 1) / 7) + 1
                    )) {
                        dqczSubAcTitleEntity.ChangePageStr = ` 转至第[${item["@prt_page"]}]页`;
                        dqczSubAcTitleEntity.PrintLine = parseInt(item["@prtl"], 10);
                    }
                }
            } else {
                const subAcTitleEntity = new DQCZSubAcTitleEntity();
                subAcTitleEntity.PrintLine = parseInt(item["@prtl"], 10) - 1;
                subAcTitleEntity.PrintAcSeqn = parseInt(item["@ac_seqn"], 10);
                subAcTitleEntity.SubAcData = subAcTitleEntity.GetNullData();
                subAcTitleEntity.ChangePageStr = ` 转至第[${item["@prt_page"]}]页`;
                dqcz.SubAcTitleList.push(subAcTitleEntity);
            }

            const subAcTitleEntity2 = new DQCZSubAcTitleEntity();
            subAcTitleEntity2.AcName = item["@ac_name"];
            subAcTitleEntity2.AcNoSeqn = parseInt(item["@ac_seqn"], 10);
            subAcTitleEntity2.AcAmt = item["@tx_amt"];
            subAcTitleEntity2.AutoDeposit = item["@zdzc"];
            subAcTitleEntity2.TxDate = item["@tx_date"];
            subAcTitleEntity2.DualDate = item["@mtr_date"];
            subAcTitleEntity2.Term = item["@tmp_term"] + item["@term_type"];
            subAcTitleEntity2.Rate = item["@rate"] + "%";
            subAcTitleEntity2.PrintLine = (parseInt(item["@prt_page"], 10) - 1) * 7 + 1;
            subAcTitleEntity2.SubAcData = subAcTitleEntity2.GetNullData();
            dqcz.SubAcTitleList.push(subAcTitleEntity2);
        }

        for (const item of YBTZ) {
            const dqczData = new DQCZSubAcEntity();
            dqczData.Amt = item["@tx_amt"];
            dqczData.Anotherbr = item["@opn_br_no"];
            dqczData.BegDate = item["@tx_date"];
            dqczData.CurNo = item["@cur_no"];
            dqczData.DepoistType = item["@cz"].includes("通知存款") ? "通知" : item["@cz"].substring(0, 2);
            dqczData.DespInfo = item["@brf"];

            const LXQD = msg.AnilyzePrintFile(
                "LXQD|@ac_no|@name|@tx_amt|@title|@opn_date|@opn_br_no|@av_bal|@all_inter|@in_tax|@tax_rate|@tax_inter|@after_inter|@c_txamt|@zero_amt|@inst_type|@memo|@ac_seqn|@term||@brf|@ac_no1||@mtr_date||@rate|@prdt_no||||||"
            );
            if (LXQD.length > 0 && !item["@brf"].includes("余额")) {
                dqczData.RateAmt = LXQD[0]["@all_inter"];
            }

            dqczData.Seqn = item["@ac_seqn"];
            dqczData.Teller = item["@tel"];
            dqczData.Term = item["@tmp_term"] + item["@term_type"];
            dqczData.TxDate = item["@tx_date"];
            const prtl = parseInt(item["@prtl"], 10);
            dqczData.PrintNo = prtl % 7 === 0 ? 7 : prtl % 7;
            dqczData.IsVisibility = true; // 这里需要根据实际情况设置

            const subAcTitle = dqcz.SubAcTitleList.filter(
                (p) => p.AcNoSeqn === parseInt(item["@ac_seqn"], 10)
            );
            if (subAcTitle.length === 0) {
                const subAcTitleEntity = new DQCZSubAcTitleEntity();
                subAcTitleEntity.PrintAcSeqn = parseInt(item["@ac_seqn"], 10);
                subAcTitleEntity.SubAcData = subAcTitleEntity.GetNullData();
                subAcTitleEntity.SubAcData[dqczData.PrintNo - 1] = dqczData;
                subAcTitleEntity.PrintLine = parseInt(item["@prtl"], 10);
                dqcz.SubAcTitleList.push(subAcTitleEntity);
            } else if (subAcTitle.length === 1) {
                subAcTitle[0].SubAcData[dqczData.PrintNo - 1] = dqczData;
                subAcTitle[0].PrintLine = parseInt(item["@prtl"], 10);
            } else {
                for (const dqczSubAcTitleEntity of subAcTitle.filter(
                    (dqczSubAcTitleEntity) =>
                        dqczSubAcTitleEntity.PrintPageNo === Math.floor((parseInt(item["@prtl"], 10) - 1) / 7) + 1
                )) {
                    dqczSubAcTitleEntity.SubAcData[dqczData.PrintNo - 1] = dqczData;
                    dqczSubAcTitleEntity.PrintLine = parseInt(item["@prtl"], 10);
                }
            }
        }

        dqcz.SubAcTitleList = dqcz.SubAcTitleList.sort((a, b) => a.PrintPageNo - b.PrintPageNo);
        return true;
    }

    static PrintDQCZ(msg: Message): PrintContainer[] {
        const aDQCZBankbook = DQCZBankbook.GetInstance();
        this.AnalyzeDQCZData(aDQCZBankbook, msg);
        return aDQCZBankbook.PrintDQCZ();
    }
}

export { BankBookHelper };