﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using Platform.Common.RunningParameters;
using Platform.Presentation.Interfaces;

namespace TellerSystem.Library.Ext.TradeExtension
{
    /// <summary>
    /// 总线数据实体
    /// </summary>
    public class TradeBusData
    {
        #region 基本存储方案
        /*************************************
         * 内置字典，且字典仅接受string和ISerializable这2种类型的数据，并通过key前缀区分
         * 提供序列化和反序列化方法
         * ***********************************/
        IDictionary<string, object> tradeBusData;
        public TradeBusData()
        {
            tradeBusData = new Dictionary<string, object>();
        }

        /// <summary>
        /// 清理掉所有的数据项
        /// </summary>
        public void Clear()
        {
            tradeBusData.Clear();
        }

        /// <summary>
        /// 获取或设置总线数据信息
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public string this[string key]
        {
            get
            {
                //key = "String_" + key;
                if (tradeBusData.ContainsKey(key))
                    return tradeBusData[key].ToString();
                return string.Empty;
            }
            set
            {
                //key = "String_" + key;
                if (tradeBusData.ContainsKey(key))
                    tradeBusData[key] = value;
                else
                    tradeBusData.Add(key, value);
            }
        }

        /// <summary>
        /// 获取复杂总线数据
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <returns></returns>
        public T GetDataItem<T>(string key) where T : ISerializable
        {
            key = "Serialize_" + key;
            var result = default(T);
            if (tradeBusData.ContainsKey(key))
                result = (T)tradeBusData[key];
            return result;
        }

        /// <summary>
        /// 设置复杂总线数据
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <param name="value"></param>
        public void SetDataItem<T>(string key, T value) where T : ISerializable
        {
            key = "Serialize_" + key;
            if (tradeBusData.ContainsKey(key))
                tradeBusData[key] = value;
            else
                tradeBusData.Add(key, value);
        }

        /// <summary>
        /// 序列化
        /// </summary>
        /// <returns></returns>
        public IDictionary<string, string> Serialize()
        {
            var result = new Dictionary<string, string>();
            //组织输出数据
            foreach (var item in tradeBusData)
            {
                if (item.Key.StartsWith("Serialize_"))
                {
                    using (var stream = new MemoryStream())
                    {
                        formater.Serialize(stream, item.Value);
                        result.Add(item.Key, PlatformSettings.Encoding.GetString(stream.ToArray()));
                    }
                }
                else
                {
                    result.Add(item.Key, item.Value.ToString());
                }
            }
            return result;
        }

        /// <summary>
        /// 反序列化
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public bool Deserialize(IDictionary<string, string> data)
        {
            if (data == null) return false;
            //格式化类，还原操作
            tradeBusData.Clear();
            foreach (var item in data)
            {
                if (item.Key.StartsWith("Serialize_"))
                {
                    using (var stream = new MemoryStream(PlatformSettings.Encoding.GetBytes(item.Value)))
                    {
                        tradeBusData.Add(item.Key, formater.Deserialize(stream));
                    }
                }
                else
                {
                    tradeBusData.Add(item.Key, item.Value);
                }
            }
            return true;
        }

        /// <summary>
        /// 是否包含指定键的元素
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public bool ContainsKey(string key)
        {
            return tradeBusData.ContainsKey(key);
        }

        /// <summary>
        /// 序列化对象
        /// </summary>
        private BinaryFormatter formater = new BinaryFormatter();
        #endregion

        #region 常用总线数据

        #endregion
    }

    /**************************************
     * 序列化数据演示
     * ************************************/
    [Serializable]
    public class DemoItem : System.Runtime.Serialization.ISerializable
    {
        public string A { get; set; }

        public string B { get; set; }

        /// <summary>
        /// 序列化
        /// </summary>
        /// <param name="info"></param>
        /// <param name="context"></param>
        public void GetObjectData(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context)
        {
            info.AddValue("A", A);
            info.AddValue("B", B);
            //throw new NotImplementedException();
        }

        public DemoItem()
        {
        }

        /// <summary>
        /// 反序列化
        /// </summary>
        /// <param name="info"></param>
        /// <param name="context"></param>
        protected DemoItem(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context)
        {
            A = info.GetString("A");
            B = info.GetString("B");
        }
    }
}
