// 模拟 ISerializable 接口
interface ISerializable {
    serialize(): string;
    static deserialize(data: string): any;
}

class TradeBusData {
    private tradeBusData: { [key: string]: any } = {};

    constructor() {
        this.tradeBusData = {};
    }

    public Clear(): void {
        this.tradeBusData = {};
    }

    public get(key: string): string {
        if (this.tradeBusData[key] !== undefined) {
            return String(this.tradeBusData[key]);
        }
        return "";
    }

    public set(key: string, value: string): void {
        this.tradeBusData[key] = value;
    }

    public GetDataItem<T extends ISerializable>(key: string): T | null {
        const newKey = "Serialize_" + key;
        if (this.tradeBusData[newKey] !== undefined) {
            return this.tradeBusData[newKey] as T;
        }
        return null;
    }

    public SetDataItem<T extends ISerializable>(key: string, value: T): void {
        const newKey = "Serialize_" + key;
        this.tradeBusData[newKey] = value;
    }

    public Serialize(): { [key: string]: string } {
        const result: { [key: string]: string } = {};
        for (const key in this.tradeBusData) {
            if (key.startsWith("Serialize_")) {
                const value = this.tradeBusData[key] as ISerializable;
                result[key] = value.serialize();
            } else {
                result[key] = String(this.tradeBusData[key]);
            }
        }
        return result;
    }

    public Deserialize(data: { [key: string]: string }): boolean {
        if (!data) {
            return false;
        }
        this.tradeBusData = {};
        for (const key in data) {
            if (key.startsWith("Serialize_")) {
                const type = this.getDataType(key);
                if (type && type.deserialize) {
                    this.tradeBusData[key] = type.deserialize(data[key]);
                }
            } else {
                this.tradeBusData[key] = data[key];
            }
        }
        return true;
    }

    private getDataType(key: string): { deserialize: (data: string) => any } | null {
        // 这里需要根据实际情况实现类型查找逻辑
        // 示例中假设所有 Serialize_ 开头的键对应 DemoItem 类型
        return DemoItem;
    }

    public ContainsKey(key: string): boolean {
        return this.tradeBusData[key] !== undefined;
    }
}

class DemoItem implements ISerializable {
    public A: string = "";
    public B: string = "";

    constructor(A?: string, B?: string) {
        if (A) this.A = A;
        if (B) this.B = B;
    }

    public serialize(): string {
        return JSON.stringify({ A: this.A, B: this.B });
    }

    public static deserialize(data: string): DemoItem {
        const obj = JSON.parse(data);
        return new DemoItem(obj.A, obj.B);
    }
}

export { TradeBusData, DemoItem };