﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Platform.Presentation.Interfaces;
using Platform.Common.BasicFunctions;
using System.Windows;
using Platform.Presentation.PagesHelper;
using Platform.Presentation.PageFunctions;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class TradeInfoData
    {
        #region 基础
        #region 附加到IPage对象
        public static TradeInfoData GetTradeInfo(DependencyObject obj)
        {
            return (TradeInfoData)obj.GetValue(TradeInfoProperty);
        }

        public static void SetTradeInfo(DependencyObject obj, TradeInfoData value)
        {
            obj.SetValue(TradeInfoProperty, value);
        }

        // Using a DependencyProperty as the backing store for TradeInfo.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TradeInfoProperty =
            DependencyProperty.RegisterAttached("TradeInfo", typeof(TradeInfoData), typeof(TradeInfoData), new UIPropertyMetadata(null));
        #endregion
        #endregion
        public TradeInfoData()
        {
            SerialNumber = string.Empty;
            SystemSerialNumber = string.Empty;
            TradeCode = string.Empty;
            TradeName = string.Empty;
            TradeBus = new TradeBusData();
        }
        #region 交易信息属性
        /// <summary>
        /// 柜员前端流水号
        /// </summary>
        public string SerialNumber
        {
            get;
            set;
        }
        /// <summary>
        /// 系统前端流水号
        /// </summary>
        public string SystemSerialNumber
        {
            get;
            internal set;
        }

        /// <summary>
        /// 打印票号
        /// </summary>
        public string PrintNum
        {
            get;
            internal set;
        }
        /// <summary>
        /// 交易号，指代一个完整的业务流程
        /// </summary>
        public string TradeCode
        {
            get;
            internal set;
        }
        /// <summary>
        /// 交易名称，指代一个完整的业务流程
        /// </summary>
        public string TradeName
        {
            get;
            internal set;
        }
        /// <summary>
        /// 总线数据集合
        /// </summary>
        public TradeBusData TradeBus
        {
            get;
            internal set;
        }
        #endregion

        #region 公共方法
        /// <summary>
        /// 检查交易是否拥有制定的权限编号
        /// </summary>
        /// <param name="powerCode"></param>
        /// <returns></returns>
        public bool CheckTradePower(string powerCode)
        {
            var list = TradePowerManager.Instance.GetTradePowerByCode(TradeCode);
            if (list == null || list.Count == 0) return false;
            return list.Any(x => x.RuleNo == powerCode);
        }
        #endregion
    }
}
