// 模拟 DependencyObject 类
class DependencyObject {
    private properties: { [key: string]: any } = {};

    getValue(propertyName: string): any {
        return this.properties[propertyName];
    }

    setValue(propertyName: string, value: any): void {
        this.properties[propertyName] = value;
    }
}

// 模拟 UIPropertyMetadata 类
class UIPropertyMetadata {
    constructor(public defaultValue: any) {}
}

// 模拟 TradePowerManager 类
class TradePowerManager {
    static Instance = new TradePowerManager();

    GetTradePowerByCode(tradeCode: string): { RuleNo: string }[] {
        // 这里需要根据实际情况实现获取权限列表的逻辑
        return [];
    }
}

// 模拟 TradeBusData 类
class TradeBusData {}

class TradeInfoData {
    // 静态属性，模拟依赖属性
    static readonly TradeInfoProperty = {
        name: "TradeInfo",
        type: TradeInfoData,
        metadata: new UIPropertyMetadata(null)
    };

    // 静态方法，获取附加的 TradeInfoData 实例
    static GetTradeInfo(obj: DependencyObject): TradeInfoData {
        return obj.getValue(this.TradeInfoProperty.name) as TradeInfoData;
    }

    // 静态方法，设置附加的 TradeInfoData 实例
    static SetTradeInfo(obj: DependencyObject, value: TradeInfoData): void {
        obj.setValue(this.TradeInfoProperty.name, value);
    }

    // 交易信息属性
    SerialNumber: string;
    SystemSerialNumber: string;
    PrintNum: string;
    TradeCode: string;
    TradeName: string;
    TradeBus: TradeBusData;

    constructor() {
        this.SerialNumber = "";
        this.SystemSerialNumber = "";
        this.PrintNum = "";
        this.TradeCode = "";
        this.TradeName = "";
        this.TradeBus = new TradeBusData();
    }

    // 检查交易是否拥有指定的权限编号
    CheckTradePower(powerCode: string): boolean {
        const list = TradePowerManager.Instance.GetTradePowerByCode(this.TradeCode);
        if (!list || list.length === 0) {
            return false;
        }
        return list.some(item => item.RuleNo === powerCode);
    }
}

export { TradeInfoData };