// 模拟 IPage 接口
interface IPage {}

// 模拟 TradeInfoData 类
class TradeInfoData {
    static GetTradeInfo(page: any) {
        return {
            TradeBus: new TradeBusData()
        };
    }
}

// 模拟 TradeBusData 类
class TradeBusData {
    private data: { [key: string]: string } = {};

    get(key: string): string {
        return this.data[key] || "";
    }

    set(key: string, value: string): void {
        this.data[key] = value;
    }

    remove(key: string): void {
        if (this.data.hasOwnProperty(key)) {
            delete this.data[key];
        }
    }
}

// 定义 WorkFlowDataHelper 类
class WorkFlowDataHelper {
    private _tradeBusData: TradeBusData;

    constructor(page: IPage | null) {
        if (page) {
            this._tradeBusData = TradeInfoData.GetTradeInfo(page).TradeBus;
        } else {
            this._tradeBusData = new TradeBusData();
        }
    }

    get tradeBusData(): TradeBusData {
        return this._tradeBusData;
    }

    get IsWorkFlow(): string {
        return this._tradeBusData.get("IsWorkFlow");
    }

    set IsWorkFlow(value: string) {
        this._tradeBusData.set("IsWorkFlow", value);
    }

    get ProcessId(): string {
        return this._tradeBusData.get("ProcessId");
    }

    set ProcessId(value: string) {
        this._tradeBusData.set("ProcessId", value);
    }

    get CheckFlag(): string {
        return this._tradeBusData.get("CheckFlag");
    }

    set CheckFlag(value: string) {
        this._tradeBusData.set("CheckFlag", value);
    }

    get ImageId(): string {
        return this._tradeBusData.get("ImageId");
    }

    set ImageId(value: string) {
        this._tradeBusData.set("ImageId", value);
    }

    get WorkItemId(): string {
        return this._tradeBusData.get("WorkItemId");
    }

    set WorkItemId(value: string) {
        this._tradeBusData.set("WorkItemId", value);
    }

    get Lvl(): string {
        return this._tradeBusData.get("lvl");
    }

    set Lvl(value: string) {
        this._tradeBusData.set("lvl", value);
    }

    get Branch(): string {
        return this._tradeBusData.get("Branch");
    }

    set Branch(value: string) {
        this._tradeBusData.set("Branch", value);
    }

    ClearWorkFlowData(): void {
        const keysToRemove = [
            "IsWorkFlow",
            "ProcessId",
            "CheckFlag",
            "ImageId",
            "WorkItemId",
            "lvl",
            "Branch"
        ];
        keysToRemove.forEach(key => {
            this._tradeBusData.remove(key);
        });
    }
}

export { WorkFlowDataHelper };