// 模拟 DependencyObject 类
class DependencyObject {
    private properties: { [key: string]: any } = {};

    getValue(property: string): any {
        return this.properties[property];
    }

    setValue(property: string, value: any): void {
        this.properties[property] = value;
    }
}

// 模拟 UIPropertyMetadata 类
class UIPropertyMetadata {
    constructor(public defaultValue: any) {}
}

// 模拟 DependencyProperty 类
class DependencyProperty {
    constructor(
        public name: string,
        public type: any,
        public ownerType: any,
        public metadata: UIPropertyMetadata
    ) {}

    static register(name: string, type: any, ownerType: any, metadata: UIPropertyMetadata): DependencyProperty {
        return new DependencyProperty(name, type, ownerType, metadata);
    }

    static registerAttached(name: string, type: any, ownerType: any, metadata: UIPropertyMetadata): DependencyProperty {
        return new DependencyProperty(name, type, ownerType, metadata);
    }
}

// ExamObject 类
export class ExamObject extends DependencyObject {
    // 定义依赖属性
    public static readonly IsExamModelProperty: DependencyProperty =
        DependencyProperty.register("IsExamModel", Boolean, ExamObject, new UIPropertyMetadata(false));
    public static readonly QuestionKeyProperty: DependencyProperty =
        DependencyProperty.register("QuestionKey", String, ExamObject, new UIPropertyMetadata(""));
    public static readonly QuestionNameProperty: DependencyProperty =
        DependencyProperty.register("QuestionName", String, ExamObject, new UIPropertyMetadata(""));
    public static readonly TimeNowProperty: DependencyProperty =
        DependencyProperty.register("TimeNow", String, ExamObject, new UIPropertyMetadata(""));
    public static readonly StartTimeProperty: DependencyProperty =
        DependencyProperty.register("StartTime", String, ExamObject, new UIPropertyMetadata(""));
    public static readonly ExamLastProperty: DependencyProperty =
        DependencyProperty.register("ExamLast", String, ExamObject, new UIPropertyMetadata(""));
    public static readonly LeftTotalSecondsProperty: DependencyProperty =
        DependencyProperty.register("LeftTotalSeconds", String, ExamObject, new UIPropertyMetadata(""));

    // 定义附加属性
    public static readonly ExamObjectProperty: DependencyProperty =
        DependencyProperty.registerAttached("ExamObject", ExamObject, ExamObject, new UIPropertyMetadata(null));

    // 属性包装器
    get IsExamModel(): boolean {
        return this.getValue(ExamObject.IsExamModelProperty.name) as boolean;
    }

    set IsExamModel(value: boolean) {
        this.setValue(ExamObject.IsExamModelProperty.name, value);
    }

    get QuestionKey(): string {
        return this.getValue(ExamObject.QuestionKeyProperty.name) as string;
    }

    set QuestionKey(value: string) {
        this.setValue(ExamObject.QuestionKeyProperty.name, value);
    }

    get QuestionName(): string {
        return this.getValue(ExamObject.QuestionNameProperty.name) as string;
    }

    set QuestionName(value: string) {
        this.setValue(ExamObject.QuestionNameProperty.name, value);
    }

    get TimeNow(): string {
        return this.getValue(ExamObject.TimeNowProperty.name) as string;
    }

    set TimeNow(value: string) {
        this.setValue(ExamObject.TimeNowProperty.name, value);
    }

    get StartTime(): string {
        return this.getValue(ExamObject.StartTimeProperty.name) as string;
    }

    set StartTime(value: string) {
        this.setValue(ExamObject.StartTimeProperty.name, value);
    }

    get ExamLast(): string {
        return this.getValue(ExamObject.ExamLastProperty.name) as string;
    }

    set ExamLast(value: string) {
        this.setValue(ExamObject.ExamLastProperty.name, value);
    }

    get LeftTotalSeconds(): string {
        return this.getValue(ExamObject.LeftTotalSecondsProperty.name) as string;
    }

    set LeftTotalSeconds(value: string) {
        this.setValue(ExamObject.LeftTotalSecondsProperty.name, value);
    }

    // 附加属性的获取和设置方法
    static GetExamObject(obj: DependencyObject): ExamObject | null {
        return obj.getValue(ExamObject.ExamObjectProperty.name) as ExamObject;
    }

    static SetExamObject(obj: DependencyObject, value: ExamObject): void {
        obj.setValue(ExamObject.ExamObjectProperty.name, value);
    }
}