﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Platform.Presentation.Interfaces;

namespace TellerSystem.Library.Ext.TradeInterfaces
{
    /// <summary>
    /// 主页面对外接口
    /// </summary>
    public interface IMainPage
    {
        /// <summary>
        ///  打开新的Tab页交易
        /// </summary>
        /// <param name="tabTrade">Ipage对象</param>
        /// <returns>布尔型</returns>
        bool OpenTabPage(IPage tabTrade);

        /// <summary>
        /// 打开交易
        /// </summary>
        /// <param name="page">Ipage对象</param>
        /// <returns>布尔型</returns>
        bool OpenTrade(IPage page);

        /// <summary>
        /// 打开交易码指定的交易对象
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        bool OpenNewTrade(string code);

        /// <summary>
        /// 关闭标签页
        /// </summary>
        void CloseTabItem();

        /// <summary>
        /// 获取当前交易的页面
        /// </summary>
        /// <returns></returns>
        string GetTradeName();

        /// <summary>
        /// 获取隐藏交易界面的数量
        /// </summary>
        /// <returns></returns>
        int GetHideTradeCount();

        /// <summary>
        /// 隐藏page所在的Tab页，返回指定还原码
        /// </summary>
        /// <param name="page">Ipage对象</param>
        /// <returns></returns>
        Guid HideTab(IPage page);

        /// <summary>
        /// 呈现guid指定的隐藏Tab页
        /// </summary>
        /// <param name="guid"></param>
        /// <returns></returns>
        IPage ShowTab(Guid guid);

        /// <summary>
        /// 刷新任务数目
        /// </summary>
        void RefreshTaskListCount();

        /// <summary>
        /// 刷新常用交易菜单
        /// </summary>
        void ReloadMenuTree();

        /// <summary>
        /// 显示当前交易数据信息
        /// </summary>
        void ShowCurrentTradeDataInfo();

        /// <summary>
        /// 隐藏当前交易数据信息
        /// </summary>
        void HideCurrentTradeDataInfo();

        /// <summary>
        /// 添加当前交易数据信息
        /// </summary>
        void AddCurrentTradeDataInfo();

        /// <summary>
        /// 删除当前交易数据信息
        /// </summary>
        void RemoveCurrentTradeDataInfo();
    }
}
