// 模拟 IPage 类型
interface IPage {}

// 定义 IMainPage 接口
interface IMainPage {
    /**
     * 打开新的 Tab 页交易
     * @param tabTrade IPage 对象
     * @returns 布尔型，操作是否成功
     */
    OpenTabPage(tabTrade: IPage): boolean;

    /**
     * 打开交易
     * @param page IPage 对象
     * @returns 布尔型，操作是否成功
     */
    OpenTrade(page: IPage): boolean;

    /**
     * 打开交易码指定的交易对象
     * @param code 交易码
     * @returns 布尔型，操作是否成功
     */
    OpenNewTrade(code: string): boolean;

    /**
     * 关闭标签页
     */
    CloseTabItem(): void;

    /**
     * 获取当前交易的页面
     * @returns 当前交易页面名称
     */
    GetTradeName(): string;

    /**
     * 获取隐藏交易界面的数量
     * @returns 隐藏交易界面的数量
     */
    GetHideTradeCount(): number;

    /**
     * 隐藏 page 所在的 Tab 页，返回指定还原码
     * @param page IPage 对象
     * @returns 用于还原隐藏 Tab 页的 Guid
     */
    HideTab(page: IPage): string;

    /**
     * 呈现 guid 指定的隐藏 Tab 页
     * @param guid 用于还原隐藏 Tab 页的 Guid
     * @returns 被呈现的 IPage 对象
     */
    ShowTab(guid: string): IPage;

    /**
     * 刷新任务数目
     */
    RefreshTaskListCount(): void;

    /**
     * 刷新常用交易菜单
     */
    ReloadMenuTree(): void;

    /**
     * 显示当前交易数据信息
     */
    ShowCurrentTradeDataInfo(): void;

    /**
     * 隐藏当前交易数据信息
     */
    HideCurrentTradeDataInfo(): void;

    /**
     * 添加当前交易数据信息
     */
    AddCurrentTradeDataInfo(): void;

    /**
     * 删除当前交易数据信息
     */
    RemoveCurrentTradeDataInfo(): void;
}

export { IMainPage, IPage };