// 引入必要的类型
import { promises as fsPromises } from 'fs';
import { parse, Document, Element } from 'xml2js';

// 定义 OftenUseTrade 类
class OftenUseTrade {
    // 定义静态字段
    static openNum: { [key: string]: string } = {};
    static relativeDir: string;
    static dir: string = "Config";
    static fileName: string = "OpenTradeInfo.xml";
    static config: { fullName: string };

    // 加载数据的方法
    static async LoadData(): Promise<void> {
        // 假设 LoginUserInfo 是一个全局对象，这里简单模拟
        OftenUseTrade.relativeDir = 'tellerNo'; // 模拟 LoginUserInfo.TellerNo
        const path = `${OftenUseTrade.dir}/${OftenUseTrade.relativeDir}/${OftenUseTrade.fileName}`;
        OftenUseTrade.config = { fullName: path };

        // 清空 openNum
        OftenUseTrade.openNum = {};
        // 添加当前时间
        OftenUseTrade.openNum["DateTime"] = new Date().toString();

        try {
            // 检查文件是否存在
            await fsPromises.access(OftenUseTrade.config.fullName);
            const data = await fsPromises.readFile(OftenUseTrade.config.fullName, 'utf8');
            const doc: Document = await new Promise((resolve, reject) => {
                parse(data, (err, result) => {
                    if (err) {
                        reject(err);
                    } else {
                        resolve(result);
                    }
                });
            });
            const elementCollection = doc.root.ele as Element[];
            if (elementCollection) {
                for (const element of elementCollection) {
                    const elementKey = element.$.key;
                    const elementValue = element.$.value;
                    if (elementKey) {
                        if (OftenUseTrade.openNum[elementKey]) {
                            OftenUseTrade.openNum[elementKey] = elementValue;
                        } else {
                            OftenUseTrade.openNum[elementKey] = elementValue;
                        }
                    }
                }
            }
        } catch (error) {
            // 文件不存在或读取失败，这里可以添加日志记录等操作
            console.log('文件读取失败或不存在:', error);
        }
    }

    // 保存数据的方法
    static async SaveData(): Promise<void> {
        if (OftenUseTrade.config) {
            // 移除 DateTime 字段
            delete OftenUseTrade.openNum["DateTime"];

            let xmlContent = '<?xml version="1.0" encoding="utf-8"?>\n<root>\n';
            for (const [key, value] of Object.entries(OftenUseTrade.openNum)) {
                xmlContent += `  <ele key="${key}" value="${value}" />\n`;
            }
            xmlContent += '</root>';

            try {
                // 确保目录存在
                await fsPromises.mkdir(`${OftenUseTrade.dir}/${OftenUseTrade.relativeDir}`, { recursive: true });
                // 保存文件
                await fsPromises.writeFile(OftenUseTrade.config.fullName, xmlContent, 'utf8');
            } catch (error) {
                // 保存失败，这里可以添加日志记录等操作
                console.log('文件保存失败:', error);
            }
        }
    }

    // 获取配置值的方法
    static GetConfigValue(key: string): string {
        let ret = '';
        if (key) {
            if (OftenUseTrade.openNum[key]) {
                ret = OftenUseTrade.openNum[key];
            }
        }
        return ret;
    }

    // 设置配置值的方法
    static SetConfigValue(key: string, value: string): boolean {
        let ret = false;
        if (key) {
            if (OftenUseTrade.openNum[key]) {
                OftenUseTrade.openNum[key] = value;
            } else {
                OftenUseTrade.openNum[key] = value;
            }
            ret = true;
        }
        return ret;
    }
}

export default OftenUseTrade;