﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;

namespace TellerSystem.PrintTemplate.Base
{
    public static class CanvasHelper
    {
        #region Property
        #region IsGridLayout
        public static bool GetIsGridLayout(DependencyObject obj)
        {
            return (bool)obj.GetValue(IsGridLayoutProperty);
        }

        public static void SetIsGridLayout(DependencyObject obj, bool value)
        {
            obj.SetValue(IsGridLayoutProperty, value);
        }

        /// <summary>
        /// 是否使用Grid布局
        /// </summary>
        public static readonly DependencyProperty IsGridLayoutProperty =
            DependencyProperty.RegisterAttached("IsGridLayout", typeof(bool), typeof(CanvasHelper), new UIPropertyMetadata(false));
        #endregion

        #region InternalGrid
        public static Grid GetInternalGrid(Canvas obj)
        {
            return (Grid)obj.GetValue(InternalGridProperty);
        }

        public static void SetInternalGrid(Canvas obj, Grid value)
        {
            obj.SetValue(InternalGridProperty, value);
        }

        /// <summary>
        /// 内部的Grid要素
        /// </summary>
        private static readonly DependencyProperty InternalGridProperty =
            DependencyProperty.RegisterAttached("InternalGrid", typeof(Grid), typeof(CanvasHelper), new UIPropertyMetadata(null));
        #endregion

        #region Offset
        public static Point GetOffset(Canvas obj)
        {
            return (Point)obj.GetValue(OffsetProperty);
        }

        public static void SetOffset(Canvas obj, Point value)
        {
            obj.SetValue(OffsetProperty, value);
        }

        /// <summary>
        /// 起始位置偏移
        /// </summary>
        public static readonly DependencyProperty OffsetProperty =
            DependencyProperty.RegisterAttached("Offset", typeof(Point), typeof(CanvasHelper), new UIPropertyMetadata(new Point(0, 0)));
        #endregion
        #endregion

        #region Method
        /// <summary>
        /// 刷新布局
        /// </summary>
        /// <param name="canvas"></param>
        public static void Refresh(this Canvas canvas)
        {
            if (canvas == null) return;
            //擒拿参数
            var grid = GetInternalGrid(canvas);
            if (grid == null) return;
            var offset = GetOffset(canvas);
            if (canvas.Children.Contains(grid)) canvas.Children.Remove(grid);
            //grid.Children.Clear();
            //整理布局
            var items = canvas.Children.Cast<UIElement>().ToList();
            foreach (var item in items)
            {
                if (GetIsGridLayout(item))
                {
                    var index = canvas.Children.IndexOf(item);
                    canvas.Children.Remove(item);
                    canvas.Children.Insert(index, new UIElement());
                    grid.Children.Add(item);
                }
            }
            Canvas.SetLeft(grid, offset.X);
            Canvas.SetTop(grid, offset.Y);
            canvas.Children.Add(grid);
        }
        #endregion
    }
}
