﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using Platform.Common.RunningParameters;
using Platform.Common.BasicFunctions;
using Platform.Presentation.Interfaces;
using TellerSystem.Communication;
using TellerSystem.Communication.MessageHelper;
using Platform.Print;
using TellerSystem.PrintTemplate.Extension;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.PrintTemplate.Base
{
    /// <summary>
    /// E_Seal.xaml 的交互逻辑
    /// </summary>
    public partial class E_Seal
    {

        #region static
        private static Brush _brush = Brushes.Transparent;
        /// <summary>
        /// 是否开启电子印章
        /// </summary>
        public static bool HasE_Seal
        {
            get
            {
                if (_brush == Brushes.Transparent)
                {
                    //按照资源加载印章画刷
                    _brush = RES.GetBrush("ESeal/" + LoginUserInfo.KinbrName + ".png");
                }
                return _brush != null;
            }
        }
        #endregion

        /// <summary>
        /// 印章
        /// </summary>
        public E_Seal(string keyCode, bool local = true)
        {
            InitializeComponent();
            if (DesignerProperties.GetIsInDesignMode(this)) return;
            tb_Code.Text = string.Empty;
            //当前环境无电子印章配置，则不处理
            if (local && !HasE_Seal) return;
            tb_Code.Text = keyCode;
            if (local)
                this.Loaded += (sender, e) =>
                {
                    if (bd_seal.Background != null) return;
                    //加载时，默认从父对象中找PrintContainer.PrintAttributes
                    var p = this.FindVisualTreeAncestor(x => x is PrintContainer) as PrintContainer;
                    if (p != null)
                        Print(PrintContainerExt.GetPrintElement(p));
                };
            string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            if (flag == "V13" || flag == "V12" || flag == "V10" || flag == "V18" || flag == "V16" || flag == "V11" || flag == "V21" || flag == "V22" || flag == "V14" || flag == "V5" || flag == "V17")
            {
                if (flag == "V13")
                {
                    Thickness thick = new Thickness(10, 30, 10, 35);
                    tb_Code.Margin = thick;
                    tb_Code.SetValue(ForegroundProperty, Brushes.Red);
                }
                else if (flag == "V12")
                {
                    Thickness thick = new Thickness(10, 25, 10, 40);
                    tb_Code.Margin = thick;
                    Panel.Width = 172;
                    Panel.Height = 122;
                }
                else if (flag == "V10")
                {
                    Thickness thick = new Thickness(10, 30, 10, 35);
                    tb_Code.Margin = thick;
                    tb_Code.FontSize = 10;
                }
                else if (flag == "V18")
                {
                    Thickness thick = new Thickness(10, 10, 10, 45);
                    tb_Code.Margin = thick;
                    tb_Code.FontSize = 14;
                    tb_Code.SetValue(ForegroundProperty, Brushes.Red);
                    Panel.Width = 172;
                    Panel.Height = 122;
                }
                else if (flag == "V16")
                {
                    Thickness thick = new Thickness(10, 30, 10, 26);
                    tb_Code.Margin = thick;
                    tb_Code.FontSize = 11;
                }
                else if (flag == "V11")
                {
                    Thickness thick = new Thickness(10, 25, 10, 40);
                    tb_Code.Margin = thick;
                    //tb_Code.SetValue(ForegroundProperty, Brushes.Red);
                    Panel.Width = 172;
                    Panel.Height = 122;
                }
                else if (flag == "V21")
                {
                    Thickness thick = new Thickness(10, 30, 10, 32);
                    tb_Code.Margin = thick;
                    tb_Code.FontSize = 12;
                    //tb_Code.SetValue(ForegroundProperty, Brushes.Red);
                }
                else if (flag == "V22")
                {
                    Thickness thick = new Thickness(10, 10, 10, 55);
                    tb_Code.Margin = thick;
                    tb_Code.FontSize = 14;
                    tb_Code.SetValue(ForegroundProperty, Brushes.Red);
                    Panel.Width = 172;
                    Panel.Height = 122;
                }
                else if (flag == "V5")
                {
                    //Thickness thick = new Thickness(10, 30, 10, 26);
                    // tb_Code.Margin = thick;
                    tb_Code.FontSize = 10;
                }
                else if (flag == "V14")
                {
                    Thickness thick = new Thickness(10, 30, 10, 35);
                    tb_Code.Margin = thick;
                    tb_Code.FontSize = 10;
                    tb_Code.SetValue(ForegroundProperty, Brushes.Red);
                }
                else if (flag == "V17")
                {
                    Thickness thick = new Thickness(10, 10, 10, 45);
                    tb_Code.Margin = thick;
                    tb_Code.FontSize = 14;
                    tb_Code.SetValue(ForegroundProperty, Brushes.Red);
                    Panel.Width = 172;
                    Panel.Height = 122;
                }
            }

        }

        private double fontSize = -1;

        /// <summary>
        /// 显示标志
        /// </summary>
        public bool Display
        {
            get
            {
                return tb_Code.FontSize > 1;
            }
            set
            {
                if (Display == value) return;
                if (!value) fontSize = tb_Code.FontSize;
                tb_Code.FontSize = value ? fontSize : 1;
                tb_Code.Width = value ? double.NaN : 1;
            }
        }

        /// <summary>
        /// 加载印章
        /// </summary>
        /// <param name="code"></param>
        private void Print(PrintAttributes printAttributes)
        {
            //if (string.IsNullOrEmpty(business)) return;
            if (printAttributes == null) return;
            //加载印章信息
            bd_seal.Background = _brush;
            string flag = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            if (flag == "V21" || flag == "V22")
            {
                tb_Code.Text = LoginUserInfo.TradeDate.Substring(0, 4) + "." + LoginUserInfo.TradeDate.Substring(4, 2) + "." + LoginUserInfo.TradeDate.Substring(6, 2);
            }
            else
            {
                var index = tb_Code.Text.IndexOf('-');
                if (index > 0)
                    tb_Code.Text = tb_Code.Text.Substring(0, index);
            }
        }
    }
}
