﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Data;
using System.Windows;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using System.IO;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using EnvDTE;
using System.Runtime.InteropServices.ComTypes;
using System.Runtime.InteropServices;
using Platform.Common.RunningParameters;

namespace TellerSystem.PrintTemplate.Base
{
    public static class RES
    {
        static RES()
        {
            Converter = new RESConverter();
        }

        #region Converter
        /// <summary>
        /// 资源转换器
        /// </summary>
        public static IValueConverter Converter
        {
            get;
            private set;
        }

        class RESConverter : IValueConverter
        {
            public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
            {
                if (parameter == null) throw new ArgumentNullException("RESConverter：必须指定ConverterParameter！");
                object o = null;
                if (targetType.FullName.Contains(".ImageSource"))
                    o = RES.GetImageSource(parameter.ToString());
                if (targetType.FullName.Contains(".Brush"))
                    o = RES.GetBrush(parameter.ToString());
                return o ?? value;
            }

            public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
            {
                throw new NotImplementedException();
            }
        }
        #endregion

        #region EnvDte
        static class EnvDteHelper
        {
            private static DirectoryInfo _develop;

            /// <summary>
            /// 获取Develop目录
            /// </summary>
            public static DirectoryInfo Develop
            {
                get
                {
                    if (_develop == null)
                    {
                        var dtes = GetAllInstances();
                        foreach (var dte in dtes)
                        {
                            //判断dte
                            if (dte.Solution.Properties.Item("Name").Value == "TellerSystem")
                            {
                                var dir = new FileInfo(dte.Solution.FullName).Directory;
                                if (dir.Name == "App")
                                    dir = new DirectoryInfo(Path.Combine(dir.FullName, "../Develop"));
                                else if (dir.Name != "Develop")
                                    dir = null;
                                if (dir != null && dir.Exists)
                                {
                                    _develop = dir;
                                    break;
                                }
                            }
                        }
                    }
                    return _develop;
                }
            }

            private static IEnumerable<DTE> GetAllInstances()
            {
                IRunningObjectTable rot;
                IEnumMoniker enumMoniker;
                int retVal = GetRunningObjectTable(0, out rot);

                if (retVal == 0)
                {
                    rot.EnumRunning(out enumMoniker);

                    IntPtr fetched = IntPtr.Zero;
                    IMoniker[] moniker = new IMoniker[1];
                    object punkObject = null;
                    while (enumMoniker.Next(1, moniker, fetched) == 0)
                    {
                        IBindCtx bindCtx;
                        CreateBindCtx(0, out bindCtx);
                        string displayName;
                        moniker[0].GetDisplayName(bindCtx, null, out displayName);
                        // Console.WriteLine("Display Name: {0}", displayName);
                        bool isVisualStudio = displayName.StartsWith("!VisualStudio");
                        if (isVisualStudio)
                        {
                            //var dte = rot.GetObject(moniker) as DTE;
                            //yield return dte;
                            rot.GetObject(moniker[0], out punkObject);
                            var dte = (DTE)(punkObject);
                            yield return dte;
                        }
                    }
                }
            }

            [DllImport("ole32.dll")]
            private static extern void CreateBindCtx(int reserved, out IBindCtx ppbc);

            [DllImport("ole32.dll")]
            private static extern int GetRunningObjectTable(int reserved, out IRunningObjectTable prot);
        }
        #endregion

        private static string _version = null;
        /// <summary>
        /// 获取资源版本号
        /// </summary>
        /// <returns></returns>
        private static string GetVersion()
        {
            if (_version != null) return _version;
            //检查版本标志,优先检查system配置文件,若不存在则采用版本号
            _version = ConfigManager.GetInstance().GetConfigValue("ResVersion", ConfigType.System);
            if (string.IsNullOrEmpty(_version))
                _version = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
            return _version;
        }

        /// <summary>
        /// 获取资源引用
        /// </summary>
        /// <param name="path"></param>
        /// <param name="searchVersion"></param>
        /// <returns></returns>
        public static Uri GetUri(string path, bool searchVersion = true)
        {
            try
            {
                path = System.Text.RegularExpressions.Regex.Replace(path, "^(/RES/|RES/|/)", string.Empty);
                if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(new DependencyObject()))
                {
                    var develop = EnvDteHelper.Develop;
                    if (searchVersion)
                    {
                        //分析是否指定版本
                        var version = new FileInfo(Path.Combine(develop.FullName, "RES", "Version.txt"));
                        if (version.Exists)
                        {
                            var lines = File.ReadAllLines(version.FullName);
                            if (lines.Length > 0)
                            {
                                //优先查找对应版本
                                var res = Path.Combine(develop.FullName, "RES", lines[0], path);
                                if (File.Exists(res) || Directory.Exists(res))
                                    return new Uri(res, UriKind.Absolute);
                            }
                        }
                    }
                    //分析项目状态，找到对应的Develop目录下资源
                    return new Uri(Path.Combine(develop.FullName, "RES", path), UriKind.Absolute);
                }
                else
                {
                    Platform.Common.LogSystem.PlatformLogger.SystemInfo("RES.Uri执行:" + path);
                    if (searchVersion)
                    {
                        //优先检查指定版本
                        var res = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "RES", GetVersion(), path);
                        //若版本分类文件存在，则取版本对应资源
                        if (File.Exists(res) || Directory.Exists(res))
                            return new Uri(res, UriKind.Absolute);
                    }
                    return new Uri(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "RES", path), UriKind.Absolute);
                }
            }
            catch
            {
                //设计时登记日志会导致vs奔溃
                //Platform.Common.LogSystem.PlatformLogger.SystemErrorInfo("RES.Uri执行失败:" + path, ex);
                return null;
            }
        }

        /// <summary>
        /// 获取ImageSource对象
        /// </summary>
        /// <param name="path"></param>
        /// <param name="target"></param>
        /// <returns></returns>
        public static ImageSource GetImageSource(string path)
        {
            try
            {
                var uri = GetUri(path);
                //设计时采用内存资源，加载；防止文件占用导致的重新生成动作
                if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(new DependencyObject()))
                {
                    if (uri != null && File.Exists(uri.LocalPath))
                    {
                        var bi = new BitmapImage();
                        bi.BeginInit();
                        bi.StreamSource = new MemoryStream(File.ReadAllBytes(uri.LocalPath));
                        bi.EndInit();
                        return bi;
                    }
                }
                return new BitmapImage(uri);
            }
            catch
            {
                return null;
            }
        }

        /// <summary>
        /// 获取Brush对象
        /// </summary>
        /// <param name="path"></param>
        /// <param name="target"></param>
        /// <returns></returns>
        public static Brush GetBrush(string path)
        {
            var image = GetImageSource(path);
            if (image == null) return null;
            return new ImageBrush(image);
        }
    }
}
