﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Platform.Print;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Controls;
using TellerSystem.PrintTemplate.Extension;
using System.ComponentModel;
using Platform.Common.RunningParameters;
using System.IO;
using Platform.Common.BasicFunctions;
using Platform.Common.LogSystem;
using TellerSystem.Devices;
using TellerSystem.ServiceProxy.Ext;
using TellerSystem.PrintTemplate.ElecSeal;

namespace TellerSystem.PrintTemplate.Base
{
    public class TemplateBase : PrintContainer
    {
        #region ctor
        static TemplateBase()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(TemplateBase), new FrameworkPropertyMetadata(typeof(TemplateBase)));
        }
        //string flag = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
        public TemplateBase()
        {
            this.FontFamily = new FontFamily("SimSun");
            this.FontSize = 13;
            this.SignPoint = new List<Point>();
            this.CommonLayer = new Canvas();

            this.Loaded += TemplateBase_Loaded;
        }

        void TemplateBase_Loaded(object sender, RoutedEventArgs e)
        {
            this.Loaded -= TemplateBase_Loaded;//仅执行一次
            DrawingCommonLayer(PrintContainerExt.GetPrintElement(this));
            if (DesignerProperties.GetIsInDesignMode(this)) return;
            //填充印章
            if (E_Seal_Location.HasValue) this.ElecSealHelper.Sign2Point(E_Seal_Location.Value, E_Seal_Kind);
            //执行电子印章的绘制动作
            this.ElecSealHelper.CreateElecSealList(PrintContainerExt.GetPrintElement(this)).ToList().ForEach(x => this.DrawingUIElement(x.Key.X, x.Key.Y, x.Value));
            //通过页面检索签字区域，填充数据
            this.FindVisualTreeChildren(x => (x is TextBlock) && (x as TextBlock).Visibility == Visibility.Visible && (
                (x as TextBlock).Text.StartsWith("客户签字") || (x as TextBlock).Text.StartsWith("客户确认") || (x as TextBlock).Text.StartsWith("客户签名"))).Cast<TextBlock>()
                .ToList().ForEach(x => SignPoint.Add(x.TranslatePoint(new Point(70, 0), this)));
            //绘制印章编码区域
            if (PrintElecKeyCode.HasValue) this.DrawingUIElement(PrintElecKeyCode.Value.X, PrintElecKeyCode.Value.Y, new TextBlock
            {
                Name = "E_Seal_KeyCode",
                Text = "签章:" + this.ElecSealHelper.KeyCode,
                Visibility = Visibility.Collapsed,
            });
            //配置参数控制是否启用文本溢出缩放效果
            if (ServiceSettings.GetConfig("Print.ProcTextNoWrap", false))
                this.FindVisualTreeChildren(x => x is TextBlock).ForEach(x => PrintContainerExt.SetIsTextNoWrap2Scale(x, true));
        }

        /// <summary>
        /// 打印印章编码
        /// </summary>
        /// <param name="p"></param>
        protected Point? PrintElecKeyCode { get; set; }

        /// <summary>
        /// 采用防伪字体
        /// </summary>
        public bool IsNewFont
        {
            get { return (bool)GetValue(IsNewFontProperty); }
            set { SetValue(IsNewFontProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsNewFont.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsNewFontProperty =
            DependencyProperty.Register("IsNewFont", typeof(bool), typeof(TemplateBase), new UIPropertyMetadata(false,
                (d, e) =>
                {
                    string flag = TellerSystem.ServiceProxy.Ext.ServiceHelper.TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
                    var p = d as TemplateBase;
                    if (true.Equals(e.NewValue))
                    {

                        //固始开启防伪字体
                        if (flag == "V21")
                        {
                            var uri = RES.GetUri("Font/number.ttf");
                            if (uri != null)
                                //p.FontFamily = new System.Windows.Media.FontFamily(uri.LocalPath + "#number,Yu Gothic");
                                p.FontFamily = new System.Windows.Media.FontFamily(uri.LocalPath + "#number,Microsoft YaHei");
                        }
                    }
                    else
                    {
                        p.FontFamily = new FontFamily("SimSun");
                    }
                }));

        public override void OnApplyTemplate()
        {
            if (DesignerProperties.GetIsInDesignMode(this))
                this.LoadTemplate(this.PrintType);
            base.OnApplyTemplate();
            //_CommonLayer = GetTemplateChild("PART_CommonLayer") as Canvas;
        }
        #endregion

        #region Property
        /// <summary>
        /// 银行标签
        /// </summary>
        public string BankFlag
        {
            get
            {
                return ServiceSettings.BankFlag;
            }
        }

        /// <summary>
        /// 打印模板类型
        /// </summary>
        public PrintTypes PrintType
        {
            get { return (PrintTypes)GetValue(PrintTypeProperty); }

            set { SetValue(PrintTypeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for PrintType.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty PrintTypeProperty =
            DependencyProperty.Register("PrintType", typeof(PrintTypes), typeof(TemplateBase), new UIPropertyMetadata(PrintTypes.None, new PropertyChangedCallback(
                (sender, e) =>
                {
                    var tb = sender as TemplateBase;
                    if (tb != null)
                        tb.LoadTemplate((PrintTypes)e.NewValue);
                })));

        public string PrintTypeExt
        {
            get { return (string)GetValue(PrintTypeExtProperty); }
            set { SetValue(PrintTypeExtProperty, value); }
        }

        // Using a DependencyProperty as the backing store for PrintTypeExt.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty PrintTypeExtProperty =
            DependencyProperty.Register("PrintTypeExt", typeof(string), typeof(TemplateBase), new UIPropertyMetadata(string.Empty, new PropertyChangedCallback((d, e) =>
            {
                var tb = d as TemplateBase;
                if (tb != null)
                    tb.LoadTemplate((string)e.NewValue ?? string.Empty);
            })));

        public PrintAttributes PrintAttribute
        {
            get
            {
                return PrintContainerExt.GetPrintElement(this);
            }
        }
        #endregion

        #region CommonLayer
        /// <summary>
        /// 公共层
        /// </summary>
        public Canvas CommonLayer
        {
            get { return (Canvas)GetValue(CommonLayerProperty); }
            private set { SetValue(CommonLayerProperty, value); }
        }

        // Using a DependencyProperty as the backing store for CommonLayer.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty CommonLayerProperty =
            DependencyProperty.Register("CommonLayer", typeof(Canvas), typeof(TemplateBase), new UIPropertyMetadata(null));

        /// <summary>
        /// 执行公共层的绘制动作
        /// </summary>
        /// <param name="attr"></param>
        protected virtual void DrawingCommonLayer(PrintAttributes attr)
        {
            if (string.Compare(ConfigManager.GetInstance().GetConfigValue("InDeviceDebugMode", ConfigType.System), "true", true) == 0)
            {
                var TestFlag = new TextBlock { Text = "测试专用", FontSize = 30, Foreground = Brushes.Red };
                Canvas.SetLeft(TestFlag, 35.00);
                Canvas.SetTop(TestFlag, 35.00);
                //  CommonLayer.Children.Add(TestFlag);
            }
            //打印预览时，是否夹杂水印效果
            if (!DesignerProperties.GetIsInDesignMode(this) && ServiceSettings.GetConfig("Watermark2Print", false))
            {
                var watermark = new Grid { Width = 180, Height = 50 };
                watermark.Children.Add(new TextBlock
                {
                    HorizontalAlignment = HorizontalAlignment.Center,
                    VerticalAlignment = VerticalAlignment.Center,
                    Foreground = Brushes.Gray,
                    Text = "测试环境(" + LoginUserInfo.TellerNo + ")",
                    RenderTransform = new RotateTransform(-6)
                });
                CommonLayer.Background = new VisualBrush
                {
                    Stretch = Stretch.Uniform,
                    TileMode = TileMode.Tile,
                    Viewport = new Rect(0, 0, watermark.Width / CommonLayer.ActualWidth, watermark.Height / CommonLayer.ActualHeight),
                    Visual = watermark
                };
            }
        }

        /// <summary>
        /// 在公共层的指定位置绘制控件
        /// </summary>
        /// <param name="left"></param>
        /// <param name="top"></param>
        /// <param name="element"></param>
        public void DrawingUIElement(double left, double top, UIElement element)
        {
            if (element == null) return;
            //清理element已存在的Parent绑定
            var parent = LogicalTreeHelper.GetParent(element);
            if (parent != null)
            {
                if (parent is Panel) (parent as Panel).Children.Remove(element);
                if (parent is ContentControl) (parent as ContentControl).Content = null;
            }
            Canvas.SetLeft(element, left);
            Canvas.SetTop(element, top);
            if (!CommonLayer.Children.Contains(element))
                CommonLayer.Children.Add(element);
        }

        /// <summary>
        /// 在公共层的指定位置绘制文本
        /// </summary>
        /// <param name="left"></param>
        /// <param name="top"></param>
        /// <param name="text"></param>
        /// <param name="width"></param>
        public void DrawingText(double left, double top, string text, double width = double.NaN)
        {
            if (string.IsNullOrEmpty(text)) return;
            var element = new TextBlock { Text = text, Width = width };
            Canvas.SetLeft(element, left);
            Canvas.SetTop(element, top);
            CommonLayer.Children.Add(element);
        }
        /// <summary>
        /// 修改打印模板背景参数的配置
        /// </summary>
        /// <param name="pt"></param>
        /// <param name="brush"></param>
        public static void ChangePTArg(PrintTypes pt, Brush brush)
        {
            //var pta = PrintTypeArguments.Instance.GetPTArg(pt);
            //if (pta == null) return;
            //pta.PrintBackground = brush;
        }
        #endregion

        #region Method
        /// <summary>
        /// 加载模板配置
        /// </summary>
        /// <param name="pt"></param>
        public void LoadTemplate(PrintTypes pt)
        {
            if (pt == PrintTypes.None) return;
            LoadTemplate(pt.ToString());
        }

        public void LoadTemplate(string pt)
        {
            var pta = PrintTypeArguments.Instance.GetPTArg(pt);
            if (pta == null) return;

            this.Width = pta.Width;
            this.Height = pta.Height;
            this.Background = pta.PrintBackground;
        }
        #endregion

        #region E_Seal
        /// <summary>
        /// 电子签章助手
        /// </summary>
        public ElecSealHelper ElecSealHelper
        {
            get
            {
                var helper = ElecSealHelper.GetHelper(this);
                if (helper == null)
                {
                    helper = new ElecSealHelper();
                    ElecSealHelper.SetHelper(this, helper);
                }
                return helper;
            }
        }

        /// <summary>
        /// 电子印章位置
        /// </summary>
        public Point? E_Seal_Location { get; set; }

        /// <summary>
        /// 电子印象类型
        /// </summary>
        public SealKind E_Seal_Kind { get; set; }

        /// <summary>
        /// 复制印章到指定位置，默认编号1
        /// </summary>
        /// <param name="p"></param>
        [Obsolete("请使用新的API:ElecSealHelper.Sign2Point")]
        public void E_Seal2Copy(Point p)
        {
            //当默认电子印章位置存在时，才允许Copy
            if (E_Seal_Location.HasValue)
                this.ElecSealHelper.Sign2Point(p, E_Seal_Kind);
        }
        #endregion

        #region Special
        private Canvas _SpecialMarkCanvas;

        /// <summary>
        /// 绘制特殊的标签，会清空所有的打印效果
        /// </summary>
        /// <param name="left"></param>
        /// <param name="top"></param>
        /// <param name="element"></param>
        public void DrawingSpecialMark(double left, double top, UIElement element)
        {
            //隐藏公共层，必须加载之后才能擒拿到_CommonLayer
            CommonLayer.Visibility = Visibility.Collapsed;

            //替换Content
            if (_SpecialMarkCanvas == null)
                _SpecialMarkCanvas = new Canvas();
            if (element != null)
            {
                Canvas.SetLeft(element, left);
                Canvas.SetTop(element, top);
                _SpecialMarkCanvas.Children.Add(element);
            }
            if (this.Content != _SpecialMarkCanvas)
                this.Content = _SpecialMarkCanvas;
        }

        /// <summary>
        /// 绘制特殊的标签，会清空所有的打印效果
        /// </summary>
        /// <param name="left"></param>
        /// <param name="top"></param>
        /// <param name="element"></param>
        public void DrawingSpecialMark(double left, double top, string markText)
        {
            var border = new Border
            {
                Child = new TextBlock
                {
                    Text = markText,
                    FontSize = 20,
                    FontWeight = FontWeights.Bold,
                    VerticalAlignment = VerticalAlignment.Center,
                    HorizontalAlignment = HorizontalAlignment.Center
                },
                BorderBrush = Brushes.Black,
                BorderThickness = new Thickness(2),
                Padding = new Thickness(10, 5, 10, 5)
            };
            //旋转-5
            //border.RenderTransform = new RotateTransform(-10);
            DrawingSpecialMark(left, top, border);
        }
        #endregion

        #region ProcessXps
        /// <summary>
        /// 针对产生的xps文件进行再加工
        /// </summary>
        /// <param name="xpsFile"></param>
        public FileInfo ProcessXps(FileInfo xpsFile)
        {
            if (ServiceSettings.GetConfig("Temp_Sign2Customer", false))
            {
                try
                {
                    if (Sign2Customer(xpsFile))
                    {
                        //解除上层父节点关系，方便重新绘制
                        (this.Parent as ContentControl).Content = null;
                        //重新创建xps文件
                        return PrintToXps();
                    }
                }
                catch (Exception ex)
                {
                    PlatformLogger.SystemErrorInfo("TemplateBase:ProcessXps发生异常！", ex);
                }
            }
            return xpsFile;
        }

        /// <summary>
        /// 电子签名位置
        /// </summary>
        public List<Point> SignPoint { get; private set; }

        private bool Sign2Customer(FileInfo xps)
        {
            try
            {
                //检查是否需要进行签名
                if (SignPoint.Count > 0)
                {
                    //抓取当前有效界面进行ModalInvoke处理外设调用
                    var current = Application.Current.MainWindow;
                    foreach (Window win in Application.Current.Windows)
                    {
                        if (!win.IsVisible) continue;
                        if (win.OwnedWindows.Count > 0) continue;
                        if (win is Platform.Controls.TradeExtension.PageDialog)
                            current = win;
                    }
                    var p = current.FirstVisualTreeChild(x => x is Platform.Presentation.Interfaces.IPage) as Platform.Presentation.Interfaces.IPage;
                    var stream = new MemoryStream();
                    p.OwnerContainer.ModalInvoke(() =>
                    {
                        //准备pdf文件
                        var pdf = new FileInfo(xps.FullName + ".pdf");
                        File.WriteAllBytes(pdf.FullName, ElecSealHelper.XpsConvert(File.ReadAllBytes(xps.FullName)));
                        //调用电子签名设备
                        var ret = DeviceHelper.Signature("请针对当前办理的业务进行电子签名！", pdf);
                        if (!ret.HasError && !ret.Data.IsCancel) stream.Write(ret.Data.SignImage, 0, ret.Data.SignImage.Length);// data = ret.Data;
                        pdf.Delete();
                    }, "请客户使用电子签名设备进行签名...");

                    if (stream.Length > 0)
                    {
                        //加盖电子签名
                        SignPoint.ForEach(point =>
                                {
                                    //因为图片格式未知，通过转换器加载
                                    var convert = new ImageSourceConverter();
                                    var imageSource = convert.ConvertFrom(stream) as ImageSource;
                                    var sign = new Image { Source = imageSource, Height = 50 };
                                    this.DrawingUIElement(point.X, point.Y, sign);
                                });
                        return true;
                    }
                }
            }
            catch (Exception ex)
            {
                PlatformLogger.SystemErrorInfo("TemplateBase:Sign2Customer发生异常！", ex);
            }
            return false;
        }
        #endregion
    }
}
