// 模拟 C# 中的 System 命名空间下的部分功能
class DateTime {
    static Now(): Date {
        return new Date();
    }
    static format(date: Date, format: string): string {
        const year = date.getFullYear();
        const month = String(date.getMonth() + 1).padStart(2, '0');
        const day = String(date.getDate()).padStart(2, '0');
        const hours = String(date.getHours()).padStart(2, '0');
        const minutes = String(date.getMinutes()).padStart(2, '0');
        const seconds = String(date.getSeconds()).padStart(2, '0');
        const milliseconds = String(date.getMilliseconds()).padStart(3, '0');

        return format.replace('yyyy', String(year))
            .replace('MM', month)
            .replace('dd', day)
            .replace('HH', hours)
            .replace('mm', minutes)
            .replace('ss', seconds)
            .replace('fff', milliseconds);
    }
}

// 模拟随机数生成
class Random {
    next(min: number, max: number): number {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    }
}

// 模拟日志系统
class PlatformLogger {
    static SystemInfo(message: string): void {
        console.log(message);
    }
}

// 模拟文件读取
async function readFile(fileName: string): Promise<Uint8Array> {
    try {
        const response = await fetch(fileName);
        if (!response.ok) {
            throw new Error('文件读取失败');
        }
        return new Uint8Array(await response.arrayBuffer());
    } catch (error) {
        return new Uint8Array(0);
    }
}

// 模拟文件写入
async function writeFile(pReadByte: Uint8Array, fileName: string): Promise<boolean> {
    try {
        // 浏览器环境无法直接写入文件，这里仅作模拟
        console.log(`模拟写入文件: ${fileName}, 数据长度: ${pReadByte.length}`);
        return true;
    } catch (error) {
        return false;
    }
}

// 模拟流读取
class Stream {
    // 模拟读取数据
    async read(data: Uint8Array, offset: number, length: number): Promise<number> {
        // 这里只是简单模拟，实际需要根据流的来源实现
        return 0;
    }
}

// 模拟哈希计算
class HashAlgorithm {
    static create(algorithm: string): HashAlgorithm {
        return new HashAlgorithm();
    }
    computeHash(data: Uint8Array): Uint8Array {
        // 简单模拟，实际需要实现具体的哈希算法
        return new Uint8Array(0);
    }
}

// 模拟 Base64 编码
function convertToBase64(data: Uint8Array): string {
    return btoa(String.fromCharCode(...data));
}

// 模拟 MultiDataBean 类
class MultiDataBean {
    getMultiData(): Uint8Array {
        return new Uint8Array(0);
    }
    getType(): string {
        return '';
    }
    getFileName(): string {
        return '';
    }
    getDescr(): string {
        return '';
    }
}

// 模拟 SealUserInfo 类
class SealUserInfo {
    getUserName(): string {
        return '';
    }
    getSealLocation(): string {
        return '';
    }
    getSealReason(): string {
        return '';
    }
    getIdentificationType(): string {
        return '';
    }
    getIdentificationNo(): string {
        return '';
    }
    getPhoneNo(): string {
        return '';
    }
}

// 模拟 SignLocation 类
class SignLocation {
    getType(): string {
        return '';
    }
    getPage(): number {
        return 0;
    }
    getLx(): number {
        return 0;
    }
    getLy(): number {
        return 0;
    }
    getKeyword(): string {
        return '';
    }
    getLocationStyle(): string {
        return '';
    }
    getOffsetX(): number {
        return 0;
    }
    getOffsetY(): number {
        return 0;
    }
    getPdfIndex(): number {
        return 0;
    }
}

// 模拟 PdfBean 类
class PdfBean {
    getPdf(): string {
        return '';
    }
    getSavedPdfId(): string {
        return '';
    }
    getBizSerialNo(): string {
        return '';
    }
    getBizTypeCode(): string {
        return '';
    }
    getOutputFilePath(): string {
        return '';
    }
    getReserved1(): string {
        return '';
    }
    getReserved2(): string {
        return '';
    }
    getReserved3(): string {
        return '';
    }
    getPdfId(): string {
        return '';
    }
}

// 模拟 SealLogBean 类
class SealLogBean {
    SealCode: string = '';
    SealType: string = '';
    FunctionType: string = '';
    BizSerialNo: string = '';
    BizTypeCode: string = '';
    FileHash: string = '';
    TimestampUsed: string = '';
    SealPerson: string = '';
    SealReason: string = '';
    SealLocation: string = '';
    Remark1: string = '';
    Remark2: string = '';
    Remark3: string = '';
}

// 模拟 FieldBean 类
class FieldBean {
    getFieldId(): string {
        return '';
    }
    getFieldName(): string {
        return '';
    }
    getFieldValue(): string {
        return '';
    }
    getFieldValueHash(): string {
        return '';
    }
    getFieldType(): string {
        return '';
    }
    getTemplateIndex(): number {
        return 0;
    }
}

// 模拟 TextBean 类
class TextBean {
    getTextValue(): string {
        return '';
    }
    getTextValueHash(): string {
        return '';
    }
    getTextFontSize(): number {
        return 0;
    }
    getTextColor(): string {
        return '';
    }
    getType(): string {
        return '';
    }
    getPageNo(): number {
        return 0;
    }
    getLx(): number {
        return 0;
    }
    getLy(): number {
        return 0;
    }
    getKeyword(): string {
        return '';
    }
    getLocationStyle(): string {
        return '';
    }
    getOffsetX(): number {
        return 0;
    }
    getOffsetY(): number {
        return 0;
    }
    getTemplateIndex(): number {
        return 0;
    }
}

// 模拟 ImageBean 类
class ImageBean {
    getValue(): string {
        return '';
    }
    getPageNo(): number {
        return 0;
    }
    getLx(): number {
        return 0;
    }
    getLy(): number {
        return 0;
    }
    getType(): string {
        return '';
    }
    getKeyword(): string {
        return '';
    }
    getLocationStyle(): string {
        return '';
    }
    getOffsetX(): number {
        return 0;
    }
    getOffsetY(): number {
        return 0;
    }
    getImageWidth(): number {
        return 0;
    }
    getImageHeight(): number {
        return 0;
    }
    getTemplateIndex(): number {
        return 0;
    }
}

// 模拟 QRCodeBean 类
class QRCodeBean {
    getQrCodeContent(): string {
        return '';
    }
    getWidth(): number {
        return 0;
    }
    getHeight(): number {
        return 0;
    }
    getPageNo(): number {
        return 0;
    }
    getLx(): number {
        return 0;
    }
    getLy(): number {
        return 0;
    }
    getTemplateIndex(): number {
        return 0;
    }
}

// 模拟 ImageLocationBean 类
class ImageLocationBean {
    Image: any;
    Lx: number;
    Ly: number;
    Page: number;
    TemplateIndex: number;
    Type: string;
    Keyword: string;
    LocationStyle: string;
    OffsetX: number;
    OffsetY: number;

    constructor(image: any, lx: number, ly: number, page: number) {
        this.Image = image;
        this.Lx = lx;
        this.Ly = ly;
        this.Page = page;
    }
}

// 模拟 DonePdfSeal 类
class DonePdfSeal {
    initPdfReader(prePdfReader: any): void { }
    updateLocationInfoByKeyword(keyword: string): void { }
    getLocationInfo(): any {
        return null;
    }
}

// 模拟 PrePdfReader 类
class PrePdfReader {
    constructor(pdf: Uint8Array) { }
}

// 模拟 LocationInfo 类
class LocationInfo {
    getCntSeal(): number {
        return 0;
    }
    getLocSignatureLandscapeInfoList(): any[] {
        return [];
    }
}

// 模拟 PdfKeywordExtractor 类
class PdfKeywordExtractor {
    static getKeywordFromPage(pdfReader: any, page: number, keyword: string): LocationInfo {
        return new LocationInfo();
    }
}

// 模拟 PdfReader 类
class PdfReader {
    NumberOfPages: number = 0;
    constructor(pdfData: Uint8Array) { }
    close(): void { }
}

// 模拟 PdfStamper 类
class PdfStamper {
    constructor(pdfReader: PdfReader, outputStream: any) { }
    getOverContent(page: number): any {
        return {
            addImage: (image: any) => { }
        };
    }
    close(): void { }
}

// 模拟 BaseColor 类
class BaseColor {
    static WHITE: any = {};
}

// 模拟 iTextSharp.text.Image 类
class Image {
    Alignment: number = 0;
    Border: number = 0;
    BorderWidth: number = 0;
    BorderColor: any = null;
    RotationDegrees: number = 0;

    static getInstance(imageBytes: Uint8Array): Image {
        return new Image();
    }

    setAbsolutePosition(x: number, y: number): void { }
    scaleToFit(width: number, height: number): void { }
}

// 模拟 MemoryStream 类
class MemoryStream {
    private data: Uint8Array;
    private position: number = 0;

    constructor(data?: Uint8Array) {
        this.data = data || new Uint8Array(0);
    }

    read(data: Uint8Array, offset: number, length: number): number {
        const bytesToRead = Math.min(length, this.data.length - this.position);
        data.set(this.data.subarray(this.position, this.position + bytesToRead), offset);
        this.position += bytesToRead;
        return bytesToRead;
    }

    write(data: Uint8Array, offset: number, length: number): void {
        // 简单模拟写入
    }

    toArray(): Uint8Array {
        return this.data;
    }

    close(): void { }
}

// 模拟 BinaryReader 类
class BinaryReader {
    private stream: MemoryStream;

    constructor(stream: MemoryStream) {
        this.stream = stream;
    }

    get BaseStream(): MemoryStream {
        return this.stream;
    }

    readBytes(length: number): Uint8Array {
        const data = new Uint8Array(length);
        const bytesRead = this.stream.read(data, 0, length);
        return data.subarray(0, bytesRead);
    }

    seek(offset: number, origin: any): void {
        // 简单模拟定位
    }
}

// 模拟 StreamReader 类
class StreamReader {
    private stream: Stream;
    private encoding: TextDecoder;

    constructor(stream: Stream, encoding: TextDecoder) {
        this.stream = stream;
        this.encoding = encoding;
    }

    async readToEnd(): Promise<string> {
        // 简单模拟读取
        return '';
    }
}

// 模拟 TextDecoder 类
class TextDecoder {
    decode(data: Uint8Array): string {
        return new TextDecoder().decode(data);
    }
}

// 模拟 System.Drawing.Image 类
class DrawingImage {
    static fromFile(filePath: string): DrawingImage {
        return new DrawingImage();
    }
    static fromStream(stream: MemoryStream): DrawingImage {
        return new DrawingImage();
    }
    width: number = 0;
    height: number = 0;
    save(stream: MemoryStream, format: any): void { }
}

// 模拟 Graphics 类
class Graphics {
    static fromImage(image: DrawingImage): Graphics {
        return new Graphics();
    }
    drawImage(img: DrawingImage, rect: any): void { }
}

// 模拟 Bitmap 类
class Bitmap {
    constructor(width: number, height: number) { }
}

// 模拟 ImageFormat 类
class ImageFormat {
    static Png: any = {};
}

// 模拟 Error 类
class Exception {
    constructor(message: string) { }
}

// 模拟 Guid 类
class Guid {
    static newGuid(): string {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            const r = Math.random() * 16 | 0;
            const v = c === 'x' ? r : (r & 0x3 | 0x8);
            return v.toString(16);
        });
    }
}

// 模拟 Regex 类
class Regex {
    static split(str: string, pattern: string, options: any): string[] {
        return str.split(new RegExp(pattern, options));
    }
}

// 模拟 StringBuilder 类
class StringBuilder {
    private content: string = '';

    constructor(initialContent?: string) {
        if (initialContent) {
            this.content = initialContent;
        }
    }

    append(value: string): StringBuilder {
        this.content += value;
        return this;
    }

    toString(): string {
        return this.content;
    }
}

// 公用方法类
export class CommonMethod {
    static rd: Random = new Random();
    static dt: string = DateTime.format(DateTime.Now(), 'yyyyMMddHHmmssfff');

    /**
     * 校验结果数据是否有效
     * @param resultData 结果数据字节数组
     * @returns bool类型的校验结果
     */
    static checkResultDataValid(resultData: Uint8Array): boolean {
        let valid = true;
        const result = "";//new TextDecoder('utf-8').decode(resultData);//hulei 转码报错
        if (result.startsWith('<Error>')) {
            PlatformLogger.SystemInfo('ElecSealHelper.PrintPdf:发生异常，电子印章加签异常！' + result);
            valid = false;
        }
        return valid;
    }

    /**
     * 从指定文件读取数据
     * @param fileName 要读取的文件名
     * @returns 读取的byte[]类型数据
     */
    static async readFile(fileName: string): Promise<Uint8Array> {
        return readFile(fileName);
    }

    /**
     * 将byte数组数据写入文件
     * @param pReadByte byte数组数据
     * @param fileName 文件名
     * @returns 是否写入成功
     */
    static async writeFile(pReadByte: Uint8Array, fileName: string): Promise<boolean> {
        return writeFile(pReadByte, fileName);
    }

    /**
     * 检查返回值是否正确，输入为stream
     * @param resStream Stream
     * @returns 检查结果字符串
     */
    static async checkResultDataValidStream(resStream: Stream): Promise<string> {
        const data = new Uint8Array(7);
        await resStream.read(data, 0, data.length);
        let valid = '';
        const result = "";// new TextDecoder('utf-8').decode(data);//hulei 转码报错
        if (result.startsWith('<Error>')) {
            valid = result;
        }
        return valid;
    }

    /**
     * streamToString用StreamReader把返回流以UTF8编码格式读成字符串
     * @param stream 输入流
     * @returns 转换后的字符串
     */
    static async streamToString(stream: Stream): Promise<string> {
        const newReader = "";// new StreamReader(stream, new TextDecoder('utf-8'));////hulei 转码报错
        return "";// newReader.readToEnd();
    }

    /**
     * GetNode取报文str的指定字段note值
     * @param str 报文字符串
     * @param node 指定字段名
     * @returns 字段值
     */
    static getNode(str: string, node: string): string {
        const startIndex = str.indexOf(node) + node.length + 1;
        const endIndex = str.indexOf('</' + node + '>');
        return str.substring(startIndex, endIndex);
    }

    /**
     * 产生摘要方法
     * @param date 输入数据字节数组
     * @returns 摘要字节数组
     */
    static hashSHA1(date: Uint8Array): Uint8Array {
        const hash = HashAlgorithm.create('SHA1');
        return hash.computeHash(date);
    }

    /**
     * 字节数组转16进制字符串
     * @param bytes 字节数组
     * @returns 16进制字符串
     */
    static byteToHexStr(bytes: Uint8Array): string {
        let returnStr = '';
        if (bytes) {
            for (let i = 0; i < bytes.length; i++) {
                returnStr += bytes[i].toString(16).padStart(2, '0').toUpperCase();
            }
        }
        return returnStr;
    }

    /**
     * writeStreamToPdf把pdf stream流写到pdf文件里
     * @param resStream 输入流
     * @param filePath 文件路径
     * @returns 是否写入成功
     */
    static async writeStreamToPdf(resStream: Stream, filePath: string): Promise<boolean> {
        const resultByte = await this.writeStreamToByte(resStream);
        try {
            const fileStream = new MemoryStream(resultByte);
            const size = 2048;
            const data = new Uint8Array(size);
            while (true) {
                const bytesRead = await fileStream.read(data, 0, data.length);
                if (bytesRead > 0) {
                    // 这里无法直接写入文件，仅作模拟
                    console.log(`模拟写入数据: ${bytesRead} 字节`);
                } else {
                    break;
                }
            }
            return true;
        } catch (error) {
            return false;
        }
    }

    /**
     * 把stream流写到byte[]里
     * @param resStream 输入流
     * @returns 转换后的字节数组
     */
    static async writeStreamToByte(resStream: Stream): Promise<Uint8Array> {
        try {
            const size = 2048;
            const data = new Uint8Array(size);
            const memoreStream = new MemoryStream();
            while (true) {
                const bytesRead = await resStream.read(data, 0, data.length);
                if (bytesRead > 0) {
                    memoreStream.write(data.subarray(0, bytesRead), 0, bytesRead);
                } else {
                    break;
                }
            }
            return memoreStream.toArray();
        } catch (error) {
            throw error;
        }
    }

    /**
     * 检验服务器返回流是否正确
     * @param resStream 服务器返回流
     * @returns bool结果
     */
    static async checkResultDataValidStreamBool(resStream: Stream): Promise<boolean> {
        const data = new Uint8Array(7);
        await resStream.read(data, 0, data.length);
        let valid = true;
        const result = new TextDecoder().decode(data);
        if (result.startsWith('<Error>')) {
            valid = false;
        }
        return valid;
    }

    /**
     * GetProofHashAndMultiData
     * @param list MultiDataBean列表
     * @returns 包含ProofHashXml和multiData字符串的数组
     */
    static getProofHashAndMultiData(list: MultiDataBean[]): string[] {
        const multiDataStringBuffer = new StringBuilder('');
        const proofHashXmlTemp = new StringBuilder('<ProofHashXml>');
        let count = 0;
        for (let i = 0; i < list.length; i++) {
            const multiDataBean = list[i];
            const data = multiDataBean.getMultiData();
            if (data) {
                if (multiDataBean.getType() !== '0') {
                    if (count > 0) {
                        multiDataStringBuffer.append(',');
                    }
                    multiDataStringBuffer.append(`{"type":"${multiDataBean.getType()}","fileName":"${multiDataBean.getFileName()}","fileData":"${convertToBase64(data)}"}`);
                    count++;
                }

                const hash = this.byteToHexStr(this.hashSHA1(data));
                proofHashXmlTemp.append(`<Proof fileName="${multiDataBean.getFileName()}" hash="${hash}" type="${multiDataBean.getType()}" descr="${multiDataBean.getDescr()}"></Proof>`);
            }
        }

        proofHashXmlTemp.append('</ProofHashXml>');
        return [proofHashXmlTemp.toString(), multiDataStringBuffer.toString()];
    }

    /**
     * createProofSealStrategyXml
     * @param handwritingImage 手写签名图片字节数组
     * @param sealUserInfo 印章用户信息
     * @param signLocation 签名位置信息
     * @param proofHashXml ProofHashXml字符串
     * @returns 生成的策略XML字符串
     */
    static createProofSealStrategyXml(handwritingImage: Uint8Array, sealUserInfo: SealUserInfo, signLocation: SignLocation, proofHashXml: string): string {
        const proofSealStrategyXml = new StringBuilder('<Request>');
        proofSealStrategyXml.append(`<Type>${signLocation.getType()}</Type>`);
        proofSealStrategyXml.append(`<HandwritingImage>${convertToBase64(handwritingImage)}</HandwritingImage>`);
        proofSealStrategyXml.append(`<SealPerson>${sealUserInfo.getUserName()}</SealPerson>`);
        proofSealStrategyXml.append(`<SealLocation>${sealUserInfo.getSealLocation()}</SealLocation>`);
        proofSealStrategyXml.append(`<SealReason>${sealUserInfo.getSealReason()}</SealReason>`);
        proofSealStrategyXml.append(`<IdentificationType>${sealUserInfo.getIdentificationType()}</IdentificationType>`);
        proofSealStrategyXml.append(`<IdentificationNo>${sealUserInfo.getIdentificationNo()}</IdentificationNo>`);
        proofSealStrategyXml.append(`<PhoneNo>${sealUserInfo.getPhoneNo()}</PhoneNo>`);

        if (signLocation.getType() === '2') {
            proofSealStrategyXml.append(`<Page>${signLocation.getPage()}</Page>`);
            proofSealStrategyXml.append(`<LX>${signLocation.getLx()}</LX>`);
            proofSealStrategyXml.append(`<LY>${signLocation.getLy()}</LY>`);
        }

        if (signLocation.getType() === '3') {
            proofSealStrategyXml.append(`<Keyword>${signLocation.getKeyword()}</Keyword>`);
            proofSealStrategyXml.append(`<LocationStyle>${signLocation.getLocationStyle()}</LocationStyle>`);
            proofSealStrategyXml.append(`<OffsetX>${signLocation.getOffsetX()}</OffsetX>`);
            proofSealStrategyXml.append(`<OffsetY>${signLocation.getOffsetY()}</OffsetY>`);
        }

        proofSealStrategyXml.append(proofHashXml);
        proofSealStrategyXml.append('</Request>');

        return proofSealStrategyXml.toString();
    }

    /**
     * createSealStrategyXml
     * @param sealCode 印章编码
     * @param sealPassword 印章密码
     * @param businessCode 业务编码
     * @param businessCodeStyle 业务编码样式
     * @param sealUserInfo 印章用户信息
     * @param signLocation 签名位置信息
     * @returns 生成的策略XML字符串
     */
    static createSealStrategyXml(sealCode: string, sealPassword: string, businessCode: string, businessCodeStyle: string, p0: string, sealUserInfo: SealUserInfo, signLocation: SignLocation): string {
        return this.createSealStrategyXml(sealCode, sealPassword, null, businessCode, businessCodeStyle, sealUserInfo, signLocation);
    }

    /**
     * createSealStrategyXml
     * @param sealCode 印章编码
     * @param sealPassword 印章密码
     * @param sealUserInfo 印章用户信息
     * @param signLocation 签名位置信息
     * @returns 生成的策略XML字符串
     */
    static createSealStrategyXmlSimple(sealCode: string, sealPassword: string, sealUserInfo: SealUserInfo, signLocation: SignLocation): string {
        return this.createSealStrategyXml(sealCode, sealPassword, null, '', '', sealUserInfo, signLocation);
    }

    /**
     * createSealStrategyXml
     * @param sealCode 印章编码
     * @param sealPassword 印章密码
     * @param sealImage 印章图片字节数组
     * @param businessCode 业务编码
     * @param businessCodeStyle 业务编码样式
     * @param sealUserInfo 印章用户信息
     * @param signLocation 签名位置信息
     * @returns 生成的策略XML字符串
     */
    static createSealStrategyXmlFull(sealCode: string, sealPassword: string, sealImage: Uint8Array | null, businessCode: string, businessCodeStyle: string, sealUserInfo: SealUserInfo, signLocation: SignLocation): string {
        const sealStrategyXml = new StringBuilder('<Request>');

        sealStrategyXml.append(`<Type>${signLocation.getType()}</Type>`);
        sealStrategyXml.append(`<SealCode>${sealCode}</SealCode>`);
        sealStrategyXml.append(`<SealPassword>${sealPassword}</SealPassword>`);
        if (sealImage) {
            const sealStrategyXMLString = convertToBase64(sealImage);
            sealStrategyXml.append(`<SealImage>${sealStrategyXMLString}</SealImage>`);
        }
        sealStrategyXml.append(`<SealPerson>${sealUserInfo.getUserName()}</SealPerson>`);
        sealStrategyXml.append(`<SealLocation>${sealUserInfo.getSealLocation()}</SealLocation>`);
        sealStrategyXml.append(`<SealReason>${sealUserInfo.getSealReason()}</SealReason>`);
        sealStrategyXml.append(`<BusinessCode>${businessCode}</BusinessCode>`);
        sealStrategyXml.append(`<BusinessCodeStyle>${businessCodeStyle}</BusinessCodeStyle>`);

        if (signLocation.getType() === '2') {
            sealStrategyXml.append(`<Page>${signLocation.getPage()}</Page>`);
            sealStrategyXml.append(`<LX>${signLocation.getLx()}</LX>`);
            sealStrategyXml.append(`<LY>${signLocation.getLy()}</LY>`);
        }

        if (signLocation.getType() === '3') {
            sealStrategyXml.append(`<Keyword>${signLocation.getKeyword()}</Keyword>`);
            sealStrategyXml.append(`<LocationStyle>${signLocation.getLocationStyle()}</LocationStyle>`);
            sealStrategyXml.append(`<OffsetX>${signLocation.getOffsetX()}</OffsetX>`);
            sealStrategyXml.append(`<OffsetY>${signLocation.getOffsetY()}</OffsetY>`);
        }
        sealStrategyXml.append('</Request>');
        return sealStrategyXml.toString();
    }

    /**
     * buildFieldBeanListXml
     * @param fieldBeanList FieldBean列表
     * @returns 生成的XML字符串
     */
    static buildFieldBeanListXml(fieldBeanList: FieldBean[]): string {
        return this.buildFieldBeanListXmlWithParams(fieldBeanList, '', '');
    }

    /**
     * buildPdfBeanXml拼接PDF报文
     * @param pdfBean PdfBean对象
     * @returns 生成的XML字符串
     */
    static buildPdfBeanXml(pdfBean: PdfBean): string {
        const pdfBeanXmlStringBuffer = new StringBuilder('');
        if (pdfBean) {
            pdfBeanXmlStringBuffer.append('<PdfBean>');
            pdfBeanXmlStringBuffer.append(`<Pdf>${pdfBean.getPdf()}</Pdf>`);
            pdfBeanXmlStringBuffer.append(`<SavedPdfId>${pdfBean.getSavedPdfId()}</SavedPdfId>`);
            pdfBeanXmlStringBuffer.append(`<BizSerialNo>${pdfBean.getBizSerialNo()}</BizSerialNo>`);
            pdfBeanXmlStringBuffer.append(`<BizTypeCode>${pdfBean.getBizTypeCode()}</BizTypeCode>`);
            pdfBeanXmlStringBuffer.append(`<CardNo>${pdfBean.getBizTypeCode()}</CardNo>`);
            pdfBeanXmlStringBuffer.append(`<OutputFilePath>${pdfBean.getOutputFilePath()}</OutputFilePath>`);
            pdfBeanXmlStringBuffer.append(`<Reserved1>${pdfBean.getReserved1()}</Reserved1>`);
            pdfBeanXmlStringBuffer.append(`<Reserved2>${pdfBean.getReserved2()}</Reserved2>`);
            pdfBeanXmlStringBuffer.append(`<Reserved3>${pdfBean.getReserved3()}</Reserved3>`);
            pdfBeanXmlStringBuffer.append('</PdfBean>');
        }
        return pdfBeanXmlStringBuffer.toString();
    }

    /**
     * 生成签章日志报文
     * @param sealLogBeans SealLogBean列表
     * @returns 生成的XML字符串
     */
    static buildSealLogsXml(sealLogBeans: SealLogBean[]): string {
        const xml = new StringBuilder('');
        xml.append('<List>');
        for (const sealLogBean of sealLogBeans) {
            xml.append('<SealLogBean>');
            xml.append(`<SealCode>${sealLogBean.SealCode}</SealCode>`);
            xml.append(`<SealType>${sealLogBean.SealType}</SealType>`);
            xml.append(`<FunctionType>${sealLogBean.FunctionType}</FunctionType>`);
            xml.append(`<BizSerialNo>${sealLogBean.BizSerialNo}</BizSerialNo>`);
            xml.append(`<BizTypeCode>${sealLogBean.BizTypeCode}</BizTypeCode>`);
            xml.append(`<FileHash>${sealLogBean.FileHash}</FileHash>`);
            xml.append(`<TimestampUsed>${sealLogBean.TimestampUsed}</TimestampUsed>`);
            xml.append(`<SealPerson>${sealLogBean.SealPerson}</SealPerson>`);
            xml.append(`<SealReason>${sealLogBean.SealReason}</SealReason>`);
            xml.append(`<SealLocation>${sealLogBean.SealLocation}</SealLocation>`);
            xml.append(`<Remark1>${sealLogBean.Remark1}</Remark1>`);
            xml.append(`<Remark2>${sealLogBean.Remark2}</Remark2>`);
            xml.append(`<Remark3>${sealLogBean.Remark3}</Remark3>`);
            xml.append('</SealLogBean>');
        }
        xml.append('</List>');
        return xml.toString();
    }

    /**
     * 从磁盘读入图片并修改图片大小，返回修改后的图片byte数组
     * @param imagePath 图片路径
     * @param width 目标宽度
     * @param height 目标高度
     * @returns 修改后的图片字节数组
     */
    static async resizeImage(imagePath: string, width: number, height: number): Promise<Uint8Array> {
        try {
            const img = DrawingImage.fromFile(imagePath);
            const bmcpy = new Bitmap(width, height);//hulei 需要实现bitmap
            // const gh = Graphics.fromImage(bmcpy);
            // gh.drawImage(img, { x: 0, y: 0, width, height });
            const ms = new MemoryStream();//hulei 这里是微软MemoryStream
            // bmcpy.save(ms, ImageFormat.Png);
            return ms.toArray();
        } catch (error) {
            throw new Exception('图片大小调整失败');
        }
    }

    /**
     * 生成要写到PDF文本域的业务数据策略报文方法
     * @param fieldBeanList FieldBean列表
     * @param savedBizXmlId 保存的业务XML ID
     * @param savedTimeIncrement 保存的时间增量
     * @returns 生成的XML字符串
     */
    static buildFieldBeanListXmlWithParams(fieldBeanList: FieldBean[], savedBizXmlId: string, savedTimeIncrement: string): string {
        const fieldListXml = new StringBuilder(`<FieldList hashAlg="sha1" savedBizXmlId="${savedBizXmlId}"  savedTimeIncrement="${savedTimeIncrement}">`);
        if (fieldBeanList) {
            for (let i = 0; i < fieldBeanList.length; i++) {
                const fieldBean = fieldBeanList[i];
                fieldListXml.append('<Field>');
                fieldListXml.append(`<FieldId>${fieldBean.getFieldId()}</FieldId>`);
                fieldListXml.append(`<FieldName>${fieldBean.getFieldName()}</FieldName>`);
                fieldListXml.append(`<FieldValue>${fieldBean.getFieldValue()}</FieldValue>`);
                fieldListXml.append(`<FieldValueHash>${fieldBean.getFieldValueHash()}</FieldValueHash>`);
                fieldListXml.append(`<FieldType>${fieldBean.getFieldType()}</FieldType>`);
                fieldListXml.append(`<TemplateIndex>${fieldBean.getTemplateIndex()}</TemplateIndex>`);
                fieldListXml.append('</Field>');
            }
        }
        fieldListXml.append('</FieldList>');
        return fieldListXml.toString();
    }

    /**
     * 组织要合成到PDF内容中的策略报文方法
     * @param textBeanList TextBean列表
     * @param savedBizXmlId 保存的业务XML ID
     * @param savedTimeIncrement 保存的时间增量
     * @returns 生成的XML字符串
     */
    static buildTextBeanListXml(textBeanList: TextBean[], savedBizXmlId: string, savedTimeIncrement: string): string {
        const textListXml = new StringBuilder(`<TextList hashAlg="sha1" savedBizXmlId="${savedBizXmlId}"  savedTimeIncrement="${savedTimeIncrement}">`);
        if (textBeanList) {
            for (let i = 0; i < textBeanList.length; i++) {
                const textBean = textBeanList[i];
                textListXml.append('<Text>');
                textListXml.append(`<TextValue>${textBean.getTextValue()}</TextValue>`);
                textListXml.append(`<TextValueHash>${textBean.getTextValueHash()}</TextValueHash>`);
                textListXml.append(`<TextFontSize>${textBean.getTextFontSize()}</TextFontSize>`);
                textListXml.append(`<TextColor>${textBean.getTextColor()}</TextColor>`);
                textListXml.append(`<Type>${textBean.getType()}</Type>`);
                textListXml.append(`<PageNo>${textBean.getPageNo()}</PageNo>`);
                textListXml.append(`<LX>${textBean.getLx()}</LX>`);
                textListXml.append(`<LY>${textBean.getLy()}</LY>`);
                textListXml.append(`<Keyword>${textBean.getKeyword()}</Keyword>`);
                textListXml.append(`<LocationStyle>${textBean.getLocationStyle()}</LocationStyle>`);
                textListXml.append(`<OffsetX>${textBean.getOffsetX()}</OffsetX>`);
                textListXml.append(`<OffsetY>${textBean.getOffsetY()}</OffsetY>`);
                textListXml.append(`<TemplateIndex>${textBean.getTemplateIndex()}</TemplateIndex>`);
                textListXml.append('</Text>');
            }
        }
        textListXml.append('</TextList>');
        return textListXml.toString();
    }

    /**
     * 生成图片列表的策略报文方法
     * @param imageBeanList ImageBean列表
     * @param qrCodeBeanList QRCodeBean列表
     * @param savedBizXmlId 保存的业务XML ID
     * @param savedTimeIncrement 保存的时间增量
     * @returns 生成的XML字符串
     */
    static buildImageBeanListXml(imageBeanList: ImageBean[], qrCodeBeanList: QRCodeBean[], savedBizXmlId: string, savedTimeIncrement: string): string {
        const imageListXml = new StringBuilder(`<ImageList hashAlg="sha1" savedBizXmlId="${savedBizXmlId}"  savedTimeIncrement="${savedTimeIncrement}">`);
        if (imageBeanList) {
            for (let i = 0; i < imageBeanList.length; i++) {
                const imageBean = imageBeanList[i];
                imageListXml.append('<Image>');
                imageListXml.append(`<ImageValue>${imageBean.getValue()}</ImageValue>`);
                imageListXml.append(`<PageNo>${imageBean.getPageNo()}</PageNo>`);
                imageListXml.append(`<LX>${imageBean.getLx()}</LX>`);
                imageListXml.append(`<LY>${imageBean.getLy()}</LY>`);
                imageListXml.append(`<Type>${imageBean.getType()}</Type>`);
                imageListXml.append(`<Keyword>${imageBean.getKeyword()}</Keyword>`);
                imageListXml.append(`<LocationStyle>${imageBean.getLocationStyle()}</LocationStyle>`);
                imageListXml.append(`<OffsetX>${imageBean.getOffsetX()}</OffsetX>`);
                imageListXml.append(`<OffsetY>${imageBean.getOffsetY()}</OffsetY>`);
                imageListXml.append(`<ImageWidth>${imageBean.getImageWidth()}</ImageWidth>`);
                imageListXml.append(`<ImageHeight>${imageBean.getImageHeight()}</ImageHeight>`);
                imageListXml.append(`<TemplateIndex>${imageBean.getTemplateIndex()}</TemplateIndex>`);
                imageListXml.append('</Image>');
            }
        }
        // qrCodeBean对象可以为空，如果不为空代表需要在pdf中生成二维码
        if (qrCodeBeanList) {
            for (const qrCodeBean of qrCodeBeanList) {
                imageListXml.append('<QRCode>');
                imageListXml.append(`<QRCodeContent>${qrCodeBean.getQrCodeContent()}</QRCodeContent>`);
                imageListXml.append(`<Width>${qrCodeBean.getWidth()}</Width>`);
                imageListXml.append(`<Height>${qrCodeBean.getHeight()}</Height>`);
                imageListXml.append(`<PageNo>${qrCodeBean.getPageNo()}</PageNo>`);
                imageListXml.append(`<LX>${qrCodeBean.getLx()}</LX>`);
                imageListXml.append(`<LY>${qrCodeBean.getLy()}</LY>`);
                imageListXml.append(`<TemplateIndex>${qrCodeBean.getTemplateIndex()}</TemplateIndex>`);
                imageListXml.append('</QRCode>');
            }
        }
        imageListXml.append('</ImageList>');
        return imageListXml.toString();
    }

    /**
     * createProofSealStrategyXml拼接单据印章报文，参数SignLocation为List形式
     * @param handwritingData 手写签名数据字节数组
     * @param sealUserInfo 印章用户信息
     * @param list SignLocation列表
     * @param proofHashXml ProofHashXml字符串
     * @returns 生成的策略XML字符串
     */
    static createProofSealStrategyXmlList(handwritingData: Uint8Array, sealUserInfo: SealUserInfo, list: SignLocation[], proofHashXml: string): string {
        const proofSealStrategyXml = new StringBuilder('<Request>');

        proofSealStrategyXml.append(`<HandwritingImage>${convertToBase64(handwritingData)}</HandwritingImage>`);

        proofSealStrategyXml.append(`<SealPerson>${sealUserInfo.getUserName()}</SealPerson>`);
        proofSealStrategyXml.append(`<SealLocation>${sealUserInfo.getSealLocation()}</SealLocation>`);
        proofSealStrategyXml.append(`<SealReason>${sealUserInfo.getSealReason()}</SealReason>`);

        proofSealStrategyXml.append(`<IdentificationType>${sealUserInfo.getIdentificationType()}</IdentificationType>`);
        proofSealStrategyXml.append(`<IdentificationNo>${sealUserInfo.getIdentificationNo()}</IdentificationNo>`);
        proofSealStrategyXml.append(`<PhoneNo>${sealUserInfo.getPhoneNo()}</PhoneNo>`);

        for (let i = 0; i < list.length; i++) {
            const signLocation = list[i];
            proofSealStrategyXml.append('<Location>');
            proofSealStrategyXml.append(`<Type>${signLocation.getType()}</Type>`);
            proofSealStrategyXml.append(`<PdfIndex>${signLocation.getPdfIndex()}</PdfIndex>`);
            if (signLocation.getType() === '2') {
                proofSealStrategyXml.append(`<Page>${signLocation.getPage()}</Page>`);
                proofSealStrategyXml.append(`<LX>${signLocation.getLx()}</LX>`);
                proofSealStrategyXml.append(`<LY>${signLocation.getLy()}</LY>`);
            }
            if (signLocation.getType() === '3') {
                proofSealStrategyXml.append(`<Keyword>${signLocation.getKeyword()}</Keyword>`);
                proofSealStrategyXml.append(`<LocationStyle>${signLocation.getLocationStyle()}</LocationStyle>`);
                proofSealStrategyXml.append(`<OffsetX>${signLocation.getOffsetX()}</OffsetX>`);
                proofSealStrategyXml.append(`<OffsetY>${signLocation.getOffsetY()}</OffsetY>`);
            }
            proofSealStrategyXml.append('</Location>');
        }
        proofSealStrategyXml.append(proofHashXml);
        proofSealStrategyXml.append('</Request>');

        return proofSealStrategyXml.toString();
    }

    /**
     * 获取数据库ID方法
     * @returns 生成的数据库ID字符串
     */
    static getDBId(): string {
        try {
            const rd1 = this.rd.next(100000, 999999);
            const rd2 = this.rd.next(1000000, 9999999);
            const result = this.dt + rd1.toString() + rd2.toString();
            return result;
        } catch (error) {
            return '获取数据库ID异常';
        }
    }

    /**
     * 生成随机数
     * @returns 生成的随机数字符串
     */
    static getRandom(): string {
        const rd = new Random();//hulei 随机生成parseInt(Guid.newGuid().replace(/-/g, ''), 16)
        const rd1 = rd.next(100000, 999999);
        const rd2 = rd.next(1000000, 9999999);
        const result = rd1.toString() + rd2.toString();
        return result;
    }

    /**
     * GetPdfBeanArr:获取PdfBean数组
     * @param pdfs PDF字符串
     * @returns PdfBean数组列表
     */
    static getPdfBeanArr(pdfs: string): string[][] {
        const list: string[][] = [];
        const pdfArr = Regex.split(pdfs, '<Bean>', 0);
        if (pdfArr && pdfArr.length > 1) {
            for (let i = 1; i < pdfArr.length; i++) {
                const pdfBeanString = Regex.split(pdfArr[i], '<Bean>', 0)[0];
                const pdfTemp = Regex.split(pdfBeanString, '<Pdf>', 0)[1];
                const pdf = Regex.split(pdfTemp, '</Pdf>', 0)[0];
                const pdfID = '';
                list.push([pdf, pdfID]);
            }
        }
        return list;
    }

    /**
     * createSealStrategyXml
     * @param sealCode 印章编码
     * @param sealPassword 印章密码
     * @param businessCode 业务编码
     * @param businessCodeStyle 业务编码样式
     * @param sealUserInfo 印章用户信息
     * @param list SignLocation列表
     * @returns 生成的策略XML字符串
     */
    static createSealStrategyXmlList(sealCode: string, sealPassword: string, businessCode: string, businessCodeStyle: string, sealUserInfo: SealUserInfo, list: SignLocation[]): string {
        return this.createSealStrategyXmlFullList(sealCode, sealPassword, null, businessCode, businessCodeStyle, sealUserInfo, list);
    }

    /**
     * createSealStrategyXml
     * @param sealCode 印章编码
     * @param sealPassword 印章密码
     * @param sealImage 印章图片字节数组
     * @param businessCode 业务编码
     * @param businessCodeStyle 业务编码样式
     * @param sealUserInfo 印章用户信息
     * @param list SignLocation列表
     * @returns 生成的策略XML字符串
     */
    static createSealStrategyXmlFullList(sealCode: string, sealPassword: string, sealImage: Uint8Array | null, businessCode: string, businessCodeStyle: string, sealUserInfo: SealUserInfo, list: SignLocation[]): string {
        const sealStrategyXml = new StringBuilder('<Request>');
        sealStrategyXml.append(`<SealCode>${sealCode}</SealCode>`);
        sealStrategyXml.append(`<SealPassword>${sealPassword}</SealPassword>`);
        if (sealImage) {
            sealStrategyXml.append(`<SealImage>${convertToBase64(sealImage)}</SealImage>`);
        }
        sealStrategyXml.append(`<SealPerson>${sealUserInfo.getUserName()}</SealPerson>`);
        sealStrategyXml.append(`<SealLocation>${sealUserInfo.getSealLocation()}</SealLocation>`);
        sealStrategyXml.append(`<SealReason>${sealUserInfo.getSealReason()}</SealReason>`);
        sealStrategyXml.append(`<BusinessCode>${businessCode}</BusinessCode>`);
        sealStrategyXml.append(`<BusinessCodeStyle>${businessCodeStyle}</BusinessCodeStyle>`);

        for (let i = 0; i < list.length; i++) {
            const signLocation = list[i];
            sealStrategyXml.append('<Location>');
            sealStrategyXml.append(`<Type>${signLocation.getType()}</Type>`);
            sealStrategyXml.append(`<PdfIndex>${signLocation.getPdfIndex()}</PdfIndex>`);
            if (signLocation.getType() === '2') {
                sealStrategyXml.append(`<Page>${signLocation.getPage()}</Page>`);
                sealStrategyXml.append(`<LX>${signLocation.getLx()}</LX>`);
                sealStrategyXml.append(`<LY>${signLocation.getLy()}</LY>`);
            }
            if (signLocation.getType() === '3') {
                sealStrategyXml.append(`<Keyword>${signLocation.getKeyword()}</Keyword>`);
                sealStrategyXml.append(`<LocationStyle>${signLocation.getLocationStyle()}</LocationStyle>`);
                sealStrategyXml.append(`<OffsetX>${signLocation.getOffsetX()}</OffsetX>`);
                sealStrategyXml.append(`<OffsetY>${signLocation.getOffsetY()}</OffsetY>`);
            }
            sealStrategyXml.append('</Location>');
        }
        sealStrategyXml.append('</Request>');
        return sealStrategyXml.toString();
    }

    /**
     * buildPdfBeanListXml构建PDF列表报文
     * @param pdfBeanList PdfBean列表
     * @returns 生成的XML字符串
     */
    static buildPdfBeanListXml(pdfBeanList: PdfBean[]): string {
        const pdfBeanListXmlStringBuffer = new StringBuilder('<List>');
        if (pdfBeanList) {
            for (let i = 0; i < pdfBeanList.length; i++) {
                const pdfBean = pdfBeanList[i];
                pdfBeanListXmlStringBuffer.append('<PdfBean>');
                pdfBeanListXmlStringBuffer.append(`<Pdf>${pdfBean.getPdf()}</Pdf>`);
                pdfBeanListXmlStringBuffer.append(`<PdfId>${pdfBean.getPdfId()}</PdfId>`);
                pdfBeanListXmlStringBuffer.append(`<SavedPdfId>${pdfBean.getSavedPdfId()}</SavedPdfId>`);
                pdfBeanListXmlStringBuffer.append(`<BizSerialNo>${pdfBean.getBizSerialNo()}</BizSerialNo>`);
                pdfBeanListXmlStringBuffer.append(`<BizTypeCode>${pdfBean.getBizTypeCode()}</BizTypeCode>`);
                pdfBeanListXmlStringBuffer.append(`<CardNo>${pdfBean.getBizTypeCode()}</CardNo>`);
                pdfBeanListXmlStringBuffer.append(`<OutputFilePath>${pdfBean.getOutputFilePath()}</OutputFilePath>`);
                pdfBeanListXmlStringBuffer.append(`<Reserved1>${pdfBean.getReserved1()}</Reserved1>`);
                pdfBeanListXmlStringBuffer.append(`<Reserved2>${pdfBean.getReserved2()}</Reserved2>`);
                pdfBeanListXmlStringBuffer.append(`<Reserved3>${pdfBean.getReserved3()}</Reserved3>`);
                pdfBeanListXmlStringBuffer.append('</PdfBean>');
            }
        }
        pdfBeanListXmlStringBuffer.append('</List>');
        return pdfBeanListXmlStringBuffer.toString();
    }

    /**
     * 判断字符串是否不为空
     * @param str 输入字符串
     * @returns 是否不为空
     */
    static isNotEmpty(str: string): boolean {
        if (str && str.trim() !== '' && str.trim().toLowerCase() !== 'null') {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 无证书盖章采用合成图片方式
     * @param pdf PDF字节数组
     * @param imageBytes 图片字节数组
     * @param signLocation 签名位置信息
     * @param n 缩放比例
     * @returns 处理后的PDF字节数组
     */
    static doSealAutoPdfWithoutCert(pdf: Uint8Array, imageBytes: Uint8Array, signLocation: SignLocation, n: number): Uint8Array {
        const pdfImageMap: { [key: string]: ImageLocationBean } = {};
        const sizeArr = this.calculateImageSize(imageBytes, n);
        const width = sizeArr[0];
        const height = sizeArr[1];
        if (signLocation.getType() === '2') {
            const image = Image.getInstance(imageBytes);
            // image.Alignment = Image.LEFT_ALIGN | Image.TEXTWRAP;
            // image.Border = Image.BOX; //模拟image hulei 这里要重构
            image.BorderWidth = 0;
            image.BorderColor = BaseColor.WHITE;
            image.RotationDegrees = 0;
            image.setAbsolutePosition(signLocation.getLx(), signLocation.getLy());
            image.scaleToFit(width, height);

            const imageLocation = new ImageLocationBean(image, signLocation.getLx(), signLocation.getLy(), signLocation.getPage());
            pdfImageMap[Object.keys(pdfImageMap).length.toString()] = imageLocation;
        } else if (signLocation.getType() === '3') { // keywords
            const donePdfSeal = new DonePdfSeal();
            const prePdfReader = new PrePdfReader(pdf);
            donePdfSeal.initPdfReader(prePdfReader);
            donePdfSeal.updateLocationInfoByKeyword(signLocation.getKeyword());
            const locationInfo = donePdfSeal.getLocationInfo();
            if (locationInfo && locationInfo.getLocSignatureLandscapeInfoList().length > 0) {
                const list = locationInfo.getLocSignatureLandscapeInfoList();
                for (let i = 0; i < list.length; i++) {
                    const pageNo = list[i].page;
                    const lx = list[i].rect.Left;
                    const ly = list[i].rect.Bottom;
                    const image = Image.getInstance(imageBytes);

                    // image.Alignment = Image.LEFT_ALIGN | Image.TEXTWRAP;
                    // image.Border = Image.BOX;模拟image hulei 这里要重构
                    image.BorderWidth = 0;
                    image.BorderColor = BaseColor.WHITE;
                    image.RotationDegrees = 0;
                    image.setAbsolutePosition(signLocation.getLx(), signLocation.getLy());
                    image.scaleToFit(width, height);

                    const imageLocation = new ImageLocationBean(image, lx, ly, pageNo);
                    pdfImageMap[Object.keys(pdfImageMap).length.toString()] = imageLocation;
                }
            } else {
                throw new Exception(`keyword:${signLocation.getKeyword()}`);
            }
        }
        pdf = this.compoundPdfDataByImage(pdf, 1, pdfImageMap);

        return pdf;
    }

    /**
     * 将图片合成到pdf
     * @param pdfData PDF字节数组
     * @param pdfIndex PDF索引
     * @param map 图片位置信息映射
     * @returns 处理后的PDF字节数组
     */
    static compoundPdfDataByImage(pdfData: Uint8Array, pdfIndex: number, map: { [key: string]: ImageLocationBean }): Uint8Array {
        let pdfReader: PdfReader | null = null;
        let pdfStamper: PdfStamper | null = null;
        try {
            pdfReader = new PdfReader(pdfData);
            const pages = pdfReader.NumberOfPages;
            for (let i = 0; i < Object.keys(map).length; i++) {
                const il = map[i.toString()];
                // 判断该字段是否合成到pdf
                if (il.TemplateIndex !== 0) {
                    if (pdfIndex !== il.TemplateIndex) {
                        continue;
                    }
                }
                const image = il.Image;
                pdfReader = new PdfReader(pdfData);
                const byteArrayOutputStream = new MemoryStream();
                pdfStamper = new PdfStamper(pdfReader, byteArrayOutputStream);
                if (il.Type === '2') {
                    if (pages < il.Page) {
                        throw new Exception(`pageNo不能超过pdf总页数: pageNo=${il.Page} pdf总页数=${pages}`);
                    }
                    image.setAbsolutePosition(il.Lx, il.Ly);
                    pdfStamper.getOverContent(il.Page).addImage(image);
                } else {
                    const keyword = il.Keyword;
                    const keywordLength = keyword.length;
                    const locationStyle = il.LocationStyle;
                    const offsetX = il.OffsetX;
                    const offsetY = il.OffsetY;
                    const num = pdfReader.NumberOfPages;
                    const scaledImageHeight = image.ScaledHeight;
                    const scaledImageWidth = image.ScaledWidth;
                    for (let j = 1; j <= num; j++) {
                        const li = PdfKeywordExtractor.getKeywordFromPage(pdfReader, j, keyword);
                        if (li.getCntSeal() !== 0) {
                            const signatureLandscapes = li.getLocSignatureLandscapeInfoList();
                            for (let k = 0; k < signatureLandscapes.length; k++) {
                                const signatureLandscape = signatureLandscapes[k];
                                let llx = signatureLandscape.rect.Left;
                                let lly = signatureLandscape.rect.Bottom;
                                const charTextWidth = signatureLandscape.charTextWidth;
                                llx = llx + charTextWidth * keywordLength / 2.0 - scaledImageWidth / 2.0;
                                lly = lly + charTextWidth / 2.0 - scaledImageWidth / 2.0;
                                if (locationStyle === 'U') {
                                    lly += charTextWidth / 2.0;
                                    lly += scaledImageHeight / 2.0;
                                } else if (locationStyle === 'D') {
                                    lly -= charTextWidth / 2.0;
                                    lly -= scaledImageHeight / 2.0;
                                } else if (locationStyle === 'L') {
                                    llx -= charTextWidth * keywordLength / 2.0;
                                    llx -= scaledImageWidth / 2.0;
                                } else if (locationStyle === 'R') {
                                    llx += charTextWidth * keywordLength / 2.0;
                                    llx += scaledImageWidth / 2.0;
                                } else if (locationStyle === 'C') {
                                    // 无需处理
                                } else {
                                    throw new Exception('LocationStyle取值应为U、D、L、R、C');
                                }
                                llx += offsetX;
                                lly += offsetY;
                                image.setAbsolutePosition(llx, lly);
                                pdfStamper.getOverContent(j).addImage(image);
                            }
                        }
                    }
                }
                pdfStamper.close();
                pdfReader.close();
                pdfData = byteArrayOutputStream.toArray();
            }
            return pdfData;
        } catch (error) {
            throw new Exception('pdf合成图片错误!');
        } finally {
            try {
                if (pdfReader) {
                    pdfReader.close();
                }
                if (pdfStamper) {
                    pdfStamper.close();
                }
            } catch (error) {
                throw new Exception('pdf合成图片,关闭pdfReader和pdfStamper异常！');
            }
        }
    }

    /**
     * 计算图片大小
     * @param imageByte 图片字节数组
     * @param n 缩放比例
     * @returns 包含宽度和高度的数组
     */
    static calculateImageSize(imageByte: Uint8Array, n: number): number[] {
        try {
            // MemoryStream hulei 这里要重构
            const ms = new MemoryStream(imageByte);
            // ms.Position = 0;
            const img = DrawingImage.fromStream(ms);
            ms.close();
            const w = img.width;
            const h = img.height;
            const width = w * n;
            const height = h / n;
            return [Math.floor(width), Math.floor(height)];
        } catch (error) {
            throw new Exception('CalculateImageSize error!');
        }
    }
}
