/**
 * 多媒体数据bean类
 */
export class MultiDataBean {
    private multiData: Uint8Array;
    private fileName: string;
    private descr: string;
    // type取值：0=不放到pdf文档中；1=注释方式；2=附件方式
    private type: string;

    constructor();
    constructor(multiData: Uint8Array, fileName: string, type: string);
    constructor(multiData: Uint8Array, fileName: string, descr: string, type: string);
    constructor(multiData?: Uint8Array, fileName?: string, descrOrType?: string, type?: string) {
        if (multiData && fileName && type) {
            this.multiData = multiData;
            this.fileName = fileName;
            this.type = type;
            if (descrOrType) {
                this.descr = descrOrType;
            } else {
                this.descr = "";
            }
        } else {
            // 默认构造函数
            this.multiData = new Uint8Array();
            this.fileName = "";
            this.descr = "";
            this.type = "";
        }
    }

    /**
     * 获取多媒体数据
     */
    public getMultiData(): Uint8Array {
        return this.multiData;
    }

    /**
     * 设置多媒体数据
     * @param multiData - 要设置的多媒体数据
     */
    public setMultiData(multiData: Uint8Array): void {
        this.multiData = multiData;
    }

    /**
     * 获取文件名
     */
    public getFileName(): string {
        return this.fileName;
    }

    /**
     * 设置文件名
     * @param fileName - 要设置的文件名
     */
    public setFileName(fileName: string): void {
        this.fileName = fileName;
    }

    /**
     * 获取类型
     */
    public getType(): string {
        return this.type;
    }

    /**
     * 设置类型
     * @param type - 要设置的类型
     */
    public setType(type: string): void {
        this.type = type;
    }

    /**
     * 获取描述
     */
    public getDescr(): string {
        return this.descr;
    }

    /**
     * 设置描述
     * @param descr - 要设置的描述
     */
    public setDescr(descr: string): void {
        this.descr = descr;
    }
}