﻿using System;
using System.IO;
using System.Net;
using System.Text;

namespace TellerSystem.PrintTemplate.ElecSeal.Cfca
{
    /// <summary>
    /// 组织要发送到服务器的xml报文类
    /// </summary>
    public class PaperlessClient
    {
        /// <summary>
        /// 组织Pdf自动化合成模板业务数据报文，发送数据并接收返回流
        /// </summary>
        /// <param name="url"></param>
        /// <param name="templateCode"></param>
        /// <param name="savedPdfId"></param>
        /// <param name="fieldBeanListXml"></param>
        /// <param name="textBeanListXml"></param>
        /// <param name="imageBeanListXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] SynthesizeAutoTemplate(string url, String templateCode, String savedPdfId, String fieldBeanListXml, String textBeanListXml, String imageBeanListXml,
           String operatorCode, String channelCode)
        {
            try
            {
                fieldBeanListXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(fieldBeanListXml));
                textBeanListXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(textBeanListXml));
                imageBeanListXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(imageBeanListXml));
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=synthesizeAutoTemplate");
                StringBuilderParameter.Append("&savedPdfId=").Append(savedPdfId)
                .Append("&templateCode=").Append(templateCode)
                .Append("&fieldBeanListXml=").Append(fieldBeanListXml)
                .Append("&textBeanListXml=").Append(textBeanListXml)
                .Append("&imageBeanListXml=").Append(imageBeanListXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);

                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// 组织Pdf自动化合成模板业务数据报文，发送数据并接收返回流(无savedPdfId)
        /// </summary>
        /// <param name="url"></param>
        /// <param name="templateCode"></param>
        /// <param name="fieldBeanListXml"></param>
        /// <param name="textBeanListXml"></param>
        /// <param name="imageBeanListXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] SynthesizeAutoTemplate(string url, String templateCode, String fieldBeanListXml, String textBeanListXml, String imageBeanListXml,
           String operatorCode, String channelCode)
        {
            return SynthesizeAutoTemplate(url, templateCode, "", fieldBeanListXml, textBeanListXml, imageBeanListXml, operatorCode, channelCode);
        }
        /// <summary>
        /// synthesizeAutoBusinessPdf
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdf"></param>
        /// <param name="fieldBeanListXml"></param>
        /// <param name="textBeanListXml"></param>
        /// <param name="imageBeanListXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>

        public byte[] SynthesizeAutoBusinessPdf(string url, byte[] pdf, String fieldBeanListXml, String textBeanListXml, String imageBeanListXml,
            String operatorCode, String channelCode)
        {
            return SynthesizeAutoBusinessPdf(url, pdf, "", fieldBeanListXml, textBeanListXml, imageBeanListXml, operatorCode, channelCode);
        }

        /// <summary>
        /// 组织Pdf自动化合成业务数据报文，发送数据并接收返回流
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdf"></param>
        /// <param name="savedPdfId"></param>
        /// <param name="fieldBeanListXml"></param>
        /// <param name="textBeanListXml"></param>
        /// <param name="imageBeanListXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] SynthesizeAutoBusinessPdf(string url, byte[] pdf, String savedPdfId, String fieldBeanListXml, String textBeanListXml, String imageBeanListXml, String operatorCode, String channelCode)
        {
            try
            {
                string pdfString = Convert.ToBase64String(pdf);
                fieldBeanListXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(fieldBeanListXml));
                textBeanListXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(textBeanListXml));
                imageBeanListXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(imageBeanListXml));

                StringBuilder StringBuilderParameter = new StringBuilder("functionType=synthesizeAutoBusinessPdf");
                StringBuilderParameter.Append("&pdf=").Append(pdfString)
                .Append("&savedPdfId=").Append(savedPdfId)
                .Append("&fieldBeanListXml=").Append(fieldBeanListXml)
                .Append("&textBeanListXml=").Append(textBeanListXml)
                .Append("&imageBeanListXml=").Append(imageBeanListXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);


                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// pdf自动化合成多媒体数据组织报文发送并获取返回流
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdf"></param>
        /// <param name="savedPdfId"></param>
        /// <param name="multiData"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] SynthesizeAutoMultiPdf(string url, byte[] pdf, String savedPdfId, String multiData, String operatorCode, String channelCode)
        {
            try
            {

                string pdfString = Convert.ToBase64String(pdf);
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=synthesizeAutoMultiPdf");
                StringBuilderParameter.Append("&pdf=").Append(pdfString).Append("&savedPdfId=").Append(savedPdfId).Append("&multiData=").Append(multiData)
                        .Append("&operatorCode=").Append(operatorCode).Append("&channelCode=").Append(channelCode);


                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// sealAutoPdf
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdf"></param>
        /// <param name="sealStrategyXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] SealAutoPdf(string url, byte[] pdf, String sealStrategyXml, String operatorCode, String channelCode)
        {
            return SealAutoPdf(url, pdf, "", sealStrategyXml, operatorCode, channelCode);
        }
        /// <summary>
        /// html/word转换pdf
        /// </summary>
        /// <param name="url"></param>
        /// <param name="source"></param>
        /// <param name="fileType"></param>
        /// <param name="savedPdfId"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] TransformToPdf(string url, byte[] source, String fileType, String savedPdfId, String operatorCode, String channelCode)
        {
            try
            {

                string sourceString = Convert.ToBase64String(source);
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=transformToPdf");
                StringBuilderParameter.Append("&source=").Append(sourceString)
                .Append("&fileType=").Append(fileType)
                .Append("&savedPdfId=").Append(savedPdfId)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);


                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        /// <summary>
        /// PDF自动化转换图片
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdf"></param>
        /// <param name="pageNo"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public string TransformPdfToImage(string url, byte[] pdf, String pageNo, String operatorCode, String channelCode)
        {
            try
            {
                string pdfString = Convert.ToBase64String(pdf);
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=transformPdfToImage");
                StringBuilderParameter.Append("&pdf=").Append(pdfString)
                .Append("&pageNo=").Append(pageNo)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);

                return CommonMethod.StreamToString(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// 合并PDF列表接口
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdfListXml"></param>
        /// <param name="outputFilePath"></param>
        /// <param name="savedPdfId"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] AutoConcatPdfList(string url, String pdfListXml, String outputFilePath, String savedPdfId, String operatorCode, String channelCode)
        {
            try
            {
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=autoConcatPdfList");
                StringBuilderParameter.Append("&pdfListXml=").Append(pdfListXml)
                .Append("&outputFilePath=").Append(outputFilePath)
                .Append("&savedPdfId=").Append(savedPdfId)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);

                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// 合并PDF列表接口
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdfListXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] AutoConcatPdfList(string url, String pdfListXml, String operatorCode, String channelCode)
        {
            return AutoConcatPdfList(url, pdfListXml, "", "", operatorCode, channelCode);
        }
        /// <summary>
        /// 组织PDF签章报文，发送数据并接收返回流
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdf"></param>
        /// <param name="savedPdfId"></param>
        /// <param name="sealStrategyXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] SealAutoPdf(string url, byte[] pdf, string savedPdfId, string sealStrategyXml, string operatorCode, string channelCode)
        {
            try
            {
                string pdfString = Convert.ToBase64String(pdf);
                sealStrategyXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(sealStrategyXml));
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=sealAutoPdf");
                StringBuilderParameter.Append("&pdf=").Append(pdfString)
                .Append("&savedPdfId=").Append(savedPdfId)
                .Append("&sealStrategyXml=").Append(sealStrategyXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);

                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// 组织PDF自动化复合签章，发送数据并接收返回流
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdfBeanXml"></param>
        /// <param name="multiData"></param>
        /// <param name="compoundSealStrategyXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelBeanXml"></param>
        /// <returns></returns>
        public byte[] CompoundSealAutoPdf(string url, String pdfBeanXml, String multiData, String compoundSealStrategyXml, String operatorCode, String channelBeanXml)
        {
            try
            {
                compoundSealStrategyXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(compoundSealStrategyXml));

                channelBeanXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(channelBeanXml));
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=compoundSealAutoPdf");
                StringBuilderParameter.Append("&pdfBeanXml=").Append(pdfBeanXml)
                .Append("&multiData=").Append(multiData)
                .Append("&compoundSealStrategyXml=").Append(compoundSealStrategyXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelBeanXml=").Append(channelBeanXml);

                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// 组织PDF自动化复合签章报文，发送数据并接收返回流
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdfBeanListXml"></param>
        /// <param name="multiData"></param>
        /// <param name="compoundSealStrategyXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelBeanXml"></param>
        /// <returns></returns>
        public string CompoundSealAutoPdfList(string url, String pdfBeanListXml, String multiData, String compoundSealStrategyXml, String operatorCode, String channelBeanXml)
        {
            try
            {

                compoundSealStrategyXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(compoundSealStrategyXml));
                channelBeanXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(channelBeanXml));
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=compoundSealAutoPdfList");
                StringBuilderParameter.Append("&pdfBeanListXml=").Append(pdfBeanListXml)
                .Append("&multiData=").Append(multiData)
                .Append("&compoundSealStrategyXml=").Append(compoundSealStrategyXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelBeanXml=").Append(channelBeanXml);

                return CommonMethod.StreamToString(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// 组织PDF列表自动化签章分离式接口报文，发送数据并接收返回流
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdfBeanListXml"></param>
        /// <param name="multiData"></param>
        /// <param name="compoundSealStrategyXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelBeanXml"></param>
        /// <returns></returns>
        public string CompoundSealAutoPdfListDetached(string url, String pdfBeanListXml, String multiData, String compoundSealStrategyXml, String operatorCode, String channelBeanXml)
        {
            try
            {
                compoundSealStrategyXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(compoundSealStrategyXml));
                channelBeanXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(channelBeanXml));
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=compoundSealAutoPdfListDetached");
                StringBuilderParameter.Append("&pdfBeanListXml=").Append(pdfBeanListXml)
                .Append("&multiData=").Append(multiData)
                .Append("&compoundSealStrategyXml=").Append(compoundSealStrategyXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelBeanXml=").Append(channelBeanXml);

                return CommonMethod.StreamToString(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        public Stream SealAutoCrossPdf(string url, string pdfString, String crossSealStrategyXml, String operatorCode, String channelCode)
        {
            try
            {


                StringBuilder StringBuilderParameter = new StringBuilder("functionType=sealAutoCrossPdf");
                StringBuilderParameter.Append("&pdf=").Append(pdfString)
                .Append("&crossSealStrategyXml=").Append(crossSealStrategyXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);


                return CallServlet(url, StringBuilderParameter.ToString());
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// PDF添加水印接口
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdf"></param>
        /// <param name="waterMarkStrategyXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] AddWaterMark2Pdf(string url, byte[] pdf, String waterMarkStrategyXml, String operatorCode, String channelCode)
        {
            try
            {

                String pdfString = Convert.ToBase64String(pdf);
                waterMarkStrategyXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(waterMarkStrategyXml));

                StringBuilder StringBuilderParameter = new StringBuilder("functionType=addWaterMark2Pdf");
                StringBuilderParameter.Append("&pdf=").Append(pdfString)
                .Append("&waterMarkStrategyXml=").Append(waterMarkStrategyXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);

                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// 组织PDF验章接口报文并发送给服务器，接收返回流
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdf"></param>
        /// <param name="verifyStrategyXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public string VerifyPdfSeal(string url, byte[] pdf, String verifyStrategyXml, String operatorCode, String channelCode)
        {
            try
            {

                string pdfString = Convert.ToBase64String(pdf, 0, Convert.ToInt32(pdf.Length));
                verifyStrategyXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(verifyStrategyXml));
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=verifyPdfSeal");
                StringBuilderParameter.Append("&pdf=").Append(pdfString)
                .Append("&verifyStrategyXml=").Append(verifyStrategyXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);


                return CommonMethod.StreamToString(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// 撤销PDF印章接口并发送数据，接收返回流
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdf"></param>
        /// <param name="count">需要撤销印章的个数，0：全部撤销，默认：1</param>
        /// <param name="operatorCode"></param>
        /// <returns></returns>
        public byte[] RevokePdfSeal(string url, byte[] pdf, int count, String operatorCode)
        {
            try
            {

                string pdfString = Convert.ToBase64String(pdf);
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=revokePdfSeal");
                StringBuilderParameter.Append("&pdf=").Append(pdfString)
                .Append("&count=").Append(count)
                .Append("&operatorCode=").Append(operatorCode);

                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        public Stream ApplyAndDownloadCert(string url, String applyCertStrategyXml, String operatorCode, String channelCode)
        {
            try
            {


                StringBuilder StringBuilderParameter = new StringBuilder("functionType=applyAndDownloadCert");
                StringBuilderParameter.Append("&applyCertStrategyXml=").Append(applyCertStrategyXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);


                return CallServlet(url, StringBuilderParameter.ToString());
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        public Stream GetTimestampURL(string url, String flag, String operatorCode)
        {
            try
            {
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=getTimestampURL");
                StringBuilderParameter.Append("&flag=").Append(flag)
                .Append("&operatorCode=").Append(operatorCode);


                return CallServlet(url, StringBuilderParameter.ToString());
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// 自助生成图片
        /// </summary>
        /// <param name="url"></param>
        /// <param name="imageStrategyXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] AutoGenerateImage(string url, String imageStrategyXml, String operatorCode, String channelCode)
        {
            try
            {
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=autoGenerateImage");
                StringBuilderParameter.Append("&imageStrategyXml=").Append(imageStrategyXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);


                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// Image转换Pdf功能
        /// </summary>
        /// <param name="url"></param>
        /// <param name="imageListXml"></param>
        /// <param name="savedPdfId"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] TransformImageToPdf(string url, string imageListXml, String savedPdfId, String operatorCode, String channelCode)
        {
            try
            {
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=transformImageToPdf");
                StringBuilderParameter.Append("&imageListXml=").Append(imageListXml)
                .Append("&savedPdfId=").Append(savedPdfId)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);
                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// 组织模板报文并发送数据，接收返回流
        /// </summary>
        /// <param name="url"></param>
        /// <param name="templateCode"></param>
        /// <param name="operatorCode"></param>
        /// <returns></returns>
        public Stream GetTemplate(string url, string templateCode, String operatorCode)
        {
            try
            {
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=getTemplate");
                StringBuilderParameter.Append("&templateCode=").Append(templateCode).Append("&operatorCode=").Append(operatorCode);


                return CallServlet(url, StringBuilderParameter.ToString());
            }
            catch (Exception e)
            {
                throw e;
            }
        }
        /// <summary>
        /// Pdf自动化骑缝章签章
        /// </summary>
        /// <param name="url"></param>
        /// <param name="pdf"></param>
        /// <param name="crossSealStrategyXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public byte[] SealAutoCrossPdf(string url, byte[] pdf, String crossSealStrategyXml, String operatorCode, String channelCode)
        {
            try
            {
                string pdfString = Convert.ToBase64String(pdf);
                crossSealStrategyXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(crossSealStrategyXml));

                StringBuilder StringBuilderParameter = new StringBuilder("functionType=sealAutoCrossPdf");
                StringBuilderParameter.Append("&pdf=").Append(pdfString).Append("&crossSealStrategyXml=").Append(crossSealStrategyXml).Append("&operatorCode=")
                        .Append(operatorCode).Append("&channelCode=").Append(channelCode);

                return CommonMethod.WriteStreamToByte(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        /// <summary>
        /// 下载印章
        /// </summary>
        /// <param name="url"></param>
        /// <param name="requestXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public string DownloadSeal(string url, string requestXml, string operatorCode, string channelCode)
        {
            try
            {
                requestXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(requestXml));
                StringBuilder StringBuilderParameter = new StringBuilder("functionType=downloadSeal");
                StringBuilderParameter.Append("&requestXml=").Append(requestXml)
                .Append("&operatorCode=").Append(operatorCode)
                .Append("&channelCode=").Append(channelCode);
                return CommonMethod.StreamToString(CallServlet(url, StringBuilderParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        /// <summary>
        /// 上传签章日志
        /// </summary>
        /// <param name="url"></param>
        /// <param name="sealLogBeanXml"></param>
        /// <param name="operatorCode"></param>
        /// <param name="channelCode"></param>
        /// <returns></returns>
        public string UploadSealLog(string url, string sealLogBeanXml, string operatorCode, string channelCode)
        {
            try
            {
                sealLogBeanXml = Convert.ToBase64String(Encoding.UTF8.GetBytes(sealLogBeanXml));

                StringBuilder stringBufferParameter = new StringBuilder("functionType=uploadSealLog");
                stringBufferParameter.Append("&sealLogBeanXml=").Append(sealLogBeanXml)
                .Append("&channelCode=").Append(channelCode).Append("&operatorCode=").Append(operatorCode);

                return CommonMethod.StreamToString(CallServlet(url, stringBufferParameter.ToString()));
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        /// <summary>
        ///  根据URL创建连接并发送数据，接收数据并返回
        /// </summary>
        /// <param name="url">服务器地址</param>
        /// <param name="buffer">发送给服务器的参数</param>
        /// <returns>服务器返回流</returns>
        public Stream CallServlet(string url, string buffer)
        {
            try
            {
                HttpWebRequest webrequest = (HttpWebRequest)WebRequest.Create(url);
                webrequest.Method = "POST";
                webrequest.ContentType = "application/x-www-form-urlencoded";
                webrequest.Timeout = 10000;//传输超时
                webrequest.ReadWriteTimeout = 10000;//读取超时
                byte[] data = Encoding.UTF8.GetBytes(buffer);//以UTF8格式转换报文
                webrequest.ContentLength = Convert.ToInt32(data.Length);
                using (Stream Stream = webrequest.GetRequestStream())
                {
                    Stream.Write(data, 0, Convert.ToInt32(data.Length));
                }
                string[] values = webrequest.Headers.GetValues("Content-Type");
                WebResponse myResponse = webrequest.GetResponse();
                Stream resStream = myResponse.GetResponseStream();
                return resStream;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
    }
}


