// 模拟 CommonMethod 工具类
class CommonMethod {
    static async WriteStreamToByte(response: Response): Promise<Uint8Array> {
        const arrayBuffer = await response.arrayBuffer();
        return new Uint8Array(arrayBuffer);
    }

    static async StreamToString(response: Response): Promise<string> {
        return response.text();
    }
}

// 定义 PaperlessClient 类
export class PaperlessClient {
    /**
     * 组织Pdf自动化合成模板业务数据报文，发送数据并接收返回流
     * @param url 
     * @param templateCode 
     * @param savedPdfId 
     * @param fieldBeanListXml 
     * @param textBeanListXml 
     * @param imageBeanListXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async SynthesizeAutoTemplate(
        url: string,
        templateCode: string,
        savedPdfId: string,
        fieldBeanListXml: string,
        textBeanListXml: string,
        imageBeanListXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        try {
            fieldBeanListXml = btoa(unescape(encodeURIComponent(fieldBeanListXml)));
            textBeanListXml = btoa(unescape(encodeURIComponent(textBeanListXml)));
            imageBeanListXml = btoa(unescape(encodeURIComponent(imageBeanListXml)));
            let StringBuilderParameter = "functionType=synthesizeAutoTemplate";
            StringBuilderParameter += `&savedPdfId=${savedPdfId}`
                + `&templateCode=${templateCode}`
                + `&fieldBeanListXml=${fieldBeanListXml}`
                + `&textBeanListXml=${textBeanListXml}`
                + `&imageBeanListXml=${imageBeanListXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 组织Pdf自动化合成模板业务数据报文，发送数据并接收返回流(无savedPdfId)
     * @param url 
     * @param templateCode 
     * @param fieldBeanListXml 
     * @param textBeanListXml 
     * @param imageBeanListXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async SynthesizeAutoTemplateWithoutSavedPdfId(
        url: string,
        templateCode: string,
        fieldBeanListXml: string,
        textBeanListXml: string,
        imageBeanListXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        return this.SynthesizeAutoTemplate(url, templateCode, "", fieldBeanListXml, textBeanListXml, imageBeanListXml, operatorCode, channelCode);
    }

    /**
     * synthesizeAutoBusinessPdf
     * @param url 
     * @param pdf 
     * @param fieldBeanListXml 
     * @param textBeanListXml 
     * @param imageBeanListXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async SynthesizeAutoBusinessPdf(
        url: string,
        pdf: Uint8Array,
        fieldBeanListXml: string,
        textBeanListXml: string,
        imageBeanListXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        return this.SynthesizeAutoBusinessPdfWithSavedPdfId(url, pdf, "", fieldBeanListXml, textBeanListXml, imageBeanListXml, operatorCode, channelCode);
    }

    /**
     * 组织Pdf自动化合成业务数据报文，发送数据并接收返回流
     * @param url 
     * @param pdf 
     * @param savedPdfId 
     * @param fieldBeanListXml 
     * @param textBeanListXml 
     * @param imageBeanListXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async SynthesizeAutoBusinessPdfWithSavedPdfId(
        url: string,
        pdf: Uint8Array,
        savedPdfId: string,
        fieldBeanListXml: string,
        textBeanListXml: string,
        imageBeanListXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        try {
            let pdfString = btoa(String.fromCharCode.apply(null, Array.from(pdf)));
            fieldBeanListXml = btoa(unescape(encodeURIComponent(fieldBeanListXml)));
            textBeanListXml = btoa(unescape(encodeURIComponent(textBeanListXml)));
            imageBeanListXml = btoa(unescape(encodeURIComponent(imageBeanListXml)));

            let StringBuilderParameter = "functionType=synthesizeAutoBusinessPdf";
            StringBuilderParameter += `&pdf=${pdfString}`
                + `&savedPdfId=${savedPdfId}`
                + `&fieldBeanListXml=${fieldBeanListXml}`
                + `&textBeanListXml=${textBeanListXml}`
                + `&imageBeanListXml=${imageBeanListXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * pdf自动化合成多媒体数据组织报文发送并获取返回流
     * @param url 
     * @param pdf 
     * @param savedPdfId 
     * @param multiData 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async SynthesizeAutoMultiPdf(
        url: string,
        pdf: Uint8Array,
        savedPdfId: string,
        multiData: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        try {
            let pdfString = btoa(String.fromCharCode.apply(null, Array.from(pdf)));
            let StringBuilderParameter = "functionType=synthesizeAutoMultiPdf";
            StringBuilderParameter += `&pdf=${pdfString}&savedPdfId=${savedPdfId}&multiData=${multiData}`
                + `&operatorCode=${operatorCode}&channelCode=${channelCode}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * sealAutoPdf
     * @param url 
     * @param pdf 
     * @param sealStrategyXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async SealAutoPdf(
        url: string,
        pdf: Uint8Array,
        sealStrategyXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        return this.SealAutoPdfWithSavedPdfId(url, pdf, "", sealStrategyXml, operatorCode, channelCode);
    }

    /**
     * html/word转换pdf
     * @param url 
     * @param source 
     * @param fileType 
     * @param savedPdfId 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async TransformToPdf(
        url: string,
        source: Uint8Array,
        fileType: string,
        savedPdfId: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        try {
            let sourceString = btoa(String.fromCharCode.apply(null, Array.from(source)));
            let StringBuilderParameter = "functionType=transformToPdf";
            StringBuilderParameter += `&source=${sourceString}`
                + `&fileType=${fileType}`
                + `&savedPdfId=${savedPdfId}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * PDF自动化转换图片
     * @param url 
     * @param pdf 
     * @param pageNo 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async TransformPdfToImage(
        url: string,
        pdf: Uint8Array,
        pageNo: string,
        operatorCode: string,
        channelCode: string
    ): Promise<string> {
        try {
            let pdfString = btoa(String.fromCharCode.apply(null, Array.from(pdf)));
            let StringBuilderParameter = "functionType=transformPdfToImage";
            StringBuilderParameter += `&pdf=${pdfString}`
                + `&pageNo=${pageNo}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.StreamToString(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 合并PDF列表接口
     * @param url 
     * @param pdfListXml 
     * @param outputFilePath 
     * @param savedPdfId 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async AutoConcatPdfList(
        url: string,
        pdfListXml: string,
        outputFilePath: string,
        savedPdfId: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        try {
            let StringBuilderParameter = "functionType=autoConcatPdfList";
            StringBuilderParameter += `&pdfListXml=${pdfListXml}`
                + `&outputFilePath=${outputFilePath}`
                + `&savedPdfId=${savedPdfId}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 合并PDF列表接口
     * @param url 
     * @param pdfListXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async AutoConcatPdfListWithoutOutput(
        url: string,
        pdfListXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        return this.AutoConcatPdfList(url, pdfListXml, "", "", operatorCode, channelCode);
    }

    /**
     * 组织PDF签章报文，发送数据并接收返回流
     * @param url 
     * @param pdf 
     * @param savedPdfId 
     * @param sealStrategyXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async SealAutoPdfWithSavedPdfId(
        url: string,
        pdf: Uint8Array,
        savedPdfId: string,
        sealStrategyXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        try {
            let pdfString = btoa(String.fromCharCode.apply(null, Array.from(pdf)));
            sealStrategyXml = btoa(unescape(encodeURIComponent(sealStrategyXml)));
            let StringBuilderParameter = "functionType=sealAutoPdf";
            StringBuilderParameter += `&pdf=${pdfString}`
                + `&savedPdfId=${savedPdfId}`
                + `&sealStrategyXml=${sealStrategyXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 组织PDF自动化复合签章，发送数据并接收返回流
     * @param url 
     * @param pdfBeanXml 
     * @param multiData 
     * @param compoundSealStrategyXml 
     * @param operatorCode 
     * @param channelBeanXml 
     * @returns 
     */
    async CompoundSealAutoPdf(
        url: string,
        pdfBeanXml: string,
        multiData: string,
        compoundSealStrategyXml: string,
        operatorCode: string,
        channelBeanXml: string
    ): Promise<Uint8Array> {
        try {
            compoundSealStrategyXml = btoa(unescape(encodeURIComponent(compoundSealStrategyXml)));
            channelBeanXml = btoa(unescape(encodeURIComponent(channelBeanXml)));
            let StringBuilderParameter = "functionType=compoundSealAutoPdf";
            StringBuilderParameter += `&pdfBeanXml=${pdfBeanXml}`
                + `&multiData=${multiData}`
                + `&compoundSealStrategyXml=${compoundSealStrategyXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelBeanXml=${channelBeanXml}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 组织PDF自动化复合签章报文，发送数据并接收返回流
     * @param url 
     * @param pdfBeanListXml 
     * @param multiData 
     * @param compoundSealStrategyXml 
     * @param operatorCode 
     * @param channelBeanXml 
     * @returns 
     */
    async CompoundSealAutoPdfList(
        url: string,
        pdfBeanListXml: string,
        multiData: string,
        compoundSealStrategyXml: string,
        operatorCode: string,
        channelBeanXml: string
    ): Promise<string> {
        try {
            compoundSealStrategyXml = btoa(unescape(encodeURIComponent(compoundSealStrategyXml)));
            channelBeanXml = btoa(unescape(encodeURIComponent(channelBeanXml)));
            let StringBuilderParameter = "functionType=compoundSealAutoPdfList";
            StringBuilderParameter += `&pdfBeanListXml=${pdfBeanListXml}`
                + `&multiData=${multiData}`
                + `&compoundSealStrategyXml=${compoundSealStrategyXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelBeanXml=${channelBeanXml}`;

            return CommonMethod.StreamToString(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 组织PDF列表自动化签章分离式接口报文，发送数据并接收返回流
     * @param url 
     * @param pdfBeanListXml 
     * @param multiData 
     * @param compoundSealStrategyXml 
     * @param operatorCode 
     * @param channelBeanXml 
     * @returns 
     */
    async CompoundSealAutoPdfListDetached(
        url: string,
        pdfBeanListXml: string,
        multiData: string,
        compoundSealStrategyXml: string,
        operatorCode: string,
        channelBeanXml: string
    ): Promise<string> {
        try {
            compoundSealStrategyXml = btoa(unescape(encodeURIComponent(compoundSealStrategyXml)));
            channelBeanXml = btoa(unescape(encodeURIComponent(channelBeanXml)));
            let StringBuilderParameter = "functionType=compoundSealAutoPdfListDetached";
            StringBuilderParameter += `&pdfBeanListXml=${pdfBeanListXml}`
                + `&multiData=${multiData}`
                + `&compoundSealStrategyXml=${compoundSealStrategyXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelBeanXml=${channelBeanXml}`;

            return CommonMethod.StreamToString(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * Pdf自动化骑缝章签章
     * @param url 
     * @param pdfString 
     * @param crossSealStrategyXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async SealAutoCrossPdf(
        url: string,
        pdfString: string,
        crossSealStrategyXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Response> {
        try {
            let StringBuilderParameter = "functionType=sealAutoCrossPdf";
            StringBuilderParameter += `&pdf=${pdfString}`
                + `&crossSealStrategyXml=${crossSealStrategyXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return this.CallServlet(url, StringBuilderParameter);
        } catch (e) {
            throw e;
        }
    }

    /**
     * PDF添加水印接口
     * @param url 
     * @param pdf 
     * @param waterMarkStrategyXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async AddWaterMark2Pdf(
        url: string,
        pdf: Uint8Array,
        waterMarkStrategyXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        try {
            let pdfString = btoa(String.fromCharCode.apply(null, Array.from(pdf)));
            waterMarkStrategyXml = btoa(unescape(encodeURIComponent(waterMarkStrategyXml)));

            let StringBuilderParameter = "functionType=addWaterMark2Pdf";
            StringBuilderParameter += `&pdf=${pdfString}`
                + `&waterMarkStrategyXml=${waterMarkStrategyXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 组织PDF验章接口报文并发送给服务器，接收返回流
     * @param url 
     * @param pdf 
     * @param verifyStrategyXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async VerifyPdfSeal(
        url: string,
        pdf: Uint8Array,
        verifyStrategyXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<string> {
        try {
            let pdfString = btoa(String.fromCharCode.apply(null, Array.from(pdf)));
            verifyStrategyXml = btoa(unescape(encodeURIComponent(verifyStrategyXml)));
            let StringBuilderParameter = "functionType=verifyPdfSeal";
            StringBuilderParameter += `&pdf=${pdfString}`
                + `&verifyStrategyXml=${verifyStrategyXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.StreamToString(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 撤销PDF印章接口并发送数据，接收返回流
     * @param url 
     * @param pdf 
     * @param count 需要撤销印章的个数，0：全部撤销，默认：1
     * @param operatorCode 
     * @returns 
     */
    async RevokePdfSeal(
        url: string,
        pdf: Uint8Array,
        count: number,
        operatorCode: string
    ): Promise<Uint8Array> {
        try {
            let pdfString = btoa(String.fromCharCode.apply(null, Array.from(pdf)));
            let StringBuilderParameter = "functionType=revokePdfSeal";
            StringBuilderParameter += `&pdf=${pdfString}`
                + `&count=${count}`
                + `&operatorCode=${operatorCode}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 申请并下载证书
     * @param url 
     * @param applyCertStrategyXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async ApplyAndDownloadCert(
        url: string,
        applyCertStrategyXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Response> {
        try {
            let StringBuilderParameter = "functionType=applyAndDownloadCert";
            StringBuilderParameter += `&applyCertStrategyXml=${applyCertStrategyXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return this.CallServlet(url, StringBuilderParameter);
        } catch (e) {
            throw e;
        }
    }

    /**
     * 获取时间戳URL
     * @param url 
     * @param flag 
     * @param operatorCode 
     * @returns 
     */
    async GetTimestampURL(
        url: string,
        flag: string,
        operatorCode: string
    ): Promise<Response> {
        try {
            let StringBuilderParameter = "functionType=getTimestampURL";
            StringBuilderParameter += `&flag=${flag}`
                + `&operatorCode=${operatorCode}`;

            return this.CallServlet(url, StringBuilderParameter);
        } catch (e) {
            throw e;
        }
    }

    /**
     * 自助生成图片
     * @param url 
     * @param imageStrategyXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async AutoGenerateImage(
        url: string,
        imageStrategyXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        try {
            let StringBuilderParameter = "functionType=autoGenerateImage";
            StringBuilderParameter += `&imageStrategyXml=${imageStrategyXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * Image转换Pdf功能
     * @param url 
     * @param imageListXml 
     * @param savedPdfId 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async TransformImageToPdf(
        url: string,
        imageListXml: string,
        savedPdfId: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        try {
            let StringBuilderParameter = "functionType=transformImageToPdf";
            StringBuilderParameter += `&imageListXml=${imageListXml}`
                + `&savedPdfId=${savedPdfId}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 组织模板报文并发送数据，接收返回流
     * @param url 
     * @param templateCode 
     * @param operatorCode 
     * @returns 
     */
    async GetTemplate(
        url: string,
        templateCode: string,
        operatorCode: string
    ): Promise<Response> {
        try {
            let StringBuilderParameter = "functionType=getTemplate";
            StringBuilderParameter += `&templateCode=${templateCode}&operatorCode=${operatorCode}`;

            return this.CallServlet(url, StringBuilderParameter);
        } catch (e) {
            throw e;
        }
    }

    /**
     * Pdf自动化骑缝章签章
     * @param url 
     * @param pdf 
     * @param crossSealStrategyXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async SealAutoCrossPdfWithBytes(
        url: string,
        pdf: Uint8Array,
        crossSealStrategyXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<Uint8Array> {
        try {
            let pdfString = btoa(String.fromCharCode.apply(null, Array.from(pdf)));
            crossSealStrategyXml = btoa(unescape(encodeURIComponent(crossSealStrategyXml)));

            let StringBuilderParameter = "functionType=sealAutoCrossPdf";
            StringBuilderParameter += `&pdf=${pdfString}&crossSealStrategyXml=${crossSealStrategyXml}&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.WriteStreamToByte(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 下载印章
     * @param url 
     * @param requestXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async DownloadSeal(
        url: string,
        requestXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<string> {
        try {
            requestXml = btoa(unescape(encodeURIComponent(requestXml)));
            let StringBuilderParameter = "functionType=downloadSeal";
            StringBuilderParameter += `&requestXml=${requestXml}`
                + `&operatorCode=${operatorCode}`
                + `&channelCode=${channelCode}`;

            return CommonMethod.StreamToString(await this.CallServlet(url, StringBuilderParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 上传签章日志
     * @param url 
     * @param sealLogBeanXml 
     * @param operatorCode 
     * @param channelCode 
     * @returns 
     */
    async UploadSealLog(
        url: string,
        sealLogBeanXml: string,
        operatorCode: string,
        channelCode: string
    ): Promise<string> {
        try {
            sealLogBeanXml = btoa(unescape(encodeURIComponent(sealLogBeanXml)));
            let stringBufferParameter = "functionType=uploadSealLog";
            stringBufferParameter += `&sealLogBeanXml=${sealLogBeanXml}`
                + `&channelCode=${channelCode}&operatorCode=${operatorCode}`;

            return CommonMethod.StreamToString(await this.CallServlet(url, stringBufferParameter));
        } catch (e) {
            throw e;
        }
    }

    /**
     * 根据URL创建连接并发送数据，接收数据并返回
     * @param url 服务器地址
     * @param buffer 发送给服务器的参数
     * @returns 服务器返回流
     */
    async CallServlet(url: string, buffer: string): Promise<Response> {
        try {
            const response = await fetch(url, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded',
                },
                body: buffer,
                signal: AbortSignal.timeout(10000),
            });

            if (!response.ok) {
                throw new Error(`HTTP error! status: ${response.status}`);
            }

            return response;
        } catch (e) {
            throw new Error((e as Error).message);
        }
    }
}