/**
 * 生成到PDF内容里的信息类
 */
class TextBean {
    private textValue: string;
    private textValueHash: string;
    private textFontSize: string | null;
    private textColor: string | null;
    private type: string;
    private pageNo: number;
    private lx: number;
    private ly: number;
    private keyword: string | null;
    private locationStyle: string | null;
    private offsetX: number;
    private offsetY: number;
    private templateIndex: number;

    constructor(textValue: string, textValueHash: string, pageNoOrKeyword: number | string, lxOrLocationStyle?: number | string, lyOrOffsetX?: number, offsetY?: number, templateIndex?: number) {
        this.textValue = textValue;
        this.textValueHash = textValueHash;
        this.textFontSize = null;
        this.textColor = null;
        this.keyword = null;
        this.locationStyle = null;

        if (typeof pageNoOrKeyword === 'number') {
            // 处理前两种构造函数
            this.type = "2";
            this.pageNo = pageNoOrKeyword;
            this.lx = lxOrLocationStyle as number;
            this.ly = lyOrOffsetX as number;
            this.templateIndex = templateIndex !== undefined ? templateIndex : 0;
        } else {
            // 处理后两种构造函数
            this.type = "3";
            this.keyword = pageNoOrKeyword;
            this.locationStyle = lxOrLocationStyle as string;
            this.offsetX = lyOrOffsetX as number;
            this.offsetY = offsetY as number;
            this.templateIndex = templateIndex !== undefined ? templateIndex : 0;
            this.pageNo = 1;
            this.lx = 0;
            this.ly = 0;
        }
    }

    /**
     * 获取文本值
     */
    get TextValue(): string {
        return this.textValue;
    }

    /**
     * 设置文本值
     */
    set TextValue(textValue: string) {
        this.textValue = textValue;
    }

    /**
     * 获取文本值的哈希值
     */
    get TextValueHash(): string {
        return this.textValueHash;
    }

    /**
     * 设置文本值的哈希值
     */
    set TextValueHash(textValueHash: string) {
        this.textValueHash = textValueHash;
    }

    /**
     * 获取类型
     */
    get Type(): string {
        return this.type;
    }

    /**
     * 设置类型
     */
    set Type(type: string) {
        this.type = type;
    }

    /**
     * 获取页码
     */
    get PageNo(): number {
        return this.pageNo;
    }

    /**
     * 设置页码
     */
    set PageNo(pageNo: number) {
        this.pageNo = pageNo;
    }

    /**
     * 获取 x 坐标
     */
    get Lx(): number {
        return this.lx;
    }

    /**
     * 设置 x 坐标
     */
    set Lx(lx: number) {
        this.lx = lx;
    }

    /**
     * 获取 y 坐标
     */
    get Ly(): number {
        return this.ly;
    }

    /**
     * 设置 y 坐标
     */
    set Ly(ly: number) {
        this.ly = ly;
    }

    /**
     * 获取关键字
     */
    get Keyword(): string | null {
        return this.keyword;
    }

    /**
     * 设置关键字
     */
    set Keyword(keyword: string | null) {
        this.keyword = keyword;
    }

    /**
     * 获取定位样式
     */
    get LocationStyle(): string | null {
        return this.locationStyle;
    }

    /**
     * 设置定位样式
     */
    set LocationStyle(locationStyle: string | null) {
        this.locationStyle = locationStyle;
    }

    /**
     * 获取 x 偏移量
     */
    get OffsetX(): number {
        return this.offsetX;
    }

    /**
     * 设置 x 偏移量
     */
    set OffsetX(offsetX: number) {
        this.offsetX = offsetX;
    }

    /**
     * 获取 y 偏移量
     */
    get OffsetY(): number {
        return this.offsetY;
    }

    /**
     * 设置 y 偏移量
     */
    set OffsetY(offsetY: number) {
        this.offsetY = offsetY;
    }

    /**
     * 获取文本字体大小
     */
    get TextFontSize(): string | null {
        return this.textFontSize;
    }

    /**
     * 设置文本字体大小
     */
    set TextFontSize(textFontSize: string | null) {
        this.textFontSize = textFontSize;
    }

    /**
     * 获取文本颜色
     */
    get TextColor(): string | null {
        return this.textColor;
    }

    /**
     * 设置文本颜色
     */
    set TextColor(textColor: string | null) {
        this.textColor = textColor;
    }

    /**
     * 获取模板索引
     */
    get TemplateIndex(): number {
        return this.templateIndex;
    }

    /**
     * 设置模板索引
     */
    set TemplateIndex(templateIndex: number) {
        this.templateIndex = templateIndex;
    }
}

export default TextBean;