// 模拟引入外部模块，实际需要根据项目情况调整
//import { DependencyObject, Point, UIElement } from 'your-ui-library';
//import { FileInfo } from 'fs';
import Message from "../../Communication/Message";
import LoginUserInfo from "@/views/front/platfrom/common/RunningParameters/LoginUserInfo";
import { ServiceSettings } from "@/views/front/develop/ServiceProxy.Ext/ServiceSettings"
//import PlatformLogger from "@/views/front/platfrom/common/LogSystem/PlatformLogger" //日志登记报错
import { TradeHandle } from '../../ServiceProxy.Ext/ServiceHelper/TradeHandle'
import { CommonMethod } from "@/views/front/develop/PrintTemplate/ElecSeal/Cfca/CommonMethod"
import { PaperlessClient } from "@/views/front/develop/PrintTemplate/ElecSeal/Cfca/PaperlessClient"
import SealUserInfo from "@/views/front/develop/PrintTemplate/ElecSeal/Cfca/SealUserInfo"
import SignLocation from "@/views/front/develop/PrintTemplate/ElecSeal/Cfca/SealUserInfo";

// 签章模式
export enum SealMode {
    None,
    Local,
    Cfca,
    Paperless,
}

// 签章类型
export enum SealKind {
    Biz,
    Receipt,
    Verify,
}

export class ElecSealHelper {
    static _MODE_: SealMode;

    // 静态成员
    // public static _MODE_: SealMode = ServiceSettings.GetConfig("ElecSeal.Mode", (x: string) => {
    //     let mode: SealMode;
    //     const parsed = Object.values(SealMode).find((value) => value.toString() === x);
    //     if (parsed !== undefined) {
    //         mode = parsed as SealMode;
    //         // 若无纸化情况下，检测当前机构匹配情况
    //         if (mode === SealMode.Paperless && !new RegExp(ServiceSettings.GetConfig("PaperlessBranchMatch", "")).test(LoginUserInfo.KinbrNo)) {
    //             return SealMode.None;
    //         }
    //         return mode;
    //     }
    //     return SealMode.None;
    // });
    //hulei 获取签章模式
    // 辅助方法 hulei依赖属性不能照搬
    // public static GetHelper(): ElecSealHelper {
    //     return obj.GetValue(HelperProperty) as ElecSealHelper;
    // }

    // public static SetHelper(obj: DependencyObject, value: ElecSealHelper): void {
    //     obj.SetValue(HelperProperty, value);
    // }

    // public static readonly HelperProperty = {
    //     // 这里只是模拟，实际需要根据 UI 库的依赖属性实现
    //     // 可能需要自定义逻辑来处理动画、样式、绑定等
    // };

    // 实例属性
    public get Mode(): SealMode {
        return null;//ElecSealHelper._MODE_;
    }
    public watch = {
        start: () => { /* 实现开始计时逻辑 */ },
        stop: () => { /* 实现停止计时逻辑 */ },
        restart: () => { /* 实现重新开始计时逻辑 */ },
        ElapsedMilliseconds: 0
    };
    public totaltime = {
        start: () => { /* 实现开始计时逻辑 */ },
        stop: () => { /* 实现停止计时逻辑 */ },
        ElapsedMilliseconds: 0
    };
    public tmplog: string = "";
    private _points: Map<string, [Point, SealKind]> = new Map();

    // 指定位置加盖电子印章，若 kind=null，则代表取消
    public Sign2Point(p: Point, kind: SealKind | null = SealKind.Biz): string {
        const key = `[${p.X},${p.Y}]`;
        if (kind !== null) {
            this._points.set(key, [p, ServiceSettings.GetConfig("ElecSeal.All2Biz", true) ? SealKind.Biz : kind]);
        } else {
            this._points.delete(key);
        }
        return key;
    }

    public KeyCode: string | null = null;

    // 获取编码
    public static async GetCode(p: any): Promise<string> {
        const business = p.BusinessTransaction;
        let defV = "None";
        if (p.SystemSerialNumber) {
            // 采用对系统流水做 16 进制转换
            defV = parseInt(p.SystemSerialNumber).toString(16).toUpperCase();
        }
        // 无纸化模式下，采用新的编码方案
        if (ElecSealHelper._MODE_ === SealMode.Paperless) {
            return defV;
        }
        try {
            const _msg = new Message();
            _msg.Init();
            this.CommSend(_msg, business);
            _msg.Fd16 = "8001";
            _msg.Fd18 = "DZQZ";
            _msg.Fd37 = "CIBAS";
            await this.CallAgn(_msg, "CallAgn");
            if (_msg.Fd12 === "0000") {
                return _msg.Fd27.trim();
            } else {
                TradeHandle.WriteImportantLog("电子印章编码错误", "通讯返回 12 域内容：" + _msg.Fd12, _msg.Fd16, "DoTrade", "1", []);
            }
        } catch (ex) {
            PlatformLogger.SystemErrorInfo("ElecSealHelper.GetCode发生异常！", ex as Error);
        }
        return defV;
    }

    // 初始化公共域
    private static CommSend(message: Message, business: string): void {
        // 开户行机构号
        message.Fd2 = LoginUserInfo.KinbrNo;
        // 机构号
        message.Fd3 = LoginUserInfo.KinbrNo;
        // 业务流水
        message.Fd4 = business;
        // 交易日期
        message.Fd5 = LoginUserInfo.TradeDate;
        // 发生时间
        message.Fd6 = new Date().toLocaleTimeString("zh-CN", { hour12: false, hour: '2-digit', minute: '2-digit', second: '2-digit' }).replace(/:/g, "");
        // 柜员号
        message.Fd7 = LoginUserInfo.TellerNo;
        // 终端号
        message.Fd10 = LoginUserInfo.TtyName;
        message.Fd96_9 = "0100"; // 本行柜面
        message.Fd96_7 = "0";
        message.Fd96_8 = "0";
        message.Fd96_C = "0100"; // 本行柜面
        message.Fd96_D = LoginUserInfo.TradeDate;
        message.Fd96_E = message.Fd6;
    }

    // 和平台通讯 API
    private static async CallAgn(message: Message, transitType: string): Promise<boolean> {
        let returnValue = false;
        const preTradeNo = message.Fd16;
        message.TransitNode = "CallAgnRun"; // 假设 TransitType.CallAgnRun 对应字符串 "CallAgnRun"
        message.BitMapType = "Agn"; // 假设 TellerSystem.Communication.Message.BitMap.Agn 对应字符串 "Agn"
        returnValue = await message.DoTransit();
        if (message.Fd16 !== preTradeNo && message.Fd12 === "0000") {
            message.Fd12 = "COMN";
        }
        return returnValue;
    }

    // 创建电子印章的列表
    public async CreateElecSealList(p: any): Promise<Map<Point, UIElement>> {
        const ret = new Map<Point, UIElement>();
        if (this.Mode === SealMode.None || this._points.size === 0) {
            return ret;
        }
        if (!this.KeyCode) {
            this.KeyCode = await ElecSealHelper.GetCode(p);
        }
        // 增加一个隐藏控件，用于登记 KeyCode
        const textBlock = new UIElement(); // 实际需要根据 UI 库创建具体元素
        ret.set(new Point(0, 0), textBlock);
        this._points.forEach(([point, kind]) => {
            const codekey = this.GenKeyCode(kind);
            const seal = new UIElement(); // 实际需要根据 UI 库创建具体的印章元素
            if (codekey.endsWith("-R") || this.Mode === SealMode.Paperless) {
                // 无纸化模式下，也默认不显示签章编码
                // 假设 seal 有 Display 属性
                (seal as any).Display = false;
            }
            ret.set(point, seal);
        });
        return ret;
    }

    private GenKeyCode(kind: SealKind): string {
        let code = this.KeyCode || "";
        switch (kind) {
            case SealKind.Biz:
                code += "-B";
                break;
            case SealKind.Receipt:
                code += "-R";
                break;
            case SealKind.Verify:
                code += "-V";
                break;
            default:
                break;
        }
        return code;
    }

    // 是否需要加签，打印时；暂仅支持 cfca
    public get NeedSignature(): boolean {
        return this.Mode === SealMode.Cfca && this._points.size > 0;
    }

    //// C:\Program Files (x86)\Reference Assemblies\Microsoft\Framework\.NETFramework\v4.0\mscorlib.dll hulei
    // 处理打印的 xps 文件，执行加签操作
    public async Process2Signature(xps: FileInfo): Promise<boolean> {
        this.watch.start();
        this.totaltime.start();
        this.tmplog += "##**加签开始: ";
        if (!this.NeedSignature) {
            return false;
        }
        let data: Buffer | null = null;
        try {
            data = await new Promise((resolve, reject) => {
                require('fs').readFile(xps.path, (err: any, buffer: Buffer<ArrayBufferLike> | PromiseLike<Buffer<ArrayBufferLike>>) => {
                    if (err) reject(err);
                    else resolve(buffer);
                });
            });
            // xps 转换成 pdf
            data = await this.XpsConvert(data);
            this.watch.stop();
            this.tmplog += `##**XPS转换成PDF:${this.watch.ElapsedMilliseconds}`;
            // 按照印章的类型进行多次加签操作
            const kinds = Array.from(new Set(Array.from(this._points.values()).map(([_, kind]) => kind)));
            switch (this.Mode) {
                case SealMode.Cfca:
                    data = await this.PdfSeal_Cfca(data, kinds);
                    break;
                default:
                    throw new Error(`暂不支持的印章模型：${this.Mode}`);
            }
            // pdf 转回 xps
            this.watch.restart();
            data = await this.XpsConvert(data, false);
            this.watch.stop();
            this.tmplog += `##**PDF转换成XPS:${this.watch.ElapsedMilliseconds}`;
            this.totaltime.stop();
            this.tmplog += `##**凭证总耗时:${this.totaltime.ElapsedMilliseconds}`;
            TradeHandle.WriteImportantLog("无纸化耗时日志", this.tmplog, "无纸化凭证打印");
            this.tmplog = "";
            await new Promise((resolve, reject) => {
                require('fs').writeFile(xps.path, data, (err: any) => {
                    if (err) reject(err);
                    else resolve(true);
                });
            });
            return true;
        } catch (ex) {
            PlatformLogger.SystemErrorInfo("xps 加电子印章发生错误！", ex as Error);
        } finally {
            // 转换后的数据 不上传
            // 可以在这里添加注释掉的上传逻辑
        }
        return false;
    }

    private async PdfSeal_Cfca(pdf: Buffer, kinds: SealKind[]): Promise<Buffer> {
        if (!pdf || pdf.length === 0 || !kinds || kinds.length === 0) {
            throw new Error("PdfSeal_Cfca:参数异常，无法进行有效处理！");
        }
        if (kinds.length === 1) {
            return this.PdfSeal_Cfca(pdf, [kinds[0]]);
        }
        try {
            PlatformLogger.SystemInfo("ElecSealHelper.PdfSeal_Cfca:进入加签！复合签章");
            this.watch.restart();
            const list = kinds.map((kind) => {
                const keyCode = this.GenKeyCode(kind);
                let cp: string[];
                if (kind === SealKind.Receipt) {
                    cp = ServiceSettings.GetConfig(`ElecSeal.Cfca.${LoginUserInfo.KinbrNo.substring(0, 3)}.${kind.toString()}`).split('|');
                } else {
                    cp = ServiceSettings.GetConfig(`ElecSeal.Cfca.${LoginUserInfo.KinbrNo}.${kind.toString()}`).split('|');
                }
                const sealCode = cp[0];
                const sealPassword = cp[1];
                const sealUserInfo = new SealUserInfo(LoginUserInfo.TellerNo, LoginUserInfo.KinbrNo, "");
                // 3 = 关键字签章
                let signLocation: SignLocation;
                if (kind === SealKind.Biz) {
                    signLocation = new SignLocation(keyCode, "C", 0, 20);
                } else {
                    signLocation = new SignLocation(keyCode, "C", 0, 0);
                }
                // 生成签章策略文件
                const sealStrategyXml = CommonMethod.CreateSealStrategyXml(sealCode, sealPassword, "", "font-size:13;x-ratio:0.0;y-ratio:0.0", sealUserInfo, signLocation);
                return sealStrategyXml;
            });
            // 获取参数
            const url_prefix = ServiceSettings.GetConfig("ElecSeal.Cfca.Url_Prefix");
            const channelCode = ServiceSettings.GetConfig("ElecSeal.Cfca.ChannelCode");
            const operatorCode = ServiceSettings.GetConfig(`ElecSeal.Cfca.OperatorCode_${LoginUserInfo.KinbrNo.substring(0, 3)}`);
            const paperlessClient = new PaperlessClient();
            const pdfBeanXml = CommonMethod.BuildPdfBeanXml({ pdf: pdf.toString('base64') });
            let multiData = "";
            const compoundSealStrategyXml = `<List>${list.join("")}</List>`;
            const channelBeanXml = `<Channel><ChannelCode>${channelCode}</ChannelCode><MerchantNo></MerchantNo><MerchantName></MerchantName><TerminalNo></TerminalNo></Channel>`;
            this.watch.stop();
            this.tmplog += `  多签章通讯前数据准备完成:${this.watch.ElapsedMilliseconds}`;
            this.watch.restart();
            PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:多签章通讯开始！");
            const resultByte = await paperlessClient.CompoundSealAutoPdf(url_prefix, pdfBeanXml, multiData, compoundSealStrategyXml, operatorCode, channelBeanXml);
            PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:多签章通讯结束！");
            this.watch.stop();
            this.tmplog += `  多签章通讯:${this.watch.ElapsedMilliseconds}`;
            if (CommonMethod.CheckResultDataValid(resultByte)) {
                try {
                    if (resultByte) {
                        const backup = `/sealpdf/${LoginUserInfo.TradeDate}_${LoginUserInfo.TellerNo}_${this.KeyCode}.PDF`;
                        this.watch.restart();
                        PlatformLogger.SystemInfo(`ElecSealHelper.PrintPdf:开始上传服务器！${this.KeyCode}`);
                        // 上传文件到服务器
                        // 这里需要实现上传逻辑
                        PlatformLogger.SystemInfo(`ElecSealHelper.PrintPdf:上传完成！${this.KeyCode}`);
                        this.watch.stop();
                        this.tmplog += `  多次签章后数据上传至服务器:${this.watch.ElapsedMilliseconds} keycode${this.KeyCode}`;
                    }
                } catch (ex) {
                    PlatformLogger.SystemErrorInfo("xps 加电子印章上传失败！", ex as Error);
                }
            } else {
                const result = Buffer.from(resultByte).toString('utf-8');
                TradeHandle.WriteImportantLog("电子印章", result, "交易");
                PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:发生异常，电子印章加签异常！");
            }
            return resultByte;
        } catch (ex) {
            PlatformLogger.SystemErrorInfo("ElecSealHelper.PrintPdf:发生异常，不支持 pdf 打印！", ex as Error);
            return Buffer.alloc(0);
        }
    }

    // pdf 加签操作
    private async PdfSeal_Cfca(pdf: Buffer, kind: SealKind): Promise<Buffer> {
        this.watch.restart();
        const keyCode = this.GenKeyCode(kind);
        let resultByte: Buffer | null = null;
        PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:进入加签！");
        // 获取参数
        const url_prefix = ServiceSettings.GetConfig("ElecSeal.Cfca.Url_Prefix");
        const channelCode = ServiceSettings.GetConfig("ElecSeal.Cfca.ChannelCode");
        const operatorCode = ServiceSettings.GetConfig(`ElecSeal.Cfca.OperatorCode_${LoginUserInfo.KinbrNo.substring(0, 3)}`);
        let cp: string[];
        if (kind.toString() === "Receipt") {
            cp = ServiceSettings.GetConfig(`ElecSeal.Cfca.${LoginUserInfo.KinbrNo.substring(0, 3)}.${kind.toString()}`).split('|');
        } else {
            cp = ServiceSettings.GetConfig(`ElecSeal.Cfca.${LoginUserInfo.KinbrNo}.${kind.toString()}`).split('|');
        }
        const sealCode = cp[0];
        const sealPassword = cp[1];

        // pdf 加签
        try {
            const paperlessClient = new PaperlessClient();
            const sealUserInfo = new SealUserInfo(LoginUserInfo.TellerNo, LoginUserInfo.KinbrNo, "");

            // 3 = 关键字签章
            let signLocation: SignLocation;
            if (kind.toString() === "Biz") {
                signLocation = new SignLocation(keyCode, "C", 0, 20);
            } else {
                signLocation = new SignLocation(keyCode, "C", 0, 0);
            }

            // 生成签章策略文件
            const sealStrategyXml = CommonMethod.CreateSealStrategyXml(sealCode, sealPassword, "", "font-size:13;x-ratio:0.0;y-ratio:0.0", sealUserInfo, signLocation);
            this.watch.stop();
            this.tmplog += `  签章通讯前数据准备完成:${this.watch.ElapsedMilliseconds}`;
            this.watch.restart();
            PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:签章通讯开始！");
            resultByte = await paperlessClient.SealAutoPdf(url_prefix, pdf, sealStrategyXml, operatorCode, channelCode);
            PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:签章通讯结束！");
            this.watch.stop();
            this.tmplog += `  签章通讯:${this.watch.ElapsedMilliseconds}`;
            if (CommonMethod.CheckResultDataValid(resultByte)) {
                try {
                    if (resultByte) {
                        const backup = `/sealpdf/${LoginUserInfo.TradeDate}_${LoginUserInfo.TellerNo}_${this.KeyCode}.PDF`;
                        this.watch.restart();
                        PlatformLogger.SystemInfo(`ElecSealHelper.PrintPdf:开始上传服务器！${this.KeyCode}`);
                        // 上传文件到服务器
                        // 这里需要实现上传逻辑
                        PlatformLogger.SystemInfo(`ElecSealHelper.PrintPdf:上传完成！${this.KeyCode}`);
                        this.watch.stop();
                        this.tmplog += `  签章后数据上传至服务器:${this.watch.ElapsedMilliseconds} keycode${this.KeyCode}`;
                    }
                } catch (ex) {
                    PlatformLogger.SystemErrorInfo("xps 加电子印章上传失败！", ex as Error);
                }
            } else {
                const result = Buffer.from(resultByte).toString('utf-8');
                TradeHandle.WriteImportantLog("电子印章", result, "交易");
                PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:发生异常，电子印章加签异常！");
            }
            return resultByte;
        } catch (ex) {
            PlatformLogger.SystemErrorInfo("ElecSealHelper.PrintPdf:发生异常，不支持 pdf 打印！", ex as Error);
            return Buffer.alloc(0);
        }
    }

    // 触发补打操作
    public static async Reprint(xps: FileInfo, bizNo: string): Promise<boolean> {
        const helper = new ElecSealHelper();
        let title = "";
        let size = { width: 0, height: 0 };
        if (helper.Mode !== SealMode.None && helper.Mode !== SealMode.Local) {
            // 分析补打文件，捕获加签的位置
            const { XpsDocument } = require('your-xps-library');
            const xpsDocument = new XpsDocument(xps.path, 'r');
            try {
                // 获取凭证参数
                title = xpsDocument.CoreDocumentProperties.Title;
                // 解析 xps 文件
                const reader = xpsDocument.FixedDocumentSequenceReader;
                const page = reader.FixedDocuments[0].FixedPages[0];
                const xrdr = page.XmlReader;
                while (await xrdr.Read()) {
                    switch (xrdr.NodeType) {
                        case 1: // 对应 XmlNodeType.Element
                            // 命中页面，提取凭证尺寸
                            if (xrdr.Name === "FixedPage") {
                                size.width = parseFloat(xrdr.getAttribute("Width"));
                                size.height = parseFloat(xrdr.getAttribute("Height"));
                            } else if (xrdr.Name === "Glyphs") {
                                const key = xrdr.getAttribute("UnicodeString");
                                // 命中隐藏编号，提取 KeyCode
                                if (key && key.startsWith("KeyCode=>")) {
                                    helper.KeyCode = key.substring(9);
                                }
                                // 命中印章编号，分析印章位置和类型
                                else if (helper.KeyCode && key && key.startsWith(helper.KeyCode)) {
                                    const point = { X: parseFloat(xrdr.getAttribute("OriginX")), Y: parseFloat(xrdr.getAttribute("OriginY")) };
                                    // 判断 SealKind
                                    let kind = SealKind.Biz;
                                    if (key.endsWith("-R")) {
                                        kind = SealKind.Receipt;
                                    } else if (key.endsWith("-V")) {
                                        kind = SealKind.Verify;
                                    }
                                    helper.Sign2Point(point, kind);
                                }
                            }
                            break;
                        default:
                            break;
                    }
                }
            } finally {
                xpsDocument.Close();
            }
        }
        // 补打是否需要走加签流程
        const flag = helper._points.size > 0;
        // 预览，走加签流程则不送打印机
        // 这里需要实现 PrintPreviewWindow 相关逻辑
        // const dialog = new PrintPreviewWindow(xps, flag ? PrintDevice.Screen : PrintDevice.Printer, true);
        // dialog.Owner = Application.Current.MainWindow;
        // dialog.ShowInTaskbar = false;
        // await dialog.ShowDialog();
        // if (!flag) {
        //     return dialog.IsPrinted;
        // }
        // 获取新的打印文件
        const xps_re = new FileInfo(require('path').join("print", xps.name));
        // 补打文件不存在，客户取消
        if (!xps_re.existsSync()) {
            return false;
        }
        // 加签操作
        await helper.Process2Signature(xps_re);
        // 这里需要实现 PrintManager 相关逻辑
        // PrintManager.SendToPrinter(xps_re, size.width, size.height, title);
        return false;
    }

    // Xps2Pdf 转换，由参数指定流动方向
    public static async XpsConvert(data: Buffer, xps2Pdf: boolean = true): Promise<Buffer> {
        // 这里需要实现 Spire.Pdf 相关的转换逻辑
        // 以下是模拟实现，实际需要使用对应的库
        return new Promise((resolve) => {
            // 简单返回原始数据，需要替换为实际转换逻辑
            resolve(data);
        });
    }

    // pdf 合成处理
    public static async Composite(data: Buffer, pos: Map<Buffer, Point[]>): Promise<Buffer> {
        if (!pos || pos.size === 0) {
            return data;
        }
        // 这里需要实现 Spire.Pdf 相关的合成逻辑
        // 以下是模拟实现，实际需要使用对应的库
        return new Promise((resolve) => {
            // 简单返回原始数据，需要替换为实际合成逻辑
            resolve(data);
        });
    }

    // 获取机构签署情况
    public GenBranchSignature(): Map<SealKind, string> {
        const result = new Map<SealKind, string>();
        if (this.Mode !== SealMode.Paperless) {
            return result;
        }
        const kinds = Array.from(new Set(Array.from(this._points.values()).map(([_, kind]) => kind)));
        kinds.forEach((kind) => {
            result.set(kind, this.GenKeyCode(kind));
        });
        return result;
    }
}
