﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Spire.Pdf;
using System.Drawing.Printing;
using Spire.Pdf.Print;

namespace TellerSystem.PrintTemplate.Extension
{
    public partial class PdfViewer : Form
    {
        private PdfViewer()
        {
            InitializeComponent();
            this.MouseWheel += new MouseEventHandler(PdfViewer_MouseWheel);
        }

        void PdfViewer_MouseWheel(object sender, MouseEventArgs e)
        {
            var zoom = printPreviewControl.Zoom + (e.Delta > 0 ? 0.1 : -0.1);
            if (zoom > 0)
                printPreviewControl.Zoom = zoom;
        }

        public static bool Open(FileInfo pdf, bool preview = true, string desc = null)
        {
            if (pdf == null || !File.Exists(pdf.FullName)) return false;
            var doc = new PdfDocument();
            try
            {
                doc.LoadFromBytes(File.ReadAllBytes(pdf.FullName));
                doc.PrintSettings.PrintController = new StandardPrintController();//静默打印
                //获取原文档第一页的纸张大小,这里的单位是Point
                SizeF size = doc.Pages[0].Size;
                //需要特别注意的是这里涉及到单位的转换，PaperSize的宽高参数默认单位是百英寸 = n / 72 * 100
                doc.PrintSettings.PaperSize = new PaperSize("Custom", (int)(size.Width / 72 * 100), (int)(size.Height / 72 * 100)) { RawKind = (int)PaperKind.Custom };
                //需要选择ActualSize打印模式
                doc.PrintSettings.SelectSinglePageLayout(PdfSinglePageScalingMode.ActualSize, false);

                var flag = true;
                if (preview)
                {
                    var dialog = new PdfViewer();
                    doc.Preview(dialog.printPreviewControl);
                    flag = dialog.ShowDialog() == DialogResult.OK;
                }
                //发送打印机
                if (flag)
                    doc.Print();
            }
            finally
            {
                doc.Close();
            }
            return true;
        }

        private void button_Print_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        private void button_Cancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }
    }
}
