﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Platform.Common.BasicFunctions;
using Newtonsoft.Json;

namespace TellerSystem.PrintTemplate.Extension
{
    public class PrintAttributes : DependencyObject
    {
        #region 柜员流水
        /// <summary>
        /// 柜员流水号
        /// </summary>
        public string SerialNumber
        {
            get { return (string)GetValue(SerialNumberProperty); }
            set { SetValue(SerialNumberProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SerialNumber.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SerialNumberProperty =
            DependencyProperty.Register("SerialNumber", typeof(string), typeof(PrintAttributes), new UIPropertyMetadata(null));
        #endregion

        #region 系统前端流水号
        /// <summary>
        /// 系统前端流水号
        /// </summary>
        public string SystemSerialNumber
        {
            get { return (string)GetValue(SystemSerialNumberProperty); }
            set { SetValue(SystemSerialNumberProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SerialNumber.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SystemSerialNumberProperty =
            DependencyProperty.Register("SystemSerialNumber", typeof(string), typeof(PrintAttributes), new UIPropertyMetadata(null));
        #endregion

        #region 业务流水
        /// <summary>
        /// 业务流水号
        /// </summary>
        public string BusinessTransaction
        {
            get { return (string)GetValue(BusinessTransactionProperty); }
            set { SetValue(BusinessTransactionProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SerialNumber.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty BusinessTransactionProperty =
            DependencyProperty.Register("BusinessTransaction", typeof(string), typeof(PrintAttributes), new UIPropertyMetadata(null));
        #endregion

        #region 票号
        /// <summary>
        /// 票号
        /// </summary>
        public string PrintNumber
        {
            get { return (string)GetValue(PrintNumberProperty); }
            set { SetValue(PrintNumberProperty, value); }
        }

        // Using a DependencyProperty as the backing store for PrintNumber.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty PrintNumberProperty =
            DependencyProperty.Register("PrintNumber", typeof(string), typeof(PrintAttributes), new UIPropertyMetadata("CCCCCCC"));
        #endregion

        #region 授权人
        /// <summary>
        /// 交易授权人
        /// </summary>
        public string Authentic
        {
            get { return (string)GetValue(AuthenticProperty); }
            set { SetValue(AuthenticProperty, value); }
        }

        // Using a DependencyProperty as the backing store for AuthTeller.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty AuthenticProperty =
            DependencyProperty.Register("Authentic", typeof(string), typeof(PrintAttributes), new UIPropertyMetadata(string.Empty));
        #endregion

        #region 授权柜员号
        /// <summary>
        /// 授权柜员号
        /// </summary>
        public string Authno
        {
            get { return (string)GetValue(AuthnoProperty); }
            set { SetValue(AuthnoProperty, value); }
        }

        // Using a DependencyProperty as the backing store for AuthTeller.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty AuthnoProperty =
            DependencyProperty.Register("Authno", typeof(string), typeof(PrintAttributes), new UIPropertyMetadata(string.Empty));
        #endregion

        #region 复核人
        /// <summary>
        /// 交易复核人
        /// </summary>
        public string Review
        {
            get { return (string)GetValue(ReviewProperty); }
            set { SetValue(ReviewProperty, value); }
        }

        // Using a DependencyProperty as the backing store for AuthTeller.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ReviewProperty =
            DependencyProperty.Register("Review", typeof(string), typeof(PrintAttributes), new UIPropertyMetadata(string.Empty));
        #endregion

        #region 交易信息
        /// <summary>
        /// 所属交易的TradeModel对象
        /// </summary>
        public TradeModel TradeModel
        {
            get { return (TradeModel)GetValue(TradeModelProperty); }
            set { SetValue(TradeModelProperty, value); }
        }

        // Using a DependencyProperty as the backing store for TradeModel.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TradeModelProperty =
            DependencyProperty.Register("TradeModel", typeof(TradeModel), typeof(PrintAttributes), new UIPropertyMetadata(null));
        #endregion

        //#region 主机流水号
        ///// <summary>
        ///// 主机流水号
        ///// </summary>
        //public string HostNumber
        //{
        //    get { return (string)GetValue(HostNumberProperty); }
        //    set { SetValue(HostNumberProperty, value); }
        //}

        //// Using a DependencyProperty as the backing store for SerialNumber.  This enables animation, styling, binding, etc...
        //public static readonly DependencyProperty HostNumberProperty =
        //    DependencyProperty.Register("HostNumber", typeof(string), typeof(PrintAttributes), new UIPropertyMetadata(null));
        //#endregion

        //#region 核心流水号
        ///// <summary>
        ///// 核心流水号
        ///// </summary>
        //public string CoreNumber
        //{
        //    get { return (string)GetValue(HCoreNumberProperty); }
        //    set { SetValue(HCoreNumberProperty, value); }
        //}

        //// Using a DependencyProperty as the backing store for SerialNumber.  This enables animation, styling, binding, etc...
        //public static readonly DependencyProperty HCoreNumberProperty =
        //    DependencyProperty.Register("CoreNumber", typeof(string), typeof(PrintAttributes), new UIPropertyMetadata(null));
        //#endregion

        /// <summary>
        /// 序列化
        /// </summary>
        /// <returns></returns>
        public string Serialize()
        {
            var data = new PrintAttributesData();
            data.SystemSerialNumber = this.SystemSerialNumber;
            data.BusinessTransaction = this.BusinessTransaction;
            data.PrintNumber = this.PrintNumber;
            data.Authentic = this.Authentic;
            data.Authno = this.Authno;
            data.Review = this.Review;
            data.TradeModel = new TradeModelData()
            {
                Action = this.TradeModel.Action,
                Code = this.TradeModel.Code,
                GUID = this.TradeModel.GUID,
                Metadata = this.TradeModel.Metadata,
                Name = this.TradeModel.Name,
                ParentID = this.TradeModel.ParentID,
                Right = this.TradeModel.Right,
                Type = this.TradeModel.Type
            };
            return JsonConvert.SerializeObject(data);
        }

        /// <summary>
        /// 反序列化
        /// </summary>
        /// <param name="json"></param>
        public void Deserialize(string json)
        {
            PrintAttributesData result = JsonConvert.DeserializeObject<PrintAttributesData>(json);
            this.SystemSerialNumber = result.SystemSerialNumber;
            this.BusinessTransaction = result.BusinessTransaction;
            this.PrintNumber = result.PrintNumber;
            this.Authentic = result.Authentic;
            this.Authno = result.Authno;
            this.Review = result.Review;
            this.TradeModel = new TradeModel();
            this.TradeModel.Action = result.TradeModel.Action;
            this.TradeModel.Code = result.TradeModel.Code;
            this.TradeModel.GUID = result.TradeModel.GUID;
            this.TradeModel.Name = result.TradeModel.Name;
            this.TradeModel.ParentID = result.TradeModel.ParentID;
            this.TradeModel.Type = result.TradeModel.Type;
            if (result.TradeModel.Metadata != null)
            {
                this.TradeModel.SetMetadata(result.TradeModel.Metadata);
            }
            if (result.TradeModel.Right != null)
            {
                this.TradeModel.SetRight(result.TradeModel.Right);
            }
        }
    }

    /// <summary>
    /// 打印数据实体
    /// </summary>
    public class PrintAttributesData
    {
        public string SystemSerialNumber { get; set; }
        public string BusinessTransaction { get; set; }
        public string PrintNumber { get; set; }
        public string Authentic { get; set; }
        public string Authno { get; set; }
        public string Review { get; set; }
        public TradeModelData TradeModel { get; set; }
    }

    /// <summary>
    /// 
    /// </summary>
    public class TradeModelData
    {
        public string Action { get; set; }
        public string Code { get; set; }
        public string GUID { get; set; }
        public IDictionary<string, string> Metadata { get; set; }
        public string Name { get; set; }
        public string ParentID { get; set; }
        public string[] Right { get; set; }
        public string Type { get; set; }
    }
}
