// 由于 TypeScript 没有 DependencyProperty 的概念，这里简化处理，使用普通的类属性
// 假设 TradeModel 类型已经定义
interface TradeModel {
    Action: string;
    Code: string;
    GUID: string;
    Metadata: { [key: string]: string };
    Name: string;
    ParentID: string;
    Right: string[];
    Type: string;
    SetMetadata(metadata: { [key: string]: string }): void;
    SetRight(right: string[]): void;
}

// 打印数据实体
class PrintAttributesData {
    SystemSerialNumber: string = "";
    BusinessTransaction: string = "";
    PrintNumber: string = "";
    Authentic: string = "";
    Authno: string = "";
    Review: string = "";
    TradeModel: TradeModelData = new TradeModelData();
}

// 交易模型数据
class TradeModelData {
    Action: string = "";
    Code: string = "";
    GUID: string = "";
    Metadata: { [key: string]: string } | null = null;
    Name: string = "";
    ParentID: string = "";
    Right: string[] | null = null;
    Type: string = "";
}

// 打印属性类
class PrintAttributes {
    SerialNumber: string = "";
    SystemSerialNumber: string = "";
    BusinessTransaction: string = "";
    PrintNumber: string = "CCCCCCC";
    Authentic: string = "";
    Authno: string = "";
    Review: string = "";
    TradeModel: TradeModel | null = null;

    /**
     * 序列化
     */
    Serialize(): string {
        const data = new PrintAttributesData();
        data.SystemSerialNumber = this.SystemSerialNumber;
        data.BusinessTransaction = this.BusinessTransaction;
        data.PrintNumber = this.PrintNumber;
        data.Authentic = this.Authentic;
        data.Authno = this.Authno;
        data.Review = this.Review;
        if (this.TradeModel) {
            data.TradeModel = {
                Action: this.TradeModel.Action,
                Code: this.TradeModel.Code,
                GUID: this.TradeModel.GUID,
                Metadata: this.TradeModel.Metadata,
                Name: this.TradeModel.Name,
                ParentID: this.TradeModel.ParentID,
                Right: this.TradeModel.Right,
                Type: this.TradeModel.Type
            };
        }
        return JSON.stringify(data);
    }

    /**
     * 反序列化
     * @param json - 要反序列化的 JSON 字符串
     */
    Deserialize(json: string): void {
        const result: PrintAttributesData = JSON.parse(json);
        this.SystemSerialNumber = result.SystemSerialNumber;
        this.BusinessTransaction = result.BusinessTransaction;
        this.PrintNumber = result.PrintNumber;
        this.Authentic = result.Authentic;
        this.Authno = result.Authno;
        this.Review = result.Review;
        this.TradeModel = {
            Action: result.TradeModel.Action,
            Code: result.TradeModel.Code,
            GUID: result.TradeModel.GUID,
            Metadata: result.TradeModel.Metadata || {},
            Name: result.TradeModel.Name,
            ParentID: result.TradeModel.ParentID,
            Right: result.TradeModel.Right || [],
            Type: result.TradeModel.Type,
            SetMetadata(metadata: { [key: string]: string }): void {
                this.Metadata = metadata;
            },
            SetRight(right: string[]): void {
                this.Right = right;
            }
        };
    }
}

export { PrintAttributes, PrintAttributesData, TradeModelData };