﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using TellerSystem.PrintTemplate.OldPrint;
using Platform.Print;
using System.Windows.Media;
using TellerSystem.ServiceProxy.Ext;
using System.ComponentModel;

namespace TellerSystem.PrintTemplate.Extension
{
    /// <summary>
    /// 
    /// </summary>
    public class PrintContainerExt
    {
        #region 打印附加属性
        //public static PrintAttributes GetPrintAttributes(DependencyObject obj)
        //{
        //    return (PrintAttributes)obj.GetValue(PrintAttributesProperty);
        //}

        //public static void SetPrintAttributes(DependencyObject obj, PrintAttributes value)
        //{
        //    obj.SetValue(PrintAttributesProperty, value);
        //}

        //// Using a DependencyProperty as the backing store for PrintAttributes.  This enables animation, styling, binding, etc...
        //public static readonly DependencyProperty PrintAttributesProperty =
        //    DependencyProperty.RegisterAttached("PrintAttributes", typeof(PrintAttributes), typeof(PrintContainerExt), new UIPropertyMetadata(null));
        public static bool GetCreatePrintNum(DependencyObject obj)
        {
            return (bool)obj.GetValue(CreatePrintNumProperty);
        }

        public static void SetCreatePrintNum(DependencyObject obj, bool value)
        {
            obj.SetValue(CreatePrintNumProperty, value);
        }
        /// <summary>
        /// 是否打印票号
        /// </summary>
        public static readonly DependencyProperty CreatePrintNumProperty =
            DependencyProperty.RegisterAttached("CreatePrintNum", typeof(bool), typeof(PrintContainerExt), new UIPropertyMetadata(true));

        public static PrintAttributes GetPrintElement(DependencyObject obj)
        {
            return (PrintAttributes)obj.GetValue(PrintElementProperty);
        }

        public static void SetPrintElement(DependencyObject obj, PrintAttributes value)
        {
            obj.SetValue(PrintElementProperty, value);
        }

        // Using a DependencyProperty as the backing store for PrintElement.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty PrintElementProperty =
            DependencyProperty.RegisterAttached("PrintElement", typeof(PrintAttributes), typeof(PrintContainerExt), new UIPropertyMetadata(null));

        /// <summary>
        /// 是否选择打印机列表
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public static bool GetIsChoosePrinter(PrintContainer obj)
        {
            return (bool)obj.GetValue(IsChoosePrinterProperty);
        }

        public static void SetIsChoosePrinter(PrintContainer obj, bool value)
        {
            obj.SetValue(IsChoosePrinterProperty, value);
        }

        // Using a DependencyProperty as the backing store for IsChoosePrinter.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsChoosePrinterProperty =
            DependencyProperty.RegisterAttached("IsChoosePrinter", typeof(bool), typeof(PrintContainerExt), new UIPropertyMetadata(false));
        #endregion

        #region TextBlock 文本溢出缩放效果
        public static bool GetIsTextNoWrap2Scale(DependencyObject obj)
        {
            return (bool)obj.GetValue(IsTextNoWrap2ScaleProperty);
        }

        public static void SetIsTextNoWrap2Scale(DependencyObject obj, bool value)
        {
            obj.SetValue(IsTextNoWrap2ScaleProperty, value);
        }

        // Using a DependencyProperty as the backing store for IsTextNoWrap2Scale.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsTextNoWrap2ScaleProperty =
            DependencyProperty.RegisterAttached("IsTextNoWrap2Scale", typeof(bool), typeof(PrintContainerExt), new UIPropertyMetadata(false, (d, e) =>
            {
                var textBlock = d as TextBlock;
                if (textBlock == null || DesignerProperties.GetIsInDesignMode(textBlock)) return;
                //检查缩放比例配置项，若为1则不启用效果
                var scale = ServiceSettings.GetConfig("TextNoWrap2Scale", x =>
                {
                    double v;
                    return double.TryParse(x, out v) ? v : 0.5;
                });
                if (scale == 1) return;
                //初始化准备
                if (IsTextNoWrap2Scale_TextEffect == null)
                    IsTextNoWrap2Scale_TextEffect = new TextEffect { PositionCount = int.MaxValue, Transform = new ScaleTransform { ScaleX = scale } };
                descriptor2TextBlock.RemoveValueChanged(textBlock, IsTextNoWrap2Scale_TextChanged);
                if (true.Equals(e.NewValue))
                {
                    //先手动触发一次
                    IsTextNoWrap2Scale_TextChanged(textBlock, null);
                    descriptor2TextBlock.AddValueChanged(textBlock, IsTextNoWrap2Scale_TextChanged);
                }
            }));

        private static DependencyPropertyDescriptor descriptor2TextBlock = DependencyPropertyDescriptor.FromProperty(TextBlock.TextProperty, typeof(TextBlock));

        private static TextEffect IsTextNoWrap2Scale_TextEffect;

        private static void IsTextNoWrap2Scale_Loaded(object sender, RoutedEventArgs e)
        {
            var textBlock = sender as TextBlock;
            if (textBlock == null) return;
            textBlock.Loaded -= IsTextNoWrap2Scale_Loaded;
            //不可见/空文本/允许溢出时，均不处理
            if (textBlock.Visibility != Visibility.Visible || string.IsNullOrEmpty(textBlock.Text) || textBlock.TextWrapping != TextWrapping.NoWrap) return;
            textBlock.TextEffects.Remove(IsTextNoWrap2Scale_TextEffect);
            //检查是否存在文本溢出情况
            FormattedText formattedText = new FormattedText(
                textBlock.Text,
                System.Threading.Thread.CurrentThread.CurrentCulture,
                textBlock.FlowDirection,
                new Typeface(
                    textBlock.FontFamily,
                    textBlock.FontStyle,
                    textBlock.FontWeight,
                    textBlock.FontStretch),
                textBlock.FontSize,
                textBlock.Foreground);
            bool isTrimmed = formattedText.Width > textBlock.DesiredSize.Width;
            //针对Canvas布局，需要特殊计算
            if (!isTrimmed && textBlock.Parent is Canvas)
            {
                var left = Canvas.GetLeft(textBlock);
                isTrimmed = textBlock.DesiredSize.Width > ((textBlock.Parent as Canvas).RenderSize.Width - (double.IsNaN(left) ? 0 : left));
            }
            if (isTrimmed)
                textBlock.TextEffects.Add(IsTextNoWrap2Scale_TextEffect);
        }

        private static void IsTextNoWrap2Scale_TextChanged(object sender, EventArgs e)
        {
            var textBlock = sender as TextBlock;
            if (textBlock == null) return;
            if (textBlock.IsLoaded)
                IsTextNoWrap2Scale_Loaded(textBlock, null);
            else
                textBlock.Loaded += IsTextNoWrap2Scale_Loaded;
        }
        #endregion

        public static FormartMode GetTextBlockFormart(DependencyObject obj)
        {
            return (FormartMode)obj.GetValue(TextBlockFormartProperty);
        }

        public static void SetTextBlockFormart(DependencyObject obj, FormartMode value)
        {
            obj.SetValue(TextBlockFormartProperty, value);
        }

        // Using a DependencyProperty as the backing store for TextBlockFormart.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TextBlockFormartProperty =
            DependencyProperty.RegisterAttached("TextBlockFormart", typeof(FormartMode), typeof(PrintContainerExt), new UIPropertyMetadata(FormartMode.String, new PropertyChangedCallback(FormartTextBlockText)));

        private static void FormartTextBlockText(object sender, DependencyPropertyChangedEventArgs e)
        {
            if ((sender as TextBlock) != null)
                (sender as TextBlock).Loaded += new RoutedEventHandler(TextFormartHelper_TextChanged);
            else if ((sender as PrintStringElement) != null)
            {
                (sender as PrintStringElement).Loaded += new RoutedEventHandler(TextFormartHelper_PrintStringElementTextChanged);
            }
        }

        static void TextFormartHelper_TextChanged(object sender, RoutedEventArgs e)
        {
            string txt = (sender as TextBlock).Text;
            FormartMode formart = PrintContainerExt.GetTextBlockFormart((sender as TextBlock));
            (sender as TextBlock).Loaded -= TextFormartHelper_TextChanged;
            switch (formart)
            {
                case FormartMode.Time:
                    DateTime dtNow = DateTime.Now;
                    if (DateTime.TryParse(txt, out dtNow))
                    {
                        (sender as TextBlock).Text = dtNow.ToString("HH:mm:ss");
                    }
                    else
                    {
                        if (txt.Length == 6)
                        {
                            (sender as TextBlock).Text = txt.Substring(0, 2) + ":" + txt.Substring(2, 2) + ":" + txt.Substring(4, 2);
                        }
                        else
                        {
                            double d = 0;
                            if (double.TryParse(txt, out d))
                            {
                                (sender as TextBlock).Text = DateTime.FromOADate(d).ToString("HH:mm:ss");
                            }
                        }
                    }
                    break;
                case FormartMode.Date:
                    DateTime dt = DateTime.Now;
                    if (DateTime.TryParse(txt, out dt))
                    {
                        (sender as TextBlock).Text = dt.ToString("yyyyMMdd");
                    }
                    else
                    {
                        if (txt.Length == 8)
                        {
                            (sender as TextBlock).Text = txt.Substring(0, 4) + "-" + txt.Substring(4, 2) + "-" + txt.Substring(6, 2);
                        }
                        else
                        {
                            double d = 0;
                            if (double.TryParse(txt, out d))
                            {
                                (sender as TextBlock).Text = DateTime.FromOADate(d).ToString("yyyyMMdd");
                            }
                        }
                    }
                    break;
                case FormartMode.Money:
                    decimal dec = 0;
                    if (Decimal.TryParse(txt, out dec))
                    {
                        (sender as TextBlock).Text = string.Format("¥{0:N}", dec);
                        if (string.IsNullOrEmpty((sender as TextBlock).Text))
                        {
                            (sender as TextBlock).Text = "¥0.00";
                        }
                        else if ((sender as TextBlock).Text.IndexOf('.') == -1)
                        {
                            (sender as TextBlock).Text += ".00";
                        }
                    }
                    break;
                default:
                    break;
            }
            (sender as TextBlock).Loaded += TextFormartHelper_TextChanged;

        }
        static void TextFormartHelper_PrintStringElementTextChanged(object sender, RoutedEventArgs e)
        {
            string txt = (sender as PrintStringElement).Value;
            FormartMode formart = PrintContainerExt.GetTextBlockFormart((sender as PrintStringElement));
            (sender as PrintStringElement).Loaded -= TextFormartHelper_PrintStringElementTextChanged;
            switch (formart)
            {
                case FormartMode.Time:
                    DateTime dtNow = DateTime.Now;
                    if (DateTime.TryParse(txt, out dtNow))
                    {
                        (sender as PrintStringElement).Value = dtNow.ToString("HH:mm:ss");
                    }
                    else
                    {
                        if (txt.Length == 6)
                        {
                            (sender as PrintStringElement).Value = txt.Substring(0, 2) + ":" + txt.Substring(2, 2) + ":" + txt.Substring(4, 2);
                        }
                        else
                        {
                            double d = 0;
                            if (double.TryParse(txt, out d))
                            {
                                (sender as PrintStringElement).Value = DateTime.FromOADate(d).ToString("HH:mm:ss");
                            }
                        }
                    }
                    break;
                case FormartMode.Date:
                    DateTime dt = DateTime.Now;
                    if (DateTime.TryParse(txt, out dt))
                    {
                        (sender as PrintStringElement).Value = dt.ToString("yyyy-MM-dd");
                    }
                    else
                    {
                        if (txt.Length == 8)
                        {
                            (sender as PrintStringElement).Value = txt.Substring(0, 4) + "-" + txt.Substring(4, 2) + "-" + txt.Substring(6, 2);
                        }
                        else
                        {
                            double d = 0;
                            if (double.TryParse(txt, out d))
                            {
                                (sender as PrintStringElement).Value = DateTime.FromOADate(d).ToString("yyyy-MM-dd");
                            }
                        }
                    }
                    break;
                case FormartMode.Money:
                    decimal dec = 0;
                    if (Decimal.TryParse(txt, out dec))
                    {
                        (sender as PrintStringElement).Value = string.Format("{0:N}", dec);
                        //(sender as PrintStringElement).Value = dec.ToString("###,###");
                        if (string.IsNullOrEmpty((sender as PrintStringElement).Value))
                        {
                            (sender as PrintStringElement).Value = "0.00";
                        }
                        else if ((sender as PrintStringElement).Value.IndexOf('.') == -1)
                        {
                            (sender as PrintStringElement).Value += ".00";
                        }
                    }
                    break;
                default:
                    break;
            }
            (sender as PrintStringElement).Loaded += TextFormartHelper_PrintStringElementTextChanged;

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="can"></param>
        /// <param name="curString"></param>
        public static void SetCanvas(Panel can, String curString)
        {
            can.Children.Clear();
            string str = curString.Replace(",", "");
            str = str.Replace(" ", "");
            if (str.Contains('.'))
            {
                var Decimalpoint = str.Split('.')[1].Substring(0, 2);

                str = str.Split('.')[0] + Decimalpoint;
            }
            else //如不含小数点 填充空格
            {
                str += "00";
            }
            str = "¥" + str;
            double width = can.Width;
            double newWidth = width / 11;
            for (int i = 0; i < 11; i++)
            {
                TextBlock newT = new TextBlock();
                newT.Width = newWidth;
                newT.Margin = new Thickness()
                {
                    Left = i * newWidth
                };
                can.Children.Add(newT);
            }
            for (int i = 0; i < str.Length; i++)
            {
                (can.Children[11 - str.Length + i] as TextBlock).Text = str[i].ToString();
            }
        }
        public static string PrintCurString(string str)
        {
            return str;
        }
    }





    public enum FormartMode
    {
        String = 0,
        Date,
        Time,
        Money
    }
}
