// 将日期转换成大写
/**
 * 将 8 位日期字符串按指定类型转换为大写日期
 * @param date - 8 位日期字符串
 * @param type - 转换类型，'y' 表示年，'m' 表示月，'d' 表示日
 * @returns 转换后的大写日期字符串
 */
export function DateToBig(date: string, type: string): string {
    let year = date.substring(0, 4); 
    let month = date.substring(4, 6); 
    let day = date.substring(6, 8); 

    const numMap: { [key: string]: string } = {
        '0': '零',
        '1': '壹',
        '2': '贰',
        '3': '叁',
        '4': '肆',
        '5': '伍',
        '6': '陆',
        '7': '柒',
        '8': '捌',
        '9': '玖'
    };

    const monthMap: { [key: string]: string } = {
        '01': '  零壹',
        '02': '  零贰',
        '03': '  零叁',
        '04': '  零肆',
        '05': '  零伍',
        '06': '  零陆',
        '07': '  零柒',
        '08': '  零捌',
        '09': '  零玖',
        '10': '零壹拾',
        '11': '壹拾壹',
        '12': '壹拾贰'
    };

    const dayMap: { [key: string]: string } = {
        '01': '  零壹',
        '02': '  零贰',
        '03': '  零叁',
        '04': '  零肆',
        '05': '  零伍',
        '06': '  零陆',
        '07': '  零柒',
        '08': '  零捌',
        '09': '  零玖',
        '10': '零壹拾',
        '11': '壹拾壹',
        '12': '壹拾贰',
        '13': '壹拾叁',
        '14': '壹拾肆',
        '15': '壹拾伍',
        '16': '壹拾陆',
        '17': '壹拾柒',
        '18': '壹拾捌',
        '19': '壹拾玖',
        '20': '零贰拾',
        '21': '贰拾壹',
        '22': '贰拾贰',
        '23': '贰拾叁',
        '24': '贰拾肆',
        '25': '贰拾伍',
        '26': '贰拾陆',
        '27': '贰拾柒',
        '28': '贰拾捌',
        '29': '贰拾玖',
        '30': '零叁拾',
        '31': '叁拾壹'
    };

    if (type === 'y') {
        let byear = '';
        for (let i = 0; i < 4; i++) {
            byear += numMap[year[i]];
        }
        return byear;
    } else if (type === 'm') {
        return monthMap[month] || '';
    } else if (type === 'd') {
        return dayMap[day] || '';
    } else {
        return '';
    }
}