﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Shapes;
using System.Windows.Media;
using System.Globalization;

namespace TellerSystem.PrintTemplate.Extension
{
    public class PrintLayoutPanel : Panel
    {
        public PrintLayoutPanel()
        {
            this.Margin = new Thickness(0, 5, 0, 0);
        }

        #region 字段
        //奇数列与偶数列的默认右边距
        private double _marginKV = 10;

        //偶数列占容器总宽度的比例
        private double evenColumn = 0.4;

        //奇数列占容器总宽度的比例
        private double oddColumn = 0.6;

        //元素上下之间的间隔值
        private double _marginTop = 8;

        //各列宽度
        private List<double> _columnSize = new List<double>();

        //用于画线
        List<Line> lineList = new List<Line>();

        //用于存储每行的行高信息
        private Dictionary<int, double> rowDetail = new Dictionary<int, double>();

        private int _columns = 4;

        FormattedText ft;
        #endregion

        #region 属性
        public bool IsShowLine
        {
            get { return (bool)GetValue(IsShowLineProperty); }
            set { SetValue(IsShowLineProperty, value); }
        }

        public static readonly DependencyProperty IsShowLineProperty =
            DependencyProperty.Register("IsShowLine", typeof(bool), typeof(PrintLayoutPanel), new FrameworkPropertyMetadata(false, FrameworkPropertyMetadataOptions.AffectsRender));

        public string Header
        {
            get { return (string)GetValue(HeaderProperty); }
            set { SetValue(HeaderProperty, value); }
        }

        public static readonly DependencyProperty HeaderProperty =
            DependencyProperty.Register("Header", typeof(string), typeof(PrintLayoutPanel), new FrameworkPropertyMetadata(string.Empty, FrameworkPropertyMetadataOptions.AffectsRender));

        public int Columns
        {
            get { return _columns; }
            set { _columns = value; }
        }
        #endregion

        #region  跨列属性
        public static int GetColumnSpan(DependencyObject obj)
        {
            return (int)obj.GetValue(ColumnSpanProperty);
        }
        public static void SetColumnSpan(DependencyObject obj, int value)
        {
            obj.SetValue(ColumnSpanProperty, value);
        }
        public static readonly DependencyProperty ColumnSpanProperty =
            DependencyProperty.RegisterAttached("ColumnSpan", typeof(int), typeof(PrintLayoutPanel), new FrameworkPropertyMetadata(1, FrameworkPropertyMetadataOptions.AffectsParentArrange));
        #endregion

        #region 方法
        /// <summary>
        /// 根据指定元素所在的列数，获取其横坐标
        /// </summary>
        /// <param name="col">元素所在的列数</param>
        /// <param name="ele">指定元素</param>
        /// <returns>该列元素的横坐标</returns>
        private double GetX(int col, FrameworkElement ele)
        {
            double re = 0;
            int span = GetColumnSpan(ele);
            if (span + col > _columnSize.Count)
            {
                span = _columnSize.Count - col;
            }
            double totalWidth = 0;
            if (col == 0)
            {
                for (int i = 0; i < span; i++)
                {
                    totalWidth += _columnSize[i];
                }
                re = totalWidth - ele.DesiredSize.Width - _marginKV;
                return re;
            }
            else
            {
                for (int i = 0; i < col; i++)
                {
                    re += _columnSize[i];
                }
                if (col % 2 == 0)
                {
                    for (int i = 0; i < span; i++)
                    {
                        totalWidth += _columnSize[col + i];
                    }
                    re += (totalWidth - ele.DesiredSize.Width - _marginKV);
                }
                return re;
            }
        }

        protected override Size ArrangeOverride(Size finalSize)
        {
            double fontHeight = 0;
            if (!string.IsNullOrEmpty(Header))
            {
                FontFamily family = new FontFamily("宋体");
                Typeface headerFace = new Typeface(family, FontStyles.Normal, FontWeights.Normal, FontStretches.Normal);
                ft = new FormattedText(Header, CultureInfo.CurrentCulture, FlowDirection.LeftToRight, headerFace, 14, Brushes.Gray);
                fontHeight = ft.Height;
            }
            SetRowDetail(finalSize);
            int currentColumn = 0;
            double lineHight = 0;
            double currentY = _marginTop + fontHeight;
            int totalColumn = _columnSize.Count;
            for (int i = 0; i < Children.Count; i++)
            {
                var ele = Children[i] as FrameworkElement;
                if (ele != null)
                {
                    int cs = GetColumnSpan(ele);
                    if (cs == 0)
                    {
                        cs = 1;
                        SetColumnSpan(ele, 1);
                    }
                    if (cs + currentColumn > totalColumn) //准备换行
                    {
                        currentColumn = 0;
                        if (lineHight == 0)   //整行隐藏
                        {
                            currentY += lineHight;
                        }
                        else
                        {
                            currentY += _marginTop + lineHight;
                        }
                        lineHight = 0;
                    }
                    if (ele.DesiredSize.Height > lineHight)
                    {
                        lineHight = ele.DesiredSize.Height;
                    }
                    ele.Arrange(new Rect(GetX(currentColumn, ele), currentY + GetY(i, ele, rowDetail), ele.DesiredSize.Width, ele.DesiredSize.Height));
                    currentColumn += cs;
                }
            }
            return finalSize;
        }

        protected override Size MeasureOverride(Size availableSize)
        {
            for (int i = 0; i < Children.Count; i++)
            {
                var ele = Children[i] as FrameworkElement;
                if (ele != null)
                {
                    ele.Measure(availableSize);
                }
            }
            if (double.IsInfinity(availableSize.Height))
            {
                availableSize.Height = 200;
            }
            if (double.IsInfinity(availableSize.Width))
            {
                availableSize.Width = 780;
            }
            _columnSize.Clear();
            double total = 0;
            if (Columns % 2 != 0)
            {
                Columns += 1;
                for (int i = 0; i < Columns - 2; i++)
                {
                    double columnWidth = 0;
                    if (i % 2 == 0)
                    {
                        columnWidth = availableSize.Width * (1.0 * 2 / Columns) * evenColumn;

                    }
                    else
                    {
                        columnWidth = availableSize.Width * (1.0 * 2 / Columns) * oddColumn;

                    }
                    total += columnWidth;
                    _columnSize.Add(columnWidth);
                }
                _columnSize.Add(availableSize.Width - total);
                Columns -= 1;
            }
            else
            {
                for (int i = 0; i < Columns; i++)
                {
                    double columnWidth = 0;
                    if (i % 2 == 0)
                    {
                        columnWidth = availableSize.Width * (1.0 * 2 / Columns) * evenColumn;

                    }
                    else
                    {
                        columnWidth = availableSize.Width * (1.0 * 2 / Columns) * oddColumn;
                    }
                    _columnSize.Add(columnWidth);
                }
            }
            double sum = 0;
            lineList.Clear();
            for (int j = 0; j < Columns - 1; j++)
            {
                sum += _columnSize[j];
                Line line = new Line();
                line.X1 = sum;
                line.X2 = sum;
                line.Y1 = 0;
                line.Y2 = availableSize.Height;
                lineList.Add(line);
            }
            return availableSize;
        }

        protected override void OnRender(DrawingContext dc)
        {
            base.OnRender(dc);
            if (IsShowLine)
            {
                foreach (var line in lineList)
                {
                    dc.DrawLine(new Pen(Brushes.RoyalBlue, 1), new Point(line.X1, line.Y1), new Point(line.X2, line.Y2));
                }
            }
            if (!string.IsNullOrEmpty(Header))
            {

                Pen p = new Pen();
                p.Thickness = 0.4;
                p.Brush = Brushes.RoyalBlue;
                Pen p2 = new Pen();
                Rect rect = new Rect(0, ft.Height / 2.0, this.ActualWidth, this.ActualHeight - ft.Height / 2.0);
                RectangleGeometry rg = new RectangleGeometry(rect, 6, 5);
                Rect rect2 = new Rect(8, 1, ft.Width + 2, ft.Height);
                RectangleGeometry rg2 = new RectangleGeometry(rect2, 0, 0);
                dc.DrawGeometry(null, p, rg);
                dc.DrawGeometry(Brushes.White, p2, rg2);
                dc.DrawText(ft, new Point(9, 1));
            }
        }

        private void SetRowDetail(Size size)
        {
            rowDetail.Clear();
            double lineHight = 0;
            //每行最后一个元素的下标
            int lineeleNumber = 0;
            int totalColumn = _columnSize.Count;
            int currentColumn = 0;
            for (int i = 0; i < Children.Count; i++)
            {
                var ele = Children[i] as FrameworkElement;
                if (ele != null)
                {
                    int cs = GetColumnSpan(ele);
                    if (cs == 0)
                    {
                        cs = 1;
                        SetColumnSpan(ele, 1);
                    }
                    if (cs + currentColumn > totalColumn) //准备换行
                    {
                        currentColumn = 0;
                        lineeleNumber = i - 1;
                        rowDetail.Add(lineeleNumber, lineHight);
                        lineHight = 0;
                    }
                    if (ele.DesiredSize.Height > lineHight)
                    {
                        lineHight = ele.DesiredSize.Height;
                    }
                    currentColumn += cs;
                }
            }
            rowDetail.Add(Children.Count - 1, lineHight);
        }

        private double GetY(int index, FrameworkElement ele, Dictionary<int, double> dic)
        {
            double y = 0;
            double height = ele.DesiredSize.Height;

            foreach (KeyValuePair<int, double> kvp in dic)
            {
                if (index <= kvp.Key)
                {
                    y = (kvp.Value - height) / 2.0;
                    break;
                }

            }
            return y; ;
        }
        #endregion

    }
}
