﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using Platform.Common.BasicFunctions;
using Platform.Controls.TradeExtension;
using Platform.Presentation.Interfaces;
using Platform.Presentation.PageFunctions;
using Platform.Print;
using System.Windows.Input;
using Platform.Library;
using System.Printing;
using System.IO;
using System.ComponentModel;
using TellerSystem.PrintTemplate.Base;
using System.Threading;
using Platform.Common.RunningParameters;
using Platform.Common.LogSystem;
using TellerSystem.PrintTemplate.ElecSeal;
using System.Windows.Xps.Packaging;
using System.Windows.Data;

namespace TellerSystem.PrintTemplate.Extension
{
    /// <summary>
    ///     PrintDisplay.xaml 的交互逻辑
    /// </summary>
    public partial class PrintViewer
    {
        #region Field

        /// <summary>
        ///     打印页面容器
        /// </summary>
        private StackPanel _panle;

        /// <summary>
        ///     偏移控制
        /// </summary>
        private TranslateTransform _ttf;

        /// <summary>
        ///     外层容器
        /// </summary>
        private PageDialog _dialog;
        /// <summary>
        ///     每一项宽度
        /// </summary>
        private double _itemWidth; //每个item的高度

        #endregion

        #region Property

        /// <summary>
        ///     当前打印项索引
        /// </summary>
        public int CurrentPrintIndex { get; set; }

        #endregion

        #region DependencyProperty

        // Using a DependencyProperty as the backing store for CurrentPrintItem.  This enables animation, styling, binding, etc...
        /// <summary>
        ///     当前打印项
        /// </summary>
        public static readonly DependencyProperty CurrentPrintItemProperty =
            DependencyProperty.Register("CurrentPrintItem", typeof(PrintItem), typeof(PrintViewer));


        // Using a DependencyProperty as the backing store for MyProperty.  This enables animation, styling, binding, etc...
        /// <summary>
        ///     打印数据源
        /// </summary>
        public static readonly DependencyProperty PrintSourcePropertyProperty =
            DependencyProperty.Register("PrintSource", typeof(List<PrintItem>), typeof(PrintViewer));

        /// <summary>
        ///     当前打印选择项
        /// </summary>
        public PrintItem CurrentPrintItem
        {
            get { return (PrintItem)GetValue(CurrentPrintItemProperty); }
            set { SetValue(CurrentPrintItemProperty, value); }
        }

        /// <summary>
        ///     打印列表数据源
        /// </summary>
        public List<PrintItem> PrintSource
        {
            get { return (List<PrintItem>)GetValue(PrintSourcePropertyProperty); }
            set { SetValue(PrintSourcePropertyProperty, value); }
        }

        #endregion

        #region Ctor

        /// <summary>
        ///     构造函数
        /// </summary>
        public PrintViewer()
        {
            InitializeComponent();
        }

        #endregion

        #region Init

        /// <summary>
        ///     存储初始信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StackPanel_Loaded(object sender, RoutedEventArgs e)
        {
            _panle = sender as StackPanel;
            _ttf = new TranslateTransform();
            if (_panle != null)
            {
                _panle.RenderTransform = _ttf;
                _itemWidth = _panle.ActualWidth;
            }
            CurrentPrintItem = PrintSource[0];
        }

        #endregion

        #region Method

        #region 显示前一项

        //private void GetPrint()
        //{
        //    if (CurrentPrintIndex <= 0)
        //    {
        //        return;
        //    }
        //    CurrentPrintIndex -= 1;
        //    var sb = new Storyboard();
        //    var ks = new DoubleAnimationUsingKeyFrames();
        //    var k1 = new EasingDoubleKeyFrame
        //        {
        //            KeyTime = KeyTime.FromTimeSpan(TimeSpan.FromSeconds(0.5)),
        //            Value = _ttf.X + _itemWidth,
        //            EasingFunction = new SineEase {EasingMode = EasingMode.EaseIn}
        //        };
        //    ks.KeyFrames.Add(k1);
        //    sb.Children.Add(ks);
        //    Storyboard.SetTarget(ks, _panle);
        //    Storyboard.SetTargetProperty(ks, new PropertyPath("(UIElement.RenderTransform).(TranslateTransform.X)"));
        //    lv_Prints.SelectedIndex -= 1;
        //    sb.Begin();
        //}

        #endregion

        /// <summary>
        ///     显示前一项
        /// </summary>
        /// <summary>
        ///     显示后一项
        /// </summary>
        private void GetNext()
        {
            if (CurrentPrintIndex >= PrintSource.Count - 1)
            {
                _dialog.Close();
                _dialog = null;
                return;
            }

            Control u = lv_Prints.ItemContainerGenerator.ContainerFromIndex(CurrentPrintIndex) as Control;
            if (u != null)
            {
                var viewBox = u.Template.FindName("viewBox", u) as Viewbox;
                if (viewBox != null)
                {
                    viewBox.Width = _itemWidth;
                }
            }

            CurrentPrintIndex += 1;
            CurrentPrintItem = PrintSource[CurrentPrintIndex];

            var sb = new Storyboard();
            var ks = new DoubleAnimationUsingKeyFrames();

            var k1 = new EasingDoubleKeyFrame
            {
                KeyTime = KeyTime.FromTimeSpan(TimeSpan.FromSeconds(0.5)),
                Value = _ttf.X - _itemWidth - 3,
                EasingFunction = new SineEase { EasingMode = EasingMode.EaseIn }
            };

            ks.KeyFrames.Add(k1);
            sb.Children.Add(ks);
            Storyboard.SetTarget(ks, _panle);
            Storyboard.SetTargetProperty(ks, new PropertyPath("(UIElement.RenderTransform).(TranslateTransform.X)"));

            sb.Begin();
        }

        /// <summary>
        ///     打开打印在交易中
        /// </summary>
        public void OpenPrintInTrade(IPage page, bool isclosetrade = true)
        {
            if (isclosetrade)
            {
                page.OwnerContainer.CloseAndOpenPage(this);
            }
            else
            {
                page.OwnerContainer.PushPage(page);
            }
        }

        /// <summary>
        ///     打开打印在窗体上
        /// </summary>
        public void OpenPrintOnWindow(IPage page, bool isclosetrade = true)
        {
            _dialog = new PageDialog
            {
                WindowStyle = WindowStyle.None,
                Style = null,
                AllowsTransparency = true,
                Background = new SolidColorBrush(Colors.Transparent),
                SizeToContent = SizeToContent.WidthAndHeight,
                WindowStartupLocation = WindowStartupLocation.CenterScreen
            };
            _dialog.Loaded += (sender, e) =>
            {
                this.Dispatcher.BeginInvoke(new Action(() =>
                {
                    _dialog.Left = (SystemParameters.FullPrimaryScreenWidth - this.ActualWidth) / 2;
                    _dialog.Top = (SystemParameters.FullPrimaryScreenHeight - this.ActualHeight) / 2;
                }), System.Windows.Threading.DispatcherPriority.DataBind, null);
            };
            //var container = _dialog.Content as PageContainer;
            //if (container != null) container.Content = this;
            PrintSource.ForEach(item =>
            {
                if (item.PrintObject != null)
                {
                    item.PreviewContent = new Border
                    {
                        Width = item.PrintObject.Width,
                        Height = item.PrintObject.Height,
                        Background = item.PrintObject.BackgroundImage,
                        Child = new Border { Background = new VisualBrush(item.PrintObject) }
                    };
                }
                else
                {
                    //加载xps文件做预览
                    using (var docs = new XpsDocument(item.XpsFile.FullName, System.IO.FileAccess.Read))
                    {
                        var docv = new DocumentViewer { Document = docs.GetFixedDocumentSequence() };
                        docv.Loaded += docv_Loaded;
                        docs.Close();
                        item.PreviewContent = docv;
                        var p1 = docv.Document.DocumentPaginator.GetPage(0);
                        //补偿一个PrintObject对象
                        item.PrintObject = new PrintContainer { Width = p1.Size.Width, Height = p1.Size.Height, DocumentName = "打印预览" };
                    }
                }
            });
            lv_Prints.ItemsSource = new System.Collections.ObjectModel.ObservableCollection<PrintItem>(PrintSource);
            _dialog.Open(page, this, string.Empty);
        }

        void docv_Loaded(object sender, RoutedEventArgs e)
        {
            var docv = sender as DocumentViewer;
            docv.Loaded -= docv_Loaded;
            //查找合适的尺寸做绑定
            docv.SetBinding(WidthProperty, new Binding("ActualWidth") { Source = lv_Prints, Mode = BindingMode.OneWay });
            docv.SetBinding(HeightProperty, new Binding("ActualHeight") { Source = lv_Prints, Mode = BindingMode.OneWay });
            docv.Margin = new Thickness(-1);
            docv.FitToWidth();
            //隐藏打印按钮
            var print = docv.FirstVisualTreeChild(x => x is Button && (x as Button).Name == "PrintButton") as Button;
            if (print != null) print.Visibility = Visibility.Collapsed;
        }

        #endregion

        #region Command

        private BaseCommand _cancelCommand;
        private BaseCommand _printCommand;

        /// <summary>
        ///     打印命令
        /// </summary>
        public BaseCommand PrintCommand
        {
            get
            {
                return _printCommand ?? (_printCommand = new BaseCommand(obj =>
                    {
                        var pd = new PrintDialog();
                        if (pd.PrintQueue == null)
                        {
                            this.ShowErrorMessageBox("未配置打印机，无法进行打印作业！");
                        }
                        else
                        {
                            if (Printing != null)
                                Printing(this, new PrintEventArgs { PrintData = CurrentPrintItem });
                            var item = CurrentPrintItem;
                            var printFile = item.XpsFile;
                            string printerName = null;
                            //若打印允许自己选择打印机列表，则展示
                            if (PrintContainerExt.GetIsChoosePrinter(item.PrintObject))
                            {
                                var dialog = new PrintDialog();
                                //不选择即按照默认打印机执行
                                if (dialog.ShowDialog() == true)
                                    printerName = dialog.PrintQueue.FullName;
                            }
                            var helper = ElecSealHelper.GetHelper(item.PrintObject);
                            if (helper != null && helper.NeedSignature)
                                this.OwnerContainer.ModalInvoke(() => helper.Process2Signature(printFile), "正在加盖电子签章，请稍候...");
                            PrintManager.SendToPrinter(item.XpsFile, item.PrintObject.Width, item.PrintObject.Height, item.PrintObject.DocumentName, printerName);
                            GetNext();
                        }
                    }));
            }
        }

        /// <summary>
        ///     取消命令
        /// </summary>
        public BaseCommand CancelCommand
        {
            get { return _cancelCommand ?? (_cancelCommand = new BaseCommand(obj => GetNext())); }
        }

        #endregion

        #region Event
        /// <summary>
        /// 打印时触发
        /// </summary>
        public event EventHandler<PrintEventArgs> Printing;
        public class PrintEventArgs : EventArgs
        {
            public PrintItem PrintData { get; set; }
        }
        #endregion

        private void TradePage_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.C:
                    {
                        CancelCommand.Execute(null);
                        break;
                    }
                default: break;
            }
        }
        private bool _isdrag = false;

        private void Viewbox_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            _isdrag = true;
            this.Cursor = Cursors.Hand;
        }

        private void Viewbox_MouseMove(object sender, MouseEventArgs e)
        {
            if (!_isdrag) return;
            var control = (sender as FrameworkElement);
            var parent = control.Parent as FrameworkElement;
            var mousePoint = e.GetPosition(parent);
            control.SetValue(Canvas.LeftProperty, mousePoint.X - control.ActualWidth / 2);
            control.SetValue(Canvas.TopProperty, mousePoint.Y - control.ActualHeight / 2);
        }

        private void Viewbox_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            _isdrag = false;
            this.Cursor = Cursors.Arrow;
        }

        private void Viewbox_PreviewMouseWheel(object sender, MouseWheelEventArgs e)
        {
            //使用xps预览时，不用处理缩放
            if (CurrentPrintItem.PreviewContent is DocumentViewer)
                return;
            var control = (sender as ScrollViewer).Content as FrameworkElement;
            //var parent = control.Parent as FrameworkElement;
            //Point locationPoint = new Point();//Viewbox左上角坐标
            Point deltaLocate = new Point();//Viewbox左上角坐标改变量
            //var mousePoint = e.GetPosition(parent);
            //locationPoint.X = Canvas.GetLeft(control);//获得Viewbox缩放前相对Canvas的左上角坐标
            //locationPoint.Y = Canvas.GetTop(control);
            deltaLocate.X = control.ActualWidth;//记下Viewbox缩放前的宽和高
            deltaLocate.Y = control.ActualHeight;

            if (e.Delta > 0)//滚轮向上，放大
            {
                control.Width = deltaLocate.X * 1.1;
                control.Height = deltaLocate.Y * 1.1;
            }
            if (e.Delta < 0)//滚轮向下，缩小
            {
                control.Width = deltaLocate.X / 1.1;
                control.Height = deltaLocate.Y / 1.1;
            }

            //deltaLocate.X -= control.Width;//计算Viewbox左上角坐标的改变量
            //deltaLocate.Y -= control.Height;
            //deltaLocate.X /= 2;
            //deltaLocate.Y /= 2;

            ////重新设置Viewbox相对于Canvas的左上角坐标，使得缩放是相对于Viewbox中心进行的
            //control.SetValue(Canvas.LeftProperty, locationPoint.X + deltaLocate.X);
            //control.SetValue(Canvas.TopProperty, locationPoint.Y + deltaLocate.Y);
        }

        private void viewBox_Loaded(object sender, RoutedEventArgs e)
        {
            //使用xps预览时，不用处理缩放
            if (CurrentPrintItem.PreviewContent is DocumentViewer)
            {
                (sender as Viewbox).Stretch = Stretch.None;
                return;
            }
            var element = sender as FrameworkElement;
            element.Width = element.ActualWidth / 1.1;
            element.Height = element.ActualHeight / 1.1;
        }

        private void lv_Prints_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            var sv = e.OriginalSource as ScrollViewer;
            if (sv != null && sv.Name != "scroll")
            {
                //使用xps预览时，不要拦截内部滚动条事件
                if (CurrentPrintItem != null && CurrentPrintItem.PreviewContent is DocumentViewer && (CurrentPrintItem.PreviewContent as DocumentViewer).IsAncestorOf(sv))
                    return;
                sv.PageLeft();
            }
        }

        private void Max_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            e.Handled = true;
            var win = Window.GetWindow(this);
            if (win.WindowState == WindowState.Maximized)
            {
                win.WindowState = WindowState.Normal;
                win.MaxWidth = 800;
                win.MaxHeight = 600;
            }
            else
            {
                //win.MaxWidth = win.MaxHeight = double.PositiveInfinity;
                win.MaxWidth = Application.Current.MainWindow.ActualWidth;// -100;
                win.MaxHeight = Application.Current.MainWindow.ActualHeight;// -100;
                win.WindowState = WindowState.Maximized;
            }
        }
    }



    /// <summary>
    /// 20151020实现INotifyPropertyChanged接口防止内存泄露
    /// </summary>
    public class PrintItem : INotifyPropertyChanged
    {
        private PrintContainer _PrinteObject;
        /// <summary>
        /// 打印的对象
        /// </summary>
        public PrintContainer PrintObject
        {
            get { return _PrinteObject; }
            set { _PrinteObject = value; OnPropertyChanged("PrinteObject"); }
        }
        private FileInfo _XpsFile;
        /// <summary>
        /// 打印文件
        /// </summary>
        public FileInfo XpsFile
        {
            get { return _XpsFile; }
            set { _XpsFile = value; OnPropertyChanged("XpsFile"); }
        }

        private FrameworkElement _PreviewContent;
        /// <summary>
        /// 预览的内容
        /// </summary>
        public FrameworkElement PreviewContent
        {
            get { return _PreviewContent; }
            internal set { _PreviewContent = value; OnPropertyChanged("PreviewContent"); }
        }
        /// <summary>
        /// 
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        /// <summary>
        /// 
        /// </summary>
        /// <param name="propertyName"></param>
        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}