﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using Platform.Common.BasicFunctions;
using Platform.Controls.TradeExtension;
using Platform.Presentation.Interfaces;
using Platform.Presentation.PageFunctions;
using Platform.Print;
using System.Windows.Input;
using Platform.Library;
using System.Printing;
using System.IO;
using System.Diagnostics;

namespace TellerSystem.PrintTemplate.Extension
{
    /// <summary>
    ///     PrintDisplay.xaml 的交互逻辑
    /// </summary>
    public partial class PrintViewerManager
    {
        #region Ctor
        public List<PrintItem> PrintSource { get; set; }

        /// <summary>
        /// 打印时触发
        /// </summary>
        public event EventHandler<PrintEventArgs> Printing;
        public class PrintEventArgs : EventArgs
        {
            public PrintItem PrintData { get; set; }
        }
        #endregion

        #region Method
        public void OpenPrintOnWindow(IPage page, bool isclosetrade = true)
        {
            if (PrintSource == null) return;
            foreach (var item in PrintSource)
            {
                //检查打印进程如果不存在启进程，避免发任务比进程快 无法打印 add by jun 20150725 
                Process[] processes = Process.GetProcessesByName("Platform.PrintService");
                if (processes.Length <= 0)
                {
                    Process printProcess = new Process();
                    printProcess.StartInfo.FileName = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Platform.PrintService.exe");
                    printProcess.Start();
                }
                //执行打印操作
                var dialog = new PrintPreviewWindow(item.XpsFile, PrintDevice.Printer);
                dialog.Owner = Application.Current.MainWindow;
                dialog.ShowInTaskbar = false;
                dialog.ShowDialog();
                if (dialog.IsPrinted && Printing != null)
                {
                    Printing(dialog, new PrintEventArgs { PrintData = item });
                }
            }
        }
        #endregion
    }
}