﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml;
using Platform.Common.LogSystem;
using Platform.Common.RunningParameters;

namespace TellerSystem.PrintTemplate.Common
{
    /// <summary>
    /// 银行汇票
    /// </summary>
    public partial class Print_Bill
    {
        public Print_Bill()
        {
            InitializeComponent();
            DateInfo = DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd", CultureInfo.CurrentCulture);
        }

        #region 自定义属性

        public static readonly DependencyProperty DateInfoProperty =
            DependencyProperty.Register("DateInfo", typeof(DateTime), typeof(Print_Bill),
                                        new UIPropertyMetadata(DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd",
                                                                                   CultureInfo.CurrentCulture)));

        public static readonly DependencyProperty DateInfo_YearProperty =
   DependencyProperty.Register("DateInfo_Year", typeof(string), typeof(Print_Bill),
                               new UIPropertyMetadata(DateToBig(LoginUserInfo.TradeDate, "y")));

        public static readonly DependencyProperty DateInfo_MonProperty =
    DependencyProperty.Register("DateInfo_Mon", typeof(string), typeof(Print_Bill),
                                new UIPropertyMetadata(DateToBig(LoginUserInfo.TradeDate, "m")));

        public static readonly DependencyProperty DateInfo_DayProperty =
    DependencyProperty.Register("DateInfo_Day", typeof(string), typeof(Print_Bill),
                                new UIPropertyMetadata(DateToBig(LoginUserInfo.TradeDate, "d")));

        public static readonly DependencyProperty PayBankNameProperty =
            DependencyProperty.Register("PayBankName", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty PayBankNoProperty =
            DependencyProperty.Register("PayBankNo", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty Payee_NameProperty =
            DependencyProperty.Register("Payee_Name", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty ActnoProperty =
            DependencyProperty.Register("Actno", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty Ticket_AmtProperty =
            DependencyProperty.Register("Ticket_Amt", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty Settlement_AmtMaxProperty =
            DependencyProperty.Register("Settlement_AmtMax", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty Settlement_AmtMinProperty =
            DependencyProperty.Register("Settlement_AmtMin", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty ApplyNameProperty =
            DependencyProperty.Register("ApplyName", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty AccountNoProperty =
            DependencyProperty.Register("AccountNo", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty BrankNoProperty =
            DependencyProperty.Register("BrankNo", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty BrankNonumProperty =
            DependencyProperty.Register("BrankNonum", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty RemarkProperty =
            DependencyProperty.Register("Remark", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty ReviewProperty =
            DependencyProperty.Register("Review", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty HandlingProperty =
            DependencyProperty.Register("Handling", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(LoginUserInfo.TellerNo));

        public static readonly DependencyProperty AccountingProperty =
            DependencyProperty.Register("Accounting", typeof (string), typeof (Print_Bill),
                                        new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 日期
        /// </summary>
        public DateTime DateInfo
        {
            get { return (DateTime)GetValue(DateInfoProperty); }
            set { SetValue(DateInfoProperty, value);
            year.Text = DateToBig(value.ToString("yyyyMMdd"))[0];
            mouth.Text = DateToBig(value.ToString("yyyyMMdd"))[1];
            day.Text = DateToBig(value.ToString("yyyyMMdd"))[2];
            }
        }

        /// <summary>
        /// 付款行名称
        /// </summary>
        public string PayBankName
        {
            get { return (string) GetValue(PayBankNameProperty); }
            set { SetValue(PayBankNameProperty, value); }
        }

        /// <summary>
        /// 付款行号
        /// </summary>
        public string PayBankNo
        {
            get { return (string) GetValue(PayBankNoProperty); }
            set { SetValue(PayBankNoProperty, value); }
        }

        /// <summary>
        /// 收款人
        /// </summary>
        public string Payee_Name
        {
            get { return (string) GetValue(Payee_NameProperty); }
            set { SetValue(Payee_NameProperty, value); }
        }

        /// <summary>
        /// 账号
        /// </summary>
        public string Actno
        {
            get { return (string) GetValue(ActnoProperty); }
            set { SetValue(ActnoProperty, value); }
        }

        /// <summary>
        /// 出票金额
        /// </summary>
        public string Settlement_AmtMax
        {
            get { return (string) GetValue(Ticket_AmtProperty); }
            set { SetValue(Ticket_AmtProperty, value); }
        }

        /// <summary>
        /// 结算金额小写
        /// </summary>
        public string Settlement_AmtMin
        {
            get { return (string) GetValue(Settlement_AmtMinProperty); }
            set
            {
                SetValue(Settlement_AmtMinProperty, value);
                SetValue(Settlement_AmtMaxProperty, MoneyToChinese(value));
                
                this.Currency.Text = "￥：" + value; // Extension.PrintContainerExt.SetCanvas(Currency, value);
            }
        }

        /// <summary>
        /// 申请人
        /// </summary>
        public string ApplyName
        {
            get { return (string) GetValue(ApplyNameProperty); }
            set { SetValue(ApplyNameProperty, value); }
        }

        /// <summary>
        /// 帐号
        /// </summary>
        public string AccountNo
        {
            get { return (string) GetValue(AccountNoProperty); }
            set { SetValue(AccountNoProperty, value); }
        }

        /// <summary>
        /// 出票行
        /// </summary>
        public string BrankNo
        {
            get { return (string) GetValue(BrankNoProperty); }
            set { SetValue(BrankNoProperty, value); }
        }

        /// <summary>
        /// 出票行号
        /// </summary>
        public string BrankNonum
        {
            get { return (string) GetValue(BrankNonumProperty); }
            set { SetValue(BrankNonumProperty, value); }
        }

        /// <summary>
        /// 备注
        /// </summary>
        public string Remark
        {
            get { return (string) GetValue(RemarkProperty); }
            set { SetValue(RemarkProperty, value); }
        }

        /// <summary>
        /// 复核
        /// </summary>
        public string Review
        {
            get { return (string) GetValue(ReviewProperty); }
            set { SetValue(ReviewProperty, value); }
        }

        /// <summary>
        /// 经办
        /// </summary>
        public string Handling
        {
            get { return (string) GetValue(HandlingProperty); }
            set { SetValue(HandlingProperty, value); }
        }

        /// <summary>
        /// 记账
        /// </summary>
        public string Accounting
        {
            get { return (string) GetValue(AccountingProperty); }
            set { SetValue(AccountingProperty, value); }
        }

        /// <summary>
        /// 设置金额显示方式
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        /// 
        public void SetsumdisplayMode(string str)
        {
            //textBlock13.Text = textBlock9.Text = PrintTemplate.Extension.PrintContainerExt.PrintCurString(str);
            //return Newsum;
        }

       
        //public void SetsumdisplayMode(string str)
        //{
        //    string Newsum = ""; //新金额
        //    string Decimalpoint; //小数点
        //    string Blank = "  "; //每个字符的空格
        //    str = str.Replace(",", "");//将金额中的逗号去掉
        //    str = str.Replace("，", "");
        //    if (str.Contains('.'))
        //    {
        //        Decimalpoint = str.Split('.')[1];
        //        str = str.Split('.')[0];
        //        Decimalpoint = " " + ForeachsumappBlank(Decimalpoint, Blank);
        //    }
        //    else //如不含小数点 填充空格
        //    {
        //        Decimalpoint = "       ";
        //    }
        //    Newsum = ForeachsumappBlank(str, Blank) + " " + Decimalpoint;
        //    textBlock13.Text = Newsum;
        //    //return Newsum;
        //}
        ///// <summary>
        ///// 遍历金额添加空格
        ///// </summary>
        ///// <param name="sum"></param>
        ///// <param name="Blank"></param>
        ///// <returns></returns>
        //private string ForeachsumappBlank(string sum, string Blank)
        //{
        //    string Appblanksum = "";
        //    for (int i = 0; i < sum.Length; i++)
        //    {
        //        if (i + 1 != sum.Length)
        //        {
        //            Appblanksum += sum[i] + Blank;
        //        }
        //        else
        //        {
        //            Appblanksum += sum[i];
        //        }
        //    }
        //    return Appblanksum;
        //}


        /// <summary>
        /// 将日期转换成大写
        /// add shangyongchao 20121024
        /// </summary>
        /// <param name="date">8位日期</param>
        /// <param name="type">y-年 m-月 d-日</param>
        /// <returns>bDate大写日期</returns>
        public static string DateToBig(string date, string type)
        {
            string year; //年
            string month; //月
            string day; //日
            string byear = ""; //大写年
            string bmonth = ""; //大写月
            string bday = ""; //大写日
            int i = 0;
            year = date.Substring(0, 4);
            month = date.Substring(4, 2);
            day = date.Substring(6, 2);

            if (type == "y")
            {
                while (i < 4)
                {
                    switch (year.Substring(i, 1)) //判断年如何取大写
                    {
                        case "0":
                            byear = byear + '零';
                            break;

                        case "1":
                            byear = byear + "壹";
                            break;

                        case "2":
                            byear = byear + "贰";
                            break;

                        case "3":
                            byear = byear + "叁";
                            break;

                        case "4":
                            byear = byear + "肆";
                            break;

                        case "5":
                            byear = byear + "伍";
                            break;

                        case "6":
                            byear = byear + "陆";
                            break;

                        case "7":
                            byear = byear + "柒";
                            break;

                        case "8":
                            byear = byear + "捌";
                            break;

                        case "9":
                            byear = byear + "玖";
                            break;
                    }
                    i = i + 1;
                }
                return byear;
            }
            else if (type == "m")
            {
                switch (month) //判断月如何取大写
                {
                    case "01":
                        bmonth = "  零壹";
                        break;

                    case "02":
                        bmonth = "  零贰";
                        break;

                    case "03":
                        bmonth = "  零叁";
                        break;

                    case "04":
                        bmonth = "  零肆";
                        break;

                    case "05":
                        bmonth = "  零伍";
                        break;

                    case "06":
                        bmonth = "  零陆";
                        break;

                    case "07":
                        bmonth = "  零柒";
                        break;

                    case "08":
                        bmonth = "  零捌";
                        break;

                    case "09":
                        bmonth = "  零玖";
                        break;

                    case "10":
                        bmonth = "零壹拾";
                        break;

                    case "11":
                        bmonth = "壹拾壹";
                        break;

                    case "12":
                        bmonth = "壹拾贰";
                        break;
                }
                return bmonth;
            }
            else if (type == "d")
            {
                switch (day) //判断日如何取大写
                {
                    case "01":
                        bday = "  零壹";
                        break;

                    case "02":
                        bday = "  零贰";
                        break;

                    case "03":
                        bday = "  零叁";
                        break;

                    case "04":
                        bday = "  零肆";
                        break;

                    case "05":
                        bday = "  零伍";
                        break;

                    case "06":
                        bday = "  零陆";
                        break;

                    case "07":
                        bday = "  零柒";
                        break;

                    case "08":
                        bday = "  零捌";
                        break;

                    case "09":
                        bday = "  零玖";
                        break;

                    case "10":
                        bday = "零壹拾";
                        break;

                    case "11":
                        bday = "壹拾壹";
                        break;

                    case "12":
                        bday = "壹拾贰";
                        break;

                    case "13":
                        bday = "壹拾叁";
                        break;

                    case "14":
                        bday = "壹拾肆";
                        break;

                    case "15":
                        bday = "壹拾伍";
                        break;

                    case "16":
                        bday = "壹拾陆";
                        break;

                    case "17":
                        bday = "壹拾柒";
                        break;

                    case "18":
                        bday = "壹拾捌";
                        break;

                    case "19":
                        bday = "壹拾玖";
                        break;

                    case "20":
                        bday = "零贰拾";
                        break;

                    case "21":
                        bday = "贰拾壹";
                        break;

                    case "22":
                        bday = "贰拾贰";
                        break;

                    case "23":
                        bday = "贰拾叁";
                        break;

                    case "24":
                        bday = "贰拾肆";
                        break;

                    case "25":
                        bday = "贰拾伍";
                        break;

                    case "26":
                        bday = "贰拾陆";
                        break;

                    case "27":
                        bday = "贰拾柒";
                        break;

                    case "28":
                        bday = "贰拾捌";
                        break;

                    case "29":
                        bday = "贰拾玖";
                        break;

                    case "30":
                        bday = "零叁拾";
                        break;

                    case "31":
                        bday = "叁拾壹";
                        break;
                }
                return bday;
            }
            else
            {
                return "";
            }

        }
        /// <summary>
        /// 将日期转换成大写
        /// add shangyongchao 20121024
        /// </summary>
        /// <param name="date">8位日期</param>
        /// <returns>bDate大写日期</returns>
        private string[] DateToBig(string date)
        {
            string year; //年
            string month; //月
            string day; //日
            string byear = ""; //大写年
            string bmonth = ""; //大写月
            string bday = ""; //大写日
            int i = 0;
            year = date.Substring(0,4);
            month = date.Substring(4, 2);
            day = date.Substring(6, 2);
            while (i < 4)
            {
                switch (year.Substring(i, 1)) //判断年如何取大写
                {
                    case "0":
                        byear = byear + '零';
                        break;

                    case "1":
                        byear = byear + "壹";
                        break;

                    case "2":
                        byear = byear + "贰";
                        break;

                    case "3":
                        byear = byear + "叁";
                        break;

                    case "4":
                        byear = byear + "肆";
                        break;

                    case "5":
                        byear = byear + "伍";
                        break;

                    case "6":
                        byear = byear + "陆";
                        break;

                    case "7":
                        byear = byear + "柒";
                        break;

                    case "8":
                        byear = byear + "捌";
                        break;

                    case "9":
                        byear = byear + "玖";
                        break;
                }
                i = i + 1;
            }

            switch (month) //判断月如何取大写
            {
                case "01":
                    bmonth = "  零壹";
                    break;

                case "02":
                    bmonth = "  零贰";
                    break;

                case "03":
                    bmonth = "  零叁";
                    break;

                case "04":
                    bmonth = "  零肆";
                    break;

                case "05":
                    bmonth = "  零伍";
                    break;

                case "06":
                    bmonth = "  零陆";
                    break;

                case "07":
                    bmonth = "  零柒";
                    break;

                case "08":
                    bmonth = "  零捌";
                    break;

                case "09":
                    bmonth = "  零玖";
                    break;

                case "10":
                    bmonth = "零壹拾";
                    break;

                case "11":
                    bmonth = "壹拾壹";
                    break;

                case "12":
                    bmonth = "壹拾贰";
                    break;
            }
            switch (day) //判断日如何取大写
            {
                case "01":
                    bday = "  零壹";
                    break;

                case "02":
                    bday = "  零贰";
                    break;

                case "03":
                    bday = "  零叁";
                    break;

                case "04":
                    bday = "  零肆";
                    break;

                case "05":
                    bday = "  零伍";
                    break;

                case "06":
                    bday = "  零陆";
                    break;

                case "07":
                    bday = "  零柒";
                    break;

                case "08":
                    bday = "  零捌";
                    break;

                case "09":
                    bday = "  零玖";
                    break;

                case "10":
                    bday = "零壹拾";
                    break;

                case "11":
                    bday = "壹拾壹";
                    break;

                case "12":
                    bday = "壹拾贰";
                    break;

                case "13":
                    bday = "壹拾叁";
                    break;

                case "14":
                    bday = "壹拾肆";
                    break;

                case "15":
                    bday = "壹拾伍";
                    break;

                case "16":
                    bday = "壹拾陆";
                    break;

                case "17":
                    bday = "壹拾柒";
                    break;

                case "18":
                    bday = "壹拾捌";
                    break;

                case "19":
                    bday = "壹拾玖";
                    break;

                case "20":
                    bday = "零贰拾";
                    break;

                case "21":
                    bday = "贰拾壹";
                    break;

                case "22":
                    bday = "贰拾贰";
                    break;

                case "23":
                    bday = "贰拾叁";
                    break;

                case "24":
                    bday = "贰拾肆";
                    break;

                case "25":
                    bday = "贰拾伍";
                    break;

                case "26":
                    bday = "贰拾陆";
                    break;

                case "27":
                    bday = "贰拾柒";
                    break;

                case "28":
                    bday = "贰拾捌";
                    break;

                case "29":
                    bday = "贰拾玖";
                    break;

                case "30":
                    bday = "零叁拾";
                    break;

                case "31":
                    bday = "叁拾壹";
                    break;
            }
            return new string[] { byear, bmonth, bday };
        }
        private string MoneyToChinese(string LowerMoney)
        {
            string functionReturnValue = null;
            if (LowerMoney == "")
            {
                functionReturnValue = "";
                return functionReturnValue;
            }
            else
            {
                bool IsNegative = false; // 是否是负数             
                if (LowerMoney.Trim().Substring(0, 1) == "-")
                {
                    // 是负数则先转为正数     
                    LowerMoney = LowerMoney.Trim().Remove(0, 1);
                    IsNegative = true;
                }
                string strLower = null;
                string strUpart = null;
                string strUpper = null;
                int iTemp = 0;
                // 保留两位小数 123.489→123.49  123.4→123.4     
                LowerMoney = Math.Round(double.Parse(LowerMoney), 2).ToString();
                if (LowerMoney.IndexOf(".") > 0)
                {
                    if (LowerMoney.IndexOf(".") == LowerMoney.Length - 2)
                    {
                        LowerMoney = LowerMoney + "0";
                    }
                }
                else
                {
                    LowerMoney = LowerMoney + ".00";
                }
                strLower = LowerMoney;
                iTemp = 1;
                strUpper = "";
                while (iTemp <= strLower.Length)
                {
                    switch (strLower.Substring(strLower.Length - iTemp, 1))
                    {
                        case ".":
                            strUpart = "圆";
                            break;
                        case "0":
                            strUpart = "零";
                            break;
                        case "1":
                            strUpart = "壹";
                            break;
                        case "2":
                            strUpart = "贰";
                            break;
                        case "3":
                            strUpart = "叁";
                            break;
                        case "4":
                            strUpart = "肆";
                            break;
                        case "5":
                            strUpart = "伍";
                            break;
                        case "6":
                            strUpart = "陆";
                            break;
                        case "7":
                            strUpart = "柒";
                            break;
                        case "8":
                            strUpart = "捌";
                            break;
                        case "9":
                            strUpart = "玖";
                            break;
                    }
                    switch (iTemp)
                    {
                        case 1:
                            strUpart = strUpart + "分";
                            break;
                        case 2:
                            strUpart = strUpart + "角";
                            break;
                        case 3:
                            strUpart = strUpart + "";
                            break;
                        case 4:
                            strUpart = strUpart + "";
                            break;
                        case 5:
                            strUpart = strUpart + "拾";
                            break;
                        case 6:
                            strUpart = strUpart + "佰";
                            break;
                        case 7:
                            strUpart = strUpart + "仟";
                            break;
                        case 8:
                            strUpart = strUpart + "万";
                            break;
                        case 9:
                            strUpart = strUpart + "拾";
                            break;
                        case 10:
                            strUpart = strUpart + "佰";
                            break;
                        case 11:
                            strUpart = strUpart + "仟";
                            break;
                        case 12:
                            strUpart = strUpart + "亿";
                            break;
                        case 13:
                            strUpart = strUpart + "拾";
                            break;
                        case 14:
                            strUpart = strUpart + "佰";
                            break;
                        case 15:
                            strUpart = strUpart + "仟";
                            break;
                        case 16:
                            strUpart = strUpart + "万";
                            break;
                        default:
                            strUpart = strUpart + "";
                            break;
                    }
                    strUpper = strUpart + strUpper;
                    iTemp = iTemp + 1;
                }
                strUpper = strUpper.Replace("零拾", "零");
                strUpper = strUpper.Replace("零佰", "零");
                strUpper = strUpper.Replace("零仟", "零");
                strUpper = strUpper.Replace("零零零", "零");
                strUpper = strUpper.Replace("零零", "零");
                strUpper = strUpper.Replace("零角零分", "整");
                strUpper = strUpper.Replace("零分", "整");
                strUpper = strUpper.Replace("零角", "零");
                strUpper = strUpper.Replace("零亿零万零圆", "亿圆");
                strUpper = strUpper.Replace("亿零万零圆", "亿圆");
                strUpper = strUpper.Replace("零亿零万", "亿");
                strUpper = strUpper.Replace("零万零圆", "万圆");
                strUpper = strUpper.Replace("零亿", "亿");
                strUpper = strUpper.Replace("零万", "万");
                strUpper = strUpper.Replace("零圆", "圆");
                strUpper = strUpper.Replace("零零", "零");
                // 对壹圆以下的金额的处理               
                if (strUpper.Substring(0, 1) == "圆")
                {
                    strUpper = strUpper.Substring(1, strUpper.Length - 1);
                }
                if (strUpper.Substring(0, 1) == "零")
                {
                    strUpper = strUpper.Substring(1, strUpper.Length - 1);
                }
                if (strUpper.Substring(0, 1) == "角")
                {
                    strUpper = strUpper.Substring(1, strUpper.Length - 1);
                }
                if (strUpper.Substring(0, 1) == "分")
                {
                    strUpper = strUpper.Substring(1, strUpper.Length - 1);
                }
                if (strUpper.Substring(0, 1) == "整")
                {
                    strUpper = "零圆整";
                }
                functionReturnValue = strUpper;
                if (IsNegative == true)
                {
                    return "负" + functionReturnValue;
                }
                else
                {
                    return functionReturnValue;
                }
            }
        }
        #endregion 自定义属性

    }
}