﻿using System;
using System.Globalization;
using System.Linq;
using System.Windows;
using Platform.Common.RunningParameters;

namespace TellerSystem.PrintTemplate.Common
{
    /// <summary>
    /// Print_ComeProof.xaml 的交互逻辑
    /// </summary>
    public partial class Print_ComeProof
    {
        public Print_ComeProof()
        {
            InitializeComponent();
        }

        #region 自定义属性

        public static readonly DependencyProperty DateInfoProperty =
            DependencyProperty.Register("DateInfo", typeof(DateTime), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd",
                                                                                   CultureInfo.CurrentCulture)));

        public static readonly DependencyProperty PayNameProperty =
            DependencyProperty.Register("PayName", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty Payee_NameProperty =
            DependencyProperty.Register("Payee_Name", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty Payee_ActnoProperty =
            DependencyProperty.Register("Payee_Actno", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty Pay_ActnoProperty =
            DependencyProperty.Register("Pay_Actno", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty PayBankNameProperty =
            DependencyProperty.Register("PayBankName", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty PayeeBankNameProperty =
            DependencyProperty.Register("PayeeBankName", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty Settlement_AmtMaxProperty =
            DependencyProperty.Register("Settlement_AmtMax", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty Settlement_AmtMinProperty =
            DependencyProperty.Register("Settlement_AmtMin", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty UseProperty =
            DependencyProperty.Register("Use", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty Payee_NameStampProperty =
            DependencyProperty.Register("Payee_NameStamp", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty ClearCenterProperty =
            DependencyProperty.Register("ClearCenter", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty ReviewProperty =
            DependencyProperty.Register("Review", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty HandlingProperty =
            DependencyProperty.Register("Handling", typeof(string), typeof(Print_ComeProof),
                                        new UIPropertyMetadata(LoginUserInfo.TellerNo));

        /// <summary>
        /// 日期
        /// </summary>
        public DateTime DateInfo
        {
            get { return (DateTime)GetValue(DateInfoProperty); }
            set { SetValue(DateInfoProperty, value); }
        }

        /// <summary>
        /// 付款人
        /// </summary>
        public string PayName
        {
            get { return (string)GetValue(PayNameProperty); }
            set { SetValue(PayNameProperty, value); }
        }

        /// <summary>
        /// 收款人
        /// </summary>
        public string Payee_Name
        {
            get { return (string)GetValue(Payee_NameProperty); }
            set { SetValue(Payee_NameProperty, value); }
        }

        /// <summary>
        /// 收款人账号
        /// </summary>
        public string Payee_Actno
        {
            get { return (string)GetValue(Payee_ActnoProperty); }
            set { SetValue(Payee_ActnoProperty, value); }
        }

        /// <summary>
        /// 付款人账号
        /// </summary>
        public string Pay_Actno
        {
            get { return (string)GetValue(Pay_ActnoProperty); }
            set { SetValue(Pay_ActnoProperty, value); }
        }

        /// <summary>
        /// 付款开户行
        /// </summary>
        public string PayBankName
        {
            get { return (string)GetValue(PayBankNameProperty); }
            set { SetValue(PayBankNameProperty, value); }
        }

        /// <summary>
        /// 收款开户行
        /// </summary>
        public string PayeeBankName
        {
            get { return (string)GetValue(PayeeBankNameProperty); }
            set { SetValue(PayeeBankNameProperty, value); }
        }

        /// <summary>
        /// 结算金额大写
        /// </summary>
        public string Settlement_AmtMax
        {
            get { return (string)GetValue(Settlement_AmtMaxProperty); }
            set { SetValue(Settlement_AmtMaxProperty, value); }
        }

        /// <summary>
        /// 结算金额小写
        /// </summary>
        public string Settlement_AmtMin
        {
            get { return (string)GetValue(Settlement_AmtMinProperty); }
            set
            {
                SetValue(Settlement_AmtMinProperty, value);
                Extension.PrintContainerExt.SetCanvas(Currency, value);
            }
        }

        /// <summary>
        /// 用途
        /// </summary>
        public string Use
        {
            get { return (string)GetValue(UseProperty); }
            set { SetValue(UseProperty, value); }
        }

        /// <summary>
        /// 付款人盖章
        /// </summary>
        public string Payee_NameStamp
        {
            get { return (string)GetValue(Payee_NameStampProperty); }
            set { SetValue(Payee_NameStampProperty, value); }
        }

        /// <summary>
        /// 清算中心
        /// </summary>
        public string ClearCenter
        {
            get { return (string)GetValue(ClearCenterProperty); }
            set { SetValue(ClearCenterProperty, value); }
        }

        /// <summary>
        /// 复核
        /// </summary>
        public string Review
        {
            get { return (string)GetValue(ReviewProperty); }
            set { SetValue(ReviewProperty, value); }
        }

        /// <summary>
        /// 经办
        /// </summary>
        public string Handling
        {
            get { return (string)GetValue(HandlingProperty); }
            set { SetValue(HandlingProperty, value); }
        }

        /// <summary>
        /// 设置金额显示方式
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public void SetsumdisplayMode(string str)
        {
            //textBlock11.Text = textBlock9.Text = PrintTemplate.Extension.PrintContainerExt.PrintCurString(str);
            //return Newsum;
        }

        #endregion 自定义属性
    }
}