﻿using System;
using System.Globalization;
using System.Linq;
using System.Windows;
using Platform.Common.RunningParameters;

namespace TellerSystem.PrintTemplate.Common
{
    /// <summary>
    ///单位定期存款证实书
    /// </summary>
    public partial class Print_DepositBook
    {
        public Print_DepositBook()
        {
            InitializeComponent();
            Date = LoginUserInfo.TradeDate;
        }

        #region 自定义属性

        public static readonly DependencyProperty DateInfoProperty =
            DependencyProperty.Register("DateInfo", typeof(DateTime), typeof(Print_DepositBook),
                                        new UIPropertyMetadata(DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd",
                                                                                   CultureInfo.CurrentCulture)));

        public static readonly DependencyProperty DepositNameProperty =
            DependencyProperty.Register("DepositName", typeof(string), typeof(Print_DepositBook),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty DepositAccountProperty =
            DependencyProperty.Register("DepositAccount", typeof(string), typeof(Print_DepositBook),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty DepositperiodProperty =
            DependencyProperty.Register("Depositperiod", typeof(string), typeof(Print_DepositBook),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty InterestrateProperty =
            DependencyProperty.Register("Interestrate", typeof(string), typeof(Print_DepositBook),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty MaturityProperty =
            DependencyProperty.Register("Maturity", typeof(DateTime?), typeof(Print_DepositBook),
                                        new UIPropertyMetadata(null));

        public static readonly DependencyProperty Settlement_AmtMaxProperty =
            DependencyProperty.Register("Settlement_AmtMax", typeof(string), typeof(Print_DepositBook),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty Settlement_AmtMinProperty =
            DependencyProperty.Register("Settlement_AmtMin", typeof(string), typeof(Print_DepositBook),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty AccountingProperty =
            DependencyProperty.Register("Accounting", typeof(string), typeof(Print_DepositBook),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty ReviewProperty =
            DependencyProperty.Register("Review", typeof(string), typeof(Print_DepositBook),
                                        new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty HandlingProperty =
            DependencyProperty.Register("Handling", typeof(string), typeof(Print_DepositBook),
                                        new UIPropertyMetadata(LoginUserInfo.TellerNo));


        #region 柜员
        /// <summary>
        /// 柜员号
        /// </summary>
        public string TellerNo
        {
            get { return (string)GetValue(TellerNoProperty); }
            set { SetValue(TellerNoProperty, value); }
        }

        public static readonly DependencyProperty TellerNoProperty =
            DependencyProperty.Register("TellerNo", typeof(string), typeof(Print_DepositBook), new UIPropertyMetadata(string.Empty));

        public string TellerNoStr
        {
            get { return (string)GetValue(TellerNoStrProperty); }
            set { SetValue(TellerNoStrProperty, value); }
        }

        public static readonly DependencyProperty TellerNoStrProperty =
            DependencyProperty.Register("TellerNoStr", typeof(string), typeof(Print_DepositBook), new UIPropertyMetadata("操作员号:"));
        #endregion

        #region 流水
        public string SerialNumberStr
        {
            get { return (string)GetValue(SerialNumberStrProperty); }
            set { SetValue(SerialNumberStrProperty, value); }
        }

        public static readonly DependencyProperty SerialNumberStrProperty =
            DependencyProperty.Register("SerialNumberStr", typeof(string), typeof(Print_DepositBook), new UIPropertyMetadata("流水号:"));
        #endregion

        #region 时间
        public string Date
        {
            get
            {
                return (string)GetValue(DateProperty);
            }
            set
            {
                SetValue(DateProperty,value);
            }
        }

        public static readonly DependencyProperty DateProperty =
            DependencyProperty.Register("Date",typeof(string),typeof(Print_DepositBook),new UIPropertyMetadata(string.Empty));

        public string DataTimeStr
        {
            get { return (string)GetValue(DataTimeStrProperty); }
            set { SetValue(DataTimeStrProperty, value); }
        }

        public static readonly DependencyProperty DataTimeStrProperty =
           DependencyProperty.Register("DataTimeStr", typeof(string), typeof(Print_DepositBook), new UIPropertyMetadata("交易日期:"));
        #endregion


        /// <summary>
        /// 日期
        /// </summary>
        public DateTime DateInfo
        {
            get { return (DateTime)GetValue(DateInfoProperty); }
            set { SetValue(DateInfoProperty, value); }
        }

        /// <summary>
        /// 存款人名称
        /// </summary>
        public string DepositName
        {
            get { return (string)GetValue(DepositNameProperty); }
            set { SetValue(DepositNameProperty, value); }
        }

        /// <summary>
        /// 存款账号
        /// </summary>
        public string DepositAccount
        {
            get { return (string)GetValue(DepositAccountProperty); }
            set { SetValue(DepositAccountProperty, value); }
        }

        /// <summary>
        /// 存期
        /// </summary>
        public string Depositperiod
        {
            get { return (string)GetValue(DepositperiodProperty); }
            set { SetValue(DepositperiodProperty, value); }
        }

        /// <summary>
        /// 利率
        /// </summary>
        public string Interestrate
        {
            get { return (string)GetValue(InterestrateProperty); }
            set { SetValue(InterestrateProperty, value); }
        }

        /// <summary>
        /// 到期日
        /// </summary>
        public DateTime? Maturity
        {
            get { return (DateTime?)GetValue(MaturityProperty); }
            set { SetValue(MaturityProperty, value); }
        }

        /// <summary>
        /// 结算金额大写
        /// </summary>
        public string Settlement_AmtMax
        {
            get { return (string)GetValue(Settlement_AmtMaxProperty); }
            set { SetValue(Settlement_AmtMaxProperty, value); }
        }

        /// <summary>
        /// 结算金额小写
        /// </summary>
        public string Settlement_AmtMin
        {
            get { return (string)GetValue(Settlement_AmtMinProperty); }
            set
            {
                SetValue(Settlement_AmtMinProperty, value);
                Extension.PrintContainerExt.SetCanvas(stackPanel1, value);
            }
        }

        /// <summary>
        ///会计
        /// </summary>
        public string Accounting
        {
            get { return (string)GetValue(AccountingProperty); }
            set { SetValue(AccountingProperty, value); }
        }

        /// <summary>
        /// 复核
        /// </summary>
        public string Review
        {
            get { return (string)GetValue(ReviewProperty); }
            set { SetValue(ReviewProperty, value); }
        }

        /// <summary>
        ///记账
        /// </summary>
        public string Handling
        {
            get { return (string)GetValue(HandlingProperty); }
            set { SetValue(HandlingProperty, value); }
        }

        /// <summary>
        /// 设置金额显示方式
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>

        #endregion 自定义属性
    }
}