﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Printing;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Brushes = System.Drawing.Brushes;
using FontFamily = System.Windows.Media.FontFamily;

namespace TellerSystem.PrintTemplate.OldPrint
{
    public abstract class PrintElement<TPrintDataType> : Control, IPrintElement<TPrintDataType>
    {
        protected PrintElement()
        {
            this.Initialized += PrintElementInitialized;
            //设置默认字体
            this.Group = "Default";
        }

        void PrintElementInitialized(object sender, EventArgs e)
        {
            this.Initialized -= PrintElementInitialized;
            this.Left = (float)(double)this.GetValue(Canvas.LeftProperty);
            this.Top = (float)(double)this.GetValue(Canvas.TopProperty);

        }

        public virtual bool IsPrint
        {
            get { return this.Visibility == Visibility.Visible; }
            set { this.Visibility = value ? Visibility.Visible : Visibility.Hidden; }
        }
        public abstract TPrintDataType Value { get; set; }

        #region Description

        /// <summary>
        /// Description Dependency Property
        /// </summary>
        public static readonly DependencyProperty DescriptionProperty =
            DependencyProperty.Register("Description", typeof(string), typeof(PrintElement<TPrintDataType>),
                new FrameworkPropertyMetadata(@"请添加该打印条目的描述信息"));

        /// <summary>
        /// Gets or sets the Description property. This dependency property 
        /// indicates ....
        /// </summary>
        public string Description
        {
            get { return (string)GetValue(DescriptionProperty); }
            set { SetValue(DescriptionProperty, value); }
        }

        #endregion

        #region PropertyName

        /// <summary>
        /// PropertyName Dependency Property
        /// </summary>
        public static readonly DependencyProperty PropertyNameProperty =
            DependencyProperty.Register("PropertyName", typeof(string), typeof(PrintElement<TPrintDataType>),
                new FrameworkPropertyMetadata(@"请添加该打印条目的描述信息"));

        /// <summary>
        /// Gets or sets the PropertyName property. This dependency property 
        /// indicates ....
        /// </summary>
        public string PropertyName
        {
            get { return (string)GetValue(PropertyNameProperty); }
            set { SetValue(PropertyNameProperty, value); }
        }

        #endregion

        public virtual bool Print(PrintPageEventArgs e)
        {
            //if (this.Value == null || !this.IsPrint) return false;
            if (!this.IsPrint) return false;
            var printfont = this.FontWeight == FontWeights.Bold ?
                new Font(this.FontFamily.ToString(), (float)this.FontSize, System.Drawing.FontStyle.Bold, GraphicsUnit.Pixel)
                : new Font(this.FontFamily.ToString(), (float)this.FontSize, System.Drawing.FontStyle.Regular, GraphicsUnit.Pixel);
            e.Graphics.DrawString(this.Value != null ? this.Value.ToString() : ""
                                    , printfont, Brushes.Black
                                    , (float)e.MarginBounds.X + this.Left
                                    , (float)e.MarginBounds.Y + this.Top);
            return true;
        }

        public float Left { get; set; }

        public float Top { get; set; }

        public string Group { get; set; }

        public virtual bool IsValueEditable
        {
            get { return true; }
        }

        public abstract void GetObjectData(SerializationInfo info, StreamingContext context);
    }
}
