﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TellerSystem.Communication;
using Platform.Presentation.Interfaces;
using System.ComponentModel;
// using TellerSystem.Library.Ext.Function;
using Platform.Common.RunningParameters;

namespace TellerSystem.PrintTemplate.Bankbook
{
    /// <summary>
    /// PrintElement_AccountDetail.xaml 的交互逻辑
    /// </summary>
    [DesignTimeVisible(false)]
    public partial class PrintElement_AccountDetail : UserControl
    {

        public PrintElement_AccountDetail()
        {
            InitializeComponent();
            if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
                return;
            Panel_PrintData.Children.Clear();
        }
        private bool _isBorrowLoan = false;
        /// <summary>
        /// 打印借贷
        /// </summary>
        public bool IsBorrowLoan
        {
            get { return _isBorrowLoan; }
            set { _isBorrowLoan = value; }
        }
        private bool _isPrint = true;

        public bool IsPrint
        {
            get { return _isPrint; }
            set { _isPrint = value; }
        }

        /// <summary>
        /// 加载分录数据
        /// </summary>
        /// <param name="serialNo"></param>
        public void LoadData(string Fd123Data)
        {
            if (!_isPrint) return;
            var printData = new List<string>();
            try
            {
                printData = Fd123Data.Split('\n').ToList();
                var printDataSort = printData;
                printDataSort.RemoveAt(0);
                printDataSort.RemoveAt(printDataSort.Count() - 1);

                if (printDataSort.Count == 0)
                {
                    Panel_PrintData.Visibility = Visibility.Hidden;
                    return;
                }
                Panel_PrintData.Visibility = Visibility.Visible;
                foreach (var item in printDataSort)
                {
                    Decimal result;
                    string[] tag = item.Split('|');
                    Decimal.TryParse(tag[2], out result);
                    if (tag[1].Trim().Equals("120"))
                        Panel_PrintData.Children.Add(new TextBlock { Text = tag[0] + "：(" + tag[1].PadRight(9, ' ') + ")" + result.ToString("N2", System.Globalization.CultureInfo.CreateSpecificCulture("zh-CN")).PadLeft(20) + "  开户机构:" + tag[3] });
                    else
                        Panel_PrintData.Children.Add(new TextBlock { Text = tag[0] + "：(" + tag[1].PadRight(9, ' ') + ")" + result.ToString("C2", System.Globalization.CultureInfo.CreateSpecificCulture("zh-CN")).PadLeft(20) + "  开户机构:" + tag[3] });
                }
            }
            catch
            {
                //解析失败
                printData.Clear();
                printData.Add("获取分录信息失败：解析错误！");
            }
        }

        private string Comma(string sourceData, int length)
        {
            Decimal result;
            if (!Decimal.TryParse(sourceData, out result))
            {
                var exception = new ArgumentException(
                    string.Format("Comma函数执行出错！[{0}]不是有效的数字类型！", sourceData));
                //TradeLogger.Error(exception);
                throw exception;
            }
            return result.ToString("N").PadLeft(length);
        }

        public decimal ToDecimal(string src)
        {
            return string.IsNullOrWhiteSpace(src) ? 0 : decimal.Parse(src);
        }

        public class DinoComparer : IComparer<string>
        {
            public int Compare(string x, string y)
            {
                if (x.Contains("收"))
                {
                    return 1;
                }
                if (x.Contains("付"))
                {
                    return y.Contains("收") ? 0 : 1;
                }
                if (x.Contains("借"))
                {
                    if (y.Contains("收"))
                    {
                        return 0;
                    }
                    return y.Contains("付") ? 0 : 1;
                }
                if (y.Contains("收"))
                {
                    return 0;
                }
                if (y.Contains("付"))
                {
                    return 0;
                }
                return y.Contains("借") ? 0 : 1;
            }
        }
    }
}
