﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing.Printing;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Media;
using TellerSystem.PrintTemplate.Template;


namespace TellerSystem.PrintTemplate.OldPrint
{

    public class PrintFormatControl : Print_Blank
    {

        #region 属性
        /// <summary>
        /// 文档名称
        /// </summary>
        public string DocumentName { get; set; }
        /// <summary>
        /// 文档类别
        /// </summary>
        public string DocumentGroup { set; get; }
        #endregion

        /// <summary>
        /// 设置打印元素组的IsPrint属性
        /// </summary>
        /// <param name="groupName"></param>
        /// <param name="isPrint"></param>
        public void SetIsPrintByGroup(string groupName, bool isPrint)
        {
            var panel = this.Content as Canvas;
            if (panel == null) return;
            foreach (var item in panel.Children)
            {
                var element = item as IPrintElement;
                if (element != null && element.Group == groupName)
                {
                    element.IsPrint = isPrint;
                }
            }
        }

        /// <summary>
        /// 此表单实例的最大打印次数
        /// （模板的最大权限次数是可以修改和删除的，实例的最大打印次数是一旦保存，就不能再变动了，打印时控制权限是按实例权限次数来控制，但是实例权限次数来源于模板权限次数）
        /// guanjian
        /// </summary>
        public int FormRoleCount
        {
            get;
            set;
        }

        /// <summary>
        /// 此表单的打印数据编号，保存在历史数据中的唯一编号。每打印一次都要根据此编号新增一条打印记录
        /// guanjian
        /// </summary>
        public string FormDataId
        {
            get;
            set;
        }

        /// <summary>
        /// 表单的中文名称，补打的时候要显示给柜员看
        /// guanjian
        /// </summary>
        public string FormDataName
        {
            get;
            set;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="top"></param>
        /// <param name="left"></param>
        /// <param name="printValue"></param>
        /// <param name="fontSize"></param>
        /// <param name="isBold"></param>
        /// <param name="fontFamily"></param>
        /// <returns></returns>
        public bool AddPrintLine(double top, double left, string printValue, double fontSize = double.NaN, bool isBold = false, FontFamily fontFamily = null)
        {
            var panel = this.Content as Canvas;
            if (panel == null) return false;
            var element = new PrintStringElement();
            Canvas.SetTop(element, top);
            Canvas.SetLeft(element, left);
            element.Value = printValue;

            if (!double.IsNaN(fontSize))
                element.FontSize = fontSize;
            if (isBold)
                element.FontWeight = System.Windows.FontWeights.Bold;
            if (fontFamily != null)
                element.FontFamily = fontFamily;
            panel.Children.Add(element);
            return true;
        }
    }
}
