﻿using System;
using System.Drawing;
using System.Drawing.Printing;
using System.Runtime.Serialization;
using System.Windows;
using System.Windows.Controls;

namespace TellerSystem.PrintTemplate.OldPrint
{
    
    [Serializable]
    public class PrintLableElement : PrintElement<string>
    {
        public PrintLableElement() : base() { }

        static PrintLableElement()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(PrintLableElement), new FrameworkPropertyMetadata(typeof(PrintLableElement)));
        }

        public override sealed string Value { get; set; }
        public override bool IsValueEditable { get { return false; } }
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("Left", Canvas.GetLeft(this));
            info.AddValue("Top", Canvas.GetTop(this));
            info.AddValue("IsPrint", IsPrint);
            info.AddValue("Value", this.Value);
            info.AddValue("Group", this.Group);
            info.AddValue("Description", this.Description);
        }

        protected PrintLableElement(SerializationInfo info, StreamingContext context)
        {
            Canvas.SetLeft(this, (float)info.GetDouble("Left"));
            Canvas.SetTop(this, (float)info.GetDouble("Top"));
            IsPrint = info.GetBoolean("IsPrint");
            Value = info.GetString("Value");
            Group = info.GetString("Group");
            Description = info.GetString("Description");
        }

        public override bool Print(PrintPageEventArgs e)
        {
            if (IsPrint && this.Value == null) this.Value = this.Description;
            return base.Print(e);
        }

        const string PRINT_ATT = "  ";

        private string _printValue;

        public string PrintValue
        {
            get { return _printValue; }
            set 
            { 
                _printValue = value;

                this.Description += PRINT_ATT + value;
            }
        }

        public void Hide()
        {
            this.Visibility = System.Windows.Visibility.Collapsed;
            this.IsPrint = false;

            UIElement ui = this.Parent as UIElement;
            if (ui == null) return;

            ui.UpdateLayout();
        }
    }
}
