﻿using System;
using System.Drawing;
using System.Drawing.Printing;
using System.Runtime.Serialization;
using System.Windows;
using System.Windows.Controls;

namespace TellerSystem.PrintTemplate.OldPrint
{

    [Serializable]
    public class PrintStringElement : PrintElement<string>
    {
        public PrintStringElement()
            : base()
        {
            this.IsPrint = true;
        }

        static PrintStringElement()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(PrintStringElement), new FrameworkPropertyMetadata(typeof(PrintStringElement)));
        }

        private string _value = string.Empty;

        public override sealed string Value
        {
            get { return _value; }
            set
            {
                _value = value;
                if (!System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
                    Description = value;
            }
        }

        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("Left", Canvas.GetLeft(this));
            info.AddValue("Top", Canvas.GetTop(this));
            info.AddValue("IsPrint", IsPrint);
            info.AddValue("Value", this.Value);
            info.AddValue("Group", this.Group);
            info.AddValue("Description", this.Description);
        }

        protected PrintStringElement(SerializationInfo info, StreamingContext context)
        {
            this.IsPrint = true;
            Canvas.SetLeft(this, (float)info.GetDouble("Left"));
            Canvas.SetTop(this, (float)info.GetDouble("Top"));
            IsPrint = info.GetBoolean("IsPrint");
            Value = info.GetString("Value");
            Group = info.GetString("Group");
            Description = info.GetString("Description");
        }

        public override bool Print(PrintPageEventArgs e)
        {
            if (this.Value == null) this.Value = string.Empty;
            return base.Print(e);
        }
    }
}
