﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using Platform.Common;
using Platform.Common.RunningParameters;
using Platform.Print;
using TellerSystem.PrintTemplate.Extension;
using TellerSystem.PrintTemplate.OldPrint;

namespace TellerSystem.PrintTemplate.Template
{
    /// <summary>
    /// 空白凭证基类
    /// </summary>
    public class Print_Etc : PrintContainer
    {
        static Print_Etc()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(Print_Etc), new FrameworkPropertyMetadata(typeof(Print_Etc)));
        }

        public Print_Etc()
        {
            
            if (DesignerProperties.GetIsInDesignMode(this))
            {
                TradeYear = "2012";
                TradeMonth = "01";
                TradeDay = "01";
                KinbrNo = "00101";
                TellerNo = "000100";
                KinbrName = "xx支行";
                TradeTime = "TradeTime";
                HostSerialNumber = "000012223";
                PrintNumber = "02";
                Authentic = "XXX";
                TradeCode = "2005";
                SerialNumber="000012";
            }
            else
            {
                PrintContainerExt.SetCreatePrintNum(this, false);
                if (LoginUserInfo.GetUserInfo("TradeDate").Length == 8)
                {
                    TradeYear = LoginUserInfo.GetUserInfo("TradeDate").Substring(0, 4);
                    TradeMonth = LoginUserInfo.GetUserInfo("TradeDate").Substring(4, 2);
                    TradeDay = LoginUserInfo.GetUserInfo("TradeDate").Substring(6, 2);
                }
                KinbrNo = LoginUserInfo.KinbrNo;
                TellerNo = LoginUserInfo.TellerNo;
                KinbrName = LoginUserInfo.KinbrName;
                TradeTime = System.DateTime.Now.ToString("HH:mm:dd");
            }
        }
        Canvas canvas;

        /// <summary>
        /// 
        /// </summary>
        /// <summary>
        /// 文档名称
        /// </summary>
        public string DocumentName { get; set; }

        /// <summary>
        /// 文档类别
        /// </summary>
        public string DocumentGroup { set; get; }

        /// <summary>
        /// 设置打印元素组的IsPrint属性
        /// </summary>
        /// <param name="groupName"></param>
        /// <param name="isPrint"></param>
        public void SetIsPrintByGroup(string groupName, bool isPrint)
        {
            var panel = this.Content as Canvas;
            if (panel == null) return;
            foreach (var item in panel.Children)
            {
                var element = item as IPrintElement;
                if (element != null && element.Group == groupName)
                {
                    element.IsPrint = isPrint;
                }
            }
        }

        /// <summary>
        /// 此表单实例的最大打印次数
        /// （模板的最大权限次数是可以修改和删除的，实例的最大打印次数是一旦保存，就不能再变动了，打印时控制权限是按实例权限次数来控制，但是实例权限次数来源于模板权限次数）
        /// guanjian
        /// </summary>
        public int FormRoleCount
        {
            get;
            set;
        }

        /// <summary>
        /// 此表单的打印数据编号，保存在历史数据中的唯一编号。每打印一次都要根据此编号新增一条打印记录
        /// guanjian
        /// </summary>
        public string FormDataId
        {
            get;
            set;
        }

        /// <summary>
        /// 表单的中文名称，补打的时候要显示给柜员看
        /// guanjian
        /// </summary>
        public string FormDataName
        {
            get;
            set;
        }

        /// <summary>
        ///
        /// </summary>
        /// <param name="top"></param>
        /// <param name="left"></param>
        /// <param name="printValue"></param>
        /// <param name="fontSize"></param>
        /// <param name="isBold"></param>
        /// <param name="fontFamily"></param>
        /// <returns></returns>
        public bool AddPrintLine(double top, double left, string printValue, double fontSize = double.NaN, bool isBold = false, FontFamily fontFamily = null)
        {
            var panel = this.Content as Canvas;
            //若Content不存在，则给默认数据
            //if (panel == null) return false;
            if (panel == null)
            {
                panel = new Canvas();
                this.Content = panel;
            }
            var element = new PrintStringElement();
            Canvas.SetTop(element, top);
            Canvas.SetLeft(element, left);
            element.Value = printValue;

            if (!double.IsNaN(fontSize))
                element.FontSize = fontSize;
            if (isBold)
                element.FontWeight = System.Windows.FontWeights.Bold;
            if (fontFamily != null)
                element.FontFamily = fontFamily;
            panel.Children.Add(element);
            return true;
        }

        /// <summary>
        /// 交易年
        /// </summary>
        public string TradeYear
        {
            get { return (string)GetValue(TradeYearProperty); }
            set { SetValue(TradeYearProperty, value); }
        }

        public static readonly DependencyProperty TradeYearProperty =
            DependencyProperty.Register("TradeYear", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 交易月号
        /// </summary>
        public string TradeMonth
        {
            get { return (string)GetValue(TradeMonthProperty); }
            set { SetValue(TradeMonthProperty, value); }
        }

        public static readonly DependencyProperty TradeMonthProperty =
            DependencyProperty.Register("TradeMonth", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 交易日号
        /// </summary>
        public string TradeDay
        {
            get { return (string)GetValue(TradeDayProperty); }
            set { SetValue(TradeDayProperty, value); }
        }

        /// <summary>
        /// 是否显示日期
        /// </summary>
        public string IsShowDate
        {
            get { return (string)GetValue(IsShowDateProperty); }
            set { SetValue(IsShowDateProperty, value); }
        }

        public static readonly DependencyProperty IsShowDateProperty =
            DependencyProperty.Register("IsShowDate", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));


        public static readonly DependencyProperty TradeDayProperty =
            DependencyProperty.Register("TradeDay", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        ///         /// </summary>
        public string IsHasHeader
        {
            get { return (string)GetValue(IsHasHeaderProperty); }
            set { SetValue(IsHasHeaderProperty, value); }
        }

        public static readonly DependencyProperty IsHasHeaderProperty =
            DependencyProperty.Register("IsHasHeader", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));


        /// <summary>
        /// 机构号
        /// </summary>
        public string KinbrNo
        {
            get { return (string)GetValue(KinbrNoProperty); }
            set { SetValue(KinbrNoProperty, value); }
        }

        public static readonly DependencyProperty KinbrNoProperty =
            DependencyProperty.Register("KinbrNo", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        public string IsShowKinbrNo
        {
            get { return (string)GetValue(IsShowKinbrNoProperty); }
            set { SetValue(IsShowKinbrNoProperty, value); }
        }

        public static readonly DependencyProperty IsShowKinbrNoProperty =
            DependencyProperty.Register("IsShowKinbrNo", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 柜员号
        /// </summary>
        public string TellerNo
        {
            get { return (string)GetValue(TellerNoProperty); }
            set { SetValue(TellerNoProperty, value); }
        }

        /// <summary>
        ///
        /// </summary>
        public static readonly DependencyProperty TellerNoProperty =
            DependencyProperty.Register("TellerNo", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        public string IsShowTellerNo
        {
            get { return (string)GetValue(IsShowTellerNoProperty); }
            set { SetValue(IsShowTellerNoProperty, value); }
        }

        public static readonly DependencyProperty IsShowTellerNoProperty =
            DependencyProperty.Register("IsShowTellerNo", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 序号
        /// </summary>
        public string PrintNumber
        {
            get { return (string)GetValue(PrintNumberProperty); }
            set { SetValue(PrintNumberProperty, value); }
        }

        // Using a DependencyProperty as the backing store for PrintNumber.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty PrintNumberProperty =
            DependencyProperty.Register("PrintNumber", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        public string IsShowPrintNumber
        {
            get { return (string)GetValue(IsShowPrintNumberProperty); }
            set { SetValue(IsShowPrintNumberProperty, value); 

            }
        }

        public static readonly DependencyProperty IsShowPrintNumberProperty =
            DependencyProperty.Register("IsShowPrintNumber", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 柜员流水号
        /// </summary>
        public string SerialNumber
        {
            get { return (string)GetValue(SerialNumberProperty); }
            set { SetValue(SerialNumberProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SerialNumber.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SerialNumberProperty =
            DependencyProperty.Register("SerialNumber", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        public string IsShowSerialNumber
        {
            get { return (string)GetValue(IsShowSerialNumberProperty); }
            set { SetValue(IsShowSerialNumberProperty, value); }
        }

        public static readonly DependencyProperty IsShowSerialNumberProperty =
            DependencyProperty.Register("IsShowSerialNumber", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 交易号
        /// </summary>
        public string TradeCode
        {
            get { return (string)GetValue(TradeCodeProperty); }
            set { SetValue(TradeCodeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for TradeCode.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TradeCodeProperty =
            DependencyProperty.Register("TradeCode", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));
        public string IsShowTradeCode
        {
            get { return (string)GetValue(IsShowTradeCodeProperty); }
            set { SetValue(IsShowTradeCodeProperty, value); }
        }

        public static readonly DependencyProperty IsShowTradeCodeProperty =
            DependencyProperty.Register("IsShowTradeCode", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 授权人
        /// </summary>
        public string Authentic
        {
            get { return (string)GetValue(AuthenticProperty); }
            set { SetValue(AuthenticProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Authentic.  This enables animation, styling, binding, etc...
        /// <summary>
        ///
        /// </summary>
        public static readonly DependencyProperty AuthenticProperty =
            DependencyProperty.Register("Authentic", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));
        public string IsShowAuthentic
        {
            get { return (string)GetValue(IsShowAuthenticProperty); }
            set { SetValue(IsShowAuthenticProperty, value); }
        }

        public static readonly DependencyProperty IsShowAuthenticProperty =
            DependencyProperty.Register("IsShowAuthentic", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 主机流水值
        /// </summary>
        public string HostSerialNumber
        {
            get { return (string)GetValue(HostSerialNumberProperty); }
            set { SetValue(HostSerialNumberProperty, value); }
        }

        /// <summary>
        ///
        /// </summary>
        public static readonly DependencyProperty HostSerialNumberProperty =
            DependencyProperty.Register("HostSerialNumber", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));
        public string IsShowHostSerialNumber
        {
            get { return (string)GetValue(IsShowHostSerialNumberProperty); }
            set { SetValue(IsShowHostSerialNumberProperty, value); }
        }

        public static readonly DependencyProperty IsShowHostSerialNumberProperty =
            DependencyProperty.Register("IsShowHostSerialNumber", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 机构名称
        /// </summary>
        public string KinbrName
        {
            get { return (string)GetValue(KinbrNameProperty); }
            set { SetValue(KinbrNameProperty, value); }
        }

        /// <summary>
        ///
        /// </summary>
        public static readonly DependencyProperty KinbrNameProperty =
            DependencyProperty.Register("KinbrName", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));
        public string IsShowKinbrName
        {
            get { return (string)GetValue(IsShowKinbrNameProperty); }
            set { SetValue(IsShowKinbrNameProperty, value); }
        }

        public static readonly DependencyProperty IsShowKinbrNameProperty =
            DependencyProperty.Register("IsShowKinbrName", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 交易时间
        /// </summary>
        public string TradeTime
        {
            get { return (string)GetValue(TradeTimeProperty); }
            set { SetValue(TradeTimeProperty, value); }
        }

        /// <summary>
        ///
        /// </summary>
        public static readonly DependencyProperty TradeTimeProperty =
            DependencyProperty.Register("TradeTime", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));
        public string IsShowTradeTime
        {
            get { return (string)GetValue(IsShowTradeTimeProperty); }
            set { SetValue(IsShowTradeTimeProperty, value); }
        }

        public static readonly DependencyProperty IsShowTradeTimeProperty =
            DependencyProperty.Register("IsShowTradeTime", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// 交易名称
        /// </summary>
        public string TradeName
        {
            get { return (string)GetValue(TradeNameProperty); }
            set { SetValue(TradeNameProperty, value); }
        }

        /// <summary>
        ///
        /// </summary>
        public static readonly DependencyProperty TradeNameProperty =
            DependencyProperty.Register("TradeName", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        public string IsShowTradeName
        {
            get { return (string)GetValue(IsShowTradeNameProperty); }
            set { SetValue(IsShowTradeNameProperty, value); }
        }

        public static readonly DependencyProperty IsShowTradeNameProperty =
            DependencyProperty.Register("IsShowTradeName", typeof(string), typeof(Print_Etc), new UIPropertyMetadata(string.Empty));

        public string IsForSeach
        {
            get { return (string)GetValue(IsForSeachProperty); }
            set { SetValue(IsForSeachProperty, value); }
        }

        public static readonly DependencyProperty IsForSeachProperty =
            DependencyProperty.Register("IsForSeach", typeof(string), typeof(Print_Etc), new UIPropertyMetadata("True"));
    }
}