﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows;
using Platform.Common;
using System.Windows.Media;
using System.Windows.Input;
using System.ComponentModel;
using Platform.Common.RunningParameters;
using Platform.Print;
using TellerSystem.PrintTemplate.Base;
using System.Windows.Media.Imaging;
using TellerSystem.Communication;
using TellerSystem.Communication.MessageHelper;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.PrintTemplate.PrintTemplateData
{
    /// <summary>
    /// 业务凭证
    /// </summary>
    public class BlankPrint_BusinessVoucher : TemplateBase
    {
        public BlankPrint_BusinessVoucher()
        {
            PrintType = PrintTypes.BusinessVoucher;
            //左侧预留1cm的装订区域
            this.Padding = new Thickness(1 * 0.3937 * 96, 0, 0, 0);
            IsDrawingOraData = true;
        }

        protected override void OnContentChanged(object oldContent, object newContent)
        {
            base.OnContentChanged(oldContent, newContent);
            var canvas = newContent as Canvas;
            if (canvas == null) return;
            //定制格式
            var grid = new Grid();
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(100) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(200) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(100) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(200) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });
            for (int i = 0; i < 15; i++)
            {
                grid.RowDefinitions.Add(new RowDefinition { Height = GridLength.Auto, MinHeight = 25 });
            }
            CanvasHelper.SetInternalGrid(canvas, grid);
            CanvasHelper.SetOffset(canvas, new Point(100, 100));
            //加载事件
            canvas.Loaded += (sender, e) =>
            {
                (sender as Canvas).Refresh();
            };
        }

        /// <summary>
        /// 是否绘制默认的数据
        /// </summary>
        public bool IsDrawingOraData { get; set; }

        protected override void DrawingCommonLayer(Extension.PrintAttributes attr)
        {
            base.DrawingCommonLayer(attr);
            if (IsDrawingOraData)
            {
                #region
                if (!string.IsNullOrEmpty(LoginUserInfo.KinbrNo))
                {
                    string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
                    if (LoginUserInfo.KinbrNo.Substring(0, 3) == "801")
                    {
                        //芝罘打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "807")
                    {
                        //莱山打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "803")
                    {
                        //新泰打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "805")
                    {
                        //冠县打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "820")
                    {
                        //大石桥打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "108")
                    {
                        //固始天骄打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "118")
                    {
                        //乳山天骄打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (FLAG == "V5")
                    {
                        //青隆不统一打印电子签章
                        if (LoginUserInfo.KinbrNo.Substring(0, 3) == "302")
                        {
                            this.ElecSealHelper.Sign2Point(new Point(550, 323), TellerSystem.PrintTemplate.ElecSeal.SealKind.Biz);
                        }
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "802")
                    {
                        //泗水打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (BankFlag == "V0")
                    {
                        //中牟打印印章
                        this.E_Seal_Location = new Point(290, 380);
                        this.PrintElecKeyCode = new Point(300, 476);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "806")
                    {
                        //蒙阴打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else
                    {
                        //阳光打印印章
                        this.E_Seal_Location = new Point(580, 358);
                    }
                }
                #endregion
                if (DesignerProperties.GetIsInDesignMode(this))
                {
                    ////this.DrawingText(500, 45, "票    号:[0123456]");
                    //this.DrawingText(95, 60, "机构名称:[XXX支行]");
                    //this.DrawingText(374, 60, "[2010-10-23 00:00:00]");
                    //this.DrawingText(95, 80, "交易名称:[个人储蓄存款]（[2000]）");
                    //this.DrawingText(95, 456, "业务流水:[000010121]");
                    //this.DrawingText(245, 456, "授权柜员:[XXX(xxxxxx)]");
                    //this.DrawingText(420, 456, "复核柜员:[XXX(xxxxxx)]");
                    //this.DrawingText(600, 456, "经办柜员:[XXX(xxxxxx)]");
                    //this.DrawingText(95, 476, "柜面流水:[20141010123456789]");
                    //this.DrawingText(420, 476, "柜员流水:[20141010123456789]");
                }
                else
                {
                    ////解析日期
                    //var date = DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd", null);
                    //var time = DateTime.ParseExact(LoginUserInfo.GetUserInfo("TradeTime"), "HHmmss", null);
                    ////this.DrawingText(500, 45, "票    号:" + attr.PrintNumber.PadLeft(7, '0'));
                    //this.DrawingText(95, 60, "机构名称:" + LoginUserInfo.KinbrName);
                    //this.DrawingText(374, 60, string.Format("{0} {1}", date.ToString("yyyy-MM-dd"), time.ToString("HH:mm:dd")));
                    //this.DrawingText(95, 80, "交易名称:" + attr.TradeModel.Name + "（" + attr.TradeModel.Code + "）");
                    //this.DrawingText(95, 456, "业务流水:" + attr.BusinessTransaction);
                    //this.DrawingText(245, 456, "授权柜员:" + attr.Authentic);
                    //this.DrawingText(420, 456, "复核柜员:" + attr.Review);
                    //this.DrawingText(600, 456, string.Format("经办柜员:{0}({1})", LoginUserInfo.TellerName, LoginUserInfo.TellerNo));
                    ////隐藏柜面流水的打印
                    //// this.DrawingText(95, 476, "柜面流水:" + attr.SystemSerialNumber);
                    //this.DrawingText(95, 476, "柜员流水:" + attr.SerialNumber);
                    //#region
                    //if (!string.IsNullOrEmpty(LoginUserInfo.KinbrNo))
                    //{
                    //    if (LoginUserInfo.KinbrNo.Substring(0, 3) == "801")
                    //    {
                    //        //芝罘打印印章
                    //        this.E_Seal_Location = new Point(550, 323);
                    //    }
                    //    else
                    //    {
                    //        //阳光打印印章
                    //        this.E_Seal_Location = new Point(580, 358);
                    //    }
                    //}
                    //#endregion
                }
            }
            //var canvas = this.Content as Canvas;
            //if (canvas == null) return;
            //canvas.Refresh();
        }
    }
    public class BlankPrint_业务凭证 : BlankPrint_BusinessVoucher
    {
    }

    public class BlankPrint_空白凭条 : BlankPrint_BusinessVoucher
    {
    }
}
