﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows;
using Platform.Common;
using System.Windows.Media;
using System.Windows.Input;
using System.ComponentModel;
using System.Windows.Media.Imaging;
using Platform.Common.RunningParameters;
using Platform.Print;
using TellerSystem.PrintTemplate.Base;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.Communication;
using TellerSystem.Communication.MessageHelper;

namespace TellerSystem.PrintTemplate.PrintTemplateData
{
    /// <summary>
    /// 业务凭证
    /// </summary>
    public class Print_BusinessVoucher : TemplateBase
    {
        public Print_BusinessVoucher()
        {
            PrintType = PrintTypes.BusinessVoucher;
            //左侧预留1cm的装订区域
            this.Padding = new Thickness(1 * 0.3937 * 96, 0, 0, 0);
            IsDrawingDefaultData = true;
            IsAuthVisible = true;
        }
        /// <summary>
        /// 报文通讯对象
        /// </summary>
        protected Message _msg = new Message();
        string FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");

        /// <summary>
        /// 初始化域
        /// </summary>
        /// <param name="message"></param>
        protected void InitFd(Message message)
        {
            message.Init();
        }
        /// <summary>
        /// 初始化公共域
        /// </summary>
        /// <param name="message"></param>
        protected void CommSend(Message message)
        {
            //开户行机构号
            message.Fd2 = LoginUserInfo.KinbrNo;// LoginUserInfo.Opnbr;
            //机构号
            message.Fd3 = LoginUserInfo.KinbrNo;
            //业务流水
            message.Fd4 = this.PrintAttribute.BusinessTransaction;
            //交易日期
            message.Fd5 = LoginUserInfo.TradeDate;
            //发生时间
            message.Fd6 = DateTime.Now.ToString("HHmmss");
            //柜员号
            message.Fd7 = LoginUserInfo.TellerNo;
            //终端号
            message.Fd10 = LoginUserInfo.TtyName;
            message.Fd96_9 = "0100"; //本行柜面
            message.Fd96_7 = "0";
            message.Fd96_8 = "0";
            message.Fd96_C = "0100"; //本行柜面
            message.Fd96_D = LoginUserInfo.TradeDate;
            message.Fd96_E = message.Fd6;

        }
        /// <summary>
        /// 和平台通讯API
        /// </summary>
        /// <param name="message"></param>
        /// <param name="transitType">TransitType枚举值</param>
        /// <returns></returns>
        protected bool CallAgn(Message message, string transitType)
        {
            var returnValue = false;
            string preTradeNo = message.Fd16;
            message.TransitNode = TransitType.CallAgnRun;
            message.BitMapType = TellerSystem.Communication.Message.BitMap.Agn;
            returnValue = message.DoTransit();
            if (message.Fd16 != preTradeNo && message.Fd12 == "0000")
            {
                message.Fd12 = "COMN";
            }
            return returnValue;
        }

        protected override void OnContentChanged(object oldContent, object newContent)
        {
            base.OnContentChanged(oldContent, newContent);
            var canvas = newContent as Canvas;
            if (canvas == null) return;
            //定制格式
            var grid = new Grid();
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(100) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(200) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(100) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(200) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });
            for (int i = 0; i < 15; i++)
            {
                grid.RowDefinitions.Add(new RowDefinition { Height = GridLength.Auto, MinHeight = 25 });
            }
            CanvasHelper.SetInternalGrid(canvas, grid);
            CanvasHelper.SetOffset(canvas, new Point(100, 100));
            //加载事件
            canvas.Loaded += (sender, e) =>
            {
                (sender as Canvas).Refresh();
            };
        }

        /// <summary>
        /// 是否绘制默认的数据
        /// </summary>
        public bool IsDrawingDefaultData { get; set; }
        public bool IsAuthVisible { get; set; }

        protected override void DrawingCommonLayer(Extension.PrintAttributes attr)
        {
            base.DrawingCommonLayer(attr);
            if (IsDrawingDefaultData)
            {
                if (!string.IsNullOrEmpty(LoginUserInfo.KinbrNo))
                {
                    if (LoginUserInfo.KinbrNo.Substring(0, 3) == "801")
                    {
                        //芝罘打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "807")
                    {
                        //莱山打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "803")
                    {
                        //新泰打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "805")
                    {
                        //冠县打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "820")
                    {
                        if (this.E_Seal_Location == new Point(-1, -1))
                        {
                            this.E_Seal_Location = null;
                        }
                        else
                        {
                            //大石桥打印印章
                            this.E_Seal_Location = new Point(550, 323);
                        }
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "108")
                    {
                        //固始天骄打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "118")
                    {
                        //乳山天骄打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (FLAG == "V5")
                    {
                        var str = "2353|2355|3998|3401|3317|5863|5872|8084|8170|4422|5020|5021";
                        //青隆电子签章 分为不同的印章
                        if (LoginUserInfo.KinbrNo.Substring(0, 3) == "302" && !str.Contains(attr.TradeModel.Code.Trim()))
                        {
                            this.ElecSealHelper.Sign2Point(new Point(550, 323), TellerSystem.PrintTemplate.ElecSeal.SealKind.Biz);
                        }
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "802")
                    {
                        //泗水打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else if (BankFlag == "V0")
                    {
                        //中牟打印印章
                        this.E_Seal_Location = new Point(290, 380);
                        this.PrintElecKeyCode = new Point(300, 476);
                    }
                    else if (LoginUserInfo.KinbrNo.Substring(0, 3) == "806")
                    {
                        //蒙阴打印印章
                        this.E_Seal_Location = new Point(550, 323);
                    }
                    else
                    {
                        //阳光打印印章
                        this.E_Seal_Location = new Point(580, 358);
                    }
                }
                if (DesignerProperties.GetIsInDesignMode(this))
                {
                    //this.DrawingText(500, 45, "票    号:[0123456]");
                    //this.DrawingText(95, 60, "机构名称:[XXX支行]");
                    //this.DrawingText(374, 60, "[2010-10-23 00:00:00]");
                    //this.DrawingText(95, 80, "交易名称:[个人储蓄存款]（[2000]）");
                    //this.DrawingText(450, 476, "业务流水:[000010121]");
                    //this.DrawingText(245, 456, "授权柜员:[XXX(xxxxxx)]");
                    //this.DrawingText(420, 456, "复核柜员:[XXX(xxxxxx)]");
                    //this.DrawingText(600, 456, "经办柜员:[XXX(xxxxxx)]");
                    //this.DrawingText(95, 476, "柜面流水:[20141010123456789]");
                    //this.DrawingText(420, 476, "柜员流水:[20141010123456789]");
                    this.FontSize = 14;
                    this.DrawingText(70, 100, "机构名称:[XXX支行]");
                    this.DrawingText(400, 100, "代码:[2000]");
                    this.DrawingText(500, 100, "交易:[XXX]");
                    this.DrawingText(100, 456, "交易日期:[00000000 00:00:00]");
                    this.DrawingText(300, 456, "柜员:[(xxxxxx)]");
                    this.DrawingText(450, 456, "流水号:[000010121]");
                    this.DrawingText(100, 476, "备注:");
                    this.DrawingText(260, 476, "柜员流水:");

                    if (!string.IsNullOrEmpty(LoginUserInfo.KinbrNo) && LoginUserInfo.KinbrName.Contains("齐丰"))
                    {
                        this.Margin = new Thickness(0, -40, 0, 0);
                    }
                    //this.DrawingText(150, 436, "授权柜员:" + attr.Authentic);
                    //this.DrawingText(300, 436, "复核柜员:" + attr.Review);
                    if (FLAG == "V14") { this.DrawingText(300, 20, "存款保险，保护您珍贵的存款。"); }
                }
                else
                {
                    this.FontSize = 14;
                    //解析日期
                    var date = DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd", null);
                    var time = DateTime.ParseExact(LoginUserInfo.GetUserInfo("TradeTime"), "HHmmss", null);
                    //this.DrawingText(500, 45, "票    号:" + attr.PrintNumber.PadLeft(7, '0'));

                    this.DrawingText(100, 100, "机构名称:" + LoginUserInfo.KinbrName);
                    this.DrawingText(400, 100, "代码:" + attr.TradeModel.Code);
                    this.DrawingText(500, 100, "交易:" + attr.TradeModel.Name);
                    this.DrawingText(100, 456, "交易日期:" + string.Format("{0} {1}", date.ToString("yyyyMMdd"), time.ToString("HH:mm:ss")));
                    if (FLAG == "V22" || FLAG == "V21")
                    {
                        this.DrawingText(300, 456, "柜员:" + LoginUserInfo.TellerName + "(" + LoginUserInfo.TellerNo + ")");
                    }
                    else if (FLAG == "V5" && LoginUserInfo.KinbrNo.Substring(0, 3) == "302")
                    {
                        this.DrawingText(300, 456, "柜员:" + LoginUserInfo.TellerNo + "(" + LoginUserInfo.TellerName + ")");
                    }
                    else
                    {
                        this.DrawingText(300, 456, "柜员:" + LoginUserInfo.TellerNo);
                    }
                    if (FLAG == "V0")//中牟
                    {
                        //var list = TradeMenuManager.Instance.GetTradeModel(x => x.Code == "2381");
                        if (attr.TradeModel.Code.Trim() != "2381" && attr.TradeModel.Code.Trim() != "5045")//2381交易不打印、5045
                        {
                            this.DrawingText(450, 476, "业务流水: " + attr.BusinessTransaction);
                        }
                    }
                    else
                    {
                        this.DrawingText(450, 456, "业务流水:" + attr.BusinessTransaction);
                    }
                    this.DrawingText(100, 476, "备注：");

                    if (!string.IsNullOrEmpty(LoginUserInfo.KinbrNo) && LoginUserInfo.KinbrName.Contains("齐丰"))
                    {
                        this.Margin = new Thickness(0, -40, 0, 0);
                    }
                    if (attr.Authentic != null && attr.Authentic.Trim() != "")
                    {
                        if (IsAuthVisible)
                        {
                            //var list = TradeMenuManager.Instance.GetTradeModel(x => x.Code == "2381");
                            if (FLAG == "V0")
                            {
                                if (attr.TradeModel.Code.Trim() != "2381")//2381交易不打印
                                {
                                    this.DrawingText(150, 436, "授权柜员:" + attr.Authentic);
                                }
                            }
                            else
                            {
                                this.DrawingText(150, 436, "授权柜员:" + attr.Authentic);
                            }
                        }
                        else
                        {

                        }
                    }
                    if (attr.Review != null && attr.Review.Trim() != "")
                    {
                        this.DrawingText(350, 436, "复核柜员:" + attr.Review);
                    }
                    // this.DrawingText(600, 456, string.Format("经办柜员:{0}({1})", LoginUserInfo.TellerName, LoginUserInfo.TellerNo));
                    //隐藏柜面流水的打印
                    //this.DrawingText(95, 476, "柜面流水:" + attr.SystemSerialNumber);
                    if (FLAG == "V0")//中牟  业务流水和柜员流水调换位置
                    {
                        //var list = TradeMenuManager.Instance.GetTradeModel(x => x.Code == "2381");
                        if (attr.TradeModel.Code.Trim() != "2381" && attr.TradeModel.Code.Trim() != "5045")//2381交易不打印、5045
                        {
                            this.DrawingText(450, 456, "柜员流水: " + attr.SerialNumber);
                        }
                    }
                    else
                    {
                        this.DrawingText(450, 476, "柜员流水:" + attr.SerialNumber);
                    }
                }
            }
            if (FLAG == "V14") { this.DrawingText(300, 20, "存款保险，保护您珍贵的存款。"); }
            //var canvas = this.Content as Canvas;
            //if (canvas == null) return;
            //canvas.Refresh();
        }
    }
    /// <summary>
    /// 山东结算转账凭证
    /// </summary>
    public class BlankPrint_SettlementdepositsSD : TemplateBase
    {
        public BlankPrint_SettlementdepositsSD()
        {
            PrintType = PrintTypes.SettlementdepositsSD;
            //左侧预留1cm的装订区域
            this.Padding = new Thickness(1 * 0.3937 * 96, 0, 0, 0);
            IsDrawingDefaultData = true;
            IsAuthVisible = true;
        }

        protected override void OnContentChanged(object oldContent, object newContent)
        {
            base.OnContentChanged(oldContent, newContent);
            var canvas = newContent as Canvas;
            if (canvas == null) return;
            //定制格式
            var grid = new Grid();
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(100) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(200) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(100) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(200) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });
            for (int i = 0; i < 15; i++)
            {
                grid.RowDefinitions.Add(new RowDefinition { Height = GridLength.Auto, MinHeight = 25 });
            }
            CanvasHelper.SetInternalGrid(canvas, grid);
            CanvasHelper.SetOffset(canvas, new Point(100, 100));
            //加载事件
            canvas.Loaded += (sender, e) =>
            {
                (sender as Canvas).Refresh();
            };
        }

        /// <summary>
        /// 是否绘制默认的数据
        /// </summary>
        public bool IsDrawingDefaultData { get; set; }
        public bool IsAuthVisible { get; set; }

        protected override void DrawingCommonLayer(Extension.PrintAttributes attr)
        {
            base.DrawingCommonLayer(attr);
            if (IsDrawingDefaultData)
            {
                if (DesignerProperties.GetIsInDesignMode(this))
                {
                    //this.DrawingText(500, 45, "票    号:[0123456]");
                    //this.DrawingText(95, 60, "机构名称:[XXX支行]");
                    //this.DrawingText(374, 60, "[2010-10-23 00:00:00]");
                    //this.DrawingText(95, 80, "交易名称:[个人储蓄存款]（[2000]）");
                    //this.DrawingText(450, 476, "业务流水:[000010121]");
                    //this.DrawingText(245, 456, "授权柜员:[XXX(xxxxxx)]");
                    //this.DrawingText(420, 456, "复核柜员:[XXX(xxxxxx)]");
                    //this.DrawingText(600, 456, "经办柜员:[XXX(xxxxxx)]");
                    //this.DrawingText(95, 476, "柜面流水:[20141010123456789]");
                    //this.DrawingText(420, 476, "柜员流水:[20141010123456789]");
                    this.FontSize = 14;
                    this.DrawingText(100, 126, "机构名称:[XXX支行]");
                    this.DrawingText(350, 126, "代码:[2000]");
                    this.DrawingText(450, 126, "交易:[XXX]");
                    this.DrawingText(505, 105, "交易日期:[yyyy    MM    dd]");
                    this.DrawingText(493, 484, "柜员:[(xxxxxx)]");
                    this.DrawingText(100, 105, "流水号:[000010121]");
                    this.DrawingText(350, 105, "柜员流水:");
                    //this.DrawingText(150, 436, "授权柜员:" + attr.Authentic);
                    //this.DrawingText(300, 436, "复核柜员:" + attr.Review);
                }
                else
                {
                    this.FontSize = 14;
                    //解析日期
                    var date = DateTime.ParseExact(LoginUserInfo.TradeDate, "yyyyMMdd", null);
                    var time = DateTime.ParseExact(LoginUserInfo.GetUserInfo("TradeTime"), "HHmmss", null);
                    //this.DrawingText(500, 45, "票    号:" + attr.PrintNumber.PadLeft(7, '0'));
                    this.DrawingText(100, 126, "机构名称:" + LoginUserInfo.KinbrName);
                    this.DrawingText(350, 126, "代码:" + attr.TradeModel.Code);
                    this.DrawingText(450, 126, "交易:" + attr.TradeModel.Name);
                    this.DrawingText(505, 105, "交易日期:" + string.Format("{0}", date.ToString("yyyy    MM    dd")));
                    this.DrawingText(513, 484, LoginUserInfo.TellerNo);//柜员
                    this.DrawingText(100, 105, "业务流水:" + attr.BusinessTransaction);
                    // this.DrawingText(100, 476, "备注：");
                    if (attr.Authentic != null && attr.Authentic.Trim() != "")
                    {
                        if (IsAuthVisible)
                        {
                            //授权柜员
                            this.DrawingText(325, 484, attr.Authentic);
                        }
                        else
                        {

                        }
                    }
                    if (attr.Review != null && attr.Review.Trim() != "")
                    {
                        //复核柜员
                        this.DrawingText(137, 484, attr.Review);
                    }
                    // this.DrawingText(600, 456, string.Format("经办柜员:{0}({1})", LoginUserInfo.TellerName, LoginUserInfo.TellerNo));
                    //隐藏柜面流水的打印
                    //this.DrawingText(95, 476, "柜面流水:" + attr.SystemSerialNumber);
                    this.DrawingText(350, 105, "柜员流水:" + attr.SerialNumber);
                }
            }
            //var canvas = this.Content as Canvas;
            //if (canvas == null) return;
            //canvas.Refresh();
        }
    }

    public class Print_业务凭证 : Print_BusinessVoucher
    {
    }

    public class Print_空白凭条 : Print_BusinessVoucher
    {
    }
    public class BlankPrint_转账凭条 : BlankPrint_SettlementdepositsSD
    {

    }
}
