// 引入必要的类型声明，在 TypeScript 中没有与 C# 这些命名空间严格对应的直接导入，这里只是形式上保留
// 实际使用时需要根据具体的类型定义和依赖情况进行调整
// 这里假设后续使用到的类型在全局作用域或者其他合适的地方有定义

// 定义 AuthResult 类，对应 C# 中的 AuthResult 类
class AuthResult {
    // 授权结果，对应 C# 中的 IsSuccess 属性
    public IsSuccess: boolean;
    // 授权结果提示信息，对应 C# 中的 Msg 属性
    public Msg: string;

    // 构造函数，用于初始化对象
    constructor(isSuccess: boolean = false, msg: string = "") {
        this.IsSuccess = isSuccess;
        this.Msg = msg;
    }

    // 将对象属性转换为字符串数组的方法，对应 C# 中的 ChangeToTable 方法
    public ChangeToTable(): string[] {
        return [this.IsSuccess.toString(), this.Msg];
    }

    // 从对象数组中读取数据并赋值给当前对象属性的方法，对应 C# 中的 ChangeFromTable 方法
    public ChangeFromTable(parms: any[]): void {
        this.IsSuccess = Boolean(parms[0]);
        this.Msg = String(parms[1]);
    }
}

// 导出 AuthResult 类，方便在其他地方使用
export { AuthResult };