// 导入必要的类型
// 这里假设存在类似 IEntity 的接口定义，若没有则需要根据实际情况补充
interface IEntity {
    // 可以根据实际情况补充 IEntity 接口的具体属性和方法
}

/**
 * BatInfo 类，对应批量详情查询表 bat_info 的实体类
 */
export class BatInfo implements IEntity {
    /**
     * 系统编号
     */
    public syscode: string;

    /**
     * 批量编码
     */
    public batcode: string;

    /**
     * 子系统编号
     */
    public batsubcode: string;

    /**
     * 批量名称
     */
    public batname: string;

    /**
     * 批量说明
     */
    public batdesp: string;

    /**
     * 批量处理状态
     * 0: 未处理
     * 1: 正在处理
     * 2: 已成功
     * 3: 批量失败
     */
    public stat: string;

    /**
     * 是否需要重做
     */
    public isredo: string;

    /**
     * 本批次子批次数量
     */
    public batnum: string;

    /**
     * 已执行子批次数量
     */
    public batovernum: string;

    /**
     * 剩余子批次数
     */
    public remainingnum: string;

    /**
     * 已执行时间
     */
    public batovertime: string;

    /**
     * 预计执行时间
     */
    public expecttime: string;

    /**
     * 营业日期，格式为 "yyyyMMdd"
     */
    public tx_date: string;

    /**
     * 最后更新日期，格式为 "yyyy-MM-dd HH:mm:ss"
     */
    public end_date: string;

    /**
     * 操作员
     */
    public opteller: string;

    /**
     * 复核员
     */
    public chkteller: string;

    /**
     * 预留字段1
     */
    public filler1: string;

    /**
     * 从对象数组中初始化 BatInfo 实例的属性
     * @param parms - 对象数组，包含各属性的值
     */
    public ChangeFromTable(parms: any[]): void {
        this.syscode = parms[0].toString();
        this.batcode = parms[1].toString();
        this.batsubcode = parms[2].toString();
        this.batname = parms[3].toString();
        this.batdesp = parms[4].toString();
        this.stat = parms[5].toString();
        this.isredo = parms[6].toString();
        this.batnum = parms[7].toString();
        this.batovernum = parms[8].toString();
        this.remainingnum = parms[9].toString();
        this.batovertime = parms[10].toString();
        this.expecttime = parms[11].toString();
        this.end_date = parms[12].toString();
        this.opteller = parms[13].toString();
        this.chkteller = parms[14].toString();
        this.filler1 = parms[15].toString();
        this.tx_date = parms[16].toString();
    }

    /**
     * 将 BatInfo 实例的属性转换为字符串数组
     * @returns 包含各属性值的字符串数组
     */
    public ChangeToTable(): string[] {
        return [
            this.syscode,
            this.batcode,
            this.batsubcode,
            this.batname,
            this.batdesp,
            this.stat,
            this.isredo,
            this.batnum,
            this.batovernum,
            this.remainingnum,
            this.batovertime,
            this.expecttime,
            this.end_date,
            this.opteller,
            this.chkteller,
            this.filler1,
            this.tx_date
        ];
    }
}