// 定义接口 IEntity，类似于 C# 中的接口定义
interface IEntity {
    ChangeFromTable(parms: any[]): void;
    ChangeToTable(): string[];
}

// 定义 BatSysInfo 类，实现 IEntity 接口
class BatSysInfo implements IEntity {
    // 批量编码
    batcode: string;
    // 批量名称
    batname: string;
    // 批量说明
    batdesp: string;
    // 系统编号
    syscode: string;
    // 主顺序号
    mainseqn: string;
    // 批量处理状态  0:未处理  1:正在处理  2:已成功  3:批量失败
    stat: string;
    // 营业日期   "yyyyMMdd"
    tx_date: string;
    // 开始时间 "yyyy-MM-dd HH:mm:ss"
    beg_date: string;
    // 结束日期 "yyyy-MM-dd HH:mm:ss"
    end_date: string;
    // 用时     "hh:mm:ss"
    elapsetime: TimeSpan;
    // 操作员
    opteller: string;
    // 复核员
    chkteller: string;
    // 批量子批量数
    subcnt: string;
    // 是否自动批量  0:不参与自动跑批  1:参与自动跑批
    bat_ind: string;
    // 预留字段1
    filler1: string;
    // 预留字段2
    filler2: string;
    // 查看详情时定位自动刷新的时间
    refreshtime: string;
    // 批量启动前置校验, 0:不校验 1: 校验
    batcheck: string;

    constructor() {
        // 初始化 elapsetime 为 TimeSpan 的最小值
        this.elapsetime = TimeSpan.MinValue;
    }

    // 从数组参数中更新对象属性
    ChangeFromTable(parms: any[]): void {
        this.batcode = parms[0].toString();
        this.batname = parms[1].toString();
        this.batdesp = parms[2].toString();
        this.syscode = parms[3].toString();
        this.mainseqn = parms[4].toString();
        this.stat = parms[5].toString();
        this.tx_date = parms[6].toString();
        this.beg_date = parms[7].toString();
        this.end_date = parms[8].toString();
        try {
            this.elapsetime = TimeSpan.parse(parms[9].toString());
        } catch (error) {
            this.elapsetime = TimeSpan.MinValue;
        }
        this.opteller = parms[10].toString();
        this.chkteller = parms[11].toString();
        this.subcnt = parms[12].toString();
        this.bat_ind = parms[13].toString();
        this.filler1 = parms[14].toString();
        this.filler2 = parms[15].toString();
        this.refreshtime = parms[16].toString();
        this.batcheck = parms[17].toString();
    }

    // 将对象属性转换为字符串数组
    ChangeToTable(): string[] {
        return [
            this.batcode,
            this.batname,
            this.batdesp,
            this.syscode,
            this.mainseqn,
            this.stat,
            this.tx_date,
            this.beg_date,
            this.end_date,
            this.elapsetime.toString('hh:mm:ss'),
            this.opteller,
            this.chkteller,
            this.subcnt,
            this.bat_ind,
            this.filler1,
            this.filler2,
            this.refreshtime,
            this.batcheck
        ];
    }
}

// 自定义 TimeSpan 类，用于处理时间间隔
class TimeSpan {
    static MinValue: TimeSpan = new TimeSpan(0, 0, 0);

    private hours: number;
    private minutes: number;
    private seconds: number;

    constructor(hours: number, minutes: number, seconds: number) {
        this.hours = hours;
        this.minutes = minutes;
        this.seconds = seconds;
    }

    static parse(str: string): TimeSpan {
        const parts = str.split(':');
        const hours = parseInt(parts[0], 10);
        const minutes = parseInt(parts[1], 10);
        const seconds = parseInt(parts[2], 10);
        return new TimeSpan(hours, minutes, seconds);
    }

    toString(format: string): string {
        return `${this.hours.toString().padStart(2, '0')}:${this.minutes.toString().padStart(2, '0')}:${this.seconds.toString().padStart(2, '0')}`;
    }
}

export { BatSysInfo };