// 假设 Platform.ServiceProxy.CommonFunction 对应的工具函数在下面的模块中实现
import { ChangeDicToStr, ChangeStrToDic } from './CommonFunction';

// 定义 IEntity 接口，由于原 C# 中未给出，这里简单定义
interface IEntity {}

/**
 * 核查记录类，包含核查相关的各种信息，以及表格数据转换方法
 */
export class PicpCheckRecords implements IEntity {
    /**
     * 核查流水号
     */
    seqNo: string;

    /**
     * 流水号
     */
    serialNo: string;

    /**
     * 报文类型
     */
    msgNo: string;

    /**
     * 交易号
     */
    tradeNo: string;

    /**
     * 核查柜员
     */
    checkTel: string;

    /**
     * 核查机构
     */
    checkBrNo: string;

    /**
     * 核查方式
     */
    checkType: string;

    /**
     * 核查日期
     */
    checkDate: string;

    /**
     * 核查时间
     */
    checkTime: string;

    /**
     * 核查信息:身份证号,姓名
     */
    checkInfoKVs: { [key: string]: string };

    /**
     * 核查是否成功
     */
    success: string;

    /**
     * 备注
     */
    comments: string;

    constructor() {
        this.seqNo = '';
        this.serialNo = '';
        this.msgNo = '';
        this.tradeNo = '';
        this.checkTel = '';
        this.checkBrNo = '';
        this.checkType = '';
        this.checkDate = '';
        this.checkTime = '';
        this.checkInfoKVs = {};
        this.success = '';
        this.comments = '';
    }

    /**
     * 将当前对象的属性转换为字符串数组，用于表格展示或存储
     * @returns 包含所有属性值的字符串数组
     */
    ChangeToTable(): string[] {
        return [
            this.seqNo,
            this.serialNo,
            this.msgNo,
            this.tradeNo,
            this.checkTel,
            this.checkBrNo,
            this.checkType,
            this.checkDate,
            this.checkTime,
            ChangeDicToStr(this.checkInfoKVs),
            this.success,
            this.comments
        ];
    }

    /**
     * 从对象数组中读取数据并赋值给当前对象的属性
     * @param parms 包含数据的对象数组
     */
    ChangeFromTable(parms: any[]): void {
        this.seqNo = parms[0].toString();
        this.serialNo = parms[1].toString();
        this.msgNo = parms[2].toString();
        this.tradeNo = parms[3].toString();
        this.checkTel = parms[4].toString();
        this.checkBrNo = parms[5].toString();
        this.checkType = parms[6].toString();
        this.checkDate = parms[7].toString();
        this.checkTime = parms[8].toString();
        this.checkInfoKVs = ChangeStrToDic(parms[9].toString());
        this.success = parms[10].toString();
        this.comments = parms[11].toString();
    }
}