// 假设 IEntity 是一个接口，这里先简单定义，你可按需修改
interface IEntity {}

/**
 * 该类表示 PICP 核查结果信息，包含返回报文类型、身份证号、姓名等核查相关信息，
 * 并提供了将信息转换为数组和从数组加载信息的方法。
 */
class PicpCheckResInfo implements IEntity {
    /**
     * 返回报文类型
     */
    resMsgNo: string;

    /**
     * 身份证号
     */
    idNo: string;

    /**
     * 姓名
     */
    name: string;

    /**
     * 核查结果
     */
    checkResult: string;

    /**
     * 签发机构
     */
    issue: string;

    /**
     * 曾用名
     */
    formerName: string;

    /**
     * 性别
     */
    sex: string;

    /**
     * 生日
     */
    birthday: string;

    /**
     * 出生地
     */
    birthPlace: string;

    /**
     * 籍贯
     */
    nativePlace: string;

    /**
     * 文化程度
     */
    eduLevel: string;

    /**
     * 婚姻状况
     */
    marriage: string;

    /**
     * 职业
     */
    job: string;

    /**
     * 服务处所
     */
    engageAddr: string;

    /**
     * 住址
     */
    address: string;

    /**
     * 照片 64bit 压缩编码
     */
    photoData: number[];

    /**
     * 备注
     */
    comments: string;

    /**
     * 照片数据长度
     */
    photoDataLength: number;

    constructor() {
        this.resMsgNo = "";
        this.idNo = "";
        this.name = "";
        this.checkResult = "";
        this.issue = "";
        this.formerName = "";
        this.sex = "";
        this.birthday = "";
        this.birthPlace = "";
        this.nativePlace = "";
        this.eduLevel = "";
        this.marriage = "";
        this.job = "";
        this.engageAddr = "";
        this.address = "";
        this.photoData = [];
        this.comments = "";
        this.photoDataLength = 0;
    }

    /**
     * 将对象的属性转换为字符串数组
     * @returns 包含对象属性值的字符串数组
     */
    ChangeToTable(): string[] {
        return [
            this.resMsgNo,
            this.idNo,
            this.name,
            this.checkResult,
            this.issue,
            this.formerName,
            this.sex,
            this.birthday,
            this.birthPlace,
            this.nativePlace,
            this.eduLevel,
            this.marriage,
            this.job,
            this.engageAddr,
            this.address,
            this.comments,
            this.photoData.length.toString()
        ];
    }

    /**
     * 从对象数组中加载数据到当前对象的属性中
     * @param parms 包含数据的对象数组
     */
    ChangeFromTable(parms: any[]): void {
        this.resMsgNo = parms[0].toString();
        this.idNo = parms[1].toString();
        this.name = parms[2].toString();
        this.checkResult = parms[3].toString();
        this.issue = parms[4].toString();
        this.formerName = parms[5].toString();
        this.sex = parms[6].toString();
        this.birthday = parms[7].toString();
        this.birthPlace = parms[8].toString();
        this.nativePlace = parms[9].toString();
        this.eduLevel = parms[10].toString();
        this.marriage = parms[11].toString();
        this.job = parms[12].toString();
        this.engageAddr = parms[13].toString();
        this.address = parms[14].toString();
        this.comments = parms[15].toString();
        this.photoDataLength = parseInt(parms[16].toString(), 10);
    }
}

export { PicpCheckResInfo };