// 假设 Platform.ServiceProxy.CommonFunction 相关方法会在别处实现
// 这里先声明函数类型
declare namespace Platform {
  namespace ServiceProxy {
    namespace CommonFunction {
      function ChangeDicToStr(dic: { [key: string]: string }): string;
      function ChangeStrToDic(str: string): { [key: string]: string };
    }
  }
}

// 定义 IEntity 接口，由于原文件未给出，这里先空着
interface IEntity {}

/**
 * 工作项描述类，实现了 IEntity 接口，用于存储工作项的相关信息。
 */
class WorkItemDesc implements IEntity {
  /**
   * 流水号
   */
  SerialNo: string;
  /**
   * 交易号
   */
  TradeNo: string;
  /**
   * 发起柜员
   */
  BegTeller: string;
  /**
   * 流程创建时间
   */
  BegTime: string;
  /**
   * 完成时间
   */
  EndTime: string;
  /**
   * 任务描述
   */
  TaskDes: string;
  /**
   * 工作项 Id
   */
  WorkItemId: string;
  /**
   * 流程 Id
   */
  ProcessId: string;
  /**
   * 数据总线数据
   */
  ProcessVariables: { [key: string]: string };
  /**
   * 任务变量数据
   */
  ItemVariables: { [key: string]: string };
  /**
   * 节点名称
   */
  NodeName: string;
  /**
   * 任务所有人
   */
  Holder: boolean;
  /**
   * 任务发起柜员机构号
   * @author YuanXiaoLong
   */
  begTelKinbrNo: string;
  /**
   * 任务发起柜员机构名称
   * @author YuanXiaoLong
   */
  begTelKinbrName: string;
  /**
   * 任务发起柜员名称
   * @author YuanXiaoLong
   */
  begTelName: string;

  /**
   * 构造函数，初始化工作项描述对象的属性。
   * @param SerialNo - 流水号
   * @param TradeNo - 交易号
   * @param BegTeller - 发起柜员
   * @param BegTime - 流程创建时间
   * @param EndTime - 完成时间
   * @param TaskDes - 任务描述
   * @param WorkItemId - 工作项 Id
   * @param ProcessId - 流程 Id
   * @param ProcessVariables - 数据总线数据
   * @param ItemVariables - 任务变量数据
   * @param NodeName - 节点名称
   * @param Holder - 任务所有人
   * @param begTelKinbrNo - 任务发起柜员机构号
   * @param begTelKinbrName - 任务发起柜员机构名称
   * @param begTelName - 任务发起柜员名称
   */
  constructor(
    SerialNo: string = '',
    TradeNo: string = '',
    BegTeller: string = '',
    BegTime: string = '',
    EndTime: string = '',
    TaskDes: string = '',
    WorkItemId: string = '',
    ProcessId: string = '',
    ProcessVariables: { [key: string]: string } = {},
    ItemVariables: { [key: string]: string } = {},
    NodeName: string = '',
    Holder: boolean = false,
    begTelKinbrNo: string = '',
    begTelKinbrName: string = '',
    begTelName: string = ''
  ) {
    this.SerialNo = SerialNo;
    this.TradeNo = TradeNo;
    this.BegTeller = BegTeller;
    this.BegTime = BegTime;
    this.EndTime = EndTime;
    this.TaskDes = TaskDes;
    this.WorkItemId = WorkItemId;
    this.ProcessId = ProcessId;
    this.ProcessVariables = ProcessVariables;
    this.ItemVariables = ItemVariables;
    this.NodeName = NodeName;
    this.Holder = Holder;
    this.begTelKinbrNo = begTelKinbrNo;
    this.begTelKinbrName = begTelKinbrName;
    this.begTelName = begTelName;
  }

  /**
   * 将工作项描述对象的属性转换为字符串数组。
   * @returns 包含工作项描述信息的字符串数组。
   */
  ChangeToTable(): string[] {
    return [
      this.SerialNo,
      this.TradeNo,
      this.BegTeller,
      this.BegTime,
      this.EndTime,
      this.TaskDes,
      this.WorkItemId,
      this.ProcessId,
      Platform.ServiceProxy.CommonFunction.ChangeDicToStr(this.ProcessVariables),
      Platform.ServiceProxy.CommonFunction.ChangeDicToStr(this.ItemVariables),
      this.NodeName,
      this.Holder.toString(),
      this.begTelKinbrNo,
      this.begTelKinbrName,
      this.begTelName
    ];
  }

  /**
   * 从对象数组中读取数据并更新工作项描述对象的属性。
   * @param parms - 包含工作项描述信息的对象数组。
   */
  ChangeFromTable(parms: any[]): void {
    this.SerialNo = parms[0].toString();
    this.TradeNo = parms[1].toString();
    this.BegTeller = parms[2].toString();
    this.BegTime = parms[3].toString();
    this.EndTime = parms[4].toString();
    this.TaskDes = parms[5].toString();
    this.WorkItemId = parms[6].toString();
    this.ProcessId = parms[7].toString();
    this.ProcessVariables = Platform.ServiceProxy.CommonFunction.ChangeStrToDic(parms[8].toString());
    this.ItemVariables = Platform.ServiceProxy.CommonFunction.ChangeStrToDic(parms[9].toString());
    this.NodeName = parms[10].toString();
    this.Holder = Boolean.parse(parms[11].toString());
    this.begTelKinbrNo = parms[12].toString();
    this.begTelKinbrName = parms[13].toString();
    this.begTelName = parms[14].toString();
  }
}

export default WorkItemDesc;