// 模拟一些类型，实际使用时需要根据具体情况定义
type TransitEntity = {
    ServiceName: string;
    FuncName: string;
    Parameters?: { [key: string]: string };
    AttachFiles?: AttachFile[];
};

type AttachFile = {
    AttachIndex: string;
    AttachType: any; // 这里不清楚 Types.TableType 的具体类型，先用 any 代替
    AttachValue: any;
};

type AfterShip = any;
type AfterTask = any;
type AfterRecords = any;
type IdCheckRecords = any;
type AfterRecordInfo = any;

// 模拟 ServiceManager 类
class ServiceManager {
    private static instance: ServiceManager;

    static GetInstance() {
        if (!this.instance) {
            this.instance = new ServiceManager();
        }
        return this.instance;
    }

    Commit(entity: TransitEntity): AttachFile[] {
        // 这里只是模拟，实际需要实现具体的提交逻辑
        return [];
    }
}

// 模拟 CommonFunction 类
class CommonFunction {
    static ChangeEntityToTable(tr: any[]) {
        // 这里只是模拟，实际需要实现具体的转换逻辑
        return {};
    }

    static ChangStrToArray(str: string): string[] {
        // 这里只是模拟，实际需要实现具体的转换逻辑
        return [];
    }
}

/**
 * 事后监督发送服务端方法
 */
class AfterSuperviseHandle {
    /**
     * 查询按人员分配监督信息 
     */
    static AfterShipSelect(): any {
        const entity: TransitEntity = {
            ServiceName: "aftersupervise",
            FuncName: "SuperVisorDistributeSelect"
            // Parameters: new Dictionary<string, string> {}
        };
        // 文件byte[]数据组织 begin
        // 特殊结构转换
        // 文件byte[]数据组织 end
        const tmpresult = ServiceManager.GetInstance().Commit(entity);
        const dt = tmpresult[0]?.AttachValue;
        return dt;
    }

    /**
     * 通过机构号和柜员号查询相关录入信息
     */
    static GetAfterShipInfoBynoAndCode(): any {
        const entity: TransitEntity = {
            ServiceName: "aftersupervise",
            FuncName: "SuperVisorDistributeSelect",
        };
        // 文件byte[]数据组织 begin
        // 特殊结构转换
        // 文件byte[]数据组织 end
        const tmpresult = ServiceManager.GetInstance().Commit(entity);
        const dt = tmpresult[0]?.AttachValue;
        if (dt && dt.Columns) {
            dt.Columns[0].ColumnName = "机构号";
            dt.Columns[1].ColumnName = "机构名称";
            dt.Columns[2].ColumnName = "柜员号";
            dt.Columns[3].ColumnName = "柜员名称";
        }
        return dt;
    }

    /**
     * 通过机构号获取机构名称
     * @param brno 机构号
     */
    static GetBrnameByNo(brno: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "getBrnameByno",
                Parameters: { "1": brno }
                // Parameters: new Dictionary<string, string> {}
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 通过用户代码获取用户名
     * @param userCode 用户代码
     */
    static GetUserNameByCode(userCode: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "getUserName",
                Parameters: { "1": userCode }
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 更新按人员分配监督信息
     * @param tr 事后监督信息数组
     */
    static AfterShipUpdate(tr: AfterShip[]): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "SuperVisorDistributeUpdate"
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: null, // 这里不清楚 Types.TableType 的具体类型，先用 null 代替
                AttachValue: CommonFunction.ChangeEntityToTable(tr)
            };
            if (!entity.AttachFiles) {
                entity.AttachFiles = [];
            }
            entity.AttachFiles.push(attcheds);
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 查询按任务分配监督信息 
     */
    static AfterTaskSelect(): any {
        const entity: TransitEntity = {
            ServiceName: "aftersupervise",
            FuncName: "TaskDistributeSelect",
            // Parameters: new Dictionary<string, string> {}
        };
        // 文件byte[]数据组织 begin
        // 特殊结构转换
        // 文件byte[]数据组织 end
        const tmpresult = ServiceManager.GetInstance().Commit(entity);
        const dt = tmpresult[0]?.AttachValue;
        return dt;
    }

    /**
     * 更新按任务分配监督信息
     * @param tr 事后任务信息数组
     */
    static AfterTaskUpdate(tr: AfterTask[]): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "TaskDistributeUpdate"
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: null, // 这里不清楚 Types.TableType 的具体类型，先用 null 代替
                AttachValue: CommonFunction.ChangeEntityToTable(tr)
            };
            if (!entity.AttachFiles) {
                entity.AttachFiles = [];
            }
            entity.AttachFiles.push(attcheds);
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 判断是否可以开启日前交易
     */
    static JudgeIfStart(): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "JudgeIfDayStart"
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 初始化日前判断
     */
    static DayStartInit(): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "DayStartInit"
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 判断服务器上是否有对应的开启日前的txt数据文件，返回bool
     * @param time 时间
     */
    static ExistTxt(time: string): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "IfExistTxt",
                Parameters: { "1": time }
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 判断日终表中是否有记录以判断可以开启日前的日期
     */
    static HasRecord(): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "JudgeIfDayStart"
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 查询可开启日前的日期
     */
    static GetDayStartDate(): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "SelectDayStartDay"
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 开启日前
     * @param startdate 开始日期
     */
    static insertDayStartInfo(startdate: string): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "DayStart",
                Parameters: { "1": startdate }
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 查询日终状态表，返回机构号，机构名称，监督状态等相关信息
     * @param datetime 日期时间
     */
    static getDayEndInfo(datetime: string): any {
        const entity: TransitEntity = {
            ServiceName: "aftersupervise",
            FuncName: "GetAfterSuperviseInfo",
            Parameters: { "1": datetime }
        };
        // 文件byte[]数据组织 begin
        // 特殊结构转换
        // 文件byte[]数据组织 end
        const tmpresult = ServiceManager.GetInstance().Commit(entity);
        const dt = tmpresult[0]?.AttachValue;
        return dt;
    }

    /**
     * 查询是否都完成监督
     * @param datetime 日期时间
     */
    static JudgeIfDayEnd(datetime: string): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "JudgeIfDayEnd",
                Parameters: { "1": datetime }
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 提交日终记录
     * @param datetime 日期时间
     */
    static SubmitDayEndInfo(datetime: string): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "UpdateDayEndInfo",
                Parameters: { "1": datetime }
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 获取工作日期
     */
    static GetWorkDate(): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "GetDayEndWorkDate"
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 柜员提交日终信息
     * @param userno 柜员号
     */
    static SubmitDayEndInfoByUser(userno: string): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "UpdateDayEndByUser",
                Parameters: { "1": userno }
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 通过柜员号查询当前可以监督的机构
     * @param teller 柜员号
     */
    static getBrnoByTeller(teller: string): string[] | null {
        let result: string[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "SelectBrnoByTeller",
                Parameters: { "1": teller }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangStrToArray(tmpresult[0].AttachValue.toString());
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 事后监督查询回显
     * @param date 日期
     * @param tellerno 柜员号
     * @param printno 票号
     */
    static getAfterCoreRecord(date: string, tellerno: string, printno: string): string[] | null {
        let result: string[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "SelectAfterCoreRecords",
                Parameters: { "1": date, "2": tellerno, "3": printno }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangStrToArray(tmpresult[0].AttachValue.toString());
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 判断是否有此票号,且无重复
     * @param printno 票号
     * @param date 日期
     * @param teller 柜员号
     */
    static GetIsExistPirntNo(printno: string, date: string, teller: string): string {
        let tradeReturn = "";
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "JudgeIsExistPirntno",
                Parameters: { "1": printno, "2": date, "3": teller }
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 提交操作，将事后监督信息插入表中
     * @param ar 事后记录信息
     */
    static InsertAfterRecords(ar: AfterRecords): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "InsertAfterRecords"
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: null, // 这里不清楚 Types.TableType 的具体类型，先用 null 代替
                AttachValue: CommonFunction.ChangeEntityToTable([ar])
            };
            if (!entity.AttachFiles) {
                entity.AttachFiles = [];
            }
            entity.AttachFiles.push(attcheds);
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 根据营业日期，柜员号，票号查询第三方流水号
     * @param date 营业日期
     * @param tellerno 柜员号
     * @param printno 票号
     */
    static GetThirdSerialNo(date: string, tellerno: string, printno: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "GetThirdSerial",
                Parameters: { "1": date, "2": tellerno, "3": printno }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 查询
     * @param idno 身份证号
     * @param type 类型
     * @param checkresult 检查结果
     * @param marking 标记
     * @param branch 机构
     * @param teller 柜员
     * @param startdate 开始日期
     * @param enddate 结束日期
     */
    static GetIdCheckInfoFromService(
        idno: string,
        type: string = "",
        checkresult: string = "",
        marking: string = "",
        branch: string = "",
        teller: string = "",
        startdate: string = "20120202",
        enddate: string = ""
    ): any {
        let tradeReturn = {};
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "GetIdCheckInfo ",
                Parameters: {
                    "1": idno,
                    "2": type,
                    "3": checkresult,
                    "4": marking,
                    "5": branch,
                    "6": teller,
                    "7": startdate,
                    "8": enddate
                }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue;
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 插入
     * @param idr 身份检查记录信息
     */
    static InsertIntoIdCheckInfo(idr: IdCheckRecords): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "InserIdCheckInfo "
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: null, // 这里不清楚 Types.TableType 的具体类型，先用 null 代替
                AttachValue: CommonFunction.ChangeEntityToTable([idr])
            };
            if (!entity.AttachFiles) {
                entity.AttachFiles = [];
            }
            entity.AttachFiles.push(attcheds);
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 获取工作类型
     */
    static GetWorkType(): string[] | null {
        let result: string[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "GetWorkType"
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangStrToArray(tmpresult[0].AttachValue.toString());
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 判断是否已监督
     * @param printno 票号
     * @param date 日期
     * @param teller 柜员号
     */
    static IfSupervise(printno: string, date: string, teller: string): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "IfSupervise",
                Parameters: { "1": printno, "2": date, "3": teller }
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 获取事后记录信息
     * @param ar 事后记录信息对象
     * @param longinuser 登录用户
     */
    static GetAfterRecordInfo(ar: AfterRecordInfo, longinuser: string): any {
        let dt = null;
        const entity: TransitEntity = {
            ServiceName: "aftersupervise",
            FuncName: "GetAfterRecordBySolution",
            Parameters: { "2": longinuser }
        };
        const attcheds: AttachFile = {
            AttachIndex: "1",
            AttachType: null, // 这里不清楚 Types.TableType 的具体类型，先用 null 代替
            AttachValue: CommonFunction.ChangeEntityToTable([ar])
        };
        if (!entity.AttachFiles) {
            entity.AttachFiles = [];
        }
        entity.AttachFiles.push(attcheds);
        // 文件byte[]数据组织 begin
        // 特殊结构转换
        // 文件byte[]数据组织 end
        const tmpresult = ServiceManager.GetInstance().Commit(entity);
        if (tmpresult && tmpresult.length === 1) {
            dt = tmpresult[0].AttachValue;
        }
        return dt;
    }

    /**
     * 通过柜员号和日期查询监督信息
     * @param teller 柜员号
     * @param datetime 日期时间
     */
    static GetSuperviseInfoByTeller(teller: string, datetime: string): any {
        let dt = null;
        const entity: TransitEntity = {
            ServiceName: "aftersupervise",
            FuncName: "GetSuperViseInfoByCurrentTeller",
            Parameters: { "1": teller, "2": datetime }
        };
        // 文件byte[]数据组织 begin
        // 特殊结构转换
        // 文件byte[]数据组织 end
        const tmpresult = ServiceManager.GetInstance().Commit(entity);
        if (tmpresult && tmpresult.length === 1) {
            dt = tmpresult[0].AttachValue;
        }
        return dt;
    }

    /**
     * 获取监督信息
     * @param money 金额
     * @param tradeno 交易号
     */
    static GetSuperInfo(money: string, tradeno: string): string | null {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "GetSuperInfo",
                Parameters: { "1": money, "2": tradeno }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 获取监督比例
     * @param date 日期
     * @param tradeno 交易号
     * @param kinbrno 机构号
     */
    static GetSuperRatio(date: string, tradeno: string, kinbrno: string): string[] | null {
        let tradeReturn: string[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "GetSuperRatio",
                Parameters: { "1": date, "2": tradeno, "3": kinbrno }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = CommonFunction.ChangStrToArray(tmpresult[0].AttachValue.toString());
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 柜员查询可监督机构以及相关业务比例
     * @param teller 柜员号
     */
    static TellerGetSuperviseInfo(teller: string): any {
        let dt = null;
        const entity: TransitEntity = {
            ServiceName: "aftersupervise",
            FuncName: "GetSuperInfoByTeller",
            Parameters: { "1": teller }
        };
        // 文件byte[]数据组织 begin
        // 特殊结构转换
        // 文件byte[]数据组织 end
        const tmpresult = ServiceManager.GetInstance().Commit(entity);
        if (tmpresult && tmpresult.length === 1) {
            dt = tmpresult[0].AttachValue;
        }
        return dt;
    }

    /**
     * 查询监督机构下对应业务的监督情况
     */
    static GetSuperInfoForBranch(): any {
        let dt = null;
        const entity: TransitEntity = {
            ServiceName: "aftersupervise",
            FuncName: "TaskDistributeSelect"
        };
        // 文件byte[]数据组织 begin
        // 特殊结构转换
        // 文件byte[]数据组织 end
        const tmpresult = ServiceManager.GetInstance().Commit(entity);
        if (tmpresult && tmpresult.length === 1) {
            dt = tmpresult[0].AttachValue;
        }
        return dt;
    }

    /**
     * 事后监督查询
     * @param date 日期
     * @param branch 机构
     * @param teller 柜员
     * @param stat 状态
     */
    static GetSuperRecords(date: string, branch: string, teller: string, stat: string): any {
        let dt = null;
        const entity: TransitEntity = {
            ServiceName: "aftersupervise",
            FuncName: "SelectAfterSuperviseInfo",
            Parameters: { "1": date, "2": branch, "3": teller, "4": stat }
        };
        const tmpresult = ServiceManager.GetInstance().Commit(entity);
        if (tmpresult && tmpresult.length === 1) {
            dt = tmpresult[0].AttachValue;
        }
        return dt;
    }

    /**
     * 统计凭证信息
     * @param date 日期
     */
    static GetNoteInfo(date: string): any {
        let dt = null;
        const entity: TransitEntity = {
            ServiceName: "aftersupervise",
            FuncName: "GetNoteInfo",
            Parameters: { "1": date }
        };
        const tmpresult = ServiceManager.GetInstance().Commit(entity);
        if (tmpresult && tmpresult.length === 1) {
            dt = tmpresult[0].AttachValue;
        }
        return dt;
    }

    /**
     * 查询柜员指定日期最大票号
     * @param date 日期
     */
    static GetMaxPrintnum(date: string): any {
        let dt = null;
        const entity: TransitEntity = {
            ServiceName: "aftersupervise",
            FuncName: "GetMaxPrintnum",
            Parameters: { "1": date }
        };
        const tmpresult = ServiceManager.GetInstance().Commit(entity);
        if (tmpresult && tmpresult.length === 1) {
            dt = tmpresult[0].AttachValue;
        }
        return dt;
    }

    /**
     * 获取日终状态
     */
    static GetDanEndStat(): boolean {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "aftersupervise",
                FuncName: "getDayEndStat"
            };
            // 文件byte[]数据组织 begin
            // 特殊结构转换
            // 文件byte[]数据组织 end
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }
}

export { AfterSuperviseHandle };