// 模拟部分类型和函数，实际需要根据项目情况补充
type TransitEntity = {
    ServiceName: string;
    FuncName: string;
    Parameters?: { [key: string]: string };
    AttachFiles: AttachFile[];
};

type AttachFile = {
    AttachIndex: string;
    AttachType: Types;
    AttachValue: any;
};

enum Types {
    TableType,
    ByteType
}

class ServiceManager {
    private static instance: ServiceManager;
    private constructor() { }

    static getInstance() {
        if (!this.instance) {
            this.instance = new ServiceManager();
        }
        return this.instance;
    }

    Commit(entity: TransitEntity): AttachFile[] {
        // 这里只是模拟，实际需要实现具体逻辑
        return [];
    }
}

class CommonFunction {
    static ChangeEntityToTable(pic: any[]): any {
        // 这里只是模拟，实际需要实现具体逻辑
        return {};
    }

    static ChangeTableToEntitys<T>(dataTable: any): T[] {
        // 这里只是模拟，实际需要实现具体逻辑
        return [] as T[];
    }
}

type PicpCheckRecords = any;
type PicpCheckResInfo = {
    photoData?: Uint8Array;
};

export class PicpHandle {
    public static PicpCommonRequest(msgStr: string): Uint8Array | null {
        let tradeReturn: Uint8Array | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "PicpService",
                FuncName: "PicpCommonRequest",
                Parameters: { "1": msgStr },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue as Uint8Array;
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    public static PicpCheckRequest(picpCheckRecords: PicpCheckRecords): PicpCheckResInfo | null {
        let tradeReturn: PicpCheckResInfo | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "PicpService",
                FuncName: "PicpCheckRequest",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable([picpCheckRecords])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 2) {
                tradeReturn = CommonFunction.ChangeTableToEntitys<PicpCheckResInfo>(tmpresult[0].AttachValue)[0];
                if (tradeReturn) {
                    tradeReturn.photoData = tmpresult[1].AttachValue as Uint8Array;
                }
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    public static GenCheckReqMessage(picpCheckRecords: PicpCheckRecords): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "PicpService",
                FuncName: "GenCheckReqMessage",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable([picpCheckRecords])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    public static AnalyzeCheckResMessage(recvData: Uint8Array): PicpCheckResInfo | null {
        let tradeReturn: PicpCheckResInfo | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "PicpService",
                FuncName: "AnalyzeCheckResMessage",
                AttachFiles: []
            };
            const attcheds2: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.ByteType,
                AttachValue: recvData
            };
            entity.AttachFiles.push(attcheds2);
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = CommonFunction.ChangeTableToEntitys<PicpCheckResInfo>(tmpresult[0].AttachValue)[0];
                if (tradeReturn) {
                    // 原代码此处索引 1 可能有误，因为 tmpresult 长度为 1
                    // tradeReturn.photoData = tmpresult[1].AttachValue as Uint8Array; 
                }
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    public static HnSaveCheckdata(picpCheckRecords: PicpCheckRecords, checkResInfo: PicpCheckResInfo): number {
        let result = 0;
        try {
            const entity: TransitEntity = {
                ServiceName: "PicpService",
                FuncName: "HnSaveCheckdata",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable([picpCheckRecords])
            };
            const attcheds2: AttachFile = {
                AttachIndex: "2",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable([checkResInfo])
            };
            const attcheds3: AttachFile = {
                AttachIndex: "3",
                AttachType: Types.ByteType,
                AttachValue: checkResInfo.photoData
            };
            entity.AttachFiles.push(attcheds3);
            entity.AttachFiles.push(attcheds);
            entity.AttachFiles.push(attcheds2);
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Number(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }
}