// 模拟引入必要的类型
type TransitEntity = {
    ServiceName: string;
    FuncName: string;
    Parameters?: Record<string, string>;
    AttachFiles?: AttachFile[];
};

type AttachFile = {
    AttachIndex: string;
    AttachType: any;
    AttachValue: any;
};

type TT_KinbrInfo = any;
type TBL_Sys_TerminalNumber = any;
type TBL_Sys_NotifyMessage = any;
type TBL_Sys_SendGroupRole = any;
type TT_GroupRule = any;
type TT_LoginUser = any;
type TT_LoginSign = any;
type TBL_Sys_ReceiveName = any;
type TT_MenuInfo = any;

// 模拟 ServiceManager 类
class ServiceManager {
    private static instance: ServiceManager;

    static getInstance() {
        if (!this.instance) {
            this.instance = new ServiceManager();
        }
        return this.instance;
    }

    async Commit(entity: TransitEntity): Promise<AttachFile[]> {
        // 这里模拟服务调用，实际使用时需要替换为真实的请求逻辑
        console.log('模拟服务调用:', entity);
        return [];
    }
}

// 模拟 CommonFunction 类
class CommonFunction {
    static ChangeTableToEntitys<T>(dataTable: any): T[] {
        // 这里模拟数据转换，实际使用时需要替换为真实的转换逻辑
        console.log('模拟数据转换:', dataTable);
        return [] as T[];
    }

    static ChangeEntityToTable(entities: any[]): any {
        // 这里模拟数据转换，实际使用时需要替换为真实的转换逻辑
        console.log('模拟数据转换:', entities);
        return {};
    }

    static ChangArrayToStr(arr: string[]): string {
        return arr.join(',');
    }

    static ChangStrToArray(str: string): string[] {
        return str.split(',');
    }
}

// 模拟 Types 常量
const Types = {
    TableType: 'TableType',
    ByteType: 'ByteType'
};

/**
 * 平台管理操作类
 */
export class PlatformManagerHandle {
    /**
     * 终端号操作
     */
    /**
     * 查询机构信息
     * @param code 编码
     * @returns Id、Code、Name
     */
    static async KinbrInfo_GetList_IDCodeName(code: string): Promise<TT_KinbrInfo[]> {
        let result: TT_KinbrInfo[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "KinbrInfo_GetList_IDCodeName",
                Parameters: { "1": code }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TT_KinbrInfo>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result || [];
    }

    /**
     * 查询终端号列表
     * @param kinbrno 
     * @returns 
     */
    static async TerminalNumber_GetList(kinbrno: string): Promise<TBL_Sys_TerminalNumber[]> {
        let result: TBL_Sys_TerminalNumber[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "TerminalNumber_GetList",
                Parameters: { "1": kinbrno }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TBL_Sys_TerminalNumber>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result || [];
    }

    /**
     * 
     * @param disk 
     * @returns 
     */
    static async TerminalNumber_GetListByDisk(disk: string): Promise<TBL_Sys_TerminalNumber[]> {
        let result: TBL_Sys_TerminalNumber[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "TerminalNumberList_SelectByDisk",
                Parameters: { "1": disk }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TBL_Sys_TerminalNumber>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result || [];
    }

    /**
     * 添加终端号
     * @param terminalNumberList 终端号
     * @returns "0000"-成功|"9999"-失败
     */
    static async TerminalNumber_Add(terminalNumberList: TBL_Sys_TerminalNumber[]): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "TerminalNumber_Add",
                AttachFiles: []
            };
            const file: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(terminalNumberList)
            };
            entity.AttachFiles.push(file);
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 终端号和激活码的匹配
     * @param tellerNumber 柜员号
     * @param activationcode 激活码
     * @param registerTime 注册时间
     * @param remakInfo 备份信息
     * @returns 
     */
    static async TerminalNumberList_Add(tellerNumber: string, activationcode: string, registerTime: string, remakInfo: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "TerminalNumberList_Add",
                Parameters: { "1": tellerNumber, "2": activationcode, "3": registerTime, "4": remakInfo }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 解除终端号和设备号匹配
     * @param code where条件
     * @returns "0000"-成功|"9999"-失败
     */
    static async UnTerminalNumber_Matching(code: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "UnTerminalNumber_Matching",
                Parameters: { "1": code }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 删除终端号
     * @param Id where条件
     * @returns "0000"-成功|"9999"-失败
     */
    static async TerminalNumber_DeleteByID(Id: string[]): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "TerminalNumber_DeleteByID",
                Parameters: { "1": CommonFunction.ChangArrayToStr(Id) }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     *  更新终端号匹配
     * @param valueData 终端号
     * @param whereData where激活码
     * @returns "0000"-成功|"9999"-失败
     */
    static async TerminalNumber_Matching(valueData: string, whereData: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "TerminalNumberUpdate",
                Parameters: { "1": valueData, "2": whereData }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 获取终端号最大的值
     * @param kinbrNo 机构号
     * @returns 终端号
     */
    static async TerminalNumber_GetMaxNumber(kinbrNo: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "TerminalNumber_GetMaxNumber",
                Parameters: { "1": kinbrNo }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 查询终端号注册状态
     * @param activationCode 激活码
     * @returns 0-没有注册，1-已经注册
     */
    static async TerminalNumber_GetStatus(activationCode: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "TerminalNumber_GetStatus",
                Parameters: { "1": activationCode }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 根据激活码获取机构号
     * @param activationCode 激活码
     * @returns 0-没有机构，1-机构号
     */
    static async TerminalNumber_GetKinbrNO(activationCode: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "TerminalNumber_GetKinbrno",
                Parameters: { "1": activationCode }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 通知下发信息操作
     */
    /**
     * 查询公告信息
     * @param id id
     * @param flag 0-普通查询，1-查询值包括附件
     * @param userCode 
     * @param dt 
     * @returns TBL_Sys__NotifyMessage集合列表
     */
    static async NotifyMessage_GetInfobyId(id: string, flag: string, userCode: string): Promise<[TBL_Sys_NotifyMessage[], any | null]> {
        let result: TBL_Sys_NotifyMessage[] | null = null;
        let dt: any | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "NotifyMessage_GetInfoByID",
                Parameters: { "1": id, "2": flag, "3": userCode }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                dt = tmpresult[0].AttachValue;
                result = CommonFunction.ChangeTableToEntitys<TBL_Sys_NotifyMessage>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return [result || [], dt];
    }

    /**
     * 查看公告信息对应机构和岗位
     * @param type 0-机构，1-岗位
     * @param userid 公告信息id
     * @returns 机构|岗位 列表
     */
    static async NotifyMessage_GetGroupRolebyInfoId(type: string, userid: string): Promise<TBL_Sys_SendGroupRole[]> {
        let result: TBL_Sys_SendGroupRole[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "NotifyMessage_GetGroupRolebyInfoId",
                Parameters: { "1": type, "2": userid }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TBL_Sys_SendGroupRole>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result || [];
    }

    /**
     * 添加公告信息
     * @param notifyMessagelist 信息列表
     * @returns "0000"-成功|"9999"-失败
     */
    static async NotifyMessage_Add(notifyMessagelist: TBL_Sys_NotifyMessage): Promise<string | null> {
        let result: string | null = null;
        const list = [notifyMessagelist];
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "NotifyMessage_Add",
                AttachFiles: []
            };
            const file: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(list)
            };
            entity.AttachFiles.push(file);
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 添加公告信息发送的机构和岗位
     * @param tblsysSendGroupRoleList 
     * @returns 
     */
    static async NotifyMessage_SendGroupRoleAdd(tblsysSendGroupRoleList: TBL_Sys_SendGroupRole[]): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "NotifyMessage_SendGroupRoleAdd",
                AttachFiles: []
            };
            const file: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(tblsysSendGroupRoleList)
            };
            entity.AttachFiles.push(file);
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     *  修改公告信息
     * @param notifyMessagelist 信息列表
     * @returns "0000"-成功|"9999"-失败
     */
    static async NotifyMessage_Update(notifyMessagelist: TBL_Sys_NotifyMessage): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "NotifyMessage_Update",
                AttachFiles: []
            };
            const file: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: notifyMessagelist
            };
            entity.AttachFiles.push(file);
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 更新附件
     * @param info_Annex 附件
     * @param info_id Id
     * @returns "0000"-成功|"9999"-失败
     */
    static async NotifyMessage_AttachedFile_Update(info_Annex: string, info_id: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "NotifyMessage_AttachedFile_Del",
                Parameters: { "1": info_Annex, "2": info_id }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 取得本柜员的公告信息
     * @param userCode 柜员号
     * @param isStatus 0-未读，1-已读
     * @returns TBL_Sys__NotifyMessage集合列表
     */
    static async NotifyMessage_GetNewsBoxByID(userCode: string, isStatus: string): Promise<TBL_Sys_NotifyMessage[]> {
        let result: TBL_Sys_NotifyMessage[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "NotifyMessage_GetNewsBoxByID",
                Parameters: { "1": userCode, "2": isStatus }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TBL_Sys_NotifyMessage>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result || [];
    }

    /**
     * 更新公告查看的状态
     * @param userId 信息Id
     * @param infoID 用户Id
     * @returns "0000"-成功|"9999"-失败
     */
    static async NotifyMessage_UpdateStateByinfoID(userId: string, infoID: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "NotifyMessage_UpdateStateByinfoID",
                Parameters: { "1": userId, "2": infoID }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 根据用户编码获取登陆用户Id
     * @param userCode 用户编码
     * @returns 登陆用户Id
     */
    static async Get_TT_LOGINUSER_UserID(userCode: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "GetUserIdByCode",
                Parameters: { "1": userCode }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 删除已发送的信息
     * @param infoId 信息ID
     * @returns "0000"-成功|"9999"-失败
     */
    static async NotifyMessageReceive_DeleteByID(infoId: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "NotifyMessageReceive_DeleteByID",
                Parameters: { "1": infoId }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 删除公告信息和接收人信息
     * @param infoId 信息ID
     * @returns "0000"-成功|"9999"-失败
     */
    static async NotifyMessage_DelByInfoId(infoId: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "NotifyMessage_DelByInfoID",
                Parameters: { "1": infoId }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 查询角色信息
     * @param code 过滤条件
     * @returns 角色Id、code、name
     */
    static async TT_ROLEGROUP_Get_IDCodeName(code: string): Promise<TT_GroupRule[]> {
        let result: TT_GroupRule[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "TT_ROLEGROUP_Get_IDCodeName",
                Parameters: { "1": code }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TT_GroupRule>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result || [];
    }

    /**
     * 机构和角色下对应的柜员Code
     * @param kinbrId 机构号
     * @param roleId 角色号
     * @returns 柜员Code列表
     */
    static async TT_LOGINUSER_Get_Code(kinbrId: string, roleId: string): Promise<string[] | null> {
        let result: string[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "TT_LOGINUSER_Get_Code",
                Parameters: { "1": kinbrId, "2": roleId }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangStrToArray(tmpresult[0].AttachValue.toString());
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 用户登录信息列表
     * @returns 返回用户登陆信息列表
     */
    static async LoginUser_List(): Promise<TT_LoginUser[]> {
        let result: TT_LoginUser[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "LoginUser_List"
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TT_LoginUser>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result || [];
    }

    /**
     * 用户在线状态
     * @returns 返回在线用户Id列表
     */
    static async LoginSign_List(): Promise<TT_LoginSign[]> {
        let result: TT_LoginSign[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "LoginSign_List"
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TT_LoginSign>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result || [];
    }

    /**
     * 添加接收人信息
     * @param receiveName 接收人List集合
     * @returns "0000"-成功|"9999"-失败
     */
    static async ReceiveName_Add(receiveName: TBL_Sys_ReceiveName[]): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "RECEIVENAME_Add",
                AttachFiles: []
            };
            const file: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(receiveName)
            };
            entity.AttachFiles.push(file);
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 帮助文件编辑
     */
    /**
     * 添加帮助文档
     * @param code 交易号
     * @param md5 
     * @returns "0000"-成功|"9999"-失败
     */
    static async AddHelpDocument(code: string, md5: string): Promise<string> {
        let result = "0000";
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "AddHelpDocument",
                Parameters: { "1": code, "2": md5 }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 删除帮助文档
     * @param code 
     * @returns "0000"-成功|"9999"-失败
     */
    static async GetHelpDocumentMd5ByCode(code: string): Promise<string> {
        let result = "0000";
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "GetHelpDocumentMd5ByCode",
                Parameters: { "1": code }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 菜单信息管理
     */
    /**
     * 删除菜单信息
     * @param code 
     * @returns 
     */
    static async TT_MenuConfig_DeleteByID(code: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "TradeFlowService",
                FuncName: "TT_MenuConfig_DeleteByID",
                Parameters: { "1": code }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                // TODO 待修改
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 更新菜单信息
     * @param add_infos 
     * @param Modify_infos 
     * @param Delete_infos 
     * @returns 
     */
    static async MenuInfoConfig_Update(add_infos: TT_MenuInfo[], Modify_infos: TT_MenuInfo[], Delete_infos: TT_MenuInfo[]): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "MenuInfoConfig_Update",
                AttachFiles: []
            };
            const file_add: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(add_infos)
            };
            entity.AttachFiles.push(file_add);

            const file_delete: AttachFile = {
                AttachIndex: "2",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(Delete_infos)
            };
            entity.AttachFiles.push(file_delete);

            const file_update: AttachFile = {
                AttachIndex: "3",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(Modify_infos)
            };
            entity.AttachFiles.push(file_update);

            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 更新菜单信息
     * @returns 
     */
    static async MenuInfoConfig_SyncFile(): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "MenuInfoConfig_SyncFile"
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 更新菜单信息
     * @param add_infos 
     * @returns 
     */
    static async MenuInfoConfig_SyncData(add_infos: TT_MenuInfo[]): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "tradeManagerService",
                FuncName: "MenuInfoConfig_SyncData",
                AttachFiles: []
            };
            const file_add: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(add_infos)
            };
            entity.AttachFiles.push(file_add);

            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     *  保存柜员个人配置文件
     * @param userCode 
     * @param userDefinedInfo 
     * @returns "0000"-成功|"9999"-失败
     */
    static async SaveUserDefinedInfo(userCode: string, userDefinedInfo: Uint8Array): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "PlatFormService",
                FuncName: "SaveUserDefinedInfo",
                Parameters: { "1": userCode },
                AttachFiles: []
            };
            const files: AttachFile = {
                AttachIndex: "2",
                AttachType: Types.ByteType,
                AttachValue: userDefinedInfo
            };
            entity.AttachFiles.push(files);

            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }
}