// 模拟 C# 中的 DataTable 类型，这里简单用 any[][] 表示
type DataTable = any[][];

// 模拟 TransitEntity 类型
interface TransitEntity {
    ServiceName: string;
    FuncName: string;
    Parameters?: { [key: string]: string };
    AttachFiles?: AttachFile[];
}

// 模拟 AttachFile 类型
interface AttachFile {
    AttachIndex: string;
    AttachType: TableType;
    AttachValue: any;
}

// 模拟 Types.TableType
enum TableType {
    // 这里可以根据实际情况补充具体值
}

// 模拟 ServiceManager 类，假设 Commit 方法是异步的
class ServiceManager {
    private static instance: ServiceManager;

    static GetInstance() {
        if (!this.instance) {
            this.instance = new ServiceManager();
        }
        return this.instance;
    }

    async Commit(entity: TransitEntity): Promise<AttachFile[]> {
        // 这里只是模拟，实际需要实现具体逻辑
        return [];
    }
}

// 模拟 Platform.ServiceProxy.CommonFunction
namespace Platform {
    export namespace ServiceProxy {
        export class CommonFunction {
            static ChangeEntityToTable<T>(entities: T[]): DataTable {
                // 这里只是模拟，实际需要实现具体逻辑
                return [];
            }

            static ChangeTableToEntitys<T>(dt: DataTable): T[] {
                // 这里只是模拟，实际需要实现具体逻辑
                return [];
            }
        }
    }
}

// 模拟 TdoutbfulPay 类型
interface TdoutbfulPay {}

// 模拟 Tcashvarb 类型
interface Tcashvarb {}

// 模拟 TcheckMachine 类型
interface TcheckMachine {}

// 模拟 TcheckAccount 类型
interface TcheckAccount {}

// 模拟 TatmDbpnr 类型
interface TatmDbpnr {}

// 模拟 ResgistryHandle 类
class ResgistryHandle {
    /**
     * 大额可以支付查询
     * @param s1 参数1
     * @param s2 参数2
     * @param s3 参数3
     */
    static async LargeSumPaySelect(s1: string, s2: string, s3: string): Promise<DataTable | null> {
        const entity: TransitEntity = {
            ServiceName: "registerservice",
            FuncName: "LargeSumPayTradeSelect",
            Parameters: { "1": s1, "2": s2, "3": s3 }
        };
        const tmpresult = await ServiceManager.GetInstance().Commit(entity);
        const dt = tmpresult[0]?.AttachValue as DataTable;
        return dt;
    }

    /**
     * 大额可以支付插入
     * @param tr 支付信息实体
     */
    static async LargeSumPayInsert(tr: TdoutbfulPay): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "LargeSumPayTradeInsert",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: TableType,
                AttachValue: Platform.ServiceProxy.CommonFunction.ChangeEntityToTable([tr])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 大额可以支付更新
     * @param tr 支付信息实体
     */
    static async LargeSumPayUpdate(tr: TdoutbfulPay): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "LargeSumPayTradeUpdate",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: TableType,
                AttachValue: Platform.ServiceProxy.CommonFunction.ChangeEntityToTable([tr])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 大额可以支付删除
     * @param f_id 要删除记录的 ID
     */
    static async LargeSumPayDelete(f_id: string): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "LargeSumPayTradeDelete",
                Parameters: { "1": f_id }
            };
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 大额口头预约查询
     * @param s1 参数1
     * @param s2 参数2
     * @param s3 参数3
     */
    static async LargeSumVerBarContractSelect(s1: string, s2: string, s3: string): Promise<DataTable | null> {
        const entity: TransitEntity = {
            ServiceName: "registerservice",
            FuncName: "LargeSumVerbarContractSelect",
            Parameters: { "1": s1, "2": s2, "3": s3 }
        };
        const tmpresult = await ServiceManager.GetInstance().Commit(entity);
        const dt = tmpresult[0]?.AttachValue as DataTable;
        return dt;
    }

    /**
     * 大额口头预约插入
     * @param tr 预约信息实体
     */
    static async LargeSumVerBarContractInsert(tr: Tcashvarb): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "LargeSumVerbarContractInsert",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: TableType,
                AttachValue: Platform.ServiceProxy.CommonFunction.ChangeEntityToTable([tr])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 大额口头预约更新
     * @param tr 预约信息实体
     */
    static async LargeSumVerBarContractUpdate(tr: Tcashvarb): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "LargeSumVerbarContractUpdate",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: TableType,
                AttachValue: Platform.ServiceProxy.CommonFunction.ChangeEntityToTable([tr])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 大额口头预约删除
     * @param f_id 要删除记录的 ID
     */
    static async LargeSumVerBarContractDelete(f_id: string): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "LargeSumVerbarContractDelete",
                Parameters: { "1": f_id }
            };
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 机器检查查询
     * @param s1 参数1
     * @param s2 参数2
     * @param s3 参数3
     */
    static async MachineCheckSelect(s1: string, s2: string, s3: string): Promise<DataTable | null> {
        const entity: TransitEntity = {
            ServiceName: "registerservice",
            FuncName: "MachineCheckSelect",
            Parameters: { "1": s1, "2": s2, "3": s3 }
        };
        const tmpresult = await ServiceManager.GetInstance().Commit(entity);
        const dt = tmpresult[0]?.AttachValue as DataTable;
        return dt;
    }

    /**
     * 机器检查插入
     * @param tr 机器检查信息实体
     */
    static async MachineCheckInsert(tr: TcheckMachine): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "MachineCheckInsert",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: TableType,
                AttachValue: Platform.ServiceProxy.CommonFunction.ChangeEntityToTable([tr])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 机器检查更新
     * @param tr 机器检查信息实体
     */
    static async MachineCheckUpdate(tr: TcheckMachine): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "MachineCheckUpdate",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: TableType,
                AttachValue: Platform.ServiceProxy.CommonFunction.ChangeEntityToTable([tr])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 机器检查删除
     * @param f_id 要删除记录的 ID
     */
    static async MachineCheckDelete(f_id: string): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "MachineCheckDelete",
                Parameters: { "1": f_id }
            };
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 账户检查查询
     * @param s1 参数1
     * @param s2 参数2
     * @param s3 参数3
     */
    static async CheckAccountSelect(s1: string, s2: string, s3: string): Promise<DataTable | null> {
        const entity: TransitEntity = {
            ServiceName: "registerservice",
            FuncName: "CheckAccountSelect",
            Parameters: { "1": s1, "2": s2, "3": s3 }
        };
        const tmpresult = await ServiceManager.GetInstance().Commit(entity);
        const dt = tmpresult[0]?.AttachValue as DataTable;
        return dt;
    }

    /**
     * 账户检查插入
     * @param tr 账户检查信息实体
     */
    static async CheckAccountInsert(tr: TcheckAccount): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "CheckAccountInsert",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: TableType,
                AttachValue: Platform.ServiceProxy.CommonFunction.ChangeEntityToTable([tr])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 账户检查更新
     * @param tr 账户检查信息实体
     */
    static async CheckAccountUpdate(tr: TcheckAccount): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "CheckAccountUpdate",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: TableType,
                AttachValue: Platform.ServiceProxy.CommonFunction.ChangeEntityToTable([tr])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 账户检查删除
     * @param f_id 要删除记录的 ID
     */
    static async CheckAccountDelete(f_id: string): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "CheckAccountDelete",
                Parameters: { "1": f_id }
            };
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * ATM 查询
     * @param s1 参数1
     * @param s2 参数2
     * @param s3 参数3
     */
    static async AtmSelect(s1: string, s2: string, s3: string): Promise<DataTable | null> {
        const entity: TransitEntity = {
            ServiceName: "registerservice",
            FuncName: "AtmSelect",
            Parameters: { "1": s1, "2": s2, "3": s3 }
        };
        const tmpresult = await ServiceManager.GetInstance().Commit(entity);
        const dt = tmpresult[0]?.AttachValue as DataTable;
        return dt;
    }

    /**
     * ATM 插入
     * @param tr ATM 信息实体
     */
    static async AtmInsert(tr: TatmDbpnr): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "AtmInsert",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: TableType,
                AttachValue: Platform.ServiceProxy.CommonFunction.ChangeEntityToTable([tr])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * ATM 更新
     * @param tr ATM 信息实体
     */
    static async AtmUpdate(tr: TatmDbpnr): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "AtmUpdate",
                AttachFiles: []
            };
            const attcheds: AttachFile = {
                AttachIndex: "1",
                AttachType: TableType,
                AttachValue: Platform.ServiceProxy.CommonFunction.ChangeEntityToTable([tr])
            };
            entity.AttachFiles.push(attcheds);
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * ATM 删除
     * @param f_id 要删除记录的 ID
     */
    static async AtmDelete(f_id: string): Promise<boolean> {
        let tradeReturn = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "registerservice",
                FuncName: "AtmDelete",
                Parameters: { "1": f_id }
            };
            const tmpresult = await ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 将普通字符串格式化为日期字符串
     * @param normalString 普通字符串
     */
    static FormatNormalStringToDateString(normalString: string): string {
        if (normalString.length !== 8) return normalString;
        const s1 = normalString.slice(0, 4) + '-' + normalString.slice(4);
        const s2 = s1.slice(0, 7) + '-' + s1.slice(7);
        return s2;
    }
}

export { ResgistryHandle };