// 假设的类型定义
type TransitEntity = {
    ServiceName: string;
    FuncName: string;
    Parameters?: Record<string, string>;
    AttachFiles?: AttachFile[];
};

type AttachFile = {
    AttachIndex: string;
    AttachType: any;
    AttachValue: any;
};

type BatSysInfo = any;
type BatInfo = any;
type BatSysConfig = any;
type Types = {
    TableType: any;
};

// 假设的 ServiceManager 类
class ServiceManager {
    private static instance: ServiceManager;

    static GetInstance() {
        if (!this.instance) {
            this.instance = new ServiceManager();
        }
        return this.instance;
    }

    Commit(entity: TransitEntity): AttachFile[] {
        // 这里应该实现实际的提交逻辑，暂时返回空数组
        return [];
    }
}

// 假设的 CommonFunction 类
class CommonFunction {
    static ChangeTableToEntitys<T>(dataTable: any): T[] {
        // 这里应该实现实际的转换逻辑，暂时返回空数组
        return [];
    }

    static ChangeEntityToTable(entities: any[]): any {
        // 这里应该实现实际的转换逻辑，暂时返回 null
        return null;
    }
}

/**
 * 跑批流程处理
 */
class SysBatchHandle {
    /**
     * 单点登录
     * @param TGT 
     * @returns 
     */
    static SSOAuthentication(TGT: string): string {
        let result = '';
        try {
            const entity: TransitEntity = {
                ServiceName: "CommonService",
                FuncName: "SSOAuthentication",
                Parameters: { "1": TGT }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 读取批量顺序控制表数据（按mainseqn、subseqn排序出来的）
     * @param wherecond 条件
     * @returns 
     */
    static GetBatSysInfos(wherecond: string): BatSysInfo[] | null {
        let result: BatSysInfo[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "GetBatSysList",
                Parameters: { "1": wherecond }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<BatSysInfo>(tmpresult[0].AttachValue as any);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 在批量顺序表中插入数据
     * @param batSysInfos 数据串
     * @returns 
     */
    static InsertBatSysInfo(batSysInfos: BatSysInfo[]): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "BatSysAdd",
                AttachFiles: []
            };
            const file: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(batSysInfos)
            };
            entity.AttachFiles.push(file);
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 更新批量顺序控制表数据
     * @param batSysInfos 数据串
     * @returns 
     */
    static UpDateBatSysInfo(batSysInfos: BatSysInfo[]): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "BatSysUpdata",
                AttachFiles: []
            };
            const file: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(batSysInfos)
            };
            entity.AttachFiles.push(file);
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 更新批量顺序控制表数据
     * @param batSysInfo 
     * @returns 
     */
    static UpDateBatSysInfoSingle(batSysInfo: BatSysInfo): boolean {
        const data: BatSysInfo[] = [];
        if (batSysInfo) {
            data.push(batSysInfo);
        }
        return this.UpDateBatSysInfo(data);
    }

    /**
     * 跑批过程中更新顺序控制表
     * @param batSysInfo 跑批流程实体
     * @returns 布尔型
     */
    static UpProcessBatSysInfo(batSysInfo: BatSysInfo): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "UpdateProcessBatInfo",
                AttachFiles: []
            };
            const file: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable([batSysInfo])
            };
            entity.AttachFiles.push(file);
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 删除批量顺序表数据
     * @param batSysInfos 被删除对象
     * @returns 
     */
    static DeleteBatSysInfo(batSysInfos: BatSysInfo[]): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "BatSysDelete",
                AttachFiles: []
            };
            const file: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(batSysInfos)
            };
            entity.AttachFiles.push(file);
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 验证顺序控制表记录是否存在
     * @param batSysInfo 验证对象
     * @returns 
     */
    static IsBatSysInfoExistSingle(batSysInfo: BatSysInfo): boolean {
        if (!batSysInfo) {
            return false;
        }

        try {
            const data = this.GetBatSysInfos(` batcode='${batSysInfo.batcode}' and syscode='${batSysInfo.syscode}'`);
            if (!data || data.length === 0) {
                return false;
            }
            return true;
        } catch (e) {
            throw e;
        }
    }

    /**
     * 验证顺序控制表记录是否存在
     * @param batcode 批量编号
     * @param syscode 系统编号
     * @returns 
     */
    static IsBatSysInfoExist(batcode: string, syscode: string): boolean {
        try {
            const data = this.GetBatSysInfos(` batcode='${batcode}' and syscode='${syscode}'`);
            if (!data || data.length === 0) {
                return false;
            }
            return true;
        } catch (e) {
            throw e;
        }
    }

    /**
     * 把批量顺序控制表数据备份到历史记录表中
     * @param nextdate 
     * @param iscontentmonth 
     * @param nextmonth 
     * @returns 
     */
    static BackUpBatSysInfo(nextdate: string, iscontentmonth: boolean, nextmonth: string): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "BatSysHisAdd",
                Parameters: { "1": nextdate, "2": iscontentmonth ? "1" : "0", "3": nextmonth }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 取批量历史记录表数据
     * @param begDate 条件
     * @returns 
     */
    static GetBatSysHisInfos(begDate: string): BatSysInfo[] | null {
        let result: BatSysInfo[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "GetBatSysHisList",
                Parameters: { "1": begDate }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<BatSysInfo>(tmpresult[0].AttachValue as any);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 在详情表中插入数据
     * @param batSysInfos 数据串
     * @returns 布尔型
     */
    static InsertBatInfo(batSysInfos: BatInfo[]): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "BatSysInfoAdd",
                AttachFiles: []
            };
            const file: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(batSysInfos)
            };
            entity.AttachFiles.push(file);
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 备份详情数据到历史表
     * @param end_date 
     * @param tx_date 
     * @returns 
     */
    static BackUpBatInfo(end_date: string, tx_date: string): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "BatSysInfoHisAdd",
                Parameters: { "1": end_date, "2": tx_date }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 获取批量详情表信息
     * @param syscode 
     * @param batcode 
     * @returns 
     */
    static GetBatSysInfoInfos(syscode: string, batcode: string): BatInfo[] | null {
        let result: BatInfo[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "GetBatSysInfoList",
                Parameters: { "1": syscode, "2": batcode }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<BatInfo>(tmpresult[0].AttachValue as any);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 获取跑批系统列表
     * @returns 
     */
    static GetBatSystemList(): BatSysConfig[] | null {
        let result: BatSysConfig[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "GetBatSystemList"
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<BatSysConfig>(tmpresult[0].AttachValue as any);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 查询批量详情历史信息
     * @param textdate 
     * @param syscode 
     * @param batcode 
     * @returns 
     */
    static GetBatSysInfoHisInfos(textdate: string, syscode: string, batcode: string): BatInfo[] | null {
        let result: BatInfo[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "GetBatSysInfoHisList",
                Parameters: { "1": textdate, "2": syscode, "3": batcode }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<BatInfo>(tmpresult[0].AttachValue as any);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 更新主控制表
     * @param nextDate 
     * @param opteller 
     * @param suncnt 
     * @returns 
     */
    static UpSysControl(nextDate: string, opteller: string, suncnt: string): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "UpSysControl",
                Parameters: { "1": nextDate, "2": opteller, "3": suncnt }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 校验当前日期是否可以开启跑批
     * @param tradeDate 营业日期
     * @param mainseqn 序号
     * @param stat 状态
     * @returns 
     */
    static IsAllowStart(tradeDate: string, mainseqn: string, stat: string): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "IsAllowStart",
                Parameters: { "1": tradeDate, "2": mainseqn, "3": stat }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 获取跑批系统列表
     * @returns 
     */
    static UpSysControlStat(): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "UpSysControlStat"
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 查询总控表跑批日期
     * @returns 
     */
    static GetSysBatStartDay(): string {
        let result = '';
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "GetSysBatStartDay"
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 卸载数据中心数据
     * @param tradeDate 
     * @param serverName 
     * @returns 
     */
    static UnLoadData(tradeDate: string, serverName: string): boolean {
        let ret = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "UnLoadData",
                Parameters: { "1": tradeDate, "2": serverName }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                ret = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return ret;
    }

    /**
     * 拉起备份流水表数据
     * @param tradeDate 
     * @returns 0:拉起失败；1：拉起成功
     */
    static pullBackupTradedata(tradeDate: string): string {
        let result = '';
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "pullBackupTradedata",
                Parameters: { "1": tradeDate }
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 获取备份状态
     * @returns 1：正在执行；2：备份完成；3：备份失败
     */
    static getBackupState(): string {
        let result = '';
        try {
            const entity: TransitEntity = {
                ServiceName: "sysbatchservice",
                FuncName: "getBackupState"
            };
            const tmpresult = ServiceManager.GetInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }
}

export default SysBatchHandle;