// 假设的类型定义
interface TransitEntity {
    ServiceName: string;
    FuncName: string;
    Parameters: { [key: string]: string };
    AttachFiles?: AttachFile[];
}

interface AttachFile {
    AttachIndex: string;
    AttachType: any;
    AttachValue: any;
}

interface ServiceManagerStatic {
    GetInstance(): {
        Commit(entity: TransitEntity): AttachFile[];
    };
}

// 假设的全局变量和函数
const ServiceManager: ServiceManagerStatic = {
    GetInstance: () => ({
        Commit: (entity: TransitEntity) => {
            // 这里只是模拟实现，实际需要替换为真实逻辑
            return [];
        }
    })
};

namespace Platform {
    export namespace ServiceProxy {
        export namespace CommonFunction {
            export function ChangArrayToStr(arr: string[]): string {
                // 简单模拟，实际需要替换为真实逻辑
                return arr.join(',');
            }

            export function ChangStrToArray(str: string): string[] {
                // 简单模拟，实际需要替换为真实逻辑
                return str.split(',');
            }
        }
    }
}

namespace TellerSystem {
    export namespace ServiceProxy {
        export namespace Ext {
            export namespace ServiceHelper {
                export class UserManagerHandle {
                    // 柜员管理
                    /**
                     * 发送公告信息
                     * @param key 唯一标示
                     * @param sender 发送人
                     * @param time 时间
                     * @param title 标题
                     */
                    public static SendMessage(key: string, sender: string, time: string, title: string): void {
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "SocketSendMessage",
                                Parameters: { "1": key, "2": sender, "3": time, "4": title }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                        } catch (e) {
                            throw e;
                        }
                    }

                    /**
                     * 发送信息
                     * @param sender 发送人
                     * @param message 发送信息
                     * @param TradeNo 接收信息柜员号列表
                     */
                    public static SendMessage2(sender: string, message: string, TradeNo: string[]): void {
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "SendMessage",
                                Parameters: {
                                    "1": sender,
                                    "2": message,
                                    "3": Platform.ServiceProxy.CommonFunction.ChangArrayToStr(TradeNo)
                                }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                        } catch (e) {
                            throw e;
                        }
                    }

                    /**
                     * 获取外部存储数据
                     * @param tradeno 
                     * @param tradetype 
                     * @param key_words 
                     * @param branch_no 
                     * @returns 
                     */
                    public static GetOutOfData(tradeno: string, tradetype: string, key_words: string, branch_no: string): string | null {
                        let result: string | null = null;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "TradeService",
                                FuncName: "GetOutOfData",
                                Parameters: { "1": tradeno, "2": tradetype, "3": key_words, "4": branch_no }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = tmpresult[0].AttachValue.toString();
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    /**
                     * 插入外部存储数据
                     * @param guid 
                     * @param tradenum 
                     * @param tradetype 
                     * @param keywords 
                     * @param tradedata 
                     * @param tellerno 
                     * @param branch_no 
                     * @returns 
                     */
                    public static SetOutOfData(guid: string, tradenum: string, tradetype: string, keywords: string, tradedata: string, tellerno: string, branch_no: string): boolean {
                        let result = false;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "TradeService",
                                FuncName: "SetOutOfData",
                                Parameters: {
                                    "1": guid, "2": tradenum, "3": tradetype, "4": keywords,
                                    "5": tradedata, "6": tellerno, "7": branch_no
                                }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = Boolean(tmpresult[0].AttachValue.toString());
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    /**
                     * 更新外部存储数据
                     * @param Guid 
                     * @param tradedata 
                     * @returns 
                     */
                    public static UpdateOutOfData(Guid: string, tradedata: string): boolean {
                        let result = false;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "TradeService",
                                FuncName: "UpdateOutOfData",
                                Parameters: { "1": Guid, "2": tradedata }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = Boolean(tmpresult[0].AttachValue.toString());
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    /**
                     * 向表中插入Blob字段的值
                     * @param tableName 表名
                     * @param columnName 列名
                     * @param strWhere where条件
                     * @param data 插入的数据
                     * @returns Ture成功|False失败
                     */
                    public static InsertBlob(tableName: string, columnName: string, strWhere: string, data: Uint8Array): boolean {
                        let result = false;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "InsertBlob",
                                Parameters: { "1": tableName, "2": columnName, "3": strWhere },
                                AttachFiles: []
                            };
                            const files: AttachFile = {
                                AttachIndex: "4",
                                AttachType: "ByteType",
                                AttachValue: data
                            };
                            entity.AttachFiles.push(files);

                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = Boolean(tmpresult[0].AttachValue.toString());
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    /**
                     * 删除文件
                     * @param sPath 文件的路径
                     * @returns 
                     */
                    public static DeleteFolderOrFile(sPath: string): boolean {
                        let result = false;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "DeleteFolder",
                                Parameters: { "1": sPath }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = Boolean(tmpresult[0].AttachValue.toString());
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    public static GetBlob(tableName: string, columnName: string, strWhere: string): Uint8Array | null {
                        let result: Uint8Array | null = null;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "GetBlobColumn",
                                Parameters: { "1": tableName, "2": columnName, "3": strWhere }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = tmpresult[0].AttachValue as Uint8Array;
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    /**
                     * 保存或更新客户端配置文件
                     * @param fileName 文件名称
                     * @param file 文件内容
                     * @param flag 追加还是覆盖
                     * @returns 
                     */
                    public static SaveConfFile(fileName: string, file: Uint8Array, flag: boolean): boolean {
                        let result = false;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "SaveFile",
                                Parameters: { "1": "ServiceSettings.ConfigFilePath" + fileName, "3": flag.toString() },
                                AttachFiles: []
                            };
                            const files: AttachFile = {
                                AttachIndex: "2",
                                AttachType: "ByteType",
                                AttachValue: file
                            };
                            entity.AttachFiles.push(files);
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = Boolean(tmpresult[0].AttachValue.toString());
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    /**
                     * 查找服务端报表文件列表
                     * @param path 
                     * @returns 
                     */
                    public static findReportFile(path: string): string[] | null {
                        let result: string[] | null = null;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "FindReportFile",
                                Parameters: { "1": path }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = Platform.ServiceProxy.CommonFunction.ChangStrToArray(tmpresult[0].AttachValue.toString());
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    /**
                     * 提取客户端配置文件内容
                     * @param fileName 文件名称
                     * @returns 
                     */
                    public static GetConfFile(fileName: string): Uint8Array | null {
                        let result: Uint8Array | null = null;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "GetFile",
                                Parameters: { "1": "ServiceSettings.ConfigFilePath" + fileName }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = tmpresult[0].AttachValue as Uint8Array;
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    /**
                     * 提取客户端配置文件内容
                     * @param date 
                     * @param fileName 文件名称
                     * @returns 
                     */
                    public static GetLogFile(date: string, fileName: string): Uint8Array | null {
                        let result: Uint8Array | null = null;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "GetFile",
                                Parameters: { "1": "Logs/" + date + "/LoginUserInfo.TellerNo/" + fileName }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = tmpresult[0].AttachValue as Uint8Array;
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    /**
                     * 执行一条SQL语句,返回执行影响的行数
                     * @param strsql Sql语句
                     * @returns 
                     */
                    public static ExecuteSql(strsql: string): number {
                        let result = 0;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "ExecuteSql",
                                Parameters: { "1": strsql }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = parseInt(tmpresult[0].AttachValue.toString(), 10);
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    /**
                     * 执行一条SQL查询语句，返回查询结果数据集
                     * @param sql Sql语句
                     * @returns 
                     */
                    public static ExecuteQuery(sql: string): string[] | null {
                        let result: string[] | null = null;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "ExecuteQuery",
                                Parameters: { "1": sql }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = Platform.ServiceProxy.CommonFunction.ChangStrToArray(tmpresult[0].AttachValue.toString());
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    /**
                     * 执行多条SQL语句，实现数据库事务
                     * @param SQLStringList Sql语句集合
                     * @returns 
                     */
                    public static ExecuteSQLTran(SQLStringList: string[]): boolean {
                        let result = false;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "ExecuteSQLTran",
                                Parameters: { "1": Platform.ServiceProxy.CommonFunction.ChangArrayToStr(SQLStringList) }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = Boolean(tmpresult[0].AttachValue.toString());
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    public static SortSockAddress(Address: string[]): string | null {
                        let result: string | null = null;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "SortSockAddress",
                                Parameters: { "1": Platform.ServiceProxy.CommonFunction.ChangArrayToStr(Address) }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = tmpresult[0].AttachValue.toString();
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    public static ExecuteProcudure(procedureName: string, arg: string[]): boolean {
                        let result = false;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "ExecuteProcudure",
                                Parameters: {
                                    "1": procedureName,
                                    "2": Platform.ServiceProxy.CommonFunction.ChangArrayToStr(arg)
                                }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = Boolean(tmpresult[0].AttachValue.toString());
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    public static ExecuteProcudureWithResult(procedureName: string, arg: string[]): string[] | null {
                        let result: string[] | null = null;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "ExecuteProcudureWithResult",
                                Parameters: {
                                    "1": procedureName,
                                    "2": Platform.ServiceProxy.CommonFunction.ChangArrayToStr(arg)
                                }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = Platform.ServiceProxy.CommonFunction.ChangStrToArray(tmpresult[0].AttachValue.toString());
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }

                    public static ExecuteProcudureWithResultOutIndex(procedureName: string, outParameterIndex: number, arg: string[]): string | null {
                        let result: string | null = null;
                        try {
                            const entity: TransitEntity = {
                                ServiceName: "UtilService",
                                FuncName: "ExecuteProcudureWithResultOutIndex",
                                Parameters: {
                                    "1": procedureName,
                                    "2": outParameterIndex.toString(),
                                    "3": Platform.ServiceProxy.CommonFunction.ChangArrayToStr(arg)
                                }
                            };
                            const tmpresult = ServiceManager.GetInstance().Commit(entity);
                            if (tmpresult && tmpresult.length === 1) {
                                result = tmpresult[0].AttachValue.toString();
                            }
                        } catch (e) {
                            throw e;
                        }
                        return result;
                    }
                }
            }
        }
    }
}