// 假设的类型定义，需要根据实际情况调整
interface TransitEntity {
    ServiceName: string;
    FuncName: string;
    Parameters: { [key: string]: string };
    AttachFiles?: AttachFile[];
}

interface AttachFile {
    AttachIndex: string;
    AttachType: any;
    AttachValue: any;
}

interface TellerInfo {
    // 这里可以补充 TellerInfo 的具体属性
}

interface WorkItemDesc {
    // 这里可以补充 WorkItemDesc 的具体属性
}

interface ProcessInfo {
    // 这里可以补充 ProcessInfo 的具体属性
}

class ServiceManager {
    private static instance: ServiceManager;

    private constructor() { }

    static getInstance(): ServiceManager {
        if (!this.instance) {
            this.instance = new ServiceManager();
        }
        return this.instance;
    }

    Commit(entity: TransitEntity): AttachFile[] {
        // 这里需要实现实际的提交逻辑
        return [];
    }
}

namespace Platform {
    export namespace ServiceProxy {
        export namespace CommonFunction {
            export function ChangeDicToStr(dic: { [key: string]: string }): string {
                // 这里需要实现实际的转换逻辑
                return '';
            }

            export function ChangeEntityToTable<T>(entities: T[]): any {
                // 这里需要实现实际的转换逻辑
                return null;
            }

            export function ChangeTableToEntitys<T>(table: any): T[] {
                // 这里需要实现实际的转换逻辑
                return [];
            }

            export function ChangeStrToDic(str: string): { [key: string]: string } {
                // 这里需要实现实际的转换逻辑
                return {};
            }
        }
    }
}

export class WorkFlowHandle {
    /**
     * 获取工作流请求返回
     * @param workflowId 
     * @returns 
     */
    static GetWorkFlowTradeReturn(workflowId: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "TradeRequest",
                Parameters: { "1": workflowId },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 获取工作流请求返回
     * @param tellerinfo 基本信息
     * @param tradePageData 页面数据
     * @param alldatas 总线数据
     * @param id ID
     * @param serialNo 流水号
     * @param isWorkFlow 是否走工作流
     * @param firstNodeFlag 首节点标志位
     * @returns 
     */
    static GetWorkFlowTradeSubmit(tellerinfo: TellerInfo, tradePageData: Uint8Array, alldatas: { [key: string]: string }, id: string, serialNo: string, isWorkFlow: boolean, firstNodeFlag: boolean): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "TradeSubmit",
                Parameters: {
                    "3": Platform.ServiceProxy.CommonFunction.ChangeDicToStr(alldatas),
                    "4": id,
                    "5": serialNo,
                    "6": isWorkFlow.toString(),
                    "7": firstNodeFlag.toString()
                },
                AttachFiles: []
            };
            // 做 tellerinfo 类型转换
            const teller: AttachFile = {
                AttachIndex: "1",
                AttachType: "TableType", // 假设 Types.TableType 对应 "TableType"
                AttachValue: Platform.ServiceProxy.CommonFunction.ChangeEntityToTable([tellerinfo])
            };
            entity.AttachFiles.push(teller);
            // 做 tradePageData 类型转换
            const pagedata: AttachFile = {
                AttachIndex: "2",
                AttachType: "ByteType", // 假设 Types.ByteType 对应 "ByteType"
                AttachValue: tradePageData
            };
            entity.AttachFiles.push(pagedata);
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 根据柜员号和任务状态获取任务列表
     * @param tellerNo 
     * @param state 
     * @returns 
     */
    static GetWorkFlowWorkItem(tellerNo: string, state: string): WorkItemDesc[] | null {
        let tradeReturn: WorkItemDesc[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "GetWorkItems",
                Parameters: { "1": tellerNo, "2": state },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Platform.ServiceProxy.CommonFunction.ChangeTableToEntitys<WorkItemDesc>(tmpresult[0].AttachValue as any);
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 挂起
     * @param processid 
     * @returns 
     */
    static ProcessSuspend(processid: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "ProcessSuspend",
                Parameters: { "1": processid },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 
     * @param processid 
     * @returns 
     */
    static ProcessResume(processid: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "ProcessResume",
                Parameters: { "1": processid },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 工作流撤销
     * @param processid 
     * @returns 
     */
    static ProcessDelete(processid: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "ProcessDelete",
                Parameters: { "1": processid },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    static GetTradePageData(serialNo: string, ActiviteId: string): Uint8Array | null {
        let result: Uint8Array | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "GetTradePageData",
                Parameters: { "1": serialNo, "2": ActiviteId },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as Uint8Array;
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 根据柜员号，获取流程状态信息
     * @param serialNo 
     * @returns 
     */
    static GetTradedataBySerialNo(serialNo: string): { [key: string]: string } | null {
        let result: { [key: string]: string } | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "GetTradedataBySerialNo",
                Parameters: { "1": serialNo },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Platform.ServiceProxy.CommonFunction.ChangeStrToDic(tmpresult[0].AttachValue.toString());
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 根据柜员号，获取该柜员发起的流程状态信息
     * @param userId 
     * @returns 
     */
    static getProcessInfosByUserId(userId: string): ProcessInfo[] | null {
        let result: ProcessInfo[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "GetProcessInfosByUserId",
                Parameters: { "1": userId },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Platform.ServiceProxy.CommonFunction.ChangeTableToEntitys<ProcessInfo>(tmpresult[0].AttachValue as any);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 根据流程实例ID，删除流程实例
     * @param proInsId 
     * @param delReason 
     */
    static deleteProcessInsById(proInsId: string, delReason: string): void {
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "DeleteProcessInsById",
                Parameters: { "1": proInsId, "2": delReason },
                AttachFiles: []
            };
            ServiceManager.getInstance().Commit(entity);
        } catch (e) {
            throw e;
        }
    }

    /**
     * 根据发起柜员号删除历史任务
     * @param TellerNo 
     */
    static deleteHisTaskInsByuserId(TellerNo: string): void {
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "DeleteHisTaskInsByuserId",
                Parameters: { "1": TellerNo },
                AttachFiles: []
            };
            ServiceManager.getInstance().Commit(entity);
        } catch (e) {
            throw e;
        }
    }

    /**
     * 根据发起柜员号删除该柜员发起的流程实例信息记录
     * @param TellerNo 
     */
    static deleteHisProcessInsByuserId(TellerNo: string): void {
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "DeleteHisProcessInsByuserId",
                Parameters: { "1": TellerNo },
                AttachFiles: []
            };
            ServiceManager.getInstance().Commit(entity);
        } catch (e) {
            throw e;
        }
    }

    /**
     * 根据任务id，放弃任务
     * @param taskid 
     * @param begTeller 
     * @returns 
     */
    static GiveUpTask(taskid: string, begTeller: string): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "GiveUpTask",
                Parameters: { "1": taskid, "2": begTeller },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 根据任务id，提交任务
     * @param taskid 
     */
    static CompleteTaskById(taskid: string): void {
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "CompleteTaskById",
                Parameters: { "1": taskid },
                AttachFiles: []
            };
            ServiceManager.getInstance().Commit(entity);
        } catch (e) {
            throw e;
        }
    }

    /**
     * 根据任务id，柜员号，查询该任务是否被他人认领
     * @param taskid 
     * @param TellerNo 
     * @param lvl 
     * @returns 
     */
    static CheckTask(taskid: string, TellerNo: string, lvl: string): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "CheckTask",
                Parameters: { "1": taskid, "2": TellerNo, "3": lvl },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 获取主流水
     * @param tradeNo 交易号
     * @param telNo 柜员号
     * @param brNo 机构号
     * @param businessdate 营业日期
     * @param localip 本地IP地址
     * @returns 
     */
    static GenNextSerialNo(tradeNo: string, telNo: string, brNo: string, businessdate: string, localip: string): string | null {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "GenNextSerialNo",
                Parameters: { "1": tradeNo, "2": telNo, "3": brNo, "4": businessdate, "5": localip },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 获取柜员流水
     * @param telNo 柜员号
     * @param brNo 机构号
     * @param businessdate 营业日期
     * @param serialid 
     * @returns 
     */
    static GenTellerSerialNo(telNo: string, brNo: string, businessdate: string, serialid: string): string | null {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "genTellerSerialNo",
                Parameters: { "1": telNo, "2": brNo, "3": businessdate, "4": serialid },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 获取某个柜员是否还有未完成的任务
     * @param telNo 柜员号
     * @returns 
     */
    static IsExistProcessByTellerNO(telNo: string): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "getWorkItemsProcessByTellerNO",
                Parameters: { "1": telNo },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 获取柜员任务个数
     * @param TellerNo 柜员号
     * @returns 
     */
    static getWorkItemsProcessNum(TellerNo: string): number {
        let result = 0;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "getWorkItemsProcessNum",
                Parameters: { "1": TellerNo },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Number(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }

    /**
     * 根据任务id，柜员号，查询该复核任务是否被他人认领
     * 只有会计结算部才启用任务认领的功能
     * @param taskid 
     * @param TellerNo 
     * @returns 
     */
    static CheckSuper(taskid: string, TellerNo: string): boolean {
        let result = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "ProcessService",
                FuncName: "CheckSuper",
                Parameters: { "1": taskid, "2": TellerNo },
                AttachFiles: []
            };
            const tmpresult = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = Boolean(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }
        return result;
    }
}