// 模拟 ConfigType 枚举
enum ConfigType {
    System,
    Client
}

// 模拟 TT_SysConfig 类
class TT_SysConfig {
    Configid: string;
    Configname: string;
    Configvalue: string;

    constructor(Configid: string, Configname: string, Configvalue: string) {
        this.Configid = Configid;
        this.Configname = Configname;
        this.Configvalue = Configvalue;
    }
}

// 模拟 ConfigManager 类
class ConfigManager {
    private static instance: ConfigManager;
    private constructor() { }

    static GetInstance(): ConfigManager {
        if (!this.instance) {
            this.instance = new ConfigManager();
        }
        return this.instance;
    }

    GetConfigValue(key: string, type: ConfigType): string {
        // 这里需要根据实际情况实现获取配置值的逻辑
        return '';
    }
}

// 模拟 TradeManagerHandle 类
class TradeManagerHandle {
    static TT_SYSCONFIG_GetConfigValueByConfigID(id: string): string {
        // 这里需要根据实际情况实现获取配置值的逻辑
        return '';
    }

    static GetSysconfigList(prefix: string): TT_SysConfig[] {
        // 这里需要根据实际情况实现获取配置列表的逻辑
        return [];
    }
}

// 模拟 LoginUserInfo 类
class LoginUserInfo {
    static KinbrNo: number = 0;
}

// 模拟 Func 类型
type Func<T, R> = (arg: T) => R;

class ServiceSettings {
    /**
     * 服务端采用的临时目录
     */
    static get TempFilePath(): string {
        return ConfigManager.GetInstance().GetConfigValue('TempFilePath', ConfigType.System);
    }

    /**
     * 服务端采用的临时目录
     */
    static get ConfigFilePath(): string {
        return ConfigManager.GetInstance().GetConfigValue('ConfigFilePath', ConfigType.System);
    }

    /**
     * 服务端地址
     */
    static get AuthServiceUri(): string {
        return ConfigManager.GetInstance().GetConfigValue('AuthServiceAddress', ConfigType.Client) + "mdps";
    }

    static get AuthSystemName(): string {
        const FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID('2ea3a7e6cfa04f89a335bf1e363295de');
        if (LoginUserInfo.KinbrNo.toString().substring(0, 3) === '106' && FLAG === 'V6') {
            return "pycz";
        } else {
            return ConfigManager.GetInstance().GetConfigValue('AuthSystemName', ConfigType.Client);
        }
    }

    static get AuthSystemSecret(): string {
        const FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID('2ea3a7e6cfa04f89a335bf1e363295de');
        if (LoginUserInfo.KinbrNo.toString().substring(0, 3) === '106' && FLAG === 'V6') {
            return "PyczDefaultKey";
        } else {
            return ConfigManager.GetInstance().GetConfigValue('AuthSystemSecret', ConfigType.Client);
        }
    }

    /**
     * 文件地址
     */
    static get AuthFileUri(): string {
        return ConfigManager.GetInstance().GetConfigValue('AuthServiceAddress', ConfigType.Client) + "upload/";
    }

    /**
     * 远程授权文件服务器地址
     */
    static get AuthFileSever(): string {
        return ConfigManager.GetInstance().GetConfigValue('AuthServiceAddress', ConfigType.Client) + "fileService/";
    }

    private static _SysConfig_: TT_SysConfig[] = TradeManagerHandle.GetSysconfigList("");

    // static _SysConfig_: { Configid: string; Configname: string; Configvalue: string }[] = [];
    static GetConfig(id: string, defV: number): number;
    static GetConfig(id: string, defV: boolean): boolean;
    static GetConfig(id: string, defV: string): string;
    static GetConfig<T>(id: string, defV?: T): T;

    // 函数实现
    static GetConfig<T>(id: string, defV?: T): T {
        if (typeof defV === 'number') {
            const value = this.GetConfig(id, '');
            const v = parseInt(value, 10);
            return isNaN(v) ? defV : v as unknown as T;
        } else if (typeof defV === 'boolean') {
            const value = this.GetConfig(id, '');
            return value.toLowerCase() === 'true' ? true as unknown as T : false as unknown as T;
        } else {
            // 设计时不执行
            // 由于 TypeScript 环境没有类似 WPF 的设计时模式，这里暂时注释掉 hulei
            // if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(new System.Windows.DependencyObject())) return defV;
            const item = this._SysConfig_.find(x => x.Configid === id || x.Configname === id);
            // 找到配置项，返回
            if (item) return item.Configvalue as unknown as T;
            // 数据库未配置的情况，以 System 配置文件为准
            const value = ConfigManager.GetInstance().GetConfigValue(id, ConfigType.System);
            if (value) return value as unknown as T;
            return defV as T;
        }
    }

    /**
     * 获取参数集合
     */
    static GetConfigList(prefix?: string): { [key: string]: string } {
        if (!prefix) return {};
        // 优先匹配 id
        const dict = this._SysConfig_.filter(x => x.Configid.startsWith(prefix)).reduce((acc, x) => {
            acc[x.Configid.substring(prefix.length)] = x.Configvalue;
            return acc;
        }, {} as { [key: string]: string });
        if (Object.keys(dict).length > 0) return dict;
        return this._SysConfig_.filter(x => x.Configname.startsWith(prefix)).reduce((acc, x) => {
            acc[x.Configname.substring(prefix.length)] = x.Configvalue;
            return acc;
        }, {} as { [key: string]: string });
    }



    /**
     * 是否登记打印数据
     */
    static get IsRecordPrintData(): boolean {
        return this.GetConfig('45befe979ea74ef39b1fe2289c58bb63', false);
    }

    /**
     * 银行标签
     */
    static get BankFlag(): string {
        return TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID('2ea3a7e6cfa04f89a335bf1e363295de');
    }
}

export { ServiceSettings };
