// 定义 IEntity 接口，由于不清楚具体内容，先设为空
interface IEntity {}

// 定义 PropertyChangedEventHandler 类型
type PropertyChangedEventHandler = (sender: any, args: { propertyName: string }) => void;

// 定义 INotifyPropertyChanged 接口
interface INotifyPropertyChanged {
    PropertyChanged: PropertyChangedEventHandler[];
    RaisePropertyChanged(propertyName: string): void;
}

// 定义 TT_MenuInfo 类
class TT_MenuInfo implements IEntity, INotifyPropertyChanged {
    // 事件处理函数数组
    PropertyChanged: PropertyChangedEventHandler[] = [];

    // 字段
    private _menuid: string = '';
    private _pmenuid: string = '';
    private _menuorder: string = '';
    private _menuname: string = '';
    private _tradeno: string = '';
    private _tradefilename: string = '';
    private _state: string = '';
    private _comments: string = '';
    private _metadata: { [key: string]: string } = {};

    // 构造函数
    constructor(element?: { attributes: () => { toList: () => { forEach: (callback: (c: { name: { toString: () => string }, value: string }) => void ) => void } } }, pid?: string) {
        if (element && pid) {
            element.attributes().toList().forEach(c => {
                this.SetValue(c.name.toString(), c.value);
            });
            this.PMenuID = pid;
        }
    }

    // 属性
    get MenuID(): string {
        return this._menuid;
    }
    set MenuID(value: string) {
        if (this._menuid !== value) {
            this._menuid = value;
            this.RaisePropertyChanged('MenuID');
        }
    }

    get PMenuID(): string {
        return this._pmenuid;
    }
    set PMenuID(value: string) {
        if (this._pmenuid !== value) {
            this._pmenuid = value;
            this.RaisePropertyChanged('PMenuID');
        }
    }

    get MenuOrder(): string {
        return this._menuorder;
    }
    set MenuOrder(value: string) {
        if (this._menuorder !== value) {
            this._menuorder = value;
            this.RaisePropertyChanged('MenuOrder');
        }
    }

    get MenuName(): string {
        return this._menuname;
    }
    set MenuName(value: string) {
        if (this._menuname !== value) {
            this._menuname = value;
            this.RaisePropertyChanged('MenuName');
        }
    }

    get TradeNo(): string {
        return this._tradeno;
    }
    set TradeNo(value: string) {
        if (this._tradeno !== value) {
            this._tradeno = value;
            this.RaisePropertyChanged('TradeNo');
        }
    }

    get TradeFileName(): string {
        return this._tradefilename;
    }
    set TradeFileName(value: string) {
        if (this._tradefilename !== value) {
            this._tradefilename = value;
            this.RaisePropertyChanged('TradeFileName');
        }
    }

    get State(): string {
        return this._state;
    }
    set State(value: string) {
        if (this._state !== value) {
            this._state = value;
            this.RaisePropertyChanged('State');
        }
    }

    get Comments(): string {
        return this._comments;
    }
    set Comments(value: string) {
        if (this._comments !== value) {
            this._comments = value;
            this.RaisePropertyChanged('Comments');
        }
    }

    get Metadata(): { [key: string]: string } {
        if (!this._metadata) {
            this._metadata = {};
        }
        return this._metadata;
    }
    set Metadata(value: { [key: string]: string }) {
        this._metadata = value;
    }

    // 扩展方法
    UpdataMetadataValue(key: string, value: string): void {
        this.Metadata[key] = value;
        this.RaisePropertyChanged('Metadata');
    }

    AddMetadataValue(key: string, value: string): void {
        if (!this.Metadata[key]) {
            this.Metadata[key] = value;
        }
        this.RaisePropertyChanged('Metadata');
    }

    // 方法
    ChangeFromTable(parms: any[]): void {
        this.MenuID = parms[0].toString();
        this.PMenuID = parms[1].toString();
        this.MenuOrder = parms[2].toString();
        this.MenuName = parms[3].toString();
        this.TradeNo = parms[4].toString();
        this.TradeFileName = parms[5].toString();
        this.State = parms[6].toString();
        this.Comments = parms[7].toString();
        // 假设 Platform.ServiceProxy.CommonFunction.ChangeStrToDic 有对应的 TypeScript 实现
        // 这里先简单模拟
        this.Metadata = JSON.parse(parms[8].toString());
    }

    ChangeToTable(): string[] {
        // 假设 Platform.ServiceProxy.CommonFunction.ChangeDicToStr 有对应的 TypeScript 实现
        // 这里先简单模拟
        const metadataStr = Object.keys(this.Metadata).length > 0 ? JSON.stringify(this.Metadata) : '';
        return [
            this.MenuID,
            this.PMenuID,
            this.MenuOrder,
            this.MenuName,
            this.TradeNo,
            this.TradeFileName,
            this.State,
            this.Comments,
            metadataStr
        ];
    }

    private SetValue(name: string, value: string): void {
        const properties = Object.getOwnPropertyNames(this);
        const found = properties.some(prop => {
            if (this.isEqual(prop, name)) {
                (this as any)[prop] = value.trim();
                return true;
            }
            return false;
        });
        if (!found) {
            this.AddMetadataValue(name, value);
        }
    }

    // 触发属性变更事件
    RaisePropertyChanged(propertyName: string): void {
        this.PropertyChanged.forEach(handler => {
            handler(this, { propertyName });
        });
    }

    // 字符串比较方法
    private isEqual(str1: string, str2: string): boolean {
        const ConvertStr: { [key: string]: string } = { "name": "MenuName", "menucode": "TradeNo", "action": "TradeFileName" };
        if (str1 === str2) {
            return true;
        } else if (ConvertStr[str2]) {
            return str1 === ConvertStr[str2];
        }
        return false;
    }
}

export { TT_MenuInfo };