// 定义 PropertyChangedEventHandler 类型
type PropertyChangedEventHandler = (sender: any, event: { propertyName: string }) => void;

// 定义 IEntity 接口，这里暂时留空，可根据实际情况补充
interface IEntity {}

// 定义 TT_PrintData 类
class TT_PrintData implements IEntity {
    // 定义 PropertyChanged 事件
    private _propertyChanged: PropertyChangedEventHandler[] = [];
    public get PropertyChanged(): { add: (handler: PropertyChangedEventHandler) => void, remove: (handler: PropertyChangedEventHandler) => void } {
        return {
            add: (handler) => this._propertyChanged.push(handler),
            remove: (handler) => {
                const index = this._propertyChanged.indexOf(handler);
                if (index !== -1) {
                    this._propertyChanged.splice(index, 1);
                }
            }
        };
    }

    private id: string;
    private tradeNo: string;
    private formId: string;
    private printData: Uint8Array;
    private date: string;
    private businessId: string;
    private description: string;
    private byteLength: string;
    private tellerSerialNo: string;

    public get Id(): string {
        return this.id;
    }
    public set Id(value: string) {
        this.id = value;
        this.notifyPropertyChanged('Id');
    }

    public get TradeNo(): string {
        return this.tradeNo;
    }
    public set TradeNo(value: string) {
        this.tradeNo = value;
        this.notifyPropertyChanged('tradeNo');
    }

    public get FormId(): string {
        return this.formId;
    }
    public set FormId(value: string) {
        this.formId = value;
        this.notifyPropertyChanged('FormId');
    }

    public get ByteLength(): string {
        return this.byteLength;
    }
    public set ByteLength(value: string) {
        this.byteLength = value;
        this.notifyPropertyChanged('ByteLength');
    }

    public get PrintData(): Uint8Array {
        return this.printData;
    }
    public set PrintData(value: Uint8Array) {
        this.printData = value;
        this.notifyPropertyChanged('PrintData');
    }

    public get Date(): string {
        return this.date;
    }
    public set Date(value: string) {
        this.date = value;
        this.notifyPropertyChanged('Date');
    }

    public get BusinessId(): string {
        return this.businessId;
    }
    public set BusinessId(value: string) {
        this.businessId = value;
        this.notifyPropertyChanged('BusinessId');
    }

    public get Description(): string {
        return this.description;
    }
    public set Description(value: string) {
        this.description = value;
        this.notifyPropertyChanged('Description');
    }

    public get TellerSerialNo(): string {
        return this.tellerSerialNo;
    }
    public set TellerSerialNo(value: string) {
        this.tellerSerialNo = value;
        this.notifyPropertyChanged('TellerSerialNo');
    }

    // 触发属性变更通知
    private notifyPropertyChanged(propertyName: string): void {
        this._propertyChanged.forEach(handler => handler(this, { propertyName }));
    }

    // 从数组参数中更新对象属性
    public ChangeFromTable(parms: any[]): void {
        this.Id = parms[0].toString();
        this.TradeNo = parms[1].toString();
        this.FormId = parms[2].toString();
        this.byteLength = parms[3].toString();
        // this.PrintData = new Uint8Array(0); // 原注释部分
        this.Date = parms[4].toString();
        this.BusinessId = parms[5].toString();
        this.Description = parms[6].toString();
        this.TellerSerialNo = parms[7].toString();
    }

    // 将对象属性转换为字符串数组
    public ChangeToTable(): string[] {
        return [this.Id, this.TradeNo, this.FormId, this.byteLength, this.Date, this.BusinessId, this.Description, this.TellerSerialNo];
    }
}

export { TT_PrintData };