// 定义 PropertyChangedEventHandler 类型，模拟 C# 中的事件处理函数类型
type PropertyChangedEventHandler = (sender: any, args: { propertyName: string }) => void;

// 定义 IEntity 接口，由于原 C# 中未给出具体定义，这里先留空
interface IEntity {}

// 定义 INotifyPropertyChanged 接口
interface INotifyPropertyChanged {
    PropertyChanged: PropertyChangedEventHandler | null;
}

// 定义 TT_TradeForm 类
class TT_TradeForm implements IEntity, INotifyPropertyChanged {
    PropertyChanged: PropertyChangedEventHandler | null = null;

    private id: string = '';
    private formId: string = '';
    private tradeNo: string = '';
    private sort: string = '';
    private authLevel: string = '';
    private printFlag: string = '';
    private t_formid: string = '';
    private t_formname: string = '';

    get Id(): string {
        return this.id;
    }

    set Id(value: string) {
        this.id = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, { propertyName: 'Id' });
        }
    }

    get FormId(): string {
        return this.formId;
    }

    set FormId(value: string) {
        this.formId = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, { propertyName: 'FormId' });
        }
    }

    get TradeNo(): string {
        return this.tradeNo;
    }

    set TradeNo(value: string) {
        this.tradeNo = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, { propertyName: 'TradeNo' });
        }
    }

    get Sort(): string {
        return this.sort;
    }

    set Sort(value: string) {
        this.sort = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, { propertyName: 'Sort' });
        }
    }

    get AuthLevel(): string {
        return this.authLevel;
    }

    set AuthLevel(value: string) {
        this.authLevel = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, { propertyName: 'AuthLevel' });
        }
    }

    /**
     * 业务上的表单编号
     */
    get T_Form_ID(): string {
        return this.t_formid;
    }

    set T_Form_ID(value: string) {
        this.t_formid = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, { propertyName: 'T_Form_ID' });
        }
    }

    get T_Form_Name(): string {
        return this.t_formname;
    }

    set T_Form_Name(value: string) {
        this.t_formname = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, { propertyName: 'T_Form_Name' });
        }
    }

    get PrintFlag(): string {
        return this.printFlag;
    }

    set PrintFlag(value: string) {
        this.printFlag = value;
        if (this.PropertyChanged) {
            this.PropertyChanged(this, { propertyName: 'PrintFlag' });
        }
    }

    ChangeFromTable(parms: any[]): void {
        this.Id = parms[0].toString();
        this.FormId = parms[1].toString();
        this.TradeNo = parms[2].toString();
        this.Sort = parms[3].toString();
        this.AuthLevel = parms[4].toString();
        this.PrintFlag = parms[5].toString();
        this.T_Form_ID = parms[6].toString();
        this.T_Form_Name = parms[7].toString();
    }

    ChangeToTable(): string[] {
        return [this.Id, this.FormId, this.TradeNo, this.Sort, this.AuthLevel, this.PrintFlag, this.T_Form_ID, this.T_Form_Name];
    }
}

export { TT_TradeForm };