// 模拟导入相关模块，需要根据实际情况调整
import { RestClient } from '@/views/front/develop/ServiceProxy.Ext/WebAPI/RestClient'; // 假设 RestClient 定义在此路径
import { ServiceSettings } from "@/views/front/develop/ServiceProxy.Ext/ServiceSettings"
import LoginUserInfo from "@/views/front/platfrom/common/RunningParameters/LoginUserInfo";
//import { RmAuthResult, FlawTradeListResult, AuthHistoryInfoResult, FlawInfoReq } from "@/views/front/develop/ServiceProxy.Ext/WebAPI/Auth/Entity/"// 假设这些实体定义在此路径
import { RSAUtil } from '@/views/front/develop/ServiceProxy.Ext/WebAPI/RSAUtil'; // 假设 RSAUtil 定义在此路径
import PlatformLogger from "@/views/front/platfrom/common/LogSystem/PlatformLogger"; //定义在此路径 hulei 日志登记还没有
import GetTokenReq from "@/views/front/develop/ServiceProxy.Ext/WebAPI/Auth/Entity/GetTokenReq"
import { TokenResult } from "@/views/front/develop/ServiceProxy.Ext/WebAPI/Auth/Entity/TokenResult"
import { ElementInfoResult } from "@/views/front/develop/ServiceProxy.Ext/WebAPI/Auth/Entity/ElementInfoResult"
import { ApplyInfoReq } from "@/views/front/develop/ServiceProxy.Ext/WebAPI/Auth/Entity/ApplyInfoReq"
import { RmAuthResult } from "@/views/front/develop/ServiceProxy.Ext/WebAPI/Auth/Entity/RmAuthResult"
import FlawTradeListResult from "@/views/front/develop/ServiceProxy.Ext/WebAPI/Auth/Entity/FlawTradeListResult"
import { AuthHistoryInfoResult } from "@/views/front/develop/ServiceProxy.Ext/WebAPI/Auth/Entity/AuthHistoryInfoResult"
import FlawInfoReq from "@/views/front/develop/ServiceProxy.Ext/WebAPI/Auth/Entity/FlawInfoReq"
export class AuthHandle {
    private static restClient: RestClient = new RestClient(ServiceSettings.AuthServiceUri);
    private static authSystemName: string = ServiceSettings.AuthSystemName;
    private static authSystemSecret: string = ServiceSettings.AuthSystemSecret;
    private static tellerNo: string = LoginUserInfo.TellerNo.trim();
    private static tellerName: string = LoginUserInfo.TellerName.trim();
    private static threadTimer: NodeJS.Timer | null = null;

    private static authToken: string | null = null;

    public static get AuthToken(): string {
        if (this.authToken === null) {
            this.authToken = this.GetToken();
            this.threadTimer = setInterval(() => this.TimerUp(null), 300000);
        }
        return this.authToken;
    }

    private static TimerUp(value: any): void {
        try {
            this.authToken = this.GetToken();
        } catch (e) {
            if (e instanceof Error) {
                PlatformLogger.TradeErrorInfo('GetToken Exception', e);
            }
        }
    }

    private static GetToken(): string {
        const req: GetTokenReq = new GetTokenReq();
        req.sysName = this.authSystemName;
        req.secret = new RSAUtil(this.GetPublicKey()).encrypt(this.authSystemSecret);
        let result: TokenResult = null
        this.restClient.Post<TokenResult>("auth/getToken", req).then(x => {
            result = x

        });
        return result.accessToken;
    }

    private static GetPublicKey(): string {
        const para: { [key: string]: string } = {};
        let result = "";
        para["sysName"] = this.authSystemName;
        this.restClient.Get<string>("auth/getPublicKey", para).then(x => {
            result = x;
        });
        return result;
    }

    public static GetRmAuthElementByMenu(menuCode: string, menuNo: string = "0"): ElementInfoResult {
        const para: { [key: string]: string } = {};
        para["menuCode"] = menuCode;
        para["menuNo"] = menuNo;
        let result: ElementInfoResult = null;
        this.restClient.Get<ElementInfoResult>("client/getRmAuthElementByMenu", para, this.AuthToken).then(x => {
            result = x;
        });
        return result;
    }

    public static ApplyRmAuth(applyInfoReq: ApplyInfoReq): string {
        let result = "";
        this.restClient.Post<string>("client/applyRmAuth", applyInfoReq, this.AuthToken).then(x => {
            result = x
        });
        return result;
    }

    public static GetRmAuthResultByRecord(rmAuthRecord: string): RmAuthResult {
        let result: RmAuthResult = null;
        const para: { [key: string]: string } = {};
        para["rmAuthRecord"] = rmAuthRecord;
        this.restClient.Get<RmAuthResult>("client/getRmAuthResultByRecord", para, this.AuthToken).then(x => {
            result = x

        });
        return result;
    }

    public static UndoRmAuth(rmAuthRecord: string): string {
        let result = "";
        const para: { [key: string]: string } = {};
        para["rmAuthRecord"] = rmAuthRecord;
        this.restClient.Get<string>("client/undoRmAuth", para, this.AuthToken).then(x => {
            result = x

        });
        return result;
    }

    public static UploadFile(fileName: string, fileByte: Uint8Array): string {
        let resultUploadeFile = "";
        try {
            const startime = new Date().toISOString().replace(/[^0-9]/g, '');
            PlatformLogger.SystemInfo(`--------------------------start--${startime}::${fileName}------------------------`);
            this.restClient.UploadFile<string>("file/upload", fileName, fileByte, this.AuthToken).then(x => {
                resultUploadeFile = x;

            });
        } catch (e) {
            // 原代码未处理异常，这里保持原样
        }
        const endtime = new Date().toISOString().replace(/[^0-9]/g, '');
        PlatformLogger.SystemInfo(`-----------------------end-----${endtime}::${resultUploadeFile}------------------------`);
        return resultUploadeFile;
    }

    public static UploadFileWithoutParams(): string {
        // 由于 Node.js 中文件操作和浏览器不同，这里只是模拟
        // 实际使用时需要正确实现文件读取逻辑
        // 以下代码仅为示例，无法直接运行
        // const filePath = "D:/pictrue/oYYBAFsfnJ-AI-bzAAAhtqb3L2s004.jpg";
        // const fileBuffer = fs.readFileSync(filePath);
        // return this.UploadFile("oYYBAFsfnJ-AI-bzAAAhtqb3L2s004.jpg", new Uint8Array(fileBuffer));
        throw new Error('需要实现文件读取逻辑');
    }

    public static GetFlawTradeList(): FlawTradeListResult[] {
        let result = null;
        const para: { [key: string]: string } = {};
        para["userCode"] = this.tellerNo;
        this.restClient.Get<FlawTradeListResult[]>("record/getFlawTradeList", para, this.AuthToken).then(x => {

            result = x
        });
        return result
    }

    public static GetHistoryInfo(recordCode: string): AuthHistoryInfoResult {
        const para: { [key: string]: string } = {};
        let result: AuthHistoryInfoResult = null;
        para["recordCode"] = recordCode;
        this.restClient.Get<AuthHistoryInfoResult>("/client/getHistoryInfo", para, this.AuthToken).then(x => {
            result = x;
        });
        return result;
    }

    public static AdditionalRmauthInfo(flawInfoReq: FlawInfoReq): string {
        let result = null;
        this.restClient.Post<string>("record/additionalRmauthInfo", flawInfoReq, this.AuthToken);
        return result
    }
}
