import crypto from 'crypto';

export class RSAUtil {
    private publicKeyRsaProvider: crypto.KeyObject;

    constructor(publicKey: string) {
        this.publicKeyRsaProvider = this.createRsaProviderFromPublicKey(publicKey);
    }

    private createRsaProviderFromPublicKey(publicKeyString: string): crypto.KeyObject {
        const publicKeyBuffer = Buffer.from(publicKeyString, 'base64');
        return crypto.createPublicKey({
            key: publicKeyBuffer,
            format: 'der',
            type: 'spki'
        });
    }

    public encrypt(text: string): string {
        const encrypted = crypto.publicEncrypt(
            {
                key: this.publicKeyRsaProvider,
                padding: crypto.constants.RSA_PKCS1_PADDING
            },
            Buffer.from(text, 'utf8')
        );
        return encrypted.toString('base64');
    }
}