import { ref, reactive } from 'vue';

interface AttachFile {
    // 附件文件结构定义
    // 根据实际业务需要补充具体属性
}

export class TransitEntity {
    // 服务方法名
    public FuncName: string = '';
    // 服务类名
    public ServiceName: string = '';
    // 方法参数(以Dictionary方式，key为参数在方法中的位置，value为参数值)
    public Parameters: Map<string, string> = new Map();
    // 附件列表，特殊结构(转换成table结构)与大数据byte[]参数做为附件传递
    public AttachFiles: AttachFile[] = [];
    // 超时设置，单位ms
    public Timeout: number = 0;

    constructor() {
        this.Parameters = new Map<string, string>();
        this.AttachFiles = [];
    }
}

// Vue 3 组合式API使用示例
export function useTransitEntity() {
    const transitEntity = reactive(new TransitEntity());

    const setFuncName = (name: string) => {
        transitEntity.FuncName = name;
    };

    const setServiceName = (name: string) => {
        transitEntity.ServiceName = name;
    };

    const addParameter = (key: string, value: string) => {
        transitEntity.Parameters.set(key, value);
    };

    const addAttachFile = (file: AttachFile) => {
        transitEntity.AttachFiles.push(file);
    };

    const setTimeoutValue = (timeout: number) => {
        transitEntity.Timeout = timeout;
    };

    return {
        transitEntity,
        setFuncName,
        setServiceName,
        addParameter,
        addAttachFile,
        setTimeoutValue
    };
}